// swad.c: main

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <stdlib.h>		// For exit

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_con.h"
#include "swad_db.h"
#include "swad_deg.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_prf.h"
#include "swad_ntf.h"

/****************************************************************************/
/******************************** Constants *********************************/
/****************************************************************************/

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Actions Actions[Act_NUM_ACTIONS];

/****************************************************************************/
/******************************* Internal types *****************************/
/****************************************************************************/

/****************************************************************************/
/************************ Internal global variables *************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

/****************************************************************************/
/****************************** Main function *******************************/
/****************************************************************************/

int main (int argc, char *argv[])
  {
   extern const char *Txt_Your_session_has_expired_due_to_inactivity[Txt_NUM_LANGUAGES];
   extern const char *Txt_You_dont_have_permission_to_perform_this_action[Txt_NUM_LANGUAGES];
/*
   fprintf (stdout,"Content-type: text/html\n\n");
   fprintf (stdout,"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">" \
                   "<head><title>%s</title></head><body><br /><br /><br /><br />" \
                   "<h1 align=\"center\">Fuera de servicio por labores de mantenimiento</h1>" \
                   "<br /><br /><h2 align=\"center\">Dentro de unos minutos se restablecer&aacute; el funcionamiento normal</h2>" \
                   "<br /><h2 align=\"center\">Disculpe las molestias</h2></body></html>",Cfg_PLATFORM_FULL_NAME);
   exit (0);
*/
   /***** Initialize global variables *****/
   Gbl_InitializeGlobals ();

   /***** Open database connection *****/
   DB_OpenDBConnection ();

   /***** Read parameters *****/
   if (Par_GetQueryString ())
     {
      Par_GetMainParameters ();
      Deg_InitCurrentCourse ();
     }

   if (Gbl.CurrentAct != ActWebSvc)
     {
      /***** Create file for HTML output *****/
      Fil_CreateFileForXHTMLOutput ();

      /***** Remove old (expired) sessions *****/
      Ses_RemoveExpiredSessions ();

      /***** Remove old users from connected list *****/
      Con_RemoveOldConnected ();

      /***** Get number of connected users *****/
      if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW)
       {
        Con_GetConnectedUsrsTotal ();
        Ses_GetNumSessions ();
       }

      /***** Check user's password and get user's data *****/
      if (!Gbl.Session.HasBeenDisconnected)
        {
         if (Gbl.Session.IsOpen)
           {
            if (Usr_CheckUsrAndGetUsrData ())
              {
               if (Gbl.CurrentAct == ActRef)
                  Ses_UpdateSessionLastRefreshInDB ();
               else	// If refreshing connected users ==> don't refresh session
                 {
                  Ses_UpdateSessionDataInDB ();
                  Con_UpdateMeInConnectedList ();
                 }
               Act_AdjustCurrentAction ();
              }
            else
               Lay_ShowErrorAndExit (NULL);
           }
         else if (Gbl.CurrentAct == ActAutUsrInt ||
                  Gbl.CurrentAct == ActAutUsrExt)
           {
            if (Usr_CheckUsrAndGetUsrData ())
              {
               Act_AdjustCurrentAction ();
               Ses_CreateSession ();

               /* Set preferences from current IP */
               Prf_SetPrefsFromIP ();
              }
            else
               Lay_ShowErrorAndExit (NULL);

            if (Gbl.CurrentAct == ActAutUsrExt)
               Imp_InsertImpSessionInDB ();
           }
        }
     }

   if (Gbl.CurrentAct != ActRef &&
       Gbl.CurrentAct != ActWebSvc)
     {
      /***** If session has been disconnected... *****/
      if (Gbl.Session.HasBeenDisconnected)
        {
         Gbl.CurrentTab = TabUnk;
         Gbl.CurrentAct = ActUnk;
         Lay_ShowAlert (WARNING,Txt_Your_session_has_expired_due_to_inactivity[Gbl.Prefs.Language]);
         Lay_ShowErrorAndExit (NULL);
        }

      if (Gbl.Usrs.Me.Logged)
        { 
         if (Gbl.CurrentTab == TabUnk)
           {
            /***** Set tab, when unknown, to last tab of current user *****/
            /* If I have log in now, the current tab will be that stored in database, else it will be that coming from the last form */
            Gbl.CurrentTab = Gbl.Usrs.Me.UsrDat.LastTab;
            if ((Gbl.CurrentTab == TabCrs) && (Gbl.CurrentCrs.Crs.CrsCod < 0))
               Gbl.CurrentTab = TabDeg;
           }

         Usr_UpdateMyLastTabAndTime ();
        }
     }

   /***** Check if the user have permission to execute the action *****/
   if ((Gbl.CurrentCrs.Crs.CrsCod >= 0 && (( Gbl.Usrs.Me.IHaveAccessToCurrentCrs && !(Actions[Gbl.CurrentAct].PermisIfIBelongToCrs     & (1 << Gbl.Usrs.Me.UsrTypeLog)))   ||
                                           (!Gbl.Usrs.Me.IHaveAccessToCurrentCrs && !(Actions[Gbl.CurrentAct].PermisIfIDontBelongToCrs & (1 << Gbl.Usrs.Me.UsrTypeLog))))) ||
       (Gbl.CurrentCrs.Crs.CrsCod < 0 && !(Actions[Gbl.CurrentAct].PermisIfNoCrsSelected & (1 << Gbl.Usrs.Me.UsrTypeLog))))
      Lay_ShowErrorAndExit (Txt_You_dont_have_permission_to_perform_this_action[Gbl.Prefs.Language]);

   /***** Wait if other users are updating database or files *****/
   if (Actions[Gbl.CurrentAct].ExecuteLockfile)
      if (!Fil_WaitSemaphore ())
         Lay_ShowErrorAndExit (NULL);

   /***** Execute a function depending on the action *****/
   if (Actions[Gbl.CurrentAct].FunctionPriori != NULL)
      Actions[Gbl.CurrentAct].FunctionPriori ();

   /***** Start writing XHTML output *****/
   Lay_WriteStartOfPage ();

   /***** Make a processing or other depending on the action *****/
   if (Actions[Gbl.CurrentAct].FunctionPosteriori != NULL)
      Actions[Gbl.CurrentAct].FunctionPosteriori ();

   /***** Send pending notifications by e-mail *****/
   if (Actions[Gbl.CurrentAct].ExecuteLockfile)
      Ntf_SendPendingNotifByEMailToAllUsrs ();

   /***** Cleanup and exit *****/
   Lay_ShowErrorAndExit (NULL);

   return 0; // Control don't reach this point. Used to avoid warning.
  }
