// swad_asg.c: assignments

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_asg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_grp.h"
#include "swad_pag.h"
#include "swad_par.h"
#include "swad_str.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Asg_ShowAllAssignments (void);
static void Asg_ShowOneAssignment (long AsgCod,bool ThisCourseHasGroups);
static void Asg_WriteAsgAuthor (long UsrCod);
static void Asg_WriteAssignmentFolder (struct Assignment *Asg);
static void Asg_GetParamAsgOrderType (void);
static void Asg_PutFormToCreateNewAsg (void);
static void Asg_PutFormsToRemEditOneAsg (long AsgCod);
static void Asg_GetDataOfAssignment (struct Assignment *Asg,const char *Query);
static void Asg_GetAssignmentTxtFromDB (long AsgCod,char *Txt);
static void Asg_PutParamAsgCod (long AsgCod);
static bool Asg_CheckIfSimilarAssignmentExists (const char *Field,const char *Value,long AsgCod);
static void Asg_ShowLstGrpsToEditAssignment (long AsgCod);
static void Asg_CreateAssignment (struct Assignment *Asg);
static void Asg_UpdateAssignment (struct Assignment *Asg);
static bool Asg_CheckIfAsgIsAssociatedToGrps (long AsgCod);
static void Asg_RemoveAllTheGrpsAssociatedToAndAssignment (long AsgCod);
static void Asg_CreateGrps (long AsgCod);
static void Asg_GetAndWriteNamesOfGrpsAssociatedToAsg (long AsgCod);
static bool Asg_CheckIfICanDoThisAssignment (long AsgCod);

/****************************************************************************/
/************************ List all the assignments **************************/
/****************************************************************************/

void Asg_SeeAssignments (void)
  {
   /***** Get parameters *****/ 
   Asg_GetParamAsgOrderType ();
   Grp_GetParamWhichGrps ();
   Pag_GetParamPagNum (Pag_ASSIGNMENTS);

   /***** Put link (form) to edit assignments *****/
   if (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER)
      Asg_PutFormToCreateNewAsg ();

   /***** Show all the assignments *****/
   Asg_ShowAllAssignments ();
  }

/****************************************************************************/
/************************ Show all the assignments **************************/
/****************************************************************************/

static void Asg_ShowAllAssignments (void)
  {
   extern const char *Txt_ASSIGNMENTS_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_ASSIGNMENTS_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_Assignment[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_files[Txt_NUM_LANGUAGES];
   extern const char *Txt_Folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_assignments[Txt_NUM_LANGUAGES];
   bool ThisCourseHasGroups;
   tAsgsOrderType Order;
   struct Pagination Pagination;
   unsigned NumAsg;

   /***** Get number of groups in current course *****/
   ThisCourseHasGroups = (Grp_CountNumGrpsInCurrentCrs () != 0);	// This course has groups
   if (!ThisCourseHasGroups)
      Gbl.CurrentCrs.Grps.WhichGrps = Grp_ALL_GROUPS;

   /***** Get list of assignments *****/
   Asg_GetListAssignments ();

   /* Select whether show only my groups or all groups */
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActSeeAsg);
   Asg_PutHiddenParamAsgOrderType ();
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
   if (ThisCourseHasGroups)
      Grp_ShowSelectorWhichGrps ();
   fprintf (Gbl.F.Out,"</form></div>");

   if (Gbl.Asgs.Num)	// There are assignments in current course
     {
      /***** Allocate memory for assignment text *****/
      if (!Str_AllocateText ())
         Lay_ShowErrorAndExit (Gbl.Message);

      /***** Compute variables related to pagination *****/
      Pagination.NumItems = (int) Gbl.Asgs.Num;
      Pagination.CurrentPage = (int) Gbl.Pag.CurrentPage;
      Pag_CalculatePagination (&Pagination);
      Gbl.Pag.CurrentPage = (unsigned) Pagination.CurrentPage;

      /***** Write links to pages *****/
      if (Pagination.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered (Pag_ASSIGNMENTS,0,&Pagination);

      /***** Table head *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
      fprintf (Gbl.F.Out,"<tr>");
      for (Order=Asg_ORDER_BY_START_DATE; Order<=Asg_ORDER_BY_END_DATE; Order++)
        {
         fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">");
         Act_FormStart (ActSeeAsg);
         Grp_PutParamWhichGrps ();
         Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_ASSIGNMENTS_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
         if (Order == Gbl.Asgs.SelectedOrderType)
            fprintf (Gbl.F.Out,"<u>");
         fprintf (Gbl.F.Out,"%s",Txt_ASSIGNMENTS_ORDER[Order][Gbl.Prefs.Language]);
         if (Order == Gbl.Asgs.SelectedOrderType)
            fprintf (Gbl.F.Out,"</u>");
         fprintf (Gbl.F.Out,"</acronym></a>&nbsp;</form></td>");
        }
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">%s</td>",
               Txt_Assignment[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">%s</td>",
               Txt_Upload_files[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">%s</td></tr>\n",
               Txt_Folder[Gbl.Prefs.Language]);

      /***** Write all the assignments *****/
      for (NumAsg=1; NumAsg<=Gbl.Asgs.Num; NumAsg++)
         if ((int) NumAsg >= Pagination.FirstItemVisible &&
             (int) NumAsg <= Pagination.LastItemVisible)
            Asg_ShowOneAssignment (Gbl.Asgs.LstAsgCods[NumAsg-1],ThisCourseHasGroups);

      /***** Table end *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Free memory for assignment text *****/
      Str_FreeTxt ();

      /***** Write again links to pages *****/
      if (Pagination.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered (Pag_ASSIGNMENTS,0,&Pagination);
     }
   else	// There are no assignments in current course
      Lay_ShowAlert (INFO,Txt_No_assignments[Gbl.Prefs.Language]);

   /***** Free list of assignments *****/
   Asg_FreeListAssignments ();
  }

/****************************************************************************/
/*************************** Show one assignment ****************************/
/****************************************************************************/

static void Asg_ShowOneAssignment (long AsgCod,bool ThisCourseHasGroups)
  {
   extern const char *Txt_ACTIVITY_TYPES[Asg_NUM_TYPES_SEND_WORK][Txt_NUM_LANGUAGES];
   extern const char *Txt_Yes[Txt_NUM_LANGUAGES];
   extern const char *Txt_No[Txt_NUM_LANGUAGES];
   struct Assignment Asg;

   /***** Get data of this assignment *****/
   Asg.AsgCod = AsgCod;
   Asg_GetDataOfAssignmentByCod (&Asg);

   fprintf (Gbl.F.Out,"<tr>");

   /***** Write first row of data of this assignment *****/
   /* Start date/time */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;<br />%02u:%02u h</td>",
            Asg.Open ? "ASG_DATE_GREEN" : "ASG_DATE_RED",
            Asg.DateTimes[Asg_START_TIME].Day,
            Asg.DateTimes[Asg_START_TIME].Month,
	    Asg.DateTimes[Asg_START_TIME].Year % 100,
	    Asg.DateTimes[Asg_START_TIME].Hour,
	    Asg.DateTimes[Asg_START_TIME].Minute);
   /* End date/time */
   fprintf (Gbl.F.Out,"<td align=\"left\"  valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;<br />%02u:%02u h</td>",
            Asg.Open ? "ASG_DATE_GREEN" : "ASG_DATE_RED",
            Asg.DateTimes[Asg_END_TIME  ].Day,
            Asg.DateTimes[Asg_END_TIME  ].Month,
            Asg.DateTimes[Asg_END_TIME  ].Year % 100,
	    Asg.DateTimes[Asg_END_TIME  ].Hour,
	    Asg.DateTimes[Asg_END_TIME  ].Minute);
   /* Assignment title and groups */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\">" \
                      "<p class=\"ASG_TITLE\">%s</p>",
            Asg.Title);
   if (ThisCourseHasGroups)
      Asg_GetAndWriteNamesOfGrpsAssociatedToAsg (Asg.AsgCod);
   fprintf (Gbl.F.Out,"</td>");
   /* Send work? */
   fprintf (Gbl.F.Out,"<td align=\"center\"  valign=\"top\" class=\"%s\">" \
                      "<img src=\"%s/%s12x12.gif\" alt=\"%s\" title=\"%s\" width=\"12\" height=\"12\" />" \
                      "<br />%s</td>",
            (Asg.SendWork == Asg_SEND_WORK) ? "DAT_N" : "DAT",
            Gbl.Prefs.IconsURL,
            (Asg.SendWork == Asg_SEND_WORK) ? "cdok_on" : "cdno_off",
            Txt_ACTIVITY_TYPES[Asg.SendWork][Gbl.Prefs.Language],
            Txt_ACTIVITY_TYPES[Asg.SendWork][Gbl.Prefs.Language],
            (Asg.SendWork == Asg_SEND_WORK) ? Txt_Yes[Gbl.Prefs.Language] : Txt_No[Gbl.Prefs.Language]);

   /* Assignment folder */
   Asg_WriteAssignmentFolder (&Asg);

   /***** Write second row of data of this assignment *****/
   fprintf (Gbl.F.Out,"</tr><tr><td colspan=\"2\" align=\"left\" valign=\"top\">");

   /* Author of the assignment */
   Asg_WriteAsgAuthor (Asg.UsrCod);

   /* Forms to remove/edit this assignment */
   switch (Gbl.Usrs.Me.UsrTypeLog)
     {
      case Usr_TEACHER:
      case Usr_SUPERUSER:
         Asg_PutFormsToRemEditOneAsg (Asg.AsgCod);
         break;
      default:
         break;
     }
   fprintf (Gbl.F.Out,"</td>");

   /* Text of the assignment */
   Asg_GetAssignmentTxtFromDB (Asg.AsgCod,Gbl.Txt);
   Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.Txt,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to recpectful HTML
   Str_InsertLinkInURLs (Gbl.Txt,Cte_MAX_BYTES_TEXT,60);	// Insert links
   fprintf (Gbl.F.Out,"<td align=\"justify\" class=\"DAT\">" \
                      "%s<br />&nbsp;<br />&nbsp;</td>" \
                      "<td colspan=\"2\"></td>" \
                      "</tr>\n",
            Gbl.Txt);
  }

/*****************************************************************************/
/********************* Write the author of an assignment *********************/
/*****************************************************************************/

static void Asg_WriteAsgAuthor (long UsrCod)
  {
   bool ShowPhoto = false;
   char PhotoURL[PATH_MAX+1];
   char FirstName[Usr_MAX_BYTES_NAME+1];
   char Surnames[2*(Usr_MAX_BYTES_NAME+1)];

   /***** Get data of author *****/
   Gbl.Usrs.Other.UsrDat.UsrCod = UsrCod;
   if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat))	// Get of the database the data of the author
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (&Gbl.Usrs.Other.UsrDat,PhotoURL);

   /***** Show photo *****/
   Usr_ShowUsrPhoto (&Gbl.Usrs.Other.UsrDat,ShowPhoto ? PhotoURL : NULL,12,16,true);

   /***** Write name *****/
   strcpy (FirstName,Gbl.Usrs.Other.UsrDat.FirstName);
   strcpy (Surnames,Gbl.Usrs.Other.UsrDat.Surname1);
   if (Gbl.Usrs.Other.UsrDat.Surname2[0])
     {
      strcat (Surnames," ");
      strcat (Surnames,Gbl.Usrs.Other.UsrDat.Surname2);
     }
   Str_LimitLengthHTMLStr (FirstName,9);
   Str_LimitLengthHTMLStr (Surnames,9);
   fprintf (Gbl.F.Out,"<span class=\"MSG_AUT\">%s %s</span>",
            FirstName,Surnames);
  }

/*****************************************************************************/
/********************* Write the folder of an assignment *********************/
/*****************************************************************************/

static void Asg_WriteAssignmentFolder (struct Assignment *Asg)
  {
   extern const char *Txt_Upload_file_or_create_folder_in_FOLDER[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\">");
   if (Asg->SendWork == Asg_SEND_WORK)
     {
      /***** Folder icon *****/
      fprintf (Gbl.F.Out,"<table cellspacing=\"2\" cellpadding=\"0\"><tr>" \
                         "<td class=\"BT\">");
      if (Asg->Open && Asg->ICanDo)	// I can send files to this assignment folder
        {
         /* Form to create a new file or folder */
         Act_FormStart (ActFrmCreAsgUsr);
	 Brw_ParamAssignmentsZoneUsr ();
         Brw_PutParamsPathAndFile (Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,Asg->Folder);
         sprintf (Gbl.MsgWindowStatus,Txt_Upload_file_or_create_folder_in_FOLDER[Gbl.Prefs.Language],
                  Asg->Folder);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/folder_closed_plus12x16.gif\" alt=\"%s\" width=\"12\" height=\"16\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
        }
      else			// I can't send files to this assignment folder
         fprintf (Gbl.F.Out,"<img src=\"%s/folder_closed12x16.gif\" width=\"12\" height=\"16\" />",
                  Gbl.Prefs.IconsURL);

      /***** Folder name *****/
      fprintf (Gbl.F.Out,"</td>" \
                         "<td align=\"left\" class=\"DAT\">%s</td>" \
                         "</tr></table>",
               Asg->Folder);
     }
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/******* Get parameter with the type or order in list of assignments *********/
/*****************************************************************************/

static void Asg_GetParamAsgOrderType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Asgs.SelectedOrderType = (tAsgsOrderType) UnsignedNum;
   else
      Gbl.Asgs.SelectedOrderType = Asg_ORDER_BY_START_DATE;
  }

/****************************************************************************/
/*** Put a hidden parameter with the type of order in list of assignments ***/
/****************************************************************************/

void Asg_PutHiddenParamAsgOrderType (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Asgs.SelectedOrderType);
  }

/****************************************************************************/
/******************* Put a link (form) to edit assignments ******************/
/****************************************************************************/

static void Asg_PutFormToCreateNewAsg (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_assignment[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActFrmNewAsg);
   Asg_PutHiddenParamAsgOrderType ();
   Grp_PutParamWhichGrps ();
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
   sprintf (Gbl.MsgWindowStatus,Txt_New_assignment[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_New_assignment[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/new12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></div>",
            Gbl.Prefs.IconsURL,Txt_New_assignment[Gbl.Prefs.Language],Txt_New_assignment[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/***************** Put a link (form) to edit one assignment *****************/
/****************************************************************************/

static void Asg_PutFormsToRemEditOneAsg (long AsgCod)
  {
   extern const char *Txt_Remove[Txt_NUM_LANGUAGES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");

   /***** Put form to remove assignment *****/
   fprintf (Gbl.F.Out,"<td align=\"left\">");
   Act_FormStart (ActAskRemAsg);
   Asg_PutParamAsgCod (AsgCod);
   Asg_PutHiddenParamAsgOrderType ();
   Grp_PutParamWhichGrps ();
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove[Gbl.Prefs.Language],NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      "</acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Txt_Remove[Gbl.Prefs.Language]);

   /***** Put form to edit assignment *****/
   fprintf (Gbl.F.Out,"<td align=\"left\">");
   Act_FormStart (ActEdiOneAsg);
   Asg_PutParamAsgCod (AsgCod);
   Asg_PutHiddenParamAsgOrderType ();
   Grp_PutParamWhichGrps ();
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Edit[Gbl.Prefs.Language],NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      "</acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Txt_Edit[Gbl.Prefs.Language]);

   fprintf (Gbl.F.Out,"</tr></table>");
  }

/****************************************************************************/
/************************ List all the assignments **************************/
/****************************************************************************/

void Asg_GetListAssignments (void)
  {
   char OrderBySubQuery[256];
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumAsg;

   if (Gbl.Asgs.LstIsRead)
      Asg_FreeListAssignments ();

   /***** Get list of assignments from database *****/
   switch (Gbl.Asgs.SelectedOrderType)
     {
      case Asg_ORDER_BY_START_DATE:
         sprintf (OrderBySubQuery,"StartTime DESC,EndTime DESC");
         break;
      case Asg_ORDER_BY_END_DATE:
         sprintf (OrderBySubQuery,"EndTime DESC,StartTime DESC");
     }
   if (Gbl.CurrentCrs.Grps.WhichGrps == Grp_ONLY_MY_GROUPS)
      sprintf (Query,"SELECT AsgCod" \
                     " FROM assignments" \
                     " WHERE CrsCod='%ld'" \
                     " AND (AsgCod NOT IN (SELECT AsgCod FROM asg_grp) OR"
                     " AsgCod IN (SELECT asg_grp.AsgCod FROM asg_grp,crs_grp_usr" \
                     " WHERE crs_grp_usr.UsrCod='%ld' AND asg_grp.GrpCod=crs_grp_usr.GrpCod))" \
                     " ORDER BY %s",
               Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod,OrderBySubQuery);
   else	// Gbl.CurrentCrs.Grps.WhichGrps == Grp_ALL_GROUPS
      sprintf (Query,"SELECT AsgCod" \
                     " FROM assignments" \
                     " WHERE CrsCod='%ld' ORDER BY %s",
               Gbl.CurrentCrs.Crs.CrsCod,OrderBySubQuery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get assignments");
   
   if (NumRows) // Assignments found...
     {
      Gbl.Asgs.Num = (unsigned) NumRows;

      /***** Create list of assignments *****/
      if ((Gbl.Asgs.LstAsgCods = (long *) calloc (NumRows,sizeof (long))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store list of assignments.");

      /***** Get the assignments codes *****/
      for (NumAsg=0; NumAsg<Gbl.Asgs.Num; NumAsg++)
        {
         /* Get next assignment code */
         row = mysql_fetch_row (mysql_res);
         if ((Gbl.Asgs.LstAsgCods[NumAsg] = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Error: wrong assignment code.");
        }
     }
   else
      Gbl.Asgs.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   Gbl.Asgs.LstIsRead = true;
  }

/*****************************************************************************/
/******************* Get assignment data using its code **********************/
/*****************************************************************************/

void Asg_GetDataOfAssignmentByCod (struct Assignment *Asg)
  {
   char Query[1024];

   /***** Build query *****/
   sprintf (Query,"SELECT AsgCod,UsrCod," \
                  "DATE_FORMAT(StartTime,'%%Y%%m%%d%%H%%i%%S')," \
                  "DATE_FORMAT(EndTime,'%%Y%%m%%d%%H%%i%%S')," \
                  "NOW() BETWEEN StartTime AND EndTime," \
                  "Title,Folder" \
                  " FROM assignments" \
                  " WHERE AsgCod='%ld' AND CrsCod='%ld'",
            Asg->AsgCod,Gbl.CurrentCrs.Crs.CrsCod);

   /***** Get data of assignment *****/
   Asg_GetDataOfAssignment (Asg,Query);
  }

/*****************************************************************************/
/*************** Get assignment data using its folder name *******************/
/*****************************************************************************/

void Asg_GetDataOfAssignmentByFolder (struct Assignment *Asg)
  {
   char Query[1024];

   /***** Query database *****/
   sprintf (Query,"SELECT AsgCod,UsrCod," \
                  "DATE_FORMAT(StartTime,'%%Y%%m%%d%%H%%i%%S')," \
                  "DATE_FORMAT(EndTime,'%%Y%%m%%d%%H%%i%%S')," \
                  "NOW() BETWEEN StartTime AND EndTime," \
                  "Title,Folder" \
                  " FROM assignments" \
                  " WHERE CrsCod='%ld' AND Folder='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,Asg->Folder);

   /***** Get data of assignment *****/
   Asg_GetDataOfAssignment (Asg,Query);
  }

/*****************************************************************************/
/************************* Get assignment data *******************************/
/*****************************************************************************/

static void Asg_GetDataOfAssignment (struct Assignment *Asg,const char *Query)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Clear data *****/
   Asg->DateTimes[Asg_START_TIME].Day   =
   Asg->DateTimes[Asg_START_TIME].Month =
   Asg->DateTimes[Asg_START_TIME].Year  = 0;
   Asg->Title[0] = '\0';

   /***** Get data of assignment from database *****/
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get assignment data");

   if (NumRows) // Assignment found...
     {
      /* Get row */
      row = mysql_fetch_row (mysql_res);

      /* Get code of the assignment (row[0]) */
      Asg->AsgCod = Str_ConvertStrCodToLongCod (row[0]);

      /* Get author of the assignment (row[1]) */
      Asg->UsrCod = Str_ConvertStrCodToLongCod (row[1]);

      /* Get start date (row[2] holds the start date in YYYYMMDDHHMMSS format) */
      if (!(Dat_GetDateTimeFromYYYYMMDDHHMMSS (&(Asg->DateTimes[Asg_START_TIME]),row[2])))
	 Lay_ShowErrorAndExit ("Error when reading start date of assignment from database.");

      /* Get end date (row[3] holds the end date in YYYYMMDDHHMMSS format) */
      if (!(Dat_GetDateTimeFromYYYYMMDDHHMMSS (&(Asg->DateTimes[Asg_END_TIME]),row[3])))
	 Lay_ShowErrorAndExit ("Error when reading end date of assignment from database.");

      /* Get whether the assignment is open or closed (row(4)) */
      Asg->Open = row[4][0] == '1';

      /* Get the title of the assignment (row[5]) */
      strcpy (Asg->Title,row[5]);

      /* Get the folder for the assignment files (row[6]) */
      strcpy (Asg->Folder,row[6]);
      Asg->SendWork = (Asg->Folder[0] != '\0');

      /* Can I do this assignment? */
      Asg->ICanDo = Asg_CheckIfICanDoThisAssignment (Asg->AsgCod);
     }
   else
     {
      /* Initialize to empty assignment */
      Asg->AsgCod = -1L;
      Asg->Open = false;
      Asg->Title[0] = '\0';
      Asg->SendWork = false;
      Asg->Folder[0] = '\0';
      Asg->ICanDo = false;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************************* Free list of assignments **************************/
/*****************************************************************************/

void Asg_FreeListAssignments (void)
  {
   if (Gbl.Asgs.LstIsRead && Gbl.Asgs.LstAsgCods)
     {
      /***** Free memory used by the list of courses in degree *****/
      free ((void *) Gbl.Asgs.LstAsgCods);
      Gbl.Asgs.LstAsgCods = NULL;
      Gbl.Asgs.Num = 0;
      Gbl.Asgs.LstIsRead = false;
     }
  }

/****************************************************************************/
/******************** Get assignment text from database *********************/
/****************************************************************************/

static void Asg_GetAssignmentTxtFromDB (long AsgCod,char *Txt)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get text of assignment from database *****/
   sprintf (Query,"SELECT Txt FROM assignments WHERE AsgCod='%ld' AND CrsCod='%ld'",
            AsgCod,Gbl.CurrentCrs.Crs.CrsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get assignment text");

   /***** The result of the query must have one row or none *****/
   if (NumRows == 1)
     {
      /* Get info text */
      row = mysql_fetch_row (mysql_res);
      strcpy (Txt,row[0]);
     }
   else
      Txt[0] = '\0';

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   if (NumRows > 1)
      Lay_ShowErrorAndExit ("Error when getting assignment text.");
  }

/****************************************************************************/
/***************** Write parameter with code of assignment ******************/
/****************************************************************************/

static void Asg_PutParamAsgCod (long AsgCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AsgCod\" value=\"%ld\" />",AsgCod);
  }

/****************************************************************************/
/****************** Get parameter with code of assignment *******************/
/****************************************************************************/

long Asg_GetParamAsgCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of assignment *****/
   Par_GetParam (Par_PARAM_SINGLE,"AsgCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/*****************************************************************************/
/************ Ask for confirmation of removing of an assignment **************/
/*****************************************************************************/

void Asg_AskRemAssignment (void)
  {
   extern const char *Txt_Do_you_really_want_to_remove_the_assignment_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_assignment[Txt_NUM_LANGUAGES];
   struct Assignment Asg;

   /***** Get parameters *****/ 
   Asg_GetParamAsgOrderType ();
   Grp_GetParamWhichGrps ();
   Pag_GetParamPagNum (Pag_ASSIGNMENTS);

   /***** Get assignment code *****/
   if ((Asg.AsgCod = Asg_GetParamAsgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of assignment is missing.");

   /***** Get data of the assignment from database *****/
   Asg_GetDataOfAssignmentByCod (&Asg);

   /***** Button of confirmation of removing *****/
   Act_FormStart (ActRemAsg);
   Asg_PutParamAsgCod (Asg.AsgCod);
   Asg_PutHiddenParamAsgOrderType ();
   Grp_PutParamWhichGrps ();
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);

   /***** Ask for confirmation of removing *****/
   sprintf (Gbl.Message,Txt_Do_you_really_want_to_remove_the_assignment_X[Gbl.Prefs.Language],
            Asg.Title);
   Lay_ShowAlert (WARNING,Gbl.Message);
   fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"%s\" /></div></form>\n",
            Txt_Remove_assignment[Gbl.Prefs.Language]);

   /***** Show assignments again *****/
   Asg_SeeAssignments ();
  }

/****************************************************************************/
/*************************** Remove an assignment ***************************/
/****************************************************************************/

void Asg_RemoveAssignment (void)
  {
   extern const char *Txt_Assignment_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Assignment Asg;

   /***** Get assignment code *****/
   if ((Asg.AsgCod = Asg_GetParamAsgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of assignment is missing.");

   /***** Get data of the assignment from database *****/
   Asg_GetDataOfAssignmentByCod (&Asg);

   /***** Remove all the folders associated to this assignment *****/
   if (Asg.Folder[0])
      Brw_RemoveFoldersAssigmentsIfExistForAllUsrs (Asg.Folder);

   /***** Remove all the groups of this assignment *****/
   Asg_RemoveAllTheGrpsAssociatedToAndAssignment (Asg.AsgCod);

   /***** Remove assignment *****/
   sprintf (Query,"DELETE FROM assignments WHERE AsgCod='%ld' AND CrsCod='%ld'",
            Asg.AsgCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryDELETE (Query,"can not remove assignment");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Assignment_X_removed[Gbl.Prefs.Language],
            Asg.Title);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show assignments again *****/
   Asg_SeeAssignments ();
  }

/****************************************************************************/
/******** Check if the title or the folder of an assignment exists **********/
/****************************************************************************/

static bool Asg_CheckIfSimilarAssignmentExists (const char *Field,const char *Value,long AsgCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of assignments with a field value from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM assignments WHERE CrsCod='%ld' AND %s='%s' AND AsgCod<>'%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,Field,Value,AsgCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get similar assignments");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when searching for similar assignments.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/****************** Put a form to create a new assignment *******************/
/****************************************************************************/

void Asg_RequestCreatOrEditAsg (void)
  {
   extern const char *Txt_New_assignment[Txt_NUM_LANGUAGES];
   extern const char *Txt_Edit_assignment[Txt_NUM_LANGUAGES];
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Start_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_End_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Title[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_files[Txt_NUM_LANGUAGES];
   extern const char *Txt_Folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_Description[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_assignment[Txt_NUM_LANGUAGES];
   extern const char *Txt_Modify_assignment[Txt_NUM_LANGUAGES];
   struct Assignment Asg;
   bool ItsANewAssignment;
   Asg_StartOrEndTime_t StartOrEndTime;
   const char *NameSelectYear  [Asg_NUM_DATES] = {"StartYear"  ,"EndYear"  };
   const char *NameSelectMonth [Asg_NUM_DATES] = {"StartMonth" ,"EndMonth" };
   const char *NameSelectDay   [Asg_NUM_DATES] = {"StartDay"   ,"EndDay"   };
   const char *NameSelectHour  [Asg_NUM_DATES] = {"StartHour"  ,"EndHour"  };
   const char *NameSelectMinute[Asg_NUM_DATES] = {"StartMinute","EndMinute"};
   const char **Txt_Dates[Asg_NUM_DATES] = {Txt_Start_date,Txt_End_date};

   /***** Get parameters *****/ 
   Asg_GetParamAsgOrderType ();
   Grp_GetParamWhichGrps ();
   Pag_GetParamPagNum (Pag_ASSIGNMENTS);

   /***** Get the code of the assignment *****/
   ItsANewAssignment = ((Asg.AsgCod = Asg_GetParamAsgCod ()) == -1L);

   /***** Get from the database the data of the assignment *****/
   if (ItsANewAssignment)
     {
      /* Initialize to empty assignment */
      Asg.AsgCod = -1L;
      Asg.DateTimes[Asg_START_TIME].Day    = Gbl.Now.Day;
      Asg.DateTimes[Asg_START_TIME].Month  = Gbl.Now.Month;
      Asg.DateTimes[Asg_START_TIME].Year   = Gbl.Now.Year;
      Asg.DateTimes[Asg_START_TIME].Hour   = Gbl.Now.Hour;
      Asg.DateTimes[Asg_START_TIME].Minute = Gbl.Now.Minute;
      Asg.DateTimes[Asg_START_TIME].Second = Gbl.Now.Second;
      Asg.DateTimes[Asg_END_TIME  ].Day    = Gbl.Now.Day;
      Asg.DateTimes[Asg_END_TIME  ].Month  = Gbl.Now.Month;
      Asg.DateTimes[Asg_END_TIME  ].Year   = Gbl.Now.Year;
      Asg.DateTimes[Asg_END_TIME  ].Hour   = 23;
      Asg.DateTimes[Asg_END_TIME  ].Minute = 59;
      Asg.DateTimes[Asg_END_TIME  ].Second = 59;
      Asg.Open = true;
      Asg.Title[0] = '\0';
      Asg.SendWork = false;
      Asg.Folder[0] = '\0';
      Asg.ICanDo = false;
     }
   else
     { 
      /* Get data of the assignment from database */
      Asg_GetDataOfAssignmentByCod (&Asg);

      /* Allocate memory for assignment text */
      if (!Str_AllocateText ())
         Lay_ShowErrorAndExit (Gbl.Message);

      /* Get text of the assignment from database */
      Asg_GetAssignmentTxtFromDB (Asg.AsgCod,Gbl.Txt);
     }

   /***** Form start *****/
   if (ItsANewAssignment)
      Act_FormStart (ActNewAsg);
   else
     {
      Act_FormStart (ActChgAsg);
      Asg_PutParamAsgCod (Asg.AsgCod);
     }
   Asg_PutHiddenParamAsgOrderType ();
   Grp_PutParamWhichGrps ();
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"DAT\">");
   Lay_WriteTitle (ItsANewAssignment ? Txt_New_assignment[Gbl.Prefs.Language] :
                                     Txt_Edit_assignment[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<br /><table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Assignment title *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\">" \
                      "<input type=\"text\" name=\"Title\" size=\"80\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td></tr>",
            Txt_Title[Gbl.Prefs.Language],
            Asg_MAX_LENGTH_ACTIVITY_TITLE,Asg.Title);

   /***** Assignment start and end dates *****/
   for (StartOrEndTime=Asg_START_TIME; StartOrEndTime<=Asg_END_TIME; StartOrEndTime++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s:</td>" \
                         "<td align=\"left\" valign=\"top\">" \
                         "<table cellspacing=\"0\" cellpadding=\"0\">" \
                         "<tr>" \
                         "<td align=\"left\" valign=\"top\">",
               Txt_Dates[StartOrEndTime][Gbl.Prefs.Language]);
      Dat_WriteFormDate (Gbl.Today.Year-1,Gbl.Today.Year+1,
                         NameSelectDay  [StartOrEndTime],
                         NameSelectMonth[StartOrEndTime],
                         NameSelectYear [StartOrEndTime],
                         Asg.DateTimes[StartOrEndTime].Day,
                         Asg.DateTimes[StartOrEndTime].Month,
                         Asg.DateTimes[StartOrEndTime].Year,
                         false,false);
      fprintf (Gbl.F.Out,"</td>" \
                         "<td align=\"left\" valign=\"top\">");
      Dat_WriteFormHourMinute (NameSelectHour  [StartOrEndTime],
                               NameSelectMinute[StartOrEndTime],
		               Asg.DateTimes[StartOrEndTime].Hour,Asg.DateTimes[StartOrEndTime].Minute,
                               false,false);
      fprintf (Gbl.F.Out,"</td></tr></table></td></tr>");
     }

   /***** Send work? *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"middle\" class=\"TIT_TBL\">%s:</td>" \
                      "<td align=\"left\" valign=\"middle\" class=\"TIT_TBL\">" \
                      "%s: " \
                      "<input type=\"text\" name=\"Folder\" size=\"%u\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td></tr>",
            Txt_Upload_files[Gbl.Prefs.Language],
            Txt_Folder[Gbl.Prefs.Language],
            Asg_MAX_LENGTH_FOLDER,Asg_MAX_LENGTH_FOLDER,Asg.Folder);

   /***** Assignment text *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\">" \
                      "<textarea name=\"Txt\" cols=\"60\" rows=\"10\">",
            Txt_Description[Gbl.Prefs.Language]);
   if (!ItsANewAssignment)
      fprintf (Gbl.F.Out,"%s",Gbl.Txt);
   fprintf (Gbl.F.Out,"</textarea>" \
                      "</td></tr>");
   
   if (!ItsANewAssignment)
      /* Free memory for assignment text */
      Str_FreeTxt ();

   /***** Groups *****/
   Asg_ShowLstGrpsToEditAssignment (Asg.AsgCod);

   /***** Table end *****/
   fprintf (Gbl.F.Out,"</table></td></tr>\n");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** New assignment *****/
   Lay_PutSendButton (ItsANewAssignment ? Txt_Create_assignment[Gbl.Prefs.Language] :
                                          Txt_Modify_assignment[Gbl.Prefs.Language]);

   /***** Form end *****/
   fprintf (Gbl.F.Out,"</form><br />\n");

   /***** Show current assignments *****/
   Asg_ShowAllAssignments ();
  }

/****************************************************************************/
/**************** Show list of groups to edit and assignment ****************/
/****************************************************************************/

static void Asg_ShowLstGrpsToEditAssignment (long AsgCod)
  {
   extern const char *Txt_Groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_whole_course[Txt_NUM_LANGUAGES];
   unsigned NumGrpTyp;

   /***** Get list of groups types and groups in this course *****/
   Grp_GetListGrpTypesAndGrpsInThisCrs ();

   if (Gbl.CurrentCrs.Grps.GrpTypes.Num)
     {
      /***** Start table *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s:</td>" \
                         "<td align=\"left\" valign=\"top\">",
               Txt_Groups[Gbl.Prefs.Language]);
      Lay_StartRoundFrameTable10 ("frame_white_corner","frame_white",WHITE,"100%%");

      /***** First row: checkbox to select the whole course *****/
      fprintf (Gbl.F.Out,"<tr><td colspan=\"7\" align=\"left\" valign=\"middle\" class=\"DAT\">" \
                         "<input type=\"checkbox\" id=\"WholeCrs\" name=\"WholeCrs\" value=\"Y\"");
      if (!Asg_CheckIfAsgIsAssociatedToGrps (AsgCod))
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," onclick=\"uncheckChildren(this,'GrpCods')\" />%s %s</td></tr>",
               Txt_The_whole_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.ShortName);

      /***** List the groups for each group type *****/
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
         if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps)
            Grp_ListGrpsToEditAssignments (&Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp],AsgCod);

      /***** End table *****/
      Lay_EndRoundFrameTable10 ("frame_white_corner","frame_white");
      fprintf (Gbl.F.Out,"</td></tr>");
     }

   /***** Free list of groups types and groups in this course *****/
   Grp_FreeListGrpTypesAndGrps ();
  }

/****************************************************************************/
/****************** Receive form to create a new assignment *****************/
/****************************************************************************/

void Asg_RecFormAssignment (void)
  {
   extern const char *Txt_Already_existed_an_assignment_with_the_title_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Already_existed_an_assignment_with_the_folder_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_You_must_specify_the_title_of_the_assignment[Txt_NUM_LANGUAGES];
   extern const char *Txt_You_cant_disable_file_uploading_once_folders_have_been_created[Txt_NUM_LANGUAGES];
   struct Assignment OldAsg,NewAsg;
   bool ItsANewAssignment;
   bool NewAssignmentIsCorrect = true;

   /***** Get the code of the assignment *****/
   ItsANewAssignment = ((NewAsg.AsgCod = Asg_GetParamAsgCod ()) == -1L);

   if (!ItsANewAssignment)
     {
      /* Get data of the old (current) assignment from database */
      OldAsg.AsgCod = NewAsg.AsgCod;
      Asg_GetDataOfAssignmentByCod (&OldAsg);
     }

   /***** Get start date *****/
   Dat_GetDateFromForm ("StartDay","StartMonth","StartYear",
                        &(NewAsg.DateTimes[Asg_START_TIME].Day),&(NewAsg.DateTimes[Asg_START_TIME].Month),&(NewAsg.DateTimes[Asg_START_TIME].Year));
   Dat_GetHourMinuteFromForm ("StartHour","StartMinute",
                              &(NewAsg.DateTimes[Asg_START_TIME].Hour),&(NewAsg.DateTimes[Asg_START_TIME].Minute));

   /***** Get end date *****/
   Dat_GetDateFromForm ("EndDay","EndMonth","EndYear",
                        &(NewAsg.DateTimes[Asg_END_TIME].Day),&(NewAsg.DateTimes[Asg_END_TIME].Month),&(NewAsg.DateTimes[Asg_END_TIME].Year));
   Dat_GetHourMinuteFromForm ("EndHour","EndMinute",
                              &(NewAsg.DateTimes[Asg_END_TIME].Hour),&(NewAsg.DateTimes[Asg_END_TIME].Minute));

   /***** Get assignment title *****/
   Par_GetParam (Par_PARAM_SINGLE,"Title",NewAsg.Title,Asg_MAX_LENGTH_ACTIVITY_TITLE);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewAsg.Title,Asg_MAX_LENGTH_ACTIVITY_TITLE,true);

   /***** Get folder name where to send works of the assignment *****/
   Par_GetParam (Par_PARAM_SINGLE,"Folder",NewAsg.Folder,Asg_MAX_LENGTH_FOLDER);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewAsg.Folder,Asg_MAX_LENGTH_FOLDER,true);
   NewAsg.SendWork = (NewAsg.Folder[0]) ? Asg_SEND_WORK : Asg_DO_NOT_SEND_WORK;

   /***** Get assignment text and insert links *****/
   Str_AllocateAndGetText (Str_TO_HTML,true);	// Store in HTML format (not rigorous)

   /***** Adjust dates *****/
   if (NewAsg.DateTimes[Asg_START_TIME].Day   == 0 ||
       NewAsg.DateTimes[Asg_START_TIME].Month == 0 ||
       NewAsg.DateTimes[Asg_START_TIME].Year  == 0)
     {
      NewAsg.DateTimes[Asg_START_TIME].Day    = Gbl.Now.Day;
      NewAsg.DateTimes[Asg_START_TIME].Month  = Gbl.Now.Month;
      NewAsg.DateTimes[Asg_START_TIME].Year   = Gbl.Now.Year;
      NewAsg.DateTimes[Asg_START_TIME].Hour   = Gbl.Now.Hour;
      NewAsg.DateTimes[Asg_START_TIME].Minute = Gbl.Now.Minute;
      NewAsg.DateTimes[Asg_START_TIME].Second = Gbl.Now.Second;
     }
   if (NewAsg.DateTimes[Asg_END_TIME].Day   == 0 ||
       NewAsg.DateTimes[Asg_END_TIME].Month == 0 ||
       NewAsg.DateTimes[Asg_END_TIME].Year  == 0)
     {
      NewAsg.DateTimes[Asg_END_TIME].Day    = Gbl.Now.Day;
      NewAsg.DateTimes[Asg_END_TIME].Month  = Gbl.Now.Month;
      NewAsg.DateTimes[Asg_END_TIME].Year   = Gbl.Now.Year;
      NewAsg.DateTimes[Asg_END_TIME].Hour   = 23;
      NewAsg.DateTimes[Asg_END_TIME].Minute = 59;
     }
   NewAsg.DateTimes[Asg_END_TIME].Second = 59;

   /***** Check if title is correct *****/
   if (NewAsg.Title[0])	// If there's an assignment title
     {
      /* If title of assignment was in database... */
      if (Asg_CheckIfSimilarAssignmentExists ("Title",NewAsg.Title,NewAsg.AsgCod))
        {
         NewAssignmentIsCorrect = false;
         sprintf (Gbl.Message,Txt_Already_existed_an_assignment_with_the_title_X[Gbl.Prefs.Language],
                  NewAsg.Title);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else	// Title is correct
        {
         if (NewAsg.SendWork == Asg_SEND_WORK)
           {
            if (Brw_CheckIfFileOrFolderNameIsValid (NewAsg.Folder))	// If folder name is valid...
              {
               if (Asg_CheckIfSimilarAssignmentExists ("Folder",NewAsg.Folder,NewAsg.AsgCod))	// If folder of assignment was in database...
                 {
                  NewAssignmentIsCorrect = false;
                  sprintf (Gbl.Message,Txt_Already_existed_an_assignment_with_the_folder_X[Gbl.Prefs.Language],
                           NewAsg.Folder);
                  Lay_ShowAlert (WARNING,Gbl.Message);
                 }
              }
            else	// Folder name not valid
              {
               NewAssignmentIsCorrect = false;
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
           }
         else	// NewAsg.SendWork == Asg_DO_NOT_SEND_WORK
           {
            if (OldAsg.SendWork == Asg_SEND_WORK)
              {
               if (Brw_CheckIfExistsFolderAssigmentForAnyUsr (OldAsg.Folder))
                 {
                  NewAssignmentIsCorrect = false;
                  Lay_ShowAlert (WARNING,Txt_You_cant_disable_file_uploading_once_folders_have_been_created[Gbl.Prefs.Language]);
                 }
              }
           }
        } 
     }
   else	// If there isn't an assignment title
     {
      NewAssignmentIsCorrect = false;
      Lay_ShowAlert (WARNING,Txt_You_must_specify_the_title_of_the_assignment[Gbl.Prefs.Language]);
     }

   /***** Create a new assignment or update an existing one *****/
   if (NewAssignmentIsCorrect)
     {
      /* Get groups for this assignments */
      Grp_GetParCodsSeveralGrpsToEditAssignments ();

      if (ItsANewAssignment)
         Asg_CreateAssignment (&NewAsg);	// Add new assignment to database
      else
        {
         if (OldAsg.Folder[0] && NewAsg.Folder[0])
            if (strcmp (OldAsg.Folder,NewAsg.Folder))	// Folder name has changed
               NewAssignmentIsCorrect = Brw_UpdateFoldersAssigmentsIfExistForAllUsrs (OldAsg.Folder,NewAsg.Folder);
         if (NewAssignmentIsCorrect)
            Asg_UpdateAssignment (&NewAsg);
        }

      /* Free memory for list of selected groups */
      Grp_FreeListCodSelectedGrps ();
     }
   else
      Asg_RequestCreatOrEditAsg ();

   /***** Free assignment text *****/
   Str_FreeTxt ();

   /***** Show assignments again *****/
   Asg_SeeAssignments ();
  }

/****************************************************************************/
/************************ Create a new assignment ***************************/
/****************************************************************************/

static void Asg_CreateAssignment (struct Assignment *Asg)
  {
   extern const char *Txt_Created_new_assignment_X[Txt_NUM_LANGUAGES];
   char Query[1024+Cte_MAX_BYTES_TEXT];

   /***** Create a new assignment *****/
   sprintf (Query,"INSERT INTO assignments (CrsCod,UsrCod,StartTime,EndTime,Title,Folder,Txt)" \
                  " VALUES ('%ld','%ld','%04u%02u%02u%02u%02u%02u','%04u%02u%02u%02u%02u%02u','%s','%s','%s')",
            Gbl.CurrentCrs.Crs.CrsCod,
            Gbl.Usrs.Me.UsrDat.UsrCod,
            Asg->DateTimes[Asg_START_TIME].Year,Asg->DateTimes[Asg_START_TIME].Month,Asg->DateTimes[Asg_START_TIME].Day,
            Asg->DateTimes[Asg_START_TIME].Hour,Asg->DateTimes[Asg_START_TIME].Minute,Asg->DateTimes[Asg_START_TIME].Second,
            Asg->DateTimes[Asg_END_TIME].Year,Asg->DateTimes[Asg_END_TIME].Month,Asg->DateTimes[Asg_END_TIME].Day,
            Asg->DateTimes[Asg_END_TIME].Hour,Asg->DateTimes[Asg_END_TIME].Minute,Asg->DateTimes[Asg_END_TIME].Second,
            Asg->Title,
            Asg->Folder,
            Gbl.Txt);
   Asg->AsgCod = DB_QueryINSERTandReturnCode (Query,"can not create new assignment");

   /***** Create groups *****/
   if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)
      Asg_CreateGrps (Asg->AsgCod);

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_assignment_X[Gbl.Prefs.Language],
            Asg->Title);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/********************* Update an existing assignment ************************/
/****************************************************************************/

static void Asg_UpdateAssignment (struct Assignment *Asg)
  {
   extern const char *Txt_The_assignment_has_been_modified[Txt_NUM_LANGUAGES];
   char Query[1024+Cte_MAX_BYTES_TEXT];

   /***** Update the data of the assignment *****/
   sprintf (Query,"UPDATE assignments SET StartTime='%04u%02u%02u%02u%02u%02u',EndTime='%04u%02u%02u%02u%02u%02u',Title='%s',Folder='%s',Txt='%s'" \
                  " WHERE AsgCod='%ld' AND CrsCod='%ld'",
            Asg->DateTimes[Asg_START_TIME].Year,Asg->DateTimes[Asg_START_TIME].Month ,Asg->DateTimes[Asg_START_TIME].Day   ,
            Asg->DateTimes[Asg_START_TIME].Hour,Asg->DateTimes[Asg_START_TIME].Minute,Asg->DateTimes[Asg_START_TIME].Second,
            Asg->DateTimes[Asg_END_TIME  ].Year,Asg->DateTimes[Asg_END_TIME  ].Month ,Asg->DateTimes[Asg_END_TIME  ].Day   ,
            Asg->DateTimes[Asg_END_TIME  ].Hour,Asg->DateTimes[Asg_END_TIME  ].Minute,Asg->DateTimes[Asg_END_TIME  ].Second,
            Asg->Title,
            Asg->Folder,
            Gbl.Txt,
            Asg->AsgCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryUPDATE (Query,"can not update assignment");

   /***** Update groups *****/
   /* Remove old groups */
   Asg_RemoveAllTheGrpsAssociatedToAndAssignment (Asg->AsgCod);

   /* Create new groups */
   if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)
      Asg_CreateGrps (Asg->AsgCod);

   /***** Write success message *****/
   Lay_ShowAlert (INFO,Txt_The_assignment_has_been_modified[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/*********** Check if an assignment is associated to any group **************/
/****************************************************************************/

static bool Asg_CheckIfAsgIsAssociatedToGrps (long AsgCod)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if an assignment is associated to a group from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM asg_grp WHERE AsgCod='%ld'",
            AsgCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if an assignment is associated to groups");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("can not check if an assignment is associated to groups");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/************ Check if an assignment is associated to a group ***************/
/****************************************************************************/

bool Asg_CheckIfAsgIsAssociatedToGrp (long AsgCod,long GrpCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if an assignment is associated to a group from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM asg_grp WHERE AsgCod='%ld' AND GrpCod='%ld'",
            AsgCod,GrpCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if an assignment is associated to a group");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("can not check if an assignment is associated to a group");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************* Remove groups of an assignment ***********************/
/****************************************************************************/

static void Asg_RemoveAllTheGrpsAssociatedToAndAssignment (long AsgCod)
  {
   char Query[256];

   /***** Remove groups of the assignment *****/
   sprintf (Query,"DELETE FROM asg_grp WHERE AsgCod='%ld'",AsgCod);
   DB_QueryDELETE (Query,"can not remove the groups associated to an assignment");
  }

/****************************************************************************/
/**************** Remove one group from all the assignments *****************/
/****************************************************************************/

void Asg_RemoveGroup (long GrpCod)
  {
   char Query[256];

   /***** Remove group from all the assignments *****/
   sprintf (Query,"DELETE FROM asg_grp WHERE GrpCod='%ld'",GrpCod);
   DB_QueryDELETE (Query,"can not remove group from the associations between assignments and groups");
  }

/****************************************************************************/
/**************** Remove one group from all the assignments *****************/
/****************************************************************************/

void Asg_RemoveGroupsOfType (long GrpTypCod)
  {
   char Query[256];

   /***** Remove group from all the assignments *****/
   sprintf (Query,"DELETE FROM asg_grp USING crs_grp,asg_grp" \
                  " WHERE crs_grp.GrpTypCod='%ld' AND crs_grp.GrpCod=asg_grp.GrpCod",
            GrpTypCod);
   DB_QueryDELETE (Query,"can not remove groups of a type from the associations between assignments and groups");
  }

/****************************************************************************/
/********************* Create groups of an assignment ***********************/
/****************************************************************************/

static void Asg_CreateGrps (long AsgCod)
  {
   unsigned NumGrpSel;
   char Query[256];

   /***** Create groups of the assignment *****/
   for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
     {
      /* Create group */
      sprintf (Query,"INSERT INTO asg_grp (AsgCod,GrpCod) VALUES ('%ld','%ld')",
               AsgCod,Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel]);
      DB_QueryINSERT (Query,"can not associate a group to an assignment");
     }
  }

/*****************************************************************************/
/********* Get and write the names of the groups of an assignment ************/
/*****************************************************************************/

static void Asg_GetAndWriteNamesOfGrpsAssociatedToAsg (long AsgCod)
  {
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_and[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_whole_course[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow;
   unsigned long NumRows;

   /***** Get groups associated to an assignment from database *****/
   sprintf (Query,"SELECT crs_grp_types.GrpTypName,crs_grp.GrpName FROM asg_grp,crs_grp,crs_grp_types" \
                  " WHERE asg_grp.AsgCod='%ld' AND asg_grp.GrpCod=crs_grp.GrpCod AND crs_grp.GrpTypCod=crs_grp_types.GrpTypCod" \
                  " ORDER BY crs_grp_types.GrpTypName,crs_grp.GrpName",
            AsgCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get groups of an assignment");

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<p class=\"TIT_TBL\">%s: ",
            (NumRows == 1) ?
            Txt_Group [Gbl.Prefs.Language] :
            Txt_Groups[Gbl.Prefs.Language]);

   /***** Write groups *****/
   if (NumRows) // Groups found...
     {
      /* Get and write the group types and names */
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         /* Get next group */
         row = mysql_fetch_row (mysql_res);

         /* Write group type name and group name */
         fprintf (Gbl.F.Out,"%s %s",row[0],row[1]);

         if (NumRows >= 2)
           {
            if (NumRow == NumRows-2)
               fprintf (Gbl.F.Out," %s ",Txt_and[Gbl.Prefs.Language]);
            if (NumRows >= 3)
              if (NumRow < NumRows-2)
                  fprintf (Gbl.F.Out,", ");
           }
        }
     }
   else
      fprintf (Gbl.F.Out,"%s %s",
               Txt_The_whole_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.ShortName);

   fprintf (Gbl.F.Out,"</p>");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/****************** Remove all the assignments of a course ******************/
/****************************************************************************/

void Asg_RemoveCrsAssignments (long CrsCod)
  {
   char Query[512];

   /***** Remove groups *****/
   sprintf (Query,"DELETE FROM asg_grp USING assignments,asg_grp" \
                  " WHERE assignments.CrsCod='%ld' AND assignments.AsgCod=asg_grp.AsgCod",
            CrsCod);
   DB_QueryDELETE (Query,"can not remove all the groups associated to assignments of a course");

   /***** Remove assignments *****/
   sprintf (Query,"DELETE FROM assignments WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove all the assignments of a course");
  }

/****************************************************************************/
/********* Check if I belong to any of the groups of an assignment **********/
/****************************************************************************/

static bool Asg_CheckIfICanDoThisAssignment (long AsgCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if I can do an assignment from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM assignments" \
                  " WHERE AsgCod='%ld'" \
                  " AND (AsgCod NOT IN (SELECT AsgCod FROM asg_grp) OR"
                  " AsgCod IN (SELECT asg_grp.AsgCod FROM asg_grp,crs_grp_usr" \
                  " WHERE crs_grp_usr.UsrCod='%ld' AND asg_grp.GrpCod=crs_grp_usr.GrpCod))",
            AsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if I can do an assignment");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Internal error while checking if I can do an assigment.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/******************* Get number of assigments in a course *******************/
/****************************************************************************/

unsigned Asg_GetNumAssigmentsInCrs (long CrsCod)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of assignments in a course from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM assignments WHERE CrsCod='%ld'",
            CrsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of assigments in course");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Internal error while getting number of assigments in course.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumRows;
  }
