// swad_brw.h: file browsers

#ifndef _SWAD_EXP
#define _SWAD_EXP
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_grp.h"

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

#define Brw_NUM_TYPES_FILE_BROWSER 16
// The following types are stored in clipboard, expanded_folders, file_browser_size tables as numeric fields, so don't change numbers!
typedef enum
  {
   Brw_FILE_BRW_UNKNOWN            =  0,
   Brw_FILE_BRW_SEE_DOWNLOAD_CRS   =  1,
   Brw_FILE_BRW_SEE_MARKS_CRS      =  2,
   Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS =  3,
   Brw_FILE_BRW_COMMON_CRS         =  4,
   Brw_FILE_BRW_COMMON_GRP         =  5,
   Brw_FILE_BRW_WORKS_USR          =  6,
   Brw_FILE_BRW_WORKS_CRS          =  7,
   Brw_FILE_BRW_ADMIN_MARKS_CRS    =  8,
   Brw_FILE_BRW_BRIEFCASE_USR      =  9,
   Brw_FILE_BRW_SEE_DOWNLOAD_GRP   = 10,
   Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP = 11,
   Brw_FILE_BRW_SEE_MARKS_GRP      = 12,
   Brw_FILE_BRW_ADMIN_MARKS_GRP    = 13,
   Brw_FILE_BRW_ASSIGNMENTS_USR    = 14,
   Brw_FILE_BRW_ASSIGNMENTS_CRS    = 15,
  } Brw_FileBrowser_t;

/****************************************************************************/
/****************************** Public constants ****************************/
/****************************************************************************/

#define Brw_MAX_DIR_LEVELS	10	// Maximum number of subdirectory levels in file browsers

#define Brw_MAX_BYTES_MIME_TYPE	256	// Maximum length of "image/jpeg", "text/html", etc.

#define Brw_NAME_OF_PARAM_FILENAME_ORG "FilenameOrg"

#define Brw_INTERNAL_NAME_ROOT_FOLDER_DOWNLOAD		"descarga"
#define Brw_INTERNAL_NAME_ROOT_FOLDER_COMMON		"comun"
#define Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS	"actividades"
#define Brw_INTERNAL_NAME_ROOT_FOLDER_WORKS		"trabajos"
#define Brw_INTERNAL_NAME_ROOT_FOLDER_MARKS		"calificaciones"
#define Brw_INTERNAL_NAME_ROOT_FOLDER_BRIEFCASE		"maletin"

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Brw_PutParamsPathAndFile (const char *PathInTree,const char *FileFolderName);
bool Brw_CheckIfExistsFolderAssigmentForAnyUsr (const char *FolderName);
bool Brw_UpdateFoldersAssigmentsIfExistForAllUsrs (const char *OldFolderName,const char *NewFolderName);
void Brw_RemoveFoldersAssigmentsIfExistForAllUsrs (const char *FolderName);
void Brw_ShowFileBrowserOrWorks (void);
void Brw_RemoveSizeOfFileTreeFromDB (long CrsCod,long GrpCod,long UsrCod);
void Brw_AskEditWorksCrs (void);
void Brw_AskRemFileFromTree (void);
void Brw_RemFileFromTree (void);
void Brw_RemFolderFromTree (void);
void Brw_ExpandFileTree (void);
void Brw_ContractFileTree (void);
void Brw_CopyFromFileBrowser (void);
void Brw_PasteIntoFileBrowser (void);
void Brw_RemSubtreeInFileBrowser (void);
void Brw_ShowFormFileBrowser (void);
void Brw_RecFolderFileBrowser (void);
void Brw_RenFolderFileBrowser (void);
void Brw_ReceiveFileInFileBrowser (void);
void Brw_ShowDownload (void);
void Brw_HideDownload (void);

void Brw_ParamSeeAssZoneCrs (void);
void Brw_ParamSeeAssZoneGrp (void);
void Brw_ParamAdmAssZoneCrs (void);
void Brw_ParamAdmAssZoneGrp (void);
void Brw_ParamAssignmentsZoneUsr (void);
void Brw_ParamWorksZoneUsr (void);
void Brw_ParamAssignmentsZoneCrs (void);
void Brw_ParamWorksZoneCrs (void);
void Brw_ParamBriefZoneUsr (void);

void Brw_ChangeNumRowsHeader (void);
void Brw_ChangeNumRowsFooter (void);
void Brw_RemoveUsrClipboard (long UsrCod);
void Brw_RemoveGrpClipboards (long GrpCod);
void Brw_RemoveUsrClipboardInCrs (long UsrCod,long CrsCod);
void Brw_RemoveUsrGrpClipboard (long UsrCod,long GrpCod);
void Brw_RemoveUsrExpandedFolders (long UsrCod);
void Brw_RemoveUsrExpandedFoldersInCrs (long UsrCod,long CrsCod);

void Brw_RemoveOldTmpFiles (const char *Path,bool RemoveDirectory);
void Brw_RemoveTree (const char *PathRelFromCGI_BIN);
void Brw_CalcSizeOfDir (char *PathRel);

bool Brw_CheckIfFileOrFolderNameIsValid (const char *FileName);

void Brw_RemoveZonesOfGroupsOfType (long GrpTypCod);
void Brw_RemoveGrpZonesVerbose (struct GroupData *GrpDat);
void Brw_RemoveGrpZones (long CrsCod,long GrpCod);

void Brw_RemoveUsrWorksInCrs (struct UsrData *UsrDat,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose);
void Brw_RemoveUsrWorksInAllCrss (struct UsrData *UsrDat,Cte_QuietOrVerbose_t QuietOrVerbose);

bool Brw_CheckMarksFile (const char *Path,unsigned *NumRowsHead,unsigned *NumRowsFoot);
void Brw_ShowMyMarks (void);

#endif
