// swad_cfg.h: configuration

#ifndef _SWAD_CFG
#define _SWAD_CFG
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/****** Constants that should be hidden when publishing source code! ********/
/****************************************************************************/

#define DONT_PUBLISH_DATABASE_PASSWORD		""

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

/* Platform name and server */
#define Cfg_PLATFORM_SHORT_NAME		"SWAD"
#define Cfg_PLATFORM_FULL_NAME		"SWAD (Sistema Web de Apoyo a la Docencia)"
#define Cfg_PLATFORM_PAGE_TITLE		"Plataforma SWAD"
#define Cfg_PLATFORM_URL		"swad.ugr.es"
#define Cfg_PLATFORM_SERVER		"swad.ugr.es"

/* Database */
#define Cfg_DATABASE_HOST		"swad.ugr.es"			// Database host
#define Cfg_DATABASE_DBNAME		"swad"				// Database name
#define Cfg_DATABASE_USER		"swad"				// Database user
#define Cfg_DATABASE_PASSWORD		DONT_PUBLISH_DATABASE_PASSWORD	// Database password

/* Footer of page whit info about the institution where this platform is installed */
#define Cfg_ABOUT_INSTITUTION_NAME	"Universidad de Granada"
#define Cfg_ABOUT_INSTITUTION_URL	"http://www.ugr.es/"
#define Cfg_ABOUT_INSTITUTION_LOGO	"ugr100x32.png"

#define Cfg_ABOUT_CENTRE_NAME		"Centro de Ense&ntilde;anzas Virtuales"
#define Cfg_ABOUT_CENTRE_URL		"http://cevug.ugr.es/"
#define Cfg_ABOUT_CENTRE_LOGO		"cevug100x32.png"

#define Cfg_ABOUT_LOGO_WIDTH		100
#define Cfg_ABOUT_LOGO_HEIGHT		32

/* Link to download Java: keep updated! */
#define Cfg_JAVA_URL			"http://www.java.com/es/download/"
#define Cfg_JAVA_NAME			"Java Runtime Environment"

/* Users */
#define Cfg_MAX_USRS_IN_LIST				5000	// If the number of users in a list is greater than this, don't show the list
#define Cfg_MIN_PHOTOS_TO_COMPUTE_AVERAGE		  10	// If the number of students with photo in a degree is less than this, don't show average photo of the degree

/****************************************************************************/
/*********************** Directories, folder and files **********************/
/****************************************************************************/

/*                        /
                          |
                          +
                          |
                         var
                          |
                          +
                          |
                         www
                          |
   +----------------------+--------------------------------------------------+
   |                      |                                                  |
 cgi-bin                 html (public)                                     swad (private)
   |                      |                                                  |
   |         +---+----+---+-------------+------------------+---+        +----+-----+----------+--------+---------+
   |         |   |    |                 |                  |   |        |          |          |        |         |
 swad (CGI) crs chat photo             icon               tmp logo     crs       photo       out      tmp       usr
             |        |                 |                  |            |                              |         |
             |  +-----+---+   +-----+---+--+------+        +            |                            +-+-+   +-+-+---+
             |  |         |   |     |   |  |      |        |            |                            | | |   | | |...|
             | links     tmp  |  filext | plugin  |    tmp.dirs.        |                          tmp.dirs. 0 1 2   9
             |            |  degree institution theme      |            |                              |     | | |...|
             |            +                                +            |                            +-+-+         +-+-+
             |          processing                       links          |                            | | |         | | |
             |          of                               to             |                           tmp. links      xxxxxxx9
             |          received                         download       |                           to                 |
             |          photos                           files          |                           assignments        +
             |                                                          |                           and                |
             |                                                          |                           works           maletin
           +-+-+                                                      +-+-+                         for
           | | |...                                                   | | |                         compression
          crs.code                                                  crs.code
             |                                                          |
        +----+----+              +----------+------+-------------+------+---+-----------+------------+
        |...                     |          |      |             |          |           |            |
   bibliografia            calificaciones comun descarga        grp        pra         lec          usr
                               +-+-+      +-+-+  +-+-+       +---+---+      +           +          +-+--------+
                               | | |      | | |  | | |       |       |      |           |          |          |
                                ...        ...    ...      cd.    cd. syllabus.xml syllabus.xml DNI        DNI
                                                             +       +                        +----+---+
                                                             |       |                        |        |
                                                           comun   comun                actividades trabajos
                                                           +-+-+   +-+-+                    +-+-+    +-+-+
                                                           | | |   | | |                    | | |    | | |
                                                            ...     ...                      ...      ...
*/

/* Directory for this CGI and other commands called by it */
#define Cfg_PATH_ABS_CGI_BIN				"/var/www/cgi-bin"	// If not exists, it should be created during installation!

/* Main directories */
#define Cfg_PATH_REL_SWAD_PRIVATE			"../swad"		// If not exists, it should be created during installation!
#define Cfg_PATH_REL_SWAD_PUBLIC			"../html"		// If not exists, it should be created during installation!

/* Private a publi directories for courses */
#define Cfg_PATH_ABS_CRS_PRIVATE			"/var/www/swad/crs"
#define Cfg_PATH_REL_CRS_PRIVATE			"../swad/crs"		// If not exists, it should be created during installation!
#define Cfg_PATH_REL_CRS_PUBLIC				"../html/crs"		// If not exists, it should be created during installation!
#define Cfg_FOLDER_CRS_PUBLIC 				"crs"

/* Private directory for users */
#define Cfg_PATH_ABS_USR_PRIVATE			"/var/www/swad/usr"
#define Cfg_PATH_REL_USR_PRIVATE			"../swad/usr"		// If not exists, it should be created during installation!

/* Directory for temporary XHTML output of this CGI */
#define Cfg_PATH_REL_XHTML_OUTPUT_PRIVATE		"../swad/out"		// Created automatically the first time it is accessed

/* Directory for temporary public links to file zones, used when displaying file browsers */
#define Cfg_PATH_ABS_FILE_BROWSER_TMP_PUBLIC		"/var/www/html/tmp"
#define Cfg_PATH_REL_FILE_BROWSER_TMP_PUBLIC		"../html/tmp"		// Created automatically the first time it is accessed
#define Cfg_URL_FILE_BROWSER_TMP_PUBLIC			"/tmp"

/* Relative path to the private directory for compression of assignments and works into a zip file */
#define Cfg_PATH_REL_ZIP_TMP_PRIVATE			"../swad/tmp"		// Created automatically the first time it is accessed

/* Directories for photo storing and processing */
#define Cfg_PATH_REL_PHOTOS_PRIVATE			"../swad/photo"		// Created automatically the first time it is accessed
#define Cfg_PATH_REL_PHOTOS_PRIVATE_FROM_PHOTOS_PUBLIC	"../../swad/photo"
#define Cfg_PATH_REL_PHOTOS_TMP_PRIVATE			"../swad/photo/tmp"	// Created automatically the first time it is accessed
#define Cfg_PATH_REL_PHOTOS_PUBLIC			"../html/photo"		// Created automatically the first time it is accessed
#define Cfg_PATH_REL_PHOTOS_TMP_PUBLIC			"../html/photo/tmp"	// Created automatically the first time it is accessed
#define Cfg_URL_PHOTOS_PUBLIC				"/photo"
#define Cfg_URL_PHOTOS_TMP_PUBLIC			"/photo/tmp"
#define Cfg_FOLDER_DEGREE_PHOTO_MEDIAN			"mdn_all"
#define Cfg_FOLDER_DEGREE_PHOTO_AVERAGE			"avg_all"

/* Folders with icons. They should be created with icons during installation */
#define Cfg_FOLDER_PUBLIC_LOGO				"logo"
#define Cfg_FOLDER_PUBLIC_ICON				"icon"
#define Cfg_ICON_FOLDER_THEMES				"theme"		// Directory with all the images for each theme
#define Cfg_ICON_FOLDER_INSTITUTIONS			"institution"	// Directory with icons for institutions
#define Cfg_ICON_FOLDER_DEGREES				"degree"	// Directory with icons for degrees
#define Cfg_ICON_FOLDER_FILEXT				"filext"	// Directory with icons for file extensions
#define Cfg_ICON_FOLDER_PLUGINS				"plugin"	// Directory with icons for plugins

/* Folders and file used to store syllabus */
#define Cfg_SYLLABUS_FOLDER_LECTURES			"lec"
#define Cfg_SYLLABUS_FOLDER_PRACTICALS			"pra"
#define Cfg_SYLLABUS_FILENAME				"syllabus.xml"

/* Main folders in file browsers */
#define Cfg_CRS_INFO_TEACHING_GUIDE			"description"
#define Cfg_CRS_INFO_LECTURES				"teoria"
#define Cfg_CRS_INFO_PRACTICALS				"practicas"
#define Cfg_CRS_INFO_BIBLIOGRAPHY			"bibliografia"
#define Cfg_CRS_INFO_FAQ				"FAQ"
#define Cfg_CRS_INFO_LINKS				"enlaces"
#define Cfg_CRS_INFO_ASSESSMENT				"evaluacion"

/****************************************************************************/
/************************ Commands called by this CGI ***********************/
/****************************************************************************/

/* Template for chat window. It should be copied during installation */
#define Cfg_PATH_AND_FILE_REL_CHAT_PRIVATE		"./chat.html"

/* Command to make image processing / face detection, programmed by Daniel J. Calandria-Hernndez */
// %s must be substituted by temporary file with the image received:
#define Cfg_COMMAND_FACE_DETECTION			"/home/lithium/fotomaton13/fotomaton13 /home/lithium/fotomaton13/options %s 540"

/* Commands to compute the average photo of a degree */
#define Cfg_COMMAND_DEGREE_PHOTO_MEDIAN			"./foto_mediana"
#define Cfg_COMMAND_DEGREE_PHOTO_AVERAGE		"./foto_promedio"

/* Command to send automatic e-mails, programmed by Antonio F. Daz-Garca y Antonio Caas-Vargas */
#define Cfg_COMMAND_SEND_AUTOMATIC_E_MAIL		"./swad_smtp.py"
// E-mail address where failed automatic e-mails will be sent:
#define Cfg_AUTOMATIC_E_MAIL_FROM			"acanas@ugr.es"

/* Command to process external login of users. It works as a client of a external web service */
#define Cfg_EXTERNAL_LOGIN_CLIENT_COMMAND		"./prado"	// Client of the web service called by this CGI
#define Cfg_EXTERNAL_LOGIN_SERVICE_SHORT_NAME		"PRADO"
#define Cfg_EXTERNAL_LOGIN_SERVICE_FULL_NAME		"PRADO (Plataforma de Recursos de Apoyo a la Docencia)"
#define Cfg_NAME_PARAM_IMPORTED_USR_ID			"pradouser"
#define Cfg_NAME_PARAM_IMPORTED_SESSION_ID		"pradosession"
#define Cfg_MAX_LENGTH_IMPORTED_USR_ID			255	// The same size as that of column ImportedUsrId in table imported_sessions
#define Cfg_MAX_LENGTH_IMPORTED_SESSION_ID		255	// The same size as that of column ImportedSessionId in table imported_sessions

/****************************************************************************/
/******************************** Time periods ******************************/
/****************************************************************************/

#define Cfg_DAYS_IN_RECENT_LOG				 15	// Only accesses in these last days + 1 are stored in recent log.
								// Important!!! Must be 1 <= Cfg_DAYS_IN_RECENT_LOG <= 29
#define Cfg_TIMES_PER_SECOND_REFRESH_CONNECTED		  4	// Execute this CGI to refresh connected users about these times per second
#define Cfg_MIN_TIME_TO_REFRESH_CONNECTED		((time_t)(                   10UL))	// Refresh period of connected users in seconds
#define Cfg_MAX_TIME_TO_REFRESH_CONNECTED		((time_t)(              15UL*60UL))	// Refresh period of connected users in seconds
#define Cfg_TIME_TO_CLOSE_SESSION_FROM_LAST_REFRESH	((time_t)(Cfg_MAX_TIME_TO_REFRESH_CONNECTED * 3))	// After these seconds without refresh of connected users, session is closed
#define Cfg_TIME_TO_CLOSE_SESSION_FROM_LAST_CLICK	((time_t)(              60UL*60UL))	// After these seconds without user's clicks, session is closed

#define Cfg_TIME_TO_DELETE_WS_KEY			((time_t)(         24UL*60UL*60UL))	// After these seconds, a web service key is removed

#define Cfg_TIME_TO_DELETE_HTML_OUTPUT			((time_t)(              30UL*60UL))	// Remove the XHTML output files older than these seconds

#define Cfg_TIME_TO_ABORT_FILE_UPLOAD			((time_t)(              55UL*60UL))	// After these seconds uploading data, abort upload.

#define Cfg_TIME_TO_DELETE_BROWSER_TMP_FILES		((time_t)(          2UL*60UL*60UL))  	// Temporary files are deleted after these seconds
#define Cfg_TIME_TO_DELETE_BROWSER_EXPANDED_FOLDERS	((time_t)(3UL*30UL*24UL*60UL*60UL))	// Past these seconds, remove expired expanded folders
#define Cfg_TIME_TO_DELETE_BROWSER_CLIPBOARD		((time_t)(              15UL*60UL))	// Paths older than these seconds are removed from clipboard

#define Cfg_TIME_TO_DELETE_THREAD_CLIPBOARD		((time_t)(              15UL*60UL))	// Threads older than these seconds are removed from clipboard

#define Cfg_TIME_TO_DELETE_OLD_PENDING_PASSWORDS 	((time_t)(     7UL*24UL*60UL*60UL))	// Past these seconds, remove expired pending passwords

#define Cfg_TIME_TO_DELETE_IP_PREFS			((time_t)(     7UL*24UL*60UL*60UL))	// Past these seconds, remove expired IP preferences

#define Cfg_TIME_TO_SEND_PENDING_NOTIF 			((time_t)(          6UL*60UL*60UL))	// After these seconds after first pending notification, all the pending notifications are sent by e-mail

#define Cfg_MIN_TIME_TO_RECOMPUTE_AVG_PHOTO		((time_t)(         12UL*60UL*60UL))	// After these seconds, users can recompute the average photos of a degree

#endif
