// swad_con.c: connected users

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <limits.h>		// For maximum values
#include <stdio.h>		// For fprintf
#include <string.h>		// For string functions

#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_str.h"
#include "swad_usr.h"

/****************************************************************************/
/*************************** Internal constants *****************************/
/****************************************************************************/

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/*************************** Internal prototypes ****************************/
/****************************************************************************/

unsigned Con_GetConnectedStdsTotal (void);
unsigned Con_GetConnectedTchsTotal (void);
unsigned Con_GetConnectedUsrsBelongingCurrentCrs (Usr_UsrType_t UsrType);
void Con_ShowConnectedUsrsThisCrs (Usr_UsrType_t UsrType);
void Con_WriteRowConnectedUsr (long UsrCod,const char *Font,const char *StrTimeDiff);

/****************************************************************************/
/******************** Show total number of connected users ******************/
/****************************************************************************/

void Con_ShowGlobalConnectedUsrs (void)
  {
   extern const char *Txt_session[Txt_NUM_LANGUAGES];
   extern const char *Txt_sessions[Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_SINGULAR_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_PLURAL_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_from[Txt_NUM_LANGUAGES];
   char CrsShortName[Deg_MAX_LENGTH_COURSE_SHORT_NAME+1];
   char StrConnectedToCurrentCourse[128];

   /***** Put number of connected users who belong to current course into a string *****/
   if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
     {
      strcpy (CrsShortName,Gbl.CurrentCrs.Crs.ShortName);
      Str_LimitLengthHTMLStr (CrsShortName,12);
      Str_ReplaceSpecialCharByCodes (CrsShortName,256);
      sprintf (StrConnectedToCurrentCourse,"%u %s %s",
               Con_GetConnectedUsrsBelongingCurrentCrs (Usr_UNKNOWN),
               Txt_from[Gbl.Prefs.Language],
               CrsShortName);
     }
   else
      sprintf (StrConnectedToCurrentCourse,"&nbsp;");

   /***** Write total number of sessions *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"120\">" \
                      "<tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/frame_sessions_corner_1_4x32.gif');\">" \
                      "<img src=\"%s/tr4x14.gif\" width=\"4\" height=\"14\" alt=\"\" /></td>" \
                      "<td align=\"center\" valign=\"top\" style=\"background-image: url('%s/frame_sessions_top_1x32.gif'); background-repeat: repeat-x;\" class=\"USRS_CON\">%u&nbsp;%s</td>" \
                      "<td align=\"left\" width=\"4\" style=\"background-image: url('%s/frame_sessions_corner_2_4x32.gif');\">" \
                      "<img src=\"%s/tr4x14.gif\" width=\"4\" height=\"14\" alt=\"\" /></td></tr>",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,
            Gbl.Session.NumSessions,
            (Gbl.Session.NumSessions == 1) ? Txt_session[Gbl.Prefs.Language] : Txt_sessions[Gbl.Prefs.Language],
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);

   /***** Write number of connected students and teachers *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/frame_sessions_left_4x1.gif'); background-repeat: repeat-y;\"></td>" \
                      "<td width=\"112\" align=\"center\" valign=\"top\" bgcolor=\"#FFFFFF\" class=\"USRS_CON\">%u&nbsp;%s<br />%u&nbsp;%s<br />%s</td>" \
                      "<td align=\"left\" width=\"4\" style=\"background-image: url('%s/frame_sessions_right_4x1.gif'); background-repeat: repeat-y;\"></td></tr>" \
                      "<tr><td align=\"left\" width=\"4\"><img src=\"%s/frame_sessions_corner_3_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td>" \
                      "<td align=\"left\" style=\"background-image: url('%s/frame_sessions_bottom_1x4.gif'); background-repeat: repeat-x;\"></td>" \
                      "<td align=\"left\" width=\"4\"><img src=\"%s/frame_sessions_corner_4_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td></tr></table>",
            Gbl.Prefs.IconsURL,
            Gbl.Usrs.Connected.TchsTotal,
            (Gbl.Usrs.Connected.TchsTotal == 1) ? Txt_USR_TYPES_SINGULAR_abc[Usr_TEACHER][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language] :
                                                  Txt_USR_TYPES_PLURAL_abc[Usr_TEACHER][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            Gbl.Usrs.Connected.StdsTotal,
            (Gbl.Usrs.Connected.StdsTotal == 1) ? Txt_USR_TYPES_SINGULAR_abc[Usr_STUDENT][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language] :
                                                  Txt_USR_TYPES_PLURAL_abc[Usr_STUDENT][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            StrConnectedToCurrentCourse,
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);
  }

/****************************************************************************/
/******** Show number of connected users who belong to current course *******/
/****************************************************************************/

void Con_ShowConnectedUsrsBelongingToCurrentCrs (Usr_UsrType_t UsrType)
  {
   extern const char *Txt_USR_TYPES_SINGULAR_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_PLURAL_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   char CrsShortName[256+1];
   unsigned NumUsrs;

   /***** Start table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"120\">" \
                      "<tr><td align=\"left\" width=\"4\"><img src=\"%s/ewgw1_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td>" \
                      "<td align=\"left\" style=\"background-image: url('%s/mgw1_1x4.gif'); background-repeat: repeat-x;\"></td>" \
                      "<td align=\"left\" width=\"4\"><img src=\"%s/ewgw2_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td></tr>",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);

   /***** Write number of connected users who belong to current course *****/
   strcpy (CrsShortName,Gbl.CurrentCrs.Crs.ShortName);
   Str_LimitLengthHTMLStr (CrsShortName,14);
   Str_ReplaceSpecialCharByCodes (CrsShortName,256);
   NumUsrs = Con_GetConnectedUsrsBelongingCurrentCrs (UsrType);
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/mgw2_4x1.gif'); background-repeat: repeat-y;\"></td>" \
                      "<td width=\"112\" align=\"center\" valign=\"top\" class=\"USRS_CON\">%u %s</td>" \
                      "<td align=\"left\" width=\"4\" style=\"background-image: url('%s/mgw3_4x1.gif'); background-repeat: repeat-y;\"></td></tr>",
            Gbl.Prefs.IconsURL,NumUsrs,
            (NumUsrs == 1) ? Txt_USR_TYPES_SINGULAR_abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language] :
                             Txt_USR_TYPES_PLURAL_abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            Gbl.Prefs.IconsURL);

   /***** List connected users belonging to this course *****/
   if (Gbl.Usrs.Me.IHaveAccessToCurrentCrs)	// If I belongs to this course or I am administrator
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/mgw2_4x1.gif'); background-repeat: repeat-y;\"></td>" \
                         "<td width=\"112\" align=\"center\" valign=\"top\">" \
                         "<table cellspacing=\"0\" cellpadding=\"0\" width=\"112\">",
               Gbl.Prefs.IconsURL);
      Con_ShowConnectedUsrsThisCrs (UsrType);
      fprintf (Gbl.F.Out,"</table></td><td align=\"left\" width=\"4\" style=\"background-image: url('%s/mgw3_4x1.gif'); background-repeat: repeat-y;\"></td></tr>",
               Gbl.Prefs.IconsURL);
     }

   /***** End table *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"4\"><img src=\"%s/ewgw3_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td>" \
                      "<td align=\"left\" style=\"background-image: url('%s/mgw4_1x4.gif'); background-repeat: repeat-x;\"></td>" \
                      "<td align=\"left\" width=\"4\"><img src=\"%s/ewgw4_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td></tr>" \
                      "</table>",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);
  }

/****************************************************************************/
/********************* Modify my entry in connected list ********************/
/****************************************************************************/

void Con_UpdateMeInConnectedList (void)
  {
   char Query[512];

   /***** Update my entry in connected list *****/
   sprintf (Query,"REPLACE INTO connected (UsrCod,UsrType,CrsCod,LastTime)" \
                  " VALUES('%ld','%u','%ld',NOW())",
            Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.Usrs.Me.UsrTypeLog,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryREPLACE (Query,"can not update list of connected users");
  }

/*****************************************************************************/
/************************** Remove old connected uses ************************/
/*****************************************************************************/

void Con_RemoveOldConnected (void)
  {
   char Query[512];

   /***** Remove old users from connected list *****/
   sprintf (Query,"DELETE FROM connected WHERE UsrCod NOT IN" \
                  " (SELECT DISTINCT(UsrCod) FROM sessions)");
   DB_QueryDELETE (Query,"can not remove old users from list of connected users");
  }

/****************************************************************************/
/**************************** Get connected users ***************************/
/****************************************************************************/

void Con_GetConnectedUsrsTotal (void)
  {
   Gbl.Usrs.Connected.StdsTotal = Con_GetConnectedStdsTotal ();
   Gbl.Usrs.Connected.TchsTotal = Con_GetConnectedTchsTotal ();
  }

/****************************************************************************/
/*************************** Get connected students *************************/
/****************************************************************************/

unsigned Con_GetConnectedStdsTotal (void)
  {
   char Query[128];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumStds;

   /***** Get number of connected students from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM connected WHERE UsrType='%u'",
            Usr_STUDENT);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of connected students");

   /***** Get number of connected students *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumStds) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of connected students.");

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return NumStds;
  }

/****************************************************************************/
/*************************** Get connected students *************************/
/****************************************************************************/

unsigned Con_GetConnectedTchsTotal (void)
  {
   char Query[128];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumTchs;

   /***** Get number of connected teachers from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM connected WHERE UsrType>'%u'",
            Usr_STUDENT);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of connected teachers");

   /***** Get number of connected teachers *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumTchs) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of connected teachers.");

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return NumTchs;
  }

/****************************************************************************/
/************* Get connected users belonging to current course **************/
/****************************************************************************/

unsigned Con_GetConnectedUsrsBelongingCurrentCrs (Usr_UsrType_t UsrType)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrs;

   /***** Get number of connected users who belong to current course from database *****/
   if (UsrType == Usr_UNKNOWN)	// Here Usr_UNKNOWN stands for all types of user
      sprintf (Query,"SELECT COUNT(connected.UsrCod) FROM connected,crs_usr" \
                     " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=connected.UsrCod",
              Gbl.CurrentCrs.Crs.CrsCod);
   else
      sprintf (Query,"SELECT COUNT(connected.UsrCod) FROM connected,crs_usr,usr_data" \
                     " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=connected.UsrCod" \
                     " AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
              Gbl.CurrentCrs.Crs.CrsCod,(unsigned) UsrType);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of connected users who belong to this course");

   /***** Get number of users *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumUsrs) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of connected users who belong to this course.");

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return NumUsrs;
  }

/****************************************************************************/
/**************************** Show connected users **************************/
/****************************************************************************/

void Con_ShowConnectedUsrsThisCrs (Usr_UsrType_t UsrType)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long CrsCod;
   long UsrCod;

   Gbl.GlobalRowEvenOdd = 0;

   /***** Get connected users who belong to current course from database *****/
   sprintf (Query,"SELECT connected.UsrCod,connected.CrsCod," \
                  "UNIX_TIMESTAMP()-UNIX_TIMESTAMP(connected.LastTime) AS Dif" \
                  " FROM connected,crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=connected.UsrCod" \
                  " AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u' ORDER BY Dif",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) UsrType);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get list of connected users who belong to this course");

   /***** Write list of connected users *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get user code (row[0]) */
      UsrCod = Str_ConvertStrCodToLongCod (row[0]);

      /* Get course code (row[1]) */
      CrsCod = Str_ConvertStrCodToLongCod (row[1]);

      /* Write row in screen */
      Con_WriteRowConnectedUsr (UsrCod,
                                (CrsCod == Gbl.CurrentCrs.Crs.CrsCod) ? "CON_CRS" : "CON",
                                row[2]);
     }

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);
  }

/****************************************************************************/
/********************* Write the name of a connected user *******************/
/****************************************************************************/

void Con_WriteRowConnectedUsr (long UsrCod,const char *Font,const char *StrTimeDiff)
  {
   struct UsrData UsrDat;
   const char *Color = Gbl.ColorRows[Gbl.GlobalRowEvenOdd];
   bool ShowPhoto,UsrHasWebPage;
   char PhotoURL[PATH_MAX+1];
   time_t TimeDiff;
   long Hours;
   int Minutes;

   /***** Initialize user's data *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Get user's data *****/
   UsrDat.UsrCod = UsrCod;
   Usr_GetAllUsrDataFromUsrCod (&UsrDat);

   /***** Show photo *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"18\" valign=\"middle\" bgcolor=\"#%s\">",Color);
   if ((UsrHasWebPage = Str_URLLooksValid (UsrDat.WWW)))
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\">",UsrDat.WWW);
   ShowPhoto = Usr_ShowUsrPhotoIsAllowed (&UsrDat,PhotoURL);
   Usr_ShowUsrPhoto (&UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,true);
   if (UsrHasWebPage)
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write full name and link *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"56\" valign=\"middle\" bgcolor=\"#%s\" class=\"%s\">",Color,Font);
   Usr_RestrictLengthAndWriteName (&UsrDat,9);
   fprintf (Gbl.F.Out,"</td>");

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDat);

   /***** Write time from last access *****/
   fprintf (Gbl.F.Out,"<td width=\"38\" align=\"right\" valign=\"middle\" bgcolor=\"#%s\" class=\"%s\">",
            Color,Font);
   if (sscanf (StrTimeDiff,"%ld",&TimeDiff) == 1)
     {
      if ((Hours = TimeDiff / 3600))
	 fprintf (Gbl.F.Out,"%ld h<br />",Hours);
      Minutes = (TimeDiff % 3600) / 60;
      if (Hours)
	 fprintf (Gbl.F.Out,"%02d'",Minutes);
      else if (Minutes)
	 fprintf (Gbl.F.Out,"%2d'",Minutes);
      if (Hours || Minutes)
	 fprintf (Gbl.F.Out,"%02ld&quot;",TimeDiff % 60);
      else
	 fprintf (Gbl.F.Out,"%2ld&quot;",TimeDiff % 60);
     }
   fprintf (Gbl.F.Out,"</td></tr>");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }
