// swad_crs.c: info about course

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <limits.h>		// For maximum values
#include <stdlib.h>		// For getenv, etc.
#include <string.h>		// For string functions

#include "swad_act.h"
#include "swad_crs.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_str.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/*************************** Public constants *******************************/
/****************************************************************************/


/****************************************************************************/
/***************************** Private types ********************************/
/****************************************************************************/


/****************************************************************************/
/**************************** Private constants *****************************/
/****************************************************************************/

/* Functions to write forms in course edition (FAQ, links, etc.) */
void (*Crs_FormsForEditionTypes[Crs_NUM_INFO_SOURCES])(Crs_InfoType_t InfoType) =
  {
   NULL,
   Crs_FormToEnterIntegratedEditor,
   Crs_FormToEnterTextEditor,
   Crs_FormToSendPage,
   Crs_FormToSendURL,
  };


const char *Crs_NamesInDBForInfoSrc[Crs_NUM_INFO_SOURCES] =
  {
   "none",
   "editor",
   "text",
   "page",
   "URL",
  };
const Act_Action_t Crs_ActionsEditInfo[Crs_NUM_INFO_TYPES] =
  {
   ActEdiTchGui,
   ActEdiSylLec,
   ActEdiSylPra,
   ActEdiBib,
   ActEdiFAQ,
   ActEdiCrsLnk,
   ActEdiAss,
  };
const Act_Action_t Crs_ActionsSelecInfoSrc[Crs_NUM_INFO_TYPES] =
  {
   ActSelInfSrcTchGui,
   ActSelInfSrcSylLec,
   ActSelInfSrcSylPra,
   ActSelInfSrcBib,
   ActSelInfSrcFAQ,
   ActSelInfSrcCrsLnk,
   ActSelInfSrcAss,
  };
const Act_Action_t Crs_ActionsInfo[Crs_NUM_INFO_SOURCES][Crs_NUM_INFO_TYPES] =
  {
   {
    ActUnk,
    ActUnk,
    ActUnk,
    ActUnk,
    ActUnk,
    ActUnk,
    ActUnk,
   },
   {
    ActEditorTchGui,
    ActEditorSylLec,
    ActEditorSylPra,
    ActEditorBib,
    ActEditorFAQ,
    ActEditorCrsLnk,
    ActEditorAss,
   },
   {
    ActTxtEdiTchGui,
    ActTxtEdiSylLec,
    ActTxtEdiSylPra,
    ActTxtEdiBib,
    ActTxtEdiFAQ,
    ActTxtEdiCrsLnk,
    ActTxtEdiAss,
   },
   {
    ActRecPagTchGui,
    ActRecPagSylLec,
    ActRecPagSylPra,
    ActRecPagBib,
    ActRecPagFAQ,
    ActRecPagCrsLnk,
    ActRecPagAss,
   },
   {
    ActRecURLTchGui,
    ActRecURLSylLec,
    ActRecURLSylPra,
    ActRecURLBib,
    ActRecURLFAQ,
    ActRecURLCrsLnk,
    ActRecURLAss,
   }
  };

const Act_Action_t AccionesEditorBibEvEtc[Crs_NUM_INFO_TYPES] =
  {
   ActEditorTchGui,
   ActEditorSylLec,
   ActEditorSylPra,
   ActEditorBib,
   ActEditorFAQ,
   ActEditorCrsLnk,
   ActEditorAss,
  };
const Act_Action_t AccionesRecFicBibEvEtc[Crs_NUM_INFO_TYPES] =
  {
   ActRecPagTchGui,
   ActRecPagSylLec,
   ActRecPagSylPra,
   ActRecPagBib,
   ActRecPagFAQ,
   ActRecPagCrsLnk,
   ActRecPagAss,
  };
const Act_Action_t AccionesRecURLBibEvEtc[Crs_NUM_INFO_TYPES] =
  {
   ActRecURLTchGui,
   ActRecURLSylLec,
   ActRecURLSylPra,
   ActRecURLBib,
   ActRecURLFAQ,
   ActRecURLCrsLnk,
   ActRecURLAss,
  };
const Act_Action_t Crs_ActionsRecTxtInfo[Crs_NUM_INFO_TYPES] =
  {
   ActRecTxtTchGui,
   ActRecTxtSylLec,
   ActRecTxtSylPra,
   ActRecTxtBib,
   ActRecTxtFAQ,
   ActRecTxtCrsLnk,
   ActRecTxtAss,
  };
const char *Crs_NamesInDBForInfoType[Crs_NUM_INFO_TYPES] =
  {
   "description",	// Change this to "guide"!
   "theory",		// Change this to "lectures"!
   "practices",
   "bibliography",
   "FAQ",
   "links",
   "assessment",
  };

/****************************************************************************/
/**************************** Private prototypes ****************************/
/****************************************************************************/


/****************************************************************************/
/******** Show course info (theory, practices, bibliography, etc.) **********/
/****************************************************************************/

void Crs_ShowInfo (void)
  {
   extern const char *Txt_No_information_available[Txt_NUM_LANGUAGES];
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();
   Crs_InfoSrc_t InfoSrc;
   char FichHTML[NAME_MAX+1];
   char PathRelFichHTML[PATH_MAX+1];
   char PathAbsFichHTML[PATH_MAX+1];

   /***** Put link (form) to edit course info *****/
   switch (Gbl.Usrs.Me.UsrTypeLog)
     {
      case Usr_TEACHER:
      case Usr_DEG_ADMIN:
      case Usr_SUPERUSER:
         Crs_PutFormToEditInfo (InfoType);
         break;
      default:
         break;
     }

   /***** Get info source from database *****/
   InfoSrc = Crs_GetInfoSrcFromDB (Gbl.CurrentCrs.Crs.CrsCod,InfoType);

   switch (InfoSrc)
     {
      case Crs_INFO_SRC_NONE:
	 Lay_ShowAlert (WARNING,Txt_No_information_available[Gbl.Prefs.Language]);
         break;
      case Crs_INFO_SRC_EDITOR:
         switch (InfoType)
           {
            case Crs_LECTURES:
            case Crs_PRACTICALS:
               Syl_LoadSyllabusToMem ();
               Syl_EditSyllabus ();
               break;
            case Crs_TEACHING_GUIDE:
            case Crs_BIBLIOGRAPHY:
            case Crs_FAQ:
            case Crs_LINKS:
            case Crs_ASSESSMENT:
	       Lay_ShowAlert (WARNING,Txt_No_information_available[Gbl.Prefs.Language]);
           }
         break;
      case Crs_INFO_SRC_TEXT:
         Crs_ShowTxtInfo ();
         break;
      case Crs_INFO_SRC_PAGE:
         /***** Open file with web page *****/
         // !!!!!!!!!!!! If the page is hosted in server ==> it should be created a temporary public directory
         //                                                  and host the page in a private directory !!!!!!!!!!!!!!!!!
         strcpy (FichHTML,"index.html");
         sprintf (PathRelFichHTML,"%s/%s",Gbl.CurrentCrs.Info.Links[InfoType].PathRelWebPage,FichHTML);
         if (Fil_CheckIfPathExists (PathRelFichHTML))
           {
            sprintf (PathAbsFichHTML,"%s/%s",Gbl.CurrentCrs.Info.Links[InfoType].PathAbsWebPage,FichHTML);
            Crs_ShowPage (PathAbsFichHTML);
           }
         else
           {
            // If index.html not exists, try index.htm
            strcpy (FichHTML,"index.htm");
            sprintf (PathRelFichHTML,"%s/%s",Gbl.CurrentCrs.Info.Links[InfoType].PathRelWebPage,FichHTML);
            if (Fil_CheckIfPathExists (PathRelFichHTML))
              {
               sprintf (PathAbsFichHTML,"%s/%s",Gbl.CurrentCrs.Info.Links[InfoType].PathAbsWebPage,FichHTML);
               Crs_ShowPage (PathAbsFichHTML);
              }
            else
               Lay_ShowAlert (WARNING,Txt_No_information_available[Gbl.Prefs.Language]);
           }
         break;
      case Crs_INFO_SRC_URL:
         /***** Check if file with URL exists *****/
         Gbl.CurrentCrs.Info.FileURL  = fopen (Gbl.CurrentCrs.Info.Links[InfoType].PathRelFileURL ,"rt");
         if (Gbl.CurrentCrs.Info.FileURL)
           {
            fgets (Gbl.CurrentCrs.Info.URL,Cte_MAX_BYTES_URL,Gbl.CurrentCrs.Info.FileURL);
            /* File is not longer needed  ==> close it */
            fclose (Gbl.CurrentCrs.Info.FileURL);
            if (Gbl.CurrentCrs.Info.URL[0])
               Crs_ShowPage (Gbl.CurrentCrs.Info.URL);
            else
               Lay_ShowAlert (WARNING,Txt_No_information_available[Gbl.Prefs.Language]);
           }
         else
            Lay_ShowAlert (WARNING,Txt_No_information_available[Gbl.Prefs.Language]);
     }
  }

/****************************************************************************/
/******************** Put a link (form) to edit course info *****************/
/****************************************************************************/

void Crs_PutFormToEditInfo (Crs_InfoType_t InfoType)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (Crs_ActionsEditInfo[InfoType]);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/*********************** Show central iframe with a page ********************/
/****************************************************************************/

void Crs_ShowPage (const char *URI)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_View_in_a_new_window[Txt_NUM_LANGUAGES];

   /***** Link for view in a new window *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<a href=\"%s\" target=\"_blank\" class=\"%s\">%s</a>" \
                      "</div>",
            URI,The_ClassFormul[Gbl.Prefs.Theme],Txt_View_in_a_new_window[Gbl.Prefs.Language]);

   if (BrowserSupportIFrame ())
     {
      /***** Start of frame *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

      /***** iframe with the page *****/
      /*
      fprintf (Gbl.F.Out,"<tr><td align=\"left\"><IFRAME onLoad=\"iFrameHeight();\" src=\"%s\" id=\"iframe_central\" name=\"iframe_central\"" \
                     " style=\"width:100%%; height:480px;\"" \
                     " frameborder=\"0\" marginwidth=\"0\" marginheight=\"0\" vspace=\"0\" hspace=\"0\">" \
                     "Su navegador no admite <em>iframes</em>.</IFRAME></td></tr>",URI);
      */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\"><iframe onload=\"iFrameHeight();\" src=\"%s\" id=\"iframe_central\" name=\"iframe_central\"" \
                         " style=\"width:100%%; height:480px;\">" \
                         "Su navegador no admite <em>iframes</em>.</iframe></td></tr>",URI);

      /***** End of frame *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
  }

/****************************************************************************/
/*************************** Browse support iframe **************************/
/****************************************************************************/

#define MAX_LENGTH_HTTP_USER_AGENT 1024

bool BrowserSupportIFrame (void)
  {
   char HTTPUserAgent[MAX_LENGTH_HTTP_USER_AGENT+1];

   strncpy (HTTPUserAgent,getenv ("HTTP_USER_AGENT"),MAX_LENGTH_HTTP_USER_AGENT);
   HTTPUserAgent[MAX_LENGTH_HTTP_USER_AGENT] = '\0';

   if (strstr (HTTPUserAgent,"MSIE 5"))
      return false;

   return true;
  }

/****************************************************************************/
/************************** Set course info source **************************/
/****************************************************************************/

void Crs_SetInfoSrc (void)
  {
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();
   Crs_InfoSrc_t InfoSrcSelected = Crs_GetInfoSrcFromForm ();

   /***** Set info source into database *****/
   Crs_SetInfoSrcIntoDB (Gbl.CurrentCrs.Crs.CrsCod,InfoType,InfoSrcSelected);

   /***** Show again the form to select and send course info *****/
   Crs_FormsToSelSendInfo ();
  }

/****************************************************************************/
/************** Select course info source and send course info **************/
/****************************************************************************/

void Crs_FormsToSelSendInfo (void)
  {
   extern const char *Txt_Source_of_information[Txt_NUM_LANGUAGES];
   extern const char *Txt_INFO_SRC_FULL_TEXT[Crs_NUM_INFO_SOURCES][Txt_NUM_LANGUAGES];
   extern const char *Txt_INFO_SRC_HELP[Crs_NUM_INFO_SOURCES][Txt_NUM_LANGUAGES];
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();
   Crs_InfoSrc_t InfoSrc,InfoSrcSelected;

   /***** Get info source from database *****/
   InfoSrcSelected = Crs_GetInfoSrcFromDB (Gbl.CurrentCrs.Crs.CrsCod,InfoType);

   /***** Form to choice between alternatives *****/
   /* Start of form and table */
   Act_FormStart (Crs_ActionsSelecInfoSrc[InfoType]);
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* Title */
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TITULO\">%s</td></tr>",
            Txt_Source_of_information[Gbl.Prefs.Language]);

   /* Options */
   for (InfoSrc=(Crs_InfoSrc_t) 0; InfoSrc<Crs_NUM_INFO_SOURCES; InfoSrc++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\"");
      if (InfoSrc == InfoSrcSelected)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
      fprintf (Gbl.F.Out," class=\"DAT\"><input type=\"radio\" name=\"InfoSrc\" value=\"%d\"",(int) InfoSrc);
      if (InfoSrc == InfoSrcSelected)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," onclick=\"javascript:document.forms[%d].submit();\" />%s</td></tr>",
               Gbl.NumForm,Txt_INFO_SRC_FULL_TEXT[InfoSrc][Gbl.Prefs.Language]);
     }

   /* End of table and form */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</form>\n");

   /***** Forms for the different edition alternatives *****/
   /* Start of table */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* Forms */
   for (InfoSrc=(Crs_InfoSrc_t) 1; InfoSrc<Crs_NUM_INFO_SOURCES; InfoSrc++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"center\"");
      if (InfoSrc == InfoSrcSelected)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
      fprintf (Gbl.F.Out," class=\"TITULO\">&nbsp;<br />%s",
               Txt_INFO_SRC_FULL_TEXT[InfoSrc][Gbl.Prefs.Language]);
      if (Txt_INFO_SRC_HELP[InfoSrc][Gbl.Prefs.Language])
         fprintf (Gbl.F.Out,"<br />(%s)",Txt_INFO_SRC_HELP[InfoSrc][Gbl.Prefs.Language]);
      Act_FormStart (Crs_ActionsInfo[InfoSrc][InfoType]);
      Crs_FormsForEditionTypes[InfoSrc] (InfoType);
      fprintf (Gbl.F.Out,"</form><br /></td></tr>\n");
     }

   /* End of table */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/****************** Form to enter in integrated editor **********************/
/****************************************************************************/

void Crs_FormToEnterIntegratedEditor (Crs_InfoType_t InfoType)
  {
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   /* Send button */
   Lay_PutSendButton (Txt_Edit[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/********************** Form to enter in text editor ************************/
/****************************************************************************/

void Crs_FormToEnterTextEditor (Crs_InfoType_t InfoType)
  {
   extern const char *Txt_Edit_text[Txt_NUM_LANGUAGES];

   /* Send button */
   Lay_PutSendButton (Txt_Edit_text[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************* Form to upload a file with a page **********************/
/****************************************************************************/

void Crs_FormToSendPage (Crs_InfoType_t InfoType)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_File[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_file[Txt_NUM_LANGUAGES];

   /* File */
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>" \
                      "<td align=\"right\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\"><input type=\"file\" name=\"%s\" size=\"50\" maxlength=\"100\" value=\"\" /></td></tr></table>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_File[Gbl.Prefs.Language],
            Brw_NAME_OF_PARAM_FILENAME_ORG);

   /* Send button */
   Lay_PutSendButton (Txt_Upload_file[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/********************* Form to send a link to a web page ********************/
/****************************************************************************/

void Crs_FormToSendURL (Crs_InfoType_t InfoType)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Send_URL[Txt_NUM_LANGUAGES];
   FILE *FichURL;

   /***** Link *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>" \
                      "<td align=\"right\" class=\"%s\">URL:<br /></td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"BibEvEtcURL\" size=\"50\" maxlength=\"256\" value=\"",
            The_ClassFormul[Gbl.Prefs.Theme]);
   if ((FichURL = fopen (Gbl.CurrentCrs.Info.Links[InfoType].PathRelFileURL,"rt")) == NULL)
      fprintf (Gbl.F.Out,"http://");
   else
     {
      fgets (Gbl.CurrentCrs.Info.URL,Cte_MAX_BYTES_URL,FichURL);
      fprintf (Gbl.F.Out,"%s",Gbl.CurrentCrs.Info.URL);
      /* File is not needed now. Close it */
      fclose (FichURL);
     }
   fprintf (Gbl.F.Out,"\" /></td></tr></table>");

   /***** Send button *****/
   Lay_PutSendAndUndoButtons (Txt_Send_URL[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******** Returns bibliography, assessment, etc. from Gbl.CurrentAct ********/
/****************************************************************************/

Crs_InfoType_t Crs_AsignInfoType (void)
  {
   switch (Gbl.CurrentAct)
     {
      case ActSeeTchGui:	case ActEdiTchGui:		case ActSelInfSrcTchGui:	case ActEditorTchGui:	case ActRecURLTchGui:	case ActRecPagTchGui:	case ActTxtEdiTchGui:	case ActRecTxtTchGui:	return Crs_TEACHING_GUIDE;
      case ActSeeSylLec:	case ActEdiSylLec:		case ActSelInfSrcSylLec:	case ActEditorSylLec:	case ActRecURLSylLec:	case ActRecPagSylLec:	case ActTxtEdiSylLec:	case ActRecTxtSylLec:	return Crs_LECTURES;
      case ActSeeSylPra:	case ActEdiSylPra:		case ActSelInfSrcSylPra:	case ActEditorSylPra:	case ActRecURLSylPra:	case ActRecPagSylPra:	case ActTxtEdiSylPra:	case ActRecTxtSylPra:	return Crs_PRACTICALS;
      case ActSeeBib:	case ActEdiBib:		case ActSelInfSrcBib:	case ActEditorBib:	case ActRecURLBib:	case ActRecPagBib:	case ActTxtEdiBib:	case ActRecTxtBib:	return Crs_BIBLIOGRAPHY;
      case ActSeeFAQ:	case ActEdiFAQ:		case ActSelInfSrcFAQ:	case ActEditorFAQ:	case ActRecURLFAQ:	case ActRecPagFAQ:	case ActTxtEdiFAQ:	case ActRecTxtFAQ:	return Crs_FAQ;
      case ActSeeCrsLnk:case ActEdiCrsLnk:	case ActSelInfSrcCrsLnk:case ActEditorCrsLnk:	case ActRecURLCrsLnk:	case ActRecPagCrsLnk:	case ActTxtEdiCrsLnk:	case ActRecTxtCrsLnk:	return Crs_LINKS;
      case ActSeeAss:	case ActEdiAss:		case ActSelInfSrcAss:	case ActEditorAss:	case ActRecURLAss:	case ActRecPagAss:	case ActTxtEdiAss:	case ActRecTxtAss:	return Crs_ASSESSMENT;
     }
   return (Crs_InfoType_t) 0; // If action is not one of the former ones, the returned value is not important
  }

/****************************************************************************/
/********** Get info source for bibliography, FAQ, etc. from form ***********/
/****************************************************************************/

Crs_InfoSrc_t Crs_GetInfoSrcFromForm (void)
  {
   char UnsignedStr[10+1];
   int InfoSrcInt;
   Crs_InfoSrc_t InfoSrc;

   /* Get info source for a specific type of course information (bibliography, FAQ, links or evaluation) */
   Par_GetParam (Par_PARAM_SINGLE,"InfoSrc",UnsignedStr,2);
   if (sscanf (UnsignedStr,"%d",&InfoSrcInt) != 1)
      return Crs_INFO_SRC_NONE;
   InfoSrc = (Crs_InfoSrc_t) InfoSrcInt;
   return InfoSrc;
  }

/****************************************************************************/
/******** Set info source for a type of course info from database ***********/
/****************************************************************************/

void Crs_SetInfoSrcIntoDB (long CrsCod,Crs_InfoType_t InfoType,Crs_InfoSrc_t InfoSrc)
  {
   char Query[512];

   /***** Insert or replace info source for a specific type of course information *****/
   sprintf (Query,"REPLACE INTO crs_info_src (CrsCod,InfoType,InfoSrc) VALUES ('%ld','%s','%s')",
            CrsCod,Crs_NamesInDBForInfoType[InfoType],Crs_NamesInDBForInfoSrc[InfoSrc]);
   DB_QueryREPLACE (Query,"can not update info source");
  }

/****************************************************************************/
/******** Get info source for a type of course info from database ***********/
/****************************************************************************/

Crs_InfoSrc_t Crs_GetInfoSrcFromDB (long CrsCod,Crs_InfoType_t InfoType)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   Crs_InfoSrc_t InfoSrc = Crs_INFO_SRC_NONE;

   /***** Get info source for a specific type of course information
          (bibliography, FAQ, links or evaluation) from database *****/
   sprintf (Query,"SELECT InfoSrc FROM crs_info_src WHERE CrsCod='%ld' AND InfoType='%s'",
            CrsCod,Crs_NamesInDBForInfoType[InfoType]);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get info source");

   /***** The result of the query must have one row or none *****/
   if (NumRows == 1)
     {
      /* Get info source */
      row = mysql_fetch_row (mysql_res);
      InfoSrc = Crs_ConvertFromStrDBToInfoSrc (row[0]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   if (NumRows > 1)
      Lay_ShowErrorAndExit ("Error when getting info source.");

   return InfoSrc;
  }

/*****************************************************************************/
/*** Convert a string with info source in database to a Crs_InfoSrc_t value ****/
/*****************************************************************************/

Crs_InfoSrc_t Crs_ConvertFromStrDBToInfoSrc (const char *StrInfoSrcDB)
  {
   Crs_InfoSrc_t InfoSrc;

   for (InfoSrc=(Crs_InfoSrc_t) 0; InfoSrc<Tst_NUM_ANS_TYPES; InfoSrc++)
      if (!strcmp (StrInfoSrcDB,Crs_NamesInDBForInfoSrc[InfoSrc]))
         return InfoSrc;

   return (Crs_InfoSrc_t) 0;
  }

/****************************************************************************/
/********** Set info text for a type of course info from database ***********/
/****************************************************************************/

void Crs_SetInfoTxtIntoDB (long CrsCod,Crs_InfoType_t InfoType,const char *InfoTxt)
  {
   char Query[256+Cte_MAX_BYTES_TEXT];

   /***** Insert or replace info source for a specific type of course information *****/
   sprintf (Query,"REPLACE INTO crs_info_txt (CrsCod,InfoType,InfoTxt)" \
                  " VALUES ('%ld','%s','%s')",
            CrsCod,Crs_NamesInDBForInfoType[InfoType],InfoTxt);
   DB_QueryREPLACE (Query,"can not update info text");
  }

/****************************************************************************/
/********** Get info text for a type of course info from database ***********/
/****************************************************************************/

void Crs_GetInfoTxtFromDB (Crs_InfoType_t InfoType,char *InfoTxt)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get info source for a specific type of course information
          (bibliography, FAQ, links or evaluation) from database *****/
   sprintf (Query,"SELECT InfoTxt FROM crs_info_txt" \
                  " WHERE CrsCod='%ld' AND InfoType='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,Crs_NamesInDBForInfoType[InfoType]);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get info text");

   /***** The result of the query must have one row or none *****/
   if (NumRows == 1)
     {
      /* Get info text */
      row = mysql_fetch_row (mysql_res);
      strcpy (InfoTxt,row[0]);
     }
   else
      InfoTxt[0] = '\0';

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   if (NumRows > 1)
      Lay_ShowErrorAndExit ("Error when getting info text.");
  }

/****************************************************************************/
/********************* Show information about the course ********************/
/****************************************************************************/

void Crs_ShowTxtInfo (void)
  {
   extern const char *Txt_No_information_available[Txt_NUM_LANGUAGES];
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();

   /***** Allocate memory for text of course info *****/
   if (!Str_AllocateText ())
      Lay_ShowErrorAndExit (Gbl.Message);

   /***** Get info text from database *****/
   Crs_GetInfoTxtFromDB (InfoType,Gbl.Txt);

   if (Gbl.Txt[0])
     {
      /***** Start table *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      if (InfoType == Crs_TEACHING_GUIDE)
         Lay_WriteHeaderClassPhoto (3,false,false,Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"4%%\"></td>" \
                         "<td align=\"left\" width=\"92%%\"><p align=\"justify\" class=\"DAT\"><br />");

      /***** Convert to respectful HTML and insert links *****/
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.Txt,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to recpectful HTML
      Str_InsertLinkInURLs (Gbl.Txt,Cte_MAX_BYTES_TEXT,60);	// Insert links

      /***** Write text *****/
      fprintf (Gbl.F.Out,"%s",Gbl.Txt);

      /***** Finish table *****/
      fprintf (Gbl.F.Out,"<br />&nbsp;</p></td><td align=\"left\" width=\"4%%\"></td></tr>\n");
      if (InfoType == Crs_TEACHING_GUIDE)
        Lay_WriteFootClassPhoto ();
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
   else
      Lay_ShowAlert (WARNING,Txt_No_information_available[Gbl.Prefs.Language]);

   /***** Free memory for text of course info *****/
   Str_FreeTxt ();
  }

/****************************************************************************/
/********************* Edit information about the course ********************/
/****************************************************************************/

void Crs_EditTxtInfo (void)
  {
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();

   /***** Start table *****/
   Act_FormStart (Crs_ActionsRecTxtInfo[InfoType]);
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   if (InfoType == Crs_TEACHING_GUIDE)
      Lay_WriteHeaderClassPhoto (1,false,false,Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);

   /***** Asign memory for text of course info *****/
   if (!Str_AllocateText ())
      Lay_ShowErrorAndExit (Gbl.Message);

   /***** Get info text from database *****/
   Crs_GetInfoTxtFromDB (InfoType,Gbl.Txt);

   /***** Edition area *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"><textarea name=\"Txt\" cols=\"80\" rows=\"20\">%s</textarea></td></tr>",
            Gbl.Txt);

   /***** End of table *****/
   if (InfoType == Crs_TEACHING_GUIDE)
      Lay_WriteFootClassPhoto ();
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"<br />");

   /***** Send and undo buttons *****/
   Lay_PutSendAndUndoButtons (Txt_Send[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/*****************************************************************************/
/****************** Receive and change text of course info *******************/
/*****************************************************************************/

void Crs_RecAndChangeInfoTxt (void)
  {
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();

   /***** Get text with course information from form *****/
   Str_AllocateAndGetText (Str_TO_HTML,true);	// Store in HTML format (not rigorous)

   /***** Update text of course info in database *****/
   Crs_SetInfoTxtIntoDB (Gbl.CurrentCrs.Crs.CrsCod,InfoType,Gbl.Txt);

   /***** Free text with course info *****/
   Str_FreeTxt ();

   /***** Show the updated info *****/
   Crs_ShowInfo ();
  }

/****************************************************************************/
/********* Receive a link to a page of syllabus, bibliography, etc. *********/
/****************************************************************************/

void Crs_ReceiveURLInfo (void)
  {
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();
   FILE *FichURL;

   /***** Get parameter with URL *****/
   Par_GetParam (Par_PARAM_SINGLE,"BibEvEtcURL",Gbl.CurrentCrs.Info.URL,Cte_MAX_BYTES_URL);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.CurrentCrs.Info.URL,PATH_MAX,true);

   /***** Open file with URL *****/
   if ((FichURL = fopen (Gbl.CurrentCrs.Info.Links[InfoType].PathRelFileURL,"wt")) == NULL)
       Lay_ShowErrorAndExit ("Error when saving URL to file.");

   /***** Write URL *****/
   fprintf (FichURL,"%s",Gbl.CurrentCrs.Info.URL);

   /***** Close file *****/
   fclose (FichURL);

   /***** Write message *****/
   sprintf (Gbl.Message,"URL <strong>%s</strong> almacenado correctamente.",	// Need translation!
            Gbl.CurrentCrs.Info.URL);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/************** Receive a page of syllabus, bibliography, etc. **************/
/****************************************************************************/

void Crs_ReceivePagInfo (void)
  {
   Crs_InfoType_t InfoType = Crs_AsignInfoType ();
   char NombreArchFuente[PATH_MAX+1];
   char PathRelFichHTML[PATH_MAX+1];
   char FileType[Brw_MAX_BYTES_MIME_TYPE+1];
   char CadenaUnzip[100+PATH_MAX*2+1];
   char *PathWebPage;

   /***** First of all, store in disk the file from stdin (really from Gbl.F.Tmp) *****/
   Fil_StartReceptionOfFile (NombreArchFuente,FileType);

   /***** Check that MIME type is HTML or ZIP *****/
   if (strcmp (FileType,"text/html") &&
       strcmp (FileType,"text/plain") &&
       strcmp (FileType,"application/x-zip-compressed") &&
       strcmp (FileType,"application/zip") &&
       strcmp (FileType,"application/octet-stream"))
      Lay_ShowAlert (WARNING,"El archivo debe ser <em>HTML</em> o <em>ZIP</em>.");	// Need translation!
   else
     {
      /***** End the reception of the data *****/
      PathWebPage = Gbl.CurrentCrs.Info.Links[InfoType].PathRelWebPage;
      if (Str_FileIs (NombreArchFuente,"html") ||
          Str_FileIs (NombreArchFuente,"htm" )) // .html or .htm file
        {
         Brw_RemoveTree (PathWebPage);
         Fil_CreateDirIfNotExists (PathWebPage);
         sprintf (PathRelFichHTML,"%s/index.html",PathWebPage);
         if (Fil_EndReceptionOfFile (PathRelFichHTML))
            Lay_ShowAlert (INFO,"Archivo <em>HTML</em> recibido correctamente.");	// Need translation!
        }
      else if (Str_FileIs (NombreArchFuente,"zip")) // .zip file
        {
         Brw_RemoveTree (PathWebPage);
         Fil_CreateDirIfNotExists (PathWebPage);
         if (Fil_EndReceptionOfFile (Gbl.CurrentCrs.Info.Links[InfoType].PathRelFileZIP))
           {
            Lay_ShowAlert (INFO,"Archivo <em>ZIP</em> recibido correctamente.");	// Need translation!

            /* Uncompress ZIP */
            sprintf (CadenaUnzip,"unzip -qq -o %s -d %s",Gbl.CurrentCrs.Info.Links[InfoType].PathRelFileZIP,PathWebPage);
            if (system (CadenaUnzip) == 0)
              {
               /* Check if uploaded file is index.html or index.htm */
               sprintf (PathRelFichHTML,"%s/index.html",PathWebPage);
               if (Fil_CheckIfPathExists (PathRelFichHTML))
                  Lay_ShowAlert (INFO,"Archivo <em>ZIP</em> descomprimido correctamente (se ha encontrado un archivo index.html).");	// Need translation!
	       else
	         {
	          sprintf (PathRelFichHTML,"%s/index.htm",PathWebPage);
	          if (Fil_CheckIfPathExists (PathRelFichHTML))
	             Lay_ShowAlert (INFO,"Archivo <em>ZIP</em> descomprimido correctamente (se ha encontrado un archivo index.htm).");	// Need translation!
	          else
                     Lay_ShowAlert (WARNING,"No se ha encontrado un archivo <strong>index.html</strong> o <strong>index.htm</strong> dentro del archivo <em>ZIP</em>.");	// Need translation!
	         }
	      }
            else
               Lay_ShowErrorAndExit ("Can not uncompress file.");
           }
        }
      else
         Lay_ShowAlert (WARNING,"El archivo debe ser <em>HTML</em> o <em>ZIP</em>.");	// Need translation!
     }
  }

/****************************************************************************/
/****************** Integrated editor for teaching guide ********************/
/****************************************************************************/

void Crs_EditorTeachingGuide (void)
  {
   extern const char *Txt_The_integrated_editor_is_not_yet_available[Txt_NUM_LANGUAGES];

   Lay_ShowAlert (WARNING,Txt_The_integrated_editor_is_not_yet_available[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****************** Integrated editor for bibliography **********************/
/****************************************************************************/

void Crs_EditorBibliography (void)
  {
   extern const char *Txt_The_integrated_editor_is_not_yet_available[Txt_NUM_LANGUAGES];

   Lay_ShowAlert (WARNING,Txt_The_integrated_editor_is_not_yet_available[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/********************** Integrated editor for FAQ ***************************/
/****************************************************************************/

void Crs_EditorFAQ (void)
  {
   extern const char *Txt_The_integrated_editor_is_not_yet_available[Txt_NUM_LANGUAGES];

   Lay_ShowAlert (WARNING,Txt_The_integrated_editor_is_not_yet_available[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/********************* Integrated editor for links **************************/
/****************************************************************************/

void Crs_EditorLinks (void)
  {
   extern const char *Txt_The_integrated_editor_is_not_yet_available[Txt_NUM_LANGUAGES];

   Lay_ShowAlert (WARNING,Txt_The_integrated_editor_is_not_yet_available[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****************** Integrated editor for assessment system *****************/
/****************************************************************************/

void Crs_EditorAssessment (void)
  {
   extern const char *Txt_The_integrated_editor_is_not_yet_available[Txt_NUM_LANGUAGES];

   Lay_ShowAlert (WARNING,Txt_The_integrated_editor_is_not_yet_available[Gbl.Prefs.Language]);
  }
