// swad_crs.c: info about course

#ifndef _SWAD_CRS
#define _SWAD_CRS
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_boo.h"

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

#define Crs_NUM_INFO_TYPES 7
typedef enum
  {
   Crs_TEACHING_GUIDE = 0,
   Crs_LECTURES       = 1,
   Crs_PRACTICALS     = 2,
   Crs_BIBLIOGRAPHY   = 3,
   Crs_FAQ            = 4,
   Crs_LINKS          = 5,
   Crs_ASSESSMENT     = 6
  } Crs_InfoType_t;

#define Crs_NUM_INFO_SOURCES 5
typedef enum
  {
   Crs_INFO_SRC_NONE   = 0,
   Crs_INFO_SRC_EDITOR = 1,
   Crs_INFO_SRC_TEXT   = 2,
   Crs_INFO_SRC_PAGE   = 3,
   Crs_INFO_SRC_URL    = 4
  } Crs_InfoSrc_t;

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Crs_ShowInfo (void);
void Crs_PutFormToEditInfo (Crs_InfoType_t InfoType);
void Crs_ShowPage (const char *URI);
bool BrowserSupportIFrame (void);
void Crs_SetInfoSrc (void);
void Crs_FormsToSelSendInfo (void);
void Crs_FormToEnterIntegratedEditor (Crs_InfoType_t InfoType);
void Crs_FormToEnterTextEditor (Crs_InfoType_t InfoType);
void Crs_FormToSendPage (Crs_InfoType_t InfoType);
void Crs_FormToSendURL (Crs_InfoType_t InfoType);
Crs_InfoType_t Crs_AsignInfoType (void);
Crs_InfoSrc_t Crs_GetInfoSrcFromForm (void);
void Crs_SetInfoSrcIntoDB (long CrsCod,Crs_InfoType_t InfoType,Crs_InfoSrc_t InfoSrc);
Crs_InfoSrc_t Crs_GetInfoSrcFromDB (long CrsCod,Crs_InfoType_t InfoType);
Crs_InfoSrc_t Crs_ConvertFromStrDBToInfoSrc (const char *StrInfoSrcDB);
void Crs_SetInfoTxtIntoDB (long CrsCod,Crs_InfoType_t InfoType,const char *InfoTxt);
void Crs_GetInfoTxtFromDB (Crs_InfoType_t InfoType,char *InfoTxt);
void Crs_ShowTxtInfo (void);
void Crs_EditTxtInfo (void);
void Crs_RecAndChangeInfoTxt (void);
void Crs_ReceiveURLInfo (void);
void Crs_ReceivePagInfo (void);
void Crs_EditorTeachingGuide (void);
void Crs_EditorBibliography (void);
void Crs_EditorFAQ (void);
void Crs_EditorLinks (void);
void Crs_EditorAssessment (void);

#endif
