// swad_ctr.c: centres

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_boo.h"
#include "swad_cns.h"
#include "swad_ctr.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_ins.h"
#include "swad_par.h"
#include "swad_str.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Ctr_GetParamCtrOrderType (void);
static void Ctr_PutFormToEditCentres (void);
static void Ctr_ListCentresForEdition (void);
static void Ctr_PutParamCtrCod (long CtrCod);
static long Ctr_GetParamCtrCod (void);
static void Ctr_RenameCentre (Cte_ShortOrFullName_t ShortOrFullName);
static bool Ctr_CheckIfCentreNameExists (const char *FieldName,const char *Name,long CtrCod);
static void Ctr_PutFormToCreateCentre (void);
static void Ctr_PutHeadCentres (void);
static void Ctr_CreateCentre (struct Centre *Ctr);

/****************************************************************************/
/*************************** List all the centres ***************************/
/****************************************************************************/

void Ctr_SeeCentres (void)
  {
   extern const char *Txt_Institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_CENTRES_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_CENTRES_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_Other_centres[Txt_NUM_LANGUAGES];
   extern const char *Txt_Centre_unspecified[Txt_NUM_LANGUAGES];
   tCtrsOrderType Order;
   unsigned NumCtr;
   struct Institution Ins;
   unsigned NumTchsWithCentre = 0;
   unsigned NumTchsInOtherCentres;

   /***** Get parameter with the type of order in the list of centres *****/
   Ctr_GetParamCtrOrderType ();

   /***** Get list of centres *****/
   Ctr_GetListCentres ();

   /***** Put link (form) to edit centres *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
       Ctr_PutFormToEditCentres ();

   /***** Table head *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td>",
            Txt_Institution[Gbl.Prefs.Language]);
   for (Order=CTR_ORDER_BY_CENTRE; Order<=CTR_ORDER_BY_NUM_TCHS; Order++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">");
      Act_FormStart (ActSeeCtr);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_CENTRES_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
      if (Order == Gbl.Ctrs.SelectedOrderType)
         fprintf (Gbl.F.Out,"<u>");
      fprintf (Gbl.F.Out,Txt_CENTRES_ORDER[Order][Gbl.Prefs.Language]);
      if (Order == Gbl.Ctrs.SelectedOrderType)
         fprintf (Gbl.F.Out,"</u>");
      fprintf (Gbl.F.Out,"</acronym></a></form></td>");
     }
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write all the centres and their nuber of teachers *****/
   for (NumCtr=0; NumCtr<Gbl.Ctrs.Num; NumCtr++)
     {
      /* Get data of institution of this centre */
      Ins.InsCod = Gbl.Ctrs.Lst[NumCtr].InsCod;
      Ins_GetDataOfInstitutionByCod (&Ins);

      /* Write data of this centre */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\">" \
                         "<a href=\"%s\" target=\"_blank\" class=\"DAT\" alt=\"%s\">%s</a></td>",
               Ins.WWW,Ins.FullName,Ins.ShortName);
      fprintf (Gbl.F.Out,"<td align=\"left\">" \
                         "<a href=\"%s\" target=\"_blank\" class=\"DAT\">%s</a></td>",
               Gbl.Ctrs.Lst[NumCtr].WWW,Gbl.Ctrs.Lst[NumCtr].FullName);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Gbl.Ctrs.Lst[NumCtr].NumTchs);
      NumTchsWithCentre += Gbl.Ctrs.Lst[NumCtr].NumTchs;
     }

   /***** Separation row *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;</td>" \
                      "</tr>");

   /***** Write teachers with other centre *****/
   NumTchsInOtherCentres = Usr_GetNumberOfTeachersInCentre (0);
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "</tr>",
            Txt_Other_centres[Gbl.Prefs.Language],NumTchsInOtherCentres);
   NumTchsWithCentre += NumTchsInOtherCentres;

   /***** Write teachers with no centre *****/
   fprintf (Gbl.F.Out,"<tr>"
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "</tr>",
            Txt_Centre_unspecified[Gbl.Prefs.Language],
            Sta_GetTotalNumberOfUsers (Sta_LOCATION_RANGE_ALL,Usr_TEACHER) - NumTchsWithCentre);

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of centres *****/
   Ctr_FreeListCentres ();
  }

/*****************************************************************************/
/********** Get parameter with the type or order in list of centres **********/
/*****************************************************************************/

static void Ctr_GetParamCtrOrderType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Ctrs.SelectedOrderType = (tCtrsOrderType) UnsignedNum;
   else
      Lay_ShowErrorAndExit ("Wrong sort order.");
  }

/****************************************************************************/
/***** Put a hidden parameter with the type of order in list of centres *****/
/****************************************************************************/

void Ctr_PutHiddenParamCtrOrderType (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Ctrs.SelectedOrderType);
  }

/****************************************************************************/
/******************** Put a link (form) to edit centres *****************/
/****************************************************************************/

static void Ctr_PutFormToEditCentres (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiCtr);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************ Put forms to edit degree types ********************/
/****************************************************************************/

void Ctr_EditCentres (void)
  {
   extern const char *Txt_Centres[Txt_NUM_LANGUAGES];

   /***** Get list of institutions *****/
   Ins_GetListInstitutions (false);
   if (!Gbl.Inss.Num)
     {
      Lay_ShowAlert (WARNING,"No hay lista de instituciones." \
                             " Debe crear al menos una instituci&oacute;n antes de crear centros.");	// Need translation!
      return;
     }

   /***** Get list of centres *****/
   Ctr_GetListCentres ();

   /***** Help message *****/
   if (Gbl.Ctrs.Num) // Centres found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo centro rellene el nombre del centro y su direcci&oacute;n web," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar un centro existente pulse el aspa roja.<br />" \
                          "Para modificar un centro existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Centres[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No degree types found
      Lay_ShowAlert (INFO,"No existen centros.<br />" \
                          "Para a&ntilde;adir un centro rellene el nombre del centro y su direcci&oacute;n web," \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit centres *****/
   /* Start table with centres */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current centres */
   if (Gbl.Ctrs.Num)
      Ctr_ListCentresForEdition ();

   /* Put a form to create a new centre */
   Ctr_PutFormToCreateCentre ();

   /* End of table with centres */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of centres *****/
   Ctr_FreeListCentres ();

   /***** Free list of institutions *****/
   Ins_FreeListInstitutions ();
  }

/****************************************************************************/
/*************************** List all the centres ***************************/
/****************************************************************************/

void Ctr_GetListCentres (void)
  {
   char OrderBySubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumCtr;
   struct Centre *Ctr;

   /***** Get centres from database *****/
   switch (Gbl.Ctrs.SelectedOrderType)
     {
      case CTR_ORDER_BY_CENTRE:
         sprintf (OrderBySubQuery,"FullName");
         break;
      case CTR_ORDER_BY_NUM_TCHS:
         sprintf (OrderBySubQuery,"NumTchs DESC,FullName");
     }
   sprintf (Query,"(SELECT centres.CtrCod,centres.InsCod," \
                  "centres.ShortName,centres.FullName," \
                  "centres.WWW,COUNT(*) AS NumTchs" \
                  " FROM centres,usr_data" \
                  " WHERE usr_data.UsrType='%u' AND usr_data.CtrCod=centres.CtrCod" \
                  " GROUP BY centres.CtrCod)" \
                  " UNION " \
                  "(SELECT CtrCod,InsCod,ShortName,FullName,WWW,0 AS NumTchs" \
                  " FROM centres" \
                  " WHERE CtrCod NOT IN" \
                  " (SELECT DISTINCT CtrCod FROM usr_data WHERE UsrType='%u'))" \
                  " ORDER BY %s",
            (unsigned) Usr_TEACHER,(unsigned) Usr_TEACHER,OrderBySubQuery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get centres");

   if (NumRows) // Centres found...
     {
      // NumRows should be equal to Deg->NumCourses
      Gbl.Ctrs.Num = (unsigned) NumRows;

      /***** Create list with courses in degree *****/
      if ((Gbl.Ctrs.Lst = (struct Centre *) calloc (NumRows,sizeof (struct Centre))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store centres.");

      /***** Get the centres *****/
      for (NumCtr=0; NumCtr<Gbl.Ctrs.Num; NumCtr++)
        {
         Ctr = &(Gbl.Ctrs.Lst[NumCtr]);

         /* Get next centre */
         row = mysql_fetch_row (mysql_res);

         /* Get centre code (row[0]) */
         if ((Ctr->CtrCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of centre.");

         /* Get institution code (row[1]) */
         if ((Ctr->InsCod = Str_ConvertStrCodToLongCod (row[1])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of institution.");

         /* Get the short name of the centre (row[2]) */
         strcpy (Ctr->ShortName,row[2]);

         /* Get the full name of the centre (row[3]) */
         strcpy (Ctr->FullName,row[3]);

         /* Get the URL of the centre (row[4]) */
         strcpy (Ctr->WWW,row[4]);

         /* Get number of teachers in this centre (row[5]) */
         if (sscanf (row[5],"%u",&Ctr->NumTchs) != 1)
            Ctr->NumTchs = 0;
        }
     }
   else
      Gbl.Ctrs.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);   
  }

/*****************************************************************************/
/**************************** Get centre full name ***************************/
/*****************************************************************************/

void Ctr_GetDataOfCentreByCod (struct Centre *Ctr)
  {
   extern const char *Txt_Another_centre[Txt_NUM_LANGUAGES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Clear data *****/
   Ctr->ShortName[0] = Ctr->FullName[0] = Ctr->WWW[0] = '\0';
   Ctr->NumTchs = 0;

   /***** Check if centre code is correct *****/
   if (Ctr->CtrCod == 0)
     {
      strcpy (Ctr->ShortName,Txt_Another_centre[Gbl.Prefs.Language]);
      strcpy (Ctr->FullName,Txt_Another_centre[Gbl.Prefs.Language]);
     }
   else if (Ctr->CtrCod > 0)
     {
      /***** Get data of a centre from database *****/
      sprintf (Query,"(SELECT centres.ShortName,centres.FullName,centres.WWW,COUNT(*)" \
                     " FROM centres,usr_data" \
                     " WHERE centres.CtrCod='%ld' AND usr_data.UsrType='%u' AND centres.CtrCod=usr_data.CtrCod" \
                     " GROUP BY centres.CtrCod)"
                     " UNION " \
                     "(SELECT ShortName,FullName,WWW,0" \
                     " FROM centres" \
                     " WHERE CtrCod='%ld' AND CtrCod NOT IN" \
                     " (SELECT DISTINCT CtrCod FROM usr_data WHERE UsrType='%u'))",
               Ctr->CtrCod,(unsigned) Usr_TEACHER,Ctr->CtrCod,(unsigned) Usr_TEACHER);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a centre");

      if (NumRows) // Centre found...
        {
         /* Get row */
         row = mysql_fetch_row (mysql_res);

         /* Get the short name of the centre (row[0]) */
         strcpy (Ctr->ShortName,row[0]);

         /* Get the full name of the centre (row[1]) */
         strcpy (Ctr->FullName,row[1]);

         /* Get the URL of the centre (row[2]) */
         strcpy (Ctr->WWW,row[2]);

         /* Get number of teachers in this centre (row[3]) */
         if (sscanf (row[3],"%u",&Ctr->NumTchs) != 1)
            Ctr->NumTchs = 0;
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/**************************** Free list of centres ***************************/
/*****************************************************************************/

void Ctr_FreeListCentres (void)
  {
   if (Gbl.Ctrs.Lst)
     {
      /***** Free memory used by the list of courses in degree *****/
      free ((void *) Gbl.Ctrs.Lst);
      Gbl.Ctrs.Lst = NULL;
      Gbl.Ctrs.Num = 0;
     }
  }

/****************************************************************************/
/*************************** List all the centres ***************************/
/****************************************************************************/

static void Ctr_ListCentresForEdition (void)
  {
   extern const char *Txt_Remove_centre[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   unsigned NumCtr;
   struct Centre *Ctr;
   struct Institution Ins;
   unsigned NumIns;

   /***** Table head *****/
   Ctr_PutHeadCentres ();

   /***** Write all the centres *****/
   for (NumCtr=0; NumCtr<Gbl.Ctrs.Num; NumCtr++)
     {
      Ctr = &Gbl.Ctrs.Lst[NumCtr];

      /* Get data of institution of this centre */
      Ins.InsCod = Ctr->InsCod;
      Ins_GetDataOfInstitutionByCod (&Ins);

      /* Put icon to remove centre */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      if (Ctr->NumTchs)	// Centre has teachers ==> deletion forbidden
         fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActRemCtr);
         Ctr_PutParamCtrCod (Ctr->CtrCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_centre[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Txt_Remove_centre[Gbl.Prefs.Language]);
        }
      fprintf (Gbl.F.Out,"</td>");

      /* Centre code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Ctr->CtrCod);

      /* Institution */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgCtrIns);
      Ctr_PutParamCtrCod (Ctr->CtrCod);
      fprintf (Gbl.F.Out,"<select name=\"InsCod\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
      fprintf (Gbl.F.Out,"<option value=\"0\"");
      if (Ctr->InsCod == 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
      for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                  Gbl.Inss.Lst[NumIns].InsCod,
                  (Gbl.Inss.Lst[NumIns].InsCod == Ctr->InsCod) ? " selected=\"selected\"" : "",
                  Gbl.Inss.Lst[NumIns].ShortName);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Centre short name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenCtrSho);
      Ctr_PutParamCtrCod (Ctr->CtrCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_CENTRE_SHORT_NAME,Ctr->ShortName,Gbl.NumForm);

      /* Centre full name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenCtrFul);
      Ctr_PutParamCtrCod (Ctr->CtrCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_CENTRE_FULL_NAME,Ctr->FullName,Gbl.NumForm);

      /* Centre WWW */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgCtrWWW);
      Ctr_PutParamCtrCod (Ctr->CtrCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"WWW\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Cte_MAX_LENGTH_WWW,Ctr->WWW,Gbl.NumForm);

      /* Number of teachers */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Ctr->NumTchs);
     }
  }

/****************************************************************************/
/******************** Write parameter with code of centre *******************/
/****************************************************************************/

static void Ctr_PutParamCtrCod (long CtrCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CtrCod\" value=\"%ld\" />",CtrCod);
  }

/****************************************************************************/
/******************* Get parameter with code of degree type *****************/
/****************************************************************************/

static long Ctr_GetParamCtrCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of centre *****/
   Par_GetParam (Par_PARAM_SINGLE,"CtrCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/******************************* Remove a centre ****************************/
/****************************************************************************/

void Ctr_RemoveCentre (void)
  {
   extern const char *Txt_Centre_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Centre Ctr;

   /***** Get centre code *****/
   if ((Ctr.CtrCod = Ctr_GetParamCtrCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of centre is missing.");

   /***** Get data of the centre from database *****/
   Ctr_GetDataOfCentreByCod (&Ctr);

   /***** Check if this centre has teachers *****/
   if (Ctr.NumTchs)	// Centre has teachers ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar un centro" \
                             " primero debe eliminar" \
                             " todos los profesores de ese centro.");	// Need translation!
   else	// Centre has no teachers ==> remove it
     {
      /***** Remove centre *****/
      sprintf (Query,"DELETE FROM centres WHERE CtrCod='%ld'",Ctr.CtrCod);
      DB_QueryDELETE (Query,"can not remove a centre");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Centre_X_removed[Gbl.Prefs.Language],
               Ctr.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Ctr_EditCentres ();
  }

/****************************************************************************/
/********************* Change the institution of a centre *******************/
/****************************************************************************/

void Ctr_ChangeCentreIns (void)
  {
   struct Centre *Ctr;
   char Query[512];

   Ctr = &Gbl.Ctrs.EditingCtr;

   /***** Get parameters from form *****/
   /* Get the code of the centre */
   if ((Ctr->CtrCod = Ctr_GetParamCtrCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of centre is missing.");

   /* Get parameter with institution code */
   Ctr->InsCod = Ins_GetParamInsCod ();

   /***** Update institution in table of centres *****/
   sprintf (Query,"UPDATE centres SET InsCod='%ld' WHERE CtrCod='%ld'",
            Ctr->InsCod,Ctr->CtrCod);
   DB_QueryUPDATE (Query,"can not update the institution of a centre");

   /***** Write message to show the change made *****/
   Lay_ShowAlert (INFO,"Se ha cambiado la instituci&oacute;n del centro.");	// Need translation!

   /***** Show the form again *****/
   Ctr_EditCentres ();
  }

/****************************************************************************/
/********************* Change the short name of a centre ********************/
/****************************************************************************/

void Ctr_RenameCentreShort (void)
  {
   Ctr_RenameCentre (Cte_SHORT_NAME);
  }

/****************************************************************************/
/********************* Change the full name of a centre *********************/
/****************************************************************************/

void Ctr_RenameCentreFull (void)
  {
   Ctr_RenameCentre (Cte_FULL_NAME);
  }

/****************************************************************************/
/************************ Change the name of a degree ***********************/
/****************************************************************************/

static void Ctr_RenameCentre (Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_centre_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Centre *Ctr;
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentCtrName = NULL;		// Initialized to avoid warning
   char NewCtrName[MAX_LENGTH_CENTRE_FULL_NAME+1];

   Ctr = &Gbl.Ctrs.EditingCtr;
   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = MAX_LENGTH_CENTRE_SHORT_NAME;
         CurrentCtrName = Ctr->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = MAX_LENGTH_CENTRE_FULL_NAME;
         CurrentCtrName = Ctr->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the centre */
   if ((Ctr->CtrCod = Ctr_GetParamCtrCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of centre is missing.");

   /* Get the new name for the centre */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewCtrName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewCtrName,MaxLength,true);

   /***** Get from the database the old names of the centre *****/
   Ctr_GetDataOfCentreByCod (Ctr);

   /***** Check if new name is empty *****/
   if (!NewCtrName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del centro <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentCtrName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentCtrName,NewCtrName))	// Different names
        {
         /***** If degree was in database... *****/
         if (Ctr_CheckIfCentreNameExists (ParamName,NewCtrName,Ctr->CtrCod))
           {
            sprintf (Gbl.Message,Txt_The_centre_X_already_exists[Gbl.Prefs.Language],
                     NewCtrName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE centres SET %s='%s' WHERE CtrCod='%ld'",
                     FieldName,NewCtrName,Ctr->CtrCod);
            DB_QueryUPDATE (Query,"can not update the name of a centre");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El centro <strong>%s</strong> ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     CurrentCtrName,NewCtrName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del centro <strong>%s</strong> no ha cambiado.",	// Need translation
                  CurrentCtrName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (CurrentCtrName,NewCtrName);
   Ctr_EditCentres ();
  }

/****************************************************************************/
/********************* Check if the name of centre exists *******************/
/****************************************************************************/

static bool Ctr_CheckIfCentreNameExists (const char *FieldName,const char *Name,long CtrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of centres with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM centres WHERE %s='%s' AND CtrCod<>'%ld'",
            FieldName,Name,CtrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a centre already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of a centre already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/************************* Change the URL of a centre ***********************/
/****************************************************************************/

void Ctr_ChangeCtrWWW (void)
  {
   struct Centre *Ctr;
   char Query[256+Cte_MAX_LENGTH_WWW];
   char NewWWW[Cte_MAX_LENGTH_WWW+1];

   Ctr = &Gbl.Ctrs.EditingCtr;

   /***** Get parameters from form *****/
   /* Get the code of the centre */
   if ((Ctr->CtrCod = Ctr_GetParamCtrCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of centre is missing.");

   /* Get the new WWW for the centre */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",NewWWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewWWW,Cte_MAX_LENGTH_WWW,true);

   /***** Check if new WWW is empty *****/
   if (NewWWW[0])
     {
      /* Update the table changing old WWW by new WWW */
      sprintf (Query,"UPDATE centres SET WWW='%s' WHERE CtrCod='%ld'",
               NewWWW,Ctr->CtrCod);
      DB_QueryUPDATE (Query,"can not update the web of a centre");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva direcci&oacute;n web es <strong>%s</strong>.",	// Need translation!
               NewWWW);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar la direcci&oacute;n web vac&iacute;a.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Ctr->WWW,NewWWW);
   Ctr_EditCentres ();
  }

/****************************************************************************/
/********************* Put a form to create a new centre ********************/
/****************************************************************************/

static void Ctr_PutFormToCreateCentre (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_centre[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_centre[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   struct Centre *Ctr;
   unsigned NumIns;

   Ctr = &Gbl.Ctrs.EditingCtr;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"7\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"7\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_centre[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewCtr);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Ctr_PutHeadCentres ();

   /***** New centre *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_centre[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Centre code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Institution *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"InsCod\" style=\"width:50px;\">");
   fprintf (Gbl.F.Out,"<option value=\"0\"");
   if (Ctr->InsCod == 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
   for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
      fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
               Gbl.Inss.Lst[NumIns].InsCod,
               (Gbl.Inss.Lst[NumIns].InsCod == Ctr->InsCod) ? " selected=\"selected\"" : "",
               Gbl.Inss.Lst[NumIns].ShortName);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Centre short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_CENTRE_SHORT_NAME,Ctr->ShortName);

   /***** Centre full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_CENTRE_FULL_NAME,Ctr->FullName);

   /***** Centre WWW *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"WWW\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Cte_MAX_LENGTH_WWW,Ctr->WWW);
  }

/****************************************************************************/
/******************** Write header with ields of a degree ******************/
/****************************************************************************/

static void Ctr_PutHeadCentres (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_Name[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" class=\"TIT_TBL\"></td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">WWW</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">Prof.</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Institution[Gbl.Prefs.Language],
            Txt_Short_Name[Gbl.Prefs.Language],
            Txt_Full_Name[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************* Receive form to create a new centre ********************/
/****************************************************************************/

void Ctr_RecFormNewCtr (void)
  {
   extern const char *Txt_The_centre_X_already_exists[Txt_NUM_LANGUAGES];
   struct Centre *Ctr;

   Ctr = &Gbl.Ctrs.EditingCtr;

   /***** Get parameters from form *****/
   /* Get institution */
   Ctr->InsCod = Ins_GetParamInsCod ();

   /* Get centre short name */
   Par_GetParam (Par_PARAM_SINGLE,"ShortName",Ctr->ShortName,MAX_LENGTH_CENTRE_SHORT_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ctr->ShortName,MAX_LENGTH_CENTRE_SHORT_NAME,true);

   /* Get centre full name */
   Par_GetParam (Par_PARAM_SINGLE,"FullName",Ctr->FullName,MAX_LENGTH_CENTRE_FULL_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ctr->FullName,MAX_LENGTH_CENTRE_FULL_NAME,true);

   /* Get centre WWW */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",Ctr->WWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ctr->WWW,Cte_MAX_LENGTH_WWW,true);

   if (Ctr->ShortName[0] && Ctr->FullName[0])	// If there's a centre name
     {
      if (Ctr->WWW[0])
        {
         /***** If name of centre was in database... *****/
         if (Ctr_CheckIfCentreNameExists ("ShortName",Ctr->ShortName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_centre_X_already_exists[Gbl.Prefs.Language],
                     Ctr->ShortName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else if (Ctr_CheckIfCentreNameExists ("FullName",Ctr->FullName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_centre_X_already_exists[Gbl.Prefs.Language],
                     Ctr->FullName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else	// Add new centre to database
            Ctr_CreateCentre (Ctr);
        }
      else	// If there isn't a web
        {
         sprintf (Gbl.Message,"Debe especificar la direcci&oacute;n web del nuevo centro.");	// Need translation!
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else	// If there isn't a centre name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre breve y el nombre completo del nuevo centro.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Ctr_EditCentres ();
  }

/****************************************************************************/
/***************************** Create a new centre **************************/
/****************************************************************************/

static void Ctr_CreateCentre (struct Centre *Ctr)
  {
   extern const char *Txt_Created_new_centre_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new centre *****/
   sprintf (Query,"INSERT INTO centres (InsCod,ShortName,FullName,WWW)" \
                  " VALUES ('%ld','%s','%s','%s')",
            Ctr->InsCod,Ctr->ShortName,Ctr->FullName,Ctr->WWW);
   DB_QueryINSERT (Query,"can not create a new centre");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_centre_X[Gbl.Prefs.Language],
            Ctr->FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/************************** Get number of centres ***************************/
/****************************************************************************/

unsigned Ctr_GetTotalNumberOfCentres (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumCtrs;

   /***** Get total number of centres from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM centres");
   DB_QuerySELECT (Query,&mysql_res,"can not get total number of centres");

   /***** Get number of centres *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumCtrs) != 1)
      NumCtrs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumCtrs;
  }

/****************************************************************************/
/***************** Get number of centres in a institution *******************/
/****************************************************************************/

unsigned Ctr_GetNumberOfCentresInInstitution (long InsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumCtrs;

   /***** Get number of centres of an institution from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM centres WHERE InsCod='%ld'",InsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of centres of an institution");

   /***** Get number of centres *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumCtrs) != 1)
      NumCtrs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumCtrs;
  }
