// swad_dat.c: dates

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <ctype.h>		// For isdigit
#include <string.h>		// For string functions
#include <time.h>		// For time

#include "swad_exa.h"
#include "swad_gbl.h"
#include "swad_par.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/**************************** Private constants *****************************/
/****************************************************************************/

const unsigned NumDaysMonth[1+12] = {0,31,28,31,30,31,30,31,31,30,31,30,31};

/****************************************************************************/
/***************************** Get current time *****************************/
/****************************************************************************/

void Dat_GetTimeStartExecution (void)
  {
   Gbl.TimeStartExecution = time (NULL);
  }

/****************************************************************************/
/****************** Convert the tiempo actual a estructura tm ****************/
/****************************************************************************/

void Dat_GetAndConvertCurrentDateTime (void)
  {
   /***** Convert current local time to a struct tblock *****/
   Dat_GetLocalTimeFromClock (&Gbl.TimeStartExecution);

   Gbl.Now.Year   = Gbl.Today.Year  = Gbl.tblock->tm_year + 1900;
   Gbl.Now.Month  = Gbl.Today.Month = Gbl.tblock->tm_mon + 1;
   Gbl.Now.Day    = Gbl.Today.Day   = Gbl.tblock->tm_mday;
   Gbl.Now.Hour   = Gbl.tblock->tm_hour;
   Gbl.Now.Minute = Gbl.tblock->tm_min;
   Gbl.Now.Second = Gbl.tblock->tm_sec;
   Gbl.AcademicYear = (Gbl.Today.Month >= 10 || (Gbl.Today.Month == 9 && Gbl.Today.Day >= 28)) ? Gbl.Today.Year : Gbl.Today.Year - 1;

   /***** Initialize current date in format YYYYMMDD *****/
   sprintf (Gbl.Today.YYYYMMDD,"%04u%02u%02u",
            Gbl.Today.Year,Gbl.Today.Month,Gbl.Today.Day);

   /***** Initialize current time in format YYYYMMDDHHMMSS *****/
   sprintf (Gbl.Now.YYYYMMDDHHMMSS,"%04u%02u%02u%02u%02u%02u",
            Gbl.Now.Year,Gbl.Now.Month,Gbl.Now.Day,Gbl.Now.Hour,Gbl.Now.Minute,Gbl.Now.Second);

   /***** Compute what day was yesterday *****/
   Dat_GetDayBefore (&Gbl.Today,&Gbl.Yesterday);
  }

/****************************************************************************/
/*********** Get a struct Date from a string in YYYYMMDD format *************/
/****************************************************************************/

bool Dat_GetDateFromYYYYMMDD (struct Date *Date,const char *YYYYMMDD)
  {
   if (sscanf (YYYYMMDD,"%04u%02u%02u",&(Date->Year),&(Date->Month),&(Date->Day)) == 3)
     {
      strncpy (Date->YYYYMMDD,YYYYMMDD,4+2+2);
      Date->YYYYMMDD[4+2+2] = '\0';
      return true;
     }
   else
     {
      Date->Year = Date->Month = Date->Day = 0;
      Date->YYYYMMDD[0] = '\0';
      return false;
     }
  }

/****************************************************************************/
/******* Get a struct DateTime from a string in YYYYMMDDHHMMSS format *******/
/****************************************************************************/

bool Dat_GetDateTimeFromYYYYMMDDHHMMSS (struct DateTime *DateTime,const char *YYYYMMDDHHMMSS)
  {
   if (sscanf (YYYYMMDDHHMMSS,"%04u%02u%02u%02u%02u%02u",
               &(DateTime->Year),&(DateTime->Month),&(DateTime->Day),&(DateTime->Hour),&(DateTime->Minute),&(DateTime->Second)) == 6)
     {
      strncpy (DateTime->YYYYMMDDHHMMSS,YYYYMMDDHHMMSS,4+2+2+2+2+2);
      DateTime->YYYYMMDDHHMMSS[4+2+2+2+2+2] = '\0';
      return true;
     }
   else
     {
      DateTime->Year = DateTime->Month = DateTime->Day = DateTime->Hour = DateTime->Minute = DateTime->Second = 0;
      DateTime->YYYYMMDDHHMMSS[0] = '\0';
      return false;
     }
  }

/****************************************************************************/
/**************************** Draw month and hour ***************************/
/****************************************************************************/

void Dat_DrawMonthAndHour (void)
  {
   /***** Get list of holidays *****/
   if (!Gbl.Hlds.LstIsRead)
     {
      Gbl.Hlds.SelectedOrderType = Hld_ORDER_BY_START_DATE;
      Hld_GetListHolidays ();
     }

   /***** Start table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">\n");

   /***** Month *****/
   /* Create list of calls for examination */
   Exa_CreateExamAnnouncement ();

   /* Draw the month */
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"middle\">\n");
   Dat_DrawMonth (Gbl.Today.Year,Gbl.Today.Month,false,true,false);
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /* Free list of dates of exam announcements */
   Exa_FreeListExamAnnouncements ();

   /***** Clock with hour:minute (server hour is shown) *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\">" \
                      "<div id=\"hm\" class=\"FECHA_AV\">%02u:%02u h</div></td></tr>\n",
            Gbl.Now.Hour,Gbl.Now.Minute);

   /***** End table *****/
   fprintf (Gbl.F.Out,"</table>\n");
  }

/****************************************************************************/
/************************ Draw an academic calendar *************************/
/****************************************************************************/

void Deg_DrawCalendar (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_print_view[Txt_NUM_LANGUAGES];
   unsigned Fila,Col;
   unsigned Year = (Gbl.Today.Month >= 10 || (Gbl.Today.Month == 9 && Gbl.Today.Day >= 29)) ? Gbl.Today.Year : Gbl.Today.Year - 1;
   unsigned Month = 10;
   bool PutLinkToEvents = (Gbl.CurrentAct == ActSeeCal);

   /***** Get list of holidays *****/
   if (!Gbl.Hlds.LstIsRead)
     {
      Gbl.Hlds.SelectedOrderType = Hld_ORDER_BY_START_DATE;
      Hld_GetListHolidays ();
     }

   /***** Start of table and title *****/
   switch (Gbl.CurrentAct)
     {
      case ActSeeCal:
	 /* Link to print view */
	 fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">");
	 fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"middle\">");
	 Act_FormStart (ActPrnCal);
         strcpy (Gbl.MsgWindowStatus,Txt_Show_print_view[Gbl.Prefs.Language]);
         Act_LinkFormSubmit (Act_NEW_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/print12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                            " %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);

	 /* Calendar head */
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         Lay_WriteHeaderClassPhoto (1,false,false,
                                  Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);
	 break;
      case ActPrnCal:
	 /* Calendar head */
	 Lay_StartTableFrame (DARK_BLUE,WHITE,NULL);
         Lay_WriteHeaderClassPhoto (1,true,false,
                                  Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);
     }

   /***** Create list of calls for examination *****/
   Exa_CreateExamAnnouncement ();

   /***** Draw todos los months del calendario *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"><table cellspacing=\"5\" cellpadding=\"0\">");
   for (Fila=0; Fila<3; Fila++)
     {
      fprintf (Gbl.F.Out,"<tr>");
      for (Col=0; Col<4; Col++)
	{
	 fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" width=\"120\">");
	 Dat_DrawMonth (Year,Month,true,PutLinkToEvents,(Gbl.CurrentAct == ActPrnCal));
	 fprintf (Gbl.F.Out,"</td>");
	 if (++Month == 13)
	   {
	    Month = 1;
	    Year++;
	   }
	}
      fprintf (Gbl.F.Out,"</tr>");
     }
   fprintf (Gbl.F.Out,"</table></td></tr>");

   /***** Free list of dates of exam announcements *****/
   Exa_FreeListExamAnnouncements ();

   /***** End of the marco *****/
   Lay_WriteFootClassPhoto ();
   switch (Gbl.CurrentAct)
     {
      case ActSeeCal:
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
	 break;
      case ActPrnCal:
         Lay_EndTableFrame ();
     }
  }

/****************************************************************************/
/******************************** Draw a month ******************************/
/****************************************************************************/

void Dat_DrawMonth (unsigned RealYear,unsigned RealMonth,bool DrawingCalendar,bool PutLinkToEvents,bool PrintView)
  {
   extern const char *Txt_DAYS_CAPS[7][Txt_NUM_LANGUAGES];
   extern const char *Txt_MONTHS_CAPS[12][Txt_NUM_LANGUAGES];
   unsigned Week;
   int DayOfWeek, /* 0, 1, 2, 3, 4, 5  6 */
       DayOfMonth,
       NumDaysInMonth;
   unsigned Year = RealYear;
   int Month = (int) RealMonth;
   char YYYYMMDD[4+2+2+1];
   unsigned NumHld;
   char *ClassForDay;	// Class of day depending on type of day
   char *TextForDay;	// Text associated to a day, for example the name of the holiday
   unsigned NumExamAnnouncement;	// Number of exam announcement
   int ResultOfCmpStartDate;
   bool ContinueSearching;
   bool ThisDayHasEvent = false;
   bool IsToday;
   char Header[512];

   /***** Compute number of day of month for the first box *****/
   /* The initial day of month (DayOfMonth) can be -5, -4, -3, -2, -1, 0, or 1
      If it's -5 then write 6 boxes of the previous month.
      If it's -4 then write 5 boxes of the previous month.
      If it's -3 then write 4 boxes of the previous month.
      If it's -2 then write 3 boxes of the previous month.
      If it's -1 then write 2 boxes of the previous month.
      If it's  0 then write 1 box   of the previous month.
      If it's  1 then write 0 boxes of the previous month. */
   if ((DayOfMonth = 1 - Dat_GetDayOfWeek (Year,Month,1)) <= 0)
     {
      if (--Month <= 0)
	{
	 Month = 12;
	 Year--;
	}
      NumDaysInMonth = (Month == 2) ? Dat_GetNumDaysFebruary (Year) : NumDaysMonth[Month];
      DayOfMonth += NumDaysInMonth;
     }

   /***** Month title *****/
   if (PrintView)
     {
      Lay_StartTableFrame (DARK_BLUE,WHITE,"100%");
      fprintf (Gbl.F.Out,"<tr><td colspan=\"7\" align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"MES\">%s %u</td></tr>\n",
               VERY_LIGHT_BLUE,Txt_MONTHS_CAPS[RealMonth-1][Gbl.Prefs.Language],RealYear);
     }
   else
     {
      sprintf (Header,"<td align=\"center\" valign=\"top\" style=\"background-image: url('%s/mbb1_1x32.gif'); background-repeat: repeat-x;\" class=\"MES\">%s %u</td>",
               Gbl.Prefs.IconsURL,Txt_MONTHS_CAPS[RealMonth-1][Gbl.Prefs.Language],RealYear);
      Lay_StartFrameRound4 ("ewbb","mbw",WHITE,"120",Header);
     }

   /***** Month head: first two letters for each day of week *****/
   fprintf (Gbl.F.Out,"<tr>");
   for (DayOfWeek=0; DayOfWeek<7; DayOfWeek++)
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"%s\">%c</td>",
               (DayOfWeek == 6) ? "T_DIA_NL" : "T_DIA_L",
               Txt_DAYS_CAPS[DayOfWeek][Gbl.Prefs.Language][0]);
   fprintf (Gbl.F.Out,"</tr>\n");

   /***** Draw every week of the month *****/
   for (Week=0; Week<6; Week++)
     {
      fprintf (Gbl.F.Out,"<tr>");

      /***** Draw every day of the week *****/
      for (DayOfWeek=0; DayOfWeek<7; DayOfWeek++)
	{
         /***** Set class for day being drawn *****/
         ClassForDay = (Month == RealMonth) ? "DIA_L" : "DIA_L_LIGHT";
	 TextForDay = NULL;

         /* Check if day is a holiday or a school day */
         sprintf (YYYYMMDD,"%04u%02u%02u",Year,(unsigned) Month,(unsigned) DayOfMonth);
         for (NumHld=0, ContinueSearching=true; NumHld<Gbl.Hlds.Num && ContinueSearching; NumHld++)
            if (Gbl.Hlds.Lst[NumHld].PlcCod <= 0 || Gbl.Hlds.Lst[NumHld].PlcCod == Gbl.CurrentDeg.Deg.PlcCod)
              {
               ResultOfCmpStartDate = strcmp (Gbl.Hlds.Lst[NumHld].StartDate.YYYYMMDD,YYYYMMDD);
               if (ResultOfCmpStartDate > 0)	// List is ordered by start date. If start date is greater than date being drawn, don't continue searching

                  ContinueSearching = false;
               else	// ResultOfCmpStartDate <= 0 <==> start date <= date being drawn
                  switch (Gbl.Hlds.Lst[NumHld].HldTyp)
                    {
                     case Hld_HOLIDAY:
                        if (ResultOfCmpStartDate == 0)	// If start date == date being drawn
                          {
                           ClassForDay = (Month == RealMonth) ? "DIA_F" : "DIA_F_LIGHT";
                           TextForDay = Gbl.Hlds.Lst[NumHld].Name;
                           ContinueSearching = false;
                          }
                        break;
                     case Hld_NON_SCHOOL_PERIOD:
                        if (strcmp (Gbl.Hlds.Lst[NumHld].EndDate.YYYYMMDD,YYYYMMDD) >= 0)	// If start date <= date being drawn <= end date
                          {
                           ClassForDay = (Month == RealMonth) ? "DIA_NL" : "DIA_NL_LIGHT";
                           TextForDay = Gbl.Hlds.Lst[NumHld].Name;
                          }
                    }
              }

         /* Day being drawn is sunday? */
	 if (DayOfWeek == 6) // All the sundays are holidays
	    ClassForDay = (Month == RealMonth) ? "DIA_F" : "DIA_F_LIGHT";

         /* Date being drawn is today? */
	 IsToday = (Gbl.CurrentAct != ActPrnCal && Month == RealMonth &&
                    Year       == Gbl.Today.Year &&
                    Month      == Gbl.Today.Month &&
                    DayOfMonth == Gbl.Today.Day);

         /* Check if day has an exam announcement */
         ThisDayHasEvent = false;
	 if (!DrawingCalendar || Month == RealMonth)	// If drawing calendar and the month is not the real one, don't draw exam announcements
	    for (NumExamAnnouncement=0; NumExamAnnouncement<Gbl.LstExamAnnouncements.NumExamAnnounc; NumExamAnnouncement++)
               if (Year       == Gbl.LstExamAnnouncements.Lst[NumExamAnnouncement].Year &&
                   Month      == Gbl.LstExamAnnouncements.Lst[NumExamAnnouncement].Month &&
                   DayOfMonth == Gbl.LstExamAnnouncements.Lst[NumExamAnnouncement].Day)
                 {
		  ThisDayHasEvent = true;
		  if (PutLinkToEvents)
   	             sprintf (Gbl.MsgWindowStatus,"Examen de %s: %02u/%02u/%04u",	// Need translation!
                              Gbl.CurrentCrs.Crs.FullName,
                              Gbl.LstExamAnnouncements.Lst[NumExamAnnouncement].Day,
                              Gbl.LstExamAnnouncements.Lst[NumExamAnnouncement].Month,
                              Gbl.LstExamAnnouncements.Lst[NumExamAnnouncement].Year);
                  break;
                 }

         /***** Write the box with the day *****/
	 fprintf (Gbl.F.Out,"<td align=\"center\" class=\"%s\">",
                  IsToday ? (ThisDayHasEvent ? "TODAY_EVENT" : "TODAY") :
                            (ThisDayHasEvent ? "DAY_EVENT"   : "DAY"  ));

         /* If day has an exam announcement */
	 if (PutLinkToEvents && ThisDayHasEvent)
           {
            Act_FormStart (ActSeeExaAnn);
            fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">" \
                               "<tr><td align=\"center\" class=\"%s\">",
                     ClassForDay);
            Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,ClassForDay);
           }
         else if (TextForDay == NULL || PrintView)
	    fprintf (Gbl.F.Out,"<span class=\"%s\">",
                     ClassForDay);
         else
	    fprintf (Gbl.F.Out,"<acronym title=\"%s\" class=\"%s\">",
                     TextForDay,ClassForDay);

	 /* Write the day of month */
	 fprintf (Gbl.F.Out,"%u",DayOfMonth);

         /* If day has an exam announcement */
	 if (PutLinkToEvents && ThisDayHasEvent)
            fprintf (Gbl.F.Out,"</acronym></a></td></tr></table></form>");
         else if (TextForDay == NULL || PrintView)
            fprintf (Gbl.F.Out,"</span>");
         else
            fprintf (Gbl.F.Out,"</acronym>");

	 fprintf (Gbl.F.Out,"</td>");

         /***** Set the next day *****/
	 NumDaysInMonth = (Month == 2) ? Dat_GetNumDaysFebruary (Year) : NumDaysMonth[Month];
	 if (++DayOfMonth > NumDaysInMonth)
	   {
	    DayOfMonth = 1;
	    if (++Month == 13)
	      {
	       Month = 1;
	       Year++;
	      }
	   }
	}
      fprintf (Gbl.F.Out,"</tr>\n");
     }
   if (PrintView)
      Lay_EndTableFrame ();
   else
      Lay_EndFrameRound4 ("ewbw","mbw");
  }

/*****************************************************************************/
/***************** Compute local time, adjusting day of week *****************/
/*****************************************************************************/

void Dat_GetLocalTimeFromClock (const time_t *clock)
  {
   if ((Gbl.tblock = localtime (clock)) != NULL)
     {
      // Convert from sunday, monday, tuesday... to monday, tuesday, wednesday...
      if (Gbl.tblock->tm_wday == 0)	// If sunday
	Gbl.tblock->tm_wday = 6;
      else				// If no sunday
	Gbl.tblock->tm_wday--;
     }
  }

/*****************************************************************************/
/***************************** Write current date ****************************/
/*****************************************************************************/

void Dat_WriteDateFromtblock (void)
  {
   fprintf (Gbl.F.Out,"%02d/%02d/%02d&nbsp;%02d:%02d",
            Gbl.tblock->tm_mday,1+Gbl.tblock->tm_mon,Gbl.tblock->tm_year%100,
            Gbl.tblock->tm_hour,Gbl.tblock->tm_min);
  }

/*****************************************************************************/
/********* Convert a struct with Day, Month and Year to a date string ********/
/*****************************************************************************/

void Dat_ConvDateToDateStr (struct Date *Date,char *DateStr)
  {
   if (Date->Day == 0 || Date->Month == 0 || Date->Year == 0)
      DateStr[0] = '\0';
   else
      sprintf (DateStr,"%02u/%02u/%04u",Date->Day,Date->Month,Date->Year);
  }

/*****************************************************************************/
/*************** Show forms to enter initial and ending dates ****************/
/*****************************************************************************/

void Dat_WriteFormIniEndDates (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Start_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_End_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Yesterday[Txt_NUM_LANGUAGES];
   extern const char *Txt_Today[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr><td align=\"center\">" \
                      "<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"right\" class=\"%s\">%s: </td>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Start_date[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"left\">");
   Dat_WriteFormDate (Sta_LOG_START_YEAR,Gbl.Today.Year,"StartDay","StartMonth","StartYear",
                      Gbl.DateRange.DateIni.Day,Gbl.DateRange.DateIni.Month,Gbl.DateRange.DateIni.Year,
                      false,false);
   fprintf (Gbl.F.Out,"</td><td rowspan=\"2\" align=\"center\" valign=\"middle\">" \
                      "<input type=\"button\" name=\"Yesterday\" value=\"%s\" onclick=\"setDateTo(this,%u,%u,%u)\" />" \
                      "<input type=\"button\" name=\"Today\" value=\"%s\" onclick=\"setDateTo(this,%u,%u,%u)\" /></td></tr>" \
                      "<tr><td align=\"right\" class=\"%s\">%s: </td>",
            Txt_Yesterday[Gbl.Prefs.Language],
	    Gbl.Yesterday.Day,Gbl.Yesterday.Month,Gbl.Yesterday.Year-Sta_LOG_START_YEAR+1,
            Txt_Today[Gbl.Prefs.Language],
            Gbl.Today.Day,Gbl.Today.Month,Gbl.Today.Year-Sta_LOG_START_YEAR+1,
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_End_date[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"left\">");
   Dat_WriteFormDate (Sta_LOG_START_YEAR,Gbl.Today.Year,"EndDay","EndMonth","EndYear",
                      Gbl.DateRange.DateEnd.Day,Gbl.DateRange.DateEnd.Month,Gbl.DateRange.DateEnd.Year,
                      false,false);
   fprintf (Gbl.F.Out,"</td></tr></table></td></tr>");
  }

/*****************************************************************************/
/************************* Show a form to enter a date ***********************/
/*****************************************************************************/
/*
See the following pages:
http://javascript.internet.com/forms/date-selection-form.html
http://www.java-scripts.net/javascripts/Calendar-Popup.phtml
http://webmonkey.wired.com/webmonkey/98/04/index3a_page10.html?tw=programming
http://www.jsmadeeasy.com/javascripts/Calendars/Select-A-Mo://tech.irt.org/articles/js068/calendar.htm
http://javascript.internet.com/calendars/select-a-month.html
http://javascript.internet.com/forms/country.html
http://javascript.internet.com/forms/category-selection.html
See also http://www.ashleyit.com/rs/jsrs/select/php/select.php
*/
void Dat_WriteFormDate (unsigned FirstYear,unsigned LastYear,
	                const char *NameSelectDay,const char *NameSelectMonth,const char *NameSelectYear,
		        unsigned DaySelected,unsigned MonthSelected,unsigned YearSelected,
                        bool SubmitFormOnChange,bool Disabled)
  {
   unsigned Day,Month,Year;
   unsigned NumDaysSelectedMonth;
   extern const char *Txt_MONTHS_SMALL[12][Txt_NUM_LANGUAGES];

   /***** Day *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"left\"><select id=\"%s\" name=\"%s\"",
            NameSelectDay,NameSelectDay);
   if (SubmitFormOnChange)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"",Gbl.NumForm);
   if (Disabled)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,"><option value=\"0\">-</option>");
   NumDaysSelectedMonth = (MonthSelected == 0) ? 31 : ((MonthSelected == 2) ? Dat_GetNumDaysFebruary (YearSelected) : NumDaysMonth[MonthSelected]);
   for (Day=1; Day<=NumDaysSelectedMonth; Day++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",Day);
      if (Day == DaySelected)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%u</option>",Day);
     }

   /***** Month *****/
   fprintf (Gbl.F.Out,"</select></td><td align=\"left\">" \
                      "<select id=\"%s\" name=\"%s\" onchange=\"adjustDateForm(this.form.%s,this.form.%s,this.form.%s)",
	    NameSelectMonth,NameSelectMonth,NameSelectDay,NameSelectMonth,NameSelectYear);
   if (SubmitFormOnChange)
      fprintf (Gbl.F.Out,";javascript:document.forms[%d].submit();",Gbl.NumForm);
   fprintf (Gbl.F.Out,"\"");
   if (Disabled)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,"><option value=\"0\">-</option>");
   for (Month=1; Month<=12; Month++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",Month);
      if (Month == MonthSelected)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_MONTHS_SMALL[Month-1][Gbl.Prefs.Language]);
     }

   /***** Year *****/
   fprintf (Gbl.F.Out,"</select></td><td align=\"left\">" \
                      "<select id=\"%s\" name=\"%s\" onchange=\"adjustDateForm(this.form.%s,this.form.%s,this.form.%s)",
	    NameSelectYear,NameSelectYear,NameSelectDay,NameSelectMonth,NameSelectYear);
   if (SubmitFormOnChange)
      fprintf (Gbl.F.Out,";javascript:document.forms[%d].submit();",Gbl.NumForm);
   fprintf (Gbl.F.Out,"\"");
   if (Disabled)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,"><option value=\"0\">-</option>");
   for (Year=FirstYear; Year<=LastYear; Year++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",Year);
      if (Year == YearSelected)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%u</option>",Year);
     }
   fprintf (Gbl.F.Out,"</select></td></tr></table>\n");
  }


/*****************************************************************************/
/*************** Show a form to enter a hour-minute time *********************/
/*****************************************************************************/

void Dat_WriteFormHourMinute (const char *NameSelectHour,const char *NameSelectMinute,
		              unsigned HourSelected,unsigned MinuteSelected,
                              bool SubmitFormOnChange,bool Disabled)
  {
   unsigned Hour,Minute;

   /***** Hour *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"left\"><select name=\"%s\"",
            NameSelectHour);
   if (SubmitFormOnChange)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"",Gbl.NumForm);
   if (Disabled)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,">");
   for (Hour=0; Hour<=23; Hour++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",Hour);
      if (Hour == HourSelected)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%02u h</option>",Hour);
     }

   /***** Minute *****/
   fprintf (Gbl.F.Out,"</select></td><td align=\"left\">" \
                      "<select name=\"%s\"",
	    NameSelectMinute);
   if (SubmitFormOnChange)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"",Gbl.NumForm);
   if (Disabled)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,">");
   for (Minute=0; Minute<=59; Minute++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",Minute);
      if (Minute == MinuteSelected)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%02u &#39;</option>",Minute);
     }
   fprintf (Gbl.F.Out,"</select></td></tr></table>\n");
  }

/****************************************************************************/
/*************************** Get a date from a form *************************/
/****************************************************************************/

void Dat_GetDateFromForm (const char *NameParamDay,const char *NameParamMonth,const char *NameParamYear,
                          unsigned *Day,unsigned *Month,unsigned *Year)
  {
   char UnsignedStr[10+1];

   /**** Get day ****/
   Par_GetParam (Par_PARAM_SINGLE,NameParamDay,UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",Day) != 1)
      *Day = 0;

   /**** Get month ****/
   Par_GetParam (Par_PARAM_SINGLE,NameParamMonth,UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",Month) != 1)
      *Month = 0;

   /**** Get year ****/
   Par_GetParam (Par_PARAM_SINGLE,NameParamYear,UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",Year) != 1)
      *Year = 0;
  }

/****************************************************************************/
/***************** Get an hour-minute time from a form **********************/
/****************************************************************************/

void Dat_GetHourMinuteFromForm (const char *NameParamHour,const char *NameParamMinute,
                                unsigned *Hour,unsigned *Minute)
  {
   char UnsignedStr[10+1];

   /**** Get hour ****/
   Par_GetParam (Par_PARAM_SINGLE,NameParamHour,UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",Hour) != 1)
      *Hour = 0;

   /**** Get minute ****/
   Par_GetParam (Par_PARAM_SINGLE,NameParamMinute,UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",Minute) != 1)
      *Minute = 0;
  }

/*****************************************************************************/
/************************** Get initial and end dates ************************/
/*****************************************************************************/

void Dat_GetIniEndDatesFromForm (void)
  {
   /***** Get initial date *****/
   Dat_GetDateFromForm ("StartDay","StartMonth","StartYear",
                        &Gbl.DateRange.DateIni.Day,
                        &Gbl.DateRange.DateIni.Month,
                        &Gbl.DateRange.DateIni.Year);
   if (Gbl.DateRange.DateIni.Day   == 0 ||
       Gbl.DateRange.DateIni.Month == 0 ||
       Gbl.DateRange.DateIni.Year  == 0)
     {
      Gbl.DateRange.DateIni.Day   = Sta_LOG_START_DAY;
      Gbl.DateRange.DateIni.Month = Sta_LOG_START_MONTH;
      Gbl.DateRange.DateIni.Year  = Sta_LOG_START_YEAR;
     }

   /***** Get end date *****/
   Dat_GetDateFromForm ("EndDay","EndMonth","EndYear",
                        &Gbl.DateRange.DateEnd.Day,
                        &Gbl.DateRange.DateEnd.Month,
                        &Gbl.DateRange.DateEnd.Year);
   if (Gbl.DateRange.DateEnd.Day   == 0 ||
       Gbl.DateRange.DateEnd.Month == 0 ||
       Gbl.DateRange.DateEnd.Year  == 0)
     {
      Gbl.DateRange.DateEnd.Day   = Gbl.Today.Day;
      Gbl.DateRange.DateEnd.Month = Gbl.Today.Month;
      Gbl.DateRange.DateEnd.Year  = Gbl.Today.Year;
     }
  }

/*****************************************************************************/
/******************************** Write a date *******************************/
/*****************************************************************************/
// DateTime must be in YYYYMMDD[...] format

void Dat_WriteDate (const char *YYYYMMDD)
  {
   extern const char *Txt_Today[Txt_NUM_LANGUAGES];

   if (strncmp (YYYYMMDD,Gbl.Today.YYYYMMDD,8))	// If date isn't today
      fprintf (Gbl.F.Out,"%c%c/%c%c/%c%c",
               YYYYMMDD[6],YYYYMMDD[7],
               YYYYMMDD[4],YYYYMMDD[5],
               YYYYMMDD[2],YYYYMMDD[3]);
   else						// If date is today
      fprintf (Gbl.F.Out,"%s",Txt_Today[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/************************* Write hour and minute *****************************/
/*****************************************************************************/
// DateTime must be in HHMM[SS] format

void Dat_WriteHourMinute (const char *HHMM)
  {
   fprintf (Gbl.F.Out,"%c%c:%c%c",
            HHMM[0],HHMM[1],
            HHMM[2],HHMM[3]);
  }

/****************************************************************************/
/************** Compute the preceding date of a given date ******************/
/****************************************************************************/

void Dat_GetDayBefore (struct Date *Date,struct Date *PrecedingDate)
  {
   if (Date->Day == 1)
     {
      if (Date->Month == 1)
        {
         PrecedingDate->Year  = Date->Year - 1;
         PrecedingDate->Month = 12;
         PrecedingDate->Day   = 31;
        }
      else
        {
         PrecedingDate->Year  = Date->Year;
         PrecedingDate->Month = Date->Month - 1;
         PrecedingDate->Day   = (PrecedingDate->Month == 2) ? Dat_GetNumDaysFebruary (PrecedingDate->Year) : NumDaysMonth[PrecedingDate->Month];
        }
     }
   else
     {
      PrecedingDate->Year  = Date->Year;
      PrecedingDate->Month = Date->Month;
      PrecedingDate->Day   = Date->Day - 1;
     }
  }

/****************************************************************************/
/**************** Compute the week before to a given week *******************/
/****************************************************************************/

void Dat_GetWeekBefore (struct Date *Date,struct Date *PrecedingDate)
  {
   if (Date->Week == 1)
     {
      PrecedingDate->Year = Date->Year - 1;
      PrecedingDate->Week = Dat_GetNumWeeksInYear (PrecedingDate->Year);
     }
   else
     {
      PrecedingDate->Year = Date->Year;
      PrecedingDate->Week = Date->Week - 1;
     }
  }

/****************************************************************************/
/*************** Compute the month before to a given month ******************/
/****************************************************************************/

void Dat_GetMonthBefore (struct Date *Date,struct Date *PrecedingDate)
  {
   if (Date->Month == 1)
     {
      PrecedingDate->Year  = Date->Year - 1;
      PrecedingDate->Month = 12;
     }
   else
     {
      PrecedingDate->Year  = Date->Year;
      PrecedingDate->Month = Date->Month - 1;
     }
  }

/*****************************************************************************/
/************** Compute the number of days beteen two dates ******************/
/*****************************************************************************/
// If the dates are the same, return 1
// If the old date is the day before the new data, return 2
// ...

unsigned Dat_GetNumDaysBetweenDates (struct Date *DateIni,struct Date *DateEnd)
  {
   int DiffDays;
   unsigned Year;

   /***** If initial year is less than end year, return 0
          (actually the difference in days should be negative) *****/
   if (DateIni->Year > DateEnd->Year)
      return 0;

   /***** Initial year is less or equal than end year ==> compute difference in days *****/
   DiffDays = (int) Dat_GetDayOfYear (DateEnd) - (int) Dat_GetDayOfYear (DateIni) + 1;
   for (Year=DateIni->Year; Year<DateEnd->Year; Year++)
      DiffDays += (int) Dat_GetNumDaysInYear (Year);
   return (DiffDays >= 0) ? (unsigned) DiffDays : 0;
  }

/*****************************************************************************/
/*************** Compute the number of weeks between two dates ***************/
/*****************************************************************************/
// If the two dates are in the same week, return 1

unsigned Dat_GetNumWeeksBetweenDates (struct Date *DateIni,struct Date *DateEnd)
  {
   int DiffWeeks;
   unsigned Year;

   /***** If initial year is lower than the ending year, return 0
          (actually the difference should be negative) *****/
   if (DateIni->Year > DateEnd->Year)
      return 0;

   /***** Initial year is lower or equal to ending year ==> compute difference in weeks *****/
   DiffWeeks = (int) DateEnd->Week - (int) DateIni->Week + 1;
   for (Year=DateIni->Year; Year<DateEnd->Year; Year++)
      DiffWeeks += (int) Dat_GetNumWeeksInYear (Year);
   return (DiffWeeks >= 0) ? (unsigned) DiffWeeks : 0;
  }

/*****************************************************************************/
/************* Compute the number of months between two dates ****************/
/*****************************************************************************/
// If the two dates are in the same month, return 1

unsigned Dat_GetNumMonthsBetweenDates (struct Date *DateIni,struct Date *DateEnd)
  {
   int DiffMonths;

   /***** Compute the difference in months *****/
   DiffMonths = ((int) DateEnd->Year - (int) DateIni->Year) * 12 + (int) DateEnd->Month - (int) DateIni->Month + 1;
   return (DiffMonths >= 0) ? (unsigned) DiffMonths : 0;
  }

/*****************************************************************************/
/*************** Compute the number of days in a given year ******************/
/*****************************************************************************/

#define NUM_DAYS_YEAR_EXCEPT_FEBRUARY (365 - 28)

unsigned Dat_GetNumDaysInYear (unsigned Year)
  {
   return NUM_DAYS_YEAR_EXCEPT_FEBRUARY + Dat_GetNumDaysFebruary (Year);
  }

/****************************************************************************/
/****************** Return the number of days of february *******************/
/****************************************************************************/

unsigned Dat_GetNumDaysFebruary (unsigned Year)
  {
   return (Dat_GetIfLeapYear (Year) ? 29 : 28);
  }

/****************************************************************************/
/************************* Return true if year is leap **********************/
/****************************************************************************/

bool Dat_GetIfLeapYear (unsigned Year)
  {
   return (Year % 4 == 0) && ((Year % 100 != 0) || (Year % 400 == 0));
  }

/*****************************************************************************/
/******* Compute the number of weeks starting in monday of a given year ******/
/*****************************************************************************/
// A week starting on monday is considered belonging to a year only if it has most of its days (4 to 7) belonging to that year
// A year may have 52 (usually) or 53 of those weeks

unsigned Dat_GetNumWeeksInYear (unsigned Year)
  {
   unsigned December31DayOfWeek = Dat_GetDayOfWeek (Year,12,31); // From 0 to 6

   if (December31DayOfWeek == 3)
      return 53;
   if (December31DayOfWeek == 4)
      if (Dat_GetIfLeapYear (Year))
         return 53;
   return 52;
  }

/****************************************************************************/
/***************** Compute day of the week from a given date ****************/
/****************************************************************************/
// Return 0 for monday, 1 for tuesday,... 6 for sunday

unsigned Dat_GetDayOfWeek (unsigned Year,unsigned Month,unsigned Day)
  {
   if (Month <= 2)
     {
      Month += 12;
      Year--;
     }
   return (((Day+(Month*2)+(((Month+1)*3)/5)+Year+(Year/4-Year/100+Year/400)+2) % 7) + 5) % 7;
  }

/****************************************************************************/
/***************** Compute the day of year (from 1 to 366) ******************/
/****************************************************************************/

unsigned Dat_GetDayOfYear (struct Date *Date)
  {
   unsigned DayYear,Month;

   /* Compute day of year */
   for (DayYear=0, Month=1; Month<Date->Month; Month++)
      DayYear += (Month == 2) ? Dat_GetNumDaysFebruary (Date->Year) : NumDaysMonth[Month];
   DayYear += Date->Day;

   return DayYear;
  }

/****************************************************************************/
/******** Compute Week (and possibly updated Year) from a given date ********/
/****************************************************************************/
/* The weeks always are counted from monday to sunday.
   01/01/2006 was sunday => it is counted in the week 52 of 2005, which goes from the 26/12/2005 (monday) until the 01/01/2006 (sunday).
   Week 1 of 2006 goes from the 02/01/2006 (monday) until the 08/01/2006 (sunday) */

void Dat_CalculateWeekOfYear (struct Date *Date)
  {
   /*
   If January 1 of Year is...   | ...then first week of Year starts the day  |
   -----------------------------+--------------------------------------------+
   (0) Monday                   | January,   1 of Year                       |
   (1) Tuesday                  | December, 31 of Year-1                     |
   (2) Wednesday                | December, 30 of Year-1                     |
   (3) Thursday                 | December, 29 of Year-1                     |
   (4) Friday                   | January,   4 of Year                       |
   (5) Saturday                 | January,   5 of Year                       |
   (6) Sunday                   | January,   6 of Year                       |
   If December 31 of Year is... | ...then last week of Year ends the day     |
   -----------------------------+--------------------------------------------+
   (0) Monday                   | December, 30 of Year                       |
   (1) Tuesday                  | December, 29 of Year                       |
   (2) Wednesday                | December, 28 of Year                       |
   (3) Thursday                 | January,   3 of Year+1                     |
   (4) Friday                   | January,   4 of Year+1                     |
   (5) Saturday                 | January,   5 of Year+1                     |
   (6) Sunday                   | December, 31 of Year                       |
   */
   int DayThatFirstWeekStarts[7] = {1,0,-1,-2,4,3,2};
   unsigned DayThatLastWeekEnds[7] = {30,29,28,34,33,32,31};
   unsigned January1DayOfWeek = Dat_GetDayOfWeek (Date->Year,1,1); // From 0 to 6
   unsigned December31DayOfWeek = Dat_GetDayOfWeek (Date->Year,12,31); // From 0 to 6

   if (Date->Month == 1 && (int) Date->Day < DayThatFirstWeekStarts[January1DayOfWeek])	// Week is the last week of the year before this
     {
      Date->Year--;
      Date->Week = Dat_GetNumWeeksInYear (Date->Year);
     }
   else if (Date->Month == 12 && Date->Day > DayThatLastWeekEnds[December31DayOfWeek])	// Week is the first week of the year after this
     {
      Date->Year++;
      Date->Week = 1;
     }
   else	// Week corresponds to this year
      Date->Week = ((int) Dat_GetDayOfYear (Date) - DayThatFirstWeekStarts[January1DayOfWeek]) / 7 + 1;
  }

/****************************************************************************/
/******************* Assign the values of a date to another *****************/
/****************************************************************************/

void Dat_AssignDate (struct Date *DateDst,struct Date *DateSrc)
  {
   DateDst->Year  = DateSrc->Year;
   DateDst->Month = DateSrc->Month;
   DateDst->Day   = DateSrc->Day;
   DateDst->Week  = DateSrc->Week;
  }

/****************************************************************************/
/****************************** Compare dates *******************************/
/****************************************************************************/
// Return <0 if Date1<Date2
// Return >0 if Date1>Date2
// Return  0 if Date1==Date2

int Dat_CompareDates (struct Date *Date1,struct Date *Date2)
  {
   if (Date1->Year  < Date2->Year ) return -1;
   if (Date1->Year  > Date2->Year ) return 1;

   if (Date1->Month < Date2->Month) return -1;
   if (Date1->Month > Date2->Month) return 1;

   if (Date1->Day   < Date2->Day  ) return -1;
   if (Date1->Day   > Date2->Day  ) return 1;

   return 0;
  }

/****************************************************************************/
/************************** Compare datetimes *******************************/
/****************************************************************************/
// Return <0 if DateTime1<DateTime2
// Return >0 if DateTime1>DateTime2
// Return  0 if DateTime1==DateTime2

int Dat_CompareDateTimes (struct DateTime *DateTime1,struct DateTime *DateTime2)
  {
   if (DateTime1->Year   < DateTime2->Year  ) return -1;
   if (DateTime1->Year   > DateTime2->Year  ) return 1;

   if (DateTime1->Month  < DateTime2->Month ) return -1;
   if (DateTime1->Month  > DateTime2->Month ) return 1;

   if (DateTime1->Day    < DateTime2->Day   ) return -1;
   if (DateTime1->Day    > DateTime2->Day   ) return 1;

   if (DateTime1->Hour   < DateTime2->Hour  ) return -1;
   if (DateTime1->Hour   > DateTime2->Hour  ) return 1;

   if (DateTime1->Minute < DateTime2->Minute) return -1;
   if (DateTime1->Minute > DateTime2->Minute) return 1;

   if (DateTime1->Second < DateTime2->Second) return -1;
   if (DateTime1->Second > DateTime2->Second) return 1;

   return 0;
  }
