// swad_dat.h: dates

#ifndef _SWAD_DAT
#define _SWAD_DAT
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <time.h>

#include "swad_boo.h"

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

#define Dat_SECONDS_IN_ONE_MONTH (30UL*24UL*60UL*60UL)

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

struct Date
  {
   unsigned Day,Month,Year,Week;
   char YYYYMMDD[4+2+2+1];
  };
struct Hour
  {
   unsigned Hour,Minute;
  };
struct DateTime
  {
   unsigned Year,Month,Day,Hour,Minute,Second,Week;
   char YYYYMMDDHHMMSS[4+2+2+2+2+2+1];
  };

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Dat_GetTimeStartExecution (void);
void Dat_GetAndConvertCurrentDateTime (void);

bool Dat_GetDateFromYYYYMMDD (struct Date *Date,const char *YYYYMMDDString);
bool Dat_GetDateTimeFromYYYYMMDDHHMMSS (struct DateTime *DateTime,const char *YYYYMMDDHHMMSS);

void Dat_DrawMonthAndHour (void);
void Deg_DrawCalendar (void);
void Dat_DrawMonth (unsigned RealYear,unsigned RealMonth,bool DrawingCalendar,bool PutLinkToEvents,bool PrintView);

void Dat_GetLocalTimeFromClock (const time_t *clock);
void Dat_WriteDateFromtblock (void);
void Dat_ConvDateToDateStr (struct Date *Date,char *DateStr);

void Dat_WriteFormIniEndDates (void);
void Dat_WriteFormDate (unsigned FirstYear,unsigned LastYear,
	                const char *NameSelectDay,const char *NameSelectMonth,const char *NameSelectYear,
		        unsigned DaySelected,unsigned MonthSelected,unsigned YearSelected,
                        bool SubmitFormOnChange,bool Disabled);
void Dat_WriteFormHourMinute (const char *NameSelectHour,const char *NameSelectMinute,
		              unsigned HourSelected,unsigned MinuteSelected,
                              bool SubmitFormOnChange,bool Disabled);
void Dat_GetDateFromForm (const char *NameParamDay,const char *NameParamMonth,const char *NameParamYear,
                          unsigned *Day,unsigned *Month,unsigned *Year);
void Dat_GetHourMinuteFromForm (const char *NameParamHour,const char *NameParamMinute,
                                unsigned *Hour,unsigned *Minute);

void Dat_GetIniEndDatesFromForm (void);
void Dat_WriteDate (const char *YYYYMMDD);
void Dat_WriteHourMinute (const char *HHMM);
void Dat_GetDayBefore (struct Date *Date,struct Date *PrecedingDate);
void Dat_GetWeekBefore (struct Date *Date,struct Date *PrecedingDate);
void Dat_GetMonthBefore (struct Date *Date,struct Date *PrecedingDate);
unsigned Dat_GetNumDaysBetweenDates (struct Date *DateIni,struct Date *DateEnd);
unsigned Dat_GetNumWeeksBetweenDates (struct Date *DateIni,struct Date *DateEnd);
unsigned Dat_GetNumMonthsBetweenDates (struct Date *DateIni,struct Date *DateEnd);
unsigned Dat_GetNumDaysInYear (unsigned Year);
unsigned Dat_GetNumDaysFebruary (unsigned Year);
bool Dat_GetIfLeapYear (unsigned Year);
unsigned Dat_GetNumWeeksInYear (unsigned Year);
unsigned Dat_GetDayOfWeek (unsigned Year,unsigned Month,unsigned Day);
unsigned Dat_GetDayOfYear (struct Date *Date);
void Dat_CalculateWeekOfYear (struct Date *Date);
void Dat_AssignDate (struct Date *DateDst,struct Date *DateSrc);

int Dat_CompareDates (struct Date *Date1,struct Date *Date2);
int Dat_CompareDateTimes (struct DateTime *DateTime1,struct DateTime *DateTime2);

#endif
