// swad_db.c: database

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <mysql/mysql.h>	// To access MySQL databases
#include <stdio.h>		// For FILE,fprintf

#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/************************ Internal global variables *************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Database tables ******************************/
/****************************************************************************/

/*
Database:
shell> mysql -h swad.ugr.es -u swad -p
mysql> CREATE DATABASE IF NOT EXIST swad;
mysql> SHOW databases;
mysql> USE swad
mysql> SHOW tables;
mysql> DROP TABLE table_to_delete;							// Delete a table
mysql> TRUNCATE TABLE table_to_delete;							// Delete all rows
shell> mysqldump -l -h swad.ugr.es -u swad -p swad > copia_base_datos_swad		// Backup of the whole database
mysql> UPDATE table SET CrsCod='new_code' WHERE CrsCod='old_code';			// Change a course code
mysql> ALTER TABLE table ADD COLUMN new_column_definition AFTER Columna;		// Add a new column to a table
mysql> ALTER TABLE table RENAME TO new_table_name;					// Rename a table
MYSQL> ALTER TABLE table MODIFY COLUMN new_column_definition;
mysql> UPDATE messages SET Content=REPLACE(Content,'<BR>','<br />');			// Change <BR> by <br /> in messages
*/

/****************************************************************************/
/****************** Create database tables if not exist *********************/
/****************************************************************************/

void DB_CreateTablesIfNotExist (void)
  {
   extern const char *Txt_Created_tables_in_the_database_that_did_not_exist[Txt_NUM_LANGUAGES];
   char Query[4096];

   /***** Information message *****/
   Lay_ShowAlert (INFO,"Creating database tables if not exist...");	// Need translation!
   fprintf (Gbl.F.Out,"<ol>");

   /***** Table IP_prefs *****/
/*
mysql> DESCRIBE IP_prefs;
+------------+------------+------+-----+---------+-------+
| Field      | Type       | Null | Key | Default | Extra |
+------------+------------+------+-----+---------+-------+
| IP         | char(40)   | NO   | PRI | NULL    |       |
| UsrCod     | int(11)    | NO   | MUL | -1      |       |
| LastChange | datetime   | NO   | MUL | NULL    |       |
| Theme      | char(16)   | NO   |     | NULL    |       |
| Language   | char(2)    | NO   |     | NULL    |       |
| SideCols   | tinyint(4) | NO   |     | 3       |       |
+------------+------------+------+-----+---------+-------+
6 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS IP_prefs (" \
                  "IP CHAR(40) NOT NULL," \
                  "UsrCod INT NOT NULL DEFAULT -1," \
                  "LastChange DATETIME NOT NULL," \
                  "Theme CHAR(16) NOT NULL," \
                  "Language CHAR(2) NOT NULL," \
                  "SideCols TINYINT NOT NULL," \
                  "PRIMARY KEY (IP),INDEX(UsrCod),INDEX(LastChange))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table asg_grp *****/
/*
mysql> DESCRIBE asg_grp;
+--------+---------+------+-----+---------+-------+
| Field  | Type    | Null | Key | Default | Extra |
+--------+---------+------+-----+---------+-------+
| AsgCod | int(11) | NO   | PRI | NULL    |       |
| GrpCod | int(11) | NO   | PRI | NULL    |       |
+--------+---------+------+-----+---------+-------+
2 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS asg_grp (" \
                  "AsgCod INT NOT NULL," \
                  "GrpCod INT NOT NULL," \
                  "UNIQUE INDEX(AsgCod,GrpCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");
      
   /***** Table assignments *****/
/*
mysql> DESCRIBE assignments;
+-----------+---------------+------+-----+---------+----------------+
| Field     | Type          | Null | Key | Default | Extra          |
+-----------+---------------+------+-----+---------+----------------+
| AsgCod    | int(11)       | NO   | PRI | NULL    | auto_increment |
| CrsCod    | int(11)       | NO   | MUL | -1      |                |
| UsrCod    | int(11)       | NO   |     | NULL    |                |
| StartTime | datetime      | NO   |     | NULL    |                |
| EndTime   | datetime      | NO   |     | NULL    |                |
| Title     | varchar(255)  | NO   |     | NULL    |                |
| SendWork  | enum('N','Y') | NO   |     | N       |                |
| Folder    | varbinary(32) | NO   |     | NULL    |                |
| Txt       | text          | NO   |     | NULL    |                |
+-----------+---------------+------+-----+---------+----------------+
9 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS assignments (" \
                  "AsgCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "UsrCod INT NOT NULL," \
                  "StartTime DATETIME NOT NULL," \
                  "EndTime DATETIME NOT NULL," \
                  "Title VARCHAR(255) NOT NULL," \
                  "SendWork ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Folder VARBINARY(32) NOT NULL," \
                  "Txt TEXT NOT NULL," \
                  "UNIQUE INDEX (AsgCod),INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table centres *****/
/*
mysql> DESCRIBE centres;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| CtrCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| InsCod    | int(11)      | NO   | MUL |         |                |
| ShortName | varchar(32)  | NO   |     |         |                |
| FullName  | varchar(255) | NO   |     |         |                |
| WWW       | varchar(255) | NO   |     |         |                |
+-----------+--------------+------+-----+---------+----------------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS centres (" \
                  "CtrCod INT NOT NULL AUTO_INCREMENT," \
                  "InsCod INT NOT NULL," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "WWW VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(CtrCod), INDEX(InsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table chat *****/
/*
mysql> DESCRIBE chat;
+----------+--------------+------+-----+---------+-------+
| Field    | Type         | Null | Key | Default | Extra |
+----------+--------------+------+-----+---------+-------+
| RoomCode | varchar(255) | NO   | PRI |         |       |
| NumUsrs  | int(11)      | NO   |     |         |       |
+----------+--------------+------+-----+---------+-------+
2 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS chat ("\
                  "RoomCode VARCHAR(255) NOT NULL," \
                  "NumUsrs INT NOT NULL," \
                  "UNIQUE INDEX(RoomCode))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table clicks_without_photo *****/
/*
mysql> DESCRIBE clicks_without_photo;
+-----------+---------+------+-----+---------+-------+
| Field     | Type    | Null | Key | Default | Extra |
+-----------+---------+------+-----+---------+-------+
| UsrCod    | int(11) | NO   | PRI |         |       |
| NumClicks | int(11) | NO   |     |         |       |
+-----------+---------+------+-----+---------+-------+
2 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS clicks_without_photo (" \
                  "UsrCod INT NOT NULL," \
                  "NumClicks INT NOT NULL," \
                  "UNIQUE INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table clipboard *****/
/*
mysql> DESCRIBE clipboard;
+-------------+------------+------+-----+-------------------+-------+
| Field       | Type       | Null | Key | Default           | Extra |
+-------------+------------+------+-----+-------------------+-------+
| UsrCod      | int(11)    | NO   | PRI | NULL              |       |
| FileBrowser | tinyint(4) | NO   | MUL | NULL              |       |
| CrsCod      | int(11)    | NO   | MUL | -1                |       |
| GrpCod      | int(11)    | NO   |     | NULL              |       |
| WorksUsrCod | int(11)    | NO   |     | NULL              |       |
| Path        | text       | NO   |     | NULL              |       |
| CopyTime    | timestamp  | NO   |     | CURRENT_TIMESTAMP |       |
+-------------+------------+------+-----+-------------------+-------+
7 rows in set (0.70 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS clipboard (" \
                  "UsrCod INT NOT NULL," \
                  "FileBrowser TINYINT NOT NULL," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL," \
                  "WorksUsrCod INT NOT NULL," \
                  "Path TEXT COLLATE latin1_bin NOT NULL," \
                  "CopyTime TIMESTAMP," \
                  "UNIQUE INDEX(UsrCod),INDEX(FileBrowser),INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table connected *****/
/*
mysql> DESCRIBE connected;
+----------+------------+------+-----+---------+-------+
| Field    | Type       | Null | Key | Default | Extra |
+----------+------------+------+-----+---------+-------+
| UsrCod   | int(11)    | NO   | PRI |         |       |
| UsrType  | tinyint(4) | NO   |     |         |       |
| CrsCod   | int(11)    | NO   | MUL | -1      |       |
| LastTime | datetime   | YES  |     | NULL    |       |
+----------+------------+------+-----+---------+-------+
4 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS connected (" \
                  "UsrCod INT NOT NULL," \
                  "UsrType TINYINT NOT NULL," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "LastTime DATETIME," \
                  "UNIQUE INDEX(UsrCod),INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table courses *****/
/*
mysql> DESCRIBE courses;
+----------------+--------------+------+-----+---------+----------------+
| Field          | Type         | Null | Key | Default | Extra          |
+----------------+--------------+------+-----+---------+----------------+
| CrsCod         | int(11)      | NO   | PRI | NULL    | auto_inrement |
| DegCod         | int(11)      | NO   | MUL | -1      |                |
| Year           | tinyint(4)   | NO   |     | 0       |                |
| Semester       | tinyint(4)   | NO   |     | 0       |                |
| ShortName      | varchar(32)  | NO   |     |         |                |
| FullName       | varchar(255) | NO   |     |         |                |
+----------------+--------------+------+-----+---------+----------------+
7 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS courses (" \
                  "CrsCod INT NOT NULL AUTO_INCREMENT," \
                  "DegCod INT NOT NULL DEFAULT -1," \
                  "Year TINYINT NOT NULL DEFAULT 0," \
                  "Semester TINYINT NOT NULL DEFAULT 0," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(CrsCod), INDEX(DegCod,Year,Semester))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_common_files *****/
/*
mysql> DESCRIBE crs_common_files;
+--------+---------+------+-----+---------+-------+
| Field  | Type    | Null | Key | Default | Extra |
+--------+---------+------+-----+---------+-------+
| CrsCod | int(11) | NO   | MUL | -1      |       |
| GrpCod | int(11) | NO   |     |         |       |
| UsrCod | int(11) | NO   | MUL |         |       |
| Path   | text    | NO   |     |         |       |
+--------+---------+------+-----+---------+-------+
4 rows in set (0.01 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_common_files (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "Path TEXT COLLATE latin1_bin NOT NULL," \
                  "INDEX(CrsCod,GrpCod),INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_grp *****/
/*
mysql> DESCRIBE crs_grp;
+-------------+---------------+------+-----+---------+----------------+
| Field       | Type          | Null | Key | Default | Extra          |
+-------------+---------------+------+-----+---------+----------------+
| GrpCod      | int(11)       | NO   | PRI | NULL    | auto_increment |
| GrpTypCod   | int(11)       | NO   | MUL | NULL    |                |
| GrpName     | varchar(255)  | NO   |     | NULL    |                |
| MaxStudents | int(11)       | NO   |     | NULL    |                |
| NumStudents | int(11)       | NO   |     | NULL    |                |
| Open        | enum('N','Y') | NO   |     | N       |                |
| FileZones   | enum('N','Y') | NO   |     | N       |                |
+-------------+---------------+------+-----+---------+----------------+
7 rows in set (0.12 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_grp (" \
                  "GrpCod INT NOT NULL AUTO_INCREMENT," \
                  "GrpTypCod INT NOT NULL," \
                  "GrpName VARCHAR(255) NOT NULL," \
                  "MaxStudents INT NOT NULL," \
                  "NumStudents INT NOT NULL," \
                  "Open ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "FileZones ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "UNIQUE INDEX(GrpCod),INDEX(GrpTypCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_grp_types *****/
/*
mysql> DESCRIBE crs_grp_types;
+------------+---------------+------+-----+---------+----------------+
| Field      | Type          | Null | Key | Default | Extra          |
+------------+---------------+------+-----+---------+----------------+
| GrpTypCod  | int(11)       | NO   | PRI | NULL    | auto_increment |
| CrsCod     | int(11)       | NO   | MUL | -1      |                |
| GrpTypName | varchar(255)  | NO   |     |         |                |
| Mandatory  | enum('N','Y') | NO   |     | N       |                |
| Multiple   | enum('N','Y') | NO   |     | N       |                |
+------------+---------------+------+-----+---------+----------------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_grp_types (" \
                  "GrpTypCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpTypName VARCHAR(255) NOT NULL," \
                  "Mandatory ENUM('N','Y') NOT NULL," \
                  "Multiple ENUM('N','Y') NOT NULL," \
                  "UNIQUE INDEX(GrpTypCod), INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_grp_usr *****/
/*
mysql> DESCRIBE crs_grp_usr;
+--------------------+----------+------+-----+---------+-------+
| Field              | Type     | Null | Key | Default | Extra |
+--------------------+----------+------+-----+---------+-------+
| GrpCod             | int(11)  | NO   | PRI | NULL    |       |
| UsrCod             | int(11)  | NO   | PRI | NULL    |       |
| LastAccDownloadGrp | datetime | NO   |     | NULL    |       |
| LastAccCommonGrp   | datetime | NO   |     | NULL    |       |
| LastAccMarksGrp    | datetime | NO   |     | NULL    |       |
+--------------------+----------+------+-----+---------+-------+
5 rows in set (0.01 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_grp_usr (" \
                  "GrpCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "LastAccDownloadGrp DATETIME NOT NULL," \
                  "LastAccCommonGrp DATETIME NOT NULL," \
                  "LastAccMarksGrp DATETIME NOT NULL," \
                  "UNIQUE INDEX(GrpCod,UsrCod),INDEX(GrpCod),INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_info_src *****/
/*
mysql> DESCRIBE crs_info_src;
+----------+------------------------------------------------------------------------------------+------+-----+-------------+-------+
| Field    | Type                                                                               | Null | Key | Default     | Extra |
+----------+------------------------------------------------------------------------------------+------+-----+-------------+-------+
| CrsCod   | int(11)                                                                            | NO   | PRI | -1          |       |
| InfoType | enum('description','theory','practices','bibliography','FAQ','links','assessment') | NO   | PRI | description |       |
| InfoSrc  | enum('none','editor','text','page','URL')                                          | NO   |     | none        |       |
+----------+------------------------------------------------------------------------------------+------+-----+-------------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_info_src (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "InfoType ENUM('description','theory','practices','bibliography','FAQ','links','assessment') NOT NULL," \
                  "InfoSrc ENUM('none','editor','text','page','URL') NOT NULL," \
                  "UNIQUE INDEX(CrsCod,InfoType))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_info_txt *****/
/*
mysql> DESCRIBE crs_info_txt;
+----------+------------------------------------------------------------------------------------+------+-----+-------------+-------+
| Field    | Type                                                                               | Null | Key | Default     | Extra |
+----------+------------------------------------------------------------------------------------+------+-----+-------------+-------+
| CrsCod   | int(11)                                                                            | NO   | PRI | -1          |       |
| InfoType | enum('description','theory','practices','bibliography','FAQ','links','assessment') | NO   | PRI | description |       |
| InfoTxt  | longtext                                                                           | NO   |     |             |       |
+----------+------------------------------------------------------------------------------------+------+-----+-------------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_info_txt (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "InfoType ENUM('description','theory','practices','bibliography','FAQ','links','assessment') NOT NULL," \
                  "InfoTxt LONGTEXT NOT NULL," \
                  "UNIQUE INDEX(CrsCod,InfoType))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_record_fields *****/
/*
mysql> DESCRIBE crs_record_fields;
+------------+--------------+------+-----+---------+----------------+
| Field      | Type         | Null | Key | Default | Extra          |
+------------+--------------+------+-----+---------+----------------+
| FieldCod   | int(11)      | NO   | PRI | NULL    | auto_increment |
| CrsCod     | int(11)      | NO   | MUL | -1      |                |
| FieldName  | varchar(255) | NO   |     | NULL    |                |
| NumLines   | int(11)      | NO   |     | NULL    |                |
| Visibility | tinyint(4)   | NO   |     | NULL    |                |
+------------+--------------+------+-----+---------+----------------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_record_fields (" \
                  "FieldCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "FieldName VARCHAR(255) NOT NULL," \
                  "NumLines INT NOT NULL," \
                  "Visibility TINYINT NOT NULL," \
                  "UNIQUE INDEX(FieldCod),INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_records *****/
/*
mysql> DESCRIBE crs_records;
+----------+---------+------+-----+---------+-------+
| Field    | Type    | Null | Key | Default | Extra |
+----------+---------+------+-----+---------+-------+
| UsrCod   | int(11) | NO   | PRI | NULL    |       |
| FieldCod | int(11) | NO   | PRI | NULL    |       |
| Txt      | text    | NO   |     | NULL    |       |
+----------+---------+------+-----+---------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_records (" \
                  "FieldCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "Txt TEXT NOT NULL," \
                  "UNIQUE INDEX(FieldCod,UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table crs_usr *****/
/*
mysql> DESCRIBE crs_usr;
+--------------------+---------------------------+------+-----+------------+-------+
| Field              | Type                      | Null | Key | Default    | Extra |
+--------------------+---------------------------+------+-----+------------+-------+
| CrsCod             | int(11)                   | NO   | PRI | -1         |       |
| UsrCod             | int(11)                   | NO   | PRI | NULL       |       |
| Accepted           | enum('N','Y')             | NO   |     | N          |       |
| LastAccDownloadCrs | datetime                  | NO   |     | NULL       |       |
| LastAccCommonCrs   | datetime                  | NO   |     | NULL       |       |
| LastAccMyWorks     | datetime                  | NO   |     | NULL       |       |
| LastAccCrsWorks    | datetime                  | NO   |     | NULL       |       |
| LastAccMarksCrs    | datetime                  | NO   |     | NULL       |       |
| LastDowGrpCod      | int(11)                   | NO   |     | -1         |       |
| LastComGrpCod      | int(11)                   | NO   |     | -1         |       |
| LastAssGrpCod      | int(11)                   | NO   |     | -1         |       |
| NumAccTst          | int(11)                   | NO   |     | 0          |       |
| NextAccTst         | datetime                  | NO   |     | NULL       |       |
| UsrListType        | enum('classphoto','list') | NO   |     | classphoto |       |
| ColsClassPhoto     | tinyint(4)                | NO   |     | NULL       |       |
| ListWithPhotos     | enum('N','Y')             | NO   |     | Y          |       |
+--------------------+---------------------------+------+-----+------------+-------+
16 rows in set (0.01 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS crs_usr (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "UsrCod INT NOT NULL," \
                  "Accepted ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "LastAccDownloadCrs DATETIME NOT NULL," \
                  "LastAccCommonCrs DATETIME NOT NULL," \
                  "LastAccMyWorks DATETIME NOT NULL," \
                  "LastAccCrsWorks DATETIME NOT NULL," \
                  "LastAccMarksCrs DATETIME NOT NULL," \
                  "LastDowGrpCod INT NOT NULL DEFAULT -1," \
                  "LastComGrpCod INT NOT NULL DEFAULT -1," \
                  "LastAssGrpCod INT NOT NULL DEFAULT -1," \
                  "NumAccTst INT NOT NULL DEFAULT 0," \
                  "NextAccTst DATETIME NOT NULL," \
                  "UsrListType ENUM('classphoto','list') NOT NULL DEFAULT 'classphoto'," \
                  "ColsClassPhoto TINYINT NOT NULL," \
                  "ListWithPhotos ENUM('N','Y') NOT NULL DEFAULT 'Y'," \
                  "UNIQUE INDEX(CrsCod,UsrCod),INDEX(CrsCod),INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table deg_admin *****/
/*
mysql> DESCRIBE deg_admin;
+--------+---------+------+-----+---------+-------+
| Field  | Type    | Null | Key | Default | Extra |
+--------+---------+------+-----+---------+-------+
| UsrCod | int(11) | NO   | PRI |         |       |
| DegCod | int(11) | NO   | PRI |         |       |
+--------+---------+------+-----+---------+-------+
2 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS deg_admin (" \
                  "UsrCod INT NOT NULL," \
                  "DegCod INT NOT NULL," \
                  "UNIQUE INDEX(UsrCod,DegCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table deg_types *****/
/*
mysql> DESCRIBE deg_types;
+------------+--------------+------+-----+---------+----------------+
| Field      | Type         | Null | Key | Default | Extra          |
+------------+--------------+------+-----+---------+----------------+
| DegTypCod  | int(11)      | NO   | PRI | NULL    | auto_increment |
| DegTypName | varchar(255) | NO   |     |         |                |
+------------+--------------+------+-----+---------+----------------+
2 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS deg_types (" \
                  "DegTypCod INT NOT NULL AUTO_INCREMENT," \
                  "DegTypName VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(DegTypCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table degrees *****/
/*
mysql> DESCRIBE degrees;
+-----------+---------------+------+-----+---------+----------------+
| Field     | Type          | Null | Key | Default | Extra          |
+-----------+---------------+------+-----+---------+----------------+
| DegCod    | int(11)       | NO   | PRI | NULL    | auto_increment |
| DegTypCod | int(11)       | NO   | MUL |         |                |
| InsCod    | int(11)       | NO   | MUL |         |                |
| PlcCod    | int(11)       | NO   | MUL |         |                |
| ShortName | varchar(32)   | NO   |     |         |                |
| FullName  | varchar(255)  | NO   |     |         |                |
| FirstYear | tinyint(4)    | NO   |     | 0       |                |
| LastYear  | tinyint(4)    | NO   |     | 0       |                |
| OptYear   | enum('N','Y') | NO   |     | N       |                |
| Logo      | varchar(16)   | NO   |     |         |                |
| WWW       | varchar(255)  | NO   |     |         |                |
+-----------+---------------+------+-----+---------+----------------+
11 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS degrees (" \
                  "DegCod INT NOT NULL AUTO_INCREMENT," \
                  "DegTypCod INT NOT NULL," \
                  "InsCod INT NOT NULL," \
                  "PlcCod INT NOT NULL," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "FirstYear TINYINT NOT NULL DEFAULT 0," \
                  "LastYear TINYINT NOT NULL DEFAULT 0," \
                  "OptYear ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Logo VARCHAR(16) NOT NULL," \
                  "WWW VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(DegCod), INDEX(DegTypCod), INDEX(InsCod), INDEX(PlcCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table departments *****/
/*
mysql> DESCRIBE departments;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| DptCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| InsCod    | int(11)      | NO   | MUL |         |                |
| ShortName | varchar(32)  | NO   |     |         |                |
| FullName  | varchar(255) | NO   |     |         |                |
| WWW       | varchar(255) | NO   |     |         |                |
+-----------+--------------+------+-----+---------+----------------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS departments (" \
                  "DptCod INT NOT NULL AUTO_INCREMENT," \
                  "InsCod INT NOT NULL," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "WWW VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(DptCod), INDEX(InsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table exam_announcements *****/
/*
mysql> DESCRIBE exam_announcements;
+-------------+--------------+------+-----+---------+----------------+
| Field       | Type         | Null | Key | Default | Extra          |
+-------------+--------------+------+-----+---------+----------------+
| ExaCod      | int(11)      | NO   | PRI | NULL    | auto_increment |
| CrsCod      | int(11)      | NO   | MUL | -1      |                |
| Status      | tinyint(4)   | NO   | MUL | 0       |                |
| NumNotif    | int(11)      | NO   |     | 0       |                |
| CrsFullName | varchar(255) | NO   |     | NULL    |                |
| Year        | tinyint(4)   | NO   |     | NULL    |                |
| ExamSession | varchar(255) | NO   |     | NULL    |                |
| CallDate    | datetime     | NO   |     | NULL    |                |
| ExamDate    | datetime     | NO   |     | NULL    |                |
| Duration    | time         | NO   |     | NULL    |                |
| Place       | text         | NO   |     | NULL    |                |
| ExamMode    | text         | NO   |     | NULL    |                |
| Structure   | text         | NO   |     | NULL    |                |
| DocRequired | text         | NO   |     | NULL    |                |
| MatRequired | text         | NO   |     | NULL    |                |
| MatAllowed  | text         | NO   |     | NULL    |                |
| OtherInfo   | text         | NO   |     | NULL    |                |
+-------------+--------------+------+-----+---------+----------------+
17 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS exam_announcements (" \
                  "ExaCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "Status TINYINT NOT NULL DEFAULT 0," \
                  "NumNotif INT NOT NULL DEFAULT 0," \
                  "CrsFullName VARCHAR(255) NOT NULL," \
                  "Year TINYINT NOT NULL," \
                  "ExamSession VARCHAR(255) NOT NULL," \
                  "CallDate DATETIME NOT NULL," \
                  "ExamDate DATETIME NOT NULL," \
                  "Duration TIME NOT NULL," \
                  "Place TEXT NOT NULL," \
                  "ExamMode TEXT NOT NULL," \
                  "Structure TEXT NOT NULL," \
                  "DocRequired TEXT NOT NULL," \
                  "MatRequired TEXT NOT NULL," \
                  "MatAllowed TEXT NOT NULL," \
                  "OtherInfo TEXT NOT NULL," \
                  "UNIQUE INDEX(ExaCod), INDEX(CrsCod,Status), INDEX(Status))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table expanded_folders *****/
/*
mysql> DESCRIBE expanded_folders;
+-------------+------------+------+-----+---------------------+-------+
| Field       | Type       | Null | Key | Default             | Extra |
+-------------+------------+------+-----+---------------------+-------+
| UsrCod      | int(11)    | NO   | MUL |                     |       |
| FileBrowser | tinyint(4) | NO   |     | 0                   |       |
| CrsCod      | int(11)    | NO   | MUL | -1                  |       |
| GrpCod      | int(11)    | NO   |     | 0                   |       |
| WorksUsrCod | int(11)    | NO   |     |                     |       |
| Path        | text       | NO   |     |                     |       |
| ClickTime   | datetime   | NO   |     | 0000-00-00 00:00:00 |       |
+-------------+------------+------+-----+---------------------+-------+
7 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS expanded_folders (" \
                  "UsrCod INT NOT NULL," \
                  "FileBrowser TINYINT NOT NULL," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL," \
                  "WorksUsrCod INT NOT NULL," \
                  "Path TEXT COLLATE latin1_bin NOT NULL," \
                  "ClickTime DATETIME NOT NULL," \
                  "INDEX(UsrCod,FileBrowser),INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table file_browser_size *****/
/*
mysql> DESCRIBE file_browser_size;
+-------------+------------+------+-----+---------+-------+
| Field       | Type       | Null | Key | Default | Extra |
+-------------+------------+------+-----+---------+-------+
| FileBrowser | tinyint(4) | NO   | PRI | NULL    |       |
| CrsCod      | int(11)    | NO   | PRI | -1      |       |
| GrpCod      | int(11)    | NO   | PRI | -1      |       |
| UsrCod      | int(11)    | NO   | PRI | -1      |       |
| NumLevels   | int(11)    | NO   |     | NULL    |       |
| NumFolders  | int(11)    | NO   |     | NULL    |       |
| NumFiles    | int(11)    | NO   |     | NULL    |       |
| TotalSize   | bigint(20) | NO   |     | NULL    |       |
+-------------+------------+------+-----+---------+-------+
8 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS file_browser_size (" \
                  "FileBrowser TINYINT NOT NULL," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL DEFAULT -1," \
                  "UsrCod INT NOT NULL DEFAULT -1," \
                  "NumLevels INT NOT NULL," \
                  "NumFolders INT NOT NULL," \
                  "NumFiles INT NOT NULL," \
                  "TotalSize BIGINT NOT NULL," \
                  "UNIQUE INDEX(FileBrowser,CrsCod,GrpCod,UsrCod),INDEX(CrsCod),INDEX(GrpCod),INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table forum_disabled_post *****/
/*
mysql> DESCRIBE forum_disabled_post;
+-------------+----------+------+-----+---------+-------+
| Field       | Type     | Null | Key | Default | Extra |
+-------------+----------+------+-----+---------+-------+
| PstCod      | int(11)  | NO   | PRI |         |       |
| UsrCod      | int(11)  | NO   |     |         |       |
| DisableTime | datetime | NO   |     |         |       |
+-------------+----------+------+-----+---------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS forum_disabled_post (" \
                  "PstCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "DisableTime DATETIME NOT NULL," \
                  "UNIQUE INDEX(PstCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table forum_post *****/
/*
mysql> DESCRIBE forum_post;
+-----------+----------+------+-----+---------------------+----------------+
| Field     | Type     | Null | Key | Default             | Extra          |
+-----------+----------+------+-----+---------------------+----------------+
| PstCod    | int(11)  | NO   | PRI | NULL                | auto_increment |
| ThrCod    | int(11)  | NO   | MUL | NULL                |                |
| UsrCod    | int(11)  | NO   | MUL | NULL                |                |
| CreatTime | datetime | NO   | MUL | NULL                |                |
| ModifTime | datetime | NO   | MUL | 0000-00-00 00:00:00 |                |
| NumNotif  | int(11)  | NO   |     | 0                   |                |
| Subject   | text     | NO   |     | NULL                |                |
| Content   | longtext | NO   |     | NULL                |                |
+-----------+----------+------+-----+---------------------+----------------+
8 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS forum_post (" \
                  "PstCod INT NOT NULL AUTO_INCREMENT," \
                  "ThrCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "CreatTime DATETIME NOT NULL," \
                  "ModifTime DATETIME NOT NULL," \
                  "NumNotif INT NOT NULL DEFAULT 0," \
                  "Subject TEXT NOT NULL," \
                  "Content LONGTEXT NOT NULL," \
                  "UNIQUE INDEX(PstCod),INDEX(ThrCod),INDEX(UsrCod),INDEX(CreatTime),INDEX(ModifTime))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table forum_thr_clip *****/
/*
mysql> DESCRIBE forum_thr_clip;
+------------+-----------+------+-----+-------------------+-------+
| Field      | Type      | Null | Key | Default           | Extra |
+------------+-----------+------+-----+-------------------+-------+
| ThrCod     | int(11)   | NO   | PRI |                   |       |
| UsrCod     | int(11)   | NO   | UNI |                   |       |
| TimeInsert | timestamp | NO   |     | CURRENT_TIMESTAMP |       |
+------------+-----------+------+-----+-------------------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS forum_thr_clip (" \
                  "ThrCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "TimeInsert TIMESTAMP NOT NULL," \
                  "UNIQUE INDEX(ThrCod),UNIQUE INDEX (UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table forum_thr_read *****/
/*
mysql> DESCRIBE forum_thr_read;
+----------+----------+------+-----+---------------------+-------+
| Field    | Type     | Null | Key | Default             | Extra |
+----------+----------+------+-----+---------------------+-------+
| ThrCod   | int(11)  | NO   | PRI | 0                   |       |
| UsrCod   | int(11)  | NO   | PRI |                     |       |
| ReadTime | datetime | NO   |     | 0000-00-00 00:00:00 |       |
+----------+----------+------+-----+---------------------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS forum_thr_read (" \
                  "ThrCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "ReadTime DATETIME NOT NULL," \
                  "UNIQUE INDEX(ThrCod,UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table forum_thread *****/
/*
mysql> DESCRIBE forum_thread;
+-------------+------------+------+-----+---------+----------------+
| Field       | Type       | Null | Key | Default | Extra          |
+-------------+------------+------+-----+---------+----------------+
| ThrCod      | int(11)    | NO   | PRI | NULL    | auto_increment |
| ForumType   | tinyint(4) | NO   | MUL | 0       |                |
| Location    | int(11)    | NO   | MUL | -1      |                |
| FirstPstCod | int(11)    | NO   | UNI | NULL    |                |
| LastPstCod  | int(11)    | NO   | UNI | NULL    |                |
+-------------+------------+------+-----+---------+----------------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS forum_thread (" \
                  "ThrCod INT NOT NULL AUTO_INCREMENT," \
                  "ForumType TINYINT NOT NULL," \
                  "Location INT NOT NULL DEFAULT -1," \
                  "FirstPstCod INT NOT NULL," \
                  "LastPstCod INT NOT NULL," \
                  "UNIQUE INDEX(ThrCod), INDEX(ForumType), INDEX(Location), UNIQUE INDEX(FirstPstCod), UNIQUE INDEX(LastPstCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table hidden_downloads *****/
/*
mysql> DESCRIBE hidden_downloads;
+--------+---------+------+-----+---------+-------+
| Field  | Type    | Null | Key | Default | Extra |
+--------+---------+------+-----+---------+-------+
| CrsCod | int(11) | NO   | MUL | -1      |       |
| GrpCod | int(11) | NO   | MUL | -1      |       |
| Path   | text    | NO   |     |         |       |
+--------+---------+------+-----+---------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS hidden_downloads (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL DEFAULT -1," \
                  "Path TEXT COLLATE latin1_bin NOT NULL," \
                  "INDEX(CrsCod,GrpCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table hidden_params *****/
/*
mysql> DESCRIBE hidden_params;
+------------+--------------+------+-----+---------+-------+
| Field      | Type         | Null | Key | Default | Extra |
+------------+--------------+------+-----+---------+-------+
| SessionId  | char(43)     | NO   | MUL | NULL    |       |
| Action     | int(11)      | NO   |     | NULL    |       |
| ParamName  | varchar(255) | NO   |     | NULL    |       |
| ParamValue | text         | NO   |     | NULL    |       |
+------------+--------------+------+-----+---------+-------+
4 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS hidden_params (" \
                  "SessionId CHAR(43) NOT NULL," \
                  "Action INT NOT NULL," \
                  "ParamName VARCHAR(255) NOT NULL," \
                  "ParamValue TEXT NOT NULL," \
                  "INDEX(SessionId,Action))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table holidays *****/
/*
mysql> DESCRIBE holidays;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| HldCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| PlcCod    | int(11)      | NO   | MUL | -1      |                |
| HldTyp    | tinyint(4)   | NO   |     |         |                |
| StartDate | date         | NO   |     |         |                |
| EndDate   | date         | NO   |     |         |                |
| Name      | varchar(255) | NO   |     |         |                |
+-----------+--------------+------+-----+---------+----------------+
6 rows in set (0.02 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS holidays (" \
                  "HldCod INT NOT NULL AUTO_INCREMENT," \
                  "PlcCod INT NOT NULL DEFAULT -1," \
                  "HldTyp TINYINT NOT NULL," \
                  "StartDate DATE NOT NULL," \
                  "EndDate DATE NOT NULL," \
                  "Name VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX (HldCod),INDEX(PlcCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table institutions *****/
/*
mysql> DESCRIBE institutions;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| InsCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| ShortName | varchar(32)  | NO   |     |         |                |
| FullName  | varchar(255) | NO   |     |         |                |
| Logo      | varchar(16)  | NO   |     |         |                |
| WWW       | varchar(255) | NO   |     |         |                |
+-----------+--------------+------+-----+---------+----------------+
5 rows in set (0.03 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS institutions (" \
                  "InsCod INT NOT NULL AUTO_INCREMENT," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "Logo VARCHAR(16) NOT NULL," \
                  "WWW VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(InsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table links *****/
/*
mysql> DESCRIBE links;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| LnkCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| ShortName | varchar(32)  | NO   |     |         |                |
| FullName  | varchar(255) | NO   |     |         |                |
| WWW       | varchar(255) | NO   |     |         |                |
+-----------+--------------+------+-----+---------+----------------+
4 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS links (" \
                  "LnkCod INT NOT NULL AUTO_INCREMENT," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "WWW VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(LnkCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table log *****/
/*
mysql> DESCRIBE log;
+----------------+--------------+------+-----+---------+-------+
| Field          | Type         | Null | Key | Default | Extra |
+----------------+--------------+------+-----+---------+-------+
| DegCod         | int(11)      | NO   | MUL | -1      |       |
| CrsCod         | int(11)      | NO   | MUL | -1      |       |
| DNI            | char(16)     | NO   | MUL | NULL    |       |
| UsrType        | tinyint(4)   | NO   |     | NULL    |       |
| ClickTime      | datetime     | NO   | MUL | NULL    |       |
| TimeToGenerate | int(11)      | NO   |     | NULL    |       |
| TimeToSend     | int(11)      | NO   |     | NULL    |       |
| Action         | varchar(32)  | NO   |     | NULL    |       |
| IP             | char(40)     | NO   |     | NULL    |       |
| Comments       | varchar(255) | YES  |     | NULL    |       |
+----------------+--------------+------+-----+---------+-------+
10 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS log (" \
                  "DegCod INT NOT NULL DEFAULT -1," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "DNI CHAR(16) NOT NULL," \
                  "UsrType TINYINT NOT NULL," \
                  "ClickTime DATETIME NOT NULL," \
                  "TimeToGenerate INT NOT NULL," \
                  "TimeToSend INT NOT NULL," \
                  "Action VARCHAR(32) NOT NULL," \
                  "IP CHAR(40) NOT NULL," \
                  "Comments VARCHAR(255)," \
                  "INDEX (DegCod),INDEX(CrsCod),INDEX(DNI),INDEX(ClickTime,UsrType))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table log_recent *****/
/*
mysql> DESCRIBE log_recent;
+----------------+--------------+------+-----+---------+-------+
| Field          | Type         | Null | Key | Default | Extra |
+----------------+--------------+------+-----+---------+-------+
| DegCod         | int(11)      | NO   | MUL | -1      |       |
| CrsCod         | int(11)      | NO   | MUL | -1      |       |
| DNI            | char(16)     | NO   | MUL |         |       |
| UsrType        | tinyint(4)   | NO   |     |         |       |
| ClickTime      | datetime     | NO   | MUL |         |       |
| TimeToGenerate | int(11)      | NO   |     |         |       |
| TimeToSend     | int(11)      | NO   |     |         |       |
| Action         | varchar(32)  | NO   |     |         |       |
| IP             | char(40)     | NO   |     |         |       |
| Comments       | varchar(255) | YES  |     | NULL    |       |
+----------------+--------------+------+-----+---------+-------+
10 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS log_recent (" \
                  "DegCod INT NOT NULL DEFAULT -1," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "DNI CHAR(16) NOT NULL," \
                  "UsrType TINYINT NOT NULL," \
                  "ClickTime DATETIME NOT NULL," \
                  "TimeToGenerate INT NOT NULL," \
                  "TimeToSend INT NOT NULL," \
                  "Action VARCHAR(32) NOT NULL," \
                  "IP CHAR(40) NOT NULL," \
                  "Comments VARCHAR(255)," \
                  "INDEX (DegCod),INDEX(CrsCod),INDEX(DNI),INDEX(ClickTime,UsrType))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table marks *****/
/*
mysql> DESCRIBE marks;
+--------+---------+------+-----+---------+-------+
| Field  | Type    | Null | Key | Default | Extra |
+--------+---------+------+-----+---------+-------+
| CrsCod | int(11) | NO   | MUL | -1      |       |
| GrpCod | int(11) | NO   |     | -1      |       |
| Path   | text    | NO   |     | NULL    |       |
| Header | int(11) | NO   |     | NULL    |       |
| Footer | int(11) | NO   |     | NULL    |       |
+--------+---------+------+-----+---------+-------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS marks (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL DEFAULT -1," \
                  "Path TEXT COLLATE latin1_bin NOT NULL," \
                  "Header INT NOT NULL," \
                  "Footer INT NOT NULL," \
                  "INDEX(CrsCod,GrpCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table messages *****/
/*
mysql> DESCRIBE messages;
+-----------+---------------+------+-----+---------------------+----------------+
| Field     | Type          | Null | Key | Default             | Extra          |
+-----------+---------------+------+-----+---------------------+----------------+
| MsgCod    | int(11)       | NO   | PRI | NULL                | auto_increment |
| CrsCod    | int(11)       | NO   | MUL | -1                  |                |
| UsrCod    | int(11)       | NO   | MUL |                     |                |
| Deleted   | enum('N','Y') | NO   | MUL | N                   |                |
| Expanded  | enum('N','Y') | NO   |     | N                   |                |
| CreatTime | datetime      | NO   | MUL | 0000-00-00 00:00:00 |                |
| Subject   | text          | NO   | MUL |                     |                |
| Content   | longtext      | NO   |     |                     |                |
+-----------+---------------+------+-----+---------------------+----------------+
8 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS messages (" \
                  "MsgCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "UsrCod INT NOT NULL," \
                  "Deleted ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Expanded ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "CreatTime DATETIME NOT NULL," \
                  "Subject TEXT NOT NULL," \
                  "Content LONGTEXT NOT NULL," \
                  "UNIQUE INDEX(MsgCod),INDEX(CrsCod),INDEX(UsrCod),INDEX(Deleted),INDEX(CreatTime),FULLTEXT(Subject,Content))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table msg_new *****/
/*
mysql> DESCRIBE msg_new;
+------------+---------+------+-----+---------+-------+
| Field      | Type    | Null | Key | Default | Extra |
+------------+---------+------+-----+---------+-------+
| UsrCod     | int(11) | NO   | PRI | NULL    |       |
| NumNewMsgs | int(11) | NO   |     | 0       |       |
+------------+---------+------+-----+---------+-------+
2 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS msg_new (" \
                  "UsrCod INT NOT NULL," \
                  "NumNewMsgs INT NOT NULL," \
                  "UNIQUE INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table notices *****/
/*
mysql> DESCRIBE notices;
+-----------+------------+------+-----+---------+----------------+
| Field     | Type       | Null | Key | Default | Extra          |
+-----------+------------+------+-----+---------+----------------+
| NotCod    | int(11)    | NO   | PRI | NULL    | auto_increment |
| CrsCod    | int(11)    | NO   | MUL | -1      |                |
| UsrCod    | int(11)    | NO   | MUL | NULL    |                |
| CreatTime | datetime   | NO   | MUL | NULL    |                |
| Content   | longtext   | NO   |     | NULL    |                |
| Status    | tinyint(4) | NO   | MUL | 0       |                |
| NumNotif  | int(11)    | NO   |     | 0       |                |
+-----------+------------+------+-----+---------+----------------+
7 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS notices (" \
                  "NotCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "UsrCod INT NOT NULL," \
                  "CreatTime DATETIME NOT NULL," \
                  "Content LONGTEXT NOT NULL," \
                  "Status TINYINT NOT NULL DEFAULT 0," \
                  "NumNotif INT NOT NULL DEFAULT 0," \
                  "UNIQUE INDEX(NotCod), INDEX(CrsCod,Status), INDEX(UsrCod), INDEX(CreatTime), INDEX(Status))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table msg_received *****/
/*
mysql> DESCRIBE msg_received;
+----------+---------------+------+-----+---------+-------+
| Field    | Type          | Null | Key | Default | Extra |
+----------+---------------+------+-----+---------+-------+
| MsgCod   | int(11)       | NO   | PRI | 0       |       |
| UsrCod   | int(11)       | NO   | PRI |         |       |
| Deleted  | enum('N','Y') | NO   | MUL | N       |       |
| Open     | enum('N','Y') | NO   |     | N       |       |
| Replied  | enum('N','Y') | NO   |     | N       |       |
| Expanded | enum('N','Y') | NO   |     | N       |       |
+----------+---------------+------+-----+---------+-------+
6 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS msg_received (" \
                  "MsgCod INT NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "Deleted ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Notified ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Open ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Replied ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "Expanded ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "UNIQUE INDEX (UsrCod,MsgCod),INDEX(MsgCod),INDEX(UsrCod),INDEX(Deleted),INDEX(Notified))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table notif *****/
/*
mysql> DESCRIBE notif;
+-------------+------------+------+-----+---------+-------+
| Field       | Type       | Null | Key | Default | Extra |
+-------------+------------+------+-----+---------+-------+
| NotifyEvent | tinyint(4) | NO   |     | NULL    |       |
| ToUsrCod    | int(11)    | NO   | MUL | NULL    |       |
| FromUsrCod  | int(11)    | NO   |     | NULL    |       |
| InsCod      | int(11)    | NO   |     | -1      |       |
| DegCod      | int(11)    | NO   |     | -1      |       |
| CrsCod      | int(11)    | NO   |     | -1      |       |
| MsgCod      | int(11)    | NO   |     | -1      |       |
| ForumType   | tinyint(4) | NO   |     | NULL    |       |
| TimeNotif   | datetime   | NO   | MUL | NULL    |       |
+-------------+------------+------+-----+---------+-------+
9 rows in set (0.37 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS notif (" \
                  "NotifyEvent TINYINT NOT NULL," \
                  "ToUsrCod INT NOT NULL," \
                  "FromUsrCod INT NOT NULL," \
                  "InsCod INT NOT NULL DEFAULT -1," \
                  "DegCod INT NOT NULL DEFAULT -1," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "MsgCod INT NOT NULL DEFAULT -1," \
                  "ForumType TINYINT NOT NULL," \
                  "TimeNotif DATETIME NOT NULL," \
                  "INDEX (ToUsrCod),INDEX(TimeNotif))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table pending_passwd *****/
/*
mysql> DESCRIBE pending_passwd;
+-----------------+----------+------+-----+---------+-------+
| Field           | Type     | Null | Key | Default | Extra |
+-----------------+----------+------+-----+---------+-------+
| UsrCod          | int(11)  | NO   | PRI |         |       |
| PendingPassword | char(86) | NO   |     |         |       |
| DateAndTime     | datetime | NO   |     |         |       |
+-----------------+----------+------+-----+---------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS pending_passwd (" \
                  "UsrCod INT NOT NULL," \
                  "PendingPassword CHAR(86) NOT NULL," \
                  "DateAndTime DATETIME NOT NULL," \
                  "PRIMARY KEY (UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table places *****/
/*
mysql> DESCRIBE places;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| PlcCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| ShortName | varchar(32)  | NO   |     |         |                |
| FullName  | varchar(255) | NO   |     |         |                |
+-----------+--------------+------+-----+---------+----------------+
3 rows in set (0.01 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS places (" \
                  "PlcCod INT NOT NULL AUTO_INCREMENT," \
                  "ShortName VARCHAR(32) NOT NULL," \
                  "FullName VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX(PlcCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table plugins *****/
/*
mysql> DESCRIBE plugins;
+-------------+--------------+------+-----+---------+----------------+
| Field       | Type         | Null | Key | Default | Extra          |
+-------------+--------------+------+-----+---------+----------------+
| PlgCod      | int(11)      | NO   | PRI | NULL    | auto_increment |
| Name        | varchar(255) | NO   |     | NULL    |                |
| Description | text         | NO   |     | NULL    |                |
| Logo        | varchar(16)  | NO   |     | NULL    |                |
| URL         | varchar(255) | NO   |     | NULL    |                |
| IP          | char(40)     | NO   |     | NULL    |                |
+-------------+--------------+------+-----+---------+----------------+
6 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS plugins (PlgCod INT NOT NULL AUTO_INCREMENT," \
                  " Name VARCHAR(255) NOT NULL," \
                  " Description TEXT NOT NULL," \
                  " Logo VARCHAR(16) NOT NULL," \
                  " URL VARCHAR(255) NOT NULL," \
                  " IP CHAR(40) NOT NULL," \
                  " UNIQUE INDEX(PlgCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table imported_groups *****/
/*
mysql> DESCRIBE imported_groups;
+-----------+--------------+------+-----+---------+----------------+
| Field     | Type         | Null | Key | Default | Extra          |
+-----------+--------------+------+-----+---------+----------------+
| GrpCod    | int(11)      | NO   | PRI | NULL    | auto_increment |
| SessionId | char(43)     | NO   | MUL | NULL    |                |
| DegName   | varchar(255) | NO   |     | NULL    |                |
| CrsName   | varchar(255) | NO   |     | NULL    |                |
| GrpName   | varchar(255) | NO   |     | NULL    |                |
| GrpType   | varchar(255) | NO   |     | NULL    |                |
+-----------+--------------+------+-----+---------+----------------+
6 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS imported_groups (" \
                  "GrpCod INT NOT NULL AUTO_INCREMENT," \
                  "SessionId CHAR(43) NOT NULL," \
                  "DegName VARCHAR(255) NOT NULL," \
                  "CrsName VARCHAR(255) NOT NULL," \
                  "GrpName VARCHAR(255) NOT NULL," \
                  "GrpType VARCHAR(255) NOT NULL," \
                  "UNIQUE INDEX (GrpCod),INDEX (SessionId))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table imported_sessions *****/
/*
mysql> DESCRIBE imported_sessions;
+-------------------+------------+------+-----+---------+-------+
| Field             | Type       | Null | Key | Default | Extra |
+-------------------+------------+------+-----+---------+-------+
| SessionId         | char(43)   | NO   | PRI | NULL    |       |
| UsrCod            | int(11)    | NO   |     | NULL    |       |
| ImportedUsrId     | char(255)  | NO   |     | NULL    |       |
| ImportedSessionId | char(255)  | NO   |     | NULL    |       |
| ImportedUsrType   | tinyint(4) | NO   |     | NULL    |       |
+-------------------+------------+------+-----+---------+-------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS imported_sessions (" \
                  "SessionId CHAR(43) NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "ImportedUsrId CHAR(255) NOT NULL," \
                  "ImportedSessionId CHAR(255) NOT NULL," \
                  "ImportedUsrType TINYINT NOT NULL," \
                  "UNIQUE INDEX(SessionId))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table imported_students *****/
/*
mysql> DESCRIBE imported_students;
+-----------+---------------------------------+------+-----+---------+----------------+
| Field     | Type                            | Null | Key | Default | Extra          |
+-----------+---------------------------------+------+-----+---------+----------------+
| GrpCod    | int(11)                         | NO   | MUL | NULL    | auto_increment |
| DNI       | char(16)                        | NO   |     | NULL    |                |
| Surname1  | varchar(255)                    | NO   |     | NULL    |                |
| Surname2  | varchar(255)                    | NO   |     | NULL    |                |
| FirstName | varchar(255)                    | NO   |     | NULL    |                |
| Gender    | enum('unknown','female','male') | NO   |     | unknown |                |
| E_mail    | varchar(255)                    | NO   |     | NULL    |                |
+-----------+---------------------------------+------+-----+---------+----------------+
7 rows in set (0.01 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS imported_students (" \
                  "GrpCod INT NOT NULL AUTO_INCREMENT," \
                  "DNI CHAR(16) NOT NULL," \
                  "Surname1 VARCHAR(255) NOT NULL," \
                  "Surname2 VARCHAR(255) NOT NULL," \
                  "FirstName VARCHAR(255) NOT NULL," \
                  "Gender ENUM ('unknown','female','male') NOT NULL DEFAULT 'unknown'," \
                  "E_mail VARCHAR(255) NOT NULL," \
                  "INDEX(GrpCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table sessions *****/
/*
mysql> DESCRIBE sessions;
+----------------+------------+------+-----+---------+-------+
| Field          | Type       | Null | Key | Default | Extra |
+----------------+------------+------+-----+---------+-------+
| SessionId      | char(43)   | NO   | PRI |         |       |
| UsrCod         | int(11)    | NO   | MUL |         |       |
| Password       | char(86)   | NO   |     |         |       |
| UsrType        | tinyint(4) | NO   |     |         |       |
| DegTypCod      | int(11)    | NO   |     | -1      |       |
| DegCod         | int(11)    | NO   |     | -1      |       |
| CrsCod         | int(11)    | NO   |     | -1      |       |
| LastTime       | datetime   | YES  |     | NULL    |       |
| LastRefresh    | datetime   | YES  |     | NULL    |       |
| LastPageMsgRec | int(11)    | NO   |     | 1       |       |
| LastPageMsgSnt | int(11)    | NO   |     | 1       |       |
| SideCols       | tinyint(4) | NO   |     | 3       |       |
+----------------+------------+------+-----+---------+-------+
12 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS sessions (" \
                  "SessionId CHAR(43) NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "Password CHAR(86) NOT NULL," \
                  "UsrType TINYINT NOT NULL," \
                  "DegTypCod INT NOT NULL DEFAULT -1," \
                  "DegCod INT NOT NULL DEFAULT -1," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "LastTime DATETIME," \
                  "LastRefresh DATETIME," \
                  "LastPageMsgRec INT NOT NULL DEFAULT 1," \
                  "LastPageMsgSnt INT NOT NULL DEFAULT 1," \
                  "UNIQUE INDEX(SessionId),INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table sta_degrees *****/
/*
mysql> DESCRIBE sta_degrees;
+-----------------------+---------------------------------------+------+-----+---------+-------+
| Field                 | Type                                  | Null | Key | Default | Extra |
+-----------------------+---------------------------------------+------+-----+---------+-------+
| DegCod                | int(11)                               | NO   | PRI | -1      |       |
| Gender                | enum('unknown','female','male','all') | NO   | PRI | all     |       |
| NumStds               | int(11)                               | NO   |     |         |       |
| NumStdsWithPhoto      | int(11)                               | NO   |     |         |       |
| TimeAvgPhoto          | datetime                              | NO   | MUL |         |       |
| TimeToComputeAvgPhoto | int(11)                               | NO   |     | 0       |       |
+-----------------------+---------------------------------------+------+-----+---------+-------+
6 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS sta_degrees (" \
                  "DegCod INT NOT NULL DEFAULT -1," \
                  "Gender ENUM ('unknown','female','male','all') NOT NULL DEFAULT 'all'," \
                  "NumStds INT NOT NULL," \
                  "NumStdsWithPhoto INT NOT NULL," \
                  "TimeAvgPhoto DATETIME NOT NULL," \
                  "TimeToComputeAvgPhoto INT NOT NULL DEFAULT -1," \
                  "UNIQUE INDEX(DegCod,Gender))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table sta_notif *****/
/*
mysql> DESCRIBE sta_notif;
+-------------+------------+------+-----+---------+-------+
| Field       | Type       | Null | Key | Default | Extra |
+-------------+------------+------+-----+---------+-------+
| DegCod      | int(11)    | NO   | PRI | NULL    |       |
| CrsCod      | int(11)    | NO   | PRI | NULL    |       |
| NotifyEvent | tinyint(4) | NO   | PRI | NULL    |       |
| NumEvents   | int(11)    | NO   |     | NULL    |       |
| NumMails    | int(11)    | NO   |     | NULL    |       |
+-------------+------------+------+-----+---------+-------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS sta_notif (" \
                  "DegCod INT NOT NULL," \
                  "CrsCod INT NOT NULL," \
                  "NotifyEvent TINYINT NOT NULL," \
                  "NumEvents INT NOT NULL," \
                  "NumMails INT NOT NULL," \
                  "UNIQUE INDEX (DegCod,CrsCod,NotifyEvent))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table timetable_crs *****/
/*
mysql> DESCRIBE timetable_crs;
+-----------+------------------------------------+------+-----+---------+-------+
| Field     | Type                               | Null | Key | Default | Extra |
+-----------+------------------------------------+------+-----+---------+-------+
| CrsCod    | int(11)                            | NO   | MUL | -1      |       |
| GrpCod    | int(11)                            | NO   |     | -1      |       |
| Day       | enum('L','M','X','J','V')          | NO   |     | NULL    |       |
| Hour      | tinyint(4)                         | NO   |     | NULL    |       |
| Duration  | tinyint(4)                         | NO   |     | NULL    |       |
| ClassType | enum('libre','teoria','practicas') | NO   |     | NULL    |       |
| Place     | varchar(255)                       | NO   |     | NULL    |       |
| GroupName | varchar(255)                       | NO   |     | NULL    |       |
+-----------+------------------------------------+------+-----+---------+-------+
8 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS timetable_crs (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "GrpCod INT NOT NULL DEFAULT -1," \
                  "Day ENUM('L','M','X','J','V') NOT NULL," \
                  "Hour TINYINT NOT NULL," \
                  "Duration TINYINT NOT NULL," \
                  "ClassType ENUM('libre','teoria','practicas') NOT NULL," \
                  "Place VARCHAR(255) NOT NULL," \
                  "GroupName VARCHAR(255) NOT NULL," \
                  "INDEX(CrsCod,GrpCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table timetable_tut *****/
/*
mysql> DESCRIBE timetable_tut;
+----------+---------------------------+------+-----+---------+-------+
| Field    | Type                      | Null | Key | Default | Extra |
+----------+---------------------------+------+-----+---------+-------+
| UsrCod   | int(11)                   | NO   | MUL | NULL    |       |
| Day      | enum('L','M','X','J','V') | NO   |     | NULL    |       |
| Hour     | tinyint(4)                | NO   |     | NULL    |       |
| Duration | tinyint(4)                | NO   |     | NULL    |       |
| Place    | varchar(255)              | NO   |     | NULL    |       |
+----------+---------------------------+------+-----+---------+-------+
5 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS timetable_tut (" \
                  "UsrCod INT NOT NULL," \
                  "Day ENUM('L','M','X','J','V') NOT NULL," \
                  "Hour TINYINT NOT NULL," \
                  "Duration TINYINT NOT NULL," \
                  "Place VARCHAR(255) NOT NULL," \
                  "INDEX(UsrCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table tst_answers *****/
/*
mysql> DESCRIBE tst_answers;
+---------+---------------+------+-----+---------+-------+
| Field   | Type          | Null | Key | Default | Extra |
+---------+---------------+------+-----+---------+-------+
| QstCod  | int(11)       | NO   | MUL | NULL    |       |
| AnsInd  | tinyint(4)    | NO   |     | NULL    |       |
| Answer  | text          | NO   |     | NULL    |       |
| Correct | enum('N','Y') | NO   |     | NULL    |       |
+---------+---------------+------+-----+---------+-------+
4 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS tst_answers (" \
                  "QstCod INT NOT NULL," \
                  "AnsInd TINYINT NOT NULL," \
                  "Answer TEXT NOT NULL," \
                  "Correct ENUM('N','Y') NOT NULL," \
                  "INDEX(QstCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table tst_config *****/
/*
mysql> DESCRIBE tst_config;
+---------------------+--------------------------------------------------------------+------+-----+---------+-------+
| Field               | Type                                                         | Null | Key | Default | Extra |
+---------------------+--------------------------------------------------------------+------+-----+---------+-------+
| CrsCod              | int(11)                                                      | NO   | PRI | -1      |       |
| Min                 | int(11)                                                      | NO   |     | NULL    |       |
| Def                 | int(11)                                                      | NO   |     | NULL    |       |
| Max                 | int(11)                                                      | NO   |     | NULL    |       |
| MinTimeNxtTstPerQst | int(11)                                                      | NO   |     | 0       |       |
| Feedback            | enum('nothing','total_result','each_result','each_good_bad') | NO   |     | NULL    |       |
+---------------------+--------------------------------------------------------------+------+-----+---------+-------+
6 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS tst_config (" \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "Min INT NOT NULL," \
                  "Def INT NOT NULL," \
                  "Max INT NOT NULL," \
                  "MinTimeNxtTstPerQst INT NOT NULL DEFAULT 0," \
                  "Feedback ENUM('nothing','total_result','each_result','each_good_bad') NOT NULL," \
                  "UNIQUE INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table tst_question_tags *****/
/*
mysql> DESCRIBE tst_question_tags;
+--------+------------+------+-----+---------+-------+
| Field  | Type       | Null | Key | Default | Extra |
+--------+------------+------+-----+---------+-------+
| QstCod | int(11)    | NO   | PRI | 0       |       |
| TagCod | int(11)    | NO   | PRI |         |       |
| IndDes | tinyint(4) | NO   |     | 0       |       |
+--------+------------+------+-----+---------+-------+
3 rows in set (0.01 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS tst_question_tags (" \
                  "QstCod INT NOT NULL," \
                  "TagCod INT NOT NULL," \
                  "IndDes TINYINT NOT NULL," \
                  "UNIQUE INDEX(QstCod,TagCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table tst_questions *****/
/*
mysql> DESCRIBE tst_questions;
+----------+--------------------------------------------------------------------+------+-----+---------+----------------+
| Field    | Type                                                               | Null | Key | Default | Extra          |
+----------+--------------------------------------------------------------------+------+-----+---------+----------------+
| QstCod   | int(11)                                                            | NO   | PRI | NULL    | auto_increment |
| CrsCod   | int(11)                                                            | NO   | MUL | -1      |                |
| EditTime | datetime                                                           | NO   |     | NULL    |                |
| AnsType  | enum('T/F','unique_choice','multiple_choice','text','int','float') | NO   |     | NULL    |                |
| Shuffle  | enum('N','Y')                                                      | NO   |     | NULL    |                |
| Stem     | text                                                               | NO   |     | NULL    |                |
| NumHits  | int(11)                                                            | NO   |     | NULL    |                |
| Score    | double                                                             | NO   |     | NULL    |                |
+----------+--------------------------------------------------------------------+------+-----+---------+----------------+
8 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS tst_questions (" \
                  "QstCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "EditTime DATETIME NOT NULL," \
                  "AnsType ENUM ('T/F','unique_choice','multiple_choice','text','int','float') NOT NULL," \
                  "Shuffle ENUM('N','Y') NOT NULL," \
                  "Stem TEXT NOT NULL," \
                  "NumHits INT NOT NULL," \
                  "Score DOUBLE PRECISION NOT NULL," \
                  "UNIQUE INDEX(QstCod), INDEX(CrsCod))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table tst_status *****/
/*
mysql> DESCRIBE tst_status;
+-----------+------------+------+-----+---------+-------+
| Field     | Type       | Null | Key | Default | Extra |
+-----------+------------+------+-----+---------+-------+
| SessionId | char(43)   | NO   | PRI |         |       |
| CrsCod    | int(11)    | NO   | PRI |         |       |
| NumTst    | int(11)    | NO   | PRI |         |       |
| Status    | tinyint(4) | NO   |     |         |       |
+-----------+------------+------+-----+---------+-------+
4 rows in set (0.04 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS tst_status (" \
                  "SessionId CHAR(43) NOT NULL," \
                  "CrsCod INT NOT NULL," \
                  "NumTst INT NOT NULL," \
                  "Status TINYINT NOT NULL," \
                  "UNIQUE INDEX(SessionId,CrsCod,NumTst))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table tst_tags *****/
/*
mysql> DESCRIBE tst_tags;
+-----------+---------------+------+-----+---------+----------------+
| Field     | Type          | Null | Key | Default | Extra          |
+-----------+---------------+------+-----+---------+----------------+
| TagCod    | int(11)       | NO   | PRI | NULL    | auto_increment |
| CrsCod    | int(11)       | NO   | MUL | -1      |                |
| TagTxt    | varchar(255)  | NO   | MUL |         |                |
| TagHidden | enum('N','Y') | NO   |     |         |                |
+-----------+---------------+------+-----+---------+----------------+
4 rows in set (0.00 sec)
*/
// CrsCod is redundant for speed in querys
   sprintf (Query,"CREATE TABLE IF NOT EXISTS tst_tags (" \
                  "TagCod INT NOT NULL AUTO_INCREMENT," \
                  "CrsCod INT NOT NULL DEFAULT -1," \
                  "TagTxt VARCHAR(255) NOT NULL," \
                  "TagHidden ENUM('N','Y') NOT NULL," \
                  "UNIQUE INDEX(TagCod), INDEX(CrsCod), INDEX(TagTxt))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table usr_data *****/
/*
mysql> DESCRIBE usr_data;
+------------------+---------------------------------+------+-----+---------+----------------+
| Field            | Type                            | Null | Key | Default | Extra          |
+------------------+---------------------------------+------+-----+---------+----------------+
| UsrCod           | int(11)                         | NO   | PRI | NULL    | auto_increment |
| EncryptedUsrCod  | char(43)                        | NO   | UNI | NULL    |                |
| DNI              | char(16)                        | NO   | UNI | NULL    |                |
| Nickname         | char(16)                        | NO   | MUL | NULL    |                |
| Password         | char(86)                        | NO   |     | NULL    |                |
| UsrType          | tinyint(4)                      | NO   | MUL | NULL    |                |
| Surname1         | varchar(255)                    | NO   |     | NULL    |                |
| Surname2         | varchar(255)                    | NO   |     | NULL    |                |
| FirstName        | varchar(255)                    | NO   |     | NULL    |                |
| Gender           | enum('unknown','female','male') | NO   |     | unknown |                |
| Theme            | char(16)                        | NO   | MUL | NULL    |                |
| Language         | char(2)                         | NO   | MUL | NULL    |                |
| E_mail           | varchar(255)                    | NO   |     |         |                |
| WWW              | varchar(255)                    | NO   |     |         |                |
| Skype            | varchar(255)                    | NO   |     |         |                |
| Photo            | char(43)                        | NO   |     | NULL    |                |
| PublicPhoto      | enum('N','Y')                   | NO   |     | N       |                |
| InsCod           | int(11)                         | NO   | MUL | -1      |                |
| DptCod           | int(11)                         | NO   | MUL | -1      |                |
| CtrCod           | int(11)                         | NO   | MUL | -1      |                |
| Office           | varchar(255)                    | NO   |     | NULL    |                |
| OfficePhone      | char(16)                        | NO   |     | NULL    |                |
| LocalAddress     | varchar(255)                    | NO   |     | NULL    |                |
| LocalPhone       | char(16)                        | NO   |     | NULL    |                |
| FamilyAddress    | varchar(255)                    | NO   |     | NULL    |                |
| FamilyPhone      | char(16)                        | NO   |     | NULL    |                |
| OriginPlace      | varchar(255)                    | NO   |     | NULL    |                |
| Birthday         | date                            | NO   |     | NULL    |                |
| Comments         | text                            | NO   |     | NULL    |                |
| NumNewMsgs       | int(11)                         | NO   |     | NULL    |                |
| SideCols         | tinyint(4)                      | NO   | MUL | 3       |                |
| SecureIcons      | enum('N','Y')                   | NO   | MUL | Y       |                |
| NotifyEvents     | int(11)                         | NO   |     | 0       |                |
| LastTab          | tinyint(4)                      | NO   |     | NULL    |                |
| LastTime         | datetime                        | NO   | MUL | NULL    |                |
| LastAccBriefcase | datetime                        | NO   |     | NULL    |                |
+------------------+---------------------------------+------+-----+---------+----------------+
36 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS usr_data (" \
                  "UsrCod INT NOT NULL AUTO_INCREMENT," \
                  "EncryptedUsrCod CHAR(43) NOT NULL," \
                  "DNI CHAR(16) NOT NULL," \
                  "Nickname CHAR(16) NOT NULL," \
                  "Password CHAR(86) NOT NULL," \
                  "UsrType TINYINT NOT NULL," \
                  "Surname1 VARCHAR(255) NOT NULL," \
                  "Surname2 VARCHAR(255) NOT NULL," \
                  "FirstName VARCHAR(255) NOT NULL," \
                  "Gender ENUM ('unknown','female','male') NOT NULL DEFAULT 'unknown'," \
                  "Theme CHAR(16) NOT NULL," \
                  "Language CHAR(2) NOT NULL," \
                  "E_mail VARCHAR(255) NOT NULL," \
                  "WWW VARCHAR(255) NOT NULL," \
                  "Skype VARCHAR(255) NOT NULL," \
                  "Photo CHAR(43) NOT NULL," \
                  "PublicPhoto ENUM('N','Y') NOT NULL DEFAULT 'N'," \
                  "InsCod INT NOT NULL DEFAULT -1," \
                  "DptCod INT NOT NULL DEFAULT -1," \
                  "CtrCod INT NOT NULL DEFAULT -1," \
                  "Office VARCHAR(255) NOT NULL," \
                  "OfficePhone CHAR(16) NOT NULL," \
                  "LocalAddress VARCHAR(255) NOT NULL," \
                  "LocalPhone CHAR(16) NOT NULL," \
                  "FamilyAddress VARCHAR(255) NOT NULL," \
                  "FamilyPhone CHAR(16) NOT NULL," \
                  "OriginPlace VARCHAR(255) NOT NULL," \
                  "Birthday DATE NOT NULL," \
                  "Comments TEXT NOT NULL," \
                  "NumNewMsgs INT NOT NULL," \
                  "SideCols TINYINT NOT NULL DEFAULT 3," \
                  "SecureIcons ENUM('N','Y') NOT NULL DEFAULT 'Y'," \
                  "NotifyEvents INT NOT NULL DEFAULT 0," \
                  "LastTab TINYINT NOT NULL," \
                  "LastTime DATETIME NOT NULL," \
                  "LastAccBriefcase DATETIME NOT NULL," \
                  "PRIMARY KEY (UsrCod),UNIQUE INDEX(EncryptedUsrCod),UNIQUE INDEX(DNI),INDEX(Nickname),INDEX(UsrType),INDEX(Theme),INDEX(Language),INDEX(SideCols),INDEX(SecureIcons),INDEX(InsCod),INDEX(DptCod),INDEX(CtrCod),INDEX(LastTime))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");

   /***** Table ws_keys *****/
/*
mysql> DESCRIBE ws_keys;
+----------+----------+------+-----+---------+-------+
| Field    | Type     | Null | Key | Default | Extra |
+----------+----------+------+-----+---------+-------+
| WSKey    | char(43) | NO   | PRI | NULL    |       |
| UsrCod   | int(11)  | NO   | MUL | NULL    |       |
| LastTime | datetime | NO   | MUL | NULL    |       |
+----------+----------+------+-----+---------+-------+
3 rows in set (0.00 sec)
*/
   sprintf (Query,"CREATE TABLE IF NOT EXISTS ws_keys (" \
                  "WSKey CHAR(43) NOT NULL," \
                  "UsrCod INT NOT NULL," \
                  "LastTime DATETIME NOT NULL," \
                  "UNIQUE INDEX(WSKey),INDEX(UsrCod),INDEX(LastTime))");
   fprintf (Gbl.F.Out,"<li class=\"DAT\">%s</li>",Query);       
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError ("can not create table");
   
   /***** Show success message *****/
   fprintf (Gbl.F.Out,"</ol>");
   Lay_ShowAlert (INFO,Txt_Created_tables_in_the_database_that_did_not_exist[Gbl.Prefs.Language]);    
  }
	
/****************************************************************************/
/********************* Open connection to the database **********************/
/****************************************************************************/

void DB_OpenDBConnection (void)
  {
   if (mysql_init (&Gbl.mysql) == NULL)
      Lay_ShowErrorAndExit ("Can not init MySQL.");

   if (mysql_real_connect (&Gbl.mysql,Cfg_DATABASE_HOST,Cfg_DATABASE_USER,Cfg_DATABASE_PASSWORD,Cfg_DATABASE_DBNAME,0,NULL,0) == NULL)
      DB_ExitOnMySQLError ("can not connect to database");

   Gbl.DatabaseIsOpen = true; 
  }

/****************************************************************************/
/********************* Close connection to the database *********************/
/****************************************************************************/

void DB_CloseDBConnection (void)
  {
   if (Gbl.DatabaseIsOpen)
     {
      mysql_close (&Gbl.mysql);	// Close the connection to the database
      Gbl.DatabaseIsOpen = false;
     }
  }

/****************************************************************************/
/******************** Make a SELECT query from database *********************/
/****************************************************************************/

unsigned long DB_QuerySELECT (const char *Query,MYSQL_RES **mysql_res,const char *MsgError)
  {
   /***** Query database *****/
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError (MsgError);

   /***** Store query result *****/
   if ((*mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      DB_ExitOnMySQLError (MsgError);

   /***** Return number of rows of result *****/
   return (unsigned long) mysql_num_rows (*mysql_res);
  }

/****************************************************************************/
/******************* Make an INSERT query from database *********************/
/****************************************************************************/

void DB_QueryINSERT (const char *Query,const char *MsgError)
  {
   /***** Query database *****/
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError (MsgError);
  }

/****************************************************************************/
/* Make an INSERT query from database and return code of last inserted item */
/****************************************************************************/

long DB_QueryINSERTandReturnCode (const char *Query,const char *MsgError)
  {
   /***** Query database *****/
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError (MsgError);
      
   /***** Return the code of the inserted item *****/
   return (long) mysql_insert_id (&Gbl.mysql);
  }

/****************************************************************************/
/******************* Make an REPLACE query from database ********************/
/****************************************************************************/

void DB_QueryREPLACE (const char *Query,const char *MsgError)
  {
   /***** Query database *****/
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError (MsgError);
  }
  
/****************************************************************************/
/******************** Make a UPDATE query from database *********************/
/****************************************************************************/

void DB_QueryUPDATE (const char *Query,const char *MsgError)
  {
   /***** Query database *****/
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError (MsgError);
  }

/****************************************************************************/
/******************** Make a DELETE query from database *********************/
/****************************************************************************/

void DB_QueryDELETE (const char *Query,const char *MsgError)
  {
   /***** Query database *****/
   if (mysql_query (&Gbl.mysql,Query))
      DB_ExitOnMySQLError (MsgError);
  }

/*****************************************************************************/
/********** Free structure that stores the result of a SELECT query **********/
/*****************************************************************************/

void DB_FreeMySQLResult (MYSQL_RES **mysql_res)
  {
   if (*mysql_res)
     {
      mysql_free_result (*mysql_res);
      *mysql_res = NULL;
     }
  }

/****************************************************************************/
/*********** Abort program due to an error in the MySQL database ************/
/****************************************************************************/

void DB_ExitOnMySQLError (const char *Message)
  {
   char BigErrorMsg[1024*1024];
   
   sprintf (BigErrorMsg,"Database error: %s (%s).",
            Message,mysql_error (&Gbl.mysql));
   Lay_ShowErrorAndExit (BigErrorMsg);
  }
