// swad_deg.h: degrees

#ifndef _SWAD_DEG
#define _SWAD_DEG
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_cns.h"

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

#define Deg_MAX_YEARS_PER_DEGREE	  6 // Max number of academic years per degree

#define Deg_MAX_DEGREES_PER_USR		 20 // Used in list of my degrees
#define Deg_MAX_COURSES_PER_USR		100 // Used in list of my courses and list of distinct courses in sent or received messages

#define Deg_MAX_LENGTH_DEGREE_TYPE_NAME		255
#define Deg_MAX_LENGTH_DEGREE_SHORT_NAME	 32
#define Deg_MAX_LENGTH_DEGREE_FULL_NAME		255
#define Deg_MAX_LENGTH_DEGREE_LOGO		 16

#define Deg_MAX_LENGTH_INSTITUTIONAL_CRS_COD	 10
#define Deg_MAX_LENGTH_COURSE_SHORT_NAME	 32
#define Deg_MAX_LENGTH_COURSE_FULL_NAME		255

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

struct Course
  {
   long CrsCod;
   char InstitutionalCrsCod[Deg_MAX_LENGTH_INSTITUTIONAL_CRS_COD+1];	// Institutional code of the course
   long DegCod;
   unsigned Year;					// Year: 0 (optatives), 1, 2, 3...
   unsigned Semester;					// Semester: 0 (full year), 1 or 2
   char ShortName[Deg_MAX_LENGTH_COURSE_SHORT_NAME+1];	// Short name of course
   char FullName[Deg_MAX_LENGTH_COURSE_FULL_NAME+1];	// Full name of course
   unsigned NumStds;					// Number of students
   unsigned NumTchs;					// Number of teachers
   unsigned NumUsrs;					// Number of users (students + teachers)
  };
struct Degree
  {
   long DegTypCod;					// Degree type code
   long DegCod;						// Degree code
   long InsCod;						// Institution code
   long PlcCod;						// Place code
   char ShortName[Deg_MAX_LENGTH_DEGREE_SHORT_NAME+1];	// Short name of degree
   char FullName[Deg_MAX_LENGTH_DEGREE_FULL_NAME+1];	// Full name of degree
   unsigned FirstYear;
   unsigned LastYear;
   bool OptYear;
   char Logo[Deg_MAX_LENGTH_DEGREE_LOGO+1];
   char WWW[Cte_MAX_LENGTH_WWW+1];
   unsigned NumCourses;					// Number of courses in the degree
   struct Course *LstCrss;				// List of courses in this degree
  };
struct DegreeType
  {
   long DegTypCod;					// Cdigo del tipo de degree
   char DegTypName[Deg_MAX_LENGTH_DEGREE_TYPE_NAME+1];	// Degree type name
   unsigned NumDegs;					// Nmero de degrees de este tipo
  };

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Deg_WriteMenuAllCourses (void);
void Deg_WriteSelectorOfDegreeType (void);
void Deg_WriteSelectorOfDegree (void);
void Deg_WriteSelectorOfYear (void);
void Deg_WriteSelectorOfCourse (void);
void Deg_WriteSelectorMyCourses (void);
void Deg_XHTMLDirAccToCrs (void);
void Deg_WriteDegreeAndCourseNames (void);
void Deg_InitCurrentCourse (void);

void Deg_ReqEditDegreeTypes (void);
void Deg_ReqEditDegrees (void);

void Deg_GetListAllDegs (void);
void Deg_FreeListAllDegs (void);
void Deg_FreeListDegTypes (void);
void Deg_FreeListMyAdminDegs (void);

void Deg_RecFormNewDegTyp (void);
void Deg_RecFormNewDeg (void);
void Deg_RemoveDegreeType (void);
void Deg_RemoveDegree (void);
long Deg_GetParamOtherDegCod (void);
long Deg_GetDataOfDegreeTypeByCod (struct DegreeType *DegTyp);
bool Deg_GetDataOfDegreeByCod (struct Degree *Deg);
void Deg_RenameDegreeType (void);
void Deg_RenameDegreeShort (void);
void Deg_RenameDegreeFull (void);
void Deg_ChangeDegreeType (void);
void Deg_ChangeDegreeIns (void);
void Deg_ChangeDegreePlace (void);
void Deg_ChangeDegFirstYear (void);
void Deg_ChangeDegLastYear (void);
void Deg_ChangeDegOptYear (void);
void Deg_ChangeDegLogo (void);
void Deg_ChangeDegWWW (void);

unsigned Deg_GetTotalNumberOfDegrees (void);
unsigned Deg_GetNumberOfDegreesInInstitution (long InsCod);
unsigned Deg_GetNumberOfDegreesInPlace (long PlcCod);
unsigned Deg_GetTotalNumberOfCourses (void);
unsigned Deg_GetNumCoursesInDeg (long DegCod);

void Deg_ReqEditCourses (void);
void Deg_FreeListCoursesInDegree (struct Degree *Deg);
void Deg_RecFormNewCrs (void);
void Deg_RemoveCourse (void);
bool Deg_GetDataOfCourseByCod (struct Course *Crs);
void Deg_ChangeCrsDegree (void);
void Deg_ChangeCrsYear (void);
void Deg_ChangeCrsSemester (void);
void Deg_RenameCourseShort (void);
void Deg_RenameCourseFull (void);
void Deg_ContEditAfterChgCrs (void);

void Deg_PutFormSearchCourse (void);
void Deg_ReqSearchCrsTch (void);
void Deg_GetParamSearchCrs (void);
void Deg_SearchCrs (void);
void Deg_SearchCrsTch (void);

void Deg_GetAndWriteCoursesOf (long UsrCod);
void Deg_GetAndWriteDegreesAdminBy (long UsrCod,unsigned ColSpan);

#endif
