// swad_dpt.c: departments

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_boo.h"
#include "swad_cns.h"
#include "swad_db.h"
#include "swad_dpt.h"
#include "swad_gbl.h"
#include "swad_ins.h"
#include "swad_par.h"
#include "swad_str.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Dpt_GetParamDptOrderType (void);
static void Dpt_PutFormToEditDpts (void);
static void Dpt_ListDepartmentsForEdition (void);
static void Dpt_PutParamDptCod (long DptCod);
static long Dpt_GetParamDptCod (void);
static void Dpt_RenameDepartment (Cte_ShortOrFullName_t ShortOrFullName);
static bool Dpt_CheckIfDepartmentNameExists (const char *FieldName,const char *Name,long DptCod);
static void Dpt_PutFormToCreateDepartment (void);
static void Dpt_PutHeadDepartments (void);
static void Dpt_CreateDepartment (struct Department *Dpt);

/****************************************************************************/
/************************* List all the departments *************************/
/****************************************************************************/

void Dpt_SeeDepts (void)
  {
   extern const char *Txt_Institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_DEPARTMENTS_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_DEPARTMENTS_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_Other_departments[Txt_NUM_LANGUAGES];
   extern const char *Txt_Department_unspecified[Txt_NUM_LANGUAGES];
   tDptsOrderType Order;
   unsigned NumDpt;
   struct Institution Ins;
   unsigned NumTchsWithDpt = 0;
   unsigned NumTchsInOtherDpts;

   /***** Get parameter with the type of order in the list of departments *****/
   Dpt_GetParamDptOrderType ();

   /***** Get list of departments *****/
   Dpt_GetListDepartments ();

   /***** Put link (form) to edit departments *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
       Dpt_PutFormToEditDpts ();

   /***** Table head *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td>",
            Txt_Institution[Gbl.Prefs.Language]);
   for (Order=DPT_ORDER_BY_DEPARTMENT; Order<=DPT_ORDER_BY_NUM_TCHS; Order++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">");
      Act_FormStart (ActSeeDpt);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_DEPARTMENTS_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
      if (Order == Gbl.Dpts.SelectedOrderType)
         fprintf (Gbl.F.Out,"<u>");
      fprintf (Gbl.F.Out,Txt_DEPARTMENTS_ORDER[Order][Gbl.Prefs.Language]);
      if (Order == Gbl.Dpts.SelectedOrderType)
         fprintf (Gbl.F.Out,"</u>");
      fprintf (Gbl.F.Out,"</acronym></a></form></td>");
     }
   fprintf (Gbl.F.Out,"</tr>");


   /***** Write all the departments and their nuber of teachers *****/
   for (NumDpt=0; NumDpt<Gbl.Dpts.Num; NumDpt++)
     {
      /* Get data of institution of this department */
      Ins.InsCod = Gbl.Dpts.Lst[NumDpt].InsCod;
      Ins_GetDataOfInstitutionByCod (&Ins);

      /* Write data of this department */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\">" \
                         "<a href=\"%s\" target=\"_blank\" class=\"DAT\" alt=\"%s\">%s</a></td>",
               Ins.WWW,Ins.FullName,Ins.ShortName);
      fprintf (Gbl.F.Out,"<td align=\"left\">" \
                         "<a href=\"%s\" target=\"_blank\" class=\"DAT\">%s</a></td>",
               Gbl.Dpts.Lst[NumDpt].WWW,Gbl.Dpts.Lst[NumDpt].FullName);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Gbl.Dpts.Lst[NumDpt].NumTchs);
      NumTchsWithDpt += Gbl.Dpts.Lst[NumDpt].NumTchs;
     }

   /***** Separation row *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;</td>" \
                      "</tr>");

   /***** Write teachers with other department *****/
   NumTchsInOtherDpts = Usr_GetNumberOfTeachersInDepartment (0);
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "</tr>",
            Txt_Other_departments[Gbl.Prefs.Language],NumTchsInOtherDpts);
   NumTchsWithDpt += NumTchsInOtherDpts;

   /***** Write teachers with no department *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "</tr>",
            Txt_Department_unspecified[Gbl.Prefs.Language],
            Sta_GetTotalNumberOfUsers (Sta_LOCATION_RANGE_ALL,Usr_TEACHER) - NumTchsWithDpt);

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of departments *****/
   Dpt_FreeListDepartments ();
  }

/*****************************************************************************/
/******** Get parameter with the type or order in list of departments ********/
/*****************************************************************************/

static void Dpt_GetParamDptOrderType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Dpts.SelectedOrderType = (tDptsOrderType) UnsignedNum;
   else
      Lay_ShowErrorAndExit ("Wrong sort order.");
  }

/****************************************************************************/
/*** Put a hidden parameter with the type of order in list of departments ***/
/****************************************************************************/

void Dpt_PutHiddenParamDptOrderType (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Dpts.SelectedOrderType);
  }

/****************************************************************************/
/******************** Put a link (form) to edit departments *****************/
/****************************************************************************/

static void Dpt_PutFormToEditDpts (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiDpt);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************ Put forms to edit degree types ********************/
/****************************************************************************/

void Dpt_EditDepartments (void)
  {
   extern const char *Txt_Departments[Txt_NUM_LANGUAGES];

   /***** Get list of institutions *****/
   Ins_GetListInstitutions (false);
   if (!Gbl.Inss.Num)
     {
      Lay_ShowAlert (WARNING,"No hay lista de instituciones." \
                             " Debe crear al menos una instituci&oacute;n antes de crear departamentos.");	// Need translation!
      return;
     }

   /***** Get list of departments *****/
   Dpt_GetListDepartments ();

   /***** Help message *****/
   if (Gbl.Dpts.Num) // Departments found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo departamento rellene el nombre del departamento y su direcci&oacute;n web," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar un departamento existente pulse el aspa roja.<br />" \
                          "Para modificar un departamento existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Departments[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No degree types found
      Lay_ShowAlert (INFO,"No existen departamentos.<br />" \
                          "Para a&ntilde;adir un departamento rellene el nombre del departamento y su direcci&oacute;n web," \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit departments *****/
   /* Start table with departments */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current departments */
   if (Gbl.Dpts.Num)
      Dpt_ListDepartmentsForEdition ();

   /* Put a form to create a new department */
   Dpt_PutFormToCreateDepartment ();

   /* End of table with departments */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of departments *****/
   Dpt_FreeListDepartments ();

   /***** Free list of institutions *****/
   Ins_FreeListInstitutions ();
  }

/****************************************************************************/
/************************* List all the departments *************************/
/****************************************************************************/

void Dpt_GetListDepartments (void)
  {
   char OrderBySubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumDpt;
   struct Department *Dpt;

   /***** Get departments from database *****/
   switch (Gbl.Dpts.SelectedOrderType)
     {
      case DPT_ORDER_BY_DEPARTMENT:
         sprintf (OrderBySubQuery,"FullName");
         break;
      case DPT_ORDER_BY_NUM_TCHS:
         sprintf (OrderBySubQuery,"NumTchs DESC,FullName");
     }
   sprintf (Query,"(SELECT departments.DptCod,departments.InsCod,departments.ShortName,departments.FullName,departments.WWW,COUNT(*) AS NumTchs" \
                  " FROM departments,usr_data" \
                  " WHERE usr_data.UsrType='%u' AND usr_data.DptCod=departments.DptCod" \
                  " GROUP BY departments.DptCod)" \
                  " UNION " \
                  "(SELECT DptCod,InsCod,ShortName,FullName,WWW,0 AS NumTchs" \
                  " FROM departments" \
                  " WHERE DptCod NOT IN (SELECT DISTINCT DptCod FROM usr_data WHERE UsrType='%u'))" \
                  " ORDER BY %s",
            (unsigned) Usr_TEACHER,(unsigned) Usr_TEACHER,OrderBySubQuery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get departments");

   if (NumRows) // Departments found...
     {
      // NumRows should be equal to Deg->NumCourses
      Gbl.Dpts.Num = (unsigned) NumRows;

      /***** Create list with courses in degree *****/
      if ((Gbl.Dpts.Lst = (struct Department *) calloc (NumRows,sizeof (struct Department))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store departments.");

      /***** Get the departments *****/
      for (NumDpt=0; NumDpt<Gbl.Dpts.Num; NumDpt++)
        {
         Dpt = &(Gbl.Dpts.Lst[NumDpt]);

         /* Get next department */
         row = mysql_fetch_row (mysql_res);

         /* Get department code (row[0]) */
         if ((Dpt->DptCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of department.");

         /* Get institution code (row[1]) */
         if ((Dpt->InsCod = Str_ConvertStrCodToLongCod (row[1])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of institution.");

         /* Get the short name of the department (row[2]) */
         strcpy (Dpt->ShortName,row[2]);

         /* Get the full name of the department (row[3]) */
         strcpy (Dpt->FullName,row[3]);

         /* Get the URL of the department (row[4]) */
         strcpy (Dpt->WWW,row[4]);

         /* Get number of teachers in this department (row[5]) */
         if (sscanf (row[5],"%u",&Dpt->NumTchs) != 1)
            Dpt->NumTchs = 0;
        }
     }
   else
      Gbl.Dpts.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);   
  }

/*****************************************************************************/
/************************** Get department full name *************************/
/*****************************************************************************/

void Dpt_GetDataOfDepartmentByCod (struct Department *Dpt)
  {
   extern const char *Txt_Another_department[Txt_NUM_LANGUAGES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Clear data *****/
   Dpt->ShortName[0] = Dpt->FullName[0] = Dpt->WWW[0] = '\0';
   Dpt->NumTchs = 0;

   /***** Check if department code is correct *****/
   if (Dpt->DptCod == 0)
     {
      strcpy (Dpt->ShortName,Txt_Another_department[Gbl.Prefs.Language]);
      strcpy (Dpt->FullName,Txt_Another_department[Gbl.Prefs.Language]);
     }
   else if (Dpt->DptCod > 0)
     {
      /***** Get data of a department from database *****/
      sprintf (Query,"(SELECT departments.ShortName,departments.FullName,departments.WWW,COUNT(*)" \
                     " FROM departments,usr_data" \
                     " WHERE departments.DptCod='%ld' AND usr_data.UsrType='%u' AND departments.DptCod=usr_data.DptCod" \
                     " GROUP BY departments.DptCod)"
                     " UNION " \
                     "(SELECT ShortName,FullName,WWW,0" \
                     " FROM departments" \
                     " WHERE DptCod='%ld' AND DptCod NOT IN (SELECT DISTINCT DptCod FROM usr_data WHERE UsrType='%u'))",
               Dpt->DptCod,(unsigned) Usr_TEACHER,Dpt->DptCod,(unsigned) Usr_TEACHER);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a department");

      if (NumRows) // Department found...
        {
         /* Get row */
         row = mysql_fetch_row (mysql_res);

         /* Get the short name of the department (row[0]) */
         strcpy (Dpt->ShortName,row[0]);

         /* Get the full name of the department (row[1]) */
         strcpy (Dpt->FullName,row[1]);

         /* Get the URL of the department (row[2]) */
         strcpy (Dpt->WWW,row[2]);

         /* Get number of teachers in this department (row[3]) */
         if (sscanf (row[3],"%u",&Dpt->NumTchs) != 1)
            Dpt->NumTchs = 0;
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/************************** Free list of departments *************************/
/*****************************************************************************/

void Dpt_FreeListDepartments (void)
  {
   if (Gbl.Dpts.Lst)
     {
      /***** Free memory used by the list of courses in degree *****/
      free ((void *) Gbl.Dpts.Lst);
      Gbl.Dpts.Lst = NULL;
      Gbl.Dpts.Num = 0;
     }
  }

/****************************************************************************/
/************************* List all the departments *************************/
/****************************************************************************/

static void Dpt_ListDepartmentsForEdition (void)
  {
   extern const char *Txt_Remove_department[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   unsigned NumDpt;
   struct Department *Dpt;
   struct Institution Ins;
   unsigned NumIns;

   /***** Table head *****/
   Dpt_PutHeadDepartments ();

   /***** Write all the departments *****/
   for (NumDpt=0; NumDpt<Gbl.Dpts.Num; NumDpt++)
     {
      Dpt = &Gbl.Dpts.Lst[NumDpt];

      /* Get data of institution of this department */
      Ins.InsCod = Dpt->InsCod;
      Ins_GetDataOfInstitutionByCod (&Ins);

      /* Put icon to remove department */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      if (Dpt->NumTchs)	// Department has teachers ==> deletion forbidden
         fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActRemDpt);
         Dpt_PutParamDptCod (Dpt->DptCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_department[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Txt_Remove_department[Gbl.Prefs.Language]);
        }
      fprintf (Gbl.F.Out,"</td>");

      /* Department code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Dpt->DptCod);

      /* Institution */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDptIns);
      Dpt_PutParamDptCod (Dpt->DptCod);
      fprintf (Gbl.F.Out,"<select name=\"InsCod\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
      fprintf (Gbl.F.Out,"<option value=\"0\"");
      if (Dpt->InsCod == 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
      for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                  Gbl.Inss.Lst[NumIns].InsCod,
                  (Gbl.Inss.Lst[NumIns].InsCod == Dpt->InsCod) ? " selected=\"selected\"" : "",
                  Gbl.Inss.Lst[NumIns].ShortName);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Department short name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenDptSho);
      Dpt_PutParamDptCod (Dpt->DptCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_DEPARTMENT_SHORT_NAME,Dpt->ShortName,Gbl.NumForm);

      /* Department full name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenDptFul);
      Dpt_PutParamDptCod (Dpt->DptCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_DEPARTMENT_FULL_NAME,Dpt->FullName,Gbl.NumForm);

      /* Department WWW */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDptWWW);
      Dpt_PutParamDptCod (Dpt->DptCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"WWW\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Cte_MAX_LENGTH_WWW,Dpt->WWW,Gbl.NumForm);

      /* Number of teachers */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Dpt->NumTchs);
     }
  }

/****************************************************************************/
/****************** Write parameter with code of department *****************/
/****************************************************************************/

static void Dpt_PutParamDptCod (long DptCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"DptCod\" value=\"%ld\" />",DptCod);
  }

/****************************************************************************/
/******************* Get parameter with code of degree type *****************/
/****************************************************************************/

static long Dpt_GetParamDptCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of department *****/
   Par_GetParam (Par_PARAM_SINGLE,"DptCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/***************************** Remove a department **************************/
/****************************************************************************/

void Dpt_RemoveDepartment (void)
  {
   extern const char *Txt_Department_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Department Dpt;

   /***** Get department code *****/
   if ((Dpt.DptCod = Dpt_GetParamDptCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of department is missing.");

   /***** Get data of the department from database *****/
   Dpt_GetDataOfDepartmentByCod (&Dpt);

   /***** Check if this department has teachers *****/
   if (Dpt.NumTchs)	// Department has teachers ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar un departamento," \
                             " primero debe eliminar todos los profesores de ese departamento.");	// Need translation!
   else	// Department has no teachers ==> remove it
     {
      /***** Remove department *****/
      sprintf (Query,"DELETE FROM departments WHERE DptCod='%ld'",Dpt.DptCod);
      DB_QueryDELETE (Query,"can not remove a department");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Department_X_removed[Gbl.Prefs.Language],
               Dpt.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Dpt_EditDepartments ();
  }

/****************************************************************************/
/****************** Change the institution of a department ******************/
/****************************************************************************/

void Dpt_ChangeDepartIns (void)
  {
   struct Department *Dpt;
   char Query[512];

   Dpt = &Gbl.Dpts.EditingDpt;

   /***** Get parameters from form *****/
   /* Get the code of the department */
   if ((Dpt->DptCod = Dpt_GetParamDptCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of department is missing.");

   /* Get parameter with institution code */
   Dpt->InsCod = Ins_GetParamInsCod ();

   /***** Update institution in table of departments *****/
   sprintf (Query,"UPDATE departments SET InsCod='%ld' WHERE DptCod='%ld'",
            Dpt->InsCod,Dpt->DptCod);
   DB_QueryUPDATE (Query,"can not update the institution of a department");

   /***** Write message to show the change made *****/
   Lay_ShowAlert (INFO,"Se ha cambiado la instituci&oacute;n del departamento.");	// Need translation!

   /***** Show the form again *****/
   Dpt_EditDepartments ();
  }

/****************************************************************************/
/******************* Change the short name of a department ******************/
/****************************************************************************/

void Dpt_RenameDepartShort (void)
  {
   Dpt_RenameDepartment (Cte_SHORT_NAME);
  }

/****************************************************************************/
/******************* Change the full name of a department *******************/
/****************************************************************************/

void Dpt_RenameDepartFull (void)
  {
   Dpt_RenameDepartment (Cte_FULL_NAME);
  }

/****************************************************************************/
/************************ Change the name of a degree ***********************/
/****************************************************************************/

static void Dpt_RenameDepartment (Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_department_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Department *Dpt;
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentDptName = NULL;		// Initialized to avoid warning
   char NewDptName[MAX_LENGTH_DEPARTMENT_FULL_NAME+1];

   Dpt = &Gbl.Dpts.EditingDpt;
   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = MAX_LENGTH_DEPARTMENT_SHORT_NAME;
         CurrentDptName = Dpt->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = MAX_LENGTH_DEPARTMENT_FULL_NAME;
         CurrentDptName = Dpt->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the department */
   if ((Dpt->DptCod = Dpt_GetParamDptCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of department is missing.");

   /* Get the new name for the department */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewDptName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewDptName,MaxLength,true);

   /***** Get from the database the old names of the department *****/
   Dpt_GetDataOfDepartmentByCod (Dpt);

   /***** Check if new name is empty *****/
   if (!NewDptName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del departamento <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentDptName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentDptName,NewDptName))	// Different names
        {
         /***** If degree was in database... *****/
         if (Dpt_CheckIfDepartmentNameExists (ParamName,NewDptName,Dpt->DptCod))
           {
            sprintf (Gbl.Message,Txt_The_department_X_already_exists[Gbl.Prefs.Language],
                     NewDptName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE departments SET %s='%s' WHERE DptCod='%ld'",
                     FieldName,NewDptName,Dpt->DptCod);
            DB_QueryUPDATE (Query,"can not update the name of a department");	// Need translation!

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El departamento <strong>%s</strong>" \
                                 " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     CurrentDptName,NewDptName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del departamento <strong>%s</strong> no ha cambiado.",	// Need translation!
                  CurrentDptName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (CurrentDptName,NewDptName);
   Dpt_EditDepartments ();
  }

/****************************************************************************/
/******************* Check if the name of department exists *****************/
/****************************************************************************/

static bool Dpt_CheckIfDepartmentNameExists (const char *FieldName,const char *Name,long DptCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of departments with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM departments WHERE %s='%s' AND DptCod<>'%ld'",
            FieldName,Name,DptCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a department already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of a department already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/*********************** Change the URL of a department *********************/
/****************************************************************************/

void Dpt_ChangeDptWWW (void)
  {
   struct Department *Dpt;
   char Query[256+Cte_MAX_LENGTH_WWW];
   char NewWWW[Cte_MAX_LENGTH_WWW+1];

   Dpt = &Gbl.Dpts.EditingDpt;

   /***** Get parameters from form *****/
   /* Get the code of the department */
   if ((Dpt->DptCod = Dpt_GetParamDptCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of department is missing.");

   /* Get the new WWW for the department */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",NewWWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewWWW,Cte_MAX_LENGTH_WWW,true);

   /***** Check if new WWW is empty *****/
   if (NewWWW[0])
     {
      /* Update the table changing old WWW by new WWW */
      sprintf (Query,"UPDATE departments SET WWW='%s' WHERE DptCod='%ld'",
               NewWWW,Dpt->DptCod);
      DB_QueryUPDATE (Query,"can not update the web of a department");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva direcci&oacute;n web es <strong>%s</strong>.",	// Need translation!
               NewWWW);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar la direcci&oacute;n web vac&iacute;a.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Dpt->WWW,NewWWW);
   Dpt_EditDepartments ();
  }

/****************************************************************************/
/******************* Put a form to create a new department ******************/
/****************************************************************************/

static void Dpt_PutFormToCreateDepartment (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_department[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_department[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   struct Department *Dpt;
   unsigned NumIns;

   Dpt = &Gbl.Dpts.EditingDpt;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"7\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"7\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_department[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewDpt);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Dpt_PutHeadDepartments ();

   /***** New department *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_department[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Department code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Institution *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"InsCod\" style=\"width:50px;\">");
   fprintf (Gbl.F.Out,"<option value=\"0\"");
   if (Dpt->InsCod == 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
   for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
      fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
               Gbl.Inss.Lst[NumIns].InsCod,
               (Gbl.Inss.Lst[NumIns].InsCod == Dpt->InsCod) ? " selected=\"selected\"" : "",
               Gbl.Inss.Lst[NumIns].ShortName);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Department short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_DEPARTMENT_SHORT_NAME,Dpt->ShortName);

   /***** Department full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_DEPARTMENT_FULL_NAME,Dpt->FullName);

   /***** Department WWW *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"WWW\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Cte_MAX_LENGTH_WWW,Dpt->WWW);
  }

/****************************************************************************/
/******************** Write header with fields of a degree ******************/
/****************************************************************************/

static void Dpt_PutHeadDepartments (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_Name[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" class=\"TIT_TBL\"></td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">WWW</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">Prof.</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Institution[Gbl.Prefs.Language],
            Txt_Short_Name[Gbl.Prefs.Language],
            Txt_Full_Name[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/***************** Receive form to create a new department ******************/
/****************************************************************************/

void Dpt_RecFormNewDpt (void)
  {
   extern const char *Txt_The_department_X_already_exists[Txt_NUM_LANGUAGES];
   struct Department *Dpt;

   Dpt = &Gbl.Dpts.EditingDpt;

   /***** Get parameters from form *****/
   /* Get institution */
   Dpt->InsCod = Ins_GetParamInsCod ();

   /* Get department short name */
   Par_GetParam (Par_PARAM_SINGLE,"ShortName",Dpt->ShortName,MAX_LENGTH_DEPARTMENT_SHORT_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Dpt->ShortName,MAX_LENGTH_DEPARTMENT_SHORT_NAME,true);

   /* Get department full name */
   Par_GetParam (Par_PARAM_SINGLE,"FullName",Dpt->FullName,MAX_LENGTH_DEPARTMENT_FULL_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Dpt->FullName,MAX_LENGTH_DEPARTMENT_FULL_NAME,true);

   /* Get department WWW */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",Dpt->WWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Dpt->WWW,Cte_MAX_LENGTH_WWW,true);

   if (Dpt->ShortName[0] && Dpt->FullName[0])	// If there's a department name
     {
      if (Dpt->WWW[0])
        {
         /***** If name of department was in database... *****/
         if (Dpt_CheckIfDepartmentNameExists ("ShortName",Dpt->ShortName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_department_X_already_exists[Gbl.Prefs.Language],
                     Dpt->ShortName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else if (Dpt_CheckIfDepartmentNameExists ("FullName",Dpt->FullName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_department_X_already_exists[Gbl.Prefs.Language],
                     Dpt->FullName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else	// Add new department to database
            Dpt_CreateDepartment (Dpt);
        }
      else	// If there isn't a web
        {
         sprintf (Gbl.Message,"Debe especificar la direcci&oacute;n web del nuevo departamento.");	// Need translation!
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else	// If there isn't a department name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre breve y el nombre completo del nuevo departamento.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Dpt_EditDepartments ();
  }

/****************************************************************************/
/************************** Create a new department *************************/
/****************************************************************************/

static void Dpt_CreateDepartment (struct Department *Dpt)
  {
   extern const char *Txt_Created_new_department_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new department *****/
   sprintf (Query,"INSERT INTO departments (InsCod,ShortName,FullName,WWW)" \
                  " VALUES ('%ld','%s','%s','%s')",
            Dpt->InsCod,Dpt->ShortName,Dpt->FullName,Dpt->WWW);
   DB_QueryINSERT (Query,"can not create a new department");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_department_X[Gbl.Prefs.Language],
            Dpt->FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/************************** Get number of departments ***********************/
/****************************************************************************/

unsigned Dpt_GetTotalNumberOfDepartments (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDpts;

   /***** Get number of departments from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM departments");
   DB_QuerySELECT (Query,&mysql_res,"can not get number of departments");

   /***** Get number of departments *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumDpts) != 1)
      NumDpts = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumDpts;
  }

/****************************************************************************/
/**************** Get number of departments in a institution ****************/
/****************************************************************************/

unsigned Dpt_GetNumberOfDepartmentsInInstitution (long InsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDpts;

   /***** Get departments in an institution from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM departments" \
                  " WHERE InsCod='%ld'",InsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of departments in an institution");

   /***** Get number of departments *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumDpts) != 1)
      NumDpts = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumDpts;
  }
