// swad_exa.c: exam announcements

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdio.h>		// For sscanf, etc.
#include <stdlib.h>		// For exit, system, malloc, calloc, free, etc.
#include <string.h>		// For string functions

#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_exa.h"
#include "swad_gbl.h"
#include "swad_ntf.h"
#include "swad_par.h"
#include "swad_str.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static long Exa_GetParamsExamAnnouncement (void);
static void Exa_AllocMemExamAnnouncement (void);
static void Exa_UpdateNumUsrsNotifiedAboutExamAnnouncement (long ExaCod,unsigned NumUsrsToBeNotified);
static void Exa_ListExamAnnouncementsEdit (void);
static void Exa_ListExamAnnouncements (Exa_tTypeViewExamAnnouncement_t TypeViewExamAnnouncement);
static long Exa_AddExamAnnouncementToDB (void);
static void Exa_ModifyExamAnnouncementInDB (long ExaCod);
static void Exa_GetDataExamAnnouncementFromDB (long ExaCod);
static void Exa_ShowExamAnnouncement (long ExaCod,Exa_tTypeViewExamAnnouncement_t TypeViewExamAnnouncement);

/****************************************************************************/
/********************** Form to edit an exam announcement *******************/
/****************************************************************************/

void Exa_PutFrmEditAExamAnnouncement (void)
  {
   long ExaCod;

   /***** Allocate memory for the exam announcement *****/
   Exa_AllocMemExamAnnouncement ();

   /***** Get the code of the exam announcement *****/
   ExaCod = Exa_GetParamsExamAnnouncement ();

   if (ExaCod >= 0)	// -1 indicates that this is a new exam announcement
      /***** Read exam announcement from the database *****/
      Exa_GetDataExamAnnouncementFromDB (ExaCod);

   /***** Show exam announcement *****/
   Exa_ShowExamAnnouncement (ExaCod,Exa_FORM_VIEW);

   /***** Free memory of the exam announcement *****/
   Exa_FreeMemExamAnnouncement ();
  }

/****************************************************************************/
/**************** Get parameters of an exam announcement ********************/
/****************************************************************************/

static long Exa_GetParamsExamAnnouncement (void)
  {
   char UnsignedStr[10+1];
   char LongStr[1+10+1];
   long ExaCod;

   /***** Get the code of the exam announcement *****/
   Par_GetParam (Par_PARAM_SINGLE,"ExaCod",LongStr,1+10);
   if (sscanf (LongStr,"%ld",&ExaCod) != 1)
      ExaCod = -1;

   /***** Get the name of the course (it is allowed to be different from the official name of the course) *****/
   Par_GetParam (Par_PARAM_SINGLE,"CrsName",Gbl.ExamAnnouncement.CrsFullName,Cte_MAX_BYTES_STRING);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.CrsFullName,Cte_MAX_BYTES_STRING,true);
   // If the parameter is not present or is empty, initialize the string to the full name of the current course
   if (!Gbl.ExamAnnouncement.CrsFullName[0])
      strcpy (Gbl.ExamAnnouncement.CrsFullName,Gbl.CurrentCrs.Crs.FullName);

   /***** Get the year *****/
   Par_GetParam (Par_PARAM_SINGLE,"Year",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.ExamAnnouncement.Year) != 1)
      Gbl.ExamAnnouncement.Year = Gbl.CurrentCrs.Crs.Year;
   if (Gbl.ExamAnnouncement.Year > Deg_MAX_YEARS_PER_DEGREE)
      Gbl.ExamAnnouncement.Year = Gbl.CurrentCrs.Crs.Year;

   /***** Get the type of exam announcement *****/
   Par_GetParam (Par_PARAM_SINGLE,"ExamSession",Gbl.ExamAnnouncement.Session,Cte_MAX_BYTES_STRING);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.Session,Cte_MAX_BYTES_STRING,true);

   /***** Get the data of the exam *****/
   Dat_GetDateFromForm ("ExamDay","ExamMonth","ExamYear",&Gbl.ExamAnnouncement.ExamDate.Day,&Gbl.ExamAnnouncement.ExamDate.Month,&Gbl.ExamAnnouncement.ExamDate.Year);
   if (Gbl.ExamAnnouncement.ExamDate.Day   == 0 ||
       Gbl.ExamAnnouncement.ExamDate.Month == 0 ||
       Gbl.ExamAnnouncement.ExamDate.Year  == 0)
     {
      Gbl.ExamAnnouncement.ExamDate.Day   = Gbl.Today.Day;
      Gbl.ExamAnnouncement.ExamDate.Month = Gbl.Today.Month;
      Gbl.ExamAnnouncement.ExamDate.Year  = Gbl.Today.Year;
     }

   /***** Get the hour of the exam *****/
   Par_GetParam (Par_PARAM_SINGLE,"ExamHour",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.ExamAnnouncement.StartTime.Hour) != 1)
      Gbl.ExamAnnouncement.StartTime.Hour = 0;
   if (Gbl.ExamAnnouncement.StartTime.Hour > 23)
      Gbl.ExamAnnouncement.StartTime.Hour = 0;
   Par_GetParam (Par_PARAM_SINGLE,"ExamMinute",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.ExamAnnouncement.StartTime.Minute) != 1)
      Gbl.ExamAnnouncement.StartTime.Minute = 0;
   if (Gbl.ExamAnnouncement.StartTime.Minute > 59)
      Gbl.ExamAnnouncement.StartTime.Minute = 0;

   /***** Get the duration of the exam *****/
   Par_GetParam (Par_PARAM_SINGLE,"DurationHour",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.ExamAnnouncement.Duration.Hour) != 1)
      Gbl.ExamAnnouncement.Duration.Hour = 0;
   if (Gbl.ExamAnnouncement.Duration.Hour > 23)
      Gbl.ExamAnnouncement.Duration.Hour = 0;
   Par_GetParam (Par_PARAM_SINGLE,"DurationMinute",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.ExamAnnouncement.Duration.Minute) != 1)
      Gbl.ExamAnnouncement.Duration.Minute = 0;
   if (Gbl.ExamAnnouncement.Duration.Minute > 59)
      Gbl.ExamAnnouncement.Duration.Minute = 0;

   /***** Get the place where the exam will happen *****/
   Par_GetParam (Par_PARAM_SINGLE,"Place",Gbl.ExamAnnouncement.Place,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.Place,Cte_MAX_BYTES_TEXT,true);

   /***** Get the modality of exam *****/
   Par_GetParam (Par_PARAM_SINGLE,"ExamMode",Gbl.ExamAnnouncement.Mode,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.Mode,Cte_MAX_BYTES_TEXT,true);

   /***** Get the structure of exam *****/
   Par_GetParam (Par_PARAM_SINGLE,"Structure",Gbl.ExamAnnouncement.Structure,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.Structure,Cte_MAX_BYTES_TEXT,true);

   /***** Get the mandatory documentation *****/
   Par_GetParam (Par_PARAM_SINGLE,"DocRequired",Gbl.ExamAnnouncement.DocRequired,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.DocRequired,Cte_MAX_BYTES_TEXT,true);

   /***** Get the mandatory material *****/
   Par_GetParam (Par_PARAM_SINGLE,"MatRequired",Gbl.ExamAnnouncement.MatRequired,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.MatRequired,Cte_MAX_BYTES_TEXT,true);

   /***** Get the allowed material *****/
   Par_GetParam (Par_PARAM_SINGLE,"MatAllowed",Gbl.ExamAnnouncement.MatAllowed,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.MatAllowed,Cte_MAX_BYTES_TEXT,true);

   /***** Get other information *****/
   Par_GetParam (Par_PARAM_SINGLE,"OtherInfo",Gbl.ExamAnnouncement.OtherInfo,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.ExamAnnouncement.OtherInfo,Cte_MAX_BYTES_TEXT,true);

   return ExaCod;
  }

/****************************************************************************/
/* Allocate memory for those parameters of a call for e. with a lot of text */
/****************************************************************************/

static void Exa_AllocMemExamAnnouncement (void)
  {
   if ((Gbl.ExamAnnouncement.Place = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");

   if ((Gbl.ExamAnnouncement.Mode = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");

   if ((Gbl.ExamAnnouncement.Structure = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");

   if ((Gbl.ExamAnnouncement.DocRequired = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");

   if ((Gbl.ExamAnnouncement.MatRequired = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");

   if ((Gbl.ExamAnnouncement.MatAllowed = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");

   if ((Gbl.ExamAnnouncement.OtherInfo = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store exam announcement.");
  }

/****************************************************************************/
/*** Free memory of those parameters of a call for e. with a lot of text ****/
/****************************************************************************/

void Exa_FreeMemExamAnnouncement (void)
  {
   if (Gbl.ExamAnnouncement.Place)
     {
      free ((void *) Gbl.ExamAnnouncement.Place);
      Gbl.ExamAnnouncement.Place = NULL;
     }
   if (Gbl.ExamAnnouncement.Mode)
     {
      free ((void *) Gbl.ExamAnnouncement.Mode);
      Gbl.ExamAnnouncement.Mode = NULL;
     }
   if (Gbl.ExamAnnouncement.Structure)
     {
      free ((void *) Gbl.ExamAnnouncement.Structure);
      Gbl.ExamAnnouncement.Structure = NULL;
     }
   if (Gbl.ExamAnnouncement.DocRequired)
     {
      free ((void *) Gbl.ExamAnnouncement.DocRequired);
      Gbl.ExamAnnouncement.DocRequired = NULL;
     }
   if (Gbl.ExamAnnouncement.MatRequired)
     {
      free ((void *) Gbl.ExamAnnouncement.MatRequired);
      Gbl.ExamAnnouncement.MatRequired = NULL;
     }
   if (Gbl.ExamAnnouncement.MatAllowed)
     {
      free ((void *) Gbl.ExamAnnouncement.MatAllowed);
      Gbl.ExamAnnouncement.MatAllowed = NULL;
     }
   if (Gbl.ExamAnnouncement.OtherInfo)
     {
      free ((void *) Gbl.ExamAnnouncement.OtherInfo);
      Gbl.ExamAnnouncement.OtherInfo = NULL;
     }
  }

/****************************************************************************/
/************************ Receive an exam announcement **********************/
/****************************************************************************/

void Exa_ReceiveExamAnnouncement (void)
  {
   extern const char *Txt_Created_new_announcement_of_exam[Txt_NUM_LANGUAGES];
   long ExaCod;
   bool NewExamAnnouncement;
   unsigned NumUsrsToBeNotified;

   /***** Allocate memory for the exam announcement *****/
   Exa_AllocMemExamAnnouncement ();

   /***** Get parameters of the exam announcement *****/
   ExaCod = Exa_GetParamsExamAnnouncement ();
   NewExamAnnouncement = (ExaCod == -1L);	// -1 indicates that it is a new exam announcement

   /***** Add the exam announcement to the database and read it again from the database *****/
   if (NewExamAnnouncement)
      ExaCod = Exa_AddExamAnnouncementToDB ();
   else
      Exa_ModifyExamAnnouncementInDB (ExaCod);

   /***** Free memory of the exam announcement *****/
   Exa_FreeMemExamAnnouncement ();

   /***** Show message *****/
   Lay_ShowAlert (INFO,NewExamAnnouncement ? Txt_Created_new_announcement_of_exam[Gbl.Prefs.Language] :
                                             "La convocatoria de examen se ha actualizado correctamente.");	// Need translation!

   /***** Notify by e-mail about the new exam announcement *****/
   if ((NumUsrsToBeNotified = Ntf_StoreNotifyEventsToAllUsrs (Ntf_NEW_EXAM_ANNOUNCEMENT,-1L,-1L)))
      Exa_UpdateNumUsrsNotifiedAboutExamAnnouncement (ExaCod,NumUsrsToBeNotified);
   Ntf_ShowAlertNumUsrsToBeNotified (NumUsrsToBeNotified);

   /***** Show exam announcement *****/
   Exa_ListExamAnnouncementsEdit ();
  }

/****************************************************************************/
/***** Update number of users notified in table of exam announcements *******/
/****************************************************************************/

static void Exa_UpdateNumUsrsNotifiedAboutExamAnnouncement (long ExaCod,unsigned NumUsrsToBeNotified)
  {
   char Query[512];

   /***** Update number of users notified *****/
   sprintf (Query,"UPDATE exam_announcements SET NumNotif=NumNotif+'%u' WHERE ExaCod='%ld'",
            NumUsrsToBeNotified,ExaCod);
   DB_QueryUPDATE (Query,"can not update number of notifications of exam announcements");
  }

/****************************************************************************/
/************************* Print an exam announcement ***********************/
/****************************************************************************/

void Exa_PrintExamAnnouncement (void)
  {
   char LongStr[1+10+1];
   long ExaCod;

   /***** Allocate memory for the exam announcement *****/
   Exa_AllocMemExamAnnouncement ();

   /***** Get the code of the exam announcement *****/
   Par_GetParam (Par_PARAM_SINGLE,"ExaCod",LongStr,1+10);
   if (sscanf (LongStr,"%ld",&ExaCod) != 1)
      Lay_ShowErrorAndExit ("Code of exam announcement is missing.");

   /***** Read exam announcement from the database *****/
   Exa_GetDataExamAnnouncementFromDB (ExaCod);

   /***** Show exam announcement *****/
   Exa_ShowExamAnnouncement (ExaCod,Exa_PRINT_VIEW);

   /***** Free memory of the exam announcement *****/
   Exa_FreeMemExamAnnouncement ();
  }

/****************************************************************************/
/*********************** Remove an exam announcement ************************/
/****************************************************************************/

void Exa_RemoveExamAnnouncement (void)
  {
   extern const char *Txt_Announcement_of_exam_removed[Txt_NUM_LANGUAGES];
   char Query[1024];
   char LongStr[1+10+1];
   long ExaCod;

   /***** Get the code of the exam announcement *****/
   Par_GetParam (Par_PARAM_SINGLE,"ExaCod",LongStr,1+10);
   if (sscanf (LongStr,"%ld",&ExaCod) != 1)
      Lay_ShowErrorAndExit ("Code of exam announcement is missing.");

   /***** Mark the exam announcement as deleted in the database *****/
   sprintf (Query,"UPDATE exam_announcements SET Status='%u' WHERE ExaCod='%ld'",
            (unsigned) Exa_DELETED_EXAM_ANNOUNCEMENT,ExaCod);
   DB_QueryUPDATE (Query,"can not remove an exam announcement");

   /***** Write message *****/
   Lay_ShowAlert (INFO,Txt_Announcement_of_exam_removed[Gbl.Prefs.Language]);

   /***** List again all the remaining exam announcements *****/
   Exa_ListExamAnnouncementsEdit ();
  }

/****************************************************************************/
/*************** List all the exam announcements to see them ****************/
/****************************************************************************/

void Exa_ListExamAnnouncementsSee (void)
  {
   Exa_ListExamAnnouncements (Exa_NORMAL_VIEW);
  }

/****************************************************************************/
/********** List all the exam announcements to edit or remove them **********/
/****************************************************************************/

static void Exa_ListExamAnnouncementsEdit (void)
  {
   Exa_ListExamAnnouncements (Exa_NORMAL_VIEW);
  }

/****************************************************************************/
/******************** List all the exam announcements ***********************/
/****************************************************************************/

static void Exa_ListExamAnnouncements (Exa_tTypeViewExamAnnouncement_t TypeViewExamAnnouncement)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_No_announcements_of_exams_of_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_announcement_of_exam[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long ExaCod;

   /***** Get exam announcements in current course from database *****/
   sprintf (Query,"SELECT ExaCod,ExamDate FROM exam_announcements" \
                  " WHERE CrsCod='%ld' AND Status<>'%u' ORDER BY ExamDate",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Exa_DELETED_EXAM_ANNOUNCEMENT);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get exam announcements in this course");

   /***** The result of the query puede estar vaco *****/
   if (NumRows == 0)
     {
      sprintf (Gbl.Message,Txt_No_announcements_of_exams_of_X[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Create link for creation of a new exam announcement *****/
   switch (Gbl.Usrs.Me.UsrTypeLog)
     {
      case Usr_TEACHER:
      case Usr_DEG_ADMIN:
      case Usr_SUPERUSER:
         fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\">");
         Act_FormStart (ActEdiExaAnn);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_New_announcement_of_exam[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/new12x12.gif\" width=\"12\" height=\"12\"" \
                            " alt=\"%s\" />" \
                            " %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,
                  Txt_New_announcement_of_exam[Gbl.Prefs.Language],
                  Txt_New_announcement_of_exam[Gbl.Prefs.Language]);
         break;
      default:
         break;
     }

   /***** List the existing exam announcements *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /***** Get the code of the exam announcement (row[0]) *****/
      row = mysql_fetch_row (mysql_res);

      if (sscanf (row[0],"%ld",&ExaCod) != 1)
         Lay_ShowErrorAndExit ("Wrong code of exam announcement.");

      /***** Allocate memory for the exam announcement *****/
      Exa_AllocMemExamAnnouncement ();

      /***** Read the data of the exam announcement *****/
      Exa_GetDataExamAnnouncementFromDB (ExaCod);

      /***** Show exam announcement *****/
      Exa_ShowExamAnnouncement (ExaCod,TypeViewExamAnnouncement);
      fprintf (Gbl.F.Out,"<br />");

      /***** Free memory of the exam announcement *****/
      Exa_FreeMemExamAnnouncement ();
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/****************** Add an exam announcement to the database ****************/
/****************************************************************************/
// Return the code of the exam announcement just added

static long Exa_AddExamAnnouncementToDB (void)
  {
   char *Query;
   long ExaCod;

   /***** Add exam announcement *****/
   if ((Query = malloc (512+Cte_MAX_BYTES_STRING+7*Cte_MAX_BYTES_TEXT)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to query database.");
   sprintf (Query,"INSERT INTO exam_announcements (CrsCod,Status,NumNotif," \
                  "CrsFullName,Year,ExamSession,CallDate,ExamDate,Duration,Place," \
                  "ExamMode,Structure,DocRequired,MatRequired,MatAllowed,OtherInfo)" \
                  " VALUES ('%ld','%u','0','%s','%u','%s',NOW(),'%04u-%02u-%02u %02u:%02u:00','%02u:%02u:00','%s','%s','%s','%s','%s','%s','%s')",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Exa_ACTIVE_EXAM_ANNOUNCEMENT,
            Gbl.ExamAnnouncement.CrsFullName,Gbl.ExamAnnouncement.Year,Gbl.ExamAnnouncement.Session,
            Gbl.ExamAnnouncement.ExamDate.Year,Gbl.ExamAnnouncement.ExamDate.Month,Gbl.ExamAnnouncement.ExamDate.Day,
            Gbl.ExamAnnouncement.StartTime.Hour,Gbl.ExamAnnouncement.StartTime.Minute,
	    Gbl.ExamAnnouncement.Duration.Hour,Gbl.ExamAnnouncement.Duration.Minute,
            Gbl.ExamAnnouncement.Place,
	    Gbl.ExamAnnouncement.Mode,Gbl.ExamAnnouncement.Structure,
            Gbl.ExamAnnouncement.DocRequired,
            Gbl.ExamAnnouncement.MatRequired,Gbl.ExamAnnouncement.MatAllowed,
            Gbl.ExamAnnouncement.OtherInfo);
   ExaCod = DB_QueryINSERTandReturnCode (Query,"can not create a new exam announcement");
   free ((void *) Query);

   return ExaCod;
  }

/****************************************************************************/
/*************** Modify an exam announcement in the database ****************/
/****************************************************************************/

static void Exa_ModifyExamAnnouncementInDB (long ExaCod)
  {
   char *Query;

   /***** Modify exam announcement *****/
   if ((Query = malloc (512+Cte_MAX_BYTES_STRING+7*Cte_MAX_BYTES_TEXT)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to query database.");
   sprintf (Query,"UPDATE exam_announcements" \
                  " SET CrsFullName='%s',Year='%u',ExamSession='%s'," \
                  "ExamDate='%04u-%02u-%02u %02u:%02u:00',Duration='%02u:%02u:00'," \
                  "Place='%s',ExamMode='%s',Structure='%s',DocRequired='%s',MatRequired='%s',MatAllowed='%s',OtherInfo='%s'" \
                  " WHERE ExaCod='%ld'",
            Gbl.ExamAnnouncement.CrsFullName,Gbl.ExamAnnouncement.Year,Gbl.ExamAnnouncement.Session,
            Gbl.ExamAnnouncement.ExamDate.Year,Gbl.ExamAnnouncement.ExamDate.Month,Gbl.ExamAnnouncement.ExamDate.Day,
            Gbl.ExamAnnouncement.StartTime.Hour,Gbl.ExamAnnouncement.StartTime.Minute,
	    Gbl.ExamAnnouncement.Duration.Hour,Gbl.ExamAnnouncement.Duration.Minute,
	    Gbl.ExamAnnouncement.Place,
            Gbl.ExamAnnouncement.Mode,Gbl.ExamAnnouncement.Structure,
            Gbl.ExamAnnouncement.DocRequired,
            Gbl.ExamAnnouncement.MatRequired,Gbl.ExamAnnouncement.MatAllowed,
            Gbl.ExamAnnouncement.OtherInfo,
            ExaCod);
   DB_QueryUPDATE (Query,"can not update an exam announcement");
   free ((void *) Query);
  }

/****************************************************************************/
/******* Create a list with the dates of all the exam announcements *********/
/****************************************************************************/

void Exa_CreateExamAnnouncement (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long ExaCod;
   struct Date ExamDate;
   unsigned Hour,Minute,Second;

   /***** Get exam announcements in current course from database *****/
   sprintf (Query,"SELECT ExaCod,ExamDate FROM exam_announcements" \
                  " WHERE CrsCod='%ld' AND Status<>'%u' ORDER BY ExamDate",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Exa_DELETED_EXAM_ANNOUNCEMENT);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get exam announcements in this course");

   /***** The result of the query may be empty *****/
   Gbl.LstExamAnnouncements.Lst = NULL;
   Gbl.LstExamAnnouncements.NumExamAnnounc = 0;
   if (NumRows)
     {
      /***** Allocate memory for the list *****/
      if ((Gbl.LstExamAnnouncements.Lst = (struct Date *) calloc (NumRows,sizeof (struct Date))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store dates of exam announcements.");

      /***** Get the dates of the existing exam announcements *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         /***** Get the code of the exam announcement (row[0]) *****/
         row = mysql_fetch_row (mysql_res);

         if (sscanf (row[0],"%ld",&ExaCod) != 1)
            Lay_ShowErrorAndExit ("Wrong code of exam announcement.");

         /***** Read the date of the exam (row[1]) *****/
         /* Date of exam (row[6]) */
         if (sscanf (row[1],"%04u-%02u-%02u %02u:%02u:%02u",
         	     &ExamDate.Year,&ExamDate.Month,&ExamDate.Day,&Hour,&Minute,&Second) != 6)
            Lay_ShowErrorAndExit ("Wrong date of exam.");

         /***** Add exam announcement the list *****/
         Gbl.LstExamAnnouncements.Lst[Gbl.LstExamAnnouncements.NumExamAnnounc].Year  = ExamDate.Year;
         Gbl.LstExamAnnouncements.Lst[Gbl.LstExamAnnouncements.NumExamAnnounc].Month = ExamDate.Month;
         Gbl.LstExamAnnouncements.Lst[Gbl.LstExamAnnouncements.NumExamAnnounc].Day   = ExamDate.Day;
         Gbl.LstExamAnnouncements.NumExamAnnounc++;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/***************** Free list of dates of exam announcements ******************/
/*****************************************************************************/

void Exa_FreeListExamAnnouncements (void)
  {
   if (Gbl.LstExamAnnouncements.Lst)
     {
      free ((void *) Gbl.LstExamAnnouncements.Lst);
      Gbl.LstExamAnnouncements.Lst = NULL;
      Gbl.LstExamAnnouncements.NumExamAnnounc = 0;
     }
  }

/****************************************************************************/
/******** Read the data of an exam announcement from the database ***********/
/****************************************************************************/

static void Exa_GetDataExamAnnouncementFromDB (long ExaCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned Hour,Minute,Second;

   /***** Get data of an exam announcement from database *****/
   sprintf (Query,"SELECT CrsFullName,Year,ExamSession,CallDate,ExamDate,Duration," \
                  "Place,ExamMode,Structure,DocRequired,MatRequired,MatAllowed,OtherInfo" \
                  " FROM exam_announcements WHERE ExaCod='%ld'",ExaCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of an exam announcement");

   /***** The result of the query must have one row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting data of an exam announcement.");

   /***** Get the data of the exam announcement *****/
   row = mysql_fetch_row (mysql_res);

   /* Name of the course (row[0]) */
   strcpy (Gbl.ExamAnnouncement.CrsFullName,row[0]);

   /* Year (row[1]) */
   if (sscanf (row[1],"%u",&Gbl.ExamAnnouncement.Year) != 1)
      Lay_ShowErrorAndExit ("Wrong year.");

   /* Exam session (row[2]) */
   strcpy (Gbl.ExamAnnouncement.Session,row[2]);

   /* Date of exam announcement (row[3]) */
   if (sscanf (row[3],"%04u-%02u-%02u %02u:%02u:%02u",
               &Gbl.ExamAnnouncement.CallDate.Year,
               &Gbl.ExamAnnouncement.CallDate.Month,
               &Gbl.ExamAnnouncement.CallDate.Day,
               &Hour,&Minute,&Second) != 6)
      Lay_ShowErrorAndExit ("Wrong date of exam announcement.");

   /* Date of exam (row[4]) */
   if (sscanf (row[4],"%04u-%02u-%02u %02u:%02u:%02u",
   	       &Gbl.ExamAnnouncement.ExamDate.Year,&Gbl.ExamAnnouncement.ExamDate.Month,&Gbl.ExamAnnouncement.ExamDate.Day,
               &Gbl.ExamAnnouncement.StartTime.Hour,&Gbl.ExamAnnouncement.StartTime.Minute,&Second) != 6)
      Lay_ShowErrorAndExit ("Wrong date of exam.");

   /* Approximate duration (row[5]) */
   if (sscanf (row[5],"%02u:%02u:%02u",&Gbl.ExamAnnouncement.Duration.Hour,&Gbl.ExamAnnouncement.Duration.Minute,&Second) != 3)
      Lay_ShowErrorAndExit ("Wrong duration of exam.");

   /* Place (row[6]) */
   strncpy (Gbl.ExamAnnouncement.Place,row[6],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.Place[Cte_MAX_BYTES_TEXT] = '\0';

   /* Exam mode (row[7]) */
   strncpy (Gbl.ExamAnnouncement.Mode,row[7],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.Mode[Cte_MAX_BYTES_TEXT] = '\0';

   /* Structure (row[8]) */
   strncpy (Gbl.ExamAnnouncement.Structure,row[8],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.Structure[Cte_MAX_BYTES_TEXT] = '\0';

   /* Documentation required (row[9]) */
   strncpy (Gbl.ExamAnnouncement.DocRequired,row[9],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.DocRequired[Cte_MAX_BYTES_TEXT] = '\0';

   /* Material required (row[10]) */
   strncpy (Gbl.ExamAnnouncement.MatRequired,row[10],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.MatRequired[Cte_MAX_BYTES_TEXT] = '\0';

   /* Material allowed (row[11]) */
   strncpy (Gbl.ExamAnnouncement.MatAllowed,row[11],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.MatAllowed[Cte_MAX_BYTES_TEXT] = '\0';

   /* Other information for students (row[12]) */
   strncpy (Gbl.ExamAnnouncement.OtherInfo,row[12],Cte_MAX_BYTES_TEXT);
   Gbl.ExamAnnouncement.OtherInfo[Cte_MAX_BYTES_TEXT] = '\0';

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/************ Show a form with the data of an exam announcement *************/
/****************************************************************************/

static void Exa_ShowExamAnnouncement (long ExaCod,Exa_tTypeViewExamAnnouncement_t TypeViewExamAnnouncement)
  {
   extern const char *Deg_StrsDegYear[1+Deg_MAX_YEARS_PER_DEGREE];
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_print_view[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Year[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Session[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Exam_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Start_time[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Approximate_duration[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Place_of_exam[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Mode[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Structure_of_the_exam[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Documentation_required[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Material_required[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Material_allowed[Txt_NUM_LANGUAGES];
   extern const char *Txt_EXAM_ANNOUNCEMENT_Other_information[Txt_NUM_LANGUAGES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];
   extern const char *Txt_Send_announcement_of_exam[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_announcement_of_exam[Txt_NUM_LANGUAGES];
   const char *StyleTitle  = "CONV_TIT";
   const char *StyleForm   = "CONV_NEG";
   const char *StyleNormal = "CONV";
   struct Institution Ins;
   char StrExamDate[Cte_MAX_LENGTH_DATE+1];
   unsigned Year,Hour,Minute;

   /***** Get data of institution of this degree *****/
   Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);

   switch (TypeViewExamAnnouncement)
     {
      case Exa_NORMAL_VIEW:
         /***** Link to print view *****/
         fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">" \
                            "<tr><td align=\"center\" valign=\"middle\">");
         Act_FormStart (ActPrnExaAnn);
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ExaCod\" value=\"%ld\" />",ExaCod);
         strcpy (Gbl.MsgWindowStatus,Txt_Show_print_view[Gbl.Prefs.Language]);
         Act_LinkFormSubmit (Act_NEW_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/print12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                            " %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"500");
	 break;
      case Exa_PRINT_VIEW:
         StyleTitle  = "CONV_TIT_IMPR";
         StyleForm   = "CONV_NEG_IMPR";
         StyleNormal = "CONV_IMPR";
         Lay_StartTableFrame (DARK_GRAY,WHITE,"500");
         break;
      case Exa_FORM_VIEW:
         StyleForm = The_ClassFormul[Gbl.Prefs.Theme];
         /***** Start form *****/
         Act_FormStart (ActRecExaAnn);
         if (ExaCod >= 0)
            fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ExaCod\" value=\"%ld\" />",ExaCod);
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"500");
     }

   /***** Start table *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">" \
                      "<table cellspacing=\"0\" cellpadding=\"20\" width=\"100%%\">" \
                      "<tr><td align=\"center\">" \
                      "<table cellspacing=\"3\" cellpadding=\"0\" width=\"100%%\">");

   /***** Institution logo *****/
   fprintf (Gbl.F.Out,"<tr><td colspan=\"2\" align=\"center\">");
   if (TypeViewExamAnnouncement == Exa_PRINT_VIEW)
      fprintf (Gbl.F.Out,"<span class=\"%s\">",StyleTitle);
   else
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
               Ins.WWW,StyleTitle);
   fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s64x64.gif\" width=\"64\" height=\"64\" alt=\"%s\" /><br />%s",
            Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Ins.Logo,Ins.FullName,Ins.FullName);
   if (TypeViewExamAnnouncement == Exa_PRINT_VIEW)
      fprintf (Gbl.F.Out,"</span>");
   else
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Degree *****/
   fprintf (Gbl.F.Out,"<tr><td colspan=\"2\" align=\"center\" class=\"%s\">",StyleTitle);
   if (TypeViewExamAnnouncement == Exa_NORMAL_VIEW)
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
               Gbl.CurrentDeg.Deg.WWW,StyleTitle);
   fprintf (Gbl.F.Out,"%s",Gbl.CurrentDeg.Deg.FullName);
   if (TypeViewExamAnnouncement == Exa_NORMAL_VIEW)
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Title *****/
   fprintf (Gbl.F.Out,"<tr><td colspan=\"2\" align=\"center\" class=\"%s\">&nbsp;<br /><strong>%s</strong></td></tr>\n",
            StyleNormal,Txt_EXAM_ANNOUNCEMENT[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<tr><td colspan=\"2\" align=\"left\" class=\"%s\">&nbsp;</td></tr>\n",StyleNormal);

   /***** Name of the course *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Course[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
     {
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"CrsName\" size=\"54\" maxlength=\"%u\" value=\"%s\" />",
               Cte_MAX_LENGTH_STRING,Gbl.ExamAnnouncement.CrsFullName);
     }
   else
      fprintf (Gbl.F.Out,"<strong>%s</strong>",Gbl.ExamAnnouncement.CrsFullName);
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Year (optativa, 1, 2, 3, 4, 5) *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Year[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
     {
      fprintf (Gbl.F.Out,"<select name=\"Year\">");
      for (Year=0; Year<=Deg_MAX_YEARS_PER_DEGREE; Year++)
        {
	 fprintf (Gbl.F.Out,"<option");
	 if (Gbl.ExamAnnouncement.Year == Year)
            fprintf (Gbl.F.Out," selected=\"selected\"");
	 fprintf (Gbl.F.Out," value=\"%u\">%s</option>",Year,Deg_StrsDegYear[Year]);
	}
      fprintf (Gbl.F.Out,"</select>");
     }
   else
      fprintf (Gbl.F.Out,"%s",Deg_StrsDegYear[Gbl.ExamAnnouncement.Year]);
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Exam session *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Session[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ExamSession\" size=\"54\" maxlength=\"%u\" value=\"%s\" />",
               Cte_MAX_LENGTH_STRING,Gbl.ExamAnnouncement.Session);
   else
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.Session);
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Date of the exam *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Exam_date[Gbl.Prefs.Language]);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
     {
      fprintf (Gbl.F.Out,"<td align=\"left\">");
      Dat_WriteFormDate ((Gbl.ExamAnnouncement.ExamDate.Year < Gbl.AcademicYear) ? Gbl.ExamAnnouncement.ExamDate.Year :
                                                                                   Gbl.AcademicYear,
                         Gbl.AcademicYear+1,
                         "ExamDay","ExamMonth","ExamYear",
                         Gbl.ExamAnnouncement.ExamDate.Day,
                         Gbl.ExamAnnouncement.ExamDate.Month,
                         Gbl.ExamAnnouncement.ExamDate.Year,
                         false,false);
      fprintf (Gbl.F.Out,"</td>");
     }
   else
     {
      Dat_ConvDateToDateStr (&Gbl.ExamAnnouncement.ExamDate,StrExamDate);
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%s</td>",
               StyleNormal,StrExamDate);
     }
   fprintf (Gbl.F.Out,"</tr>\n");

   /***** Start time *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Start_time[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
     {
      fprintf (Gbl.F.Out,"<select name=\"ExamHour\"><option value=\"0\"");
      if (Gbl.ExamAnnouncement.StartTime.Hour == 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">-</option>");
      for (Hour=7; Hour<=22; Hour++)
        {
	 fprintf (Gbl.F.Out,"<option value=\"%u\"",Hour);
	 if (Gbl.ExamAnnouncement.StartTime.Hour == Hour)
            fprintf (Gbl.F.Out," selected=\"selected\"");
	 fprintf (Gbl.F.Out,">%02u h</option>",Hour);
	}
      fprintf (Gbl.F.Out,"</select><select name=\"ExamMinute\">");
      for (Minute=0; Minute<=59; Minute++)
        {
	 fprintf (Gbl.F.Out,"<option value=\"%u\"",Minute);
	 if (Gbl.ExamAnnouncement.StartTime.Minute == Minute)
            fprintf (Gbl.F.Out," selected=\"selected\"");
	 fprintf (Gbl.F.Out,">%02u &#39;</option>",Minute);
	}
      fprintf (Gbl.F.Out,"</select>");
     }
   else if (Gbl.ExamAnnouncement.StartTime.Hour)
      fprintf (Gbl.F.Out,"%2u:%02u h",
               Gbl.ExamAnnouncement.StartTime.Hour,
               Gbl.ExamAnnouncement.StartTime.Minute);
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Approximate duration of the exam *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Approximate_duration[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
     {
      fprintf (Gbl.F.Out,"<select name=\"DurationHour\">");
      for (Hour=0; Hour<=8; Hour++)
        {
	 fprintf (Gbl.F.Out,"<option value=\"%u\"",Hour);
	 if (Gbl.ExamAnnouncement.Duration.Hour == Hour)
            fprintf (Gbl.F.Out," selected=\"selected\"");
	 fprintf (Gbl.F.Out,">%02u h</option>",Hour);
	}
      fprintf (Gbl.F.Out,"</select><select name=\"DurationMinute\">");
      for (Minute=0; Minute<=59; Minute++)
        {
	 fprintf (Gbl.F.Out,"<option value=\"%u\"",Minute);
	 if (Gbl.ExamAnnouncement.Duration.Minute == Minute)
            fprintf (Gbl.F.Out," selected=\"selected\"");
	 fprintf (Gbl.F.Out,">%02u &#39;</option>",Minute);
	}
      fprintf (Gbl.F.Out,"</select>");
     }
   else if (Gbl.ExamAnnouncement.Duration.Hour ||
            Gbl.ExamAnnouncement.Duration.Minute)
     {
      if (Gbl.ExamAnnouncement.Duration.Hour)
        {
         fprintf (Gbl.F.Out,"%u h",Gbl.ExamAnnouncement.Duration.Hour);
         if (Gbl.ExamAnnouncement.Duration.Minute)
            fprintf (Gbl.F.Out," %u min",Gbl.ExamAnnouncement.Duration.Minute);
         else
           {
            fprintf (Gbl.F.Out,"ora");
            if (Gbl.ExamAnnouncement.Duration.Hour > 1)
               fprintf (Gbl.F.Out,"s");
           }
        }
      else if (Gbl.ExamAnnouncement.Duration.Minute)
        {
         fprintf (Gbl.F.Out,"%u minuto",Gbl.ExamAnnouncement.Duration.Minute);
         if (Gbl.ExamAnnouncement.Duration.Minute > 1)
            fprintf (Gbl.F.Out,"s");
        }
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Place where the exam will be made *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Place_of_exam[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"Place\" cols=\"40\" rows=\"4\">%s</textarea>",
               Gbl.ExamAnnouncement.Place);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.Place,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.Place);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Exam mode *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Mode[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"ExamMode\" cols=\"40\" rows=\"2\">%s</textarea>",
               Gbl.ExamAnnouncement.Mode);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.Mode,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.Mode);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Structure of the exam *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Structure_of_the_exam[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"Structure\" cols=\"40\" rows=\"8\">%s</textarea>",
               Gbl.ExamAnnouncement.Structure);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.Structure,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.Structure);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Documentation required *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Documentation_required[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"DocRequired\" cols=\"40\" rows=\"2\">%s</textarea>",
               Gbl.ExamAnnouncement.DocRequired);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.DocRequired,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.DocRequired);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Material required *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Material_required[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"MatRequired\" cols=\"40\" rows=\"4\">%s</textarea>",
               Gbl.ExamAnnouncement.MatRequired);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.MatRequired,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.MatRequired);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Material allowed *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Material_allowed[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"MatAllowed\" cols=\"40\" rows=\"4\">%s</textarea>",
               Gbl.ExamAnnouncement.MatAllowed);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.MatAllowed,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.MatAllowed);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Other information to students *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">",
            StyleForm,Txt_EXAM_ANNOUNCEMENT_Other_information[Gbl.Prefs.Language],
            StyleNormal);
   if (TypeViewExamAnnouncement == Exa_FORM_VIEW)
      fprintf (Gbl.F.Out,"<textarea name=\"OtherInfo\" cols=\"40\" rows=\"5\">%s</textarea>",
               Gbl.ExamAnnouncement.OtherInfo);
   else
     {
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.ExamAnnouncement.OtherInfo,Cte_MAX_BYTES_TEXT,false);	// Convert from HTML to rigorous HTML
      fprintf (Gbl.F.Out,"%s",Gbl.ExamAnnouncement.OtherInfo);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Bottom space used for signatures, or links to edit / remove the exam announcement *****/
   switch (TypeViewExamAnnouncement)
     {
      case Exa_NORMAL_VIEW:
         switch (Gbl.Usrs.Me.UsrTypeLog)
           {
            case Usr_TEACHER:
            case Usr_DEG_ADMIN:
            case Usr_SUPERUSER:
               /* Links to edit / remove the exam announcement */
               fprintf (Gbl.F.Out,"<tr><td colspan=\"2\" align=\"center\"><table width=\"100%%\" cellspacing=\"0\" cellpadding=\"0\"><tr>");

               /***** Create link to edit this exam announcement *****/
               fprintf (Gbl.F.Out,"<td align=\"left\"><table cellspacing=\"0\" cellpadding=\"0\">");
               fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
               Act_FormStart (ActEdiExaAnn);
               fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ExaCod\" value=\"%ld\" />",ExaCod);
               Act_LinkFormSubmit (Act_MAIN_WINDOW,"Editar esta convocatoria",The_ClassFormul[Gbl.Prefs.Theme]);
               fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"Editar esta convocatoria\" />" \
	   	                  " %s</acronym></a>",Gbl.Prefs.IconsURL,Txt_Edit[Gbl.Prefs.Language]);
               fprintf (Gbl.F.Out,"</form></td></tr></table></td>");

               /***** Create link to remove this exam announcement *****/
               fprintf (Gbl.F.Out,"<td align=\"right\"><table cellspacing=\"0\" cellpadding=\"0\">");
               fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
               Act_FormStart (ActRemExaAnn);
               fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ExaCod\" value=\"%ld\" />",ExaCod);
               Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_announcement_of_exam[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
               fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
   		                  " %s</acronym></a>",
   		        Gbl.Prefs.IconsURL,
   		        Txt_Remove_announcement_of_exam[Gbl.Prefs.Language],
   		        Txt_Remove[Gbl.Prefs.Language]);
               fprintf (Gbl.F.Out,"</form></td></tr></table></td>");

               fprintf (Gbl.F.Out,"</tr></table></td></tr>");
               break;
            default:
               break;
           }
         break;
      case Exa_PRINT_VIEW:
         /* Bottom space used for signatures */
         fprintf (Gbl.F.Out,"<tr><td colspan=\"2\" align=\"left\" class=\"%s\">&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;</td></tr>\n",StyleForm);
         break;
      default:
         break;
     }

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table></td></tr></table></td></tr>\n");
   switch (TypeViewExamAnnouncement)
     {
      case Exa_NORMAL_VIEW:
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
	 break;
      case Exa_PRINT_VIEW:
         Lay_EndTableFrame ();
         break;
      case Exa_FORM_VIEW:
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         Lay_PutSendAndUndoButtons (Txt_Send_announcement_of_exam[Gbl.Prefs.Language]);
         fprintf (Gbl.F.Out,"</form>\n");
     }
  }
