// swad_for.c: forums

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <string.h>
#include <mysql/mysql.h>	// To access MySQL databases

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_for.h"
#include "swad_gbl.h"
#include "swad_lay.h"
#include "swad_ntf.h"
#include "swad_par.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Actions Actions[Act_NUM_ACTIONS];

/****************************************************************************/
/*********************** Private constants and types ************************/
/****************************************************************************/

#define For_FORUM_MAX_LEVELS 3

const unsigned PermissionThreadDeletion[For_NUM_TYPES_FORUM] =
  {
   0x38,
   0x38,
   0x30,
   0x30,
   0x30,
   0x30,
   0x20,
   0x20,
   0x20,
   0x20,
   0x20,
   0x20,
  };

const Act_Action_t ActionsSeeFor[For_NUM_TYPES_FORUM] =
  {
   ActSeeForCrsUsr,
   ActSeeForCrsTch,
   ActSeeForDegUsr,
   ActSeeForDegTch,
   ActSeeForCtrUsr,
   ActSeeForCtrTch,
   ActSeeForInsUsr,
   ActSeeForInsTch,
   ActSeeForGenUsr,
   ActSeeForGenTch,
   ActSeeForSWAUsr,
   ActSeeForSWATch,
  };
const Act_Action_t ActionsSeePstFor[For_NUM_TYPES_FORUM] =
  {
   ActSeePstForCrsUsr,
   ActSeePstForCrsTch,
   ActSeePstForDegUsr,
   ActSeePstForDegTch,
   ActSeePstForCtrUsr,
   ActSeePstForCtrTch,
   ActSeePstForInsUsr,
   ActSeePstForInsTch,
   ActSeePstForGenUsr,
   ActSeePstForGenTch,
   ActSeePstForSWAUsr,
   ActSeePstForSWATch,
  };
const Act_Action_t ActionsRecThrFor[For_NUM_TYPES_FORUM] =
  {
   ActRecThrForCrsUsr,
   ActRecThrForCrsTch,
   ActRecThrForDegUsr,
   ActRecThrForDegTch,
   ActRecThrForCtrUsr,
   ActRecThrForCtrTch,
   ActRecThrForInsUsr,
   ActRecThrForInsTch,
   ActRecThrForGenUsr,
   ActRecThrForGenTch,
   ActRecThrForSWAUsr,
   ActRecThrForSWATch,
  };
const Act_Action_t ActionsRecRepFor[For_NUM_TYPES_FORUM] =
  {
   ActRecRepForCrsUsr,
   ActRecRepForCrsTch,
   ActRecRepForDegUsr,
   ActRecRepForDegTch,
   ActRecRepForCtrUsr,
   ActRecRepForCtrTch,
   ActRecRepForInsUsr,
   ActRecRepForInsTch,
   ActRecRepForGenUsr,
   ActRecRepForGenTch,
   ActRecRepForSWAUsr,
   ActRecRepForSWATch,
  };
const Act_Action_t ActionsReqDelThr[For_NUM_TYPES_FORUM] =
  {
   ActReqDelThrCrsUsr,
   ActReqDelThrCrsTch,
   ActReqDelThrDegUsr,
   ActReqDelThrDegTch,
   ActReqDelThrCtrUsr,
   ActReqDelThrCtrTch,
   ActReqDelThrInsUsr,
   ActReqDelThrInsTch,
   ActReqDelThrGenUsr,
   ActReqDelThrGenTch,
   ActReqDelThrSWAUsr,
   ActReqDelThrSWATch,
  };
const Act_Action_t ActionsDelThrFor[For_NUM_TYPES_FORUM] =
  {
   ActDelThrForCrsUsr,
   ActDelThrForCrsTch,
   ActDelThrForDegUsr,
   ActDelThrForDegTch,
   ActDelThrForCtrUsr,
   ActDelThrForCtrTch,
   ActDelThrForInsUsr,
   ActDelThrForInsTch,
   ActDelThrForGenUsr,
   ActDelThrForGenTch,
   ActDelThrForSWAUsr,
   ActDelThrForSWATch,
  };
const Act_Action_t ActionsCutThrFor[For_NUM_TYPES_FORUM] =
  {
   ActCutThrForCrsUsr,
   ActCutThrForCrsTch,
   ActCutThrForDegUsr,
   ActCutThrForDegTch,
   ActCutThrForCtrUsr,
   ActCutThrForCtrTch,
   ActCutThrForInsUsr,
   ActCutThrForInsTch,
   ActCutThrForGenUsr,
   ActCutThrForGenTch,
   ActCutThrForSWAUsr,
   ActCutThrForSWATch,
  };
const Act_Action_t ActionsPasThrFor[For_NUM_TYPES_FORUM] =
  {
   ActPasThrForCrsUsr,
   ActPasThrForCrsTch,
   ActPasThrForDegUsr,
   ActPasThrForDegTch,
   ActPasThrForCtrUsr,
   ActPasThrForCtrTch,
   ActPasThrForInsUsr,
   ActPasThrForInsTch,
   ActPasThrForGenUsr,
   ActPasThrForGenTch,
   ActPasThrForSWAUsr,
   ActPasThrForSWATch,
  };
const Act_Action_t ActionsDelPstFor[For_NUM_TYPES_FORUM] =
  {
   ActDelPstForCrsUsr,
   ActDelPstForCrsTch,
   ActDelPstForDegUsr,
   ActDelPstForDegTch,
   ActDelPstForCtrUsr,
   ActDelPstForCtrTch,
   ActDelPstForInsUsr,
   ActDelPstForInsTch,
   ActDelPstForGenUsr,
   ActDelPstForGenTch,
   ActDelPstForSWAUsr,
   ActDelPstForSWATch,
  };
const Act_Action_t ActionsEnbPstFor[For_NUM_TYPES_FORUM] =
  {
   ActEnbPstForCrsUsr,
   ActEnbPstForCrsTch,
   ActEnbPstForDegUsr,
   ActEnbPstForDegTch,
   ActEnbPstForCtrUsr,
   ActEnbPstForCtrTch,
   ActEnbPstForInsUsr,
   ActEnbPstForInsTch,
   ActEnbPstForGenUsr,
   ActEnbPstForGenTch,
   ActEnbPstForSWAUsr,
   ActEnbPstForSWATch,
  };
const Act_Action_t ActionsDisPstFor[For_NUM_TYPES_FORUM] =
  {
   ActDisPstForCrsUsr,
   ActDisPstForCrsTch,
   ActDisPstForDegUsr,
   ActDisPstForDegTch,
   ActDisPstForCtrUsr,
   ActDisPstForCtrTch,
   ActDisPstForInsUsr,
   ActDisPstForInsTch,
   ActDisPstForGenUsr,
   ActDisPstForGenTch,
   ActDisPstForSWAUsr,
   ActDisPstForSWATch,
  };

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void For_ShowThreadPosts (long ThrCod,char *LastSubject);
static void For_PutParamWhichForum (void);
static void For_PutParamForumOrder (void);
static void For_PutFormWhichForums (void);
static void For_PutParamsForumIndDegCrs (void);
static void For_WriteLinksToGblForums (bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static void For_WriteLinksToPlatformForums (bool IsLastForum,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static long For_WriteLinksToInsForums (long InsCod,bool IsLastIns,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static long For_WriteLinksToDegForums (long DegCod,bool IsLastDeg,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static long For_WriteLinksToCrsForums (long CrsCod,bool IsLastCrs,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static void For_WriteLinkToAForum (For_ForumType_t ForumType,bool ShowNumOfPosts,
                                   unsigned Level,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static void For_WriteLinkToForum (For_ForumType_t ForumType,Act_Action_t NextAct,const char *Icon,const char *ForumName,bool ShowNumOfPosts,
                                  unsigned Level,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS]);
static unsigned For_GetNumOfThreadsInForumNewerThan (For_ForumType_t ForumType,const char *Time);
static unsigned For_GetNumOfUnreadPostsInThr (long ThrCod,unsigned NumPostsInThr);
static unsigned For_GetNumOfPostsInThrNewerThan (long ThrCod,const char *Time);
static void For_UpdateNumUsrsNotifiedAboutPost (long PstCod,unsigned NumUsrsToBeNotified);
static void For_WriteNumberOfThrs (unsigned NumThrs,unsigned NumThrsWithNewPosts);
static long For_GetParamThrCod (void);
static void For_PutHiddenParamPstCod (long PstCod);
static long For_GetParamPstCod (void);
static void For_ShowAForumPost (struct ForumThread *Thr,unsigned long PstNum,long PstCod,bool LastPst,char *LastSubject,bool NewPst,bool ICanModerateForum);
static void For_GetPstData (long PstCod,long *UsrCod,char *CreatTime,char *Subject, char *Content);

/*****************************************************************************/
/****************************** Enable a forum post **************************/
/*****************************************************************************/

void For_EnbPst (void)
  {
   long PstCod,ThrCod;

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get the post code to enable *****/
   PstCod = For_GetParamPstCod ();

   /***** Get the thread number *****/
   ThrCod = For_GetParamThrCod ();

   /***** Delete post from table of disabled posts *****/
   For_DeletePstFromDisabledPstTable (PstCod);
   Lay_ShowAlert (INFO,"Mensaje permitido.");	// Need translation!

   /***** Show the posts again *****/
   For_ShowForumLevel2 (ThrCod);
  }

/*****************************************************************************/
/***************************** Disable a forum post **************************/
/*****************************************************************************/

void For_DisPst (void)
  {
   long PstCod,ThrCod;

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get the post code to enable *****/
   PstCod = For_GetParamPstCod ();

   /***** Get the thread number *****/
   ThrCod = For_GetParamThrCod ();

   /***** Check if post really exists, if it hasn't been removed *****/
   if (For_GetIfForumPstExists (PstCod))
     {
      /***** Insert post into table of disabled posts *****/
      For_InsertPstIntoDisabledPstTable (PstCod);
      Lay_ShowAlert (INFO,"Mensaje censurado.");	// Need translation!
     }
   else
      Lay_ShowErrorAndExit ("The post to be disabled no longer exists.");

   /***** Show the posts again *****/
   For_ShowForumLevel2 (ThrCod);
  }

/*****************************************************************************/
/******************** Get if a forum post exists in database *****************/
/*****************************************************************************/

bool For_GetIfForumPstExists (long PstCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a forum post exists from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM forum_post WHERE PstCod='%ld'",PstCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a post of a forum already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a post of a forum already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);	// Post exists if it appears in table of forum posts
  }

/*****************************************************************************/
/*********************** Get if a forum post is enabled **********************/
/*****************************************************************************/

bool For_GetIfPstIsEnabled (long PstCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if post is disabled from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM forum_disabled_post" \
                  " WHERE PstCod='%ld'",PstCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a post of a forum is disabled");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a post of a forum is disabled.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows == 0);	// Post is enabled if it doesn't appear in table of disabled posts
  }

/*****************************************************************************/
/****************** Delete post from table of disabled posts *****************/
/*****************************************************************************/

void For_DeletePstFromDisabledPstTable (long PstCod)
  {
   char Query[512];

   /***** Remove post from disabled posts table *****/
   sprintf (Query,"DELETE FROM forum_disabled_post" \
                  " WHERE PstCod='%ld'",PstCod);
   DB_QueryDELETE (Query,"can not enable a post of a forum");
  }

/*****************************************************************************/
/****************** Insert post into table of disabled posts *****************/
/*****************************************************************************/

void For_InsertPstIntoDisabledPstTable (long PstCod)
  {
   char Query[512];

   /***** Remove post from disabled posts table *****/
   sprintf (Query,"REPLACE INTO forum_disabled_post (PstCod,UsrCod,DisableTime)" \
                  " VALUES ('%ld','%ld',NOW())",
            PstCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryREPLACE (Query,"can not disable a post of a forum");
  }

/*****************************************************************************/
/************** Insert a post new in the table of posts of forums ************/
/*****************************************************************************/

long For_InsertForumPst (long ThrCod,long UsrCod,const char *Subject,const char *Content)
  {
   char Query[512+Cte_MAX_BYTES_SUBJECT+Cte_MAX_BYTES_TEXT];

   /***** Insert forum post in the database *****/
   sprintf (Query,"INSERT INTO forum_post (ThrCod,UsrCod,CreatTime,ModifTime,NumNotif,Subject,Content)" \
                  " VALUES ('%ld','%ld',NOW(),NOW(),'0','",
            ThrCod,UsrCod);
   Str_AddStrToQuery (Query,Subject,sizeof (Query));
   strcat (Query,"','");
   Str_AddStrToQuery (Query,Content,sizeof (Query));
   strcat (Query,"')");
   return DB_QueryINSERTandReturnCode (Query,"can not create a new post in a forum");
  }

/*****************************************************************************/
/***************** Delete a post from the forum post table *******************/
/*****************************************************************************/
// Return true if the post thread is deleted

bool For_RemoveForumPst (long PstCod)
  {
   char Query[512];
   long ThrCod;
   bool ThreadDeleted = false;

   /***** If the post is the only one in its thread, delete that thread *****/
   if (For_NumPstsInThrWithPstCod (PstCod,&ThrCod) < 2)
     {
      For_RemoveThreadOnly (ThrCod);
      ThreadDeleted = true;
     }

   /***** Delete post from forum post table *****/
   sprintf (Query,"DELETE FROM forum_post WHERE PstCod='%ld'",PstCod);
   DB_QueryDELETE (Query,"can not remove a post from a forum");

   /***** Delete the post from the table of disabled forum posts *****/
   For_DeletePstFromDisabledPstTable (PstCod);

   /***** Update the last post of the thread *****/
   if (!ThreadDeleted)
      For_UpdateThrLastPst (ThrCod,For_GetLastPstCod (ThrCod));

   return ThreadDeleted;
  }

/*****************************************************************************/
/*********** Get the number of posts in the thread than holds a post *********/
/*****************************************************************************/

unsigned For_NumPstsInThrWithPstCod (long PstCod,long *ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPsts;

   /***** Get number of posts in the thread that holds a post from database *****/
   sprintf (Query,"SELECT COUNT(PstCod),ThrCod FROM forum_post"
                  " WHERE ThrCod IN" \
                  " (SELECT ThrCod FROM forum_post" \
                  " WHERE PstCod='%ld') GROUP BY ThrCod;",PstCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of posts in a thread of a forum");

   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumPsts) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of posts in a thread of a forum.");
   if (sscanf (row[1],"%ld",ThrCod) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of posts in a thread of a forum.");
   DB_FreeMySQLResult (&mysql_res);

   return NumPsts;
  }

/*****************************************************************************/
/*************** Insert a new thread in table of forum threads ***************/
/*****************************************************************************/
// Returns the code of the new inserted thread

long For_InsertForumThread (For_ForumType_t ForumType,long FirstPstCod)
  {
   char Query[512];

   /***** Insert new thread in the database *****/
   switch (ForumType)
     {
      case For_FORUM_SWAD_USRS:		case For_FORUM_SWAD_TCHS:
      case For_FORUM_GLOBAL_USRS:		case For_FORUM_GLOBAL_TCHS:
      case For_FORUM_CENTRE_USRS:		case For_FORUM_CENTRE_TCHS:
         sprintf (Query,"INSERT INTO forum_thread (ForumType,Location,FirstPstCod,LastPstCod)" \
                        " VALUES ('%u','-1','%ld','%ld')",
                  (unsigned) ForumType,FirstPstCod,FirstPstCod);
         break;
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (Query,"INSERT INTO forum_thread (ForumType,Location,FirstPstCod,LastPstCod)" \
                        " VALUES ('%u','%ld','%ld','%ld')",
                  (unsigned) ForumType,Gbl.Forum.Ins.InsCod,FirstPstCod,FirstPstCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (Query,"INSERT INTO forum_thread (ForumType,Location,FirstPstCod,LastPstCod)" \
                        " VALUES ('%u','%ld','%ld','%ld')",
                  (unsigned) ForumType,Gbl.Forum.Deg.DegCod,FirstPstCod,FirstPstCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (Query,"INSERT INTO forum_thread (ForumType,Location,FirstPstCod,LastPstCod)" \
                        " VALUES ('%u','%ld','%ld','%ld')",
                  (unsigned) ForumType,Gbl.Forum.Crs.CrsCod,FirstPstCod,FirstPstCod);
         break;
     }
   return DB_QueryINSERTandReturnCode (Query,"can not create a new thread in a forum");
  }

/*****************************************************************************/
/*************** Delete a thread from the forum thread table *****************/
/*****************************************************************************/

void For_RemoveThreadOnly (long ThrCod)
  {
   char Query[512];

   /***** Indicate that this thread has not been read by anyone *****/
   For_DeleteThrFromReadThrs (ThrCod);

   /***** Remove thread code from thread clipboard *****/
   For_RemoveThrCodFromThrClipboard (ThrCod);

   /***** Delete thread from forum thread table *****/
   sprintf (Query,"DELETE FROM forum_thread WHERE ThrCod='%ld'",ThrCod);
   DB_QueryDELETE (Query,"can not remove a thread from a forum");
  }

/*****************************************************************************/
/*************** Delete a thread from the forum thread table *****************/
/*****************************************************************************/

void For_RemoveThreadAndItsPsts (long ThrCod)
  {
   char Query[512];

   /***** Delete disabled posts in thread *****/
   sprintf (Query,"DELETE forum_disabled_post FROM forum_post,forum_disabled_post" \
                  " WHERE forum_post.ThrCod='%ld' AND forum_post.PstCod=forum_disabled_post.PstCod",
            ThrCod);
   DB_QueryDELETE (Query,"can not enable then posts of a thread of a forum");

   /***** Delete thread posts *****/
   sprintf (Query,"DELETE FROM forum_post WHERE ThrCod='%ld'",ThrCod);
   DB_QueryDELETE (Query,"can not remove the posts of a thread of a forum");

   /***** Delete thread from forum thread table *****/
   For_RemoveThreadOnly (ThrCod);
  }

/*****************************************************************************/
/********************* Get the thread subject from a thread ******************/
/*****************************************************************************/

void For_GetThrSubject (long ThrCod,char *Subject,size_t MaxSize)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;

   /***** Get subject of a thread from database *****/
   sprintf (Query,"SELECT forum_post.Subject FROM forum_thread,forum_post" \
                  " WHERE forum_thread.ThrCod='%ld' AND forum_thread.FirstPstCod=forum_post.PstCod",ThrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the subject of a thread of a forum");

   /***** Write the subject of the thread *****/
   row = mysql_fetch_row (mysql_res);
   strncpy (Subject,row[0],MaxSize);
   Subject[MaxSize] = '\0';
   Str_LimitLengthHTMLStr (Subject,20);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********* Modify the codes of the first and last posts of a thread **********/
/*****************************************************************************/

void For_UpdateThrFirstAndLastPst (long ThrCod,long FirstPstCod,long LastPstCod)
  {
   char Query[512];

   /***** Update the code of the first and last posts of a thread *****/
   sprintf (Query,"UPDATE forum_thread SET FirstPstCod='%ld',LastPstCod='%ld'" \
                  " WHERE ThrCod='%ld'",
            FirstPstCod,LastPstCod,ThrCod);
   DB_QueryUPDATE (Query,"can not update a thread of a forum");
  }

/*****************************************************************************/
/************** Modify the code of the last post of a thread *****************/
/*****************************************************************************/

void For_UpdateThrLastPst (long ThrCod,long LastPstCod)
  {
   char Query[512];

   /***** Update the code of the last post of a thread *****/
   sprintf (Query,"UPDATE forum_thread SET LastPstCod='%ld' WHERE ThrCod='%ld'",
            LastPstCod,ThrCod);
   DB_QueryUPDATE (Query,"can not update a thread of a forum");
  }

/*****************************************************************************/
/**************** Get the code of the last post of a thread ******************/
/*****************************************************************************/

long For_GetLastPstCod (long ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   long LastPstCod;

   /***** Get the code of the last post of a thread from database *****/
   sprintf (Query,"SELECT PstCod FROM forum_post" \
                  " WHERE ThrCod='%ld' ORDER BY CreatTime DESC LIMIT 1",
            ThrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the most recent post of a thread of a forum");

   /***** Write the subject of the thread *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%ld",&LastPstCod) != 1)
      Lay_ShowErrorAndExit ("Error when getting the most recent post of a thread of a forum.");

   return LastPstCod;
  }

/*****************************************************************************/
/************* Update read date of a thread for the current user *************/
/*****************************************************************************/
// Update forum_thr_read table indicating that this thread page and previous ones
// have been read and have no new posts for the current user
// (even if any previous pages have been no read actually)

void For_UpdateThrReadTime (long ThrCod,const char *ReadTime)
  {
   char Query[512];

   /***** Insert or replace pair ThrCod-UsrCod in forum_thr_read *****/
   sprintf (Query,"REPLACE INTO forum_thr_read (ThrCod,UsrCod,ReadTime)" \
                  " VALUES ('%ld','%ld','%s')",
            ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod,ReadTime);
   DB_QueryREPLACE (Query,"can not update the status of reading of a thread of a forum");
  }

/*****************************************************************************/
/**************** Get number of users that have read a thread ****************/
/*****************************************************************************/

unsigned For_GetNumOfReadersOfThr (long ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumReaders;

   /***** Get number of distinct readers of a thread from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM forum_thr_read WHERE ThrCod='%ld'",ThrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of readers of a thread of a forum");

   /* Get row with number of readers */
   row = mysql_fetch_row (mysql_res);

   /* Get number of readers (row[0]) */
   if (sscanf (row[0],"%u",&NumReaders) != 1)
      Lay_ShowErrorAndExit ("Internal error while getting the number of readers of a thread of a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumReaders;
  }

/*****************************************************************************/
/********** Get number of users that have write posts in a thread ************/
/*****************************************************************************/

unsigned For_GetNumOfWritersInThr (long ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumWriters;

   /***** Get number of distinct writers in a thread from database *****/
   sprintf (Query,"SELECT COUNT(DISTINCT UsrCod) FROM forum_post" \
                  " WHERE ThrCod='%ld'",ThrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of writers in a thread of a forum");

   /* Get row with number of writers */
   row = mysql_fetch_row (mysql_res);

   /* Get number of writers (row[0]) */
   if (sscanf (row[0],"%u",&NumWriters) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of writers in a thread of a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumWriters;
  }

/*****************************************************************************/
/********************** Get number of posts in a thread **********************/
/*****************************************************************************/

unsigned For_GetNumPstsInThr (long ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPosts;

   /***** Get number of posts in a thread from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM forum_post WHERE ThrCod='%ld'",ThrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of posts in a thread of a forum");

   /* Get row with number of posts */
   row = mysql_fetch_row (mysql_res);

   /* Get number of posts (row[0]) */
   if (sscanf (row[0],"%u",&NumPosts) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of posts in a thread of a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumPosts;
  }

/*****************************************************************************/
/************** Get whether there are posts of mine in a thread **************/
/*****************************************************************************/

unsigned For_GetNumMyPstInThr (long ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPosts;

   /***** Get if I have write posts in a thread from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM forum_post" \
                  " WHERE forum_post.ThrCod='%ld' AND forum_post.UsrCod='%ld'",
            ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if you have written post in a thead of a forum");

   /***** Get number of posts of mine *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumPosts) != 1)
      Lay_ShowErrorAndExit ("Error when checking if you have written post in a thead of a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumPosts;
  }

/*****************************************************************************/
/****************** Get thread read time for the current user ****************/
/*****************************************************************************/
// ReadTime is return in YYYYMMDDHHMMSS format

void For_GetThrReadTime (long ThrCod,char *ReadTime)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get read time of a thread from database *****/
   sprintf (Query,"SELECT DATE_FORMAT(ReadTime,'%%Y%%m%%d%%H%%i%%S')" \
                  " FROM forum_thr_read WHERE ThrCod='%ld' AND UsrCod='%ld'",
            ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get date of reading of a thread of a forum");

   /***** Check if there is a row with read time *****/
   if (NumRows)
     {
      /***** There is a row ==> get read time inf format YYYYMMDDHHMMSS *****/
      row = mysql_fetch_row (mysql_res);
      strncpy (ReadTime,row[0],4+2+2+2+2+2);
      ReadTime[4+2+2+2+2+2] = '\0';
     }
   else
      strcpy (ReadTime,"00000000000000");	// If there is no row for this thread and current user, then current user has not read this thread

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********************* Delete thread read status for a thread ****************/
/*****************************************************************************/

void For_DeleteThrFromReadThrs (long ThrCod)
  {
   char Query[512];

   /***** Delete pairs ThrCod-UsrCod in forum_thr_read for a thread *****/
   sprintf (Query,"DELETE FROM forum_thr_read WHERE ThrCod='%ld'",ThrCod);
   DB_QueryDELETE (Query,"can not remove the status of reading of a thread of a forum");
  }

/*****************************************************************************/
/********************** Delete thread read status for an user ****************/
/*****************************************************************************/

void For_DeleteUsrFromReadThrs (long UsrCod)
  {
   char Query[512];

   /***** Delete pairs ThrCod-UsrCod in forum_thr_read for a user *****/
   sprintf (Query,"DELETE FROM forum_thr_read WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove the status of reading by a user of all the threads of a forum");
  }

/*****************************************************************************/
/****************************** Show forum posts *****************************/
/*****************************************************************************/

static void For_ShowThreadPosts (long ThrCod,char *LastSubject)
  {
   bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS];
   struct ForumThread Thr;
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   unsigned long NumPst = 0;		// Initialized to avoid warning
   unsigned NumPsts;
   char ReadTime[4+2+2+2+2+2+1];	// Read time of thread for the current user in YYYYMMDDHHMMSS format
   struct Pagination Pagination;
   long PstCod;
   bool NewPst = false;
   bool ICanModerateForum = false;
   bool ICanMoveThreads = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER);	// If I have permission to move threads...

   /***** Get data of the thread *****/
   Thr.ThrCod = ThrCod;
   For_GetThrData (&Thr);

   /***** Get the page number *****/
   Pag_GetParamPagNum (Pag_POSTS_FORUM);

   /***** Write title *****/
   /* Get if there is a thread ready to be moved */
   if (ICanMoveThreads)
      Gbl.Forum.ThreadToMove = For_GetThrInMyClipboard ();

   /* Get thread read time for the current user */
   For_GetThrReadTime (ThrCod,ReadTime);

   /* Table start */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /* Put a form to select which forums */
   For_PutFormWhichForums ();

   /* Write a link to top level of forums */
   For_WriteLinkToTopLevelOfForums ();

   /* Write a link to current forum */
   IsLastItemInLevel[1] = true;
   For_WriteLinkToAForum (Gbl.Forum.ForumType,true,1,IsLastItemInLevel);

   /* Write thread title */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   IsLastItemInLevel[2] = true;
   Msg_IndentDependingOnLevel (2,IsLastItemInLevel);

   fprintf (Gbl.F.Out,"<td align=\"left\" class=\"TITULO\"><img src=\"%s/thread%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" /> ",
            Gbl.Prefs.IconsURL,Thr.NumUnreadPosts ? "new" : "");
   For_WriteThrSubject (ThrCod);

   /***** Get posts of a thread from database *****/
   sprintf (Query,"SELECT PstCod,DATE_FORMAT(CreatTime,'%%Y%%m%%d%%H%%i%%S') FROM forum_post" \
                  " WHERE ThrCod='%ld' ORDER BY PstCod",ThrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get posts of a thread");

   NumPsts = (unsigned) NumRows;

   /* Write number of posts and number of new posts */
   fprintf (Gbl.F.Out," ");
   Msg_WriteNumMsgs (NumPsts,0,0);

   fprintf (Gbl.F.Out,"</td></tr></table></td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   LastSubject[0] = '\0';
   if (NumPsts)		// If there are posts...
     {
      /***** Check if I can moderate posts in forum *****/
      switch (Gbl.Forum.ForumType)
        {
         case For_FORUM_SWAD_USRS:		case For_FORUM_SWAD_TCHS:
         case For_FORUM_GLOBAL_USRS:	case For_FORUM_GLOBAL_TCHS:
         case For_FORUM_CENTRE_USRS:	case For_FORUM_CENTRE_TCHS:
            ICanModerateForum = Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER;
            break;
         case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
            ICanModerateForum = Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER;
            break;
         case For_FORUM_DEGREE_USRS:	case For_FORUM_DEGREE_TCHS:
         				case For_FORUM_COURSE_TCHS:
            ICanModerateForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_DEG_ADMIN ||
                                 Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER);
            break;
         case For_FORUM_COURSE_USRS:
            ICanModerateForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_TEACHER ||
                                 Gbl.Usrs.Me.UsrTypeForumsChat == Usr_DEG_ADMIN ||
                                 Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER);
            break;
        }

      /***** Compute variables related to pagination *****/
      Pagination.NumItems = NumPsts;
      Pagination.CurrentPage = (int) Gbl.Pag.CurrentPage;
      Pag_CalculatePagination (&Pagination);
      Gbl.Pag.CurrentPage = (unsigned) Pagination.CurrentPage;

      /***** Write links to pages *****/
      if (Pagination.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered (Pag_POSTS_FORUM,ThrCod,&Pagination);

      /***** Show posts from this page, the author and the date of last reply *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" width=\"100%%\"><table cellspacing=\"0\" cellpadding=\"2\" width=\"100%%\">");

      for (NumRow=1; NumRow<=NumRows; NumRow++)
        {
         row = mysql_fetch_row (mysql_res);
         if (NumRow >= Pagination.FirstItemVisible &&
             NumRow <= Pagination.LastItemVisible)
           {
            if (sscanf (row[0],"%ld",&PstCod) != 1)
               Lay_ShowErrorAndExit ("Wrong code of post.");

            NumPst = NumRow;
            NewPst = (strcmp (row[1],ReadTime) > 0);

            if (NewPst && NumRow == Pagination.LastItemVisible)
               /* Update forum_thr_read table indicating that this thread page and previous ones
                  have been read and have no new posts for the current user
                  (even if any previous pages have been no read actually) */
               For_UpdateThrReadTime (ThrCod,row[1]);

            /* Show post */
            For_ShowAForumPost (&Thr,NumPst,PstCod,(NumRow == NumRows),LastSubject,NewPst,ICanModerateForum);
           }
        }

      fprintf (Gbl.F.Out,"</table></td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Write again links to pages *****/
      if (Pagination.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered (Pag_POSTS_FORUM,ThrCod,&Pagination);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/**************************** Show a post from forum *************************/
/*****************************************************************************/

static void For_ShowAForumPost (struct ForumThread *Thr,unsigned long PstNum,long PstCod,bool LastPst,char *LastSubject,bool NewPst,bool ICanModerateForum)
  {
   extern const char *Txt_Remove_post[Txt_NUM_LANGUAGES];
   extern const char *Txt_no_subject[Txt_NUM_LANGUAGES];
   struct UsrData UsrDat;
   char CreatTime[4+2+2+2+2+2+1];	// Creation time of a post in YYYYMMDDHHMMSS format
   char OriginalContent[Cte_MAX_BYTES_TEXT+1];
   char Subject[Cte_MAX_BYTES_SUBJECT+1];
   char Content[Cte_MAX_BYTES_TEXT+1];
   bool Enabled;

   /***** Initialize structure with user's data *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Check if post is enabled *****/
   Enabled = For_GetIfPstIsEnabled (PstCod);

   /***** Get data of post *****/
   For_GetPstData (PstCod,&UsrDat.UsrCod,CreatTime,Subject,OriginalContent);
   if (Enabled)
     {
      /* Return this subject as last subject */
      strncpy (LastSubject,Subject,Cte_MAX_BYTES_SUBJECT);
      LastSubject[Cte_MAX_BYTES_SUBJECT] = '\0';
     }

   /***** Put an icon with post status *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\" width=\"18\" bgcolor=\"#%s\">" \
                      "<img src=\"%s/mail_16x16.gif\" alt=\"\" width=\"16\" height=\"16\" /></td>",
            NewPst ? LIGHTEST_GREEN : LIGHTEST_BLUE,Gbl.Prefs.IconsURL);

   /***** Write post number *****/
   Msg_WriteMsgNumber (PstNum,NewPst);

   /***** Write date *****/
   Msg_WriteMsgDate (CreatTime,NewPst);

   /***** Write subject *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">",
            NewPst ? LIGHTEST_GREEN : LIGHTEST_BLUE,NewPst ? "MSG_TIT_NEW" : "MSG_TIT");
   if (Enabled)
     {
      if (Subject[0])
         fprintf (Gbl.F.Out,"%s",Subject);
      else
         fprintf (Gbl.F.Out,"[%s]",Txt_no_subject[Gbl.Prefs.Language]);
     }
   else
      fprintf (Gbl.F.Out,"[mensaje no permitido]");
   fprintf (Gbl.F.Out,"</td></tr>");

    /***** Form to remove post *****/
   fprintf (Gbl.F.Out,"<tr><td width=\"18\" align=\"center\" valign=\"top\">");
   if (LastPst && Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat.UsrCod)
      // Post can be removed if post is the last (without answers) and it's mine
     {
      Act_FormStart (ActionsDelPstFor[Gbl.Forum.ForumType]);
      Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
      For_PutHiddenParamPstCod (PstCod);
      For_PutAllHiddenParamsForum ();
      For_PutHiddenParamThrCod (Thr->ThrCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_post[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\"/></acronym></a></form>",
               Gbl.Prefs.IconsURL,Txt_Remove_post[Gbl.Prefs.Language]);
     }

   /***** Form to disable/enable post *****/
   else
     {
      if (ICanModerateForum)
        {
         Act_FormStart (Enabled ? ActionsDisPstFor[Gbl.Forum.ForumType] : ActionsEnbPstFor[Gbl.Forum.ForumType]);
         Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
         For_PutHiddenParamPstCod (PstCod);
         For_PutAllHiddenParamsForum ();
         For_PutHiddenParamThrCod (Thr->ThrCod);

         sprintf (Gbl.MsgWindowStatus,"Mensaje %lu %s.",PstNum,	// Need translation!
                  Enabled ? "permitido. Pulse para censurarlo" :
                            "censurado. Pulse para permitirlo");	// Need translation!
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/%s12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Enabled ? "finp" : "proh",Gbl.MsgWindowStatus);
        }
      else
         fprintf (Gbl.F.Out,"<acronym title=\"Mensaje %s\">" \
                            "<img src=\"%s/%s_off12x12.gif\" alt=\"Mensaje %s\" width=\"12\" height=\"12\" /></acronym>",	// Need translation!
                  Enabled ? "permitido" : "censurado",	// Need translation!
                  Gbl.Prefs.IconsURL,Enabled ? "finp" : "proh",
                  Enabled ? "permitido" : "censurado");	// Need translation!
     }

   /***** Write author or destinatary, and form to reply (in case of received post) *****/
   fprintf (Gbl.F.Out,"</td><td align=\"left\" width=\"120\" colspan=\"2\" valign=\"top\">" \
                      "<table width=\"120\" cellspacing=\"0\" cellpadding=\"2\">");

   /* Write author */
   fprintf (Gbl.F.Out,"<tr>");
   Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat);
   Msg_WriteMsgAuthor (&UsrDat,100,16,"MSG_AUT","MSG_AUT",Enabled,NULL);
   fprintf (Gbl.F.Out,"</tr>");
   if (Enabled)
     {
      /* Write number of posts from this user */
      fprintf (Gbl.F.Out,"<tr>");
      For_WriteNumberOfPosts (Gbl.Forum.ForumType,UsrDat.UsrCod);
      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Write post content *****/
   fprintf (Gbl.F.Out,"</table></td><td align=\"left\" valign=\"top\" class=\"MSJ\"><tt>");
   if (Enabled)
     {
      strncpy (Content,OriginalContent,Cte_MAX_BYTES_TEXT);
      Msg_WriteMsgContent (Content,Cte_MAX_BYTES_TEXT,true,false);
     }
   else
      fprintf (Gbl.F.Out,"Este mensaje ha sido censurado," \
                         " probablemente por no cumplir las <a href=\"/wiki/index.php/Netiquette\" target=\"_blank\">normas del foro</a>.");
   fprintf (Gbl.F.Out,"</tt><br />&nbsp;</td></tr>");

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDat);
  }

/*****************************************************************************/
/*************************** Get data of a forum post ************************/
/*****************************************************************************/

static void For_GetPstData (long PstCod,long *UsrCod,char *CreatTime,char *Subject, char *Content)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get data of a post from database *****/
   sprintf (Query,"SELECT UsrCod,DATE_FORMAT(CreatTime,'%%Y%%m%%d%%H%%i%%S'),Subject,Content" \
                  " FROM forum_post WHERE PstCod='%ld'",PstCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a post");

   /***** Result should have a unique row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Internal error in database when getting data of a post.");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);

   /****** Get author code (row[1]) *****/
   *UsrCod = Str_ConvertStrCodToLongCod (row[0]);

   /****** Get creation time (row[1]) *****/
   strcpy (CreatTime,row[1]);

   /****** Get subject (row[2]) *****/
   strcpy (Subject,row[2]);

   /****** Get location (row[3]) *****/
   strcpy (Content,row[3]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/*************** Write number of posts in a forum of an user *****************/
/*****************************************************************************/

void For_WriteNumberOfPosts (For_ForumType_t ForumType,long UsrCod)
  {
   extern const char *Txt_post[Txt_NUM_LANGUAGES];
   extern const char *Txt_posts[Txt_NUM_LANGUAGES];
   char SubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPsts;

   /***** Star table cell *****/
   fprintf (Gbl.F.Out,"<td width=\"120\" colspan=\"2\" align=\"center\" valign=\"top\" class=\"MSG_AUT\">");

   /***** Get number of posts from database *****/
   switch (ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
         SubQuery[0] = '\0';
     }
   sprintf (Query,"SELECT COUNT(*) FROM forum_post,forum_thread" \
                  " WHERE forum_post.UsrCod='%ld' AND forum_post.ThrCod=forum_thread.ThrCod AND forum_thread.ForumType='%u'%s",
            UsrCod,(unsigned) ForumType,SubQuery);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of posts of a user in a forum");

   /* Get row with number of threads and number of messages */
   row = mysql_fetch_row (mysql_res);

   /* Get number of messages (row[0]) */
   if (sscanf (row[0],"%u",&NumPsts) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of posts of a user in a forum.");

   /***** Write number of threads and number of messages *****/
   if (NumPsts == 1)
      fprintf (Gbl.F.Out,"[1 %s]",Txt_post[Gbl.Prefs.Language]);
   else
      fprintf (Gbl.F.Out,"[%u %s]",NumPsts,Txt_posts[Gbl.Prefs.Language]);

   /***** End table cell *****/
   fprintf (Gbl.F.Out,"</td>");
  }

/****************************************************************************/
/************ Put all the hidden parameters related to forums ***************/
/****************************************************************************/

void For_PutAllHiddenParamsForum (void)
  {
   For_PutParamWhichForum ();
   For_PutParamForumOrder ();
   For_PutParamsForumIndDegCrs ();
  }

/****************************************************************************/
/*********** Put a hidden parameter with which forum I want to see **********/
/****************************************************************************/

static void For_PutParamWhichForum (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AllMyForums\" value=\"%c\" />",
            (Gbl.Forum.WhichForums == For_ALL_MY_FORUMS) ? 'Y' : 'N');
  }

/****************************************************************************/
/******** Put a hidden parameter with the order criterium for forums ********/
/****************************************************************************/

static void For_PutParamForumOrder (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Forum.SelectedOrderType);
  }

/****************************************************************************/
/** Put hidden parameters related to forum institution, degree and course ***/
/****************************************************************************/

static void For_PutParamsForumIndDegCrs (void)
  {
   /***** Put a hidden parameter with the institution of the forum *****/
   if (Gbl.Forum.Ins.InsCod > 0)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ForInsCod\" value=\"%ld\" />",
               Gbl.Forum.Ins.InsCod);

   /***** Put a hidden parameter with the degree of the forum *****/
   if (Gbl.Forum.Deg.DegCod > 0)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ForDegCod\" value=\"%ld\" />",
               Gbl.Forum.Deg.DegCod);

   /***** Put a hidden parameter with the course of the forum *****/
   if (Gbl.Forum.Crs.CrsCod > 0)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ForCrsCod\" value=\"%ld\" />",
               Gbl.Forum.Crs.CrsCod);
  }

/*****************************************************************************/
/***************************** Set the type of forum *************************/
/*****************************************************************************/

void For_SetForumTypeAndRestrictAccess (void)
  {
   bool ICanSeeForum = false;

   switch (Gbl.CurrentAct)
     {
      case ActSeeForSWAUsr:	case ActSeePstForSWAUsr:
      case ActRecThrForSWAUsr:	case ActRecRepForSWAUsr:
      case ActReqDelThrSWAUsr:	case ActDelThrForSWAUsr:
      case ActCutThrForSWAUsr:	case ActPasThrForSWAUsr:
      case ActDelPstForSWAUsr:
      case ActEnbPstForSWAUsr:	case ActDisPstForSWAUsr:
         Gbl.Forum.ForumType = For_FORUM_SWAD_USRS;
         Gbl.Forum.Level = 1;
         break;
      case ActSeeForSWATch:	case ActSeePstForSWATch:
      case ActRecThrForSWATch:	case ActRecRepForSWATch:
      case ActReqDelThrSWATch:	case ActDelThrForSWATch:
      case ActCutThrForSWATch:	case ActPasThrForSWATch:
      case ActDelPstForSWATch:
      case ActEnbPstForSWATch:	case ActDisPstForSWATch:  
         Gbl.Forum.ForumType = For_FORUM_SWAD_TCHS;
         Gbl.Forum.Level = 1;
         break;
      case ActSeeForGenUsr:	case ActSeePstForGenUsr:
      case ActRecThrForGenUsr:	case ActRecRepForGenUsr:
      case ActReqDelThrGenUsr:	case ActDelThrForGenUsr:
      case ActCutThrForGenUsr:	case ActPasThrForGenUsr:
      case ActDelPstForGenUsr:
      case ActEnbPstForGenUsr:	case ActDisPstForGenUsr:
         Gbl.Forum.ForumType = For_FORUM_GLOBAL_USRS;
         Gbl.Forum.Level = 1;
         break;
      case ActSeeForGenTch:	case ActSeePstForGenTch:
      case ActRecThrForGenTch:	case ActRecRepForGenTch:
      case ActReqDelThrGenTch:	case ActDelThrForGenTch:
      case ActCutThrForGenTch:	case ActPasThrForGenTch:
      case ActDelPstForGenTch:
      case ActEnbPstForGenTch:	case ActDisPstForGenTch:  
         Gbl.Forum.ForumType = For_FORUM_GLOBAL_TCHS;
         Gbl.Forum.Level = 1;
         break;
      case ActSeeForInsUsr:	case ActSeePstForInsUsr:
      case ActRecThrForInsUsr:	case ActRecRepForInsUsr:
      case ActReqDelThrInsUsr:	case ActDelThrForInsUsr:
      case ActCutThrForInsUsr:	case ActPasThrForInsUsr:
      case ActDelPstForInsUsr:
      case ActEnbPstForInsUsr:	case ActDisPstForInsUsr:  
         Gbl.Forum.ForumType = For_FORUM_INSTITUTION_USRS;
         Gbl.Forum.Level = 1;
         break;
      case ActSeeForInsTch:	case ActSeePstForInsTch:
      case ActRecThrForInsTch:	case ActRecRepForInsTch:
      case ActReqDelThrInsTch:	case ActDelThrForInsTch:
      case ActCutThrForInsTch:	case ActPasThrForInsTch:
      case ActDelPstForInsTch:
      case ActEnbPstForInsTch:	case ActDisPstForInsTch:  
	 Gbl.Forum.ForumType = For_FORUM_INSTITUTION_TCHS;
         Gbl.Forum.Level = 1;
	 break;
      case ActSeeForCtrUsr:	case ActSeePstForCtrUsr:
      case ActRecThrForCtrUsr:	case ActRecRepForCtrUsr:
      case ActReqDelThrCtrUsr:	case ActDelThrForCtrUsr:
      case ActCutThrForCtrUsr:	case ActPasThrForCtrUsr:
      case ActDelPstForCtrUsr:
      case ActEnbPstForCtrUsr:	case ActDisPstForCtrUsr:  
         Gbl.Forum.ForumType = For_FORUM_CENTRE_USRS;
         Gbl.Forum.Level = 1;
         break;
      case ActSeeForCtrTch:	case ActSeePstForCtrTch:
      case ActRecThrForCtrTch:	case ActRecRepForCtrTch:
      case ActReqDelThrCtrTch:	case ActDelThrForCtrTch:
      case ActCutThrForCtrTch:	case ActPasThrForCtrTch:
      case ActDelPstForCtrTch:
      case ActEnbPstForCtrTch:	case ActDisPstForCtrTch:  
	 Gbl.Forum.ForumType = For_FORUM_CENTRE_TCHS;
         Gbl.Forum.Level = 1;
	 break;
      case ActSeeForDegUsr:	case ActSeePstForDegUsr:
      case ActRecThrForDegUsr:	case ActRecRepForDegUsr:
      case ActReqDelThrDegUsr:	case ActDelThrForDegUsr:
      case ActCutThrForDegUsr:	case ActPasThrForDegUsr:
      case ActDelPstForDegUsr:
      case ActEnbPstForDegUsr:	case ActDisPstForDegUsr:  
         Gbl.Forum.ForumType = For_FORUM_DEGREE_USRS;
         Gbl.Forum.Level = 2;
         break;
      case ActSeeForDegTch:	case ActSeePstForDegTch:
      case ActRecThrForDegTch:	case ActRecRepForDegTch:
      case ActReqDelThrDegTch:	case ActDelThrForDegTch:
      case ActCutThrForDegTch:	case ActPasThrForDegTch:
      case ActDelPstForDegTch:
      case ActEnbPstForDegTch:	case ActDisPstForDegTch:  
	 Gbl.Forum.ForumType = For_FORUM_DEGREE_TCHS;
         Gbl.Forum.Level = 2;
	 break;
      case ActSeeForCrsUsr:	case ActSeePstForCrsUsr:
      case ActRecThrForCrsUsr:	case ActRecRepForCrsUsr:
      case ActReqDelThrCrsUsr:	case ActDelThrForCrsUsr:
      case ActCutThrForCrsUsr:	case ActPasThrForCrsUsr:
      case ActDelPstForCrsUsr:
      case ActEnbPstForCrsUsr:	case ActDisPstForCrsUsr:  
         Gbl.Forum.ForumType = For_FORUM_COURSE_USRS;
         Gbl.Forum.Level = 3;
         break;
      case ActSeeForCrsTch:	case ActSeePstForCrsTch:
      case ActRecThrForCrsTch:	case ActRecRepForCrsTch:
      case ActReqDelThrCrsTch:	case ActDelThrForCrsTch:
      case ActCutThrForCrsTch:	case ActPasThrForCrsTch:
      case ActDelPstForCrsTch:
      case ActEnbPstForCrsTch:	case ActDisPstForCrsTch:  
         Gbl.Forum.ForumType = For_FORUM_COURSE_TCHS;
         Gbl.Forum.Level = 3;
         break;
     }

   /***** Restrict access *****/
   switch (Gbl.Forum.ForumType)
     {
      case For_FORUM_COURSE_USRS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                         Usr_CheckIfIBelongToCrs (Gbl.Forum.Crs.CrsCod));
         break;
      case For_FORUM_COURSE_TCHS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                         (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER &&
                          Usr_CheckIfIBelongToCrs (Gbl.Forum.Crs.CrsCod)));
         break;
      case For_FORUM_DEGREE_USRS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                         Usr_CheckIfIBelongToDeg (Gbl.Forum.Deg.DegCod));
         break;
      case For_FORUM_DEGREE_TCHS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                         (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER &&
                          Usr_CheckIfIBelongToDeg (Gbl.Forum.Deg.DegCod)));
         break;
      case For_FORUM_CENTRE_USRS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_STUDENT);
         break;
      case For_FORUM_CENTRE_TCHS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER);
         break;
      case For_FORUM_INSTITUTION_USRS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                         Usr_CheckIfIBelongToIns (Gbl.Forum.Ins.InsCod));
         break;
      case For_FORUM_INSTITUTION_TCHS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                         (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER &&
                          Usr_CheckIfIBelongToIns (Gbl.Forum.Ins.InsCod)));
         break;
      case For_FORUM_GLOBAL_USRS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_STUDENT);
         break;
      case For_FORUM_GLOBAL_TCHS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER);
         break;
      case For_FORUM_SWAD_USRS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_STUDENT);
         break;
      case For_FORUM_SWAD_TCHS:
         ICanSeeForum = (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER);
         break;
     }
   if (!ICanSeeForum)
      Lay_ShowErrorAndExit ("You don't have permission to access to forum.");
  }

/*****************************************************************************/
/************************** Show list of available forums ********************/
/*****************************************************************************/

void For_ShowForumList (void)
  {
   bool ICanMoveThreads = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER);	// If I have permission to move threads...
   bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS];
   unsigned NumMyIns;
   MYSQL_RES *mysql_resDeg,*mysql_resCrs;
   MYSQL_ROW row;
   unsigned NumDeg,NumDegs;
   unsigned NumCrs,NumCrss;
   bool ICanSeeInsForum;
   bool ICanSeeDegForum;

   /***** Get if there is a thread ready to be moved *****/
   if (ICanMoveThreads)
      Gbl.Forum.ThreadToMove = For_GetThrInMyClipboard ();

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Fill the list with the institutions I belong to *****/ 
   Usr_GetMyInstitutions ();

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /***** Put a form to select which forums *****/
   For_PutFormWhichForums ();

   /***** Write a link to top level of forums *****/
   For_WriteLinkToTopLevelOfForums ();

   /***** Links to global forums *****/
   For_WriteLinksToGblForums (IsLastItemInLevel);

   switch (Gbl.Forum.WhichForums)
     {
      case For_ALL_MY_FORUMS:
         /***** Links to forums about the platform *****/
         For_WriteLinksToPlatformForums ((Gbl.Usrs.Me.MyInstitutions.Num == 0),IsLastItemInLevel);

         /***** Links to forums of users from my institutions, the degrees in each institution and the courses in each degree *****/
         for (NumMyIns=0; NumMyIns<Gbl.Usrs.Me.MyInstitutions.Num; NumMyIns++)
           {
            /* Links to forums of this institution */
            For_WriteLinksToInsForums (Gbl.Usrs.Me.MyInstitutions.InsCods[NumMyIns],(NumMyIns == Gbl.Usrs.Me.MyInstitutions.Num-1),IsLastItemInLevel);

            /* Get my degrees in this institution from database */
            if ((NumDegs = Usr_GetDegsFromUsr (Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Forum.Ins.InsCod,&mysql_resDeg)) > 0) // Degrees found in this institution
               for (NumDeg=0; NumDeg<NumDegs; NumDeg++)
                 {
                  /* Get next degree */
                  row = mysql_fetch_row (mysql_resDeg);

                  /* Links to forums of this degree */
                  if (For_WriteLinksToDegForums (Str_ConvertStrCodToLongCod (row[0]),(NumDeg == NumDegs-1),IsLastItemInLevel) > 0)
                    {
                     /* Get my courses in this degree from database */
                     if ((NumCrss = Usr_GetCrssFromUsr (Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Forum.Deg.DegCod,&mysql_resCrs)) > 0) // Courses found in this degree
                        for (NumCrs=0; NumCrs<NumCrss; NumCrs++)
                          {
                           /* Get next course */
                           row = mysql_fetch_row (mysql_resCrs);

                           /* Links to forums of this course */
                           For_WriteLinksToCrsForums (Str_ConvertStrCodToLongCod (row[0]),(NumCrs == NumCrss-1),IsLastItemInLevel);
                          }

                     /* Free structure that stores the query result */
                     DB_FreeMySQLResult (&mysql_resCrs);
	            }
                 }

            /* Free structure that stores the query result */
            DB_FreeMySQLResult (&mysql_resDeg);
           }
         break;
      case For_ONLY_FORUMS_CURRENT_INS_DEG_CRS:
         ICanSeeInsForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                            Gbl.Usrs.Me.UsrTypeForumsChat == Usr_DEG_ADMIN ||
                            Usr_CheckIfIBelongToIns (Gbl.CurrentIns.Ins.InsCod));

         /***** Links to forums about the platform *****/
         For_WriteLinksToPlatformForums (!ICanSeeInsForum,IsLastItemInLevel);

         if (ICanSeeInsForum)
           {
            ICanSeeDegForum = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER ||
                               Gbl.Usrs.Me.UsrTypeForumsChat == Usr_DEG_ADMIN ||
                               Usr_CheckIfIBelongToDeg (Gbl.CurrentDeg.Deg.DegCod));

            /***** Links to forums of current institution *****/
            if (For_WriteLinksToInsForums (Gbl.CurrentIns.Ins.InsCod,true,IsLastItemInLevel) > 0)
               if (ICanSeeDegForum)
                  /***** Links to forums of current degree *****/
                  if (For_WriteLinksToDegForums (Gbl.CurrentDeg.Deg.DegCod,true,IsLastItemInLevel) > 0)
                     if (Gbl.Usrs.Me.IHaveAccessToCurrentCrs)
                        /***** Links to forums of current degree *****/
                        For_WriteLinksToCrsForums (Gbl.CurrentCrs.Crs.CrsCod,true,IsLastItemInLevel);
           }
         break;
      default:
         break;
     }

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/**************** Write title and link to top level of forums ****************/
/*****************************************************************************/

static void For_PutFormWhichForums (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_FORUM_WHICH_FORUM[2][Txt_NUM_LANGUAGES];
   For_WhichForums_t WhichForums;

   /***** Form to select which forums I want to see
          (all my forums or only the forums of current institution/degree/course) *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">",
            The_ClassFormul[Gbl.Prefs.Theme]);
   Act_FormStart (ActSeeFor);
   For_PutParamForumOrder ();
   For_PutParamsForumIndDegCrs ();
   for (WhichForums=(For_WhichForums_t) 0; WhichForums<For_NUM_WHICH_FORUMS; WhichForums++)
     {
      fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"AllMyForums\" value=\"%c\"",
               (WhichForums == For_ALL_MY_FORUMS) ? 'Y' : 'N');
      if (WhichForums == Gbl.Forum.WhichForums)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," onclick=\"javascript:document.forms[%d].submit();\" />%s",
               Gbl.NumForm,Txt_FORUM_WHICH_FORUM[WhichForums][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</form></td></tr>");
  }

/*****************************************************************************/
/**************** Write title and link to top level of forums ****************/
/*****************************************************************************/

void For_WriteLinkToTopLevelOfForums (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Forums[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"%s\">",
            The_ClassFormul[Gbl.Prefs.Theme]);
   Act_FormStart (ActSeeFor);
   For_PutAllHiddenParamsForum ();
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Forums[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/forum16x16.gif\" width=\"16\" height=\"16\" align=\"top\" title=\"%s\" alt=\"%s\" />" \
                      "&nbsp;%s</acronym></a></form></td></tr>",
            Gbl.Prefs.IconsURL,
            Txt_Forums[Gbl.Prefs.Language],
            Txt_Forums[Gbl.Prefs.Language],
            Txt_Forums[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/************************* Write links to global forums **********************/
/*****************************************************************************/

static void For_WriteLinksToGblForums (bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   /***** Link to forum global *****/
   IsLastItemInLevel[1] = false;
   For_WriteLinkToAForum (For_FORUM_GLOBAL_USRS,false,1,IsLastItemInLevel);

   /***** Link to forum of teachers global *****/
   if (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER)
     {
      IsLastItemInLevel[1] = false;
      For_WriteLinkToAForum (For_FORUM_GLOBAL_TCHS,false,1,IsLastItemInLevel);
     }
  }

/*****************************************************************************/
/****************** Write links to forums about the platform *****************/
/*****************************************************************************/

static void For_WriteLinksToPlatformForums (bool IsLastForum,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   /***** Link to forum of users about the platform *****/
   IsLastItemInLevel[1] = (Gbl.Usrs.Me.UsrTypeForumsChat < Usr_TEACHER && IsLastForum);
   For_WriteLinkToAForum (For_FORUM_SWAD_USRS,false,1,IsLastItemInLevel);

   /***** Link to forum of teachers about the platform *****/
   if (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER)
     {
      IsLastItemInLevel[1] = IsLastForum;
      For_WriteLinkToAForum (For_FORUM_SWAD_TCHS,false,1,IsLastItemInLevel);
     }
  }

/*****************************************************************************/
/********************** Write links to institution forums ********************/
/*****************************************************************************/
// Returns institution code

static long For_WriteLinksToInsForums (long InsCod,bool IsLastIns,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   if (InsCod > 0)
     {
      /***** Get data of this institution *****/
      Gbl.Forum.Ins.InsCod = InsCod;
      if (!Ins_GetDataOfInstitutionByCod (&Gbl.Forum.Ins))
         Lay_ShowErrorAndExit ("Institution not found.");

      /***** Link to the forum of users from this institution *****/
      IsLastItemInLevel[1] = (IsLastIns && Gbl.Usrs.Me.UsrTypeForumsChat < Usr_TEACHER);
      For_WriteLinkToAForum (For_FORUM_INSTITUTION_USRS,false,1,IsLastItemInLevel);

      /***** Link to forum of teachers from this institution *****/
      if (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER)
        {
         IsLastItemInLevel[1] = IsLastIns;
         For_WriteLinkToAForum (For_FORUM_INSTITUTION_TCHS,false,1,IsLastItemInLevel);
        }
     }
   return InsCod;
  }

/*****************************************************************************/
/************************ Write links to degree forums ***********************/
/*****************************************************************************/
// Returns degree code

static long For_WriteLinksToDegForums (long DegCod,bool IsLastDeg,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   if (DegCod > 0)
     {
      /***** Get data of this degree *****/
      Gbl.Forum.Deg.DegCod = DegCod;
      if (!Deg_GetDataOfDegreeByCod (&Gbl.Forum.Deg))
         Lay_ShowErrorAndExit ("Degree not found.");

      /***** Link to the forum of users from this degree *****/
      IsLastItemInLevel[2] = (IsLastDeg && Gbl.Usrs.Me.UsrTypeForumsChat < Usr_TEACHER);
      For_WriteLinkToAForum (For_FORUM_DEGREE_USRS,false,2,IsLastItemInLevel);

      /***** Link to forum of teachers from this degree *****/
      if (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER)
        {
         IsLastItemInLevel[2] = IsLastDeg;
         For_WriteLinkToAForum (For_FORUM_DEGREE_TCHS,false,2,IsLastItemInLevel);
        }
     }
   return DegCod;
  }

/*****************************************************************************/
/************************ Write links to course forums ***********************/
/*****************************************************************************/
// Returns course code

static long For_WriteLinksToCrsForums (long CrsCod,bool IsLastCrs,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   if (CrsCod > 0)
     {
      /***** Get data of this course *****/
      Gbl.Forum.Crs.CrsCod = CrsCod;
      if (!Deg_GetDataOfCourseByCod (&Gbl.Forum.Crs))
         Lay_ShowErrorAndExit ("Course not found.");

      /***** Link to the forum of users from this course *****/
      IsLastItemInLevel[3] = (IsLastCrs && Gbl.Usrs.Me.UsrTypeForumsChat < Usr_TEACHER);
      For_WriteLinkToAForum (For_FORUM_COURSE_USRS,false,3,IsLastItemInLevel);

      /***** Link to forum of teachers from this course *****/
      if (Gbl.Usrs.Me.UsrTypeForumsChat >= Usr_TEACHER)
        {
         IsLastItemInLevel[3] = IsLastCrs;
         For_WriteLinkToAForum (For_FORUM_COURSE_TCHS,false,3,IsLastItemInLevel);
        }
     }
   return CrsCod;
  }

/*****************************************************************************/
/********************** Write title and link to a forum **********************/
/*****************************************************************************/

static void For_WriteLinkToAForum (For_ForumType_t ForumType,bool ShowNumOfPosts,
                                   unsigned Level,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   char Icon[512];
   char ForumName[512];

   For_SetForumName (ForumType,&Gbl.Forum.Ins,&Gbl.Forum.Deg,&Gbl.Forum.Crs,ForumName,Gbl.Prefs.Language,true);

   switch (ForumType)
     {
      case For_FORUM_COURSE_USRS:
         sprintf (Icon,"<img src=\"%s/course16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL);
         break;
      case For_FORUM_COURSE_TCHS:
         sprintf (Icon,"<img src=\"%s/course16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL);
         break;
      case For_FORUM_DEGREE_USRS:
         sprintf (Icon,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Gbl.Forum.Deg.Logo);
         break;
      case For_FORUM_DEGREE_TCHS:
         sprintf (Icon,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Gbl.Forum.Deg.Logo);
         break;
      case For_FORUM_CENTRE_USRS:
         sprintf (Icon,"<img src=\"%s\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.CurrentDeg.Logo16);
         break;
      case For_FORUM_CENTRE_TCHS:
         sprintf (Icon,"<img src=\"%s\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.CurrentDeg.Logo16);
         break;
      case For_FORUM_INSTITUTION_USRS:
         sprintf (Icon,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.Forum.Ins.Logo);
         break;
      case For_FORUM_INSTITUTION_TCHS:
         sprintf (Icon,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.Forum.Ins.Logo);
         break;
      case For_FORUM_GLOBAL_USRS:
         sprintf (Icon,"<img src=\"%s/ballon16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL);
         break;
      case For_FORUM_GLOBAL_TCHS:
         sprintf (Icon,"<img src=\"%s/ballon16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL);
         break;
      case For_FORUM_SWAD_USRS:
         sprintf (Icon,"<img src=\"%s/swad16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL);
         break;
      case For_FORUM_SWAD_TCHS:
         sprintf (Icon,"<img src=\"%s/swad16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                  Gbl.Prefs.IconsURL);
         break;
     }

   For_WriteLinkToForum (ForumType,ActionsSeeFor[ForumType],Icon,ForumName,ShowNumOfPosts,
                         Level,IsLastItemInLevel);
  }

/*****************************************************************************/
/************************** Set the name of a forum **************************/
/*****************************************************************************/

void For_SetForumName (For_ForumType_t ForumType,struct Institution *Ins,struct Degree *Deg,struct Course *Crs,char *ForumName,Txt_Language_t Language,bool UseHTMLEntities)
  {
   extern const char *Txt_General[Txt_NUM_LANGUAGES];
   extern const char *Txt_General_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_only_teachers[Txt_NUM_LANGUAGES];
   extern const char *Txt_only_teachers_NO_HTML[Txt_NUM_LANGUAGES];

   switch (ForumType)
     {
      case For_FORUM_COURSE_USRS:
         strcpy (ForumName,Crs->ShortName);
         break;
      case For_FORUM_COURSE_TCHS:
         sprintf (ForumName,"%s%s",Crs->ShortName,
                  UseHTMLEntities ? Txt_only_teachers[Language] :
                                    Txt_only_teachers_NO_HTML[Language]);
         break;
      case For_FORUM_DEGREE_USRS:
         strcpy (ForumName,Deg->ShortName);
         break;
      case For_FORUM_DEGREE_TCHS:
         sprintf (ForumName,"%s%s",Deg->ShortName,
                  UseHTMLEntities ? Txt_only_teachers[Language] :
                                    Txt_only_teachers_NO_HTML[Language]);
         break;
      case For_FORUM_CENTRE_USRS:	// Change !!!!
         strcpy (ForumName,"centro (sin implementar)");
         break;
      case For_FORUM_CENTRE_TCHS:	// Change !!!!
         sprintf (ForumName,"centro (sin implementar)%s",
                  UseHTMLEntities ? Txt_only_teachers[Language] :
                                    Txt_only_teachers_NO_HTML[Language]);
         break;
      case For_FORUM_INSTITUTION_USRS:
         strcpy (ForumName,Ins->ShortName);
         break;
      case For_FORUM_INSTITUTION_TCHS:
         sprintf (ForumName,"%s%s",Ins->ShortName,
                  UseHTMLEntities ? Txt_only_teachers[Language] :
                                    Txt_only_teachers_NO_HTML[Language]);
         break;
      case For_FORUM_GLOBAL_USRS:
         strcpy (ForumName,UseHTMLEntities ? Txt_General[Language] : 
                                             Txt_General_NO_HTML[Language]);
         break;
      case For_FORUM_GLOBAL_TCHS:
         sprintf (ForumName,"%s%s",
                  UseHTMLEntities ? Txt_General[Language] : 
                                    Txt_General_NO_HTML[Language],
                  UseHTMLEntities ? Txt_only_teachers[Language] :
                                    Txt_only_teachers_NO_HTML[Language]);
         break;
      case For_FORUM_SWAD_USRS:
         strcpy (ForumName,Cfg_PLATFORM_SHORT_NAME);
         break;
      case For_FORUM_SWAD_TCHS:
         sprintf (ForumName,"%s%s",Cfg_PLATFORM_SHORT_NAME,
                  UseHTMLEntities ? Txt_only_teachers[Language] :
                                    Txt_only_teachers_NO_HTML[Language]);
         break;
     }
  }

/*****************************************************************************/
/*********************** Write title and link to a forum *********************/
/*****************************************************************************/

static void For_WriteLinkToForum (For_ForumType_t ForumType,Act_Action_t NextAct,const char *Icon,const char *ForumName,bool ShowNumOfPosts,
                                  unsigned Level,bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS])
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *The_ClassFormulB[The_NUM_THEMES];
   unsigned NumThrs;
   unsigned NumThrsWithNewPosts;
   unsigned NumPosts;
   const char *Style;

   /***** Get number of threads and number of posts *****/
   NumThrs = For_GetNumThrsInForum (ForumType);
   NumThrsWithNewPosts = For_GetNumThrsWithNewPstsInForum (ForumType,NumThrs);
   Style = (NumThrsWithNewPosts ? The_ClassFormulB[Gbl.Prefs.Theme] : The_ClassFormul[Gbl.Prefs.Theme]);

   /***** Start row *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\"><tr>");

   /***** Indent forum title *****/
   Msg_IndentDependingOnLevel (Level,IsLastItemInLevel);

   /***** Write paste button used to move a thread in clipboard to this forum *****/
   if (Gbl.Forum.ThreadToMove >= 0) // If I have permission to paste threads and there is a thread ready to be pasted...
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" width=\"16\">");

      /* Check if thread to move is yet in current forum */
      if (For_CheckIfThrBelongsToForum (Gbl.Forum.ThreadToMove,ForumType))
         fprintf (Gbl.F.Out,"<img src=\"%s/paste_off12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActionsPasThrFor[ForumType]);
         Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
         For_PutAllHiddenParamsForum ();
         For_PutHiddenParamThrCod (Gbl.Forum.ThreadToMove);
         sprintf (Gbl.MsgWindowStatus,"Pegar hebra");	// Need translation!
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/paste12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
        }

      fprintf (Gbl.F.Out,"</td>");
     }

   /***** Write link to forum *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" class=\"%s\">",Style);
   Act_FormStart (NextAct);
   For_PutAllHiddenParamsForum ();
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Actions[NextAct].LogMessage,Style);
   fprintf (Gbl.F.Out,"%s&nbsp;%s",
            Icon,ForumName);

   /***** Write total number of threads and messages in this forum *****/
   if (ShowNumOfPosts)
     {
      NumPosts = For_GetNumPstsInForum (ForumType);
      For_WriteNumThrsAndPsts (NumThrs,NumThrsWithNewPosts,NumPosts);
     }
   else
      For_WriteNumberOfThrs (NumThrs,NumThrsWithNewPosts);

   fprintf (Gbl.F.Out,"</acronym></a></form></td></tr></table></td></tr>");
  }

/*****************************************************************************/
/***** Get number of threads with new posts since my last read of a forum ****/
/*****************************************************************************/

unsigned For_GetNumThrsWithNewPstsInForum (For_ForumType_t ForumType,unsigned NumThreads)
  {
   char SubQuery[256];
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumThrsWithNewPosts = NumThreads;	// By default, all the threads are new to me

   /***** Get last time I read this forum from database *****/
   switch (ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
         SubQuery[0] = '\0';
     }
   sprintf (Query,"SELECT MAX(forum_thr_read.ReadTime) FROM forum_thr_read,forum_thread" \
                  " WHERE forum_thr_read.UsrCod='%ld' AND forum_thr_read.ThrCod=forum_thread.ThrCod" \
                  " AND forum_thread.ForumType='%u'%s",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) ForumType,SubQuery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the date of reading of a forum");

   if (NumRows)
     {
      /***** Get number of threads with a last message modify time > newest read time (row[0]) *****/
      row = mysql_fetch_row (mysql_res);
      NumThrsWithNewPosts = For_GetNumOfThreadsInForumNewerThan (ForumType,row[0]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumThrsWithNewPosts;
  }

/*****************************************************************************/
/**** Get number of threads in forum with a modify time > a specified time ***/
/*****************************************************************************/

static unsigned For_GetNumOfThreadsInForumNewerThan (For_ForumType_t ForumType,const char *Time)
  {
   char SubQuery[256];
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumThreads;

   /***** Get number of threads with a last message modify time > specified time from database *****/
   switch (ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
         SubQuery[0] = '\0';
     }
   sprintf (Query,"SELECT COUNT(*) FROM forum_thread,forum_post WHERE forum_thread.ForumType='%u'%s" \
                  " AND forum_thread.LastPstCod=forum_post.PstCod AND forum_post.ModifTime>'%s'",
                  (unsigned) ForumType,SubQuery,Time);
   DB_QuerySELECT (Query,&mysql_res,"can not check if there are new posts in a forum");

   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumThreads) != 1)
      Lay_ShowErrorAndExit ("Error when checking if there are new posts in a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumThreads;
  }

/*****************************************************************************/
/** Get number of unread posts in a thread since my last read of that thread */
/*****************************************************************************/

static unsigned For_GetNumOfUnreadPostsInThr (long ThrCod,unsigned NumPostsInThr)
  {
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumUnreadPosts = NumPostsInThr;	// By default, all the posts are unread by me

   /***** Get last time I read this thread from database *****/
   sprintf (Query,"SELECT ReadTime FROM forum_thr_read" \
                  " WHERE ThrCod='%ld' AND UsrCod='%ld'",
            ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the date of reading of a thread");

   /***** Get if last time I read this thread exists in database *****/
   if (NumRows)
     {
      /***** Get the number of posts in thread with a modify time > newest read time for me (row[0]) *****/
      row = mysql_fetch_row (mysql_res);
      NumUnreadPosts = For_GetNumOfPostsInThrNewerThan (ThrCod,row[0]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumUnreadPosts;
  }

/*****************************************************************************/
/*** Get number of messages in thread with a modify time > a specified time **/
/*****************************************************************************/

static unsigned For_GetNumOfPostsInThrNewerThan (long ThrCod,const char *Time)
  {
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPosts;

   /***** Get the number of messages in thread with a modify time > a specified time from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM forum_post" \
                  " WHERE ThrCod='%ld' AND ModifTime>'%s'",
            ThrCod,Time);
   DB_QuerySELECT (Query,&mysql_res,"can not check if there are new posts in a thread of a forum");

   /***** Get number of posts *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumPosts) != 1)
      Lay_ShowErrorAndExit ("Error when checking if there are new posts in a thread of a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumPosts;
  }

/*****************************************************************************/
/********************** Show available threads of a forum ********************/
/*****************************************************************************/

void For_ShowForumThrs (void)
  {
   extern const char *Txt_MSG_Subject[Txt_NUM_LANGUAGES];
   extern const char *Txt_FORUM_THREAD_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_FORUM_THREAD_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_No_BR_msgs[Txt_NUM_LANGUAGES];
   extern const char *Txt_Unread_BR_msgs[Txt_NUM_LANGUAGES];
   extern const char *Txt_WriBRters[Txt_NUM_LANGUAGES];
   extern const char *Txt_ReaBRders[Txt_NUM_LANGUAGES];
   bool IsLastItemInLevel[1+For_FORUM_MAX_LEVELS];
   char SubQuery[256];
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumThr,NumThrs;
   unsigned NumThrInScreen;	// From 0 to Pag_ITEMS_PER_PAGE-1
   For_ForumOrderType_t Order;
   long ThrCods[Pag_ITEMS_PER_PAGE];
   struct Pagination PaginationThrs;
   bool ICanMoveThreads = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER);	// If I have permission to move threads...

   /***** Get if there is a thread ready to be moved *****/
   if (ICanMoveThreads)
      Gbl.Forum.ThreadToMove = For_GetThrInMyClipboard ();

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get page number *****/
   Pag_GetParamPagNum (Pag_THREADS_FORUM);

   /***** Get threads of a forum from database *****/
   switch (Gbl.Forum.ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
      	 SubQuery[0] = '\0';
     }
   switch (Gbl.Forum.SelectedOrderType)
     {
      case For_FIRST_MSG:
         sprintf (Query,"SELECT forum_thread.ThrCod FROM forum_thread,forum_post" \
                        " WHERE forum_thread.ForumType='%u'%s" \
                        " AND forum_thread.FirstPstCod=forum_post.PstCod" \
                        " ORDER BY forum_post.CreatTime DESC",
                  (unsigned) Gbl.Forum.ForumType,SubQuery);
         break;
      case For_LAST_MSG:
         sprintf (Query,"SELECT forum_thread.ThrCod FROM forum_thread,forum_post" \
                        " WHERE forum_thread.ForumType='%u'%s" \
                        " AND forum_thread.LastPstCod=forum_post.PstCod" \
                        " ORDER BY forum_post.CreatTime DESC",
                  (unsigned) Gbl.Forum.ForumType,SubQuery);
     }
   NumThrs = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get thread of a forum");

   /***** Compute variables related to pagination of threads *****/
   PaginationThrs.NumItems = NumThrs;
   PaginationThrs.CurrentPage = (int) Gbl.Pag.CurrentPage;
   Pag_CalculatePagination (&PaginationThrs);
   Gbl.Pag.CurrentPage = (unsigned) PaginationThrs.CurrentPage;

   /***** Fill the list of threads for current page *****/
   for (NumThr=1, NumThrInScreen=0; NumThr<=NumThrs; NumThr++)
     {
      row = mysql_fetch_row (mysql_res);

      if (NumThr >= PaginationThrs.FirstItemVisible &&
          NumThr <= PaginationThrs.LastItemVisible)
        {
         /* Get thread code(row[0]) */
         if ((ThrCods[NumThrInScreen] = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Error when getting thread of a forum.");
         NumThrInScreen++;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Header whith the name of this forum, the number of threads, and the total number of messages *****/
   /* Table start */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /* Put a form to select which forums */
   For_PutFormWhichForums ();

   /* Write a link to top level of forums */
   For_WriteLinkToTopLevelOfForums ();

   /* Write a link to current forum */
   IsLastItemInLevel[1] = true;
   For_WriteLinkToAForum (Gbl.Forum.ForumType,true,1,IsLastItemInLevel);

   /* End table */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** List the threads *****/
   if (NumThrs)
     {
      /***** Write links to all the pages in the listing of threads *****/
      if (PaginationThrs.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered (Pag_THREADS_FORUM,0,&PaginationThrs);

      /***** Start table *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" width=\"100%%\">" \
                         "<table cellspacing=\"0\" cellpadding=\"2\" width=\"100%%\">");

      /***** Heading row *****/
      fprintf (Gbl.F.Out,"<tr><td width=\"14\" align=\"left\" class=\"TIT_TBL\">&nbsp;</td>" \
                         "<td width=\"18\" align=\"left\" class=\"TIT_TBL\">&nbsp;</td>" \
                         "<td align=\"left\" class=\"TIT_TBL\">%s</td>",
               Txt_MSG_Subject[Gbl.Prefs.Language]);
      for (Order=For_FIRST_MSG; Order<=For_LAST_MSG; Order++)
	{
	 fprintf (Gbl.F.Out,"<td colspan=\"3\" align=\"center\">");
         Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
         Pag_PutHiddenParamPagNum (PaginationThrs.CurrentPage);
         For_PutParamWhichForum ();
         For_PutParamsForumIndDegCrs ();
	 fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
	 Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_FORUM_THREAD_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
         if (Order == Gbl.Forum.SelectedOrderType)
            fprintf (Gbl.F.Out,"<u>");
	 fprintf (Gbl.F.Out,Txt_FORUM_THREAD_ORDER[Order][Gbl.Prefs.Language]);
         if (Order == Gbl.Forum.SelectedOrderType)
            fprintf (Gbl.F.Out,"</u>");
         fprintf (Gbl.F.Out,"</acronym></a></form></td>");
	}
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                         "</tr>",
               Txt_No_BR_msgs[Gbl.Prefs.Language],
               Txt_Unread_BR_msgs[Gbl.Prefs.Language],
               Txt_WriBRters[Gbl.Prefs.Language],
               Txt_ReaBRders[Gbl.Prefs.Language]);

      /***** List the threads *****/
      For_ListForumThrs (NumThrs,ThrCods,&PaginationThrs);

      /***** End table *****/
      fprintf (Gbl.F.Out,"</table></td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Write links to all the pages in the listing of threads *****/
      if (PaginationThrs.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered (Pag_THREADS_FORUM,0,&PaginationThrs);
     }

   /***** Put a form to write the first message of a new thread *****/
   For_WriteFormForumPst (false,-1,NULL);
  }

/*****************************************************************************/
/************* Get and write total number of threads and messages ************/
/*****************************************************************************/

static void For_WriteNumberOfThrs (unsigned NumThrs,unsigned NumThrsWithNewPosts)
  {
   extern const char *Txt_thread[Txt_NUM_LANGUAGES];
   extern const char *Txt_threads[Txt_NUM_LANGUAGES];
   extern const char *Txt_with_new_posts[Txt_NUM_LANGUAGES];

   /***** Write number of threads and number of posts *****/
   fprintf (Gbl.F.Out," [");
   if (NumThrs == 1)
      fprintf (Gbl.F.Out,"1 %s",Txt_thread[Gbl.Prefs.Language]);
   else
      fprintf (Gbl.F.Out,"%u %s",NumThrs,Txt_threads[Gbl.Prefs.Language]);
   if (NumThrsWithNewPosts)
      fprintf (Gbl.F.Out,", %u %s",NumThrsWithNewPosts,Txt_with_new_posts[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"]");
  }

/*****************************************************************************/
/************* Get and write total number of threads and messages ************/
/*****************************************************************************/

void For_WriteNumThrsAndPsts (unsigned NumThrs,unsigned NumThrsWithNewPosts,unsigned NumPosts)
  {
   extern const char *Txt_thread[Txt_NUM_LANGUAGES];
   extern const char *Txt_threads[Txt_NUM_LANGUAGES];
   extern const char *Txt_post[Txt_NUM_LANGUAGES];
   extern const char *Txt_posts[Txt_NUM_LANGUAGES];
   extern const char *Txt_with_new_posts[Txt_NUM_LANGUAGES];

   /***** Write number of threads and number of posts *****/
   fprintf (Gbl.F.Out," [");
   if (NumThrs == 1)
     {
      fprintf (Gbl.F.Out,"1 %s",Txt_thread[Gbl.Prefs.Language]);
      if (NumThrsWithNewPosts)
         fprintf (Gbl.F.Out,", 1 %s",Txt_with_new_posts[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"; ");
      if (NumPosts == 1)
         fprintf (Gbl.F.Out,"1 %s",Txt_post[Gbl.Prefs.Language]);
      else
         fprintf (Gbl.F.Out,"%u %s",NumPosts,Txt_posts[Gbl.Prefs.Language]);
     }
   else
     {
      fprintf (Gbl.F.Out,"%u %s",NumThrs,Txt_threads[Gbl.Prefs.Language]);
      if (NumThrsWithNewPosts)
         fprintf (Gbl.F.Out,", %u %s",NumThrsWithNewPosts,Txt_with_new_posts[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"; %u %s",NumPosts,Txt_posts[Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"]");
  }

/*****************************************************************************/
/********************** Get number of forums of a type ***********************/
/*****************************************************************************/

unsigned For_GetNumTotalForumsOfType (For_ForumType_t ForumType,long DegCod,long CrsCod)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumForums;

   /***** Get number of forums of a type from database *****/
   switch (ForumType)
     {
      case For_FORUM_SWAD_USRS:		case For_FORUM_SWAD_TCHS:
      case For_FORUM_GLOBAL_USRS:		case For_FORUM_GLOBAL_TCHS:
         return 1;	// Only one forum
      case For_FORUM_CENTRE_USRS:		case For_FORUM_CENTRE_TCHS:
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         // 0 <= number of forums <= number of centres or institutions
         sprintf (Query,"SELECT COUNT(DISTINCT Location) FROM forum_thread WHERE ForumType='%u'",
                  (unsigned) ForumType);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         if (DegCod < 0)	// Number of degree forums for all the degrees
            sprintf (Query,"SELECT COUNT(DISTINCT Location) FROM forum_thread WHERE ForumType='%u'",
                     (unsigned) ForumType);
         else			// DegCod >= 0 ==> 0 <= number of degree forums for a degree <= 1
            sprintf (Query,"SELECT COUNT(DISTINCT Location) FROM forum_thread WHERE ForumType='%u' AND Location='%ld'",
                     (unsigned) ForumType,DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         if (CrsCod < 0)
           {
            if (DegCod < 0)	// CrsCod < 0 && DegCod < 0 ==> Number of course forums for all the courses
               sprintf (Query,"SELECT COUNT(DISTINCT Location) FROM forum_thread WHERE ForumType='%u'",
                        (unsigned) ForumType);
            else		// CrsCod < 0 && DegCod >= 0 ==> Number of course forums for a degree
               sprintf (Query,"SELECT COUNT(DISTINCT Location) FROM forum_thread WHERE ForumType='%u' AND Location IN (SELECT DISTINCT CrsCod FROM courses WHERE DegCod='%ld')",
                        (unsigned) ForumType,DegCod);
           }
         else			// CrsCod >= 0 ==> 0 <= number of course forums for a course <= 1
           sprintf (Query,"SELECT COUNT(DISTINCT Location) FROM forum_thread WHERE ForumType='%u' AND Location='%ld'",
                     (unsigned) ForumType,CrsCod);
         break;
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of forums of a type");

   /* Get row with number of threads and number of messages */
   row = mysql_fetch_row (mysql_res);

   /* Get number of threads (row[0]) */
   if (sscanf (row[0],"%u",&NumForums) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of forums of a type.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumForums;
  }

/*****************************************************************************/
/*********** Get total number of threads in forums of this type **************/
/*****************************************************************************/

unsigned For_GetNumTotalThrsInForumsOfType (For_ForumType_t ForumType,long DegCod,long CrsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumThrs;

   /***** Get total number of threads in forums of this type from database *****/
   switch (ForumType)
     {
      case For_FORUM_SWAD_USRS:		case For_FORUM_SWAD_TCHS:
      case For_FORUM_GLOBAL_USRS:		case For_FORUM_GLOBAL_TCHS:
      case For_FORUM_CENTRE_USRS:		case For_FORUM_CENTRE_TCHS:
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         // Total number of threads in forums of this type
         sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u'",
                  (unsigned) ForumType);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         if (DegCod < 0)	// Number of threads in degree forums for all the degrees
            sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u'",
                     (unsigned) ForumType);
         else			// DegCod >= 0 ==> number of threads in degree forums for a degree
            sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u' AND Location='%ld'",
                     (unsigned) ForumType,DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         if (CrsCod < 0)
           {
            if (DegCod < 0)	// CrsCod < 0 && DegCod < 0 ==> Number of threads in course forums for all the courses
               sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u'",
                        (unsigned) ForumType);
            else		// CrsCod < 0 && DegCod >= 0 ==> Number of threads in course forums for a degree
               sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u' AND Location IN (SELECT DISTINCT CrsCod FROM courses WHERE DegCod='%ld')",
                        (unsigned) ForumType,DegCod);
           }
         else			// CrsCod >= 0 ==> 0 <= number of threads in course forums for a course
            sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u' AND Location='%ld'",
                     (unsigned) ForumType,CrsCod);
         break;
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of threads in forums of a type");

   /* Get row with number of threads and number of messages */
   row = mysql_fetch_row (mysql_res);

   /* Get number of threads (row[0]) */
   if (sscanf (row[0],"%u",&NumThrs) != 1)
      Lay_ShowErrorAndExit ("Internal error while getting the number of threads in forums of a type.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumThrs;
  }

/*****************************************************************************/
/******************* Get number of threads in a forum ************************/
/*****************************************************************************/

unsigned For_GetNumThrsInForum (For_ForumType_t ForumType)
  {
   char SubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumThrs;

   /***** Get number of threads in a forum from database *****/
   switch (ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
         SubQuery[0] = '\0';
     }
   sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ForumType='%u'%s",
            (unsigned) ForumType,SubQuery);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of threads in a forum");

   /* Get row with number of threads and number of messages */
   row = mysql_fetch_row (mysql_res);

   /* Get number of threads (row[0]) */
   if (sscanf (row[0],"%u",&NumThrs) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of threads in a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumThrs;
  }

/*****************************************************************************/
/********************* Get total number of posts in forums of a type ************************/
/*****************************************************************************/

unsigned For_GetNumTotalPstsInForumsOfType (For_ForumType_t ForumType,long DegCod,long CrsCod,unsigned *NumUsrsToBeNotified)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPosts;

   /***** Get total number of posts in forums of this type from database *****/
   switch (ForumType)
     {
      case For_FORUM_SWAD_USRS:		case For_FORUM_SWAD_TCHS:
      case For_FORUM_GLOBAL_USRS:		case For_FORUM_GLOBAL_TCHS:
      case For_FORUM_CENTRE_USRS:		case For_FORUM_CENTRE_TCHS:
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         // Total number of posts in forums of this type
         sprintf (Query,"SELECT COUNT(*),SUM(forum_post.NumNotif) FROM forum_thread,forum_post " \
                        " WHERE forum_thread.ForumType='%u' AND forum_thread.ThrCod=forum_post.ThrCod",
                  (unsigned) ForumType);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         if (DegCod < 0)	// Number of threads in degree forums for all the degrees
            sprintf (Query,"SELECT COUNT(*),SUM(forum_post.NumNotif) FROM forum_thread,forum_post " \
                           " WHERE forum_thread.ForumType='%u' AND forum_thread.ThrCod=forum_post.ThrCod",
                     (unsigned) ForumType);
         else			// DegCod >= 0 ==> number of threads in degree forums for a degree
            sprintf (Query,"SELECT COUNT(*),SUM(forum_post.NumNotif) FROM forum_thread,forum_post " \
                           " WHERE forum_thread.ForumType='%u' AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_post.ThrCod",
                     (unsigned) ForumType,DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         if (CrsCod < 0)
           {
            if (DegCod < 0)	// CrsCod < 0 && DegCod < 0 ==> Number of threads in course forums for all the courses
               sprintf (Query,"SELECT COUNT(*),SUM(forum_post.NumNotif) FROM forum_thread,forum_post " \
                              " WHERE forum_thread.ForumType='%u' AND forum_thread.ThrCod=forum_post.ThrCod",
                        (unsigned) ForumType);
            else		// CrsCod < 0 && DegCod >= 0 ==> Number of threads in course forums for a degree
               sprintf (Query,"SELECT COUNT(*),SUM(forum_post.NumNotif) FROM forum_thread,forum_post " \
                              " WHERE forum_thread.ForumType='%u' AND forum_thread.Location IN (SELECT DISTINCT CrsCod FROM courses WHERE DegCod='%ld') AND forum_thread.ThrCod=forum_post.ThrCod",
                        (unsigned) ForumType,DegCod);
           }
         else			// CrsCod >= 0 ==> 0 <= number of threads in course forums for a course
           sprintf (Query,"SELECT COUNT(*),SUM(forum_post.NumNotif) FROM forum_thread,forum_post " \
                          " WHERE forum_thread.ForumType='%u' AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_post.ThrCod",
                     (unsigned) ForumType,CrsCod);
         break;
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get the total number of forums of a type");

   /* Get row with number of messages */
   row = mysql_fetch_row (mysql_res);

   /* Get number of messages (row[0]) */
   if (sscanf (row[0],"%u",&NumPosts) != 1)
      Lay_ShowErrorAndExit ("Error when getting the total number of forums of a type.");

   /* Get number of users notified (row[1]) */
   if (row[1])
     {
      if (sscanf (row[1],"%u",NumUsrsToBeNotified) != 1)
         Lay_ShowErrorAndExit ("Error when getting the total number of forums of a type.");
     }
   else
      *NumUsrsToBeNotified = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumPosts;
  }

/*****************************************************************************/
/********************* Get number of posts in a forum ************************/
/*****************************************************************************/

unsigned For_GetNumPstsInForum (For_ForumType_t ForumType)
  {
   char SubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumPosts;

   /***** Get number of posts in a forum from database *****/
   switch (ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
         SubQuery[0] = '\0';
     }
   sprintf (Query,"SELECT COUNT(*) FROM forum_thread,forum_post " \
                  " WHERE forum_thread.ForumType='%u'%s AND forum_thread.ThrCod=forum_post.ThrCod",
            (unsigned) ForumType,SubQuery);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of posts in a forum");

   /* Get row with number of messages */
   row = mysql_fetch_row (mysql_res);

   /* Get number of messages (row[0]) */
   if (sscanf (row[0],"%u",&NumPosts) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of posts in a forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumPosts;
  }

/*****************************************************************************/
/************************ List the threads of a forum ************************/
/*****************************************************************************/

void For_ListForumThrs (unsigned NumThrs,long ThrCods[Pag_ITEMS_PER_PAGE],struct Pagination *PaginationThrs)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *The_ClassFormulB[The_NUM_THEMES];
   extern const char *Txt_post[Txt_NUM_LANGUAGES];
   extern const char *Txt_posts[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_thread[Txt_NUM_LANGUAGES];
   unsigned NumThr;
   unsigned NumThrInScreen;	// From 0 to Pag_ITEMS_PER_PAGE-1
   struct ForumThread Thr;
   struct UsrData UsrDat;
   For_ForumOrderType_t Order;
   const char *DateTime;
   struct Pagination PaginationPsts;
   const char *Style;
   bool ButtonsDelCut = false;
   bool ICanMoveThreads = (Gbl.Usrs.Me.UsrTypeForumsChat == Usr_SUPERUSER);	// If I have permission to move threads...
   long ThreadInMyClipboard = -1L;
   bool ThisThreadIsInMyClipboard;
   unsigned NumMyPostsInThr;

   /***** Get if there is a thread ready to be moved *****/
   if (ICanMoveThreads)
      ThreadInMyClipboard = For_GetThrInMyClipboard ();

   /***** Initialize structure with user's data *****/
   Usr_UsrDataConstructor (&UsrDat);

   for (NumThr=PaginationThrs->FirstItemVisible, NumThrInScreen=0; NumThr<=PaginationThrs->LastItemVisible; NumThr++)
     {
      /***** Get the data of this thread *****/
      Thr.ThrCod = ThrCods[NumThrInScreen++];
      For_GetThrData (&Thr);
      Style = (Thr.NumUnreadPosts ? "MSG_AUT_NEW" : "MSG_AUT");
      ThisThreadIsInMyClipboard = (Thr.ThrCod == ThreadInMyClipboard);

      /***** Show my photo if I have any posts in this thread *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" width=\"14\" bgcolor=\"#%s\">",
               ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2]);
      if ((NumMyPostsInThr = For_GetNumMyPstInThr (Thr.ThrCod)) != 0)
        {
         fprintf (Gbl.F.Out,"<acronym title=\"Usted ha escrito ");
         if (NumMyPostsInThr == 1)
            fprintf (Gbl.F.Out,"1 %s",Txt_post[Gbl.Prefs.Language]);
         else
            fprintf (Gbl.F.Out,"%u %s",NumMyPostsInThr,Txt_posts[Gbl.Prefs.Language]);
         fprintf (Gbl.F.Out," en esta hebra\"><img src=\"");	// Need translation!
         if (Gbl.Usrs.Me.PhotoURL[0])	// If I have photo
            fprintf (Gbl.F.Out,"%s",Gbl.Usrs.Me.PhotoURL);
         else
            fprintf (Gbl.F.Out,"%s/usr_bl.jpg",Gbl.Prefs.IconsURL);
         fprintf (Gbl.F.Out,"\" class=\"F12x16\" alt=\"Hebra con mensajes suyos\" /></acronym>");	// Need translation
        }
      fprintf (Gbl.F.Out,"</td>");

      /***** Put an icon with thread status *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" width=\"24\" bgcolor=\"#%s\">",
               ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2]);

      fprintf (Gbl.F.Out,"<img src=\"%s/",Gbl.Prefs.IconsURL);
      if (Thr.NumUnreadPosts)
         fprintf (Gbl.F.Out,"threadnew16x16.gif\" alt=\"Hay mensajes nuevos");
      else
         fprintf (Gbl.F.Out,"thread16x16.gif\" alt=\"No hay mensajes nuevos");
      fprintf (Gbl.F.Out,"\" width=\"16\" height=\"16\" />");

      if (PermissionThreadDeletion[Gbl.Forum.ForumType] & (1 << Gbl.Usrs.Me.UsrTypeForumsChat)) // If I have permission to remove thread in this forum...
        {
         fprintf (Gbl.F.Out,"<br /><table cellspacing=\"0\" cellpadding=\"0\"><tr>");
         ButtonsDelCut = true;

         /* Put button to remove the thread */
         fprintf (Gbl.F.Out,"<td class=\"BT\">");
         Act_FormStart (ActionsReqDelThr[Gbl.Forum.ForumType]);
         For_PutAllHiddenParamsForum ();
         For_PutHiddenParamThrCod (Thr.ThrCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_thread[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
                  Gbl.Prefs.IconsURL,Txt_Remove_thread[Gbl.Prefs.Language]);
        }

      if (ICanMoveThreads)
        {
         if (!ButtonsDelCut)
           {
            fprintf (Gbl.F.Out,"<br /><table cellspacing=\"0\" cellpadding=\"0\"><tr>");
            ButtonsDelCut = true;
           }

         /* Put button to cut the thread for moving it to another forum */
         fprintf (Gbl.F.Out,"<td class=\"BT\">");
         Act_FormStart (ActionsCutThrFor[Gbl.Forum.ForumType]);
         Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
         For_PutAllHiddenParamsForum ();
         For_PutHiddenParamThrCod (Thr.ThrCod);
         sprintf (Gbl.MsgWindowStatus,"Mover hebra");	// Need translation!
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/cut12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
        }

      if (ButtonsDelCut)
         fprintf (Gbl.F.Out,"</tr></table>");

      fprintf (Gbl.F.Out,"</td>");

      /***** Write subject and links to thread pages *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\">",
               ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2]);
      PaginationPsts.NumItems = Thr.NumPosts;
      PaginationPsts.CurrentPage = 1;	// First page
      Pag_CalculatePagination (&PaginationPsts);
      Str_LimitLengthHTMLStr (Thr.Subject,80);
      Pag_WriteLinksToPages (Pag_POSTS_FORUM,Thr.ThrCod,&PaginationPsts,
                             Thr.Enabled[For_FIRST_MSG],
                             Thr.Subject,
                             Thr.NumUnreadPosts ? The_ClassFormulB[Gbl.Prefs.Theme] : The_ClassFormul[Gbl.Prefs.Theme],
                             true);
      fprintf (Gbl.F.Out,"</td>");

      /***** Write the authors and dates of first and last messages *****/
      for (Order=For_FIRST_MSG; Order<=For_LAST_MSG; Order++)
        {
         /* Write the author of first or last message */
         UsrDat.UsrCod = Thr.UsrCod[Order];
         Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat);
         Msg_WriteMsgAuthor (&UsrDat,55,9,Style,Style,Thr.Enabled[Order],ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2]);

         /* Write the date of first or last message (it's in YYYYMMDDHHMMSS format) */
         fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\" bgcolor=\"#%s\">",
                  Style,ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2]);
         DateTime = Thr.WriteTime[Order];
         Dat_WriteDate (DateTime);
         fprintf (Gbl.F.Out,"<br />");
         Dat_WriteHourMinute (&DateTime[8]);
         fprintf (Gbl.F.Out,"</td>");
        }

      /***** Write number of posts in this thread *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\" bgcolor=\"#%s\">%u&nbsp;</td>",
               Style,ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2],Thr.NumPosts);

      /***** Write number of new posts in this thread *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\" bgcolor=\"#%s\">%u&nbsp;</td>",
               Style,ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2],Thr.NumUnreadPosts);

      /***** Write number of users who have write posts in this thread *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\" bgcolor=\"#%s\">%u&nbsp;</td>",
               Style,ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2],Thr.NumWriters);

      /***** Write number of users who have read this thread *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\" bgcolor=\"#%s\">%u&nbsp;</td></tr>",
               Style,ThisThreadIsInMyClipboard ? LIGHT_GREEN : Gbl.ColorRows[NumThrInScreen % 2],Thr.NumReaders);
     }

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDat);
  }

/*****************************************************************************/
/***************************** Get data of a thread **************************/
/*****************************************************************************/

void For_GetThrData (struct ForumThread *Thr)
  {
   extern const char *Txt_no_subject[Txt_NUM_LANGUAGES];
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   For_ForumOrderType_t Order;

   /***** Get data of a thread from database *****/
   sprintf (Query,"SELECT m0.PstCod,m1.PstCod,m0.UsrCod,m1.UsrCod," \
                  "DATE_FORMAT(m0.CreatTime,'%%Y%%m%%d%%H%%i%%S'),DATE_FORMAT(m1.CreatTime,'%%Y%%m%%d%%H%%i%%S')," \
                  "m0.Subject" \
                  " FROM forum_thread,forum_post AS m0,forum_post AS m1" \
                  " WHERE forum_thread.ThrCod='%ld' AND forum_thread.FirstPstCod=m0.PstCod AND forum_thread.LastPstCod=m1.PstCod",
            Thr->ThrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a thread of a forum");

   /***** The result of the query should have one row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting data of a thread of a forum.");
   row = mysql_fetch_row (mysql_res);

   /***** Get the code of the first message in this thread (row[0]) *****/
   if (sscanf (row[0],"%ld",&(Thr->PstCod[For_FIRST_MSG])) != 1)
      Lay_ShowErrorAndExit ("Wrong code of post.");

   /***** Get the code of the last message in this thread (row[1]) *****/
   if (sscanf (row[1],"%ld",&(Thr->PstCod[For_LAST_MSG])) != 1)
      Lay_ShowErrorAndExit ("Wrong code of post.");

   /***** Get the author of the first message in this thread (row[2]) *****/
   Thr->UsrCod[For_FIRST_MSG] = Str_ConvertStrCodToLongCod (row[2]);

   /***** Get the author of the last message in this thread (row[3]) *****/
   Thr->UsrCod[For_LAST_MSG] = Str_ConvertStrCodToLongCod (row[3]);

   /***** Get the date of the first message in this thread (row[4]) *****/
   strncpy (Thr->WriteTime[For_FIRST_MSG],row[4],Cte_MAX_BYTES_DATE);
   Thr->WriteTime[For_FIRST_MSG][Cte_MAX_BYTES_DATE] = '\0';

   /***** Get the date of the last message in this thread (row[5]) *****/
   strncpy (Thr->WriteTime[For_LAST_MSG],row[5],Cte_MAX_BYTES_DATE);
   Thr->WriteTime[For_LAST_MSG][Cte_MAX_BYTES_DATE] = '\0';

   /***** Get the subject of this thread (row[6]) *****/
   strncpy (Thr->Subject,row[6],Cte_MAX_BYTES_SUBJECT);
   Thr->Subject[Cte_MAX_BYTES_SUBJECT] = '\0';
   if (!Thr->Subject[0])
      sprintf (Thr->Subject,"[%s]",Txt_no_subject[Gbl.Prefs.Language]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Get if first or last message are enabled *****/
   for (Order=For_FIRST_MSG; Order<=For_LAST_MSG; Order++)
      Thr->Enabled[Order] = For_GetIfPstIsEnabled (Thr->PstCod[Order]);

   /***** Get number of posts in this thread *****/
   Thr->NumPosts = For_GetNumPstsInThr (Thr->ThrCod);

   /***** Get number of unread posts in this thread *****/
   Thr->NumUnreadPosts = For_GetNumOfUnreadPostsInThr (Thr->ThrCod,Thr->NumPosts);

   /***** Get number of users who have write posts in this thread *****/
   Thr->NumWriters = For_GetNumOfWritersInThr (Thr->ThrCod);

   /***** Get number of users who have read this thread *****/
   Thr->NumReaders = For_GetNumOfReadersOfThr (Thr->ThrCod);
  }

/*****************************************************************************/
/************* Write the subject of the first message of a thread ************/
/*****************************************************************************/

void For_WriteThrSubject (long ThrCod)
  {
   extern const char *Txt_no_subject[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long FirstPstCod;

   /***** Get subject of a thread from database *****/
   sprintf (Query,"SELECT forum_post.PstCod,forum_post.Subject FROM forum_thread,forum_post" \
                  " WHERE forum_thread.ThrCod='%ld' AND forum_thread.FirstPstCod=forum_post.PstCod",
            ThrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the subject of a thread");

   /***** The result of the query should have only one row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting the subject of a thread.");
   row = mysql_fetch_row (mysql_res);

   /***** Get message code *****/
   if (sscanf (row[0],"%ld",&FirstPstCod) != 1)
      Lay_ShowErrorAndExit ("Error when getting the subject of a thread.");

   /***** Write the subject of the thread *****/
   if (For_GetIfPstIsEnabled (FirstPstCod))
     {
      if (row[1][0])
         fprintf (Gbl.F.Out,"%s",row[1]);
      else
         fprintf (Gbl.F.Out,"[%s]",Txt_no_subject[Gbl.Prefs.Language]);
     }
   else
      fprintf (Gbl.F.Out,"[primer mensaje no permitido]");

   /***** Free the structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************** Show messages of a thread in a discussion forum **************/
/*****************************************************************************/

void For_ShowThrPsts (void)
  {
   long ThrCod;

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Get the identifier of the thread that user want to show *****/
   ThrCod = For_GetParamThrCod ();

   /***** Show the messages of that thread *****/
   For_ShowForumLevel2 (ThrCod);
  }

/*****************************************************************************/
/********************* Get parameters related to a forum *********************/
/*****************************************************************************/

void For_GetParamsForum (void)
  {
   char YN[1+1];
   char UnsignedStr[10+1];
   unsigned UnsignedNum;
   char LongStr[1+10+1];

   /***** Get which forums I want to see *****/
   Par_GetParam (Par_PARAM_SINGLE,"AllMyForums",YN,1);
   Gbl.Forum.WhichForums = (Str_ConvertToUpperLetter (YN[0]) == 'Y') ? For_ALL_MY_FORUMS : For_ONLY_FORUMS_CURRENT_INS_DEG_CRS;

   /***** Get order type *****/
   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Forum.SelectedOrderType = (For_ForumOrderType_t) UnsignedNum;
   else
      Lay_ShowErrorAndExit ("Sort order is missing.");

   /***** Get parameter with code of institution *****/
   Par_GetParam (Par_PARAM_SINGLE,"ForInsCod",LongStr,1+10);
   Gbl.Forum.Ins.InsCod = Str_ConvertStrCodToLongCod (LongStr);
   Ins_GetDataOfInstitutionByCod (&Gbl.Forum.Ins);

   /***** Get parameter with code of degree *****/
   Par_GetParam (Par_PARAM_SINGLE,"ForDegCod",LongStr,1+10);
   Gbl.Forum.Deg.DegCod = Str_ConvertStrCodToLongCod (LongStr);
   Deg_GetDataOfDegreeByCod (&Gbl.Forum.Deg);

   /***** Get parameter with code of course *****/
   Par_GetParam (Par_PARAM_SINGLE,"ForCrsCod",LongStr,1+10);
   Gbl.Forum.Crs.CrsCod = Str_ConvertStrCodToLongCod (LongStr);
   Deg_GetDataOfCourseByCod (&Gbl.Forum.Crs);
  }

/*****************************************************************************/
/************ Write a form parameter to specify a thread code ****************/
/*****************************************************************************/

void For_PutHiddenParamThrCod (long ThrCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ThrCod\" value=\"%ld\" />",ThrCod);
  }

/*****************************************************************************/
/********* Get the value of the parameter with the code of a thread **********/
/*****************************************************************************/

static long For_GetParamThrCod (void)
  {
   char StrThrCod[1+10+1];	// String that holds the thread code
   long ThrCod;

   /* Get thread code */
   Par_GetParam (Par_PARAM_SINGLE,"ThrCod",StrThrCod,1+10);
   if (sscanf (StrThrCod,"%ld",&ThrCod) != 1)
      Lay_ShowErrorAndExit ("Wrong thread code.");

   return ThrCod;
  }

/*****************************************************************************/
/************* Write a form parameter to specify a post code *****************/
/*****************************************************************************/

static void For_PutHiddenParamPstCod (long PstCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"PstCod\" value=\"%ld\" />",PstCod);
  }

/*****************************************************************************/
/********** Get the value of the parameter with the code of a post ***********/
/*****************************************************************************/

static long For_GetParamPstCod (void)
  {
   char StrPstCod[1+10+1];	// String that holds the post code
   long PstCod;

   /* Get post code */
   Par_GetParam (Par_PARAM_SINGLE,"PstCod",StrPstCod,1+10);
   if (sscanf (StrPstCod,"%ld",&PstCod) != 1)
      Lay_ShowErrorAndExit ("Wrong post code.");

   return PstCod;
  }

/*****************************************************************************/
/************ Show messages of a thread of the forum of discusin **********/
/*****************************************************************************/

void For_ShowForumLevel2 (long ThrCod)
  {
   char Subject[Cte_MAX_BYTES_SUBJECT+1];

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Show the messages of this thread *****/
   For_ShowThreadPosts (ThrCod,Subject);

   /***** Form to write a new message in the thread *****/
   For_WriteFormForumPst (true,ThrCod,Subject);
  }

/*****************************************************************************/
/********************** Show an area to write a message **********************/
/*****************************************************************************/

void For_WriteFormForumPst (bool IsReply,long ThrCod,const char *Subject)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_message[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_thread[Txt_NUM_LANGUAGES];
   extern const char *Txt_If_you_send_this_message_you_will_appear_as_its_author[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_Subject[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_Message[Txt_NUM_LANGUAGES];
   extern const char *Txt_Send_message[Txt_NUM_LANGUAGES];

   if (IsReply)	// Form to write a reply to a message of an existing thread
     {
      Act_FormStart (ActionsRecRepFor[Gbl.Forum.ForumType]);
      For_PutHiddenParamThrCod (ThrCod);
      Pag_PutHiddenParamPagNum (0);
     }
   else		// Form to write the first message of a new thread
      Act_FormStart (ActionsRecThrFor[Gbl.Forum.ForumType]);
   For_PutAllHiddenParamsForum ();

   fprintf (Gbl.F.Out,"<div align=\"center\"><table>\n<tr><td align=\"left\"></td>" \
                      "<td align=\"left\"><span class=\"%s\">%s</span>" \
                      "<span class=\"DAT\"> (%s)</span></td>\n",
            The_ClassFormul[Gbl.Prefs.Theme],IsReply ? Txt_New_message[Gbl.Prefs.Language] : Txt_New_thread[Gbl.Prefs.Language],
            Txt_If_you_send_this_message_you_will_appear_as_its_author[Gbl.Prefs.Language]);

   fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"2\">");
   Msg_WriteLinkToNetiquette ();
   fprintf (Gbl.F.Out,"</td></tr>");

   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\"><textarea name=\"Subject\" cols=\"75\" rows=\"2\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_MSG_Subject[Gbl.Prefs.Language]);
   if (IsReply)	// If writing a reply to a message of an existing thread
      fprintf (Gbl.F.Out,Subject);
   fprintf (Gbl.F.Out,"</textarea></td></tr><tr><td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\"><textarea name=\"Txt\" cols=\"75\" rows=\"15\"></textarea></td>" \
                      "</tr></table></div>\n",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_MSG_Message[Gbl.Prefs.Language]);

   Lay_PutSendAndUndoButtons (Txt_Send_message[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/*****************************************************************************/
/************************** Receive message of a forum ***********************/
/*****************************************************************************/

void For_RecForumPst (void)
  {
   bool PstIsAReply = false;
   long ThrCod = 0;
   long PstCod = 0;
   unsigned NumUsrsToBeNotified;
   long DegCod;
   long CrsCod;

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get the code of the thread y the nmero of page *****/
   if (Gbl.CurrentAct == ActRecRepForCrsUsr || Gbl.CurrentAct == ActRecRepForCrsTch ||
       Gbl.CurrentAct == ActRecRepForDegUsr || Gbl.CurrentAct == ActRecRepForDegTch ||
       Gbl.CurrentAct == ActRecRepForCtrUsr || Gbl.CurrentAct == ActRecRepForCtrTch ||
       Gbl.CurrentAct == ActRecRepForInsUsr || Gbl.CurrentAct == ActRecRepForInsTch ||
       Gbl.CurrentAct == ActRecRepForGenUsr || Gbl.CurrentAct == ActRecRepForGenTch ||
       Gbl.CurrentAct == ActRecRepForSWAUsr || Gbl.CurrentAct == ActRecRepForSWATch)
     {
      PstIsAReply = true;

      /* Get the code of the thread */
      ThrCod = For_GetParamThrCod ();
     }

   /***** Get page number *****/
   Pag_GetParamPagNum (Pag_POSTS_FORUM);

   /***** Get message subject *****/
   Par_GetParam (Par_PARAM_SINGLE,"Subject",Gbl.Msg.Subject,Cte_MAX_BYTES_SUBJECT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.Msg.Subject,Cte_MAX_BYTES_SUBJECT,true);

   /***** Get message body *****/
   Str_AllocateAndGetText (Str_TO_RIGOROUS_HTML,false);

   /***** Create a new message *****/
   if (PstIsAReply)
     {
      // ThrCod has been received from form

      /***** Create last message of the thread *****/
      PstCod = For_InsertForumPst (ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Msg.Subject,Gbl.Txt);

      /***** Modify last message of the thread *****/
      For_UpdateThrLastPst (ThrCod,PstCod);
     }
   else	// New thread
     {
      /***** Create new thread with unknown first and last message codes *****/
      ThrCod = For_InsertForumThread (Gbl.Forum.ForumType,-1L);

      /***** Create first (and last) message of the thread *****/
      PstCod = For_InsertForumPst (ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Msg.Subject,Gbl.Txt);

      /***** Update first and last messages of new thread *****/
      For_UpdateThrFirstAndLastPst (ThrCod,PstCod,PstCod);
     }

   /***** Free memory used by the body of the message *****/
   Str_FreeTxt ();

   /***** Write message of success *****/
   Lay_ShowAlert (INFO,"Mensaje enviado." \
                       " Se recomienda que no recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a" \
                       " de nuevo y llegar&iacute;a repetido.");	// Need translation!

   /***** Notify the new post to previous writers in this thread *****/
   if (PstIsAReply)
     {
      if ((NumUsrsToBeNotified = Ntf_StoreNotifyEventsToAllUsrs (Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS,PstCod,ThrCod)))
        {
         For_UpdateNumUsrsNotifiedAboutPost (PstCod,NumUsrsToBeNotified);

         switch (Gbl.Forum.ForumType)
           {
            case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
               DegCod = Gbl.Forum.Deg.DegCod;
               CrsCod = -1L;
               break;
            case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
               CrsCod = Gbl.Forum.Crs.CrsCod;
               DegCod = Gbl.Forum.Crs.DegCod;
               break;
            default:
               DegCod = CrsCod = -1L;
               break;
           }
        }
      Ntf_ShowAlertNumUsrsToBeNotified (NumUsrsToBeNotified);
     }

   /***** Show again the messages of this thread of the forum *****/
   For_ShowForumLevel2 (ThrCod);
  }

/****************************************************************************/
/******** Update number of users notified in table of forum messages ********/
/****************************************************************************/

static void For_UpdateNumUsrsNotifiedAboutPost (long PstCod,unsigned NumUsrsToBeNotified)
  {
   char Query[512];

   /***** Update number of users notified *****/
   sprintf (Query,"UPDATE forum_post SET NumNotif='%u' WHERE PstCod='%ld'",
            NumUsrsToBeNotified,PstCod);
   DB_QueryUPDATE (Query,"can not update the number of notifications of a post");
  }

/*****************************************************************************/
/***************************** Delete a forum post ***************************/
/*****************************************************************************/

void For_DelPst (void)
  {
   extern const char *Txt_Post_and_thread_removed[Txt_NUM_LANGUAGES];
   extern const char *Txt_Post_removed[Txt_NUM_LANGUAGES];
   long PstCod,ThrCod;
   struct UsrData UsrDat;
   char CreatTime[4+2+2+2+2+2+1];	// Creation time of a message in YYYYMMDDHHMMSS format
   char Subject[Cte_MAX_BYTES_SUBJECT+1];
   char OriginalContent[Cte_MAX_BYTES_TEXT+1];
   bool ThreadDeleted = false;

   /***** Get the code of the message to delete *****/
   PstCod = For_GetParamPstCod ();

   /***** Delete the message *****/
   /* Get order type, degree and course of the forum */
   For_GetParamsForum ();
   /* Set forum type */
   For_SetForumTypeAndRestrictAccess ();
   /* Get the thread number */
   ThrCod = For_GetParamThrCod ();

   /* Check if the message really exists, if it hasn't been removed */
   if (!For_GetIfForumPstExists (PstCod))
      Lay_ShowErrorAndExit ("The post to remove no longer exists.");

   /* Check if I am the author of the message */
   For_GetPstData (PstCod,&UsrDat.UsrCod,CreatTime,Subject,OriginalContent);
   if (Gbl.Usrs.Me.UsrDat.UsrCod != UsrDat.UsrCod)
      Lay_ShowErrorAndExit ("You can not remove post because you aren't the author.");

   /* Check if the message is the last message in the thread */
   if (PstCod != For_GetLastPstCod (ThrCod))
      Lay_ShowErrorAndExit ("You can not remove post because it isn't the last of the thread.");

   /* Remove the message */
   ThreadDeleted = For_RemoveForumPst (PstCod);

   /* Remove pending notification of this post */
   Ntf_DeletePendingNotif (Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS,PstCod);

   if (ThreadDeleted)
     {
      Lay_ShowAlert (INFO,Txt_Post_and_thread_removed[Gbl.Prefs.Language]);

      /* Show the remaining threads */
      For_ShowForumThrs ();
     }
   else
     {
      Lay_ShowAlert (INFO,Txt_Post_removed[Gbl.Prefs.Language]);

      /* Show the remaining messages */
      For_ShowForumLevel2 (ThrCod);
     }
  }

/*****************************************************************************/
/**************** Solicitar the eliminacin of a existing thread **************/
/*****************************************************************************/

void For_ReqDelThr (void)
  {
   long ThrCod;
   char Subject[Cte_MAX_BYTES_SUBJECT+1];

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Initialize the type of forum *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get the identifier of the thread to remove *****/
   ThrCod = For_GetParamThrCod ();

   /***** Get subject of thread to delete *****/
   For_GetThrSubject (ThrCod,Subject,Cte_MAX_BYTES_SUBJECT);

   /***** Request confirmation to remove the thread *****/
   if (Subject[0])
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar toda la hebra <strong>%s</strong>?",	// Need translation!
               Subject);
   else
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar toda la hebra?");	// Need translation!
   Lay_ShowAlert (WARNING,Gbl.Message);

   Act_FormStart (ActionsDelThrFor[Gbl.Forum.ForumType]);
   For_PutAllHiddenParamsForum ();
   For_PutHiddenParamThrCod (ThrCod);
   fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"S&iacute;, eliminar hebra\" /></div>\n");	// Need translation!
   fprintf (Gbl.F.Out,"</form>\n");
  }

/*****************************************************************************/
/*************************** Remove an existing thread ***********************/
/*****************************************************************************/

void For_DelThr (void)
  {
   extern const char *Txt_Thread_X_removed[Txt_NUM_LANGUAGES];
   extern const char *Txt_Thread_removed[Txt_NUM_LANGUAGES];
   long ThrCod;
   char Subject[Cte_MAX_BYTES_SUBJECT+1];

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   if (PermissionThreadDeletion[Gbl.Forum.ForumType] & (1 << Gbl.Usrs.Me.UsrTypeForumsChat)) // If I have permission to remove thread in this forum...
     {
      /***** Get code of thread to delete *****/
      ThrCod = For_GetParamThrCod ();

      /***** Get subject of thread to delete *****/
      For_GetThrSubject (ThrCod,Subject,Cte_MAX_BYTES_SUBJECT);

      /***** Remove the thread and all its posts *****/
      For_RemoveThreadAndItsPsts (ThrCod);

      /***** Write message confirming the deletion *****/
      if (Subject[0])
      	{
         sprintf (Gbl.Message,Txt_Thread_X_removed[Gbl.Prefs.Language],
                  Subject);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
      else
         Lay_ShowAlert (INFO,Txt_Thread_removed[Gbl.Prefs.Language]);
     }
   else
      Lay_ShowErrorAndExit ("You can not remove threads in this forum.");

   /***** Show the threads again *****/
   For_ShowForumThrs ();
  }

/*****************************************************************************/
/*************** Cut a thread to move it to another forum ********************/
/*****************************************************************************/

void For_CutThr (void)
  {
   long ThrCod;
   char Subject[Cte_MAX_BYTES_SUBJECT+1];

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get code of thread to cut *****/
   ThrCod = For_GetParamThrCod ();

   /***** Get subject of thread to cut *****/
   For_GetThrSubject (ThrCod,Subject,Cte_MAX_BYTES_SUBJECT);

   /***** Mark the thread as cut *****/
   For_InsertThrInClipboard (ThrCod);

   /***** Write message confirming that thread has been marked to move it *****/
   if (Subject[0])
      sprintf (Gbl.Message,"Hebra <strong>%s</strong> marcada para moverla.",	// Need translation!
               Subject);
   else
      sprintf (Gbl.Message,"Hebra marcada para moverla.");	// Need translation!
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the threads again *****/
   For_ShowForumThrs ();
  }

/*****************************************************************************/
/************* Paste the thread in clipboard into current forum **************/
/*****************************************************************************/

void For_PasteThr (void)
  {
   long ThrCod;
   char Subject[Cte_MAX_BYTES_SUBJECT+1];

   /***** Get order type, degree and course of the forum *****/
   For_GetParamsForum ();

   /***** Set forum type *****/
   For_SetForumTypeAndRestrictAccess ();

   /***** Get code of thread to paste *****/
   ThrCod = For_GetParamThrCod ();

   /***** Get subject of thread to paste *****/
   For_GetThrSubject (ThrCod,Subject,Cte_MAX_BYTES_SUBJECT);

   /***** Paste (move) the thread to current forum *****/
   if (For_CheckIfThrBelongsToForum (ThrCod,Gbl.Forum.ForumType))
     {
      if (Subject[0])
         sprintf (Gbl.Message,"La hebra <strong>%s</strong> ya est&aacute; en este foro.",	// Need translation!
                  Subject);
      else
         sprintf (Gbl.Message,"La hebra ya est&aacute; en este foro.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      For_MoveThrToCurrentForum (ThrCod);
      if (Subject[0])
         sprintf (Gbl.Message,"Hebra <strong>%s</strong> movida a este foro.",	// Need translation!
                  Subject);
      else
         sprintf (Gbl.Message,"Hebra movida a este foro.");	// Need translation!
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the threads again *****/
   For_ShowForumThrs ();
  }

/****************************************************************************/
/**************** Get if there is a thread ready to be moved ****************/
/****************************************************************************/

long For_GetThrInMyClipboard (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long ThrCod = -1L;;

   /***** Get if there is a thread ready to move in my clipboard from database *****/
   sprintf (Query,"SELECT ThrCod FROM forum_thr_clip WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not check if there is any thread ready to be moved");

   if (NumRows == 1)
     {
      /* Get thread code */
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%ld",&ThrCod) != 1)
         Lay_ShowErrorAndExit ("Error when checking if there is any thread ready to be moved.");
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return ThrCod;
  }

/****************************************************************************/
/**************** Get if a thread belongs to current forum ******************/
/****************************************************************************/

bool For_CheckIfThrBelongsToForum (long ThrCod,For_ForumType_t ForumType)
  {
   char SubQuery[256];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a thread belong to current forum from database *****/
   switch (ForumType)
     {
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Ins.InsCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Deg.DegCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (SubQuery," AND forum_thread.Location='%ld'",Gbl.Forum.Crs.CrsCod);
         break;
      default:
         SubQuery[0] = '\0';
     }
   sprintf (Query,"SELECT COUNT(*) FROM forum_thread WHERE ThrCod='%ld' AND ForumType='%u'%s",
            ThrCod,(unsigned) ForumType,SubQuery);
   DB_QuerySELECT (Query,&mysql_res,"can not get if a thread belong to current forum");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when getting if a thread belong to current forum.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/*****************************************************************************/
/************************ Move a thread to current forum *********************/
/*****************************************************************************/

void For_MoveThrToCurrentForum (long ThrCod)
  {
   char Query[512];

   /***** Move a thread to current forum *****/
   switch (Gbl.Forum.ForumType)
     {
      case For_FORUM_SWAD_USRS:		case For_FORUM_SWAD_TCHS:
      case For_FORUM_GLOBAL_USRS:		case For_FORUM_GLOBAL_TCHS:
      case For_FORUM_CENTRE_USRS:		case For_FORUM_CENTRE_TCHS:
         sprintf (Query,"UPDATE forum_thread SET ForumType='%u',Location='-1' WHERE ThrCod='%ld'",
                  (unsigned) Gbl.Forum.ForumType,ThrCod);
         break;
      case For_FORUM_INSTITUTION_USRS:	case For_FORUM_INSTITUTION_TCHS:
         sprintf (Query,"UPDATE forum_thread SET ForumType='%u',Location='%ld' WHERE ThrCod='%ld'",
                  (unsigned) Gbl.Forum.ForumType,Gbl.Forum.Ins.InsCod,ThrCod);
         break;
      case For_FORUM_DEGREE_USRS:		case For_FORUM_DEGREE_TCHS:
         sprintf (Query,"UPDATE forum_thread SET ForumType='%u',Location='%ld' WHERE ThrCod='%ld'",
                  (unsigned) Gbl.Forum.ForumType,Gbl.Forum.Deg.DegCod,ThrCod);
         break;
      case For_FORUM_COURSE_USRS:		case For_FORUM_COURSE_TCHS:
         sprintf (Query,"UPDATE forum_thread SET ForumType='%u',Location='%ld' WHERE ThrCod='%ld'",
                  (unsigned) Gbl.Forum.ForumType,Gbl.Forum.Crs.CrsCod,ThrCod);
         break;
     }
   DB_QueryUPDATE (Query,"can not move a thread to current forum");
  }

/****************************************************************************/
/********************* Insert thread in thread clipboard ********************/
/****************************************************************************/

void For_InsertThrInClipboard (long ThrCod)
  {
   char Query[512];

   /***** Remove expired thread clipboards *****/
   For_RemoveExpiredThrsClipboards ();

   /***** Add thread to my clipboard *****/
   sprintf (Query,"REPLACE INTO forum_thr_clip (ThrCod,UsrCod) VALUES ('%ld','%ld')",
            ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryREPLACE (Query,"can not add thread to clipboard");
  }

/*****************************************************************************/
/************* Remove expired thread clipboards (from all users) *************/
/*****************************************************************************/

void For_RemoveExpiredThrsClipboards (void)
  {
   char Query[512];

   /***** Remove all expired clipboards *****/
   sprintf (Query,"DELETE LOW_PRIORITY FROM forum_thr_clip" \
                  " WHERE UNIX_TIMESTAMP() > UNIX_TIMESTAMP(TimeInsert)+%ld",
            Cfg_TIME_TO_DELETE_THREAD_CLIPBOARD);
   DB_QueryDELETE (Query,"can not remove old threads from clipboards");
  }

/*****************************************************************************/
/**************** Remove thread code from thread clipboard *******************/
/*****************************************************************************/

void For_RemoveThrCodFromThrClipboard (long ThrCod)
  {
   char Query[512];

   /***** Remove thread from thread clipboard *****/
   sprintf (Query,"DELETE FROM forum_thr_clip WHERE ThrCod='%ld'",ThrCod);
   DB_QueryDELETE (Query,"can not remove a thread from clipboard");
  }

/*****************************************************************************/
/********************* Remove thread clipboard of a user *********************/
/*****************************************************************************/

void For_RemoveUsrFromThrClipboard (long UsrCod)
  {
   char Query[512];

   /***** Remove clipboard of specified user *****/
   sprintf (Query,"DELETE FROM forum_thr_clip WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove a thread from the clipboard of a user");
  }
