// swad_gbl.c: global variables

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <locale.h>		// For setlocale
#include <stdlib.h>		// For exit, system, malloc, calloc, free, etc
#include <string.h>		// For string functions
#include <sys/time.h>		// For gettimeofday
#include <sys/types.h>		// For getpid
#include <unistd.h>		// For getpid

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_cns.h"
#include "swad_exa.h"
#include "swad_gbl.h"
#include "swad_prf.h"
#include "swad_the.h"

/****************************************************************************/
/****************************** Public variables ****************************/
/****************************************************************************/

struct Globals Gbl;	// All the global parameters and variables must be in this structure

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Actions Actions[Act_NUM_ACTIONS];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

/*****************************************************************************/
/************* Intialize globals variables when starting program *************/
/*****************************************************************************/

void Gbl_InitializeGlobals (void)
  {
   extern const char *The_ThemeId[The_NUM_THEMES];
   char UniqueNamePlain[Cte_MAX_LENGTH_IP+1+10+1+10+1];

   setlocale (LC_ALL,"es_ES.iso88591");

   gettimeofday (&Gbl.tvStart, &Gbl.tz);
   Dat_GetTimeStartExecution ();
   Dat_GetAndConvertCurrentDateTime ();
   Gbl.TimeGenerationInMicroseconds = -1L;
   Gbl.TimeSendInMicroseconds = -1L;
   Gbl.PID = getpid ();
   Sta_GetRemoteAddr ();
   sprintf (UniqueNamePlain,"%s-%lx-%x",Gbl.IP,Gbl.TimeStartExecution,Gbl.PID);
   Cry_EncryptSHA256Base64 (UniqueNamePlain,Gbl.UniqueNameEncrypted);	// Make difficult to guess a unique name
 
   srand ((unsigned int) Gbl.TimeStartExecution);	// Initialize seed for rand()

   Gbl.WebService = false;

   Gbl.F.Out = stdout;
   Gbl.F.Tmp = NULL;
   Gbl.F.Syllabus = NULL;

   Gbl.NumForm = -1;      // Number of form. It's increased by 1 at the begin of each form

   Gbl.Error = false;

   Gbl.GetMethod = false;

   Gbl.WritingHTMLStart = false;
   Gbl.HTMLStartWritten = false;
   Gbl.TablEndWritten = false;
   Gbl.HTMLEndWritten = false;

   Gbl.DatabaseIsOpen = false;

   Gbl.HiddenParamsInsertedIntoDB = false;

   Gbl.Prefs.SecureIcons = true;	// By default, the icons are sent via https
   Prf_SetIconsURL ();
   Gbl.Prefs.Theme = THEME_DEFAULT;	// Default theme
   sprintf (Gbl.Prefs.PathTheme,"%s/%s/%s",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_THEMES,The_ThemeId[Gbl.Prefs.Theme]);
   Gbl.Prefs.Language = Txt_LANGUAGE_DEFAULT;
   Gbl.Prefs.SideCols = 3;	// Show both side columns, left and right

   /* Related with lockfile */
   Gbl.LockCreated = false;
   sprintf (Gbl.PathLockFile,"%s/swad.lock",Cfg_PATH_REL_SWAD_PRIVATE);

   Gbl.Session.NumSessions = 0;
   Gbl.Session.IsOpen = false;
   Gbl.Session.HasBeenDisconnected = false;

   Gbl.YearOK = false;

   Gbl.Usrs.Me.UsrIdLogin[0] = '\0';
   Gbl.Usrs.Me.LoginPlainPassword[0] = '\0';
   Gbl.Usrs.Me.UsrDat.UsrCod = -1L;
   Gbl.Usrs.Me.UsrDat.DNI[0] = '\0';
   Usr_UsrDataConstructor (&Gbl.Usrs.Me.UsrDat);
   Gbl.Session.Id[0] = '\0';
   Gbl.Usrs.Me.Logged = false;
   Gbl.Usrs.Me.AvailableUsrTypes = 0;
   Gbl.Usrs.Me.UsrTypeFromSession = Usr_UNKNOWN;
   Gbl.Usrs.Me.UsrTypeLog = Usr_UNKNOWN;
   Gbl.Usrs.Me.UsrTypeForumsChat = Usr_UNKNOWN;
   Gbl.Usrs.Me.IBelongToThisCrs = false;
   Gbl.Usrs.Me.IHaveAccessToCurrentCrs = false;
   Gbl.Usrs.Me.MyPhotoExists = false;
   Gbl.Usrs.Me.NumAccWithoutPhoto = 0;
   Gbl.Usrs.Me.TimeLastAccToThisFileBrowser = LONG_MAX;	// Initialized to a big value, so by default files are not shown as recent or new
   Gbl.Usrs.Me.MyInstitutions.Filled = false;
   Gbl.Usrs.Me.MyDegrees.Filled = false;
   Gbl.Usrs.Me.MyCourses.Filled = false;
   Gbl.Usrs.Me.MyAdminDegs.Num = 0;
   Gbl.Usrs.Me.MyAdminDegs.Lst = NULL;

   Gbl.Usrs.Other.UsrDat.UsrCod = -1L;
   Gbl.Usrs.Other.UsrDat.DNI[0] = '\0';
   Usr_UsrDataConstructor (&Gbl.Usrs.Other.UsrDat);

   Gbl.CurrentTab = TabUnk;
   Gbl.CurrentAct = ActUnk;

   Gbl.Usrs.LstStds.NumUsrs = 0;
   Gbl.Usrs.LstStds.Lst = NULL;
   Gbl.Usrs.LstStds.Remove = NULL;
   Gbl.Usrs.LstStds.Ind = NULL;

   Gbl.Usrs.LstTchs.NumUsrs = 0;
   Gbl.Usrs.LstTchs.Lst = NULL;
   Gbl.Usrs.LstTchs.Remove = NULL;
   Gbl.Usrs.LstTchs.Ind = NULL;

   Gbl.Usrs.LstAdms.NumUsrs = 0;
   Gbl.Usrs.LstAdms.Lst = NULL;
   Gbl.Usrs.LstAdms.Remove = NULL;
   Gbl.Usrs.LstAdms.Ind = NULL;

   Gbl.LstExamAnnouncements.NumExamAnnounc = 0;
   Gbl.LstExamAnnouncements.Lst = NULL;

   Gbl.Usrs.Select.All = Gbl.Usrs.Select.Std = Gbl.Usrs.Select.Tch = Gbl.Usrs.ListDNIOrNicks = NULL;

   Gbl.Txt = NULL;

   Gbl.Msg.Subject[0] = '\0';
   Gbl.Msg.ShowOnlyUnreadMsgs = false;

   Gbl.Pag.CurrentPage = 1;	// By default, go to the first page

   Gbl.CurrentIns.Ins.InsCod = -1L;
   Gbl.CurrentIns.Ins.ShortName[0] = '\0';
   Gbl.CurrentIns.Ins.FullName[0] = '\0';
   Gbl.CurrentIns.Ins.Logo[0] = '\0';
   Gbl.CurrentIns.Ins.WWW[0] = '\0';
   Gbl.CurrentIns.Ins.NumStds = Gbl.CurrentIns.Ins.NumTchs = Gbl.CurrentIns.Ins.NumUsrs =
   Gbl.CurrentIns.Ins.NumCtrs = Gbl.CurrentIns.Ins.NumDpts = Gbl.CurrentIns.Ins.NumDegs = 0;

   Gbl.CurrentDegTyp.DegTyp.DegTypCod = -1L;
   Gbl.CurrentDegTyp.DegTyp.DegTypName[0] = '\0';

   Gbl.CurrentDeg.Deg.DegTypCod = -1L;
   Gbl.CurrentDeg.Deg.DegCod = -1L;
   Gbl.CurrentDeg.Deg.InsCod = -1L;
   Gbl.CurrentDeg.Deg.PlcCod = -1L;
   Gbl.CurrentDeg.Deg.ShortName[0] = Gbl.CurrentDeg.Deg.FullName[0] = '\0';

   Gbl.CurrentCrs.Crs.CrsCod = -1L;
   Gbl.CurrentCrs.Crs.DegCod = -1L;
   Gbl.CurrentCrs.Crs.ShortName[0] = Gbl.CurrentCrs.Crs.FullName[0] = '\0';
   
   Gbl.Inss.Num = 0;
   Gbl.Inss.Lst = NULL;
   Gbl.Inss.SelectedOrderType = INS_ORDER_BY_NUM_USRS;
   Gbl.Inss.EditingIns.InsCod = -1L;
   Gbl.Inss.EditingIns.ShortName[0] = '\0';
   Gbl.Inss.EditingIns.FullName[0] = '\0';
   Gbl.Inss.EditingIns.Logo[0] = '\0';
   Gbl.Inss.EditingIns.WWW[0] = '\0';
   Gbl.Inss.EditingIns.NumStds = Gbl.Inss.EditingIns.NumTchs = Gbl.Inss.EditingIns.NumUsrs =
   Gbl.Inss.EditingIns.NumCtrs = Gbl.Inss.EditingIns.NumDpts = Gbl.Inss.EditingIns.NumDegs = 0;

   Gbl.Ctrs.Num = 0;
   Gbl.Ctrs.Lst = NULL;
   Gbl.Ctrs.SelectedOrderType = CTR_ORDER_BY_CENTRE;
   Gbl.Ctrs.EditingCtr.CtrCod = -1L;
   Gbl.Ctrs.EditingCtr.ShortName[0] = '\0';
   Gbl.Ctrs.EditingCtr.FullName[0] = '\0';
   Gbl.Ctrs.EditingCtr.WWW[0] = '\0';

   Gbl.Dpts.Num = 0;
   Gbl.Dpts.Lst = NULL;
   Gbl.Dpts.SelectedOrderType = DPT_ORDER_BY_DEPARTMENT;
   Gbl.Dpts.EditingDpt.DptCod = -1L;
   Gbl.Dpts.EditingDpt.ShortName[0] = '\0';
   Gbl.Dpts.EditingDpt.FullName[0] = '\0';
   Gbl.Dpts.EditingDpt.WWW[0] = '\0';

   Gbl.Plcs.Num = 0;
   Gbl.Plcs.Lst = NULL;
   Gbl.Plcs.SelectedOrderType = PLC_ORDER_BY_NUM_DEGS;
   Gbl.Plcs.EditingPlc.PlcCod = -1L;
   Gbl.Plcs.EditingPlc.ShortName[0] = '\0';
   Gbl.Plcs.EditingPlc.FullName[0] = '\0';

   Gbl.Hlds.LstIsRead = false;	// List is not read
   Gbl.Hlds.Num = 0;
   Gbl.Hlds.Lst = NULL;
   Gbl.Hlds.SelectedOrderType = Hld_ORDER_BY_START_DATE;
   Gbl.Hlds.EditingHld.HldCod = -1L;
   Gbl.Hlds.EditingHld.PlcCod = -1L;
   Gbl.Hlds.EditingHld.HldTyp = Hld_HOLIDAY;
   Gbl.Hlds.EditingHld.StartDate.Year  = Gbl.Hlds.EditingHld.EndDate.Year  = Gbl.Today.Year;
   Gbl.Hlds.EditingHld.StartDate.Month = Gbl.Hlds.EditingHld.EndDate.Month = Gbl.Today.Month;
   Gbl.Hlds.EditingHld.StartDate.Day   = Gbl.Hlds.EditingHld.EndDate.Day   = Gbl.Today.Day;
   Gbl.Hlds.EditingHld.Name[0] = '\0';

   Gbl.Degs.DegTypes.Num = 0;
   Gbl.Degs.DegTypes.Lst = NULL;

   Gbl.Degs.AllDegs.Num = 0;
   Gbl.Degs.AllDegs.Lst = NULL;

   Gbl.Degs.EditingDegTyp.DegTypCod = -1L;
   Gbl.Degs.EditingDegTyp.DegTypName[0] = '\0';

   Gbl.Degs.EditingDeg.DegCod = -1L;
   Gbl.Degs.EditingDeg.InsCod = -1L;
   Gbl.Degs.EditingDeg.PlcCod = -1L;
   Gbl.Degs.EditingDeg.ShortName[0] = '\0';
   Gbl.Degs.EditingDeg.FullName[0] = '\0';
   Gbl.Degs.EditingDeg.FirstYear = Gbl.Degs.EditingDeg.LastYear = 0;
   Gbl.Degs.EditingDeg.OptYear = false;
   Gbl.Degs.EditingDeg.Logo[0] = '\0';
   Gbl.Degs.EditingDeg.WWW[0] = '\0';
   Gbl.Degs.EditingDeg.NumCourses = 0;
   Gbl.Degs.EditingDeg.LstCrss = NULL;

   Gbl.Degs.EditingCrs.CrsCod = -1L;
   Gbl.Degs.EditingCrs.DegCod = -1L;
   Gbl.Degs.EditingCrs.Year = 0;				
   Gbl.Degs.EditingCrs.Semester = 0;			
   Gbl.Degs.EditingCrs.ShortName[0] = '\0';
   Gbl.Degs.EditingCrs.FullName[0] = '\0';

   Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes = NULL;
   Gbl.CurrentCrs.Grps.GrpTypes.Num = 0;
   Gbl.CurrentCrs.Grps.GrpTypes.NestedCalls = 0;
   Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName[0] = '\0';
   Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration = true;
   Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration = false;
   Gbl.CurrentCrs.Grps.GrpCod = -1L; // -1L stands for the whole course
   Gbl.CurrentCrs.Grps.GrpName[0] = '\0';
   Gbl.CurrentCrs.Grps.MaxStudents = INT_MAX;
   Gbl.CurrentCrs.Grps.Open = false;
   Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod  = NULL;
   Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = 0;
   Gbl.Usrs.ClassPhoto.AllGroups = true;

   Gbl.CurrentCrs.Records.Field.Name[0] = '\0';
   Gbl.CurrentCrs.Records.Field.NumLines = Rec_MIN_LINES_IN_EDITION_FIELD;
   Gbl.CurrentCrs.Records.Field.Visibility = HIDDEN_FIELD;

   Gbl.CurrentCrs.Records.LstFields.Lst = NULL;
   Gbl.CurrentCrs.Records.LstFields.Num = 0;
   Gbl.CurrentCrs.Records.LstFields.NestedCalls = 0;

   Gbl.CurrentCrs.Syllabus.EditionIsActive = false;

   Gbl.Search.Crs[0] = '\0';

   Gbl.Asgs.LstIsRead = false;	// List is not read
   Gbl.Asgs.Num = 0;
   Gbl.Asgs.LstAsgCods = NULL;
   Gbl.Asgs.SelectedOrderType = Asg_ORDER_BY_START_DATE;

   Gbl.Links.Num = 0;
   Gbl.Links.Lst = NULL;
   Gbl.Links.EditingLnk.LnkCod = -1L;
   Gbl.Links.EditingLnk.ShortName[0] = '\0';
   Gbl.Links.EditingLnk.FullName[0] = '\0';
   Gbl.Links.EditingLnk.WWW[0] = '\0';

   Gbl.Usrs.Listing.Type = Usr_USER_LIST_UNKNOWN;
   Gbl.Usrs.Listing.CrdsPerPag = 1;
   Gbl.Usrs.Listing.WithPhotos = true;

   Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;

   Gbl.DateRange.DateIni.Year  = Sta_LOG_START_YEAR;
   Gbl.DateRange.DateIni.Month = Sta_LOG_START_MONTH;
   Gbl.DateRange.DateIni.Day   = Sta_LOG_START_DAY;
   Gbl.DateRange.DateEnd.Year  = Gbl.Today.Year;
   Gbl.DateRange.DateEnd.Month = Gbl.Today.Month;
   Gbl.DateRange.DateEnd.Day   = Gbl.Today.Day;

   Gbl.Stat.LocationRange  = Sta_LOCATION_RANGE_ALL;
   Gbl.Stat.ClicksStatType = ACC_CRS_PER_USR;
   Gbl.Stat.CountType      = Sta_TOTAL_CLICKS;
   Gbl.Stat.UsrType        = Sta_IDENTIFIED_USRS;
   Gbl.Stat.NumAction      = ActAll;
   Gbl.Stat.RowsPerPage    = 50;
   Gbl.Stat.UseStatType    = Sta_DEGREES_AND_COURSES;

   Gbl.Usrs.ListRange = Usr_LIST_USRS_RANGE_THIS_COURSE_ALL;

   Gbl.Usrs.Connected.StdsTotal = Gbl.Usrs.Connected.TchsTotal = 0;
   Gbl.Usrs.Connected.TimeToRefreshInMs = Con_MAX_TIME_TO_REFRESH_CONNECTED_IN_MS;

   Gbl.Test.NumQuestions = Tst_CONFIG_DEFAULT_DEF_QUESTIONS;
   Gbl.Test.AllTags = Gbl.Test.AllAnsTypes = false;
   Gbl.Test.TagsList = NULL;
   Gbl.Test.ListAnsTypes[0] = '\0';

   Gbl.Forum.ForumType = (For_ForumType_t) 0;
   Gbl.Forum.WhichForums = For_ALL_MY_FORUMS;
   Gbl.Forum.SelectedOrderType = For_LAST_MSG;
   Gbl.Forum.ThreadToMove = -1L;

   /* Related to user nickname */
   Gbl.Usrs.Me.Nickname[0] = '\0';
   Gbl.Usrs.NewNickname[0] = '\0';

   /* Public directory where create links for the file explorer */
   Gbl.FileBrowser.Type = Brw_FILE_BRW_UNKNOWN;

   /* By default, a file of marks doesn't have header or footer rows */
   Gbl.Marks.Header = Gbl.Marks.Footer = 0;

   /* To alternate colors where listing rows */
   Gbl.GlobalRowEvenOdd = 0;
   Gbl.ColorRows[0] = "FBFBE1";
   Gbl.ColorRows[1] = "FEFEF2";

   /* Related to imported data from external site */
   Gbl.Imported.UsrId[0] = '\0';
   Gbl.Imported.SessionId[0] = '\0';
   Gbl.Imported.UsrType = Usr_UNKNOWN;
  }

/****************************************************************************/
/************* Free memory, close files, remove lock file, etc. *************/
/****************************************************************************/

void Gbl_Cleanup (void)
  {
   if (Gbl.CurrentAct != ActRef &&
       Gbl.CurrentAct != ActWebSvc &&
       Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW &&
       !Gbl.HiddenParamsInsertedIntoDB)
      Ses_RemoveHiddenParFromThisSession ();
   Usr_UsrDataDestructor (&Gbl.Usrs.Me.UsrDat);
   Usr_UsrDataDestructor (&Gbl.Usrs.Other.UsrDat);
   Rec_FreeListFields ();
   Grp_FreeListGrpTypesAndGrps ();
   Grp_FreeListCodSelectedGrps ();
   Deg_FreeListCoursesInDegree (&Gbl.Degs.EditingDeg);
   Deg_FreeListMyAdminDegs ();
   Deg_FreeListAllDegs ();
   Deg_FreeListDegTypes ();
   Ins_FreeListInstitutions ();
   Ctr_FreeListCentres ();
   Dpt_FreeListDepartments ();
   Plc_FreeListPlaces ();
   Hld_FreeListHolidays ();
   Lnk_FreeListLinks ();
   Plg_FreeListPlugins ();
   Usr_FreeUsrsList (&Gbl.Usrs.LstAdms);
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);
   Usr_FreeListNicks ();
   Usr_FreeListsEncryptedUsrCods ();
   Str_FreeTxt ();
   Syl_FreeListItemsSyllabus ();
   Tst_FreeTagsList ();
   Exa_FreeMemExamAnnouncement ();
   Exa_FreeListExamAnnouncements ();
   if (Gbl.F.Tmp)
      fclose (Gbl.F.Tmp);
   Syl_CloseFileSyllabus ();
   Fil_SignalSemaphore ();
  }
