// swad_gbl.h: global variables

#ifndef _SWAD_GBL
#define _SWAD_GBL
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <limits.h>		// For maximum values
#include <stdio.h>		// For FILE
#include <sys/time.h>		// For tz
#include <mysql/mysql.h>	// To access MySQL databases

#include "swad_asg.h"
#include "swad_brw.h"
#include "swad_cfg.h"
#include "swad_cry.h"
#include "swad_ctr.h"
#include "swad_dpt.h"
#include "swad_fil.h"
#include "swad_for.h"
#include "swad_hld.h"
#include "swad_ins.h"
#include "swad_imp.h"
#include "swad_lay.h"
#include "swad_lnk.h"
#include "swad_msg.h"
#include "swad_plc.h"
#include "swad_plg.h"
#include "swad_rec.h"
#include "swad_ses.h"
#include "swad_syl.h"
#include "swad_tst.h"
#include "swad_tt.h"

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

struct Globals
  {
   struct Files F;
   MYSQL mysql;
   pid_t PID;	// PID of current process
   int NumForm;	// Number of form used in form submit links
   bool Error;
   bool GetMethod;	// Am I accessed using GET method?
   bool WebService;	// Must generate HTML output (WebService==false) or SOAP-XML output (WebService==true)?
   Cte_Content_t ContentReceivedByCGI;	/* Content send by the form and received by the CGI:
						Cte_NORMAL (if CONTENT_TYPE==text/plain) or
						Cte_DAT (if CONTENT_TYPE==multipart/form-data) */
   char DelimiterString[1000];
   char DelimiterStringIncludingInitialRet[2+1000];
   bool WritingHTMLStart;	// Used to avoid writing the HTML head when aborting program on error
   bool HTMLStartWritten;	// Used to avoid writing more than once the HTML head
   bool TablEndWritten;	// Used to avoid writing more than once the HTML end
   bool HTMLEndWritten;	// Used to avoid writing more than once the HTML end
   bool DatabaseIsOpen;
   bool HiddenParamsInsertedIntoDB;	// Indica si se ha insertado algn parameter in the database in esta ejecucin
   bool LockCreated;
   char PathLockFile[PATH_MAX+1];

   /* To compute execution time of the program */
   struct timeval tvStart,tvPageCreated,tvPageSent;
   struct timezone tz;
   long TimeGenerationInMicroseconds;
   long TimeSendInMicroseconds;

   char IP[Cte_MAX_LENGTH_IP+1];
   char UniqueNameEncrypted[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1];	// Used for session id, temporary directory names, etc.

   struct
     {
      char UsrId[Cfg_MAX_LENGTH_IMPORTED_USR_ID+1];
      char SessionId[Cfg_MAX_LENGTH_IMPORTED_SESSION_ID+1];
      Usr_UsrType_t UsrType;
     } Imported;
   struct
     {
      tTheme Theme;
      char PathTheme[PATH_MAX+1];
      Txt_Language_t Language;
      unsigned SideCols;
      bool SecureIcons;
      char IconsURL[PATH_MAX+1];
     } Prefs;

   struct
     {
      unsigned NumSessions;
      bool IsOpen;
      bool HasBeenDisconnected;
      char Id[Ses_LENGTH_SESSION_ID+1];
      long UsrCod;
     } Session;

   bool YearOK;
   Act_Tab_t CurrentTab;
   Act_Action_t CurrentAct;
   time_t TimeStartExecution;
   struct tm *tblock;
   struct Date Today,Yesterday;
   struct DateTime Now;
   unsigned AcademicYear;	// First year of the academic year and current month
   char Message[Lay_MAX_BYTES_ALERT];			// Auxiliar string for alerts
   char MsgWindowStatus[Lay_MAX_BYTES_MSG_WINDOW_STATUS];	// String for help message in lower part of browser
   int GlobalRowEvenOdd;	// To alternate row colors in listings
   char *ColorRows[2];
   struct
     {
      char FileName[PATH_MAX+1];
     } HTMLOutput;
   struct
     {
      unsigned Num;		// Number of institutions
      struct Institution *Lst;	// List of institutions
      struct Institution EditingIns;
      tInssOrderType SelectedOrderType;
     } Inss;
   struct
     {
      unsigned Num;		// Number of centres
      struct Centre *Lst;	// List of centres
      struct Centre EditingCtr;
      tCtrsOrderType SelectedOrderType;
     } Ctrs;
   struct
     {
      unsigned Num;		// Number of departments
      struct Department *Lst;	// List of departments
      struct Department EditingDpt;
      tDptsOrderType SelectedOrderType;
     } Dpts;
   struct
     {
      unsigned Num;		// Number of places
      struct Place *Lst;	// List of places
      struct Place EditingPlc;
      Plc_PlcsOrderType_t SelectedOrderType;
     } Plcs;
   struct
     {
      bool LstIsRead;		// Is the list already read from database, or it needs to be read?
      unsigned Num;		// Number of holidays
      struct Holiday *Lst;	// List of holidays
      struct Holiday EditingHld;
      Hld_OrderType_t SelectedOrderType;
     } Hlds;
   struct
     {
      unsigned Num;		// Number of institutional links
      struct Link *Lst;		// List of institutional links
      struct Link EditingLnk;
     } Links;
   struct
     {
      struct
        {
         unsigned Num;			// Number of degree types
         struct DegreeType *Lst;	// List of degree types
        } DegTypes;
      struct
        {
         unsigned Num;			// Number of degrees
         struct Degree *Lst;		// List of degrees
        } AllDegs;
      struct DegreeType EditingDegTyp;
      struct Degree EditingDeg;
      struct Course EditingCrs;
     } Degs;
   struct
     {
      char Crs[Deg_MAX_LENGTH_COURSE_FULL_NAME+1];
     } Search;
  struct
     {
      unsigned Num;		// Number of plugins
      struct Plugin *Lst;	// List of plugins
      struct Plugin EditingPlg;
     } Plugins;
   struct
     {
      struct
	{
	 struct UsrData UsrDat;
         unsigned AvailableUsrTypes;
         Usr_UsrType_t UsrTypeFromSession;
         Usr_UsrType_t UsrTypeLog;
         Usr_UsrType_t UsrTypeForumsChat;
	 char UsrIdLogin[Usr_MAX_LENGTH_USR_ID+1];	// String to store the DNI or the nickname entered in the user's login
	 char Nickname[Usr_MAX_LENGTH_NICKNAME+1];
         char LoginPlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1];
         char LoginEncryptedPassword[Cry_LENGTH_ENCRYPTED_STR_SHA512_BASE64+1];
         char PendingPassword[Cry_LENGTH_ENCRYPTED_STR_SHA512_BASE64+1];
	 char PathAbsDir[PATH_MAX+1];
	 char PathRelDir[PATH_MAX+1];
	 bool Logged;
         bool IBelongToThisCrs;
	 bool IHaveAccessToCurrentCrs;
         bool MyPhotoExists;
         unsigned NumAccWithoutPhoto;
         char PhotoURL[PATH_MAX+1];
         time_t TimeLastAccToThisFileBrowser;
         struct
           {
            bool Filled;	// My institutions are already filled?
            unsigned Num;
            long InsCods[MAX_INSTITUTIONS_PER_USR];
           } MyInstitutions;
         struct
           {
            bool Filled;	// My degrees are already filled?
            unsigned Num;
            long DegCods[Deg_MAX_DEGREES_PER_USR];
           } MyDegrees;
         struct
           {
            bool Filled;	// My courses are already filled?
            unsigned Num;
            long CrsCods[Deg_MAX_COURSES_PER_USR];
           } MyCourses;
         struct
           {
            unsigned Num;
            struct Degree *Lst;
           } MyAdminDegs;	// List of degrees administrated by me
	 Usr_ShowUsrsType_t ListType;	// My preference about user's list type
	} Me;		// The user logged
      struct
	{
	 struct UsrData UsrDat;
	} Other;		// Another user, used for example to register in / remove from a course
      struct ListUsers LstStds,LstTchs,LstAdms;
      char NewNickname[Usr_MAX_LENGTH_NICKNAME+1];
      struct
        {
         char *All;
         char *Std;
         char *Tch;
        } Select;	// Lists of DNI of users from a form
      char *ListDNIOrNicks;	// List of DNI or nicks of users written explicitely on a form
      char PathRelListDNI[PATH_MAX+1];
      struct
	{
	 Usr_ListUsrsType_t Type;	// List of students, cards of students or cards of teachers
	 unsigned CrdsPerPag;	// Number of rows of cards in cada row (slo in the impresin)
	 bool WithPhotos;
	} Listing;
      struct
	{
	 unsigned Cols;
	 bool AllGroups;
	} ClassPhoto;
      struct
        {
         unsigned StdsTotal;
         unsigned TchsTotal;
         unsigned long TimeToRefreshInMs;
        } Connected;
      unsigned MonthsWithoutAccess;	// To remove old users
      Usr_ListUsrsRange_t ListRange;
      char FileNamePhoto[NAME_MAX+1];	// File name (with no path and no .jpg) of the temporary file with the selected face
      bool Error;	// Variable global for indicar that se ha producido a error
     } Usrs;
   struct
     {
      struct Institution Ins;
      char Logo16[PATH_MAX+1];
      char Logo32[PATH_MAX+1];
      char Logo64[PATH_MAX+1];
     } CurrentIns;
   struct
     {
      struct DegreeType DegTyp;
      struct Degree *LstDegs;	// List of degrees of current degree type
     } CurrentDegTyp;
   struct
     {
      struct Degree Deg;
      char Logo16[PATH_MAX+1];
      char Logo32[PATH_MAX+1];
      char Logo64[PATH_MAX+1];
     } CurrentDeg;
   struct
     {
      struct Course Crs;
      char PathAbsPriv[PATH_MAX+1];   // Absolute path to the private directory of the course
      char PathRelPriv[PATH_MAX+1];   // Relative path to the private directory of the course
      char PathRelPubl[PATH_MAX+1];   // Relative path to the public directory of the course
      char PathURLPubl[PATH_MAX+1];   // Abolute URL to the public part of the course
      struct
        {
         struct GroupTypes GrpTypes;
         struct GroupType GrpTyp;
         long GrpCod;
         char GrpName[MAX_LENGTH_GROUP_NAME+1];
         int MaxStudents;
         bool Open;
         bool FileZones;
         struct ListCodGrps LstGrpsSel;
         Grp_WhichGroups_t WhichGrps;	// Show my groups or all groups
        } Grps;
      struct
	{
	 struct
	   {
	    char URL[Cte_MAX_BYTES_URL+1];	// URL
	    char PathRelFileURL [PATH_MAX+1]; // Relative path to a text file of texto containing the URL
	    char PathRelFileZIP [PATH_MAX+1]; // Relative path to a file ZIP with the complete web page
	    char PathRelWebPage [PATH_MAX+1]; // Relative path to a public directory with a complete web page
	    char PathAbsWebPage [PATH_MAX+1]; // Absolute path (URL) to a public directory with a complete web page
	   } Links[Crs_NUM_INFO_TYPES];
	 char URL[Cte_MAX_BYTES_URL+1];
         FILE *FileURL;
	} Info;
      struct
	{
	 char PathDir[PATH_MAX+1];
	 Syl_SyllabusType_t Type;
	 int NumItem;
	 bool EditionIsActive;
	} Syllabus;
      struct
        {
         struct RecordField Field;
         struct LstRecordFields LstFields;
        } Records;
     } CurrentCrs;
   struct
     {
      char CrsFullName[Cte_MAX_BYTES_STRING+1];
      unsigned Year; // Number of year (0 (Optional), 1, 2, 3, 4, 5, 6) in the degree
      char Session[Cte_MAX_BYTES_STRING+1];	// Exam session is june, september, etc.
      struct Date CallDate;
      struct Date ExamDate;
      struct Hour StartTime;
      struct Hour Duration;
      char *Place;
      char *Mode;
      char *Structure;
      char *DocRequired;
      char *MatRequired;
      char *MatAllowed;
      char *OtherInfo;
     } ExamAnnouncement;
   struct
     {
      struct Date *Lst;		// List of dates of announcements of exams
      unsigned NumExamAnnounc;	// Number of announcements of exam in the list
     } LstExamAnnouncements;
   struct
     {
      Brw_FileBrowser_t Type;
      bool FullTree;		// Show full tree?
      struct
        {
         unsigned long MaxFiles;
         unsigned long MaxFolds;
         unsigned long long int MaxQuota;
         unsigned NumLevls;
         unsigned long NumFolds;
         unsigned long NumFiles;
         unsigned long long int TotalSiz;
        } Size;
      struct
	{
	 char PathRelAboveAboveRootFolder[PATH_MAX+1];
	 char PathAbsAboveRootFolder[PATH_MAX+1];
	 char PathRelAboveRootFolder[PATH_MAX+1];
	 char PathRelRootFolder[PATH_MAX+1];
	 char PathInTreeExceptFileOrFolder[PATH_MAX+1];
	 char FullPathInTree[PATH_MAX+1];
	} Priv;
      char NewFileOrFolderName[NAME_MAX+1];
      char FileOrFolderName[NAME_MAX+1];
      unsigned Level;
      struct
        {
         Brw_FileBrowser_t FileBrowser;	// Type of the file browser
         long CrsCod;			// Course code
	 long GrpCod;			// Code of the group related to the file browser with the clipboard
	 long WorksUsrCod;		// User code of the user related to the works file browser with the clipboard
         char Path[PATH_MAX+1];		// Complete path in the file browser
         char FileName[NAME_MAX+1];	// File name, last part of complete path in the file browser
         bool IsThisTree;		// When showing a file browser, is it that corresponding to the clipboard?
         bool IsThisFile;		// When showing a row of a file browser, are we in the path of the clipboard?
        } Clipboard;
      char TmpPubDir[NAME_MAX+1];
      bool HiddenLevels[1+Brw_MAX_DIR_LEVELS];
      struct Assignment Asg;	// Data of assignment when browsing level 1 or an assignment zone
      struct
        {
	 bool CreateZIP;
         char TmpDir[NAME_MAX+1];
        } ZIP;
     } FileBrowser;	// Struct used for a file browser
   struct
     {
      bool LstIsRead;	// Is the list already read from database, or it needs to be read?
      unsigned Num;	// Number of assignments
      long *LstAsgCods;	// List of assigments codes
      tAsgsOrderType SelectedOrderType;
     } Asgs;
   struct
     {
      unsigned Header;
      unsigned Footer;
     } Marks;
   struct
     {
      For_WhichForums_t WhichForums;
      For_ForumType_t ForumType;
      struct Institution Ins;
      struct Degree Deg;
      struct Course Crs;
      int Level;
      For_ForumOrderType_t SelectedOrderType;
      char PathRelOld[PATH_MAX+1];
      long ThreadToMove;
     } Forum;
   struct
     {
      int MsgId;
      char Subject[Cte_MAX_BYTES_SUBJECT+1];
      unsigned NumCourses;
      struct
        {
         long CrsCod;
         char ShortName[Deg_MAX_LENGTH_COURSE_SHORT_NAME+1];
        } Courses[Deg_MAX_COURSES_PER_USR];	// Distinct courses in my messages sent or received
      long FilterCrsCod; 		// Show only messages sent from this course code
      char FilterCrsShortName[Deg_MAX_LENGTH_COURSE_SHORT_NAME+1];
      char FilterFromTo[Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3+1];	// Show only messages from/to these users
      char FilterContent[Msg_MAX_LENGTH_FILTER_CONTENT+1];		// Show only messages that match this content
      bool ShowOnlyUnreadMsgs;	// Show only unread messages (this option is applicable only for received messages)
      char FileNameMail[PATH_MAX+1];
      FILE *FileMail;
     } Msg;
   struct
     {
      unsigned CurrentPage;
     } Pag;
   struct
     {
      char WindowName[256];
     } Chat;
   struct
     {
      unsigned Day,Hour,Column;
      TT_ClassType_t ClassType;
      unsigned Duration;
      char Place[TT_MAX_BYTES_PLACE+1];
      char Group[TT_MAX_BYTES_GROUP+1];
      long GrpCod;		// Group code (-1 if no group selected)
     } TimeTable;
   struct
     {
      struct ConfigTst Config;
      long QstCod;
      Tst_AnswerType_t AnswerType;
      char TagText[Tst_MAX_TAGS_PER_QUESTION][Tst_MAX_BYTES_TAG+1];
      char AnsTF;
      struct
        {
         bool Correct;
         char *Text;
        } AnsChoice[Tst_MAX_ANSWERS_PER_QUESTION];
      long AnsInt;
      double AnsFloat[2];
      unsigned NumQuestions;
      long QstCodes[Tst_MAX_QUESTIONS_PER_EXAM];	// Codes of the sent/received questions in a test
      char StrAnswersUsr[Tst_MAX_QUESTIONS_PER_EXAM][Tst_MAX_LENGTH_ANS_USR+1];	// Answers selected by user
      bool AllTags;
      bool AllAnsTypes;
      char *TagsList;
      char ListAnsTypes[Tst_MAX_BYTES_LIST_ANSWER_TYPES+1];
      Tst_QuestionsOrder_t SelectedOrderType;
     } Test;
   char *Txt;	// Text of a message
   struct
     {
      struct Date DateIni,DateEnd;
     } DateRange;
   struct
     {
      Sta_LocationRange_t LocationRange;
      Sta_ClicksStatType_t ClicksStatType;
      Sta_UseStatType_t UseStatType;
      Sta_UsrTypeStat_t UsrType;
      Sta_CountType_t CountType;
      Act_Action_t NumAction;
      long FirstRow,LastRow,RowsPerPage;
      struct
        {
         Sta_AvgPhotoTypeOfAverage_t TypeOfAverage;
         Sta_HowComputePhotoSize_t HowComputePhotoSize;
         Sta_HowOrderDegrees_t HowOrderDegrees;
         int MaxStds;
         int MaxStdsWithPhoto;
         float MaxPercent;
        } DegPhotos;
     } Stat;
   char QueryString[Cte_MAX_LENGTH_ARGS_SENT_TO_CGI+1]; // String with the arguments sent to the CGI
  };

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Gbl_InitializeGlobals (void);
void Gbl_Cleanup (void);

#endif
