// swad_grp.c: groups

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <stdlib.h>		// For exit, system, malloc, free, rand, etc.
#include <string.h>		// For string functions

#include "swad_act.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_grp.h"
#include "swad_par.h"

/****************************************************************************/
/*************************** Internal constants *****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Internal types *******************************/
/****************************************************************************/

/****************************************************************************/
/************* External global variables from others modules ****************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/************************* Internal global variables ************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Internal prototypes **************************/
/****************************************************************************/

static void Grp_EditGroupTypes (void);
static void Grp_EditGroups (void);
static void Grp_ShowFormSeveralGrps (Act_Action_t NextAction);
static void Grp_ConstructorListGrpAlreadySelec (struct ListGrpsAlreadySelec **AlreadyExistsGroupOfType);
static void Grp_DestructorListGrpAlreadySelec (struct ListGrpsAlreadySelec **AlreadyExistsGroupOfType);
static void Grp_RemoveUsrFromGroup (struct UsrData *UsrDat,long GrpCod);
static void Grp_AddUsrToGroup (struct UsrData *UsrDat,long GrpCod);
static void Grp_ListGroupTypesForEdition (void);
static void Grp_ListGroupsForEdition (void);
static void Grp_ListGroupsForChange (struct GroupType *GrpTyp);
static void Grp_ListGroupsForMultipleSelection (struct GroupType *GrpTyp);
static void Grp_WriteGrpHead (void);
static void Grp_WriteRowGrp (struct Group *Grp,struct GroupType *GrpTyp,bool IBelongToThisGroup);
static void Grp_PutFormToCreateGroupType (void);
static void Grp_PutFormToCreateGroup (void);
static unsigned Grp_CountNumGrpsInThisCrsOfType (long GrpTypCod);
static void Grp_GetDataOfGroupTypeByCod (struct GroupType *GrpTyp);
static unsigned Grp_GetNumStdsInNoGrpsOfType (long GrpTypCod);
static long Grp_GetFirstCodGrpStdBelongsTo (long GrpTypCod,long UsrCod);
static bool Grp_GetIfGrpIsAvailable (long GrpTypCod);
static bool Grp_ChkIfIBelongToGrp (long GrpCod,struct ListCodGrps *LstGrpsIBelong);
static bool Grp_CheckIfGroupTypeNameExists (const char *GrpTypName,long GrpTypCod);
static bool Grp_CheckIfGroupNameExists (long GrpTypCod,const char *GrpName,long GrpCod);
static void Grp_CreateGroupType (void);
static void Grp_CreateGroup (void);
static void Grp_AskConfirmRemGrpTypWithGrps (unsigned NumGrps);
static void Grp_AskConfirmRemGrp (void);
static void Grp_RemoveGroupTypeCompletely (void);
static void Grp_RemoveGroupCompletely (void);
static void Grp_WriteMaxStdsGrp (unsigned MaxStudents);
static unsigned Grp_ConvertToNumMaxStdsGrp (const char *StrMaxStudents);
static long Grp_GetParamGrpTypeCod (void);
static void Grp_PutParamGrpTypCod (long GrpTypCod);

/*****************************************************************************/
/******************* Write the names of the selected groups ******************/
/*****************************************************************************/

void Grp_WriteNamesOfSelectedGrps (void)
  {
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_students_with_no_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_and[Txt_NUM_LANGUAGES];
   long GrpCod;
   unsigned NumGrpSel;
   struct GroupData GrpDat;

   /***** Show the selected groups *****/
   fprintf (Gbl.F.Out,"%s: ",
            (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps == 1) ?
            Txt_Group [Gbl.Prefs.Language] :
            Txt_Groups[Gbl.Prefs.Language]);
   for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
     {
      if ((GrpCod = Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel]) >= 0)
        {
         GrpDat.GrpCod = GrpCod;
         Grp_GetDataOfGroupByCod (&GrpDat);
         fprintf (Gbl.F.Out,"%s %s",
                  GrpDat.GrpTypName,GrpDat.GrpName);
        }
      else	// GrpCod < 0 ==> students not belonging to any group of type (-GrpCod)
        {
         Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = -GrpCod;
         Grp_GetDataOfGroupTypeByCod (&Gbl.CurrentCrs.Grps.GrpTyp);
         fprintf (Gbl.F.Out,"%s (%s)",
                  Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,
                  Txt_students_with_no_group[Gbl.Prefs.Language]);
        }

      if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps >= 2)
        {
         if (NumGrpSel == Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps-2)
            fprintf (Gbl.F.Out," %s ",Txt_and[Gbl.Prefs.Language]);
         if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps >= 3)
            if (NumGrpSel < Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps-2)
               fprintf (Gbl.F.Out,", ");
        }
     }
  }

/****************************************************************************/
/***************** Put forms to edit group types and groups *****************/
/****************************************************************************/

void Grp_ReqEditGroups (void)
  {
   /***** Get list of groups types and groups in this course *****/
   Grp_GetListGrpTypesAndGrpsInThisCrs ();

   /***** Put form to edit group types *****/
   Grp_EditGroupTypes ();

   /***** Put form to edit groups *****/
   if (Gbl.CurrentCrs.Grps.GrpTypes.Num) // If there are group types...
      Grp_EditGroups ();

   /***** Free list of groups types and groups in this course *****/
   Grp_FreeListGrpTypesAndGrps ();
  }

/****************************************************************************/
/************************* Put forms to edit group types ********************/
/****************************************************************************/

static void Grp_EditGroupTypes (void)
  {
   extern const char *Txt_Types_of_group_in_the_course_X[Txt_NUM_LANGUAGES];

   /***** Help message *****/
   if (Gbl.CurrentCrs.Grps.GrpTypes.Num) // Group types found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo tipo de grupo rellene el nombre del tipo de grupo," \
                          " el tipo de adscripci&oacute;n y pulse el icono verde.<br />" \
                          "Para eliminar un tipo de grupo existente pulse el aspa roja.<br />" \
                          "Para modificar un tipo de grupo existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,Txt_Types_of_group_in_the_course_X[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No group types found in this course
     {
      sprintf (Gbl.Message,"No existen tipos de grupo en <strong>%s</strong>.<br />" \
                           "Para a&ntilde;adir un tipo de grupo rellene el nombre del tipo de grupo," \
                           " el tipo de adscripci&oacute;n y pulse el icono verde.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Forms to edit group types *****/
   /* Start table with current group types */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current group types */
   Grp_ListGroupTypesForEdition ();

   /* Put a form to create a new group type */
   Grp_PutFormToCreateGroupType ();

   /* End of table with current group types */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/**************************** Put forms to edit groups **********************/
/****************************************************************************/

static void Grp_EditGroups (void)
  {
   extern const char *Txt_Groups_in_the_course_X[Txt_NUM_LANGUAGES];

   /***** Help message *****/
   if (Gbl.CurrentCrs.Grps.GrpTypes.NumGrpsTotal) // If there are groups...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un grupo nuevo seleccione el tipo de grupo," \
                          " rellene el nombre del grupo," \
                          " el n&ordm; m&aacute;ximo de alumnos (opcional) y pulse el icono verde.<br />" \
                          "Para eliminar un grupo existente pulse el aspa roja.<br />" \
                          "Para cerrar o abrir un grupo, pulse sobre el candado.<br />" \
                          "Para habilitar o inhabilitar una zona de archivos, pulse sobre el CD.<br />" \
                          "Para modificar el tipo, el nombre o el n&ordm; m&aacute;ximo de alumnos de un grupo existente," \
                          " ed&iacute;telos en las casillas correspondientes" \
                          " y despu&eacute;s pulse con el rat&oacute;n fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,Txt_Groups_in_the_course_X[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// There are group types, but there aren't groups
     {
      sprintf (Gbl.Message,"No existen grupos en <strong>%s</strong>.<br />" \
                           "Para a&ntilde;adir un grupo seleccione el tipo de grupo," \
                           " rellene el nombre del grupo," \
                           " el n&ordm; m&aacute;ximo de alumnos (opcional) y pulse el icono verde.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Forms to edit groups *****/
   /* Start table with current groups */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List the current groups */
   Grp_ListGroupsForEdition ();

   /* Put a form to create a new group */
   Grp_PutFormToCreateGroup ();

   /* End of table with current groups */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/********************* Request the registration in groups *******************/
/****************************************************************************/

void Grp_ReqRegisterInGrps (void)
  {
   extern const char *Txt_Groups_in_the_course_X_to_which_you_belong[Txt_NUM_LANGUAGES];
   extern const char *Txt_Change_my_groups[Txt_NUM_LANGUAGES];
   unsigned NumGrpTyp;
   struct GroupType *GrpTyp;

   /***** Check if this course has groups *****/
   if (Grp_CountNumGrpsInCurrentCrs ()) // This course has groups
     {
      /***** Get list of groups types and groups in this course *****/
      Grp_GetListGrpTypesAndGrpsInThisCrs ();

      /***** Write title *****/
      sprintf (Gbl.Message,Txt_Groups_in_the_course_X_to_which_you_belong[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_WriteTitle (Gbl.Message);

      /***** Start form *****/
      Act_FormStart (ActChgGrp);

      /***** Start table *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /***** List to selection the groups a the that belongs the user *****/
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
        {
         GrpTyp = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp];
         if (GrpTyp->NumGrps)	 // If there are groups of this type
           {
            if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)	// Los students estn obligados a apuntarse a the groups with mandatory registration; the teachers no.
               if (Grp_GetFirstCodGrpStdBelongsTo (GrpTyp->GrpTypCod,Gbl.Usrs.Me.UsrDat.UsrCod) < 0)  // If the student no is apuntado to any group
                  if (Grp_GetIfGrpIsAvailable (GrpTyp->GrpTypCod))	// If hay algn group of this type available
                    {
                     if (GrpTyp->MandatoryRegistration)
                       {
                        sprintf (Gbl.Message,"Tiene que apuntarse obligatoriamente %s de %s.",	// Need translation!
                                 GrpTyp->MultipleRegistration ? "al menos a un grupo" : "a un grupo",	// Need translation!
                                 GrpTyp->GrpTypName);
	                Lay_ShowAlert (WARNING,Gbl.Message);
                       }
                     else
                       {
                        sprintf (Gbl.Message,"Puede apuntarse voluntariamente %s de %s.",	// Need translation!
                                 GrpTyp->MultipleRegistration ? "a uno o a varios grupos" : "a un grupo",	// Need translation!
                                 GrpTyp->GrpTypName);
	                Lay_ShowAlert (INFO,Gbl.Message);
                       }
                    }
            Grp_ListGroupsForChange (GrpTyp);
           }
        }

      /***** End table *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** End form *****/
      Lay_PutSendAndUndoButtons (Txt_Change_my_groups[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form>\n");

      /***** Free list of groups types and groups in this course *****/
      Grp_FreeListGrpTypesAndGrps ();
     }
   else	// This course hasn't groups
     {
      sprintf (Gbl.Message,"No se han creado grupos dentro de <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
  }

/****************************************************************************/
/**************** Show form to select on or several groups ******************/
/****************************************************************************/

static void Grp_ShowFormSeveralGrps (Act_Action_t NextAction)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_All_groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_Update_students_according_to_selected_groups[Txt_NUM_LANGUAGES];
   unsigned NumGrpTyp;

   /***** Start form to actualizar the students in funcin of the groups selected *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (NextAction);

   /***** Put parameters needed depending on the action *****/
   switch (Gbl.CurrentAct)
     {
      case ActReqAsgWrkCrs:
//      case ActEdiAsgWrkCrs:
         Brw_ParamWorksZoneCrs ();
         break;
      case ActSeeStd:
         Usr_PutParamColsClassPhoto ();
         Usr_PutParamListWithPhotos ();
         break;
      case ActReqMsgUsr:
	 Usr_HiddenParUsrCodAll (NextAction,Gbl.Usrs.Select.All);
         break;
      default:
         break;
     }

   /***** Start table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\">");
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Select all groups *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"7\" class=\"%s\">" \
                      "<input type=\"checkbox\" id=\"AllGroups\" name=\"AllGroups\" value=\"Y\"",
            The_ClassFormul[Gbl.Prefs.Theme]);
   if (Gbl.Usrs.ClassPhoto.AllGroups)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onclick=\"togglecheckChildren(this,'GrpCods')\" />%s</td></tr>",
            Txt_All_groups[Gbl.Prefs.Language]);

   /***** Get list of groups types and groups in this course *****/
   Grp_GetListGrpTypesAndGrpsInThisCrs ();

   /***** List the groups for each group type *****/
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
      if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps)
         Grp_ListGroupsForMultipleSelection (&Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp]);

   /***** Free list of groups types and groups in this course *****/
   Grp_FreeListGrpTypesAndGrps ();

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** End form *****/
   sprintf (Gbl.MsgWindowStatus,Txt_Update_students_according_to_selected_groups[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/update12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /> %s</acronym></a></td></tr></table></form></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/********* Get parameters with the groups of students selected *******/
/****************************************************************************/

void Grp_PutParamsCodGrps (void)
  {
   long NumGrpSel;

   /***** Write the boolean parameter that indicates if all the groups must be listed *****/
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AllGroups\" value=\"%c\" />",
            Gbl.Usrs.ClassPhoto.AllGroups ? 'Y' : 'N');

   /***** Write the parameter with the list of group codes to show *****/
   if (!Gbl.Usrs.ClassPhoto.AllGroups && Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)
     {
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"GrpCods\" value=\"");
      for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
        {
         if (NumGrpSel)
            fprintf (Gbl.F.Out,"%c",Par_SEPARATOR_PARAM_MULTIPLE);
         fprintf (Gbl.F.Out,"%ld",Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel]);
        }
      fprintf (Gbl.F.Out,"\" />");
     }
  }

/****************************************************************************/
/****** Show a form to select one or several groups ********/
/****************************************************************************/

void Grp_ShowFormToSelectSeveralGroups (Act_Action_t NextAction)
  {
   /****** Get groups to show ******/
   if (Grp_GetParCodsSeveralGrpsToShowUsrs ())
      Grp_ShowFormSeveralGrps (NextAction);
  }

/****************************************************************************/
/**************** Get parameters related to groups selected *****************/
/****************************************************************************/
// Returns number of groups in current course

unsigned Grp_GetParCodsSeveralGrpsToShowUsrs (void)
  {
   char YN[1+1];
   unsigned NumGrpsInCurrentCrs;
   unsigned long MaxSizeLstGrpCods;
   char *LstCodGrps;
   struct ListCodGrps LstGrpsIBelong;
   const char *Ptr;
   char LongStr[1+10+1];
   unsigned NumGrp;

   /***** Set default for number of groups selected by me *****/
   Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = 0;

   /***** Get boolean parameter that indicates if all groups must be listed *****/
   Par_GetParam (Par_PARAM_SINGLE,"AllGroups",YN,1);
   Gbl.Usrs.ClassPhoto.AllGroups = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /***** Count number of groups in current course *****/
   if ((NumGrpsInCurrentCrs = Grp_CountNumGrpsInCurrentCrs ()))
     {
      /***** Allocate memory for the list of group codes *****/
      MaxSizeLstGrpCods = ((1+10+1) * NumGrpsInCurrentCrs) - 1;
      if ((LstCodGrps = (char *) malloc (MaxSizeLstGrpCods + 1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store the codes of the selected groups.");

      /***** Get parameter with list of groups to list *****/
      Par_GetParam (Par_PARAM_MULTIPLE,"GrpCods",LstCodGrps,MaxSizeLstGrpCods);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,LstCodGrps,MaxSizeLstGrpCods,true);

      if (LstCodGrps[0])
        {
         /***** Count number of groups selected from LstCodGrps *****/
         for (Ptr = LstCodGrps, NumGrp=0; *Ptr; NumGrp++)
            Str_GetNextStrUntilSeparParamMult (&Ptr,LongStr,1+10);
         Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = NumGrp;
        }

      if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)	// If I have selected groups...
        {
         /***** Create a list of groups selected from LstCodGrps *****/
         if ((Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod = (long *) calloc (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps,sizeof (long))) == NULL)
            Lay_ShowErrorAndExit ("Not enough memory to store the codes of the selected groups.");
         for (Ptr = LstCodGrps, NumGrp=0; *Ptr; NumGrp++)
           {
            Str_GetNextStrUntilSeparParamMult (&Ptr,LongStr,1+10);
            Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrp] = Str_ConvertStrCodToLongCod (LongStr);
           }
        }
      else						// If I haven't selected any group...
        {
         /***** I I haven't selected any group, show by default the groups I belong to *****/
         if (NumGrpsInCurrentCrs)
           {
            Grp_GetLstCodGrpsUsrBelongs (-1L,Gbl.Usrs.Me.UsrDat.UsrCod,&LstGrpsIBelong);
            if (LstGrpsIBelong.NumGrps)
              {
               if ((Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod = (long *) calloc (LstGrpsIBelong.NumGrps,sizeof (long))) == NULL)
                  Lay_ShowErrorAndExit ("Not enough memory to store the codes of the selected groups.");
               for (NumGrp=0; NumGrp<LstGrpsIBelong.NumGrps; NumGrp++)
                  Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrp] = LstGrpsIBelong.GrpCod[NumGrp];
               Grp_FreeListCodGrp (&LstGrpsIBelong);
               Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = LstGrpsIBelong.NumGrps;
              }
           }
        }

      /***** Free memory used for the list of groups to show *****/
      free ((void *) LstCodGrps);
     }

   /***** If no groups selected ==> show all groups *****/
   if (!Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)
      Gbl.Usrs.ClassPhoto.AllGroups = true;

   return NumGrpsInCurrentCrs;
  }

/****************************************************************************/
/**************** Get parameters related to groups selected *****************/
/****************************************************************************/

void Grp_GetParCodsSeveralGrpsToEditAssignments (void)
  {
   unsigned NumGrpsInCurrentCrs;
   unsigned long MaxSizeLstGrpCods;
   char *LstCodGrps;
   const char *Ptr;
   char LongStr[1+10+1];
   unsigned NumGrp;

   /***** Set default for number of groups selected by me *****/
   Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = 0;

   /***** Count number of groups in current course *****/
   if ((NumGrpsInCurrentCrs = Grp_CountNumGrpsInCurrentCrs ()))
     {
      /***** Allocate memory for the list of group codes *****/
      MaxSizeLstGrpCods = ((1+10+1) * NumGrpsInCurrentCrs) - 1;
      if ((LstCodGrps = (char *) malloc (MaxSizeLstGrpCods + 1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store the codes of the selected groups.");

      /***** Get parameter with list of groups to list *****/
      Par_GetParam (Par_PARAM_MULTIPLE,"GrpCods",LstCodGrps,MaxSizeLstGrpCods);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,LstCodGrps,MaxSizeLstGrpCods,true);

      if (LstCodGrps[0])
        {
         /***** Count number of groups selected from LstCodGrps *****/
         for (Ptr = LstCodGrps, NumGrp=0; *Ptr; NumGrp++)
            Str_GetNextStrUntilSeparParamMult (&Ptr,LongStr,1+10);
         Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = NumGrp;
        }

      if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)	// If I have selected groups...
        {
         /***** Create a list of groups selected from LstCodGrps *****/
         if ((Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod = (long *) calloc (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps,sizeof (long))) == NULL)
            Lay_ShowErrorAndExit ("Not enough memory to store the codes of the selected groups.");
         for (Ptr = LstCodGrps, NumGrp=0; *Ptr; NumGrp++)
           {
            Str_GetNextStrUntilSeparParamMult (&Ptr,LongStr,1+10);
            Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrp] = Str_ConvertStrCodToLongCod (LongStr);
           }
        }

      /***** Free memory used for the list of groups to show *****/
      free ((void *) LstCodGrps);
     }
  }

/****************************************************************************/
/********* Free memory used for the list of group codes selected ************/
/****************************************************************************/

void Grp_FreeListCodSelectedGrps (void)
  {
   if (Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod)
     {
      free ((void *) Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod);
      Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod = NULL;
      Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps = 0;
     }
  }

/****************************************************************************/
/**************** Change of groups a the that belongs a user ****************/
/****************************************************************************/

void Grp_ChangeMyGrps (void)
  {
   struct ListCodGrps LstGrpsIBelong,LstGrpsIWant;
   unsigned NumGrpTyp,NumGrpIBelong,NumGrpIWant,NumGrpThisType;
   struct GroupType *GrpTyp;
   bool ITryToLeaveAClosedGroup = false;
   bool ITryToRegisterInAClosedGroup = false;
   bool ITryToRegisterInFullGroup = false;
   bool RemoveMeFromThisGrp;
   bool RegisterMeInThisGrp;

   /***** Get list of groups types and groups in this course *****/
   Grp_GetListGrpTypesAndGrpsInThisCrs ();

   /***** Get the group codes which I want to join to *****/
   Grp_GetLstCodsGrpIWant (&LstGrpsIWant);

   /***** A student can not be registered in more than one group if the type of group is of single registration *****/
   // As the form to register in groups of single registration is a radio-based form and not a checkbox-based form...
   // ...this check is made only to avoid problems if the student manipulates the form
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT &&
       LstGrpsIWant.NumGrps >= 2)
      if (!Grp_CheckIfSelectionGrpsIsValid (&LstGrpsIWant))
         Lay_ShowErrorAndExit ("You can't belong to more than one group when registration is unique.");

   /***** Query in the database the group codes which I belong to *****/
   Grp_GetLstCodGrpsUsrBelongs (-1L,Gbl.Usrs.Me.UsrDat.UsrCod,&LstGrpsIBelong);

   if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)
     {
      /***** Go across the list of groups which I belong to and check if I try to leave a closed group *****/
      for (NumGrpIBelong=0; NumGrpIBelong<LstGrpsIBelong.NumGrps && !ITryToLeaveAClosedGroup; NumGrpIBelong++)
        {
         for (NumGrpIWant=0, RemoveMeFromThisGrp = true; NumGrpIWant<LstGrpsIWant.NumGrps && RemoveMeFromThisGrp; NumGrpIWant++)
            if (LstGrpsIBelong.GrpCod[NumGrpIBelong] == LstGrpsIWant.GrpCod[NumGrpIWant])
               RemoveMeFromThisGrp = false;
         if (RemoveMeFromThisGrp)
            /* Check if the group is closed */
            for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num && !ITryToLeaveAClosedGroup; NumGrpTyp++)
              {
               GrpTyp = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp];
               for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps && !ITryToLeaveAClosedGroup; NumGrpThisType++)
                  if ((GrpTyp->LstGrps[NumGrpThisType]).GrpCod == LstGrpsIBelong.GrpCod[NumGrpIBelong])
                     if (!((GrpTyp->LstGrps[NumGrpThisType]).Open))
                       {
                        ITryToLeaveAClosedGroup = true;
                        Lay_ShowAlert (WARNING,"No se ha realizado ning&uacute;n cambio" \
                                               " porque un grupo del que desea quitarse est&aacute; cerrado.");	// Need translation!
                       }
              }
       }

      if (!ITryToLeaveAClosedGroup)
         /***** Go across the list of groups which I want to belong and check that they are not closed or full *****/
         for (NumGrpIWant=0; NumGrpIWant<LstGrpsIWant.NumGrps && !ITryToRegisterInAClosedGroup && !ITryToRegisterInFullGroup; NumGrpIWant++)
           {
            for (NumGrpIBelong=0, RegisterMeInThisGrp = true; NumGrpIBelong<LstGrpsIBelong.NumGrps && RegisterMeInThisGrp; NumGrpIBelong++)
               if (LstGrpsIWant.GrpCod[NumGrpIWant] == LstGrpsIBelong.GrpCod[NumGrpIBelong])
                  RegisterMeInThisGrp = false;
            if (RegisterMeInThisGrp)
               /* Check if the group is closed o completo */
               for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num && !ITryToRegisterInAClosedGroup && !ITryToRegisterInFullGroup; NumGrpTyp++)
                 {
                  GrpTyp = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp];
                  for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps && !ITryToRegisterInAClosedGroup && !ITryToRegisterInFullGroup; NumGrpThisType++)
                     if ((GrpTyp->LstGrps[NumGrpThisType]).GrpCod == LstGrpsIWant.GrpCod[NumGrpIWant])
                       {
                        /* Check if the group is closed */
                        if (!((GrpTyp->LstGrps[NumGrpThisType]).Open))
                          {
                           ITryToRegisterInAClosedGroup = true;
                           Lay_ShowAlert (WARNING,"No se ha realizado ning&uacute;n cambio porque un grupo que desea est&aacute; cerrado.");	// Need translation!
                          }
                        /* Check if the group is full */
                        else if ((GrpTyp->LstGrps[NumGrpThisType]).NumStudents >= (GrpTyp->LstGrps[NumGrpThisType]).MaxStudents)
                          {
                           ITryToRegisterInFullGroup = true;
                           Lay_ShowAlert (WARNING,"No se ha realizado ning&uacute;n cambio porque un grupo que desea ya estaba completo.");	// Need translation!
                          }
                       }
                 }
           }
     }

   if (!ITryToLeaveAClosedGroup && !ITryToRegisterInAClosedGroup && !ITryToRegisterInFullGroup)
     {
      /***** Go across the list of groups I belong to, removing those groups that are not present in the list of groups I want to belong to *****/
      for (NumGrpIBelong=0; NumGrpIBelong<LstGrpsIBelong.NumGrps; NumGrpIBelong++)
        {
         for (NumGrpIWant=0, RemoveMeFromThisGrp = true; NumGrpIWant<LstGrpsIWant.NumGrps && RemoveMeFromThisGrp; NumGrpIWant++)
            if (LstGrpsIBelong.GrpCod[NumGrpIBelong] == LstGrpsIWant.GrpCod[NumGrpIWant])
               RemoveMeFromThisGrp = false;
         if (RemoveMeFromThisGrp)
            Grp_RemoveUsrFromGroup (&Gbl.Usrs.Me.UsrDat,LstGrpsIBelong.GrpCod[NumGrpIBelong]);
        }

      /***** Go across the list of groups that I want to register in, adding those groups that are not present in the list of groups I belong to *****/
      for (NumGrpIWant=0; NumGrpIWant<LstGrpsIWant.NumGrps; NumGrpIWant++)
        {
         for (NumGrpIBelong=0, RegisterMeInThisGrp = true; NumGrpIBelong<LstGrpsIBelong.NumGrps && RegisterMeInThisGrp; NumGrpIBelong++)
            if (LstGrpsIWant.GrpCod[NumGrpIWant] == LstGrpsIBelong.GrpCod[NumGrpIBelong])
               RegisterMeInThisGrp = false;
         if (RegisterMeInThisGrp)
            Grp_AddUsrToGroup (&Gbl.Usrs.Me.UsrDat,LstGrpsIWant.GrpCod[NumGrpIWant]);
        }
     }

   /***** Free memory with the list of groups which I want to belong to *****/
   Grp_FreeListCodGrp (&LstGrpsIWant);

   /***** Free memory with the list of groups which I belonged to *****/
   Grp_FreeListCodGrp (&LstGrpsIBelong);

   /***** Free list of groups types and groups in this course *****/
   Grp_FreeListGrpTypesAndGrps ();

   /***** Show again the table of selection of groups with the changes already made *****/
   Grp_ReqRegisterInGrps ();
  }

/****************************************************************************/
/***** Check if no se ha selected ms of a group of single registration *****/
/****************************************************************************/

bool Grp_CheckIfSelectionGrpsIsValid (struct ListCodGrps *LstGrps)
  {
   struct ListGrpsAlreadySelec *AlreadyExistsGroupOfType;
   unsigned NumGrpTyp,NumCodGrp;
   struct GroupData GrpDat;
   bool SelectionValid = true;

   /***** Create e inicializar list of groups already selected *****/
   Grp_ConstructorListGrpAlreadySelec (&AlreadyExistsGroupOfType);

   /***** Go across the list of groups selected comprobando if already se ha selected a group of the mismo type *****/
   for (NumCodGrp=0; SelectionValid && NumCodGrp<LstGrps->NumGrps; NumCodGrp++)
     {
      GrpDat.GrpCod = LstGrps->GrpCod[NumCodGrp];
      Grp_GetDataOfGroupByCod (&GrpDat);
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
         if (GrpDat.GrpTypCod == AlreadyExistsGroupOfType[NumGrpTyp].GrpTypCod)
           {
            if (GrpDat.MultipleRegistration || !AlreadyExistsGroupOfType[NumGrpTyp].AlreadySelected)
               AlreadyExistsGroupOfType[NumGrpTyp].AlreadySelected = true;
            else
               SelectionValid = false;
            break;
           }
     }

   /***** Free memory of the list of booleanos that indica if already se ha selected a group of cada type *****/
   Grp_DestructorListGrpAlreadySelec (&AlreadyExistsGroupOfType);

   return SelectionValid; // Return true if the selection of groups is correcta
  }

/****************************************************************************/
/*********** Create e inicializar list of groups already selected ***********/
/****************************************************************************/

static void Grp_ConstructorListGrpAlreadySelec (struct ListGrpsAlreadySelec **AlreadyExistsGroupOfType)
  {
   unsigned NumGrpTyp;

   /***** Allocate memory to a list of booleanos that indica if already se ha selected a group of cada type *****/
   if ((*AlreadyExistsGroupOfType = (struct ListGrpsAlreadySelec *) calloc (Gbl.CurrentCrs.Grps.GrpTypes.Num,sizeof (struct ListGrpsAlreadySelec))) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store type of group.");

   /***** Initialize the list *****/
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      (*AlreadyExistsGroupOfType)[NumGrpTyp].GrpTypCod = Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod;
      (*AlreadyExistsGroupOfType)[NumGrpTyp].AlreadySelected = false;
     }
  }

/****************************************************************************/
/***************** Liberar list of groups already selected ******************/
/****************************************************************************/

static void Grp_DestructorListGrpAlreadySelec (struct ListGrpsAlreadySelec **AlreadyExistsGroupOfType)
  {
   free ((void *) *AlreadyExistsGroupOfType);
   *AlreadyExistsGroupOfType = NULL;
  }

/****************************************************************************/
/******************* Register user in the groups of a list ******************/
/****************************************************************************/

void Grp_RegisterUsrIntoGroups (struct UsrData *UsrDat,struct ListCodGrps *LstGrps)
  {
   struct ListCodGrps LstGrpsHeBelongs;
   unsigned NumGrpTyp,NumGrpSel,NumGrpThisType,NumGrpHeBelongs;
   bool MultipleRegistration,AlreadyRegisteredInGrp;

   /***** For each existing type of group in the course... *****/
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      MultipleRegistration = Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].MultipleRegistration;

      /***** Query in the database the group codes of any group of this type the student belongs to *****/
      Grp_GetLstCodGrpsUsrBelongs (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod,UsrDat->UsrCod,&LstGrpsHeBelongs);

      /***** For each group selected by me... *****/
      for (NumGrpSel=0; NumGrpSel<LstGrps->NumGrps; NumGrpSel++)
        {
         /* Check if the selected group is of this type */
         for (NumGrpThisType=0; NumGrpThisType<Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps; NumGrpThisType++)
            if (LstGrps->GrpCod[NumGrpSel] == Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].LstGrps[NumGrpThisType].GrpCod)
              {
               /* The selected group is of this type */
               AlreadyRegisteredInGrp = false;
               /* For each group of this type to which the user belongs... */
               for (NumGrpHeBelongs=0; NumGrpHeBelongs<LstGrpsHeBelongs.NumGrps; NumGrpHeBelongs++)
                  if (LstGrps->GrpCod[NumGrpSel] == LstGrpsHeBelongs.GrpCod[NumGrpHeBelongs])
                     AlreadyRegisteredInGrp = true;
                  else if (!MultipleRegistration)	// If the type of group is of single registration
                    {
                     /* If the registration is single and the group to which the user belongs is different from the selected ==>
                        remove user from the group to which he belongs */
                     Grp_RemoveUsrFromGroup (UsrDat,LstGrpsHeBelongs.GrpCod[NumGrpHeBelongs]);
                     sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong>" \
                                          " ha sido dado de baja del grupo de %s al que pertenec&iacute;a.",	// Need translation!
			      UsrDat->DNI,Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName);
                     Lay_ShowAlert (INFO,Gbl.Message);
                    }
               if (!AlreadyRegisteredInGrp)	// If the user doesn't belong to the selected group
                 {
                  Grp_AddUsrToGroup (UsrDat,LstGrps->GrpCod[NumGrpSel]);
                  sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong>" \
                                       " ha sido a&ntilde;adido al grupo de %s <strong>%s</strong>.",	// Need translation!
		           UsrDat->DNI,Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName,
                           Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].LstGrps[NumGrpThisType].GrpName);
                  Lay_ShowAlert (INFO,Gbl.Message);
                 }
               break;	// Once we know the type of a selected group, it's not necessary to check the rest of types
              }
        }

      /***** Free the list of groups of this type to which the user belonged *****/
      Grp_FreeListCodGrp (&LstGrpsHeBelongs);
     }
  }

/****************************************************************************/
/**************** Remove user of the groups indicados in a list *************/
/****************************************************************************/
// Returns NumGrpsHeIsRemoved

unsigned Grp_RemoveUsrFromGroups (struct UsrData *UsrDat,struct ListCodGrps *LstGrps)
  {
   struct ListCodGrps LstGrpsHeBelongs;
   unsigned NumGrpSel,NumGrpHeBelongs,NumGrpsHeIsRemoved = 0;

   /***** Query in the database the group codes of any group the user belongs to *****/
   Grp_GetLstCodGrpsUsrBelongs (-1L,UsrDat->UsrCod,&LstGrpsHeBelongs);

   /***** For each group selected by me... *****/
   for (NumGrpSel=0; NumGrpSel<LstGrps->NumGrps; NumGrpSel++)
      /* For each group to which the user belongs... */
      for (NumGrpHeBelongs=0; NumGrpHeBelongs<LstGrpsHeBelongs.NumGrps; NumGrpHeBelongs++)
         /* If the user belongs to a selected group from which he must be removed */
         if (LstGrpsHeBelongs.GrpCod[NumGrpHeBelongs] == LstGrps->GrpCod[NumGrpSel])
           {
            Grp_RemoveUsrFromGroup (UsrDat,LstGrpsHeBelongs.GrpCod[NumGrpHeBelongs]);
            NumGrpsHeIsRemoved++;
           }

   /***** Write message to inform about how many groups the student has been removed from *****/
   if (NumGrpsHeIsRemoved == 0)
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> no ha sido dado de baja de ning&uacute;n grupo.",	// Need translation!
               UsrDat->DNI);
   else if (NumGrpsHeIsRemoved == 1)
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ha sido dado de baja de un grupo.",	// Need translation!
               UsrDat->DNI);
   else
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ha sido dado de baja de %u grupos.",	// Need translation!
               UsrDat->DNI,NumGrpsHeIsRemoved);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Free the list of groups of this type to which the user belonged *****/
   Grp_FreeListCodGrp (&LstGrpsHeBelongs);

   return NumGrpsHeIsRemoved;
  }

/****************************************************************************/
/********** Remove a user of all the groups from this course ********/
/****************************************************************************/

void Grp_RemUsrFromAllGrpsInCrs (struct UsrData *UsrDat,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[1024];

   /***** If user is a student... *****/
   if (UsrDat->UsrTypeInDB == Usr_STUDENT)
     {
      /***** ...decrease number of students in all the groups from this course donde estuviera dado of alta the student *****/
      // The comprobacin of that NumStudents>0 realmente no is necesaria
      sprintf (Query,"UPDATE crs_grp,crs_grp_types,crs_grp_usr SET crs_grp.NumStudents=crs_grp.NumStudents-1" \
                     " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                     " AND crs_grp_usr.UsrCod='%ld' AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp.NumStudents>'0'",
               Crs->CrsCod,UsrDat->UsrCod);
      DB_QueryUPDATE (Query,"can not decrease the number of students in several groups");
     }

   /***** Remove user of all the groups of the course *****/
   sprintf (Query,"DELETE FROM crs_grp_usr WHERE UsrCod='%ld' AND GrpCod IN" \
                  " (SELECT crs_grp.GrpCod FROM crs_grp_types,crs_grp" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod)",
            UsrDat->UsrCod,Crs->CrsCod);
   DB_QueryDELETE (Query,"can not remove a user from all groups of a course");

   /***** Write message to show the change made *****/
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong>" \
                           " ha sido eliminado de todos los grupos de <strong>%s</strong>.",	// Need translation!
               UsrDat->DNI,Crs->FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
  }

/****************************************************************************/
/********* Remove a student from all the groups of all the courses **********/
/****************************************************************************/

void Grp_RemUsrFromAllGrps (struct UsrData *UsrDat,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[512];

   /***** If user is a student... *****/
   if (UsrDat->UsrTypeInDB == Usr_STUDENT)
     {
      /***** ...decrease number of students in all the groups that the student belongs to *****/
      // The check about NumStudents>0 is not really necessary
      sprintf (Query,"UPDATE crs_grp,crs_grp_usr SET crs_grp.NumStudents=crs_grp.NumStudents-1" \
                     " WHERE crs_grp_usr.UsrCod='%ld' AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp.NumStudents>'0'",
               UsrDat->UsrCod);
      DB_QueryUPDATE (Query,"can not decrease the number of students in several groups");
     }

   /***** Remove user from all groups *****/
   sprintf (Query,"DELETE FROM crs_grp_usr WHERE UsrCod='%ld'",UsrDat->UsrCod);
   DB_QueryDELETE (Query,"can not remove a user from the groups he/she belongs to");

   /***** Write message to show the change made *****/
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong>" \
                           " ha sido eliminado de todos los grupos de todas sus asignaturas.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
  }

/****************************************************************************/
/************************* Remove a user from a group ***********************/
/****************************************************************************/

static void Grp_RemoveUsrFromGroup (struct UsrData *UsrDat,long GrpCod)
  {
   char Query[512];

   /***** Remove me from the group *****/
   sprintf (Query,"DELETE FROM crs_grp_usr WHERE GrpCod='%ld' AND UsrCod='%ld'",
            GrpCod,UsrDat->UsrCod);
   DB_QueryDELETE (Query,"can not remove a user from a group");

   /***** If user is a student... *****/
   if (UsrDat->UsrTypeInDB == Usr_STUDENT)
     {
      /***** ...decrease the number of students in the group *****/
      // The check about NumStudents>0 is not really necessary
      sprintf (Query,"UPDATE crs_grp SET NumStudents=NumStudents-1" \
                     " WHERE GrpCod='%ld' AND NumStudents>'0'",
               GrpCod);
      DB_QueryUPDATE (Query,"can not decrease the number of students in a group");
     }

   /***** Remove possible clipboard entry *****/
   Brw_RemoveUsrGrpClipboard (UsrDat->UsrCod,GrpCod);
  }

/****************************************************************************/
/*********************** Register a user in a group *************************/
/****************************************************************************/

static void Grp_AddUsrToGroup (struct UsrData *UsrDat,long GrpCod)
  {
   char Query[512];

   /***** Register in group *****/
   sprintf (Query,"INSERT INTO crs_grp_usr (GrpCod,UsrCod,LastAccDownloadGrp,LastAccCommonGrp,LastAccMarksGrp)" \
                  " VALUES ('%ld','%ld','0000-00-00','0000-00-00','0000-00-00')",
            GrpCod,UsrDat->UsrCod);
   DB_QueryINSERT (Query,"can not add a user to a group");

   /***** If user is a student... *****/
   if (UsrDat->UsrTypeInDB == Usr_STUDENT)
     {
      /***** ...increase the number of students in the group *****/
      sprintf (Query,"UPDATE crs_grp SET NumStudents=NumStudents+1" \
                     " WHERE GrpCod='%ld'",GrpCod);
      DB_QueryUPDATE (Query,"can not increase the number of students in a group");
     }
  }

/****************************************************************************/
/******************** List current group types for edition ******************/
/****************************************************************************/

static void Grp_ListGroupTypesForEdition (void)
  {
   extern const char *Txt_Type_of_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_eg_Theory_Practice[Txt_NUM_LANGUAGES];
   extern const char *Txt_Mandatory_registration[Txt_NUM_LANGUAGES];
   extern const char *Txt_Multiple_registration[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_BR_groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_type_of_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_It_is_optional_to_choose_a_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_It_is_mandatory_to_choose_a_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_A_student_can_belong_to_several_groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_A_student_can_only_belong_to_one_group[Txt_NUM_LANGUAGES];
   unsigned NumGrpTyp;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s<br />(%s)</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_Type_of_group[Gbl.Prefs.Language],Txt_eg_Theory_Practice[Gbl.Prefs.Language],
            Txt_Mandatory_registration[Gbl.Prefs.Language],
            Txt_Multiple_registration[Gbl.Prefs.Language],
            Txt_No_of_BR_groups[Gbl.Prefs.Language]);

   /***** List group types with forms for edition *****/
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      /* Put icon to remove group type */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      Act_FormStart (ActReqRemGrpTyp);
      Grp_PutParamGrpTypCod (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_type_of_group[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Txt_Remove_type_of_group[Gbl.Prefs.Language]);

      /* Name of group type */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\">");
      Act_FormStart (ActRenGrpTyp);
      Grp_PutParamGrpTypCod (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"GrpTypName\" size=\"30\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_GROUP_TYPE_NAME,Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName,Gbl.NumForm);

      /* Is it mandatory to register in any group? */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgMdtGrpTyp);
      Grp_PutParamGrpTypCod (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
      fprintf (Gbl.F.Out,"<select name=\"MandatoryRegistration\" style=\"width:150px;\" onchange=\"javascript:document.forms[%d].submit();\">" \
                         "<option value=\"N\"",Gbl.NumForm);
      if (!Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].MandatoryRegistration)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option><option value=\"Y\"",
               Txt_It_is_optional_to_choose_a_group[Gbl.Prefs.Language]);
      if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].MandatoryRegistration)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option></select></form></td>",
               Txt_It_is_mandatory_to_choose_a_group[Gbl.Prefs.Language]);

      /* Is it possible to register in multiple groups? */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgMulGrpTyp);
      Grp_PutParamGrpTypCod (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
      fprintf (Gbl.F.Out,"<select name=\"MultipleRegistration\" style=\"width:150px;\" onchange=\"javascript:document.forms[%d].submit();\">" \
                         "<option value=\"N\"",Gbl.NumForm);
      if (!Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].MultipleRegistration)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option><option value=\"Y\"",
               Txt_A_student_can_only_belong_to_one_group[Gbl.Prefs.Language]);
      if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].MultipleRegistration)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option></select></form></td>",
               Txt_A_student_can_belong_to_several_groups[Gbl.Prefs.Language]);

      /* Number of groups of this type */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT\">%u</td></tr>",
               Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps);
     }
  }

/****************************************************************************/
/********************** List current groups for edition *********************/
/****************************************************************************/

static void Grp_ListGroupsForEdition (void)
  {
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type_BR_of_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Group_name[Txt_NUM_LANGUAGES];
   extern const char *Txt_eg_A_B[Txt_NUM_LANGUAGES];
   extern const char *Txt_Max_stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_OF_stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_group[Txt_NUM_LANGUAGES];
   unsigned NumGrpTyp,NumTipGrpAux,NumGrpThisType;
   struct GroupType *GrpTyp,*GrpTypAux;
   struct Group *Grp;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td><td class=\"BM\"></td>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s<br />(%s)</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_Type_BR_of_group[Gbl.Prefs.Language],
            Txt_Group_name[Gbl.Prefs.Language],Txt_eg_A_B[Gbl.Prefs.Language],
            Txt_Max_stud[Gbl.Prefs.Language],
            Txt_No_OF_stud[Gbl.Prefs.Language]);

   /***** List the groups *****/
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      GrpTyp = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp];
      for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps; NumGrpThisType++)
        {
         Grp = &(GrpTyp->LstGrps[NumGrpThisType]);

         /* Write icon to remove group */
         fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
         Act_FormStart (ActReqRemGrp);
         Grp_PutParamGrpCod (Grp->GrpCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_group[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
                  Gbl.Prefs.IconsURL,Txt_Remove_group[Gbl.Prefs.Language]);

         /* Write icon to open/close group */
         fprintf (Gbl.F.Out,"<td class=\"BM\">");
         Act_FormStart (Grp->Open ? ActCloGrp : ActOpeGrp);
         Grp_PutParamGrpCod (Grp->GrpCod);
         sprintf (Gbl.MsgWindowStatus,"%s %s %s",
                  Txt_Group[Gbl.Prefs.Language],
                  Grp->GrpName,Grp->Open ? "abierto; pulse para cerrarlo" : "cerrado; pulse para abrirlo");	// Need translation!
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/%s_on12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
                  Gbl.Prefs.IconsURL,Grp->Open ? "open" : "closed",Gbl.MsgWindowStatus);

         /* Write icon to activate file zones for this group */
         fprintf (Gbl.F.Out,"<td class=\"BM\">");
         Act_FormStart (Grp->FileZones ? ActDisFilZonGrp : ActEnaFilZonGrp);
         Grp_PutParamGrpCod (Grp->GrpCod);
         sprintf (Gbl.MsgWindowStatus,"Zonas de archivos del grupo %s %s",	// Need translation!
                  Grp->GrpName,Grp->FileZones ? "habilitadas; pulse para inhabilitarlas" :
                                                "inhabilitadas; pulse para habilitarlas");	// Need translation!
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/cd%s_on12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
                  Gbl.Prefs.IconsURL,Grp->FileZones ? "ok" : "no",Gbl.MsgWindowStatus);

         /* Group type */
         fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
         Act_FormStart (ActChgGrpTyp);
         Grp_PutParamGrpCod (Grp->GrpCod);
         fprintf (Gbl.F.Out,"<select name=\"GrpTypCod\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
         for (NumTipGrpAux=0; NumTipGrpAux<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumTipGrpAux++)
           {
            GrpTypAux = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumTipGrpAux];
            fprintf (Gbl.F.Out,"<option value=\"%ld\"",GrpTypAux->GrpTypCod);
            if (GrpTypAux->GrpTypCod == GrpTyp->GrpTypCod)
	       fprintf (Gbl.F.Out," selected=\"selected\"");
            fprintf (Gbl.F.Out,">%s</option>",GrpTypAux->GrpTypName);
           }
         fprintf (Gbl.F.Out,"</select></form></td>");

         /* Group name */
         fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
         Act_FormStart (ActRenGrp);
         Grp_PutParamGrpCod (Grp->GrpCod);
         fprintf (Gbl.F.Out,"<input type=\"text\" name=\"GrpName\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                        " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
                  MAX_LENGTH_GROUP_NAME,Grp->GrpName,Gbl.NumForm);

         /* Mximo number of students of the group (row[3]) */
         fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
         Act_FormStart (ActChgMaxStdGrp);
         Grp_PutParamGrpCod (Grp->GrpCod);
         fprintf (Gbl.F.Out,"<input type=\"text\" name=\"MaxStudents\" size=\"3\" maxlength=\"10\" value=\"");
         Grp_WriteMaxStdsGrp (Grp->MaxStudents);
         fprintf (Gbl.F.Out,"\" onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",Gbl.NumForm);

         /* Current number of students in this group */
         fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT\">%d</td></tr>\n",
                  Grp->NumStudents);
        }
     }
  }

/****************************************************************************/
/** List groups of a type to that the user pueda cambiarse of group ***/
/****************************************************************************/

static void Grp_ListGroupsForChange (struct GroupType *GrpTyp)
  {
   struct ListCodGrps LstGrpsIBelong;
   unsigned NumGrpThisType;
   struct Group *Grp;
   bool IBelongToThisGroup;

   /***** Write heading *****/
   Grp_WriteGrpHead ();

   /***** Query in the database the group of this type al that I belong to *****/
   Grp_GetLstCodGrpsUsrBelongs (GrpTyp->GrpTypCod,Gbl.Usrs.Me.UsrDat.UsrCod,&LstGrpsIBelong);

   /***** List the groups *****/
   for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps; NumGrpThisType++)
     {
      Grp = &(GrpTyp->LstGrps[NumGrpThisType]);
      IBelongToThisGroup = Grp_ChkIfIBelongToGrp (Grp->GrpCod,&LstGrpsIBelong);

      /* Put icon to select the group */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\"");
      if (IBelongToThisGroup)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
      fprintf (Gbl.F.Out,"><input type=\"");
      // If user is a student and the registration is single and there are more than a group, put a radio item
      if ((Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT) &&
          !GrpTyp->MultipleRegistration &&
          GrpTyp->NumGrps > 1)
         fprintf (Gbl.F.Out,"radio\" name=\"GrpCod%ld\" value=\"%ld\" onclick=\"selectUnselectRadio(this,this.form.GrpCod%ld,%u)\"",
                  GrpTyp->GrpTypCod,Grp->GrpCod,GrpTyp->GrpTypCod,GrpTyp->NumGrps);
      else // Put a checkbox item
         fprintf (Gbl.F.Out,"checkbox\" name=\"GrpCod%ld\" value=\"%ld\"",GrpTyp->GrpTypCod,Grp->GrpCod);
      if (IBelongToThisGroup)
	 fprintf (Gbl.F.Out," checked=\"checked\"");
      else if ((Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT) &&
               ((!Grp->Open) || (Grp->NumStudents >= Grp->MaxStudents)))
         fprintf (Gbl.F.Out," disabled=\"disabled\"");
      fprintf (Gbl.F.Out," /></td>");

      Grp_WriteRowGrp (Grp,GrpTyp,IBelongToThisGroup);

      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Free memory with the list of groups a the that belongs the user *****/
   Grp_FreeListCodGrp (&LstGrpsIBelong);
  }

/****************************************************************************/
/**************** List groups of a type to edit assignments *****************/
/****************************************************************************/

void Grp_ListGrpsToEditAssignments (struct GroupType *GrpTyp,long AsgCod)
  {
   struct ListCodGrps LstGrpsIBelong;
   unsigned NumGrpThisType;
   bool IBelongToThisGroup;
   struct Group *Grp;

   /***** Write heading *****/
   Grp_WriteGrpHead ();

   /***** Query from the database the groups of this type which I belong to *****/
   Grp_GetLstCodGrpsUsrBelongs (GrpTyp->GrpTypCod,Gbl.Usrs.Me.UsrDat.UsrCod,&LstGrpsIBelong);

   /***** List the groups *****/
   for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps; NumGrpThisType++)
     {
      Grp = &(GrpTyp->LstGrps[NumGrpThisType]);
      IBelongToThisGroup = Grp_ChkIfIBelongToGrp (Grp->GrpCod,&LstGrpsIBelong);

      /* Put checkbox to select the group */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\"");
      if (IBelongToThisGroup)
	 fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
      fprintf (Gbl.F.Out,"><input type=\"checkbox\" name=\"GrpCods\" value=\"%ld\"",
               Grp->GrpCod);
      if (AsgCod > 0)	// AsgCod == -1L means new assignment
         if (Asg_CheckIfAsgIsAssociatedToGrp (AsgCod,Grp->GrpCod))
            fprintf (Gbl.F.Out," checked=\"checked\"");
      if (!(IBelongToThisGroup ||
            Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER))
         fprintf (Gbl.F.Out," disabled=\"disabled\"");
      fprintf (Gbl.F.Out," onclick=\"uncheckParent(this,'WholeCrs')\" /></td>");

      Grp_WriteRowGrp (Grp,GrpTyp,IBelongToThisGroup);

      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Free memory with the list of groups which I belongs to *****/
   Grp_FreeListCodGrp (&LstGrpsIBelong);
  }

/****************************************************************************/
/*************** List groups of a type to add or remove users ***************/
/****************************************************************************/

void Grp_ListGrpsToAddOrRemUsrs (struct GroupType *GrpTyp)
  {
   struct ListCodGrps LstGrpsIBelong;
   unsigned NumGrpThisType;
   bool IBelongToThisGroup;
   struct Group *Grp;

   /***** Write heading *****/
   Grp_WriteGrpHead ();

   /***** Query from the database the groups of this type which I belong to *****/
   Grp_GetLstCodGrpsUsrBelongs (GrpTyp->GrpTypCod,Gbl.Usrs.Me.UsrDat.UsrCod,&LstGrpsIBelong);

   /***** List the groups *****/
   for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps; NumGrpThisType++)
     {
      Grp = &(GrpTyp->LstGrps[NumGrpThisType]);
      IBelongToThisGroup = Grp_ChkIfIBelongToGrp (Grp->GrpCod,&LstGrpsIBelong);

      /* Put checkbox to select the group */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\"");
      if (IBelongToThisGroup)
	 fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
      fprintf (Gbl.F.Out,"><input type=\"checkbox\" name=\"GrpCod%ld\" value=\"%ld\"",
               GrpTyp->GrpTypCod,Grp->GrpCod);
      if (!(IBelongToThisGroup ||
            Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER))
         fprintf (Gbl.F.Out," disabled=\"disabled\"");
      fprintf (Gbl.F.Out," /></td>");

      Grp_WriteRowGrp (Grp,GrpTyp,IBelongToThisGroup);

      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Free memory with the list of groups which I belongs to *****/
   Grp_FreeListCodGrp (&LstGrpsIBelong);
  }

/****************************************************************************/
/******* Write a list of groups as checkbox form for unique selection *******/
/****************************************************************************/

static void Grp_ListGroupsForMultipleSelection (struct GroupType *GrpTyp)
  {
   extern const char *Txt_students_with_no_group[Txt_NUM_LANGUAGES];
   unsigned NumGrpThisType;
   unsigned NumGrpSel;
   struct ListCodGrps LstGrpsIBelong;
   bool IBelongToThisGroup;
   struct Group *Grp;

   /***** Write heading *****/
   Grp_WriteGrpHead ();

   /***** Query from the database the groups of this type which I belong to *****/
   Grp_GetLstCodGrpsUsrBelongs (GrpTyp->GrpTypCod,Gbl.Usrs.Me.UsrDat.UsrCod,&LstGrpsIBelong);

   /***** List the groups *****/
   for (NumGrpThisType=0; NumGrpThisType<GrpTyp->NumGrps; NumGrpThisType++)
     {
      Grp = &(GrpTyp->LstGrps[NumGrpThisType]);
      IBelongToThisGroup = Grp_ChkIfIBelongToGrp (Grp->GrpCod,&LstGrpsIBelong);

      /* Put checkbox to select the group */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\"");
      if (IBelongToThisGroup)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
      fprintf (Gbl.F.Out,"><input type=\"checkbox\" name=\"GrpCods\" value=\"%ld\"",
               Grp->GrpCod);
      if (Gbl.Usrs.ClassPhoto.AllGroups)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      else
         for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
            if (Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel] == Grp->GrpCod)
              {
               fprintf (Gbl.F.Out," checked=\"checked\"");
               break;
              }
      fprintf (Gbl.F.Out," onclick=\"checkParent(this,'AllGroups')\" /></td>");

      Grp_WriteRowGrp (Grp,GrpTyp,IBelongToThisGroup);

      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Free memory with the list of groups which I belongs to *****/
   Grp_FreeListCodGrp (&LstGrpsIBelong);

   /***** Write rows to select the students who don't belong to any group *****/
   /* To get the students who don't belong to a type of group, use group code -(GrpTyp->GrpTypCod) */
   /* Write checkbox to select the group */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\">" \
                      "<input type=\"checkbox\" name=\"GrpCods\" value=\"%ld\"",
            -(GrpTyp->GrpTypCod));
   if (Gbl.Usrs.ClassPhoto.AllGroups)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   else
      for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
         if (Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel] == -(GrpTyp->GrpTypCod))
           {
            fprintf (Gbl.F.Out," checked=\"checked\"");
            break;
           }
   fprintf (Gbl.F.Out," onclick=\"checkParent(this,'AllGroups')\" /></td>");

   /* Column closed/open */
   fprintf (Gbl.F.Out,"<td align=\"left\"></td>");

   /* Group type */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" class=\"DAT\">%s&nbsp;</td>",
            GrpTyp->GrpTypName);

   /* Group name = students with no group */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" colspan=\"2\" class=\"DAT\">%s&nbsp;</td>",
            Txt_students_with_no_group[Gbl.Prefs.Language]);

   /* Number of students who don't belong to any group of this type */
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT\">%u</td>",
            Grp_GetNumStdsInNoGrpsOfType (GrpTyp->GrpTypCod));

   /* Last column */
   fprintf (Gbl.F.Out,"<td align=\"left\"></td></tr>");
  }

/****************************************************************************/
/************** Write a row with the head for list of groups ****************/
/****************************************************************************/

static void Grp_WriteGrpHead (void)
  {
   extern const char *Txt_Type[Txt_NUM_LANGUAGES];
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Max_stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_OF_stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_Vacants[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" colspan=\"2\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "</tr>\n",
            Txt_Type[Gbl.Prefs.Language],
            Txt_Group[Gbl.Prefs.Language],
            Txt_Max_stud[Gbl.Prefs.Language],
            Txt_No_OF_stud[Gbl.Prefs.Language],
            Txt_Vacants[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****************** Write a row with the data of a group ********************/
/****************************************************************************/

static void Grp_WriteRowGrp (struct Group *Grp,struct GroupType *GrpTyp,bool IBelongToThisGroup)
  {
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   int Vacant;

   /***** Write icon to show if group is open or closed *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" width=\"12\"");
   if (IBelongToThisGroup)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
   fprintf (Gbl.F.Out,"><img src=\"%s/%s_off12x12.gif\" width=\"12\" height=\"12\" alt=\"%s %s %s\" /></td>",
            Gbl.Prefs.IconsURL,Grp->Open ? "open" : "closed",
            Txt_Group[Gbl.Prefs.Language],
            Grp->GrpName,Grp->Open ? "abierto" : "cerrado");

   /***** Group type *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\"");
   if (IBelongToThisGroup)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
   fprintf (Gbl.F.Out," class=\"DAT\">%s&nbsp;</td>",GrpTyp->GrpTypName);

   /***** Group name *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\"");
   if (IBelongToThisGroup)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
   fprintf (Gbl.F.Out," class=\"DAT\">%s&nbsp;</td>",Grp->GrpName);

   /***** Max. number of students in this group *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"");
   if (IBelongToThisGroup)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
   fprintf (Gbl.F.Out," class=\"DAT\">");
   Grp_WriteMaxStdsGrp (Grp->MaxStudents);
   fprintf (Gbl.F.Out,"&nbsp;</td>");

   /***** Current number of students in this group *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"");
   if (IBelongToThisGroup)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
   fprintf (Gbl.F.Out," class=\"DAT\">%d</td>",Grp->NumStudents);

   /***** Vacants in this group *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"");
   if (IBelongToThisGroup)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",VERY_LIGHT_BLUE);
   fprintf (Gbl.F.Out," class=\"DAT\">");
   if (Grp->MaxStudents > 10000)
      fprintf (Gbl.F.Out,"-");
   else
     {
      Vacant = (int) Grp->MaxStudents - (int) Grp->NumStudents;
      fprintf (Gbl.F.Out,"%u",(Vacant > 0) ? (unsigned) Vacant : 0);
     }
   fprintf (Gbl.F.Out,"</td>");
  }

/****************************************************************************/
/********************* Put a form to create a new group type ****************/
/****************************************************************************/

static void Grp_PutFormToCreateGroupType (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_type_of_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_type_of_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_It_is_optional_to_choose_a_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_It_is_mandatory_to_choose_a_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_A_student_can_belong_to_several_groups[Txt_NUM_LANGUAGES];
   extern const char *Txt_A_student_can_only_belong_to_one_group[Txt_NUM_LANGUAGES];

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"4\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"4\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_New_type_of_group[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewGrpTyp);

   /***** New group type *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_type_of_group[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Group type name *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"GrpTypName\" size=\"30\" maxlength=\"%u\" value=\"%s\" /></td>",
            MAX_LENGTH_GROUP_TYPE_NAME,Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);

   /***** Is it mandatory to register in any groups of this type? *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"MandatoryRegistration\" style=\"width:150px;\"><option value=\"N\"");
   if (!Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option><option value=\"Y\"",
            Txt_It_is_optional_to_choose_a_group[Gbl.Prefs.Language]);
   if (Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option></select></td>",
            Txt_It_is_mandatory_to_choose_a_group[Gbl.Prefs.Language]);

   /***** Is it possible to register in multiple groups of this type? *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"MultipleRegistration\" style=\"width:150px;\"><option value=\"N\"");
   if (!Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option><option value=\"Y\"",
            Txt_A_student_can_only_belong_to_one_group[Gbl.Prefs.Language]);
   if (Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option></select></td>" \
                      "</tr></table></form></td></tr>\n",
            Txt_A_student_can_belong_to_several_groups[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/*********************** Put a form to create a new group *******************/
/****************************************************************************/

static void Grp_PutFormToCreateGroup (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_group[Txt_NUM_LANGUAGES];
   unsigned NumGrpTyp;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"6\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"6\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_New_group[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewGrp);

   /***** New group *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_group[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Group is closed by default *****/
   fprintf (Gbl.F.Out,"<td class=\"BM\">" \
                      "<img src=\"%s/closed_off12x12.gif\" width=\"12\" height=\"12\" alt=\"Grupo cerrado\" />" \
                      "</td>",	// Need translation!
            Gbl.Prefs.IconsURL);

   /***** File zones are disabled by default *****/
   fprintf (Gbl.F.Out,"<td class=\"BM\">" \
                      "<img src=\"%s/cdno_off12x12.gif\" width=\"12\" height=\"12\" alt=\"Zonas de archivos inhabilitadas\" />" \
                      "</td>",	// Need translation!
            Gbl.Prefs.IconsURL);

   /***** Group type *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                  "<select name=\"GrpTypCod\">");
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
      if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod == Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName);
     }
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Group name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"GrpName\" size=\"40\" maxlength=\"%u\" value=\"%s\" /></td>",
            MAX_LENGTH_GROUP_NAME,Gbl.CurrentCrs.Grps.GrpName);

   /***** Mximo of students *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><input type=\"text\" name=\"MaxStudents\" size=\"3\" maxlength=\"10\" value=\"");
   Grp_WriteMaxStdsGrp (Gbl.CurrentCrs.Grps.MaxStudents);
   fprintf (Gbl.F.Out,"\" /></td></tr></table></form></td></tr>\n");
  }

/****************************************************************************/
/*********** Create a list with current group types in this course **********/
/****************************************************************************/

void Grp_GetListGrpTypesInThisCrs (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow;

   if (++Gbl.CurrentCrs.Grps.GrpTypes.NestedCalls > 1) // If list is created yet, there's nothing to do
      return;

   /***** Get group types with groups + groups types without groups from database *****/
   sprintf (Query,"(SELECT crs_grp_types.GrpTypCod,crs_grp_types.GrpTypName AS GrpTypName,crs_grp_types.Mandatory,crs_grp_types.Multiple,COUNT(crs_grp.GrpCod)" \
                  " FROM crs_grp_types,crs_grp WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                  " GROUP BY crs_grp.GrpTypCod)" \
                  " UNION " \
                  "(SELECT GrpTypCod,GrpTypName,Mandatory,Multiple,'0'" \
                  " FROM crs_grp_types WHERE CrsCod='%ld'" \
                  " AND GrpTypCod NOT IN (SELECT GrpTypCod FROM crs_grp))" \
                  " ORDER BY GrpTypName",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod);
   Gbl.CurrentCrs.Grps.GrpTypes.Num = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get types of group of a course");

   /***** Get group types *****/
   Gbl.CurrentCrs.Grps.GrpTypes.NumGrpsTotal = 0;
   if (Gbl.CurrentCrs.Grps.GrpTypes.Num)
     {
      /***** Create a list of group types *****/
      if ((Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes = (struct GroupType *) calloc (Gbl.CurrentCrs.Grps.GrpTypes.Num,sizeof (struct GroupType))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store types of group.");

      /***** Get group types *****/
      for (NumRow=0; NumRow<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumRow++)
        {
         /* Get next group type */
         row = mysql_fetch_row (mysql_res);

         /* Get group type code (row[0]) */
         if ((Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].GrpTypCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong type of group.");

         /* Get group type name (row[1]) */
         strncpy (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].GrpTypName,row[1],MAX_LENGTH_GROUP_TYPE_NAME);

         /* Is it mandatory to register in any groups of this type? (row[2]) */
         Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].MandatoryRegistration = (Str_ConvertToUpperLetter (row[2][0]) != 'N');

         /* Is it possible to register in multiple groups of this type? (row[3]) */
         Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].MultipleRegistration = (Str_ConvertToUpperLetter (row[3][0]) != 'N');

         /* Number of groups of this type (row[4]) */
         if (sscanf (row[4],"%u",&Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].NumGrps) != 1)
            Lay_ShowErrorAndExit ("Wrong number of groups of a type.");

         /* Add number of groups to total number of groups */
         Gbl.CurrentCrs.Grps.GrpTypes.NumGrpsTotal += Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].NumGrps;

	 /* Initialize pointer to the list of groups of this type */
         Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumRow].LstGrps = NULL;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/********* Create a list with group types and groups in this course *********/
/****************************************************************************/

void Grp_GetListGrpTypesAndGrpsInThisCrs (void)
  {
   unsigned NumGrpTyp;
   unsigned NumGrp;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   struct GroupType *GrpTyp;
   struct Group *Grp;

   /***** First we get the list of group types *****/
   Grp_GetListGrpTypesInThisCrs ();

   /***** Then we get the list of groups for each group type *****/
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      GrpTyp = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp];
      if (GrpTyp->NumGrps)	 // If there are groups of this type...
        {
         /***** Query database *****/
         if ((NumRows = Grp_GetGrpsOfType (GrpTyp->GrpTypCod,&mysql_res)) > 0) // Groups found...
           {
	    // NumRows should be equal to GrpTyp->NumGrps
            GrpTyp->NumGrps = (unsigned) NumRows;

            /***** Create list with groups of this type *****/
            if ((GrpTyp->LstGrps = (struct Group *) calloc (GrpTyp->NumGrps,sizeof (struct Group))) == NULL)
               Lay_ShowErrorAndExit ("Not enough memory to store groups of a type.");

            /***** Get the groups of this type *****/
            for (NumGrp=0; NumGrp<GrpTyp->NumGrps; NumGrp++)
              {
               Grp = &(GrpTyp->LstGrps[NumGrp]);

               /* Get next group */
               row = mysql_fetch_row (mysql_res);

               /* Get group code (row[0]) */
               if ((Grp->GrpCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
                  Lay_ShowErrorAndExit ("Wrong code of group.");

               /* Get group name (row[1]) */
               strcpy (Grp->GrpName,row[1]);

               /* Get max number of students of group (row[2]) and number of current students */
               Grp->MaxStudents = Grp_ConvertToNumMaxStdsGrp (row[2]);
               Grp->NumStudents = Grp_GetNumStdsInGrp (Grp->GrpCod);

               /* Get whether group is open ('Y') or closed ('N') (row[3]) */
               Grp->Open = (Str_ConvertToUpperLetter (row[3][0]) != 'N');

               /* Get whether group have file zones ('Y') or not ('N') (row[4]) */
               Grp->FileZones = (Str_ConvertToUpperLetter (row[4][0]) != 'N');
              }
           }
         else	// Error: groups should be found, but really they haven't be found. This never should happen.
            GrpTyp->NumGrps = 0;

         /***** Free structure that stores the query result *****/
         DB_FreeMySQLResult (&mysql_res);
        }
     }
  }

/*****************************************************************************/
/********* Free list of groups types and list of groups of each type *********/
/*****************************************************************************/

void Grp_FreeListGrpTypesAndGrps (void)
  {
   unsigned NumGrpTyp;
   struct GroupType *GrpTyp;

   if (Gbl.CurrentCrs.Grps.GrpTypes.NestedCalls > 0)
      if (--Gbl.CurrentCrs.Grps.GrpTypes.NestedCalls == 0)
         if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes)
           {
	    /***** Free memory used for each list of groups (one list for each group type) *****/
	    for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
              {
               GrpTyp = &Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp];
               if (GrpTyp->LstGrps)
                 {
                  free ((void *) GrpTyp->LstGrps);
		  GrpTyp->LstGrps = NULL;
		  GrpTyp->NumGrps = 0;
                 }
              }

	    /***** Free memory used by the list of group types *****/
            free ((void *) Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes);
            Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes = NULL;
            Gbl.CurrentCrs.Grps.GrpTypes.Num = 0;
           }
  }

/****************************************************************************/
/****** Query the types of group that already existen in this course ******/
/****************************************************************************/

unsigned long Grp_GetAllGrpTypesInThisCrs (MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get types of group in current course from database *****/
   sprintf (Query,"SELECT GrpTypCod,GrpTypName,Mandatory,Multiple FROM crs_grp_types" \
                  " WHERE CrsCod='%ld' ORDER BY GrpTypName",
            Gbl.CurrentCrs.Crs.CrsCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get types of group of a course");
  }

/****************************************************************************/
/********* Query the number of groups that hay in this course *********/
/****************************************************************************/

unsigned Grp_CountNumGrpsInCurrentCrs (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumGrps;

   /***** Get number of group in current course from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_grp_types,crs_grp" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod",
            Gbl.CurrentCrs.Crs.CrsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of groups in this course");

   /***** Get the number of groups of ese type *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumGrps) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of groups in this course.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumGrps;
  }

/****************************************************************************/
/****************** Count number of groups in a group type ******************/
/****************************************************************************/

static unsigned Grp_CountNumGrpsInThisCrsOfType (long GrpTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   int NumGrps;

   /***** Get number of groups of a type from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_grp WHERE GrpTypCod='%ld'",
            GrpTypCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of groups of a type");

   /***** Get number of groups of this type *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumGrps) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of groups of a type.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumGrps;
  }

/****************************************************************************/
/***************** Get current groups of a type in this course **************/
/****************************************************************************/

unsigned long Grp_GetGrpsOfType (long GrpTypCod,MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get groups of a type from database *****/
   sprintf (Query,"SELECT GrpCod,GrpName,MaxStudents,Open,FileZones" \
                  " FROM crs_grp WHERE GrpTypCod='%ld' ORDER BY GrpName",GrpTypCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get groups of a type");
  }

/****************************************************************************/
/******************* Get data of a group type from its code *****************/
/****************************************************************************/
// GrpTyp->GrpTypCod must have the code of the type of group

static void Grp_GetDataOfGroupTypeByCod (struct GroupType *GrpTyp)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   
   /***** Get data of a type of group from database *****/
   sprintf (Query,"SELECT GrpTypName,Mandatory,Multiple FROM crs_grp_types" \
                  " WHERE CrsCod='%ld' AND GrpTypCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpTyp->GrpTypCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get type of group");

   /***** Count number of rows in result *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting type of group.");

   /***** Get some data of group type *****/
   row = mysql_fetch_row (mysql_res);
   strcpy (GrpTyp->GrpTypName,row[0]);
   GrpTyp->MandatoryRegistration = (Str_ConvertToUpperLetter (row[1][0]) != 'N');
   GrpTyp->MultipleRegistration  = (Str_ConvertToUpperLetter (row[2][0]) != 'N');

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/********************** Get data of a group from its code *******************/
/****************************************************************************/

void Grp_GetDataOfGroupByCod (struct GroupData *GrpDat)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get data of a group from database *****/
   sprintf (Query,"SELECT crs_grp_types.GrpTypCod,crs_grp_types.GrpTypName,crs_grp_types.Multiple,crs_grp.GrpName,crs_grp.MaxStudents,crs_grp.Open,crs_grp.FileZones FROM crs_grp_types,crs_grp" \
                  " WHERE crs_grp_types.GrpTypCod=crs_grp.GrpTypCod AND crs_grp.GrpCod='%ld'",
            GrpDat->GrpCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a group");

   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting group.");

   /***** Get data of group *****/
   row = mysql_fetch_row (mysql_res);
   /* Get the code of the group type (row[0]) */
   if ((GrpDat->GrpTypCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of type of group.");
   /* Get the name of the group type (row[1]) */
   strcpy (GrpDat->GrpTypName,row[1]);
   /* Get whether a student may be in one or multiple groups (row[2]) */
   GrpDat->MultipleRegistration = (Str_ConvertToUpperLetter (row[2][0]) != 'N');
   /* Get the name of the group (row[3]) */
   strcpy (GrpDat->GrpName,row[3]);
   /* Get maximum number of students (row[4]) */
   GrpDat->MaxStudents = Grp_ConvertToNumMaxStdsGrp (row[4]);
   /* Get whether group is open or closed (row[5]) */
   GrpDat->Open = (Str_ConvertToUpperLetter (row[5][0]) != 'N');
   /* Get whether group has file zones (row[6]) */
   GrpDat->FileZones = (Str_ConvertToUpperLetter (row[6][0]) != 'N');

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/******************** Check if a group exists in database *******************/
/****************************************************************************/

bool Grp_CheckIfGroupExists (long GrpCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a group exists from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_grp WHERE GrpCod='%ld'",GrpCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a group exists");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a group exists.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************* Get number of students in a group ********************/
/****************************************************************************/

unsigned Grp_GetNumStdsInGrp (long GrpCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;
   unsigned NumStudents;

   /***** Get number of students in a group from database ******/
   sprintf (Query,"SELECT NumStudents FROM crs_grp WHERE GrpCod='%ld'",GrpCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get number of students in a group");

   /***** Count number of rows in result *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting number of students in a group.");

   /***** Get number of students (row[0]) *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumStudents) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of students in a group.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumStudents;
  }

/****************************************************************************/
/** Get no. of students of current course not belonging to groups of a type */
/****************************************************************************/

static unsigned Grp_GetNumStdsInNoGrpsOfType (long GrpTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumStudents;

   /***** Get number of students not belonging to groups of a type from database ******/
   sprintf (Query,"SELECT COUNT(crs_usr.UsrCod) FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'" \
                  " AND crs_usr.UsrCod NOT IN" \
                  " (SELECT DISTINCT crs_grp_usr.UsrCod FROM crs_grp,crs_grp_usr" \
                  " WHERE crs_grp.GrpTypCod='%ld' AND crs_grp.GrpCod=crs_grp_usr.GrpCod)",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Usr_STUDENT,GrpTypCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of students not belonging to groups of a type");

   /***** Get the number of students (row[0]) *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumStudents) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of students not belonging to groups of a type.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumStudents;
  }

/****************************************************************************/
/**** Get the first code of group of cierto type al that pert. a student ****/
/****************************************************************************/
// Return -GrpTypCod if the student doesn't belongs to any group of type GrpTypCod

static long Grp_GetFirstCodGrpStdBelongsTo (long GrpTypCod,long UsrCod)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long CodGrpIBelong;

   /***** Get a group which a user belong to from database *****/
   sprintf (Query,"SELECT crs_grp.GrpCod FROM crs_grp,crs_grp_usr WHERE crs_grp.GrpTypCod='%ld'" \
                  " AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod='%ld'",
            GrpTypCod,UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the group which a user belongs to");

   /***** Get the group *****/
   if (NumRows == 0)
      CodGrpIBelong = -GrpTypCod;
   else	// If there are more than a group, only get the first one
     {
      row = mysql_fetch_row (mysql_res);

      /* Get the code of group (row[0]) */
      if ((CodGrpIBelong = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of group.");
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return CodGrpIBelong;
  }

/****************************************************************************/
/********************* Check if a user belongs to a group *******************/
/****************************************************************************/
// Return true if the user identificado belongs al group with cdigo GrpCod

bool Grp_CheckIfIBelongToGrp (long GrpCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned UnsignedNum = 0;

   /***** Get if I belong to a group from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_grp_usr" \
                  " WHERE GrpCod='%ld' AND UsrCod='%ld'",
            GrpCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if you belong to a group");

   /***** Get if I belong or not to the group *****/
   row = mysql_fetch_row (mysql_res);
   sscanf (row[0],"%u",&UnsignedNum);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (UnsignedNum != 0);
  }

/****************************************************************************/
/* Get the number of types of group with mandatory registration             */
/* that have any group open and with any vacant                             */
/* and in cuyos groups no est apuntado the student                         */
/****************************************************************************/

unsigned Grp_NumGrpTypesMandatIDontBelong (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumGrpTypes;

   /***** Get the number of types of groups with mandatory registration which I don't belong to, from database *****/
   // The following query counts the number of students instead of get them directly, so it is slower
/*   sprintf (Query," SELECT COUNT(DISTINCT GrpTypCod) FROM" \
                  " (SELECT crs_grp_types.GrpTypCod AS GrpTypCod,COUNT(*) AS NumStudents,crs_grp.MaxStudents as MaxStudents" \
                  " FROM crs_grp_types,crs_grp,crs_grp_usr,usr_data" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.Mandatory='Y'" \
                  " AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod AND crs_grp.Open='Y'" \
                  " AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'" \
                  " GROUP BY crs_grp.GrpCod HAVING NumStudents<MaxStudents) AS tipos_abiertos_o_no_llenos" \
                  " WHERE GrpTypCod NOT IN" \
                  " (SELECT DISTINCT crs_grp_types.GrpTypCod FROM crs_grp_types,crs_grp,crs_grp_usr" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.Mandatory='Y' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                  " AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod='%ld')",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Usr_STUDENT,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod); */
   // The following query is faster
   sprintf (Query," SELECT COUNT(DISTINCT crs_grp_types.GrpTypCod) FROM crs_grp_types,crs_grp" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.Mandatory='Y' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                  " AND crs_grp.Open='Y' AND crs_grp.NumStudents<crs_grp.MaxStudents" \
                  " AND crs_grp_types.GrpTypCod NOT IN" \
                  " (SELECT DISTINCT crs_grp_types.GrpTypCod FROM crs_grp_types,crs_grp,crs_grp_usr" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.Mandatory='Y' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                  " AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod='%ld')",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of types of group of mandatory registration to which you don't belong to");

   /***** Get the number of type of group *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumGrpTypes) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of types of group of mandatory registration to which you don't belong to.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumGrpTypes;
  }

/****************************************************************************/
/***** Query if algn group of a type is open and with vacants ******/
/****************************************************************************/

static bool Grp_GetIfGrpIsAvailable (long GrpTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumGrpTypes;

   /***** Get the number of types of group (0 or 1) of a type
          with one or more open groups with vacants, from database *****/
   sprintf (Query," SELECT COUNT(DISTINCT crs_grp_types.GrpTypCod) FROM crs_grp_types,crs_grp" \
                  " WHERE crs_grp_types.GrpTypCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                  " AND crs_grp.Open='Y' AND crs_grp.NumStudents<crs_grp.MaxStudents",
            GrpTypCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a type of group has available groups");

   /***** Get the number of types of group (0 if there are no groups available or 1 if there are) *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumGrpTypes) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a type of group has available groups.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumGrpTypes != 0);
  }

/****************************************************************************/
/****** Query list of group codes of a type to which a user belongs to ******/
/****************************************************************************/
// If GrpTypCod < 0 ==> get the groups of any type

void Grp_GetLstCodGrpsUsrBelongs (long GrpTypCod,long UsrCod,struct ListCodGrps *LstGrps)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumGrpSel;

   /***** Get groups which a user belong to from database *****/
   if (GrpTypCod < 0)	// Query the groups of any type
      sprintf (Query,"SELECT crs_grp.GrpCod FROM crs_grp_types,crs_grp,crs_grp_usr" \
                     " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                     " AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod='%ld'" \
                     " ORDER BY crs_grp_types.GrpTypName,crs_grp.GrpName",
               Gbl.CurrentCrs.Crs.CrsCod,UsrCod);
   else			// Query only the groups of specified type
      sprintf (Query,"SELECT crs_grp.GrpCod FROM crs_grp_types,crs_grp,crs_grp_usr" \
                     " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod" \
                     " AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod='%ld'" \
                     " ORDER BY crs_grp.GrpName",
               Gbl.CurrentCrs.Crs.CrsCod,GrpTypCod,UsrCod);
   LstGrps->NumGrps = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get the groups which a user belongs to");

   /***** Get the groups *****/
   if (LstGrps->NumGrps)
     {
      /***** Create a list of groups a the that belongs the user *****/
      if ((LstGrps->GrpCod = (long *) calloc (LstGrps->NumGrps,sizeof (long))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store codes of groups you belong to.");
      for (NumGrpSel=0; NumGrpSel<LstGrps->NumGrps; NumGrpSel++)
        {
         row = mysql_fetch_row (mysql_res);

         /* Get the code of group (row[0]) */
         if ((LstGrps->GrpCod[NumGrpSel] = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of group.");
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/******** Check if a group is in a list of groups which I belong to *********/
/****************************************************************************/

static bool Grp_ChkIfIBelongToGrp (long GrpCod,struct ListCodGrps *LstGrpsIBelong)
  {
   unsigned NumGrpIBelong;

   for (NumGrpIBelong=0; NumGrpIBelong<LstGrpsIBelong->NumGrps; NumGrpIBelong++)
      if (GrpCod == LstGrpsIBelong->GrpCod[NumGrpIBelong])
         return true;
   return false;
  }

/****************************************************************************/
/********** Query names of groups of a type which user belongs to ***********/
/****************************************************************************/

void Grp_GetNamesGrpsStdBelongsTo (long GrpTypCod,long UsrCod,char *GroupNames)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;

   /***** Get the names of groups which a user belongs to, from database *****/
   sprintf (Query,"SELECT crs_grp.GrpName FROM crs_grp,crs_grp_usr" \
                  " WHERE crs_grp.GrpTypCod='%ld' AND crs_grp.GrpCod=crs_grp_usr.GrpCod AND crs_grp_usr.UsrCod='%ld'" \
                  " ORDER BY crs_grp.GrpName",GrpTypCod,UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the names of groups a user belongs to");

   /***** Get the groups *****/
   GroupNames[0] = '\0';
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next group */
      row = mysql_fetch_row (mysql_res);

      /* El group name in row[0] */
      if (NumRow)
         strcat (GroupNames,", ");
      strcat (GroupNames,row[0]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/****************** Receive form to create a new group type *****************/
/****************************************************************************/

void Grp_RecFormNewGrpTyp (void)
  {
   extern const char *Txt_The_type_of_group_X_already_exists[Txt_NUM_LANGUAGES];
   char YN[1+1];

   /***** Get parameters from form *****/
   /* Get the name of group type */
   Par_GetParam (Par_PARAM_SINGLE,"GrpTypName",Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,MAX_LENGTH_GROUP_TYPE_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,MAX_LENGTH_GROUP_TYPE_NAME,true);

   /* Get whether it is mandatory to regisrer in any group of this type */
   Par_GetParam (Par_PARAM_SINGLE,"MandatoryRegistration",YN,1);
   Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /* Get whether it is possible to register in multiple groups of this type */
   Par_GetParam (Par_PARAM_SINGLE,"MultipleRegistration",YN,1);
   Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   if (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName[0])	// If there's a group type name
     {
      /***** If name of group type was in database... *****/
      if (Grp_CheckIfGroupTypeNameExists (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_type_of_group_X_already_exists[Gbl.Prefs.Language],
                  Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else	// Add new group type to database
         Grp_CreateGroupType ();
     }
   else	// If there isn't a group type name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre del nuevo tipo de grupo.");	// Need translation!
      Lay_ShowAlert (ERROR,Gbl.Message);
     }

   /***** Show the form again *****/
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******************** Receive form to create a new group ********************/
/****************************************************************************/

void Grp_RecFormNewGrp (void)
  {
   extern const char *Txt_The_group_X_already_exists[Txt_NUM_LANGUAGES];
   char UnsignedStr[10+1];

   /***** Get parameters from form *****/
   if ((Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = Grp_GetParamGrpTypeCod ()) >= 0) // Group type valid
     {
      /* Get group name */
      Par_GetParam (Par_PARAM_SINGLE,"GrpName",Gbl.CurrentCrs.Grps.GrpName,MAX_LENGTH_GROUP_NAME);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.CurrentCrs.Grps.GrpName,MAX_LENGTH_GROUP_NAME,true);

      /* Get maximum number of students */
      Par_GetParam (Par_PARAM_SINGLE,"MaxStudents",UnsignedStr,10);
      Gbl.CurrentCrs.Grps.MaxStudents = Grp_ConvertToNumMaxStdsGrp (UnsignedStr);

      if (Gbl.CurrentCrs.Grps.GrpName[0])	// If there's a group name
        {
         /***** If name of group was in database... *****/
         if (Grp_CheckIfGroupNameExists (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod,Gbl.CurrentCrs.Grps.GrpName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_group_X_already_exists[Gbl.Prefs.Language],
                     Gbl.CurrentCrs.Grps.GrpName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else	// Add new group to database
            Grp_CreateGroup ();
        }
      else	// If there isn't a group name
        {
         sprintf (Gbl.Message,"Debe especificar el nombre del nuevo grupo.");	// Need translation!
         Lay_ShowAlert (ERROR,Gbl.Message);
        }
     }
   else	// Group type not valid
      Lay_ShowAlert (ERROR,"Tipo de grupo no v&aacute;lido.");	// Need translation!

   /***** Show the form again *****/
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******************* Check if name of group type exists *********************/
/****************************************************************************/

static bool Grp_CheckIfGroupTypeNameExists (const char *GrpTypName,long GrpTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of group types with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_grp_types" \
                  " WHERE CrsCod='%ld' AND GrpTypName='%s' AND GrpTypCod<>'%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpTypName,GrpTypCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of type of group already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of type of group already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/************************ Check if name of group exists *********************/
/****************************************************************************/

static bool Grp_CheckIfGroupNameExists (long GrpTypCod,const char *GrpName,long GrpCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of groups with a type and a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_grp" \
                  " WHERE GrpTypCod='%ld' AND GrpName='%s' AND GrpCod<>'%ld'",
            GrpTypCod,GrpName,GrpCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of group already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of group already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/************************** Create a new group type *************************/
/****************************************************************************/

static void Grp_CreateGroupType (void)
  {
   extern const char *Txt_Created_new_type_of_group_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new group type *****/
   sprintf (Query,"INSERT INTO crs_grp_types (CrsCod,GrpTypName,Mandatory,Multiple)" \
                  " VALUES ('%ld','%s','%c','%c')",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,
            Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration ? 'Y' : 'N',
            Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration ? 'Y' : 'N');
   DB_QueryINSERT (Query,"can not create type of group");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_type_of_group_X[Gbl.Prefs.Language],
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/***************************** Create a new group ***************************/
/****************************************************************************/

static void Grp_CreateGroup (void)
  {
   extern const char *Txt_Created_new_group_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /*****  Create a new group *****/
   sprintf (Query,"INSERT INTO crs_grp (GrpTypCod,GrpName,MaxStudents,NumStudents,Open,FileZones)" \
                  " VALUES ('%ld','%s','%u','0','N','N')",
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod,Gbl.CurrentCrs.Grps.GrpName,Gbl.CurrentCrs.Grps.MaxStudents);
   DB_QueryINSERT (Query,"can not create group");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_group_X[Gbl.Prefs.Language],
            Gbl.CurrentCrs.Grps.GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/********************* Request removing of a group type *********************/
/****************************************************************************/

void Grp_ReqRemGroupType (void)
  {
   unsigned NumGrps;

   /***** Get the code of the group type *****/
   if ((Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = Grp_GetParamGrpTypeCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Check if this group type has groups *****/
   if ((NumGrps = Grp_CountNumGrpsInThisCrsOfType (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod)))	// Group type has groups ==> Ask for confirmation
      Grp_AskConfirmRemGrpTypWithGrps (NumGrps);
   else	// Group type has no groups ==> remove directly
      Grp_RemoveGroupTypeCompletely ();
  }

/****************************************************************************/
/************************* Request removal of a group ***********************/
/****************************************************************************/

void Grp_ReqRemGroup (void)
  {
   /***** Get group code *****/
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Confirm removing *****/
   Grp_AskConfirmRemGrp ();
  }

/*****************************************************************************/
/********** Ask for confirmation to remove a group type with groups **********/
/*****************************************************************************/

static void Grp_AskConfirmRemGrpTypWithGrps (unsigned NumGrps)
  {
   /***** Get data of the group type from database *****/
   Grp_GetDataOfGroupTypeByCod (&Gbl.CurrentCrs.Grps.GrpTyp);

   /***** Write message to ask confirmation of removing *****/
   if (NumGrps == 1)
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />" \
                           "Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
   else
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />" \
                           "Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,NumGrps);
   Lay_ShowAlert (WARNING,Gbl.Message);

   /***** Put button to confirm the removing *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActRemGrpTyp);
   Grp_PutParamGrpTypCod (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
   Lay_PutSendButton ("S&iacute;, eliminar tipo");	// Need translation!
   fprintf (Gbl.F.Out,"</form></div>\n");
  }

/*****************************************************************************/
/******************* Ask for confirmation to remove a group ******************/
/*****************************************************************************/

static void Grp_AskConfirmRemGrp (void)
  {
   struct GroupData GrpDat;
   unsigned NumStudents;

   /***** Get name and type of the group from database *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);
   NumStudents = Grp_GetNumStdsInGrp (Gbl.CurrentCrs.Grps.GrpCod);

   /***** Write message to ask confirmation of removing *****/
   if (NumStudents == 0)
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?",	// Need translation!
               GrpDat.GrpName);
   else if (NumStudents == 1)
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />" \
                           "Si lo hace, un alumno dejar&aacute; de pertenecer a ese grupo.",	// Need translation!
               GrpDat.GrpName);
   else
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />" \
                           "Si lo hace, %u alumnos dejar&aacute;n de pertenecer a ese grupo.",	// Need translation!
               GrpDat.GrpName,NumStudents);
   Lay_ShowAlert (WARNING,Gbl.Message);

   /***** Put button to confirm the removing *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActRemGrp);
   Grp_PutParamGrpCod (GrpDat.GrpCod);
   Lay_PutSendButton ("S&iacute;, eliminar grupo");
   fprintf (Gbl.F.Out,"</form></div>\n");
  }

/****************************************************************************/
/****************************** Remove a group type *************************/
/****************************************************************************/

void Grp_RemoveGroupType (void)
  {
   /***** Get param with code of group type *****/
   if ((Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = Grp_GetParamGrpTypeCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of type of group is missing.");

   /***** Remove group type and its groups *****/
   Grp_RemoveGroupTypeCompletely ();
  }

/****************************************************************************/
/******************************* Remove a group *****************************/
/****************************************************************************/

void Grp_RemoveGroup (void)
  {
   /***** Get param with group code *****/
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Remove group *****/
   Grp_RemoveGroupCompletely ();
  }

/****************************************************************************/
/***** Remove a group type from data base and remove group common zones *****/
/****************************************************************************/

static void Grp_RemoveGroupTypeCompletely (void)
  {
   extern const char *Txt_Type_of_group_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];

   /***** Get name and type of the group from database *****/
   Grp_GetDataOfGroupTypeByCod (&Gbl.CurrentCrs.Grps.GrpTyp);

   /***** Remove file zones of all the groups of this type *****/
   Brw_RemoveZonesOfGroupsOfType (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);

   /***** Remove the associations of assignments to groups of this type *****/
   Asg_RemoveGroupsOfType (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);

   /***** Change all groups of this type in course timetable *****/
   sprintf (Query,"UPDATE timetable_crs SET GrpCod='-1'" \
                  " WHERE GrpCod IN (SELECT GrpCod FROM crs_grp WHERE GrpTypCod='%ld')",
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
   DB_QueryUPDATE (Query,"can not update all groups of a type in course timetable");

   /***** Remove all the students in groups of this type *****/
   sprintf (Query,"DELETE FROM crs_grp_usr WHERE GrpCod IN" \
                  " (SELECT GrpCod FROM crs_grp WHERE GrpTypCod='%ld')",
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
   DB_QueryDELETE (Query,"can not remove users from all groups of a type");

   /***** Remove all the groups of this type *****/
   sprintf (Query,"DELETE FROM crs_grp WHERE GrpTypCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
   DB_QueryDELETE (Query,"can not remove groups of a type");

   /***** Remove the group type *****/
   sprintf (Query,"DELETE FROM crs_grp_types WHERE GrpTypCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
   DB_QueryDELETE (Query,"can not remove a type of group");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Type_of_group_X_removed[Gbl.Prefs.Language],
            Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******* Remove a group from data base and remove group common zone *********/
/****************************************************************************/

static void Grp_RemoveGroupCompletely (void)
  {
   extern const char *Txt_Group_X_removed[Txt_NUM_LANGUAGES];
   struct GroupData GrpDat;
   char Query[512];

   /***** Get name and type of the group from database *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Remove file zones of this group *****/
   Brw_RemoveGrpZonesVerbose (&GrpDat);

   /***** Remove this group from all the assignments *****/
   Asg_RemoveGroup (GrpDat.GrpCod);

   /***** Change this group in course timetable *****/
   sprintf (Query,"UPDATE timetable_crs SET GrpCod='-1' WHERE GrpCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryUPDATE (Query,"can not update a group in course timetable");

   /***** Remove all the students in this group *****/
   sprintf (Query,"DELETE FROM crs_grp_usr WHERE GrpCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryDELETE (Query,"can not remove users from a group");

   /***** Remove the group *****/
   sprintf (Query,"DELETE FROM crs_grp WHERE GrpCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryDELETE (Query,"can not remove a group");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Group_X_removed[Gbl.Prefs.Language],
            GrpDat.GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******************************* Open a group *****************************/
/****************************************************************************/

void Grp_OpenGroup (void)
  {
   struct GroupData GrpDat;
   char Query[512];

   /***** Get group code *****/
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Get group data from database *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Update the table of groups changing open/close status *****/
   sprintf (Query,"UPDATE crs_grp SET Open='Y' WHERE GrpCod='%ld'",Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryUPDATE (Query,"can not open a group");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"El grupo <strong>%s</strong> est&aacute; ahora abierto.",	// Need translation!
            GrpDat.GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.Open = true;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******************************* Close a group ******************************/
/****************************************************************************/

void Grp_CloseGroup (void)
  {
   struct GroupData GrpDat;
   char Query[512];

   /***** Get group code *****/
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Get group data from database *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Update the table of groups changing open/close status *****/
   sprintf (Query,"UPDATE crs_grp SET Open='N' WHERE GrpCod='%ld'",Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryUPDATE (Query,"can not close a group");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"El grupo <strong>%s</strong> est&aacute; ahora cerrado.",	// Need translation!
            GrpDat.GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.Open = false;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/************************ Enable file zones of a group **********************/
/****************************************************************************/

void Grp_EnableFileZonesGrp (void)
  {
   struct GroupData GrpDat;
   char Query[512];

   /***** Get group code *****/
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Get group data from database *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Update the table of groups changing file zones status *****/
   sprintf (Query,"UPDATE crs_grp SET FileZones='Y' WHERE GrpCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryUPDATE (Query,"can not enable file zones of a group");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"Las zonas de archivos del grupo <strong>%s</strong>" \
                        " est&aacute;n ahora habilitadas.",	// Need translation!
            GrpDat.GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.FileZones = true;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/*********************** Disable file zones of a group **********************/
/****************************************************************************/

void Grp_DisableFileZonesGrp (void)
  {
   struct GroupData GrpDat;
   char Query[512];

   /***** Get group code *****/
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /***** Get group data from database *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Update the table of groups changing file zones status *****/
   sprintf (Query,"UPDATE crs_grp SET FileZones='N' WHERE GrpCod='%ld'",
            Gbl.CurrentCrs.Grps.GrpCod);
   DB_QueryUPDATE (Query,"can not disable file zones of a group");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"Las zonas de archivos del grupo <strong>%s</strong>" \
                        " est&aacute;n ahora inhabilitadas.",	// Need translation!
            GrpDat.GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.FileZones = false;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/*********************** Change the group type of a group *******************/
/****************************************************************************/

void Grp_ChangeGroupType (void)
  {
   extern const char *Txt_The_group_X_already_exists[Txt_NUM_LANGUAGES];
   long NewGrpTypCod;
   struct GroupData GrpDat;
   char Query[512];

   /***** Get parameters from form *****/
   /* Get group code */
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /* Get the new group type */
   NewGrpTypCod = Grp_GetParamGrpTypeCod ();

   /* Get from the database the type and the name of the group */
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** If group was in database... *****/
   if (Grp_CheckIfGroupNameExists (NewGrpTypCod,GrpDat.GrpName,-1L))
     {
      sprintf (Gbl.Message,Txt_The_group_X_already_exists[Gbl.Prefs.Language],
               GrpDat.GrpName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /* Update the table of groups changing old type by new type */
      sprintf (Query,"UPDATE crs_grp SET GrpTypCod='%ld' WHERE GrpCod='%ld'",
               NewGrpTypCod,Gbl.CurrentCrs.Grps.GrpCod);
      DB_QueryUPDATE (Query,"can not update the type of a group");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"Se ha cambiado el tipo de grupo del grupo <strong>%s</strong>.",	// Need translation!
               GrpDat.GrpName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = NewGrpTypCod;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/************ Change mandatory registration to a group of a type ************/
/****************************************************************************/

void Grp_ChangeMandatGrpTyp (void)
  {
   char Query[1024];
   char YN[1+1];
   int NewMandatoryRegistration;

   /***** Get parameters of the form *****/
   /* Get the cdigo of type of group */
   if ((Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = Grp_GetParamGrpTypeCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of type of group is missing.");

   /* Get the new type of registration (mandatory or voluntaria) of this type of group */
   Par_GetParam (Par_PARAM_SINGLE,"MandatoryRegistration",YN,1);
   NewMandatoryRegistration = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /* Get from the database the name of the type and the old type of registration */
   Grp_GetDataOfGroupTypeByCod (&Gbl.CurrentCrs.Grps.GrpTyp);

   /***** Check if the old type of registration match the new
          (this happens when return is pressed without changes in the form) *****/
   if (Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration == NewMandatoryRegistration)
     {
      sprintf (Gbl.Message,"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      /***** Update of the table of types of group changing the old type of registration by the new *****/
      sprintf (Query,"UPDATE crs_grp_types SET Mandatory='%c' WHERE GrpTypCod='%ld'",
               NewMandatoryRegistration ? 'Y' : 'N',Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
      DB_QueryUPDATE (Query,"can not update registration type of a type of group");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La adscripci&oacute;n de alumnos a grupos del tipo <strong>%s</strong> es ahora <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,NewMandatoryRegistration ? "obligatoria" : "voluntaria");	// Need translation!
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.GrpTyp.MandatoryRegistration = NewMandatoryRegistration;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******* Change multiple registration to one or more groups of a type *******/
/****************************************************************************/

void Grp_ChangeMultiGrpTyp (void)
  {
   char Query[1024];
   char YN[1+1];
   int NewMultipleRegistration;

   /***** Get parameters from the form *****/
   /* Get the code of type of group */
   if ((Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = Grp_GetParamGrpTypeCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of type of group is missing.");

   /* Get the new type of registration (single or multiple) of this type of group */
   Par_GetParam (Par_PARAM_SINGLE,"MultipleRegistration",YN,1);
   NewMultipleRegistration = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /* Get from the database the name of the type and the old type of registration */
   Grp_GetDataOfGroupTypeByCod (&Gbl.CurrentCrs.Grps.GrpTyp);

   /***** Check if the old type of registration coincide with the new (esto pasa cuando se pulsa return sin cambiar nada in the form) *****/
   if (Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration == NewMultipleRegistration)
     {
      sprintf (Gbl.Message,"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      /***** Update of the table of types of group changing the old type of registration by the new *****/
      sprintf (Query,"UPDATE crs_grp_types SET Multiple='%c' WHERE GrpTypCod='%ld'",
               NewMultipleRegistration ? 'Y' : 'N',Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
      DB_QueryUPDATE (Query,"can not update registration type of a type of group");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"Ahora cada alumno <strong>%s</strong> de tipo <strong>%s</strong>.",	// Need translation!
               NewMultipleRegistration ? "puede pertenecer a varios grupos" : "s&oacute;lo puede pertenecer a un grupo",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration = NewMultipleRegistration;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/***************** Change maximum of students in a group ********************/
/****************************************************************************/

void Grp_ChangeMaxStdsGrp (void)
  {
   struct GroupData GrpDat;
   char Query[1024];
   int NewMaxStds;
   char UnsignedStr[10+1];

   /***** Get parameters of the form *****/
   /* Get group code */
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /* Get the new maximum number of students of the group */
   Par_GetParam (Par_PARAM_SINGLE,"MaxStudents",UnsignedStr,10);
   NewMaxStds = Grp_ConvertToNumMaxStdsGrp (UnsignedStr);

   /* Get from the database the type, name, and antiguo maximum of students of the group */
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Check if the old maximum of students equals the new one (this happens when user press return without change the form) *****/
   if (GrpDat.MaxStudents == NewMaxStds)
     {
      sprintf (Gbl.Message,"El n&ordm; m&aacute;ximo de alumnos del grupo <strong>%s</strong> no ha cambiado.",	// Need translation!
               GrpDat.GrpName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      /***** Update the table of groups changing the old maximum of students to the new *****/
      sprintf (Query,"UPDATE crs_grp SET MaxStudents='%d' WHERE GrpCod='%ld'",
               NewMaxStds,Gbl.CurrentCrs.Grps.GrpCod);
      DB_QueryUPDATE (Query,"can not update the maximum number of students in a group");

      /***** Write message to show the change made *****/
      if (NewMaxStds == INT_MAX)
         sprintf (Gbl.Message,"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de alumnos.",	// Need translation!
                  GrpDat.GrpName);
      else
         sprintf (Gbl.Message,"El n&ordm; m&aacute;ximo de alumnos del grupo <strong>%s</strong> es ahora <strong>%d</strong>.",	// Need translation!
                  GrpDat.GrpName,NewMaxStds);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.CurrentCrs.Grps.MaxStudents = NewMaxStds;
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/************ Write the number maximum of students in a group **************/
/****************************************************************************/

static void Grp_WriteMaxStdsGrp (unsigned MaxStudents)
  {
   if (MaxStudents > 10000)
      fprintf (Gbl.F.Out,"-");
   else
      fprintf (Gbl.F.Out,"%u",MaxStudents);
  }

/****************************************************************************/
/********* Convert string to maximum number of students in a group **********/
/****************************************************************************/

static unsigned Grp_ConvertToNumMaxStdsGrp (const char *StrMaxStudents)
  {
   unsigned MaxStudents;

   if (sscanf (StrMaxStudents,"%u",&MaxStudents) != 1)
      return INT_MAX;
   else if (MaxStudents > 10000)
      return INT_MAX;
   return MaxStudents;
  }

/****************************************************************************/
/***************************** Rename a group type **************************/
/****************************************************************************/

void Grp_RenameGroupType (void)
  {
   extern const char *Txt_The_type_of_group_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[1024];
   char NewNameGrpTyp[MAX_LENGTH_GROUP_TYPE_NAME+1];

   /***** Get parameters from form *****/
   /* Get the code of the group type */
   if ((Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = Grp_GetParamGrpTypeCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of type of group is missing.");

   /* Get the new name for the group type */
   Par_GetParam (Par_PARAM_SINGLE,"GrpTypName",NewNameGrpTyp,MAX_LENGTH_GROUP_TYPE_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewNameGrpTyp,MAX_LENGTH_GROUP_TYPE_NAME,true);

   /***** Get from the database the old name of the group type *****/
   Grp_GetDataOfGroupTypeByCod (&Gbl.CurrentCrs.Grps.GrpTyp);

   /***** Check if new name is empty *****/
   if (!NewNameGrpTyp[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del tipo de grupo <strong>%s</strong> vac&iacute;o.",	// Need translation!
               Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,NewNameGrpTyp))	// Different names
        {
         /***** If group type was in database... *****/
         if (Grp_CheckIfGroupTypeNameExists (NewNameGrpTyp,Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod))
           {
            sprintf (Gbl.Message,Txt_The_type_of_group_X_already_exists[Gbl.Prefs.Language],
                     NewNameGrpTyp);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE crs_grp_types SET GrpTypName='%s' WHERE GrpTypCod='%ld'",
                     NewNameGrpTyp,Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod);
            DB_QueryUPDATE (Query,"can not update the type of a group");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El tipo de grupo <strong>%s</strong> ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,NewNameGrpTyp);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.",	// Need translation!
                  NewNameGrpTyp);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,NewNameGrpTyp);
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******************************* Rename a group *****************************/
/****************************************************************************/

void Grp_RenameGroup (void)
  {
   extern const char *Txt_The_group_X_already_exists[Txt_NUM_LANGUAGES];
   struct GroupData GrpDat;
   char Query[512];
   char NewNameGrp[MAX_LENGTH_GROUP_NAME+1];

   /***** Get parameters from form *****/
   /* Get the code of the group */
   if ((Gbl.CurrentCrs.Grps.GrpCod = Grp_GetParamGrpCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of group is missing.");

   /* Get the new name for the group */
   Par_GetParam (Par_PARAM_SINGLE,"GrpName",NewNameGrp,MAX_LENGTH_GROUP_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewNameGrp,MAX_LENGTH_GROUP_NAME,true);

   /***** Get from the database the type and the old name of the group *****/
   GrpDat.GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
   Grp_GetDataOfGroupByCod (&GrpDat);

   /***** Check if new name is empty *****/
   if (!NewNameGrp[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del grupo <strong>%s</strong> vac&iacute;o.",	// Need translation!
               GrpDat.GrpName);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (GrpDat.GrpName,NewNameGrp))	// Different names
        {
         /***** If group was in database... *****/
         if (Grp_CheckIfGroupNameExists (GrpDat.GrpTypCod,NewNameGrp,Gbl.CurrentCrs.Grps.GrpCod))
           {
            sprintf (Gbl.Message,Txt_The_group_X_already_exists[Gbl.Prefs.Language],
                     NewNameGrp);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE crs_grp SET GrpName='%s' WHERE GrpCod='%ld'",
                     NewNameGrp,Gbl.CurrentCrs.Grps.GrpCod);
            DB_QueryUPDATE (Query,"can not update the name of a group");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El grupo <strong>%s</strong> ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     GrpDat.GrpName,NewNameGrp);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del grupo <strong>%s</strong> no ha cambiado.",	// Need translation!
                  NewNameGrp);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (Gbl.CurrentCrs.Grps.GrpName,NewNameGrp);
   Grp_ReqEditGroups ();
  }

/****************************************************************************/
/******************* Get parameter with code of group type ******************/
/****************************************************************************/

static long Grp_GetParamGrpTypeCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of group type *****/
   Par_GetParam (Par_PARAM_SINGLE,"GrpTypCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/*********************** Get parameter with group code **********************/
/****************************************************************************/

long Grp_GetParamGrpCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with group code *****/
   Par_GetParam (Par_PARAM_SINGLE,"GrpCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/****************** Write parameter with code of group type *****************/
/****************************************************************************/

static void Grp_PutParamGrpTypCod (long GrpTypCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"GrpTypCod\" value=\"%ld\" />",
            GrpTypCod);
  }

/****************************************************************************/
/********************* Write parameter with code of group *******************/
/****************************************************************************/

void Grp_PutParamGrpCod (long GrpCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"GrpCod\" value=\"%ld\" />",
            GrpCod);
  }

/****************************************************************************/
/************************* Get list of group codes selected *****************/
/****************************************************************************/

void Grp_GetLstCodsGrpIWant (struct ListCodGrps *LstGrps)
  {
   unsigned NumGrpTyp;
   char Param[8+10+1];
   char LongStr[1+10+1];
   char **LstCodGrps;
   const char *Ptr;
   unsigned NumGrpSel;

   /***** Allocate memory for the lists of group codes of cada type *****/
   if ((LstCodGrps = (char **) calloc (Gbl.CurrentCrs.Grps.GrpTypes.Num,sizeof (char *))) == NULL)
      Lay_ShowErrorAndExit ("Not enoguh memory to store codes of groups in which you want to be registered.");

   /***** Get lists with the groups selected of each type and count the total number of groups selected *****/
   LstGrps->NumGrps = 0;
   for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
     {
      /***** Allocate memory for the list of group codes of this type *****/
      if ((LstCodGrps[NumGrpTyp] = (char *) malloc ((1+10+1) * Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps)) == NULL)
         Lay_ShowErrorAndExit ("Not enoguh memory to store codes of groups in which you want to be registered.");

      /***** Get the multiple parameter code of group of this type *****/
      sprintf (Param,"GrpCod%ld",Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
      Par_GetParam (Par_PARAM_MULTIPLE,Param,LstCodGrps[NumGrpTyp],((1+10+1) * Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps) - 1);
      if (LstCodGrps[NumGrpTyp][0])
        {
         /***** Count the number of groups selected of this type a partir of LstCodGrps[NumGrpTyp] *****/
         for (Ptr = LstCodGrps[NumGrpTyp], NumGrpSel=0; *Ptr; NumGrpSel++)
            Str_GetNextStrUntilSeparParamMult (&Ptr,LongStr,1+10);

         /***** Add the number of groups selected of this type to the number of groups selected total *****/
         LstGrps->NumGrps += NumGrpSel;
        }
      }

   /***** Create the list (argument passed to this function) with all the groups selected (of all the types) *****/
   if (LstGrps->NumGrps)
     {
      if ((LstGrps->GrpCod = (long *) calloc (LstGrps->NumGrps,sizeof (long))) == NULL)
         Lay_ShowErrorAndExit ("Not enoguh memory to store codes of groups in which you want to be registered.");

      /***** Get the groups *****/
      NumGrpSel = 0;
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
        {
         /* Add the groups selected of this type to the complete list of groups selected */
         for (Ptr = LstCodGrps[NumGrpTyp]; *Ptr; NumGrpSel++)
           {
            Str_GetNextStrUntilSeparParamMult (&Ptr,LongStr,1+10);
            LstGrps->GrpCod[NumGrpSel] = Str_ConvertStrCodToLongCod (LongStr);
           }
         /* Free memory used by the list of group codes of this type */
         free ((void *) LstCodGrps[NumGrpTyp]);
        }
     }

   /***** Free memory used by the lists of group codes of each type *****/
   free ((void *) LstCodGrps);
  }

/****************************************************************************/
/************************* Liberar list of group codes **********************/
/****************************************************************************/

void Grp_FreeListCodGrp (struct ListCodGrps *LstGrps)
  {
   if (LstGrps->NumGrps)
      free ((void *) LstGrps->GrpCod);
  }

/****************************************************************************/
/*********** Put parameter that indicates all groups selected ***************/
/****************************************************************************/

void Grp_PutParamAllGroups (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AllGroups\" value=\"Y\" />");
  }

/****************************************************************************/
/****** Parameter to show only my groups or all groups or in timetable ******/
/****************************************************************************/

void Grp_PutParamWhichGrps (void)
  {
   Grp_GetParamWhichGrps ();

   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"WhichGrps\" value=\"%u\" />",
            (unsigned) Gbl.CurrentCrs.Grps.WhichGrps);
  }

/****************************************************************************/
/********** Show selector to choice whether to show only my groups **********/
/********** or all groups in timetable                            **********/
/****************************************************************************/

void Grp_ShowSelectorWhichGrps (void)
  {
   extern const char *Txt_Show_WHICH_groups[2][Txt_NUM_LANGUAGES];
   Grp_WhichGroups_t WhichGrps;

   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   for (WhichGrps=Grp_ONLY_MY_GROUPS; WhichGrps<=Grp_ALL_GROUPS; WhichGrps++)
     {
      fprintf (Gbl.F.Out,"<td align=\"left\" class=\"DAT\">" \
                         "<input type=\"radio\" name=\"WhichGrps\" value=\"%u\"",
               (unsigned) WhichGrps);
      if (WhichGrps == Gbl.CurrentCrs.Grps.WhichGrps)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," onclick=\"javascript:document.forms[%d].submit();\" />" \
                         "%s</td>",
               Gbl.NumForm,Txt_Show_WHICH_groups[WhichGrps][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</tr></table>");
  }

/****************************************************************************/
/***** Get whether to show only my groups or all groups or in timetable *****/
/****************************************************************************/

void Grp_GetParamWhichGrps (void)
  {
   static bool FirstTime = true;
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   if (FirstTime)
     {
      FirstTime = false;

      /***** Get groups type (my groups or all groups) *****/
      Par_GetParam (Par_PARAM_SINGLE,"WhichGrps",UnsignedStr,1);
      if (UnsignedStr[0])
        {
         if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
            Lay_ShowErrorAndExit ("Types of groups to show is missing.");
         if (UnsignedNum >= 2)
            Lay_ShowErrorAndExit ("Wrong types of groups to show.");
         Gbl.CurrentCrs.Grps.WhichGrps = (Grp_WhichGroups_t) UnsignedNum;
        }
      else	// This parameter doesn't exist ==> set default value
         switch (Gbl.CurrentAct)
           {
            case ActSeeCrsTimTbl:
            case ActPrnCrsTimTbl:
            case ActSeeAsg:
               Gbl.CurrentCrs.Grps.WhichGrps = Gbl.Usrs.Me.IBelongToThisCrs ? Grp_ONLY_MY_GROUPS :	// If I belong to this course ==> see only my groups
                                                                              Grp_ALL_GROUPS;		// If I don't belong to this course ==> see all groups
	       break;
            case ActSeeMyTimTbl:
            case ActPrnMyTimTbl:
               Gbl.CurrentCrs.Grps.WhichGrps = Grp_ONLY_MY_GROUPS;	// By default, see only my groups
	       break;
            default:	// Control never must enter here
               Gbl.CurrentCrs.Grps.WhichGrps = Grp_ALL_GROUPS;
           }
     }
  }
