// swad_grp.h: groups

#ifndef _SWAD_GRP
#define _SWAD_GRP
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_crs.h"
#include "swad_usr.h"

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

#define MAX_LENGTH_GROUP_TYPE_NAME	255
#define MAX_LENGTH_GROUP_NAME		255

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

// Related with groups
struct GroupData
  {
   long GrpCod;
   long GrpTypCod;
   char GrpTypName[MAX_LENGTH_GROUP_TYPE_NAME+1];
   char GrpName[MAX_LENGTH_GROUP_NAME+1];
   unsigned MaxStudents,NumStudents;
   int  Vacant;
   bool Open;					// Group is open?
   bool FileZones;				// Group has file zones?
   bool MultipleRegistration;
  };
struct Group
  {
   long GrpCod;					// Code of group
   char GrpName[MAX_LENGTH_GROUP_NAME+1];	// Name of group
   unsigned MaxStudents;
   unsigned NumStudents;			// Number of students in the group
   bool Open;					// Group is open?
   bool FileZones;				// Group has file zones?
  };
struct GroupType
  {
   long GrpTypCod;					// Code of type of group
   char GrpTypName[MAX_LENGTH_GROUP_TYPE_NAME+1];	// Name of type of group
   bool MandatoryRegistration;				// Registration is mandatory?
   bool MultipleRegistration;				// Registration is miltiple?
   unsigned NumGrps;					// Number of groups of this type
   struct Group *LstGrps;				// List of groups of this type
  };
struct GroupTypes
  {
   struct GroupType *LstGrpTypes;		// List of types of group
   unsigned Num;				// Number of types of group
   unsigned NumGrpsTotal;			// Number of groups of any type
   int NestedCalls;				// Number of nested calls to the function that allocates memory for this list
  };
struct ListCodGrps
  {
   long *GrpCod;
   unsigned NumGrps;
  };
struct ListGrpsAlreadySelec
  {
   long GrpTypCod;
   bool AlreadySelected;
  };
typedef enum {Grp_ONLY_MY_GROUPS,Grp_ALL_GROUPS} Grp_WhichGroups_t;

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Grp_WriteNamesOfSelectedGrps (void);
void Grp_ReqEditGroups (void);
void Grp_ReqRegisterInGrps (void);
void Grp_PutParamsCodGrps (void);
void Grp_ShowFormToSelectSeveralGroups (Act_Action_t NextAction);
unsigned Grp_GetParCodsSeveralGrpsToShowUsrs (void);
void Grp_GetParCodsSeveralGrpsToEditAssignments (void);
void Grp_FreeListCodSelectedGrps (void);
void Grp_ChangeMyGrps (void);
bool Grp_CheckIfSelectionGrpsIsValid (struct ListCodGrps *LstGrps);
void Grp_RegisterUsrIntoGroups (struct UsrData *UsrDat,struct ListCodGrps *LstGrps);
unsigned Grp_RemoveUsrFromGroups (struct UsrData *UsrDat,struct ListCodGrps *LstGrps);
void Grp_RemUsrFromAllGrpsInCrs (struct UsrData *UsrDat,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose);
void Grp_RemUsrFromAllGrps (struct UsrData *UsrDat,Cte_QuietOrVerbose_t QuietOrVerbose);
void Grp_ListGrpsToEditAssignments (struct GroupType *GrpTyp,long AsgCod);
void Grp_ListGrpsToAddOrRemUsrs (struct GroupType *GrpTyp);
void Grp_GetListGrpTypesInThisCrs (void);
void Grp_FreeListGrpTypesAndGrps (void);
void Grp_GetListGrpTypesAndGrpsInThisCrs (void);
unsigned Grp_CountNumGrpsInCurrentCrs (void);
unsigned long Grp_GetAllGrpTypesInThisCrs (MYSQL_RES **mysql_res);
unsigned long Grp_GetGrpsOfType (long GrpTypCod,MYSQL_RES **mysql_res);
void Grp_GetDataOfGroupByCod (struct GroupData *GrpDat);
bool Grp_CheckIfGroupExists (long GrpCod);
unsigned Grp_GetNumStdsInGrp (long GrpCod);
bool Grp_CheckIfIBelongToGrp (long GrpCod);
unsigned Grp_NumGrpTypesMandatIDontBelong (void);
void Grp_GetLstCodGrpsUsrBelongs (long GrpTypCod,long UsrCod,struct ListCodGrps *LstGrps);
void Grp_GetNamesGrpsStdBelongsTo (long GrpTypCod,long UsrCod,char *GrpNames);
void Grp_RecFormNewGrpTyp (void);
void Grp_RecFormNewGrp (void);
void Grp_ReqRemGroupType (void);
void Grp_ReqRemGroup (void);
void Grp_RemoveGroupType (void);
void Grp_RemoveGroup (void);
void Grp_OpenGroup (void);
void Grp_CloseGroup (void);
void Grp_EnableFileZonesGrp (void);
void Grp_DisableFileZonesGrp (void);
void Grp_ChangeGroupType (void);
void Grp_ChangeMandatGrpTyp (void);
void Grp_ChangeMultiGrpTyp (void);
void Grp_ChangeMaxStdsGrp (void);
void Grp_RenameGroupType (void);
void Grp_RenameGroup (void);
long Grp_GetParamGrpCod (void);
void Grp_PutParamGrpCod (long GrpCod);
void Grp_GetLstCodsGrpIWant (struct ListCodGrps *LstGrps);
void Grp_FreeListCodGrp (struct ListCodGrps *LstGrps);
void Grp_PutParamAllGroups (void);

void Grp_PutParamWhichGrps (void);
void Grp_ShowSelectorWhichGrps (void);
void Grp_GetParamWhichGrps (void);

#endif
