// swad_hld.c: holidays

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_hld.h"
#include "swad_par.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Hld_GetParamHldOrderType (void);
static void Hld_PutFormToEditHlds (void);
static Hld_HolidayType_t Hld_GetParamHldType (void);
static Hld_HolidayType_t Hld_GetTypeOfHoliday (const char *UnsignedStr);
static void Hld_ListHolidaysForEdition (void);
static void Hld_PutParamHldCod (long HldCod);
static void Hld_ChangeDate (Hld_StartOrEndDate_t StartOrEndDate);
static bool Hld_CheckIfHolidayNameExists (const char *Name,long HldCod);
static void Hld_PutFormToCreateHoliday (void);
static void Hld_PutHeadHolidays (void);
static void Hld_CreateHoliday (struct Holiday *Hld);

/****************************************************************************/
/*************************** List all the holidays **************************/
/****************************************************************************/

void Hld_SeeHolidays (void)
  {
   extern const char *Txt_HOLIDAYS_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_HOLIDAYS_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_End_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Holiday[Txt_NUM_LANGUAGES];
   extern const char *Txt_All_places[Txt_NUM_LANGUAGES];
   Hld_OrderType_t Order;
   unsigned NumHld;

   /***** Get parameter with the type of order in the list of holidays *****/
   Hld_GetParamHldOrderType ();

   /***** Get list of holidays *****/
   Hld_GetListHolidays ();

   /***** Put link (form) to edit holidays *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      Hld_PutFormToEditHlds ();

   /***** Table head *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr>");
   for (Order=Hld_ORDER_BY_PLACE; Order<=Hld_ORDER_BY_START_DATE; Order++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">");
      Act_FormStart (ActSeeHld);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_HOLIDAYS_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
      if (Order == Gbl.Hlds.SelectedOrderType)
         fprintf (Gbl.F.Out,"<u>");
      fprintf (Gbl.F.Out,"%s",Txt_HOLIDAYS_ORDER[Order][Gbl.Prefs.Language]);
      if (Order == Gbl.Hlds.SelectedOrderType)
         fprintf (Gbl.F.Out,"</u>");
      fprintf (Gbl.F.Out,"</acronym></a></form></td>");
     }
   fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">&nbsp;%s&nbsp;</td>",Txt_End_date[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">%s</td></tr>",Txt_Holiday[Gbl.Prefs.Language]);


   /***** Write all the holidays *****/
   for (NumHld=0; NumHld<Gbl.Hlds.Num; NumHld++)
     {
      /* Write data of this holiday */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">%s</td>",
               (Gbl.Hlds.Lst[NumHld].PlcCod <= 0) ? Txt_All_places[Gbl.Prefs.Language] : Gbl.Hlds.Lst[NumHld].PlaceFullName);
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"DAT\">&nbsp;%02u/%02u/%04u</td>",
               Gbl.Hlds.Lst[NumHld].StartDate.Day,
               Gbl.Hlds.Lst[NumHld].StartDate.Month,
               Gbl.Hlds.Lst[NumHld].StartDate.Year);
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"DAT\">&nbsp;");
      switch (Gbl.Hlds.Lst[NumHld].HldTyp)
        {
         case Hld_HOLIDAY:
            fprintf (Gbl.F.Out,"-");
            break;
         case Hld_NON_SCHOOL_PERIOD:
            fprintf (Gbl.F.Out,"%02u/%02u/%04u",
                     Gbl.Hlds.Lst[NumHld].EndDate.Day,
                     Gbl.Hlds.Lst[NumHld].EndDate.Month,
                     Gbl.Hlds.Lst[NumHld].EndDate.Year);
        }
      fprintf (Gbl.F.Out,"</td><td align=\"left\" class=\"DAT\">&nbsp;%s</td></tr>",
               Gbl.Hlds.Lst[NumHld].Name);
     }

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of holidays *****/
   Hld_FreeListHolidays ();
  }

/*****************************************************************************/
/********** Get parameter with the type or order in list of holidays ***********/
/*****************************************************************************/

static void Hld_GetParamHldOrderType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Hlds.SelectedOrderType = (Hld_OrderType_t) UnsignedNum;
   else
      Lay_ShowErrorAndExit ("Sort order is missing.");
  }

/****************************************************************************/
/**** Put a hidden parameter with the type of order in list of holidays *****/
/****************************************************************************/

void Hld_PutHiddenParamHldOrderType (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Hlds.SelectedOrderType);
  }

/****************************************************************************/
/********************* Put a link (form) to edit holidays *******************/
/****************************************************************************/

static void Hld_PutFormToEditHlds (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiHld);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************ Put forms to edit degree types ********************/
/****************************************************************************/

void Hld_EditHolidays (void)
  {
   extern const char *Txt_Holidays[Txt_NUM_LANGUAGES];

   /***** Get list of places *****/
   Plc_GetListPlaces ();

   /***** Get list of holidays *****/
   Hld_GetListHolidays ();

   /***** Help message *****/
   if (Gbl.Hlds.Num) // Holidays found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir una nueva festividad seleccione el lugar, el tipo, rellene la(s) fecha(s) y el texto de la festividad," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar una festividad existente pulse el aspa roja.<br />" \
                          "Para modificar una festividad existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Holidays[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No degree types found
      Lay_ShowAlert (INFO,"No existen d&iacute;as festivos.<br />" \
                          "Para a&ntilde;adir una festividad seleccione el lugar, el tipo, rellene la(s) fecha(s) y el texto de la festividad," \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit holidays *****/
   /* Start table with holidays */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current holidays */
   if (Gbl.Hlds.Num)
      Hld_ListHolidaysForEdition ();

   /* Put a form to create a new holiday */
   Hld_PutFormToCreateHoliday ();

   /* End of table with holidays */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of holidays *****/
   Hld_FreeListHolidays ();

   /***** Free list of places *****/
   Plc_FreeListPlaces ();
  }

/****************************************************************************/
/*************************** List all the holidays **************************/
/****************************************************************************/

void Hld_GetListHolidays (void)
  {
   char OrderBySubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumHld;
   struct Holiday *Hld;

   if (Gbl.Hlds.LstIsRead)
      Hld_FreeListHolidays ();

   /***** Get holidays from database *****/
   switch (Gbl.Hlds.SelectedOrderType)
     {
      case Hld_ORDER_BY_PLACE:
         sprintf (OrderBySubQuery,"Place,StartDate");
         break;
      case Hld_ORDER_BY_START_DATE:
         sprintf (OrderBySubQuery,"StartDate,Place");
     }
   sprintf (Query,"(SELECT holidays.HldCod,holidays.PlcCod,places.FullName as Place,HldTyp,DATE_FORMAT(holidays.StartDate,'%%Y%%m%%d') AS StartDate,DATE_FORMAT(holidays.EndDate,'%%Y%%m%%d') AS EndDate,holidays.Name" \
                  " FROM holidays,places" \
                  " WHERE holidays.PlcCod=places.PlcCod)" \
                  " UNION " \
                  "(SELECT HldCod,PlcCod,'' as Place,HldTyp,DATE_FORMAT(StartDate,'%%Y%%m%%d') AS StartDate,DATE_FORMAT(EndDate,'%%Y%%m%%d') AS EndDate,Name" \
                  " FROM holidays" \
                  " WHERE PlcCod NOT IN (SELECT DISTINCT PlcCod FROM places))" \
                  " ORDER BY %s",
            OrderBySubQuery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get holidays");

   if (NumRows) // Holidays found...
     {
      Gbl.Hlds.Num = (unsigned) NumRows;

      /***** Create list of holidays *****/
      if ((Gbl.Hlds.Lst = (struct Holiday *) calloc (NumRows,sizeof (struct Holiday))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store holidays.");

      /***** Get the holidays *****/
      for (NumHld=0; NumHld<Gbl.Hlds.Num; NumHld++)
        {
         Hld = &(Gbl.Hlds.Lst[NumHld]);

         /* Get next holiday */
         row = mysql_fetch_row (mysql_res);

         /* Get holiday code (row[0]) */
         if ((Hld->HldCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong holiday code.");

         /* Get place code (row[1]) */
         Hld->PlcCod = Str_ConvertStrCodToLongCod (row[1]);

         /* Get the full name of the place (row[2]) */
         strcpy (Hld->PlaceFullName,row[2]);

         /* Get type (row[3]) */
         Hld->HldTyp = Hld_GetTypeOfHoliday (row[3]);

         /* Get start date (row[4] holds the start date in YYYYMMDD format) */
         if (!(Dat_GetDateFromYYYYMMDD (&(Hld->StartDate),row[4])))
	    Lay_ShowErrorAndExit ("Wrong start date.");

         /* Get end date (row[5] holds the end date in YYYYMMDD format) */
         if (!(Dat_GetDateFromYYYYMMDD (&(Hld->EndDate),row[5])))
	    Lay_ShowErrorAndExit ("Wrong end date.");

         /* Get the name of the holiday/non school period (row[6]) */
         strcpy (Hld->Name,row[6]);
        }
     }
   else
      Gbl.Hlds.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   Gbl.Hlds.LstIsRead = true;
  }

/*****************************************************************************/
/**************************** Get holiday full name ****************************/
/*****************************************************************************/

void Hld_GetDataOfHolidayByCod (struct Holiday *Hld)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Clear data *****/
   Hld->PlcCod = -1L;
   Hld->PlaceFullName[0] = '\0';
   Hld->StartDate.Day =  Hld->StartDate.Month = Hld->StartDate.Year = 0;
   Hld->Name[0] = '\0';

   /***** Check if holiday code is correct *****/
   if (Hld->HldCod <= 0)
      Lay_ShowErrorAndExit ("Wrong code of holiday.");

   /***** Get data of holiday from database *****/
   sprintf (Query,"(SELECT holidays.PlcCod,places.FullName as Place,HldTyp," \
                  "DATE_FORMAT(holidays.StartDate,'%%Y%%m%%d'),DATE_FORMAT(holidays.EndDate,'%%Y%%m%%d'),holidays.Name" \
                  " FROM holidays,places" \
                  " WHERE holidays.HldCod='%ld' AND holidays.PlcCod=places.PlcCod)" \
                  " UNION " \
                  "(SELECT PlcCod,'' as Place,HldTyp,DATE_FORMAT(StartDate,'%%Y%%m%%d'),DATE_FORMAT(EndDate,'%%Y%%m%%d'),Name" \
                  " FROM holidays" \
                  " WHERE HldCod='%ld' AND PlcCod NOT IN (SELECT DISTINCT PlcCod FROM places))",
            Hld->HldCod,Hld->HldCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a holiday");

   if (NumRows) // Holiday found...
     {
      /* Get row */
      row = mysql_fetch_row (mysql_res);

      /* Get place code (row[0]) */
      Hld->PlcCod = Str_ConvertStrCodToLongCod (row[0]);

      /* Get the full name of the place (row[1]) */
      strcpy (Hld->PlaceFullName,row[1]);

      /* Get type (row[2]) */
      Hld->HldTyp = Hld_GetTypeOfHoliday (row[2]);

      /* Get start date (row[3] holds the start date in YYYYMMDD format) */
      if (!(Dat_GetDateFromYYYYMMDD (&(Hld->StartDate),row[3])))
	 Lay_ShowErrorAndExit ("Wrong start date.");

      /* Get end date (row[4] holds the end date in YYYYMMDD format) */
      if (!(Dat_GetDateFromYYYYMMDD (&(Hld->EndDate),row[4])))
	 Lay_ShowErrorAndExit ("Wrong end date.");

      /* Get the name of the holiday/non school period (row[5]) */
      strcpy (Hld->Name,row[5]);
     }

  /***** Free structure that stores the query result *****/
  DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/************ Get parameter from form with the type of a holiday ************/
/****************************************************************************/

static Hld_HolidayType_t Hld_GetParamHldType (void)
  {
   char UnsignedStr[10+1];

   Par_GetParam (Par_PARAM_SINGLE,"HldTyp",UnsignedStr,10);
   return Hld_GetTypeOfHoliday (UnsignedStr);
  }

/*****************************************************************************/
/********************* Get type of holiday from string ***********************/
/*****************************************************************************/

static Hld_HolidayType_t Hld_GetTypeOfHoliday (const char *UnsignedStr)
  {
   unsigned UnsignedNum;

   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      Lay_ShowErrorAndExit ("Wrong type of holiday.");

   if (UnsignedNum >= Hld_NUM_TYPES_HOLIDAY)
      Lay_ShowErrorAndExit ("Wrong type of holiday.");

   return (Hld_HolidayType_t) UnsignedNum;
  }

/*****************************************************************************/
/**************************** Free list of holidays **************************/
/*****************************************************************************/

void Hld_FreeListHolidays (void)
  {
   if (Gbl.Hlds.LstIsRead && Gbl.Hlds.Lst)
     {
      /***** Free memory used by the list of courses in degree *****/
      free ((void *) Gbl.Hlds.Lst);
      Gbl.Hlds.Lst = NULL;
      Gbl.Hlds.Num = 0;
      Gbl.Hlds.LstIsRead = false;
     }
  }

/****************************************************************************/
/********************* List all the holidays for edition ********************/
/****************************************************************************/

static void Hld_ListHolidaysForEdition (void)
  {
   extern const char *Txt_Remove_holiday[Txt_NUM_LANGUAGES];
   extern const char *Txt_All_places[Txt_NUM_LANGUAGES];
   extern const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY][Txt_NUM_LANGUAGES];
   unsigned NumHld;
   unsigned NumPlc;
   struct Holiday *Hld;
   Hld_HolidayType_t HolidayType;

   /***** Table head *****/
   Hld_PutHeadHolidays ();

   /***** Write all the holidays *****/
   for (NumHld=0; NumHld<Gbl.Hlds.Num; NumHld++)
     {
      Hld = &Gbl.Hlds.Lst[NumHld];

      /* Put icon to remove holiday */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      Act_FormStart (ActRemHld);
      Hld_PutParamHldCod (Hld->HldCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_holiday[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Txt_Remove_holiday[Gbl.Prefs.Language]);

      /* Holiday code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Hld->HldCod);

      /* Holiday place */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgHldPlc);
      Hld_PutParamHldCod (Hld->HldCod);
      fprintf (Gbl.F.Out,"<select name=\"PlcCod\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
      fprintf (Gbl.F.Out,"<option value=\"-1\"");
      if (Hld->PlcCod <= 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_All_places[Gbl.Prefs.Language]);
      for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                  Gbl.Plcs.Lst[NumPlc].PlcCod,
                  (Gbl.Plcs.Lst[NumPlc].PlcCod == Hld->PlcCod) ? " selected=\"selected\"" : "",
                  Gbl.Plcs.Lst[NumPlc].ShortName);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Holiday type */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgHldTyp);
      Hld_PutParamHldCod (Hld->HldCod);
      fprintf (Gbl.F.Out,"<select name=\"HldTyp\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
      for (HolidayType=(Hld_HolidayType_t) 0; HolidayType<Hld_NUM_TYPES_HOLIDAY; HolidayType++)
         fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%s</option>",
                  (unsigned) HolidayType,
                  (HolidayType == Hld->HldTyp) ? " selected=\"selected\"" : "",
                  Txt_HOLIDAY_TYPES[HolidayType][Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Holiday date / Non school period start date */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgHldStrDat);
      Hld_PutParamHldCod (Hld->HldCod);
      Dat_WriteFormDate (Gbl.Today.Year-1,Gbl.Today.Year+1,"StartDay","StartMonth","StartYear",
                         Gbl.Hlds.Lst[NumHld].StartDate.Day,
                         Gbl.Hlds.Lst[NumHld].StartDate.Month,
                         Gbl.Hlds.Lst[NumHld].StartDate.Year,
                         true,false);
      fprintf (Gbl.F.Out,"</form></td>");

      /* Non school period end date */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgHldEndDat);
      Hld_PutParamHldCod (Hld->HldCod);
      Dat_WriteFormDate (Gbl.Today.Year-1,Gbl.Today.Year+1,"EndDay","EndMonth","EndYear",
                         Gbl.Hlds.Lst[NumHld].EndDate.Day,
                         Gbl.Hlds.Lst[NumHld].EndDate.Month,
                         Gbl.Hlds.Lst[NumHld].EndDate.Year,
                         true,(Hld->HldTyp == Hld_HOLIDAY));
      fprintf (Gbl.F.Out,"</form></td>");

      /* Holiday name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenHld);
      Hld_PutParamHldCod (Hld->HldCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Name\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td></tr>\n",
               Hld_MAX_LENGTH_HOLIDAY_NAME,Hld->Name,Gbl.NumForm);
     }
  }

/****************************************************************************/
/******************** Write parameter with code of holiday ********************/
/****************************************************************************/

static void Hld_PutParamHldCod (long HldCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"HldCod\" value=\"%ld\" />",HldCod);
  }

/****************************************************************************/
/********************* Get parameter with code of holiday *********************/
/****************************************************************************/

long Hld_GetParamHldCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of holiday *****/
   Par_GetParam (Par_PARAM_SINGLE,"HldCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/******************************* Remove a holiday *****************************/
/****************************************************************************/

void Hld_RemoveHoliday (void)
  {
   extern const char *Txt_Holiday_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Holiday Hld;

   /***** Get holiday code *****/
   if ((Hld.HldCod = Hld_GetParamHldCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of holiday is missing.");

   /***** Get data of the holiday from database *****/
   Hld_GetDataOfHolidayByCod (&Hld);

   /***** Remove holiday *****/
   sprintf (Query,"DELETE FROM holidays WHERE HldCod='%ld'",Hld.HldCod);
   DB_QueryDELETE (Query,"can not remove a holiday");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Holiday_X_removed[Gbl.Prefs.Language],
            Hld.Name);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Hld_EditHolidays ();
  }

/****************************************************************************/
/************************* Change the place of a holiday *********************/
/****************************************************************************/

void Hld_ChangeHolidayPlace (void)
  {
   char Query[512];
   struct Holiday *Hld;
   struct Place NewPlace;

   Hld = &Gbl.Hlds.EditingHld;

   /***** Get parameters from form *****/
   /* Get the code of the holiday */
   if ((Hld->HldCod = Hld_GetParamHldCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of holiday is missing.");

   /* Get the new place for the holiday */
   NewPlace.PlcCod = Plc_GetParamPlcCod ();

   /***** Get from the database the data of the place *****/
   Plc_GetDataOfPlaceByCod (&NewPlace);

   /***** Get from the database the data of the holiday *****/
   Hld_GetDataOfHolidayByCod (Hld);

   /***** Update the place in database *****/
   sprintf (Query,"UPDATE holidays SET PlcCod='%ld' WHERE HldCod='%ld'",
            NewPlace.PlcCod,Hld->HldCod);
   DB_QueryUPDATE (Query,"can not update the place of a holiday");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"El lugar de la festividad <strong>%s</strong>" \
                        " ha cambiado a <strong>%s</strong>.",	// Need translation!
            Hld->Name,NewPlace.FullName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Hld->PlcCod = NewPlace.PlcCod;
   strcpy (Hld->PlaceFullName,NewPlace.FullName);
   Hld_EditHolidays ();
  }

/****************************************************************************/
/************************* Change the type of a holiday *********************/
/****************************************************************************/

void Hld_ChangeHolidayType (void)
  {
   char Query[512];
   struct Holiday *Hld;

   Hld = &Gbl.Hlds.EditingHld;

   /***** Get the code of the holiday *****/
   if ((Hld->HldCod = Hld_GetParamHldCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of holiday is missing.");

   /***** Get from the database the data of the holiday *****/
   Hld_GetDataOfHolidayByCod (Hld);

   /***** Get the new type for the holiday *****/
   Hld->HldTyp = Hld_GetParamHldType ();

   /***** Update holiday/no school period in database *****/
   switch (Hld->HldTyp)
     {
      case Hld_HOLIDAY:
         Hld->EndDate.Day = Hld->EndDate.Month = Hld->EndDate.Year = 0;
         sprintf (Query,"UPDATE holidays SET HldTyp='%u',EndDate='00000000' WHERE HldCod='%ld'",
                  (unsigned) Hld_HOLIDAY,Hld->HldCod);
         break;
      case Hld_NON_SCHOOL_PERIOD:
         Hld->EndDate.Day   = Hld->StartDate.Day;
         Hld->EndDate.Month = Hld->StartDate.Month;
         Hld->EndDate.Year  = Hld->StartDate.Year;
         sprintf (Query,"UPDATE holidays SET HldTyp='%u',EndDate=StartDate WHERE HldCod='%ld'",
                  (unsigned) Hld_NON_SCHOOL_PERIOD,Hld->HldCod);
     }
   DB_QueryUPDATE (Query,"can not update the type of a holiday");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"El tipo de la festividad <strong>%s</strong> ha cambiado.",	// Need translation!
            Hld->Name);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Hld_EditHolidays ();
  }

/****************************************************************************/
/*** Change the date of a holiday / the start date of a non school period ***/
/****************************************************************************/

void Hld_ChangeStartDate (void)
  {
   Hld_ChangeDate (HLD_START_DATE);
  }

/****************************************************************************/
/*************** Change the end date of a non school period *****************/
/****************************************************************************/

void Hld_ChangeEndDate (void)
  {
   Hld_ChangeDate (HLD_END_DATE);
  }

/****************************************************************************/
/**************** Change the start/end date of a holiday ********************/
/****************************************************************************/

static void Hld_ChangeDate (Hld_StartOrEndDate_t StartOrEndDate)
  {
   char Query[512];
   struct Holiday *Hld;
   struct Date NewDate;
   struct Date *PtrDate;
   const char *StrDate;

   Hld = &Gbl.Hlds.EditingHld;

   /***** Get the code of the holiday *****/
   if ((Hld->HldCod = Hld_GetParamHldCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of holiday is missing.");

   /***** Get from the database the data of the holiday *****/
   Hld_GetDataOfHolidayByCod (Hld);

   /***** Get the new date for the holiday *****/
   switch (StartOrEndDate)
     {
      case HLD_START_DATE:
         StrDate = "StartDate";
         PtrDate = &(Hld->StartDate);
         Dat_GetDateFromForm ("StartDay","StartMonth","StartYear",
                              &(NewDate.Day),&(NewDate.Month),&(NewDate.Year));
         if (NewDate.Day == 0 || NewDate.Month == 0 || NewDate.Year == 0)
           {
            NewDate.Day   = Gbl.Today.Day;
            NewDate.Month = Gbl.Today.Month;
            NewDate.Year  = Gbl.Today.Year;
           }
         break;
      default:	// HLD_END_DATE
         StrDate = "EndDate";
         PtrDate = &(Hld->EndDate);
         switch (Hld->HldTyp)
           {
            case Hld_HOLIDAY:
               NewDate.Day = NewDate.Month = NewDate.Year = 0;
               break;
            case Hld_NON_SCHOOL_PERIOD:
               Dat_GetDateFromForm ("EndDay","EndMonth","EndYear",
                                    &(NewDate.Day),&(NewDate.Month),&(NewDate.Year));
               if (NewDate.Day == 0 || NewDate.Month == 0 || NewDate.Year == 0)
                 {
                  NewDate.Day   = Gbl.Today.Day;
                  NewDate.Month = Gbl.Today.Month;
                  NewDate.Year  = Gbl.Today.Year;
                 }
           }
     }

   /***** Update the date in database *****/
   sprintf (Query,"UPDATE holidays SET %s='%04u%02u%02u' WHERE HldCod='%ld'",
            StrDate,NewDate.Year,NewDate.Month,NewDate.Day,Hld->HldCod);
   DB_QueryUPDATE (Query,"can not update the date of a holiday");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"La fecha de <strong>%s</strong>" \
                        " ha cambiado a &quot;<strong>%02u/%02u/%04u</strong>&quot;.",	// Need translation!
            Hld->Name,NewDate.Day,NewDate.Month,NewDate.Year);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Dat_AssignDate (PtrDate,&NewDate);
   Hld_EditHolidays ();
  }

/****************************************************************************/
/************************ Change the name of a degree ***********************/
/****************************************************************************/

void Hld_RenameHoliday (void)
  {
   extern const char *Txt_The_holiday_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Holiday *Hld;
   char NewHldName[Hld_MAX_LENGTH_HOLIDAY_NAME+1];

   Hld = &Gbl.Hlds.EditingHld;

   /***** Get parameters from form *****/
   /* Get the code of the holiday */
   if ((Hld->HldCod = Hld_GetParamHldCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of holiday is missing.");

   /* Get the new name for the holiday */
   Par_GetParam (Par_PARAM_SINGLE,"Name",NewHldName,Hld_MAX_LENGTH_HOLIDAY_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewHldName,Hld_MAX_LENGTH_HOLIDAY_NAME,true);

   /***** Get from the database the old names of the holiday *****/
   Hld_GetDataOfHolidayByCod (Hld);

   /***** Check if new name is empty *****/
   if (!NewHldName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el texto (festividad) de la festividad <strong>%s</strong> vac&iacute;o.",	// Need translation!
               Hld->Name);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (Hld->Name,NewHldName))	// Different names
        {
         /***** If degree was in database... *****/
         if (Hld_CheckIfHolidayNameExists (NewHldName,Hld->HldCod))
           {
            sprintf (Gbl.Message,Txt_The_holiday_X_already_exists[Gbl.Prefs.Language],
                     NewHldName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE holidays SET Name='%s' WHERE HldCod='%ld'",
                     NewHldName,Hld->HldCod);
            DB_QueryUPDATE (Query,"can not update the text of a holiday");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"La festividad <strong>%s</strong>" \
                                 " ha sido renombrada como <strong>%s</strong>.",	// Need translation!
                     Hld->Name,NewHldName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El texto (festividad) de la festividad <strong>%s</strong> no ha cambiado.",	// Need translation!
                 Hld->Name);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (Hld->Name,NewHldName);
   Hld_EditHolidays ();
  }

/****************************************************************************/
/******************** Check if the name of holiday exists *******************/
/****************************************************************************/

static bool Hld_CheckIfHolidayNameExists (const char *Name,long HldCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of holidays with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM holidays WHERE Name='%s' AND HldCod<>'%ld'",
            Name,HldCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the text of a holiday already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the text of a holiday already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************* Put a form to create a new holiday *********************/
/****************************************************************************/

static void Hld_PutFormToCreateHoliday (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_All_places[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_holiday[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_holiday[Txt_NUM_LANGUAGES];
   extern const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY][Txt_NUM_LANGUAGES];
   struct Holiday *Hld;
   unsigned NumPlc;
   Hld_HolidayType_t HolidayType;

   Hld = &Gbl.Hlds.EditingHld;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"7\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"7\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_holiday[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewHld);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Hld_PutHeadHolidays ();

   /***** New holiday *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_holiday[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Holiday code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Holiday place *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
   fprintf (Gbl.F.Out,"<select name=\"PlcCod\" style=\"width:50px;\"><option value=\"-1\"");
   if (Hld->PlcCod <= 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Txt_All_places[Gbl.Prefs.Language]);
   for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
      fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
               Gbl.Plcs.Lst[NumPlc].PlcCod,
               (Gbl.Plcs.Lst[NumPlc].PlcCod == Hld->PlcCod) ? " selected=\"selected\"" : "",
              Gbl.Plcs.Lst[NumPlc].ShortName);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Holiday type *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"HldTyp\" style=\"width:50px;\">");
   for (HolidayType=(Hld_HolidayType_t) 0; HolidayType<Hld_NUM_TYPES_HOLIDAY; HolidayType++)
      fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%s</option>",
               (unsigned) HolidayType,
               (HolidayType == Hld->HldTyp) ? " selected=\"selected\"" : "",
               Txt_HOLIDAY_TYPES[HolidayType][Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Holiday date / Non school period start date *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
   Dat_WriteFormDate (Gbl.Today.Year-1,Gbl.Today.Year+1,"StartDay","StartMonth","StartYear",
                      Hld->StartDate.Day,
                      Hld->StartDate.Month,
                      Hld->StartDate.Year,
                      false,false);
   fprintf (Gbl.F.Out,"</td>");

   /***** Non school period end date *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
   Dat_WriteFormDate (Gbl.Today.Year-1,Gbl.Today.Year+1,"EndDay","EndMonth","EndYear",
                      Hld->EndDate.Day,
                      Hld->EndDate.Month,
                      Hld->EndDate.Year,
                      false,false);
   fprintf (Gbl.F.Out,"</td>");

   /***** Holiday name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"Name\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Hld_MAX_LENGTH_HOLIDAY_NAME,Hld->Name);
  }

/****************************************************************************/
/******************* Write header with fields of a holiday ******************/
/****************************************************************************/

static void Hld_PutHeadHolidays (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Place[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type[Txt_NUM_LANGUAGES];
   extern const char *Txt_Start_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_End_date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Holiday[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Place[Gbl.Prefs.Language],
            Txt_Type[Gbl.Prefs.Language],
            Txt_Start_date[Gbl.Prefs.Language],
            Txt_End_date[Gbl.Prefs.Language],
            Txt_Holiday[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************* Receive form to create a new holiday *******************/
/****************************************************************************/

void Hld_RecFormNewHoliday (void)
  {
   extern const char *Txt_The_holiday_X_already_exists[Txt_NUM_LANGUAGES];
   struct Holiday *Hld;

   Hld = &Gbl.Hlds.EditingHld;

   /***** Get place code *****/
   Hld->PlcCod = Plc_GetParamPlcCod ();

   /***** Get the type of holiday *****/
   Hld->HldTyp = Hld_GetParamHldType ();

   /***** Get start date *****/
   Dat_GetDateFromForm ("StartDay","StartMonth","StartYear",
                        &(Hld->StartDate.Day),&(Hld->StartDate.Month),&(Hld->StartDate.Year));

   /***** Get end date *****/
   Dat_GetDateFromForm ("EndDay","EndMonth","EndYear",
                        &(Hld->EndDate.Day),&(Hld->EndDate.Month),&(Hld->EndDate.Year));

   /***** Get holiday name *****/
   Par_GetParam (Par_PARAM_SINGLE,"Name",Hld->Name,Hld_MAX_LENGTH_HOLIDAY_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Hld->Name,Hld_MAX_LENGTH_HOLIDAY_NAME,true);

   /***** Adjust dates *****/
   if (Hld->StartDate.Day == 0 || Hld->StartDate.Month == 0 || Hld->StartDate.Year == 0)
     {
      Hld->StartDate.Day   = Gbl.Today.Day;
      Hld->StartDate.Month = Gbl.Today.Month;
      Hld->StartDate.Year  = Gbl.Today.Year;
     }
   switch (Hld->HldTyp)
     {
      case Hld_HOLIDAY:
         Hld->EndDate.Day = Hld->EndDate.Month = Hld->EndDate.Year = 0;
         break;
      case Hld_NON_SCHOOL_PERIOD:
         if (Hld->EndDate.Day == 0 || Hld->EndDate.Month == 0 || Hld->EndDate.Year == 0)
           {
            Hld->EndDate.Day   = Gbl.Today.Day;
            Hld->EndDate.Month = Gbl.Today.Month;
            Hld->EndDate.Year  = Gbl.Today.Year;
           } 
     }

   /***** Create the new holiday or write warning message *****/
   if (Hld->Name[0])	// If there's a holiday name
     {
      /* If name of holiday was in database... */
      if (Hld_CheckIfHolidayNameExists (Hld->Name,-1L))
        {
         sprintf (Gbl.Message,Txt_The_holiday_X_already_exists[Gbl.Prefs.Language],
                  Hld->Name);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else	// Add new holiday to database
         Hld_CreateHoliday (Hld);
     }
   else	// If there isn't a holiday name
     {
      sprintf (Gbl.Message,"Debe especificar el texto (festividad) de la nueva festividad.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Hld_EditHolidays ();
  }

/****************************************************************************/
/**************************** Create a new holiday ***************************/
/****************************************************************************/

static void Hld_CreateHoliday (struct Holiday *Hld)
  {
   extern const char *Txt_Created_new_holiday_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new holiday or no school period *****/
   sprintf (Query,"INSERT INTO holidays (PlcCod,HldTyp,StartDate,EndDate,Name) VALUES ('%ld','%u','%04u%02u%02u','%04u%02u%02u','%s')",
            Hld->PlcCod,(unsigned) Hld->HldTyp,
            Hld->StartDate.Year,Hld->StartDate.Month,Hld->StartDate.Day,
            Hld->EndDate.Year,Hld->EndDate.Month,Hld->EndDate.Day,
            Hld->Name);
   DB_QueryINSERT (Query,"can not create holiday");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_holiday_X[Gbl.Prefs.Language],
            Hld->Name);
   Lay_ShowAlert (INFO,Gbl.Message);
  }
