// swad_imp.c: import courses and students from another web application using web services

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For exit, system, malloc, free, rand, etc.
#include <string.h>		// For string functions
#include <sys/wait.h>		// For the macro WEXITSTATUS

#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_imp.h"
#include "swad_par.h"
#include "swad_usr.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern const char *StringsGenderDB[Usr_NUM_GENDERS];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

#define Imp_OK				0
#define Imp_ERROR_BAD_SESSION		1
#define Imp_ERROR_NOT_ENOUGH_MEMORY	2
#define Imp_ERROR_SOAP			3

#define Imp_MAX_BYTES_DNI 16
#define Imp_MAX_BYTES_NAME 255
#define Imp_MAX_BYTES_EMAIL 255

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

struct ImportedStudent
  {
   char DNI[Imp_MAX_BYTES_DNI+1];
   char FirstName[Imp_MAX_BYTES_NAME+1];
   char Surname1[Imp_MAX_BYTES_NAME+1];
   char Surname2[Imp_MAX_BYTES_NAME+1];
   char E_mail[Imp_MAX_BYTES_EMAIL+1];
   Usr_Gender_t Gender;
  };

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Imp_ListMyImpGrpsAndStds (bool ItsAFormToRegRemStds);
static void Imp_RemoveOldImpSessions (void);
static void Imp_RemoveOldImpStdsAndGrps (void);
static unsigned Imp_GetAndListImpGrpsAndStdsFromDB (bool ItsAFormToRegRemStds);

/****************************************************************************/
/************** Get parameters with imported user and session ***************/
/****************************************************************************/

void Imp_GetImpUsrAndSession (void)
  {
   Par_GetParam (Par_PARAM_SINGLE,Cfg_NAME_PARAM_IMPORTED_USR_ID    ,Gbl.Imported.UsrId    ,Cfg_MAX_LENGTH_IMPORTED_USR_ID    );
   Par_GetParam (Par_PARAM_SINGLE,Cfg_NAME_PARAM_IMPORTED_SESSION_ID,Gbl.Imported.SessionId,Cfg_MAX_LENGTH_IMPORTED_SESSION_ID);
  }

/****************************************************************************/
/********** Import the students of an official group from database **********/
/****************************************************************************/

void Imp_ImportStdsFromAnImpGrp (long ImpGrpCod,unsigned NumGrpsInCurrentCrs,struct ListCodGrps *LstGrps,unsigned *NumUsrsRegistered)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumStds,NumStd;
   struct ImportedStudent ImpStd;
   Usr_Gender_t Gender;
   bool IsStudent;

   /***** Get data of the imported students belonging to this group from database *****/
   sprintf (Query,"SELECT DNI,Surname1,Surname2,FirstName,Gender,E_mail" \
                  " FROM imported_students WHERE GrpCod='%ld'" \
                  " ORDER BY Surname1,Surname2,FirstName,DNI",
                  ImpGrpCod);
   NumStds = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get imported students");

   /***** Import the students from this official group *****/
   for (NumStd=0; NumStd<NumStds; NumStd++)	// For each student inside the official group...
     {
      /***** Get official data of the student *****/
      row = mysql_fetch_row (mysql_res);

      /* Get DNI (row[0]) */
      strncpy (ImpStd.DNI,row[0],Imp_MAX_BYTES_DNI);
      ImpStd.DNI[Imp_MAX_BYTES_DNI] = '\0';

      /* Get Surname1 (row[1]) */
      strncpy (ImpStd.Surname1,row[1],Imp_MAX_BYTES_NAME);
      ImpStd.Surname1[Imp_MAX_BYTES_NAME] = '\0';

      /* Get Surname2 (row[2]) */
      strncpy (ImpStd.Surname2,row[2],Imp_MAX_BYTES_NAME);
      ImpStd.Surname2[Imp_MAX_BYTES_NAME] = '\0';

      /* Get FirstName (row[3]) */
      strncpy (ImpStd.FirstName,row[3],Imp_MAX_BYTES_NAME);
      ImpStd.FirstName[Imp_MAX_BYTES_NAME] = '\0';

      /* Get Gender (row[4]) */
      ImpStd.Gender = Usr_GENDER_UNKNOWN;
      for (Gender=(Usr_Gender_t) 0; Gender<Usr_NUM_GENDERS; Gender++)
         if (!strcasecmp (row[4],StringsGenderDB[Gender]))
           {
	    ImpStd.Gender = Gender;
            break;
           }

      /* Get E_mail (row[5]) */
      strncpy (ImpStd.E_mail,row[5],Imp_MAX_BYTES_EMAIL);
      ImpStd.E_mail[Imp_MAX_BYTES_EMAIL] = '\0';

      /***** Copy data of the student *****/
      strncpy (Gbl.Usrs.Other.UsrDat.DNI,ImpStd.DNI,Imp_MAX_BYTES_DNI);
      Gbl.Usrs.Other.UsrDat.DNI[Imp_MAX_BYTES_DNI] = '\0';
      if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Other.UsrDat.DNI))	// If it's not a valid DNI, skip it
        {
         IsStudent = true;
         /* Check if the record of the user exists and get user's data */
         if (Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Other.UsrDat))		// Get from the database the data of the student to be registered
           {
            if (Gbl.Usrs.Other.UsrDat.UsrTypeInDB == Usr_STUDENT)
              {// The user doesn't exist. Register him/her as student
               /* Fill empty data */
               if (!Gbl.Usrs.Other.UsrDat.Surname1[0])
                  strcpy (Gbl.Usrs.Other.UsrDat.Surname1,ImpStd.Surname1);
               if (!Gbl.Usrs.Other.UsrDat.Surname2[0])
                  strcpy (Gbl.Usrs.Other.UsrDat.Surname2,ImpStd.Surname2);
               if (!Gbl.Usrs.Other.UsrDat.FirstName[0])
                  strcpy (Gbl.Usrs.Other.UsrDat.FirstName,ImpStd.FirstName);
               if (Gbl.Usrs.Other.UsrDat.Gender == Usr_GENDER_UNKNOWN)
                  Gbl.Usrs.Other.UsrDat.Gender = ImpStd.Gender;
               if (!Gbl.Usrs.Other.UsrDat.E_mail[0])
                  strcpy (Gbl.Usrs.Other.UsrDat.E_mail,ImpStd.E_mail);
              }
            else							
              {// The user exists, but not as student
               IsStudent = false;
               sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong>" \
                                    " no ha podido darse de alta como alumno.",	// Need translation!
                        Gbl.Usrs.Other.UsrDat.DNI);
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
           }
         else
           {// The user doesn't exist. Register him/her as student
	    Gbl.Usrs.Other.UsrDat.UsrTypeInDB = Usr_STUDENT;
            strcpy (Gbl.Usrs.Other.UsrDat.Surname1,ImpStd.Surname1);
            strcpy (Gbl.Usrs.Other.UsrDat.Surname2,ImpStd.Surname2);
            strcpy (Gbl.Usrs.Other.UsrDat.FirstName,ImpStd.FirstName);
            Gbl.Usrs.Other.UsrDat.Gender = ImpStd.Gender;
            strcpy (Gbl.Usrs.Other.UsrDat.E_mail,ImpStd.E_mail);
           }
         if (IsStudent)
           {
            Usr_CreatUsrIfNotExistsAndRegisterHimInCurrentCrs (&Gbl.Usrs.Other.UsrDat,Usr_REPLACE_DATA,Cte_QUIET);	// Create user's record if his/her did not exist, and register his/her in current course
 	    if (NumGrpsInCurrentCrs)	// If there are groups in current course
               Grp_RegisterUsrIntoGroups (&Gbl.Usrs.Other.UsrDat,LstGrps);					// Register student in the selected groups
            (*NumUsrsRegistered)++;
           }
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/************** Insert new imported session in the database *****************/
/****************************************************************************/

void Imp_InsertImpSessionInDB (void)
  {
   char Query[1024];

   /***** Remove old imported sessions *****/
   Imp_RemoveOldImpSessions ();

   /***** Insert session in the database *****/
   sprintf (Query,"INSERT INTO imported_sessions (SessionId,UsrCod,ImportedUsrId,ImportedSessionId,ImportedUsrType)" \
                  " VALUES ('%s','%ld','%s','%s','%u')",
            Gbl.Session.Id,
            Gbl.Usrs.Me.UsrDat.UsrCod,
            Gbl.Imported.UsrId,
            Gbl.Imported.SessionId,
            (unsigned) Gbl.Imported.UsrType);
   DB_QueryINSERT (Query,"can not create imported session");
  }

/*****************************************************************************/
/******************** Get the data of a imported session *********************/
/*****************************************************************************/

bool Imp_GetImpSessionData (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool Result = false;

   /***** Get data of the imported session from database *****/
   sprintf (Query,"SELECT ImportedUsrId,ImportedSessionId,ImportedUsrType FROM imported_sessions" \
                  " WHERE SessionId='%s'",Gbl.Session.Id);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get imported session data");

   /***** Check if the session existed in the database *****/
   if (NumRows)
     {
      row = mysql_fetch_row (mysql_res);

      /***** Get imported user identifier (row[0]) *****/
      strcpy (Gbl.Imported.UsrId,row[0]);

      /***** Get imported session identifier (row[1]) *****/
      strcpy (Gbl.Imported.SessionId,row[1]);

      /***** Get imported user type (row[2]) *****/
      if (sscanf (row[2],"%u",&Gbl.Imported.UsrType) != 1)
         Gbl.Imported.UsrType = Usr_UNKNOWN;

      Result = true;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return Result;
  }

/****************************************************************************/
/************* Get lists of my imported groups and students *****************/
/****************************************************************************/

void Imp_ListMyImpGrpsAndStdsNoForm (void)
  {
   Imp_ListMyImpGrpsAndStds (false);
  }

/****************************************************************************/
/************* Get lists of my imported groups and students *****************/
/****************************************************************************/

void Imp_ListMyImpGrpsAndStdsForm (void)
  {
   Imp_ListMyImpGrpsAndStds (true);
  }

/****************************************************************************/
/************* Get lists of my imported groups and students *****************/
/****************************************************************************/

static void Imp_ListMyImpGrpsAndStds (bool ItsAFormToRegRemStds)
  {
   char Command[2048];
   int ReturnCode;

   /***** Try to get lists from database *****/
   if (!Imp_GetAndListImpGrpsAndStdsFromDB (ItsAFormToRegRemStds))
     {
      /***** Remove old imported students and groups *****/
      Imp_RemoveOldImpStdsAndGrps ();

      if (Gbl.Imported.UsrId[0] &&
          Gbl.Imported.SessionId[0] &&
          Gbl.Imported.UsrType == Usr_TEACHER)
        {
         /***** Get new imported lists and store them in database *****/
         sprintf (Command,"%s 2 %s %s %s",
                  Cfg_EXTERNAL_LOGIN_CLIENT_COMMAND,Gbl.Session.Id,Gbl.Imported.UsrId,Gbl.Imported.SessionId);
         ReturnCode = system (Command);
         if (ReturnCode == -1)
            Lay_ShowErrorAndExit ("Error when running command to import lists of groups and students.");

         /***** Write message depending on return code *****/
         ReturnCode = WEXITSTATUS(ReturnCode);
         switch (ReturnCode)
           {
            case 0:	// Success
               /***** Try again to get lists from database *****/
               if (!Imp_GetAndListImpGrpsAndStdsFromDB (ItsAFormToRegRemStds))
                 {
                  sprintf (Gbl.Message,"No se ha podido obtener la lista oficial de sus grupos y alumnos." \
                                       "Es posible que haya expirado el plazo para obtener dicha lista." \
                                       "Entre de nuevo a %s desde %s y reint&eacute;ntelo.",	// Need translation!
                           Cfg_PLATFORM_SHORT_NAME,Cfg_EXTERNAL_LOGIN_SERVICE_FULL_NAME);
                  Lay_ShowAlert (ERROR,Gbl.Message);
                 }
               break;
            default:	// Error
               sprintf (Gbl.Message,"Error %d while importing lists of groups and students.",ReturnCode);
               Lay_ShowAlert (ERROR,Gbl.Message);
           }
        }
      else	// I am not entered from external program as teacher
        {
         sprintf (Gbl.Message,"Para obtener la lista oficial de sus grupos y alumnos" \
                              " debe entrar a %s desde %s.",	// Need translation!
                  Cfg_PLATFORM_SHORT_NAME,Cfg_EXTERNAL_LOGIN_SERVICE_FULL_NAME);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
  }

/****************************************************************************/
/*********************** Remove old imported sessions ***********************/
/****************************************************************************/

static void Imp_RemoveOldImpSessions (void)
  {
   char Query[1024];

   /***** Remove old imported sessions *****/
   sprintf (Query,"DELETE FROM imported_sessions WHERE SessionId NOT IN (SELECT SessionId FROM sessions)");
   DB_QueryDELETE (Query,"can not remove old imported sessions");
  }

/****************************************************************************/
/***************** Remove old imported students and groups ******************/
/****************************************************************************/

static void Imp_RemoveOldImpStdsAndGrps (void)
  {
   char Query[1024];

   /***** Remove old imported students *****/
   sprintf (Query,"DELETE FROM imported_students USING imported_groups,imported_students,sessions" \
                  " WHERE imported_groups.SessionId NOT IN (SELECT SessionId FROM sessions) AND imported_groups.GrpCod=imported_students.GrpCod");
   DB_QueryDELETE (Query,"can not remove old imported students");

   /***** Remove old imported groups *****/
   sprintf (Query,"DELETE FROM imported_groups WHERE SessionId NOT IN (SELECT SessionId FROM sessions)");
   DB_QueryDELETE (Query,"can not remove old imported groups");
  }

/****************************************************************************/
/******** Get from database and list imported groups and students ***********/
/****************************************************************************/

static unsigned Imp_GetAndListImpGrpsAndStdsFromDB (bool ItsAFormToRegRemStds)
  {
   char Query[1024];
   MYSQL_RES *mysql_res_grp;
   MYSQL_RES *mysql_res_std;
   MYSQL_ROW row;
   unsigned NumGrps,NumGrp;
   long GrpCod;
   unsigned NumStds,NumStd;

   /***** Get data of imported groups from database *****/
   sprintf (Query,"SELECT GrpCod,DegName,CrsName,GrpName,GrpType FROM imported_groups" \
                  " WHERE SessionId='%s' ORDER BY DegName,CrsName,GrpName,GrpType",
            Gbl.Session.Id);
   NumGrps = (unsigned) DB_QuerySELECT (Query,&mysql_res_grp,"can not get imported groups");

   if (NumGrps)
     {
      /***** Start table with groups *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /***** Get groups and write them *****/
      for (NumGrp=0; NumGrp<NumGrps; NumGrp++)	// For each course
        {
         row = mysql_fetch_row (mysql_res_grp);

         GrpCod = Str_ConvertStrCodToLongCod (row[0]);

         /***** Write degree, course and group *****/
         fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\">");
         if (ItsAFormToRegRemStds)
            /* Put checkbox to select the group */
            fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"ImpGrpCod\" value=\"%ld\" />",
                     GrpCod);
         else
            fprintf (Gbl.F.Out,"&nbsp;");
         fprintf (Gbl.F.Out,"</td>" \
                            "<td align=\"left\" class=\"TIT_TBL\">%s &gt; %s &gt; %s (%s)</td>" \
                            "</tr>",
                  row[1],row[2],row[3],row[4]);

         /***** Get data of the imported students belonging to this group from database *****/
         sprintf (Query,"SELECT DNI,Surname1,Surname2,FirstName FROM imported_students" \
                        " WHERE GrpCod='%ld' ORDER BY Surname1,Surname2,FirstName,DNI",
                  GrpCod);
         NumStds = (unsigned) DB_QuerySELECT (Query,&mysql_res_std,"can not get imported students");

         fprintf (Gbl.F.Out,"<tr>" \
                            "<td align=\"left\" class=\"DAT\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>" \
                            "<td align=\"left\" class=\"DAT\">%u alumnos:<br />" \
                            "<textarea cols=\"60\" rows=\"4\" readonly>",NumStds);
         for (NumStd=0; NumStd<NumStds; NumStd++)	// For each student inside the group
           {
            row = mysql_fetch_row (mysql_res_std);
            fprintf (Gbl.F.Out,"%s %s",row[0],row[1]);	// DNI and surname 1
            if (row[2][0])
               fprintf (Gbl.F.Out," %s",row[2]);	// surname 2
            fprintf (Gbl.F.Out,", %s\n",row[3]);	// first name
           }
         fprintf (Gbl.F.Out,"</textarea></td></tr>");

         /***** Free structure that stores the query result *****/
         DB_FreeMySQLResult (&mysql_res_std);
        }

      /***** End of table with courses *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res_grp);

   return NumGrps;
  }
