// swad_ins.c: institutions

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_cfg.h"
#include "swad_cns.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_ins.h"
#include "swad_par.h"
#include "swad_txt.h"
#include "swad_usr.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Ins_GetParamInsOrderType (void);
static void Ins_PutFormToEditInstitutions (void);
static void Ins_ListInstitutionsForEdition (void);
static void Ins_PutParamInsCod (long InsCod);
static void Ins_RenameInstitution (Cte_ShortOrFullName_t ShortOrFullName);
static bool Ins_CheckIfInstitutionNameExists (const char *FieldName,const char *Name,long InsCod);
static void Ins_PutFormToCreateInstitution (void);
static void Ins_PutHeadInstitutions (void);
static void Ins_CreateInstitution (struct Institution *Ins);

/****************************************************************************/
/************************* List all the institutions ************************/
/****************************************************************************/

void Ins_SeeInstitutions (void)
  {
   extern const char *Txt_INSTITUTIONS_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_INSTITUTIONS_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_PLURAL_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Centres[Txt_NUM_LANGUAGES];
   extern const char *Txt_Departments[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degrees[Txt_NUM_LANGUAGES];
   extern const char *Txt_Other_institutions[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institution_unspecified[Txt_NUM_LANGUAGES];
   tInssOrderType Order;
   unsigned NumIns;
   unsigned NumStdsWithInstitution = 0;
   unsigned NumTchsWithInstitution = 0;
   unsigned NumUsrsWithInstitution = 0;
   unsigned NumCtrsWithInstitution = 0;
   unsigned NumDptsWithInstitution = 0;
   unsigned NumDegsWithInstitution = 0;
   unsigned NumStdsInOtherInstitutions;
   unsigned NumTchsInOtherInstitutions;
   unsigned NumUsrsInOtherInstitutions;
   unsigned NumCtrsInOtherInstitutions;
   unsigned NumDptsInOtherInstitutions;
   unsigned NumDegsInOtherInstitutions;
   unsigned NumTotalStds;
   unsigned NumTotalTchs;
   unsigned NumTotalUsrs;
   unsigned NumTotalCtrs;
   unsigned NumTotalDpts;
   unsigned NumTotalDegs;

   /***** Get parameter with the type of order in the list of institutions *****/
   Ins_GetParamInsOrderType ();

   /***** Get list of institutions *****/
   Ins_GetListInstitutions (true);

   /***** Put link (form) to edit institutions *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
       Ins_PutFormToEditInstitutions ();

   /***** Table head *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td width=\"20\" class=\"TIT_TBL\">&nbsp;</td>");
   for (Order=INS_ORDER_BY_INSTITUTION; Order<=INS_ORDER_BY_NUM_USRS; Order++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">");
      Act_FormStart (ActSeeIns);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_INSTITUTIONS_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
      if (Order == Gbl.Inss.SelectedOrderType)
         fprintf (Gbl.F.Out,"<u>");
      fprintf (Gbl.F.Out,Txt_INSTITUTIONS_ORDER[Order][Gbl.Prefs.Language]);
      if (Order == Gbl.Inss.SelectedOrderType)
         fprintf (Gbl.F.Out,"</u>");
      fprintf (Gbl.F.Out,"</acronym></a></form></td>");
     }
   fprintf (Gbl.F.Out,"<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_USR_TYPES_PLURAL_Abc[Usr_STUDENT][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            Txt_USR_TYPES_PLURAL_Abc[Usr_TEACHER][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            Txt_Centres[Gbl.Prefs.Language],
            Txt_Departments[Gbl.Prefs.Language],
            Txt_Degrees[Gbl.Prefs.Language]);

   /***** Write all the institutions and their nuber of users *****/
   for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"20\">" \
                         "<a href=\"%s\" target=\"_blank\">" \
                         "<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" alt=\"%s\" />" \
                         "</a></td>",
               Gbl.Inss.Lst[NumIns].WWW,
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.Inss.Lst[NumIns].Logo,Gbl.Inss.Lst[NumIns].ShortName);
      fprintf (Gbl.F.Out,"<td align=\"left\">" \
                         "<a href=\"%s\" target=\"_blank\" class=\"DAT\">%s</a></td>",
               Gbl.Inss.Lst[NumIns].WWW,Gbl.Inss.Lst[NumIns].FullName);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>",
               Gbl.Inss.Lst[NumIns].NumUsrs);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>",
               Gbl.Inss.Lst[NumIns].NumStds);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>",
               Gbl.Inss.Lst[NumIns].NumTchs);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>",
               Gbl.Inss.Lst[NumIns].NumCtrs);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>",
               Gbl.Inss.Lst[NumIns].NumDpts);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Gbl.Inss.Lst[NumIns].NumDegs);

      NumStdsWithInstitution += Gbl.Inss.Lst[NumIns].NumStds;
      NumTchsWithInstitution += Gbl.Inss.Lst[NumIns].NumTchs;
      NumUsrsWithInstitution += Gbl.Inss.Lst[NumIns].NumUsrs;
      NumCtrsWithInstitution += Gbl.Inss.Lst[NumIns].NumCtrs;
      NumDptsWithInstitution += Gbl.Inss.Lst[NumIns].NumDpts;
      NumDegsWithInstitution += Gbl.Inss.Lst[NumIns].NumDegs;
     }

   /***** Separation row *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"5\">&nbsp;</td></tr>");
 
   /***** Write users with other institution *****/
   NumStdsInOtherInstitutions = Usr_GetNumberOfUsersInInstitution (0,Usr_STUDENT);
   NumTchsInOtherInstitutions = Usr_GetNumberOfUsersInInstitution (0,Usr_TEACHER);
   NumUsrsInOtherInstitutions = NumStdsInOtherInstitutions + NumTchsInOtherInstitutions;
   NumCtrsInOtherInstitutions = Ctr_GetNumberOfCentresInInstitution (0);
   NumDptsInOtherInstitutions = Dpt_GetNumberOfDepartmentsInInstitution (0);
   NumDegsInOtherInstitutions = Deg_GetNumberOfDegreesInInstitution (0);
   fprintf (Gbl.F.Out,"<tr><td width=\"20\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
            Txt_Other_institutions[Gbl.Prefs.Language],
            NumUsrsInOtherInstitutions,NumStdsInOtherInstitutions,NumTchsInOtherInstitutions,
            NumCtrsInOtherInstitutions,NumDptsInOtherInstitutions,NumDegsInOtherInstitutions);
   NumStdsWithInstitution += NumStdsInOtherInstitutions;
   NumTchsWithInstitution += NumTchsInOtherInstitutions;
   NumUsrsWithInstitution += NumUsrsInOtherInstitutions;
   NumCtrsWithInstitution += NumCtrsInOtherInstitutions;
   NumDptsWithInstitution += NumDptsInOtherInstitutions;
   NumDegsWithInstitution += NumDegsInOtherInstitutions;

   /***** Write users with no institution *****/
   NumTotalStds = Sta_GetTotalNumberOfUsers (Sta_LOCATION_RANGE_ALL,Usr_STUDENT);
   NumTotalTchs = Sta_GetTotalNumberOfUsers (Sta_LOCATION_RANGE_ALL,Usr_TEACHER);
   NumTotalUsrs = NumTotalStds + NumTotalTchs;
   NumTotalCtrs = Ctr_GetTotalNumberOfCentres ();
   NumTotalDpts = Dpt_GetTotalNumberOfDepartments ();
   NumTotalDegs = Deg_GetTotalNumberOfDegrees ();

   fprintf (Gbl.F.Out,"<tr><td width=\"20\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
            Txt_Institution_unspecified[Gbl.Prefs.Language],
            NumTotalUsrs - NumUsrsWithInstitution,
            NumTotalStds - NumStdsWithInstitution,
            NumTotalTchs - NumTchsWithInstitution,
            NumTotalCtrs - NumCtrsWithInstitution,
            NumTotalDpts - NumDptsWithInstitution,
            NumTotalDegs - NumDegsWithInstitution);

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of institutions *****/
   Ins_FreeListInstitutions ();
  }

/*****************************************************************************/
/******* Get parameter with the type or order in list of institutions ********/
/*****************************************************************************/

static void Ins_GetParamInsOrderType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Inss.SelectedOrderType = (tInssOrderType) UnsignedNum;
   else
      Lay_ShowErrorAndExit ("Sort order is missing.");
  }

/****************************************************************************/
/** Put a hidden parameter with the type of order in list of institutions ***/
/****************************************************************************/

void Ins_PutHiddenParamInsOrderType (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Inss.SelectedOrderType);
  }

/****************************************************************************/
/******************** Put a link (form) to edit institutions ****************/
/****************************************************************************/

static void Ins_PutFormToEditInstitutions (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiIns);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************ Put forms to edit institutions ********************/
/****************************************************************************/

void Ins_EditInstitutions (void)
  {
   extern const char *Txt_Institutions[Txt_NUM_LANGUAGES];

   /***** Get list of institutions *****/
   Ins_GetListInstitutions (true);

   /***** Help message *****/
   if (Gbl.Inss.Num) // Institutions found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir una nueva instituci&oacute;n rellene el nombre de la instituci&oacute;n, su logo y su direcci&oacute;n web," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar una instituci&oacute;n existente pulse el aspa roja.<br />" \
                          "Para modificar una instituci&oacute;n existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Institutions[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No institutions found
      Lay_ShowAlert (INFO,"No existen instituciones.<br />" \
                          "Para a&ntilde;adir una instituci&oacute;n rellene el nombre de la instituci&oacute;n, su logo y su direcci&oacute;n web," \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit institutions *****/
   /* Start table with institutions */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current institutions */
   if (Gbl.Inss.Num)
      Ins_ListInstitutionsForEdition ();

   /* Put a form to create a new institution */
   Ins_PutFormToCreateInstitution ();

   /* End of table with institutions */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of institutions *****/
   Ins_FreeListInstitutions ();
  }

/****************************************************************************/
/********************** Get list of current institutions ********************/
/****************************************************************************/

void Ins_GetListInstitutions (bool GetExtraData)
  {
   char OrderBySubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumIns;
   struct Institution *Ins;

   /***** Get institutions from database *****/
   if (GetExtraData)
     {
      switch (Gbl.Inss.SelectedOrderType)
        {
         case INS_ORDER_BY_INSTITUTION:
            sprintf (OrderBySubQuery,"FullName");
            break;
         case INS_ORDER_BY_NUM_USRS:
            sprintf (OrderBySubQuery,"NumUsrs DESC,FullName");
        }
      sprintf (Query,"(SELECT institutions.InsCod,institutions.ShortName,institutions.FullName," \
                     "institutions.Logo,institutions.WWW,COUNT(*) AS NumUsrs" \
                     " FROM institutions,usr_data" \
                     " WHERE usr_data.InsCod=institutions.InsCod" \
                     " GROUP BY institutions.InsCod)" \
                     " UNION " \
                     "(SELECT InsCod,ShortName,FullName,Logo,WWW,0 AS NumUsrs" \
                     " FROM institutions" \
                     " WHERE InsCod NOT IN (SELECT DISTINCT InsCod FROM usr_data))" \
                     " ORDER BY %s",
            OrderBySubQuery);
     }
   else
      sprintf (Query,"SELECT InsCod,ShortName,FullName,Logo,WWW" \
                     " FROM institutions ORDER BY FullName");
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get institutions");

   if (NumRows) // Institutions found...
     {
      // NumRows should be equal to Deg->NumCourses
      Gbl.Inss.Num = (unsigned) NumRows;

      /***** Create list with institutions *****/
      if ((Gbl.Inss.Lst = (struct Institution *) calloc (NumRows,sizeof (struct Institution))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store institutions.");

      /***** Get the institutions *****/
      for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
        {
         Ins = &(Gbl.Inss.Lst[NumIns]);

         /* Get next institution */
         row = mysql_fetch_row (mysql_res);

         /* Get institution code (row[0]) */
         if ((Ins->InsCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of institution.");

         /* Get the short name of the institution (row[1]) */
         strcpy (Ins->ShortName,row[1]);

         /* Get the full name of the institution (row[2]) */
         strcpy (Ins->FullName,row[2]);

         /* Get the logo of the institution (row[3]) */
         strcpy (Ins->Logo,row[3]);

         /* Get the URL of the institution (row[4]) */
         strcpy (Ins->WWW,row[4]);

         /* Get extra data */
         if (GetExtraData)
           {
            /* Get number of users in this institution (row[5]) */
            if (sscanf (row[5],"%u",&Ins->NumUsrs) == 1)
              {
               if (Ins->NumUsrs)
                 {
                  Ins->NumStds = Usr_GetNumberOfUsersInInstitution (Ins->InsCod,Usr_STUDENT);
                  Ins->NumTchs = Usr_GetNumberOfUsersInInstitution (Ins->InsCod,Usr_TEACHER);
                  Ins->NumUsrs = Ins->NumStds + Ins->NumTchs;
                 }
              }
            else
               Ins->NumStds = Ins->NumTchs = Ins->NumUsrs = 0;

            /* Get number of centres in this institution */
            Ins->NumCtrs = Ctr_GetNumberOfCentresInInstitution (Ins->InsCod);

            /* Get number of centres in this institution */
            Ins->NumDpts = Dpt_GetNumberOfDepartmentsInInstitution (Ins->InsCod);

            /* Get number of centres in this institution */
            Ins->NumDegs = Deg_GetNumberOfDegreesInInstitution (Ins->InsCod);
           }
         else
            Ins->NumStds = Ins->NumTchs = Ins->NumUsrs =
            Ins->NumCtrs = Ins->NumDpts = Ins->NumDegs = 0;
        }
     }
   else
      Gbl.Inss.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);   
  }

/*****************************************************************************/
/************************* Get data of an institution ************************/
/*****************************************************************************/

bool Ins_GetDataOfInstitutionByCod (struct Institution *Ins)
  {
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool InsFound;

   /***** Clear data *****/
   Ins->ShortName[0] = Ins->FullName[0] = Ins->Logo[0] = Ins->WWW[0] = '\0';
   Ins->NumStds = Ins->NumTchs = Ins->NumUsrs =
   Ins->NumCtrs = Ins->NumDpts = Ins->NumDegs = 0;

   /***** Check if institution code is correct *****/
   if (Ins->InsCod < 0)
      return false;
   if (Ins->InsCod == 0)
     {
      strcpy (Ins->ShortName,Txt_Another_institution[Gbl.Prefs.Language]);
      strcpy (Ins->FullName,Txt_Another_institution[Gbl.Prefs.Language]);
      return false;
     }
   // Ins->InsCod > 0

   /***** Get data of an institution from database *****/
   sprintf (Query,"(SELECT institutions.ShortName,institutions.FullName,institutions.Logo,institutions.WWW,COUNT(*)" \
                  " FROM institutions,usr_data" \
                  " WHERE institutions.InsCod='%ld' AND institutions.InsCod=usr_data.InsCod" \
                  " GROUP BY institutions.InsCod)" \
                  " UNION " \
                  "(SELECT ShortName,FullName,Logo,WWW,0" \
                  " FROM institutions" \
                  " WHERE InsCod='%ld' AND InsCod NOT IN (SELECT DISTINCT InsCod FROM usr_data))",
            Ins->InsCod,Ins->InsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of an institution");

   /***** Count number of rows in result *****/
   if (NumRows) // Institution found...
     {
      InsFound = true;

      /* Get row */
      row = mysql_fetch_row (mysql_res);

      /* Get the short name of the institution (row[0]) */
      strcpy (Ins->ShortName,row[0]);

      /* Get the full name of the institution (row[1]) */
      strcpy (Ins->FullName,row[1]);

      /* Get the logo of the institution (row[2]) */
      strcpy (Ins->Logo,row[2]);

      /* Get the URL of the institution (row[3]) */
      strcpy (Ins->WWW,row[3]);

      /* Get number of users in this institution (row[4]) */
      if (sscanf (row[4],"%u",&Ins->NumUsrs) == 1)
        {
         if (Ins->NumUsrs)
           {
            Ins->NumStds = Usr_GetNumberOfUsersInInstitution (Ins->InsCod,Usr_STUDENT);
            Ins->NumTchs = Usr_GetNumberOfUsersInInstitution (Ins->InsCod,Usr_TEACHER);
            Ins->NumUsrs = Ins->NumStds + Ins->NumTchs;
           }
        }
      else
         Ins->NumUsrs = 0;
     }
   else
      InsFound = false;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return InsFound;
  }

/*****************************************************************************/
/************************* Free list of institutions *************************/
/*****************************************************************************/

void Ins_FreeListInstitutions (void)
  {
   if (Gbl.Inss.Lst)
     {
      /***** Free memory used by the list of institutions *****/
      free ((void *) Gbl.Inss.Lst);
      Gbl.Inss.Lst = NULL;
      Gbl.Inss.Num = 0;
     }
  }

/****************************************************************************/
/************************* List all the institutions ************************/
/****************************************************************************/

static void Ins_ListInstitutionsForEdition (void)
  {
   extern const char *Txt_Remove_institution[Txt_NUM_LANGUAGES];
   unsigned NumIns;
   struct Institution *Ins;

   /***** Table head *****/
   Ins_PutHeadInstitutions ();

   /***** Write all the institutions *****/
   for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
     {
      Ins = &Gbl.Inss.Lst[NumIns];

      /* Put icon to remove institution */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      if (Ins->NumUsrs)	// Institution has users ==> deletion forbidden
         fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActRemIns);
         Ins_PutParamInsCod (Ins->InsCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_institution[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Txt_Remove_institution[Gbl.Prefs.Language]);
        }
      fprintf (Gbl.F.Out,"</td>");

      /* Institution code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Ins->InsCod);

      /* Institution logo */
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"20\"><img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" alt=\"%s\" /></td>",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.Inss.Lst[NumIns].Logo,Gbl.Inss.Lst[NumIns].ShortName);

      /* Institution short name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenInsSho);
      Ins_PutParamInsCod (Ins->InsCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"12\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_INSTITUTION_SHORT_NAME,Ins->ShortName,Gbl.NumForm);

      /* Institution full name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenInsFul);
      Ins_PutParamInsCod (Ins->InsCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_INSTITUTION_FULL_NAME,Ins->FullName,Gbl.NumForm);

      /* Institution logo */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgInsLog);
      Ins_PutParamInsCod (Ins->InsCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Logo\" size=\"8\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_INSTITUTION_LOGO,Ins->Logo,Gbl.NumForm);

      /* Institution WWW */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgInsWWW);
      Ins_PutParamInsCod (Ins->InsCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"WWW\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Cte_MAX_LENGTH_WWW,Ins->WWW,Gbl.NumForm);

      /* Number of users */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Ins->NumUsrs);
     }
  }

/****************************************************************************/
/***************** Write parameter with code of institution *****************/
/****************************************************************************/

static void Ins_PutParamInsCod (long InsCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"InsCod\" value=\"%ld\" />",InsCod);
  }

/****************************************************************************/
/******************* Get parameter with code of institution *****************/
/****************************************************************************/

long Ins_GetParamInsCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of institution *****/
   Par_GetParam (Par_PARAM_SINGLE,"InsCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/**************************** Remove a institution **************************/
/****************************************************************************/

void Ins_RemoveInstitution (void)
  {
   extern const char *Txt_Institution_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Institution Ins;

   /***** Get institution code *****/
   if ((Ins.InsCod = Ins_GetParamInsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institution is missing.");

   /***** Get data of the institution from database *****/
   Ins_GetDataOfInstitutionByCod (&Ins);

   /***** Check if this institution has users *****/
   if (Ins.NumUsrs)	// Institution has users ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar una instituci&oacute;n" \
                             " primero debe eliminar todos los usuarios de esa instituci&oacute;n.");	// Need translation!
   else	// Institution has no users ==> remove it
     {
      /***** Remove institution *****/
      sprintf (Query,"DELETE FROM institutions WHERE InsCod='%ld'",Ins.InsCod);
      DB_QueryDELETE (Query,"can not remove an institution");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Institution_X_removed[Gbl.Prefs.Language],
               Ins.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Ins_EditInstitutions ();
  }

/****************************************************************************/
/***************** Change the short name of an institution ******************/
/****************************************************************************/

void Ins_RenameInsShort (void)
  {
   Ins_RenameInstitution (Cte_SHORT_NAME);
  }

/****************************************************************************/
/***************** Change the full name of an institution *******************/
/****************************************************************************/

void Ins_RenameInsFull (void)
  {
   Ins_RenameInstitution (Cte_FULL_NAME);
  }

/****************************************************************************/
/******************** Change the name of an institution *********************/
/****************************************************************************/

static void Ins_RenameInstitution (Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_institution_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Institution *Ins;
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentInsName = NULL;		// Initialized to avoid warning
   char NewInsName[MAX_LENGTH_INSTITUTION_FULL_NAME+1];

   Ins = &Gbl.Inss.EditingIns;
   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = MAX_LENGTH_INSTITUTION_SHORT_NAME;
         CurrentInsName = Ins->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = MAX_LENGTH_INSTITUTION_FULL_NAME;
         CurrentInsName = Ins->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the institution */
   if ((Ins->InsCod = Ins_GetParamInsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institution is missing.");

   /* Get the new name for the institution */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewInsName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewInsName,MaxLength,true);

   /***** Get from the database the old names of the institution *****/
   Ins_GetDataOfInstitutionByCod (Ins);

   /***** Check if new name is empty *****/
   if (!NewInsName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentInsName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentInsName,NewInsName))	// Different names
        {
         /***** If institution was in database... *****/
         if (Ins_CheckIfInstitutionNameExists (ParamName,NewInsName,Ins->InsCod))
           {
            sprintf (Gbl.Message,Txt_The_institution_X_already_exists[Gbl.Prefs.Language],
                     NewInsName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE institutions SET %s='%s' WHERE InsCod='%ld'",
                     FieldName,NewInsName,Ins->InsCod);
            DB_QueryUPDATE (Query,"can not update the name of an institution");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"La instituci&oacute;n <strong>%s</strong> ha sido renombrada como <strong>%s</strong>.",	// Need translation!
                     CurrentInsName,NewInsName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.",	// Need translation!
                  CurrentInsName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (CurrentInsName,NewInsName);
   Ins_EditInstitutions ();
  }

/****************************************************************************/
/****************** Check if the name of institution exists *****************/
/****************************************************************************/

static bool Ins_CheckIfInstitutionNameExists (const char *FieldName,const char *Name,long InsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of institutions with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM institutions" \
                  " WHERE %s='%s' AND InsCod<>'%ld'",
            FieldName,Name,InsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of an institution already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of an institution already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************* Change the logo of a institution *********************/
/****************************************************************************/

void Ins_ChangeInsLogo (void)
  {
   struct Institution *Ins;
   char Query[256+MAX_LENGTH_INSTITUTION_LOGO];
   char NewLogo[MAX_LENGTH_INSTITUTION_LOGO+1];

   Ins = &Gbl.Inss.EditingIns;

   /***** Get parameters from form *****/
   /* Get the code of the institution */
   if ((Ins->InsCod = Ins_GetParamInsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institution is missing.");

   /* Get the new logo for the institution */
   Par_GetParam (Par_PARAM_SINGLE,"Logo",NewLogo,MAX_LENGTH_INSTITUTION_LOGO);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewLogo,MAX_LENGTH_INSTITUTION_LOGO,true);

   /***** Check if new logo is empty *****/
   if (NewLogo[0])
     {
      /* Update the table changing old logo by new logo */
      sprintf (Query,"UPDATE institutions SET Logo='%s' WHERE InsCod='%ld'",
               NewLogo,Ins->InsCod);
      DB_QueryUPDATE (Query,"can not update the logo of an institution");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"El nuevo logo es <strong>%s</strong>.",	// Need translation!
               NewLogo);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
      Lay_ShowAlert (WARNING,"No puede dejar el logo vac&iacute;o.");	// Need translation!

   /***** Show the form again *****/
   strcpy (Ins->Logo,NewLogo);
   Ins_EditInstitutions ();
  }

/****************************************************************************/
/********************** Change the URL of a institution *********************/
/****************************************************************************/

void Ins_ChangeInsWWW (void)
  {
   struct Institution *Ins;
   char Query[256+Cte_MAX_LENGTH_WWW];
   char NewWWW[Cte_MAX_LENGTH_WWW+1];

   Ins = &Gbl.Inss.EditingIns;

   /***** Get parameters from form *****/
   /* Get the code of the institution */
   if ((Ins->InsCod = Ins_GetParamInsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institution is missing.");

   /* Get the new WWW for the institution */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",NewWWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewWWW,Cte_MAX_LENGTH_WWW,true);

   /***** Check if new WWW is empty *****/
   if (NewWWW[0])
     {
      /* Update the table changing old WWW by new WWW */
      sprintf (Query,"UPDATE institutions SET WWW='%s' WHERE InsCod='%ld'",
               NewWWW,Ins->InsCod);
      DB_QueryUPDATE (Query,"can not update the web of an institution");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva direcci&oacute;n web es <strong>%s</strong>.",	// Need translation!
               NewWWW);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
      Lay_ShowAlert (WARNING,"No puede dejar la direcci&oacute;n web vac&iacute;a.");	// Need translation!

   /***** Show the form again *****/
   strcpy (Ins->WWW,NewWWW);
   Ins_EditInstitutions ();
  }

/****************************************************************************/
/******************* Put a form to create a new institution *****************/
/****************************************************************************/

static void Ins_PutFormToCreateInstitution (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_institution[Txt_NUM_LANGUAGES];
   struct Institution *Ins;

   Ins = &Gbl.Inss.EditingIns;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"8\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"8\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_institution[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewIns);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Ins_PutHeadInstitutions ();

   /***** New institution *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_institution[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Institution code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Institution logo *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"20\"></td>");

   /***** Institution short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"ShortName\" size=\"12\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_INSTITUTION_SHORT_NAME,Ins->ShortName);

   /***** Institution full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_INSTITUTION_FULL_NAME,Ins->FullName);

   /***** Institution logo *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"Logo\" size=\"8\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_INSTITUTION_LOGO,Ins->Logo);

   /***** Institution WWW *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"WWW\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Cte_MAX_LENGTH_WWW,Ins->WWW);
  }

/****************************************************************************/
/**************** Write header with fields of an institution ****************/
/****************************************************************************/

static void Ins_PutHeadInstitutions (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_Name[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" class=\"TIT_TBL\">&nbsp;</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\" width=\"20\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">Logo</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">WWW</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">Usrs.</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Short_Name[Gbl.Prefs.Language],
            Txt_Full_Name[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/**************** Receive form to create a new institution ******************/
/****************************************************************************/

void Ins_RecFormNewIns (void)
  {
   extern const char *Txt_The_institution_X_already_exists[Txt_NUM_LANGUAGES];
   struct Institution *Ins;

   Ins = &Gbl.Inss.EditingIns;

   /***** Get parameters from form *****/
   /* Get institution short name */
   Par_GetParam (Par_PARAM_SINGLE,"ShortName",Ins->ShortName,MAX_LENGTH_INSTITUTION_SHORT_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ins->ShortName,MAX_LENGTH_INSTITUTION_SHORT_NAME,true);

   /* Get institution full name */
   Par_GetParam (Par_PARAM_SINGLE,"FullName",Ins->FullName,MAX_LENGTH_INSTITUTION_FULL_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ins->FullName,MAX_LENGTH_INSTITUTION_FULL_NAME,true);

   /* Get institution logo */
   Par_GetParam (Par_PARAM_SINGLE,"Logo",Ins->Logo,MAX_LENGTH_INSTITUTION_LOGO);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ins->Logo,MAX_LENGTH_INSTITUTION_LOGO,true);

   /* Get institution WWW */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",Ins->WWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Ins->WWW,Cte_MAX_LENGTH_WWW,true);

   if (Ins->ShortName[0] && Ins->FullName[0])	// If there's a institution name
     {
      if (Ins->Logo[0] && Ins->WWW[0])
        {
         /***** If name of institution was in database... *****/
         if (Ins_CheckIfInstitutionNameExists ("ShortName",Ins->ShortName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_institution_X_already_exists[Gbl.Prefs.Language],
                     Ins->ShortName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else if (Ins_CheckIfInstitutionNameExists ("FullName",Ins->FullName,-1L))
           {
            sprintf (Gbl.Message,Txt_The_institution_X_already_exists[Gbl.Prefs.Language],
                     Ins->FullName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else	// Add new institution to database
            Ins_CreateInstitution (Ins);
        }
      else	// If there isn't a logo or a web
         Lay_ShowAlert (WARNING,"Debe especificar el logo y la direcci&oacute;n web de la nueva instituci&oacute;n.");	// Need translation!
     }
   else	// If there isn't a institution name
      Lay_ShowAlert (WARNING,"Debe especificar el nombre breve y el nombre completo de la nueva instituci&oacute;n.");	// Need translation!

   /***** Show the form again *****/
   Ins_EditInstitutions ();
  }

/****************************************************************************/
/************************** Create a new institution ************************/
/****************************************************************************/

static void Ins_CreateInstitution (struct Institution *Ins)
  {
   extern const char *Txt_Created_new_institution_X[Txt_NUM_LANGUAGES];
   char Query[2048];

   /***** Create a new institution *****/
   sprintf (Query,"INSERT INTO institutions (ShortName,FullName,Logo,WWW)" \
                  " VALUES ('%s','%s','%s','%s')",
            Ins->ShortName,Ins->FullName,Ins->Logo,Ins->WWW);
   DB_QueryINSERT (Query,"can not create institution");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_institution_X[Gbl.Prefs.Language],
            Ins->FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }
