// swad_lay.c: page layout

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For exit, system, malloc, calloc, free, etc
#include <string.h>		// For string functions

#include "swad_act.h"
#include "swad_con.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_not.h"
#include "swad_prf.h"
#include "swad_the.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Tabs Tabs[Act_NUM_TABS];
extern struct Actions Actions[Act_NUM_ACTIONS];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Lay_WritePageTitle (void);

static void Lay_WriteScripts(void);
static void Lay_WriteScriptInit (void);
static void Lay_WriteScriptConnectedUsrs (void);
static void Lay_WriteScriptSkype (void);

static void Lay_WritePageTopHeading (void);
static void Lay_StartLogoTable (void);
static void Lay_EndLogoTable (void);

static void Lay_DrawTabs (void);
static void Lay_WriteMenuThisTab (void);
static void Lay_WriteTitleAction (void);

/****************************************************************************/
/*************************** List all the centres ***************************/
/****************************************************************************/

/****************************************************************************/
/*********************** Write the start of the page ************************/
/****************************************************************************/

void Lay_WriteStartOfPage (void)
  {
   extern const char *The_CentralBgColors[The_NUM_THEMES];
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_NEW_YEAR_GREETING[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_photo[Txt_NUM_LANGUAGES];

   /***** If, when this function is called, the head is being written, or the head is already written ==>
           ==> don't do anything *****/
   if (Gbl.WritingHTMLStart || Gbl.HTMLStartWritten)
      return;

   /***** Send head width the file type for the HTTP protocol *****/
   if (Gbl.CurrentAct == ActRef)	// Don't generate a full HTML page, only refresh connected users
     {
      fprintf (Gbl.F.Out,"Content-Type: text/html; charset=ISO-8859-1\n");
      fprintf (Gbl.F.Out,"Cache-Control: max-age=0, no-cache, must-revalidate\n\n");

      // fprintf (Gbl.F.Out,"Expires: Thu, 29 Oct 1998 17:04:19 GMT\n\n");
      Gbl.WritingHTMLStart = false;
      Gbl.HTMLStartWritten = Gbl.TablEndWritten = true;
      return;
     }

   /***** If serving a web service ==> don't do anything *****/
   if (Gbl.CurrentAct == ActWebSvc)
     {
      Gbl.WritingHTMLStart = false;
      Gbl.HTMLStartWritten = Gbl.TablEndWritten = true;
      return;
     }

   Gbl.WritingHTMLStart = true;

   // Write next header to standard output to avoid timeout
   fprintf (stdout,"Content-type: text/html; charset=ISO-8859-1\n\n"); /* Ojo. Debe haber dos retornos of carro al final */
   fprintf (stdout,"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");

   /***** Write start of HTML code *****/
   fprintf (Gbl.F.Out,"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n" \
                      "<head>\n");

   /* Character encoding declaration */
   fprintf (Gbl.F.Out,"<meta http-equiv=\"Content-Type\" content=\"text/html;charset=iso-8859-1\" />\n");

   /* Title */
   Lay_WritePageTitle ();

   /* Style sheet */
   fprintf (Gbl.F.Out,"<link rel=\"StyleSheet\" href=\"/swad.css\" type=\"text/css\" />\n");

   /* Write initial scripts depending on the action */
   Lay_WriteScripts ();

   /* Soft transition between pages (avoid white screen) for Microsoft Internet Explorer */
   if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW)
      fprintf (Gbl.F.Out,"<meta http-equiv=\"Page-Exit\" content=\"blendTrans(Duration=0)\" />\n");

   fprintf (Gbl.F.Out,"</head>\n");

   /***** HTML body *****/
   fprintf (Gbl.F.Out,"<body bgcolor=\"#%s\"",WHITE);
   if (Actions[Gbl.CurrentAct].BrowserWindow != Act_MAIN_WINDOW)
     {
      fprintf (Gbl.F.Out,">\n");
      Gbl.WritingHTMLStart = false;
      Gbl.HTMLStartWritten = Gbl.TablEndWritten = true;
      return;
     }
   fprintf (Gbl.F.Out," onload=\"inicio()\">\n");

   if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW)
      fprintf (Gbl.F.Out,"<div id=\"zoomLyr\" class=\"ZOOM\">" \
                         "<img id=\"zoomImg\" src=\"%s/_.gif\" alt=\"\" class=\"IMG_USR\" /><br />" \
                         "<div id=\"zoomTxt\" align=\"center\"></div>" \
                         "</div>\n",
	       Gbl.Prefs.IconsURL);

   fprintf (Gbl.F.Out,"<table width=\"100%%\" cellpadding=\"0\" cellspacing=\"0\">" \
                      "<colgroup><col width=\"128\" /><col width=\"*\" /><col width=\"128\" /></colgroup>");

   /***** Header of layout *****/
   Lay_WritePageTopHeading ();

   fprintf (Gbl.F.Out,"<tr>");

   /***** Left column *****/
   if (Gbl.Prefs.SideCols & 2)	// Left column visible
     {
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" width=\"128\"" \
                         " style=\"background-image: url('%s/head_base_background_1x20.gif'); background-repeat: repeat-x;\">" \
                         "<table width=\"100%%\" cellspacing=\"4\" cellpadding=\"0\">",
               Gbl.Prefs.PathTheme);

      /* Menu with links to all degrees and courses */
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\">");
      Deg_WriteMenuAllCourses ();
      fprintf (Gbl.F.Out,"</td></tr>");

      /* Month and hour */
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\">");
      Dat_DrawMonthAndHour ();
      fprintf (Gbl.F.Out,"</td></tr>");

      /* Notices */
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\">");
      Not_ShowNotices (Not_LIST_NOTICES_TO_SEE);
      fprintf (Gbl.F.Out,"</td></tr></table></td>\n");
     }

   /***** Central (main) part *****/
   fprintf (Gbl.F.Out,"<td");
   if (Gbl.Prefs.SideCols == 2 ||
       Gbl.Prefs.SideCols == 1)	// Only one side column visible, left or right
      fprintf (Gbl.F.Out," colspan=\"2\"");
   else if (Gbl.Prefs.SideCols == 0)	// Both side columns hidden, left and right
      fprintf (Gbl.F.Out," colspan=\"3\"");
   fprintf (Gbl.F.Out," align=\"left\" valign=\"top\"" \
                      " style=\"background-image: url('%s/head_base_background_1x20.gif'); background-repeat: repeat-x;\">" \
                      "<table width=\"100%%\" cellspacing=\"0\" cellpadding=\"0\">" \
                      "<colgroup><col width=\"10\" /><col width=\"*\" align=\"center\" /><col width=\"10\" /></colgroup>" \
                      "<tr><td align=\"left\" width=\"10\" style=\"background-image: url('%s/central_corner_1_10x64.gif');\">" \
                      "<img src=\"%s/tr10x28.gif\" width=\"10\" height=\"28\" alt=\"\" /></td>" \
                      "<td align=\"center\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">",
            Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,Gbl.Prefs.IconsURL);

   /* Tabs menu */
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\" style=\"background-image: url('%s/central_top_1x64.gif'); background-repeat: repeat-x;\">",
            Gbl.Prefs.PathTheme);
   Lay_DrawTabs ();
   fprintf (Gbl.F.Out,"</td></tr></table></td>" \
                      "<td align=\"left\" width=\"10\" style=\"background-image: url('%s/central_corner_2_10x64.gif');\">" \
                      "<img src=\"%s/tr10x28.gif\" width=\"10\" height=\"28\" alt=\"\" /></td>",
	    Gbl.Prefs.PathTheme,Gbl.Prefs.IconsURL);
   fprintf (Gbl.F.Out,"</tr>");

   /* Left bar used to expand-contract central zone */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" width=\"10\"" \
                      " style=\"background-image: url('%s/central_left_10x1.gif'); background-repeat: repeat-y;\">",
            Gbl.Prefs.PathTheme);
   Prf_PutLeftIconToHideShowCols ();
   fprintf (Gbl.F.Out,"</td>");

   /* Tab content, including vertical menu (left) and main zone (right) */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">" \
                      "<tr><td align=\"left\" valign=\"top\" width=\"124\">",The_CentralBgColors[Gbl.Prefs.Theme]);
   Lay_WriteMenuThisTab ();
   fprintf (Gbl.F.Out,"</td>");

   /* Main zone */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\">\n");

   if (Actions[Actions[Gbl.CurrentAct].SuperAction].IndexInMenu >= 0)
      Lay_WriteTitleAction ();

   Gbl.WritingHTMLStart = false;
   Gbl.HTMLStartWritten = true;

   /* Write new year greeting */
   if (Gbl.CurrentAct == ActAutUsrInt || Gbl.CurrentAct == ActAutUsrExt)
      if (Gbl.Today.Month == 1 && Gbl.Today.Day == 1)
        {
         fprintf (Gbl.F.Out,"<div align=\"center\" class=\"ASG_TITLE\">" \
                            " <br /> <br /> <br />");
         fprintf (Gbl.F.Out,Txt_NEW_YEAR_GREETING[Gbl.Prefs.Language],Gbl.Today.Year);
         fprintf (Gbl.F.Out,"<br /> <br /> <br /> " \
                            "</div>");
        }

   /* Write message indicating number of clicks allowed before sending my photo */
   Usr_InformAboutNumClicksBeforePhoto ();

   /* If user has just log in, show warnings */
   if (Gbl.CurrentAct == ActAutUsrInt && Gbl.Usrs.Me.Logged && Gbl.Usrs.Me.UsrDat.Password[0])
     {
      if (Gbl.Usrs.Me.UsrDat.FirstName[0] && Gbl.Usrs.Me.UsrDat.Surname1[0] &&
          !Gbl.Usrs.Me.MyPhotoExists)	// User has a name but has no photo
        {
         sprintf (Gbl.Message,"Usted no dispone de fotograf&iacute;a en <strong>%s</strong>.",	// Need translation!
                  Cfg_PLATFORM_SHORT_NAME);
         Lay_ShowAlert (WARNING,Gbl.Message);
	 fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">" \
                            "<tr><td align=\"center\" valign=\"middle\">");
	 Act_FormStart (ActReqMyPho);
         strcpy (Gbl.MsgWindowStatus,Txt_Upload_photo[Gbl.Prefs.Language]);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/photo12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /> %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
        }
     }
  }

/****************************************************************************/
/************************ Write the end of the page *************************/
/****************************************************************************/

void Lay_WriteEndOfPage (void)
  {
   if (!Gbl.TablEndWritten)
     {
      Gbl.TablEndWritten = true;
      fprintf (Gbl.F.Out,"</td></tr></table></td>");

      /* Right bar used to expand-contract central zone */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" width=\"10\"" \
                         " style=\"background-image: url('%s/central_right_10x1.gif'); background-repeat: repeat-y;\">",
               Gbl.Prefs.PathTheme);
      Prf_PutRigthIconToHideShowCols ();
      fprintf (Gbl.F.Out,"</td></tr>");

      /* Bottom of the central zone */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"10\"><img src=\"%s/central_corner_3_10x10.gif\" width=\"10\" height=\"10\" alt=\"\" /></td>" \
                         "<td align=\"left\" style=\"background-image: url('%s/central_bottom_1x10.gif'); background-repeat: repeat-x;\"></td>" \
                         "<td align=\"right\" width=\"10\">" \
                         "<img src=\"%s/central_corner_4_10x10.gif\" width=\"10\" height=\"10\" alt=\"\" /></td>" \
                         "</tr></table></td>\n",
               Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme);

      if (Gbl.Prefs.SideCols & 1)	// Right column visible
         Usr_ShowConnectedUsrsAndRightMenu ();

      fprintf (Gbl.F.Out,"</tr></table>\n");
     }
  }

/****************************************************************************/
/************************ Write the title of the page ***********************/
/****************************************************************************/

static void Lay_WritePageTitle (void)
  {
   fprintf (Gbl.F.Out,"<title>");
   if (Gbl.GetMethod && Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      fprintf (Gbl.F.Out,"%s &gt; %s",Cfg_PLATFORM_PAGE_TITLE,Gbl.CurrentDeg.Deg.ShortName);
      if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
         fprintf (Gbl.F.Out," &gt; %s",Gbl.CurrentCrs.Crs.ShortName);
     }
   else
      fprintf (Gbl.F.Out,"%s",Cfg_PLATFORM_PAGE_TITLE);
   fprintf (Gbl.F.Out,"</title>\n");
  }

/****************************************************************************/
/************ Write some scripts depending on the current action ************/
/****************************************************************************/

static void Lay_WriteScripts (void)
  {
   fprintf (Gbl.F.Out,"<script type=\"text/javascript\" src=\"swad.js\"></script>\n");

   if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW)
     {
      Lay_WriteScriptInit ();
      Lay_WriteScriptConnectedUsrs ();
     }

   switch (Gbl.CurrentAct)
     {
      case ActSeeStd:
      case ActSeeCrdCrsStd:
      case ActRecCrdCrsOthUsr:
      case ActActMyData:
      case ActUpdOthUsrDat:
	 Lay_WriteScriptSkype ();
	 break;
     }
  }

/****************************************************************************/
/******* Write script with init function executed after loading page ********/
/****************************************************************************/

static void Lay_WriteScriptInit (void)
  {
   fprintf (Gbl.F.Out,"<script type=\"text/javascript\" language=\"JavaScript\">\n");

   fprintf (Gbl.F.Out,"function inicio(){\n");
   if (Gbl.Prefs.SideCols & 2)	// Left column visible
     {
      fprintf (Gbl.F.Out,"	Hour = %u;\n",Gbl.Now.Hour);
      fprintf (Gbl.F.Out,"	Minute = %u;\n",Gbl.Now.Minute);
      fprintf (Gbl.F.Out,"	writeClock();\n");
     }
   fprintf (Gbl.F.Out,"	setTimeout(\"refreshConnected()\",%lu);\n",Gbl.Usrs.Connected.TimeToRefreshInMs);
   fprintf (Gbl.F.Out,"}\n");

   fprintf (Gbl.F.Out,"</script>\n");
  }

/****************************************************************************/
/************** Write script to show connected users using AJAX *************/
/****************************************************************************/

static void Lay_WriteScriptConnectedUsrs (void)
  {
   fprintf (Gbl.F.Out,"<script type=\"text/javascript\" language=\"JavaScript\">\n");
   fprintf (Gbl.F.Out,"var RefreshParam1 = \"NxtAct=ActRef\";\n");
   fprintf (Gbl.F.Out,"var RefreshParam2 = \"IdSes=%s\";\n",Gbl.Session.Id);
   fprintf (Gbl.F.Out,"var RefreshParam3 = \"CrsCod=%ld\";\n",Gbl.CurrentCrs.Crs.CrsCod);
   fprintf (Gbl.F.Out,"</script>\n");
  }

/****************************************************************************/
/******************* Write script for the zoom of the photos ****************/
/****************************************************************************/

static void Lay_WriteScriptSkype (void)
  {
   fprintf (Gbl.F.Out,"<script type=\"text/javascript\" src=\"https://%s/skypeCheck.js\">\n" \
                      "</script>\n",
            Cfg_PLATFORM_URL);
  }

/****************************************************************************/
/************************ Write top heading of the page *********************/
/****************************************************************************/

static void Lay_WritePageTopHeading (void)
  {
   extern const char *The_ClassHead[The_NUM_THEMES];

   /***** 1st. row, 1st. column: left logo *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"middle\" width=\"128\"" \
                      " style=\"background-image: url('%s/head_row1_1x512.gif'); background-repeat: repeat-x;\">",
            Gbl.Prefs.PathTheme);

   /* Left logo */
   Lay_StartLogoTable ();
   fprintf (Gbl.F.Out,"<a href=\"http://%s/\" target=\"_blank\">" \
	              "<img src=\"%s/swad112x48.png\" width=\"112\" height=\"48\" alt=\"%s\" />" \
                      "</a>",
            Cfg_PLATFORM_URL,Gbl.Prefs.IconsURL,Cfg_PLATFORM_FULL_NAME);
   Lay_EndLogoTable ();
   fprintf (Gbl.F.Out,"</td>");

   /***** 1st. row, 2nd. column: logged user *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"%s\"" \
                      " style=\"background-image: url('%s/head_row1_1x512.gif'); background-repeat: repeat-x;\">",
            The_ClassHead[Gbl.Prefs.Theme],Gbl.Prefs.PathTheme);
   if (Gbl.Usrs.Me.Logged)
      /* Logged user */
      Usr_WriteLoggedUsr ();
   else
      /* Form to login */
      Usr_WriteFormLogin ();
   fprintf (Gbl.F.Out,"</td>");

   /***** 1st. row, 3rd. column: language and theme selection *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" width=\"128\" class=\"%s\"" \
                      " style=\"background-image: url('%s/head_row1_1x512.gif'); background-repeat: repeat-x;\">",
            The_ClassHead[Gbl.Prefs.Theme],Gbl.Prefs.PathTheme);

   /* Language selection */
   Prf_PutSelectorToSelectLanguage ();

   /* Theme selection */
   The_PutIconsToSelectTheme ();
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** 2nd. row, 1st. column: direct link to my courses *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"middle\" width=\"128\" class=\"%s\"" \
                      " style=\"background-image: url('%s/head_row2_1x512.gif'); background-repeat: repeat-x;\">",
            The_ClassHead[Gbl.Prefs.Theme],Gbl.Prefs.PathTheme);

   /* Find course */
   Deg_PutFormSearchCourse ();

   /* Direct link to my courses */
   Deg_WriteSelectorMyCourses ();
   fprintf (Gbl.F.Out,"</td>");

   /***** 2nd. row, 2nd. column: degree and course *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\"" \
                      " style=\"background-image: url('%s/head_row2_1x512.gif'); background-repeat: repeat-x;\">",
            Gbl.Prefs.PathTheme);
   /* Degree and course */
   Deg_WriteDegreeAndCourseNames ();
   fprintf (Gbl.F.Out,"</td>");

   /***** 2nd. row, 3rd. column: right logo and number of connected users *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" width=\"128\"" \
                      " style=\"background-image: url('%s/head_row2_1x512.gif'); background-repeat: repeat-x;\">",
            Gbl.Prefs.PathTheme);

   /* Number of connected users */
   fprintf (Gbl.F.Out,"<div id=\"globalconnected\">");	// Used for AJAX based refresh
   Con_ShowGlobalConnectedUsrs ();
   fprintf (Gbl.F.Out,"</div>");			// Used for AJAX based refresh
   fprintf (Gbl.F.Out,"</td></tr>");
  }

/****************************************************************************/
/************** Start table for logo in the top of the page *****************/
/****************************************************************************/

static void Lay_StartLogoTable (void)
  {
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"120\">" \
                      "<tr><td align=\"left\" width=\"4\"><img src=\"%s/frame_logo_corner_1_4x8.gif\" width=\"4\" height=\"8\" alt=\"\" /></td>" \
                      "<td align=\"left\" width=\"112\"><img src=\"%s/frame_logo_top_1x8.gif\" width=\"112\" height=\"8\" alt=\"\" /></td>" \
                      "<td align=\"left\" width=\"4\"><img src=\"%s/frame_logo_corner_2_4x8.gif\" width=\"4\" height=\"8\" alt=\"\" /></td></tr>" \
                      "<tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/frame_logo_left_4x1.gif'); background-repeat: repeat-y;\"></td>" \
                      "<td width=\"112\" align=\"center\" valign=\"middle\" bgcolor=\"#%s\">",
            Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,WHITE);
  }

/****************************************************************************/
/************** End table for logo in the top of the page *****************/
/****************************************************************************/

static void Lay_EndLogoTable (void)
  {
   fprintf (Gbl.F.Out,"</td><td align=\"left\" width=\"4\" style=\"background-image: url('%s/frame_logo_right_4x1.gif'); background-repeat: repeat-y;\"></td></tr>" \
                      "<tr><td align=\"left\" width=\"4\"><img src=\"%s/frame_logo_corner_3_4x8.gif\" width=\"4\" height=\"8\" alt=\"\" /></td>" \
                      "<td align=\"left\" width=\"112\"><img src=\"%s/frame_logo_bottom_1x8.gif\" width=\"112\" height=\"8\" alt=\"\" /></td>" \
                      "<td align=\"left\" width=\"4\"><img src=\"%s/frame_logo_corner_4_4x8.gif\" width=\"4\" height=\"8\" alt=\"\" /></td></tr>" \
                      "</table>",
            Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme,Gbl.Prefs.PathTheme);
  }

/****************************************************************************/
/**************** Draw tabs with the current tab highlighted ****************/
/****************************************************************************/

static void Lay_DrawTabs (void)
  {
   extern const char *Txt_TABS_FULL_TXT[Act_NUM_TABS][Txt_NUM_LANGUAGES];
   extern const char *Txt_TABS_SHORT_TXT[Act_NUM_TABS][Txt_NUM_LANGUAGES];
   extern const char *The_ClassTabOn[The_NUM_THEMES];
   extern const char *The_ClassTabOff[The_NUM_THEMES];
   Act_Tab_t NumTab;
   enum {NO,OFF,ON} LeftTab = NO;
   Usr_UsrType_t UsrTypeLog = Gbl.Usrs.Me.Logged ? Gbl.Usrs.Me.UsrTypeLog : Usr_UNKNOWN;
   unsigned BitUsrType = (1 << UsrTypeLog);

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");

   /***** Draw the tabs *****/
   for (NumTab=(Act_Tab_t) 1; NumTab<=(Act_Tab_t) Act_NUM_TABS-1; NumTab++)
      // If I have permission to see this tab...
      if ((
           (Gbl.CurrentCrs.Crs.CrsCod >= 0)  &&
           (
            ( Gbl.Usrs.Me.IBelongToThisCrs   && (Tabs[NumTab].PermisIfIBelongToCrs     & BitUsrType))
              ||
            (!Gbl.Usrs.Me.IBelongToThisCrs   && (Tabs[NumTab].PermisIfIDontBelongToCrs & BitUsrType))
           )
          )
          ||
          (
           (Gbl.CurrentCrs.Crs.CrsCod  < 0)  && (Tabs[NumTab].PermisIfNoCrsSelected    & BitUsrType)
          )
          ||
          (
           (NumTab == TabPlg) && (!strcmp (Gbl.Usrs.Me.UsrDat.DNI,"P23226424") || !strcmp (Gbl.Usrs.Me.UsrDat.DNI,"76249962"))
          )
         )
	{
	 /* If current tab is unknown, then activate the first one with access allowed */
	 if (Gbl.CurrentTab == TabUnk)
	    Gbl.CurrentTab = NumTab;

	 /* Border between the tab on the left and this one */
	 fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\"><img src=\"%s/tab_",
                  Gbl.Prefs.PathTheme);
	 if (NumTab == Gbl.CurrentTab)
	   {
	    fprintf (Gbl.F.Out,(LeftTab == NO) ? "no_on_10x28.gif\" width=\"10\" height=\"28\" alt=\"\" />" :
				                 "off_on_20x28.gif\" width=\"20\" height=\"28\" alt=\"\" />");
	    LeftTab = ON;
	   }
	 else
	   {
	    fprintf (Gbl.F.Out,(LeftTab == NO)   ? "no_off_10x28.gif\" width=\"10\" height=\"28\" alt=\"\" />" :
		              ((LeftTab == OFF)) ? "off_off_20x28.gif\" width=\"20\" height=\"28\" alt=\"\" />" :
						   "on_off_20x28.gif\" width=\"20\" height=\"28\" alt=\"\" />");
	    LeftTab = OFF;
	   }
	 fprintf (Gbl.F.Out,"</td>");

	 /* Form, icon and text of the tab */
	 fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" style=\"background-image: url('%s/tab_%s_1x64.gif'); background-repeat: repeat-x;\" class=\"NO_BR\">",
		  Gbl.Prefs.PathTheme,(NumTab == Gbl.CurrentTab) ? "on" : "off");
	 if (NumTab != Gbl.CurrentTab)
           {
	    Act_FormStart (ActMnu);
            fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NxtTab\" value=\"%s\" />",Tabs[NumTab].Str);
           }
	 if (NumTab == Gbl.CurrentTab)
	    fprintf (Gbl.F.Out,"<span class=\"%s\">",The_ClassTabOn[Gbl.Prefs.Theme]);
	 else
	    Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_TABS_FULL_TXT[NumTab][Gbl.Prefs.Language],The_ClassTabOff[Gbl.Prefs.Theme]);

	 fprintf (Gbl.F.Out,"<img src=\"%s/%s16x17.gif\" width=\"16\" height=\"17\" align=\"bottom\" alt=\"%s\" />",
		  Gbl.Prefs.PathTheme,
                  (NumTab == Gbl.CurrentTab) ? Tabs[NumTab].IconOn : Tabs[NumTab].IconOff,
                  Txt_TABS_FULL_TXT[NumTab][Gbl.Prefs.Language]);
	 fprintf (Gbl.F.Out,"%s",Txt_TABS_SHORT_TXT[NumTab][Gbl.Prefs.Language]);
	 fprintf (Gbl.F.Out,(NumTab == Gbl.CurrentTab) ? "</span>" : "</acronym></a></form>");
	 fprintf (Gbl.F.Out,"</td>");
	}

   /***** Right border of the last tab *****/
   if (LeftTab != NO)
     {
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\"><img src=\"%s/tab_",
               Gbl.Prefs.PathTheme);
      fprintf (Gbl.F.Out,(LeftTab == OFF) ? "off_no" : "on_no");
      fprintf (Gbl.F.Out,"_10x28.gif\" width=\"10\" height=\"28\" alt=\"\" /></td>");
     }

   /***** End of the table *****/
   fprintf (Gbl.F.Out,"</tr></table>");
  }

/****************************************************************************/
/************************ Write the menu of current tab *********************/
/****************************************************************************/

static void Lay_WriteMenuThisTab (void)
  {
   extern const char *Txt_MENU[Act_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB][Txt_NUM_LANGUAGES];
   extern const char *The_ClassMenuOn[The_NUM_THEMES];
   extern const char *The_ClassMenuOff[The_NUM_THEMES];
   unsigned i;
   Act_Action_t NumAct;
   Usr_UsrType_t UsrTypeLog = Gbl.Usrs.Me.Logged ? Gbl.Usrs.Me.UsrTypeLog : Usr_UNKNOWN;
   const char *Title;

   /* Header of the table */
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"124\">");

   /***** Loop to write all options in menu. Each row holds an option *****/
   for (i=0, NumAct = Tabs[Gbl.CurrentTab].Menu.FirstOption; NumAct <= Tabs[Gbl.CurrentTab].Menu.LastOption; i++, NumAct++)
      if ((Gbl.CurrentCrs.Crs.CrsCod >= 0 && (( Gbl.Usrs.Me.IBelongToThisCrs && (Actions[NumAct].PermisIfIBelongToCrs    & (1 << UsrTypeLog)))   ||
                                              (!Gbl.Usrs.Me.IBelongToThisCrs && (Actions[NumAct].PermisIfIDontBelongToCrs & (1 << UsrTypeLog))))) ||
          (Gbl.CurrentCrs.Crs.CrsCod < 0 && (Actions[NumAct].PermisIfNoCrsSelected & (1 << UsrTypeLog)))) // If I have permission to execute this action...
	{
         Title = Act_GetTitleAction (NumAct);

	 /* Icon */
	 fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"24\">");
	 Act_FormStart (NumAct);
	 if (Actions[NumAct].FunctionExtraParams != NULL)
	    Actions[NumAct].FunctionExtraParams ();
	 Act_LinkFormSubmit (Actions[NumAct].BrowserWindow,Title,NULL);
	 fprintf (Gbl.F.Out,"<img src=\"%s/%s32x32.gif\" width=\"32\" height=\"32\" vspace=\"4\" alt=\"%s\" />" \
                            "</acronym></a></form></td>",
		  Gbl.Prefs.PathTheme,Actions[NumAct].Icon,Title);

	 /* Text */
	 fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" width=\"100\" style=\"padding-left:2px;\">");
	 Act_FormStart (NumAct);
	 if (Actions[NumAct].FunctionExtraParams != NULL)
	    Actions[NumAct].FunctionExtraParams ();
	 Act_LinkFormSubmit (Actions[NumAct].BrowserWindow,Title,
                             (NumAct == Actions[Gbl.CurrentAct].SuperAction) ? The_ClassMenuOn[Gbl.Prefs.Theme] : The_ClassMenuOff[Gbl.Prefs.Theme]);
	 fprintf (Gbl.F.Out,"%s</acronym></a></form></td></tr>",
                  Txt_MENU[Gbl.CurrentTab][i][Gbl.Prefs.Language]);
	}

   /* End of table */
   fprintf (Gbl.F.Out,"</table>");
  }

/****************************************************************************/
/*********** Write icon and title associated to the current action **********/
/****************************************************************************/

static void Lay_WriteTitleAction (void)
  {
   extern const char *The_ClassAction[The_NUM_THEMES];
   const char *Title = Act_GetTitleAction (Gbl.CurrentAct);

   /***** Start of rectangle *****/
   fprintf (Gbl.F.Out,"<div align=\"center\" style=\"padding-bottom:5px;\"><table cellspacing=\"0\" cellpadding=\"0\"><tr>");

   /***** Icon *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"bottom\" width=\"32\"><img src=\"%s/%s32x32.gif\" width=\"32\" height=\"32\" alt=\"%s\" /></td>",
            Gbl.Prefs.PathTheme,
            Actions[Actions[Gbl.CurrentAct].SuperAction].Icon,
            Title);

   /***** Title *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"bottom\" style=\"padding-left:5px; padding-right:5px;\" class=\"%s\">%s</td>",
            The_ClassAction[Gbl.Prefs.Theme],Title);

   /***** End of rectangle *****/
   fprintf (Gbl.F.Out,"</tr></table></div>");
  }

/****************************************************************************/
/********************** Put a button to submit a form ***********************/
/****************************************************************************/

void Lay_PutSendButton (const char *TextSendButton)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<input type=\"submit\" value=\"%s\" />" \
                      "</div>\n",
            TextSendButton);
  }

/****************************************************************************/
/******************* Put buttons to submit and undo a form ******************/
/****************************************************************************/

void Lay_PutSendAndUndoButtons (const char *TextSendButton)
  {
   extern const char *Txt_Undo[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<input type=\"submit\" value=\"%s\"/>" \
                      "<input type=\"reset\" value=\"%s\" />" \
                      "</div>\n",
            TextSendButton,Txt_Undo[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/**** Draw a cell of a table with a icon to create or register something *****/
/*****************************************************************************/

void Lay_PutButtonCreate (void)
  {
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/new12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/********** Write a message centrado with the ttulo of a table *********/
/****************************************************************************/

void Lay_WriteTitle (const char *Title)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"TITULO\">%s</div>",Title);
  }

/****************************************************************************/
/*********************** Start and end a table with frame *******************/
/****************************************************************************/

void Lay_StartTableFrame (const char *BorderColor,const char *BgColor,const char *Width)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"1\" cellpadding=\"0\"");
   if (BorderColor != NULL)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BorderColor);
   if (Width != NULL)
      fprintf (Gbl.F.Out," width=\"%s\"",Width);
   fprintf (Gbl.F.Out,"><tr><td align=\"center\" valign=\"top\"");
   if (BgColor != NULL)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
   fprintf (Gbl.F.Out,"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">");
  }

void Lay_EndTableFrame (void)
  {
   fprintf (Gbl.F.Out,"</table></td></tr></table></div>");
  }

/****************************************************************************/
/************ Start and end a table with frame redondeado ***********/
/****************************************************************************/

void Lay_StartRoundFrameTable10 (const char *Corner,const char *Frame,const char *BgColor,const char *Width)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"");
   if (Width != NULL)
      fprintf (Gbl.F.Out," width=\"%s\"",Width);
   if (BgColor != NULL)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
   fprintf (Gbl.F.Out,"><tr><td align=\"left\" width=\"10\">" \
                      "<img src=\"%s/%s_1_10x10.gif\" width=\"10\" height=\"10\" alt=\"\" /></td>" \
                      "<td align=\"left\" style=\"background-image: url('%s/%s_top_1x10.gif'); background-repeat: repeat-x;\"></td>" \
                      "<td align=\"left\" width=\"10\">" \
                      "<img src=\"%s/%s_2_10x10.gif\" width=\"10\" height=\"10\" alt=\"\" /></td></tr>" \
                      "<tr><td align=\"left\" width=\"10\" style=\"background-image: url('%s/%s_left_10x1.gif'); background-repeat: repeat-y;\"></td>" \
                      "<td align=\"center\" valign=\"top\"",
            Gbl.Prefs.PathTheme,Corner,Gbl.Prefs.PathTheme,Frame,Gbl.Prefs.PathTheme,Corner,Gbl.Prefs.PathTheme,Frame);
   if (BgColor != NULL)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
   fprintf (Gbl.F.Out,"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">");
  }

void Lay_EndRoundFrameTable10 (const char *Corner,const char *Frame)
  {
   fprintf (Gbl.F.Out,"</table></td><td align=\"left\" width=\"10\" style=\"background-image: url('%s/%s_right_10x1.gif'); background-repeat: repeat-y;\"></td></tr>" \
                      "<tr><td align=\"left\" width=\"10\">" \
                      "<img src=\"%s/%s_3_10x10.gif\" width=\"10\" height=\"10\" alt=\"\" /></td>" \
                      "<td align=\"left\" style=\"background-image: url('%s/%s_bottom_1x10.gif'); background-repeat: repeat-x;\"></td>" \
                      "<td align=\"left\" width=\"10\">" \
                      "<img src=\"%s/%s_4_10x10.gif\" width=\"10\" height=\"10\" alt=\"\" /></td></tr>" \
                      "</table></div>",
            Gbl.Prefs.PathTheme,Frame,Gbl.Prefs.PathTheme,Corner,Gbl.Prefs.PathTheme,Frame,Gbl.Prefs.PathTheme,Corner);
  }

void Lay_StartFrameRound4 (const char *Corner,const char *Frame,const char *BgColor,const char *Width,const char *Header)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"");
   if (Width != NULL)
      fprintf (Gbl.F.Out," width=\"%s\"",Width);
   if (BgColor != NULL)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
   fprintf (Gbl.F.Out,"><tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/%s1_4x32.gif');\">" \
                      "<img src=\"%s/tr4x14.gif\" width=\"4\" height=\"14\" alt=\"\" /></td>" \
                      "%s" \
                      "<td align=\"left\" width=\"4\" style=\"background-image: url('%s/%s2_4x32.gif');\">" \
                      "<img src=\"%s/tr4x14.gif\" width=\"4\" height=\"14\" alt=\"\" /></td></tr>" \
                      "<tr><td align=\"left\" width=\"4\" style=\"background-image: url('%s/%s2_4x1.gif'); background-repeat: repeat-y;\"></td>" \
                      "<td align=\"center\" valign=\"top\"",
            Gbl.Prefs.IconsURL,Corner,Gbl.Prefs.IconsURL,Header,Gbl.Prefs.IconsURL,Corner,Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL,Frame);
   if (BgColor != NULL)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
   fprintf (Gbl.F.Out,"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">");
  }

void Lay_EndFrameRound4 (const char *Corner,const char *Frame)
  {
   fprintf (Gbl.F.Out,"</table></td><td align=\"left\" width=\"4\" style=\"background-image: url('%s/%s3_4x1.gif'); background-repeat: repeat-y;\"></td></tr>" \
                      "<tr><td align=\"left\" width=\"4\"><img src=\"%s/%s3_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td>" \
                      "<td align=\"left\" style=\"background-image: url('%s/%s4_1x4.gif'); background-repeat: repeat-x;\"></td>" \
                      "<td align=\"left\" width=\"4\"><img src=\"%s/%s4_4x4.gif\" width=\"4\" height=\"4\" alt=\"\" /></td></tr>" \
                      "</table></div>",
            Gbl.Prefs.IconsURL,Frame,Gbl.Prefs.IconsURL,Corner,Gbl.Prefs.IconsURL,Frame,Gbl.Prefs.IconsURL,Corner);
  }

/****************************************************************************/
/******* Write error message, close files, remove lock file, and exit *******/
/****************************************************************************/

void Lay_ShowErrorAndExit (const char *Message)
  {
   /***** Write error message and finish the page, except </body> and </html> *****/
   Lay_ShowAlert (ERROR,Message);
   Lay_WriteEndOfPage ();

   /***** Free memory and close all the open files *****/
   Gbl_Cleanup ();

   /***** Page is generated (except </body> and </html>). Compute time to generate page *****/
   if (Gbl.CurrentAct != ActRef &&	// Don't log anything when refreshing connected users
       Gbl.CurrentAct != ActWebSvc)	// Don't log anything when serving a plugin request
      Sta_ComputeTimeToGeneratePage ();

   /***** Send page. The HTML output is now in Gbl.F.Out file ==> copy it to standard output *****/
   if (Gbl.CurrentAct != ActWebSvc)
     {
      rewind (Gbl.F.Out);
      Fil_FastCopyOfOpenFiles (Gbl.F.Out,stdout);
      Fil_CloseAndRemoveFileForHTMLOutput ();
     }

   /***** Compute time to send page *****/
   if (Gbl.CurrentAct != ActRef &&	// Don't log anything when refreshing connected users
       Gbl.CurrentAct != ActWebSvc)	// Don't log anything when serving a plugin request
      Sta_ComputeTimeToSendPage ();

   /***** Write page footer *****/
   if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW &&
       Gbl.CurrentAct != ActRef &&
       Gbl.CurrentAct != ActWebSvc)
      Lay_WritePageFooter ();

   /***** Log access *****/
   if (Gbl.CurrentAct != ActRef &&	// Don't log anything when refreshing connected users
       Gbl.CurrentAct != ActWebSvc)	// Don't log anything when serving a plugin request
      Sta_LogAccess (Message);

   /***** Close database connection *****/
   DB_CloseDBConnection ();

   /***** End the output *****/
   if (!Gbl.HTMLEndWritten)
     {
      fprintf (Gbl.F.Out," </body>\n</html>\n");
      Gbl.HTMLEndWritten = true;
     }
   exit (0);
  }

/****************************************************************************/
/******************** Show an alert message to the user *********************/
/****************************************************************************/

void Lay_ShowAlert (tMsgType MsgType,const char *Message)
  {
   static const char *MsgIcons[3] =
     {
      "info",
      "warning",
      "error",
     };
   char *Corner;
   char *Frame;
   char *Background;

   /****** If start of page is not written yet, do it now ******/
   if (!Gbl.HTMLStartWritten)
      Lay_WriteStartOfPage ();

   if (Message != NULL)
      if (Message[0])
	{
         if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW)
           {
            Corner = "frame_help_corner";
            Frame  = "frame_help";
            Background = YELLOW_BACKGROUND_MSG;
           }
         else
           {
            Corner = "frame_white_corner";
            Frame  = "frame_white";
            Background = WHITE;
           }

         /****** Write the message ******/
         Lay_StartRoundFrameTable10 (Corner,Frame,Background,NULL);
	 fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" width=\"20\"><img src=\"%s/%s16x16.gif\" width=\"16\" height=\"16\" /></td>" \
                            "<td align=\"left\" valign=\"middle\"><p align=\"justify\" class=\"MSJ_AVISO\">%s</p></td></tr>",
		  Gbl.Prefs.IconsURL,MsgIcons[MsgType],Message);
         Lay_EndRoundFrameTable10 (Corner,Frame);
	}
  }

/****************************************************************************/
/******************** Refresh connected users via AJAX **********************/
/****************************************************************************/

void Lay_RefreshMsgsAndConnected (void)
  {
   // Send, before the HTML, the refresh time
   fprintf (Gbl.F.Out,"%lu|",Gbl.Usrs.Connected.TimeToRefreshInMs);
   if (Gbl.Usrs.Me.Logged)
      Msg_WriteNumberOfNewMsgs ();
   fprintf (Gbl.F.Out,"|");
   Con_ShowGlobalConnectedUsrs ();
   fprintf (Gbl.F.Out,"|");
   if (Gbl.Prefs.SideCols & 1)	// Right column visible
      if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
        {
         Con_ShowConnectedUsrsBelongingToCurrentCrs (Usr_TEACHER);
	 fprintf (Gbl.F.Out,"<img src=\"%s/tr1x4.gif\" width=\"1\" height=\"4\" alt=\"\" />",Gbl.Prefs.IconsURL);	// Image used as separation
         Con_ShowConnectedUsrsBelongingToCurrentCrs (Usr_STUDENT);
        }

   /***** All the output is made, so don't write anymore *****/
   Gbl.TablEndWritten = Gbl.HTMLEndWritten = true;
  }

/*****************************************************************************/
/*************************** Write footer of page ****************************/
/*****************************************************************************/

void Lay_WritePageFooter (void)
  {
   extern const char *Txt_About[Txt_NUM_LANGUAGES];
   extern const char *Txt_We_need_your_help[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"FOOT\"><br />");

   /***** Write time to generate and send page *****/
   Sta_WriteTimeToGenerateAndSendPage ();

   /***** About this platform *****/
   fprintf (Gbl.F.Out,"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" \
                      "<a href=\"http://%s/info/\" class=\"FOOT\" target=\"_blank\">%s %s</a>" \
                      "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" \
                      "<a href=\"http://%s/wiki/index.php/Colaborar\" class=\"FOOT\" target=\"_blank\">%s</a>" \
                      "<br />" \
                      "<a href=\"%s\" class=\"FOOT\" target=\"_blank\">" \
                      "<img src=\"/%s/%s\" width=\"%u\" height=\"%u\" alt=\"%s\" /></a> " \
                      "<a href=\"%s\" class=\"FOOT\" target=\"_blank\">" \
                      "<img src=\"/%s/%s\" width=\"%u\" height=\"%u\" alt=\"%s\" /></a>" \
                      "</div>",
            Cfg_PLATFORM_URL,Txt_About[Gbl.Prefs.Language],Cfg_PLATFORM_SHORT_NAME,
            Cfg_PLATFORM_URL,Txt_We_need_your_help[Gbl.Prefs.Language],
            Cfg_ABOUT_INSTITUTION_URL,
            Cfg_FOLDER_PUBLIC_ICON,Cfg_ABOUT_INSTITUTION_LOGO,Cfg_ABOUT_LOGO_WIDTH,Cfg_ABOUT_LOGO_HEIGHT,
            Cfg_ABOUT_INSTITUTION_NAME,
            Cfg_ABOUT_CENTRE_URL,
            Cfg_FOLDER_PUBLIC_ICON,Cfg_ABOUT_CENTRE_LOGO,Cfg_ABOUT_LOGO_WIDTH,Cfg_ABOUT_LOGO_HEIGHT,
            Cfg_ABOUT_CENTRE_NAME);
  }

/*****************************************************************************/
/****** Write header and footer of the class photo or academic calendar ******/
/*****************************************************************************/

void Lay_WriteHeaderClassPhoto (unsigned NumColumns,bool PrintView,bool DrawingClassPhoto,long InsCod,long DegCod,long CrsCod)
  {
   extern const char *Txt_Academic_year[Txt_NUM_LANGUAGES];
   struct Institution Ins;
   struct Degree Deg;
   struct Course Crs;
   unsigned Size = (PrintView ? 64 : 32);

   /***** Get data of institution of this degree *****/
   Ins.InsCod = InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);

   /***** Get data of this degree *****/
   Deg.DegCod = DegCod;
   Deg_GetDataOfDegreeByCod (&Deg);

   /***** Get data of course *****/
   Crs.CrsCod = CrsCod;
   Deg_GetDataOfCourseByCod (&Crs);

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"><table cellspacing=\"0\" cellpadding=\"5\" width=\"100%%\">" \
                      "<tr><td align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">");

   /***** First row: title *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"%u\">\n<table width=\"100%%\" cellspacing=\"0\" cellpadding=\"0\"><tr>\n",
            NumColumns);

   /* First column: institution logo */
   fprintf (Gbl.F.Out,"<td width=\"64\" align=\"left\" valign=\"top\">");
   if (InsCod > 0)
     {
      if (!PrintView)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\">",Ins.WWW);
      fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s%ux%u.gif\" width=\"32\" height=\"32\" alt=\"%s\" />",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Ins.Logo,Size,Size,Ins.ShortName);
      if (!PrintView)
        fprintf (Gbl.F.Out,"</a>");
     }
   fprintf (Gbl.F.Out,"</td>\n");

   /* Second column: class photo title */
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"TIT_ORL\">");
   if (InsCod > 0)
     {
      if (!PrintView)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"TIT_ORL\">",
                  Ins.WWW);
      fprintf (Gbl.F.Out,"%s",Ins.FullName);
      if (!PrintView)
         fprintf (Gbl.F.Out,"</a>");
     }
   if (DegCod > 0)
     {
      if (Ins.InsCod > 0)
         fprintf (Gbl.F.Out," - ");
      if (!PrintView)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"TIT_ORL\">",
                  Deg.WWW);
      fprintf (Gbl.F.Out,"%s",Deg.FullName);
      if (!PrintView)
         fprintf (Gbl.F.Out,"</a>");
     }
   fprintf (Gbl.F.Out,"<br />");
   if (CrsCod > 0)
      fprintf (Gbl.F.Out,"%s - ",Crs.FullName);
   fprintf (Gbl.F.Out,"%s %u-%u",
            Txt_Academic_year[Gbl.Prefs.Language],
            Gbl.AcademicYear,Gbl.AcademicYear+1);
   if (CrsCod > 0 && DrawingClassPhoto && !Gbl.Usrs.ClassPhoto.AllGroups)
     {
      fprintf (Gbl.F.Out,"<br />");
      Grp_WriteNamesOfSelectedGrps ();
     }
   fprintf (Gbl.F.Out,"</td>\n");

   /* Third column: degree logo */
   fprintf (Gbl.F.Out,"<td width=\"64\" align=\"right\" valign=\"top\">");
   if (DegCod > 0)
     {
      if (!PrintView)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"TIT_ORL\">",
                  Deg.WWW);
      fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s%ux%u.gif\" width=\"32\" height=\"32\" alt=\"%s\" />",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Deg.Logo,Size,Size,Deg.ShortName);
      if (!PrintView)
         fprintf (Gbl.F.Out,"</a>");
     }
   fprintf (Gbl.F.Out,"</td></tr></table></td></tr>\n");

   /***** Second row: separation space *****/
   if (DrawingClassPhoto)
     fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"TIT_ORL\">&nbsp;</td></tr>\n");
  }

void Lay_WriteFootClassPhoto (void)
  {
   fprintf (Gbl.F.Out,"</table></td></tr></table></td></tr>\n");
  }
