// swad_lnk.c: institutional links

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_cns.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_lnk.h"
#include "swad_par.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Lnk_PutFormToEditLinks (void);
static void Lnk_ListLinksForEdition (void);
static void Lnk_PutParamLnkCod (long LnkCod);
static void Lnk_RenameLink (Cte_ShortOrFullName_t ShortOrFullName);
static bool Lnk_CheckIfLinkNameExists (const char *FieldName,const char *Name,long LnkCod);
static void Lnk_PutFormToCreateLink (void);
static void Lnk_PutHeadLinks (void);
static void Lnk_CreateLink (struct Link *Lnk);

/****************************************************************************/
/*************************** List all the links ****************************/
/****************************************************************************/

void Lnk_SeeLinks (void)
  {
   extern const char *Txt_Institutional_links[Txt_NUM_LANGUAGES];
   unsigned NumLnk;

   /***** Get list of links *****/
   Lnk_GetListLinks ();

   /***** Put link (form) to edit links *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
       Lnk_PutFormToEditLinks ();

   /***** Table head *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"TIT_TBL\">%s</td></tr>",
            Txt_Institutional_links[Gbl.Prefs.Language]);

   /***** Write all the links *****/
   for (NumLnk=0; NumLnk<Gbl.Links.Num; NumLnk++)
      /* Write data of this link */
      fprintf (Gbl.F.Out,"<tr>" \
                         "<td align=\"left\" class=\"DAT\"><a href=\"%s\" target=\"_blank\" class=\"DAT\">" \
                         "<acronym title=\"%s\" class=\"DAT\">%s</acronym></a></td>" \
                         "</tr>",
               Gbl.Links.Lst[NumLnk].WWW,
               Gbl.Links.Lst[NumLnk].FullName,
               Gbl.Links.Lst[NumLnk].ShortName);

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of links *****/
   Lnk_FreeListLinks ();
  }

/****************************************************************************/
/********************** Put a link (form) to edit links ********************/
/****************************************************************************/

static void Lnk_PutFormToEditLinks (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiLnk);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************** Put forms to edit links *************************/
/****************************************************************************/

void Lnk_EditLinks (void)
  {
   extern const char *Txt_Institutional_links[Txt_NUM_LANGUAGES];

   /***** Get list of links *****/
   Lnk_GetListLinks ();

   /***** Help message *****/
   if (Gbl.Links.Num) // Places found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo enlace rellene los datos del enlace" \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar un enlace existente pulse el aspa roja.<br />" \
                          "Para modificar un enlace existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Institutional_links[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No links found
      Lay_ShowAlert (INFO,"No existen enlaces.<br />" \
                          "Para a&ntilde;adir un enlace rellene los datos del enlace" \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit links *****/
   /* Start table with links */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current links */
   Lnk_ListLinksForEdition ();

   /* Put a form to create a new link */
   Lnk_PutFormToCreateLink ();

   /* End of table with links */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of links *****/
   Lnk_FreeListLinks ();
  }

/****************************************************************************/
/**************************** List all the links ***************************/
/****************************************************************************/

void Lnk_GetListLinks (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumLnk;
   struct Link *Lnk;

   /***** Get institutional links from database *****/
   sprintf (Query,"SELECT LnkCod,ShortName,FullName,WWW FROM links ORDER BY ShortName");
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get institutional links");

   if (NumRows) // Places found...
     {
      Gbl.Links.Num = (unsigned) NumRows;

      /***** Create list with places *****/
      if ((Gbl.Links.Lst = (struct Link *) calloc (NumRows,sizeof (struct Link))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store institutional links.");

      /***** Get the links *****/
      for (NumLnk=0; NumLnk<Gbl.Links.Num; NumLnk++)
        {
         Lnk = &(Gbl.Links.Lst[NumLnk]);

         /* Get next link */
         row = mysql_fetch_row (mysql_res);

         /* Get link code (row[0]) */
         if ((Lnk->LnkCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of institutional link.");

         /* Get the short name of the link (row[1]) */
         strcpy (Lnk->ShortName,row[1]);

         /* Get the full name of the link (row[2]) */
         strcpy (Lnk->FullName,row[2]);

         /* Get the URL of the link (row[3]) */
         strcpy (Lnk->WWW,row[3]);
        }
     }
   else
      Gbl.Links.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);   
  }

/*****************************************************************************/
/**************************** Get link full name ****************************/
/*****************************************************************************/

void Lnk_GetDataOfLinkByCod (struct Link *Lnk)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   
   /***** Clear data *****/
   Lnk->ShortName[0] = Lnk->FullName[0] = Lnk->WWW[0] = '\0';

   /***** Check if link code is correct *****/
   if (Lnk->LnkCod > 0)
     {
      /***** Get data of an institutional link from database *****/
      sprintf (Query,"SELECT ShortName,FullName,WWW FROM links WHERE LnkCod='%ld'",
               Lnk->LnkCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of an institutional link");

      if (NumRows) // Link found...
        {
         /* Get row */
         row = mysql_fetch_row (mysql_res);

         /* Get the short name of the link (row[0]) */
         strcpy (Lnk->ShortName,row[0]);

         /* Get the full name of the link (row[1]) */
         strcpy (Lnk->FullName,row[1]);

         /* Get the URL of the link (row[2]) */
         strcpy (Lnk->WWW,row[2]);
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/**************************** Free list of links ****************************/
/*****************************************************************************/

void Lnk_FreeListLinks (void)
  {
   if (Gbl.Links.Lst)
     {
      /***** Free memory used by the list of links *****/
      free ((void *) Gbl.Links.Lst);
      Gbl.Links.Lst = NULL;
      Gbl.Links.Num = 0;
     }
  }

/****************************************************************************/
/*************************** List all the links ****************************/
/****************************************************************************/

static void Lnk_ListLinksForEdition (void)
  {
   extern const char *Txt_Remove_link[Txt_NUM_LANGUAGES];
   unsigned NumLnk;
   struct Link *Lnk;

   /***** Table head *****/
   Lnk_PutHeadLinks ();

   /***** Write all the links *****/
   for (NumLnk=0; NumLnk<Gbl.Links.Num; NumLnk++)
     {
      Lnk = &Gbl.Links.Lst[NumLnk];

      /* Put icon to remove link */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      Act_FormStart (ActRemLnk);
      Lnk_PutParamLnkCod (Lnk->LnkCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_link[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
               Gbl.Prefs.IconsURL,Txt_Remove_link[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</td>");

      /* Link code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Lnk->LnkCod);

      /* Link short name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenLnkSho);
      Lnk_PutParamLnkCod (Lnk->LnkCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_LINK_SHORT_NAME,Lnk->ShortName,Gbl.NumForm);

      /* Link full name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenLnkFul);
      Lnk_PutParamLnkCod (Lnk->LnkCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_LINK_FULL_NAME,Lnk->FullName,Gbl.NumForm);

      /* Link WWW */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgLnkWWW);
      Lnk_PutParamLnkCod (Lnk->LnkCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"WWW\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td></tr>",
               Cte_MAX_LENGTH_WWW,Lnk->WWW,Gbl.NumForm);

     }
  }

/****************************************************************************/
/******************** Write parameter with code of link ********************/
/****************************************************************************/

static void Lnk_PutParamLnkCod (long LnkCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"LnkCod\" value=\"%ld\" />",LnkCod);
  }

/****************************************************************************/
/********************* Get parameter with code of link *********************/
/****************************************************************************/

long Lnk_GetParamLnkCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of link *****/
   Par_GetParam (Par_PARAM_SINGLE,"LnkCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/******************************* Remove a link *****************************/
/****************************************************************************/

void Lnk_RemoveLink (void)
  {
   extern const char *Txt_Link_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Link Lnk;

   /***** Get link code *****/
   if ((Lnk.LnkCod = Lnk_GetParamLnkCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institutional link is missing.");

   /***** Get data of the link from database *****/
   Lnk_GetDataOfLinkByCod (&Lnk);

   /***** Remove link *****/
   sprintf (Query,"DELETE FROM links WHERE LnkCod='%ld'",Lnk.LnkCod);
   DB_QueryDELETE (Query,"can not remove an institutional link");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Link_X_removed[Gbl.Prefs.Language],
            Lnk.ShortName);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Lnk_EditLinks ();
  }

/****************************************************************************/
/********************* Change the short name of a link *********************/
/****************************************************************************/

void Lnk_RenameLinkShort (void)
  {
   Lnk_RenameLink (Cte_SHORT_NAME);
  }

/****************************************************************************/
/********************* Change the full name of a link **********************/
/****************************************************************************/

void Lnk_RenameLinkFull (void)
  {
   Lnk_RenameLink (Cte_FULL_NAME);
  }

/****************************************************************************/
/************************ Change the name of a link *************************/
/****************************************************************************/

static void Lnk_RenameLink (Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_link_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Link *Lnk;
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentLnkName = NULL;		// Initialized to avoid warning
   char NewLnkName[MAX_LENGTH_LINK_FULL_NAME+1];

   Lnk = &Gbl.Links.EditingLnk;
   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = MAX_LENGTH_LINK_SHORT_NAME;
         CurrentLnkName = Lnk->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = MAX_LENGTH_LINK_FULL_NAME;
         CurrentLnkName = Lnk->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the link */
   if ((Lnk->LnkCod = Lnk_GetParamLnkCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institutional link is missing.");

   /* Get the new name for the link */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewLnkName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewLnkName,MaxLength,true);

   /***** Get from the database the old names of the link *****/
   Lnk_GetDataOfLinkByCod (Lnk);

   /***** Check if new name is empty *****/
   if (!NewLnkName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del enlace <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentLnkName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentLnkName,NewLnkName))	// Different names
        {
         /***** If link was in database... *****/
         if (Lnk_CheckIfLinkNameExists (ParamName,NewLnkName,Lnk->LnkCod))
           {
            sprintf (Gbl.Message,Txt_The_link_X_already_exists[Gbl.Prefs.Language],
                     NewLnkName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE links SET %s='%s' WHERE LnkCod='%ld'",
                     FieldName,NewLnkName,Lnk->LnkCod);
            DB_QueryUPDATE (Query,"can not update the name of an institutional link");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El enlace <strong>%s</strong>" \
                                 " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     CurrentLnkName,NewLnkName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del enlace <strong>%s</strong> no ha cambiado.",	// Need translation!
                  CurrentLnkName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (CurrentLnkName,NewLnkName);
   Lnk_EditLinks ();
  }

/****************************************************************************/
/********************** Check if the name of link exists ********************/
/****************************************************************************/

static bool Lnk_CheckIfLinkNameExists (const char *FieldName,const char *Name,long LnkCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of links with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM links WHERE %s='%s' AND LnkCod<>'%ld'",
            FieldName,Name,LnkCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of an institutional link already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of an institutional link already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/**************** Change the WWW of an institutional link *******************/
/****************************************************************************/

void Lnk_ChangeLinkWWW (void)
  {
   struct Link *Lnk;
   char Query[256+Cte_MAX_LENGTH_WWW];
   char NewWWW[Cte_MAX_LENGTH_WWW+1];

   Lnk = &Gbl.Links.EditingLnk;

   /***** Get parameters from form *****/
   /* Get the code of the link */
   if ((Lnk->LnkCod = Lnk_GetParamLnkCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of institutional link is missing.");

   /* Get the new WWW for the link */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",NewWWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewWWW,Cte_MAX_LENGTH_WWW,true);

   /***** Check if new WWW is empty *****/
   if (NewWWW[0])
     {
      /* Update the table changing old WWW by new WWW */
      sprintf (Query,"UPDATE links SET WWW='%s' WHERE LnkCod='%ld'",
               NewWWW,Lnk->LnkCod);
      DB_QueryUPDATE (Query,"can not update the web of an institutional link");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva direcci&oacute;n web es <strong>%s</strong>.",	// Need translation!
               NewWWW);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     Lay_ShowAlert (WARNING,"No puede dejar la direcci&oacute;n web vac&iacute;a.");	// Need translation!

   /***** Show the form again *****/
   strcpy (Lnk->WWW,NewWWW);
   Lnk_EditLinks ();
  }

/****************************************************************************/
/********************* Put a form to create a new link *********************/
/****************************************************************************/

static void Lnk_PutFormToCreateLink (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_link[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_link[Txt_NUM_LANGUAGES];
   struct Link *Lnk;

   Lnk = &Gbl.Links.EditingLnk;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"5\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_link[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewLnk);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Lnk_PutHeadLinks ();

   /***** New link *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_link[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Link code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Link short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_LINK_SHORT_NAME,Lnk->ShortName);

   /***** Link full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_LINK_FULL_NAME,Lnk->FullName);

   /***** Link WWW *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"WWW\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Cte_MAX_LENGTH_WWW,Lnk->WWW);
  }

/****************************************************************************/
/******************** Write header with fields of a link ******************/
/****************************************************************************/

static void Lnk_PutHeadLinks (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_Name[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">WWW</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Short_Name[Gbl.Prefs.Language],
            Txt_Full_Name[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************* Receive form to create a new link ********************/
/****************************************************************************/

void Lnk_RecFormNewLink (void)
  {
   extern const char *Txt_The_link_X_already_exists[Txt_NUM_LANGUAGES];
   struct Link *Lnk;

   Lnk = &Gbl.Links.EditingLnk;

   /***** Get parameters from form *****/
   /* Get link short name */
   Par_GetParam (Par_PARAM_SINGLE,"ShortName",Lnk->ShortName,MAX_LENGTH_LINK_SHORT_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Lnk->ShortName,MAX_LENGTH_LINK_SHORT_NAME,true);

   /* Get link full name */
   Par_GetParam (Par_PARAM_SINGLE,"FullName",Lnk->FullName,MAX_LENGTH_LINK_FULL_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Lnk->FullName,MAX_LENGTH_LINK_FULL_NAME,true);

   /* Get link URL */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",Lnk->WWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Lnk->WWW,Cte_MAX_LENGTH_WWW,true);

   if (Lnk->ShortName[0] && Lnk->FullName[0])	// If there's a link name
     {
      /***** If name of link was in database... *****/
      if (Lnk_CheckIfLinkNameExists ("ShortName",Lnk->ShortName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_link_X_already_exists[Gbl.Prefs.Language],
                  Lnk->ShortName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else if (Lnk_CheckIfLinkNameExists ("FullName",Lnk->FullName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_link_X_already_exists[Gbl.Prefs.Language],
                  Lnk->FullName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else if (!Lnk->WWW[0])
         Lay_ShowAlert (WARNING,"Debe especificar la URL del nuevo enlace.");	// Need translation!
      else	// Add new link to database
         Lnk_CreateLink (Lnk);
     }
   else	// If there isn't a link name
      Lay_ShowAlert (WARNING,"Debe especificar el nombre breve y el nombre completo del nuevo enlace.");	// Need translation!

   /***** Show the form again *****/
   Lnk_EditLinks ();
  }

/****************************************************************************/
/**************************** Create a new link ***************************/
/****************************************************************************/

static void Lnk_CreateLink (struct Link *Lnk)
  {
   extern const char *Txt_Created_new_link_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new link *****/
   sprintf (Query,"INSERT INTO links (ShortName,FullName,WWW) VALUES ('%s','%s','%s')",
            Lnk->ShortName,Lnk->FullName,Lnk->WWW);
   DB_QueryINSERT (Query,"can not create institutional link");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_link_X[Gbl.Prefs.Language],
            Lnk->ShortName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/***************** Write menu with some institutional links *****************/
/****************************************************************************/

void Lnk_WriteMenuWithInstitutionalLinks (void)
  {
   extern const char *Txt_Institutional_links[Txt_NUM_LANGUAGES];
   unsigned NumLnk;
   char Header[1024];

   /***** Get list of links *****/
   Lnk_GetListLinks ();

   /***** Start and first link *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   sprintf (Header,"<td align=\"center\" valign=\"top\" style=\"background-image: url('%s/mbb1_1x32.gif'); background-repeat: repeat-x;\" class=\"MENUD\">%s</td>",
            Gbl.Prefs.IconsURL,Txt_Institutional_links[Gbl.Prefs.Language]);
   Lay_StartFrameRound4 ("ewbb","mbw",WHITE,"120",Header);

   /***** Write all the links *****/
   for (NumLnk=0; NumLnk<Gbl.Links.Num; NumLnk++)
      /* Write data of this link */
      fprintf (Gbl.F.Out,"<tr>" \
                         "<td align=\"left\" class=\"MENUD\"><a href=\"%s\" target=\"_blank\" class=\"MENUD\">" \
                         "<acronym title=\"%s\" class=\"MENUD\">%s</acronym></a></td>" \
                         "</tr>",
               Gbl.Links.Lst[NumLnk].WWW,
               Gbl.Links.Lst[NumLnk].FullName,
               Gbl.Links.Lst[NumLnk].ShortName);

   /***** End *****/
   Lay_EndFrameRound4 ("ewbw","mbw");
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Free list of links *****/
   Lnk_FreeListLinks ();
  }
