// swad_msg.c: messages between users

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For free
#include <string.h>		// For string functions
#include <time.h>		// For time

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_for.h"
#include "swad_gbl.h"
#include "swad_grp.h"
#include "swad_msg.h"
#include "swad_ntf.h"
#include "swad_par.h"
#include "swad_usr.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern const Act_Action_t ActionsSeeFor[For_NUM_TYPES_FORUM];
extern const Act_Action_t ActionsSeePstFor[For_NUM_TYPES_FORUM];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

#define MAX_LENGTH_MESSAGES_QUERY 4096
#define MAX_LENGTH_STR_ADDR (32*5000)

/****************************************************************************/
/******************************** Private types *****************************/
/****************************************************************************/

/*****************************************************************************/
/**************************** Internal prototypes ****************************/
/*****************************************************************************/

static void Msg_ShowSentOrReceivedMessages (Msg_TypeOfMessages_t TypeOfMessages);
static void Msg_ConstructQueryToSelectSentOrReceivedMsgs (char *Query,Msg_TypeOfMessages_t TypeOfMessages,long UsrCod,
                                                          long FilterCrsCod,const char *FilterFromToSubquery,const char *FilterContentSubquery);
static void Msg_ShowFormToShowOnlyUnreadMessages (void);
static void Msg_GetParamOnlyUnreadMsgs (void);
static void Msg_ShowASentOrReceivedMessage (Msg_TypeOfMessages_t TypeOfMessages,long MsgNum,long MsgCod,bool NewMsg);
static void Msg_GetStatusOfSentMsg (long MsgCod,bool *Expanded);
static void Msg_GetStatusOfReceivedMsg (long MsgCod,bool *Open,bool *Replied,bool *Expanded);
static void Msg_PutFormToDeleteMessage (unsigned long MsgNum,long MsgCod,Msg_TypeOfMessages_t TypeOfMessages);
static void Msg_PutHiddenParamMsgCod (long MsgCod);
static long Msg_GetParamMsgCod (void);

static void Msg_ExpandSentMsg (long MsgCod);
static void Msg_ExpandReceivedMsg (long MsgCod);

static void Msg_InsertReceivedMsgIntoDB (long MsgCod,long UsrCod);
static void Msg_SetReceivedMsgAsNotified (long MsgCod,long UsrCod);
static void Msg_SetReceivedMsgAsReplied (long MsgCod);
static unsigned Msg_GetNumUnreadMsgs (long FilterCrsCod,const char *FilterFromToSubquery,const char *FilterContentSubquery);

static void Msg_GetMsgData (long MsgCod,long *CrsCod,long *UsrCod,bool *Deleted,char *CreatTime,char *Subject);
static void Msg_GetMsgContent (long MsgCod,char *Content);

static void Msg_WriteSentOrReceivedMsgSubject (Msg_TypeOfMessages_t TypeOfMessages,long MsgCod,const char *Subject,bool Open,bool Expanded);
static void Msg_WriteFormToReply (long MsgCod,long CrsCod,const char *Subject,bool ThisCrs,bool Replied,const char EncryptedUsrCod[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1]);
static void Msg_WriteMsgFrom (struct UsrData *UsrDat,bool Deleted);
static void Msg_WriteMsgTo (Msg_TypeOfMessages_t TypeOfMessages,long MsgCod);

/*****************************************************************************/
/******************* List the e-mails of all the students ********************/
/*****************************************************************************/

void Msg_ListEMails (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_X_students_who_have_e_mail[Txt_NUM_LANGUAGES];
   extern const char *Txt_X_students_who_have_accepted_and_who_have_e_mail[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_e_mail_message[Txt_NUM_LANGUAGES];
   unsigned NumUsr,NumStdsWithEmail = 0,NumAcceptedStdsWithEmail = 0;
   char StrAddresses[MAX_LENGTH_STR_ADDR+1];
   unsigned int LengthStrAddr = 0;
   struct UsrData *UsrDat;

   /***** Form to select groups *****/
   Grp_ShowFormToSelectSeveralGroups (ActMaiStd);

   /***** Get and order list of students in this course *****/
   Usr_GetAndOrderStdsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);

   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      /***** Count the number of students with e-mail *****/
      for (NumUsr=0, NumStdsWithEmail=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
         if (Gbl.Usrs.LstStds.Lst[NumUsr].E_mail[0]) // If e-mail address exists (not empty)
           {
	    NumStdsWithEmail++;
            if (Gbl.Usrs.LstStds.Lst[NumUsr].Accepted) // If student accepted joining to this course
	       NumAcceptedStdsWithEmail++;
           }

      /***** Start of the frame used to list the e-mails *****/
      sprintf (Gbl.Message,Txt_X_students_who_have_e_mail[Gbl.Prefs.Language],
               NumStdsWithEmail,((float) NumStdsWithEmail / (float) Gbl.Usrs.LstStds.NumUsrs) * 100.0,Gbl.Usrs.LstStds.NumUsrs);
      Lay_WriteTitle (Gbl.Message);
      sprintf (Gbl.Message,Txt_X_students_who_have_accepted_and_who_have_e_mail[Gbl.Prefs.Language],
               NumAcceptedStdsWithEmail,((float) NumAcceptedStdsWithEmail / (float) Gbl.Usrs.LstStds.NumUsrs) * 100.0,Gbl.Usrs.LstStds.NumUsrs);
      Lay_WriteTitle (Gbl.Message);
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT_SMALL\">\n");

      /***** List the students' e-mail addresses *****/
      for (NumUsr=0, NumAcceptedStdsWithEmail=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
        {
         UsrDat = &Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.Ind[NumUsr]];
         if (UsrDat->Accepted && UsrDat->E_mail[0]) // If student has accepted and has e-mail
	   {
	    if (NumAcceptedStdsWithEmail > 0)
	      {
	       fprintf (Gbl.F.Out,", ");
	       LengthStrAddr += 2;
	       if (LengthStrAddr <= MAX_LENGTH_STR_ADDR)
	          strcat (StrAddresses,",");
	       else
	          Lay_ShowErrorAndExit ("The space allocated to store e-mail addresses is full.");
	      }
            LengthStrAddr += strlen (UsrDat->E_mail);
	    if (LengthStrAddr <= MAX_LENGTH_STR_ADDR)
	       strcat (StrAddresses,UsrDat->E_mail);
	    else
	       Lay_ShowErrorAndExit ("The space allocated to store e-mail addresses is full.");

	    fprintf (Gbl.F.Out,"<a href=\"mailto:%s?subject=%s\">%s</a>",
                     UsrDat->E_mail,Gbl.CurrentCrs.Crs.FullName,UsrDat->E_mail);

	    NumAcceptedStdsWithEmail++;
	   }
        }

      /***** End of the frame used to list the e-mails *****/
      fprintf (Gbl.F.Out,"</td></tr>\n");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Icon to open the client e-mail program *****/
      fprintf (Gbl.F.Out,"<div align=\"center\">" \
                         "<a href=\"mailto:?subject=%s&cc=%s&bcc=%s\" class=\"%s\">" \
	                 " <img src=\"%s/email32x32.gif\" width=\"32\" height=\"32\" alt=\"%s\" />" \
                         "<br clear=\"all\" />%s</a></div>",
	      Gbl.CurrentCrs.Crs.FullName,Gbl.Usrs.Me.UsrDat.E_mail,StrAddresses,
              The_ClassFormul[Gbl.Prefs.Theme],Gbl.Prefs.IconsURL,
              Txt_Create_e_mail_message[Gbl.Prefs.Language],
              Txt_Create_e_mail_message[Gbl.Prefs.Language]);
     }
   else
      Usr_ShowWarningNoUsersFound (Usr_STUDENT);

   /***** Free memory for students list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();
  }

/*****************************************************************************/
/***************** Put a form to write a new message to users ****************/
/*****************************************************************************/

void Msg_FormMsgUsrs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_MSG_To[Txt_NUM_LANGUAGES];
   extern const char *Txt_Send_message[Txt_NUM_LANGUAGES];
   char YN[1+1];
   bool IsReply;		// Is the message I am editing a reply to a received message?
   long RepliedMsgCod = -1;	// Initialized to avoid warning

   Gbl.Usrs.LstTchs.NumUsrs = Gbl.Usrs.LstStds.NumUsrs = 0;
   if (Gbl.Usrs.Me.IHaveAccessToCurrentCrs)	// If course selected and I belong to it
     {
      /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
      Usr_GetAndUpdatePrefsAboutUsrList ();

      /***** Form to select groups *****/
      Grp_ShowFormToSelectSeveralGroups (ActReqMsgUsr);

      /***** Form to select type of list used for select several users *****/
      Usr_ShowFormsToSelectUsrListType (ActReqMsgUsr);

      /***** Get and order lists of users from this course *****/
      Usr_GetAndOrderTchsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);
      Usr_GetAndOrderStdsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);

      if (Gbl.Usrs.LstTchs.NumUsrs || Gbl.Usrs.LstStds.NumUsrs)
         /***** Get list of selected users *****/
         Usr_GetListSelectedUsrs ();
      else
         Usr_ShowWarningNoUsersFound (Usr_UNKNOWN);
     }

   /***** Get list of DNI or nicknames written explicitely *****/
   Usr_GetListDNIOrNicks (false);

   /***** Get other parameters *****/
   /* Get parameter that indicates if the message is a reply to another message */
   Par_GetParam (Par_PARAM_SINGLE,"IsReply",YN,1);
   if ((IsReply = (Str_ConvertToUpperLetter (YN[0]) == 'Y')))
     {
      /* Get original message code */
      RepliedMsgCod = Msg_GetParamMsgCod ();
      /* Get the user's code of the original user who sent the first message */
      Usr_GetParamOtherUsrCodEncrypted ();
     }

   /***** Draw two class photos with the recipients: one with the teachers of the course and another one with the students *****/
   Act_FormStart (ActRecMsgUsr);
   if (IsReply)
     {
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"IsReply\" value=\"Y\" />");
      Msg_PutHiddenParamMsgCod (RepliedMsgCod);
      Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
     }

   /* Start table */
   fprintf (Gbl.F.Out,"<div align=\"center\"><table><tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td><td align=\"left\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_MSG_To[Gbl.Prefs.Language]);
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /* Put list of users to select destinatary users */
   Usr_ListUsersToSelect (Usr_TEACHER);
   Usr_ListUsersToSelect (Usr_STUDENT);

   /* Other users (nicknames) */
   Msg_WriteFormDNIOrNicksOtherRecipients (IsReply);
   Usr_FreeListNicks ();

   /* Free memory used for por the listas */
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Free memory used by list of users *****/
   Usr_FreeListsEncryptedUsrCods ();

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();

   /* End of table */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Subject and content *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"2\">");
   Msg_WriteLinkToNetiquette ();
   fprintf (Gbl.F.Out,"</td></tr>");

   Msg_WriteFormSubjectAndContentMsgToUsrs ();

   fprintf (Gbl.F.Out,"</table></div>");

   /***** Send and undo buttons *****/
   Lay_PutSendAndUndoButtons (Txt_Send_message[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/*****************************************************************************/
/************** Nicknames of recipients of a message to users ****************/
/*****************************************************************************/

void Msg_WriteFormDNIOrNicksOtherRecipients (bool IsReply)
  {
   extern const char *Txt_Other_recipients[Txt_NUM_LANGUAGES];
   extern const char *Txt_Recipients[Txt_NUM_LANGUAGES];
   extern const char *Txt_IDs_or_nicknames_separated_by_commas[Txt_NUM_LANGUAGES];
   char Nickname[Usr_MAX_LENGTH_NICKNAME+1];
   unsigned Colspan = Usr_GetColumnsForSelectUsrs ();

   /***** Textarea with DNIs or nicknames *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" bgcolor=\"#%s\"",
            VERY_LIGHT_BLUE);
   if (Gbl.Usrs.Me.IHaveAccessToCurrentCrs)
      fprintf (Gbl.F.Out," colspan=\"%u\" class=\"TIT_TBL\">%s:",Colspan,Txt_Other_recipients[Gbl.Prefs.Language]);
   else
      fprintf (Gbl.F.Out," class=\"TIT_TBL\">%s:",Txt_Recipients[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<span class=\"DAT\"> (%s)</span></td></tr>" \
                      "<tr><td align=\"left\"",
            Txt_IDs_or_nicknames_separated_by_commas[Gbl.Prefs.Language]);
   if (Gbl.Usrs.Me.IHaveAccessToCurrentCrs)
      fprintf (Gbl.F.Out," colspan=\"%u\"",Colspan);
   fprintf (Gbl.F.Out,"><textarea name=\"Nicks\" cols=\"72\" rows=\"2\">");
   if (IsReply && !Gbl.Usrs.ListDNIOrNicks[0])	// If this is a reply message and there's no list of explicit DNI or nicks, write nickname of original sender
      if (Usr_GetNicknameFromUsrCod (Gbl.Usrs.Other.UsrDat.UsrCod,Nickname))
         fprintf (Gbl.F.Out,"%s",Nickname);
   fprintf (Gbl.F.Out,"%s</textarea></td></tr>\n",Gbl.Usrs.ListDNIOrNicks);
  }

/*****************************************************************************/
/****** Write form fields with subject and content of a message to users *****/
/*****************************************************************************/

void Msg_WriteFormSubjectAndContentMsgToUsrs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_MSG_Subject[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_Message[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   char LongStr[1+10+1];
   long MsgCod = -1;
   char OriginalMsg[Cte_MAX_BYTES_TEXT+1];

   /***** Subject of new message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\"><textarea name=\"Subject\" cols=\"75\" rows=\"2\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_MSG_Subject[Gbl.Prefs.Language]);

   /***** If message is a reply ==> get original message *****/
   /* Get code of original message */
   Par_GetParam (Par_PARAM_SINGLE,"MsgCod",LongStr,1+10);
   if (sscanf (LongStr,"%ld",&MsgCod) == 1)	// It's a reply
     {
      /***** Get subject and content of message from database *****/
      sprintf (Query,"SELECT Subject,Content FROM messages WHERE MsgCod='%ld'",MsgCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get message");

      /***** Result should have a unique row *****/
      if (NumRows != 1)
         Lay_ShowErrorAndExit ("Error when getting message from database.");

      /***** Subject of new message *****/
      row = mysql_fetch_row (mysql_res);
      fprintf (Gbl.F.Out,"Re: %s</textarea></td></tr>",row[0]);

      /***** Content of new message *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>" \
                         "<td align=\"left\"><textarea name=\"Txt\" cols=\"75\" rows=\"20\">",
               The_ClassFormul[Gbl.Prefs.Theme],Txt_MSG_Message[Gbl.Prefs.Language]);

      fprintf (Gbl.F.Out,"\n\n\n\n\n----- Mensaje original -----\n");
      strncpy (OriginalMsg,row[1],Cte_MAX_BYTES_TEXT);
      OriginalMsg[Cte_MAX_BYTES_TEXT] = '\0';
      Msg_WriteMsgContent (OriginalMsg,Cte_MAX_BYTES_TEXT,false,true);

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
   else	// It's not a reply
     {
      /***** Subject of new message *****/
      fprintf (Gbl.F.Out,Gbl.Msg.Subject);
      fprintf (Gbl.F.Out,"</textarea></td></tr>");

      /***** Content of new message *****/
      fprintf (Gbl.F.Out,"<tr>\n<td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>\n" \
                         "<td align=\"left\"><textarea name=\"Txt\" cols=\"75\" rows=\"20\">",
               The_ClassFormul[Gbl.Prefs.Theme],
               Txt_MSG_Message[Gbl.Prefs.Language]);
      if (Gbl.Txt)
         fprintf (Gbl.F.Out,Gbl.Txt);
     }
   fprintf (Gbl.F.Out,"</textarea></td></tr>\n");
  }

/*****************************************************************************/
/********************** Receive a new message from a user ********************/
/*****************************************************************************/

void Msg_RecMsgFromUsr (void)
  {
   extern const char *Txt_You_must_select_one_ore_more_recipients[Txt_NUM_LANGUAGES];
   char YN[1+1];
   bool IsReply;
   bool Replied = false;
   long RepliedMsgCod = -1;	// Initialized to avoid warning
   const char *Ptr;
   int NumRecipients;
   struct UsrData UsrDstData;
   int NumErrors = 0;
   char *ListUsrsDst;
   long MsgCod = -1;		// Initiliazed to avoid warning
   bool MsgInserted = false;
   bool Error = false;
   unsigned NumUsrsToBeNotified = 0;

   /***** Get data from form *****/
   /* Get the subject of the message */
   Par_GetParam (Par_PARAM_SINGLE,"Subject",Gbl.Msg.Subject,Cte_MAX_BYTES_SUBJECT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.Msg.Subject,Cte_MAX_BYTES_SUBJECT,true);

   /* Get the body of the message */
   Str_AllocateAndGetText (Str_DONT_CHANGE,false);

   /* Get parameter that indicates if the message is a reply to another message */
   Par_GetParam (Par_PARAM_SINGLE,"IsReply",YN,1);
   if ((IsReply = (Str_ConvertToUpperLetter (YN[0]) == 'Y')))
     {
      /* Get original message code */
      RepliedMsgCod = Msg_GetParamMsgCod ();
      /* Get the user's code of the original user who sent the first message */
      Usr_GetParamOtherUsrCodEncrypted ();
     }

   /* Get list of selected users */
   Usr_GetListSelectedUsrs ();

   /* Get list of DNI or nicknames written explicitely */
   Error = Usr_GetListDNIOrNicks (true);

   /***** If there are no recipients... *****/
   if ((NumRecipients = Usr_CountNumUsrsInEncryptedList ()) == 0)
     {
      /* Write warning message */
      Lay_ShowAlert (WARNING,Txt_You_must_select_one_ore_more_recipients[Gbl.Prefs.Language]);
      Error = true;
     }

   /***** If error in list of recipients, show again the form used to write a message *****/
   if (Error)
     {
      /* Show the form again, with the subject and the message filled */
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Txt,Cte_MAX_BYTES_TEXT,true);
      Msg_FormMsgUsrs ();
      return;
     }

   /***** Allocate space to store a list of recipients with the following format:
	  "FirstName Surname1 Surname2; FirstName Surname1 Surname2; FirstName Surname1 Surname2" *****/
   if ((ListUsrsDst = (char *) malloc (((Usr_MAX_BYTES_NAME+1)*3+1)*NumRecipients)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store e-mail addresses of recipients.");
   ListUsrsDst[0] = '\0';

   /***** Initialize structure with user's data *****/
   Usr_UsrDataConstructor (&UsrDstData);

   /***** Loop over the list Gbl.Usrs.Select.All, that holds the list of the
	  recipients, creating a received message for each recipient *****/
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_RIGOROUS_HTML,Gbl.Txt,Cte_MAX_BYTES_TEXT,false);
   Ptr = Gbl.Usrs.Select.All;
   NumRecipients = 0;

   while (*Ptr)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,UsrDstData.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
      Usr_GetUsrCodFromEncryptedUsrCod (&UsrDstData);
      if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDstData))		// Get recipient's data from the database
        {
         /***** Create message *****/
         if (!MsgInserted)
           {
            // The message is inserted only once in the table of messages
            MsgCod = Msg_InsertNewMsg (Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Msg.Subject,Gbl.Txt);
            MsgInserted = true;
           }

         /***** Create the received message for this recipient *****/
         Msg_InsertReceivedMsgIntoDB (MsgCod,UsrDstData.UsrCod);
         Msg_IncreaseNumNewMsgs (UsrDstData.UsrCod);        // Increment number of new messages received by this recipient

         /***** If this recipient is the original sender of a message been replied, set Replied to true *****/
         if (IsReply)
            if (UsrDstData.UsrCod == Gbl.Usrs.Other.UsrDat.UsrCod)
               Replied = true;

         /***** If this recipient wants to receive notifications by mail, try to send it a notification *****/
         if ((UsrDstData.UsrCod != Gbl.Usrs.Me.UsrDat.UsrCod) &&
             (UsrDstData.Prefs.NotifyEvents & (1 << Ntf_NEW_MESSAGE)))
           {
            Ntf_StoreNotifyEventToOneUser (Ntf_NEW_MESSAGE,&UsrDstData,MsgCod);
            Msg_SetReceivedMsgAsNotified (MsgCod,UsrDstData.UsrCod);
            NumUsrsToBeNotified++;

            /* Show an alert indicating that the message has been sent successfully */
            sprintf (Gbl.Message,"Mensaje enviado correctamente a %s (ser&aacute; notificado por e-mail).",	// Need translation!
                     UsrDstData.FullName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
         else
           {
            /* Show an alert indicating that the message has been sent successfully */
            sprintf (Gbl.Message,"Mensaje enviado correctamente a %s (no ser&aacute; notificado por e-mail).",	// Need translation!
                     UsrDstData.FullName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }

         NumRecipients++;
        }
      else
        {
         Lay_ShowAlert (ERROR,"Error obteniendo datos de un usuario destino.");	// Need translation!
         NumErrors++;
        }
     }

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDstData);

   /***** Free memory *****/
   /* Free memory used for list of recipients' names */
   free (ListUsrsDst);

   /* Free memory used for list of users and the text of the message */
   Usr_FreeListNicks ();
   Usr_FreeListsEncryptedUsrCods ();
   Str_FreeTxt ();

   /***** Update received message setting Replied field to true *****/
   if (Replied)
      Msg_SetReceivedMsgAsReplied (RepliedMsgCod);

   /***** Show message about number of users to be notified *****/
   Ntf_ShowAlertNumUsrsToBeNotified (NumUsrsToBeNotified);

   /***** Write final message *****/
   sprintf (Gbl.Message,"El mensaje se ha enviado a %d destinatario%s." \
                        " Se recomienda que no recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a" \
                        " de nuevo y llegar&iacute;a repetido.",	// Need translation!
            NumRecipients,(NumRecipients > 1) ? "s" : "");
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show alert about errors on sending message *****/
   if (NumErrors)
     {
      sprintf (Gbl.Message,"Ha habido %d error%s enviando el mensaje.",	// Need translation!
               NumErrors,(NumErrors > 1) ? "es" : "");
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
  }

/*****************************************************************************/
/****************** Create temporary file for mail content *******************/
/*****************************************************************************/

 void Msg_CreateFileNameMail (void)
  {
   sprintf (Gbl.Msg.FileNameMail,"%s/%s_mail.txt",Cfg_PATH_REL_XHTML_OUTPUT_PRIVATE,Gbl.UniqueNameEncrypted);
   if ((Gbl.Msg.FileMail = fopen (Gbl.Msg.FileNameMail,"wt")) == NULL)
      Lay_ShowErrorAndExit ("Can not open file to sen e-mail.");
  }

/*****************************************************************************/
/************ Write a welcome note heading the automatic e-mail **************/
/*****************************************************************************/

void Msg_WriteWelcomeNoteEMail (struct UsrData *UsrDat)
  {
   extern const char *Txt_Dear_NO_HTML[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_user_NO_HTML[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];

   fprintf (Gbl.Msg.FileMail,"%s %s:\n",
            Txt_Dear_NO_HTML[UsrDat->Gender][UsrDat->Prefs.Language],
            (UsrDat->FirstName[0] ? UsrDat->FirstName :
                                    Txt_user_NO_HTML[UsrDat->Gender][UsrDat->Prefs.Language]));
  }

/*****************************************************************************/
/****************** Write a foot note in the automatic e-mail ****************/
/*****************************************************************************/

void Msg_WriteFootNoteEMail (Txt_Language_t Language)
  {
   extern const char *Txt_Please_do_not_reply_to_this_automatically_generated_email_NO_HTML[Txt_NUM_LANGUAGES];

   fprintf (Gbl.Msg.FileMail,"%s\n" \
                             "%s\n" \
                             "http://%s/\n",
            Txt_Please_do_not_reply_to_this_automatically_generated_email_NO_HTML[Language],
            Cfg_PLATFORM_FULL_NAME,
            Cfg_PLATFORM_URL);
  }

/*****************************************************************************/
/******************** Write number of new messages received ******************/
/*****************************************************************************/

void Msg_WriteNumberOfNewMsgs (void)
  {
   extern const char *The_ClassHead[The_NUM_THEMES];
   extern const char *Txt_no_new_messages[Txt_NUM_LANGUAGES];
   extern const char *Txt_new_message[Txt_NUM_LANGUAGES];
   extern const char *Txt_new_messages[Txt_NUM_LANGUAGES];

   Act_FormStart (ActSeeNewMsg);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,"Ver mensajes recibidos",The_ClassHead[Gbl.Prefs.Theme]);
   if (Gbl.Usrs.Me.UsrDat.NumNewMsgs)
     {
      /***** Blinking icon *****/
      fprintf (Gbl.F.Out,"<img src=\"%s/mail13x10.gif\" width=\"13\" height=\"10\" align=\"bottom\" /> ",
               Gbl.Prefs.IconsURL);
      /***** Messages *****/
      if (Gbl.Usrs.Me.UsrDat.NumNewMsgs == 1)
         fprintf (Gbl.F.Out,"(1 %s)",Txt_new_message[Gbl.Prefs.Language]);
      else
         fprintf (Gbl.F.Out,"(%u %s)",Gbl.Usrs.Me.UsrDat.NumNewMsgs,Txt_new_messages[Gbl.Prefs.Language]);
     }
   else
      fprintf (Gbl.F.Out,"(%s)",Txt_no_new_messages[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</acronym></a></form>");
  }

/*****************************************************************************/
/******* Increase in the database the number of new received messages ********/
/*****************************************************************************/

void Msg_IncreaseNumNewMsgs (long UsrCod)
  {
   char Query[512];

   /***** Increase entry in the table of number of new messages *****/
   sprintf (Query,"UPDATE usr_data SET NumNewMsgs=NumNewMsgs+1" \
                  " WHERE UsrCod='%ld'",UsrCod);
   DB_QueryUPDATE (Query,"can not increase the number of new messages");
  }

/*****************************************************************************/
/************** Reset my number of new received messages to 0 ****************/
/*****************************************************************************/

void Msg_ClearMyNumNewMsgs (void)
  {
   char Query[512];

   /***** Reset to 0 my number of new received messages *****/
   sprintf (Query,"UPDATE usr_data SET NumNewMsgs='0' WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not reset the number of new messages");
  }

/*****************************************************************************/
/*************** Show messages enviados a otros usuarios ******************/
/*****************************************************************************/

void Msg_ShowSntMsgs (void)
  {
   /***** Show the sent messages *****/
   Msg_ShowSentOrReceivedMessages (Msg_MESSAGES_SENT);
  }

/*****************************************************************************/
/************** Show messages recibidos of otros usuarios *****************/
/*****************************************************************************/

void Msg_ShowRecMsgs (void)
  {
   /***** Show the received messages *****/
   Msg_ShowSentOrReceivedMessages (Msg_MESSAGES_RECEIVED);

   /***** Reset to 0 number of new messages *****/
   Msg_ClearMyNumNewMsgs ();
  }

/*****************************************************************************/
/***************** Request deletion of all received messages *****************/
/*****************************************************************************/

void Msg_ReqDelAllRecMsgs (void)
  {
   char StrFilterContent[256+Msg_MAX_LENGTH_FILTER_CONTENT+1];

   /***** Get parameters *****/
   Msg_GetParamMsgsCrsCod ();
   Msg_GetParamFilterFromTo ();
   Msg_GetParamFilterContent ();
   Msg_GetParamOnlyUnreadMsgs ();

   /***** Request confirmation to delete messages *****/
   if (Gbl.Msg.FilterContent[0])
      sprintf (StrFilterContent," relacionados con <strong>%s</strong>",	// Need translation!
               Gbl.Msg.FilterContent);
   else
      StrFilterContent[0] = '\0';
   sprintf (Gbl.Message,"&iquest;Realmente desea eliminar %s de %s desde %s%s?",	// Need translation!
            Gbl.Msg.ShowOnlyUnreadMsgs ? "los mensajes recibidos no le&iacute;dos" :
                                         "todos los mensajes recibidos",	// Need translation!
            Gbl.Msg.FilterFromTo[0] ? Gbl.Msg.FilterFromTo : "cualquier usuario",	// Need translation!
            Gbl.Msg.FilterCrsShortName,StrFilterContent);
   Lay_ShowAlert (WARNING,Gbl.Message);

   Act_FormStart (ActDelAllRecMsg);
   Msg_PutHiddenParamsMsgsFilters ();
   fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"S&iacute;, eliminar mensajes recibidos\" /></div></form>\n");	// Need translation!
  }

/*****************************************************************************/
/******************* Request deletion of all sent messages *******************/
/*****************************************************************************/

void Msg_ReqDelAllSntMsgs (void)
  {
   char StrFilterContent[256+Msg_MAX_LENGTH_FILTER_CONTENT+1];

   /***** Get parameters *****/
   Msg_GetParamMsgsCrsCod ();
   Msg_GetParamFilterFromTo ();
   Msg_GetParamFilterContent ();

   /***** Request confirmation to delete messages *****/
   if (Gbl.Msg.FilterContent[0])
      sprintf (StrFilterContent," relacionados con <strong>%s</strong>",	// Need translation!
               Gbl.Msg.FilterContent);
   else
      StrFilterContent[0] = '\0';
   sprintf (Gbl.Message,"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s%s?",	// Need translation!
            Gbl.Msg.FilterFromTo[0] ? Gbl.Msg.FilterFromTo : "cualquier usuario",	// Need translation!
            Gbl.Msg.FilterCrsShortName,StrFilterContent);
   Lay_ShowAlert (WARNING,Gbl.Message);
   Act_FormStart (ActDelAllSntMsg);
   Msg_PutHiddenParamsMsgsFilters ();
   fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"S&iacute;, eliminar mensajes enviados\" /></div></form>\n");	// Need translation!
  }

/*****************************************************************************/
/*********************** Delete all received messages ************************/
/*****************************************************************************/

void Msg_DelAllRecMsgs (void)
  {
   char FilterFromToSubquery[MAX_LENGTH_MESSAGES_QUERY+1];
   char FilterContentSubquery[MAX_LENGTH_MESSAGES_QUERY+1];

   /***** Get parameters *****/
   Msg_GetParamMsgsCrsCod ();
   Msg_GetParamFilterFromTo ();
   Msg_GetParamFilterContent ();
   Msg_GetParamOnlyUnreadMsgs ();
   Msg_MakeFilterFromToSubquery (FilterFromToSubquery);
   Msg_MakeFilterContentSubquery (FilterContentSubquery);

   /***** Delete messages *****/
   Msg_DelAllRecOrSntMsgsUsr (Msg_MESSAGES_RECEIVED,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Msg.FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
   sprintf (Gbl.Message,"Se han eliminado %s de %s desde %s.",	// Need translation!
            Gbl.Msg.ShowOnlyUnreadMsgs ? "los mensajes recibidos no le&iacute;dos" :
                                         "todos los mensajes recibidos",	// Need translation!
            Gbl.Msg.FilterFromTo[0] ? Gbl.Msg.FilterFromTo : "cualquier usuario",	// Need translation!
            Gbl.Msg.FilterCrsShortName);
   Lay_ShowAlert (INFO,Gbl.Message);
   Msg_ShowRecMsgs ();
  }

/*****************************************************************************/
/************************* Delete all sent messages **************************/
/*****************************************************************************/

void Msg_DelAllSntMsgs (void)
  {
   char FilterFromToSubquery[MAX_LENGTH_MESSAGES_QUERY+1];
   char FilterContentSubquery[MAX_LENGTH_MESSAGES_QUERY+1];

   /***** Get parameters *****/
   Msg_GetParamMsgsCrsCod ();
   Msg_GetParamFilterFromTo ();
   Msg_GetParamFilterContent ();
   Msg_MakeFilterFromToSubquery (FilterFromToSubquery);
   Msg_MakeFilterContentSubquery (FilterContentSubquery);

   /***** Delete messages *****/
   Msg_DelAllRecOrSntMsgsUsr (Msg_MESSAGES_SENT,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Msg.FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
   sprintf (Gbl.Message,"Se han eliminado todos los mensajes enviados a %s desde %s.",	// Need translation!
            Gbl.Msg.FilterFromTo[0] ? Gbl.Msg.FilterFromTo : "cualquier usuario",	// Need translation!
            Gbl.Msg.FilterCrsShortName);
   Lay_ShowAlert (INFO,Gbl.Message);
   Msg_ShowSntMsgs ();
  }

/*****************************************************************************/
/**************** Get parameter with course origin of messages ***************/
/*****************************************************************************/

void Msg_GetParamMsgsCrsCod (void)
  {
   extern const char *Txt_any_course[Txt_NUM_LANGUAGES];
   char LongStr[1+10+1];
   struct Course Crs;

   Par_GetParam (Par_PARAM_SINGLE,"FilterCrsCod",LongStr,1+10);
   Gbl.Msg.FilterCrsCod = Str_ConvertStrCodToLongCod (LongStr);
   if (Gbl.Msg.FilterCrsCod >= 0)	// If origin course specified
     {
      /* Get data of course */
      Crs.CrsCod = Gbl.Msg.FilterCrsCod;
      Deg_GetDataOfCourseByCod (&Crs);

      strcpy (Gbl.Msg.FilterCrsShortName,Crs.ShortName);
     }
   else
      strcpy (Gbl.Msg.FilterCrsShortName,Txt_any_course[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/******************* Get parameter with "from"/"to" filter ********************/
/*****************************************************************************/

void Msg_GetParamFilterFromTo (void)
  {
   /***** Get "from"/"to" filter *****/
   Par_GetParam (Par_PARAM_SINGLE,"FilterFromTo",Gbl.Msg.FilterFromTo,Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Msg.FilterFromTo,Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3,true);
  }

/*****************************************************************************/
/********************* Get parameter with content filter *********************/
/*****************************************************************************/

void Msg_GetParamFilterContent (void)
  {
   /***** Get content filter *****/
   Par_GetParam (Par_PARAM_SINGLE,"FilterContent",Gbl.Msg.FilterContent,Msg_MAX_LENGTH_FILTER_CONTENT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Msg.FilterContent,Msg_MAX_LENGTH_FILTER_CONTENT,true);
  }

/*****************************************************************************/
/************************* Make "from"/"to" subquery *************************/
/*****************************************************************************/

void Msg_MakeFilterFromToSubquery (char *FilterFromToSubquery)
  {
   const char *Ptr;
   char SearchWord[Usr_MAX_LENGTH_USR_NAME_OR_SURNAME+1];

   /***** Split "from"/"to" string into words *****/
   if (Gbl.Msg.FilterFromTo[0])
     {
      Ptr = Gbl.Msg.FilterFromTo;
      strcpy (FilterFromToSubquery," AND CONCAT(usr_data.FirstName,usr_data.Surname1,usr_data.Surname2) LIKE '");
      while (*Ptr)
        {
         Str_GetNextStringUntilSpace (&Ptr,SearchWord,Usr_MAX_LENGTH_USR_NAME_OR_SURNAME);
         if (strlen (FilterFromToSubquery) + strlen (SearchWord) + 512 > MAX_LENGTH_MESSAGES_QUERY)	// Prevent string overflow
            break;
         strcat (FilterFromToSubquery,"%");
         strcat (FilterFromToSubquery,SearchWord);
        }
      strcat (FilterFromToSubquery,"%'");
     }
   else
      FilterFromToSubquery[0] = '\0';
  }

/*****************************************************************************/
/**************************** Make content subquery **************************/
/*****************************************************************************/

void Msg_MakeFilterContentSubquery (char *FilterContentSubquery)
  {
   if (Gbl.Msg.FilterContent[0])
      sprintf (FilterContentSubquery," AND MATCH (messages.Subject,messages.Content) AGAINST ('%s')",Gbl.Msg.FilterContent);
   else
      FilterContentSubquery[0] = '\0';
  }

/*****************************************************************************/
/****************************** Delete a sent message ************************/
/*****************************************************************************/

void Msg_DelSntMsg (void)
  {
   extern const char *Txt_Message_removed[Txt_NUM_LANGUAGES];
   long MsgCod;

   /***** Get the code of the message to delete *****/
   MsgCod = Msg_GetParamMsgCod ();

   /***** Delete the message *****/
   /* Delete the sent message */
   Msg_SetSentMsgAsDeleted (MsgCod);
   Lay_ShowAlert (INFO,Txt_Message_removed[Gbl.Prefs.Language]);

   /* Show the remaining messages */
   Msg_ShowSntMsgs ();
  }

/*****************************************************************************/
/************************* Delete a received message *************************/
/*****************************************************************************/

void Msg_DelRecMsg (void)
  {
   extern const char *Txt_Message_removed[Txt_NUM_LANGUAGES];
   long MsgCod;

   /***** Get the code of the message to delete *****/
   MsgCod = Msg_GetParamMsgCod ();

   /***** Delete the message *****/
   /* Delete the received message */
   Msg_SetReceivedMsgAsDeleted (MsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   Lay_ShowAlert (INFO,Txt_Message_removed[Gbl.Prefs.Language]);

   /* Show the remaining messages */
   Msg_ShowRecMsgs ();
  }

/*****************************************************************************/
/**************************** Expand a sent message **************************/
/*****************************************************************************/

void Msg_ExpSntMsg (void)
  {
   long MsgCod;

   /***** Get the code of the message to expand *****/
   MsgCod = Msg_GetParamMsgCod ();

   /***** Expand the message *****/
   Msg_ExpandSentMsg (MsgCod);
         
   /***** Show again the messages *****/
   Msg_ShowSntMsgs ();
  }

/*****************************************************************************/
/**************************** Expand a sent message **************************/
/*****************************************************************************/

void Msg_ExpRecMsg (void)
  {
   long MsgCod;

   /***** Get the code of the message to expand *****/
   MsgCod = Msg_GetParamMsgCod ();

   /***** Expand the message *****/
   Msg_ExpandReceivedMsg (MsgCod);

   /***** Show again the messages *****/
   Msg_ShowRecMsgs ();
  }

/*****************************************************************************/
/**************************** Expand a sent message **************************/
/*****************************************************************************/

static void Msg_ExpandSentMsg (long MsgCod)
  {
   char Query[512];

   /***** Expand message in sent message table *****/
   sprintf (Query,"UPDATE messages SET Expanded='Y' WHERE MsgCod='%ld' AND UsrCod='%ld'",
            MsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not expand a sent message");

   /***** Contract all my other messages in sent message table *****/
   sprintf (Query,"UPDATE messages SET Expanded='N' WHERE UsrCod='%ld' AND MsgCod<>'%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod,MsgCod);
   DB_QueryUPDATE (Query,"can not contract a sent message");
  }

/*****************************************************************************/
/************************* Expand a received message *************************/
/*****************************************************************************/

static void Msg_ExpandReceivedMsg (long MsgCod)
  {
   char Query[512];

   /***** Expand message in received message table and mark it as read by me *****/
   sprintf (Query,"UPDATE msg_received SET Open='Y',Expanded='Y' WHERE MsgCod='%ld' AND UsrCod='%ld'",
            MsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not expand a received message");

   /***** Contract all my other messages in received message table *****/
   sprintf (Query,"UPDATE msg_received SET Expanded='N' WHERE UsrCod='%ld' AND MsgCod<>'%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod,MsgCod);
   DB_QueryUPDATE (Query,"can not contract a received message");
  }

/*****************************************************************************/
/********************** Insert a message in the database *********************/
/*****************************************************************************/
// Return the code of the new inserted message

long Msg_InsertNewMsg (long CrsCod,long UsrCod,const char *Subject,const char *Txt)
  {
   char Query[512+Cte_MAX_BYTES_SUBJECT+Cte_MAX_BYTES_TEXT];

   /***** Insert message in the database *****/
   sprintf (Query,"INSERT INTO messages (CrsCod,UsrCod,Deleted,Expanded,CreatTime,Subject,Content)" \
                  " VALUES ('%ld','%ld','N','N',NOW(),'",
            CrsCod,UsrCod);
   Str_AddStrToQuery (Query,Subject,sizeof (Query));
   strcat (Query,"','");
   Str_AddStrToQuery (Query,Txt,sizeof (Query));
   strcat (Query,"')");
   return DB_QueryINSERTandReturnCode (Query,"can not create message");
  }

/*****************************************************************************/
/************** Delete a message from the sent message table *****************/
/*****************************************************************************/

void Msg_SetSentMsgAsDeleted (long MsgCod)
  {
   char Query[512];

   /***** Delete message from sent message table *****/
   sprintf (Query,"UPDATE messages SET Deleted='Y' WHERE MsgCod='%ld'",MsgCod);
   DB_QueryUPDATE (Query,"can not remove a sent message");
  }

/*****************************************************************************/
/******************* Delete all the sent messages of a user ******************/
/*****************************************************************************/

void Msg_DelAllRecOrSntMsgsUsr (Msg_TypeOfMessages_t TypeOfMessages,long UsrCod,
                                long FilterCrsCod,const char *FilterFromToSubquery,const char *FilterContentSubquery)
  {
   char Query[MAX_LENGTH_MESSAGES_QUERY+1];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   
   unsigned long MsgNum,NumMsgs;
   long MsgCod;

   /***** Get all the messages sent by this user from database *****/
   Msg_ConstructQueryToSelectSentOrReceivedMsgs (Query,TypeOfMessages,UsrCod,FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
   NumMsgs = DB_QuerySELECT (Query,&mysql_res,"can not get list of sent messages");

   /***** Delete each sent message *****/
   for (MsgNum=0; MsgNum<NumMsgs; MsgNum++)
     {
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%ld",&MsgCod) != 1)
         Lay_ShowErrorAndExit ("Wrong code of message.");
      switch (TypeOfMessages)
        {
         case Msg_MESSAGES_RECEIVED:
            Msg_SetReceivedMsgAsDeleted (MsgCod,UsrCod);
            break;
         case Msg_MESSAGES_SENT:
            Msg_SetSentMsgAsDeleted (MsgCod);
            break;
         default:
            break;
        }
     }

   /***** Free the MySQL result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/** Insert a message y su destinatario in the table of messages recibidos **/
/*****************************************************************************/

static void Msg_InsertReceivedMsgIntoDB (long MsgCod,long UsrCod)
  {
   char Query[512];

   /***** Insert message received in the database *****/
   sprintf (Query,"INSERT INTO msg_received (MsgCod,UsrCod,Deleted,Notified,Open,Replied,Expanded)" \
                  " VALUES ('%ld','%ld','N','N','N','N','N')",
            MsgCod,UsrCod);
   DB_QueryINSERT (Query,"can not create received message");
  }

/*****************************************************************************/
/******** Update received message by setting Notified field to true **********/
/*****************************************************************************/

static void Msg_SetReceivedMsgAsNotified (long MsgCod,long UsrCod)
  {
   char Query[512];

   /***** Update received message by setting Notified field to true *****/
   sprintf (Query,"UPDATE msg_received SET Notified='Y' WHERE MsgCod='%ld' AND UsrCod='%ld'",
            MsgCod,UsrCod);
   DB_QueryUPDATE (Query,"can not update a received message");
  }

/*****************************************************************************/
/******** Update received message by setting Replied field to true ***********/
/*****************************************************************************/

static void Msg_SetReceivedMsgAsReplied (long MsgCod)
  {
   char Query[512];

   /***** Update received message by setting Replied field to true *****/
   sprintf (Query,"UPDATE msg_received SET Replied='Y' WHERE MsgCod='%ld' AND UsrCod='%ld'",
            MsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update a received message");
  }

/*****************************************************************************/
/************ Delete a message from the received message table ***************/
/*****************************************************************************/

void Msg_SetReceivedMsgAsDeleted (long MsgCod,long UsrCod)
  {
   char Query[512];

   /***** Delete message from received message table *****/
   sprintf (Query,"UPDATE msg_received SET Deleted='Y' WHERE MsgCod='%ld' AND UsrCod='%ld'",
            MsgCod,UsrCod);
   DB_QueryUPDATE (Query,"can not remove a received message");
  }

/*****************************************************************************/
/******** Get number of received messages that haven't been read by me *******/
/*****************************************************************************/

static unsigned Msg_GetNumUnreadMsgs (long FilterCrsCod,const char *FilterFromToSubquery,const char *FilterContentSubquery)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUnreadMsgs;

   /***** Get number of unread messages from database *****/
   if (FilterCrsCod >= 0)	// If origin course selected
     {
      if (FilterFromToSubquery[0])
         sprintf (Query,"SELECT COUNT(msg_received.MsgCod) FROM msg_received,messages,usr_data" \
                        " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N' AND msg_received.Open='N'" \
                        " AND msg_received.MsgCod=messages.MsgCod AND messages.CrsCod='%ld'" \
                        " AND messages.UsrCod=usr_data.UsrCod%s%s" \
                        " ORDER BY msg_received.MsgCod DESC",
                  Gbl.Usrs.Me.UsrDat.UsrCod,FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
      else
         sprintf (Query,"SELECT COUNT(msg_received.MsgCod) FROM msg_received,messages" \
                        " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N' AND msg_received.Open='N'" \
                        " AND msg_received.MsgCod=messages.MsgCod AND messages.CrsCod='%ld'%s" \
                        " ORDER BY msg_received.MsgCod DESC",
                  Gbl.Usrs.Me.UsrDat.UsrCod,FilterCrsCod,FilterContentSubquery);
     }
   else	// If no origin course selected
     {
      if (FilterFromToSubquery[0])
         sprintf (Query,"SELECT COUNT(msg_received.MsgCod) FROM msg_received,messages,usr_data" \
                        " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N' AND msg_received.Open='N'" \
                        " AND msg_received.MsgCod=messages.MsgCod AND messages.UsrCod=usr_data.UsrCod%s%s" \
                        " ORDER BY msg_received.MsgCod DESC",
                  Gbl.Usrs.Me.UsrDat.UsrCod,FilterFromToSubquery,FilterContentSubquery);
      else
        {
         if (FilterContentSubquery[0])
            sprintf (Query,"SELECT COUNT(msg_received.MsgCod) FROM msg_received,messages" \
                           " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N' AND msg_received.Open='N'" \
                           " AND msg_received.MsgCod=messages.MsgCod%s" \
                           " ORDER BY msg_received.MsgCod DESC",
                     Gbl.Usrs.Me.UsrDat.UsrCod,FilterContentSubquery);
         else
            sprintf (Query,"SELECT COUNT(*) FROM msg_received" \
                           " WHERE UsrCod='%ld' AND Deleted='N' AND Open='N'" \
                           " ORDER BY MsgCod DESC",
                     Gbl.Usrs.Me.UsrDat.UsrCod);
        }
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of unread messages");

   /***** Get row with number of unread messages *****/
   row = mysql_fetch_row (mysql_res);

   /***** Get number of unread messages (row[0]) *****/
   if (sscanf (row[0],"%u",&NumUnreadMsgs) != 1)
      Lay_ShowErrorAndExit ("Wrong number of unread messages.");

   DB_FreeMySQLResult (&mysql_res);

   return NumUnreadMsgs;
  }

/*****************************************************************************/
/************************ Show sent or received messages *********************/
/*****************************************************************************/

static void Msg_ShowSentOrReceivedMessages (Msg_TypeOfMessages_t TypeOfMessages)
  {
   extern const char *Txt_any_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_View_messages[Txt_NUM_LANGUAGES];
   char FilterFromToSubquery[MAX_LENGTH_MESSAGES_QUERY+1];
   char FilterContentSubquery[MAX_LENGTH_MESSAGES_QUERY+1];
   char Query[MAX_LENGTH_MESSAGES_QUERY+1];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   unsigned long NumMsg = 0;		// Initialized to avoid warning
   unsigned NumMsgs;
   unsigned NumUnreadMsgs = 0;
   unsigned NumNewMsgs = ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Gbl.Usrs.Me.UsrDat.NumNewMsgs : 0);
   struct Pagination Pagination;
   long MsgCod;
   bool NewMsg = false;

   /***** Get the page number *****/
   Pag_GetParamPagNum ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Pag_MESSAGES_RECEIVED : Pag_MESSAGES_SENT);

   /***** Write title *****/
   if (NumNewMsgs)	// If viewing received messages and number of new messages > 0...
     {
      /* Show all messages */
      Gbl.Msg.FilterCrsCod = -1L;	// ...show all messages
      strcpy (Gbl.Msg.FilterCrsShortName,Txt_any_course[Gbl.Prefs.Language]);
      Gbl.Msg.FilterFromTo[0] = '\0';
      FilterFromToSubquery[0] = '\0';
      FilterContentSubquery[0] = '\0';
     }
   else
     {
      /* Show only selected messages */
      Msg_GetParamMsgsCrsCod ();
      Msg_GetParamFilterFromTo ();
      Msg_GetParamFilterContent ();
      Msg_MakeFilterFromToSubquery (FilterFromToSubquery);
      Msg_MakeFilterContentSubquery (FilterContentSubquery);
     }
   Msg_GetDistinctCoursesInMyMessages (TypeOfMessages);

   /***** Form to see messages again *****/
   Act_FormStart ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? ActSeeRecMsg : ActSeeSntMsg);
   Msg_ShowFormSelectCourseSentOrRecMsgs (TypeOfMessages);
   Msg_ShowFormToFilterMsgs (TypeOfMessages);
   if (TypeOfMessages == Msg_MESSAGES_RECEIVED)
     {
      Msg_GetParamOnlyUnreadMsgs ();
      Msg_ShowFormToShowOnlyUnreadMessages ();
     }
   Lay_PutSendButton (Txt_View_messages[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>");

   if (TypeOfMessages == Msg_MESSAGES_RECEIVED)
      NumUnreadMsgs = Msg_GetNumUnreadMsgs (Gbl.Msg.FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);

   /***** Get messages from database *****/
   Msg_ConstructQueryToSelectSentOrReceivedMsgs (Query,TypeOfMessages,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Msg.FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get messages");

   NumMsgs = (unsigned) NumRows;

   /* Write number of messages and number of new messages */
   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"TITULO\">");
   Msg_WriteNumMsgs (NumMsgs,NumUnreadMsgs,NumNewMsgs);
   fprintf (Gbl.F.Out,"</div>");

   if (NumMsgs)		// If there are messages...
     {
      /***** Show form to delete all messages *****/
      Msg_ShowFormDelSentOrRecMsgs (TypeOfMessages,NumMsgs);

      /***** Compute variables related to pagination *****/
      Pagination.NumItems = NumMsgs;
      Pagination.CurrentPage = (int) Gbl.Pag.CurrentPage;
      Pag_CalculatePagination (&Pagination);
      Gbl.Pag.CurrentPage = (unsigned) Pagination.CurrentPage;

      /***** Save my current page in order to show it next time I'll view my received/sent messages *****/
      Pag_SaveLastPageMsgIntoSession (TypeOfMessages,Gbl.Pag.CurrentPage);

      /***** Write links to pages *****/
      if (Pagination.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Pag_MESSAGES_RECEIVED : Pag_MESSAGES_SENT,0,&Pagination);

      /***** Show messages from this page, the author and the date of last reply *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" width=\"100%%\"><table cellspacing=\"0\" cellpadding=\"2\" width=\"100%%\">");

      for (NumRow=1; NumRow<=NumRows; NumRow++)
        {
         row = mysql_fetch_row (mysql_res);
         if (NumRow >= Pagination.FirstItemVisible &&
             NumRow <= Pagination.LastItemVisible)
           {
            if (sscanf (row[0],"%ld",&MsgCod) != 1)
               Lay_ShowErrorAndExit ("Wrong code of message.");
            switch (TypeOfMessages)
              {
               case Msg_MESSAGES_RECEIVED:
                  NumMsg = NumRows - NumRow + 1;
                  NewMsg = (NumMsg > NumRows - NumNewMsgs);
	          break;
               case Msg_MESSAGES_SENT:
                  NumMsg = NumRows - NumRow + 1;
	          break;
               default: // Not aplicable here
	          break;
              }
            Msg_ShowASentOrReceivedMessage (TypeOfMessages,NumMsg,MsgCod,NewMsg);
           }
        }

      fprintf (Gbl.F.Out,"</table></td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Write again links to pages *****/
      if (Pagination.MoreThanOnePage)
         Pag_WriteLinksToPagesCentered ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Pag_MESSAGES_RECEIVED : Pag_MESSAGES_SENT,0,&Pagination);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********* Generate a query to select messages received or sent **************/
/*****************************************************************************/

static void Msg_ConstructQueryToSelectSentOrReceivedMsgs (char *Query,Msg_TypeOfMessages_t TypeOfMessages,long UsrCod,
                                                          long FilterCrsCod,const char *FilterFromToSubquery,const char *FilterContentSubquery)
  {
   const char *StrUnreadMsg;

   if (FilterCrsCod >= 0)	// If origin course selected
     {
      switch (TypeOfMessages)
        {
         case Msg_MESSAGES_RECEIVED:
            StrUnreadMsg = (Gbl.Msg.ShowOnlyUnreadMsgs ? " AND msg_received.Open='N'" : "");
            if (FilterFromToSubquery[0])
               sprintf (Query,"SELECT msg_received.MsgCod FROM msg_received,messages,usr_data" \
                              " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N'%s" \
                              " AND msg_received.MsgCod=messages.MsgCod AND messages.CrsCod='%ld'" \
                              " AND messages.UsrCod=usr_data.UsrCod%s%s" \
                              " ORDER BY msg_received.MsgCod DESC",
                        UsrCod,StrUnreadMsg,FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
            else
               sprintf (Query,"SELECT msg_received.MsgCod FROM msg_received,messages" \
                              " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N'%s" \
                              " AND msg_received.MsgCod=messages.MsgCod AND messages.CrsCod='%ld'%s" \
                              " ORDER BY msg_received.MsgCod DESC",
                        UsrCod,StrUnreadMsg,FilterCrsCod,FilterContentSubquery);
            break;
         case Msg_MESSAGES_SENT:
            if (FilterFromToSubquery[0])
               sprintf (Query,"SELECT DISTINCT messages.MsgCod FROM messages,msg_received,usr_data" \
                              " WHERE messages.UsrCod='%ld' AND messages.CrsCod='%ld' AND messages.Deleted='N'" \
                              " AND messages.MsgCod=msg_received.MsgCod AND msg_received.UsrCod=usr_data.UsrCod%s%s" \
                              " ORDER BY messages.MsgCod DESC",
                        UsrCod,FilterCrsCod,FilterFromToSubquery,FilterContentSubquery);
            else
               sprintf (Query,"SELECT MsgCod FROM messages" \
                              " WHERE UsrCod='%ld' AND CrsCod='%ld' AND Deleted='N'%s" \
                              " ORDER BY MsgCod DESC",
                        UsrCod,FilterCrsCod,FilterContentSubquery);
            break;
         default: // Not aplicable here
            break;
        }
     }
   else	// If no origin course selected
     {
      switch (TypeOfMessages)
        {
         case Msg_MESSAGES_RECEIVED:
            if (FilterFromToSubquery[0])
              {
               StrUnreadMsg = (Gbl.Msg.ShowOnlyUnreadMsgs ? " AND msg_received.Open='N'" : "");
               sprintf (Query,"SELECT msg_received.MsgCod FROM msg_received,messages,usr_data" \
                              " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N'%s" \
                              " AND msg_received.MsgCod=messages.MsgCod AND messages.UsrCod=usr_data.UsrCod%s%s" \
                              " ORDER BY msg_received.MsgCod DESC",
                        UsrCod,StrUnreadMsg,FilterFromToSubquery,FilterContentSubquery);
              }
            else
              {
               if (FilterContentSubquery[0])
                 {
                  StrUnreadMsg = (Gbl.Msg.ShowOnlyUnreadMsgs ? " AND msg_received.Open='N'" : "");
                  sprintf (Query,"SELECT msg_received.MsgCod FROM msg_received,messages" \
                                 " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N'%s" \
                                 " AND msg_received.MsgCod=messages.MsgCod%s" \
                                 " ORDER BY msg_received.MsgCod DESC",
                           UsrCod,StrUnreadMsg,FilterContentSubquery);
                 }
               else
                 {
                  StrUnreadMsg = (Gbl.Msg.ShowOnlyUnreadMsgs ? " AND Open='N'" : "");
                  sprintf (Query,"SELECT MsgCod FROM msg_received" \
                                 " WHERE UsrCod='%ld' AND Deleted='N'%s" \
                                 " ORDER BY MsgCod DESC",
                           UsrCod,StrUnreadMsg);
                 }
              }
            break;
         case Msg_MESSAGES_SENT:
            if (FilterFromToSubquery[0])
               sprintf (Query,"SELECT DISTINCT messages.MsgCod FROM messages,msg_received,usr_data" \
                              " WHERE messages.UsrCod='%ld' AND messages.Deleted='N'" \
                              " AND messages.MsgCod=msg_received.MsgCod AND msg_received.UsrCod=usr_data.UsrCod%s%s" \
                              " ORDER BY messages.MsgCod DESC",
                        UsrCod,FilterFromToSubquery,FilterContentSubquery);
            else
               sprintf (Query,"SELECT MsgCod FROM messages" \
                              " WHERE UsrCod='%ld' AND Deleted='N'%s" \
                              " ORDER BY MsgCod DESC",
                        UsrCod,FilterContentSubquery);
            break;
         default: // Not aplicable here
            break;
        }
     }
  }

/*****************************************************************************/
/***** Get the number of unique messages sent by teachers from a course ******/
/*****************************************************************************/

unsigned Msg_GetNumMsgsSentByTchsCrs (long CrsCod)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumMsgsSent;

   /***** Get the number of unique messages sent by teachers from this course from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM messages,usr_data,crs_usr" \
                  " WHERE messages.CrsCod='%ld' AND usr_data.UsrType='%u' AND crs_usr.CrsCod='%ld'" \
                  " AND messages.UsrCod=usr_data.UsrCod AND messages.UsrCod=crs_usr.UsrCod",
                  CrsCod,(unsigned) Usr_TEACHER,CrsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of messages sent by teachers");

   /* Get number of messages */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumMsgsSent) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of messages sent by teachers.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return NumMsgsSent;
  }

/*****************************************************************************/
/******** Get the number of unique messages sent from this location **********/
/******** (all the platform, current degree or current course)      **********/
/*****************************************************************************/

unsigned Msg_GetNumMsgsSent (Sta_LocationRange_t LocationRange,Msg_Status_t MsgStatus)
  {
   char Subquery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumMsgsSent;

   /***** Get the number of messages sent from this location
          (all the platform, current degree or current course) from database *****/
   switch (MsgStatus)
     {
      case Msg_STATUS_ALL:
      case Msg_STATUS_NOTIFIED:
         Subquery[0] = '\0';
         break;
      case Msg_STATUS_DELETED:
         switch (LocationRange)
           {
            case Sta_LOCATION_RANGE_ALL:
               strcpy (Subquery," WHERE Deleted='Y'");
               break;
            case Sta_LOCATION_RANGE_THIS_DEGREE:
               strcpy (Subquery," AND messages.Deleted='Y'");
            case Sta_LOCATION_RANGE_THIS_COURSE:
            default:
               strcpy (Subquery," AND Deleted='Y'");
           }
     }
   switch (LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT COUNT(*) FROM messages%s",
                  Subquery);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT COUNT(*) FROM messages,courses" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=messages.CrsCod%s",
                  Gbl.CurrentDeg.Deg.DegCod,Subquery);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         sprintf (Query,"SELECT COUNT(*) FROM messages" \
                        " WHERE CrsCod='%ld'%s",
                  Gbl.CurrentCrs.Crs.CrsCod,Subquery);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of sent messages");

   /* Get number of users */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumMsgsSent) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of sent messages.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return NumMsgsSent;
  }

/*****************************************************************************/
/****** Get the number of unique messages received from this location ********/
/****** (all the platform, current degree or current course)          ********/
/*****************************************************************************/

unsigned Msg_GetNumMsgsReceived (Sta_LocationRange_t LocationRange,Msg_Status_t MsgStatus)
  {
   char Subquery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumMsgsReceived;

   /***** Get the number of unique messages sent from this location
          (all the platform, current degree or current course) from database *****/
   switch (MsgStatus)
     {
      case Msg_STATUS_ALL:
         Subquery[0] = '\0';
         break;
      case Msg_STATUS_DELETED:
         switch (LocationRange)
           {
            case Sta_LOCATION_RANGE_ALL:
               strcpy (Subquery," WHERE Deleted='Y'");
               break;
            case Sta_LOCATION_RANGE_THIS_DEGREE:
            case Sta_LOCATION_RANGE_THIS_COURSE:
            default:
               strcpy (Subquery," AND msg_received.Deleted='Y'");
           }
         break;
      case Msg_STATUS_NOTIFIED:
         switch (LocationRange)
           {
            case Sta_LOCATION_RANGE_ALL:
               strcpy (Subquery," WHERE Notified='Y'");
               break;
            case Sta_LOCATION_RANGE_THIS_DEGREE:
            case Sta_LOCATION_RANGE_THIS_COURSE:
            default:
               strcpy (Subquery," AND msg_received.Notified='Y'");
           }
     }
   switch (LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT COUNT(*) FROM msg_received%s",
                  Subquery);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT COUNT(*) FROM msg_received,courses,messages" \
                         " WHERE courses.DegCod='%ld' AND courses.CrsCod=messages.CrsCod AND messages.MsgCod=msg_received.MsgCod%s",
                  Gbl.CurrentDeg.Deg.DegCod,Subquery);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         sprintf (Query,"SELECT COUNT(*) FROM msg_received,messages" \
                        " WHERE messages.CrsCod='%ld' AND messages.MsgCod=msg_received.MsgCod%s",
                  Gbl.CurrentCrs.Crs.CrsCod,Subquery);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of received messages");

   /* Get number of messages */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumMsgsReceived) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of sent messages.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return NumMsgsReceived;
  }

/*****************************************************************************/
/*********** Write number of messages and number of new messages *************/
/*****************************************************************************/

void Msg_WriteNumMsgs (unsigned NumMsgs,unsigned NumUnreadMsgs,unsigned NumNewMsgs)
  {
   extern const char *Txt_message[Txt_NUM_LANGUAGES];
   extern const char *Txt_messages[Txt_NUM_LANGUAGES];
   extern const char *Txt_unread_MESSAGE[Txt_NUM_LANGUAGES];
   extern const char *Txt_unread_MESSAGES[Txt_NUM_LANGUAGES];
   extern const char *Txt_new_MESSAGE[Txt_NUM_LANGUAGES];
   extern const char *Txt_new_MESSAGES[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"[");

   /***** Write total number of messages *****/
   if (NumMsgs == 1)
      fprintf (Gbl.F.Out,"1 %s",Txt_message[Gbl.Prefs.Language]);
   else
      fprintf (Gbl.F.Out,"%u %s",NumMsgs,Txt_messages[Gbl.Prefs.Language]);

   if (NumMsgs)
     {
      /***** Write number of unread messages *****/
      if (NumUnreadMsgs)
        {
         if (NumUnreadMsgs == 1)
	    fprintf (Gbl.F.Out,", 1 %s",Txt_unread_MESSAGE[Gbl.Prefs.Language]);
         else
	    fprintf (Gbl.F.Out,", %u %s",NumUnreadMsgs,Txt_unread_MESSAGES[Gbl.Prefs.Language]);
        }

      /***** Write number of new messages *****/
      if (NumNewMsgs)
        {
         if (NumNewMsgs == 1)
	    fprintf (Gbl.F.Out,", 1 %s",Txt_new_MESSAGE[Gbl.Prefs.Language]);
         else
	    fprintf (Gbl.F.Out,", %u %s",NumNewMsgs,Txt_new_MESSAGES[Gbl.Prefs.Language]);
        }
     }

   fprintf (Gbl.F.Out,"]");
  }

/*****************************************************************************/
/*************** Show form to delete sent or received messages ***************/
/*****************************************************************************/

void Msg_ShowFormDelSentOrRecMsgs (Msg_TypeOfMessages_t TypeOfMessages,unsigned NumMsgs)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_MSG_containing_the_text[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSGS_containing_the_text[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_the_MESSAGE[Txt_NUM_LANGUAGES];
   extern const char *Txt_message_received_from_A_USER[Txt_NUM_LANGUAGES];
   extern const char *Txt_message_sent_to[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_the_X_MESSAGES[Txt_NUM_LANGUAGES];
   extern const char *Txt_unread_messages_received_from[Txt_NUM_LANGUAGES];
   extern const char *Txt_messages_received_from_A_USER[Txt_NUM_LANGUAGES];
   extern const char *Txt_messages_sent_to_A_USER[Txt_NUM_LANGUAGES];
   extern const char *Txt_any_user[Txt_NUM_LANGUAGES];
   extern const char *Txt_from_A_COURSE[Txt_NUM_LANGUAGES];
   char StrFilterContent[256+Msg_MAX_LENGTH_FILTER_CONTENT+1];

   /***** Put link to request deletion of all sent or received messages *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? ActReqDelAllRecMsg : ActReqDelAllSntMsg);
   Msg_PutHiddenParamsMsgsFilters ();

   if (Gbl.Msg.FilterContent[0])
      sprintf (StrFilterContent,", %s <strong>%s</strong>",
               (NumMsgs == 1) ? Txt_MSG_containing_the_text[Gbl.Prefs.Language] :
                                Txt_MSGS_containing_the_text[Gbl.Prefs.Language],
               Gbl.Msg.FilterContent);
   else
      StrFilterContent[0] = '\0';
   
   if (NumMsgs == 1)
      sprintf (Gbl.MsgWindowStatus,"%s %s %s, %s %s%s",
               Txt_Remove_the_MESSAGE[Gbl.Prefs.Language],
               (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Txt_message_received_from_A_USER[Gbl.Prefs.Language] :
                                                       Txt_message_sent_to[Gbl.Prefs.Language],
               Gbl.Msg.FilterFromTo[0] ? Gbl.Msg.FilterFromTo : Txt_any_user[Gbl.Prefs.Language],
               Txt_from_A_COURSE[Gbl.Prefs.Language],
               Gbl.Msg.FilterCrsShortName,
               StrFilterContent);
   else
      sprintf (Gbl.MsgWindowStatus,"%s %u %s %s, %s %s%s",
               Txt_Remove_the_X_MESSAGES[Gbl.Prefs.Language],
               NumMsgs,
               (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? (Gbl.Msg.ShowOnlyUnreadMsgs ? Txt_unread_messages_received_from[Gbl.Prefs.Language] :
                                                                                     Txt_messages_received_from_A_USER[Gbl.Prefs.Language]) :
                                                       Txt_messages_sent_to_A_USER[Gbl.Prefs.Language],
               Gbl.Msg.FilterFromTo[0] ? Gbl.Msg.FilterFromTo : Txt_any_user[Gbl.Prefs.Language],
               Txt_from_A_COURSE[Gbl.Prefs.Language],
               Gbl.Msg.FilterCrsShortName,
               StrFilterContent);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" align=\"bottom\" /> %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/***** Write a form parameter to specify filter "from"/"to" for messages *****/
/*****************************************************************************/

void Msg_PutHiddenParamsMsgsFilters (void)
  {
   if (Gbl.Msg.FilterCrsCod >= 0)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FilterCrsCod\" value=\"%ld\" />",
               Gbl.Msg.FilterCrsCod);
   if (Gbl.Msg.FilterFromTo[0])
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FilterFromTo\" value=\"%s\" />",
               Gbl.Msg.FilterFromTo);
   if (Gbl.Msg.FilterContent[0])
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FilterContent\" value=\"%s\" />",
               Gbl.Msg.FilterContent);
   if (Gbl.Msg.ShowOnlyUnreadMsgs)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OnlyUnreadMsgs\" value=\"Y\" />");
  }

/*****************************************************************************/
/********************* Get dictinct courses in my messages *******************/
/*****************************************************************************/

void Msg_GetDistinctCoursesInMyMessages (Msg_TypeOfMessages_t TypeOfMessages)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct Course Crs;

   /***** Get distinct courses in my messages from database *****/
   switch (TypeOfMessages)
     {
      case Msg_MESSAGES_RECEIVED:
         sprintf (Query,"SELECT DISTINCT courses.CrsCod,courses.ShortName FROM msg_received,messages,courses" \
                        " WHERE msg_received.UsrCod='%ld' AND msg_received.Deleted='N' AND msg_received.MsgCod=messages.MsgCod AND messages.CrsCod=courses.CrsCod" \
                        " ORDER BY courses.ShortName",Gbl.Usrs.Me.UsrDat.UsrCod);
         break;
      case Msg_MESSAGES_SENT:
         sprintf (Query,"SELECT DISTINCT courses.CrsCod,courses.ShortName FROM messages,courses" \
                        " WHERE messages.UsrCod='%ld' AND messages.Deleted='N' AND messages.CrsCod=courses.CrsCod" \
                        " ORDER BY courses.ShortName",Gbl.Usrs.Me.UsrDat.UsrCod);
         break;
      default: // Not aplicable here
         break;
     }
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get distinct courses in your messages");

   /***** Get distinct courses in messages from database *****/
   Gbl.Msg.NumCourses = 0;
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next course */
      row = mysql_fetch_row (mysql_res);
      Crs.CrsCod = Str_ConvertStrCodToLongCod (row[0]);
      if (Crs.CrsCod >= 0 && Gbl.Msg.NumCourses < Deg_MAX_COURSES_PER_USR)
         if (Deg_GetDataOfCourseByCod (&Crs))
           {
            Gbl.Msg.Courses[Gbl.Msg.NumCourses].CrsCod = Crs.CrsCod;
            strcpy (Gbl.Msg.Courses[Gbl.Msg.NumCourses].ShortName,Crs.ShortName);
            Gbl.Msg.NumCourses++;
           }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********* Show form to select course for sent or received messages **********/
/*****************************************************************************/

void Msg_ShowFormSelectCourseSentOrRecMsgs (Msg_TypeOfMessages_t TypeOfMessages)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Messages_received_from_A_COURSE[Txt_NUM_LANGUAGES];
   extern const char *Txt_Messages_sent_from_A_COURSE[Txt_NUM_LANGUAGES];
   extern const char *Txt_any_course[Txt_NUM_LANGUAGES];
   unsigned NumOriginCrs;

   /***** Course selection *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   fprintf (Gbl.F.Out,"<span class=\"%s\">%s </span>" \
                      "<select name=\"FilterCrsCod\">" \
                      "<option value=\"\"",
            The_ClassFormul[Gbl.Prefs.Theme],
            (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Txt_Messages_received_from_A_COURSE[Gbl.Prefs.Language] :
                                                    Txt_Messages_sent_from_A_COURSE[Gbl.Prefs.Language]);
   if (Gbl.Msg.FilterCrsCod < 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Txt_any_course[Gbl.Prefs.Language]);

   /***** Write an option for each user's course  *****/
   for (NumOriginCrs=0; NumOriginCrs<Gbl.Msg.NumCourses; NumOriginCrs++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.Msg.Courses[NumOriginCrs].CrsCod);
      if (Gbl.Msg.Courses[NumOriginCrs].CrsCod == Gbl.Msg.FilterCrsCod)
        fprintf (Gbl.F.Out," selected=\"selected\"");	// Select origin course
      fprintf (Gbl.F.Out,">%s</option>",Gbl.Msg.Courses[NumOriginCrs].ShortName);
     }
   fprintf (Gbl.F.Out,"</select></div>\n");
  }

/*****************************************************************************/
/***** Show form to filter "from" and "to" for received or sent messages *****/
/*****************************************************************************/

void Msg_ShowFormToFilterMsgs (Msg_TypeOfMessages_t TypeOfMessages)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_MSG_From[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_To[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_Message[Txt_NUM_LANGUAGES];

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Authors/recipients of the message *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" class=\"%s\">%s:&nbsp;</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"FilterFromTo\" size=\"40\" maxlength=\"%u\" value=\"%s\" /></td>" \
                      "</tr>",
            The_ClassFormul[Gbl.Prefs.Theme],
            (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? Txt_MSG_From[Gbl.Prefs.Language] :
                                                    Txt_MSG_To[Gbl.Prefs.Language],
            Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3,Gbl.Msg.FilterFromTo);

   /***** Authors/recipients of the message *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" class=\"%s\">%s:&nbsp;</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"FilterContent\" size=\"40\" maxlength=\"%u\" value=\"%s\" /></td>" \
                      "</tr>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_MSG_Message[Gbl.Prefs.Language],
            Msg_MAX_LENGTH_FILTER_CONTENT,Gbl.Msg.FilterContent);

   /***** Table end *****/
   fprintf (Gbl.F.Out,"</table></div>\n");
  }

/*****************************************************************************/
/**** Show form to select whether to show only unread (received) messages ****/
/*****************************************************************************/

static void Msg_ShowFormToShowOnlyUnreadMessages (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_only_unread_messages[Txt_NUM_LANGUAGES];

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Put checkbox to select whether to show only unread (received) messages *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\">" \
                      "<input type=\"checkbox\" name=\"OnlyUnreadMsgs\" value=\"Y\"");
   if (Gbl.Msg.ShowOnlyUnreadMsgs)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," /></td><td align=\"left\" valign=\"middle\" class=\"%s\">&nbsp;%s</td>" \
                      "</tr>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Show_only_unread_messages[Gbl.Prefs.Language]);

   /***** Table end *****/
   fprintf (Gbl.F.Out,"</table></div>\n");
  }

/*****************************************************************************/
/*********** Get parameter to show only unread (received) messages ***********/
/*****************************************************************************/

static void Msg_GetParamOnlyUnreadMsgs (void)
  {
   char YN[1+1];

   /***** Get parameter to show only unread (received) messages *****/
   Par_GetParam (Par_PARAM_SINGLE,"OnlyUnreadMsgs",YN,1);
   Gbl.Msg.ShowOnlyUnreadMsgs = (Str_ConvertToUpperLetter (YN[0]) == 'Y');
  }

/*****************************************************************************/
/**************************** Get data of a message *************************/
/*****************************************************************************/

static void Msg_GetMsgData (long MsgCod,long *CrsCod,long *UsrCod,bool *Deleted,char *CreatTime,char *Subject)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get data of message from database *****/
   sprintf (Query,"SELECT CrsCod,UsrCod,Deleted,DATE_FORMAT(CreatTime,'%%Y%%m%%d%%H%%i%%S'),Subject" \
                  " FROM messages WHERE MsgCod='%ld'",MsgCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a message");

   /***** Result should have a unique row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting data of a message.");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);

   /****** Get location (row[0]) *****/
   *CrsCod = Str_ConvertStrCodToLongCod (row[0]);

   /****** Get author code (row[1]) *****/
   *UsrCod = Str_ConvertStrCodToLongCod (row[1]);

   /***** Get if message has been deleted by author (row[2]) *****/ 
   *Deleted = (Str_ConvertToUpperLetter (row[2][0]) != 'N');

   /****** Get creation time (row[3]) *****/
   strcpy (CreatTime,row[3]);

   /****** Get subject (row[4]) *****/
   strcpy (Subject,row[4]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************************** Get content of a message *************************/
/*****************************************************************************/

static void Msg_GetMsgContent (long MsgCod,char *Content)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get content of message from database *****/
   sprintf (Query,"SELECT Content FROM messages WHERE MsgCod='%ld'",MsgCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the content of a message");

   /***** Result should have a unique row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting content of a message.");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);

   /****** Get location (row[0]) *****/
   strncpy (Content,row[0],Cte_MAX_BYTES_TEXT);
   Content[Cte_MAX_BYTES_TEXT] = '\0';

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********************** Get if a sent message is expanded ********************/
/*****************************************************************************/

static void Msg_GetStatusOfSentMsg (long MsgCod,bool *Expanded)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get if sent message has been replied/expanded from database *****/
   sprintf (Query,"SELECT Expanded FROM messages" \
                  " WHERE MsgCod='%ld' AND UsrCod='%ld' AND Deleted='N'",
            MsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get if a sent message has been replied/expanded");

   /***** Result should have a unique row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting if a sent message has been replied/expanded.");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);

   /***** Get if message is expanded *****/
   *Expanded = (Str_ConvertToUpperLetter (row[0][0]) != 'N');

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/***************** Get if a received message has been replied ****************/
/*****************************************************************************/

static void Msg_GetStatusOfReceivedMsg (long MsgCod,bool *Open,bool *Replied,bool *Expanded)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get if received message has been replied/expanded from database *****/
   sprintf (Query,"SELECT Open,Replied,Expanded FROM msg_received" \
                  " WHERE MsgCod='%ld' AND UsrCod='%ld' AND Deleted='N'",
            MsgCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get if a received message has been replied/expanded");

   /***** Result should have a unique row *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting if a received message has been replied/expanded.");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);

   /***** Get if message has been read by me *****/
   *Open = (Str_ConvertToUpperLetter (row[0][0]) != 'N');

   /***** Get if message has been replied *****/
   *Replied = (Str_ConvertToUpperLetter (row[1][0]) != 'N');

   /***** Get if message is expanded *****/
   *Expanded = (Str_ConvertToUpperLetter (row[2][0]) != 'N');

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/******** Show a sent or a received message (from a user to another) *********/
/*****************************************************************************/

static void Msg_ShowASentOrReceivedMessage (Msg_TypeOfMessages_t TypeOfMessages,long MsgNum,long MsgCod,bool NewMsg)
  {
   extern const char *Txt_MSG_From[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_To[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_Message[Txt_NUM_LANGUAGES];
   struct UsrData UsrDat;
   bool FromThisCrs = false;		// Initialized to avoid warning
   char CreatTime[4+2+2+2+2+2+1];	// Creation time of a message in YYYYMMDDHHMMSS format
   long CrsCod;
   char Subject[Cte_MAX_BYTES_SUBJECT+1];
   char Content[Cte_MAX_BYTES_TEXT+1];
   bool Deleted;
   bool Open = true;
   bool Replied = false;		// Initialized to avoid warning
   bool Expanded = false;

   /***** Initialize structure with user's data *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Get data of message *****/
   Msg_GetMsgData (MsgCod,&CrsCod,&UsrDat.UsrCod,&Deleted,CreatTime,Subject);
   switch (TypeOfMessages)
     {
      case Msg_MESSAGES_RECEIVED:
         Msg_GetStatusOfReceivedMsg (MsgCod,&Open,&Replied,&Expanded);
         break;
      case Msg_MESSAGES_SENT:
         Msg_GetStatusOfSentMsg (MsgCod,&Expanded);
         break;
      default: // Not aplicable here
         break;
     }

   /***** Put an icon with message status *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\" width=\"16\" class=\"%s\">" \
                      "<table cellspacing=\"0\" cellpadding=\"0\" width=\"16\">" \
                      "<tr><td align=\"center\" valign=\"top\" width=\"16\">" \
                      "<acronym title=\"%s\">" \
                      "<img src=\"%s/msg_%s16x16.gif\" width=\"16\" height=\"16\" />" \
                      "</acronym></td></tr>",
            (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? (Open ? "BG_MSG_BLUE" : "BG_MSG_GREEN") :
                                                    "BG_MSG_BLUE",
            (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? (Open ? (Replied ? "Respondido" :"No respondido") :
                                                            (NewMsg ? "Nuevo" : "No le&iacute;do")) :
                                                    "Enviado",
            Gbl.Prefs.IconsURL,
            (TypeOfMessages == Msg_MESSAGES_RECEIVED) ? (Open ? (Replied ? "replied" :"open") :
                                                            (NewMsg ? "new" : "unread")) :
                                                    "fwd");

   /***** Form to delete message *****/
   fprintf (Gbl.F.Out,"<tr>");
   Msg_PutFormToDeleteMessage (MsgNum,MsgCod,TypeOfMessages);
   fprintf (Gbl.F.Out,"</tr></table></td>");

   /***** Write message number *****/
   Msg_WriteMsgNumber (MsgNum,!Open);

   /***** Write message author *****/
   Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat);
   Msg_WriteMsgAuthor (&UsrDat,80,11,Open ? "MSG_AUT_BG" : "MSG_AUT_BG_NEW",Open ? "MSG_AUT" : "MSG_AUT_NEW",true,NULL);

   /***** Write subject *****/
   Msg_WriteSentOrReceivedMsgSubject (TypeOfMessages,MsgCod,Subject,Open,Expanded);

   /***** Write date *****/
   Msg_WriteMsgDate (CreatTime,!Open);
   fprintf (Gbl.F.Out,"</tr>");

   if (Expanded)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" rowspan=\"3\" colspan=\"3\">" \
                         "<table cellspacing=\"0\" cellpadding=\"2\">");

      /***** Write course origin of message *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"left\">");
      FromThisCrs = Msg_WriteCrsOrgMsg (CrsCod);
      fprintf (Gbl.F.Out,"</td></tr>");

      /***** Form to reply message *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"left\">");
      switch (TypeOfMessages)
        {
         case Msg_MESSAGES_RECEIVED:
            Msg_WriteFormToReply (MsgCod,CrsCod,Subject,FromThisCrs,Replied,UsrDat.EncryptedUsrCod);
            break;
         case Msg_MESSAGES_SENT:
            break;
         default: // Not aplicable here
            break;
        }
      fprintf (Gbl.F.Out,"</td></tr></table></td>");

      /***** Write "From:" *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"MSG_TIT\">%s:&nbsp;</td>",
               Txt_MSG_From[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" colspan=\"2\">");
      Msg_WriteMsgFrom (&UsrDat,Deleted);
      fprintf (Gbl.F.Out,"</td></tr>");

      /***** Write "To:" *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"MSG_TIT\">%s:&nbsp;</td>",
               Txt_MSG_To[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" colspan=\"2\">");
      Msg_WriteMsgTo (TypeOfMessages,MsgCod);
      fprintf (Gbl.F.Out,"</td></tr>");

      /***** Write "Message:" *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"MSG_TIT\">%s:&nbsp;</td>",
               Txt_MSG_Message[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" colspan=\"2\" class=\"MSJ\"><tt>");
      Msg_GetMsgContent (MsgCod,Content);
      Msg_WriteMsgContent (Content,Cte_MAX_BYTES_TEXT,true,false);
      fprintf (Gbl.F.Out,"</tt><br />&nbsp;</td></tr>");
     }

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDat);
  }

/*****************************************************************************/
/************************** Write number of message **************************/
/*****************************************************************************/

void Msg_WriteMsgNumber (unsigned long MsgNum,bool NewMsg)
  {
   fprintf (Gbl.F.Out,"<td width=\"36\" align=\"center\" valign=\"top\" class=\"%s\">%lu:</td>",
	    NewMsg ? "MSG_TIT_BG_NEW" : "MSG_TIT_BG",MsgNum);
  }

/*****************************************************************************/
/******************** Write subject of a received message ********************/
/*****************************************************************************/

static void Msg_WriteSentOrReceivedMsgSubject (Msg_TypeOfMessages_t TypeOfMessages,long MsgCod,const char *Subject,bool Open,bool Expanded)
  {
   extern const char *Txt_no_subject[Txt_NUM_LANGUAGES];

   /***** Start cell *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">",
            Open ? "MSG_TIT_BG" : "MSG_TIT_BG_NEW");

   /***** Start form to expand the message *****/
   if (!Expanded)
     {
      Act_FormStart ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? ActExpRecMsg : ActExpSntMsg);
      Msg_PutHiddenParamsMsgsFilters ();
      Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
      Msg_PutHiddenParamMsgCod (MsgCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,"Ver mensaje",Open ? "MSG_TIT_BG" : "MSG_TIT_BG_NEW");
     }

   /***** Write subject *****/
   if (Subject[0])
      fprintf (Gbl.F.Out,"%s",Subject);
   else
      fprintf (Gbl.F.Out,"[%s]",Txt_no_subject[Gbl.Prefs.Language]);

   /***** End form to expand the message *****/
   if (!Expanded)
      fprintf (Gbl.F.Out,"</acronym></a></form>");

   /***** End cell *****/
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/************************ Write the author of a message **********************/
/*****************************************************************************/
// Input: UsrDat must hold user's data

void Msg_WriteMsgAuthor (struct UsrData *UsrDat,unsigned WidthOfNameColumn,unsigned MaxCharsInName,const char *Style,const char *AnchorStyle,bool Enabled,const char *BgColor)
  {
   bool ShowPhoto = false,UsrHasWebPage;
   char PhotoURL[PATH_MAX+1];
   bool WriteAuthor = false;

   /***** Start first column *****/
   fprintf (Gbl.F.Out,"<td width=\"18\" align=\"center\" valign=\"top\"");
   if (BgColor)
      fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
   fprintf (Gbl.F.Out," class=\"%s\">",Style);

   /***** Write author name or don't write it? *****/
   if (Enabled)
      if (UsrDat->UsrCod >= 0)
         WriteAuthor = true;

   if (WriteAuthor)
     {
      /***** First column with author's photo (if author has a web page, put a link to it) *****/
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      UsrHasWebPage = Str_URLLooksValid (UsrDat->WWW);
      if (UsrHasWebPage)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
                  UsrDat->WWW,AnchorStyle);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,true);
      if (UsrHasWebPage)
         fprintf (Gbl.F.Out,"</a>");
      fprintf (Gbl.F.Out,"</td>");

      /***** Second column with user name (if author has a web page, put a link to it) *****/
      fprintf (Gbl.F.Out,"<td width=\"%u\" align=\"left\" valign=\"top\" class=\"%s\"",
               WidthOfNameColumn,Style);
      if (BgColor)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
      fprintf (Gbl.F.Out,">");
      if (UsrHasWebPage)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
                  UsrDat->WWW,AnchorStyle);

      /* Restrict length of firstname and surnames */
      Usr_RestrictLengthAndWriteName (UsrDat,MaxCharsInName);

      if (UsrHasWebPage)
         fprintf (Gbl.F.Out,"</a>");
     } 
   else
     {
      fprintf (Gbl.F.Out,"<img src=\"%s/usr_bl.jpg\" class=\"F18x24\" /></td>" \
                         "<td align=\"left\" width=\"90\" class=\"%s\"",
               Gbl.Prefs.IconsURL,Style);
      if (BgColor)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",BgColor);
      fprintf (Gbl.F.Out,">&nbsp;");
     }

   /***** End seconf column *****/
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/********************* Write course origin of a message **********************/
/*****************************************************************************/
// Returns true if the origin course is the current course

bool Msg_WriteCrsOrgMsg (long CrsCod)
  {
   extern const char *Txt_from_this_BR_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Go_to_X[Txt_NUM_LANGUAGES];
   struct Course Crs;
   bool FromThisCrs = true;
   bool ThereIsOrgCrs = false;

   if (CrsCod >= 0)
     {
      /* Get new course code from old course code */
      Crs.CrsCod = CrsCod;

      /* Get data of current degree */
      if (Deg_GetDataOfCourseByCod (&Crs))
        {
         ThereIsOrgCrs = true;
         if ((FromThisCrs = (CrsCod == Gbl.CurrentCrs.Crs.CrsCod)))	// Message sent from current course
             fprintf (Gbl.F.Out,"<span class=\"MSG_AUT\">(%s)</span>",
                      Txt_from_this_BR_course[Gbl.Prefs.Language]);
         else	// Message sent from another course
            {
             /* Write course, including link */
             Act_FormStart (ActMnu);
             fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CrsCod\" value=\"%ld\" />",Crs.CrsCod);
             fprintf (Gbl.F.Out,"<span class=\"MSG_AUT\">(");
             sprintf (Gbl.MsgWindowStatus,Txt_Go_to_X[Gbl.Prefs.Language],Crs.FullName);
             Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"MSG_AUT");
             fprintf (Gbl.F.Out,"%s</acronym></a>)</span></form>",Crs.ShortName);
            }
	}
     }
   if (!ThereIsOrgCrs)	// It's an old message without origin source specified, or is a message sent from none course
      fprintf (Gbl.F.Out,"<span class=\"MSG_AUT\">(sin asignatura<br />origen)</span>");

   return FromThisCrs;
  }

/*****************************************************************************/
/************************* Write form to reply a message *********************/
/*****************************************************************************/

static void Msg_WriteFormToReply (long MsgCod,long CrsCod,const char *Subject,bool FromThisCrs,bool Replied,const char EncryptedUsrCod[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1])
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Reply[Txt_NUM_LANGUAGES];
   extern const char *Txt_Reply_BR_again[Txt_NUM_LANGUAGES];
   extern const char *Txt_Go_to_BR_course_BR_and_reply[Txt_NUM_LANGUAGES];
   extern const char *Txt_Go_to_BR_course_BR_and_reply_BR_again[Txt_NUM_LANGUAGES];

   /***** Form start and parameters *****/
   Act_FormStart (ActReqMsgUsr);
   if (!FromThisCrs)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CrsCod\" value=\"%ld\" />",CrsCod);
   Grp_PutParamAllGroups ();
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"IsReply\" value=\"Y\" />");
   Msg_PutHiddenParamMsgCod (MsgCod);
   Usr_PutParamOtherUsrCodEncrypted (EncryptedUsrCod);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrCodAll\" value=\"%s\" />",EncryptedUsrCod);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Subject\" value=\"Re: %s\" />",Subject);

   /****** Link and form end *****/
   if (FromThisCrs)
     {
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Reply[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,Replied ? Txt_Reply_BR_again[Gbl.Prefs.Language] :
                                   Txt_Reply[Gbl.Prefs.Language]);
     }
   else
     {
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Reply[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,Replied ? Txt_Go_to_BR_course_BR_and_reply_BR_again[Gbl.Prefs.Language] :
                                   Txt_Go_to_BR_course_BR_and_reply[Gbl.Prefs.Language]);

     }
   fprintf (Gbl.F.Out,"</acronym></a></form>");
  }

/*****************************************************************************/
/************************** Write author of a message ************************/
/*****************************************************************************/

static void Msg_WriteMsgFrom (struct UsrData *UsrDat,bool Deleted)
  {
   bool ShowPhoto;
   char PhotoURL[PATH_MAX+1];

   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");

   /***** Put an icon to show if user has read the message *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" width=\"12\">" \
                      "<acronym title=\"%s\">" \
                      "<img src=\"%s/%s12x12.gif\" alt=\"\" />" \
                      "</acronym></td>",
            Deleted ? "Enviado y eliminado" : "Enviado",Gbl.Prefs.IconsURL,
            Deleted ? "msg_fwd_del" : "msg_fwd");

   /***** Put user's photo *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" width=\"16\">");
   ShowPhoto = (Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL));
   Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,12,16,true);

   /***** Write user's name *****/
   fprintf (Gbl.F.Out,"</td><td align=\"left\" class=\"MSG_AUT\">");
   if (UsrDat->UsrCod >= 0)
      fprintf (Gbl.F.Out,"%s",UsrDat->FullName);
   else
      fprintf (Gbl.F.Out,"[desconocido]");	// User not found, likely an old user who has been removed
   fprintf (Gbl.F.Out,"</td></tr></table>");
  }

/*****************************************************************************/
/******************** Write list of recipients of a message ******************/
/*****************************************************************************/

#define MAX_RECIPIENTS_TO_SHOW 10	// If number of recipients <= MAX_RECIPIENTS_TO_SHOW, show all recipients
#define DEFAULT_RECIPIENTS_TO_SHOW 5	// If number of recipients > MAX_RECIPIENTS_TO_SHOW, show only DEFAULT_RECIPIENTS_TO_SHOW

static void Msg_WriteMsgTo (Msg_TypeOfMessages_t TypeOfMessages,long MsgCod)
  {
   extern const char *Txt_unknown_recipient[Txt_NUM_LANGUAGES];
   extern const char *Txt_unknown_recipients[Txt_NUM_LANGUAGES];
   extern const char *Txt_View_all_recipients[Txt_NUM_LANGUAGES];
   extern const char *Txt_and_X_other_recipients[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRcp;
   unsigned NumRecipientsTotal;
   unsigned NumRecipientsToShow;
   char YN[1+1];
   struct UsrData UsrDat;
   bool Deleted;
   bool OpenByDst;
   bool UsrValid;
   bool ShowPhoto;
   char PhotoURL[PATH_MAX+1];

   /***** Get recipients of a message from database *****/
   sprintf (Query,"SELECT UsrCod,Deleted,Open FROM msg_received" \
                  " WHERE MsgCod='%ld'",MsgCod);
   NumRecipientsTotal = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get recipients of a message");

   /***** Check number of recipients *****/
   if (NumRecipientsTotal)
     {
      /***** Start table *****/
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

      /***** How many recipients will be shown? *****/
      if (NumRecipientsTotal <= MAX_RECIPIENTS_TO_SHOW)
         NumRecipientsToShow = NumRecipientsTotal;
      else	// A lot of recipients
        {
         /***** Get parameter that indicates if I want to see all recipients *****/
         Par_GetParam (Par_PARAM_SINGLE,"SeeAllRcpts",YN,1);
         NumRecipientsToShow = (Str_ConvertToUpperLetter (YN[0]) == 'Y') ? NumRecipientsTotal : DEFAULT_RECIPIENTS_TO_SHOW;
        }
      
      /***** Initialize structure with user's data *****/
      Usr_UsrDataConstructor (&UsrDat);

      /***** Write recipients *****/
      for (NumRcp=0; NumRcp<NumRecipientsToShow; NumRcp++)
        {
         /* Get user's code */
         row = mysql_fetch_row (mysql_res);
         UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);

         /* Get if message has been deleted by recipient */ 
         Deleted = (Str_ConvertToUpperLetter (row[1][0]) != 'N');

         /* Get if message has been read by recipient */ 
         OpenByDst = (Str_ConvertToUpperLetter (row[2][0]) != 'N');

         /* Get user's data */
	 UsrValid = Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat);

         /* Put an icon to show if user has read the message */
         fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" width=\"12\">" \
                            "<acronym title=\"%s\">" \
                            "<img src=\"%s/%s12x12.gif\" alt=\"\" />" \
                            "</acronym></td>",
                  OpenByDst ? (Deleted ? "Abierto y eliminado" : "Abierto") :
                              (Deleted ? "Eliminado sin abrir" : "Sin abrir"),Gbl.Prefs.IconsURL,
                  OpenByDst ? (Deleted ? "msg_open_del" : "msg_open") :
                              (Deleted ? "msg_unread_del" : "msg_unread"));

         /* Put user's photo */
         fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" width=\"16\">");
         ShowPhoto = (UsrValid ? Usr_ShowUsrPhotoIsAllowed (&UsrDat,PhotoURL) : false);
         Usr_ShowUsrPhoto (&UsrDat,ShowPhoto ? PhotoURL : NULL,12,16,true);

         /* Write user's name */
         fprintf (Gbl.F.Out,"</td><td align=\"left\" class=\"MSG_AUT\">");
         if (UsrDat.UsrCod >= 0)
           {
	    if (UsrValid)
               fprintf (Gbl.F.Out,"%s",UsrDat.FullName);
            else
               fprintf (Gbl.F.Out,"[%s]",Txt_unknown_recipient[Gbl.Prefs.Language]);	// User not found, likely an old user who has been removed
           }
         else
            fprintf (Gbl.F.Out,"[%s]",Txt_unknown_recipients[Gbl.Prefs.Language]); 	// Users stored in database with the old system (a list of names)
         fprintf (Gbl.F.Out,"</td></tr>");
        }

      /***** If any recipients are not listed *****/
      if (NumRecipientsToShow < NumRecipientsTotal)
        {
         /***** Start form to show all the users *****/
         fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"3\" class=\"MSG_AUT\">");
         Act_FormStart ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? ActSeeRecMsg : ActSeeSntMsg);
         Msg_PutHiddenParamsMsgsFilters ();
         Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
         Msg_PutHiddenParamMsgCod (MsgCod);
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"SeeAllRcpts\" value=\"Y\" />");
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_View_all_recipients[Gbl.Prefs.Language],"MSG_AUT");
         fprintf (Gbl.F.Out,Txt_and_X_other_recipients[Gbl.Prefs.Language],
                  NumRecipientsTotal - NumRecipientsToShow);
         fprintf (Gbl.F.Out,"</acronym></a></form></td></tr>");
        }

      /***** Free memory used for user's data *****/
      Usr_UsrDataDestructor (&UsrDat);

      /***** End table *****/
      fprintf (Gbl.F.Out,"</table>");
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/******************* Write the date of creacin of a message ****************/
/*****************************************************************************/

void Msg_WriteMsgDate (const char *DateTime,bool NewMsg)
  {
   /***** Start cell *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" width=\"85\" class=\"%s\">",
            NewMsg ? "MSG_TIT_BG_NEW" : "MSG_TIT_BG");

   /***** Write date and time (DateTime holds data and time in YYYYMMDDHHMMSS format) *****/
   Dat_WriteDate (DateTime);
   fprintf (Gbl.F.Out,"&nbsp;");
   Dat_WriteHourMinute (&DateTime[8]);
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/************* Put a form to delete a received or sent message ***************/
/*****************************************************************************/

static void Msg_PutFormToDeleteMessage (unsigned long MsgNum,long MsgCod,Msg_TypeOfMessages_t TypeOfMessages)
  {
   extern const char *Txt_Remove_message[Txt_NUM_LANGUAGES];
  	
   fprintf (Gbl.F.Out,"<td width=\"18\" align=\"center\" valign=\"top\">");
   Act_FormStart ((TypeOfMessages == Msg_MESSAGES_RECEIVED) ? ActDelRecMsg : ActDelSntMsg);
   Pag_PutHiddenParamPagNum (Gbl.Pag.CurrentPage);
   Msg_PutHiddenParamMsgCod (MsgCod);
   Msg_PutHiddenParamsMsgsFilters ();
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_message[Gbl.Prefs.Language],NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\"/></acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Txt_Remove_message[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/******************** Write the texto (content) of a message *****************/
/*****************************************************************************/

void Msg_WriteMsgContent (char *Content,unsigned long MaxLength,bool InsertLinks,bool CambiarBRporRet)
  {
   /***** Insert links in URLs *****/
   if (InsertLinks)
      Str_InsertLinkInURLs (Content,MaxLength,60);

   /***** Write message to file *****/
   if (CambiarBRporRet)
      Str_FilePrintStrChangingBRToRet (Gbl.F.Out,Content);
   else
      fprintf (Gbl.F.Out,Content);
  }

/*****************************************************************************/
/*********************** Indent forum or chat title **************************/
/*****************************************************************************/

void Msg_IndentDependingOnLevel (int Level,bool IsLastItemInLevel[])
  {
   int i;

   for (i=1; i<Level; i++)
      fprintf (Gbl.F.Out,"<td width=\"16\"><img src=\"%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"\" /></td>",
               Gbl.Prefs.IconsURL,IsLastItemInLevel[i] ? "tr" : "subleft");
   fprintf (Gbl.F.Out,"<td width=\"16\"><img src=\"%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"\" /></td>",
            Gbl.Prefs.IconsURL,IsLastItemInLevel[Level] ? "subend" : "submid");
  }

/*****************************************************************************/
/*************** Get parameter with the code of a message ********************/
/*****************************************************************************/

static void Msg_PutHiddenParamMsgCod (long MsgCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"MsgCod\" value=\"%ld\" />",MsgCod);
  }

/*****************************************************************************/
/*************** Get parameter with the code of a message ********************/
/*****************************************************************************/

static long Msg_GetParamMsgCod (void)
  {
   char LongStr[1+10+1];	// String that holds the message code
   long MsgCod;

   /* Get message code */
   Par_GetParam (Par_PARAM_SINGLE,"MsgCod",LongStr,1+10);
   if (sscanf (LongStr,"%ld",&MsgCod) != 1)
      Lay_ShowErrorAndExit ("Wrong code of message.");

   return MsgCod;
  }

/*****************************************************************************/
/*********************** Write a link to netiquette rules ********************/
/*****************************************************************************/

void Msg_WriteLinkToNetiquette (void)
  {
   extern const char *Txt_Mandatory_rules_to_compose_messages[Txt_NUM_LANGUAGES];
   char Title[1024];

   sprintf (Title,"<a href=\"http://swad.ugr.es/wiki/index.php/Netiquette\" target=\"_blank\">%s</a>",
            Txt_Mandatory_rules_to_compose_messages[Gbl.Prefs.Language]);
   Lay_WriteTitle (Title);
  }
