// swad_not.c: notices (yellow notes)

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <string.h>

#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_not.h"
#include "swad_par.h"
#include "swad_ntf.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Not_DrawANotice (Not_TypeNoticesListing_t TypeNoticesListing,bool ICanEditNotices,long NotCod,const char *DateTime,const char *Content,long UsrCod,Not_NoticeStatus_t NoticeStatus);
static long Not_InsertNotice (const char *Content);
static void Not_UpdateNumUsrsNotifiedAboutNotice (long NotCod,unsigned NumUsrsToBeNotified);
static void Not_PutHiddenParamNotCod (long NotCod);
static long Not_GetParamNotCod (void);

/*****************************************************************************/
/***************************** Write a new notice ****************************/
/*****************************************************************************/

void Not_ShowFormNotice (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_notice[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_Message[Txt_NUM_LANGUAGES];
   extern const char *Txt_Send_notice[Txt_NUM_LANGUAGES];

   sprintf (Gbl.Message,"El aviso que escriba aqu&iacute; aparecer&aacute; en forma de <EM>Post-it</EM>" \
                        " a la izquierda de la p&aacute;gina y ser&aacute; <strong>visible por todo el mundo</strong>" \
                        " (no s&oacute;lo por los usuarios de %s).<br />" \
                        "&nbsp;&nbsp;&nbsp;<strong>Se aconseja que el mensaje sea breve</strong>." \
                        " Si desea crear un aviso extenso o de acceso exclusivo para sus alumnos," \
                        " puede usar el foro de la asignatura, enviar un mensaje a todos" \
                        " a trav&eacute;s del sistema de mensajer&iacute;a interna," \
                        " o incluso crear un archivo con el texto y a&ntilde;adirlo a &quot;descargas&quot;.",	// Need translation!
            Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (INFO,Gbl.Message);

   Msg_WriteLinkToNetiquette ();

   Act_FormStart (ActRecNot);
   fprintf (Gbl.F.Out,"<div align=\"center\"><table><tr><td align=\"left\"></td><td align=\"left\" class=\"%s\">%s</td></tr>" \
                      "<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\"><textarea name=\"Txt\" cols=\"30\" rows=\"10\"></textarea></td></tr></table></div>\n",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_New_notice[Gbl.Prefs.Language],
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_MSG_Message[Gbl.Prefs.Language]);
   Lay_PutSendButton (Txt_Send_notice[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/*****************************************************************************/
/******* Receive a new notice from a form and store it in database ***********/
/*****************************************************************************/

void Not_ReceiveNotice (void)
  {
   extern const char *Txt_Notice_created[Txt_NUM_LANGUAGES];
   long NotCod;
   unsigned NumUsrsToBeNotified;

   /***** Get the text of the notice *****/
   Str_AllocateAndGetText (Str_TO_RIGOROUS_HTML,true);

   /***** Create a new notice *****/
   NotCod = Not_InsertNotice (Gbl.Txt);

   /***** Free memory used for the text of the notice *****/
   Str_FreeTxt ();

   /***** Write message of success *****/
   Lay_ShowAlert (INFO,Txt_Notice_created[Gbl.Prefs.Language]);

   /***** Notify by e-mail about the new notice *****/
   if ((NumUsrsToBeNotified = Ntf_StoreNotifyEventsToAllUsrs (Ntf_NEW_NOTICE,NotCod,-1L)))
      Not_UpdateNumUsrsNotifiedAboutNotice (NotCod,NumUsrsToBeNotified);
   Ntf_ShowAlertNumUsrsToBeNotified (NumUsrsToBeNotified);
  }

/*****************************************************************************/
/******************* Insert a notice in the table of notices *****************/
/*****************************************************************************/
// Return the code of the new inserted notice

static long Not_InsertNotice (const char *Content)
  {
   char Query[512+Cte_MAX_BYTES_TEXT];
   char SubQuery[512];

   /***** Insert notice in the database *****/
   sprintf (Query,"INSERT INTO notices (CrsCod,UsrCod,CreatTime,Content,Status)" \
                  " VALUES ('%ld','%ld',NOW(),'",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   Str_AddStrToQuery (Query,Content,sizeof (Query));
   sprintf (SubQuery,"','%u')",(unsigned) Not_ACTIVE_NOTICE);
   strcat (Query,SubQuery);
   return DB_QueryINSERTandReturnCode (Query,"can not create notice");
  }

/****************************************************************************/
/********** Update number of users notified in table of notices *************/
/****************************************************************************/

static void Not_UpdateNumUsrsNotifiedAboutNotice (long NotCod,unsigned NumUsrsToBeNotified)
  {
   char Query[512];

   /***** Update number of users notified *****/
   sprintf (Query,"UPDATE notices SET NumNotif='%u' WHERE NotCod='%ld'",
            NumUsrsToBeNotified,NotCod);
   DB_QueryUPDATE (Query,"can not update the number of notifications");
  }

/****************************************************************************/
/****************************** List notices ********************************/
/****************************************************************************/

void Not_ListNotices (void)
  {
   Not_ShowNotices (Not_LIST_NOTICES_TO_EDIT);
  }

/*****************************************************************************/
/********************************* Hide a notice *****************************/
/*****************************************************************************/

void Not_HideNotice (void)
  {
   char Query[512];
   long NotCod;

   /***** Get the code of the notice to hide *****/
   NotCod = Not_GetParamNotCod ();

   /***** Set notice as hidden *****/
   sprintf (Query,"UPDATE notices SET Status='%u'" \
                  " WHERE NotCod='%ld' AND CrsCod='%ld'",
            (unsigned) Not_OBSOLETE_NOTICE,
            NotCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryUPDATE (Query,"can not hide notice");
  }

/*****************************************************************************/
/********************* Show a notice that was hidden *************************/
/*****************************************************************************/

void Not_ShowNotice (void)
  {
   char Query[512];
   long NotCod;

   /***** Get the code of the notice to show *****/
   NotCod = Not_GetParamNotCod ();

   /***** Set notice as shown *****/
   sprintf (Query,"UPDATE notices SET Status='%u'" \
                  " WHERE NotCod='%ld' AND CrsCod='%ld'",
            (unsigned) Not_ACTIVE_NOTICE,
            NotCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryUPDATE (Query,"can not show notice");
  }

/*****************************************************************************/
/******************************* Delete a notice *****************************/
/*****************************************************************************/

void Not_DeleteNotice (void)
  {
   char Query[512];
   long NotCod;

   /***** Get the code of the notice to delete *****/
   NotCod = Not_GetParamNotCod ();

   /***** Mark notice as deleted *****/
   sprintf (Query,"UPDATE notices SET Status='%u'" \
                  " WHERE NotCod='%ld' AND CrsCod='%ld'",
            (unsigned) Not_DELETED_NOTICE,
            NotCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryUPDATE (Query,"can not remove notice");

   /***** Delete pending notification of this notice *****/
   Ntf_DeletePendingNotif (Ntf_NEW_NOTICE,NotCod);
  }

/****************************************************************************/
/******************************* Show the notices ***************************/
/****************************************************************************/

void Not_ShowNotices (Not_TypeNoticesListing_t TypeNoticesListing)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_No_notices[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_notice[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   long NotCod;
   unsigned long NumNot,NumNotices;
   char Content[Cte_MAX_BYTES_TEXT+1];
   long UsrCod;
   unsigned UnsignedNum;
   Not_NoticeStatus_t NoticeStatus;
   bool ICanEditNotices = (TypeNoticesListing == Not_LIST_NOTICES_TO_EDIT &&
                          Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER);

   /***** Get notices from database *****/
   switch (TypeNoticesListing)
     {
      case Not_LIST_NOTICES_TO_SEE:
         sprintf (Query,"SELECT NotCod,DATE_FORMAT(CreatTime,'%%Y%%m%%d%%H%%i') AS F,UsrCod,Content,Status" \
                        " FROM notices WHERE CrsCod='%ld' AND Status='%u' ORDER BY CreatTime DESC",
                  Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Not_ACTIVE_NOTICE);
         break;
      case Not_LIST_NOTICES_TO_EDIT:
         sprintf (Query,"SELECT NotCod,DATE_FORMAT(CreatTime,'%%Y%%m%%d%%H%%i') AS F,UsrCod,Content,Status" \
                        " FROM notices WHERE CrsCod='%ld' AND Status<>'%u' ORDER BY CreatTime DESC",
                  Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Not_DELETED_NOTICE);
     }
   NumNotices = DB_QuerySELECT (Query,&mysql_res,"can not get notices from database");

   if (TypeNoticesListing == Not_LIST_NOTICES_TO_EDIT && !NumNotices)
      Lay_ShowAlert (WARNING,Txt_No_notices[Gbl.Prefs.Language]);

   if (TypeNoticesListing == Not_LIST_NOTICES_TO_EDIT && (NumNotices || ICanEditNotices))
     {
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
      fprintf (Gbl.F.Out,"<tr><td align=\"center\">\n");
      if (ICanEditNotices)
        {
         fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\">");
         Act_FormStart (ActWriNot);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_New_notice[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/new12x12.gif\" width=\"12\" height=\"12\"" \
                            " alt=\"%s\" />" \
                            " %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,
                  Txt_New_notice[Gbl.Prefs.Language],
                  Txt_New_notice[Gbl.Prefs.Language]);
        }
     }

   /***** Show the notices *****/
   if (NumNotices)
     {
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"120\">\n");
      for (NumNot=0; NumNot<NumNotices; NumNot++)
        {
         row = mysql_fetch_row (mysql_res);

         /* Get notice code */
         if (sscanf (row[0],"%ld",&NotCod) != 1)
            Lay_ShowErrorAndExit ("Wrong code of notice.");

         /* Get user code */
         UsrCod = Str_ConvertStrCodToLongCod (row[2]);

         /* Get the content and insert links */
         strncpy (Content,row[3],Cte_MAX_BYTES_TEXT);
         Str_InsertLinkInURLs (Content,Cte_MAX_BYTES_TEXT,18);

         /* Get status of the notice */
         NoticeStatus = Not_DELETED_NOTICE;
         if (sscanf (row[4],"%u",&UnsignedNum) == 1)
            if (UnsignedNum < Not_NUM_STATUS_NOTICE)
              NoticeStatus = (Not_NoticeStatus_t) UnsignedNum;

         /* Draw the notice */
         Not_DrawANotice (TypeNoticesListing,ICanEditNotices,NotCod,row[1],Content,UsrCod,NoticeStatus);
        }
      fprintf (Gbl.F.Out,"</table>\n");
     }

   if (TypeNoticesListing == Not_LIST_NOTICES_TO_EDIT && (NumNotices || ICanEditNotices))
     {
      fprintf (Gbl.F.Out,"</td></tr>\n");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/****************** Draw a notice as a yellow post-it ***********************/
/****************************************************************************/

static void Not_DrawANotice (Not_TypeNoticesListing_t TypeNoticesListing,bool ICanEditNotices,long NotCod,const char *DateTime,const char *Content,long UsrCod,Not_NoticeStatus_t NoticeStatus)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_NOTICE_Obsolete_SINGULAR[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTICE_Obsolete_Mark_as_active[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTICE_Active_SINGULAR[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTICE_Active_Mark_as_obsolete[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_notice[Txt_NUM_LANGUAGES];
   struct UsrData UsrDat;

   if (NoticeStatus == Not_DELETED_NOTICE)
      return;

   /***** Write the date in the top part of the post-it *****/
   /* Left */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_arriba_izquierda_10x100.png');\">" \
                      "<img src=\"%s/tr10x32.gif\" width=\"10\" height=\"32\" /></td>",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);
   /* Centre */
   fprintf (Gbl.F.Out,"<td width=\"12\" valign=\"top\" align=\"left\" style=\"background-image: url('%s/post-it_arriba_12x100.png');\" class=\"FECHA_AV\">" \
                      "<img src=\"%s/tr12x10.gif\" width=\"12\" height=\"10\" /><br />",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);
   /* Write symbol to indicate if notice is obsolete or active */
   if (TypeNoticesListing == Not_LIST_NOTICES_TO_EDIT)
     {
      if (ICanEditNotices)	// Put form to change the state of the notice
        {
         switch (NoticeStatus)
           {
            case Not_ACTIVE_NOTICE:
               Act_FormStart (ActHidNot);
               Not_PutHiddenParamNotCod (NotCod);
               Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_NOTICE_Active_Mark_as_obsolete[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
               fprintf (Gbl.F.Out,"<img src=\"%s/pinon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                                  "</acronym></a></form>",
                        Gbl.Prefs.IconsURL,Txt_NOTICE_Active_Mark_as_obsolete[Gbl.Prefs.Language]);
               break;
            case Not_OBSOLETE_NOTICE:
               Act_FormStart (ActShoNot);
               Not_PutHiddenParamNotCod (NotCod);
               Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_NOTICE_Obsolete_Mark_as_active[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
               fprintf (Gbl.F.Out,"<img src=\"%s/pinoff12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                                  "</acronym></a></form>",
                        Gbl.Prefs.IconsURL,Txt_NOTICE_Obsolete_Mark_as_active[Gbl.Prefs.Language]);
               break;
            case Not_DELETED_NOTICE:
               break;
           }
        }
      else			// Don't put form to change the state of the notice
        {
         switch (NoticeStatus)
           {
            case Not_ACTIVE_NOTICE:
               fprintf (Gbl.F.Out,"<acronym title=\"%s\">" \
                                  "<img src=\"%s/pinon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                                  "</acronym>",
                        Txt_NOTICE_Active_SINGULAR[Gbl.Prefs.Language],Gbl.Prefs.IconsURL,Txt_NOTICE_Active_SINGULAR[Gbl.Prefs.Language]);
               break;
            case Not_OBSOLETE_NOTICE:
               fprintf (Gbl.F.Out,"<acronym title=\"%s\">" \
                                  "<img src=\"%s/pinoff12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                                  "</acronym>",
                        Txt_NOTICE_Obsolete_SINGULAR[Gbl.Prefs.Language],Gbl.Prefs.IconsURL,Txt_NOTICE_Obsolete_SINGULAR[Gbl.Prefs.Language]);
               break;
            case Not_DELETED_NOTICE:
               break;
           }
        }
     }
   fprintf (Gbl.F.Out,"</td>" \
                      "<td width=\"88\" valign=\"top\" align=\"right\" style=\"background-image: url('%s/post-it_arriba_88x100.png');\" class=\"FECHA_AV\">" \
                      "<img src=\"%s/tr88x10.gif\" width=\"88\" height=\"10\" /><br />",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);
   /* Write the date (DateTime is in YYYYMMDDHHMM format) */
   Dat_WriteDate (DateTime);
   fprintf (Gbl.F.Out,"&nbsp;");
   Dat_WriteHourMinute (&DateTime[8]);
   fprintf (Gbl.F.Out,"</td>");

   /* Right */
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_arriba_derecha_10x100.png');\">" \
                      "<img src=\"%s/tr10x32.gif\" width=\"10\" height=\"32\" /></td></tr>\n",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);

   /***** Write the content of the notice *****/
   /* Left */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_izquierda_10x1.png'); background-repeat: repeat-y;\"></td>",
            Gbl.Prefs.IconsURL);
   /* Centre */
   fprintf (Gbl.F.Out,"<td colspan=\"2\" align=\"left\" width=\"100\" valign=\"top\" style=\"background-image: url('%s/post-it_centro_100x1.png'); background-repeat: repeat-y;\" bgcolor=\"#F9EE8E\">" \
                      "<div style=\"width:100px; overflow:hidden\" class=\"AVISO\">",
            Gbl.Prefs.IconsURL);
   fprintf (Gbl.F.Out,Content);
   fprintf (Gbl.F.Out,"</div></td>");
   /* Right */
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_derecha_10x1.png'); background-repeat: repeat-y;\"></td></tr>\n",
            Gbl.Prefs.IconsURL);

   /***** Write the author *****/
   /* Left */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_izquierda_10x1.png'); background-repeat: repeat-y;\"></td>",
            Gbl.Prefs.IconsURL);
   /* Centre */
   fprintf (Gbl.F.Out,"<td colspan=\"2\" width=\"100\" valign=\"top\" align=\"right\" bgcolor=\"#FAF098\"" \
                      " style=\"background-image: url('%s/post-it_centro_100x1.png'); background-repeat: repeat-y;\"" \
                      " class=\"AUTOR_AV\">",
            Gbl.Prefs.IconsURL);
   Usr_UsrDataConstructor (&UsrDat);
   UsrDat.UsrCod = UsrCod;
   if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat)) // Get from the database the data of the autor
      Usr_RestrictLengthAndWriteName (&UsrDat,18);
   Usr_UsrDataDestructor (&UsrDat);
   fprintf (Gbl.F.Out,"</td>");
   /* Right */
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_derecha_10x1.png'); background-repeat: repeat-y;\"></td></tr>\n",
            Gbl.Prefs.IconsURL);

   /***** Write form to delete this notice *****/
   if (ICanEditNotices)
     {
      /* Left */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_izquierda_10x1.png'); background-repeat: repeat-y;\"></td>",
               Gbl.Prefs.IconsURL);
      /* Centre */
      fprintf (Gbl.F.Out,"<td colspan=\"2\" align=\"center\" valign=\"top\" width=\"100\" bgcolor=\"#FAF098\" class=\"%s\"" \
                         " style=\"background-image: url('%s/post-it_centro_100x1.png'); background-repeat: repeat-y;\"><br />",
               The_ClassFormul[Gbl.Prefs.Theme],Gbl.Prefs.IconsURL);

      /* Form to delete notice */
      Act_FormStart (ActRemNot);
      Not_PutHiddenParamNotCod (NotCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_notice[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                         " %s</acronym></a></form>",
               Gbl.Prefs.IconsURL,Txt_Remove_notice[Gbl.Prefs.Language],Txt_Remove[Gbl.Prefs.Language]);

      fprintf (Gbl.F.Out,"</td>");

      /* Right */
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"10\" valign=\"top\" style=\"background-image: url('%s/post-it_derecha_10x1.png'); background-repeat: repeat-y;\"></td></tr>\n",
               Gbl.Prefs.IconsURL);
     }

   /***** Bottom part of the post-it *****/
   /* Left, centre and right */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"120\" colspan=\"4\" valign=\"top\" style=\"background-image: url('%s/post-it_abajo_120x15.png');\">" \
                      "<img src=\"%s/tr120x15.gif\" width=\"120\" height=\"15\" /></td></tr>\n",
            Gbl.Prefs.IconsURL,Gbl.Prefs.IconsURL);
  }


/*****************************************************************************/
/*************************** Get number of notices ***************************/
/*****************************************************************************/
// Returns the number of (active or obsolete) notices
// sent from this location (all the platform, current degree or current course)

unsigned Not_GetNumNotices (Sta_LocationRange_t LocationRange,Not_NoticeStatus_t NoticeStatus,unsigned *NumNotif)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumNotices;

   /***** Get number of notices from database *****/
   switch (LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT COUNT(*),SUM(NumNotif) FROM notices" \
                        " WHERE Status='%u'",NoticeStatus);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT COUNT(*),SUM(notices.NumNotif) FROM notices,courses" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=notices.CrsCod" \
                        " AND notices.Status='%u'",
                  Gbl.CurrentDeg.Deg.DegCod,NoticeStatus);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         sprintf (Query,"SELECT COUNT(*),SUM(NumNotif) FROM notices" \
                        " WHERE CrsCod='%ld'" \
                        " AND Status='%u'",
                  Gbl.CurrentCrs.Crs.CrsCod,NoticeStatus);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of notices");

   /***** Get number of notices *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumNotices) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of notices.");

   /***** Get number of notifications by e-mail *****/
   if (row[1])
     {
      if (sscanf (row[1],"%u",NumNotif) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of notifications of notices.");
     }
   else
      *NumNotif = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumNotices;
  }

/*****************************************************************************/
/*************** Get parameter with the code of a notice *********************/
/*****************************************************************************/

static void Not_PutHiddenParamNotCod (long NotCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NotCod\" value=\"%ld\" />",NotCod);
  }

/*****************************************************************************/
/*************** Get parameter with the code of a notice *********************/
/*****************************************************************************/

static long Not_GetParamNotCod (void)
  {
   char LongStr[1+10+1];	// String that holds the notice code
   long NotCod;

   /* Get notice code */
   Par_GetParam (Par_PARAM_SINGLE,"NotCod",LongStr,1+10);
   if (sscanf (LongStr,"%ld",&NotCod) != 1)
      Lay_ShowErrorAndExit ("Wrong code of notice.");

   return NotCod;
  }
