// swad_ntf.c: notifications about events, sent by e-mail

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For system
#include <string.h>
#include <sys/wait.h>		// For the macro WEXITSTATUS
#include <unistd.h>		// For unlink

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_ntf.h"
#include "swad_par.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Tabs Tabs[Act_NUM_TABS];
extern struct Actions Actions[Act_NUM_ACTIONS];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

const char *Ntf_StrNotifyEvents[Ntf_NUM_NOTIFY_EVENTS] =
  {
   "NotifyNewCalls",
   "NotifyNewMarks",
   "NotifyNewNotices",
   "NotifyNewMsgs",
   "NotifyNewResponses",
  };

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Ntf_SendPendingNotifByEMailToOneUsr (struct UsrData *ToUsrDat,unsigned *NumNotif,unsigned *NumMails);
static void Ntf_GetNumNotifSent (long DegCod,long CrsCod,Ntf_NotifyEvent_t NotifyEvent,unsigned *NumEvents,unsigned *NumMails);
static void Ntf_UpdateNumNotifSent (long DegCod,long CrsCod,Ntf_NotifyEvent_t NotifyEvent,unsigned NumEvents,unsigned NumMails);
static void Ntf_GetParamsNotifyEvents (void);

/*****************************************************************************/
/***** Get a list with user's codes of all users in current course/group  ****/
/***** who wants to be notified by e-mail about an event, and notify them ****/
/*****************************************************************************/
// Return the number of users notified by e-mail

unsigned Ntf_StoreNotifyEventsToAllUsrs (Ntf_NotifyEvent_t NotifyEvent,long MsgCod,long ThrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData UsrDat;
   unsigned NumUsrsToBeNotified = 0;
   unsigned NotifyEventMask = (1 << NotifyEvent);

   /***** Get users who want to be notified from database ******/
   switch (NotifyEvent)
     {
      case Ntf_NEW_EXAM_ANNOUNCEMENT:
      case Ntf_NEW_NOTICE:
         sprintf (Query,"SELECT crs_usr.UsrCod FROM crs_usr,usr_data" \
                        " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod" \
                        " AND usr_data.UsrCod<>'%ld'" \
                        " AND ((usr_data.NotifyEvents & %u)<>0)",
                  Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod,NotifyEventMask);
         break;
      case Ntf_NEW_MARKS_FILE:
         switch (Gbl.FileBrowser.Type)
           {
            case Brw_FILE_BRW_ADMIN_MARKS_CRS:
               sprintf (Query,"SELECT crs_usr.UsrCod FROM crs_usr,usr_data" \
                              " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod" \
                              " AND usr_data.UsrCod<>'%ld'" \
                              " AND ((usr_data.NotifyEvents & %u)<>0)",
                        Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod,NotifyEventMask);
               break;
            case Brw_FILE_BRW_ADMIN_MARKS_GRP:
               sprintf (Query,"SELECT crs_grp_usr.UsrCod FROM crs_grp_usr,usr_data" \
                              " WHERE crs_grp_usr.GrpCod='%ld' AND crs_grp_usr.UsrCod=usr_data.UsrCod" \
                              " AND usr_data.UsrCod<>'%ld'" \
                              " AND ((usr_data.NotifyEvents & %u)<>0)",
                        Gbl.CurrentCrs.Grps.GrpCod,Gbl.Usrs.Me.UsrDat.UsrCod,NotifyEventMask);
               break;
            default:	// This function should not be called in other cases
               return 0;
           }
         break;
      case Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS:
         sprintf (Query,"SELECT DISTINCT forum_post.UsrCod FROM forum_post,usr_data" \
                        " WHERE forum_post.ThrCod='%ld'"
                        " AND forum_post.UsrCod<>'%ld' AND forum_post.UsrCod=usr_data.UsrCod" \
                        " AND ((usr_data.NotifyEvents & %u)<>0)",
                  ThrCod,Gbl.Usrs.Me.UsrDat.UsrCod,NotifyEventMask);
         break;
      default:	// This function should not be called in other cases
         return 0;
     }
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get users who want to be notified");

   if (NumRows) // Users found
     {
      /***** Initialize structure with user's data *****/
      Usr_UsrDataConstructor (&UsrDat);

      /***** Notify the users one by one *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
         /* Get next user */
         row = mysql_fetch_row (mysql_res);

         /* Get user code */
         UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);

	 if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat))		// Get user's data from the database
           {
            Ntf_StoreNotifyEventToOneUser (NotifyEvent,&UsrDat,MsgCod);
            NumUsrsToBeNotified++;
           }
        }

      /***** Free memory used for user's data *****/
      Usr_UsrDataDestructor (&UsrDat);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumUsrsToBeNotified;
  }

/*****************************************************************************/
/************** Store a notify event to one user into database ***************/
/*****************************************************************************/

void Ntf_StoreNotifyEventToOneUser (Ntf_NotifyEvent_t NotifyEvent,struct UsrData *UsrDat,long MsgCod)
  {
   char Query[512];

   /***** Store notify event *****/
   sprintf (Query,"INSERT INTO notif (NotifyEvent,ToUsrCod,FromUsrCod,InsCod,DegCod,CrsCod,MsgCod,ForumType,TimeNotif)" \
                  " VALUES ('%u','%ld','%ld','%ld','%ld','%ld','%ld','%u',NOW())",
            (unsigned) NotifyEvent,
            UsrDat->UsrCod,
            Gbl.Usrs.Me.UsrDat.UsrCod,
            Gbl.CurrentIns.Ins.InsCod,
            Gbl.CurrentDeg.Deg.DegCod,
            Gbl.CurrentCrs.Crs.CrsCod,
            MsgCod,
            (unsigned) Gbl.Forum.ForumType);
   DB_QueryINSERT (Query,"can not create new notification event");
  }

/*****************************************************************************/
/******** Delete notification of notice when the notice is removed ***********/
/*****************************************************************************/

void Ntf_DeletePendingNotif (Ntf_NotifyEvent_t NotifyEvent,long MsgCod)
  {
   char Query[512];

   /***** Delete notification of notice *****/
   sprintf (Query,"DELETE FROM notif WHERE NotifyEvent='%u' AND MsgCod='%ld'",
            (unsigned) NotifyEvent,MsgCod);
   DB_QueryDELETE (Query,"can not remove pending notifications");
  }

/*****************************************************************************/
/***************** Send all pending notifications by e-mail ******************/
/*****************************************************************************/

void Ntf_SendPendingNotifByEMailToAllUsrs (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData UsrDat;
   unsigned NumNotif;
   unsigned NumTotalNotif = 0;
   unsigned NumMails;
   unsigned NumTotalMails = 0;

   /***** Get users who must be notified from database ******/
   sprintf (Query,"SELECT DISTINCT ToUsrCod FROM notif" \
                  " WHERE UNIX_TIMESTAMP(TimeNotif) < UNIX_TIMESTAMP()-%lu",
            Cfg_TIME_TO_SEND_PENDING_NOTIF);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get users who must be notified");

   if (NumRows) // Events found
     {
      /***** Initialize structure with user's data *****/
      Usr_UsrDataConstructor (&UsrDat);

      /***** Notify the users one by one *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
         /* Get next user */
         row = mysql_fetch_row (mysql_res);

         /* Get user code */
         UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);

         /* Get user's data */
	 if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat))		// Get user's data from the database 
           {
            /* Send one email to this user */
            Ntf_SendPendingNotifByEMailToOneUsr (&UsrDat,&NumNotif,&NumMails);
            NumTotalNotif += NumNotif;
            NumTotalMails += NumMails;
           }

         /* Delete all the pending notifications to this user */
         sprintf (Query,"DELETE FROM notif WHERE ToUsrCod='%ld'",UsrDat.UsrCod);
         DB_QueryDELETE (Query,"can not remove pending notifications");
        }

      /***** Free memory used for user's data *****/
      Usr_UsrDataDestructor (&UsrDat);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************ Send pending notifications of one user by e-mail ***************/
/*****************************************************************************/
// Return 0 on success

static void Ntf_SendPendingNotifByEMailToOneUsr (struct UsrData *ToUsrDat,unsigned *NumNotif,unsigned *NumMails)
  {
   extern const char *Txt_Notifications[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTIFY_EVENTS_There_is_a_new_event_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTIFY_EVENTS_There_are_X_new_events_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTIFY_EVENTS_MAIL_SUBJECT_NO_HTML[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Course_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_Forum_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSG_From_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_Go_to_NO_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_If_you_no_longer_wish_to_receive_email_notifications_NO_HTML[Txt_NUM_LANGUAGES];
   static const Act_Action_t Ntf_Actions[Ntf_NUM_NOTIFY_EVENTS] =
     {
      ActSeeExaAnn,	// Ntf_NEW_EXAM_ANNOUNCEMENT
      ActAskSeeAss,	// Ntf_NEW_MARKS_FILE
      ActSeeNot,	// Ntf_NEW_NOTICE
      ActSeeRecMsg,	// Ntf_NEW_MESSAGE
      ActSeeFor,	// Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS
     };
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData FromUsrDat;
   Ntf_NotifyEvent_t NotifyEvent;
   struct Institution Ins;
   struct Degree Deg;
   struct Course Crs;
   long MsgCod;
   For_ForumType_t ForumType;
   char ForumName[512];
   char BreadcrumbStr[512];
   unsigned UnsignedNum;
   char Command[256]; // Command to execute for sending an e-mail
   int ReturnCode;

   /***** Return 0 notifications and 0 mails when error *****/
   *NumNotif = *NumMails = 0;

   /***** Get pending notifications of a user from database ******/
   sprintf (Query,"SELECT NotifyEvent,FromUsrCod,InsCod,DegCod,CrsCod,MsgCod,ForumType,DATE_FORMAT(TimeNotif,'%%Y-%%m-%%d %%H:%%i:%%S')" \
                  " FROM notif WHERE ToUsrCod='%ld' ORDER BY TimeNotif,NotifyEvent",
            ToUsrDat->UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get pending notifications of a user");

   if (NumRows) // Events found
     {
      /***** Create temporary file for mail content *****/
      Msg_CreateFileNameMail ();

      /***** Welcome note *****/
      Msg_WriteWelcomeNoteEMail (ToUsrDat);
      if (NumRows == 1)
         fprintf (Gbl.Msg.FileMail,Txt_NOTIFY_EVENTS_There_is_a_new_event_NO_HTML[ToUsrDat->Prefs.Language],
                  Cfg_PLATFORM_SHORT_NAME);
      else
         fprintf (Gbl.Msg.FileMail,Txt_NOTIFY_EVENTS_There_are_X_new_events_NO_HTML[ToUsrDat->Prefs.Language],
                  (unsigned) NumRows,Cfg_PLATFORM_SHORT_NAME);
      fprintf (Gbl.Msg.FileMail,": \n");

      /***** Initialize structure with origin user's data *****/
      Usr_UsrDataConstructor (&FromUsrDat);

      /***** Inform about the events one by one *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
         /* Get next event */
         row = mysql_fetch_row (mysql_res);

         /* Get event type (row[0]) */
         if (sscanf (row[0],"%u",&UnsignedNum) == 1)
            if (UnsignedNum < For_NUM_TYPES_FORUM)
               NotifyEvent = (For_ForumType_t) UnsignedNum;
            else
               Lay_ShowErrorAndExit ("Wrong type of event.");
         else
            Lay_ShowErrorAndExit ("Wrong type of event.");

         /* Get origin user code (row[1]) */
         FromUsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[1]);
	 Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&FromUsrDat);		// Get origin user's data from the database 

         /* Get institution code (row[2]) */
         Ins.InsCod = Str_ConvertStrCodToLongCod (row[2]);
         Ins_GetDataOfInstitutionByCod (&Ins);

         /* Get degree code (row[3]) */
         Deg.DegCod = Str_ConvertStrCodToLongCod (row[3]);
         Deg_GetDataOfDegreeByCod (&Deg);

         /* Get course code (row[4]) */
         Crs.CrsCod = Str_ConvertStrCodToLongCod (row[4]);
         Deg_GetDataOfCourseByCod (&Crs);

         /* Get message/post code (row[5]) */
         MsgCod = Str_ConvertStrCodToLongCod (row[5]);

         /* Get forum type (row[6]) */
         ForumType = (For_ForumType_t) 0;
         if (sscanf (row[6],"%u",&UnsignedNum) == 1)
            if (UnsignedNum < For_NUM_TYPES_FORUM)
               ForumType = (For_ForumType_t) UnsignedNum;

         /* Write date and time when the event happened (row[7], in YYYYMMDDHHMMSS format) */
         fprintf (Gbl.Msg.FileMail,"\n%s: ",row[7]);

         /* Information about the type of this event */
         fprintf (Gbl.Msg.FileMail,Txt_NOTIFY_EVENTS_MAIL_SUBJECT_NO_HTML[NotifyEvent][ToUsrDat->Prefs.Language],
                  Cfg_PLATFORM_SHORT_NAME);
         fprintf (Gbl.Msg.FileMail,"\n");

         /* Course/forum: */
         switch (NotifyEvent)
           {
            case Ntf_NEW_EXAM_ANNOUNCEMENT:
            case Ntf_NEW_MARKS_FILE:
            case Ntf_NEW_NOTICE:
            case Ntf_NEW_MESSAGE:
               if (Crs.CrsCod > 0)
                  fprintf (Gbl.Msg.FileMail,"%s: %s\n",
                           Txt_Course_NO_HTML[ToUsrDat->Prefs.Language],
                           Crs.FullName);
               break;
            case Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS:
               For_SetForumName (ForumType,&Ins,&Deg,&Crs,ForumName,ToUsrDat->Prefs.Language,false);	// Set forum name in recipient's language
               fprintf (Gbl.Msg.FileMail,"%s: %s\n",
                        Txt_Forum_NO_HTML[ToUsrDat->Prefs.Language],
                        ForumName);
               break;
           }
         /* From: */
         fprintf (Gbl.Msg.FileMail,"%s: %s\n",
                  Txt_MSG_From_NO_HTML[ToUsrDat->Prefs.Language],
                  FromUsrDat.FullName);
         /* Go to: */
         Act_GetBreadcrumbStrForAction (Ntf_Actions[NotifyEvent],ToUsrDat->Prefs.Language,false,BreadcrumbStr);
         fprintf (Gbl.Msg.FileMail,"%s: %s\n",
                  Txt_Go_to_NO_HTML[ToUsrDat->Prefs.Language],
                  BreadcrumbStr);
        }

      /***** Free memory used for origin user's data *****/
      Usr_UsrDataDestructor (&FromUsrDat);

      /* Disclaimer */
      Act_GetBreadcrumbStrForAction (ActEdiPrf,ToUsrDat->Prefs.Language,false,BreadcrumbStr);
      fprintf (Gbl.Msg.FileMail,"\n%s %s.\n",
               Txt_If_you_no_longer_wish_to_receive_email_notifications_NO_HTML[ToUsrDat->Prefs.Language],
               BreadcrumbStr);

      /* Footer note */
      Msg_WriteFootNoteEMail (ToUsrDat->Prefs.Language);

      fclose (Gbl.Msg.FileMail);

      /***** Call the command to send an e-mail *****/
      sprintf (Command,"%s \"%s\" \"%s\" \"[%s] %s\" \"%s\"",
               Cfg_COMMAND_SEND_AUTOMATIC_E_MAIL,
               Cfg_AUTOMATIC_E_MAIL_FROM,ToUsrDat->E_mail,Cfg_PLATFORM_SHORT_NAME,
               Txt_Notifications[ToUsrDat->Prefs.Language],
               Gbl.Msg.FileNameMail);

      ReturnCode = system (Command);
      if (ReturnCode == -1)
         Lay_ShowErrorAndExit ("Error when running script to send e-mail.");

      /***** Remove temporary file *****/
      unlink (Gbl.Msg.FileNameMail);

      /***** Write message depending on return code *****/
      ReturnCode = WEXITSTATUS(ReturnCode);
      switch (ReturnCode)
        {
         case 0: // Message sent successfully
            *NumNotif = (unsigned) NumRows;
            *NumMails = 1;

            /* Update statistics about notifications */
            Ntf_UpdateNumNotifSent (Deg.DegCod,Crs.CrsCod,NotifyEvent,*NumNotif,*NumMails);
            break;
         case 1:
            Lay_ShowAlert (ERROR,"Ha ocurrido un problema enviando autom&aacute;ticamente un correo electr&oacute;nico.");	// Need translation!
            break;
         default:
            sprintf (Gbl.Message,"El mensaje no ha podido enviarse correctamente." \
                                 " C&oacute;digo de error devuelto por el script de env&iacute;o: %d",	// Need translation!
                     ReturnCode);
            Lay_ShowAlert (ERROR,Gbl.Message);
            break;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/******** Show an alert with the number of users notified by e-mail *********/
/****************************************************************************/

void Ntf_ShowAlertNumUsrsToBeNotified (unsigned NumUsrsToBeNotified)
  {
   extern const char *Txt_No_of_users_who_will_be_notified_by_e_mail[Txt_NUM_LANGUAGES];

   sprintf (Gbl.Message,"%s: %u",
            Txt_No_of_users_who_will_be_notified_by_e_mail[Gbl.Prefs.Language],NumUsrsToBeNotified);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/************* Get number of events notified and e-mails sent ***************/
/****************************************************************************/

static void Ntf_GetNumNotifSent (long DegCod,long CrsCod,Ntf_NotifyEvent_t NotifyEvent,unsigned *NumEvents,unsigned *NumMails)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get number of notifications sent by e-mail from database *****/
   sprintf (Query,"SELECT NumEvents,NumMails FROM sta_notif" \
                  " WHERE DegCod='%ld' AND CrsCod='%ld' AND NotifyEvent='%u'",
            DegCod,CrsCod,(unsigned) NotifyEvent);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get number of notifications sent by e-mail");

   /***** Get number of rows *****/
   if (NumRows)
     {
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",NumEvents) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of notifications sent by e-mail.");
      if (sscanf (row[1],"%u",NumMails) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of notifications sent by e-mail.");
     }
   else
      *NumEvents = *NumMails = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/******************** Update number of notify e-mails sent ******************/
/****************************************************************************/

static void Ntf_UpdateNumNotifSent (long DegCod,long CrsCod,Ntf_NotifyEvent_t NotifyEvent,unsigned NumEvents,unsigned NumMails)
  {
   char Query[512];
   unsigned CurrentNumEvents;
   unsigned CurrentNumMails;

   /***** Get number of events notified and number of mails sent *****/
   Ntf_GetNumNotifSent (DegCod,CrsCod,NotifyEvent,&CurrentNumEvents,&CurrentNumMails);

   /***** Update number of users notified *****/
   sprintf (Query,"REPLACE INTO sta_notif (DegCod,CrsCod,NotifyEvent,NumEvents,NumMails) VALUES ('%ld','%ld','%u','%u','%u')",
            DegCod,CrsCod,(unsigned) NotifyEvent,
            CurrentNumEvents + NumEvents,
            CurrentNumMails + NumMails);
   DB_QueryREPLACE (Query,"can not update the number of sent notifications");
  }

/****************************************************************************/
/*** Activate the sending of e-mail to notify me that I have new messages ***/
/****************************************************************************/

void Ntf_PutFormNotifyNewMsgs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Change_preferences[Txt_NUM_LANGUAGES];
   extern const char *Txt_Notify_me_by_e_mail_about[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTIFY_EVENTS[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES];
   Ntf_NotifyEvent_t NotifyEvent;

   /***** Start form *****/
   Act_FormStart (ActChgNotNewMsg);
   fprintf (Gbl.F.Out,"<table cellspacing=\"2\" cellpadding=\"0\"><tr>" \
                      "<td align=\"right\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\"><table cellspacing=\"2\" cellpadding=\"0\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Notify_me_by_e_mail_about[Gbl.Prefs.Language]);

   /***** Checkbox to activate the sending of e-mail to notify me about events *****/
   for (NotifyEvent=(Ntf_NotifyEvent_t) 0; NotifyEvent<Ntf_NUM_NOTIFY_EVENTS; NotifyEvent++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"right\">" \
                         "<input type=\"checkbox\" name=\"%s\" value=\"Y\"",
               Ntf_StrNotifyEvents[NotifyEvent]);
      if ((Gbl.Usrs.Me.UsrDat.Prefs.NotifyEvents & (1 << NotifyEvent)))
          fprintf (Gbl.F.Out," checked=\"checked\""); 
      fprintf (Gbl.F.Out," /></td><td align=\"left\" class=\"%s\">%s</td></tr>",
               The_ClassFormul[Gbl.Prefs.Theme],
               Txt_NOTIFY_EVENTS[NotifyEvent][Gbl.Prefs.Language]);
     }

   /***** End form *****/
   fprintf (Gbl.F.Out,"</table></td></tr></table>");
   Lay_PutSendButton (Txt_Change_preferences[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/* Get parameter with the sending of e-mail to notify me that I have msgs. **/
/****************************************************************************/

static void Ntf_GetParamsNotifyEvents (void)
  {
   Ntf_NotifyEvent_t NotifyEvent;
   char YN[1+1];

   Gbl.Usrs.Me.UsrDat.Prefs.NotifyEvents = 0;
   for (NotifyEvent=(Ntf_NotifyEvent_t) 0; NotifyEvent<Ntf_NUM_NOTIFY_EVENTS; NotifyEvent++)
     {
      Par_GetParam (Par_PARAM_SINGLE,Ntf_StrNotifyEvents[NotifyEvent],YN,1);
      if (Str_ConvertToUpperLetter (YN[0]) == 'Y')
         Gbl.Usrs.Me.UsrDat.Prefs.NotifyEvents |= (1 << NotifyEvent);
     }
  }

/****************************************************************************/
/**** Change my preference about sending me notify e-mails about events *****/
/****************************************************************************/

void Ntf_ChangeNotifyEvents (void)
  {
   char Query[512];

   /***** Get param with the sending of e-mail to notify me about events *****/
   Ntf_GetParamsNotifyEvents ();

   /***** Store the preference about sending of e-mail to notify events *****/
   sprintf (Query,"UPDATE usr_data SET NotifyEvents='%u' WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.Prefs.NotifyEvents,
            Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update user's preferences");
  }
