// swad_pag.c: pagination of assignments, forums and messages

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_act.h"
#include "swad_db.h"
#include "swad_for.h"
#include "swad_gbl.h"
#include "swad_par.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern const Act_Action_t ActionsSeeFor[For_NUM_TYPES_FORUM];
extern const Act_Action_t ActionsSeePstFor[For_NUM_TYPES_FORUM];

/****************************************************************************/
/*************************** Private constants ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

/*****************************************************************************/
/******* Calculate variables related to the pagination of the messages *******/
/*****************************************************************************/
// Return the number of subsets of N elements in a set of NumElements elements
#define NumSubsetsOfNElements(NumElements,N) ((NumElements+(N-1)) / N)
#define NUM_PAGES_BEFORE_CURRENT 1
#define NUM_PAGES_AFTER_CURRENT 1

void Pag_CalculatePagination (struct Pagination *Pagination)
  {
   Pagination->StartPage = Pagination->LeftPage = Pagination->RightPage = Pagination->EndPage = 1;
   Pagination->MoreThanOnePage = false;
   if ((Pagination->NumPags = NumSubsetsOfNElements (Pagination->NumItems,Pag_ITEMS_PER_PAGE)) > 1)
     {
      Pagination->MoreThanOnePage = true;

      /* If page to show is 0 (special code), then last page must be shown.
         If page to show is greater than number of pages, then show last page also */
      if (Pagination->CurrentPage == 0 ||
          Pagination->CurrentPage > Pagination->NumPags)
         Pagination->CurrentPage = Pagination->NumPags;

      /* Compute first page with link around the current */
      if ((Pagination->StartPage = Pagination->CurrentPage - NUM_PAGES_BEFORE_CURRENT) < 1)
         Pagination->StartPage = 1;

      /* Compute last page with link around the current */
      if ((Pagination->EndPage = Pagination->CurrentPage + NUM_PAGES_AFTER_CURRENT) > Pagination->NumPags)
         Pagination->EndPage = Pagination->NumPags;

      /* Compute left page with link in the middle of first page and current page */
      Pagination->LeftPage = (1 + Pagination->StartPage) / 2;

      /* Compute right page with link in the middle of current page and last page */
      Pagination->RightPage = (Pagination->EndPage + Pagination->NumPags) / 2;
     }
   else      // Only one page
      Pagination->CurrentPage = 1;  // If hay only a page, the nmero of page a mostrar es the 1

   Pagination->FirstItemVisible = Pagination->CurrentPage * Pag_ITEMS_PER_PAGE - (Pag_ITEMS_PER_PAGE-1);
   Pagination->LastItemVisible = Pagination->FirstItemVisible + (Pag_ITEMS_PER_PAGE-1);
   if (Pagination->LastItemVisible > Pagination->NumItems)
      Pagination->LastItemVisible = Pagination->NumItems;
  }

/*****************************************************************************/
/************** Show enlaces a distintas pginas of messages **************/
/*****************************************************************************/

void Pag_WriteLinksToPagesCentered (Pag_WhatPaginate_t WhatPaginate,long ThrCod,struct Pagination *Pagination)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Pag_WriteLinksToPages (WhatPaginate,ThrCod,Pagination,true,NULL,"TITULO",false);
   fprintf (Gbl.F.Out,"</div>");
  }

/*****************************************************************************/
/********************** Show links to pages of messages **********************/
/*****************************************************************************/

void Pag_WriteLinksToPages (Pag_WhatPaginate_t WhatPaginate,long ThrCod,struct Pagination *Pagination,
                            bool FirstMsgEnabled,const char *Subject,const char *Font,
                            bool LinkToPagCurrent)
  {
   extern const char *Txt_Page[Txt_NUM_LANGUAGES];
   extern const char *Txt_See_page_X_of_Y[Txt_NUM_LANGUAGES];
   extern const char *Txt_first_message_not_allowed[Txt_NUM_LANGUAGES];
   int NumPage;

   /***** Link to page 1, including a text *****/
   if (Subject)
     {
      if (LinkToPagCurrent)
        {
         switch (WhatPaginate)
           {
            case Pag_ASSIGNMENTS:
               Act_FormStart (ActSeeAsg);
               Pag_PutHiddenParamPagNum (1);
               Asg_PutHiddenParamAsgOrderType ();
               Grp_PutParamWhichGrps ();
               break;
            case Pag_MESSAGES_RECEIVED:
               Act_FormStart (ActSeeRecMsg);
               Pag_PutHiddenParamPagNum (1);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_MESSAGES_SENT:
               Act_FormStart (ActSeeSntMsg);
               Pag_PutHiddenParamPagNum (1);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_THREADS_FORUM:
               Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (1);
               For_PutAllHiddenParamsForum ();
               break;
            case Pag_POSTS_FORUM:
               Act_FormStart (ActionsSeePstFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (1);
               For_PutHiddenParamThrCod (ThrCod);
               For_PutAllHiddenParamsForum ();
	       break;
           }
         sprintf (Gbl.MsgWindowStatus,Txt_See_page_X_of_Y[Gbl.Prefs.Language],
                  1,(unsigned) Pagination->NumPags);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Font);
        }
      else
         fprintf (Gbl.F.Out,"<span class=\"%s\">",Font);
      if (FirstMsgEnabled)
         fprintf (Gbl.F.Out,"%s",Subject);
      else
         fprintf (Gbl.F.Out,"[%s]",
                  Txt_first_message_not_allowed[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"%s",LinkToPagCurrent ? "</acronym></a></form>" : "</span>");
     }

   if (Pagination->MoreThanOnePage)
     {
      /***** Links to several pages start here *****/
      fprintf (Gbl.F.Out,"<table cellspacing=\"5\" cellpadding=\"0\">" \
                         "<tr>" \
                         "<td align=\"left\" class=\"%s\">[</td>" \
                         "<td align=\"left\" class=\"%s\">%s</td>",
               Font,Font,Txt_Page[Gbl.Prefs.Language]);

      /***** Possible link to page 1 *****/
      if (Pagination->StartPage > 1)
        {
         fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",Font);
         switch (WhatPaginate)
           {
            case Pag_ASSIGNMENTS:
               Act_FormStart (ActSeeAsg);
               Pag_PutHiddenParamPagNum (1);
               Asg_PutHiddenParamAsgOrderType ();
               Grp_PutParamWhichGrps ();
               break;
            case Pag_MESSAGES_RECEIVED:
               Act_FormStart (ActSeeRecMsg);
               Pag_PutHiddenParamPagNum (1);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_MESSAGES_SENT:
               Act_FormStart (ActSeeSntMsg);
               Pag_PutHiddenParamPagNum (1);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_THREADS_FORUM:
               Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (1);
               For_PutAllHiddenParamsForum ();
               break;
            case Pag_POSTS_FORUM:
               Act_FormStart (ActionsSeePstFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (1);
               For_PutHiddenParamThrCod (ThrCod);
               For_PutAllHiddenParamsForum ();
	       break;
           }
         sprintf (Gbl.MsgWindowStatus,Txt_See_page_X_of_Y[Gbl.Prefs.Language],
                  1,(unsigned) Pagination->NumPags);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Font);
         fprintf (Gbl.F.Out,"1</acronym></a></form></td>");
         if (Pagination->LeftPage > 2)
            fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">...</td>",Font);
        }

      /***** Posible link to page left *****/
      if (Pagination->LeftPage > 1 && Pagination->LeftPage < Pagination->StartPage)
        {
         fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",Font);
         switch (WhatPaginate)
           {
            case Pag_ASSIGNMENTS:
               Act_FormStart (ActSeeAsg);
               Pag_PutHiddenParamPagNum (Pagination->LeftPage);
               Asg_PutHiddenParamAsgOrderType ();
               Grp_PutParamWhichGrps ();
               break;
            case Pag_MESSAGES_RECEIVED:
               Act_FormStart (ActSeeRecMsg);
               Pag_PutHiddenParamPagNum (Pagination->LeftPage);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_MESSAGES_SENT:
               Act_FormStart (ActSeeSntMsg);
               Pag_PutHiddenParamPagNum (Pagination->LeftPage);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_THREADS_FORUM:
               Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (Pagination->LeftPage);
               For_PutAllHiddenParamsForum ();
               break;
            case Pag_POSTS_FORUM:
               Act_FormStart (ActionsSeePstFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (Pagination->LeftPage);
               For_PutHiddenParamThrCod (ThrCod);
               For_PutAllHiddenParamsForum ();
	       break;
           }
         sprintf (Gbl.MsgWindowStatus,Txt_See_page_X_of_Y[Gbl.Prefs.Language],
                  (unsigned) Pagination->LeftPage,(unsigned) Pagination->NumPags);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Font);
         fprintf (Gbl.F.Out,"%u</acronym></a></form></td>",
                  (unsigned) Pagination->LeftPage);
         if (Pagination->LeftPage < Pagination->StartPage-1)
            fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">...</td>",Font);
        }

      /***** Loop to put links to the pages around the current one *****/
      for (NumPage=Pagination->StartPage; NumPage<=Pagination->EndPage; NumPage++)
        {
         fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",Font);
         if (!LinkToPagCurrent && NumPage == Pagination->CurrentPage)
            fprintf (Gbl.F.Out,"<u>%u</u>",(unsigned) NumPage);
         else
           {
            switch (WhatPaginate)
              {
               case Pag_ASSIGNMENTS:
                  Act_FormStart (ActSeeAsg);
                  Pag_PutHiddenParamPagNum (NumPage);
                  Asg_PutHiddenParamAsgOrderType ();
                  Grp_PutParamWhichGrps ();
                  break;
               case Pag_MESSAGES_RECEIVED:
                  Act_FormStart (ActSeeRecMsg);
                  Pag_PutHiddenParamPagNum (NumPage);
                  Msg_PutHiddenParamsMsgsFilters ();
                  break;
               case Pag_MESSAGES_SENT:
                  Act_FormStart (ActSeeSntMsg);
                  Pag_PutHiddenParamPagNum (NumPage);
                  Msg_PutHiddenParamsMsgsFilters ();
                  break;
               case Pag_THREADS_FORUM:
                  Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
                  Pag_PutHiddenParamPagNum (NumPage);
                  For_PutAllHiddenParamsForum ();
                  break;
               case Pag_POSTS_FORUM:
                  Act_FormStart (ActionsSeePstFor[Gbl.Forum.ForumType]);
                  Pag_PutHiddenParamPagNum (NumPage);
                  For_PutHiddenParamThrCod (ThrCod);
                  For_PutAllHiddenParamsForum ();
                  break;
              }
            sprintf (Gbl.MsgWindowStatus,Txt_See_page_X_of_Y[Gbl.Prefs.Language],
                     (unsigned) NumPage,(unsigned) Pagination->NumPags);
            Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Font);
            fprintf (Gbl.F.Out,"%u</acronym></a></form>",(unsigned) NumPage);
           }
         fprintf (Gbl.F.Out,"</td>");
        }

      /***** Posible link to page right *****/
      if (Pagination->RightPage > Pagination->EndPage && Pagination->RightPage < Pagination->NumPags)
        {
         if (Pagination->RightPage > Pagination->EndPage+1)
            fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">...</td>",Font);
         fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",Font);
         switch (WhatPaginate)
           {
            case Pag_ASSIGNMENTS:
               Act_FormStart (ActSeeAsg);
               Pag_PutHiddenParamPagNum (Pagination->RightPage);
               Asg_PutHiddenParamAsgOrderType ();
               Grp_PutParamWhichGrps ();
               break;
            case Pag_MESSAGES_RECEIVED:
               Act_FormStart (ActSeeRecMsg);
               Pag_PutHiddenParamPagNum (Pagination->RightPage);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_MESSAGES_SENT:
               Act_FormStart (ActSeeSntMsg);
               Pag_PutHiddenParamPagNum (Pagination->RightPage);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_THREADS_FORUM:
               Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (Pagination->RightPage);
               For_PutAllHiddenParamsForum ();
               break;
            case Pag_POSTS_FORUM:
               Act_FormStart (ActionsSeePstFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (Pagination->RightPage);
               For_PutHiddenParamThrCod (ThrCod);
               For_PutAllHiddenParamsForum ();
               break;
           }
         sprintf (Gbl.MsgWindowStatus,Txt_See_page_X_of_Y[Gbl.Prefs.Language],
                  (unsigned) Pagination->RightPage,(unsigned) Pagination->NumPags);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Font);
         fprintf (Gbl.F.Out,"%u</acronym></a></form></td>",(unsigned) Pagination->RightPage);
        }

      /***** Possible link to last page *****/
      if (Pagination->EndPage < Pagination->NumPags)
        {
         if (Pagination->NumPags > Pagination->RightPage+1)
            fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">...</td>",Font);
         fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",Font);
         switch (WhatPaginate)
           {
            case Pag_ASSIGNMENTS:
               Act_FormStart (ActSeeAsg);
               Pag_PutHiddenParamPagNum (Pagination->NumPags);
               Asg_PutHiddenParamAsgOrderType ();
               Grp_PutParamWhichGrps ();
               break;
            case Pag_MESSAGES_RECEIVED:
               Act_FormStart (ActSeeRecMsg);
               Pag_PutHiddenParamPagNum (Pagination->NumPags);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_MESSAGES_SENT:
               Act_FormStart (ActSeeSntMsg);
               Pag_PutHiddenParamPagNum (Pagination->NumPags);
               Msg_PutHiddenParamsMsgsFilters ();
               break;
            case Pag_THREADS_FORUM:
               Act_FormStart (ActionsSeeFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (Pagination->NumPags);
               For_PutAllHiddenParamsForum ();
               break;
            case Pag_POSTS_FORUM:
               Act_FormStart (ActionsSeePstFor[Gbl.Forum.ForumType]);
               Pag_PutHiddenParamPagNum (Pagination->NumPags);
               For_PutHiddenParamThrCod (ThrCod);
               For_PutAllHiddenParamsForum ();
               break;
           }
         sprintf (Gbl.MsgWindowStatus,Txt_See_page_X_of_Y[Gbl.Prefs.Language],
                  (unsigned) Pagination->NumPags,(unsigned) Pagination->NumPags);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Font);
         fprintf (Gbl.F.Out,"%u</acronym></a></form></td>",(unsigned) Pagination->NumPags);
        }
      fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">]</td></tr></table>",Font);
     }
  }

/*****************************************************************************/
/******************* Put hidden parameter number of page *********************/
/*****************************************************************************/

void Pag_PutHiddenParamPagNum (unsigned NumPage)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NumPage\" value=\"%u\" />",NumPage);
  }
 
/*****************************************************************************/
/************************ Get parameter number of page ***********************/
/*****************************************************************************/

void Pag_GetParamPagNum (Pag_WhatPaginate_t WhatPaginate)
  {
   static bool FirstCall = true;
   char UnsignedStr[1+10+1];
   unsigned NumPage;

   if (FirstCall)
     {
      FirstCall = false;

      /***** Try to get parameter with the number of page *****/
      Par_GetParam (Par_PARAM_SINGLE,"NumPage",UnsignedStr,10);
      if (sscanf (UnsignedStr,"%u",&NumPage) == 1)
         Gbl.Pag.CurrentPage = NumPage;
      else
         /***** If there's no parameter page, return a default value *****/
         switch (WhatPaginate)
           {
            case Pag_MESSAGES_RECEIVED:
	       // If there are new messages, show first page, else show last page
               Gbl.Pag.CurrentPage = ((Gbl.Usrs.Me.UsrDat.NumNewMsgs) ? 1 : Pag_GetLastPageMsgFromSession (Pag_MESSAGES_RECEIVED));
               break;
            case Pag_MESSAGES_SENT:
               // Show last page
               Gbl.Pag.CurrentPage = Pag_GetLastPageMsgFromSession (Pag_MESSAGES_SENT);
               break;
            default:
               Gbl.Pag.CurrentPage = 1;
           }
     }
  }

/*****************************************************************************/
/********* Save last page of received/sent messages into session *************/
/*****************************************************************************/

void Pag_SaveLastPageMsgIntoSession (Pag_WhatPaginate_t WhatPaginate,unsigned NumPage)
  {
   char Query[512];

   /***** Save last page of received/sent messages *****/
   sprintf (Query,"UPDATE sessions SET %s='%u' WHERE SessionId='%s'",
            (WhatPaginate == Pag_MESSAGES_RECEIVED) ? "LastPageMsgRec" : "LastPageMsgSnt",
            NumPage,Gbl.Session.Id);
   DB_QueryUPDATE (Query,"can not update last page of messages");
  }

/*****************************************************************************/
/********* Get last page of received/sent messages stored in session *********/
/*****************************************************************************/

unsigned Pag_GetLastPageMsgFromSession (Pag_WhatPaginate_t WhatPaginate)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumPage;

   /***** Get last page of received/sent messages from database *****/
   sprintf (Query,"SELECT %s FROM sessions WHERE SessionId='%s'",
            (WhatPaginate == Pag_MESSAGES_RECEIVED) ? "LastPageMsgRec" : "LastPageMsgSnt",
            Gbl.Session.Id);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get last page of messages");

   /***** Check number of rows of the result ****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting last page of messages.");

   /***** Get last page of messages *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumPage) == 1)
      if (NumPage == 0)
         NumPage = 1;

   return NumPage;
  }
