// swad_par.c: CGI parameters

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <ctype.h>		// For isprint, isspace, etc.
#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_prf.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Actions Actions[Act_NUM_ACTIONS];
extern struct Tabs Tabs[Act_NUM_TABS];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

/****************************************************************************/
/*** Read all parameters passed to this CGI and store for later processing **/
/****************************************************************************/
// Return false if there are no arguments, and true if there are arguments

bool Par_GetQueryString (void)
  {
   char Method[256];
   char ContentType[512];

   strcpy (Method,getenv ("REQUEST_METHOD"));

   if (!strcmp (Method,"GET"))
     {
      Gbl.GetMethod = true;
      Gbl.ContentReceivedByCGI = Cte_NORMAL;
      strcpy (Gbl.QueryString,getenv ("QUERY_STRING"));
     }
   else
     {
      /* If data are received ==> the environment variable CONTENT_TYPE will hold:
      multipart/form-data; boundary=---------------------------7d13ca2e948
      */
      if (getenv ("CONTENT_TYPE") == NULL)
         return false;

      strcpy (ContentType,getenv ("CONTENT_TYPE"));

      if (!strncmp (ContentType,"multipart/form-data",strlen ("multipart/form-data")))
        {
         Gbl.ContentReceivedByCGI = Cte_DAT;
         /* If data are received ==> the environment variable CONTENT_TYPE will hold:
         multipart/form-data; boundary=---------------------------7d13ca2e948
         Gbl.DelimiterStringIncludingInitialRet will be set to:
			        "\r\n-----------------------------7d13ca2e948"
         I.e. 0x0D, 0x0A, '-', '-', and boundary.
         */
         sprintf (Gbl.DelimiterString,"--%s",strstr (getenv ("CONTENT_TYPE"),"boundary=") + strlen ("boundary="));
         sprintf (Gbl.DelimiterStringIncludingInitialRet,"%c%c%s",0x0D,0x0A,Gbl.DelimiterString);
         return Fil_ReadStdinIntoTmpFile ();
        }
      else if (!strncmp (ContentType,"text/xml",strlen ("text/xml")))
        {
         Gbl.WebService = true;
        }
      else
        {
         Gbl.ContentReceivedByCGI = Cte_NORMAL;

         /***** Get the string sent by form *****/
         fgets (Gbl.QueryString,Cte_MAX_LENGTH_ARGS_SENT_TO_CGI,stdin);
        }
     }
   return true;
  }

/****************************************************************************/
/****************** Get the parameters sent to this CGI *********************/
/****************************************************************************/

#define MAX_LENGTH_ACTION_STR	20
#define MAX_BYTES_ACTION_STR	(MAX_LENGTH_ACTION_STR*Str_MAX_CHARACTER)

#define MAX_LENGTH_TAB_STR	20
#define MAX_BYTES_TAB_STR	(MAX_LENGTH_TAB_STR*Str_MAX_CHARACTER)

void Par_GetMainParameters (void)
  {
   extern const char *The_ThemeId[The_NUM_THEMES];
   long DegTypCodFromMenu = -1L;
   Act_Tab_t NumTab;
   Act_Action_t NumAct;
   char TabStr[MAX_BYTES_TAB_STR+1];		// String that stores the current tab
   char ActionStr[MAX_BYTES_ACTION_STR+1];	// String that stores the current action
   char LongStr[1+10+1];
   char YearStr[1+1];
   tTheme ThemeFromForm;

   /***** Reset codes of institution, degree and course *****/
   Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = Gbl.CurrentDeg.Deg.DegTypCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
   Gbl.YearOK = false;

   // First of all, get action, and session identifier.
   // So, if other parameters have been stored in the database, there will be no problems to get them.

   /***** Get action to perform *****/
   if (Gbl.WebService)
     {
      Gbl.CurrentAct = ActWebSvc;
      return;
     }
   /* Check if SWAD is called from an external site */
   Imp_GetImpUsrAndSession ();
   if (Gbl.Imported.UsrId[0] && Gbl.Imported.SessionId[0])
     {
      Gbl.CurrentAct = ActAutUsrExt;
      return;
     }
   // SWAD is not called from external site
   Par_GetParam (Par_PARAM_SINGLE,"NxtAct",ActionStr,MAX_BYTES_ACTION_STR);
   for (NumAct=(Act_Action_t) 0; NumAct<Act_NUM_ACTIONS; NumAct++)
      if (!strcmp (ActionStr,Actions[NumAct].Str))
        {
         Gbl.CurrentAct = NumAct;
         break;
        }

   /***** Get session identifier, if exists *****/
   if (strlen (Par_GetParam (Par_PARAM_SINGLE,"IdSes",Gbl.Session.Id,Ses_LENGTH_SESSION_ID)))
     {
      /***** Get user's code, password, current degree and current course from stored session *****/
      if (Ses_GetSessionData ())
        {
	 Gbl.Session.IsOpen = true;
         Imp_GetImpSessionData ();
        }
      else
	{
	 Gbl.Session.HasBeenDisconnected = true;
	 Gbl.Session.Id[0] = '\0';
	}
     }

//   if (Gbl.Session.HasBeenDisconnected)
//      return;

   /***** Get user password and login *****/
   switch (Gbl.CurrentAct)
     {
      case ActAutUsrInt:
         Usr_GetParamUsrPwdLogin ();
      case ActFrmSndNewPwd:
      case ActSndNewPwd:
         Usr_GetParamUsrIdLogin ();
	 break;
     }

   /***** Get tab to activate *****/
   Par_GetParam (Par_PARAM_SINGLE,"NxtTab",TabStr,MAX_BYTES_TAB_STR);
   for (NumTab=(Act_Tab_t) 0; NumTab<Act_NUM_TABS; NumTab++)
      if (!strcmp (TabStr,Tabs[NumTab].Str))
        {
	 Gbl.CurrentTab = NumTab;
	 break;
        }
   if (Gbl.CurrentTab == TabUnk) 			// If no tab specified in the form...
      Gbl.CurrentTab = Actions[Gbl.CurrentAct].Tab;	// ...use the tab associated to current action

   if (!Gbl.Session.IsOpen)	// When no session open (no logged user)...
     {
      /***** Try to get preferences changed from current IP *****/
      Prf_GetPrefsFromIP ();
      if (Gbl.Prefs.Theme == THEME_UNKNOWN)
         Gbl.Prefs.Theme = THEME_DEFAULT;

      /***** Try to get parameter theme (sent via GET method *****/
      if ((ThemeFromForm = The_GetParamTheme ()) != THEME_UNKNOWN)
        {
         /* Parameter is correct, so set theme */
         Gbl.Prefs.Theme = ThemeFromForm;

         /* Update preferences from current IP */
         Prf_SetPrefsFromIP ();
        }
  
      /***** Set path of theme *****/
      sprintf (Gbl.Prefs.PathTheme,"%s/%s/%s",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_THEMES,The_ThemeId[Gbl.Prefs.Theme]);
     }

   /***** Get type of degree if exists (from menu) *****/
   Par_GetParam (Par_PARAM_SINGLE,"DegTyp",LongStr,1+10);
   if (LongStr[0])	// Parameter DegTyp available
      DegTypCodFromMenu = Str_ConvertStrCodToLongCod (LongStr);

   if (DegTypCodFromMenu > 0)	// Changing degree type in menu
     {
      Gbl.CurrentDeg.Deg.DegTypCod = DegTypCodFromMenu;
      Gbl.YearOK = false;
      Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
     }
   else
     {
      /***** Get numerical degree code if exists (from menu) *****/
      Par_GetParam (Par_PARAM_SINGLE,"DegCod",LongStr,1+10);
      if (LongStr[0])	// Parameter DegCod available
        {
         Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Str_ConvertStrCodToLongCod (LongStr);
         if (Gbl.CurrentDeg.Deg.DegCod > 0)
           {
            /***** Get possible course year *****/
            Par_GetParam (Par_PARAM_SINGLE,"CrsYear",YearStr,1);
            if (YearStr[0])	// Parameter CrsYear available
              {
               if (isdigit ((int) (YearStr[0])))		
                 {
                  Gbl.CurrentCrs.Crs.Year = (unsigned) (YearStr[0]) - (unsigned) '0';
                  Gbl.YearOK = (Gbl.CurrentCrs.Crs.Year <= Deg_MAX_YEARS_PER_DEGREE);
                 }
               else
                  Gbl.YearOK = false;
               if (!Gbl.YearOK)	// Error in parameter year
                 {
                  Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
                  return;
                 }
              }

            Gbl.CurrentCrs.Crs.CrsCod = -1L;	// Reset possible course from session
           }
         else
           {
            Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
            return;
           }
        }

      /***** Get numerical course code if exists (from menu) *****/
      Par_GetParam (Par_PARAM_SINGLE,"CrsCod",LongStr,1+10);
      if (LongStr[0])	// Parameter CrsCod available
         Gbl.CurrentCrs.Crs.CrsCod = Str_ConvertStrCodToLongCod (LongStr);	// Overwrite CrsCod from session

      /***** If numerical course code is available, get course data *****/
      if (Gbl.CurrentCrs.Crs.CrsCod > 0)
        {
         if (Deg_GetDataOfCourseByCod (&Gbl.CurrentCrs.Crs))		// Course found
           {
            Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod;
            Gbl.YearOK = true;
           }
         else
           {
            Gbl.YearOK = false;
            Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = Gbl.CurrentDeg.Deg.DegTypCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
           }
        }

      /***** If numerical degree code is available, get degree data *****/
      if (Gbl.CurrentDeg.Deg.DegCod > 0)
        {
         if (!Deg_GetDataOfDegreeByCod (&Gbl.CurrentDeg.Deg))		// Degree not found
           {
            Gbl.YearOK = false;
            Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = Gbl.CurrentDeg.Deg.DegTypCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
           }
        }
      else
        {
         Gbl.YearOK = false;
         Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod = -1;
        }

      /***** If numerical institution code is available, get institution data *****/
      if (Gbl.CurrentDeg.Deg.InsCod > 0)
        {
         Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.InsCod;
         if (!Ins_GetDataOfInstitutionByCod (&Gbl.CurrentIns.Ins))	// Institution not found
           {
            Gbl.YearOK = false;
            Gbl.CurrentIns.Ins.InsCod = Gbl.CurrentDeg.Deg.DegTypCod = Gbl.CurrentDeg.Deg.DegCod = Gbl.CurrentCrs.Crs.DegCod = Gbl.CurrentCrs.Crs.CrsCod = -1L;
           }
        }
      // else
      //   Gbl.YearOK = false;
     }

   /***** If degree type code is available, get degree type data *****/
   Gbl.CurrentDegTyp.DegTyp.DegTypCod = Gbl.CurrentDeg.Deg.DegTypCod;
   if (Gbl.CurrentDegTyp.DegTyp.DegTypCod >= 0)
     {
      if ((Gbl.CurrentDegTyp.DegTyp.DegTypCod = Gbl.CurrentDeg.Deg.DegTypCod = Deg_GetDataOfDegreeTypeByCod (&Gbl.CurrentDegTyp.DegTyp)) == -1L)
         Gbl.YearOK = false;
     }   
   else
      Gbl.YearOK = false;
  }

/****************************************************************************/
/************************* Get the value of a parameter *********************/
/****************************************************************************/

// Return the pointer ParamValue

#define LENGTH_OF_STR_BEFORE_PARAM 38	// Length of "CONTENT-DISPOSITION: FORM-DATA; NAME=\""
#define MAX_BYTES_STR_AUX 1024

char *Par_GetParam (tParamType ParamType, const char *ParamName, char *ParamValue, size_t MaxBytes)
  {
   static const char *StringBeforeParam = "CONTENT-DISPOSITION: FORM-DATA; NAME=\"";
   size_t BytesToCopy;
   size_t BytesAlreadyCopied = 0;
   int Ch;
   char *PtrSrc;
   char *PtrDst;
   char *PtrStartOfParam;
   int Result;
   int NumTimesThisParam;
   bool ParamFound = false;
   bool ContinueSearching = true;
   unsigned ParamNameLength;
   char StrAux[MAX_BYTES_STR_AUX+1];

   ParamValue[0] = '\0'; // By default, the value of the parameter will be an empty string
   ParamNameLength = strlen (ParamName);

   if (Gbl.ContentReceivedByCGI == Cte_NORMAL)
     {
      if (Gbl.GetMethod)			// Only some selected parameters can be passed by GET method
        {
         if (strcmp (ParamName,"CrsCod") &&	// To enter directly to a course
             strcmp (ParamName,"Theme") &&	// To change the design an colors theme
             strcmp (ParamName,"plugin"))	// To call (via SOAP) functions associated to a determined plugin
	    return ParamValue;	// Return empty value when method is GET and parameter is not one of these
        }
      PtrSrc = Gbl.QueryString;
      PtrDst = ParamValue;
      for (NumTimesThisParam=0; (NumTimesThisParam<1 || ParamType == Par_PARAM_MULTIPLE); NumTimesThisParam++)
	{
         ParamFound = false;
         do
            if ((PtrStartOfParam = strstr (PtrSrc,ParamName)) != NULL)
              {
	       // String ParamName found inside Gbl.QueryString
               if (*(PtrStartOfParam + ParamNameLength) == '=')
                 {
		  // Just after the name of the parameter, must be found a '=' symbol
                  if (PtrStartOfParam == Gbl.QueryString)	// The parameter is just at start
                     ParamFound = true;
                  else if (*(PtrStartOfParam-1) == '&')		// The parameter is not at start, but just after an "&" separator
                     ParamFound = true;
                  else						// String has been found at the end of another parameter
                     PtrSrc = PtrStartOfParam + ParamNameLength;
                 }
               else						// String has been found, but it is not a parameter
                  PtrSrc = PtrStartOfParam + ParamNameLength;
              }
         while (PtrStartOfParam != NULL && !ParamFound);
         if (!ParamFound)	// Not found ==> PtrStartOfParam == NULL
            break;
         PtrSrc = PtrStartOfParam + ParamNameLength + 1;	// Add 1 due to the '=' symbol
	 if (NumTimesThisParam)
	   {
	    if (BytesAlreadyCopied + 1 > MaxBytes)
              {
	       sprintf (Gbl.Message,"Multiple parameter <strong>%s</strong> too large," \
	                            " it exceed the maximum allowed size (%lu bytes).",
	                ParamName,MaxBytes);
	       Lay_ShowErrorAndExit (Gbl.Message);
              }
	    *PtrDst++ = Par_SEPARATOR_PARAM_MULTIPLE;		// Separator in the destination string
	    BytesAlreadyCopied++;
	   }
	 BytesToCopy = strcspn (PtrSrc,"&");	// The & charecter is the separator of two parameters
	 if (BytesAlreadyCopied + BytesToCopy > MaxBytes)
           {
            sprintf (Gbl.Message,"Parameter <strong>%s</strong> too large," \
                                 " it exceed the maximum allowed size (%lu bytes).",
                     ParamName,MaxBytes);
	    Lay_ShowErrorAndExit (Gbl.Message);
           }
	 strncpy (PtrDst,PtrSrc,BytesToCopy);
	 BytesAlreadyCopied += BytesToCopy;
	 PtrDst += BytesToCopy;
         PtrSrc += BytesToCopy;
        }
      *PtrDst = '\0'; // strncpy() doesn't add the final NULL
     }
   else	// Gbl.ContentReceivedByCGI == Cte_DAT
     {
      rewind (Gbl.F.Tmp);
      while (ContinueSearching)
        {
         Result = Str_ReceiveFileUntilDelimitStr (Gbl.F.Tmp,NULL,StrAux,Gbl.DelimiterString,(unsigned long long) MAX_BYTES_STR_AUX);
         switch (Result)
           {
            case -1:	// Delimiter string not found
               ContinueSearching = false;
               break;
            case 0:	// MAX_BYTES_STR_AUX exceeded
               ContinueSearching = false;
               break;
            case 1:	// Delimiter string found
	       /* Skip carriage returns, spaces, etc. */
	       do
	          Ch = fgetc (Gbl.F.Tmp);
	       while (isspace (Ch) && Ch != EOF);
               if (Ch == (int) StringBeforeParam[0])
                  if (!strcasecmp (Str_GetNextStrFromFile (Gbl.F.Tmp,StrAux,LENGTH_OF_STR_BEFORE_PARAM-1),StringBeforeParam+1)) // Start of a parameter
                     if (!strcasecmp (Str_GetNextStrFromFile (Gbl.F.Tmp,StrAux,ParamNameLength),ParamName)) // Parameter found
                       {
 	                /* Skip "'" symbol after parameter name */
	                Ch = fgetc (Gbl.F.Tmp);
	                if (Ch != (int) '\"')
                          {
                           sprintf (Gbl.Message,"Error reading parameter <strong>%s</strong>." \
                                                " A <strong>&quot;</strong> character was expected, but a <strong>%c</strong> (ASCII code %d) has been found.",
                                    ParamName,(char) Ch,Ch);
	                   Lay_ShowErrorAndExit (Gbl.Message);
                          }

	                /* Skip carriage returns, spaces, etc. */
	                do
	                   Ch = fgetc (Gbl.F.Tmp);
	                while (isspace (Ch) && Ch != EOF);

	                /* Get the parameter */
	                ParamValue[0] = Ch;
	                Result = Str_ReceiveFileUntilDelimitStr (Gbl.F.Tmp,(FILE *) NULL,&ParamValue[1],Gbl.DelimiterStringIncludingInitialRet,(unsigned long long) MaxBytes);

	                /* Depending on the result... */
                        switch (Result)
                          {
                           case -1:	// Delimiter string not found
	                      sprintf (Gbl.Message,"Parameter <strong>%s</strong> has not been readed properly.",
	                               ParamName);
	                      Lay_ShowErrorAndExit (Gbl.Message);
                              break;
                           case 0:	// Parameter's too long
	                      sprintf (Gbl.Message,"Parameter <strong>%s</strong> exceed the maximum allowed size (%lu bytes).",
	                               ParamName,MaxBytes);
	                      Lay_ShowErrorAndExit (Gbl.Message);
                              break;
	             	   case 1:	// Delimiter string and parameter value found
                              ParamFound = true;
                              ContinueSearching = false;
                              break;
	                  }
                       }
               break;
           }
        }
     }

   if (!ParamValue[0])
      /***** If parameter not found or parameter empty, search from table with hidden parameters in the database *****/
      if (Ses_GetHiddenParFromDB (Gbl.CurrentAct,ParamName,ParamValue,MaxBytes))
        {
         sprintf (Gbl.Message,"Hidden parameter <strong>%s</strong> too large," \
                              " it exceed the maximum allowed size (%lu bytes).",
                  ParamName,MaxBytes);
         Lay_ShowErrorAndExit (Gbl.Message);
        }

   return ParamValue;
  }
