// swad_plc.c: places

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>		// For calloc
#include <string.h>		// For string functions

#include "swad_cns.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_plc.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Plc_GetParamPlcOrderType (void);
static void Plc_PutFormToEditPlcs (void);
static void Plc_ListPlacesForEdition (void);
static void Plc_PutParamPlcCod (long PlcCod);
static void Plc_RenamePlace (Cte_ShortOrFullName_t ShortOrFullName);
static bool Plc_CheckIfPlaceNameExists (const char *FieldName,const char *Name,long PlcCod);
static void Plc_PutFormToCreatePlace (void);
static void Plc_PutHeadPlaces (void);
static void Plc_CreatePlace (struct Place *Plc);

/****************************************************************************/
/*************************** List all the places ****************************/
/****************************************************************************/

void Plc_SeePlaces (void)
  {
   extern const char *Txt_PLACES_HELP_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_PLACES_ORDER[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_Other_places[Txt_NUM_LANGUAGES];
   extern const char *Txt_Place_unspecified[Txt_NUM_LANGUAGES];
   Plc_PlcsOrderType_t Order;
   unsigned NumPlc;
   unsigned NumDegsWithPlc = 0;
   unsigned NumDegsInOtherPlcs;

   /***** Get parameter with the type of order in the list of places *****/
   Plc_GetParamPlcOrderType ();

   /***** Get list of places *****/
   Plc_GetListPlaces ();

   /***** Put link (form) to edit places *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
       Plc_PutFormToEditPlcs ();

   /***** Table head *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr>");
   for (Order=PLC_ORDER_BY_PLACE; Order<=PLC_ORDER_BY_NUM_DEGS; Order++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"TIT_TBL\">");
      Act_FormStart (ActSeePlc);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_PLACES_HELP_ORDER[Order][Gbl.Prefs.Language],"TIT_TBL");
      if (Order == Gbl.Plcs.SelectedOrderType)
         fprintf (Gbl.F.Out,"<u>");
      fprintf (Gbl.F.Out,Txt_PLACES_ORDER[Order][Gbl.Prefs.Language]);
      if (Order == Gbl.Plcs.SelectedOrderType)
         fprintf (Gbl.F.Out,"</u>");
      fprintf (Gbl.F.Out,"</acronym></a></form></td>");
     }
   fprintf (Gbl.F.Out,"</tr>");


   /***** Write all the places and their nuber of degrees *****/
   for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
     {
      /* Write data of this place */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">%s</td>",
               Gbl.Plcs.Lst[NumPlc].FullName);
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Gbl.Plcs.Lst[NumPlc].NumDegs);
      NumDegsWithPlc += Gbl.Plcs.Lst[NumPlc].NumDegs;
     }

   /***** Separation row *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;</td>" \
                      "</tr>");

   /***** Write degrees with other place *****/
   NumDegsInOtherPlcs = Deg_GetNumberOfDegreesInPlace (0);
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "</tr>",
            Txt_Other_places[Gbl.Prefs.Language],NumDegsInOtherPlcs);
   NumDegsWithPlc += NumDegsInOtherPlcs;

   /***** Write degrees with no place *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "</tr>",
            Txt_Place_unspecified[Gbl.Prefs.Language],
            Deg_GetTotalNumberOfDegrees () - NumDegsWithPlc);

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of places *****/
   Plc_FreeListPlaces ();
  }

/*****************************************************************************/
/********** Get parameter with the type or order in list of places ***********/
/*****************************************************************************/

static void Plc_GetParamPlcOrderType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Order",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      Gbl.Plcs.SelectedOrderType = (Plc_PlcsOrderType_t) UnsignedNum;
   else
      Lay_ShowErrorAndExit ("Sort order is missing.");
  }

/****************************************************************************/
/***** Put a hidden parameter with the type of order in list of places ******/
/****************************************************************************/

void Plc_PutHiddenParamPlcOrderType (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
            (unsigned) Gbl.Plcs.SelectedOrderType);
  }

/****************************************************************************/
/********************** Put a link (form) to edit places ********************/
/****************************************************************************/

static void Plc_PutFormToEditPlcs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiPlc);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************ Put forms to edit degree types ********************/
/****************************************************************************/

void Plc_EditPlaces (void)
  {
   extern const char *Txt_Places[Txt_NUM_LANGUAGES];

   /***** Get list of places *****/
   Plc_GetListPlaces ();

   /***** Help message *****/
   if (Gbl.Plcs.Num) // Places found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo lugar rellene el nombre del lugar" \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar un lugar existente pulse el aspa roja.<br />" \
                          "Para modificar un lugar existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Places[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No degree types found
      Lay_ShowAlert (INFO,"No existen lugares.<br />" \
                          "Para a&ntilde;adir un lugar rellene el nombre del lugar" \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit places *****/
   /* Start table with places */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current places */
   if (Gbl.Plcs.Num)
      Plc_ListPlacesForEdition ();

   /* Put a form to create a new place */
   Plc_PutFormToCreatePlace ();

   /* End of table with places */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of places *****/
   Plc_FreeListPlaces ();
  }

/****************************************************************************/
/**************************** List all the places ***************************/
/****************************************************************************/

void Plc_GetListPlaces (void)
  {
   char OrderBySubQuery[256];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumPlc;
   struct Place *Plc;

   /***** Get places from database *****/
   switch (Gbl.Plcs.SelectedOrderType)
     {
      case PLC_ORDER_BY_PLACE:
         sprintf (OrderBySubQuery,"FullName");
         break;
      case PLC_ORDER_BY_NUM_DEGS:
         sprintf (OrderBySubQuery,"NumDegs DESC,FullName");
     }
   sprintf (Query,"(SELECT places.PlcCod,places.ShortName,places.FullName,COUNT(*) AS NumDegs" \
                  " FROM places,degrees" \
                  " WHERE places.PlcCod=degrees.PlcCod" \
                  " GROUP BY places.PlcCod)" \
                  " UNION " \
                  "(SELECT PlcCod,ShortName,FullName,0 AS NumDegs" \
                  " FROM places" \
                  " WHERE PlcCod NOT IN (SELECT DISTINCT PlcCod FROM degrees))" \
                  " ORDER BY %s",
            OrderBySubQuery);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get places");

   /***** Count number of rows in result *****/
   if (NumRows) // Places found...
     {
      // NumRows should be equal to Deg->NumCourses
      Gbl.Plcs.Num = (unsigned) NumRows;

      /***** Create list with courses in degree *****/
      if ((Gbl.Plcs.Lst = (struct Place *) calloc (NumRows,sizeof (struct Place))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store places.");

      /***** Get the places *****/
      for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
        {
         Plc = &(Gbl.Plcs.Lst[NumPlc]);

         /* Get next place */
         row = mysql_fetch_row (mysql_res);

         /* Get place code (row[0]) */
         if ((Plc->PlcCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of place.");

         /* Get the short name of the place (row[1]) */
         strcpy (Plc->ShortName,row[1]);

         /* Get the full name of the place (row[2]) */
         strcpy (Plc->FullName,row[2]);

         /* Get number of degrees in this place (row[3]) */
         if (sscanf (row[3],"%u",&Plc->NumDegs) != 1)
            Plc->NumDegs = 0;
        }
     }
   else
      Gbl.Plcs.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);   
  }

/*****************************************************************************/
/**************************** Get place full name ****************************/
/*****************************************************************************/

void Plc_GetDataOfPlaceByCod (struct Place *Plc)
  {
   extern const char *Txt_Another_department[Txt_NUM_LANGUAGES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Clear data *****/
   Plc->ShortName[0] = Plc->FullName[0] = '\0';
   Plc->NumDegs = 0;

   /***** Check if place code is correct *****/
   if (Plc->PlcCod == 0)
     {
      strcpy (Plc->ShortName,Txt_Another_department[Gbl.Prefs.Language]);
      strcpy (Plc->FullName,Txt_Another_department[Gbl.Prefs.Language]);
     }
   else if (Plc->PlcCod > 0)
     {
      /***** Get data of a place from database *****/
      sprintf (Query,"(SELECT places.ShortName,places.FullName,COUNT(*)" \
                     " FROM places,degrees" \
                     " WHERE places.PlcCod='%ld' AND places.PlcCod=degrees.PlcCod" \
                     " GROUP BY places.PlcCod)"
                     " UNION " \
                     "(SELECT ShortName,FullName,0" \
                     " FROM places" \
                     " WHERE PlcCod='%ld'" \
                     " AND PlcCod NOT IN (SELECT DISTINCT PlcCod FROM degrees))",
               Plc->PlcCod,Plc->PlcCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a place");

      /***** Count number of rows in result *****/
      if (NumRows) // Place found...
        {
         /* Get row */
         row = mysql_fetch_row (mysql_res);

         /* Get the short name of the place (row[0]) */
         strcpy (Plc->ShortName,row[0]);

         /* Get the full name of the place (row[1]) */
         strcpy (Plc->FullName,row[1]);

         /* Get number of degrees in this place (row[2]) */
         if (sscanf (row[2],"%u",&Plc->NumDegs) != 1)
            Plc->NumDegs = 0;
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/**************************** Free list of places ****************************/
/*****************************************************************************/

void Plc_FreeListPlaces (void)
  {
   if (Gbl.Plcs.Lst)
     {
      /***** Free memory used by the list of courses in degree *****/
      free ((void *) Gbl.Plcs.Lst);
      Gbl.Plcs.Lst = NULL;
      Gbl.Plcs.Num = 0;
     }
  }

/****************************************************************************/
/*************************** List all the places ****************************/
/****************************************************************************/

static void Plc_ListPlacesForEdition (void)
  {
   extern const char *Txt_Remove_place[Txt_NUM_LANGUAGES];
   unsigned NumPlc;
   struct Place *Plc;

   /***** Table head *****/
   Plc_PutHeadPlaces ();

   /***** Write all the places *****/
   for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
     {
      Plc = &Gbl.Plcs.Lst[NumPlc];

      /* Put icon to remove place */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      if (Plc->NumDegs)	// Place has degres ==> deletion forbidden
         fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActRemPlc);
         Plc_PutParamPlcCod (Plc->PlcCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_place[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Txt_Remove_place[Gbl.Prefs.Language]);
        }
      fprintf (Gbl.F.Out,"</td>");

      /* Place code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Plc->PlcCod);

      /* Place short name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenPlcSho);
      Plc_PutParamPlcCod (Plc->PlcCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Plc_MAX_LENGTH_PLACE_SHORT_NAME,Plc->ShortName,Gbl.NumForm);

      /* Place full name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenPlcFul);
      Plc_PutParamPlcCod (Plc->PlcCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Plc_MAX_LENGTH_PLACE_FULL_NAME,Plc->FullName,Gbl.NumForm);

      /* Number of degrees */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td></tr>",
               Plc->NumDegs);
     }
  }

/****************************************************************************/
/******************** Write parameter with code of place ********************/
/****************************************************************************/

static void Plc_PutParamPlcCod (long PlcCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"PlcCod\" value=\"%ld\" />",PlcCod);
  }

/****************************************************************************/
/********************* Get parameter with code of place *********************/
/****************************************************************************/

long Plc_GetParamPlcCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of place *****/
   Par_GetParam (Par_PARAM_SINGLE,"PlcCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/******************************* Remove a place *****************************/
/****************************************************************************/

void Plc_RemovePlace (void)
  {
   extern const char *Txt_Place_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Place Plc;

   /***** Get place code *****/
   if ((Plc.PlcCod = Plc_GetParamPlcCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of place is missing.");

   /***** Get data of the place from database *****/
   Plc_GetDataOfPlaceByCod (&Plc);

   /***** Check if this place has degrees *****/
   if (Plc.NumDegs)	// Place has degrees ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar un lugar primero debe eliminar todas las titulaciones de ese lugar.");	// Need translation!
   else	// Place has no degrees ==> remove it
     {
      /***** Remove place *****/
      sprintf (Query,"DELETE FROM places WHERE PlcCod='%ld'",Plc.PlcCod);
      DB_QueryDELETE (Query,"can not remove a place");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Place_X_removed[Gbl.Prefs.Language],
               Plc.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Plc_EditPlaces ();
  }

/****************************************************************************/
/********************* Change the short name of a place *********************/
/****************************************************************************/

void Plc_RenamePlaceShort (void)
  {
   Plc_RenamePlace (Cte_SHORT_NAME);
  }

/****************************************************************************/
/********************* Change the full name of a place **********************/
/****************************************************************************/

void Plc_RenamePlaceFull (void)
  {
   Plc_RenamePlace (Cte_FULL_NAME);
  }

/****************************************************************************/
/************************ Change the name of a degree ***********************/
/****************************************************************************/

static void Plc_RenamePlace (Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_place_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Place *Plc;
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentPlcName = NULL;		// Initialized to avoid warning
   char NewPlcName[Plc_MAX_LENGTH_PLACE_FULL_NAME+1];

   Plc = &Gbl.Plcs.EditingPlc;
   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = Plc_MAX_LENGTH_PLACE_SHORT_NAME;
         CurrentPlcName = Plc->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = Plc_MAX_LENGTH_PLACE_FULL_NAME;
         CurrentPlcName = Plc->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the place */
   if ((Plc->PlcCod = Plc_GetParamPlcCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of place is missing.");

   /* Get the new name for the place */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewPlcName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewPlcName,MaxLength,true);

   /***** Get from the database the old names of the place *****/
   Plc_GetDataOfPlaceByCod (Plc);

   /***** Check if new name is empty *****/
   if (!NewPlcName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del lugar <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentPlcName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentPlcName,NewPlcName))	// Different names
        {
         /***** If place was in database... *****/
         if (Plc_CheckIfPlaceNameExists (ParamName,NewPlcName,Plc->PlcCod))
           {
            sprintf (Gbl.Message,Txt_The_place_X_already_exists[Gbl.Prefs.Language],
                     NewPlcName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE places SET %s='%s' WHERE PlcCod='%ld'",
                     FieldName,NewPlcName,Plc->PlcCod);
            DB_QueryUPDATE (Query,"can not update the name of a place");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El lugar <strong>%s</strong>" \
                                 " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     CurrentPlcName,NewPlcName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del lugar <strong>%s</strong> no ha cambiado.",	// Need translation!
                  CurrentPlcName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (CurrentPlcName,NewPlcName);
   Plc_EditPlaces ();
  }

/****************************************************************************/
/********************* Check if the name of place exists ********************/
/****************************************************************************/

static bool Plc_CheckIfPlaceNameExists (const char *FieldName,const char *Name,long PlcCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of places with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM places WHERE %s='%s' AND PlcCod<>'%ld'",
            FieldName,Name,PlcCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a place already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when getting if the name of a place already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************* Put a form to create a new place *********************/
/****************************************************************************/

static void Plc_PutFormToCreatePlace (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_place[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_place[Txt_NUM_LANGUAGES];
   struct Place *Plc;

   Plc = &Gbl.Plcs.EditingPlc;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"5\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_place[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewPlc);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Plc_PutHeadPlaces ();

   /***** New place *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_place[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Place code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Place short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"ShortName\" size=\"15\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            Plc_MAX_LENGTH_PLACE_SHORT_NAME,Plc->ShortName);

   /***** Place full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Plc_MAX_LENGTH_PLACE_FULL_NAME,Plc->FullName);
  }

/****************************************************************************/
/******************** Write header with fields of a place ******************/
/****************************************************************************/

static void Plc_PutHeadPlaces (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_Name[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">N&ordm;<br />degrees</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Short_Name[Gbl.Prefs.Language],
            Txt_Full_Name[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************* Receive form to create a new place ********************/
/****************************************************************************/

void Plc_RecFormNewPlace (void)
  {
   extern const char *Txt_The_place_X_already_exists[Txt_NUM_LANGUAGES];
   struct Place *Plc;

   Plc = &Gbl.Plcs.EditingPlc;

   /***** Get parameters from form *****/
   /* Get place short name */
   Par_GetParam (Par_PARAM_SINGLE,"ShortName",Plc->ShortName,Plc_MAX_LENGTH_PLACE_SHORT_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plc->ShortName,Plc_MAX_LENGTH_PLACE_SHORT_NAME,true);

   /* Get place full name */
   Par_GetParam (Par_PARAM_SINGLE,"FullName",Plc->FullName,Plc_MAX_LENGTH_PLACE_FULL_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plc->FullName,Plc_MAX_LENGTH_PLACE_FULL_NAME,true);

   if (Plc->ShortName[0] && Plc->FullName[0])	// If there's a place name
     {
      /***** If name of place was in database... *****/
      if (Plc_CheckIfPlaceNameExists ("ShortName",Plc->ShortName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_place_X_already_exists[Gbl.Prefs.Language],
                  Plc->ShortName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else if (Plc_CheckIfPlaceNameExists ("FullName",Plc->FullName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_place_X_already_exists[Gbl.Prefs.Language],
                  Plc->FullName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else	// Add new place to database
         Plc_CreatePlace (Plc);
     }
   else	// If there isn't a place name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre breve y el nombre completo del nuevo lugar.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Plc_EditPlaces ();
  }

/****************************************************************************/
/**************************** Create a new place ***************************/
/****************************************************************************/

static void Plc_CreatePlace (struct Place *Plc)
  {
   extern const char *Txt_Created_new_place_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new place *****/
   sprintf (Query,"INSERT INTO places (ShortName,FullName) VALUES ('%s','%s')",
            Plc->ShortName,Plc->FullName);
   DB_QueryINSERT (Query,"can not create place");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_place_X[Gbl.Prefs.Language],
            Plc->FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }
