// swad_plg.c: plugins called from SWAD using web services

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
TO DO:
Check if web service is called from an authorized IP.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <string.h>
#include <stdio.h>	// For fprintf
#include <stdlib.h>	// For calloc, free

#include "swad_boo.h"
#include "swad_cfg.h"
#include "swad_cns.h"
#include "swad_cry.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_plg.h"
#include "swad_ses.h"
#include "swad_txt.h"
#include "swad_ws.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Plg_PutFormToEditPlugins (void);
static void Plg_ListPluginsForEdition (void);
static void Plg_PutParamPlgCod (long PlgCod);
static void Plg_GetListPlugins (void);
static void Plg_PutFormToCreatePlugin (void);
static void Plg_PutHeadPlugins (void);
static bool Plg_CheckIfPluginNameExists (const char *Name,long PlgCod);
static void Plg_CreatePlugin (struct Plugin *Plg);

/****************************************************************************/
/************************** List available plugins **************************/
/****************************************************************************/

void Plg_ListPlugins (void)
  {
   extern const char *Txt_Plugin[Txt_NUM_LANGUAGES];
   unsigned NumPlg;
   struct Plugin *Plg;
   char URL[Cte_MAX_LENGTH_WWW+Ses_LENGTH_SESSION_ID+1];

   /***** Get list of plugins *****/
   Plg_GetListPlugins ();

   /***** Put link (form) to edit plugins *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
       Plg_PutFormToEditPlugins ();

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write table heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"TIT_TBL\" width=\"32\"></td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_Plugin[Gbl.Prefs.Language]);

   /***** Write all the plugins *****/
   for (NumPlg=0; NumPlg<Gbl.Plugins.Num; NumPlg++)
     {
      Plg = &(Gbl.Plugins.Lst[NumPlg]);

      sprintf (URL,"%s%s",Plg->URL,Gbl.Session.Id);

      /* Plugin logo */
      fprintf (Gbl.F.Out,"<tr>" \
                         "<td align=\"left\" class=\"DAT\" width=\"32\">" \
                         "<a href=\"%s\" class=\"DAT\" target=\"_blank\"><acronym title=\"%s\" class=\"DAT\">" \
                         "<img src=\"%s/%s/%s24x24.gif\" width=\"24\" height=\"24\" alt=\"%s\" />" \
                         "</acronym></a>" \
                         "</td>" \
                         "<td align=\"left\" class=\"DAT\">" \
                         "<a href=\"%s\" class=\"DAT\" target=\"_blank\"><acronym title=\"%s\" class=\"DAT\">" \
                         "%s</acronym></a></td>" \
                         "</tr>",
               URL,Plg->Name,
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_PLUGINS,Gbl.Plugins.Lst[NumPlg].Logo,Plg->Name,
               URL,Plg->Name,
               Plg->Name);
     }

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of plugins *****/
   Plg_FreeListPlugins ();
  }

/****************************************************************************/
/********************** Put a link (form) to edit plugins *******************/
/****************************************************************************/

static void Plg_PutFormToEditPlugins (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActEdiPlg);
   sprintf (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/************************** Put forms to edit plugins ***********************/
/****************************************************************************/

void Plg_EditPlugins (void)
  {
   extern const char *Txt_Plugins[Txt_NUM_LANGUAGES];

   /***** Get list of plugins *****/
   Plg_GetListPlugins ();

   /***** Help message *****/
   if (Gbl.Plugins.Num) // Plugins found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo complemento rellene el nombre del complemento, su descripci&oacute;n, su logo, su URL y su direcci&oacute;n IP," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar un complemento existente pulse el aspa roja.<br />" \
                          "Para modificar un complemento existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Plugins[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No plugins found
      Lay_ShowAlert (INFO,"No existen complementos.<br />" \
                          "Para a&ntilde;adir un complemento rellene el nombre del complemento, su descripci&oacute;n, su logo, su URL y su direcci&oacute;n IP," \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit plugins *****/
   /* Start table with plugins */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current plugins */
   Plg_ListPluginsForEdition ();

   /* Put a form to create a new plugin */
   Plg_PutFormToCreatePlugin ();

   /* End of table with plugins */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of plugins *****/
   Plg_FreeListPlugins ();
  }

/****************************************************************************/
/************************* Get list of current plugins **********************/
/****************************************************************************/

static void Plg_GetListPlugins (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumPlg;
   struct Plugin *Plg;

   /***** Get plugins from database *****/
   sprintf (Query,"SELECT PlgCod,Name,Description,Logo,URL,IP" \
                  " FROM plugins ORDER BY Name");
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get plugins");

   /***** Count number of rows in result *****/
   if (NumRows) // Plugins found...
     {
      Gbl.Plugins.Num = (unsigned) NumRows;

      /***** Create list with plugins *****/
      if ((Gbl.Plugins.Lst = (struct Plugin *) calloc ((size_t) Gbl.Plugins.Num,sizeof (struct Plugin))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store plugins.");

      /***** Get the plugins *****/
      for (NumPlg=0; NumPlg<Gbl.Plugins.Num; NumPlg++)
        {
         Plg = &(Gbl.Plugins.Lst[NumPlg]);

         /* Get next plugin */
         row = mysql_fetch_row (mysql_res);

         /* Get plugin code (row[0]) */
         if ((Plg->PlgCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of plugin.");

         /* Get the name of the plugin (row[1]) */
         strncpy (Plg->Name,row[1],MAX_LENGTH_PLUGIN_NAME);
         Plg->Name[MAX_LENGTH_PLUGIN_NAME] = '\0';

         /* Get the description of the plugin (row[2]) */
         strncpy (Plg->Description,row[2],MAX_LENGTH_PLUGIN_DESCRIPTION);
         Plg->Description[MAX_LENGTH_PLUGIN_DESCRIPTION] = '\0';

         /* Get the logo of the plugin (row[3]) */
         strncpy (Plg->Logo,row[3],MAX_LENGTH_PLUGIN_LOGO);
         Plg->Logo[MAX_LENGTH_PLUGIN_LOGO] = '\0';

         /* Get the URL of the plugin (row[4]) */
         strncpy (Plg->URL,row[4],Cte_MAX_LENGTH_WWW);
         Plg->URL[Cte_MAX_LENGTH_WWW] = '\0';

         /* Get the IP of the plugin (row[5]) */
         strncpy (Plg->IP,row[5],Cte_MAX_LENGTH_IP);
         Plg->IP[Cte_MAX_LENGTH_IP] = '\0';
        }
     }
   else
      Gbl.Plugins.Num = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);   
  }

/*****************************************************************************/
/*************************** Get data of a plugin ****************************/
/*****************************************************************************/

bool Plg_GetDataOfPluginByCod (struct Plugin *Plg)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool PluginFound;

   /***** Clear data *****/
   Plg->Name[0] = Plg->Description[0] = Plg->Logo[0] = Plg->URL[0] = Plg->IP[0] = '\0';

   /***** Check if plugin code is correct *****/
   if (Plg->PlgCod <= 0)
      return false;
   // Plg->PlgCod > 0

   /***** Get data of a plugin from database *****/
   sprintf (Query,"SELECT Name,Description,Logo,URL,IP" \
                  " FROM plugins" \
                  " WHERE PlgCod='%ld'",
            Plg->PlgCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a plugin");

   /***** Count number of rows in result *****/
   if (NumRows) // Plugin found...
     {
      PluginFound = true;

      /* Get row */
      row = mysql_fetch_row (mysql_res);

      /* Get the name of the plugin (row[0]) */
      strncpy (Plg->Name,row[0],MAX_LENGTH_PLUGIN_NAME);
      Plg->Name[MAX_LENGTH_PLUGIN_NAME] = '\0';

      /* Get the description of the plugin (row[1]) */
      strncpy (Plg->Description,row[1],MAX_LENGTH_PLUGIN_DESCRIPTION);
      Plg->Description[MAX_LENGTH_PLUGIN_DESCRIPTION] = '\0';

      /* Get the logo of the plugin (row[2]) */
      strncpy (Plg->Logo,row[2],MAX_LENGTH_PLUGIN_LOGO);
      Plg->Logo[MAX_LENGTH_PLUGIN_LOGO] = '\0';

      /* Get the URL of the plugin (row[3]) */
      strncpy (Plg->URL,row[3],Cte_MAX_LENGTH_WWW);
      Plg->URL[Cte_MAX_LENGTH_WWW] = '\0';

      /* Get the IP of the plugin (row[4]) */
      strncpy (Plg->IP,row[4],Cte_MAX_LENGTH_IP);
      Plg->IP[Cte_MAX_LENGTH_IP] = '\0';
     }
   else
      PluginFound = false;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return PluginFound;
  }

/*****************************************************************************/
/*************************** Free list of plugins ****************************/
/*****************************************************************************/

void Plg_FreeListPlugins (void)
  {
   if (Gbl.Plugins.Lst)
     {
      /***** Free memory used by the list of plugins *****/
      free ((void *) Gbl.Plugins.Lst);
      Gbl.Plugins.Lst = NULL;
      Gbl.Plugins.Num = 0;
     }
  }

/****************************************************************************/
/**************************** List all the plugins **************************/
/****************************************************************************/

static void Plg_ListPluginsForEdition (void)
  {
   extern const char *Txt_Remove_plugin[Txt_NUM_LANGUAGES];
   unsigned NumPlg;
   struct Plugin *Plg;

   /***** Table head *****/
   Plg_PutHeadPlugins ();

   /***** Write all the plugins *****/
   for (NumPlg=0; NumPlg<Gbl.Plugins.Num; NumPlg++)
     {
      Plg = &Gbl.Plugins.Lst[NumPlg];

      /* Put icon to remove plugin */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      Act_FormStart (ActRemPlg);
      Plg_PutParamPlgCod (Plg->PlgCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_plugin[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Txt_Remove_plugin[Gbl.Prefs.Language]);

      /* Plugin code */
      fprintf (Gbl.F.Out,"<td align=\"right\" class=\"DAT\">%ld&nbsp;</td>",
               Plg->PlgCod);

      /* Plugin logo */
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"20\"><img src=\"%s/%s/%s24x24.gif\" width=\"24\" height=\"24\" alt=\"%s\" /></td>",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_PLUGINS,Gbl.Plugins.Lst[NumPlg].Logo,Gbl.Plugins.Lst[NumPlg].Name);

      /* Plugin name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenPlg);
      Plg_PutParamPlgCod (Plg->PlgCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Name\" size=\"12\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_PLUGIN_NAME,Plg->Name,Gbl.NumForm);

      /* Plugin description */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgPlgDes);
      Plg_PutParamPlgCod (Plg->PlgCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Description\" size=\"40\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_PLUGIN_DESCRIPTION,Plg->Description,Gbl.NumForm);

      /* Plugin logo */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgPlgLog);
      Plg_PutParamPlgCod (Plg->PlgCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Logo\" size=\"8\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               MAX_LENGTH_PLUGIN_LOGO,Plg->Logo,Gbl.NumForm);

      /* Plugin URL */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgPlgURL);
      Plg_PutParamPlgCod (Plg->PlgCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"URL\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Cte_MAX_LENGTH_WWW,Plg->URL,Gbl.NumForm);

      /* Plugin IP */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgPlgIP);
      Plg_PutParamPlgCod (Plg->PlgCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"IP\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td></tr>",
               Cte_MAX_LENGTH_IP,Plg->IP,Gbl.NumForm);
     }
  }

/****************************************************************************/
/******************* Write parameter with code of plugin ********************/
/****************************************************************************/

static void Plg_PutParamPlgCod (long PlgCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"PlgCod\" value=\"%ld\" />",PlgCod);
  }

/****************************************************************************/
/********************* Get parameter with code of plugin ********************/
/****************************************************************************/

long Plg_GetParamPlgCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of plugin *****/
   Par_GetParam (Par_PARAM_SINGLE,"PlgCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/******************************* Remove a plugin ****************************/
/****************************************************************************/

void Plg_RemovePlugin (void)
  {
   extern const char *Txt_Plugin_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Plugin Plg;

   /***** Get plugin code *****/
   if ((Plg.PlgCod = Plg_GetParamPlgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of plugin is missing.");

   /***** Get data of the plugin from database *****/
   Plg_GetDataOfPluginByCod (&Plg);

   /***** Remove plugin *****/
   sprintf (Query,"DELETE FROM plugins WHERE PlgCod='%ld'",Plg.PlgCod);
   DB_QueryDELETE (Query,"can not remove a plugin");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Plugin_X_removed[Gbl.Prefs.Language],
            Plg.Name);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Plg_EditPlugins ();
  }

/****************************************************************************/
/************************ Change the name of a plugin ***********************/
/****************************************************************************/

void Plg_RenamePlugin (void)
  {
   extern const char *Txt_The_plugin_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   struct Plugin *Plg;
   char NewPlgName[MAX_LENGTH_PLUGIN_NAME+1];

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Get parameters from form *****/
   /* Get the code of the institution */
   if ((Plg->PlgCod = Plg_GetParamPlgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of plugin is missing.");

   /* Get the new name for the plugin */
   Par_GetParam (Par_PARAM_SINGLE,"Name",NewPlgName,MAX_LENGTH_PLUGIN_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewPlgName,MAX_LENGTH_PLUGIN_NAME,true);

   /***** Get from the database the old name of the plugin *****/
   Plg_GetDataOfPluginByCod (Plg);

   /***** Check if new name is empty *****/
   if (!NewPlgName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del complemento <strong>%s</strong> vac&iacute;o.",	// Need translation!
               Plg->Name);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (Plg->Name,NewPlgName))	// Different names
        {
         /***** If plugin was in database... *****/
         if (Plg_CheckIfPluginNameExists (NewPlgName,Plg->PlgCod))
           {
            sprintf (Gbl.Message,Txt_The_plugin_X_already_exists[Gbl.Prefs.Language],
                     NewPlgName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE plugins SET Name='%s' WHERE PlgCod='%ld'",
                     NewPlgName,Plg->PlgCod);
            DB_QueryUPDATE (Query,"can not update the name of a plugin");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El complemento <strong>%s</strong>" \
                                 " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     Plg->Name,NewPlgName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del complemento <strong>%s</strong> no ha cambiado.",	// Need translation!
                  Plg->Name);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (Plg->Name,NewPlgName);
   Plg_EditPlugins ();
  }

/****************************************************************************/
/****************** Check if the name of institution exists *****************/
/****************************************************************************/

static bool Plg_CheckIfPluginNameExists (const char *Name,long PlgCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of plugins with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM plugins WHERE Name='%s' AND PlgCod<>'%ld'",
            Name,PlgCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a plugin already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when getting if the name of a plugin already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/******************* Change the description of a plugin *********************/
/****************************************************************************/

void Plg_ChangePlgDescription (void)
  {
   struct Plugin *Plg;
   char Query[256+MAX_LENGTH_PLUGIN_DESCRIPTION];
   char NewDescription[MAX_LENGTH_PLUGIN_DESCRIPTION+1];

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Get parameters from form *****/
   /* Get the code of the plugin */
   if ((Plg->PlgCod = Plg_GetParamPlgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of plugin is missing.");

   /* Get the new description for the plugin */
   Par_GetParam (Par_PARAM_SINGLE,"Description",NewDescription,MAX_LENGTH_PLUGIN_DESCRIPTION);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewDescription,MAX_LENGTH_PLUGIN_DESCRIPTION,true);

   /***** Check if new description is empty *****/
   if (NewDescription[0])
     {
      /* Update the table changing old description by new description */
      sprintf (Query,"UPDATE plugins SET Description='%s' WHERE PlgCod='%ld'",
               NewDescription,Plg->PlgCod);
      DB_QueryUPDATE (Query,"can not update the description of a plugin");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva descripci&oacute;n es <strong>%s</strong>.",	// Need translation!
               NewDescription);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar la descripci&oacute;n vac&iacute;a.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Plg->Description,NewDescription);
   Plg_EditPlugins ();
  }

/****************************************************************************/
/************************ Change the logo of a plugin ***********************/
/****************************************************************************/

void Plg_ChangePlgLogo (void)
  {
   struct Plugin *Plg;
   char Query[256+MAX_LENGTH_PLUGIN_LOGO];
   char NewLogo[MAX_LENGTH_PLUGIN_LOGO+1];

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Get parameters from form *****/
   /* Get the code of the plugin */
   if ((Plg->PlgCod = Plg_GetParamPlgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of plugin is missing.");

   /* Get the new logo for the plugin */
   Par_GetParam (Par_PARAM_SINGLE,"Logo",NewLogo,MAX_LENGTH_PLUGIN_LOGO);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewLogo,MAX_LENGTH_PLUGIN_LOGO,true);

   /***** Check if new logo is empty *****/
   if (NewLogo[0])
     {
      /* Update the table changing old logo by new logo */
      sprintf (Query,"UPDATE plugins SET Logo='%s' WHERE PlgCod='%ld'",
               NewLogo,Plg->PlgCod);
      DB_QueryUPDATE (Query,"can not update the logo of a plugin");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"El nuevo logo es <strong>%s</strong>.",	// Need translation!
               NewLogo);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar el logo vac&iacute;o.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Plg->Logo,NewLogo);
   Plg_EditPlugins ();
  }

/****************************************************************************/
/************************ Change the URL of a plugin ************************/
/****************************************************************************/

void Plg_ChangePlgURL (void)
  {
   struct Plugin *Plg;
   char Query[256+Cte_MAX_LENGTH_WWW];
   char NewURL[Cte_MAX_LENGTH_WWW+1];

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Get parameters from form *****/
   /* Get the code of the plugin */
   if ((Plg->PlgCod = Plg_GetParamPlgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of plugin is missing.");

   /* Get the new URL for the plugin */
   Par_GetParam (Par_PARAM_SINGLE,"URL",NewURL,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewURL,Cte_MAX_LENGTH_WWW,true);

   /***** Check if new URL is empty *****/
   if (NewURL[0])
     {
      /* Update the table changing old WWW by new WWW */
      sprintf (Query,"UPDATE plugins SET URL='%s' WHERE PlgCod='%ld'",
               NewURL,Plg->PlgCod);
      DB_QueryUPDATE (Query,"can not update the URL of a plugin");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"El nuevo URL es <strong>%s</strong>.",	// Need translation!
               NewURL);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar el URL vac&iacute;o.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Plg->URL,NewURL);
   Plg_EditPlugins ();
  }

/****************************************************************************/
/************************* Change the IP of a plugin ************************/
/****************************************************************************/

void Plg_ChangePlgIP (void)
  {
   struct Plugin *Plg;
   char Query[256+Cte_MAX_LENGTH_IP];
   char NewIP[Cte_MAX_LENGTH_IP+1];

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Get parameters from form *****/
   /* Get the code of the plugin */
   if ((Plg->PlgCod = Plg_GetParamPlgCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of plugin is missing.");

   /* Get the new IP for the plugin */
   Par_GetParam (Par_PARAM_SINGLE,"IP",NewIP,Cte_MAX_LENGTH_IP);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewIP,Cte_MAX_LENGTH_IP,true);

   /***** Check if new IP is empty *****/
   if (NewIP[0])
     {
      /* Update the table changing old IP by new IP */
      sprintf (Query,"UPDATE plugins SET IP='%s' WHERE PlgCod='%ld'",
               NewIP,Plg->PlgCod);
      DB_QueryUPDATE (Query,"can not update the IP address of a plugin");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva direcci&oacute;n IP es <strong>%s</strong>.",	// Need translation!
               NewIP);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar la direcci&oacute;n IP vac&iacute;a.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Plg->URL,NewIP);
   Plg_EditPlugins ();
  }

/****************************************************************************/
/********************* Put a form to create a new plugin ********************/
/****************************************************************************/

static void Plg_PutFormToCreatePlugin (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_plugin[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_plugin[Txt_NUM_LANGUAGES];
   struct Plugin *Plg;

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"8\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"8\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_plugin[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewPlg);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Plg_PutHeadPlugins ();

   /***** New plugin *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_plugin[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Plugin code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Plugin logo *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"20\"></td>");

   /***** Plugin name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"Name\" size=\"12\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_PLUGIN_NAME,Plg->Name);

   /***** Plugin description *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"Description\" size=\"40\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_PLUGIN_DESCRIPTION,Plg->Description);

   /***** Plugin logo *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"Logo\" size=\"8\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            MAX_LENGTH_PLUGIN_LOGO,Plg->Logo);

   /***** Plugin URL *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"URL\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            Cte_MAX_LENGTH_WWW,Plg->URL);

   /***** Plugin IP address *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"IP\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td></tr></table></form></td></tr>\n",
            Cte_MAX_LENGTH_IP,Plg->IP);
  }

/****************************************************************************/
/******************** Write header with fields of a plugin ******************/
/****************************************************************************/

static void Plg_PutHeadPlugins (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Description[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" class=\"TIT_TBL\">&nbsp;</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\" width=\"28\">&nbsp;</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">Logo</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">URL</td>" \
                      "<td align=\"left\" class=\"TIT_TBL\">IP</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Name[Gbl.Prefs.Language],
            Txt_Description[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****************** Receive form to create a new plugin *********************/
/****************************************************************************/

void Plg_RecFormNewPlg (void)
  {
   extern const char *Txt_The_plugin_X_already_exists[Txt_NUM_LANGUAGES];
   struct Plugin *Plg;

   Plg = &Gbl.Plugins.EditingPlg;

   /***** Get parameters from form *****/
   /* Get plugin name */
   Par_GetParam (Par_PARAM_SINGLE,"Name",Plg->Name,MAX_LENGTH_PLUGIN_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plg->Name,MAX_LENGTH_PLUGIN_NAME,true);

   /* Get plugin description */
   Par_GetParam (Par_PARAM_SINGLE,"Description",Plg->Description,MAX_LENGTH_PLUGIN_DESCRIPTION);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plg->Description,MAX_LENGTH_PLUGIN_DESCRIPTION,true);

   /* Get plugin logo */
   Par_GetParam (Par_PARAM_SINGLE,"Logo",Plg->Logo,MAX_LENGTH_PLUGIN_LOGO);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plg->Logo,MAX_LENGTH_PLUGIN_LOGO,true);

   /* Get plugin URL */
   Par_GetParam (Par_PARAM_SINGLE,"URL",Plg->URL,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plg->URL,Cte_MAX_LENGTH_WWW,true);

   /* Get plugin IP address */
   Par_GetParam (Par_PARAM_SINGLE,"IP",Plg->IP,Cte_MAX_LENGTH_IP);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Plg->IP,Cte_MAX_LENGTH_IP,true);

   if (Plg->Name[0])	// If there's a plugin name
     {
      if (Plg->Logo[0] && Plg->URL[0] && Plg->IP[0])
        {
         /***** If name of plugin was in database... *****/
         if (Plg_CheckIfPluginNameExists (Plg->Name,-1L))
           {
            sprintf (Gbl.Message,Txt_The_plugin_X_already_exists[Gbl.Prefs.Language],
                     Plg->Name);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else	// Add new plugin to database
            Plg_CreatePlugin (Plg);
        }
      else	// If there isn't a logo, a URL or a IP
        {
         sprintf (Gbl.Message,"Debe especificar el logo, el URL y la IP del nuevo complemento.");	// Need translation!
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else	// If there isn't a plugin name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Plg_EditPlugins ();
  }

/****************************************************************************/
/***************************** Create a new plugin **************************/
/****************************************************************************/

static void Plg_CreatePlugin (struct Plugin *Plg)
  {
   extern const char *Txt_Created_new_plugin_X[Txt_NUM_LANGUAGES];
   char Query[512+MAX_LENGTH_PLUGIN_NAME+MAX_LENGTH_PLUGIN_DESCRIPTION+MAX_LENGTH_PLUGIN_LOGO+Cte_MAX_LENGTH_WWW+Cte_MAX_LENGTH_IP];

   /***** Create a new plugin *****/
   sprintf (Query,"INSERT INTO plugins (Name,Description,Logo,URL,IP)" \
                  " VALUES ('%s','%s','%s','%s','%s')",
            Plg->Name,Plg->Description,Plg->Logo,Plg->URL,Plg->IP);
   DB_QueryINSERT (Query,"can not create plugin");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_plugin_X[Gbl.Prefs.Language],
            Plg->Name);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/******* Function called when a web service if required by a plugin *********/
/****************************************************************************/

void Plg_WebService (void)
  {
   /***** Call soap service *****/
   WS_WebService ();

   /***** All the output is made, so don't write anymore *****/
   Gbl.TablEndWritten = Gbl.HTMLEndWritten = true;
  }

