// swad_prf.c: user's preferences

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdio.h>	// For fprintf, etc.
#include <string.h>

#include "swad_boo.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_lay.h"
#include "swad_ntf.h"
#include "swad_par.h"
#include "swad_prf.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Prf_UpdateSideColsOnUsrDataTable (void);
static bool Prf_GetParamSecureIcons (void);
static void Prf_PutIconsToSelectSecureIcons (void);
static void Prf_UpdateSecureIconsOnUsrDataTable (void);
static void Prf_PutFormPublicPhoto (void);

/****************************************************************************/
/**************************** Edit preferences ******************************/
/****************************************************************************/

void Prf_EditPrefs (void)
  {
   extern const char *Txt_Language[Txt_NUM_LANGUAGES];
   extern const char *Txt_Design[Txt_NUM_LANGUAGES];
   extern const char *Txt_Columns[Txt_NUM_LANGUAGES];
   extern const char *Txt_Icons[Txt_NUM_LANGUAGES];

   /***** Language selection *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td></tr>" \
                      "<tr><td align=\"center\">",Txt_Language[Gbl.Prefs.Language]);
   Prf_PutSelectorToSelectLanguage ();
   fprintf (Gbl.F.Out,"</td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Theme selection *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td></tr>" \
                      "<tr><td align=\"center\">",Txt_Design[Gbl.Prefs.Language]);
   The_PutIconsToSelectTheme ();
   fprintf (Gbl.F.Out,"</td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Side columns selection *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td></tr>" \
                      "<tr><td align=\"center\">",Txt_Columns[Gbl.Prefs.Language]);
   Prf_PutIconsToSelectSideCols ();
   fprintf (Gbl.F.Out,"</td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Secure / unsecure icons selection *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td></tr>" \
                      "<tr><td align=\"center\">",Txt_Icons[Gbl.Prefs.Language]);
   Prf_PutIconsToSelectSecureIcons ();
   fprintf (Gbl.F.Out,"</td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   if (Gbl.Usrs.Me.Logged)
     {
      /***** Public / private photo *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
      fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
      Prf_PutFormPublicPhoto ();
      fprintf (Gbl.F.Out,"</td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Automatic e-mail to notify of new messages *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
      fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
      Ntf_PutFormNotifyNewMsgs ();
      fprintf (Gbl.F.Out,"</td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
  }

/****************************************************************************/
/****************** Get preferences changed from current IP *****************/
/****************************************************************************/

void Prf_GetPrefsFromIP (void)
  {
   extern const char *Txt_StrLangId[Txt_NUM_LANGUAGES];
   char Query[1024];
   unsigned long NumRows;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   Txt_Language_t Lan;

   if (!Gbl.IP[0])
      return;

   /***** Get preferences from database *****/
   sprintf (Query,"SELECT Theme,Language,SideCols" \
                  " FROM IP_prefs WHERE IP='%s'",Gbl.IP);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get preferences");

   if (NumRows == 0)
      return;

   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Internal error while getting preferences.");

   /***** Get preferences from database *****/
   row = mysql_fetch_row (mysql_res);

   /* Get theme (row[0]) */
   Gbl.Prefs.Theme = The_GetThemeFromStr (row[0]);

   /* Get language (row[1]) */
   Gbl.Prefs.Language = Txt_LANGUAGE_DEFAULT;	// Set default language
   for (Lan=(Txt_Language_t) 0; Lan<Txt_NUM_LANGUAGES; Lan++)
      if (!strcasecmp (row[1],Txt_StrLangId[Lan]))
        {
	 Gbl.Prefs.Language = Lan;
         break;
        }

   /* Get if user wants to show side columns (row[2]) */
   if (sscanf (row[2],"%u",&Gbl.Prefs.SideCols) == 1)
     {
      if (Gbl.Prefs.SideCols > 3)
         Gbl.Prefs.SideCols = 3;	// Show both side columns
     }
   else
      Gbl.Prefs.SideCols = 3;	// Show both side columns
  }

/****************************************************************************/
/*********************** Set preferences from current IP ********************/
/****************************************************************************/

void Prf_SetPrefsFromIP (void)
  {
   extern const char *Txt_StrLangId[Txt_NUM_LANGUAGES];
   extern const char *The_ThemeId[The_NUM_THEMES];
   char Query[512];

   if (!(Gbl.PID % 100))	// Do this only one of 100 times
     {
      /***** Remove old preferences *****/
      sprintf (Query,"DELETE LOW_PRIORITY FROM IP_prefs" \
                     " WHERE UNIX_TIMESTAMP() > UNIX_TIMESTAMP(LastChange)+%ld",
               Cfg_TIME_TO_DELETE_IP_PREFS);
      DB_QueryDELETE (Query,"can not remove old preferences");
     }

   /***** Update preferences from current IP in database *****/
   sprintf (Query,"REPLACE INTO IP_prefs (IP,UsrCod,LastChange,Theme,Language,SideCols)" \
                  " VALUES ('%s','%ld',NOW(),'%s','%s','%u')",
            Gbl.IP,Gbl.Usrs.Me.UsrDat.UsrCod,
            The_ThemeId[Gbl.Prefs.Theme],
            Txt_StrLangId[Gbl.Prefs.Language],
            Gbl.Prefs.SideCols);
   DB_QueryREPLACE (Query,"can not store preferences from current IP address");

   /***** If a user is logged, update its preferences in database for all its IP's *****/
   if (Gbl.Usrs.Me.Logged)
     {
      sprintf (Query,"UPDATE IP_prefs SET Theme='%s',Language='%s',SideCols='%u'" \
                     " WHERE UsrCod='%ld'",
               The_ThemeId[Gbl.Prefs.Theme],
               Txt_StrLangId[Gbl.Prefs.Language],
               Gbl.Prefs.SideCols,
               Gbl.Usrs.Me.UsrDat.UsrCod);
      DB_QueryUPDATE (Query,"can not update your preferences");
     }
  }

/****************************************************************************/
/********************* Put a selector to select language ********************/
/****************************************************************************/

void Prf_PutSelectorToSelectLanguage (void)
  {
   extern const char *Txt_StrLangName[Txt_NUM_LANGUAGES];
   Txt_Language_t Lan;

   Act_FormStart (ActChgLan);
   fprintf (Gbl.F.Out,"<select name=\"Lan\" style=\"width:120px;\" onchange=\"javascript:document.forms[%d].submit();\">",
            Gbl.NumForm);
   for (Lan=(Txt_Language_t) 0; Lan<Txt_NUM_LANGUAGES; Lan++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Lan);
      if (Lan == Gbl.Prefs.Language)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_StrLangName[Lan]);
     }
   fprintf (Gbl.F.Out,"</select></form>");
  }

/****************************************************************************/
/****************************** Change language *****************************/
/****************************************************************************/

void Prf_ChangeLanguage (void)
  {
   extern const char *Txt_StrLangId[Txt_NUM_LANGUAGES];
   char Query[512];

   /***** Get param language *****/
   Gbl.Prefs.Language = Prf_GetParamLanguage ();

   /***** Store language in database *****/
   if (Gbl.Usrs.Me.Logged)
     {
      sprintf (Query,"UPDATE usr_data SET Language='%s' WHERE UsrCod='%ld'",
               Txt_StrLangId[Gbl.Prefs.Language],Gbl.Usrs.Me.UsrDat.UsrCod);
      DB_QueryUPDATE (Query,"can not update your language");
     }

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/************************** Get parameter language **************************/
/****************************************************************************/

Txt_Language_t Prf_GetParamLanguage (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Lan",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      if (UnsignedNum < Txt_NUM_LANGUAGES)
         return (Txt_Language_t) UnsignedNum;

   return Txt_LANGUAGE_DEFAULT;
  }

/****************************************************************************/
/*********** Put icons to select the layout of the side columns *************/
/****************************************************************************/

void Prf_PutIconsToSelectSideCols (void)
  {
   unsigned SideCols;
   extern const char *Txt_LAYOUT_SIDE_COLUMNS[4][Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"1\"><tr>");
   for (SideCols=0; SideCols<=3; SideCols++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"%s\">",
               (SideCols == Gbl.Prefs.SideCols) ? "LAYOUT_ON" : "LAYOUT_OFF");
      Act_FormStart (ActChgCol);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"SideCols\" value=\"%u\" />",SideCols);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_LAYOUT_SIDE_COLUMNS[SideCols][Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/layout%u%u_32x20.gif\" width=\"32\" height=\"20\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,SideCols >> 1,SideCols & 1,Txt_LAYOUT_SIDE_COLUMNS[SideCols][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</tr></table>");
  }

/****************************************************************************/
/*************** Put left icon to hide/show side columns *******************/
/****************************************************************************/

void Prf_PutLeftIconToHideShowCols (void)
  {
   if (Gbl.Prefs.SideCols & 2)
     {
      Act_FormStart (ActHidLftCol);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,"Ocultar columna izquierda",NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/central_left_8x400.gif\" width=\"8\" height=\"400\" alt=\"\" />",
               Gbl.Prefs.PathTheme);
     }
   else
     {
      Act_FormStart (ActShoLftCol);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,"Mostrar columna izquierda",NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/central_right_8x400.gif\" width=\"8\" height=\"400\" alt=\"\" />",
               Gbl.Prefs.PathTheme);
     }
   fprintf (Gbl.F.Out,"</acronym></a></form>");
  }

/****************************************************************************/
/*************** Put right icon to hide/show side columns *******************/
/****************************************************************************/

void Prf_PutRigthIconToHideShowCols (void)
  {
   if (Gbl.Prefs.SideCols & 1)
     {
      Act_FormStart (ActHidRgtCol);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,"Ocultar columna derecha",NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/central_right_8x400.gif\" width=\"8\" height=\"400\" alt=\"\" />",
               Gbl.Prefs.PathTheme);
     }
   else
     {
      Act_FormStart (ActShoRgtCol);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,"Mostrar columna derecha",NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/central_left_8x400.gif\" width=\"8\" height=\"400\" alt=\"\" />",
               Gbl.Prefs.PathTheme);
     }
   fprintf (Gbl.F.Out,"</acronym></a></form>");
  }

/****************************************************************************/
/********************** Change layout of side columns ***********************/
/****************************************************************************/

void Prf_ChangeSideCols (void)
  {
   /***** Get param side-columns *****/
   Gbl.Prefs.SideCols = Prf_GetParamSideCols ();

   /***** Store side colums in database *****/
   if (Gbl.Usrs.Me.Logged)
      Prf_UpdateSideColsOnUsrDataTable ();

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/************************** Hide left side column ***************************/
/****************************************************************************/

void Prf_HideLeftCol (void)
  {
   Gbl.Prefs.SideCols &= 1;	//  And with 01 to hide left column
   if (Gbl.Usrs.Me.Logged)
      Prf_UpdateSideColsOnUsrDataTable ();

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/************************** Hide right side column **************************/
/****************************************************************************/

void Prf_HideRightCol (void)
  {
   Gbl.Prefs.SideCols &= 2;	//  And with 10 to hide right column
   if (Gbl.Usrs.Me.Logged)
      Prf_UpdateSideColsOnUsrDataTable ();

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/*************************** Show left side column **************************/
/****************************************************************************/

void Prf_ShowLeftCol (void)
  {
   Gbl.Prefs.SideCols |= 2;	// Or with 10 to show left column
   if (Gbl.Usrs.Me.Logged)
      Prf_UpdateSideColsOnUsrDataTable ();

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/*************************** Show right side column *************************/
/****************************************************************************/

void Prf_ShowRightCol (void)
  {
   Gbl.Prefs.SideCols |= 1;	// Or with 01 to show right column
   if (Gbl.Usrs.Me.Logged)
      Prf_UpdateSideColsOnUsrDataTable ();

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/************* Update layout of side colums on user data table **************/
/****************************************************************************/

static void Prf_UpdateSideColsOnUsrDataTable (void)
  {
   char Query[512];

   sprintf (Query,"UPDATE usr_data SET SideCols='%u' WHERE UsrCod='%ld'",
            Gbl.Prefs.SideCols,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update your preference about side columns");
  }

/****************************************************************************/
/************* Get parameter used to show/hide side columns *****************/
/****************************************************************************/

unsigned Prf_GetParamSideCols (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;	// 11 ==> by default, show both side columns

   Par_GetParam (Par_PARAM_SINGLE,"SideCols",UnsignedStr,10);
 
   if (!UnsignedStr[0])
      return 3;

   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      return 3;

   if (UnsignedNum > 3)
      return 3;

   return UnsignedNum;
  }

/****************************************************************************/
/************** Put icons to select secure / unsecure icons *****************/
/****************************************************************************/

static void Prf_PutIconsToSelectSecureIcons (void)
  {
   /***** Start form *****/
   Act_FormStart (ActChgSecIco);
   fprintf (Gbl.F.Out,"<table cellspacing=\"2\" cellpadding=\"0\">");

   /***** Unsecure icons (http) *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">" \
                      "<input type=\"radio\" name=\"SecIco\" value=\"N\" onclick=\"javascript:document.forms[%d].submit();\"",
            Gbl.NumForm);
   if (!Gbl.Prefs.SecureIcons)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (!Gbl.Usrs.Me.Logged)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," />http</td></tr>");

   /***** Secure icons (https) *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">" \
                      "<input type=\"radio\" name=\"SecIco\" value=\"Y\" onclick=\"javascript:document.forms[%d].submit();\"",
            Gbl.NumForm);
   if (Gbl.Prefs.SecureIcons)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (!Gbl.Usrs.Me.Logged)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," />https</td></tr>");

   /***** End form *****/
   fprintf (Gbl.F.Out,"</table></form>\n");
  }

/****************************************************************************/
/********************** Change secure / unsecure icons **********************/
/****************************************************************************/

void Prf_ChangeSecureIcons (void)
  {
   /***** Get param side-columns *****/
   Gbl.Prefs.SecureIcons = Prf_GetParamSecureIcons ();
   Prf_SetIconsURL ();

   /***** Store side colums in database *****/
   if (Gbl.Usrs.Me.Logged)
      Prf_UpdateSecureIconsOnUsrDataTable ();
  }

/****************************************************************************/
/*************** Set URL for icons *****************/
/****************************************************************************/

void Prf_SetIconsURL (void)
  {
   sprintf (Gbl.Prefs.IconsURL,"%s://%s/%s",
            Gbl.Prefs.SecureIcons ? "https" : "http",Cfg_PLATFORM_URL,Cfg_FOLDER_PUBLIC_ICON);
  }

/****************************************************************************/
/*************** Get parameter with secure / unsecure icons *****************/
/****************************************************************************/

static bool Prf_GetParamSecureIcons (void)
  {
   char YN[1+1];

   Par_GetParam (Par_PARAM_SINGLE,"SecIco",YN,1);
   return (Str_ConvertToUpperLetter (YN[0]) == 'Y');
  }

/****************************************************************************/
/*********** Update secure / unsecure icons on user data table **************/
/****************************************************************************/

static void Prf_UpdateSecureIconsOnUsrDataTable (void)
  {
   char Query[512];

   sprintf (Query,"UPDATE usr_data SET SecureIcons='%c' WHERE UsrCod='%ld'",
            Gbl.Prefs.SecureIcons ? 'Y' : 'N',
            Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update your preference about secure icons");
  }

/****************************************************************************/
/********************** Select public / private photo ***********************/
/****************************************************************************/

static void Prf_PutFormPublicPhoto (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Public_Photo[Txt_NUM_LANGUAGES];

   /***** Start form *****/
   Act_FormStart (ActChgPubPho);
   fprintf (Gbl.F.Out,"<table cellspacing=\"2\" cellpadding=\"0\"><tr>");

   /***** Checkbox to select between public or private photo *****/
   fprintf (Gbl.F.Out,"<td align=\"right\">" \
                      "<input type=\"checkbox\" name=\"PublicPhoto\" value=\"Y\"");
   if (Gbl.Usrs.Me.UsrDat.PublicPhoto)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\" /></td>",
            Gbl.NumForm);
   fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">%s</td>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Public_Photo[Gbl.Prefs.Language]);

   /***** End form *****/
   fprintf (Gbl.F.Out,"</tr></table></form>\n");
  }

/****************************************************************************/
/********** Get parameter with public / private photo from form *************/
/****************************************************************************/

bool Prf_GetParamPublicPhoto (void)
  {
   char YN[1+1];

   Par_GetParam (Par_PARAM_SINGLE,"PublicPhoto",YN,1);
   return (Str_ConvertToUpperLetter (YN[0]) == 'Y');
  }

/****************************************************************************/
/*********************** Change public / private photo **********************/
/****************************************************************************/

void Prf_ChangePublicPhoto (void)
  {
   char Query[512];

   /***** Get param with public/private photo *****/
   Gbl.Usrs.Me.UsrDat.PublicPhoto = Prf_GetParamPublicPhoto ();

   /***** Store public/private photo in database *****/
   sprintf (Query,"UPDATE usr_data SET PublicPhoto='%c' WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.PublicPhoto ? 'Y' : 'N',
            Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update your preference about public photo");
  }
