// swad_rec.c: users' record cards

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>	// For calloc
#include <string.h>

#include "swad_act.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_rec.h"
#include "swad_usr.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Actions Actions[Act_NUM_ACTIONS];
extern const char *StringsGenderDB[Usr_NUM_GENDERS];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static void Rec_GetFieldByCod (long FieldCod,char *Name,unsigned *NumLines,Rec_VisibilityRecordFields_t *Visibility);
static void Rec_ShowCrsRecordAfterUpdate (Rec_RecordViewType_t TypeOfView,struct UsrData *UsrDat);

/****************************************************************************/
/*************** Create, edit and remove fields of records ******************/
/****************************************************************************/

void Rec_ReqEditRecordFields (void)
  {
   /***** Form to edit the fields of the records *****/
   /* Get list of fields of records in current course */
   Rec_GetListRecordFieldsInCurrentCrs ();
   if (Gbl.CurrentCrs.Records.LstFields.Num) // Fields found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo campo rellene el nombre del campo, el n&ordm; de l&iacute;neas del formulario y la visibilidad del campo;"
                          " despu&eacute;s pulse el icono verde.<br />" \
                          "Para eliminar un campo existente pulse el aspa roja.<br />" \
                          "Para modificar un campo existente," \
                          " ed&iacute;te las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"Campos de fichas en <strong>%s</strong>:",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No fields of records found for current course in the database
     {
      sprintf (Gbl.Message,"No existen campos de fichas en <strong>%s</strong>.<br />" \
                           "Para a&ntilde;adir un campo rellene el nombre del campo," \
                           " la visibilidad y editabilidad, y pulse el icono verde.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /* Start table */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List the current fields of records for edit them */
   Rec_ListFieldsRecordsForEdition ();

   /* Show form for the creacin of a new group */
   Rec_ShowFormCreateRecordField ();

   /* End table */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /* Free list of fields of records */
   Rec_FreeListFields ();
  }

/****************************************************************************/
/****** Create a list with the fields of records from current course ********/
/****************************************************************************/

void Rec_GetListRecordFieldsInCurrentCrs (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow;
   unsigned Vis;

   if (++Gbl.CurrentCrs.Records.LstFields.NestedCalls > 1) // If the list is already created, don't do anything
      return;

   /***** Get fields of cards of a course from database *****/
   sprintf (Query,"SELECT FieldCod,FieldName,NumLines,Visibility FROM crs_record_fields" \
                  " WHERE CrsCod='%ld' ORDER BY FieldName",
            Gbl.CurrentCrs.Crs.CrsCod);
   Gbl.CurrentCrs.Records.LstFields.Num = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get fields of cards of a course");

   /***** Get the fields of records *****/
   if (Gbl.CurrentCrs.Records.LstFields.Num)
     {
      /***** Create a list of fields *****/
      if ((Gbl.CurrentCrs.Records.LstFields.Lst = (struct RecordField *) calloc (Gbl.CurrentCrs.Records.LstFields.Num,sizeof (struct RecordField))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store fields of records in current course.");

      /***** Get the fields *****/
      for (NumRow=0; NumRow<Gbl.CurrentCrs.Records.LstFields.Num; NumRow++)
        {
         /* Get next field */
         row = mysql_fetch_row (mysql_res);

         /* Get the code of field (row[0]) */
         if ((Gbl.CurrentCrs.Records.LstFields.Lst[NumRow].FieldCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of field.");

         /* Name of the field (row[1]) */
         strncpy (Gbl.CurrentCrs.Records.LstFields.Lst[NumRow].Name,row[1],Rec_MAX_LENGTH_NAME_FIELD);

         /* Number of lines (row[2]) */
         Gbl.CurrentCrs.Records.LstFields.Lst[NumRow].NumLines = Rec_ConvertToNumLinesField (row[2]);

         /* Visible or editable by students? (row[3]) */
         if (sscanf (row[3],"%u",&Vis) != 1)
	    Lay_ShowErrorAndExit ("Error when getting field of record in current course.");
         if (Vis < Rec_NUM_TYPES_VISIBILITY)
            Gbl.CurrentCrs.Records.LstFields.Lst[NumRow].Visibility = (Rec_VisibilityRecordFields_t) Vis;
         else
            Gbl.CurrentCrs.Records.LstFields.Lst[NumRow].Visibility = HIDDEN_FIELD;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/********* List the fields of records already present in database ***********/
/****************************************************************************/

void Rec_ListFieldsRecordsForEdition (void)
  {
   extern const char *Txt_Field_BR_name[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_BR_lines[Txt_NUM_LANGUAGES];
   extern const char *Txt_Visible_by_BR_the_student[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_record_field[Txt_NUM_LANGUAGES];
   extern const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES];
   unsigned NumField;
   Rec_VisibilityRecordFields_t Vis;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_Field_BR_name[Gbl.Prefs.Language],
            Txt_No_of_BR_lines[Gbl.Prefs.Language],
            Txt_Visible_by_BR_the_student[Gbl.Prefs.Language]);

   /***** List the fields *****/
   for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
     {
      fprintf (Gbl.F.Out,"<tr>");

     /* Write icon to eliminar the field */
      fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
      Act_FormStart (ActReqRemFie);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FieldCod\" value=\"%ld\" />",
               Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_record_field[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Txt_Remove_record_field[Gbl.Prefs.Language]);

      /* Name of the field */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\">");
      Act_FormStart (ActRenFie);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FieldCod\" value=\"%ld\" />",
               Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FieldName\" size=\"50\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Rec_MAX_LENGTH_NAME_FIELD,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Name,Gbl.NumForm);

      /* Number of lines in the form */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgRowFie);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FieldCod\" value=\"%ld\" />",
               Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"NumLines\" size=\"2\"" \
                         " maxlength=\"2\" value=\"%u\" onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Gbl.CurrentCrs.Records.LstFields.Lst[NumField].NumLines,Gbl.NumForm);

      /* Visibility por students of a field */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgVisFie);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FieldCod\" value=\"%ld\" />",
               Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
      fprintf (Gbl.F.Out,"<select name=\"Visibility\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
      for (Vis=(Rec_VisibilityRecordFields_t) 0; Vis<(Rec_VisibilityRecordFields_t) Rec_NUM_TYPES_VISIBILITY; Vis++)
        {
         fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Vis);
         if (Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == Vis)
	    fprintf (Gbl.F.Out," selected=\"selected\"");
         fprintf (Gbl.F.Out,">%s</option>",Txt_RECORD_FIELD_VISIBILITY_MENU[Vis][Gbl.Prefs.Language]);
        }
      fprintf (Gbl.F.Out,"</select></form></td></tr>\n");
     }
  }

/****************************************************************************/
/******************* Show form to create a new record field *****************/
/****************************************************************************/

void Rec_ShowFormCreateRecordField (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_record_field[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_record_field[Txt_NUM_LANGUAGES];
   extern const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES];
   Rec_VisibilityRecordFields_t Vis;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"4\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"4\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_record_field[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewFie);

   /***** New field in student record *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr><td class=\"BM%d\">",
            Gbl.GlobalRowEvenOdd);
   sprintf (Gbl.MsgWindowStatus,Txt_Create_record_field[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Field name *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\"><input type=\"text\" name=\"FieldName\" size=\"50\" maxlength=\"%u\" value=\"%s\" /></td>",
            Rec_MAX_LENGTH_NAME_FIELD,Gbl.CurrentCrs.Records.Field.Name);

   /***** Number of lines in form ******/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><input type=\"text\" name=\"NumLines\" size=\"2\" maxlength=\"2\" value=\"%u\" /></td>",
            Gbl.CurrentCrs.Records.Field.NumLines);

   /***** Visibility to students *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><select name=\"Visibility\">");
   for (Vis=(Rec_VisibilityRecordFields_t) 0; Vis<(Rec_VisibilityRecordFields_t) Rec_NUM_TYPES_VISIBILITY; Vis++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Vis);
      if (Gbl.CurrentCrs.Records.Field.Visibility == Vis)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_RECORD_FIELD_VISIBILITY_MENU[Vis][Gbl.Prefs.Language]);
     }

   /***** End of form *****/
   fprintf (Gbl.F.Out,"</select></td></tr></table></form></td></tr>\n");
  }

/*****************************************************************************/
/*************** Receive data from a form of record fields *******************/
/*****************************************************************************/

void Rec_ReceiveFormField (void)
  {
   extern const char *Txt_The_record_field_X_already_exists[Txt_NUM_LANGUAGES];
   char UnsignedStr[10+1];
   unsigned Vis;

   /***** Get parameters from the form *****/
   /* Get the name of the field */
   Par_GetParam (Par_PARAM_SINGLE,"FieldName",Gbl.CurrentCrs.Records.Field.Name,Rec_MAX_LENGTH_NAME_FIELD);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.CurrentCrs.Records.Field.Name,Rec_MAX_LENGTH_NAME_FIELD,true);

   /* Get the number of lines */
   Par_GetParam (Par_PARAM_SINGLE,"NumLines",UnsignedStr,10);
   Gbl.CurrentCrs.Records.Field.NumLines = Rec_ConvertToNumLinesField (UnsignedStr);

   /* Get the field visibility by students */
   Par_GetParam (Par_PARAM_SINGLE,"Visibility",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Vis) != 1)
      Lay_ShowErrorAndExit ("Visibility of field is missing.");
   if (Vis < Rec_NUM_TYPES_VISIBILITY)
      Gbl.CurrentCrs.Records.Field.Visibility = (Rec_VisibilityRecordFields_t) Vis;
   else
      Gbl.CurrentCrs.Records.Field.Visibility = HIDDEN_FIELD;

   if (Gbl.CurrentCrs.Records.Field.Name[0])	// If there's a name
     {
      /***** If the field already was in the database... *****/
      if (Rec_CheckIfRecordFieldIsRepeated (Gbl.CurrentCrs.Records.Field.Name))
        {
         sprintf (Gbl.Message,Txt_The_record_field_X_already_exists[Gbl.Prefs.Language],
                  Gbl.CurrentCrs.Records.Field.Name);
         Lay_ShowAlert (ERROR,Gbl.Message);
        }
      else	// Add the new field to the database
         Rec_CreateRecordField ();
     }
   else		// If there isn't name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre del nuevo campo.");	// Need translation!
      Lay_ShowAlert (ERROR,Gbl.Message);
     }

   /***** Show the form again *****/
   Rec_ReqEditRecordFields ();
  }

/****************************************************************************/
/********* Get number of lines of the form to edit a record field ***********/
/****************************************************************************/

unsigned Rec_ConvertToNumLinesField (const char *StrNumLines)
  {
   int NumLines;

   if (sscanf (StrNumLines,"%d",&NumLines) != 1)
      return Rec_MIN_LINES_IN_EDITION_FIELD;
   else if (NumLines < Rec_MIN_LINES_IN_EDITION_FIELD)
      return Rec_MIN_LINES_IN_EDITION_FIELD;
   else if (NumLines > Rec_MAX_LINES_IN_EDITION_FIELD)
      return Rec_MAX_LINES_IN_EDITION_FIELD;
   return (unsigned) NumLines;
  }

/****************************************************************************/
/* Check if the name of the field of record equals any of the existing ones */
/****************************************************************************/

bool Rec_CheckIfRecordFieldIsRepeated (const char *FieldName)
  {
   bool FieldIsRepeated = false;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;

   /* Query database */
   if ((NumRows = Rec_GetAllFieldsInCurrCrs (&mysql_res)) > 0)	// If se han encontrado groups...
     {
      /* Compare with all the tipos of group from the database */
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
	 /* Get next type of group */
	 row = mysql_fetch_row (mysql_res);

         /* The name of the field is in row[1] */
         if (!strcasecmp (FieldName,row[1]))
           {
            FieldIsRepeated = true;
            break;
           }
	}
     }

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return FieldIsRepeated;
  }

/****************************************************************************/
/******* Get the fields of records already present in current course ********/
/****************************************************************************/

unsigned long Rec_GetAllFieldsInCurrCrs (MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get fields of cards of current course from database *****/
   sprintf (Query,"SELECT FieldCod,FieldName,Visibility FROM crs_record_fields" \
                  " WHERE CrsCod='%ld' ORDER BY FieldName",Gbl.CurrentCrs.Crs.CrsCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get fields of cards of a course");
  }

/****************************************************************************/
/************************* Create a field of record *************************/
/****************************************************************************/

void Rec_CreateRecordField (void)
  {
   extern const char *Txt_Created_new_record_field_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create the new field *****/
   sprintf (Query,"INSERT INTO crs_record_fields (CrsCod,FieldName,NumLines,Visibility) VALUES ('%ld','%s','%u','%u')",
            Gbl.CurrentCrs.Crs.CrsCod,
            Gbl.CurrentCrs.Records.Field.Name,Gbl.CurrentCrs.Records.Field.NumLines,(unsigned) Gbl.CurrentCrs.Records.Field.Visibility);
   DB_QueryINSERT (Query,"can not create field of record");

   /***** Write mensaje of xito *****/
   sprintf (Gbl.Message,Txt_Created_new_record_field_X[Gbl.Prefs.Language],
            Gbl.CurrentCrs.Records.Field.Name);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/*****************************************************************************/
/**************** Request the removing of a field of records *****************/
/*****************************************************************************/

void Rec_ReqRemField (void)
  {
   unsigned NumRecords;

   /***** Get the code of field *****/
   if ((Gbl.CurrentCrs.Records.Field.FieldCod = Rec_GetFieldCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of field is missing.");

   /***** Check if exists alguna record with ese field relleno *****/
   if ((NumRecords = Rec_CountNumRecordsInCurrCrsWithField (Gbl.CurrentCrs.Records.Field.FieldCod)))	// Existen records with the field relleno
      Rec_AskConfirmRemFieldWithRecords (NumRecords);
   else			// No existen records with ese field relleno
      Rec_RemoveFieldFromDB ();
  }

/****************************************************************************/
/************ Get a parameter with a code of field of records ***************/
/****************************************************************************/

long Rec_GetFieldCod (void)
  {
   char LongStr[1+10+1];

   /***** Get the code of the field *****/
   Par_GetParam (Par_PARAM_SINGLE,"FieldCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/*************** Get the number of records with a field filled **************/
/****************************************************************************/

unsigned Rec_CountNumRecordsInCurrCrsWithField (long FieldCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   int NumRecords;

   /***** Get number of cards with a given field in a course from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_records WHERE FieldCod='%ld'",FieldCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of cards with a given field not empty in a course");

   /***** Get the number of records with that field filled *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRecords) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of records with a given field not empty in a course.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumRecords;
  }

/*****************************************************************************/
/******* Request confirmation for the removing of a field with records *******/
/*****************************************************************************/

void Rec_AskConfirmRemFieldWithRecords (unsigned NumRecords)
  {
   extern const char *Txt_Remove_record_field[Txt_NUM_LANGUAGES];
  	
   /***** Get from the database the name of the field *****/
   Rec_GetFieldByCod (Gbl.CurrentCrs.Records.Field.FieldCod,Gbl.CurrentCrs.Records.Field.Name,&Gbl.CurrentCrs.Records.Field.NumLines,&Gbl.CurrentCrs.Records.Field.Visibility);

   /***** Write mensaje to ask confirmation of eliminacin *****/
   if (NumRecords == 1)
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?" \
                           " (dicho campo est&aacute; relleno en la ficha de un alumno)",	// Need translation!
               Gbl.CurrentCrs.Records.Field.Name,Gbl.CurrentCrs.Crs.FullName);
   else
      sprintf (Gbl.Message,"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?" \
                           " (dicho campo est&aacute; relleno en las fichas de %u alumnos)",	// Need translation!
               Gbl.CurrentCrs.Records.Field.Name,Gbl.CurrentCrs.Crs.FullName,NumRecords);
   Lay_ShowAlert (WARNING,Gbl.Message);

   /***** Button to confirm removing *****/
   Act_FormStart (ActRemFie);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FieldCod\" value=\"%ld\" />",
            Gbl.CurrentCrs.Records.Field.FieldCod);
   fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"%s\" /></div>\n",
            Txt_Remove_record_field[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/************** Remove from the database a field of records *****************/
/****************************************************************************/

void Rec_RemoveFieldFromDB (void)
  {
   extern const char *Txt_Record_field_X_removed[Txt_NUM_LANGUAGES];
   char Query[512];

   /***** Get from the database the name of the field *****/
   Rec_GetFieldByCod (Gbl.CurrentCrs.Records.Field.FieldCod,Gbl.CurrentCrs.Records.Field.Name,&Gbl.CurrentCrs.Records.Field.NumLines,&Gbl.CurrentCrs.Records.Field.Visibility);

   /***** Remove field from all records *****/
   sprintf (Query,"DELETE FROM crs_records WHERE FieldCod='%ld'",
            Gbl.CurrentCrs.Records.Field.FieldCod);
   DB_QueryDELETE (Query,"can not remove field from all students' records");

   /***** Remove the field *****/
   sprintf (Query,"DELETE FROM crs_record_fields WHERE FieldCod='%ld'",
            Gbl.CurrentCrs.Records.Field.FieldCod);
   DB_QueryDELETE (Query,"can not remove field of record");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,Txt_Record_field_X_removed[Gbl.Prefs.Language],
            Gbl.CurrentCrs.Records.Field.Name);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Rec_ReqEditRecordFields ();
  }

/****************************************************************************/
/************** Get the data of a field of records from its code ************/
/****************************************************************************/

static void Rec_GetFieldByCod (long FieldCod,char *Name,unsigned *NumLines,Rec_VisibilityRecordFields_t *Visibility)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned Vis;

   /***** Get a field of a record in a course from database *****/
   sprintf (Query,"SELECT FieldName,NumLines,Visibility FROM crs_record_fields" \
                  " WHERE CrsCod='%ld' AND FieldCod='%ld'",Gbl.CurrentCrs.Crs.CrsCod,FieldCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get a field of a record in a course");

   /***** Count number of rows in result *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting a field of a record in a course.");

   /***** Get the field *****/
   row = mysql_fetch_row (mysql_res);

   /* Name of the field */
   strcpy (Name,row[0]);

   /* Number of lines of the field (row[1]) */
   *NumLines = Rec_ConvertToNumLinesField (row[1]);

   /* Visible or editable by students? (row[2]) */
   if (sscanf (row[2],"%u",&Vis) != 1)
      Lay_ShowErrorAndExit ("Error when getting a field of a record in a course.");
   if (Vis < Rec_NUM_TYPES_VISIBILITY)
      *Visibility = (Rec_VisibilityRecordFields_t) Vis;
   else
      *Visibility = HIDDEN_FIELD;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************************* Remove a field of records *************************/
/*****************************************************************************/

void Rec_RemoveField (void)
  {
   /***** Get the code of the field *****/
   if ((Gbl.CurrentCrs.Records.Field.FieldCod = Rec_GetFieldCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of field is missing.");

   /***** Borrarlo from the database *****/
   Rec_RemoveFieldFromDB ();
  }

/*****************************************************************************/
/************************** Rename a field of records ************************/
/*****************************************************************************/

void Rec_RenameField (void)
  {
   extern const char *Txt_The_record_field_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[1024];
   char NewFieldName[Rec_MAX_LENGTH_NAME_FIELD+1];

   /***** Get parameters of the form *****/
   /* Get the code of the field */
   if ((Gbl.CurrentCrs.Records.Field.FieldCod = Rec_GetFieldCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of field is missing.");

   /* Get the new group name */
   Par_GetParam (Par_PARAM_SINGLE,"FieldName",NewFieldName,Rec_MAX_LENGTH_NAME_FIELD);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewFieldName,Rec_MAX_LENGTH_NAME_FIELD,true);

   /***** Get from the database the antiguo group name *****/
   Rec_GetFieldByCod (Gbl.CurrentCrs.Records.Field.FieldCod,Gbl.CurrentCrs.Records.Field.Name,&Gbl.CurrentCrs.Records.Field.NumLines,&Gbl.CurrentCrs.Records.Field.Visibility);

   /***** Check if new name is empty *****/
   if (!NewFieldName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre del campo <strong>%s</strong> vac&iacute;o.",	// Need translation!
               Gbl.CurrentCrs.Records.Field.Name);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
   else
     {
      /***** Check if the name of the field antiguo coincide with the new (esto pasa cuando se pulsa return sin cambiar nada in the form) *****/
      if (strcmp (Gbl.CurrentCrs.Records.Field.Name,NewFieldName))	// Different names
        {
         /***** If the group ya estaba in the database... *****/
         if (Rec_CheckIfRecordFieldIsRepeated (NewFieldName))
           {
            sprintf (Gbl.Message,Txt_The_record_field_X_already_exists[Gbl.Prefs.Language],
                     NewFieldName);
            Lay_ShowAlert (ERROR,Gbl.Message);
           }
         else
           {
            /* Update the table of fields changing then old name by the new one */
            sprintf (Query,"UPDATE crs_record_fields SET FieldName='%s' WHERE FieldCod='%ld'",
                     NewFieldName,Gbl.CurrentCrs.Records.Field.FieldCod);
            DB_QueryUPDATE (Query,"can not update name of field of record");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El campo <strong>%s</strong>" \
                                 " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     Gbl.CurrentCrs.Records.Field.Name,NewFieldName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del campo <strong>%s</strong> no ha cambiado.",	// Need translation!
                  NewFieldName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (Gbl.CurrentCrs.Records.Field.Name,NewFieldName);
   Rec_ReqEditRecordFields ();
  }

/*****************************************************************************/
/********* Change number of lines of the form of a field of records **********/
/*****************************************************************************/

void Rec_ChangeLinesField (void)
  {
   char Query[1024];
   char UnsignedStr[10+1];
   unsigned NewNumLines;

   /***** Get parameters of the form *****/
   /* Get the code of field */
   if ((Gbl.CurrentCrs.Records.Field.FieldCod = Rec_GetFieldCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of field is missing.");

   /* Get the new number of lines */
   Par_GetParam (Par_PARAM_SINGLE,"NumLines",UnsignedStr,10);
   NewNumLines = Rec_ConvertToNumLinesField (UnsignedStr);

   /* Get from the database the number of lines of the field */
   Rec_GetFieldByCod (Gbl.CurrentCrs.Records.Field.FieldCod,Gbl.CurrentCrs.Records.Field.Name,&Gbl.CurrentCrs.Records.Field.NumLines,&Gbl.CurrentCrs.Records.Field.Visibility);

   /***** Check if the number of filas antiguo coincide with the new (esto pasa cuando se pulsa return sin cambiar nada in the form) *****/
   if (Gbl.CurrentCrs.Records.Field.NumLines == NewNumLines)
     {
      sprintf (Gbl.Message,"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.",	// Need translation!
               Gbl.CurrentCrs.Records.Field.Name);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      /***** Update of the table of fields changing the old maximum of students by the new one *****/
      sprintf (Query,"UPDATE crs_record_fields SET NumLines='%u' WHERE FieldCod='%ld'",
               NewNumLines,Gbl.CurrentCrs.Records.Field.FieldCod);
      DB_QueryUPDATE (Query,"can not update the number of lines of a field of record");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.",	// Need translation!
	       Gbl.CurrentCrs.Records.Field.Name,NewNumLines);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.CurrentCrs.Records.Field.NumLines = NewNumLines;
   Rec_ReqEditRecordFields ();
  }

/*****************************************************************************/
/************ Change wisibility by students of a field of records ************/
/*****************************************************************************/

void Rec_ChangeVisibilityField (void)
  {
   extern const char *Txt_RECORD_FIELD_VISIBILITY_MSG[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES];
   char Query[1024];
   char UnsignedStr[10+1];
   unsigned Vis;
   Rec_VisibilityRecordFields_t NewVisibility;

   /***** Get parameters of the form *****/
   /* Get the code of field */
   if ((Gbl.CurrentCrs.Records.Field.FieldCod = Rec_GetFieldCod ()) == -1)
      Lay_ShowErrorAndExit ("Code of field is missing.");

   /* Get the new visibility of the field */
   Par_GetParam (Par_PARAM_SINGLE,"Visibility",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Vis) != 1)
      Lay_ShowErrorAndExit ("Visibility of field is missing.");
   if (Vis < Rec_NUM_TYPES_VISIBILITY)
      NewVisibility = (Rec_VisibilityRecordFields_t) Vis;
   else
      NewVisibility = HIDDEN_FIELD;

   /* Get from the database the visibility of the field */
   Rec_GetFieldByCod (Gbl.CurrentCrs.Records.Field.FieldCod,Gbl.CurrentCrs.Records.Field.Name,&Gbl.CurrentCrs.Records.Field.NumLines,&Gbl.CurrentCrs.Records.Field.Visibility);

   /***** Check if the old visibility matches the new one
          (this happens whe return is pressed without changes in the form) *****/
   if (Gbl.CurrentCrs.Records.Field.Visibility == NewVisibility)
     {
      sprintf (Gbl.Message,"La visibilidad del campo <strong>%s</strong> no ha cambiado.",	// Need translation!
               Gbl.CurrentCrs.Records.Field.Name);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      /***** Update of the table of fields changing the old visibility by the new *****/
      sprintf (Query,"UPDATE crs_record_fields SET Visibility='%u' WHERE FieldCod='%ld'",
               (unsigned) NewVisibility,Gbl.CurrentCrs.Records.Field.FieldCod);
      DB_QueryUPDATE (Query,"can not update the visibility of a field of record");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_RECORD_FIELD_VISIBILITY_MSG[NewVisibility][Gbl.Prefs.Language],
	       Gbl.CurrentCrs.Records.Field.Name);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.CurrentCrs.Records.Field.Visibility = NewVisibility;
   Rec_ReqEditRecordFields ();
  }

/*****************************************************************************/
/********************** Liberar list of fields of records ********************/
/*****************************************************************************/

void Rec_FreeListFields (void)
  {
   if (Gbl.CurrentCrs.Records.LstFields.NestedCalls > 0)
      if (--Gbl.CurrentCrs.Records.LstFields.NestedCalls == 0)
         if (Gbl.CurrentCrs.Records.LstFields.Lst)
           {
            free ((void *) Gbl.CurrentCrs.Records.LstFields.Lst);
            Gbl.CurrentCrs.Records.LstFields.Lst = NULL;
            Gbl.CurrentCrs.Records.LstFields.Num = 0;
           }
  }

/****************************************************************************/
/************ Show record in the course of the user actual ***********/
/****************************************************************************/

void Rec_ShowFormMyCrsRecord (void)
  {
   extern const char *Txt_Change_my_record[Txt_NUM_LANGUAGES];

   /***** Show record common a todas the courses *****/
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"1\" cellpadding=\"0\"><tr><td align=\"center\">");
   Rec_ShowCommonRecord (RECORD_LIST,&Gbl.Usrs.Me.UsrDat,Txt_Change_my_record[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Get list of fields of records in current course *****/
   Rec_GetListRecordFieldsInCurrentCrs ();

   /***** Show record particular from current course *****/
   if (Gbl.CurrentCrs.Records.LstFields.Num) // Hay fields in the record particulares a the course...
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
      Rec_ShowCrsRecord (FORM_MY_COURSE_RECORD,&Gbl.Usrs.Me.UsrDat);
      fprintf (Gbl.F.Out,"</td></tr>");
     }

   /***** Liberar list of fields of records *****/
   Rec_FreeListFields ();

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table></div>\n");
  }

/****************************************************************************/
/**** Update record in the course of the user actual and presentarla ****/
/****************************************************************************/

void Rec_UpdateAndShowMyCrsRecord (void)
  {
   /***** Get list of fields of records in current course *****/
   Rec_GetListRecordFieldsInCurrentCrs ();

   /***** Allocate memory for the texts of the fields *****/
   Rec_AllocMemFieldsRecordsCrs ();

   /***** Get data of the record of the student in the course procedentes of the form *****/
   Rec_GetFieldsCrsRecordFromForm ();

   /***** Update the record *****/
   Rec_UpdateCrsRecord (Gbl.Usrs.Me.UsrDat.UsrCod);

   /***** Show user's record ya actualizada *****/
   Rec_ShowMyCrsRecordUpdated ();

   /***** Free memory of the textos of the fields *****/
   Rec_FreeMemFieldsRecordsCrs ();
  }

/****************************************************************************/
/* Update record in the course of another user (student) and presentarla */
/****************************************************************************/

void Rec_UpdateAndShowOtherCrsRecord (void)
  {
   /***** Get the user's code of the student whose record we want to modify *****/
   Usr_GetParamOtherUsrCodEncrypted ();

   /***** Get data of the student, because we need the name *****/
   Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat);

   /***** Get list of fields of records in current course *****/
   Rec_GetListRecordFieldsInCurrentCrs ();

   /***** Allocate memory for the texts of the fields *****/
   Rec_AllocMemFieldsRecordsCrs ();

   /***** Get data of the record of the student in the course procedentes of the form *****/
   Rec_GetFieldsCrsRecordFromForm ();

   /***** Update the record *****/
   Rec_UpdateCrsRecord (Gbl.Usrs.Other.UsrDat.UsrCod);

   /***** Show user's record ya actualizada *****/
   Rec_ShowOtherCrsRecordUpdated ();

   /***** Free memory of the textos of the fields *****/
   Rec_FreeMemFieldsRecordsCrs ();
  }

/****************************************************************************/
/*************************** Show record common **************************/
/****************************************************************************/
// Show form or only data depending on TypeOfView

void Rec_ShowCrsRecord (Rec_RecordViewType_t TypeOfView,struct UsrData *UsrDat)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES];
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];
   unsigned CardWidth=Rec_WIDTH_COURSE_RECORD,FrameWidth=10,Col1Width=150,Col2Width,LogoSize=32;
   char CadenaCardWidth[4+1];
   char *ClassHead,*ClassData,*LogoDeg;
   bool IsForm = false;
   unsigned NumField;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row = NULL; // Initialized to avoid warning
   unsigned long NumRows;
   bool ThisFieldHasText,EditField;
   char Text[Cte_MAX_BYTES_TEXT+1];

   LogoDeg = Gbl.CurrentDeg.Logo64;
   ClassHead = "CAB_FICHA";
   ClassData = "DAT_FICHA";
   switch (TypeOfView)
     {
      case FORM_MY_COURSE_RECORD:
         for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
            if (Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == EDITABLE_FIELD)
              {
               IsForm = true;
  	       Act_FormStart (ActRecCrdCrs);
               break;
              }
	 CardWidth = Rec_WIDTH_COURSE_RECORD; FrameWidth = 10;
	 ClassHead = "CAB_FICHA_PEQ";
	 ClassData = "DAT_FICHA";
	 LogoDeg = Gbl.CurrentDeg.Logo32;
	 LogoSize = 32;
	 break;
      case MY_COURSE_RECORD_CHECK:
      case OTHER_COURSE_RECORD_CHECK:
	 CardWidth = Rec_WIDTH_COURSE_RECORD; FrameWidth = 10;
	 ClassHead = "CAB_FICHA_PEQ";
	 ClassData = "DAT_FICHA_PEQN";
	 LogoDeg = Gbl.CurrentDeg.Logo32;
	 LogoSize = 32;
         break;
      case RECORD_LIST:
         IsForm = true;
	 Act_FormStart (ActRecCrdCrsOthUsr);
         Usr_PutParamOtherUsrCodEncrypted (UsrDat->EncryptedUsrCod);
	 CardWidth = Rec_WIDTH_COURSE_RECORD; FrameWidth = 10;
	 ClassHead = "CAB_FICHA_PEQ";
	 ClassData = "DAT_FICHA_PEQN";
	 LogoDeg = Gbl.CurrentDeg.Logo32;
	 LogoSize = 32;
	 break;
      case RECORD_PRINT:
	 CardWidth = Rec_WIDTH_COURSE_RECORD_PRINT; FrameWidth = 1;
	 ClassHead = "CAB_FICHA_PEQ";
	 ClassData = "DAT_FICHA_PEQN";
	 LogoDeg = Gbl.CurrentDeg.Logo64;
	 LogoSize = 48;
         break;
      default:
         break;
    }

   Col2Width = CardWidth - FrameWidth*2 - Col1Width;

   /***** Start table *****/
   sprintf (CadenaCardWidth,"%u",CardWidth);
   if (TypeOfView == RECORD_PRINT)
      Lay_StartTableFrame (DARK_BLUE,WHITE,CadenaCardWidth);
   else
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,CadenaCardWidth);
   fprintf (Gbl.F.Out,"<tr><td align=\"left\"><table cellspacing=\"0\" cellpadding=\"1\" width=\"%u\">",CardWidth - FrameWidth*2);
// fprintf (Gbl.F.Out,"<tr><td align=\"left\"><table border=\"1\" bordercolor=\"#FF00FF\" bgcolor=\"#%s\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">",WHITE);

   /***** Header *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"2\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">" \
                  "<tr><td align=\"left\" width=\"%u\"><img src=\"%s\" width=\"%u\" height=\"%u\" /></td>" \
                  "<td align=\"center\" class=\"%s\">%s<br />%s<br />%s</td></tr></table></td></tr>",
            LogoSize,LogoDeg,LogoSize,LogoSize,ClassHead,
            Gbl.CurrentDeg.Deg.FullName,Gbl.CurrentCrs.Crs.FullName,
            UsrDat->FullName);

   /***** Fields of the record dependientes of the course *****/
   for (NumField=0, Gbl.GlobalRowEvenOdd=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
      // If debe mostrarse the field...
      if (!(TypeOfView == FORM_MY_COURSE_RECORD || TypeOfView == MY_COURSE_RECORD_CHECK) ||
          Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility != HIDDEN_FIELD)
        {
         EditField = TypeOfView == RECORD_LIST ||
                     (TypeOfView == FORM_MY_COURSE_RECORD  && Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == EDITABLE_FIELD);

         /* Name of the field */
         fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">%s:",
                  Col1Width,Gbl.ColorRows[Gbl.GlobalRowEvenOdd],EditField ? The_ClassFormul[Gbl.Prefs.Theme] : "DAT_FICHA_PEQ",Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Name);
         if (TypeOfView == RECORD_LIST)
            fprintf (Gbl.F.Out,"<span class=\"DAT_SMALL\"> (%s)</span>",
                     Txt_RECORD_FIELD_VISIBILITY_RECORD[Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility][Gbl.Prefs.Language]);
         fprintf (Gbl.F.Out,"</td>");

         /***** Get the text of the field *****/
         NumRows = Rec_GetFieldFromCrsRecord (UsrDat->UsrCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod,&mysql_res);
         if (NumRows)
           {
            ThisFieldHasText = true;
            row = mysql_fetch_row (mysql_res);
           }
         else
            ThisFieldHasText = false;

         /***** Write form, text, o nada segn the type of user and the visibility of the field... */
         fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">",
                  Col2Width,Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ClassData);
         if (EditField)	// Mostrarlo with form
           {
            fprintf (Gbl.F.Out,"<textarea name=\"Campo%ld\" cols=\"40\" rows=\"%u\">",
                     Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].NumLines);
            if (ThisFieldHasText)
               fprintf (Gbl.F.Out,row[0]);
            fprintf (Gbl.F.Out,"</textarea>");
           }
         else			// Mostrarlo sin form
           {
            if (ThisFieldHasText)
              {
               strcpy (Text,row[0]);
               Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Text,Cte_MAX_BYTES_TEXT,false);	// Se convierte of HTML a HTML respetuoso
               fprintf (Gbl.F.Out,Text);
              }
            else
               fprintf (Gbl.F.Out,"-");
           }
         fprintf (Gbl.F.Out,"</td></tr>");

         /***** Free structure that stores the query result *****/
         DB_FreeMySQLResult (&mysql_res);

         Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
        }

   /***** End of table *****/
// fprintf (Gbl.F.Out,"</table></td></tr>");
   fprintf (Gbl.F.Out,"</table></td></tr>\n");
   if (TypeOfView == RECORD_PRINT)
      Lay_EndTableFrame ();
   else
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   if (IsForm)
     {
      Lay_PutSendAndUndoButtons (Txt_Send[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form>\n");
     }
  }

/****************************************************************************/
/************** Get the text of a field of a record of course ***************/
/****************************************************************************/

unsigned long Rec_GetFieldFromCrsRecord (long UsrCod,long FieldCod,MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get the text of a field of a record from database *****/
   sprintf (Query,"SELECT Txt FROM crs_records WHERE FieldCod='%ld' AND UsrCod='%ld'",
            FieldCod,UsrCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get the text of a field of a record.");
  }

/****************************************************************************/
/********* Get the fields of the record input by the student in form ********/
/****************************************************************************/

void Rec_GetFieldsCrsRecordFromForm (void)
  {
   unsigned NumField;
   char NombreParamCampo[5+10+1];

   for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
      if (Gbl.Usrs.Me.UsrTypeLog > Usr_STUDENT || Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == EDITABLE_FIELD)
        {
         /* Get text of the form */
         sprintf (NombreParamCampo,"Campo%ld",Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
         Par_GetParam (Par_PARAM_SINGLE,NombreParamCampo,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text,Cte_MAX_BYTES_TEXT);
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text,Cte_MAX_BYTES_TEXT,true);
        }
  }

/****************************************************************************/
/*********************** Update record of a user *********************/
/****************************************************************************/

void Rec_UpdateCrsRecord (long UsrCod)
  {
   unsigned NumField;
   char Query[256+Cte_MAX_BYTES_TEXT];
   MYSQL_RES *mysql_res;
   unsigned long NumRows;
   bool YaExisteCampo;

   for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
      if (Gbl.Usrs.Me.UsrTypeLog > Usr_STUDENT || Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == EDITABLE_FIELD)
        {
         /***** Check if already exists this field for this user in database *****/
         NumRows = Rec_GetFieldFromCrsRecord (UsrCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod,&mysql_res);
         YaExisteCampo = (NumRows != 0);
         DB_FreeMySQLResult (&mysql_res);
         if (YaExisteCampo)
           {
            if (Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text[0])
              {
               /***** Update text of the field of record course *****/
               sprintf (Query,"UPDATE crs_records SET Txt='%s' WHERE UsrCod='%ld' AND FieldCod='%ld'",
                        Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text,UsrCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
               DB_QueryUPDATE (Query,"can not update field of record");
              }
            else
              {
               /***** Remove text of the field of record course *****/
               sprintf (Query,"DELETE FROM crs_records WHERE UsrCod='%ld' AND FieldCod='%ld'",
                        UsrCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod);
               DB_QueryDELETE (Query,"can not remove field of record");
              }
           }
         else
           {
            if (Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text[0])
              {
               /***** Insert text field of record course *****/
               sprintf (Query,"INSERT INTO crs_records (FieldCod,UsrCod,Txt) VALUES('%ld','%ld','%s')",
                        Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod,UsrCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text);
               DB_QueryINSERT (Query,"can not create field of record");
              }
           }
       }
  }

/****************************************************************************/
/******** Remove fields of record of a user in current course *********/
/****************************************************************************/

void Rec_RemoveFieldsCrsRecordInCrs (long UsrCod,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[512];

   /***** Remove text of the field of record course *****/
   sprintf (Query,"DELETE FROM crs_records WHERE UsrCod='%ld' AND FieldCod IN" \
                  " (SELECT FieldCod FROM crs_record_fields WHERE CrsCod='%ld')",
            UsrCod,Crs->CrsCod);
   DB_QueryDELETE (Query,"can not remove user's record in a course");

   /***** Write mensaje *****/
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"Ficha del usuario en <strong>%s</strong> eliminada.",	// Need translation!
               Crs->FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
  }

/****************************************************************************/
/************* Remove fields of record of a user in all courses *************/
/****************************************************************************/

void Rec_RemoveFieldsCrsRecordAll (long UsrCod,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[128];

   /***** Remove text of the field of record course *****/
   sprintf (Query,"DELETE FROM crs_records WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove user's records in all courses");

   /***** Write mensaje *****/
   if (QuietOrVerbose == Cte_VERBOSE)
      Lay_ShowAlert (INFO,"Se han eliminado las fichas del usuario en todas las asignaturas.");	// Need translation!
  }

/****************************************************************************/
/*************** Show my record in the course already updated ***************/
/****************************************************************************/

void Rec_ShowMyCrsRecordUpdated (void)
  {
   /***** Write mensaje of confirmacin *****/
   Lay_ShowAlert (INFO,"Su ficha en la asignatura se ha actualizado:");	// Need translation!

   /***** Show user's record ya actualizada *****/
   Rec_ShowCrsRecordAfterUpdate (MY_COURSE_RECORD_CHECK,&Gbl.Usrs.Me.UsrDat);
  }

/****************************************************************************/
/* Show user's record in the course of the presente user ya actualizada **/
/****************************************************************************/

void Rec_ShowOtherCrsRecordUpdated (void)
  {
   /***** Write mensaje of confirmacin *****/
   Lay_ShowAlert (INFO,"La ficha del alumno en la asignatura se ha actualizado:");	// Need translation!

   /***** Show user's record ya actualizada *****/
   Rec_ShowCrsRecordAfterUpdate (OTHER_COURSE_RECORD_CHECK,&Gbl.Usrs.Other.UsrDat);
  }

/****************************************************************************/
/************ Show user's record in the course already updated **************/
/****************************************************************************/

static void Rec_ShowCrsRecordAfterUpdate (Rec_RecordViewType_t TypeOfView,struct UsrData *UsrDat)
  {
   /***** Show user's record *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Rec_ShowCrsRecord (TypeOfView,UsrDat);
   fprintf (Gbl.F.Out,"</div>");
  }

/****************************************************************************/
/***** Allocate memory for the text of the field of the record in course ****/
/****************************************************************************/

void Rec_AllocMemFieldsRecordsCrs (void)
  {
   unsigned NumField;

   for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
      if (Gbl.Usrs.Me.UsrTypeLog > Usr_STUDENT || Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == EDITABLE_FIELD)
         /* Allocate memory for the texts of the fields */
         if ((Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
            Lay_ShowErrorAndExit ("Not enough memory to store records of the course.");
  }

/****************************************************************************/
/**** Free memory used by the texts of the field of the record in course ****/
/****************************************************************************/

void Rec_FreeMemFieldsRecordsCrs (void)
  {
   unsigned NumField;

   for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
      if (Gbl.Usrs.Me.UsrTypeLog > Usr_STUDENT || Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility == EDITABLE_FIELD)
         /* Free memory of the text of the opcin */
         if (Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text)
           {
            free ((void *) Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text);
            Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Text = NULL;
           }
  }

/****************************************************************************/
/******************* Show form to edit my common record *********************/
/****************************************************************************/

void Rec_ShowFormMyCommRecord (void)
  {
   extern const char *Txt_Change_my_record[Txt_NUM_LANGUAGES];

   /***** If user has no gender, name and surname... *****/
   if (Gbl.Usrs.Me.UsrDat.Gender == Usr_GENDER_UNKNOWN)
      Lay_ShowAlert (WARNING,"Antes de acceder a cualquier otra opci&oacute;n debe rellenar su ficha incluyendo el tipo de usuario.");	// Need translation!
   else if (!Gbl.Usrs.Me.UsrDat.FirstName[0] || !Gbl.Usrs.Me.UsrDat.Surname1[0])
      Lay_ShowAlert (WARNING,"Antes de acceder a cualquier otra opci&oacute;n debe rellenar su ficha incluyendo su nombre y apellido(s).");	// Need translation!
   else if (!Gbl.Usrs.Me.UsrDat.E_mail[0])
      Lay_ShowAlert (WARNING,"Antes de acceder a cualquier otra opci&oacute;n debe rellenar su ficha incluyendo su direcci&oacute;n de correo electr&oacute;nico.");	// Need translation!
   else if (Gbl.Usrs.Me.UsrDat.InsCod == -1L)
      Lay_ShowAlert (WARNING,"Antes de acceder a cualquier otra opci&oacute;n debe rellenar su ficha incluyendo su instituci&oacute;n.");	// Need translation!
   else if (Gbl.Usrs.Me.UsrDat.UsrTypeInDB >= Usr_TEACHER &&
            (Gbl.Usrs.Me.UsrDat.Tch.CtrCod == -1L ||
             Gbl.Usrs.Me.UsrDat.Tch.DptCod == -1L))
      Lay_ShowAlert (WARNING,"Antes de acceder a cualquier otra opci&oacute;n debe rellenar su ficha incluyendo su centro y departamento.");	// Need translation!

   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Rec_ShowCommonRecord (FORM_MY_COMMON_RECORD,&Gbl.Usrs.Me.UsrDat,Txt_Change_my_record[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</div>");
  }

/****************************************************************************/
/*************** Show form to edit the record of a new user *****************/
/****************************************************************************/

void Rec_ShowFormOtherNewCommonRecord (struct UsrData *UsrDat)
  {
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];

   /***** Show the form *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Rec_ShowCommonRecord (FORM_NEW_RECORD_OTHER_USR,UsrDat,Txt_Send[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</div>");
  }

/****************************************************************************/
/************ Show form to edit the record of an existing user **************/
/****************************************************************************/

void Rec_ShowFormOtherExistingCommonRecord (struct UsrData *UsrDat)
  {
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];

   /***** Show the form *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Rec_ShowCommonRecord (FORM_MODIF_RECORD_OTHER_USR,UsrDat,Txt_Send[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</div>");
  }

/****************************************************************************/
/*********** Show user's record of the presente user ya actualizada *********/
/****************************************************************************/

void Rec_ShowMyCommonRecordUpd (void)
  {
   /***** Write mensaje of confirmacin *****/
   Lay_ShowAlert (INFO,"Sus datos personales se han actualizado:");	// Need translation!

   /***** Show user's record ya actualizada *****/
   Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Me.UsrDat);
  }

/****************************************************************************/
/******************* Show user's record ya actualizada **********************/
/****************************************************************************/

void Rec_ShowCommonRecordUnmodifiable (struct UsrData *UsrDat)
  {
   /***** Get password, user type and user's data from database *****/
   Usr_GetAllUsrDataFromUsrCod (UsrDat);
   UsrDat->Accepted = Usr_GetIfUserHasAccepted (UsrDat->UsrCod);

   /***** Show user's record *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Rec_ShowCommonRecord (MY_COMMON_RECORD_CHECK,UsrDat,NULL);
   fprintf (Gbl.F.Out,"</div>");
  }

/****************************************************************************/
/*************************** Show record common **************************/
/****************************************************************************/
// Show form o slo data dependiendo of TypeOfView

void Rec_ShowCommonRecord (Rec_RecordViewType_t TypeOfView,struct UsrData *UsrDat,const char *TxtSubmitButton)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Type_of_user[Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Surname_1[Txt_NUM_LANGUAGES];
   extern const char *Txt_Surname_2[Txt_NUM_LANGUAGES];
   extern const char *Txt_First_name[Txt_NUM_LANGUAGES];
   extern const char *Txt_E_mail[Txt_NUM_LANGUAGES];
   extern const char *Txt_Web_page[Txt_NUM_LANGUAGES];
   extern const char *Txt_Name_on_Skype[Txt_NUM_LANGUAGES];
   extern const char *Txt_Private_Photo[Txt_NUM_LANGUAGES];
   extern const char *Txt_Public_Photo[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Centre[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_centre[Txt_NUM_LANGUAGES];
   extern const char *Txt_Department[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_department[Txt_NUM_LANGUAGES];
   extern const char *Txt_Office[Txt_NUM_LANGUAGES];
   extern const char *Txt_Phone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Local_address[Txt_NUM_LANGUAGES];
   extern const char *Txt_Family_address[Txt_NUM_LANGUAGES];
   extern const char *Txt_Place_of_origin[Txt_NUM_LANGUAGES];
   extern const char *Txt_Date_of_birth[Txt_NUM_LANGUAGES];
   extern const char *Txt_USER_comments[Txt_NUM_LANGUAGES];
   unsigned CardWidth = Rec_WIDTH_COMMON_RECORD_BIG;
   unsigned FrameWidth = 10;
   unsigned Cols1and2Width,Col1Width = 120,Col2Width,Col3Width = 120;
   unsigned PhotoWidth = Usr_PHOTO_REAL_WIDTH;
   unsigned PhotoHeight = Usr_PHOTO_REAL_HEIGHT;
   unsigned NumRowsPhoto = 8;
   unsigned LogoSize = 64;
   unsigned LogoSizeInScreen = 64;
   const char *ClassHead,*ClassForm,*ClassData;
   char PhotoURL[PATH_MAX+1];
   bool ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
   bool ShowData = ItsMe || UsrDat->Accepted || Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN || Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;
   bool IsForm = (TypeOfView == FORM_MY_COMMON_RECORD ||
                  TypeOfView == FORM_NEW_RECORD_OTHER_USR ||
                  TypeOfView == FORM_MODIF_RECORD_OTHER_USR);
   Usr_Gender_t Gender;
   bool ShowPhoto;
   bool UsrHasWebPage;
   struct Institution Ins;
   unsigned NumIns;
   struct Centre Ctr;
   unsigned NumCtr;
   struct Department Dpt;
   unsigned NumDpt;

   ClassHead = "CAB_FICHA";
   ClassForm = The_ClassFormul[Gbl.Prefs.Theme];
   ClassData = "DAT_FICHA";
   switch (TypeOfView)
     {
      case FORM_MY_COMMON_RECORD:
	 Act_FormStart (ActActMyData);
	 CardWidth = Rec_WIDTH_COMMON_RECORD_BIG;
         FrameWidth = 10;
         Col3Width = PhotoWidth = Usr_PHOTO_REAL_WIDTH;
         PhotoHeight = Usr_PHOTO_REAL_HEIGHT;
         NumRowsPhoto = 8;
	 ClassHead = "CAB_FICHA";
	 ClassForm = The_ClassFormul[Gbl.Prefs.Theme];
	 ClassData = "DAT_FICHA";
	 LogoSize          = 64;
	 LogoSizeInScreen  = 64;
	 break;
      case FORM_NEW_RECORD_OTHER_USR:
      case FORM_MODIF_RECORD_OTHER_USR:
	 Act_FormStart ((TypeOfView == FORM_NEW_RECORD_OTHER_USR) ? ActCreOthUsrDat : ActUpdOthUsrDat);
         Usr_PutParamOtherUsrDNIPlain ();
	 CardWidth = Rec_WIDTH_COMMON_RECORD_BIG;
         FrameWidth = 10;
         Col3Width = PhotoWidth = Usr_PHOTO_REAL_WIDTH;
         PhotoHeight = Usr_PHOTO_REAL_HEIGHT;
         NumRowsPhoto = 7;
	 ClassHead = "CAB_FICHA";
         ClassForm = The_ClassFormul[Gbl.Prefs.Theme];
	 ClassData = "DAT_FICHA";
	 LogoSize          = 64;
	 LogoSizeInScreen  = 64;
	 break;
      case MY_COMMON_RECORD_CHECK:
      case RECORD_LIST:
	 CardWidth = Rec_WIDTH_COMMON_RECORD_SMALL;
         FrameWidth = 10;
         PhotoWidth = 75;
         PhotoHeight = 100;
         NumRowsPhoto = 7;
	 ClassHead = "CAB_FICHA_PEQ";
	 ClassForm = "DAT_FICHA_PEQ";
	 ClassData = "DAT_FICHA_PEQN";
	 LogoSize          = 32;
	 LogoSizeInScreen  = 32;
	 break;
      case RECORD_PRINT:
	 CardWidth = Rec_WIDTH_COMMON_RECORD_PRINT;
         FrameWidth = 1;
         Col3Width = PhotoWidth = 120;
         PhotoHeight = 160;
         NumRowsPhoto = 7;
	 ClassHead = "CAB_FICHA_PEQ";
	 ClassForm = "DAT_FICHA_PEQ";
	 ClassData = "DAT_FICHA_PEQN";
	 LogoSize          = 64;
	 LogoSizeInScreen  = 48;
         break;
      default:
         break;
    }
   Cols1and2Width = CardWidth - FrameWidth*2 - Col3Width;
   Col2Width = Cols1and2Width - Col1Width;

   /***** Start table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"%u\"><tr><td align=\"left\" width=\"%u\">",CardWidth,CardWidth);
   if (TypeOfView == RECORD_PRINT)
      Lay_StartTableFrame (DARK_BLUE,WHITE,"100%");
   else
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
   fprintf (Gbl.F.Out,"<tr>");

   /***** Header *****/
   if (UsrDat->InsCod > 0)
     {
      Ins.InsCod = UsrDat->InsCod;
      Ins_GetDataOfInstitutionByCod (&Ins);
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\" colspan=\"2\" valign=\"top\">" \
                         "<table cellspacing=\"0\" cellpadding=\"0\" width=\"%u\">" \
                         "<tr><td align=\"left\" width=\"%u\"><img src=\"%s/%s/%s%ux%u.gif\" width=\"%u\" height=\"%u\" /></td>" \
                         "<td align=\"center\" class=\"%s\">%s",
               Cols1and2Width,Cols1and2Width,LogoSizeInScreen,
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Ins.Logo,LogoSize,LogoSize,LogoSizeInScreen,LogoSizeInScreen,
               ClassHead,Ins.FullName);
     }
   else
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\" colspan=\"2\" valign=\"top\">" \
                         "<table cellspacing=\"0\" cellpadding=\"0\" width=\"%u\">" \
                         "<tr><td align=\"left\" width=\"%u\"></td>" \
                         "<td align=\"center\" class=\"%s\">-",
               Cols1and2Width,Cols1and2Width,LogoSizeInScreen,
               ClassHead);
   fprintf (Gbl.F.Out,"<br />%s</td></tr></table></td>",
            UsrDat->FullName);

   /***** Photo *****/
   ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
   fprintf (Gbl.F.Out,"<td width=\"%u\" rowspan=\"%u\" colspan=\"2\" align=\"right\" valign=\"top\">",Col3Width,NumRowsPhoto);
   Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,
                     PhotoWidth,PhotoHeight,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** DNI *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER || IsForm || TypeOfView == MY_COMMON_RECORD_CHECK ||
       ((TypeOfView == RECORD_LIST || TypeOfView == RECORD_PRINT) &&
        Gbl.Usrs.Listing.Type == Usr_USER_LIST_RECORDS_COURSE_STUDENTS))
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">DNI:</td><td align=\"left\" width=\"%u\" class=\"%s\">",
               Col1Width,ClassForm,Col2Width,ClassData);
      fprintf (Gbl.F.Out,"%s",UsrDat->DNI);
      fprintf (Gbl.F.Out,"</td></tr>");
     }

   /***** User's type *****/
   if (IsForm)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                         "<td align=\"left\" width=\"%u\" class=\"%s\">",
               Col1Width,ClassForm,
               Txt_Type_of_user[Gbl.Prefs.Language],
               Col2Width,ClassData);
      switch (TypeOfView)
        {
         case FORM_MY_COMMON_RECORD:
            for (Gender=Usr_GENDER_FEMALE; Gender<=Usr_GENDER_MALE; Gender++)
              {
  	       fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"Gender\" value=\"%u\"",(unsigned) Gender);
               if (Gender == Gbl.Usrs.Me.UsrDat.Gender)
                  fprintf (Gbl.F.Out," checked=\"checked\"");
               fprintf (Gbl.F.Out," /><img src=\"%s/%s12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" align=\"bottom\" />%s",
                        Gbl.Prefs.IconsURL,StringsGenderDB[Gender],
                        Txt_USR_TYPES_SINGULAR_Abc[Gbl.Usrs.Me.UsrDat.UsrTypeInDB][Gender][Gbl.Prefs.Language],
                        Txt_USR_TYPES_SINGULAR_Abc[Gbl.Usrs.Me.UsrDat.UsrTypeInDB][Gender][Gbl.Prefs.Language]);
              }
	    break;
         case FORM_MODIF_RECORD_OTHER_USR:
  	    fprintf (Gbl.F.Out,"%s",Txt_USR_TYPES_SINGULAR_Abc[Gbl.Usrs.Other.UsrDat.UsrTypeInDB][Gbl.Usrs.Other.UsrDat.Gender][Gbl.Prefs.Language]);
	    break;
         case FORM_NEW_RECORD_OTHER_USR:
	    fprintf (Gbl.F.Out,"<select name=\"UsrType\">");
	    switch (Gbl.Usrs.Me.UsrTypeLog)
	      {
	       case Usr_TEACHER:	// A teacher can create students
	          fprintf (Gbl.F.Out,"<option value=\"%u\" selected=\"selected\">%s</option>",
                           (unsigned) Usr_STUDENT,Txt_USR_TYPES_SINGULAR_Abc[Usr_STUDENT][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language]);
	          break;
	       case Usr_DEG_ADMIN:	// An administrator or a superuser can create students and teachers
	       case Usr_SUPERUSER:
	          fprintf (Gbl.F.Out,"<option value=\"%u\" selected=\"selected\">%s</option>" \
                                     "<option value=\"%u\">%s</option>",
                           (unsigned) Usr_STUDENT,Txt_USR_TYPES_SINGULAR_Abc[Usr_STUDENT][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
                           (unsigned) Usr_TEACHER,Txt_USR_TYPES_SINGULAR_Abc[Usr_TEACHER][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language]);
	          break;
	       default:	// The rest of users can not register other users
	          break;
	      }
	    fprintf (Gbl.F.Out,"</select>");
	    break;
         default:
	    break;
        }
      fprintf (Gbl.F.Out,"</td></tr>");
     }

   /***** Name *****/
   /* Surname 1 */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td><td align=\"left\" width=\"%u\" class=\"%s\">",
            Col1Width,ClassForm,
            Txt_Surname_1[Gbl.Prefs.Language],
            Col2Width,ClassData);
   if (IsForm)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Surname1\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",
               Usr_MAX_LENGTH_USR_NAME_OR_SURNAME,UsrDat->Surname1);
   else if (UsrDat->Surname1[0])
      fprintf (Gbl.F.Out,"%s",UsrDat->Surname1);
   fprintf (Gbl.F.Out,"</td></tr>");

   /* Surname 2 */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td><td align=\"left\" width=\"%u\" class=\"%s\">",
            Col1Width,ClassForm,
            Txt_Surname_2[Gbl.Prefs.Language],
            Col2Width,ClassData);
   if (IsForm)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Surname2\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",
               Usr_MAX_LENGTH_USR_NAME_OR_SURNAME,UsrDat->Surname2);
   else if (UsrDat->Surname2[0])
      fprintf (Gbl.F.Out,"%s",UsrDat->Surname2);
   fprintf (Gbl.F.Out,"</td></tr>");

   /* First name */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td><td align=\"left\" width=\"%u\" class=\"%s\">",
            Col1Width,ClassForm,
            Txt_First_name[Gbl.Prefs.Language],
            Col2Width,ClassData);
   if (IsForm)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FirstName\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",
               Usr_MAX_LENGTH_USR_NAME_OR_SURNAME,UsrDat->FirstName);
   else if (UsrDat->FirstName[0])
      fprintf (Gbl.F.Out,"%s",UsrDat->FirstName);
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Internet data: e-mail, web and skype *****/
   /* E-mail */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" width=\"%u\">",
            Col1Width,ClassForm,
            Txt_E_mail[Gbl.Prefs.Language],
            Col2Width);
   if (IsForm && ShowData)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"E_mail\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",
               Cte_MAX_LENGTH_STRING,UsrDat->E_mail);
   else if (UsrDat->E_mail[0])
     {
      if (ShowData)
        {
         fprintf (Gbl.F.Out,"<a href=\"mailto:%s\"",UsrDat->E_mail);
         Str_LimitLengthHTMLStr (UsrDat->E_mail,36);
         fprintf (Gbl.F.Out," class=\"%s\">%s</a>",ClassData,UsrDat->E_mail);
        }
      else
         fprintf (Gbl.F.Out,"-");
     }
   fprintf (Gbl.F.Out,"</td></tr>");

   /* Web */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" width=\"%u\" class=\"%s\">",
            Col1Width,ClassForm,
            Txt_Web_page[Gbl.Prefs.Language],
            Col2Width,ClassData);
   if (IsForm && ShowData)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"WWW\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",Cte_MAX_LENGTH_URL,UsrDat->WWW);
   else if (UsrDat->WWW[0])
     {
      if (ShowData)
        {
         UsrHasWebPage = Str_URLLooksValid (UsrDat->WWW);
         if (UsrHasWebPage)
            fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\">",UsrDat->WWW);
         Str_LimitLengthHTMLStr (UsrDat->WWW,36);
         fprintf (Gbl.F.Out,"%s",UsrDat->WWW);
         if (UsrHasWebPage)
            fprintf (Gbl.F.Out,"</a>");
        }
      else
         fprintf (Gbl.F.Out,"-");
     }
   fprintf (Gbl.F.Out,"</td></tr>");

   /* Skype */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" width=\"%u\" class=\"%s\">",
            Col1Width,ClassForm,
            Txt_Name_on_Skype[Gbl.Prefs.Language],
            Col2Width,ClassData);
   if (IsForm && ShowData)
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Skype\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",Cte_MAX_LENGTH_STRING,UsrDat->Skype);
   else if (UsrDat->Skype[0])
     {
      if (ShowData)
         fprintf (Gbl.F.Out,"<a href=\"skype:%s?call\" class=\"%s\">%s</a>",UsrDat->Skype,ClassData,UsrDat->Skype);
      else
         fprintf (Gbl.F.Out,"-");
     }
   fprintf (Gbl.F.Out,"</td>");

   /* Public / private photo */
   if (TypeOfView == FORM_MY_COMMON_RECORD)	// ShowData == true
     {
      fprintf (Gbl.F.Out,"<td width=\"%u\" colspan=\"2\" align=\"center\">",PhotoWidth);
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" width=\"100%%\"><tr><td align=\"right\">");
      fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"PublicPhoto\" value=\"Y\"");
      if (UsrDat->PublicPhoto)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," /></td><td align=\"left\" class=\"%s\">%s</td></tr></table></td>",
               ClassForm,Txt_Public_Photo[Gbl.Prefs.Language]);
     }
   else if (TypeOfView == FORM_MODIF_RECORD_OTHER_USR ||
            TypeOfView == FORM_NEW_RECORD_OTHER_USR ||
            TypeOfView == MY_COMMON_RECORD_CHECK ||
            (TypeOfView == RECORD_LIST && Gbl.Usrs.Listing.Type == Usr_USER_LIST_RECORDS_COURSE_STUDENTS))
      fprintf (Gbl.F.Out,"<td width=\"%u\" colspan=\"2\" align=\"center\" class=\"%s\">%s</td>",
               PhotoWidth,ClassData,
               ShowData ?
               (UsrDat->PublicPhoto ? Txt_Public_Photo[Gbl.Prefs.Language] :
                                      Txt_Private_Photo[Gbl.Prefs.Language]) : "-");
   fprintf (Gbl.F.Out,"</tr>");

   /* Institution */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" width=\"%u\" colspan=\"3\" class=\"%s\">",
            Col1Width,ClassForm,Txt_Institution[Gbl.Prefs.Language],CardWidth-Col1Width,ClassData);
   if (IsForm && ShowData)
     {
      /***** If list of institutions is empty, try to get it *****/
      if (!Gbl.Inss.Num)
         Ins_GetListInstitutions (false);

      fprintf (Gbl.F.Out,"<select name=\"InsCod\" style=\"width:300px;\">" \
                         "<option value=\"-1\"></option>");
      fprintf (Gbl.F.Out,"<option value=\"0\"");
      if (UsrDat->InsCod == 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
      for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
        {
         fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.Inss.Lst[NumIns].InsCod);
         if (Gbl.Inss.Lst[NumIns].InsCod == UsrDat->InsCod)
            fprintf (Gbl.F.Out," selected=\"selected\"");
         fprintf (Gbl.F.Out,">%s</option>",Gbl.Inss.Lst[NumIns].FullName);
        }
      fprintf (Gbl.F.Out,"</select>");
     }
   else if (UsrDat->InsCod > 0)
     {
      if (ShowData)
        {
         if (Ins.WWW[0])
            fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
                     Ins.WWW,ClassData);
         fprintf (Gbl.F.Out,"%s",Ins.ShortName);
         if (Ins.WWW[0])
            fprintf (Gbl.F.Out,"</a>");
        }
      else
         fprintf (Gbl.F.Out,"-");
     }
   fprintf (Gbl.F.Out,"</td></tr>");

   /**** Data depending on type of user *****/
   if (!(TypeOfView == FORM_NEW_RECORD_OTHER_USR || TypeOfView == FORM_MODIF_RECORD_OTHER_USR))
     {
      switch (UsrDat->UsrTypeInDB)
	{
	 case Usr_TEACHER:
	    /***** Centre, department, office, etc. *****/
	    /* Centre */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                               "<td align=\"left\" width=\"%u\" colspan=\"3\" class=\"%s\">",
                     Col1Width,ClassForm,Txt_Centre[Gbl.Prefs.Language],CardWidth-Col1Width,ClassData);
	    if (IsForm && ShowData)
	      {
               /***** If list of centres is empty, try to get it *****/
               if (!Gbl.Ctrs.Num)
                  Ctr_GetListCentres ();

	       fprintf (Gbl.F.Out,"<select name=\"CtrCod\" style=\"width:300px;\">" \
                                  "<option value=\"-1\"></option>");
	       fprintf (Gbl.F.Out,"<option value=\"0\"");
	       if (UsrDat->Tch.CtrCod == 0)
		  fprintf (Gbl.F.Out," selected=\"selected\"");
	       fprintf (Gbl.F.Out,">%s</option>",Txt_Another_centre[Gbl.Prefs.Language]);
	       for (NumCtr=0; NumCtr<Gbl.Ctrs.Num; NumCtr++)
		 {
		  fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.Ctrs.Lst[NumCtr].CtrCod);
		  if (Gbl.Ctrs.Lst[NumCtr].CtrCod == UsrDat->Tch.CtrCod)
		     fprintf (Gbl.F.Out," selected=\"selected\"");
		  fprintf (Gbl.F.Out,">%s</option>",Gbl.Ctrs.Lst[NumCtr].FullName);
		 }
	       fprintf (Gbl.F.Out,"</select>");
	      }
	    else if (UsrDat->Tch.CtrCod > 0)
	      {
               if (ShowData)
                 {
                  Ctr.CtrCod = UsrDat->Tch.CtrCod;
                  Ctr_GetDataOfCentreByCod (&Ctr);
                  if (Ctr.WWW[0])
                     fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
                              Ctr.WWW,ClassData);
                  fprintf (Gbl.F.Out,"%s",Ctr.ShortName);
                  if (Ctr.WWW[0])
                     fprintf (Gbl.F.Out,"</a>");
                 }
               else
                  fprintf (Gbl.F.Out,"-");
	      }
	    fprintf (Gbl.F.Out,"</td></tr>");

	    /* Department */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                               "<td align=\"left\" width=\"%u\" colspan=\"3\" class=\"%s\">",
                     Col1Width,ClassForm,Txt_Department[Gbl.Prefs.Language],CardWidth-Col1Width,ClassData);
	    if (IsForm && ShowData)
	      {
               /***** If list of departments is empty, try to get it *****/
               if (!Gbl.Dpts.Num)
                  Dpt_GetListDepartments ();

	       fprintf (Gbl.F.Out,"<select name=\"DptCod\" style=\"width:300px;\">" \
                                  "<option value=\"-1\"></option>");
	       fprintf (Gbl.F.Out,"<option value=\"0\"");
	       if (UsrDat->Tch.DptCod == 0)
		  fprintf (Gbl.F.Out," selected=\"selected\"");
	       fprintf (Gbl.F.Out,">%s</option>",Txt_Another_department[Gbl.Prefs.Language]);
	       for (NumDpt=0; NumDpt<Gbl.Dpts.Num; NumDpt++)
		 {
		  fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.Dpts.Lst[NumDpt].DptCod);
		  if (Gbl.Dpts.Lst[NumDpt].DptCod == UsrDat->Tch.DptCod)
		     fprintf (Gbl.F.Out," selected=\"selected\"");
		  fprintf (Gbl.F.Out,">%s</option>",Gbl.Dpts.Lst[NumDpt].FullName);
		 }
	       fprintf (Gbl.F.Out,"</select>");
	      }
	    else if (UsrDat->Tch.DptCod > 0)
              {
               if (ShowData)
	         {
                  Dpt.DptCod = UsrDat->Tch.DptCod;
                  Dpt_GetDataOfDepartmentByCod (&Dpt);
                  if (Dpt.WWW[0])
                     fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
                              Dpt.WWW,ClassData);
                  fprintf (Gbl.F.Out,"%s",Dpt.ShortName);
                  if (Dpt.WWW[0])
                     fprintf (Gbl.F.Out,"</a>");
                 }
               else
                  fprintf (Gbl.F.Out,"-");
	      }
	    fprintf (Gbl.F.Out,"</td></tr>");

	    /* Office */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                               "<td align=\"left\" width=\"%u\" class=\"%s\">",
                     Col1Width,ClassForm,
                     Txt_Office[Gbl.Prefs.Language],
                     Col2Width,ClassData);
	    if (IsForm && ShowData)
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Office\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",Cte_MAX_LENGTH_STRING,UsrDat->Tch.Office);
	    else if (UsrDat->Tch.Office[0])
	       fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Tch.Office : "-");
	    fprintf (Gbl.F.Out,"</td>");

	    /* Phone */
	    fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">%s:</td><td align=\"left\" class=\"%s\">",
                     ClassForm,
                     Txt_Phone[Gbl.Prefs.Language],
                     ClassData);
	    if (IsForm && ShowData)
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"OfficePhone\" size=\"8\" maxlength=\"%u\" value=\"%s\" />",
                        Usr_MAX_LENGTH_PHONE,UsrDat->Tch.OfficePhone);
	    else if (UsrDat->Tch.OfficePhone[0])
	       fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Tch.OfficePhone : "-");
	    fprintf (Gbl.F.Out,"</td></tr>");
	    break;
	 default:
	    /***** Addresses and phone *****/
	    /* Local address */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                           "<td align=\"left\" width=\"%u\" class=\"%s\">",
                     Col1Width,ClassForm,
                     Txt_Local_address[Gbl.Prefs.Language],
                     Col2Width,ClassData);
	    if (IsForm && ShowData)
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"LocalAddress\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",Cte_MAX_LENGTH_STRING,UsrDat->Std.LocalAddress);
	    else if (UsrDat->Std.LocalAddress[0])
	       fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.LocalAddress : "-");
	    fprintf (Gbl.F.Out,"</td>");

	    /* Local phone */
	    fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",ClassForm);
	    if (IsForm && ShowData)
	      {
	       fprintf (Gbl.F.Out,"%s:</td><td align=\"left\">",Txt_Phone[Gbl.Prefs.Language]);
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"LocalPhone\" size=\"8\" maxlength=\"%u\" value=\"%s\" />",
                        Usr_MAX_LENGTH_PHONE,UsrDat->Std.LocalPhone);
	      }
	    else
	      {
	       fprintf (Gbl.F.Out,"%s:</td><td align=\"left\" class=\"%s\">",Txt_Phone[Gbl.Prefs.Language],ClassData);
               if (UsrDat->Std.LocalPhone[0])
	          fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.LocalPhone : "-");
	      }
	    fprintf (Gbl.F.Out,"</td></tr>");

	    /* Family address */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                               "<td align=\"left\" width=\"%u\" class=\"%s\">",
                     Col1Width,ClassForm,Txt_Family_address[Gbl.Prefs.Language],Col2Width,ClassData);
	    if (IsForm && ShowData)
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FamilyAddress\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",
                        Cte_MAX_LENGTH_STRING,UsrDat->Std.FamilyAddress);
	    else if (UsrDat->Std.FamilyAddress[0])
	       fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.FamilyAddress : "-");
	    fprintf (Gbl.F.Out,"</td>");

	    /* Family phone */
            fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",ClassForm);
	    if (IsForm && ShowData)
	      {
	       fprintf (Gbl.F.Out,"%s:</td><td align=\"left\">",Txt_Phone[Gbl.Prefs.Language]);
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FamilyPhone\" size=\"8\" maxlength=\"%u\" value=\"%s\" />",
                        Usr_MAX_LENGTH_PHONE,UsrDat->Std.FamilyPhone);
	      }
	    else
	      {
	       fprintf (Gbl.F.Out,"%s:</td><td align=\"left\" class=\"%s\">",Txt_Phone[Gbl.Prefs.Language],ClassData);
               if (UsrDat->Std.FamilyPhone[0])
	          fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.FamilyPhone : "-");
	      }
	    fprintf (Gbl.F.Out,"</td></tr>");

	    /***** Origin place and date of birth *****/
	    /* Origin place */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\" class=\"%s\">%s:</td>" \
                               "<td align=\"left\" width=\"%u\" class=\"%s\">",
                     Col1Width,ClassForm,Txt_Place_of_origin[Gbl.Prefs.Language],Col2Width,ClassData);
	    if (IsForm && ShowData)
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"OriginPlace\" size=\"36\" maxlength=\"%u\" value=\"%s\" />",Cte_MAX_LENGTH_STRING,UsrDat->Std.OriginPlace);
	    else if (UsrDat->Std.OriginPlace[0])
	       fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.OriginPlace : "-");
	    fprintf (Gbl.F.Out,"</td><td></td><td></td></tr>");

	    /* Date of birth */
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"%s\">%s:</td>",
                     ClassForm,Txt_Date_of_birth[Gbl.Prefs.Language]);
	    if (IsForm && ShowData)
              {
               fprintf (Gbl.F.Out,"<td align=\"left\">");
	       Dat_WriteFormDate (Gbl.Today.Year-99,Gbl.Today.Year-16,"DiaNac","MesNac","AnoNac",
		                  UsrDat->Std.Birthday.Day,UsrDat->Std.Birthday.Month,UsrDat->Std.Birthday.Year,
                                  false,false);
               fprintf (Gbl.F.Out,"</td>");
              }
	    else
              {
	       fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",ClassData);
               if (UsrDat->Std.StrBirthday[0]) 
	         fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.StrBirthday : "-");
	       fprintf (Gbl.F.Out,"</td>");
              }
	    fprintf (Gbl.F.Out,"<td></td><td></td></tr>");

            /***** Common comments for all the courses *****/
	    fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" class=\"%s\">%s:</td>" \
                               "<td align=\"left\" valign=\"top\" colspan=\"3\" class=\"%s\">",
                     ClassForm,Txt_USER_comments[Gbl.Prefs.Language],ClassData);
	    if (IsForm && ShowData)
               fprintf (Gbl.F.Out,"<textarea name=\"Comments\" cols=\"48\" rows=\"3\">%s</textarea>",UsrDat->Std.Comments);
	    else if (UsrDat->Std.Comments[0])
              {
               Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,UsrDat->Std.Comments,Cte_MAX_BYTES_TEXT,false);	// Se convierte of HTML a HTML respetuoso
	       fprintf (Gbl.F.Out,"%s",ShowData ? UsrDat->Std.Comments : "-");
              }
	    fprintf (Gbl.F.Out,"</td></tr>");
	}
     }

   /***** End of table *****/
   if (TypeOfView == RECORD_PRINT)
      Lay_EndTableFrame ();
   else
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr></table>\n");

   if (IsForm)
     {
      Lay_PutSendAndUndoButtons (TxtSubmitButton);
      fprintf (Gbl.F.Out,"</form>\n");
     }
   }
