// swad_sta.h: statistics

#ifndef _SWAD_STA
#define _SWAD_STA
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_usr.h"

/****************************************************************************/
/************************** Public types and constants **********************/
/****************************************************************************/

#define Sta_LOG_START_YEAR	2005	// Year  where the log started in the database
#define Sta_LOG_START_MONTH	  01	// Month where the log started in the database
#define Sta_LOG_START_DAY	  01	// Day   where the log started in the database

#define Sta_NUM_USR_TYPES_STAT	8
typedef enum
  {
   Sta_IDENTIFIED_USRS = 0,
   Sta_ALL_USRS        = 1,
   Sta_ADMINISTRATORS  = 2,
   Sta_TEACHERS        = 3,
   Sta_STUDENTS        = 4,
   Sta_INVITED_USRS    = 5,
   Sta_UNKNOWN_USRS    = 6,
   Sta_ME              = 7,
  } Sta_UsrTypeStat_t;

#define Sta_NUM_STAT_COUNT_TYPES	5
typedef enum
  {
   Sta_TOTAL_CLICKS     = 0,
   Sta_DISTINCT_USRS    = 1,
   Sta_CLICKS_PER_USR   = 2,
   Sta_GENERATION_TIME  = 3,
   Sta_SEND_TIME	= 4,
  } Sta_CountType_t;

#define Sta_NUM_TYPES_CLICK_STATS 18
typedef enum
  {
   ACC_CRS_LISTING           =  0,
   ACC_CRS_PER_USR           =  1,
   ACC_CRS_PER_DAYS          =  2,
   ACC_CRS_PER_DAYS_AND_HOUR =  3,
   ACC_CRS_PER_WEEKS         =  4,
   ACC_CRS_PER_MONTHS        =  5,
   ACC_CRS_PER_HOUR          =  6,
   ACC_CRS_PER_MINUTE        =  7,
   ACC_CRS_PER_ACTION        =  8,
   ACC_GBL_PER_DAYS          =  9,
   ACC_GBL_PER_DAYS_AND_HOUR = 10,
   ACC_GBL_PER_WEEKS         = 11,
   ACC_GBL_PER_MONTHS        = 12,
   ACC_GBL_PER_HOUR          = 13,
   ACC_GBL_PER_MINUTE        = 14,
   ACC_GBL_PER_ACTION        = 15,
   ACC_GBL_PER_DEGREE        = 16,
   ACC_GBL_PER_COURSE        = 17,
  } Sta_ClicksStatType_t;

#define Sta_NUM_TYPES_USE_STATS 10
typedef enum
  {
   Sta_DEGREES_AND_COURSES,	// Number of degrees and courses
   Sta_USERS,			// Number of users
   Sta_FOLDERS_AND_FILES,	// Number of folders and files
   Sta_MSGS_BETWEEN_USERS,	// Number of users' (sent and received) messages
   Sta_FORUMS,			// Number of forums, threads and posts
   Sta_NOTICES,			// Number of notices
   Sta_LANGUAGES,		// Number of users per language
   Sta_THEMES,			// Number of users per theme
   Sta_COLUMNS,			// Number of users per design of columns
   Sta_NOTIFY_EVENTS,		// Number of users per notify event
  } Sta_UseStatType_t;

#define Sta_NUM_COLOR_TYPES 3
typedef enum
  {
   Sta_COLOR,
   Sta_BLACK_TO_WHITE,
   Sta_WHITE_TO_BLACK,
  } Sta_ColorType_t;

#define Sta_NUM_STAT_CRS_FILE_ZONES 9

/***** Average photos of students in degrees ******/

typedef enum
  {
   Sta_DEGREES_SEE,
   Sta_DEGREES_PRINT,
  } Sta_AvgPhotoSeeOrPrint_t;

#define Sta_NUM_AVERAGE_PHOTO_TYPES 2
typedef enum
  {
   Sta_PHOTO_MEDIAN_ALL  = 0,
   Sta_PHOTO_AVERAGE_ALL = 1,
  } Sta_AvgPhotoTypeOfAverage_t;

#define Sta_NUM_HOW_COMPUTE_PHOTO_SIZES 4
typedef enum
  {
   Sta_PROPORTIONAL_TO_NUMBER_OF_STUDENTS = 0,
   Sta_PROPORTIONAL_TO_NUMBER_OF_PHOTOS   = 1,
   Sta_PROPORTIONAL_TO_PERCENT            = 2,
   Sta_UNIQUE_SIZE                        = 3,
  } Sta_HowComputePhotoSize_t;

#define Sta_NUM_HOW_ORDER_DEGREES 4
typedef enum
  {
   Sta_NUMBER_OF_STUDENTS = 0,
   Sta_NUMBER_OF_PHOTOS   = 1,
   Sta_PERCENT            = 2,
   Sta_DEGREE_NAME        = 3,
  } Sta_HowOrderDegrees_t;

// Related with range of courses-degrees when computing statistics
#define NUM_TYPES_OF_LOCATION_RANGE	3
typedef enum
  {
   Sta_LOCATION_RANGE_ALL         = 0,
   Sta_LOCATION_RANGE_THIS_DEGREE = 1,
   Sta_LOCATION_RANGE_THIS_COURSE = 2,
  } Sta_LocationRange_t;

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Sta_GetRemoteAddr (void);
void Sta_LogAccess (const char *Comments);
void Sta_AskSeeCrsAccesses (void);
void Sta_AskSeeGblAccesses (void);
void Sta_SetIniEndDates (void);
void Sta_SeeCrsAccesses (void);
void Sta_SeeGblAccesses (void);
void Sta_ReqUseOfPlatform (void);
void Sta_ShowUseOfPlatform (void);
unsigned Sta_GetTotalNumberOfUsers (Sta_LocationRange_t LocationRange,Usr_UsrType_t UsrType);
void Sta_WriteParamsDatesSeeAccesses (void);

void Sta_ReqStatsCourses (void);
void Sta_ShowStatsCourses (void);

void Sta_CalcPhotoDegree (void);
void Sta_RemoveObsoleteStatDegrees (void);
void Sta_ShowPhotoDegree (void);
void Sta_PrintPhotoDegree (void);
void Sta_ShowOrPrintPhotoDegree (Sta_AvgPhotoSeeOrPrint_t SeeOrPrint);
void Sta_PutHiddenParamTypeOfAvg (void);
void Sta_PutHiddenParamPhotoSize (void);
void Sta_PutHiddenParamOrderDegrees (void);

void Sta_ComputeTimeToGeneratePage (void);
void Sta_ComputeTimeToSendPage (void);
void Sta_WriteTimeToGenerateAndSendPage (void);

#endif
