// swad_syl.c: syllabus

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdlib.h>	// For free ()
#include <string.h>	// For strcat (), etc.
#include <time.h>	// For time ()

#include "swad_cfg.h"
#include "swad_gbl.h"
#include "swad_log.h"
#include "swad_par.h"
#include "swad_str.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

#define MAX_LEVELS_SYLLABUS 10
#define MAX_LENGTH_TEXTO_ITEM_PRG	1024
#define MAX_BYTES_TEXTO_ITEM_PRG (MAX_LENGTH_TEXTO_ITEM_PRG*Str_MAX_CHARACTER)

#define WIDTH_NUM_SYLLABUS 16

// End of line in a file. If we put only \n the file doesn't look good in some Windows text editors
const char NEW_LINE[] = {13,10,0};

const char *StringsTypeSyllabusFolder[2] = {Cfg_SYLLABUS_FOLDER_LECTURES,Cfg_SYLLABUS_FOLDER_PRACTICALS};
const char *StringsTypeSyllabusScreen[2] = {"teor&iacute;a"       ,"pr&aacute;cticas"};

char *FuentePrg[1+MAX_LEVELS_SYLLABUS] = {"","T1","T2","T3","T3","T3","T3","T3","T3","T3","T3"};

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

struct ItemSyllabus
  {
   int Level;
   int CodItem[1+MAX_LEVELS_SYLLABUS];
   bool HaveChildren;
   char Text[MAX_BYTES_TEXTO_ITEM_PRG+1];
  };

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

struct
  {
   struct ItemSyllabus *Lst;	// List of items of a syllabus
   int NumItems;		// Number of items in the list
   int NumItemsWithChildren;	// Number of items with children
   int NumLevels;		// Number of levels in the list
  } LstItemsSyllabus;

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

/****************************************************************************/
/************** Get parameter item number in program edition ****************/
/****************************************************************************/

void Syl_GetParamItemNumber (void)
  {
   char InStr[1+10+1];

   Par_GetParam (Par_PARAM_SINGLE,"NumI",InStr,10);
   if (sscanf (InStr,"%d",&Gbl.CurrentCrs.Syllabus.NumItem) != 1)
      Lay_ShowErrorAndExit ("Item is missing.");
  }

/****************************************************************************/
/****************************** Edit a syllabus *****************************/
/****************************************************************************/

void Syl_EditSyllabus (void)
  {
   extern const char *Txt_Enter_a_new_item_here[Txt_NUM_LANGUAGES];

   if (Gbl.CurrentAct == ActEditorSylLec ||
       Gbl.CurrentAct == ActEditorSylPra)
      Gbl.CurrentCrs.Syllabus.EditionIsActive = true;

   if (Gbl.CurrentCrs.Syllabus.EditionIsActive || LstItemsSyllabus.NumItems)
     {
      /***** Start of table *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      fprintf (Gbl.F.Out,"<tr><td align=\"left\"><table cellspacing=\"0\" cellpadding=\"1\" width=\"100%%\">");
      // fprintf (Gbl.F.Out,"<tr><td align=\"left\"><table width=\"100%%\" border=\"1\" bordercolor=\"#%s\" cellspacing=\"0\" cellpadding=\"0\">",DARK_GRAY);

      /***** Write the syllabus actual *****/
      Syl_ShowSyllabus ();

      /***** If the syllabus is empty ==> show form to aadir a tema to the final *****/
      if (Gbl.CurrentCrs.Syllabus.EditionIsActive && LstItemsSyllabus.NumItems == 0)
         Syl_ShowRowSyllabus (0,1,LstItemsSyllabus.Lst[0].CodItem,Txt_Enter_a_new_item_here[Gbl.Prefs.Language],true);

      /***** End of table *****/
      // fprintf (Gbl.F.Out,"</table></td></tr>");
      fprintf (Gbl.F.Out,"</table></td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      if (!Gbl.CurrentCrs.Syllabus.EditionIsActive)
         fprintf (Gbl.F.Out,"<div align=\"center\" class=\"DAT_SMALL\"><br />" \
                            "Este programa ha sido editado por los profesores de la asignatura" \
                            " y podr&iacute;a no coincidir con el programa oficial.</div>");	// Need translation!
     }
   else
     {
      sprintf (Gbl.Message,"El programa de %s de %s no est&aacute; disponible.",	// Need translation!
               StringsTypeSyllabusScreen[Gbl.CurrentCrs.Syllabus.Type],Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
  }

/****************************************************************************/
/** Read from XML and load in memory a syllabus of lectures or practicals ***/
/****************************************************************************/

void Syl_LoadSyllabusToMem (void)
  {
   char PathFile[PATH_MAX+1];
   long PosInicioLista;
   int NumItem = 0,N;
   int CodItem[1+MAX_LEVELS_SYLLABUS];	// To make numeration
   int Result;
   int NumItemsWithChildren = 0;

   /***** Get the type of syllabus (lectures or practicals) *****/
   switch (Gbl.CurrentAct)
     {
      case ActSeeSylLec:
      case ActEditorSylLec:
      case ActDelItmSylLec:
      case ActUp_IteSylLec:
      case ActDwnIteSylLec:
      case ActRgtIteSylLec:
      case ActLftIteSylLec:
      case ActInsIteSylLec:
      case ActModIteSylLec:
      case ActEdiSylLec:
	 Gbl.CurrentCrs.Syllabus.Type = Syl_SYLLABUS_LECTURES;
	 break;
      case ActSeeSylPra:
      case ActEditorSylPra:
      case ActDelItmSylPra:
      case ActUp_IteSylPra:
      case ActDwnIteSylPra:
      case ActRgtIteSylPra:
      case ActLftIteSylPra:
      case ActInsIteSylPra:
      case ActModIteSylPra:
      case ActEdiSylPra:
	 Gbl.CurrentCrs.Syllabus.Type = Syl_SYLLABUS_PRACTICALS;
     }

   /* Path of the private directory where estar the file XML with the syllabus of lectures */
   sprintf (Gbl.CurrentCrs.Syllabus.PathDir,"%s/%s",Gbl.CurrentCrs.PathRelPriv,StringsTypeSyllabusFolder[Gbl.CurrentCrs.Syllabus.Type]);

   /***** Open the file with the syllabus *****/
   Syl_OpenSyllabusFile (Gbl.CurrentCrs.Syllabus.PathDir,PathFile);

   /***** Go to the start of the list of items *****/
   if (!Str_FindStrInFile (Gbl.F.Syllabus,"<lista>",Str_NO_SKIP_HTML_COMMENTS))
      Lay_ShowErrorAndExit ("Wrong syllabus format.");

   /***** Save the position of the start of the list *****/
   PosInicioLista = ftell (Gbl.F.Syllabus);

   /***** Loop to count the number of items *****/
   for (LstItemsSyllabus.NumItems=0; Str_FindStrInFile (Gbl.F.Syllabus,"<item",Str_NO_SKIP_HTML_COMMENTS); LstItemsSyllabus.NumItems++);

   /***** Allocate memory for the list of items *****/
   if ((LstItemsSyllabus.Lst = (struct ItemSyllabus *) calloc (LstItemsSyllabus.NumItems + 1,sizeof (struct ItemSyllabus))) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store syllabus.");

   /***** Return to the start of the list *****/
   fseek (Gbl.F.Syllabus,PosInicioLista,SEEK_SET);

   for (N=1; N<=MAX_LEVELS_SYLLABUS; N++)
      CodItem[N] = 0;
   LstItemsSyllabus.NumLevels = 1;

   /***** If the syllabus is empty ==> initialize an item to be edited *****/
   if (LstItemsSyllabus.NumItems == 0)
     {
      /* Level of the item */
      LstItemsSyllabus.Lst[0].Level = 1;

      /* Code (numeracin) of the item */
      CodItem[1] = 1;
      for (N=1; N<=MAX_LEVELS_SYLLABUS; N++)
	 LstItemsSyllabus.Lst[0].CodItem[N] = CodItem[N];

      /* Text of the item  */
      LstItemsSyllabus.Lst[0].Text[0] = '\0';
     }
   else
      /***** Loop to read and store all the items of the syllabus *****/
      for (NumItem=0; NumItem<LstItemsSyllabus.NumItems; NumItem++)
	{
	 /* Go to the start of the item */
	 if (!Str_FindStrInFile (Gbl.F.Syllabus,"<item",Str_NO_SKIP_HTML_COMMENTS))
	    Lay_ShowErrorAndExit ("Wrong syllabus format.");

	 /* Get the level */
	 LstItemsSyllabus.Lst[NumItem].Level = Syl_ReadLevelItemSyllabus ();
	 if (LstItemsSyllabus.Lst[NumItem].Level > LstItemsSyllabus.NumLevels)
	    LstItemsSyllabus.NumLevels = LstItemsSyllabus.Lst[NumItem].Level;

	 /* Set the code (number) of the item */
	 CodItem[LstItemsSyllabus.Lst[NumItem].Level]++;
	 for (N=LstItemsSyllabus.Lst[NumItem].Level+1; N<=MAX_LEVELS_SYLLABUS; N++)
	    CodItem[N] = 0;
	 for (N=1; N<=MAX_LEVELS_SYLLABUS; N++)
	    LstItemsSyllabus.Lst[NumItem].CodItem[N] = CodItem[N];

	 /* Get the text of the item */
	 Result = Str_ReceiveFileUntilDelimitStr (Gbl.F.Syllabus, NULL, LstItemsSyllabus.Lst[NumItem].Text,"</item>",(unsigned long long) MAX_BYTES_TEXTO_ITEM_PRG);
	 if (Result == 0) // Str too long
	   {
	    if (!Str_FindStrInFile (Gbl.F.Syllabus,"</item>",Str_NO_SKIP_HTML_COMMENTS)) // End the search
	       Lay_ShowErrorAndExit ("Wrong syllabus format.");
	   }
	 else if (Result == -1)
	    Lay_ShowErrorAndExit ("Wrong syllabus format.");
	}

   /***** Close the file with the syllabus *****/
   Syl_CloseFileSyllabus ();

   /***** Initialize other fields in the list *****/
   if (LstItemsSyllabus.NumItems > 0)
     {
      for (NumItem=0; NumItem<LstItemsSyllabus.NumItems-1; NumItem++)
	{
	 if (LstItemsSyllabus.Lst[NumItem].Level < LstItemsSyllabus.Lst[NumItem+1].Level)
	   {
	    LstItemsSyllabus.Lst[NumItem].HaveChildren = true;
	    NumItemsWithChildren++;
	   }
	 else
	    LstItemsSyllabus.Lst[NumItem].HaveChildren = false;
	}
      LstItemsSyllabus.Lst[LstItemsSyllabus.NumItems-1].HaveChildren = false;
     }
   LstItemsSyllabus.NumItemsWithChildren = NumItemsWithChildren;
  }

/*****************************************************************************/
/*********************** Free list of items of a syllabus ********************/
/*****************************************************************************/

void Syl_FreeListItemsSyllabus (void)
  {
   if (LstItemsSyllabus.Lst)
     {
      free ((void *) LstItemsSyllabus.Lst);
      LstItemsSyllabus.Lst = NULL;
     }
  }

/****************************************************************************/
/***************** Read the level of the item actual of a syllabus **************/
/****************************************************************************/
// XML file with the syllabus must be positioned after <item
// XML with the syllabus becomes positioned after <item nivel="x">

int Syl_ReadLevelItemSyllabus (void)
  {
   int Level;
   char StrlLevel[11+1];

   if (!Str_FindStrInFile (Gbl.F.Syllabus,"nivel=\"",Str_NO_SKIP_HTML_COMMENTS))
      Lay_ShowErrorAndExit ("Wrong syllabus format.");
   if (Str_ReceiveFileUntilDelimitStr (Gbl.F.Syllabus,NULL,StrlLevel,"\"",(unsigned long long) (11+1)) != 1)
      Lay_ShowErrorAndExit ("Wrong syllabus format.");
   if (sscanf (StrlLevel,"%d",&Level) != 1)
      Lay_ShowErrorAndExit ("Wrong syllabus format.");
   Str_FindStrInFile (Gbl.F.Syllabus,">",Str_NO_SKIP_HTML_COMMENTS);
   if (Level < 1)
      Level = 1;
   else if (Level > MAX_LEVELS_SYLLABUS)
      Level = MAX_LEVELS_SYLLABUS;
   return Level;
  }

/****************************************************************************/
/****** Store the content of a syllabus of theory or practice into XML ******/
/****************************************************************************/

void Syl_ShowSyllabus (void)
  {
   extern const char *Txt_Enter_a_new_item_here[Txt_NUM_LANGUAGES];
   int i,NumItem,NumBotones = Gbl.CurrentCrs.Syllabus.EditionIsActive ? 5 : 0;
   bool ShowRowInsertNewItem = (Gbl.CurrentAct == ActInsIteSylLec || Gbl.CurrentAct == ActInsIteSylPra ||
                                Gbl.CurrentAct == ActModIteSylLec || Gbl.CurrentAct == ActModIteSylPra ||
				Gbl.CurrentAct == ActRgtIteSylLec || Gbl.CurrentAct == ActRgtIteSylPra ||
                                Gbl.CurrentAct == ActLftIteSylLec || Gbl.CurrentAct == ActLftIteSylPra);

   /***** Set ancho of columns of the table *****/
   fprintf (Gbl.F.Out,"<colgroup>");
   for (i=0; i<NumBotones; i++)
      fprintf (Gbl.F.Out,"<col width=\"12\" />");
   for (i=1; i<=LstItemsSyllabus.NumLevels; i++)
      fprintf (Gbl.F.Out,"<col width=\"%d\" />",i*WIDTH_NUM_SYLLABUS);
   fprintf (Gbl.F.Out,"<col width=\"*\" />");
   fprintf (Gbl.F.Out,"</colgroup>\n");

   /***** Loop for writing all items of the syllabus *****/
   for (NumItem=0; NumItem<LstItemsSyllabus.NumItems; NumItem++)
     {
      Syl_ShowRowSyllabus (NumItem,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].CodItem,LstItemsSyllabus.Lst[NumItem].Text,false);
      if (ShowRowInsertNewItem && NumItem == Gbl.CurrentCrs.Syllabus.NumItem)
	 // Mostrar a new row where se puede insert a new item
	 Syl_ShowRowSyllabus (NumItem+1,LstItemsSyllabus.Lst[NumItem].Level,NULL,Txt_Enter_a_new_item_here[Gbl.Prefs.Language],true);
     }
  }

/****************************************************************************/
/********* Write a row (item) of a syllabus of theory or practice ***********/
/****************************************************************************/

void Syl_ShowRowSyllabus (int NumItem,int Level,int *CodItem,const char *Text,bool NewItem)
  {
   static const char *Txt_Remove[Txt_NUM_LANGUAGES];
   static int LastLevel = 0;
   char StrItemCod[MAX_LEVELS_SYLLABUS*(10+1)];
   struct MoveSubtrees Subtree;

   Subtree.ToGetUp.Ini = Subtree.ToGetUp.End = 0;
   Subtree.ToGetDown.Ini = Subtree.ToGetDown.End = 0;
   Subtree.MovAllowed = false;

   if (!NewItem)	// If the item is new (not stored in file), it has no number
      Syl_WriteNumItem (StrItemCod,NULL,NumItem,Level,CodItem);

   /***** Start the row *****/
   fprintf (Gbl.F.Out,"<tr>");

   if (Gbl.CurrentCrs.Syllabus.EditionIsActive)
     {
      if (NewItem)
	 fprintf (Gbl.F.Out,"<td align=\"left\" colspan=\"5\" bgcolor=\"#%s\"></td>",Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      else
	{
	 /***** Icon to eliminar the row *****/
         fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
	 if (LstItemsSyllabus.Lst[NumItem].HaveChildren)
            fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                     Gbl.Prefs.IconsURL);
	 else
	   {
	    Act_FormStart ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActDelItmSylLec : ActDelItmSylPra);
	    Syl_PutParamNumItem (NumItem);
	    sprintf (Gbl.MsgWindowStatus,"%s %s",Txt_Remove[Gbl.Prefs.Language],StrItemCod);
            Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
            fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                     Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
	   }
         fprintf (Gbl.F.Out,"</td>");

	 /***** Icon to get up an item *****/
	 Syl_CalculateUpSubtreeSyllabus (&Subtree,NumItem);
	 fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
	 if (Subtree.MovAllowed)
	   {
	    Act_FormStart ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActUp_IteSylLec : ActUp_IteSylPra);
	    Syl_PutParamNumItem (NumItem);
	    sprintf (Gbl.MsgWindowStatus,"Subir %s%s",	// Need translation!
                     StrItemCod,LstItemsSyllabus.Lst[NumItem].HaveChildren ? " y sus subapartados" : "");	// Need translation!
	    Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
	    fprintf (Gbl.F.Out,"<img src=\"%s/farriba12x12.png\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                     Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
	   }
	 else
	    fprintf (Gbl.F.Out,"<img src=\"%s/farriba_off12x12.png\" width=\"12\" height=\"12\" />",
                     Gbl.Prefs.IconsURL);
         fprintf (Gbl.F.Out,"</td>");

	 /***** Icon to get down item *****/
	 Syl_CalculateDownSubtreeSyllabus (&Subtree,NumItem);
	 fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
	 if (Subtree.MovAllowed)
	   {
	    Act_FormStart ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActDwnIteSylLec : ActDwnIteSylPra);
	    Syl_PutParamNumItem (NumItem);
	    sprintf (Gbl.MsgWindowStatus,"Bajar %s%s",StrItemCod,	// Need translation!
                     LstItemsSyllabus.Lst[NumItem].HaveChildren ? " y sus subapartados" : "");	// Need translation!
	    Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
	    fprintf (Gbl.F.Out,"<img src=\"%s/fabajo12x12.png\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                     Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
	   }
	 else
	    fprintf (Gbl.F.Out,"<img src=\"%s/fabajo_off12x12.png\" width=\"12\" height=\"12\" />",
                     Gbl.Prefs.IconsURL);
         fprintf (Gbl.F.Out,"</td>");

	 /***** Icon to increase the level of an item *****/
	 fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
	 if (Level > 1)
	   {
	    Act_FormStart ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActRgtIteSylLec : ActRgtIteSylPra);
	    Syl_PutParamNumItem (NumItem);
	    sprintf (Gbl.MsgWindowStatus,"Aumentar nivel de %s",StrItemCod);	// Need translation!
	    Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
	    fprintf (Gbl.F.Out,"<img src=\"%s/fizda12x12.png\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                     Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
	   }
	 else
	    fprintf (Gbl.F.Out,"<img src=\"%s/fizda_off12x12.png\" width=\"12\" height=\"12\" />",
                     Gbl.Prefs.IconsURL);
         fprintf (Gbl.F.Out,"</td>");

	 /***** Icon to decrease level item *****/
	 fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
	 if (Level < LastLevel+1 && Level < MAX_LEVELS_SYLLABUS)
	   {
	    Act_FormStart ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActLftIteSylLec : ActLftIteSylPra);
	    Syl_PutParamNumItem (NumItem);
	    sprintf (Gbl.MsgWindowStatus,"Disminuir nivel de %s",StrItemCod);	// Need translation!
	    Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
	    fprintf (Gbl.F.Out,"<img src=\"%s/fdcha12x12.png\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                     Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
	   }
	 else
	    fprintf (Gbl.F.Out,"<img src=\"%s/fdcha_off12x12.png\" width=\"12\" height=\"12\" />",
                     Gbl.Prefs.IconsURL);
         fprintf (Gbl.F.Out,"</td>");

	 LastLevel = Level;
	}
     }

   if (Gbl.CurrentCrs.Syllabus.EditionIsActive)
      Syl_PutFormItemSyllabus (NewItem,NumItem,Level,CodItem,Text,Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
   else
     {
      /***** Indent depending on the level *****/
      if (Level > 1)
	 fprintf (Gbl.F.Out,"<td align=\"left\" colspan=\"%d\" bgcolor=\"#%s\"></td>",
                  Level-1,Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);

      /***** Code of the item *****/
      fprintf (Gbl.F.Out,"<td width=\"%d\" align=\"right\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">",
               Level*WIDTH_NUM_SYLLABUS,Gbl.ColorRows[Gbl.GlobalRowEvenOdd],FuentePrg[Level]);
      if (Level == 1)
	 fprintf (Gbl.F.Out,"&nbsp;");
      fprintf (Gbl.F.Out,"%s&nbsp;</td>",StrItemCod);

      /***** Text of the item *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" colspan=\"%d\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">",
               LstItemsSyllabus.NumLevels-Level+1,Gbl.ColorRows[Gbl.GlobalRowEvenOdd],FuentePrg[Level]);
      fprintf (Gbl.F.Out,Text);
      fprintf (Gbl.F.Out,"</td>");
     }

   /***** End of the row *****/
   fprintf (Gbl.F.Out,"</tr>");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }

/*****************************************************************************/
/******** Show a form to modificar an item of the syllabus *********/
/*****************************************************************************/

void Syl_PutFormItemSyllabus (bool NewItem,int NumItem,int Level,int *CodItem,const char *Text,const char *BgColor)
  {
   extern const char *Txt_Enter_a_new_item_here[Txt_NUM_LANGUAGES];

   if (Level < 1)
      Level = 1;

   /***** Indent depending on the level *****/
   if (Level > 1)
      fprintf (Gbl.F.Out,"<td align=\"left\" colspan=\"%d\" bgcolor=\"#%s\"></td>",
               Level-1,BgColor);

   /***** Write the code of the item *****/
   if (NewItem)	// If the item is new (not stored in the file) ==> no tiene number
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%d\" bgcolor=\"#%s\"></td>",
               Level*WIDTH_NUM_SYLLABUS,BgColor);
   else
     {
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%d\" valign=\"middle\" bgcolor=\"#%s\" class=\"%s\">",
               Level*WIDTH_NUM_SYLLABUS,BgColor,FuentePrg[Level]);
      if (Level == 1)
	 fprintf (Gbl.F.Out,"&nbsp;");
      Syl_WriteNumItem (NULL,Gbl.F.Out,NumItem,Level,CodItem);
      fprintf (Gbl.F.Out,"&nbsp;</td>");
     }

   /***** Text of the item *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" colspan=\"%d\" valign=\"middle\" bgcolor=\"#%s\">",
            LstItemsSyllabus.NumLevels-Level+1,BgColor);
   Act_FormStart (NewItem ? ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActInsIteSylLec : ActInsIteSylPra) :
                            ((Gbl.CurrentCrs.Syllabus.Type == Syl_SYLLABUS_LECTURES) ? ActModIteSylLec : ActModIteSylPra));
   Syl_PutParamNumItem (NumItem);
   fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Txt\" size=\"80\" maxlength=\"%u\" value=\"%s\"" \
                      " onchange=\"javascript:document.forms[%d].submit();\"" \
                      " onfocus=\"javascript:if(this.value=='%s') this.value='';\" /></form></td>",
	    MAX_LENGTH_TEXTO_ITEM_PRG,Text,Gbl.NumForm,Txt_Enter_a_new_item_here[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****** Write parameter with the number of an item in a syllabus form ********/
/****************************************************************************/

void Syl_PutParamNumItem (int NumItem)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NumI\" value=\"%d\" />",NumItem);
  }

/****************************************************************************/
/******************** Write number of item in legal style *******************/
/****************************************************************************/

void Syl_WriteNumItem (char *StrDst,FILE *FileTgt,int NumItem,int Level,int *CodItem)
  {
   int N;
   char InStr[1+10+1];

   if (StrDst)
      StrDst[0] = '\0';

   for (N=1; N<=Level; N++)
     {
      if (N > 1)
	{
	 if (StrDst)
	    strcat (StrDst,".");
	 if (FileTgt)
	    fprintf (FileTgt,".");
	}
      sprintf (InStr,"%d",CodItem[N]);
      if (StrDst)
	 strcat (StrDst,InStr);
      if (FileTgt)
	 fprintf (FileTgt,InStr);
     }
  }

/****************************************************************************/
/********************** Remove an item from syllabus ************************/
/****************************************************************************/

void Syl_RemoveItemSyllabus (void)
  {
   char PathFile[PATH_MAX+1],PathOldFile[PATH_MAX+1],PathNewFile[PATH_MAX+1];
   FILE *NewFile;
   int NumItem;

   Gbl.CurrentCrs.Syllabus.EditionIsActive = true;

   /***** Get item number *****/
   Syl_GetParamItemNumber ();

   /***** Create a new file to make the update *****/
   Syl_BuildPathFileSyllabus (PathFile);
   Fil_CreateUpdateFile (PathFile,".old",PathOldFile,PathNewFile,&NewFile);

   /***** Create the new XML file *****/
   Syl_WriteStartFileSyllabus (NewFile);
   for (NumItem=0; NumItem<LstItemsSyllabus.NumItems; NumItem++)
      if (NumItem != Gbl.CurrentCrs.Syllabus.NumItem)
	 Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);
   Syl_WriteEndFileSyllabus (NewFile);

   /***** Close the files *****/
   Fil_CloseUpdateFile (PathFile,PathOldFile,PathNewFile,NewFile);

   /***** Show the updated syllabus to continue editing it *****/
   Syl_FreeListItemsSyllabus ();
   Syl_LoadSyllabusToMem ();
   Syl_EditSyllabus ();
  }

/****************************************************************************/
/*** Get up position of an item of the syllabus of lectures or prcticals ***/
/****************************************************************************/

void Syl_UpItemSyllabus (void)
  {
   Syl_ChangePlaceItemSyllabus (SUBIR_POSICION);
  }

/****************************************************************************/
/******* Get down position of an item of the syllabus of lectures or practicals ********/
/****************************************************************************/

void Syl_DownItemSyllabus (void)
  {
   Syl_ChangePlaceItemSyllabus (BAJAR_POSICION);
  }

/****************************************************************************/
/*************** Get up or get down a subtree of a syllabus ******************/
/****************************************************************************/

void Syl_ChangePlaceItemSyllabus (ChangePosItemSyllabus_t SubirOBajarPos)
  {
   char PathFile[PATH_MAX+1],PathOldFile[PATH_MAX+1],PathNewFile[PATH_MAX+1];
   FILE *NewFile;
   int NumItem;
   struct MoveSubtrees Subtree;

   Gbl.CurrentCrs.Syllabus.EditionIsActive = true;

   /***** Get item number *****/
   Syl_GetParamItemNumber ();

   Subtree.ToGetUp.Ini = Subtree.ToGetUp.End = 0;
   Subtree.ToGetDown.Ini = Subtree.ToGetDown.End = 0;
   Subtree.MovAllowed = false;

   if (Gbl.CurrentCrs.Syllabus.NumItem >= 0 && Gbl.CurrentCrs.Syllabus.NumItem < LstItemsSyllabus.NumItems)
     {
      /***** Create a new file where realizar the update *****/
      Syl_BuildPathFileSyllabus (PathFile);
      Fil_CreateUpdateFile (PathFile,".old",PathOldFile,PathNewFile,&NewFile);

      /***** Get up or get down position *****/
      switch (SubirOBajarPos)
	{
	 case SUBIR_POSICION:
	    Syl_CalculateUpSubtreeSyllabus (&Subtree,Gbl.CurrentCrs.Syllabus.NumItem);
	    break;
	 case BAJAR_POSICION:
	    Syl_CalculateDownSubtreeSyllabus (&Subtree,Gbl.CurrentCrs.Syllabus.NumItem);
	    break;
	}

      /***** Create the new XML file *****/
      Syl_WriteStartFileSyllabus (NewFile);
      if (Subtree.MovAllowed)
	{
	 for (NumItem=0; NumItem<Subtree.ToGetDown.Ini; NumItem++)
	    Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);
	 for (NumItem=Subtree.ToGetUp.Ini; NumItem<=Subtree.ToGetUp.End; NumItem++)
	    Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);
	 for (NumItem=Subtree.ToGetDown.Ini; NumItem<=Subtree.ToGetDown.End; NumItem++)
	    Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);
	 for (NumItem=Subtree.ToGetUp.End+1; NumItem<LstItemsSyllabus.NumItems; NumItem++)
	    Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);
	}
      else
	 Syl_WriteAllItemsFileSyllabus (NewFile);
      Syl_WriteEndFileSyllabus (NewFile);

      /***** Close the files *****/
      Fil_CloseUpdateFile (PathFile,PathOldFile,PathNewFile,NewFile);
     }

/*
fprintf (Gbl.F.Out,"Subtree.MovAllowed = %d Gbl.CurrentCrs.Syllabus.NumItem = %d Subtree.ToGetDown.Ini = %d Subtree.ToGetDown.End = %d Subtree.ToGetUp.Ini = %d Subtree.ToGetUp.End = %d",
	(int) Subtree.MovAllowed,Gbl.CurrentCrs.Syllabus.NumItem,Subtree.ToGetDown.Ini,Subtree.ToGetDown.End,Subtree.ToGetUp.Ini,Subtree.ToGetUp.End);
*/
   /***** Show the updated syllabus to continue editing it *****/
   Syl_FreeListItemsSyllabus ();
   Syl_LoadSyllabusToMem ();
   Syl_EditSyllabus ();
  }

/****************************************************************************/
/********** Compute the limits for get up a subtree of a syllabus ***********/
/****************************************************************************/
// If return Subtree->MovAllowed = false, the limits become undefined

void Syl_CalculateUpSubtreeSyllabus (struct MoveSubtrees *Subtree,int NumItem)
  {
   int Level = LstItemsSyllabus.Lst[NumItem].Level;

   if (NumItem == 0)
      Subtree->MovAllowed = false;
   else
     {
      /***** Compute limits of the subtree to get up *****/
      Subtree->ToGetUp.Ini = NumItem;
      /* Search down the end of the full subtree to get up */
      for (Subtree->ToGetUp.End = NumItem + 1; Subtree->ToGetUp.End < LstItemsSyllabus.NumItems; Subtree->ToGetUp.End++)
	 if (LstItemsSyllabus.Lst[Subtree->ToGetUp.End].Level <= Level)
	   {
	    Subtree->ToGetUp.End--;
	    break;
	   }
      if (Subtree->ToGetUp.End == LstItemsSyllabus.NumItems)
	 Subtree->ToGetUp.End = LstItemsSyllabus.NumItems - 1;

      /***** Compute limits of the subtree to get down *****/
      Subtree->ToGetDown.End = NumItem - 1;
      if (LstItemsSyllabus.Lst[Subtree->ToGetDown.End].Level < Level)
	 Subtree->MovAllowed = false;
      else
	{
	 Subtree->MovAllowed = true;
	 /* Find hacia arriba the inicio of the subtree to get down */
	 for (Subtree->ToGetDown.Ini = Subtree->ToGetDown.End; Subtree->ToGetDown.Ini >= 0; Subtree->ToGetDown.Ini--)
	    if (LstItemsSyllabus.Lst[Subtree->ToGetDown.Ini].Level <= Level)
	       break;
	 if (Subtree->ToGetDown.Ini < 0)
	    Subtree->ToGetDown.Ini = 0;
	}
     }
  }

/****************************************************************************/
/***** Compute the limits for the bajada of a subtree of a syllabus *****/
/****************************************************************************/
// If se devuelve Subtree->MovAllowed a false, the limits no quedan bien definidos

void Syl_CalculateDownSubtreeSyllabus (struct MoveSubtrees *Subtree,int NumItem)
  {
   int Level = LstItemsSyllabus.Lst[NumItem].Level;

   /***** Compute limits of the subtree to get down *****/
   Subtree->ToGetDown.Ini = NumItem;
   /* Search down the end of the full subtree to get down */
   for (Subtree->ToGetDown.End = NumItem + 1; Subtree->ToGetDown.End < LstItemsSyllabus.NumItems; Subtree->ToGetDown.End++)
      if (LstItemsSyllabus.Lst[Subtree->ToGetDown.End].Level <= Level)
	{
	 Subtree->ToGetDown.End--;
	 break;
	}
   if (Subtree->ToGetDown.End >= LstItemsSyllabus.NumItems - 1)
      Subtree->MovAllowed = false;
   else
     {
      /***** Compute limits of the subtree to get up *****/
      Subtree->ToGetUp.Ini = Subtree->ToGetDown.End + 1;
      if (LstItemsSyllabus.Lst[Subtree->ToGetUp.Ini].Level < Level)
	 Subtree->MovAllowed = false;
      else
	{
	 Subtree->MovAllowed = true;
	 /* Find hacia abajo the end of the subtree to get up */
	 for (Subtree->ToGetUp.End = Subtree->ToGetUp.Ini + 1; Subtree->ToGetUp.End < LstItemsSyllabus.NumItems; Subtree->ToGetUp.End++)
	    if (LstItemsSyllabus.Lst[Subtree->ToGetUp.End].Level <= Level)
	      {
	       Subtree->ToGetUp.End--;
	       break;
	      }
	 if (Subtree->ToGetUp.End == LstItemsSyllabus.NumItems)
	    Subtree->ToGetUp.End = LstItemsSyllabus.NumItems - 1;
	}
     }
  }

/****************************************************************************/
/** Increase the level of an item of the syllabus of lectures or practicals */
/****************************************************************************/

void Syl_RightItemSyllabus (void)
  {
   Syl_ChangeLevelItemSyllabus (INCREASE_LEVEL);
  }

/****************************************************************************/
/** Decrease the level of an item of the syllabus of lectures or practicals */
/****************************************************************************/

void Syl_LeftItemSyllabus (void)
  {
   Syl_ChangeLevelItemSyllabus (DECREASE_LEVEL);
  }

/****************************************************************************/
/********* Increase or decrease the level of an item of a syllabus ***********/
/****************************************************************************/

void Syl_ChangeLevelItemSyllabus (ChangeLevelItemSyllabus_t IncreaseOrDecreaseLevel)
  {
   char PathFile[PATH_MAX+1],PathOldFile[PATH_MAX+1],PathNewFile[PATH_MAX+1];
   FILE *NewFile;

   Gbl.CurrentCrs.Syllabus.EditionIsActive = true;

   /***** Get item number *****/
   Syl_GetParamItemNumber ();

   /***** Create a new file where realizar the update *****/
   Syl_BuildPathFileSyllabus (PathFile);
   Fil_CreateUpdateFile (PathFile,".old",PathOldFile,PathNewFile,&NewFile);

   /***** Increase or decrease level *****/
   switch (IncreaseOrDecreaseLevel)
     {
      case INCREASE_LEVEL:
	 if (LstItemsSyllabus.Lst[Gbl.CurrentCrs.Syllabus.NumItem].Level > 1)
	    LstItemsSyllabus.Lst[Gbl.CurrentCrs.Syllabus.NumItem].Level--;
	 break;
      case DECREASE_LEVEL:
	 if (LstItemsSyllabus.Lst[Gbl.CurrentCrs.Syllabus.NumItem].Level < MAX_LEVELS_SYLLABUS)
	    LstItemsSyllabus.Lst[Gbl.CurrentCrs.Syllabus.NumItem].Level++;
	 break;
     }

   /***** Create the file XML new *****/
   Syl_WriteStartFileSyllabus (NewFile);
   Syl_WriteAllItemsFileSyllabus (NewFile);
   Syl_WriteEndFileSyllabus (NewFile);

   /***** Close the files *****/
   Fil_CloseUpdateFile (PathFile,PathOldFile,PathNewFile,NewFile);

   /***** Show the updated syllabus to continue editing it *****/
   Syl_FreeListItemsSyllabus ();
   Syl_LoadSyllabusToMem ();
   Syl_EditSyllabus ();
  }

/****************************************************************************/
/************************ Insert an item in a syllabus **********************/
/****************************************************************************/

void Syl_InsertItemSyllabus (void)
  {
   char PathFile[PATH_MAX+1],PathOldFile[PATH_MAX+1],PathNewFile[PATH_MAX+1];
   FILE *NewFile;
   int NumItem;

   Gbl.CurrentCrs.Syllabus.EditionIsActive = true;

   /***** Get item number *****/
   Syl_GetParamItemNumber ();

   /***** Get item body *****/
   Str_AllocateAndGetText (Str_TO_HTML,true);

   /***** Create a new file where realizar the update *****/
   Syl_BuildPathFileSyllabus (PathFile);
   Fil_CreateUpdateFile (PathFile,".old",PathOldFile,PathNewFile,&NewFile);

   /***** Create the new XML file *****/
   Syl_WriteStartFileSyllabus (NewFile);

   /* Write items anteriores to that to be inserted */
   for (NumItem=0; NumItem<Gbl.CurrentCrs.Syllabus.NumItem; NumItem++)
      Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);

   /* Write the item that will be inserted */
   Syl_WriteItemFileSyllabus (NewFile,NumItem ? LstItemsSyllabus.Lst[NumItem-1].Level : 1,Gbl.Txt);

   /* Free memory used by the body of the message */
   Str_FreeTxt ();

   /* Write items after the one just inserted */
   for (; NumItem<LstItemsSyllabus.NumItems; NumItem++)
      Syl_WriteItemFileSyllabus (NewFile,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);

   Syl_WriteEndFileSyllabus (NewFile);

   /***** Close the files *****/
   Fil_CloseUpdateFile (PathFile,PathOldFile,PathNewFile,NewFile);

   /***** Show the updated syllabus to continue editing it *****/
   Syl_FreeListItemsSyllabus ();
   Syl_LoadSyllabusToMem ();
   Syl_EditSyllabus ();
  }

/****************************************************************************/
/*********** Receive an existing item modificado of the syllabus ************/
/****************************************************************************/

void Syl_ModifyItemSyllabus (void)
  {
   char PathFile[PATH_MAX+1],PathOldFile[PATH_MAX+1],PathNewFile[PATH_MAX+1];
   FILE *NewFile;

   Gbl.CurrentCrs.Syllabus.EditionIsActive = true;

   /***** Get item number *****/
   Syl_GetParamItemNumber ();

   /***** Get item body *****/
   Str_AllocateAndGetText (Str_TO_HTML,true);

   /***** Replace the text of the item by the received text *****/
   strncpy (LstItemsSyllabus.Lst[Gbl.CurrentCrs.Syllabus.NumItem].Text,Gbl.Txt,MAX_BYTES_TEXTO_ITEM_PRG);
   LstItemsSyllabus.Lst[Gbl.CurrentCrs.Syllabus.NumItem].Text[MAX_BYTES_TEXTO_ITEM_PRG] = '\0';

   /* Free memory used by the body of the message */
   Str_FreeTxt ();

   /***** Create a new file where realizar the update *****/
   Syl_BuildPathFileSyllabus (PathFile);
   Fil_CreateUpdateFile (PathFile,".old",PathOldFile,PathNewFile,&NewFile);

   /***** Create the new XML file *****/
   Syl_WriteStartFileSyllabus (NewFile);
   Syl_WriteAllItemsFileSyllabus (NewFile);
   Syl_WriteEndFileSyllabus (NewFile);

   /***** Close the files *****/
   Fil_CloseUpdateFile (PathFile,PathOldFile,PathNewFile,NewFile);

   /***** Show the updated syllabus to continue editing it *****/
   Syl_FreeListItemsSyllabus ();
   Syl_LoadSyllabusToMem ();
   Syl_EditSyllabus ();
  }

/****************************************************************************/
/************ Build the path of the file XML with the syllabus **************/
/****************************************************************************/

void Syl_BuildPathFileSyllabus (char *PathFile)
  {
   sprintf (PathFile,"%s/%s",Gbl.CurrentCrs.Syllabus.PathDir,Cfg_SYLLABUS_FILENAME);
  }

/****************************************************************************/
/*********************** Open file with the syllabus ***********************/
/****************************************************************************/

void Syl_OpenSyllabusFile (const char *PathDir,char *PathFile)
  {
   if (Gbl.F.Syllabus == NULL) // If it's not open in this moment...
     {
      /* If the directory doesn't exists, create it */
      Fil_CreateDirIfNotExists (PathDir);

      /* Open the file for reading */
      Syl_BuildPathFileSyllabus (PathFile);
      if ((Gbl.F.Syllabus = fopen (PathFile,"rt")) == NULL)
	{
	 /* Can't open the file */
	 if (!Fil_CheckIfPathExists (PathDir)) // Strange error, since it is just created
	    Lay_ShowErrorAndExit ("Can not open syllabus file.");
	 else
	   {
	    /* Create a new empty syllabus */
	    if ((Gbl.F.Syllabus = fopen (PathFile,"wt")) == NULL)
	       Lay_ShowErrorAndExit ("Can not create syllabus file.");
	    Syl_WriteStartFileSyllabus (Gbl.F.Syllabus);
	    Syl_WriteEndFileSyllabus (Gbl.F.Syllabus);
	    Syl_CloseFileSyllabus ();
	    /* Open of new the file for reading */
	    if ((Gbl.F.Syllabus = fopen (PathFile,"rt")) == NULL)
	       Lay_ShowErrorAndExit ("Can not open syllabus file.");
	   }
	}
     }
   else  // Go to the start of the file
      rewind (Gbl.F.Syllabus);
  }

/****************************************************************************/
/*********************** Write a file with a syllabus ***********************/
/****************************************************************************/

void Syl_WriteStartFileSyllabus (FILE *FileSyllabus)
  {
   Syl_WriteStartFileXML (FileSyllabus,"temario");
   fprintf (FileSyllabus," <lista>%s" ,NEW_LINE);
  }

void Syl_WriteAllItemsFileSyllabus (FILE *FileSyllabus)
  {
   int NumItem;

   for (NumItem=0; NumItem<LstItemsSyllabus.NumItems; NumItem++)
      Syl_WriteItemFileSyllabus (FileSyllabus,LstItemsSyllabus.Lst[NumItem].Level,LstItemsSyllabus.Lst[NumItem].Text);
  }

void Syl_WriteItemFileSyllabus (FILE *FileSyllabus,int Level,const char *Text)
  {
   fprintf (FileSyllabus,"  <item nivel=\"%d\">%s</item>%s",Level,Text,NEW_LINE);
  }

void Syl_WriteEndFileSyllabus (FILE *FileSyllabus)
  {
   fprintf (FileSyllabus," </lista>%s",NEW_LINE);
   Syl_WriteEndFileXML (FileSyllabus,"temario");
  }

/****************************************************************************/
/********************** Close the file with the syllabus ********************/
/****************************************************************************/

void Syl_CloseFileSyllabus (void)
  {
   if (Gbl.F.Syllabus)
     {
      fclose (Gbl.F.Syllabus);
      Gbl.F.Syllabus = NULL;	// To indicate that it is not open
     }
  }

/****************************************************************************/
/************ Write an XML file with author and date of creation ************/
/****************************************************************************/

void Syl_WriteStartFileXML (FILE *FileTgt,const char *Type)
  {
   fprintf (FileTgt,"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>%s",NEW_LINE);
   fprintf (FileTgt,"<%s>%s",Type,NEW_LINE);
   fprintf (FileTgt," <generator>%s</generator>%s",Log_PLATFORM_VERSION,NEW_LINE);
   fprintf (FileTgt," <copyright>%s</copyright>%s",Log_COPYRIGHT,NEW_LINE);
   fprintf (FileTgt," <author>%s,%s</author>%s",Gbl.Usrs.Me.UsrDat.DNI,Gbl.Usrs.Me.UsrDat.FullName,NEW_LINE);
   fprintf (FileTgt," <date>%ld</date>%s",time (NULL),NEW_LINE);
  }

void Syl_WriteEndFileXML (FILE *FileTgt,const char *Type)
  {
   fprintf (FileTgt,"</%s>%s",Type,NEW_LINE);
  }
