// swad_the.c: themes (colour layouts)

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <stdio.h>	// For fprintf, etc.
#include <string.h>

#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_prf.h"
#include "swad_the.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/******************************** Private constants *************************/
/****************************************************************************/

#define MAX_THEME_ID 16

const char *The_ThemeId[The_NUM_THEMES] =
  {
   "yellow",
   "blue"
  };

const char *The_ThemeNames[The_NUM_THEMES] =
  {
   "Yellow",
   "Blue"
  };

const char *The_CentralBgColors[The_NUM_THEMES] =
  {
   "FFFA85",
   "E8F3F6"
  };

const char *The_ClassHead[The_NUM_THEMES] =
  {
   "YELLOW_HEAD",
   "BLUE_HEAD"
  };

const char *The_ClassUsr[The_NUM_THEMES] =
  {
   "YELLOW_USR",
   "BLUE_USR"
  };

const char *The_ClassDegree[The_NUM_THEMES] =
  {
   "YELLOW_DEGREE",
   "BLUE_DEGREE"
  };

const char *The_ClassCourse[The_NUM_THEMES] =
  {
   "YELLOW_COURSE",
   "BLUE_COURSE"
  };

const char *The_ClassMenuCourses[The_NUM_THEMES] =
  {
   "YELLOW_MENU_COURSES",
   "BLUE_MENU_COURSES"
  };

const char *The_ClassConnected[The_NUM_THEMES] =
  {
   "YELLOW_CONNECTED",
   "BLUE_CONNECTED"
  };

const char *The_ClassMenuOff[The_NUM_THEMES] =
  {
   "YELLOW_MENU_OFF",
   "BLUE_MENU_OFF"
  };

const char *The_ClassMenuOn[The_NUM_THEMES] =
  {
   "YELLOW_MENU_ON",
   "BLUE_MENU_ON"
  };

const char *The_ClassTabOff[The_NUM_THEMES] =
  {
   "YELLOW_TAB_OFF",
   "BLUE_TAB_OFF"
  };

const char *The_ClassTabOn[The_NUM_THEMES] =
  {
   "YELLOW_TAB_ON",
   "BLUE_TAB_ON"
  };

const char *The_ClassAction[The_NUM_THEMES] =
  {
   "YELLOW_ACTION",
   "BLUE_ACTION"
  };

const char *The_ClassTitle[The_NUM_THEMES] =
  {
   "YELLOW_TITLE",
   "BLUE_TITLE"
  };

const char *The_ClassFormul[The_NUM_THEMES] =
  {
   "YELLOW_FORMUL",
   "BLUE_FORMUL"
  };

const char *The_ClassFormulNB[The_NUM_THEMES] =
  {
   "YELLOW_FORMUL_NB",
   "BLUE_FORMUL_NB"
  };

const char *The_ClassFormulB[The_NUM_THEMES] =
  {
   "YELLOW_FORMULB",
   "BLUE_FORMULB"
  };


/****************************************************************************/
/************************ Put icons to select a theme ***********************/
/****************************************************************************/

void The_PutIconsToSelectTheme (void)
  {
   tTheme Theme;

   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"1\"><tr>");
   for (Theme=(tTheme) 0; Theme<The_NUM_THEMES; Theme++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" class=\"%s\">",
               (Theme == Gbl.Prefs.Theme) ? "LAYOUT_ON" : "LAYOUT_OFF");
      Act_FormStart (ActChgThe);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Theme\" value=\"%s\" />",The_ThemeId[Theme]);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,The_ThemeNames[Theme],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s/theme_32x20.gif\" width=\"32\" height=\"20\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_THEMES,The_ThemeId[Theme],The_ThemeNames[Theme]);
     }
   fprintf (Gbl.F.Out,"</tr></table>\n");
  }

/****************************************************************************/
/******************************** Change theme ******************************/
/****************************************************************************/

void The_ChangeTheme (void)
  {
   char Query[512];

   /***** Get param theme *****/
   Gbl.Prefs.Theme = The_GetParamTheme ();
   sprintf (Gbl.Prefs.PathTheme,"%s/%s/%s",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_THEMES,The_ThemeId[Gbl.Prefs.Theme]);

   /***** Store theme in database *****/
   if (Gbl.Usrs.Me.Logged)
     {
      sprintf (Query,"UPDATE usr_data SET Theme='%s' WHERE UsrCod='%ld'",
               The_ThemeId[Gbl.Prefs.Theme],Gbl.Usrs.Me.UsrDat.UsrCod);
      DB_QueryUPDATE (Query,"can not update your preference about theme");
     }

   /***** Set preferences from current IP *****/
   Prf_SetPrefsFromIP ();
  }

/****************************************************************************/
/**************************** Get parameter theme ***************************/
/****************************************************************************/

tTheme The_GetParamTheme (void)
  {
   char ThemeId[MAX_THEME_ID+1];
   tTheme Theme;

   Par_GetParam (Par_PARAM_SINGLE,"Theme",ThemeId,MAX_THEME_ID);
   for (Theme=(tTheme) 0; Theme<The_NUM_THEMES; Theme++)
      if (!strcmp (ThemeId,The_ThemeId[Theme]))
         return Theme;

   return THEME_UNKNOWN;
  }

/****************************************************************************/
/*************************** Get theme from string **************************/
/****************************************************************************/

tTheme The_GetThemeFromStr (const char *Str)
  {
   tTheme Theme;

   for (Theme=(tTheme) 0; Theme<The_NUM_THEMES; Theme++)
      if (!strcasecmp (Str,The_ThemeId[Theme]))
	 return Theme;

   return THEME_UNKNOWN;
  }
