// swad_tst.h: self-assessment tests

#ifndef _SWAD_TST
#define _SWAD_TST
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

#define Tst_MAX_QUESTIONS_PER_EXAM		100	// Absolute maximum number of questions in a test exam
#define Tst_MAX_TAGS_PER_QUESTION		  5
#define Tst_MAX_TAG_LENGTH			255
#define Tst_MAX_BYTES_TAG			(Tst_MAX_TAG_LENGTH*Str_MAX_CHARACTER)
#define Tst_MAX_ANSWERS_PER_QUESTION		 10
#define Tst_MAX_LENGTH_ANS_USR			((10+1)*Tst_MAX_ANSWERS_PER_QUESTION-1)
#define Tst_NUM_ANS_TYPES			  6
#define Tst_MAX_BYTES_LIST_ANSWER_TYPES		(10+(Tst_NUM_ANS_TYPES-1)*(1+10))
#define Tst_CONFIG_DEFAULT_MIN_QUESTIONS	  1
#define Tst_CONFIG_DEFAULT_DEF_QUESTIONS	 20	// Number of questions to be generated by default in a self-assessment test
#define Tst_CONFIG_DEFAULT_MAX_QUESTIONS	 30	// Maximum number of questions to be generated in a self-assessment test
#define Tst_NUM_TYPES_FEEDBACK			  4
#define Tst_NUM_CRIT_ORDEN_TST			  3

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

typedef enum
  {
   Tst_FEEDBACK_NOTHING        = 0,
   Tst_FEEDBACK_TOTAL_RESULT   = 1,
   Tst_FEEDBACK_EACH_RESULT    = 2,
   Tst_FEEDBACK_EACH_GOOD_BAD  = 3
  } Tst_Feedback_t;
#define Tst_FEEDBACK_DEFAULT Tst_FEEDBACK_EACH_GOOD_BAD

struct ConfigTst
  {
   unsigned Min,Def,Max;
   unsigned MinTimeNxtTstPerQst;
   Tst_Feedback_t FeedbackType;
  };

typedef enum {Tst_ANS_TRUE_FALSE      = 0,
              Tst_ANS_UNIQUE_CHOICE   = 1,
              Tst_ANS_MULTIPLE_CHOICE = 2,
              Tst_ANS_TEXT            = 3,
              Tst_ANS_INT             = 4,
              Tst_ANS_FLOAT           = 5} Tst_AnswerType_t;

typedef enum {Tst_ORDER_HEADING       = 0,
              Tst_ORDER_NUM_ACCESS    = 1,
              Tst_ORDER_AVERAGE_GRADE = 2} Tst_QuestionsOrder_t;

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Tst_ShowFormAskTst (void);
void Tst_ShowTestExam (void);
void Tst_AssessTestExam (void);
void Tst_ShowFormAskEditTsts (void);
void Tst_ListQuestionsToEdit (void);
void Tst_ShowFormConfig (void);
void Tst_EnableTag (void);
void Tst_DisableTag (void);
void Tst_RenameTag (void);
void Tst_ReceiveConfigTst (void);
void Tst_ShowFormEditOneQst (void);
void Tst_ReceiveQst (void);
void Tst_RemoveQst (void);
void Tst_ChangeShuffleQst (void);
void Tst_FreeTagsList (void);

#endif
