// swad_usr.h: users

#ifndef _SWAD_USR
#define _SWAD_USR
/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <mysql/mysql.h>	// To access MySQL databases

#include "swad_act.h"
#include "swad_cns.h"
#include "swad_cry.h"
#include "swad_dat.h"
#include "swad_deg.h"
#include "swad_the.h"

/****************************************************************************/
/***************************** Public constants *****************************/
/****************************************************************************/

#define Usr_MAX_CLICKS_WITHOUT_PHOTO	 50
#define Usr_PHOTO_REAL_WIDTH		150
#define Usr_PHOTO_REAL_HEIGHT		200

#define Usr_NUM_MAIN_FIELDS_DATA_USR 11
#define Usr_NUM_MAIN_FIELDS_DATA_ADM 10
#define Usr_NUM_ALL_FIELDS_DATA_STD 15
#define Usr_NUM_ALL_FIELDS_DATA_TCH 13

#define Usr_MIN_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS  6
#define Usr_DEF_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS 12
#define Usr_MAX_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS 36

#define Usr_MIN_DIGITS_USR_ID			 3
#define Usr_MIN_LENGTH_USR_ID			 4
#define Usr_MAX_LENGTH_USR_ID			16
#define Usr_MAX_CONSECUTIVE_LETTERS_USR_ID	 5

#define Usr_MIN_LENGTH_NICKNAME	  3
#define Usr_MAX_LENGTH_NICKNAME	 16

#define Usr_MIN_LENGTH_PLAIN_PASSWORD	 10
#define Usr_INPUT_LENGTH_PLAIN_PASSWORD	 32
#define Usr_MAX_LENGTH_PLAIN_PASSWORD	256

#define Usr_MAX_LENGTH_USR_NAME_OR_SURNAME	 32
#define Usr_MAX_BYTES_NAME			255

#define Usr_MAX_LENGTH_PHONE	16
#define Usr_MAX_BYTES_PHONE	16

#define Usr_CLASS_PHOTO_COLS_DEF	10	// Default number of columns in a class photo
#define Usr_CLASS_PHOTO_COLS_MAX	50	// Maximum number of columns in a class photo

#define Usr_LIST_WITH_PHOTOS_DEF	false

#define Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS	(Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64*10000UL)

#define Usr_MAX_BYTES_LIST_NICKS		((Usr_MAX_LENGTH_NICKNAME+2)*10000UL)

/****************************************************************************/
/******************************* Public types *******************************/
/****************************************************************************/

// Related with user's types
#define Usr_NUM_USR_TYPES 6
typedef enum
  {
   Usr_UNKNOWN   = 0,
   Usr_INVITED   = 1,
   Usr_STUDENT   = 2,
   Usr_TEACHER   = 3,
   Usr_DEG_ADMIN = 4,
   Usr_SUPERUSER = 5,
  } Usr_UsrType_t;	// Only Usr_STUDENT and Usr_TEACHER are allowed as user types, but a user may be logged as other types
// Don't change these numbers! They are used for users' permissions and for user's types in database

// Related with user's genders
#define Usr_NUM_GENDERS 4	// Unknown, female, male, all
typedef enum
  {
   Usr_GENDER_UNKNOWN = 0,
   Usr_GENDER_FEMALE  = 1,
   Usr_GENDER_MALE    = 2,
   Usr_GENDER_ALL     = 3,
  } Usr_Gender_t;	// Usr_GENDER_ALL is intended for all the genders
// Don't change these numbers! They are used for user's genders in database

// Related with class photograph
typedef enum
  {
   Usr_CLASS_PHOTO_SEE,
   Usr_CLASS_PHOTO_PRN,
   Usr_CLASS_PHOTO_SEL,
  } Usr_ClassPhotoType_t;

// Related with type of list of users
#define NUM_USR_LIST_TYPES 2
typedef enum
  {
   Usr_CLASS_PHOTO,
   Usr_LIST,
  } Usr_ShowUsrsType_t;
#define Usr_SHOW_USRS_TYPE_DEFAULT Usr_CLASS_PHOTO

// Related with range of courses-degrees when getting lists of users from database
#define Usr_NUM_LIST_USRS_RANGES	7
typedef enum
  {
   Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL,
   Usr_LIST_USRS_RANGE_ANY_DEGREE_SEARCH,
   Usr_LIST_USRS_RANGE_NO_COURSE_ALL,
   Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL,
   Usr_LIST_USRS_RANGE_THIS_DEGREE_SEARCH,
   Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,
   Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE,
  } Usr_ListUsrsRange_t;

typedef enum {
   Usr_USER_LIST_UNKNOWN=0,
   Usr_USER_LIST_DATA_COURSE_STUDENTS=1,
   Usr_USER_LIST_DATA_TEACHERS=2,
   Usr_USER_LIST_DATA_ADMINISTRATORS=3,
   Usr_USER_LIST_RECORDS_COURSE_STUDENTS=4,
   Usr_USER_LIST_RECORDS_COURSE_TEACHERS=5,
  } Usr_ListUsrsType_t;

typedef enum
  {
   Usr_REPLACE_DATA,
   Usr_INSERT_NEW_DATA,
   } Usr_ReplaceInsert_t;

// Related with users adding or removing
typedef enum
  {
   Usr_REQUEST_REMOVE_USR,
   Usr_REMOVE_USR,
  } Usr_ReqDelOrDelUsr_t;

// Related with user's data
struct UsrData
  {
   long UsrCod;
   char EncryptedUsrCod [Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1];
   char DNI		[Usr_MAX_LENGTH_USR_ID+1];
   char Password        [Cry_LENGTH_ENCRYPTED_STR_SHA512_BASE64+1];
   Usr_UsrType_t UsrTypeInDB;
   char Surname1	[Usr_MAX_BYTES_NAME+1];
   char Surname2	[Usr_MAX_BYTES_NAME+1];
   char FirstName	[Usr_MAX_BYTES_NAME+1];
   char FullName	[(Usr_MAX_BYTES_NAME+1)*3];
   Usr_Gender_t Gender;
   char E_mail		[Cte_MAX_BYTES_STRING+1];
   char WWW		[Cte_MAX_BYTES_URL   +1];
   char Skype		[Cte_MAX_BYTES_STRING+1];
   char Photo		[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1];	// Name of public link to photo
   bool PublicPhoto;	// User want his photo to be public?
   long InsCod;
   bool Accepted;		// User has accepted joining to current course?
   unsigned NumNewMsgs;
   Act_Tab_t LastTab;
   struct
     {
      char LocalAddress	[Cte_MAX_BYTES_STRING+1];
      char LocalPhone	[Usr_MAX_BYTES_PHONE +1];
      char FamilyAddress[Cte_MAX_BYTES_STRING+1];
      char FamilyPhone	[Usr_MAX_BYTES_PHONE +1];
      char OriginPlace	[Cte_MAX_BYTES_STRING+1];
      char StrBirthday	[Cte_MAX_LENGTH_DATE +1];
      struct Date Birthday;
      char *Comments;
     } Std;
   struct
     {
      long CtrCod;
      long DptCod;
      char Office	[Cte_MAX_BYTES_STRING+1];
      char OfficePhone	[Usr_MAX_BYTES_PHONE +1];
     } Tch;
   struct
     {
      Txt_Language_t Language;
      tTheme Theme;
      unsigned SideCols;
      bool SecureIcons;
      unsigned NotifyEvents;	// One bit activated for each type of event
     } Prefs;
  };

struct ListUsers
  {
   unsigned *Ind;	// Index used to order the elements of the list
   bool *Remove;	// A boolean associated with each user that indicates if it must be removed
   struct UsrData *Lst;	// List of users with all their data
   unsigned NumUsrs;	// Number of users in the list
  };

/****************************************************************************/
/***************************** Public prototypes ****************************/
/****************************************************************************/

void Usr_InformAboutNumClicksBeforePhoto (void);

void Usr_UsrDataConstructor (struct UsrData *UsrDat);
void Usr_ResetUsrDataExceptUsrCodAndDNI (struct UsrData *UsrDat);
void Usr_UsrDataDestructor (struct UsrData *UsrDat);
void Usr_GetAllUsrDataFromUsrCod (struct UsrData *UsrDat);
void Usr_GetAllUsrDataFromDNI (struct UsrData *UsrDat);
void Usr_GetDNIFromUsrCod (struct UsrData *UsrDat);
void Usr_GetUsrCodFromDNI (struct UsrData *UsrDat);
void Usr_GetUsrCodFromEncryptedUsrCod (struct UsrData *UsrDat);
Usr_UsrType_t Usr_ConvertUnsignedStrToUsrType (const char *UnsignedStr);
void Usr_GetUsrDataFromUsrCod (struct UsrData *UsrDat);
void Usr_BuildFullName (struct UsrData *UsrDat);
void Usr_GetStudentCommentsFromString (char *Str,struct UsrData *UsrDat);

void Usr_RestrictLengthAndWriteName (struct UsrData *UsrDat,unsigned MaxChars);

bool Usr_ShowUsrPhotoIsAllowed (struct UsrData *UsrDat,char *PhotoURL);
bool Usr_BuildLinkToPhoto (const struct UsrData *UsrDat,char *PhotoURL);
bool Usr_CheckIfPrivPhotoExists (const struct UsrData *UsrDat,char *PathPrivRelPhoto);
bool Usr_RemovePhoto (struct UsrData *UsrDat);
void Usr_ShowUsrPhoto (const struct UsrData *UsrDat,const char *PhotoURL,int Width,int Height,bool Zoom);
bool Usr_CheckIfUsrIsAdmOfDeg (long UsrCod,long DegCod);
bool Usr_CheckIfUsrIsAdmOfAllDegs (long UsrCod);
bool Usr_CheckIfUsrIsSuperuser (long UsrCod);
void Usr_GetMyInstitutions (void);
void Usr_GetMyDegrees (void);
void Usr_GetMyCourses (void);
bool Usr_CheckIfUsrBelongsToDeg (long UsrCod,long DegCod);
bool Usr_CheckIfUsrBelongsToCrs (long UsrCod,long CrsCod);
bool Usr_CheckIfIBelongToIns (long InsCod);
bool Usr_CheckIfIBelongToDeg (long DegCod);
bool Usr_CheckIfIBelongToCrs (long CrsCod);
unsigned long Usr_GetInssFromUsr (long UsrCod,MYSQL_RES **mysql_res);
unsigned long Usr_GetDegsFromUsr (long UsrCod,long InsCod,MYSQL_RES **mysql_res);
unsigned long Usr_GetCrssFromUsr (long UsrCod,long DegCod,MYSQL_RES **mysql_res);

void Usr_AskRemoveOldUsrs (void);
void Usr_RemoveOldUsrs (void);

void Usr_ReqAcceptRegisterInCrs (void);
void Usr_AcceptRegisterMeInCrs (void);

void Usr_WriteFormLogin (void);
void Usr_WriteLoggedUsr (void);
void Usr_GetParamUsrIdLogin (void);
void Usr_GetParamUsrPwdLogin (void);
bool Usr_GetParamOtherDNIPlainAndGetUserData (void);
void Usr_PutParamOtherUsrDNIPlain (void);
void Usr_GetParamOtherUsrDNIPlain (void);
void Usr_PutParamOtherUsrCodEncrypted (const char EncryptedUsrCod[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1]);
void Usr_GetParamOtherUsrCodEncrypted (void);

bool Usr_CheckUsrAndGetUsrData (void);
void Usr_ChgUsrTypeLog1 (void);
void Usr_ChgUsrTypeLog2 (void);

unsigned Usr_UpdateMyClicksWithoutPhoto (void);
void Usr_RemoveUsrFromTableClicksWithoutPhoto (long UsrCod);

void Usr_AsignUsrListType (void);
void Usr_WriteFormToReqAnotherUserDNI (Act_Action_t NextAction);

void Usr_ShowConnectedUsrsAndRightMenu (void);

bool Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (char *DNI);
bool Usr_CheckIfDNIWithoutLeadingZerosIsValid (char *DNI);
bool Usr_CheckIfDNIWithoutEndAlphaIsValid (const char *DNI,unsigned Length);
bool Usr_CheckIfNickIsValid (const char *Nickname);

void Usr_ActChgMyPwd1 (void);
void Usr_ActChgMyPwd2 (void);
void Usr_ShowFormSendNewPwd (void);
void Usr_SendNewPwdByEMail (void);
void Usr_UpdateOtherPwd1 (void);
void Usr_UpdateOtherPwd2 (void);
bool Usr_FastCheckIfPasswordSeemsGood (const char *PlainPassword);

bool Usr_GetNicknameFromUsrCod (long UsrCod,char *Nickname);
bool Usr_GetUsrCodFromNickname (const char *Nickname,struct UsrData *UsrDat);
void Usr_ShowFormChangeNick (void);
void Usr_UpdateNick (void);
void Usr_ShowFormChgPwd (void);
void Usr_ReqDNIOthUsrPwd (void);
void Usr_ShowFormOthPwd1 (void);
void Usr_ShowFormOthPwd2 (void);
void Usr_ReqMyPhoto (void);
void Usr_ReqUsrPhoto (struct UsrData *UsrDat);
void Usr_ReqPhoto (const struct UsrData *UsrDat,bool PhotoExists,const char *PhotoURL);
void Usr_RecMyPhotoDetFaces (void);
void Usr_RecUsrPhotoDetFaces (void);
void Usr_RemoveMyPhoto1 (void);
void Usr_RemoveMyPhoto2 (void);
void Usr_RemoveUsrPhoto (void);
void Usr_ReceivePhotoAndDetectFaces (bool ItsMe,const struct UsrData *UsrDat);
void Usr_UpdateMyPhoto1 (void);
void Usr_UpdateMyPhoto2 (void);
void Usr_UpdateUsrPhoto1 (void);
void Usr_UpdateUsrPhoto2 (void);

void Usr_GetUsrCommonDataFromForm (struct UsrData *UsrDat,bool GetUsrTypeFromForm,bool GetGender,bool GetPublicPhoto);
void Usr_GetStdDataFromForm (struct UsrData *UsrDat);
void Usr_GetTchDataFromForm (struct UsrData *UsrDat);
bool Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (struct UsrData *UsrDat);
bool Usr_ChkDNIAndGetAllUsrDataFromDNI (struct UsrData *UsrDat);
bool Usr_GetIfUserHasAccepted (long UsrCod);
void Usr_GetUsrObservationsFromForm (struct UsrData *UsrDat);
void Usr_UpdateMyLastTabAndTime (void);

void Usr_UpdateMyCard (void);
void Usr_CreatAndShowUsrCardAndRegisterInCrs (void);
void Usr_ModifAndShowUsrCardAndRegisterInCrs (void);
void Usr_GetDNIShowUsrCardAndRegisterInCrs (void);
void Usr_UpdateMyNick (void);
void Usr_UpdateCommonCard (struct UsrData *UsrDat,Usr_ReplaceInsert_t ReplaceOrInsert);

void Usr_CreatUsrIfNotExistsAndRegisterHimInCurrentCrs (struct UsrData *UsrDat,Usr_ReplaceInsert_t ReplaceOrInsert,Cte_QuietOrVerbose_t QuietOrVerbose);
void Usr_CreateUsrIfNotExists (struct UsrData *UsrDat,Usr_ReplaceInsert_t ReplaceOrInsert,Cte_QuietOrVerbose_t QuietOrVerbose);
void Usr_RegisterUsrInCurrentCrs (long UsrCod,Cte_QuietOrVerbose_t QuietOrVerbose);
void Usr_RegisterAdminInCurrentDeg (long UsrCod);
void Usr_EffectivelyRemUsrFromCrs (struct UsrData *UsrDat,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose);
void Usr_EffectivelyRemAdmFromDeg (long UsrCod);
void Usr_DeleteUsrFromPlatform (struct UsrData *UsrDat,Cte_QuietOrVerbose_t QuietOrVerbose);

void Usr_RemoveUsrBriefcase (struct UsrData *UsrDat);
void Usr_RemoveUsrCard (struct UsrData *UsrDat);

void Usr_WriteRowStdMainData (unsigned NumUsr,struct UsrData *UsrDat,bool PutCheckboxToSelectUsr);
void Usr_WriteRowStdAllData (unsigned NumUsr,struct UsrData *UsrDat,char *GrpNames);
void Usr_WriteRowTchMainData (unsigned NumUsr,struct UsrData *UsrDat,bool PutCheckboxToSelectUsr);
void Usr_WriteRowTchAllData (unsigned NumUsr,struct UsrData *UsrDat);
bool Usr_ICanSeeDNIOfATeacher (struct UsrData *UsrDat);
void Usr_RestrictLengthMainData (bool ShowData,struct UsrData *UsrDat,char *MailLink,char *WWWLink,char *SkypeLink);
void Usr_WriteMainDataUsrExceptDNI (struct UsrData *UsrDat,const char *BgColor,bool ShowData,
                                    const char *MailLink,const char *WWWLink,const char *SkypeLink,
                                    const char *InstitutionName,const char *InstitutionLink);
void Usr_WriteUsrData (const char *Color,const char *Dato,const char *Enlace,bool NonBreak,bool Accepted);

unsigned Usr_GetNumStdsInCrs (long CrsCod);
unsigned Usr_GetNumTchsInCrs (long CrsCod);
unsigned Usr_CountNumStdsInDeg (long DegCod);
void Usr_GetAndOrderStdsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod);
void Usr_GetAndOrderTchsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod);
void Usr_GetAndOrderAdmsLst (Usr_ListUsrsRange_t ListUsrsRange);
void Usr_GetDNIRandomStdInCrs (char *DNI);
void Usr_GetDNIRandomStdInGrp (long GrpCod,char *DNI);

unsigned Usr_GetNumberOfTeachersInDepartment (long DptCod);
unsigned Usr_GetNumberOfUsersInInstitution (long InsCod,Usr_UsrType_t UsrType);
unsigned Usr_GetNumberOfTeachersInCentre (long CtrCod);

void Usr_GetStdsLstCrs (void);
void Usr_GetStdsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod);
void Usr_GetTchsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod,const char *TchQuery);
void Usr_GetAdmsLst (Usr_ListUsrsRange_t ListUsrsRange);
void Usr_GetStdsLstDeg (long DegCod);
void Usr_OrderUsrsList (struct ListUsers *LstUsrs);
bool Usr_NameIsLess (struct UsrData *UsrData1,struct UsrData *UsrData2);
void Usr_FreeUsrsList (struct ListUsers *LstUsrs);
void Usr_HiddenParUsrCodAll (Act_Action_t NextAction,const char *ListUsrCods);
void Usr_GetListSelectedUsrs (void);
bool Usr_GetListDNIOrNicks (bool WriteErrorMsgs);
bool Usr_FindEncryptedUsrCodInList (const char *EncryptedUsrCodToFind);
int  Usr_CountNumUsrsInEncryptedList (void);
void Usr_AllocateListEncryptedUsrCodAll (void);
void Usr_AllocateListEncryptedUsrCodStd (void);
void Usr_AllocateListEncryptedUsrCodTch (void);
void Usr_AllocateListNicks (void);
void Usr_FreeListsEncryptedUsrCods (void);
void Usr_FreeListNicks (void);

void Usr_ShowFormsToSelectUsrListType (Act_Action_t NextAction);
void Usr_PutCheckboxToSelectAllTheUsers (Usr_UsrType_t UsrType);
Usr_Gender_t Usr_GetGenderOfUsrsLst (struct ListUsers *LstUsrs);
unsigned Usr_GetColumnsForSelectUsrs (void);
void Usr_PutExtraParamsUsrList (Act_Action_t NextAction);
void Usr_ListUsersToSelect (Usr_UsrType_t UsrType);
void Usr_PutCheckboxToSelectUser (Usr_UsrType_t UsrType,const char *EncryptedUsrCod,bool UsrIsTheMsgSender);

void Usr_PutCheckboxListWithPhotos (void);
void Usr_ListMainDataStds (void);
void Usr_ListAllDataStds (void);
void Usr_ListUsrsForSelection (Usr_UsrType_t UsrType);
void Usr_ListMainDataTchs (void);
void Usr_ListAllDataTchs (void);
void Usr_ListTchsFound (Usr_ListUsrsRange_t ListUsrsRange,const char *TchQuery);
void Usr_ListDataAdms (void);
void Usr_ReqListCardsStdsCrs (void);
void Usr_StartFormLinkToStdCard (const char *EncryptedUsrCod);
void Usr_ListCardsStdsCrs (void);
void Usr_ListCardsTchsCrs (void);
void Usr_GetParamCardsPerPage (void);

void Usr_GetAndUpdatePrefsAboutUsrList (void);
void Usr_PutParamUsrListType (Usr_ShowUsrsType_t ListType);
void Usr_GetAndUpdateColsClassPhoto (void);
void Usr_PutParamColsClassPhoto (void);
void Usr_PutParamListWithPhotos (void);

void Usr_SeeStudents (void);
void Usr_SeeTeachers (void);
void Usr_SeeStdClassPhotoPrn (void);
void Usr_SeeTchClassPhotoPrn (void);
void Usr_DrawClassPhoto (Usr_ClassPhotoType_t ClassPhotoType,Usr_UsrType_t UsrTypeInClassPhoto);
void Usr_PutSelectorListUsrsRange (bool SendOnChange,Usr_UsrType_t UsrTypeToList);
void Usr_GetListUsrsRange (Usr_UsrType_t UsrTypeToList,Usr_ListUsrsRange_t DefaultLocationRange);
void Usr_PutSelectorNumColsClassPhoto (void);

void Usr_RegisterRemoveUsrsCrs (void);
void Usr_ReceiveFormUsrsCrs (void);
void Usr_AskRemAllStdsThisCrs (void);
void Usr_RemAllStdsThisCrs (void);
void Usr_ReqDNINewUsr (void);
void Usr_ReqDNINewAdm (void);
void Usr_ReqDNIPhotoUsr (void);
void Usr_AskIfAddUsrToCrs (void);
void Usr_AddAdmToDeg (void);
void Usr_SendPhotoUsr1 (void);
void Usr_SendPhotoUsr2 (void);
void Usr_ReqDNIRemUsrFromCrs (void);
void Usr_ReqDNIRemAdm (void);
void Usr_ReqRemUsrFromCrs (void);
void Usr_RemUsrFromCrs (void);
void Usr_ReqRemOrRemUsrFromCrs (Usr_ReqDelOrDelUsr_t ReqDelOrDelUsr);
void Usr_ReqDNIRemGbl (void);
void Usr_ReqRemUsrGbl (void);
void Usr_ReqRemAdm (void);
void Usr_RemAdm (void);
void Usr_ReqRemOrRemAdm (Usr_ReqDelOrDelUsr_t ReqDelOrDelUsr);
void Usr_RemUsrGbl (void);
void Usr_ReqDelOrDelUsrGbl (Usr_ReqDelOrDelUsr_t ReqDelOrDelUsr);
void Usr_AskIfRemUsrFromCrsOrPlatform (bool ItsMe);
void Usr_AskIfAddAdmToDeg (void);
void Usr_AskIfRemAdmFromDeg (bool ItsMe);
void Usr_ConstructPathUsr (char *DNI,char *PathRelUsr,char *PathAbsUsr);
bool Usr_ChkIfUsrCodExists (long UsrCod);
bool Usr_ChkIfDNIExists (const char *DNI);

void Usr_ShowWarningNoUsersFound (Usr_UsrType_t UsrType);

#endif
