// swad_act.c: actions

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <stdio.h>		// For fprintf

#include "swad_cfg.h"
#include "swad_cht.h"
#include "swad_gbl.h"
#include "swad_exa.h"
#include "swad_not.h"
#include "swad_ntf.h"
#include "swad_prf.h"
#include "swad_set.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/************************ Internal global variables *************************/
/****************************************************************************/
/*
631 CGI's in one:
			  0. ActAll			Any action (used for statistics)
			  1. ActUnk			Unknown action
			  2. ActAutUsrInt		Authentify user internally (directly from the platform)
			  3. ActAutUsrExt		Authentify user externally (remotely from an external site)
                          4. ActChgUsrTypLog		Change type of logged user
			  5. ActMnu			Show menu
			  6. ActDirAcc			Get XHTML code to access directly to a course
			  7. ActRef			Refresh number of new received messages and connected users via AJAX
			  8. ActWebSvc			Call plugin function
			  9. ActLogOut			Close session
                Degree:
			 10. ActReqSch			Request search of courses or teachers
			 11. ActSeeIns			List institutions
			 12. ActSeeCtr			List centres
			 13. ActSeeDpt			List departments
			 14. ActSeePlc			List places
			 15. ActSeeHld			List holidays

			 16. ActLstAdm			List main data of teachers

			 17. ActReqNewAdm		Request the register of a degree administrator in current degree
			 18. ActReqRemAdm		Request the removal of a degree administrator from current degree

			 19. ActReqEdiDegTyp		Request edition of types of degrees
			 20. ActReqEdiDeg		Request edition of degrees of a type
			 21. ActReqEdiCrs		Request edition of courses of a degree

			 22. ActSeeLnk			See institutional links

			 22b. ActSetUp			Initial set up

			 23. ActSchCrs			Find courses
			 24. ActSchCrsTch		Find courses or teachers

			 25. ActEdiIns			Edit institutions
			 26. ActNewIns			Request the creation of an institution
			 27. ActRemIns			Remove institution
			 28. ActRenInsSho		Change short name institution
			 29. ActRenInsFul		Change full name institution
			 30. ActChgInsLog		Change logo of institution
			 31. ActChgInsWWW		Change web of institution

			 32. ActEdiCtr			Edit centres
			 33. ActNewCtr			Request the creation of a centre
			 34. ActRemCtr			Remove centre
			 35. ActChgCtrIns		Request change of the institution of a centre
			 36. ActRenCtrSho		Change short name centre
			 37. ActRenCtrFul		Change full name centre
			 38. ActChgCtrWWW		Change web of centre

			 39. ActEdiDpt			Edit departments
			 40. ActNewDpt			Request the creation of a department
			 41. ActRemDpt			Remove department
			 42. ActChgDptIns		Request change of the institution of a department
			 43. ActRenDptSho		Change short name department
			 44. ActRenDptFul		Change full name department
			 45. ActChgDptWWW		Change web of department

			 46. ActEdiPlc			Edit places
			 47. ActNewPlc			Request the creation of a place
			 48. ActRemPlc			Remove a place
			 49. ActRenPlcSho		Change short name of a place
			 50. ActRenPlcFul		Change full name of a place

			 51. ActEdiHld			Edit holidays
			 52. ActNewHld			Request the creation of a holiday
			 53. ActRemHld			Remove a holiday
			 54. ActChgHldPlc		Change place of a holiday
			 55. ActChgHldTyp		Change type of a holiday
			 56. ActChgHldStrDat		Change start date of a holiday
			 57. ActChgHldEndDat		Change end date of a holiday
			 58. ActRenHld			Change name of a holiday

			 59. ActNewAdm			Rewgister an administrador in this degree
			 60. ActAskRemAdm		Confirm baja of a administrador of this degree
			 61. ActRemAdm			Remove an administrador of this degree

			 62. ActNewDegTyp		Request the creation of a type of degree
			 63. ActRemDegTyp		Request the removal of a type of degree
			 64. ActRenDegTyp		Request renaming of a type of degree

			 65. ActNewDeg			Request the creation of a degree
			 66. ActRemDeg			Request the removal of a degree
			 67. ActRenDegSho		Request change of the short name of a degree
			 68. ActRenDegFul		Request change of the full name of a degree
			 69. ActChgDegTyp		Request change of the type of a degree
			 70. ActChgDegIns		Request change of the institution of a degree
			 71. ActChgDegPlc		Request change of the place of a degree
			 72. ActChgDegFstYea		Request change of the first year of a degree
			 73. ActChgDegLstYea		Request change of the last year of a degree
			 74. ActChgDegOptYea		Request change of the year of optativas of a degree
			 75. ActChgDegLog		Request change of the logo of a degree
			 76. ActChgDegWWW		Request change of the web of a degree

			 77. ActNewCrs			Request the creation of a course
			 78. ActRemCrs			Request the removal of a course
			 79. ActChgCrsDeg		Request change of degree of a course
			 80. ActChgCrsYea		Request change of curso of a course dentro of su degree
			 81. ActChgCrsSem		Request change of cuatrimestre of a course
			 82. ActRenCrsSho		Request change of short name of a course
			 83. ActRenCrsFul		Request change of full name of a course

			 84. ActEdiLnk			Edit institutional links
			 85. ActNewLnk			Request the creation of an institutional link
			 86. ActRemLnk			Request the removal of an institutional link
			 87. ActRenLnkSho		Request the change of the short name of an institutional link
			 88. ActRenLnkFul		Request the change of the full name of an institutional link
			 89. ActChgLnkWWW		Request the change of the web of an institutional link 

		Course:
			 90. ActSeeTchGui		Show teaching guide of the course
			 91. ActSeeCal			Show the calendario acadmico of the course
			 92. ActSeeCrsTimTbl		Show the timetable
			 93. ActSeeSylLec		Show the syllabus of teora
			 94. ActSeeSylPra		Show the syllabus of prcticas
			 95. ActAskSeeDow		List the files of the zona of download of the course
			 96. ActAskAdmDow		Admin the files of the zona of download of the course
			 97. ActAskAdmCom		Request the administracin of the files of the common zone of the course o of a group
			 98. ActSeeBib			Show the bibliografa
			 99. ActSeeFAQ			Show the FAQ
			100. ActSeeCrsLnk		Show links related to the course
			101. ActEdiTchGui		Edit teaching guide of the course
			102. ActPrnCal			Show print view of the academic calendar of the course
			103. ActPrnCrsTimTbl		Show print view of the timetable
			104. ActEdiCrsTimTbl		Edit the timetable
			105. ActChgCrsTimTbl		Modify the timetable of the course
			106. ActEdiSylLec		Edit the syllabus of teora
			107. ActEdiSylPra		Edit the syllabus of prcticas
			108. ActDelItmSylLec		Remove a item del syllabus of teora
			109. ActDelItmSylPra		Remove a item del syllabus of prcticas
			110. ActUp_IteSylLec		Subir the posicin of a subrbol del syllabus of teora
			111. ActUp_IteSylPra		Subir the posicin of a subrbol del syllabus of prcticas
			112. ActDwnIteSylLec		Bajar the posicin of a subrbol del syllabus of teora
			113. ActDwnIteSylPra		Bajar the posicin of a subrbol del syllabus of prcticas
			114. ActRgtIteSylLec		Aumentar the level of a item del syllabus of teora
			115. ActRgtIteSylPra		Aumentar the level of a item del syllabus of prcticas
			116. ActLftIteSylLec		Disminuir the level of a item del syllabus of teora
			117. ActLftIteSylPra		Disminuir the level of a item del syllabus of prcticas
			118. ActInsIteSylLec		Insertar a nuevo item in the syllabus of teora
			119. ActInsIteSylPra		Insertar a nuevo item in the syllabus of prcticas
			120. ActModIteSylLec		Modify a item del syllabus of teora
			121. ActModIteSylPra		Modify a item del syllabus of prcticas
			122. ActSeeDowCrs		List the files of the zona of download of the course
			123. ActExpSeeDowCrs		Expand a folder in course download
			124. ActConSeeDowCrs		Contract a folder in course download
			125. ActSeeDowGrp		List the files of the zona of download of a group
			126. ActExpSeeDowGrp		Expand a folder in group download
			127. ActConSeeDowGrp		Contract a folder in group download
			128. ActAdmDowCrs		Admin the files of the zona of download of the course
			129. ActAskRemFilDowCrs		Request removal of a file of download
			130. ActRemFilDowCrs		Remove a file of download
			131. ActRemFolDowCrs		Remove a folder empty of download
			132. ActCopDowCrs		Set source of copy in download
			133. ActPasDowCrs		Paste a folder or file in download
			134. ActRemTreDowCrs		Remove a folder no empty of download
			135. ActFrmCreDowCrs		Form to crear a folder or file in download
			136. ActRecFolDowCrs		Create a new folder in download
			137. ActRenFolDowCrs		Rename a folder in download
			138. ActRecFilDowCrs		Receive a file in download
			139. ActExpAdmDowCrs		Expand a folder in download administration
			140. ActConAdmDowCrs		Contract a folder in download administration
			141. ActShoDowCrs		Show hidden folder or file of download
			142. ActHidDowCrs		Hide folder or file of download
			143. ActAdmDowGrp		Admin the files of the zona of download of the course
			144. ActAskRemFilDowGrp		Request removal of a file of download
			145. ActRemFilDowGrp		Remove a file of download
			146. ActRemFolDowGrp		Remove a folder empty of download
			147. ActCopDowGrp		Set source of copy in download
			148. ActPasDowGrp		Paste a folder or file in download
			149. ActRemTreDowGrp		Remove a folder no empty of download
			150. ActFrmCreDowGrp		Form to crear a folder or file in download
			151. ActRecFolDowGrp		Create a new folder in download
			152. ActRenFolDowGrp		Rename a folder in download
			153. ActRecFilDowGrp		Receive a file in download
			154. ActExpAdmDowGrp		Expand a folder in download administration
			155. ActConAdmDowGrp		Contract a folder in download administration
			156. ActShoDowGrp		Show hidden folder or file of download
			157. ActHidDowGrp		Hide folder or file of download
			158. ActAdmComCrs		Admin the files of the common zone of the course
			159. ActAskRemFilComCrs		Request removal of a file comn of the course
			160. ActRemFilComCrs		Remove a file comn of the course
			161. ActRemFolComCrs		Remove a folder empty comn of the course
			162. ActCopComCrs		Set source of copy in common zone of the course
			163. ActPasComCrs		Paste a folder or file in common zone of the course
			164. ActRemTreComCrs		Remove a folder no empty of common zone of the course
			165. ActFrmCreComCrs		Form to crear a folder or file in common zone of the course
			166. ActRecFolComCrs		Create a new folder in common zone of the course
			167. ActRenFolComCrs		Rename a folder in common zone of the course
			168. ActRecFilComCrs		Receive a file in common zone of the course
			169. ActExpComCrs		Expand a folder in common zone of a course
			170. ActConComCrs		Contract a folder in common zone of a course
			171. ActAdmComGrp		Admin the files of the common zone of a group
			172. ActAskRemFilComGrp		Request removal of a file comn of a group
			173. ActRemFilComGrp		Remove a file comn of a group
			174. ActRemFolComGrp		Remove a folder empty comn of a group
			175. ActCopComGrp		Set source of copy in common zone of a group
			176. ActPasComGrp		Paste a folder or file in common zone of a group
			177. ActRemTreComGrp		Remove a folder no empty of common zone of a group
			178. ActFrmCreComGrp		Form to crear a folder or file in common zone of a group
			179. ActRecFolComGrp		Create a new folder in common zone of a group
			180. ActRenFolComGrp		Rename a folder in common zone of a group
			181. ActRecFilComGrp		Receive a file in common zone of a group
			182. ActExpComGrp		Expand a folder in common zone of a group
			183. ActConComGrp		Contract a folder in common zone of a group
			184. ActEdiBib			Edit the bibliografa
			185. ActEdiFAQ			Edit the FAQ
			186. ActEdiCrsLnk		Edit the links relacionados with the course
			187. ActSelInfSrcTchGui		Select the type of info shown in the teaching guide
			188. ActSelInfSrcSylLec		Select the type of info shown in the programa of teora
			189. ActSelInfSrcSylPra		Select the type of info shown in the progrma of prcticas
			190. ActSelInfSrcBib		Select the type of info shown in the bibliografa
			191. ActSelInfSrcFAQ		Select the type of info shown in the FAQ
			192. ActSelInfSrcCrsLnk		Select the type of info shown in the links
			193. ActRecURLTchGui		Receive a link a the teaching guide
			194. ActRecURLSylLec		Receive a link al programa of teora
			195. ActRecURLSylPra		Receive a link al programa of prcticas
			196. ActRecURLBib		Receive a link a bibliografa
			197. ActRecURLFAQ		Receive a link a FAQ
			198. ActRecURLCrsLnk		Receive a link a links
			199. ActRecPagTchGui		Receive a pgina with the teaching guide
			200. ActRecPagSylLec		Receive a pgina with the programa of teora
			201. ActRecPagSylPra		Receive a pgina with the programa of prcticas
			202. ActRecPagBib		Receive a pgina with bibliografa
			203. ActRecPagFAQ		Receive a pgina with FAQ
			204. ActRecPagCrsLnk		Receive a pgina with links
			205. ActEditorTchGui		Editor integrado of the teaching guide
			206. ActEditorSylLec		Editor integrado del programa of teora
			207. ActEditorSylPra		Editor integrado del programa of prcticas
			208. ActEditorBib		Editor integrado of bibliografa
			209. ActEditorFAQ		Editor integrado of FAQ
			210. ActEditorCrsLnk		Editor integrado of links
			211. ActTxtEdiTchGui		Editor of text of the teaching guide
			212. ActTxtEdiSylLec		Editor of text del programa of teora
			213. ActTxtEdiSylPra		Editor of text del programa of prcticas
			214. ActTxtEdiBib		Editor of text of the bibliografa
			215. ActTxtEdiFAQ		Editor of text of the FAQ
			216. ActTxtEdiCrsLnk		Editor of text of the links
			217. ActRecTxtTchGui		Receive and change the text of the teaching guide
			218. ActRecTxtSylLec		Receive and change the text del programa of teora
			219. ActRecTxtSylPra		Receive and change the text del programa of prcticas
			220. ActRecTxtBib		Receive and change the text of the bibliografa
			221. ActRecTxtFAQ		Receive and change the text of the FAQ
			222. ActRecTxtCrsLnk		Receive and change the text of the links
		Assessment:
			223. ActSeeAss			Show the assessment system
			224. ActSeeAsg			Show assignments
			225. ActEdiAss			Edit the assessment system
		        226. ActSeeCrdCrs		Show fields of my card in this course
		        227. ActEdiAsgWrkUsr		One user sends works of the course
		        228. ActReqAsgWrkCrs		A teacher requests edition of works sent to the course
			229. ActReqTst			Request a test of self-assesment
			230. ActWriTstQst		Edit a pregunta of self-assesment
			231. ActEdiTstQst		Edit preguntas of self-assesment
			232. ActCfgTst			Request renaming of tags of questions of self-assesment
			233. ActSeeExaAnn		Ver las exam announcements of exmenes (opcin pendiente of implementar)
			234. ActAskSeeAss		List the files of the zona of marks
			235. ActAskAdmAss		Admin the files of the zona of marks
			236. ActRecURLAss		Receive a link a evaluacin
			237. ActRecPagAss		Receive a pgina with evaluacin
			238. ActEditorAss		Editor integrado of evaluacin
			239. ActTxtEdiAss		Editor of text of evaluacin
			240. ActRecTxtAss		Receive and change the text del system of evaluacin

			241. ActFrmNewAsg		Form to create a new assignment
			242. ActEdiOneAsg		Edit one assignment
			243. ActNewAsg			Create new assignment
			244. ActChgAsg			Modify data of an existing assignment
			245. ActAskRemAsg		Request the removal of an assignment
			246. ActRemAsg			Remove assignment

		        247. ActRecCrdCrs		Actualizar campos of my card in this course
		        248. ActEdiAsgWrkCrs		Edit all the works sent to the course

			249. ActAskRemFilAsgUsr		Request removal of a file of assignments from a user
			250. ActRemFilAsgUsr		Remove a file of assignments from a user
			251. ActRemFolAsgUsr		Remove an empty folder of assignments of a user
			252. ActCopAsgUsr		Set origin of copy in assignments of a user
			253. ActPasAsgUsr		Paste a file or folder of assignments of a user
			254. ActRemTreAsgUsr		Remove a not empty folder of assignments of a user
			255. ActFrmCreAsgUsr		Form to create a new folder or file of assignments of a user
			256. ActRecFolAsgUsr		Create a new folder of assignments of a user
			257. ActRenFolAsgUsr		Rename a folder of assignments of a user
			258. ActRecFilAsgUsr		Receive a new file of assignments of a user
			259. ActExpAsgUsr		Expand a folder of assignments of a user
			260. ActConAsgUsr		Contract a folder of assignments of a user

			261. ActAskRemFilWrkUsr		Request removal of a file of works from a user
			262. ActRemFilWrkUsr		Remove a file of works from a user
			263. ActRemFolWrkUsr		Remove an empty folder of works of a user
			264. ActCopWrkUsr		Set origin of copy in works of a user
			265. ActPasWrkUsr		Paste a file or folder of works of a user
			266. ActRemTreWrkUsr		Remove a not empty folder of works of a user
			267. ActFrmCreWrkUsr		Form to create a new folder or file of works of a user
			268. ActRecFolWrkUsr		Create a new folder of works of a user
			269. ActRenFolWrkUsr		Rename a folder of works of a user
			270. ActRecFilWrkUsr		Receive a new file of works of a user
			271. ActExpWrkUsr		Expand a folder of works of a user
			272. ActConWrkUsr		Contract a folder of works of a user

			273. ActAskRemFilAsgCrs		Request removal of a file of assignments in the course
			274. ActRemFilAsgCrs		Remove a file of assignments in the course
			275. ActRemFolAsgCrs		Remove an empty folder of assignments in the course
			276. ActCopAsgCrs		Set origin of copy in assignments in the course
			277. ActPasAsgCrs		Paste a file or folder of assignments in the course
			278. ActRemTreAsgCrs		Remove a not empty folder of assignments in the course
			279. ActFrmCreAsgCrs		Form to create a new folder or file of assignments in the course
			280. ActRecFolAsgCrs		Create a new folder of assignments in the course
			281. ActRenFolAsgCrs		Rename a folder of assignments in the course
			282. ActRecFilAsgCrs		Receive a file of assignments in the course
			283. ActExpAsgCrs		Expand a folder of assignments in a course
			284. ActConAsgCrs		Contract a folder of assignments in a course

			285. ActAskRemFilWrkCrs		Request removal of a file of works in the course
			286. ActRemFilWrkCrs		Remove a file of works in the course
			287. ActRemFolWrkCrs		Remove an empty folder of works in the course
			288. ActCopWrkCrs		Set origin of copy in works in the course
			289. ActPasWrkCrs		Paste a file or folder of works in the course
			289. ActRemTreWrkCrs		Remove a not empty folder of works in the course
			290. ActFrmCreWrkCrs		Form to create a new folder or file of works in the course
			291. ActRecFolWrkCrs		Create a new folder of works in the course
			292. ActRenFolWrkCrs		Rename a folder of works in the course
			293. ActRecFilWrkCrs		Receive a file of works in the course
			294. ActExpWrkCrs		Expand a folder of works in a course
			295. ActConWrkCrs		Contract a folder of works in a course

			296. ActSeeTst			Show the test of self-assesment
			297. ActAssTst			Evaluar a test
			298. ActLstTstQst		List for edition varias questions of self-assesment
			299. ActRecTstQst		Receive a question of self-assesment
			300. ActShfTstQst		Borrar a question of self-assesment
			301. ActRemTstQst		Change criterio of ordenacin of las answers of a question of self-assesment
			302. ActEnableTag		Permitir a tag
			303. ActDisableTag		Prohibir a tag
			304. ActRenTag			Rename a tag
			305. ActRecCfgTst		Receive configuracin of test
			306. ActEdiExaAnn		Edit an exam announcement
			307. ActRecExaAnn		Receive an exam announcement
			308. ActPrnExaAnn		Show an exam announcement ready to be printed
			309. ActRemExaAnn		Remove an exam announcement
			310. ActSeeAssCrs		Consultar las marks in course
			311. ActExpSeeAssCrs		Expand a folder in marks in course
			312. ActConSeeAssCrs		Contract a folder in marks in course
			313. ActSeeMyAssCrs		Consulta individualizada of marks in course
			314. ActSeeAssGrp		Consultar las marks in group
			315. ActExpSeeAssGrp		Expand a folder in marks in group
			316. ActConSeeAssGrp		Contract a folder in marks in group
			317. ActSeeMyAssGrp		Consulta individualizada of marks in group
			318. ActAdmAssCrs		Edit the files with las marks in course
			319. ActAskRemFilAssCrs		Request removal of a file of marks in course
			320. ActRemFilAssCrs		Remove a file of marks in course
			321. ActRemFolAssCrs		Remove a folder empty of marks in course
			322. ActCopAssCrs		Set source of copy in marks in course
			323. ActPasAssCrs		Paste a folder or file in marks in course
			324. ActRemTreAssCrs		Remove a folder no empty of marks in course
			325. ActFrmCreAssCrs		Form to crear a folder or file in marks in course
			326. ActRecFolAssCrs		Create a new folder in marks in course
			327. ActRenFolAssCrs		Rename a folder in marks in course
			328. ActRecFilAssCrs		Receive a file to marks in course
			329. ActExpAdmAssCrs		Expand a folder in marks administration in course
			330. ActConAdmAssCrs		Contract a folder in marks administration in course
			331. ActChgNumRowHeaCrs		Change the number of rows of cabecera of a file of marks in course
			332. ActChgNumRowFooCrs		Change the number of rows of pie of a file of marks in course
			333. ActAdmAssGrp		Edit the files with las marks in group
			334. ActAskRemFilAssGrp		Request removal of a file of marks in group
			335. ActRemFilAssGrp		Remove a file of marks in group
			336. ActRemFolAssGrp		Remove a folder empty of marks in group
			337. ActCopAssGrp		Set source of copy in marks in group
			338. ActPasAssGrp		Paste a folder or file in marks in group
			339. ActRemTreAssGrp		Remove a folder no empty of marks in group
			340. ActFrmCreAssGrp		Form to crear a folder or file in marks in group
			341. ActRecFolAssGrp		Create a new folder in marks in group
			342. ActRenFolAssGrp		Rename a folder in marks in group
			343. ActRecFilAssGrp		Receive a file to marks in group
			344. ActExpAdmAssGrp		Expand a folder in marks administration in group
			345. ActConAdmAssGrp		Contract a folder in marks administration in group
			346. ActChgNumRowHeaGrp		Change the number of rows of cabecera of a file of marks in group
			347. ActChgNumRowFooGrp		Change the number of rows of pie of a file of marks in group
		Users:
			348. ActSeeStd			Show orla o lista of todos the students of a course
			349. ActSeeTch			Show orla o lista of teachers
			350. ActReqCrdCrsStd		Request the visin of las fichas of the students
			351. ActEdiFieCrd		Request the edition of campos of fichas of students
			352. ActSeeCrdCrsTch		Show las fichas of the teachers
			353. ActSeeTut			Show the timetable of tutoras
			354. ActReqEdiGrp		Request the edition of groups of students
			355. ActReqSelGrp		Request the seleccin of groups of students
			356. ActReqNewUsrCrs		Request the alta a a user in this course
			357. ActMdfSevStdCrs		Request new / remove several students to / from current course
			358. ActGetExtLstStd		Get external lists of students
			359. ActReqPwdOthUsr		Request the change of the contrasea of acceso of otro user
			360. ActReqPhoUsr		Request the envo of the foto of a user of this course
			361. ActReqRemUsrCrs		Request the baja of a user of this course
			362. ActAskRemAllStd		Dar of baja a todos the students of this course
			363. ActReqDNIRemGbl		Request the removal total of a user
			364. ActAskRemOldUsr		Request the removal total of the users antiguos
			365. ActPrnStdPho		Show the orla of students ready to be printed
			366. ActPrnTchPho		Show the orla of teachers ready to be printed
			367. ActLstStdAll		List in otra window the data completos of the students of the course
			368. ActLstTchAll		List in otra window the data completos of the teachers
			369. ActSeeCrdCrsStd		Show las fichas of the students
			370. ActPrnCrdCrsStd		Show las fichas of students listas to imprimir
			371. ActPrnCrdCrsTch		Show las fichas of teachers listas to imprimir
			372. ActRecCrdCrsOthUsr		Actualizar campos of the card of a student in this course
			373. ActNewFie			Request the creation of a campo of las fichas
			374. ActReqRemFie		Request the removal of a campo of las fichas
			375. ActRemFie			Remove a campo of las fichas
			376. ActRenFie			Request renaming of a campo of las fichas
			377. ActChgRowFie		Request change in number of lneas del form of a campo of las fichas
			378. ActChgVisFie		Request change in visibilidad por students of a campo of las fichas
			379. ActNewGrpTyp		Request the creation of a type of group of students
			380. ActReqRemGrpTyp		Request the removal of a type of group of students
			381. ActRemGrpTyp		Remove a type of group of students
			382. ActRenGrpTyp		Request renaming of a type of group of students
			383. ActChgMdtGrpTyp		Request change in the obligatoriedad of apuntarse a a group of a type
			384. ActChgMulGrpTyp		Request change in the posibilidad of apuntarse a ms of a group of a type
			385. ActNewGrp			Request the creation of a group of students
			386. ActReqRemGrp		Request the removal of a group of students
			387. ActRemGrp			Remove a group of students
			388. ActOpeGrp			Abrir a group of students
			389. ActCloGrp			Cerrar a group of students
			390. ActEnaFilZonGrp		Enable zonas of files of a group
			391. ActDisFilZonGrp		Disable zonas of files of a group
			392. ActChgGrpTyp		Request change in the type of group of students
			393. ActRenGrp			Request renaming of a group of students
			394. ActChgMaxStdGrp		Request change in the number mximo of students of a group
			395. ActChgGrp			Change of groups of students
			396. ActAskNewUsrCrs		Dar of alta a a user in this course
			397. ActUsrPho			Send the photo of otro user
			398. ActDetUsrPho		Receive other user's photo and detect faces on it
			399. ActUpdUsrPho		Update other user's photo
			400. ActRemUsrPho		Remove the photo of otro user
			401. ActCreOthUsrDat		Insertar a nuevo user in this course
			402. ActUpdOthUsrDat		Actualizar the card of otro user
			403. ActRegUsr			Register a existing user in current course without modifying user's data

			404. ActReqAccRegCrs		Confirm acceptation / refusion of registration in current course
			405. ActAccRegCrs		Accept registration in current course
			406. ActRefRegCrs		Refuse registration in current course

			407. ActRecFrmMdfStdCrs		Receive a form with DNI of users to register/remove to/from current course
			408. ActFrmPwdOthUsr		Show form to the change of the contrasea of acceso of otro user
			409. ActRecPwdOthUsr		Change the contrasea of acceso of otro user
			410. ActAskRemUsrCrs		Confirm baja of a user of this course
			411. ActRemUsrCrs		Dar of baja a user of this course
			412. ActAskRemUsrGbl		Confirm the removal total of a user
			413. ActRemUsrGbl		Remove totalmente a user
			414. ActRemAllStdCrs		Borrar todos the students of this course
			415. ActRemOldUsr		Remove completely old users
		Messages:
			416. ActSeeNot			Show notices
			417. ActReqMsgUsr		Write message a varios users
			418. ActSeeSntMsg		Show the messages enviados a otros users
			419. ActSeeRecMsg		Show the messages recibidos of otros users (link in menu)
			420. ActMaiStd			Send an e-mail to students
			421. ActSeeFor			Show the level superior of the forums
			422. ActSeeChtRms		Show the chat rooms
			423. ActWriNot			Write a new notice
			424. ActRecNot			Receive a new notice
			425. ActHidNot			Hide a notice that was shown
			426. ActShoNot			Show a notice that was hidden
			427. ActRemNot			Remove a notice
			428. ActSeeNewMsg		Show the received messages from other users (link in top heading)

			429. ActSeeForCrsUsr		Show top level of forum of users of the course
			430. ActSeeForCrsTch		Show top level of forum of teachers of the course
			431. ActSeeForDegUsr		Show top level of forum of users of the degree
			432. ActSeeForDegTch		Show top level of forum of teachers of the degree
			433. ActSeeForCtrUsr		Show top level of forum of users del centre
			434. ActSeeForCtrTch		Show top level of forum of teachers del centre
			435. ActSeeForInsUsr		Show top level of forum of users of the institution
			436. ActSeeForInsTch		Show top level of forum of teachers of the institution
			437. ActSeeForGenUsr		Show top level of forum of users general
			438. ActSeeForGenTch		Show top level of forum of teachers general
			439. ActSeeForSWAUsr		Show top level of forum of users of the platform
			440. ActSeeForSWATch		Show top level of forum of teachers of the platform

			441. ActSeePstForCrsUsr		Show the messages of a thread del forum of users of the course
			442. ActSeePstForCrsTch		Show the messages of a thread del forum of teachers of the course
			443. ActSeePstForDegUsr		Show the messages of a thread del forum of users of the degree
			444. ActSeePstForDegTch		Show the messages of a thread del forum of teachers of the degree
			445. ActSeePstForCtrUsr		Show the messages of a thread del forum of users del centre
			446. ActSeePstForCtrTch		Show the messages of a thread del forum of teachers del centre
			447. ActSeePstForInsUsr		Show the messages of a thread del forum of users of the institution
			448. ActSeePstForInsTch		Show the messages of a thread del forum of teachers of the institution
			449. ActSeePstForGenUsr		Show the messages of a thread del forum of users general
			450. ActSeePstForGenTch		Show the messages of a thread del forum of teachers general
			451. ActSeePstForSWAUsr		Show the messages of a thread del forum of users of the platform
			452. ActSeePstForSWATch		Show the messages of a thread del forum of teachers of the platform

			453. ActRecThrForCrsUsr		Receive the primer message of a new thread of forum of users of the course
			454. ActRecThrForCrsTch		Receive the primer message of a new thread of forum of teachers of the course
			455. ActRecThrForDegUsr		Receive the primer message of a new thread of forum of users of the degree
			456. ActRecThrForDegTch		Receive the primer message of a new thread of forum of teachers of the degree
			457. ActRecThrForCtrUsr		Receive the primer message of a new thread of forum of users of centre
			458. ActRecThrForCtrTch		Receive the primer message of a new thread of forum of teachers of centre
			459. ActRecThrForInsUsr		Receive the primer message of a new thread of forum of users of the institution
			460. ActRecThrForInsTch		Receive the primer message of a new thread of forum of teachers of the institution
			461. ActRecThrForGenUsr		Receive the primer message of a new thread of forum of users general
			462. ActRecThrForGenTch		Receive the primer message of a new thread of forum of teachers general
			463. ActRecThrForSWAUsr		Receive the primer message of a new thread of forum of users of the platform
			464. ActRecThrForSWATch		Receive the primer message of a new thread of forum of teachers of the platform

			465. ActRecRepForCrsUsr		Receive a message of answer in a thread existente in the forum of users of the course
			466. ActRecRepForCrsTch		Receive a message of answer in a thread existente in the forum of teachers of the course
			467. ActRecRepForDegUsr		Receive a message of answer in a thread existente in the forum of users of the degree
			468. ActRecRepForDegTch		Receive a message of answer in a thread existente in the forum of teachers of the degree
			469. ActRecRepForCtrUsr		Receive a message of answer in a thread existente in the forum of users of centre
			470. ActRecRepForCtrTch		Receive a message of answer in a thread existente in the forum of teachers of centre
			471. ActRecRepForInsUsr		Receive a message of answer in a thread existente in the forum of users of the institution
			472. ActRecRepForInsTch		Receive a message of answer in a thread existente in the forum of teachers of the institution
			473. ActRecRepForGenUsr		Receive a message of answer in a thread existente in the forum of users general
			474. ActRecRepForGenTch		Receive a message of answer in a thread existente in the forum of teachers general
			475. ActRecRepForSWAUsr		Receive a message of answer in a thread existente in the forum of users of the platform
			476. ActRecRepForSWATch		Receive a message of answer in a thread existente in the forum of teachers of the platform

			477. ActReqDelThrCrsUsr		Request the removal of a thread of forum of users of the course
			478. ActReqDelThrCrsTch		Request the removal of a thread of forum of teachers of the course
			479. ActReqDelThrDegUsr		Request the removal of a thread of forum of users of the degree
			480. ActReqDelThrDegTch		Request the removal of a thread of forum of teachers of the degree
			481. ActReqDelThrCtrUsr		Request the removal of a thread of forum of users of centre
			482. ActReqDelThrCtrTch		Request the removal of a thread of forum of teachers of centre
			483. ActReqDelThrInsUsr		Request the removal of a thread of forum of users of the institution
			484. ActReqDelThrInsTch		Request the removal of a thread of forum of teachers of the institution
			485. ActReqDelThrGenUsr		Request the removal of a thread of forum of users general
			486. ActReqDelThrGenTch		Request the removal of a thread of forum of teachers general
			487. ActReqDelThrSWAUsr		Request the removal of a thread of forum of users of the platform
			488. ActReqDelThrSWATch		Request the removal of a thread of forum of teachers of the platform

			489. ActDelThrForCrsUsr		Remove a thread of forum of users of the course
			490. ActDelThrForCrsTch		Remove a thread of forum of teachers of the course
			491. ActDelThrForDegUsr		Remove a thread of forum of users of the degree
			492. ActDelThrForDegTch		Remove a thread of forum of teachers of the degree
			493. ActDelThrForCtrUsr		Remove a thread of forum of users of centre
			494. ActDelThrForCtrTch		Remove a thread of forum of teachers of centre
			495. ActDelThrForInsUsr		Remove a thread of forum of users of the institution
			496. ActDelThrForInsTch		Remove a thread of forum of teachers of the institution
			497. ActDelThrForGenUsr		Remove a thread of forum of users general
			498. ActDelThrForGenTch		Remove a thread of forum of teachers general
			499. ActDelThrForSWAUsr		Remove a thread of forum of users of the platform
			500. ActDelThrForSWATch		Remove a thread of forum of teachers of the platform

			501. ActCutThrForCrsUsr		Cut a thread of forum of users of the course
			502. ActCutThrForCrsTch		Cut a thread of forum of teachers of the course
			503. ActCutThrForDegUsr		Cut a thread of forum of users of the degree
			504. ActCutThrForDegTch		Cut a thread of forum of teachers of the degree
			505. ActCutThrForCtrUsr		Cut a thread of forum of users of centre
			506. ActCutThrForCtrTch		Cut a thread of forum of teachers of centre
			507. ActCutThrForInsUsr		Cut a thread of forum of users of the institution
			508. ActCutThrForInsTch		Cut a thread of forum of teachers of the institution
			509. ActCutThrForGenUsr		Cut a thread of forum of users general
			510. ActCutThrForGenTch		Cut a thread of forum of teachers general
			511. ActCutThrForSWAUsr		Cut a thread of forum of users of the platform
			512. ActCutThrForSWATch		Cut a thread of forum of teachers of the platform

			513. ActPasThrForCrsUsr		Paste a thread of forum of users of the course
			514. ActPasThrForCrsTch		Paste a thread of forum of teachers of the course
			515. ActPasThrForDegUsr		Paste a thread of forum of users of the degree
			516. ActPasThrForDegTch		Paste a thread of forum of teachers of the degree
			517. ActPasThrForCtrUsr		Paste a thread of forum of users of centre
			518. ActPasThrForCtrTch		Paste a thread of forum of teachers of centre
			519. ActPasThrForInsUsr		Paste a thread of forum of users of the institution
			520. ActPasThrForInsTch		Paste a thread of forum of teachers of the institution
			521. ActPasThrForGenUsr		Paste a thread of forum of users general
			522. ActPasThrForGenTch		Paste a thread of forum of teachers general
			523. ActPasThrForSWAUsr		Paste a thread of forum of users of the platform
			524. ActPasThrForSWATch		Paste a thread of forum of teachers of the platform

			525. ActDelPstForCrsUsr		Remove a message of forum of users of the course
			526. ActDelPstForCrsTch		Remove a message of forum of teachers of the course
			527. ActDelPstForDegUsr		Remove a message of forum of users of the degree
			528. ActDelPstForDegTch		Remove a message of forum of teachers of the degree
			529. ActDelPstForCtrUsr		Remove a message of forum of users of centre
			530. ActDelPstForCtrTch		Remove a message of forum of teachers of centre
			531. ActDelPstForInsUsr		Remove a message of forum of users of the institution
			532. ActDelPstForInsTch		Remove a message of forum of teachers of the institution
			533. ActDelPstForGenUsr		Remove a message of forum of users general
			534. ActDelPstForGenTch		Remove a message of forum of teachers general
			535. ActDelPstForSWAUsr		Remove a message of forum of users of the platform
			536. ActDelPstForSWATch		Remove a message of forum of teachers of the platform

			537. ActEnbPstForCrsUsr		Enable a message of forum of users of the course
			538. ActEnbPstForCrsTch		Enable a message of forum of teachers of the course
			539. ActEnbPstForDegUsr		Enable a message of forum of users of the degree
			540. ActEnbPstForDegTch		Enable a message of forum of teachers of the degree
			541. ActEnbPstForCtrUsr		Enable a message of forum of users of centre
			542. ActEnbPstForCtrTch		Enable a message of forum of teachers of centre
			543. ActEnbPstForInsUsr		Enable a message of forum of users of the institution
			544. ActEnbPstForInsTch		Enable a message of forum of teachers of the institution
			545. ActEnbPstForGenUsr		Enable a message of forum of users general
			546. ActEnbPstForGenTch		Enable a message of forum of teachers general
			547. ActEnbPstForSWAUsr		Enable a message of forum of users of the platform
			548. ActEnbPstForSWATch		Enable a message of forum of teachers of the platform

			549. ActDisPstForCrsUsr		Disable a message of forum of users of the course
			550. ActDisPstForCrsTch		Disable a message of forum of teachers of the course
			551. ActDisPstForDegUsr		Disable a message of forum of users of the degree
			552. ActDisPstForDegTch		Disable a message of forum of teachers of the degree
			553. ActDisPstForCtrUsr		Disable a message of forum of users of centre
			554. ActDisPstForCtrTch		Disable a message of forum of teachers of centre
			555. ActDisPstForInsUsr		Disable a message of forum of users of the institution
			556. ActDisPstForInsTch		Disable a message of forum of teachers of the institution
			557. ActDisPstForGenUsr		Disable a message of forum of users general
			558. ActDisPstForGenTch		Disable a message of forum of teachers general
			559. ActDisPstForSWAUsr		Disable a message of forum of users of the platform
			560. ActDisPstForSWATch		Disable a message of forum of teachers of the platform

			561. ActRecMsgUsr		Receive a message of a user
			562. ActReqDelAllSntMsg		Request the removal of todos the messages enviados a otros users
			563. ActReqDelAllRecMsg		Request the removal of todos the messages recibidos of otros users
			564. ActDelAllSntMsg		Remove todos the messages enviados a otros users
			565. ActDelAllRecMsg		Remove todos the messages recibidos of otros users
			566. ActDelSntMsg		Remove a message enviado a otros users
			567. ActDelRecMsg		Remove a message recibido of otro user
			568. ActExpSntMsg		See (expand) sent message
			569. ActExpRecMsg		See (expand) received message
			570. ActCht			Enter in a chat room to chat
		Statistics:
			571. ActReqAccCrs		Request query of accesses a the course
			572. ActReqAccGbl		Request query of accesses to the complete platform
			573. ActReqUseGbl		Request showing use of the platform
			574. ActReqStaCrs		Request statistics of courses
			575. ActSeePhoDeg		Show a class photo with the average photos of the students of each degree.
			576. ActSeeAccCrs		Query accesses to current course
			577. ActSeeAccGbl		Query accesses to the complete platform
			578. ActSeeUseGbl		Show use of the platform
			579. ActSeeStaCrs		Show statistics of courses
			580. ActPrnPhoDeg		Show vista of impresin of the class photo with the average photos of the students of each degree.
			581. ActCalPhoDeg		Compute the average photos of the students of each degree.
		Personal:
			582. ActFrmSndNewPwd		Show form to send a new password via e-mail
			583. ActFrmChgPwd		Show form to the change of the contrasea of acceso
			584. ActFrmNic			Show form to the change of apodo (nickname)
			585. ActReqEdiCrdCom		Request the edition of the card with the data personales del user
			586. ActReqMyPho		Show form to send my photo
			587. ActEdiPrf			Show forms to edit preferences
			588. ActSeeMyTimTbl		Show the timetable of todas las courses del user identificado
			589. ActEdiTut			Edit the timetable of tutoras
			590. AccAdmMlt			Show the briefcase of almacenamiento personal
			591. ActSndNewPwd		Send a new password via e-mail
			592. ActRecPwd			Change the password
			593. ActRecNic			Change the nickname
			594. ActActMyData		Update my personal data
			595. ActDetMyPho		Receive my photo and detect faces on it
			596. ActUpdMyPho		Update my photo
			597. ActRemMyPho		Remove the photo del user
			598. ActChgThe			Change theme
			599. ActChgLan			Change language
			600. ActChgCol			Change side columns
			601. ActHidLftCol		Hide left side column
			602. ActHidRgtCol		Hide right side column
			603. ActShoLftCol		Show left side column
			604. ActShoRgtCol		Show right side column
			605. ActChgSecIco		Change secure icons
			606. ActChgPubPho		Change photo privacity
			607. ActChgNotNewMsg		Change whether to notify by e-mail new messages
			608. ActPrnMyTimTbl		Show the timetable listo to impresin of all my courses
			609. ActChgTut			Modify the timetable of tutoras
			610. ActAskRemFilBrf		Request removal of a file del briefcase
			611. ActRemFilBrf		Remove a file del briefcase
			612. ActRemFolBrf		Remove a folder empty del briefcase
			613. ActCopBrf			Set source of copy in the briefcase
			614. ActPasBrf			Paste a folder or file in the briefcase
			615. ActRemTreBrf		Remove a folder no empty del briefcase
			616. ActFrmCreBrf		Form to crear a folder or file in the briefcase
			617. ActRecFolBrf		Create a new folder in the briefcase
			618. ActRenFolBrf		Rename a folder del briefcase
			619. ActRecFilBrf		Receive a file in the briefcase
			620. ActExpBrf			Expand a folder in briefcase
			621. ActConBrf			Contract a folder in briefcase
		Plugins:
			622. ActLstPlg			List plugins
			623. ActEdiPlg			Edit plugins
			624. ActNewPlg			Request the creation of a plugin
			625. ActRemPlg			Remove a plugin
			626. ActRenPlg			Rename a plugin
			627. ActChgPlgDes		Change the description of a plugin
			628. ActChgPlgLog		Change the logo of a plugin
			629. ActChgPlgURL		Change the URL of a plugin
			630. ActChgPlgIP		Change the IP address of a plugin

*/
// Permissions: 1 means permitted and 0 means forbidden
// The order is Superuser-Administrator-Teacher-Student-Invited-Unknown from msb to lsb
struct Tabs Tabs[Act_NUM_TABS] =
  {
   {{0			,0              },"TabUnk",NULL     ,NULL    ,0x3E,0x3E,0x3E},
   {{ActReqSch		,ActSetUp       },"TabDeg","deg_off","deg_on",0x3F,0x3F,0x3F},
   {{ActSeeTchGui	,ActSeeCrsLnk   },"TabCrs","crs_off","crs_on",0x3F,0x3F,0x00},
   {{ActSeeAss		,ActAskAdmAss   },"TabAss","ass_off","ass_on",0x3F,0x3F,0x00},
   {{ActSeeStd		,ActAskRemOldUsr},"TabUsr","usr_off","usr_on",0x3F,0x3F,0x3F},
   {{ActSeeNot		,ActSeeChtRms	},"TabMsg","msg_off","msg_on",0x3F,0x3F,0x3E},
   {{ActReqAccCrs	,ActSeePhoDeg   },"TabSta","sta_off","sta_on",0x3F,0x3F,0x3F},
   {{ActFrmSndNewPwd	,AccAdmMlt      },"TabPer","per_off","per_on",0x3F,0x3F,0x3F},
   {{ActLstPlg		,ActLstPlg      },"TabPlg","plg_off","plg_on",0x20,0x20,0x20},
  };
struct Actions Actions[Act_NUM_ACTIONS] =
  {
   // TabUnk *************************************************************************
   // Actions not in menu:
   {-1,"ActAll"			,TabUnk,ActAll		,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,NULL				,NULL				,"Cualquier accin"			,NULL},	// Used for statistics
   {-1,"ActUnk"			,TabUnk,ActUnk		,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,NULL				,NULL				,"Accin desconocida"			,NULL},
   {-1,"ActAutUsrInt"		,TabUnk,ActAutUsrInt	,0x3F,0x3F,0x3F,true ,Act_MAIN_WINDOW,NULL			,NULL				,NULL				,"Autenticar usuario"			,NULL},
   {-1,"ActAutUsrExt"		,TabUnk,ActAutUsrExt	,0x3F,0x3F,0x3F,true ,Act_MAIN_WINDOW,NULL			,NULL				,NULL				,"Autenticar usuario desde fuera"	,NULL},
   {-1,"ActChgUsrTypLog"	,TabUnk,ActChgUsrTypLog	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,Usr_ChgUsrTypeLog1	,Usr_ChgUsrTypeLog2		,NULL				,"Cambiar tipo de usuario identif."	,NULL},
   {-1,"ActMnu"			,TabUnk,ActMnu		,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,NULL				,NULL				,"Ver men"				,NULL},
   {-1,"ActDirAcc"		,TabUnk,ActDirAcc	,0x3F,0x3F,0x00,false,Act_NEW_WINDOW ,NULL			,Deg_XHTMLDirAccToCrs		,NULL				,"Ver HTML acceso directo"		,NULL},
   {-1,"ActRef"			,TabUnk,ActRef		,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Lay_RefreshMsgsAndConnected	,NULL				,"Refrescar mensajes y usuarios"	,NULL},
   {-1,"ActWebSvc"		,TabUnk,ActWebSvc	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Plg_WebService			,NULL				,"Ejecutar servicio web"		,NULL},
   {-1,"ActLogOut"		,TabUnk,ActLogOut	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,Ses_CloseSession		,NULL				,NULL				,"Cerrar sesin"			,NULL},

   // TabDeg *************************************************************************
   // Actions in menu:
   { 0,"ActReqSch"		,TabDeg,ActReqSch	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Deg_ReqSearchCrsTch		,NULL				,"Solicitar bsqueda asig./prof."	,"search"		},
   { 1,"ActSeeIns"		,TabDeg,ActSeeIns	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Ins_SeeInstitutions		,Ins_PutHiddenParamInsOrderType	,"Ver instituciones"			,"house"		},
   { 2,"ActSeeCtr"		,TabDeg,ActSeeCtr	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Ctr_SeeCentres			,Ctr_PutHiddenParamCtrOrderType	,"Ver centros"				,"house"		},
   { 3,"ActSeeDpt"		,TabDeg,ActSeeDpt	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Dpt_SeeDepts			,Dpt_PutHiddenParamDptOrderType	,"Ver departamentos"			,"houseteachers"	},
   { 4,"ActSeePlc"		,TabDeg,ActSeePlc	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Plc_SeePlaces			,Plc_PutHiddenParamPlcOrderType	,"Ver lugares"				,"earth"		},
   { 5,"ActSeeHld"		,TabDeg,ActSeeHld	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Hld_SeeHolidays		,Hld_PutHiddenParamHldOrderType	,"Ver d&iacute;as festivos"		,"date"			},
   { 6,"ActLstAdm"		,TabDeg,ActLstAdm	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Usr_ListDataAdms		,NULL				,"Ver lista administradores"		,"listtch"		},
   { 7,"ActReqNewAdm"		,TabDeg,ActReqNewAdm	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Usr_ReqDNINewAdm		,NULL				,"Solicitar DNI alta administrador"	,"adduser"		},
   { 8,"ActReqRemAdm"		,TabDeg,ActReqRemAdm	,0x30,0x30,0x30,false,Act_MAIN_WINDOW,NULL			,Usr_ReqDNIRemAdm		,NULL				,"Solicitar DNI baja administrador"	,"removeuser"		},
   { 9,"ActReqEdiDegTyp"	,TabDeg,ActReqEdiDegTyp	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Deg_ReqEditDegreeTypes		,NULL				,"Solicitar edicin tipos titul."	,"editpack"		},
   {10,"ActReqEdiDeg"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Deg_ReqEditDegrees		,NULL				,"Solicitar edicin titulaciones"	,"editpack"		},
   {11,"ActReqEdiCrs"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,false,Act_MAIN_WINDOW,NULL			,Deg_ReqEditCourses		,NULL				,"Solicitar edicin asignaturas"	,"editblackboard"	},
   {12,"ActSeeLnk"		,TabDeg,ActSeeLnk	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Lnk_SeeLinks			,NULL				,"Ver enlaces institucionales"		,"earth"		},
   {13,"ActSetUp"		,TabDeg,ActSetUp	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Set_SetUp			,NULL				,"Instalacin inicial"			,"lightning"		},

   // Actions not in menu:
   {-1,"ActSchCrs"		,TabDeg,ActReqSch	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Deg_GetParamSearchCrs	,Deg_SearchCrs			,NULL				,"Buscar asignaturas"			,NULL},
   {-1,"ActSchCrsTch"		,TabDeg,ActReqSch	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Deg_SearchCrsTch		,NULL				,"Buscar asignaturas/profesores"	,NULL},

   {-1,"ActEdiIns"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Ins_EditInstitutions		,NULL				,"Editar instituciones"			,NULL},
   {-1,"ActNewIns"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ins_RecFormNewIns		,NULL				,"Crear institucin"			,NULL},
   {-1,"ActRemIns"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ins_RemoveInstitution		,NULL				,"Eliminar institucin"			,NULL},
   {-1,"ActRenInsSho"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ins_RenameInsShort		,NULL				,"Cambiar nombre breve ins."		,NULL},
   {-1,"ActRenInsFul"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ins_RenameInsFull		,NULL				,"Cambiar nombre completo ins."		,NULL},
   {-1,"ActChgInsLog"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ins_ChangeInsLogo		,NULL				,"Cambiar logo de institucin"		,NULL},
   {-1,"ActChgInsWWW"		,TabDeg,ActSeeIns	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ins_ChangeInsWWW		,NULL				,"Cambiar web de institucin"		,NULL},

   {-1,"ActEdiCtr"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Ctr_EditCentres		,NULL				,"Editar centros"			,NULL},
   {-1,"ActNewCtr"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ctr_RecFormNewCtr		,NULL				,"Crear centro"				,NULL},
   {-1,"ActRemCtr"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ctr_RemoveCentre		,NULL				,"Eliminar centro"			,NULL},
   {-1,"ActChgCtrIns"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ctr_ChangeCentreIns		,NULL				,"Cambiar institucin centro"		,NULL},
   {-1,"ActRenCtrSho"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ctr_RenameCentreShort		,NULL				,"Cambiar nombre breve centro"		,NULL},
   {-1,"ActRenCtrFul"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ctr_RenameCentreFull		,NULL				,"Cambiar nombre completo centro"	,NULL},
   {-1,"ActChgCtrWWW"		,TabDeg,ActSeeCtr	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Ctr_ChangeCtrWWW		,NULL				,"Cambiar web de centro"		,NULL},

   {-1,"ActEdiDpt"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Dpt_EditDepartments		,NULL				,"Editar departamentos"			,NULL},
   {-1,"ActNewDpt"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Dpt_RecFormNewDpt		,NULL				,"Crear departamento"			,NULL},
   {-1,"ActRemDpt"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Dpt_RemoveDepartment		,NULL				,"Eliminar departamento"		,NULL},
   {-1,"ActChgDptIns"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Dpt_ChangeDepartIns		,NULL				,"Cambiar institucin depar."		,NULL},
   {-1,"ActRenDptSho"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Dpt_RenameDepartShort		,NULL				,"Cambiar nombre breve depar."		,NULL},
   {-1,"ActRenDptFul"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Dpt_RenameDepartFull		,NULL				,"Cambiar nombre completo depar."	,NULL},
   {-1,"ActChgDptWWW"		,TabDeg,ActSeeDpt	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Dpt_ChangeDptWWW		,NULL				,"Cambiar web de departamento"		,NULL},

   {-1,"ActEdiPlc"		,TabDeg,ActSeePlc	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Plc_EditPlaces			,NULL				,"Editar lugares"			,NULL},
   {-1,"ActNewPlc"		,TabDeg,ActSeePlc	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plc_RecFormNewPlace		,NULL				,"Crear lugar"				,NULL},
   {-1,"ActRemPlc"		,TabDeg,ActSeePlc	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plc_RemovePlace		,NULL				,"Eliminar lugar"			,NULL},
   {-1,"ActRenPlcSho"		,TabDeg,ActSeePlc	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plc_RenamePlaceShort		,NULL				,"Cambiar nombre breve lugar"		,NULL},
   {-1,"ActRenPlcFul"		,TabDeg,ActSeePlc	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plc_RenamePlaceFull		,NULL				,"Cambiar nombre completo lugar"	,NULL},

   {-1,"ActEdiHld"		,TabDeg,ActSeeHld	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Hld_EditHolidays		,NULL				,"Editar festivo."			,NULL},
   {-1,"ActNewHld"		,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_RecFormNewHoliday		,NULL				,"Crear festiv."			,NULL},
   {-1,"ActRemHld"		,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_RemoveHoliday		,NULL				,"Eliminar festiv."			,NULL},
   {-1,"ActChgHldPlc"		,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_ChangeHolidayPlace		,NULL				,"Cambiar lugar festiv."		,NULL},
   {-1,"ActChgHldTyp"		,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_ChangeHolidayType		,NULL				,"Cambiar tipo festiv."			,NULL},
   {-1,"ActChgHldStrDat"	,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_ChangeStartDate		,NULL				,"Cambiar fecha inicio festiv."		,NULL},
   {-1,"ActChgHldEndDat"	,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_ChangeEndDate		,NULL				,"Cambiar fecha fin festiv."		,NULL},
   {-1,"ActRenHld"		,TabDeg,ActSeeHld	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Hld_RenameHoliday		,NULL				,"Cambiar nombre festiv."		,NULL},

   {-1,"ActAskNewAdm"		,TabDeg,ActReqNewAdm	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Usr_AskIfAddAdmToDeg		,NULL				,"Confirmar alta administrador"		,NULL},
   {-1,"ActNewAdm"		,TabDeg,ActReqNewAdm	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Usr_AddAdmToDeg		,NULL				,"Dar alta administrador"		,NULL},
   {-1,"ActAskRemAdm"		,TabDeg,ActReqRemAdm	,0x30,0x30,0x30,false,Act_MAIN_WINDOW,NULL			,Usr_ReqRemAdm			,NULL				,"Confirmar baja administrador"		,NULL},
   {-1,"ActRemAdm"		,TabDeg,ActReqRemAdm	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Usr_RemAdm			,NULL				,"Dar baja administrador"		,NULL},

   {-1,"ActNewDegTyp"		,TabDeg,ActReqEdiDegTyp	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RecFormNewDegTyp		,NULL				,"Crear tipo de titulacin"		,NULL},
   {-1,"ActRemDegTyp"		,TabDeg,ActReqEdiDegTyp	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RemoveDegreeType		,NULL				,"Eliminar tipo titulacin"		,NULL},
   {-1,"ActRenDegTyp"		,TabDeg,ActReqEdiDegTyp	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RenameDegreeType		,NULL				,"Renombrar tipo de titulacin"		,NULL},

   {-1,"ActNewDeg"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RecFormNewDeg		,NULL				,"Crear titulacin"			,NULL},
   {-1,"ActRemDeg"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RemoveDegree		,NULL				,"Eliminar titulacin"			,NULL},
   {-1,"ActRenDegSho"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RenameDegreeShort		,NULL				,"Cambiar nombre breve titulacin"	,NULL},
   {-1,"ActRenDegFul"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_RenameDegreeFull		,NULL				,"Cambiar nombre completo titulac."	,NULL},
   {-1,"ActChgDegTyp"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegreeType		,NULL				,"Cambiar tipo de titulacin"		,NULL},
   {-1,"ActChgDegIns"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegreeIns		,NULL				,"Cambiar institucin de titul."	,NULL},
   {-1,"ActChgDegPlc"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegreePlace		,NULL				,"Cambiar lugar de titulacin"		,NULL},
   {-1,"ActChgDegFstYea"	,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegFirstYear		,NULL				,"Cambiar primer ao de titulacin"	,NULL},
   {-1,"ActChgDegLstYea"	,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegLastYear		,NULL				,"Cambiar ltimo ao de titulacin"	,NULL},
   {-1,"ActChgDegOptYea"	,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegOptYear		,NULL				,"Cambiar ao opcional titulacin"	,NULL},
   {-1,"ActChgDegLog"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegLogo		,NULL				,"Cambiar logo de titulacin"		,NULL},
   {-1,"ActChgDegWWW"		,TabDeg,ActReqEdiDeg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeDegWWW		,NULL				,"Cambiar web de titulacin"		,NULL},

   {-1,"ActNewCrs"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Deg_RecFormNewCrs		,NULL				,"Crear asignatura"			,NULL},
   {-1,"ActRemCrs"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Deg_RemoveCourse		,NULL				,"Eliminar asignatura"			,NULL},
   {-1,"ActChgCrsDeg"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeCrsDegree		,NULL				,"Mover asignatura a otra titulac."	,NULL},
   {-1,"ActChgCrsYea"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeCrsYear		,NULL				,"Cambiar curso de asignatura"		,NULL},
   {-1,"ActChgCrsSem"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Deg_ChangeCrsSemester		,NULL				,"Cambiar semestre de asignatura"	,NULL},
   {-1,"ActRenCrsSho"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,Deg_RenameCourseShort	,Deg_ContEditAfterChgCrs	,NULL				,"Cambiar nombre breve asignatura"	,NULL},
   {-1,"ActRenCrsFul"		,TabDeg,ActReqEdiCrs	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,Deg_RenameCourseFull	,Deg_ContEditAfterChgCrs	,NULL				,"Cambiar nombre completo asig."	,NULL},

   {-1,"ActEdiLnk"		,TabDeg,ActSeeLnk	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Lnk_EditLinks			,NULL				,"Editar enlaces institucionales"	,NULL},
   {-1,"ActNewLnk"		,TabDeg,ActSeeLnk	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Lnk_RecFormNewLink		,NULL				,"Crear enlace institucional"		,NULL},
   {-1,"ActRemLnk"		,TabDeg,ActSeeLnk	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Lnk_RemoveLink			,NULL				,"Eliminar enlace institucional"	,NULL},
   {-1,"ActRenLnkSho"		,TabDeg,ActSeeLnk	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Lnk_RenameLinkShort		,NULL				,"Cambiar nombre breve enlace ins."	,NULL},
   {-1,"ActRenLnkFul"		,TabDeg,ActSeeLnk	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Lnk_RenameLinkFull		,NULL				,"Cambiar nombre completo enlace ins."	,NULL},
   {-1,"ActChgLnkWWW"		,TabDeg,ActSeeLnk	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Lnk_ChangeLinkWWW		,NULL				,"Cambiar web de enlace institucional"	,NULL},

   // TabCrs *************************************************************************
   // Actions in menu:
   { 0,"ActSeeTchGui"		,TabCrs,ActSeeTchGui	,0x3F,0x3F,0x00,false,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver gua docente"			,"guide"		},
   { 1,"ActSeeCal"		,TabCrs,ActSeeCal	,0x3F,0x3F,0x00,false,Act_MAIN_WINDOW,NULL			,Deg_DrawCalendar		,NULL				,"Ver calendario"			,"date"			},
   { 2,"ActSeeCrsTimTbl"	,TabCrs,ActSeeCrsTimTbl	,0x3F,0x3F,0x00,false,Act_MAIN_WINDOW,NULL			,TT_ShowClassTimeTable		,NULL				,"Ver horario"				,"clock"		},
   { 3,"ActSeeSylLec"		,TabCrs,ActSeeSylLec	,0x3F,0x3F,0x00,false,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver teora"				,"presentation"		},
   { 4,"ActSeeSylPra"		,TabCrs,ActSeeSylPra	,0x3F,0x3F,0x00,false,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver prcticas"			,"lab"			},
   { 5,"ActAskSeeDow"		,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Ver descargas"			,"cd"			},
   { 6,"ActAskAdmDow"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar descargas"		,"editcd"		},
   { 7,"ActAskAdmCom"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar zona comn"		,"editcdusers"		},
   { 8,"ActSeeBib"		,TabCrs,ActSeeBib	,0x3F,0x3F,0x00,false,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver bibliografa"			,"books"		},
   { 9,"ActSeeFAQ"		,TabCrs,ActSeeFAQ	,0x2C,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver FAQ"				,"faq"			},
   {10,"ActSeeCrsLnk"		,TabCrs,ActSeeCrsLnk	,0x2C,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver enlaces"				,"earth"		},
   // Actions not in menu:
   {-1,"ActEdiTchGui"		,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar gua docente"			,NULL},
   {-1,"ActPrnCal"		,TabCrs,ActSeeCal	,0x3F,0x3F,0x00,true ,Act_NEW_WINDOW ,NULL			,Deg_DrawCalendar		,NULL				,"Imprimir calendario"			,NULL},
   {-1,"ActPrnCrsTimTbl"	,TabCrs,ActSeeCrsTimTbl	,0x3F,0x3F,0x00,true ,Act_NEW_WINDOW ,NULL			,TT_ShowClassTimeTable		,NULL				,"Imprimir horario"			,NULL},
   {-1,"ActEdiCrsTimTbl"	,TabCrs,ActSeeCrsTimTbl	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,TT_EditCrsTimeTable		,NULL				,"Editar horario asignatura"		,NULL},
   {-1,"ActChgCrsTimTbl"	,TabCrs,ActSeeCrsTimTbl	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,TT_EditCrsTimeTable		,NULL				,"Cambiar horario asignatura"		,NULL},
   {-1,"ActEdiSylLec"		,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar teora"			,NULL},
   {-1,"ActEdiSylPra"		,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar prcticas"			,NULL},
   {-1,"ActDelItmSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_RemoveItemSyllabus		,NULL				,"Eliminar item teora"			,NULL},
   {-1,"ActDelItmSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_RemoveItemSyllabus		,NULL				,"Eliminar item prcticas"		,NULL},
   {-1,"ActUp_IteSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_UpItemSyllabus		,NULL				,"Subir posicin item teora"		,NULL},
   {-1,"ActUp_IteSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_UpItemSyllabus		,NULL				,"Subir posicin item prcticas"	,NULL},
   {-1,"ActDwnIteSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_DownItemSyllabus		,NULL				,"Bajar posicin item teora"		,NULL},
   {-1,"ActDwnIteSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_DownItemSyllabus		,NULL				,"Bajar posicin item prcticas"	,NULL},
   {-1,"ActRgtIteSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_RightItemSyllabus		,NULL				,"Aumentar nivel item teora"		,NULL},
   {-1,"ActRgtIteSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_RightItemSyllabus		,NULL				,"Aumentar nivel item prcticas"	,NULL},
   {-1,"ActLftIteSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_LeftItemSyllabus		,NULL				,"Disminuir nivel item teora"		,NULL},
   {-1,"ActLftIteSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_LeftItemSyllabus		,NULL				,"Disminuir nivel item prcticas"	,NULL},
   {-1,"ActInsIteSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_InsertItemSyllabus		,NULL				,"Insertar item teora"			,NULL},
   {-1,"ActInsIteSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_InsertItemSyllabus		,NULL				,"Insertar item prcticas"		,NULL},
   {-1,"ActModIteSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_ModifyItemSyllabus		,NULL				,"Editar item teora"			,NULL},
   {-1,"ActModIteSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_ModifyItemSyllabus		,NULL				,"Editar item prcticas"		,NULL},

   {-1,"ActSeeDowCrs"		,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Ver descargas asignatura"		,NULL},
   {-1,"ActExpSeeDowCrs"	,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta ver desc. asg."	,NULL},
   {-1,"ActConSeeDowCrs"	,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta ver desc. asg."	,NULL},

   {-1,"ActSeeDowGrp"		,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Ver descargas grupo"			,NULL},
   {-1,"ActExpSeeDowGrp"	,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta ver desc. grp."	,NULL},
   {-1,"ActConSeeDowGrp"	,TabCrs,ActAskSeeDow	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta ver desc. grp."	,NULL},

   {-1,"ActAdmDowCrs"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar descargas asignatura"	,NULL},
   {-1,"ActAskRemFilDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. desc. asg."	,NULL},
   {-1,"ActRemFilDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo descarga asg."	,NULL},
   {-1,"ActRemFolDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta descarga asg."	,NULL},
   {-1,"ActCopDowCrs"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de descarga asg."		,NULL},
   {-1,"ActPasDowCrs"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en descarga asg."		,NULL},
   {-1,"ActRemTreDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol descarga asg."		,NULL},
   {-1,"ActFrmCreDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solicitar nueva descarga asg."	,NULL},
   {-1,"ActRecFolDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta descarga asg."		,NULL},
   {-1,"ActRenFolDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta descarga asg."	,NULL},
   {-1,"ActRecFilDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo descarga asg."		,NULL},
   {-1,"ActExpAdmDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta admin.desc.asg."	,NULL},
   {-1,"ActConAdmDowCrs"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta admin.desc.asg."	,NULL},
   {-1,"ActShoDowCrs"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowDownload		,NULL				,"Habilitar arch/carp desc. asg."	,NULL},
   {-1,"ActHidDowCrs"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_HideDownload		,NULL				,"Inhabilitar arch/carp desc. asg."	,NULL},

   {-1,"ActAdmDowGrp"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar descargas grupo"		,NULL},
   {-1,"ActAskRemFilDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. desc. grp."	,NULL},
   {-1,"ActRemFilDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo descarga grp."	,NULL},
   {-1,"ActRemFolDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta descarga grp."	,NULL},
   {-1,"ActCopDowGrp"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de descarga grp."		,NULL},
   {-1,"ActPasDowGrp"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en descarga grp."		,NULL},
   {-1,"ActRemTreDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol descarga grp."		,NULL},
   {-1,"ActFrmCreDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solicitar nueva descarga grp."	,NULL},
   {-1,"ActRecFolDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta descarga grp."		,NULL},
   {-1,"ActRenFolDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta descarga grp."	,NULL},
   {-1,"ActRecFilDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo descarga grp."		,NULL},
   {-1,"ActExpAdmDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta admin.desc.grp."	,NULL},
   {-1,"ActConAdmDowGrp"	,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta admin.desc.grp."	,NULL},
   {-1,"ActShoDowGrp"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowDownload		,NULL				,"Habilitar arch/carp desc. grp."	,NULL},
   {-1,"ActHidDowGrp"		,TabCrs,ActAskAdmDow	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_HideDownload		,NULL				,"Inhabilitar arch/carp desc. grp."	,NULL},

   {-1,"ActAdmComCrs"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar zona comn asg."		,NULL},
   {-1,"ActAskRemFilComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. comn asg."	,NULL},
   {-1,"ActRemFilComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo comn asg."		,NULL},
   {-1,"ActRemFolComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta comn asg."		,NULL},
   {-1,"ActCopComCrs"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de zona comn asg."		,NULL},
   {-1,"ActPasComCrs"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en zona comn asg."		,NULL},
   {-1,"ActRemTreComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol comn asg."		,NULL},
   {-1,"ActFrmCreComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic.nuevo arch/carp comn asg."	,NULL},
   {-1,"ActRecFolComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta comn asg."		,NULL},
   {-1,"ActRenFolComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta comn asg."		,NULL},
   {-1,"ActRecFilComCrs"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo comn asg."		,NULL},
   {-1,"ActExpComCrs"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta comn asg."		,NULL},
   {-1,"ActConComCrs"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta comn asg."		,NULL},

   {-1,"ActAdmComGrp"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar zona comn grp."		,NULL},
   {-1,"ActAskRemFilComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. comn grp."	,NULL},
   {-1,"ActRemFilComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo comn grp."		,NULL},
   {-1,"ActRemFolComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta comn grp."		,NULL},
   {-1,"ActCopComGrp"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de zona comn grp."		,NULL},
   {-1,"ActPasComGrp"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en zona comn grp."		,NULL},
   {-1,"ActRemTreComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol comn grp."		,NULL},
   {-1,"ActFrmCreComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic.nuevo arch/carp comn grp."	,NULL},
   {-1,"ActRecFolComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta comn grp."		,NULL},
   {-1,"ActRenFolComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta comn grp."		,NULL},
   {-1,"ActRecFilComGrp"	,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo comn grp."		,NULL},
   {-1,"ActExpComGrp"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta comn grupo"		,NULL},
   {-1,"ActConComGrp"		,TabCrs,ActAskAdmCom	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta comn grupo"		,NULL},

   {-1,"ActEdiBib"		,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar bibliografa"			,NULL},
   {-1,"ActEdiFAQ"		,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar FAQ"				,NULL},
   {-1,"ActEdiCrsLnk"		,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar enlaces"			,NULL},
   {-1,"ActSelInfSrcTchGui"	,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de gua docente"		,NULL},
   {-1,"ActSelInfSrcSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de prog. de teora"	,NULL},
   {-1,"ActSelInfSrcSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de prog. de prct."	,NULL},
   {-1,"ActSelInfSrcBib"	,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de bibliografa"		,NULL},
   {-1,"ActSelInfSrcFAQ"	,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de FAQ"			,NULL},
   {-1,"ActSelInfSrcCrsLnk"	,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de enlaces"		,NULL},
   {-1,"ActRecURLTchGui"	,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a gua docente"		,NULL},
   {-1,"ActRecURLSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a teora"		,NULL},
   {-1,"ActRecURLSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a prcticas"		,NULL},
   {-1,"ActRecURLBib"		,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a bibliografa"		,NULL},
   {-1,"ActRecURLFAQ"		,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a FAQ"			,NULL},
   {-1,"ActRecURLCrsLnk"	,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a enlaces"		,NULL},
   {-1,"ActRecPagTchGui"	,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de gua docente"	,NULL},
   {-1,"ActRecPagSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de teora"		,NULL},
   {-1,"ActRecPagSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de prcticas"		,NULL},
   {-1,"ActRecPagBib"		,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de bibliografa"	,NULL},
   {-1,"ActRecPagFAQ"		,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de FAQ"			,NULL},
   {-1,"ActRecPagCrsLnk"	,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de enlaces"		,NULL},
   {-1,"ActEditorTchGui"	,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditorTeachingGuide	,NULL				,"Editor integrado de gua docente"	,NULL},
   {-1,"ActEditorSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_EditSyllabus		,NULL				,"Editor integrado de teora"		,NULL},
   {-1,"ActEditorSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Syl_LoadSyllabusToMem	,Syl_EditSyllabus		,NULL				,"Editor integrado de prcticas"	,NULL},
   {-1,"ActEditorBib"		,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditorBibliography		,NULL				,"Editor integrado de bibliografa"	,NULL},
   {-1,"ActEditorFAQ"		,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditorFAQ   		,NULL				,"Editor integrado de FAQ"		,NULL},
   {-1,"ActEditorCrsLnk"	,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditorLinks		,NULL				,"Editor integrado de enlaces"		,NULL},
   {-1,"ActTxtEdiTchGui"	,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo		,NULL				,"Editor de texto de gua docente"	,NULL},
   {-1,"ActTxtEdiSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo		,NULL				,"Editor de texto de teora"		,NULL},
   {-1,"ActTxtEdiSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo		,NULL				,"Editor de texto de prcticas"		,NULL},
   {-1,"ActTxtEdiBib"		,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo		,NULL				,"Editor de texto de bibliografa"	,NULL},
   {-1,"ActTxtEdiFAQ"		,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo   		,NULL				,"Editor de texto de FAQ"		,NULL},
   {-1,"ActTxtEdiCrsLnk"	,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo		,NULL				,"Editor de texto de enlaces"		,NULL},
   {-1,"ActRecTxtTchGui"	,TabCrs,ActSeeTchGui	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar texto de gua docente"		,NULL},
   {-1,"ActRecTxtSylLec"	,TabCrs,ActSeeSylLec	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar texto de teora"		,NULL},
   {-1,"ActRecTxtSylPra"	,TabCrs,ActSeeSylPra	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar texto de prcticas"		,NULL},
   {-1,"ActRecTxtBib"		,TabCrs,ActSeeBib	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar texto de bibliografa"		,NULL},
   {-1,"ActRecTxtFAQ"		,TabCrs,ActSeeFAQ	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar texto de FAQ"			,NULL},
   {-1,"ActRecTxtCrsLnk"	,TabCrs,ActSeeCrsLnk	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar texto de enlaces"		,NULL},
   // TabAss *************************************************************************
   // Actions in menu:
   { 0,"ActSeeAss"		,TabAss,ActSeeAss	,0x3F,0x3F,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ShowInfo			,NULL				,"Ver evaluacin"			,"infogrades"		},
   { 1,"ActSeeAsg"		,TabAss,ActSeeAsg	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Asg_SeeAssignments		,Asg_PutHiddenParamAsgOrderType	,"Ver actividades"			,"desk"			},
   { 2,"ActSeeCrdCrs"		,TabAss,ActSeeCrdCrs	,0x24,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_ShowFormMyCrsRecord	,NULL				,"Solicitar cambio ficha asig."		,"editcard"		},
   { 3,"ActEdiAsgWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,Brw_ParamWorksZoneUsr		,"Editar mis trabajos"			,"editcduser"		},
   { 4,"ActReqAsgWrkCrs"	,TabAss,ActReqAsgWrkCrs	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskEditWorksCrs		,Brw_ParamWorksZoneCrs		,"Editar trabajos de asignatura"	,"cdusers"		},
   { 5,"ActReqTst"		,TabAss,ActReqTst	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ShowFormAskTst		,NULL				,"Solicitar test"			,"test"			},
   { 6,"ActWriTstQst"		,TabAss,ActWriTstQst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ShowFormEditOneQst		,NULL				,"Escribir pregunta test"		,"edittest"		},
   { 7,"ActEdiTstQst"		,TabAss,ActEdiTstQst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ShowFormAskEditTsts	,NULL				,"Solicitar edicin de tests"		,"edittest"		},
   { 8,"ActCfgTst"		,TabAss,ActCfgTst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ShowFormConfig		,NULL				,"Solicitar configuracin test"		,"edittest"		},
   { 9,"ActSeeExaAnn"		,TabAss,ActSeeExaAnn	,0x3F,0x3F,0x00,true ,Act_MAIN_WINDOW,NULL			,Exa_ListExamAnnouncementsSee	,NULL				,"Ver convocatorias"			,"announce"		},
   {10,"ActAskSeeAss"		,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Ver calificaciones"			,"grades"		},
   {11,"ActAskAdmAss"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Administrar calificaciones"		,"editgrades"		},
   // Actions not in menu:
   {-1,"ActEdiAss"		,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_FormsToSelSendInfo		,NULL				,"Editar evaluacin"			,NULL},
   {-1,"ActSelInfSrcAss"	,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_SetInfoSrc			,NULL				,"Selec. tipo de evaluacin"		,NULL},
   {-1,"ActRecURLAss"		,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceiveURLInfo		,NULL				,"Enviar enlace a evaluacin"		,NULL},
   {-1,"ActRecPagAss"		,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_ReceivePagInfo		,NULL				,"Enviar pgina de evaluacin"		,NULL},
   {-1,"ActEditorAss"		,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditorAssessment		,NULL				,"Editor integrado de evaluacin"	,NULL},
   {-1,"ActTxtEdiAss"		,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_EditTxtInfo		,NULL				,"Editor de texto de evaluacin"	,NULL},
   {-1,"ActRecTxtAss"		,TabAss,ActSeeAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Crs_RecAndChangeInfoTxt	,NULL				,"Enviar text de evaluacin"		,NULL},

   {-1,"ActFrmNewAsg"		,TabAss,ActSeeAsg	,0x28,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Asg_RequestCreatOrEditAsg	,NULL				,"Solicitar creacin actividad"		,NULL},
   {-1,"ActEdiOneAsg"		,TabAss,ActSeeAsg	,0x28,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Asg_RequestCreatOrEditAsg	,NULL				,"Solicitar edicin actividad"		,NULL},
   {-1,"ActNewAsg"		,TabAss,ActSeeAsg	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Asg_RecFormAssignment		,NULL				,"Crear actividad"			,NULL},
   {-1,"ActChgAsg"		,TabAss,ActSeeAsg	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Asg_RecFormAssignment		,NULL				,"Modificar actividad"			,NULL},
   {-1,"ActAskRemAsg"		,TabAss,ActSeeAsg	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Asg_AskRemAssignment		,NULL				,"Solicitar elim. actividad"		,NULL},
   {-1,"ActRemAsg"		,TabAss,ActSeeAsg	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Asg_RemoveAssignment		,NULL				,"Eliminar actividad"			,NULL},

   {-1,"ActRecCrdCrs"		,TabAss,ActSeeCrdCrs	,0x24,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_UpdateAndShowMyCrsRecord	,NULL				,"Cambiar ficha asignatura"		,NULL},
   {-1,"ActEdiAsgWrkCrs"	,TabAss,ActReqAsgWrkCrs	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,NULL				,"Editar trabajos de asignatura"	,NULL},

   {-1,"ActAskRemFilAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. mis act."	,NULL},
   {-1,"ActRemFilAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo mis activid."	,NULL},
   {-1,"ActRemFolAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta mis activid."	,NULL},
   {-1,"ActCopAsgUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de mis actividades"		,NULL},
   {-1,"ActPasAsgUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en mis actividades"		,NULL},
   {-1,"ActRemTreAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol mis activid."		,NULL},
   {-1,"ActFrmCreAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic. nuevo arch/carp mis act."	,NULL},
   {-1,"ActRecFolAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta mis actividades"	,NULL},
   {-1,"ActRenFolAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta mis activid."	,NULL},
   {-1,"ActRecFilAsgUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo mis actividades"	,NULL},
   {-1,"ActExpAsgUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta mis activid."	,NULL},
   {-1,"ActConAsgUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta mis activid."	,NULL},

   {-1,"ActAskRemFilWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. mis trab."	,NULL},
   {-1,"ActRemFilWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo mis trabajos"	,NULL},
   {-1,"ActRemFolWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta mis trabajos"	,NULL},
   {-1,"ActCopWrkUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de mis trabajos"		,NULL},
   {-1,"ActPasWrkUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en mis trabajos"		,NULL},
   {-1,"ActRemTreWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol mis trabajos"		,NULL},
   {-1,"ActFrmCreWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic. nuevo arch/carp mis trab."	,NULL},
   {-1,"ActRecFolWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta mis trabajos"		,NULL},
   {-1,"ActRenFolWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta mis trabajos"	,NULL},
   {-1,"ActRecFilWrkUsr"	,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo mis trabajos"		,NULL},
   {-1,"ActExpWrkUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta mis trabajos"	,NULL},
   {-1,"ActConWrkUsr"		,TabAss,ActEdiAsgWrkUsr	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta mis trabajos"	,NULL},

   {-1,"ActAskRemFilAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. act. asg."	,NULL},
   {-1,"ActRemFilAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo activid. asg."	,NULL},
   {-1,"ActRemFolAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta activid. asg."	,NULL},
   {-1,"ActCopAsgCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de actividades asg."		,NULL},
   {-1,"ActPasAsgCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en actividades asg."		,NULL},
   {-1,"ActRemTreAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol activid. asg."		,NULL},
   {-1,"ActFrmCreAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic. nuevo arch/carp act.asg."	,NULL},
   {-1,"ActRecFolAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta activid. asg."		,NULL},
   {-1,"ActRenFolAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta activid. asg."	,NULL},
   {-1,"ActRecFilAsgCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo activid. asg."		,NULL},
   {-1,"ActExpAsgCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta activid. asg."	,NULL},
   {-1,"ActConAsgCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta activid. asg."	,NULL},

   {-1,"ActAskRemFilWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. trab. asg."	,NULL},
   {-1,"ActRemFilWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo trabajos asg."	,NULL},
   {-1,"ActRemFolWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta trabajos asg."	,NULL},
   {-1,"ActCopWrkCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de trabajos asg."		,NULL},
   {-1,"ActPasWrkCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en trabajos asg."		,NULL},
   {-1,"ActRemTreWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol trabajos asg."		,NULL},
   {-1,"ActFrmCreWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic. nuevo arch/carp trab.asg."	,NULL},
   {-1,"ActRecFolWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta trabajos asg."		,NULL},
   {-1,"ActRenFolWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta trabajos asg."	,NULL},
   {-1,"ActRecFilWrkCrs"	,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo trabajos asg."		,NULL},
   {-1,"ActExpWrkCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta trabajos asg."	,NULL},
   {-1,"ActConWrkCrs"		,TabAss,ActReqAsgWrkCrs,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta trabajos asg."	,NULL},

   {-1,"ActSeeTst"		,TabAss,ActReqTst	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ShowTestExam		,NULL				,"Ver test"				,NULL},
   {-1,"ActAssTst"		,TabAss,ActReqTst	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_AssessTestExam		,NULL				,"Evaluar test"				,NULL},
   {-1,"ActLstTstQst"		,TabAss,ActEdiTstQst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ListQuestionsToEdit	,NULL				,"Editar preguntas test"		,NULL},
   {-1,"ActRecTstQst"		,TabAss,ActWriTstQst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ReceiveQst			,NULL				,"Enviar pregunta test"			,NULL},
   {-1,"ActRemTstQst"		,TabAss,ActEdiTstQst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_RemoveQst			,NULL				,"Eliminar pregunta test"		,NULL},
   {-1,"ActShfTstQst"		,TabAss,ActEdiTstQst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ChangeShuffleQst		,NULL				,"Cambiar orden. respuestas test"	,NULL},
   {-1,"ActEnableTag"		,TabAss,ActCfgTst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_EnableTag			,NULL				,"Permitir descriptor test"		,NULL},
   {-1,"ActDisableTag"		,TabAss,ActCfgTst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_DisableTag			,NULL				,"Prohibir descriptor test"		,NULL},
   {-1,"ActRenTag"		,TabAss,ActCfgTst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_RenameTag			,NULL				,"Renombrar descriptor test"		,NULL},
   {-1,"ActRecCfgTst"		,TabAss,ActCfgTst	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Tst_ReceiveConfigTst		,NULL				,"Recibir configuracin test"		,NULL},
   {-1,"ActEdiExaAnn"		,TabAss,ActSeeExaAnn	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Exa_PutFrmEditAExamAnnouncement,NULL				,"Editar convocatoria"			,NULL},
   {-1,"ActRecExaAnn"		,TabAss,ActSeeExaAnn	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Exa_ReceiveExamAnnouncement	,NULL				,"Enviar convocatoria"			,NULL},
   {-1,"ActPrnExaAnn"		,TabAss,ActSeeExaAnn	,0x3F,0x3F,0x00,true ,Act_NEW_WINDOW ,NULL			,Exa_PrintExamAnnouncement	,NULL				,"Imprimir convocatoria"		,NULL},
   {-1,"ActRemExaAnn"		,TabAss,ActSeeExaAnn	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Exa_RemoveExamAnnouncement	,NULL				,"Eliminar convocatoria"		,NULL},

   {-1,"ActSeeAssCrs"		,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,Brw_ParamSeeAssZoneCrs		,"Ver calificaciones asignatura"	,NULL},
   {-1,"ActExpSeeAssCrs"	,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta ver calif. asg."	,NULL},
   {-1,"ActConSeeAssCrs"	,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta ver calif. asg."	,NULL},
   {-1,"ActSeeMyAssCrs"		,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_NEW_WINDOW ,Brw_ShowMyMarks		,NULL				,NULL				,"Ver mis calificaciones asg."		,NULL},

   {-1,"ActSeeAssGrp"		,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,Brw_ParamSeeAssZoneGrp		,"Ver calificaciones grupo"		,NULL},
   {-1,"ActExpSeeAssGrp"	,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta ver calif.grp"	,NULL},
   {-1,"ActConSeeAssGrp"	,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta ver calif.grp."	,NULL},
   {-1,"ActSeeMyAssGrp"		,TabAss,ActAskSeeAss	,0x2C,0x20,0x00,true ,Act_NEW_WINDOW ,Brw_ShowMyMarks		,NULL				,NULL				,"Ver mis calificaciones grp."		,NULL},

   {-1,"ActAdmAssCrs"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,Brw_ParamAdmAssZoneCrs		,"Administrar calificaciones asg."	,NULL},
   {-1,"ActAskRemFilAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim.arch.calif.asg."	,NULL},
   {-1,"ActRemFilAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo calif. asg."		,NULL},
   {-1,"ActRemFolAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta calif. asg."		,NULL},
   {-1,"ActCopAssCrs"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de calif. asg."		,NULL},
   {-1,"ActPasAssCrs"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en calif. asg."			,NULL},
   {-1,"ActRemTreAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol calif. asg."		,NULL},
   {-1,"ActFrmCreAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic.nuevo arch/carp calif.asg."	,NULL},
   {-1,"ActRecFolAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta calif. asg."		,NULL},
   {-1,"ActRenFolAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta calif. asg."	,NULL},
   {-1,"ActRecFilAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo calif. asg."		,NULL},
   {-1,"ActExpAdmAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carp. admin.calif.asg."	,NULL},
   {-1,"ActConAdmAssCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carp. admin.calif.asg."	,NULL},
   {-1,"ActChgNumRowHeaCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ChangeNumRowsHeader	,NULL				,"Cambiar filas de cabecera asg."	,NULL},
   {-1,"ActChgNumRowFooCrs"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ChangeNumRowsFooter	,NULL				,"Cambiar filas de pie asg."		,NULL},

   {-1,"ActAdmAssGrp"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,Brw_ParamAdmAssZoneGrp		,"Administrar calificaciones grp."	,NULL},
   {-1,"ActAskRemFilAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim.arch.calif.grp."	,NULL},
   {-1,"ActRemFilAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo calif. grp."		,NULL},
   {-1,"ActRemFolAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta calif. grp."		,NULL},
   {-1,"ActCopAssGrp"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de calif. grp."		,NULL},
   {-1,"ActPasAssGrp"		,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en calif. grp."			,NULL},
   {-1,"ActRemTreAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol calif. grp."		,NULL},
   {-1,"ActFrmCreAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic.nuevo arch/carp calif.grp."	,NULL},
   {-1,"ActRecFolAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta calif. grp."		,NULL},
   {-1,"ActRenFolAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta calif. grp."	,NULL},
   {-1,"ActRecFilAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo calif. grp."		,NULL},
   {-1,"ActExpAdmAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carp. admin.calif.grp."	,NULL},
   {-1,"ActConAdmAssGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carp. admin.calif.grp."	,NULL},
   {-1,"ActChgNumRowHeaGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ChangeNumRowsHeader	,NULL				,"Cambiar filas de cabecera grp."	,NULL},
   {-1,"ActChgNumRowFooGrp"	,TabAss,ActAskAdmAss	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Brw_ChangeNumRowsFooter	,NULL				,"Cambiar filas de pie grp."		,NULL},

   // TabUsr *******************************************************************
   // Actions in menu:
   { 0,"ActSeeStd"		,TabUsr,ActSeeStd	,0x3C,0x30,0x30,false,Act_MAIN_WINDOW,NULL			,Usr_SeeStudents		,NULL				,"Ver orla o lista de alumnos"		,"classphoto"		},
   { 1,"ActGetExtLstStd"	,TabUsr,ActGetExtLstStd	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,Imp_ListMyImpGrpsAndStdsNoForm	,NULL				,"Ver lista de alumnos oficiales"	,"classphoto"		},
   { 2,"ActSeeTch"		,TabUsr,ActSeeTch	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Usr_SeeTeachers		,NULL				,"Ver orla o lista de profesores"	,"classphototch"	},
   { 3,"ActReqCrdCrsStd"	,TabUsr,ActReqCrdCrsStd	,0x28,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Usr_ReqListCardsStdsCrs	,NULL				,"Solicitar fichas alumnos"		,"card"			},
   { 4,"ActEdiFieCrd"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_ReqEditRecordFields	,NULL				,"Solicitar edicin campos fichas"	,"editcard"		},
   { 5,"ActSeeCrdCrsTch"	,TabUsr,ActSeeCrdCrsTch	,0x3E,0x3E,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ListCardsTchsCrs		,NULL				,"Ver fichas profesores"		,"cardtch"		},
   { 6,"ActSeeTut"		,TabUsr,ActSeeTut	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,TT_ShowTutTimeTable		,NULL				,"Ver tutoras"				,"tutor"		},
   { 7,"ActReqEdiGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ReqEditGroups		,NULL				,"Solicitar edicin de grupos"		,"editusers"		},
   { 8,"ActReqSelGrp"		,TabUsr,ActReqSelGrp	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ReqRegisterInGrps		,NULL				,"Solicitar unin a grupos"		,"users"		},
   { 9,"ActReqNewUsrCrs"	,TabUsr,ActReqNewUsrCrs	,0x38,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqDNINewUsr		,NULL				,"Solicitar DNI alta usuario"		,"adduser"		},
   {10,"ActMdfSevStdCrs"	,TabUsr,ActMdfSevStdCrs	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_RegisterRemoveUsrsCrs	,NULL				,"Solicitar alta/baja alumnos"		,"addremoveusers"	},
   {11,"ActReqPwdOthUsr"	,TabUsr,ActReqPwdOthUsr	,0x38,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqDNIOthUsrPwd		,NULL				,"Solicitar DNI cambio contrasea"	,"keyuser"		},
   {12,"ActReqPhoUsr"		,TabUsr,ActReqPhoUsr	,0x38,0x30,0x30,false,Act_MAIN_WINDOW,NULL			,Usr_ReqDNIPhotoUsr		,NULL				,"Solicitar DNI foto usuario"		,"photo"		},
   {13,"ActReqRemUsrCrs"	,TabUsr,ActReqRemUsrCrs	,0x3C,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqDNIRemUsrFromCrs	,NULL				,"Solicitar DNI baja usuario"		,"removeuser"		},
   {14,"ActAskRemAllStd"	,TabUsr,ActAskRemAllStd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_AskRemAllStdsThisCrs	,NULL				,"Solicitar baja todos alumnos"		,"removeusers"		},
   {15,"ActReqDNIRemGbl"	,TabUsr,ActReqDNIRemGbl	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqDNIRemGbl		,NULL				,"Solicitar DNI baja total"		,"removeuser"		},
   {16,"ActAskRemOldUsr"	,TabUsr,ActAskRemOldUsr	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Usr_AskRemoveOldUsrs		,NULL				,"Solicitar elim.usuarios antiguos"	,"removeusers"		},
   // Actions not in menu:	
   {-1,"ActPrnStdPho"		,TabUsr,ActSeeStd	,0x3C,0x30,0x30,false,Act_NEW_WINDOW ,NULL			,Usr_SeeStdClassPhotoPrn	,NULL				,"Imprimir orla alumnos"		,NULL},
   {-1,"ActPrnTchPho"		,TabUsr,ActSeeTch	,0x3F,0x3F,0x3F,false,Act_NEW_WINDOW ,NULL			,Usr_SeeTchClassPhotoPrn	,NULL				,"Imprimir orla profesores"		,NULL},
   {-1,"ActLstStdAll"		,TabUsr,ActSeeStd	,0x38,0x30,0x30,false,Act_NEW_WINDOW ,NULL			,Usr_ListAllDataStds		,NULL				,"Ver lista alumnos"			,NULL},
   {-1,"ActLstTchAll"		,TabUsr,ActSeeTch	,0x38,0x38,0x30,false,Act_NEW_WINDOW ,NULL			,Usr_ListAllDataTchs 		,NULL				,"Ver lista profesores"			,NULL},
   {-1,"ActSeeCrdCrsStd"	,TabUsr,ActReqCrdCrsStd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ListCardsStdsCrs		,NULL				,"Ver fichas alumnos"			,NULL},
   {-1,"ActPrnCrdCrsStd"	,TabUsr,ActReqCrdCrsStd	,0x28,0x20,0x00,true ,Act_NEW_WINDOW ,NULL			,Usr_ListCardsStdsCrs		,NULL				,"Imprimir fichas alumnos"		,NULL},
   {-1,"ActPrnCrdCrsTch"	,TabUsr,ActSeeCrdCrsTch	,0x3E,0x3E,0x00,true ,Act_NEW_WINDOW ,NULL			,Usr_ListCardsTchsCrs		,NULL				,"Imprimir fichas profesores"		,NULL},
   {-1,"ActRecCrdCrsOthUsr"	,TabUsr,ActReqCrdCrsStd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_UpdateAndShowOtherCrsRecord,NULL				,"Cambiar ficha alumno en asig."	,NULL},
   {-1,"ActNewFie"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_ReceiveFormField		,NULL				,"Crear campo de fichas"		,NULL},
   {-1,"ActReqRemFie"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_ReqRemField		,NULL				,"Solicitar elimin. campo fichas"	,NULL},
   {-1,"ActRemFie"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_RemoveField		,NULL				,"Eliminar campo de fichas"		,NULL},
   {-1,"ActRenFie"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_RenameField		,NULL				,"Renombrar campo de fichas"		,NULL},
   {-1,"ActChgRowFie"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_ChangeLinesField		,NULL				,"Cambiar lneas campo"			,NULL},
   {-1,"ActChgVisFie"		,TabUsr,ActEdiFieCrd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Rec_ChangeVisibilityField	,NULL				,"Cambiar visibilidad campo"		,NULL},
   {-1,"ActNewGrpTyp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_RecFormNewGrpTyp		,NULL				,"Crear tipo de grupo"			,NULL},
   {-1,"ActReqRemGrpTyp"	,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ReqRemGroupType		,NULL				,"Solicitar eliminacin tipo grupo"	,NULL},
   {-1,"ActRemGrpTyp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_RemoveGroupType		,NULL				,"Eliminar tipo de grupo"		,NULL},
   {-1,"ActRenGrpTyp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_RenameGroupType		,NULL				,"Renombrar tipo de grupo"		,NULL},
   {-1,"ActChgMdtGrpTyp"	,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ChangeMandatGrpTyp		,NULL				,"Cambiar obligator. tipo grupo"	,NULL},
   {-1,"ActChgMulGrpTyp"	,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ChangeMultiGrpTyp		,NULL				,"Cambiar multiplic. tipo grupo"	,NULL},
   {-1,"ActNewGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_RecFormNewGrp		,NULL				,"Crear grupo"				,NULL},
   {-1,"ActReqRemGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ReqRemGroup		,NULL				,"Solicitar eliminacin grupo"		,NULL},
   {-1,"ActRemGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_RemoveGroup		,NULL				,"Eliminar grupo"			,NULL},
   {-1,"ActOpeGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_OpenGroup			,NULL				,"Abrir grupo"				,NULL},
   {-1,"ActCloGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_CloseGroup			,NULL				,"Cerrar grupo"				,NULL},
   {-1,"ActEnaFilZonGrp"	,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_EnableFileZonesGrp		,NULL				,"Habilitar zonas archivos grupo"	,NULL},
   {-1,"ActDisFilZonCloGrp"	,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_DisableFileZonesGrp	,NULL				,"Inhabilitar zonas archivos grupo"	,NULL},
   {-1,"ActChgGrpTyp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ChangeGroupType		,NULL				,"Cambiar tipo de grupo"		,NULL},
   {-1,"ActRenGrp"		,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_RenameGroup		,NULL				,"Renombrar grupo"			,NULL},
   {-1,"ActChgMaxStdGrp"	,TabUsr,ActReqEdiGrp	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ChangeMaxStdsGrp		,NULL				,"Cambiar mx. alumnos en grupo"	,NULL},
   {-1,"ActChgGrp"		,TabUsr,ActReqSelGrp	,0x2C,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Grp_ChangeMyGrps		,NULL				,"Cambiar de grupos"			,NULL},
   {-1,"ActAskNewUsrCrs"	,TabUsr,ActReqNewUsrCrs	,0x38,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_AskIfAddUsrToCrs		,NULL				,"Confirmar alta usuario"		,NULL},
   {-1,"ActUsrPho"		,TabUsr,ActReqPhoUsr	,0x38,0x30,0x30,false,Act_MAIN_WINDOW,Usr_SendPhotoUsr1		,Usr_SendPhotoUsr2		,NULL				,"Solicitar cambio foto otro usr."	,NULL},
   {-1,"ActDetUsrPho"		,TabUsr,ActReqPhoUsr	,0x38,0x30,0x30,false,Act_MAIN_WINDOW,NULL			,Usr_RecUsrPhotoDetFaces	,NULL				,"Detectar rostros en foto otro usr."	,NULL},
   {-1,"ActUpdUsrPho"		,TabUsr,ActReqPhoUsr	,0x38,0x30,0x30,false,Act_MAIN_WINDOW,Usr_UpdateUsrPhoto1	,Usr_UpdateUsrPhoto2		,NULL				,"Cambiar foto otro usuario"		,NULL},
   {-1,"ActRemUsrPho"		,TabUsr,ActReqPhoUsr	,0x38,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,Usr_RemoveUsrPhoto		,NULL				,"Eliminar foto otro usuario"		,NULL},
   {-1,"ActCreOthUsrDat"	,TabUsr,ActReqNewUsrCrs	,0x38,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_CreatAndShowUsrCardAndRegisterInCrs,NULL			,"Dar alta usuario nuevo"		,NULL},
   {-1,"ActUpdOthUsrDat"	,TabUsr,ActReqNewUsrCrs	,0x38,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ModifAndShowUsrCardAndRegisterInCrs,NULL			,"Dar alta usr. cambiando datos"	,NULL},
   {-1,"ActRegUsr"		,TabUsr,ActReqNewUsrCrs	,0x38,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_GetDNIShowUsrCardAndRegisterInCrs,NULL			,"Dar alta usr. sin cambiar datos"	,NULL},

   {-1,"ActReqAccRegCrs"	,TabUsr,ActReqNewUsrCrs	,0x3E,0x00,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqAcceptRegisterInCrs	,NULL				,"Solicitar acept/rechazo mi alta"	,NULL},
   {-1,"ActAccRegCrs"		,TabUsr,ActReqNewUsrCrs	,0x3E,0x00,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_AcceptRegisterMeInCrs	,NULL				,"Aceptar mi alta"			,NULL},
   {-1,"ActRefRegCrs"		,TabUsr,ActReqRemUsrCrs	,0x3E,0x00,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqDNIRemUsrFromCrs	,NULL				,"Rechazar mi alta"			,NULL},

   {-1,"ActRecFrmMdfStdCrs"	,TabUsr,ActMdfSevStdCrs	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ReceiveFormUsrsCrs		,NULL				,"Dar alta/baja alumnos (formul.)"	,NULL},

   {-1,"ActFrmPwdOthUsr"	,TabUsr,ActReqPwdOthUsr	,0x38,0x30,0x30,true ,Act_MAIN_WINDOW,Usr_ShowFormOthPwd1	,Usr_ShowFormOthPwd2		,NULL				,"Confirm. cambio otra contrasea"	,NULL},
   {-1,"ActRecPwdOthUsr"	,TabUsr,ActReqPwdOthUsr	,0x38,0x30,0x30,true ,Act_MAIN_WINDOW,Usr_UpdateOtherPwd1	,Usr_UpdateOtherPwd2		,NULL				,"Cambiar otra contrasea"		,NULL},
   {-1,"ActAskRemUsrCrs"	,TabUsr,ActReqRemUsrCrs	,0x3C,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqRemUsrFromCrs		,NULL				,"Solicitar baja usuario"		,NULL},
   {-1,"ActRemUsrCrs"		,TabUsr,ActReqRemUsrCrs	,0x3C,0x30,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_RemUsrFromCrs		,NULL				,"Dar baja usuario"			,NULL},
   {-1,"ActAskRemUsrGbl"	,TabUsr,ActReqDNIRemGbl	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Usr_ReqRemUsrGbl		,NULL				,"Solicitar baja total usuario"		,NULL},
   {-1,"ActRemUsrGbl"		,TabUsr,ActReqDNIRemGbl	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Usr_RemUsrGbl			,NULL				,"Dar baja total usuario"		,NULL},
   {-1,"ActRemAllStdCrs"	,TabUsr,ActAskRemAllStd	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Usr_RemAllStdsThisCrs		,NULL				,"Dar baja todos alumnos"		,NULL},
   {-1,"ActRemOldUsr"		,TabUsr,ActAskRemOldUsr	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Usr_RemoveOldUsrs		,NULL				,"Eliminar usuarios antiguos"		,NULL},
   // TabMsg *******************************************************************
   // Actions in menu:
   { 0,"ActSeeNot"		,TabMsg,ActSeeNot	,0x3F,0x3F,0x00,true ,Act_MAIN_WINDOW,NULL			,Not_ListNotices		,NULL				,"Ver avisos"				,"note"			},
   { 1,"ActReqMsgUsr"		,TabMsg,ActReqMsgUsr	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,NULL			,Msg_FormMsgUsrs		,NULL				,"Solicitar mensaje nuevo"		,"editnewmsg"		},
   { 2,"ActSeeSntMsg"		,TabMsg,ActSeeSntMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ShowSntMsgs		,NULL				,"Ver mensajes enviados"		,"sntmsg"		},
   { 3,"ActSeeRecMsg"		,TabMsg,ActSeeRecMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ShowRecMsgs		,NULL				,"Ver mensajes recibidos"		,"recmsg"		},
   { 4,"ActMaiStd"		,TabMsg,ActMaiStd	,0x28,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Msg_ListEMails			,NULL				,"Solicitar envo correo"		,"email"		},
   { 5,"ActSeeFor"		,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumList		,For_PutAllHiddenParamsForum	,"Ver foros"				,"forum"		},
   { 6,"ActSeeChtRms"		,TabMsg,ActSeeChtRms	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Cht_ShowChatRooms		,NULL				,"Ver salas chat y pizarra"		,"chat"			},
   // Actions not in menu:
   {-1,"ActWriNot"		,TabMsg,ActSeeNot	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,NULL			,Not_ShowFormNotice		,NULL				,"Solicitar aviso nuevo"		,NULL},
   {-1,"ActRecNot"		,TabMsg,ActSeeNot	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Not_ReceiveNotice		,Not_ListNotices		,NULL				,"Crear aviso nuevo"			,NULL},
   {-1,"ActHidNot"		,TabMsg,ActSeeNot	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Not_HideNotice		,Not_ListNotices		,NULL				,"Ocultar aviso"			,NULL},
   {-1,"ActShoNot"		,TabMsg,ActSeeNot	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Not_ShowNotice		,Not_ListNotices		,NULL				,"Mostrar aviso"			,NULL},
   {-1,"ActRemNot"		,TabMsg,ActSeeNot	,0x28,0x20,0x00,true ,Act_MAIN_WINDOW,Not_DeleteNotice		,Not_ListNotices		,NULL				,"Eliminar aviso"			,NULL},
   {-1,"ActSeeNewMsg"		,TabMsg,ActSeeRecMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ShowRecMsgs		,NULL				,"Ver mensajes recibidos nuevos"	,NULL},
   {-1,"ActSeeForCrsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro usuarios asignatura"		,NULL},
   {-1,"ActSeeForCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro profesores asignatura"	,NULL},
   {-1,"ActSeeForDegUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro usuarios titulacin"		,NULL},
   {-1,"ActSeeForDegTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro profesores titulacin"	,NULL},
   {-1,"ActSeeForCtrUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro usuarios centro"		,NULL},
   {-1,"ActSeeForCtrTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro profesores centro"		,NULL},
   {-1,"ActSeeForInsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro usuarios instit."		,NULL},
   {-1,"ActSeeForInsTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro profesores instit."		,NULL},
   {-1,"ActSeeForGenUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro usuarios general"		,NULL},
   {-1,"ActSeeForGenTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro profesores general"		,NULL},
   {-1,"ActSeeForSWAUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro usuarios SWAD"		,NULL},
   {-1,"ActSeeForSWATch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowForumThrs		,NULL				,"Ver foro profesores SWAD"		,NULL},

   {-1,"ActSeePstForCrsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro usrs.asig."	,NULL},
   {-1,"ActSeePstForCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro prof.asig."	,NULL},
   {-1,"ActSeePstForDegUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro usrs.tit."	,NULL},
   {-1,"ActSeePstForDegTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro prof.tit."	,NULL},
   {-1,"ActSeePstForCtrUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro usrs.cent."	,NULL},
   {-1,"ActSeePstForCtrTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro prof.cent."	,NULL},
   {-1,"ActSeePstForInsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro usrs.inst."	,NULL},
   {-1,"ActSeePstForInsTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro prof.inst."	,NULL},
   {-1,"ActSeePstForGenUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro usrs.gral."	,NULL},
   {-1,"ActSeePstForGenTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro prof.gral."	,NULL},
   {-1,"ActSeePstForSWAUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro usrs.SWAD"	,NULL},
   {-1,"ActSeePstForSWATch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_ShowThrPsts		,NULL				,"Ver msjs. tema foro prof.SWAD"	,NULL},

   {-1,"ActRecThrForCrsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro usrs.asig."		,NULL},
   {-1,"ActRecThrForCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro prof.asig."		,NULL},
   {-1,"ActRecThrForDegUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro usrs.tit."		,NULL},
   {-1,"ActRecThrForDegTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro prof.tit."		,NULL},
   {-1,"ActRecThrForCtrUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro usrs.cent."		,NULL},
   {-1,"ActRecThrForCtrTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro prof.cent."		,NULL},
   {-1,"ActRecThrForInsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro usrs.inst."		,NULL},
   {-1,"ActRecThrForInsTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro prof.inst."		,NULL},
   {-1,"ActRecThrForGenUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro usrs.gral."		,NULL},
   {-1,"ActRecThrForGenTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro prof.gral."		,NULL},
   {-1,"ActRecThrForSWAUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro usrs.SWAD"		,NULL},
   {-1,"ActRecThrForSWATch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar tema foro prof.SWAD"		,NULL},

   {-1,"ActRecRepForCrsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro usrs.asig."		,NULL},
   {-1,"ActRecRepForCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro prof.asig."		,NULL},
   {-1,"ActRecRepForDegUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro usrs.tit." 		,NULL},
   {-1,"ActRecRepForDegTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro prof.tit." 		,NULL},
   {-1,"ActRecRepForCtrUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro usrs.cent."		,NULL},
   {-1,"ActRecRepForCtrTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro prof.cent."		,NULL},
   {-1,"ActRecRepForInsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro usrs.inst."		,NULL},
   {-1,"ActRecRepForInsTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro prof.inst."		,NULL},
   {-1,"ActRecRepForGenUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro usrs.gral."		,NULL},
   {-1,"ActRecRepForGenTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro prof.gral."		,NULL},
   {-1,"ActRecRepForSWAUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro usrs.SWAD" 		,NULL},
   {-1,"ActRecRepForSWATch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_RecForumPst		,NULL				,"Enviar resp. foro prof.SWAD" 		,NULL},

   {-1,"ActReqDelThrCrsUsr"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro usrs.asig"	,NULL},
   {-1,"ActReqDelThrCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro prof.asig"	,NULL},
   {-1,"ActReqDelThrDegUsr"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro usrs.tit."	,NULL},
   {-1,"ActReqDelThrDegTch"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro prof.tit."	,NULL},
   {-1,"ActReqDelThrCtrUsr"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro usrs.cent"	,NULL},
   {-1,"ActReqDelThrCtrTch"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro prof.cent"	,NULL},
   {-1,"ActReqDelThrInsUsr"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro usrs.inst."	,NULL},
   {-1,"ActReqDelThrInsTch"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro prof.inst."	,NULL},
   {-1,"ActReqDelThrGenUsr"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro usrs.gral."	,NULL},
   {-1,"ActReqDelThrGenTch"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro prof.gral."	,NULL},
   {-1,"ActReqDelThrSWAUsr"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro usrs.SWAD"	,NULL},
   {-1,"ActReqDelThrSWATch"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_ReqDelThr			,NULL				,"Solic.elim.hebra foro prof.SWAD"	,NULL},

   {-1,"ActDelThrForCrsUsr"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro usrs.asig."	,NULL},
   {-1,"ActDelThrForCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro prof.asig."	,NULL},
   {-1,"ActDelThrForDegUsr"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro usrs.tit."	,NULL},
   {-1,"ActDelThrForDegTch"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro prof.tit."	,NULL},
   {-1,"ActDelThrForCtrUsr"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro usrs.cent."	,NULL},
   {-1,"ActDelThrForCtrTch"	,TabMsg,ActSeeFor	,0x32,0x32,0x32,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro prof.cent."	,NULL},
   {-1,"ActDelThrForInsUsr"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro usrs.inst."	,NULL},
   {-1,"ActDelThrForInsTch"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro prof.inst."	,NULL},
   {-1,"ActDelThrForGenUsr"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro usrs.gral."	,NULL},
   {-1,"ActDelThrForGenTch"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro prof.gral."	,NULL},
   {-1,"ActDelThrForSWAUsr"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro usrs.SWAD"	,NULL},
   {-1,"ActDelThrForSWATch"	,TabMsg,ActSeeFor	,0x22,0x22,0x22,true ,Act_MAIN_WINDOW,NULL			,For_DelThr			,NULL				,"Eliminar hebra foro prof.SWAD"	,NULL},

   {-1,"ActCutThrForCrsUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro usrs.asig."		,NULL},
   {-1,"ActCutThrForCrsTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro prof.asig."		,NULL},
   {-1,"ActCutThrForDegUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro usrs.tit."		,NULL},
   {-1,"ActCutThrForDegTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro prof.tit."		,NULL},
   {-1,"ActCutThrForCtrUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro usrs.cent."		,NULL},
   {-1,"ActCutThrForCtrTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro prof.cent."		,NULL},
   {-1,"ActCutThrForInsUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro usrs.inst."		,NULL},
   {-1,"ActCutThrForInsTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro prof.inst."		,NULL},
   {-1,"ActCutThrForGenUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro usrs.gral."		,NULL},
   {-1,"ActCutThrForGenTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro prof.gral."		,NULL},
   {-1,"ActCutThrForSWAUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro usrs.SWAD"		,NULL},
   {-1,"ActCutThrForSWATch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_CutThr			,NULL				,"Cortar hebra foro prof.SWAD"		,NULL},

   {-1,"ActPasThrForCrsUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro usrs.asig."		,NULL},
   {-1,"ActPasThrForCrsTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro prof.asig."		,NULL},
   {-1,"ActPasThrForDegUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro usrs.tit."		,NULL},
   {-1,"ActPasThrForDegTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro prof.tit."		,NULL},
   {-1,"ActPasThrForCtrUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro usrs.cent."		,NULL},
   {-1,"ActPasThrForCtrTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro prof.cent."		,NULL},
   {-1,"ActPasThrForInsUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro usrs.inst."		,NULL},
   {-1,"ActPasThrForInsTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro prof.inst."		,NULL},
   {-1,"ActPasThrForGenUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro usrs.gral."		,NULL},
   {-1,"ActPasThrForGenTch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro prof.gral."		,NULL},
   {-1,"ActPasThrForSWAUsr"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro usrs.SWAD"		,NULL},
   {-1,"ActPasThrForSWATch"	,TabMsg,ActSeeFor	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,For_PasteThr			,NULL				,"Pegar hebra foro prof.SWAD"		,NULL},

   {-1,"ActDelPstForCrsUsr"	,TabMsg,ActSeeFor	,0x2E,0x2E,0x2E,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro usrs.asig."	,NULL},
   {-1,"ActDelPstForCrsTch"	,TabMsg,ActSeeFor	,0x2A,0x2A,0x2A,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro prof.asig."	,NULL},
   {-1,"ActDelPstForDegUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro usrs.tit."	,NULL},
   {-1,"ActDelPstForDegTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro prof.tit."	,NULL},
   {-1,"ActDelPstForCtrUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro usrs.cent."	,NULL},
   {-1,"ActDelPstForCtrTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro prof.cent."	,NULL},
   {-1,"ActDelPstForInsUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro usrs.inst."	,NULL},
   {-1,"ActDelPstForInsTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro prof.inst."	,NULL},
   {-1,"ActDelPstForGenUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro usrs.gral."	,NULL},
   {-1,"ActDelPstForGenTch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro prof.gral."	,NULL},
   {-1,"ActDelPstForSWAUsr"	,TabMsg,ActSeeFor	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro usrs.SWAD"	,NULL},
   {-1,"ActDelPstForSWATch"	,TabMsg,ActSeeFor	,0x3A,0x3A,0x3A,true ,Act_MAIN_WINDOW,NULL			,For_DelPst			,NULL				,"Eliminar mens. foro prof.SWAD"	,NULL},

   {-1,"ActEnbPstForCrsUsr"	,TabMsg,ActSeeFor	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro usrs.asig."	,NULL},
   {-1,"ActEnbPstForCrsTch"	,TabMsg,ActSeeFor	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro prof.asig."	,NULL},
   {-1,"ActEnbPstForDegUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro usrs.tit."	,NULL},
   {-1,"ActEnbPstForDegTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro prof.tit."	,NULL},
   {-1,"ActEnbPstForCtrUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro usrs.cent."	,NULL},
   {-1,"ActEnbPstForCtrTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro prof.cent."	,NULL},
   {-1,"ActEnbPstForInsUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro usrs.inst."	,NULL},
   {-1,"ActEnbPstForInsTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro prof.inst."	,NULL},
   {-1,"ActEnbPstForGenUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro usrs.gral."	,NULL},
   {-1,"ActEnbPstForGenTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro prof.gral."	,NULL},
   {-1,"ActEnbPstForSWAUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro usrs.SWAD"	,NULL},
   {-1,"ActEnbPstForSWATch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_EnbPst			,NULL				,"Permitir mens. foro prof.SWAD"	,NULL},

   {-1,"ActDisPstForCrsUsr"	,TabMsg,ActSeeFor	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro usrs.asig."	,NULL},
   {-1,"ActDisPstForCrsTch"	,TabMsg,ActSeeFor	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro prof.asig."	,NULL},
   {-1,"ActDisPstForDegUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro usrs.tit."	,NULL},
   {-1,"ActDisPstForDegTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro prof.tit."	,NULL},
   {-1,"ActDisPstForCtrUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro usrs.cent."	,NULL},
   {-1,"ActDisPstForCtrTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro prof.cent."	,NULL},
   {-1,"ActDisPstForInsUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro usrs.inst."	,NULL},
   {-1,"ActDisPstForInsTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro prof.inst."	,NULL},
   {-1,"ActDisPstForGenUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro usrs.gral."	,NULL},
   {-1,"ActDisPstForGenTch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro prof.gral."	,NULL},
   {-1,"ActDisPstForSWAUsr"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro usrs.SWAD"	,NULL},
   {-1,"ActDisPstForSWATch"	,TabMsg,ActSeeFor	,0x30,0x30,0x30,true ,Act_MAIN_WINDOW,NULL			,For_DisPst			,NULL				,"Censurar mens. foro prof.SWAD"	,NULL},

   {-1,"ActRecMsgUsr"		,TabMsg,ActReqMsgUsr	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_RecMsgFromUsr		,NULL				,"Enviar mensaje"			,NULL},
   {-1,"ActReqDelAllSntMsg"	,TabMsg,ActSeeSntMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ReqDelAllSntMsgs		,NULL				,"Solic. elim. mensajes enviados"	,NULL},
   {-1,"ActReqDelAllRecMsg"	,TabMsg,ActSeeRecMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ReqDelAllRecMsgs		,NULL				,"Solic. elim. mensajes recibidos"	,NULL},
   {-1,"ActDelAllSntMsg"	,TabMsg,ActSeeSntMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_DelAllSntMsgs		,NULL				,"Eliminar mensajes enviados"		,NULL},
   {-1,"ActDelAllRecMsg"	,TabMsg,ActSeeRecMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_DelAllRecMsgs		,NULL				,"Eliminar mensajes recibidos"		,NULL},
   {-1,"ActDelSntMsg"		,TabMsg,ActSeeSntMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_DelSntMsg			,NULL				,"Eliminar mensaje enviado"		,NULL},
   {-1,"ActDelRecMsg"		,TabMsg,ActSeeRecMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_DelRecMsg			,NULL				,"Eliminar mensaje recibido"		,NULL},
   {-1,"ActExpSntMsg"		,TabMsg,ActSeeSntMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ExpSntMsg			,NULL				,"Ver mensaje enviado"			,NULL},
   {-1,"ActExpRecMsg"		,TabMsg,ActSeeRecMsg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Msg_ExpRecMsg			,NULL				,"Ver mensaje recibido"			,NULL},

   {-1,"ActCht"			,TabMsg,ActSeeChtRms	,0x3E,0x3E,0x3E,true ,Act_CHAT_WINDOW,Cht_OpenChatWindow	,NULL				,NULL				,"Abrir chat y pizarra"			,NULL},

   // TabSta *******************************************************************
   // Actions in menu:
   { 0,"ActReqAccCrs"		,TabSta,ActReqAccCrs	,0x28,0x20,0x00,false,Act_MAIN_WINDOW,Sta_SetIniEndDates	,Sta_AskSeeCrsAccesses		,NULL				,"Solic. consulta accesos asig."	,"statsuser"		},
   { 1,"ActReqAccGbl"		,TabSta,ActReqAccGbl	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,Sta_SetIniEndDates	,Sta_AskSeeGblAccesses		,NULL				,"Solic. consulta accesos SWAD"		,"stats"		},
   { 2,"ActReqUseGbl"		,TabSta,ActReqUseGbl	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Sta_ReqUseOfPlatform		,NULL				,"Solicitar ver uso de SWAD"		,"pie"			},
   { 3,"ActReqStaCrs"		,TabSta,ActReqStaCrs	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Sta_ReqStatsCourses		,NULL				,"Solicitar estad. asignaturas"		,"blackboard"		},
   { 4,"ActSeePhoDeg"		,TabSta,ActSeePhoDeg	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Sta_ShowPhotoDegree		,NULL				,"Ver orla de titulaciones"		,"classphoto"		},
   // Actions not in menu:
   {-1,"ActSeeAccCrs"		,TabSta,ActReqAccCrs	,0x28,0x20,0x00,false,Act_MAIN_WINDOW,NULL			,Sta_SeeCrsAccesses		,NULL				,"Ver accesos asignatura"		,NULL},
   {-1,"ActSeeAccGbl"		,TabSta,ActReqAccGbl	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,NULL			,Sta_SeeGblAccesses		,NULL				,"Ver accesos SWAD"			,NULL},
   {-1,"ActSeeUseGbl"		,TabSta,ActReqUseGbl	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Sta_ShowUseOfPlatform		,NULL				,"Ver uso de SWAD"			,NULL},
   {-1,"ActSeeStaCrs"		,TabSta,ActReqStaCrs	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Sta_ShowStatsCourses		,NULL				,"Ver estad. asignaturas"		,NULL},
   {-1,"ActPrnPhoDeg"		,TabSta,ActSeePhoDeg	,0x3F,0x3F,0x3F,false,Act_NEW_WINDOW ,NULL			,Sta_PrintPhotoDegree  		,NULL				,"Imprimir orla de titulaciones"	,NULL},
   {-1,"ActCalPhoDeg"		,TabSta,ActSeePhoDeg	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Sta_CalcPhotoDegree		,NULL				,"Calcular fotos promedio"		,NULL},
   // TabPer *******************************************************************
   // Actions in menu:
   { 0,"ActFrmSndNewPwd"	,TabPer,ActFrmSndNewPwd	,0x00,0x01,0x01,false,Act_MAIN_WINDOW,NULL			,Usr_ShowFormSendNewPwd		,NULL				,"Solicitar envo contrasea"		,"key"			},
   { 1,"ActFrmChgPwd"		,TabPer,ActFrmChgPwd	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,NULL			,Usr_ShowFormChgPwd		,NULL				,"Solicitar cambio contrasea"		,"key"			},
   { 2,"ActFrmNic"		,TabPer,ActFrmNic	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,NULL			,Usr_ShowFormChangeNick		,NULL				,"Solicitar cambio apodo"		,"edituser"		},
   { 3,"ActReqEdiCrdCom"	,TabPer,ActReqEdiCrdCom	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Rec_ShowFormMyCommRecord	,NULL				,"Solicitar cambio ficha comn"		,"editcard"		},
   { 4,"ActReqMyPho"		,TabPer,ActReqMyPho	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,NULL			,Usr_ReqMyPhoto			,NULL				,"Solicitar envo foto"			,"photo"		},
   { 5,"ActEdiPrf"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,NULL			,Prf_EditPrefs			,NULL				,"Editar preferencias"			,"editheart"		},
   { 6,"ActSeeMyTimTbl"		,TabPer,ActSeeMyTimTbl	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,TT_ShowClassTimeTable		,NULL				,"Ver horario todas clases"		,"clock"		},
   { 7,"ActEdiTut"		,TabPer,ActEdiTut	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,TT_ShowMyTutTimeTable		,NULL				,"Editar horario tutoras"		,"edittutor"		},
   { 8,"AccAdmMlt"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFileBrowserOrWorks	,Brw_ParamBriefZoneUsr		,"Ver maletn"				,"pendrive"		},

   // Actions not in menu:
   {-1,"ActSndNewPwd"		,TabPer,ActFrmSndNewPwd	,0x00,0x01,0x01,false,Act_MAIN_WINDOW,NULL			,Usr_SendNewPwdByEMail		,NULL				,"Enviar contrasea por correo"		,NULL},
   {-1,"ActRecPwd"		,TabPer,ActFrmChgPwd	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,Usr_ActChgMyPwd1		,Usr_ActChgMyPwd2		,NULL				,"Cambiar contrasea"			,NULL},
   {-1,"ActRecNic"		,TabPer,ActFrmNic	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Usr_UpdateNick			,NULL				,"Cambiar apodo"			,NULL},
   {-1,"ActActMyData"		,TabPer,ActReqEdiCrdCom	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,Usr_UpdateMyCard		,Rec_ShowMyCommonRecordUpd	,NULL				,"Cambiar ficha comn"			,NULL},
   {-1,"ActDetMyPho"		,TabPer,ActReqMyPho	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,NULL			,Usr_RecMyPhotoDetFaces		,NULL				,"Detectar rostros mi foto"		,NULL},
   {-1,"ActUpdMyPho"		,TabPer,ActReqMyPho	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,Usr_UpdateMyPhoto1	,Usr_UpdateMyPhoto2		,NULL				,"Cambiar mi foto"			,NULL},
   {-1,"ActRemMyPho"		,TabPer,ActReqMyPho	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,Usr_RemoveMyPhoto1	,Usr_RemoveMyPhoto2		,NULL				,"Eliminar foto"			,NULL},

   {-1,"ActChgThe"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,The_ChangeTheme		,Prf_EditPrefs			,NULL				,"Cambiar diseo"			,NULL},
   {-1,"ActChgLan"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Prf_ChangeLanguage	,Prf_EditPrefs			,NULL				,"Cambiar idioma"			,NULL},
   {-1,"ActChgCol"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Prf_ChangeSideCols	,Prf_EditPrefs			,NULL				,"Cambiar columnas"			,NULL},
   {-1,"ActHidLftCol"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Prf_HideLeftCol		,Prf_EditPrefs			,NULL				,"Ocultar columna izquierda"		,NULL},
   {-1,"ActHidRgtCol"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Prf_HideRightCol		,Prf_EditPrefs			,NULL				,"Ocultar columna derecha"		,NULL},
   {-1,"ActShoLftCol"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Prf_ShowLeftCol		,Prf_EditPrefs			,NULL				,"Mostrar columna izquierda"		,NULL},
   {-1,"ActShoRgtCol"		,TabPer,ActEdiPrf	,0x3F,0x3F,0x3F,false,Act_MAIN_WINDOW,Prf_ShowRightCol		,Prf_EditPrefs			,NULL				,"Mostrar columna derecha"		,NULL},
   {-1,"ActChgSecIco"		,TabPer,ActEdiPrf	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,Prf_ChangeSecureIcons	,Prf_EditPrefs			,NULL				,"Cambiar iconos seguros"		,NULL},
   {-1,"ActChgPubPho"		,TabPer,ActEdiPrf	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,Prf_ChangePublicPhoto	,Prf_EditPrefs			,NULL				,"Cambiar privacidad foto"		,NULL},
   {-1,"ActChgNotNewMsg"	,TabPer,ActEdiPrf	,0x3E,0x3E,0x3E,false,Act_MAIN_WINDOW,Ntf_ChangeNotifyEvents	,Prf_EditPrefs			,NULL				,"Cambiar notificacin mensajes"	,NULL},

   {-1,"ActPrnMyTimTbl"		,TabPer,ActSeeMyTimTbl	,0x3E,0x3E,0x3E,true ,Act_NEW_WINDOW ,NULL			,TT_ShowClassTimeTable		,NULL				,"Imprimir horario todas clases"	,NULL},
   {-1,"ActChgTut"		,TabPer,ActEdiTut	,0x28,0x28,0x28,true ,Act_MAIN_WINDOW,NULL			,TT_ShowMyTutTimeTable		,NULL				,"Cambiar horario tutoras"		,NULL},
   {-1,"ActAskRemFilBrf"	,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_AskRemFileFromTree		,NULL				,"Solicitar elim. arch. maletn"	,NULL},
   {-1,"ActRemFilBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFileFromTree		,NULL				,"Eliminar archivo maletn"		,NULL},
   {-1,"ActRemFolBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_RemFolderFromTree		,NULL				,"Eliminar carpeta maletn"		,NULL},
   {-1,"ActCopBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_CopyFromFileBrowser	,NULL				,"Copiar de maletn"			,NULL},
   {-1,"ActPasBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_PasteIntoFileBrowser	,NULL				,"Pegar en maletn"			,NULL},
   {-1,"ActRemTreBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_RemSubtreeInFileBrowser	,NULL				,"Eliminar rbol maletn"		,NULL},
   {-1,"ActFrmCreBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_ShowFormFileBrowser	,NULL				,"Solic. nuevo arch/carp malet."	,NULL},
   {-1,"ActRecFolBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_RecFolderFileBrowser	,NULL				,"Crear carpeta en maletn"		,NULL},
   {-1,"ActRenFolBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_RenFolderFileBrowser	,NULL				,"Renombrar carpeta en maletn"		,NULL},
   {-1,"ActRecFilBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_ReceiveFileInFileBrowser	,NULL				,"Enviar archivo a maletn"		,NULL},
   {-1,"ActExpBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_ExpandFileTree		,NULL				,"Expandir carpeta maletn"		,NULL},
   {-1,"ActConBrf"		,TabPer,AccAdmMlt	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Brw_ContractFileTree		,NULL				,"Contraer carpeta maletn"		,NULL},

   // TabPlg *************************************************************************
   // Actions in menu:
   { 0,"ActLstPlg"		,TabPlg,ActLstPlg	,0x3E,0x3E,0x3E,true ,Act_MAIN_WINDOW,NULL			,Plg_ListPlugins		,NULL				,"Listar plugins"			,"blocks"		},

   // Actions not in menu:
   {-1,"ActEdiPlg"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,false,Act_MAIN_WINDOW,NULL			,Plg_EditPlugins		,NULL				,"Editar plugins"			,NULL},
   {-1,"ActNewPlg"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_RecFormNewPlg		,NULL				,"Crear plugin"				,NULL},
   {-1,"ActRemPlg"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_RemovePlugin		,NULL				,"Eliminar plugin"			,NULL},
   {-1,"ActRenPlg"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_RenamePlugin		,NULL				,"Cambiar nombre de plugin"		,NULL},
   {-1,"ActChgPlgDes"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_ChangePlgDescription	,NULL				,"Cambiar descripcin de plugin"	,NULL},
   {-1,"ActChgPlgLog"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_ChangePlgLogo		,NULL				,"Cambiar logo de plugin"		,NULL},
   {-1,"ActChgPlgURL"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_ChangePlgURL		,NULL				,"Cambiar URL de plugin"		,NULL},
   {-1,"ActChgPlgIP"		,TabPlg,ActLstPlg	,0x20,0x20,0x20,true ,Act_MAIN_WINDOW,NULL			,Plg_ChangePlgIP		,NULL				,"Cambiar IP de plugin"			,NULL},
  };

/****************************************************************************/
/*********** Write icon and title associated to the current action **********/
/****************************************************************************/

const char *Act_GetTitleAction (Act_Action_t Action)
  {
   extern const char *Txt_OPTION_TITLE[Act_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB][Txt_NUM_LANGUAGES];

   return Txt_OPTION_TITLE[Actions[Actions[Action].SuperAction].Tab][Actions[Actions[Action].SuperAction].IndexInMenu][Gbl.Prefs.Language];
  }

/****************************************************************************/
/***************** Create a breadcrumb string to an action ******************/
/****************************************************************************/

void Act_GetBreadcrumbStrForAction (Act_Action_t Action,Txt_Language_t Language,bool HTML,char *BreadcrumbStr)
  {
   extern const char *Txt_TABS_FULL_TXT[Act_NUM_TABS][Txt_NUM_LANGUAGES];
   extern const char *Txt_OPTION_TITLE[Act_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB][Txt_NUM_LANGUAGES];
   Act_Action_t Superaction = Actions[Action].SuperAction;
   char *Arrow;

   Arrow = HTML ? "&gt;" : ">"; 
   sprintf (BreadcrumbStr,"https://%s/ %s %s %s %s",
            Cfg_PLATFORM_URL,Arrow,
            Txt_TABS_FULL_TXT[Actions[Superaction].Tab][Language],Arrow,
            Txt_OPTION_TITLE[Actions[Superaction].Tab][Actions[Superaction].IndexInMenu][Language]);
  }

/****************************************************************************/
/******************************** Start a form ******************************/
/****************************************************************************/

void Act_FormStart (Act_Action_t NextAction)
  {
   Gbl.NumForm++; // Initialized to -1. The first time it is incremented, it will be equal to 0
   /* All is written in a unique line in order to send less characters */
   fprintf (Gbl.F.Out,"<form method=\"post\" action=\"");
   switch (Actions[NextAction].BrowserWindow)
     {
      case Act_MAIN_WINDOW:
         if (NextAction == ActSeeNewMsg)
            fprintf (Gbl.F.Out,"/\" id=\"form_msg\"");
         else
            fprintf (Gbl.F.Out,"/\"");
         break;
      case Act_NEW_WINDOW:
         fprintf (Gbl.F.Out,"/\" target=\"win_aux\"");
         break;
      case Act_CHAT_WINDOW:
         fprintf (Gbl.F.Out,"/\" target=\"_blank\"");
         break;
      }
   switch (NextAction)
     {
      case ActRecFilDowCrs:	case ActRecFilDowGrp:
      case ActRecFilComCrs:	case ActRecFilComGrp:
      case ActRecFilAssCrs:	case ActRecFilAssGrp:
      case ActRecFilAsgCrs:	case ActRecFilAsgUsr:
      case ActRecFilWrkCrs:	case ActRecFilWrkUsr:
      case ActRecFilBrf:
      case ActRecPagTchGui:
      case ActRecPagSylLec:
      case ActRecPagSylPra:
      case ActRecPagBib:
      case ActRecPagFAQ:
      case ActRecPagCrsLnk:
      case ActRecPagAss:
      case ActDetMyPho:		case ActDetUsrPho:
      case ActRemMyPho:		case ActRemUsrPho:
	 fprintf (Gbl.F.Out," enctype=\"multipart/form-data\"");
	 break;
     }
   fprintf (Gbl.F.Out,">");

   if (NextAction != ActUnk)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NxtAct\" value=\"%s\" />",
               Actions[NextAction].Str);
   if (Gbl.Session.Id[0])
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"IdSes\" value=\"%s\" />",
               Gbl.Session.Id);
   else	// Extra parameters necessary when there's no session open
     {
      if (NextAction != ActUnk)
        {
         /* If session is open, course code will be get from session data,
            but if there isn't an open session, and next action is known, it is necessary to send a parameter with course code */
         if (Gbl.CurrentCrs.Crs.CrsCod >= 0)		// If course selected...
            fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CrsCod\" value=\"%ld\" />",
                     Gbl.CurrentCrs.Crs.CrsCod);
         else if (Gbl.CurrentDeg.Deg.DegCod >= 0)	// If no course selected, but degree selected...
            fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"DegCod\" value=\"%ld\" />",
                     Gbl.CurrentDeg.Deg.DegCod);
        }
     }
  }

/****************************************************************************/
/******************* Anchor directive used to send a form *******************/
/****************************************************************************/
// Requires an extern </a>

void Act_LinkFormSubmit (Act_BrowserWindow_t Window,const char *MsgWindowStatus,const char *LinkStyle)
  {
   fprintf (Gbl.F.Out,"<a href=\"javascript:");
   switch (Window)
     {
      case Act_MAIN_WINDOW:
         if (Gbl.CurrentAct == ActRef)
            fprintf (Gbl.F.Out,"document.getElementById('form_msg').submit();\"");
         else
            fprintf (Gbl.F.Out,"document.forms[%d].submit();\"",Gbl.NumForm);
         break;
      case Act_NEW_WINDOW:
         fprintf (Gbl.F.Out,"window.open('https://%s/espere.html','win_aux','width=640,height=480,location=no,menubar=yes,status=yes,toolbar=yes,scrollbars=yes,resizable=yes');" \
  	                    "document.forms[%d].submit();\"",
	          Cfg_PLATFORM_URL,Gbl.NumForm);
         break;
      case Act_CHAT_WINDOW:
         fprintf (Gbl.F.Out,"document.forms[%d].submit();\"",Gbl.NumForm);

         break;
     }
   if (LinkStyle)
      if (LinkStyle[0])
         fprintf (Gbl.F.Out," class=\"%s\"",LinkStyle);
   fprintf (Gbl.F.Out,"><acronym title=\"%s\"",MsgWindowStatus);
   if (LinkStyle)
      if (LinkStyle[0])
         fprintf (Gbl.F.Out," class=\"%s\"",LinkStyle);
   fprintf (Gbl.F.Out,">");
  }

/****************************************************************************/
/****** Adjust current action if it's mandatory to fill any empty data ******/
/****************************************************************************/

void Act_AdjustCurrentAction (void)
  {
   /***** Don't adjust anything when refreshing users or on a web service *****/
   if (Gbl.CurrentAct == ActRef ||
       Gbl.CurrentAct == ActWebSvc)
      return;

   /***** Don't adjust anything when tab is unknown and action isn't autentify user *****/
   if (Gbl.CurrentTab == TabUnk && Gbl.CurrentAct != ActAutUsrInt)
      return;

   /***** Don't adjust nothing when current action isn't a menu option *****/
   if (Gbl.CurrentAct != Actions[Gbl.CurrentAct].SuperAction)	// It isn't a menu option
      return;

   /***** If I have no password in database, the only action possible is show a form to change my password *****/
   if (!Gbl.Usrs.Me.UsrDat.Password[0])
     {
      switch (Gbl.CurrentAct)
        {
         case ActMnu: case ActLogOut: case ActFrmChgPwd:
            break;
         default:
            Gbl.CurrentTab = TabPer;
	    Gbl.CurrentAct = ActFrmChgPwd;
            return;
        }
     }

   /***** If I my login password is not enough secure, the only action possible is show a form to change my password *****/
   if (Gbl.CurrentAct == ActAutUsrInt)
     {
      if (!Usr_FastCheckIfPasswordSeemsGood (Gbl.Usrs.Me.LoginPlainPassword))
        {
         Gbl.CurrentTab = TabPer;
         Gbl.CurrentAct = ActFrmChgPwd;
         return;
        }
     }

   /***** If I have no gender, no FirstName, no Surname1 or no E_mail in database,
          or if I'm a teacher and I have no centre or department,
          the only action possible is show a form to change my card *****/
   if (Gbl.Usrs.Me.UsrDat.Gender == Usr_GENDER_UNKNOWN ||
       !Gbl.Usrs.Me.UsrDat.FirstName[0] ||
       !Gbl.Usrs.Me.UsrDat.Surname1[0] ||
       !Gbl.Usrs.Me.UsrDat.E_mail[0] ||
        Gbl.Usrs.Me.UsrDat.InsCod == -1L ||
       (Gbl.Usrs.Me.UsrDat.UsrTypeInDB >= Usr_TEACHER &&
        (Gbl.Usrs.Me.UsrDat.Tch.CtrCod == -1L ||
         Gbl.Usrs.Me.UsrDat.Tch.DptCod == -1L)))
     {
      switch (Gbl.CurrentAct)
        {
         case ActMnu: case ActLogOut: case ActFrmChgPwd: case ActReqEdiCrdCom:
            break;
         default:
	    Gbl.CurrentTab = TabPer;
	    Gbl.CurrentAct = ActReqEdiCrdCom;
            return;
        }
     }

   /***** If I belong to this course but I have not accepted the registration, the only action possible is show a form to ask for registration *****/
   if (Gbl.Usrs.Me.IBelongToThisCrs && !Gbl.Usrs.Me.UsrDat.Accepted)
     {
      switch (Gbl.CurrentAct)
        {
         case ActReqRemUsrCrs: case ActLogOut:
            break;
         default:
	    Gbl.CurrentTab = TabUsr;
            Gbl.CurrentAct = ActReqAccRegCrs;
            return;
        }
     }

   /***** If I am logged as student and I belong to current course... *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT && Gbl.Usrs.Me.IBelongToThisCrs)
     {
      switch (Gbl.CurrentAct)
        {
         case ActAutUsrInt: case ActMnu: case ActLogOut: case ActFrmChgPwd: case ActReqEdiCrdCom: case ActReqSelGrp:
         case ActSeeCrsTimTbl: case ActPrnCrsTimTbl: case ActSeeMyTimTbl: case ActPrnMyTimTbl:
            // These last actions are allowed in order to students could see/print timetable before register in groups
            break;
         default:
            /* If there are some group types with mandatory registration and groups in this course,
	       and I don't belong to any of those groups,
	       the the only action possible is show a form to register in groups */
            if (Grp_NumGrpTypesMandatIDontBelong ())
              {
	       Gbl.CurrentTab = TabUsr;
	       Gbl.CurrentAct = ActReqSelGrp;
	      }
            /* If I have no photo, and current action is not available for unknown users,
	       then update number of accesses without photo */
            else if (!Gbl.Usrs.Me.MyPhotoExists)
               if (!(Actions[Gbl.CurrentAct].PermisIfIBelongToCrs & (1 << Usr_UNKNOWN)) &&	// If current action is not available for unknown users...
                   Gbl.CurrentAct != ActReqMyPho)	// ...and current action is not ActReqMyPho...
                  if ((Gbl.Usrs.Me.NumAccWithoutPhoto = Usr_UpdateMyClicksWithoutPhoto ()) > Usr_MAX_CLICKS_WITHOUT_PHOTO)
	            {
	             /* If limit of accesses has been reached, 
		        the only action possible is show a form to send my photo */
	             Gbl.CurrentTab = TabPer;
	             Gbl.CurrentAct = ActReqMyPho;
	            }
        }
     }
  }
