// swad_brw.c: file browsers

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <dirent.h>		// For scandir, etc.
#include <errno.h>		// For errno
#include <stdlib.h>		// For exit, system, malloc, free, etc
#include <string.h>		// For string functions
#include <time.h>		// For time
#include <sys/types.h>		// For lstat, time_t
#include <sys/stat.h>		// For lstat
#include <unistd.h>		// For access, lstat, getpid, chdir, symlink

#include "swad_brw.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_ntf.h"
#include "swad_par.h"
#include "swad_str.h"

/*****************************************************************************/
/******************** Global variables from other modules ********************/
/*****************************************************************************/

extern struct Globals Gbl;

/*****************************************************************************/
/****************************** Internal types *******************************/
/*****************************************************************************/

// Related with file browser
typedef enum
  {
   Brw_IS_FILE,
   Brw_IS_FOLDER,
  } Brw_FileType_t;
typedef enum
  {
   EXPAND_TREE_NOTHING,
   EXPAND_TREE_PLUS,
   EXPAND_TREE_MINUS,
  } ExpandTree_t;
typedef enum
  {
   Brw_HEADER=0,
   Brw_FOOTER=1,
  } Brw_HeadOrFoot_t;

/*****************************************************************************/
/**************************** Internal constants *****************************/
/*****************************************************************************/

const char *HeadOrFootStr[2] =	// Names of columns in database, so don't change
 {
  "Header",
  "Footer",
 };

const char *Brw_RootFolderInternalNames[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   "",						// Brw_FILE_BRW_UNKNOWN
   Brw_INTERNAL_NAME_ROOT_FOLDER_DOWNLOAD,	// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   Brw_INTERNAL_NAME_ROOT_FOLDER_MARKS,		// Brw_FILE_BRW_SEE_MARKS_CRS
   Brw_INTERNAL_NAME_ROOT_FOLDER_DOWNLOAD,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   Brw_INTERNAL_NAME_ROOT_FOLDER_COMMON,	// Brw_FILE_BRW_COMMON_CRS
   Brw_INTERNAL_NAME_ROOT_FOLDER_COMMON,	// Brw_FILE_BRW_COMMON_GRP
   Brw_INTERNAL_NAME_ROOT_FOLDER_WORKS,		// Brw_FILE_BRW_WORKS_USR
   Brw_INTERNAL_NAME_ROOT_FOLDER_WORKS,		// Brw_FILE_BRW_WORKS_CRS
   Brw_INTERNAL_NAME_ROOT_FOLDER_MARKS,		// Brw_FILE_BRW_ADMIN_MARKS_CRS
   Brw_INTERNAL_NAME_ROOT_FOLDER_BRIEFCASE,	// Brw_FILE_BRW_BRIEFCASE_USR
   Brw_INTERNAL_NAME_ROOT_FOLDER_DOWNLOAD,	// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   Brw_INTERNAL_NAME_ROOT_FOLDER_DOWNLOAD,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   Brw_INTERNAL_NAME_ROOT_FOLDER_MARKS,		// Brw_FILE_BRW_SEE_MARKS_GRP
   Brw_INTERNAL_NAME_ROOT_FOLDER_MARKS,		// Brw_FILE_BRW_ADMIN_MARKS_GRP
   Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
// Number of columns of a file browser
const unsigned Brw_NumColumnsInExpTree[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   6,		// Brw_FILE_BRW_UNKNOWN
   6,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   6,		// Brw_FILE_BRW_SEE_MARKS_CRS
   6,		// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   8,		// Brw_FILE_BRW_COMMON_CRS
   8,		// Brw_FILE_BRW_COMMON_GRP
   6,		// Brw_FILE_BRW_WORKS_USR
   6,		// Brw_FILE_BRW_WORKS_CRS
   8,		// Brw_FILE_BRW_ADMIN_MARKS_CRS
   6,		// Brw_FILE_BRW_BRIEFCASE_USR
   6,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   6,		// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   6,		// Brw_FILE_BRW_SEE_MARKS_GRP
   6,		// Brw_FILE_BRW_ADMIN_MARKS_GRP
   9,		// Brw_FILE_BRW_ASSIGNMENTS_USR
   9,		// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const bool Brw_FileBrowserIsEditable[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   false,		// Brw_FILE_BRW_UNKNOWN
   false,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   false,		// Brw_FILE_BRW_SEE_MARKS_CRS
   true,		// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   true,		// Brw_FILE_BRW_COMMON_CRS
   true,		// Brw_FILE_BRW_COMMON_GRP
   true,		// Brw_FILE_BRW_WORKS_USR
   true,		// Brw_FILE_BRW_WORKS_CRS
   true,		// Brw_FILE_BRW_ADMIN_MARKS_CRS
   true,		// Brw_FILE_BRW_BRIEFCASE_USR
   false,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   true,		// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   false,		// Brw_FILE_BRW_SEE_MARKS_GRP
   true,		// Brw_FILE_BRW_ADMIN_MARKS_GRP
   true,		// Brw_FILE_BRW_ASSIGNMENTS_USR
   true,		// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActAskRemoveFile[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActAskRemFilDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActAskRemFilComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActAskRemFilComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActAskRemFilWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActAskRemFilWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActAskRemFilAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActAskRemFilBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActAskRemFilDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActAskRemFilAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActAskRemFilAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActAskRemFilAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActRemoveFile[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActRemFilDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActRemFilComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActRemFilComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActRemFilWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActRemFilWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActRemFilAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActRemFilBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActRemFilDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActRemFilAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActRemFilAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActRemFilAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActRemoveFolder[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActRemFolDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActRemFolComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActRemFolComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActRemFolWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActRemFolWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActRemFolAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActRemFolBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActRemFolDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActRemFolAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActRemFolAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActRemFolAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActRemoveFolderNotEmpty[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActRemTreDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActRemTreComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActRemTreComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActRemTreWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActRemTreWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActRemTreAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActRemTreBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActRemTreDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActRemTreAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActRemTreAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActRemTreAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActCopy[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActCopDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActCopComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActCopComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActCopWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActCopWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActCopAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActCopBrf,		// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActCopDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActCopAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActCopAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActCopAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActPaste[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActPasDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActPasComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActPasComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActPasWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActPasWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActPasAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActPasBrf,		// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActPasDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActPasAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActPasAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActPasAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActFormCreate[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActFrmCreDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActFrmCreComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActFrmCreComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActFrmCreWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActFrmCreWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActFrmCreAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActFrmCreBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActFrmCreDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActFrmCreAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActFrmCreAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActFrmCreAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActCreateFolder[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActRecFolDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActRecFolComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActRecFolComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActRecFolWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActRecFolWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActRecFolAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActRecFolBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActRecFolDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActRecFolAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActRecFolAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActRecFolAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActCreateFile[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActRecFilDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActRecFilComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActRecFilComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActRecFilWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActRecFilWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActRecFilAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActRecFilBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActRecFilDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActRecFilAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActRecFilAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActRecFilAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActRenameFolder[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_CRS
   ActRenFolDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActRenFolComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActRenFolComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActRenFolWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActRenFolWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActRenFolAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActRenFolBrf,	// Brw_FILE_BRW_BRIEFCASE_USR
   ActUnk,		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActRenFolDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActUnk,		// Brw_FILE_BRW_SEE_MARKS_GRP
   ActRenFolAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActRenFolAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActRenFolAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActExpandFolder[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActExpSeeDowCrs,	// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActExpSeeAssCrs,	// Brw_FILE_BRW_SEE_MARKS_CRS
   ActExpAdmDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActExpComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActExpComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActExpWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActExpWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActExpAdmAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActExpBrf,		// Brw_FILE_BRW_BRIEFCASE_USR
   ActExpSeeDowGrp,	// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActExpAdmDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActExpSeeAssGrp,	// Brw_FILE_BRW_SEE_MARKS_GRP
   ActExpAdmAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActExpAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActExpAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
const Act_Action_t Brw_ActContractFolder[Brw_NUM_TYPES_FILE_BROWSER] =
  {
   ActUnk,		// Brw_FILE_BRW_UNKNOWN
   ActConSeeDowCrs,	// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
   ActConSeeAssCrs,	// Brw_FILE_BRW_SEE_MARKS_CRS
   ActConAdmDowCrs,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
   ActConComCrs,	// Brw_FILE_BRW_COMMON_CRS
   ActConComGrp,	// Brw_FILE_BRW_COMMON_GRP
   ActConWrkUsr,	// Brw_FILE_BRW_WORKS_USR
   ActConWrkCrs,	// Brw_FILE_BRW_WORKS_CRS
   ActConAdmAssCrs,	// Brw_FILE_BRW_ADMIN_MARKS_CRS
   ActConBrf,		// Brw_FILE_BRW_BRIEFCASE_USR
   ActConSeeDowGrp,	// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
   ActConAdmDowGrp,	// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
   ActConSeeAssGrp,	// Brw_FILE_BRW_SEE_MARKS_GRP
   ActConAdmAssGrp,	// Brw_FILE_BRW_ADMIN_MARKS_GRP
   ActConAsgUsr,	// Brw_FILE_BRW_ASSIGNMENTS_USR
   ActConAsgCrs,	// Brw_FILE_BRW_ASSIGNMENTS_CRS
  };
/* All quotas must be multiple of 1 GiB */
/* Maximum quotas for each type of file browser */
#define Brw_MAX_QUOTA_DOWNLOAD_CRS  (64ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_DOWNLOAD_CRS    5000
#define Brw_MAX_FOLDERS_DOWNLOAD_CRS  1000

#define Brw_MAX_QUOTA_DOWNLOAD_GRP  (1ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_DOWNLOAD_GRP    1000
#define Brw_MAX_FOLDERS_DOWNLOAD_GRP   500

#define Brw_MAX_QUOTA_COMMON_CRS (64ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_COMMON_CRS      5000
#define Brw_MAX_FOLDERS_COMMON_CRS    1000	// Many, because every student can create his own directories

#define Brw_MAX_QUOTA_COMMON_GRP (1ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_COMMON_GRP      1000
#define Brw_MAX_FOLDERS_COMMON_GRP     500	// Many, because every student can create his own directories

#define Brw_MAX_QUOTA_ASSIGNMENTS_PER_STD (2ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_ASSIGNMENTS_PER_STD    500
#define Brw_MAX_FOLDERS_ASSIGNMENTS_PER_STD   50

#define Brw_MAX_QUOTA_WORKS_PER_STD (2ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_WORKS_PER_STD    500
#define Brw_MAX_FOLDERS_WORKS_PER_STD   50

#define Brw_MAX_QUOTA_GRADES_CRS (1ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_GRADES_CRS       500
#define Brw_MAX_FOLDERS_GRADES_CRS      50

#define Brw_MAX_QUOTA_GRADES_GRP (1ULL*1024ULL*1024ULL*1024ULL)
#define Brw_MAX_FILES_GRADES_GRP       200
#define Brw_MAX_FOLDERS_GRADES_GRP      20

const unsigned long long int Brw_MAX_QUOTAS_BRIEFCASE[Usr_NUM_USR_TYPES] =
  {                         0ULL, // Usr_UNKNOWN
                            0ULL, // Usr_INVITED
   16ULL*1024ULL*1024ULL*1024ULL, // Usr_STUDENT
   64ULL*1024ULL*1024ULL*1024ULL, // Usr_TEACHER
                            0ULL, // Usr_DEG_ADMIN	(there shouldn't be briefcases for administrators!)
                            0ULL};// Usr_SUPERUSER	(there shouldn't be briefcases for administrators!)
#define Brw_MAX_FILES_BRIEFCASE         5000
#define Brw_MAX_FOLDERS_BRIEFCASE        500

/* Extensions allowed for uploaded files */
const char *Brw_FileExtensionsAllowed[] =
  {"asm"  ,"avi"  ,"bas"  ,"bat"  ,"bbl"  ,"bib"  ,"bin"  ,"bmp"  ,"bz2"  ,
   "c"    ,"cc"   ,"cdr"  ,"com"  ,"cpp"  ,"css"  ,"csv"  ,"doc"  ,"docx" ,
   "eps"  ,"exe"  ,"fdf"  ,"flv"  ,"gif"  ,"gz"   ,"h"    ,"hex"  ,"htm"  ,
   "html" ,"img"  ,"jpg"  ,"jpeg" ,"latex","m"    ,"mdb"  ,"mht"  ,"mhtml",
   "mid"  ,"mov"  ,"mp3"  ,"mpeg" ,"mpg"  ,"mpp"  ,"mus"  ,"nb"   ,"odb"  ,
   "odc"  ,"odf"  ,"odg"  ,"odi"  ,"odm"  ,"odp"  ,"ods"  ,"odt"  ,"otg"  ,
   "otp"  ,"ots"  ,"ott"  ,"pas"  ,"pdf"  ,"png"  ,"pps"  ,"ppsx" ,"ppt"  ,
   "pptx" ,"ps"   ,"pss"  ,"qt"   ,"r"    ,"ram"  ,"rar"  ,"raw"  ,"rdata",
   "rm"   ,"rtf"  ,"s"    ,"sav"  ,"sbs"  ,"sf3"  ,"sgp"  ,"spp"  ,"spo"  ,
   "sps"  ,"swf"  ,"tar"  ,"tex"  ,"tgz"  ,"tif"  ,"txt"  ,"voc"  ,"wav"  ,
   "wma"  ,"wmv"  ,"xls"  ,"xlsx" ,"zip"  };
const int Brw_NUM_FILE_EXT_ALLOWED = sizeof (Brw_FileExtensionsAllowed) / sizeof (Brw_FileExtensionsAllowed[0]);

/* MIME types allowed for uploades files */
const char *Brw_MIMETypesAllowed[] =
  {
   "application/acrobat",		//  PDF
   "application/arj",			//  compressed archive arj
   "application/binary",		//  
   "application/bzip2",			//  Bzip 2 UNIX Compressed File
   "application/cdr",			//  Corel Draw (CDR)
   "application/coreldraw",		//  Corel Draw (CDR)
   "application/css-stylesheet",	//  Hypertext Cascading Style Sheet
   "application/csv",			//  CSV, Comma Separated Values
   "application/download",		//  zip files in Firefox caused by an error?
   "application/excel",			//  Microsoft Excel xls
   "application/finale",		//  Finale .mus
   "application/force-download",	//  RAR uploaded from Firefox
   "application/futuresplash",		//  Flash
   "application/gzip",			//  GNU ZIP gz, gzip
   "application/gzip-compressed",	//  GNU ZIP gz, gzip
   "application/gzipped",		//  GNU ZIP gz, gzip
   "application/msaccess",		//  Microsoft Access mdb
   "application/msexcel",		//  Microsoft Excel xla, xls, xlt, xlw
   "application/mspowerpoint",		//  Microsoft PowerPoint pot, pps, ppt
   "application/mathematica",		//  Mathematica
   "application/matlab",		//  Matlab
   "application/mfile",			//  Matlab
   "application/mpp",			//  Microsoft Project mpp
   "application/msproj",		//  Microsoft Project mpp
   "application/msproject",		//  Microsoft Project mpp
   "application/msword",		//  Microsoft Word doc, word, w6w
   "application/mswrite",		//  Microsoft Write wri
   "application/octet-stream",		//  uninterpreted binary bin
   "application/pdf",			//  Adobe Acrobat pdf
   "application/postscript",		//  PostScript ai, eps, ps
   "application/powerpoint",		//  Microsoft PowerPoint pot, pps, ppt
   "application/rar",			//  RAR
   "application/rtf",			//  RTF
   "application/self-extracting",	//  Compressed file, self-extracting
   "application/vnd.fdf",		//  Forms Data Format
   "application/vnd.msexcel",		//  Microsoft Excel xls
   "application/vnd.ms-excel",		//  Microsoft Excel xls
   "application/vnd.ms-powerpoint",	//  Microsoft PowerPoint ppt or pps
   "application/vnd.ms-project",	//  Microsoft Project mpp
   "application/vnd.ms-word",		//  Microsoft Word doc
   "application/vnd.oasis.opendocument.text",			// OpenOffice Text 			.odt
   "application/vnd.oasis.opendocument.spreadsheet",		// OpenOffice Hoja of clculo 		.ods
   "application/vnd.oasis.opendocument.presentation",		// OpenOffice Presentacin 		.odp
   "application/vnd.oasis.opendocument.graphics",		// OpenOffice Dibujo 			.odg
   "application/vnd.oasis.opendocument.chart",			// OpenOffice Grfica 			.odc
   "application/vnd.oasis.opendocument.formula",		// OpenOffice Frmula matemtica 	.odf
   "application/vnd.oasis.opendocument.database",		// OpenOffice database 			.odb
   "application/vnd.oasis.opendocument.image",			// OpenOffice Imagen 			.odi
   "application/vnd.oasis.opendocument.text-master",		// OpenOffice Documento maestro 	.odm
   "application/vnd.oasis.opendocument.text-template",		// OpenOffice Text 			.ott
   "application/vnd.oasis.opendocument.spreadsheet-template",	// OpenOffice Hoja of clculo 		.ots
   "application/vnd.oasis.opendocument.presentation-template",	// OpenOffice Presentacin 		.otp
   "application/vnd.oasis.opendocument.graphics-template",	// OpenOffice Dibujo 			.otg
   "application/vnd.openxmlformats-officedocument.presentationml.presentation",	//  Power Point Microsoft Office Open XML Format Presentation Slide Show .pptx
   "application/vnd.openxmlformats-officedocument.presentationml.slideshow",	//  Power Point Microsoft Office Open XML Format Presentation Slide Show .ppsx
   "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",		//  Excel Microsoft Office Open XML Format Spreadsheet .xlsx
   "application/vnd.openxmlformats-officedocument.wordprocessingml.document",	//  Word Microsoft Office Open XML Format Document .docx
   "application/vnd.pdf",		//  PDF
   "application/x-bz2",			//  Bzip 2 UNIX Compressed File
   "application/x-bzip",		//  Bzip 2 UNIX Compressed File
   "application/x-cdr",			//  Corel Draw (CDR)
   "application/x-compress",		//  GNU ZIP gz, gzip
   "application/x-compressed",		//  GNU ZIP gz, gzip, Bzip 2 UNIX Compressed File
   "application/x-compressed-tar",	//  TGZ
   "application/x-coreldraw",		//  Corel Draw (CDR)
   "application/x-dos_ms_project",	//  Microsoft Project mpp
   "application/x-download",		//  RAR
   "application/x-forcedownload",	//  PDF
   "application/x-gtar",		//  GNU tar gtar
   "application/x-gunzip",		//  GNU ZIP gz, gzip
   "application/x-gzip",		//  GNU ZIP gz, gzip
   "application/x-latex",	   	//  LateX latex (LateX)
   "application/x-midi",		//  MIDI mid
   "application/x-msdos-program",	//  MSDOS program
   "application/x-msdownload",		//  dll, exe
   "application/x-mspowerpoint",	//  Microsoft PowerPoint pot, pps, ppt
   "application/x-msproject",		//  Microsoft Project mpp
   "application/x-ms-project",		//  Microsoft Project mpp
   "application/x-shockwave-flash",		// Flash
   "application/x-shockwave-flash2-preview",	// Flash
   "application/x-unknown",		//  Unknown file type
   "application/x-vnd.oasis.opendocument.chart",		// OpenOffice Grfica 			.odc
   "application/x-vnd.oasis.opendocument.database",		// OpenOffice database 			.odb
   "application/x-vnd.oasis.opendocument.formula",		// OpenOffice Frmula matemtica 	.odf
   "application/x-vnd.oasis.opendocument.graphics",		// OpenOffice Dibujo 			.odg
   "application/x-vnd.oasis.opendocument.graphics-template",	// OpenOffice Dibujo 			.otg
   "application/x-vnd.oasis.opendocument.image",		// OpenOffice Imagen 			.odi
   "application/x-vnd.oasis.opendocument.presentation",		// OpenOffice Presentacin 		.odp
   "application/x-vnd.oasis.opendocument.presentation-template",// OpenOffice Presentacin 		.otp
   "application/x-vnd.oasis.opendocument.spreadsheet",		// OpenOffice Hoja of clculo 		.ods
   "application/x-vnd.oasis.opendocument.spreadsheet-template",	// OpenOffice Hoja of clculo 		.ots
   "application/x-vnd.oasis.opendocument.text",			// OpenOffice Text 			.odt
   "application/x-vnd.oasis.opendocument.text-master",		// OpenOffice Documento maestro 	.odm
   "application/x-vnd.oasis.opendocument.text-template",	// OpenOffice Text			.ott
   "application/x-pdf",			//  PDF
   "application/x-shockwave-flash",	//  Macromedia Shockwave swf
   "application/x-spss",		//  SPSS File sav spp sbs sps spo
   "application/x-rar-compressed",	//  RAR archive rar
   "application/x-tar",			//  4.3BSD tar format tar
   "application/x-tex",			//  TeX tex (LateX)
   "application/x-tgz",			//  TGZ
   "application/x-zip",			//  ZIP archive zip
   "application/x-zip-compressed",	//  ZIP archive zip
   "application/zip",			//  ZIP archive zip
   "audio/basic",			//  BASIC audio (u-law) au, snd
   "audio/mpeg",			//  MP3
   "audio/midi",			//  MIDI mid, midi
   "audio/x-aiff",			//  AIFF audio aif, aifc, aiff
   "audio/x-mpeg",			//  MPEG audio mp3
   "audio/x-ms-wma",			//  WMA (Windows Media Audio File)
   "audio/x-pn-realaudio",		//  RealAudio ra, ram
   "audio/x-pn-realaudio-plugin",	//  RealAudio plug-in rpm
   "audio/x-voice",			//  Voice voc
   "audio/x-wav",			//  Microsoft Windows WAVE audio wav
   "binary/octet-stream",		//  uninterpreted binary bin
   "file/unknown",			//  Some bowsers send this (?)
   "gzip/document",			//  GNU ZIP gz, gzip
   "image/bmp",				//  Bitmap bmp
   "image/cdr",				//  Corel Draw (CDR)
   "image/gif",				//  GIF image gif
   "image/jpeg",			//  JPEG image jpe, jpeg, jpg
   "image/pjpeg",			//  JPEG image jpe, jpeg, jpg
   "image/pict",			//  Macintosh PICT pict
   "image/png",				//  Portable Network Graphic png
   "image/tiff",			//  TIFF image tif, tiff
   "image/vnd.rn-realflash",		//  Flash
   "image/x-cdr",			//  Corel Draw (CDR)
   "image/x-cmu-raster",		//  CMU raster ras
   "image/x-eps",			//  Imagen postcript
   "image/x-png",			//  Portable Network Graphic png
   "image/x-portable-anymap",		//  PBM Anymap format pnm
   "image/x-portable-bitmap",		//  PBM Bitmap format pbm
   "image/x-portable-graymap",		//  PBM Graymap format pgm
   "image/x-portable-pixmap",		//  PBM Pixmap format ppm
   "image/x-rgb",			//  RGB image rgb
   "image/x-xbitmap",			//  X Bitmap xbm
   "image/x-xpixmap",			//  X Pixmap xpm
   "image/x-xwindowdump",		//  X BrowserWindow System dump xwd
   "message/rfc822",			//  Files .mht and .mhtml
   "multipart/x-gzip",			//  GNU ZIP archive gzip
   "multipart/x-zip",			//  PKZIP archive zip
   "octet/pdf",				//  PDF
   "text/anytext",			//  CSV, Comma Separated Values?
   "text/comma-separated-values",	//  CSV, Comma Separated Values
   "text/css",				//  Hypertext Cascading Style Sheet
   "text/csv",				//  CSV, Comma Separated Values
   "text/html",				//  HTML htm, html, php
   "text/pdf",				//  PDF
   "text/plain",			//  plain text C, cc, h, txt. BAS
   "text/richtext",			//  RTF
   "text/x-chdr",			//  Cdigo fuente in C
   "text/x-csrc",			//  Cdigo fuente in C
   "text/x-c++src",			//  Cdigo fuente in C++
   "text/x-latex",			//  LateX
   "text/x-pdf",			//  PDF
   "video/avi",				//  AVI
   "video/mpeg",			//  MPEG video mpe, mpeg, mpg
   "video/msvideo",			//  Microsoft Windows video avi
   "video/quicktime",			//  QuickTime video mov, qt
   "video/x-ms-asf",			//  WMA (Windows Media Audio File)
   "video/x-ms-wmv",			//  WMV (Windows Media File)
   "x-world/x-vrml",			//  VRML Worlds wrl
   "zz-application/zz-winassoc-cdr",	//  Corel Draw (CDR)
   "zz-application/zz-winassoc-mpp"	//  Microsoft Project mpp
  };

const int Brw_NUM_MIME_TYPES_ALLOWED = sizeof (Brw_MIMETypesAllowed) / sizeof (Brw_MIMETypesAllowed[0]);

/*****************************************************************************/
/*************************** Internal prototypes *****************************/
/*****************************************************************************/

static void Brw_GetParAndInitFileBrowser (void);
static void Brw_GetGrpSettings (void);
static Brw_FileBrowser_t Brw_GetParFileBrowserType (void);
static void Brw_GetParamsPathInTreeAndFileName (void);
static void Brw_InitializeFileBrowser (void);
static void Brw_SetPathFileBrowser (void);
static void Brw_CreateTopFoldersFileBrowserIfNotExist (void);
static void Brw_CreateFoldersAssignmentsIfNotExist (void);
static void Brw_SetAndCheckQuota (void);
static void Brw_SetMaxQuota (void);
static bool Brw_CheckIfQuotaExceded (void);

static void Brw_ShowFileBrowserNormal (void);
static void Brw_ShowFileBrowsersAsgWrkCrs (void);
static void Brw_ShowFileBrowsersAsgWrkUsr (void);

static void Brw_FormToChangeZone (void);
static void Brw_GetSelectedGroupData (struct GroupData *GrpDat,bool AbortOnError);
static void Brw_ShowDataOwnerAsgWrk (struct UsrData *UsrDat);
static void Brw_ShowAgainFileBrowserOrWorks (void);
static void Brw_ShowFileBrowser (void);
static void Brw_WriteTopBeforeShowingFileBrowser (void);
static void Brw_UpdateLastAccess (void);
static void Brw_WriteTitleOfFileBrowser (void);
static void Brw_InitHiddenLevels (void);
static void Brw_ShowSizeOfFileTree (void);
static void Brw_StoreSizeOfFileTreeInDB (void);
static void Brw_WriteFormFullTree (void);
static bool Brw_GetFullTreeFromForm (void);
static bool Brw_GetCreateZIPFromForm (void);
static void Brw_CreateDirDownloadTmp (void);
static void Brw_CreateTmpDirForCompression (void);
static void Brw_CreateDirCompressionUsr (void);
static void Brw_CreateZIP (void);
static void Brw_GetAndUpdateDateLastAccFileBrowser (const char *FieldNameDB);
static long Brw_GetGrpLastAccZone (const char *FieldNameDB);
static void Brw_UpdateGrpLastAccZone (const char *FieldNameDB,long GrpCod);
static void Brw_ResetFileBrowserSize (void);
static void Brw_CalcSizeOfDirRecursive (unsigned Level,char *PathRel);
static void Brw_ListDir (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS],const char *PathRel,const char *PathInTree);
static bool Brw_WriteRowFileBrowser (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS],Brw_FileType_t IsFileOrFolder,ExpandTree_t ExpandTree,const char *PathInTree,const char *FileName,off_t FileSize,time_t TimeLastModif);
static void Brw_ConstructCompletePathInTree (const char *PathInTreeUntilFileOrFolder,const char *FileOrFolderName);
static void Brw_PutIconsRemoveCopyPaste (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName);
static bool Brw_CheckIfCanPasteIn (unsigned Level);
static void Brw_PutIconRemoveFile (unsigned Level,const char *PathInTree,const char *FileName);
static void Brw_PutIconRemoveDir (unsigned Level,const char *PathInTree,const char *FileName);
static void Brw_PutIconCopy (unsigned Level,const char *PathInTree,const char *FileName);
static void Brw_PutIconPasteOn (unsigned Level,const char *PathInTree,const char *FileName);
static void Brw_PutIconPasteOff (void);
static void Brw_IndentAndWriteIconExpandContract (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS],ExpandTree_t ExpandTree,const char *PathInTree,const char *FileName);
static void Brw_IndentDependingOnLevel (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS]);
static void Brw_PutIconShow (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName);
static void Brw_PutIconHide (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName);
static bool Brw_CheckIfAnyUpperLevelIsHidden (unsigned CurrentLevel);
static void Brw_PutIconFolder (unsigned Level,const char *PathInTree,const char *FileName,ExpandTree_t ExpandTree);
static void Brw_PutIconNewFileOrFolder (void);
static void Brw_PutIconFile (const char *FileName);
static void Brw_WriteFileName (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName,bool IsRecent);
static const char *Brw_GetFileNameToShow (Brw_FileBrowser_t FileBrowser,unsigned Level,const char *FileName);
static void Brw_CopyStrChangingSpaces (const char *StringWithSpaces,char *StringWithoutSpaces,unsigned MaxLength);
static void Brw_CreateTmpLinkFileBrowser (const char *PathInTree,const char *FileName);
static void Brw_CreatePublicDirForDownload (const char *PathInTree,const char *FileName);
static void Brw_ParamListFiles (const char *PathInTree,const char *FileName);
static void Brw_ParamSeeDownloadZoneCrs (void);
static void Brw_ParamSeeDownloadZoneGrp (void);
static void Brw_ParamAdminDownloadZoneCrs (void);
static void Brw_ParamAdminDownloadZoneGrp (void);
static void Brw_ParamAdminCommonZoneCrs (void);
static void Brw_ParamAdminCommonZoneGrp (void);
static void Brw_GetNumRowsHeaderAndFooter (unsigned *Header,unsigned *Footer);
static void Brw_AddMarksToDB (unsigned Header,unsigned Footer);
static void Brw_RemoveOneGradesFileFromDB (long GrpCod,const char *Path);
static void Brw_RemoveChildrenOfMarksFolderFromDB (long GrpCod,const char *Path);
static void Brw_RenameChildrenMarksFileInDB (long GrpCod,const char *OldPath,const char *NewPath);
static void Brw_ChangeNumRowsHeaderOrFooter (Brw_HeadOrFoot_t HeaderOrFooter);
static void Brw_WriteDatesAssignment (unsigned Level);
static void Brw_GetAndWriteNumRowsHeaderAndFooter (Brw_FileType_t IsFileOrFolder,long GrpCod,const char *PathInTree,const char *FileName,bool IsRecent);
static void Brw_WriteFileSizeAndDate (Brw_FileType_t IsFileOrFolder,off_t FileSize,time_t TimeLastModif,bool IsRecent);
static void Brw_WriteFileOrFolderCreator (unsigned Level,const char *Path,bool IsRecent);
static void Brw_AskConfirmRemoveFolderNotEmpty (void);
static inline void Brw_GetAndWriteClipboard (void);
static void Brw_WriteCurrentClipboard (void);
static bool Brw_GetMyClipboard (void);
static bool Brw_CheckIfClipboardIsInThisTree (void);
static void Brw_AddPathToClipboards (const char *Path);
static void Brw_UpdatePathInClipboard (const char *Path);
static void Brw_InsThisFolderAndUpdOtherFoldersInExpandedFolders (const char *Path);
static void Brw_RemThisFolderAndUpdOtherFoldersFromExpandedFolders (const char *Path);
static void Brw_InsertFolderInExpandedFolders (const char *Path);
static void Brw_UpdateClickTimeOfThisFileBrowserInExpandedFolders (void);
static void Brw_RemoveFolderFromExpandedFolders (const char *Path);
static void Brw_RemoveExpiredClipboards (void);
static void Brw_RemoveExpiredExpandedFolders (void);
static void Brw_RemoveAffectedClipboards (Brw_FileBrowser_t FileBrowser,long MyUsrCod,long WorksUsrCod);
static void Brw_RemoveAffectedExpandedFolders (const char *Path);
static void Brw_RenameAffectedExpandedFolders (Brw_FileBrowser_t FileBrowser,long MyUsrCod,long WorksUsrCod,const char *OldPath,const char *NewPath);
static void Brw_PasteClipboard (void);
static unsigned Brw_NumLevelsInPath (const char *Path);
static bool Brw_PasteTreeIntoFolder (const char *PathOrg,const char *PathDstEnTree,unsigned *NumFilesPasted,unsigned *NumFoldsPasted);
static bool Brw_GetIfHiddenDownload (long GrpCod,const char *Path);
static bool Brw_GetIfExpandedTree (const char *Path);
static void Brw_AddHiddenDownloadToDB (long GrpCod,const char *Path);
static void Brw_RemoveOneHiddenDownloadFromDB (long GrpCod,const char *Path);
static void Brw_RemoveChildrenOfHiddenDownloadFromDB (long GrpCod,const char *Path);
static void Brw_RenameOneHiddenDownloadFolderInDB (long GrpCod,const char *OldPath,const char *NewPath);
static void Brw_RenameChildrenHiddenDownloadsInDB (long GrpCod,const char *OldPath,const char *NewPath);
static void Brw_AddPathCommonToDB (long GrpCod,const char *Path);
static long Brw_GetCreatorFileOrFolderCommon (long GrpCod,const char *Path);
static void Brw_RemoveOneFileOrFolderCommonFromDB (long GrpCod,const char *Path);
static void Brw_RemoveChildrenOfFolderCommonFromDB (long GrpCod,const char *Path);
static void Brw_RenameOneFolderCommonInDB (long GrpCod,const char *OldPath,const char *NewPath);
static void Brw_RenameChildrenFilesOrFoldersCommonInDB (long GrpCod,const char *OldPath,const char *NewPath);
static bool Brw_CheckIfICanModifyFileOrFolder (unsigned Level);
static bool Brw_CheckIfICanCreateIntoFolder (unsigned Level);
static bool Brw_CheckIfIHavePermissionFileOrFolderCommon (long GrpCod);

/****************************************************************************/
/***************** Get parameters related to file browser *******************/
/****************************************************************************/

static void Brw_GetParAndInitFileBrowser (void)
  {
   /***** Get group code. If a group is selected, get its data *****/
   Brw_GetGrpSettings ();

   /***** Get type of file browser *****/
   switch (Gbl.CurrentAct)
     {
      case ActAskSeeDow:	// Access to a download zone from menu
         /* Set file browser type acording to last group accessed */
         Gbl.FileBrowser.Type = (Gbl.CurrentCrs.Grps.GrpCod >= 0) ? Brw_FILE_BRW_SEE_DOWNLOAD_GRP : Brw_FILE_BRW_SEE_DOWNLOAD_CRS;
         break;
      case ActAskAdmDow:	// Access to a download zone from menu
         /* Set file browser type acording to last group accessed */
         Gbl.FileBrowser.Type = (Gbl.CurrentCrs.Grps.GrpCod >= 0) ? Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP : Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS;
         break;
      case ActAskAdmCom:	// Access to a common zone from menu
         /* Set file browser type acording to last group accessed */
         Gbl.FileBrowser.Type = (Gbl.CurrentCrs.Grps.GrpCod >= 0) ? Brw_FILE_BRW_COMMON_GRP : Brw_FILE_BRW_COMMON_CRS;
         break;
      case ActAskSeeAss:	// Access to a download zone from menu
         /* Set file browser type acording to last group accessed */
         Gbl.FileBrowser.Type = (Gbl.CurrentCrs.Grps.GrpCod >= 0) ? Brw_FILE_BRW_SEE_MARKS_GRP : Brw_FILE_BRW_SEE_MARKS_CRS;
         break;
      case ActAskAdmAss:	// Access to a download zone from menu
         /* Set file browser type acording to last group accessed */
         Gbl.FileBrowser.Type = (Gbl.CurrentCrs.Grps.GrpCod >= 0) ? Brw_FILE_BRW_ADMIN_MARKS_GRP : Brw_FILE_BRW_ADMIN_MARKS_CRS;
         break;
      default:
         if ((Gbl.FileBrowser.Type = Brw_GetParFileBrowserType ()) == Brw_FILE_BRW_UNKNOWN)
            Lay_ShowErrorAndExit ("The type of file browser is wrong.");
     }

   /***** Get the path in the file browser and the name of the file or folder *****/
   Brw_GetParamsPathInTreeAndFileName ();
   Brw_ConstructCompletePathInTree (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);

   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         /* Get list of DNI of the selected users */
         Usr_GetListSelectedUsrs ();
         /* Get user whose folder will be used to make any operation */
         Usr_GetParamOtherUsrCodEncrypted ();
         /* Get if we must create the zip file */
         Gbl.FileBrowser.ZIP.CreateZIP = Brw_GetCreateZIPFromForm ();
         break;
      default:
         break;
     }

   if (Gbl.CurrentAct == ActRecFolDowCrs	|| Gbl.CurrentAct == ActRenFolDowCrs	||
       Gbl.CurrentAct == ActRecFolDowGrp	|| Gbl.CurrentAct == ActRenFolDowGrp	||
       Gbl.CurrentAct == ActRecFolComCrs	|| Gbl.CurrentAct == ActRenFolComCrs	||
       Gbl.CurrentAct == ActRecFolComGrp	|| Gbl.CurrentAct == ActRenFolComGrp	||
       Gbl.CurrentAct == ActRecFolAssCrs	|| Gbl.CurrentAct == ActRenFolAssCrs	||
       Gbl.CurrentAct == ActRecFolAssGrp	|| Gbl.CurrentAct == ActRenFolAssGrp	||
       Gbl.CurrentAct == ActRecFolAsgCrs	|| Gbl.CurrentAct == ActRenFolAsgCrs	||
       Gbl.CurrentAct == ActRecFolWrkCrs	|| Gbl.CurrentAct == ActRenFolWrkCrs	||
       Gbl.CurrentAct == ActRecFolAsgUsr	|| Gbl.CurrentAct == ActRenFolAsgUsr	||
       Gbl.CurrentAct == ActRecFolWrkUsr	|| Gbl.CurrentAct == ActRenFolWrkUsr	||
       Gbl.CurrentAct == ActRecFolBrf		|| Gbl.CurrentAct == ActRenFolBrf)
     {
      /* Get the name of the new folder */
      Par_GetParam (Par_PARAM_SINGLE,"NewFolderName",Gbl.FileBrowser.NewFileOrFolderName,NAME_MAX);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.FileBrowser.NewFileOrFolderName,NAME_MAX,true);	// Remove leading and trailing spaces from the name of the new folder
     }

   /***** Get whether to show full tree *****/
   Gbl.FileBrowser.FullTree = Brw_GetFullTreeFromForm ();

   /***** Initialize file browser *****/
   Brw_InitializeFileBrowser ();
  }

/****************************************************************************/
/********** Get group code. If a group is selected, get its data ************/
/****************************************************************************/

static void Brw_GetGrpSettings (void)
  {
   struct GroupData GrpDat;

   switch (Gbl.CurrentAct)
     {
      case ActAskSeeDow:
      case ActAskAdmDow:	// Access to a download zone from menu
         GrpDat.GrpCod = Brw_GetGrpLastAccZone ("LastDowGrpCod");
         Brw_GetSelectedGroupData (&GrpDat,false);	// Don't abort on error
         Gbl.CurrentCrs.Grps.GrpCod = GrpDat.GrpCod;
         break;
      case ActAskAdmCom:	// Access to a common zone from menu
         GrpDat.GrpCod = Brw_GetGrpLastAccZone ("LastComGrpCod");
         Brw_GetSelectedGroupData (&GrpDat,false);	// Don't abort on error
         Gbl.CurrentCrs.Grps.GrpCod = GrpDat.GrpCod;
         break;
      case ActAskSeeAss:
      case ActAskAdmAss:	// Access to a marks zone from menu
         GrpDat.GrpCod = Brw_GetGrpLastAccZone ("LastAssGrpCod");
         Brw_GetSelectedGroupData (&GrpDat,false);	// Don't abort on error
         Gbl.CurrentCrs.Grps.GrpCod = GrpDat.GrpCod;
         break;
      case ActReqAsgWrkCrs:	// Don't get parameter with group code because this parameter is multiple
      case ActEdiAsgWrkCrs:
         Gbl.CurrentCrs.Grps.GrpCod = -1L;
         break;
      default:
         GrpDat.GrpCod = Grp_GetParamGrpCod ();
         Brw_GetSelectedGroupData (&GrpDat,true);	// Abort on error
         Gbl.CurrentCrs.Grps.GrpCod = GrpDat.GrpCod;
         break;
     }
   if (Gbl.CurrentCrs.Grps.GrpCod >= 0)
     {
      Gbl.CurrentCrs.Grps.GrpTyp.GrpTypCod = GrpDat.GrpTypCod;
      strcpy (Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,GrpDat.GrpTypName);
      strcpy (Gbl.CurrentCrs.Grps.GrpName,GrpDat.GrpName);
      Gbl.CurrentCrs.Grps.MaxStudents = GrpDat.MaxStudents;
      Gbl.CurrentCrs.Grps.Open = GrpDat.Open;
      Gbl.CurrentCrs.Grps.FileZones = GrpDat.FileZones;
      Gbl.CurrentCrs.Grps.GrpTyp.MultipleRegistration = GrpDat.MultipleRegistration;
     }
  }

/****************************************************************************/
/******************* Get parameter with file browser type *******************/
/****************************************************************************/

static Brw_FileBrowser_t Brw_GetParFileBrowserType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   /* Get file browser type from form */
   Par_GetParam (Par_PARAM_SINGLE,"BrwTyp",UnsignedStr,10);

   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      return Brw_FILE_BRW_UNKNOWN;

   if (UnsignedNum < Brw_NUM_TYPES_FILE_BROWSER)
      return (Brw_FileBrowser_t) UnsignedNum;

   return Brw_FILE_BRW_UNKNOWN;
  }

/*****************************************************************************/
/** Put hidden params. with the path in the tree and the name of file/folder */
/*****************************************************************************/

void Brw_PutParamsPathAndFile (const char *PathInTree,const char *FileFolderName)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Path\" value=\"%s\" />",PathInTree);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FilFol\" value=\"%s\" />",FileFolderName);
  }

/****************************************************************************/
/************** Get parameters path and file in file browser ****************/
/****************************************************************************/

static void Brw_GetParamsPathInTreeAndFileName (void)
  {
   const char *Ptr;
   unsigned i;

   /***** Get the path inside the tree (this path doesn't include the name of the file or folder at the end) *****/
   Par_GetParam (Par_PARAM_SINGLE,"Path",Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,PATH_MAX);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,PATH_MAX,true);

   /***** Get the name of the file or folder *****/
   Par_GetParam (Par_PARAM_SINGLE,"FilFol",Gbl.FileBrowser.FileOrFolderName,NAME_MAX);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.FileBrowser.FileOrFolderName,NAME_MAX,false);   // File names with heading and trailing spaces are allowed

   /***** Set level of this file or folder inside file browser *****/
   if (!strcmp (Gbl.FileBrowser.FileOrFolderName,"."))
      Gbl.FileBrowser.Level = 0;
   else
     {
      // Level == number-of-slashes-in-path-except-file-or-folder + 1
      Gbl.FileBrowser.Level = 1;
      for (Ptr=Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder; *Ptr; Ptr++)
         if (*Ptr == '/')
            Gbl.FileBrowser.Level++;
     }

   /***** Get data of assignment *****/
   if (Gbl.FileBrowser.Level &&
       (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_USR ||
        Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS))
     {
      if (Gbl.FileBrowser.Level == 1)
        {
         // We are in this case: assignments/assignment-folder
         strncpy (Gbl.FileBrowser.Asg.Folder,Gbl.FileBrowser.FileOrFolderName,Asg_MAX_LENGTH_FOLDER);
         Gbl.FileBrowser.Asg.Folder[Asg_MAX_LENGTH_FOLDER] = '\0';
        } 
      else
        {
         // We are in this case: assignments/assignment-folder/rest-of-path
         for (Ptr=Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder; *Ptr && *Ptr!='/'; Ptr++);	// Go to first '/'
         if (*Ptr == '/')
            Ptr++;	// Skip '/'
         for (i=0; i<Asg_MAX_LENGTH_FOLDER && *Ptr && *Ptr!='/'; i++, Ptr++)
            Gbl.FileBrowser.Asg.Folder[i] = *Ptr;	// Copy assignment folder
         Gbl.FileBrowser.Asg.Folder[i] = '\0';
        }
      Asg_GetDataOfAssignmentByFolder (&Gbl.FileBrowser.Asg);
     }
  }

/****************************************************************************/
/************************* Initialize file browser **************************/
/****************************************************************************/
// Gbl.FileBrowser.Type must be set to a valid file browser

static void Brw_InitializeFileBrowser (void)
  {
   /***** Construct the relative path to the folder of file browser *****/
   Brw_SetPathFileBrowser ();

   /***** Create top folders in file browser if not exist *****/
   Brw_CreateTopFoldersFileBrowserIfNotExist ();

   /***** Other initializations *****/
   Brw_ResetFileBrowserSize ();
  }

/****************************************************************************/
/********* Construct the paths to the top folders of file browser ***********/
/****************************************************************************/

static void Brw_SetPathFileBrowser (void)
  {
   /***** Reset paths. An empty path means that we don't have to create that directory *****/
   Gbl.FileBrowser.Priv.PathRelAboveAboveRootFolder[0] = '\0';
   Gbl.FileBrowser.Priv.PathAbsAboveRootFolder[0] = '\0';
   Gbl.FileBrowser.Priv.PathRelAboveRootFolder[0] = '\0';
   Gbl.FileBrowser.Priv.PathRelRootFolder[0] = '\0';

   /***** Set paths depending on file browser *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         strcpy (Gbl.FileBrowser.Priv.PathAbsAboveRootFolder,Gbl.CurrentCrs.PathAbsPriv);
         strcpy (Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.CurrentCrs.PathRelPriv);
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Gbl.FileBrowser.Priv.PathRelAboveAboveRootFolder,"%s/grp",Gbl.CurrentCrs.PathRelPriv);
         sprintf (Gbl.FileBrowser.Priv.PathAbsAboveRootFolder,"%s/grp/%ld",Gbl.CurrentCrs.PathAbsPriv,Gbl.CurrentCrs.Grps.GrpCod);
         sprintf (Gbl.FileBrowser.Priv.PathRelAboveRootFolder,"%s/grp/%ld",Gbl.CurrentCrs.PathRelPriv,Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Gbl.FileBrowser.Priv.PathRelAboveAboveRootFolder,"%s/usr",Gbl.CurrentCrs.PathRelPriv);
         sprintf (Gbl.FileBrowser.Priv.PathAbsAboveRootFolder,"%s/usr/%s",Gbl.CurrentCrs.PathAbsPriv,Gbl.Usrs.Me.UsrDat.DNI);
         sprintf (Gbl.FileBrowser.Priv.PathRelAboveRootFolder,"%s/usr/%s",Gbl.CurrentCrs.PathRelPriv,Gbl.Usrs.Me.UsrDat.DNI);
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         if (Gbl.Usrs.Other.UsrDat.DNI[0])
           {
            sprintf (Gbl.FileBrowser.Priv.PathRelAboveAboveRootFolder,"%s/usr",Gbl.CurrentCrs.PathRelPriv);
            sprintf (Gbl.FileBrowser.Priv.PathAbsAboveRootFolder,"%s/usr/%s",Gbl.CurrentCrs.PathAbsPriv,Gbl.Usrs.Other.UsrDat.DNI);
            sprintf (Gbl.FileBrowser.Priv.PathRelAboveRootFolder,"%s/usr/%s",Gbl.CurrentCrs.PathRelPriv,Gbl.Usrs.Other.UsrDat.DNI);
           }
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         strcpy (Gbl.FileBrowser.Priv.PathAbsAboveRootFolder,Gbl.Usrs.Me.PathAbsDir);
         strcpy (Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.Usrs.Me.PathRelDir);
	 break;
      default:
	 return;
     }
   if (Gbl.FileBrowser.Priv.PathRelAboveRootFolder[0])
      sprintf (Gbl.FileBrowser.Priv.PathRelRootFolder,"%s/%s",
               Gbl.FileBrowser.Priv.PathRelAboveRootFolder,
               Brw_RootFolderInternalNames[Gbl.FileBrowser.Type]);
  }

/*****************************************************************************/
/************** Create file browser top folders if not exist *****************/
/*****************************************************************************/

static void Brw_CreateTopFoldersFileBrowserIfNotExist (void)
  {
   /***** Create top folders if not exist *****/
   if (Gbl.FileBrowser.Priv.PathRelAboveAboveRootFolder[0])
      Fil_CreateDirIfNotExists (Gbl.FileBrowser.Priv.PathRelAboveAboveRootFolder);
   if (Gbl.FileBrowser.Priv.PathRelAboveRootFolder[0])
      Fil_CreateDirIfNotExists (Gbl.FileBrowser.Priv.PathRelAboveRootFolder);
   if (Gbl.FileBrowser.Priv.PathRelRootFolder[0])
      Fil_CreateDirIfNotExists (Gbl.FileBrowser.Priv.PathRelRootFolder);

   /***** If file browser is for assignments, create folders of assignments if not exist *****/
   if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_USR ||
       Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS)
      Brw_CreateFoldersAssignmentsIfNotExist ();
  }

/*****************************************************************************/
/****** Check if exists a folder of assignments for any user in course *******/
/*****************************************************************************/
// Folders are in level 1, just under root folder

bool Brw_CheckIfExistsFolderAssigmentForAnyUsr (const char *FolderName)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows,NumRow;
   char PathRelFolderFromCGI_BIN[PATH_MAX+1];
   bool FolderExists = false;
  
   /***** Get all the users belonging to current course from database *****/
   sprintf (Query,"SELECT usr_data.DNI FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod",
            Gbl.CurrentCrs.Crs.CrsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get users from current course");

   /***** Remove folders *****/
   for (NumRow=0; NumRow<NumRows && !FolderExists; NumRow++)
     {
      /* Get next assignment with folder */
      row = mysql_fetch_row (mysql_res);

      /* Remove tree if exists */
      sprintf (PathRelFolderFromCGI_BIN,"%s/usr/%s/%s/%s",
               Gbl.CurrentCrs.PathRelPriv,
               row[0],	// DNI
               Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,
               FolderName);
      FolderExists = Fil_CheckIfPathExists (PathRelFolderFromCGI_BIN);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return FolderExists;
  }

/*****************************************************************************/
/********* Create folders of assignments if not exist for one user ***********/
/*****************************************************************************/
// Folders are created in level 1, just under root folder
// Create a folder of and assignment when:
// 1. Teacher has set that user must send work(s) for that assignment
// 2. ...and folder name is not empty
// 3. ...and the assignment is open (StartTime <= now <= EndTime)
// 4. ...the assignment is not restricted to groups or (if restricted to groups) I belong to any of the groups

static void Brw_CreateFoldersAssignmentsIfNotExist (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows,NumRow;
   char PathRelFolderAsg[PATH_MAX+1];
  
   /***** Get assignment folders from database *****/
   sprintf (Query,"SELECT Folder FROM assignments" \
                  " WHERE CrsCod='%ld' AND Folder<>'' AND StartTime<=NOW() AND EndTime>=NOW()" \
                  " AND (AsgCod NOT IN (SELECT AsgCod FROM asg_grp) OR"
                  " AsgCod IN (SELECT asg_grp.AsgCod FROM asg_grp,crs_grp_usr" \
                  " WHERE crs_grp_usr.UsrCod='%ld' AND asg_grp.GrpCod=crs_grp_usr.GrpCod))",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get folders of assignments");

   /***** Create one folder for each assignment *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next assignment with folder */
      row = mysql_fetch_row (mysql_res);

      /* Create folder if not exists */
      sprintf (PathRelFolderAsg,"%s/%s",Gbl.FileBrowser.Priv.PathRelRootFolder,row[0]);
      Fil_CreateDirIfNotExists (PathRelFolderAsg);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/**** Update folders of assignments if exist for all the users in course *****/
/*****************************************************************************/
// Folders are in level 1, just under root folder

bool Brw_UpdateFoldersAssigmentsIfExistForAllUsrs (const char *OldFolderName,const char *NewFolderName)
  {
   extern const char *Txt_Can_not_rename_a_folder_of_assignment[Txt_NUM_LANGUAGES];
   extern const char *Txt_Users[Txt_NUM_LANGUAGES];
   extern const char *Txt_Folders_renamed[Txt_NUM_LANGUAGES];
   extern const char *Txt_Folders_not_renamed[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows,NumRow;
   char OldPath[PATH_MAX+1];
   char NewPath[PATH_MAX+1];
   char PathRelOldFolderFromCGI_BIN[PATH_MAX+1];
   char PathRelNewFolderFromCGI_BIN[PATH_MAX+1];
   bool RenamingIsPossible = true;
   unsigned NumUsrsError = 0;
   unsigned NumUsrsSuccess = 0;
  
   /***** Get all the users belonging to current course from database *****/
   sprintf (Query,"SELECT usr_data.DNI FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod",
            Gbl.CurrentCrs.Crs.CrsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get users from current course");

   /***** Check if there exist folders with the new name *****/
   for (NumRow=0; NumRow<NumRows && RenamingIsPossible; NumRow++)
     {
      /* Get next assignment with folder */
      row = mysql_fetch_row (mysql_res);

      /* Rename folder if exists */
      sprintf (PathRelOldFolderFromCGI_BIN,"%s/usr/%s/%s/%s",
               Gbl.CurrentCrs.PathRelPriv,
               row[0],	// DNI
               Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,
               OldFolderName);
      sprintf (PathRelNewFolderFromCGI_BIN,"%s/usr/%s/%s/%s",
               Gbl.CurrentCrs.PathRelPriv,
               row[0],	// DNI
               Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,
               NewFolderName);
      if (Fil_CheckIfPathExists (PathRelOldFolderFromCGI_BIN) &&
          Fil_CheckIfPathExists (PathRelNewFolderFromCGI_BIN))
         RenamingIsPossible = false;
     }

   /***** Rename folder for each user *****/
   if (RenamingIsPossible)
     {
      mysql_data_seek (mysql_res,0);
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         /* Get next assignment with folder */
         row = mysql_fetch_row (mysql_res);

         /* Rename folder if exists */
         sprintf (PathRelOldFolderFromCGI_BIN,"%s/usr/%s/%s/%s",
                  Gbl.CurrentCrs.PathRelPriv,
                  row[0],	// DNI
                  Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,
                  OldFolderName);
         if (Fil_CheckIfPathExists (PathRelOldFolderFromCGI_BIN))
           {
            sprintf (PathRelNewFolderFromCGI_BIN,"%s/usr/%s/%s/%s",
                     Gbl.CurrentCrs.PathRelPriv,
                     row[0],	// DNI
                     Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,
                     NewFolderName);
            if (rename (PathRelOldFolderFromCGI_BIN,PathRelNewFolderFromCGI_BIN))
	      {
               Lay_ShowAlert (ERROR,Txt_Can_not_rename_a_folder_of_assignment[Gbl.Prefs.Language]);
               NumUsrsError++;
	      }
            else
              {
               /* Get user code from DNI */
               strcpy (Gbl.Usrs.Other.UsrDat.DNI,row[0]);
               Usr_GetUsrCodFromDNI (&Gbl.Usrs.Other.UsrDat);

               /* Remove affected clipboards */
               Brw_RemoveAffectedClipboards (Brw_FILE_BRW_ASSIGNMENTS_USR,Gbl.Usrs.Other.UsrDat.UsrCod,-1L);
               Brw_RemoveAffectedClipboards (Brw_FILE_BRW_ASSIGNMENTS_CRS,-1L,Gbl.Usrs.Other.UsrDat.UsrCod);

               /* Remove affected expanded folders */
               sprintf (OldPath,"%s/%s",Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,OldFolderName);
               sprintf (NewPath,"%s/%s",Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,NewFolderName);
               Brw_RenameAffectedExpandedFolders (Brw_FILE_BRW_ASSIGNMENTS_USR,Gbl.Usrs.Other.UsrDat.UsrCod,-1L,OldPath,NewPath);
               Brw_RenameAffectedExpandedFolders (Brw_FILE_BRW_ASSIGNMENTS_CRS,-1L,Gbl.Usrs.Other.UsrDat.UsrCod,OldPath,NewPath);

               NumUsrsSuccess++;
              }
           }
        }

      /***** Summary message *****/
      sprintf (Gbl.Message,"%s: %lu<br />"
                           "%s: %u<br />" \
                           "%s: %u.",
               Txt_Users[Gbl.Prefs.Language],NumRows,
               Txt_Folders_renamed[Gbl.Prefs.Language],NumUsrsSuccess,
               Txt_Folders_not_renamed[Gbl.Prefs.Language],NumUsrsError);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
      /***** Warning message *****/
      Lay_ShowAlert (WARNING,Txt_Can_not_rename_a_folder_of_assignment[Gbl.Prefs.Language]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return RenamingIsPossible;
  }

/*****************************************************************************/
/**** Remove folders of assignments if exist for all the users in course *****/
/*****************************************************************************/
// Folders are in level 1, just under root folder

void Brw_RemoveFoldersAssigmentsIfExistForAllUsrs (const char *FolderName)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows,NumRow;
   char PathRelFolderFromCGI_BIN[PATH_MAX+1];
  
   /***** Get all the users belonging to current course from database *****/
   sprintf (Query,"SELECT usr_data.DNI FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod",
            Gbl.CurrentCrs.Crs.CrsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get users from current course");

   /***** Remove folders *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next assignment with folder */
      row = mysql_fetch_row (mysql_res);

      /* Remove tree if exists */
      sprintf (PathRelFolderFromCGI_BIN,"%s/usr/%s/%s/%s",
               Gbl.CurrentCrs.PathRelPriv,
               row[0],	// DNI
               Brw_INTERNAL_NAME_ROOT_FOLDER_ASSIGNMENTS,
               FolderName);
      Brw_RemoveTree (PathRelFolderFromCGI_BIN);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/*** Initialize maximum quota of current file browser and check if exceded ***/
/*****************************************************************************/

static void Brw_SetAndCheckQuota (void)
  {
   extern const char *Txt_Quota_exceeded[Txt_NUM_LANGUAGES];

   /* Check the quota */
   Brw_SetMaxQuota ();
   Brw_CalcSizeOfDir (Gbl.FileBrowser.Priv.PathRelRootFolder);
   if (Brw_CheckIfQuotaExceded ())
      Lay_ShowAlert (ERROR,Txt_Quota_exceeded[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/************ Initialize maximum quota of current file browser ***************/
/*****************************************************************************/

static void Brw_SetMaxQuota (void)
  {
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_DOWNLOAD_CRS;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_DOWNLOAD_CRS;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_DOWNLOAD_CRS;
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_DOWNLOAD_GRP;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_DOWNLOAD_GRP;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_DOWNLOAD_GRP;
	 break;
      case Brw_FILE_BRW_COMMON_CRS:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_COMMON_CRS;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_COMMON_CRS;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_COMMON_CRS;
	 break;
      case Brw_FILE_BRW_COMMON_GRP:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_COMMON_GRP;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_COMMON_GRP;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_COMMON_GRP;
	 break;
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_GRADES_CRS;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_GRADES_CRS;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_GRADES_CRS;
	 break;
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_GRADES_GRP;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_GRADES_GRP;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_GRADES_GRP;
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_ASSIGNMENTS_PER_STD;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_ASSIGNMENTS_PER_STD;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_ASSIGNMENTS_PER_STD;
	 break;
      case Brw_FILE_BRW_WORKS_USR:
      case Brw_FILE_BRW_WORKS_CRS:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTA_WORKS_PER_STD;
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_WORKS_PER_STD;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_WORKS_PER_STD;
	 break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
	 Gbl.FileBrowser.Size.MaxQuota = Brw_MAX_QUOTAS_BRIEFCASE[Gbl.Usrs.Me.UsrDat.UsrTypeInDB];
         Gbl.FileBrowser.Size.MaxFiles = Brw_MAX_FILES_BRIEFCASE;
         Gbl.FileBrowser.Size.MaxFolds = Brw_MAX_FOLDERS_BRIEFCASE;
	 break;
      default:
	 break;
     }
  }

/*****************************************************************************/
/********************** Check if quota has been exceeded *********************/
/*****************************************************************************/

static bool Brw_CheckIfQuotaExceded (void)
  {
   return (Gbl.FileBrowser.Size.NumLevls > Brw_MAX_DIR_LEVELS ||
           Gbl.FileBrowser.Size.NumFolds > Gbl.FileBrowser.Size.MaxFolds ||
           Gbl.FileBrowser.Size.NumFiles > Gbl.FileBrowser.Size.MaxFiles ||
           Gbl.FileBrowser.Size.TotalSiz > Gbl.FileBrowser.Size.MaxQuota);
  }

/*****************************************************************************/
/************** Request edition of works of users of the course **************/
/*****************************************************************************/

void Brw_AskEditWorksCrs (void)
  {
   extern const char *Txt_View_works[Txt_NUM_LANGUAGES];     

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Show form to select the groups *****/
   Grp_ShowFormToSelectSeveralGroups (ActReqAsgWrkCrs);

   /***** Form to select type of list used for select several users *****/
   Usr_ShowFormsToSelectUsrListType (ActReqAsgWrkCrs);

   /***** Get and order lists of users from this course *****/
   Usr_GetAndOrderTchsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);
   Usr_GetAndOrderStdsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);

   if (Gbl.Usrs.LstTchs.NumUsrs || Gbl.Usrs.LstStds.NumUsrs)
     {
      fprintf (Gbl.F.Out,"<div align=\"center\">");

      /***** Draw a class photo with the users of the course *****/
      /* Form start */
      Act_FormStart (ActEdiAsgWrkCrs);
      Grp_PutParamsCodGrps ();
      Brw_ParamWorksZoneCrs ();

      /* Start the table */
      fprintf (Gbl.F.Out,"<table><tr><td align=\"left\">");
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /* Put list of users to select some of them */
      Usr_ListUsersToSelect (Usr_TEACHER);
      Usr_ListUsersToSelect (Usr_STUDENT);

      /* End the table */
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
      fprintf (Gbl.F.Out,"</td></tr></table>\n");

      /***** Button to send the form *****/
      Lay_PutSendButton (Txt_View_works[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form></div>\n");
     }
   else
      Usr_ShowWarningNoUsersFound (Usr_UNKNOWN);

   /***** Free memory for users' list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Free the memory used by the list of users *****/
   Usr_FreeListsEncryptedUsrCods ();

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();
  }

/*****************************************************************************/
/************ Show normal file browser (not for assignments-works) ************/
/*****************************************************************************/

static void Brw_ShowFileBrowserNormal (void)
  {
   /***** Write top before showing file browser *****/
   Brw_WriteTopBeforeShowingFileBrowser ();

   /****** Show the file browser *****/
   Brw_ShowFileBrowser ();
  }

/*****************************************************************************/
/*** Show file browsers with works files of several users of current course **/
/*****************************************************************************/

static void Brw_ShowFileBrowsersAsgWrkCrs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_You_must_select_one_ore_more_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_ZIP_file[Txt_NUM_LANGUAGES];
   const char *Ptr;

   /***** Check the number of users whose works will be shown *****/
   if (Usr_CountNumUsrsInEncryptedList () == 0)				// If no users are selected...
     {
      // ...write error alert
      Lay_ShowAlert (ERROR,Txt_You_must_select_one_ore_more_users[Gbl.Prefs.Language]);
      // ...and show again the form
      Brw_AskEditWorksCrs ();
      return;
     }

   /***** Write top before showing file browser *****/
   Brw_WriteTopBeforeShowingFileBrowser ();

   if (Gbl.FileBrowser.ZIP.CreateZIP)
     {
      /***** Create zip file with the assignments and works of the selected users *****/
      /* Create temporary directory for the compression of assignments and works */
      Brw_CreateTmpDirForCompression ();

      /* Create temporary directory for each selected user inside the directory used for compression */
      Ptr = Gbl.Usrs.Select.All;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,Gbl.Usrs.Other.UsrDat.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
         Usr_GetUsrCodFromEncryptedUsrCod (&Gbl.Usrs.Other.UsrDat);
         if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat))               // Get user's data from database
            if (Usr_CheckIfUsrBelongsToCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod))
               Brw_CreateDirCompressionUsr ();
        }

      /* Create the zip file and put a link to download it */
      if (Gbl.FileBrowser.ZIP.CreateZIP)
         Brw_CreateZIP ();
     }
   else
     {
      /***** Button to create a zip file with all the works of the selected users *****/
      fprintf (Gbl.F.Out,"<div align=\"center\">");
      Act_FormStart (ActEdiAsgWrkCrs);
      Brw_ParamWorksZoneCrs ();
      Usr_HiddenParUsrCodAll (ActEdiAsgWrkCrs,Gbl.Usrs.Select.All);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CreateZIP\" value=\"Y\" />");
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\">");
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Create_ZIP_file[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,"<img src=\"%s/%s/zip12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\"" \
                         " align=\"bottom\" />&nbsp;%s</acronym></a></td></tr></table></form></div>",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_FILEXT,
               Txt_Create_ZIP_file[Gbl.Prefs.Language],
               Txt_Create_ZIP_file[Gbl.Prefs.Language]);
     }

   /***** Header of the table with the list of users *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%%");

   /***** List the assignments and works of the selected users *****/
   Ptr = Gbl.Usrs.Select.All;
   while (*Ptr)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,Gbl.Usrs.Other.UsrDat.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
      Usr_GetUsrCodFromEncryptedUsrCod (&Gbl.Usrs.Other.UsrDat);
      if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat))               // Get of the database the data of the user
         if (Usr_CheckIfUsrBelongsToCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod))
           {
            /***** Show a row with the data of the owner of the works *****/
            fprintf (Gbl.F.Out,"<tr>");
            Brw_ShowDataOwnerAsgWrk (&Gbl.Usrs.Other.UsrDat);

            fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\">");

            /***** Show the tree with the assignments *****/
            Gbl.FileBrowser.Type = Brw_FILE_BRW_ASSIGNMENTS_CRS;
            Brw_InitializeFileBrowser ();
            Brw_ShowFileBrowser ();

            /***** Show the tree with the works *****/
            Gbl.FileBrowser.Type = Brw_FILE_BRW_WORKS_CRS;
            Brw_InitializeFileBrowser ();
            Brw_ShowFileBrowser ();

            fprintf (Gbl.F.Out,"</td></tr>");
           }
     }

   /***** End of the table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free the memory used for the list of users *****/
   Usr_FreeListsEncryptedUsrCods ();
  }

/*****************************************************************************/
/************ Show file browsers with works files of one user ****************/
/*****************************************************************************/

static void Brw_ShowFileBrowsersAsgWrkUsr (void)
  {
   /***** Write top before showing file browser *****/
   Brw_WriteTopBeforeShowingFileBrowser ();

   /***** Show the tree with the assignments *****/
   Gbl.FileBrowser.Type = Brw_FILE_BRW_ASSIGNMENTS_USR;
   Brw_InitializeFileBrowser ();
   Brw_ShowFileBrowser ();

   /***** Show the tree with the works *****/
   Gbl.FileBrowser.Type = Brw_FILE_BRW_WORKS_USR;
   Brw_InitializeFileBrowser ();
   Brw_ShowFileBrowser ();
  }

/*****************************************************************************/
/*************************** Form to change file zone ************************/
/*****************************************************************************/

static void Brw_FormToChangeZone (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *The_ClassFormulB[The_NUM_THEMES];
   extern const char *Txt_Download_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Download_management_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Common_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Marks_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Marks_management_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_group[Txt_NUM_LANGUAGES];
   struct ListCodGrps LstMyGrps;
   unsigned NumGrp;
   struct GroupData GrpDat;

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Select the complete course, not a group *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\">");
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
         Act_FormStart (ActSeeDowCrs);
         Brw_ParamSeeDownloadZoneCrs ();
         sprintf (Gbl.MsgWindowStatus,"%s (%s: %s)",
                  Txt_Download_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         Act_FormStart (ActAdmDowCrs);
         Brw_ParamAdminDownloadZoneCrs ();
         sprintf (Gbl.MsgWindowStatus,"%s (%s: %s)",
                  Txt_Download_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         break;
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_COMMON_GRP:
         Act_FormStart (ActAdmComCrs);
         Brw_ParamAdminCommonZoneCrs ();
         sprintf (Gbl.MsgWindowStatus,"%s (%s: %s)",
                  Txt_Common_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         break;
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
         Act_FormStart (ActSeeAssCrs);
         Brw_ParamSeeAssZoneCrs ();
         sprintf (Gbl.MsgWindowStatus,"%s (%s: %s)",
                  Txt_Marks_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Act_FormStart (ActAdmAssCrs);
         Brw_ParamAdmAssZoneCrs ();
         sprintf (Gbl.MsgWindowStatus,"%s (%s: %s)",
                  Txt_Marks_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         break;
      default:
         break;
     }
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,
                       (Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_DOWNLOAD_CRS ||
                        Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS ||
                        Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_CRS ||
                        Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_CRS ||
                        Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_CRS) ? The_ClassFormulB[Gbl.Prefs.Theme] :
                                                                                            The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"%s</acronym></a></form></td></tr>",Gbl.CurrentCrs.Crs.FullName);

   /***** Select my groups *****/
   if (Grp_CountNumGrpsInCurrentCrs ())	// This course has groups?
     {
      /***** Get list of group types which I belong to *****/
      Grp_GetLstCodGrpsUsrBelongs (-1L,Gbl.Usrs.Me.UsrDat.UsrCod,&LstMyGrps);

      /***** List my groups for unique selection *****/
      for (NumGrp=0; NumGrp<LstMyGrps.NumGrps; NumGrp++)
        {
         /* Get next group */
         GrpDat.GrpCod = LstMyGrps.GrpCod[NumGrp];
         Grp_GetDataOfGroupByCod (&GrpDat);

         if (GrpDat.FileZones)	// Only show groups with enabled file zones
           {
            fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\">");

            switch (Gbl.FileBrowser.Type)
              {
               case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
               case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
                  Act_FormStart (ActSeeDowGrp);
                  Brw_ParamSeeDownloadZoneGrp ();
                  sprintf (Gbl.MsgWindowStatus,"%s (%s: %s %s)",
                           Txt_Download_zone[Gbl.Prefs.Language],
                           Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
                  break;
               case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
               case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
                  Act_FormStart (ActAdmDowGrp);
                  Brw_ParamAdminDownloadZoneGrp ();
                  sprintf (Gbl.MsgWindowStatus,"%s (%s: %s %s)",
                           Txt_Download_management_zone[Gbl.Prefs.Language],
                           Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
                  break;
               case Brw_FILE_BRW_COMMON_CRS:
               case Brw_FILE_BRW_COMMON_GRP:
                  Act_FormStart (ActAdmComGrp);
                  Brw_ParamAdminCommonZoneGrp ();
                  sprintf (Gbl.MsgWindowStatus,"%s (%s: %s %s)",
                           Txt_Common_zone[Gbl.Prefs.Language],
                           Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
                  break;
               case Brw_FILE_BRW_SEE_MARKS_CRS:
               case Brw_FILE_BRW_SEE_MARKS_GRP:
                  Act_FormStart (ActSeeAssGrp);
                  Brw_ParamSeeAssZoneGrp ();
                  sprintf (Gbl.MsgWindowStatus,"%s (%s: %s %s)",
                           Txt_Marks_zone[Gbl.Prefs.Language],
                           Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
                  break;
               case Brw_FILE_BRW_ADMIN_MARKS_CRS:
               case Brw_FILE_BRW_ADMIN_MARKS_GRP:
                  Act_FormStart (ActAdmAssGrp);
                  Brw_ParamAdmAssZoneGrp ();
                  sprintf (Gbl.MsgWindowStatus,"%s (%s: %s %s)",
                           Txt_Marks_management_zone[Gbl.Prefs.Language],
                           Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
                  break;
               default:
                  break;
              }
            Grp_PutParamGrpCod (GrpDat.GrpCod);
            fprintf (Gbl.F.Out,"<img src=\"%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",
                     Gbl.Prefs.IconsURL,NumGrp<LstMyGrps.NumGrps-1 ? "submid" : "subend");
            Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,
                                ((GrpDat.GrpCod == Gbl.CurrentCrs.Grps.GrpCod) &&
                                 (Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_DOWNLOAD_GRP || 
                                  Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP || 
                                  Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP ||
                                  Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_GRP ||
                                  Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP)) ? The_ClassFormulB[Gbl.Prefs.Theme] :
                                                                                                The_ClassFormul[Gbl.Prefs.Theme]);
            fprintf (Gbl.F.Out,"%s %s</acronym></a></form></td></tr>",GrpDat.GrpTypName,GrpDat.GrpName);
           }
        }

      /***** Free memory with the list of groups which user belongs to *****/
      Grp_FreeListCodGrp (&LstMyGrps);
     }

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/*** Get the data of the selected group in order to enter its common zone ****/
/*****************************************************************************/

static void Brw_GetSelectedGroupData (struct GroupData *GrpDat,bool AbortOnError)
  {
   if (GrpDat->GrpCod >= 0)
     {
      /***** Get the data of the selected group *****/
      Grp_GetDataOfGroupByCod (GrpDat);

      /***** For security, check if group file zones are enabled, and if I belongs to the selected group *****/
      if (!GrpDat->FileZones)
        {
         if (AbortOnError)
            Lay_ShowErrorAndExit ("The file browser is disabled.");
         GrpDat->GrpCod = -1L;
        }
      else if (!Grp_CheckIfIBelongToGrp (GrpDat->GrpCod))
        {
         if (AbortOnError)
            Lay_ShowErrorAndExit ("You don't belong to the group whose files you want to access.");
         GrpDat->GrpCod = -1L;
        }
     }
  }

/*****************************************************************************/
/************* Show a row with the data of the owner of works ****************/
/*****************************************************************************/

static void Brw_ShowDataOwnerAsgWrk (struct UsrData *UsrDat)
  {
   extern const char *Txt_Write_a_message_to_X[Txt_NUM_LANGUAGES];
   bool ShowPhoto,UsrHasWebPage;
   char PhotoURL[PATH_MAX+1];
   bool ICanSeeDNI;

   /***** Show user's photo *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" width=\"40\">");
   UsrHasWebPage = Str_URLLooksValid (UsrDat->WWW);
   if (UsrHasWebPage)
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\">",UsrDat->WWW);
   ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
   Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,36,48,true);
   if (UsrHasWebPage)
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td>");

   /***** Start form to send a message to this user *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" width=\"25%%\" class=\"MSG_AUT\">");
   Act_FormStart (ActReqMsgUsr);
   Grp_PutParamAllGroups ();
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrCodAll\" value=\"%s\" />",
            UsrDat->EncryptedUsrCod);

   /***** Show user's DNI *****/
   ICanSeeDNI = (UsrDat->UsrTypeInDB == Usr_TEACHER) ?
                Usr_ICanSeeDNIOfATeacher (UsrDat) :
                (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER);
   fprintf (Gbl.F.Out,"%s<br />",ICanSeeDNI ? UsrDat->DNI : "********");

   /***** Show user's name *****/
   sprintf (Gbl.MsgWindowStatus,Txt_Write_a_message_to_X[Gbl.Prefs.Language],
            UsrDat->FullName);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"MSG_AUT");
   fprintf (Gbl.F.Out,"%s",UsrDat->Surname1);
   if (UsrDat->Surname2[0])
      fprintf (Gbl.F.Out," %s",UsrDat->Surname2);
   if (UsrDat->FirstName[0])
      fprintf (Gbl.F.Out,", %s",UsrDat->FirstName);
   fprintf (Gbl.F.Out,"</acronym></a>");

   /***** Show user's e-mail *****/
   if (UsrDat->E_mail[0])
     {
      fprintf (Gbl.F.Out,"<br /><a href=\"mailto:%s\" target=\"_blank\" class=\"MSG_AUT\">",
               UsrDat->E_mail);
      Str_LimitLengthHTMLStr (UsrDat->E_mail,25);
      fprintf (Gbl.F.Out,UsrDat->E_mail);
      fprintf (Gbl.F.Out,"</a>");
     }
   fprintf (Gbl.F.Out,"</form></td>");
  }

/*****************************************************************************/
/******************* Show a file browser or students' works  *****************/
/*****************************************************************************/

void Brw_ShowFileBrowserOrWorks (void)
  {
   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Show the file browser or works *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/******************* Show a file browser or users' works  ********************/
/*****************************************************************************/

static void Brw_ShowAgainFileBrowserOrWorks (void)
  {
   extern const char *Txt_Files_of_marks_must_contain_a_table_in_HTML_format_[Txt_NUM_LANGUAGES];
   extern const char *Txt_Help_on_the_file_browser[Txt_NUM_LANGUAGES];

   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         Brw_ShowFileBrowsersAsgWrkUsr ();
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         Brw_ShowFileBrowsersAsgWrkCrs ();
         break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Lay_ShowAlert (INFO,Txt_Files_of_marks_must_contain_a_table_in_HTML_format_[Gbl.Prefs.Language]);
      default:
         Brw_ShowFileBrowserNormal ();
     }

   /***** Link to help about file browser *****/
   sprintf (Gbl.Message,"<a href=\"https://%s/wiki/index.php/Explorador_de_archivos\" target=\"_blank\">%s</a>",
            Cfg_PLATFORM_URL,Txt_Help_on_the_file_browser[Gbl.Prefs.Language]);
   Lay_WriteTitle (Gbl.Message);
  }

/*****************************************************************************/
/**************************** Show a file browser ****************************/
/*****************************************************************************/

static void Brw_ShowFileBrowser (void)
  {
   bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS];
   struct stat FileStatus;

   /***** Check if the maximum quota has been exceeded *****/
   if (Brw_FileBrowserIsEditable[Gbl.FileBrowser.Type])
      Brw_SetAndCheckQuota ();

   /***** Check if the clipboard is in this tree *****/
   Gbl.FileBrowser.Clipboard.IsThisTree = Brw_CheckIfClipboardIsInThisTree ();

   /***** Start of table *****/
   if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS ||
       Gbl.FileBrowser.Type == Brw_FILE_BRW_WORKS_CRS)
      Lay_StartRoundFrameTable10 ("frame_white_corner","frame_white",WHITE,"100%%");
   else
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%%");

   /***** Title *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" colspan=\"%u\">",
            Brw_NumColumnsInExpTree[Gbl.FileBrowser.Type]);
   Brw_WriteTitleOfFileBrowser ();
   fprintf (Gbl.F.Out,"</td>" \
                      "</tr>\n");

   /***** Show and store number of documents found *****/
   if (Brw_FileBrowserIsEditable[Gbl.FileBrowser.Type])
     {
      Brw_ShowSizeOfFileTree ();
      Brw_StoreSizeOfFileTreeInDB ();
     }

   /***** List recursively the directory *****/
   Brw_CreateDirDownloadTmp ();
   lstat (Gbl.FileBrowser.Priv.PathRelRootFolder,&FileStatus);
   Brw_ConstructCompletePathInTree (Brw_RootFolderInternalNames[Gbl.FileBrowser.Type],".");
   if (Brw_WriteRowFileBrowser (0,IsLastItemInLevel,Brw_IS_FOLDER,EXPAND_TREE_NOTHING,Brw_RootFolderInternalNames[Gbl.FileBrowser.Type],".",FileStatus.st_size,FileStatus.st_mtime))
      Brw_ListDir (1,IsLastItemInLevel,Gbl.FileBrowser.Priv.PathRelRootFolder,Brw_RootFolderInternalNames[Gbl.FileBrowser.Type]);

   /***** End of table *****/
   if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS ||
       Gbl.FileBrowser.Type == Brw_FILE_BRW_WORKS_CRS)
      Lay_EndRoundFrameTable10 ("frame_white_corner","frame_white");
   else
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/********************** Write title of a file browser ************************/
/*****************************************************************************/

static void Brw_WriteTopBeforeShowingFileBrowser (void)
  {
   /***** Update last access to this file browser *****/
   Brw_UpdateLastAccess ();

   /***** Put form to change zone *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Brw_FormToChangeZone ();
	 break;
      default:
	 break;
     }

   /***** Initialize hidden levels *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         Brw_InitHiddenLevels ();
	 break;
      default:
	 break;
     }

   /***** Write for to show the full tree *****/
   Brw_WriteFormFullTree ();

   /***** If browser is editable, get and write current clipboard *****/
   if (Brw_FileBrowserIsEditable[Gbl.FileBrowser.Type])
      Brw_GetAndWriteClipboard ();
  }

/*****************************************************************************/
/******************* Update last access to a file browser ********************/
/*****************************************************************************/

static void Brw_UpdateLastAccess (void)
  {
   /***** Get and update date and hour of last access to works in the course *****/

   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccDownloadCrs");
         if (Gbl.CurrentAct == ActSeeDowCrs)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastDowGrpCod",-1L);
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccDownloadGrp");
         if (Gbl.CurrentAct == ActSeeDowGrp)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastDowGrpCod",Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccDownloadCrs");
         if (Gbl.CurrentAct == ActAdmDowCrs)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastDowGrpCod",-1L);
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccDownloadGrp");
         if (Gbl.CurrentAct == ActAdmDowGrp)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastDowGrpCod",Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_COMMON_CRS:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccCommonCrs");
         if (Gbl.CurrentAct == ActAdmComCrs)	// Update group of last access to a common zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastComGrpCod",-1L);
	 break;
      case Brw_FILE_BRW_COMMON_GRP:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccCommonGrp");
         if (Gbl.CurrentAct == ActAdmComGrp)	// Update group of last access to a common zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastComGrpCod",Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_SEE_MARKS_CRS:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccMarksCrs");
         if (Gbl.CurrentAct == ActSeeAssCrs)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastAssGrpCod",-1L);
	 break;
      case Brw_FILE_BRW_SEE_MARKS_GRP:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccMarksGrp");
         if (Gbl.CurrentAct == ActSeeAssGrp)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastAssGrpCod",Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccMarksCrs");
         if (Gbl.CurrentAct == ActAdmAssCrs)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastAssGrpCod",-1L);
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccMarksGrp");
         if (Gbl.CurrentAct == ActAdmAssGrp)	// Update group of last access to a download zone only when user chages zone
            Brw_UpdateGrpLastAccZone ("LastAssGrpCod",Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccMyWorks");
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccCrsWorks");
	 break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         Brw_GetAndUpdateDateLastAccFileBrowser ("LastAccBriefcase");
	 break;
      default:
	 return;
     }
  }

/*****************************************************************************/
/*********************** Write title of a file browser ***********************/
/*****************************************************************************/

static void Brw_WriteTitleOfFileBrowser (void)
  {
   extern const char *Txt_Download_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Download_management_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Common_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Assignments_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Works_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Marks_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Marks_management_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_Private_storage_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_for_reading_and_writing_by_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_accessible_only_for_reading_by_you_and_the_teachers_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_the_marks_of_a_student_chosen_at_random_[Txt_NUM_LANGUAGES];
   extern const char *Txt_nobody_else_can_access_this_content[Txt_NUM_LANGUAGES];
   char Title[1024];
   char Subtitle[1024];

   /***** Set text of title *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
         sprintf (Title,"%s<br />%s: %s",
                  Txt_Download_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
         sprintf (Title,"%s<br />%s: %s %s",
                  Txt_Download_zone[Gbl.Prefs.Language],
                  Txt_group[Gbl.Prefs.Language],Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,Gbl.CurrentCrs.Grps.GrpName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         sprintf (Title,"%s<br />%s: %s",
                  Txt_Download_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         sprintf (Title,"%s<br />%s: %s %s",
                  Txt_Download_management_zone[Gbl.Prefs.Language],
                  Txt_group[Gbl.Prefs.Language],Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,Gbl.CurrentCrs.Grps.GrpName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_COMMON_CRS:
         sprintf (Title,"%s<br />%s: %s",
                  Txt_Common_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_COMMON_GRP:
         sprintf (Title,"%s<br />%s: %s %s",
                  Txt_Common_zone[Gbl.Prefs.Language],
                  Txt_group[Gbl.Prefs.Language],Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,Gbl.CurrentCrs.Grps.GrpName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
         if (Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_CRS)
            sprintf (Title,"%s<br />%s: %s",
                     Txt_Marks_zone[Gbl.Prefs.Language],
                     Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         else
            sprintf (Title,"%s<br />%s: %s %s",
                     Txt_Marks_zone[Gbl.Prefs.Language],
                     Txt_group[Gbl.Prefs.Language],Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,Gbl.CurrentCrs.Grps.GrpName);
         if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)
            sprintf (Subtitle,"(%s)",
                     Txt_accessible_only_for_reading_by_you_and_the_teachers_of_the_course[Gbl.Prefs.Language]);
	 else
            sprintf (Subtitle,"(%s)",
                     Txt_the_marks_of_a_student_chosen_at_random_[Gbl.Prefs.Language]);
 	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         sprintf (Title,"%s<br />%s: %s",
                  Txt_Marks_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Title,"%s<br />%s: %s %s",
                  Txt_Marks_management_zone[Gbl.Prefs.Language],
                  Txt_group[Gbl.Prefs.Language],Gbl.CurrentCrs.Grps.GrpTyp.GrpTypName,Gbl.CurrentCrs.Grps.GrpName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
         sprintf (Title,"%s<br />%s, %s %s",
                  Txt_Assignments_zone[Gbl.Prefs.Language],
                  Gbl.Usrs.Me.UsrDat.FullName,
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Title,"%s<br />%s, %s %s",
                  Txt_Works_zone[Gbl.Prefs.Language],
                  Gbl.Usrs.Me.UsrDat.FullName,
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course[Gbl.Prefs.Language]);
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         sprintf (Title,"%s<br />%s",
                  Txt_Assignments_zone[Gbl.Prefs.Language],
                  Gbl.Usrs.Other.UsrDat.FullName);
         Subtitle[0] = '\0';
	 break;
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Title,"%s<br />%s",
                  Txt_Works_zone[Gbl.Prefs.Language],
                  Gbl.Usrs.Other.UsrDat.FullName);
         Subtitle[0] = '\0';
	 break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Title,"%s<br />%s",
                  Txt_Private_storage_zone[Gbl.Prefs.Language],
                  Gbl.Usrs.Me.UsrDat.FullName);
         sprintf (Subtitle,"(%s)",
                  Txt_nobody_else_can_access_this_content[Gbl.Prefs.Language]);
	 break;
      default:
	 return;
     }

   /***** Write title *****/
   fprintf (Gbl.F.Out,"<span class=\"Brw_TITLE\">%s</span>",Title);
   if (Subtitle[0])
      fprintf (Gbl.F.Out,"<span class=\"Brw_SUBTITLE\"><br />%s</span>",Subtitle);
  }

/*****************************************************************************/
/************ Initialize hidden levels of download tree to false *************/
/*****************************************************************************/

static void Brw_InitHiddenLevels (void)
  {
   unsigned Level;

   for (Level=0; Level<=Brw_MAX_DIR_LEVELS; Level++)
      Gbl.FileBrowser.HiddenLevels[Level] = false;
  }

/*****************************************************************************/
/************************* Show size of a file browser ***********************/
/*****************************************************************************/

static void Brw_ShowSizeOfFileTree (void)
  {
   extern const char *Txt_level[Txt_NUM_LANGUAGES];
   extern const char *Txt_levels[Txt_NUM_LANGUAGES];
   extern const char *Txt_folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_folders[Txt_NUM_LANGUAGES];
   extern const char *Txt_file[Txt_NUM_LANGUAGES];
   extern const char *Txt_files[Txt_NUM_LANGUAGES];
   extern const char *Txt_of_PART_OF_A_TOTAL[Txt_NUM_LANGUAGES];
   
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" colspan=\"%u\" class=\"DAT\">" \
                      "%u %s; %lu %s; %lu %s; ",
            Brw_NumColumnsInExpTree[Gbl.FileBrowser.Type],
            Gbl.FileBrowser.Size.NumLevls,(Gbl.FileBrowser.Size.NumLevls == 1) ? Txt_level [Gbl.Prefs.Language] : Txt_levels [Gbl.Prefs.Language],
            Gbl.FileBrowser.Size.NumFolds,(Gbl.FileBrowser.Size.NumFolds == 1) ? Txt_folder[Gbl.Prefs.Language] : Txt_folders[Gbl.Prefs.Language],
            Gbl.FileBrowser.Size.NumFiles,(Gbl.FileBrowser.Size.NumFiles == 1) ? Txt_file  [Gbl.Prefs.Language] : Txt_files  [Gbl.Prefs.Language]);
   Str_WriteSizeInBytes ((double) Gbl.FileBrowser.Size.TotalSiz);
   fprintf (Gbl.F.Out," (%.1f%% %s ",
            100.0 * ((double) Gbl.FileBrowser.Size.TotalSiz / (double) Gbl.FileBrowser.Size.MaxQuota),
            Txt_of_PART_OF_A_TOTAL[Gbl.Prefs.Language]);
   Str_WriteSizeInBytes ((double) Gbl.FileBrowser.Size.MaxQuota);
   fprintf (Gbl.F.Out,")</td></tr>");
  }

/*****************************************************************************/
/****************** Store size of a file browser in database *****************/
/*****************************************************************************/

static void Brw_StoreSizeOfFileTreeInDB (void)
  {
   char Query[512];
   long CrsCod = -1L;
   long GrpCod = -1L;
   long UsrCod = -1L;
   Brw_FileBrowser_t FileBrowser = Gbl.FileBrowser.Type;

   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         CrsCod = Gbl.CurrentCrs.Crs.CrsCod;
         break;
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         CrsCod = Gbl.CurrentCrs.Crs.CrsCod;
         GrpCod = Gbl.CurrentCrs.Grps.GrpCod;
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         CrsCod = Gbl.CurrentCrs.Crs.CrsCod;
         UsrCod = Gbl.Usrs.Me.UsrDat.UsrCod;
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         CrsCod = Gbl.CurrentCrs.Crs.CrsCod;
         UsrCod = Gbl.Usrs.Other.UsrDat.UsrCod;
         FileBrowser = Brw_FILE_BRW_ASSIGNMENTS_USR;
         break;
      case Brw_FILE_BRW_WORKS_CRS:
         CrsCod = Gbl.CurrentCrs.Crs.CrsCod;
         UsrCod = Gbl.Usrs.Other.UsrDat.UsrCod;
         FileBrowser = Brw_FILE_BRW_WORKS_USR;
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         UsrCod = Gbl.Usrs.Me.UsrDat.UsrCod;
         break;
      default: // Not aplicable here
         break;
     }

   /***** Update size of the file browser in database *****/
   sprintf (Query,"REPLACE INTO file_browser_size" \
                  " (FileBrowser,CrsCod,GrpCod,UsrCod,NumLevels,NumFolders,NumFiles,TotalSize)" \
                  " VALUES ('%u','%ld','%ld','%ld','%u','%lu','%lu','%llu')",
            (unsigned) FileBrowser,CrsCod,GrpCod,UsrCod,
            Gbl.FileBrowser.Size.NumLevls,Gbl.FileBrowser.Size.NumFolds,Gbl.FileBrowser.Size.NumFiles,Gbl.FileBrowser.Size.TotalSiz);
   DB_QueryREPLACE (Query,"can not store the size of a file browser");
  }

/*****************************************************************************/
/**************** Remove size of a file browser from database ****************/
/*****************************************************************************/

void Brw_RemoveSizeOfFileTreeFromDB (long CrsCod,long GrpCod,long UsrCod)
  {
   char Query[512];

   /***** Remove size of the file browser from database *****/
   if (CrsCod > 0 && GrpCod == -1L && UsrCod == -1L)	// All the zones of the course
      sprintf (Query,"DELETE FROM file_browser_size WHERE CrsCod='%ld'",
               CrsCod);
   else if (CrsCod > 0 && UsrCod > 0)			// Works of a student in the course
      sprintf (Query,"DELETE FROM file_browser_size WHERE CrsCod='%ld' AND UsrCod='%ld'",
               CrsCod,UsrCod);
   else if (GrpCod > 0)					// All the zones of the group
      sprintf (Query,"DELETE FROM file_browser_size WHERE GrpCod='%ld'",
               GrpCod);
   else if (UsrCod > 0)					// All the zones of the user
      sprintf (Query,"DELETE FROM file_browser_size WHERE UsrCod='%ld'",
               UsrCod);
   DB_QueryDELETE (Query,"can not remove the size of a file browser");
  }

/*****************************************************************************/
/************** Write a form to select whether show full tree ****************/
/*****************************************************************************/

static void Brw_WriteFormFullTree (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_all_files[Txt_NUM_LANGUAGES];

   /***** Start form depending on type of tree *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
	 Act_FormStart (ActSeeDowCrs);
	 Brw_ParamSeeDownloadZoneCrs ();
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
	 Act_FormStart (ActSeeDowGrp);
	 Brw_ParamSeeDownloadZoneGrp ();
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
	 Act_FormStart (ActAdmDowCrs);
	 Brw_ParamAdminDownloadZoneCrs ();
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
	 Act_FormStart (ActAdmDowGrp);
	 Brw_ParamAdminDownloadZoneGrp ();
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_COMMON_CRS:
	 Act_FormStart (ActAdmComCrs);
	 Brw_ParamAdminCommonZoneCrs ();
	 break;
      case Brw_FILE_BRW_COMMON_GRP:
	 Act_FormStart (ActAdmComGrp);
	 Brw_ParamAdminCommonZoneGrp ();
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_SEE_MARKS_CRS:
	 Act_FormStart (ActSeeAssCrs);
	 Brw_ParamSeeAssZoneCrs ();
	 break;
      case Brw_FILE_BRW_SEE_MARKS_GRP:
	 Act_FormStart (ActSeeAssGrp);
	 Brw_ParamSeeAssZoneGrp ();
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
	 Act_FormStart (ActAdmAssCrs);
	 Brw_ParamAdmAssZoneCrs ();
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
	 Act_FormStart (ActAdmAssGrp);
	 Brw_ParamAdmAssZoneGrp ();
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
	 Act_FormStart (ActEdiAsgWrkUsr);
	 Brw_ParamAssignmentsZoneUsr ();
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
	 Act_FormStart (ActEdiAsgWrkCrs);
	 Brw_ParamAssignmentsZoneCrs ();
	 Usr_HiddenParUsrCodAll (ActEdiAsgWrkCrs,Gbl.Usrs.Select.All);
	 break;
      case Brw_FILE_BRW_WORKS_USR:
	 Act_FormStart (ActEdiAsgWrkUsr);
	 Brw_ParamWorksZoneUsr ();
	 break;
      case Brw_FILE_BRW_WORKS_CRS:
	 Act_FormStart (ActEdiAsgWrkCrs);
	 Brw_ParamWorksZoneCrs ();
	 Usr_HiddenParUsrCodAll (ActEdiAsgWrkCrs,Gbl.Usrs.Select.All);
	 break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
	 Act_FormStart (AccAdmMlt);
	 Brw_ParamBriefZoneUsr ();
	 break;
      default:
	 break;
     }

   /***** End form *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"left\" valign=\"middle\">" \
                      "<input type=\"checkbox\" name=\"FullTree\" value=\"Y\"");
   if (Gbl.FileBrowser.FullTree)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onclick=\"javascript:document.forms[%d].submit();\" /></td>",Gbl.NumForm);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" class=\"%s\">" \
                      "&nbsp;%s" \
                      "</td></tr></table></form></div>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Show_all_files[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/******************* Get whether to show full tree from form *****************/
/*****************************************************************************/

static bool Brw_GetFullTreeFromForm (void)
  {
   char YN[1+1];

   Par_GetParam (Par_PARAM_SINGLE,"FullTree",YN,1);
   return (Str_ConvertToUpperLetter (YN[0]) == 'Y');
  }

/*****************************************************************************/
/****************** Get whether to create ZIP file from form *****************/
/*****************************************************************************/

static bool Brw_GetCreateZIPFromForm (void)
  {
   char YN[1+1];

   Par_GetParam (Par_PARAM_SINGLE,"CreateZIP",YN,1);
   return (Str_ConvertToUpperLetter (YN[0]) == 'Y');
  }

/*****************************************************************************/
/******** Create a temporary public directory used to download files *********/
/*****************************************************************************/

static void Brw_CreateDirDownloadTmp (void)
  {
   static unsigned NumDir = 0;	// When this function is called several times in the same execution of the program, each time a new directory is created
				// This happens when the trees of assignments and works of several users is being listing
   char PathPubDirTmpRelDesdeCGI_BIN[PATH_MAX+1];
   char PathPubDownloadRelFromCGI_BIN[PATH_MAX+1];

   /***** If the public directory doesn't exists, create it *****/
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_FILE_BROWSER_TMP_PUBLIC);

   /***** First of all, we remove the oldest temporary directories.
          Such temporary directories have been created by me or by other users.
          This is a bit sloppy, but they must be removed by someone.
          Here "oldest" means more than x time from their creation *****/
   if (NumDir == 0)	// Only in the first call to this function
      Brw_RemoveOldTmpFiles (Cfg_PATH_REL_FILE_BROWSER_TMP_PUBLIC,false);

   /***** Create a new temporary directory *****/
//   umask ((mode_t) 0);
   if (NumDir)
      sprintf (Gbl.FileBrowser.TmpPubDir,"%s_%u",Gbl.UniqueNameEncrypted,NumDir);
   else
      strcpy (Gbl.FileBrowser.TmpPubDir,Gbl.UniqueNameEncrypted);
   sprintf (PathPubDirTmpRelDesdeCGI_BIN,"%s/%s",Cfg_PATH_REL_FILE_BROWSER_TMP_PUBLIC,Gbl.FileBrowser.TmpPubDir);
   if (mkdir (PathPubDirTmpRelDesdeCGI_BIN,(mode_t) 0xFFF))
      Lay_ShowErrorAndExit ("Can not create a temporary folder for download.");
   NumDir++;

   /***** Create the download folder inside the temporary directory *****/
   sprintf (PathPubDownloadRelFromCGI_BIN,"%s/%s",PathPubDirTmpRelDesdeCGI_BIN,Brw_RootFolderInternalNames[Gbl.FileBrowser.Type]);
   Fil_CreateDirIfNotExists (PathPubDownloadRelFromCGI_BIN);
  }

/*****************************************************************************/
/********* Create temporary directory to put the works to compress ***********/
/*****************************************************************************/

static void Brw_CreateTmpDirForCompression (void)
  {
   char PathDirTmpRelFromCGI_BIN[PATH_MAX+1];

   /***** If the public directory doesn't exists, create it *****/
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_ZIP_TMP_PRIVATE);

   /***** First of all, we remove the oldest temporary directories.
          Such temporary directories have been created by me or by other users.
          This is a bit sloppy, but they must be removed by someone.
          Here "oldest" means more than x time from their creation *****/
   Brw_RemoveOldTmpFiles (Cfg_PATH_REL_ZIP_TMP_PRIVATE,false);

   /***** Create a new temporary directory *****/
//   umask ((mode_t) 0);
   strcpy (Gbl.FileBrowser.ZIP.TmpDir,Gbl.UniqueNameEncrypted);
   sprintf (PathDirTmpRelFromCGI_BIN,"%s/%s",Cfg_PATH_REL_ZIP_TMP_PRIVATE,Gbl.FileBrowser.ZIP.TmpDir);
   if (mkdir (PathDirTmpRelFromCGI_BIN,(mode_t) 0xFFF))
      Lay_ShowErrorAndExit ("Can not create temporary folder for compression.");
  }

/*****************************************************************************/
/**************** Create link to a user's works zone        ******************/
/**************** in the temporary directory of compression ******************/
/*****************************************************************************/
// In Gbl.Usrs.Other.UsrDat deben estar the data of the user

static void Brw_CreateDirCompressionUsr (void)
  {
   char FullNameAndDNI[(Usr_MAX_BYTES_NAME+1)*3+Usr_MAX_LENGTH_USR_ID+1];
   char PathRelFolderUsrInsideCrs[PATH_MAX+1];
   char LinkRelTmpUsr[PATH_MAX+1];

   /***** Create a link in the tree of compression with a name that identifies the owner of the assignments and works *****/
   sprintf (FullNameAndDNI,"%s%s%s%s%s%s",Gbl.Usrs.Other.UsrDat.Surname1,Gbl.Usrs.Other.UsrDat.Surname1[0] ? "_" : "",
					  Gbl.Usrs.Other.UsrDat.Surname2,Gbl.Usrs.Other.UsrDat.Surname2[0] ? "_" : "",
   					  Gbl.Usrs.Other.UsrDat.FirstName,Gbl.Usrs.Other.UsrDat.FirstName[0] ? "-" : "");
   Str_LimitLengthHTMLStr (FullNameAndDNI,50);
   strcat (FullNameAndDNI,Gbl.Usrs.Other.UsrDat.DNI);
   Str_ConvertToValidFileName (FullNameAndDNI);

   // sprintf (PathRelFolderUsrInsideCrs,"%s/usr/%s/%s",Gbl.CurrentCrs.PathAbsPriv,Gbl.Usrs.Other.UsrDat.DNI,Brw_INTERNAL_NAME_ROOT_FOLDER_WORKS);
   sprintf (PathRelFolderUsrInsideCrs,"%s/usr/%s",Gbl.CurrentCrs.PathAbsPriv,Gbl.Usrs.Other.UsrDat.DNI);
   sprintf (LinkRelTmpUsr,"%s/%s/%s",Cfg_PATH_REL_ZIP_TMP_PRIVATE,Gbl.FileBrowser.ZIP.TmpDir,FullNameAndDNI);
   if (symlink (PathRelFolderUsrInsideCrs,LinkRelTmpUsr) != 0)
      Lay_ShowErrorAndExit ("Can not create temprary folder for compression.");
  }

/*****************************************************************************/
/*************** Create the zip file with assignments and works **************/
/*************** and put a link to download it                  **************/
/*****************************************************************************/

static void Brw_CreateZIP (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_ZIP_file[Txt_NUM_LANGUAGES];
   char StrZip[100+PATH_MAX*2+1];
   char PathRelFromCGI_BIN[PATH_MAX+1];
   int Result;

   /***** Create a temporary public directory used to download the zip file *****/
   Brw_CreateDirDownloadTmp ();

   /***** Relative path of the directory with the works to compress *****/
   sprintf (PathRelFromCGI_BIN,"%s/%s",Cfg_PATH_REL_ZIP_TMP_PRIVATE,Gbl.FileBrowser.ZIP.TmpDir);

   /***** Change to directory of the assignments and works in order to start the path in the zip file from there *****/
   if (chdir (PathRelFromCGI_BIN))
      Lay_ShowErrorAndExit ("Can not change to temporary folder for compression.");

   /***** Create public zip file with the assignment and works *****/
   sprintf (StrZip,"nice -n 19 zip -q -r %s/%s/trabajos.zip *",
            Cfg_PATH_ABS_FILE_BROWSER_TMP_PUBLIC,Gbl.FileBrowser.TmpPubDir);
   Result = system (StrZip);

   /***** Return to the CGI directory *****/
   chdir (Cfg_PATH_ABS_CGI_BIN);

   /***** If the zip command has been sucessful, write the link to zip file *****/
   if (Result == 0)
      fprintf (Gbl.F.Out,"<div align=\"center\">" \
                         "<a href=\"%s/%s/trabajos.zip\" class=\"%s\">" \
                         "<img src=\"%s/%s/zip12x12.gif\" width=\"12\" height=\"12\"" \
                         " alt=\"%s\" align=\"bottom\" />" \
                         "&nbsp;%s" \
                         "</a></div>",
               Cfg_URL_FILE_BROWSER_TMP_PUBLIC,Gbl.FileBrowser.TmpPubDir,
               The_ClassFormul[Gbl.Prefs.Theme],
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_FILEXT,
               Txt_ZIP_file[Gbl.Prefs.Language],
               Txt_ZIP_file[Gbl.Prefs.Language]);
   else
      Lay_ShowErrorAndExit ("Can not compress files into zip file.");

   /***** Remove the directory of compression *****/
   Brw_RemoveTree (PathRelFromCGI_BIN);
  }

/*****************************************************************************/
/* Get and update the date of my last access to file browser in this course */
/*****************************************************************************/

static void Brw_GetAndUpdateDateLastAccFileBrowser (const char *FieldNameDB)
  {
   char Query1[256];
   char Query2[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get date of last accesss to a file browser from database *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_USR:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query1,"SELECT UNIX_TIMESTAMP(%s) FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
                  FieldNameDB,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
         sprintf (Query2,"UPDATE crs_usr SET %s=NOW() WHERE CrsCod='%ld' AND UsrCod='%ld'",
                  FieldNameDB,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query1,"SELECT UNIX_TIMESTAMP(%s) FROM crs_grp_usr WHERE GrpCod='%ld' AND UsrCod='%ld'",
                  FieldNameDB,Gbl.CurrentCrs.Grps.GrpCod,Gbl.Usrs.Me.UsrDat.UsrCod);
         sprintf (Query2,"UPDATE crs_grp_usr SET %s=NOW() WHERE GrpCod='%ld' AND UsrCod='%ld'",
                  FieldNameDB,Gbl.CurrentCrs.Grps.GrpCod,Gbl.Usrs.Me.UsrDat.UsrCod);
	 break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query1,"SELECT UNIX_TIMESTAMP(%s) FROM usr_data WHERE UsrCod='%ld'",
                  FieldNameDB,Gbl.Usrs.Me.UsrDat.UsrCod);
         sprintf (Query2,"UPDATE usr_data SET %s=NOW() WHERE UsrCod='%ld'",
                  FieldNameDB,Gbl.Usrs.Me.UsrDat.UsrCod);
	 break;
      default:
	 return;
     }
   NumRows = DB_QuerySELECT (Query1,&mysql_res,"can not get date-time of last access to a file browser");

   if (NumRows == 0)	// May be an administrator not belonging to this course
      Gbl.Usrs.Me.TimeLastAccToThisFileBrowser = LONG_MAX;	// Initialize to a big value in order to show files as old
   else if (NumRows == 1)
     {
      /* Get the date of the last access to file browser (row[0]) */
      row = mysql_fetch_row (mysql_res);
      if (row[0] == NULL)
         Gbl.Usrs.Me.TimeLastAccToThisFileBrowser = 0;
      else if (sscanf (row[0],"%lu",&Gbl.Usrs.Me.TimeLastAccToThisFileBrowser) != 1)
         Lay_ShowErrorAndExit ("Error when reading date-time of last access to a file browser.");
     }
   else
      Lay_ShowErrorAndExit ("Error when getting date-time of last access to a file browser.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Update date of my last access to file browser in this course *****/
   DB_QueryUPDATE (Query2,"can not update date of last access to a file browser");
  }

/*****************************************************************************/
/************* Get the group of my last access to a common zone **************/
/*****************************************************************************/

static long Brw_GetGrpLastAccZone (const char *FieldNameDB)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long GrpCod = -1L;

   /***** Get the group of my last access to a common zone from database *****/
   sprintf (Query,"SELECT %s FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            FieldNameDB,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the group of your last access to a file browser");

   if (NumRows == 0)	// May be an administrator not belonging to this course
      GrpCod = -1L;
   else if (NumRows == 1)
     {
      /* Get the group code (row[0]) */
      row = mysql_fetch_row (mysql_res);
      if (row[0])
        {
         if (sscanf (row[0],"%ld",&GrpCod) != 1)
            Lay_ShowErrorAndExit ("Error when reading the group of your last access to a file browser.");
        }
      else
         GrpCod = -1L;
     }
   else
      Lay_ShowErrorAndExit ("Error when getting the group of your last access to a file browser.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Check if group exists (it's possible that this group has been removed after my last access to it) *****/
   if (GrpCod >= 0)
      if (Grp_CheckIfGroupExists (GrpCod))
         /* Check if I belong to this group (it's possible that I have been removed from this group after my last access to it) */
         if (Grp_CheckIfIBelongToGrp (GrpCod))
            return GrpCod;

   return -1L;	// To indicate course zone
  }

/*****************************************************************************/
/*********** Update the group of my last access to a common zone *************/
/*****************************************************************************/

static void Brw_UpdateGrpLastAccZone (const char *FieldNameDB,long GrpCod)
  {
   char Query[512];

   /***** Update the group of my last access to a common zone *****/
   sprintf (Query,"UPDATE crs_usr SET %s='%ld' WHERE CrsCod='%ld' AND UsrCod='%ld'",
            FieldNameDB,GrpCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update the group of the last access to a file browser");
  }

/*****************************************************************************/
/********************* Reset the size of a file browser **********************/
/*****************************************************************************/

static void Brw_ResetFileBrowserSize (void)
  {
   Gbl.FileBrowser.Size.NumLevls = 0;
   Gbl.FileBrowser.Size.NumFolds =
   Gbl.FileBrowser.Size.NumFiles = 0L;
   Gbl.FileBrowser.Size.TotalSiz = 0ULL;
  }

/*****************************************************************************/
/********************** Compute the size of a directory **********************/
/*****************************************************************************/

void Brw_CalcSizeOfDir (char *PathRel)
  {
   Brw_ResetFileBrowserSize ();
   Brw_CalcSizeOfDirRecursive (1,PathRel);
  }

/*****************************************************************************/
/**************** Compute the size of a directory recursively ****************/
/*****************************************************************************/

void Brw_CalcSizeOfDirRecursive (unsigned Level,char *PathRel)
  {
   struct dirent **FileList;
   int NumFileInThisDir,NumFilesInThisDir;
   char PathFileRel[PATH_MAX+1];
   struct stat FileStatus;

   /***** Scan the directory *****/
   NumFilesInThisDir = scandir (PathRel,&FileList,NULL,NULL);

   /***** Compute recursively the total number and size of the files and folders *****/
   for (NumFileInThisDir=0; NumFileInThisDir<NumFilesInThisDir; NumFileInThisDir++)
      if (strcmp (FileList[NumFileInThisDir]->d_name,".") &&
          strcmp (FileList[NumFileInThisDir]->d_name,".."))	// Skip directories "." and ".."
	{
         /* There are files in this directory ==> update level */
         if (Level > Gbl.FileBrowser.Size.NumLevls)
            Gbl.FileBrowser.Size.NumLevls++;

         /* Update counters depending on whether it's a directory or a regular file */
	 sprintf (PathFileRel,"%s/%s",PathRel,FileList[NumFileInThisDir]->d_name);
	 lstat (PathFileRel,&FileStatus);
	 if (S_ISDIR (FileStatus.st_mode))		// It's a directory
	   {
	    Gbl.FileBrowser.Size.NumFolds++;
	    Gbl.FileBrowser.Size.TotalSiz += (unsigned long long int) FileStatus.st_size;
	    Brw_CalcSizeOfDirRecursive (Level+1,PathFileRel);
	   }
	 else if (S_ISREG (FileStatus.st_mode))		// It's a regular file
	   {
	    Gbl.FileBrowser.Size.NumFiles++;
	    Gbl.FileBrowser.Size.TotalSiz += (unsigned long long int) FileStatus.st_size;
	   }
	}
  }

/*****************************************************************************/
/************************ List a directory recursively ***********************/
/*****************************************************************************/

static void Brw_ListDir (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS],const char *PathRel,const char *PathInTree)
  {
   struct dirent **DirFileList,**SubdirFileList;
   int NumFileInThisDir,NumFilesInThisDir,NumFilesInThisSubdir;
   char PathFileRel[PATH_MAX+1];
   char PathFileInExplTree[PATH_MAX+1];
   struct stat FileStatus;
   ExpandTree_t ExpandTree;

   /***** Scan directory *****/
   NumFilesInThisDir = scandir (PathRel,&DirFileList,NULL,alphasort);

   /***** List files *****/
   for (NumFileInThisDir=0; NumFileInThisDir<NumFilesInThisDir; NumFileInThisDir++)
      if (strcmp (DirFileList[NumFileInThisDir]->d_name,".") &&
          strcmp (DirFileList[NumFileInThisDir]->d_name,".."))	// Skip directories "." and ".."
        {
         IsLastItemInLevel[Level] = (NumFileInThisDir == NumFilesInThisDir-1);
	 sprintf (PathFileRel       ,"%s/%s",PathRel   ,DirFileList[NumFileInThisDir]->d_name);
	 sprintf (PathFileInExplTree,"%s/%s",PathInTree,DirFileList[NumFileInThisDir]->d_name);

	 lstat (PathFileRel,&FileStatus);

         /***** Construct the full path of the file or folder *****/
         Brw_ConstructCompletePathInTree (PathInTree,DirFileList[NumFileInThisDir]->d_name);

	 if (S_ISDIR (FileStatus.st_mode))	// It's a directory
	   {
            if (Gbl.FileBrowser.FullTree)
               ExpandTree = EXPAND_TREE_NOTHING;
            else
              {
               /***** Check if this subdirectory have files or folders in it *****/
               if ((NumFilesInThisSubdir = scandir (PathFileRel,&SubdirFileList,NULL,NULL)) <= 2)
                  ExpandTree = EXPAND_TREE_NOTHING;
               else
                  /***** Check if the tree starting at this subdirectory must be expanded *****/
                  ExpandTree = Brw_GetIfExpandedTree (Gbl.FileBrowser.Priv.FullPathInTree) ? EXPAND_TREE_MINUS : EXPAND_TREE_PLUS;
              }

            /***** Write a row for the subdirectory *****/
	    if (Brw_WriteRowFileBrowser (Level,IsLastItemInLevel,Brw_IS_FOLDER,ExpandTree,PathInTree,DirFileList[NumFileInThisDir]->d_name,FileStatus.st_size,FileStatus.st_mtime))
               if (ExpandTree == EXPAND_TREE_MINUS ||
                   ExpandTree == EXPAND_TREE_NOTHING)
                  if (Level < Brw_MAX_DIR_LEVELS)
                     /* List subtree starting at this this directory */
                     Brw_ListDir (Level+1,IsLastItemInLevel,PathFileRel,PathFileInExplTree);
	   }
	 else if (S_ISREG (FileStatus.st_mode))	// It's a regular file
	    Brw_WriteRowFileBrowser (Level,IsLastItemInLevel,Brw_IS_FILE,EXPAND_TREE_NOTHING,PathInTree,DirFileList[NumFileInThisDir]->d_name,FileStatus.st_size,FileStatus.st_mtime);
	}
  }

/*****************************************************************************/
/*********************** Write a row of a file browser ***********************/
/*****************************************************************************/
// If it is the first row (root folder), always show it
// If it is not the first row, it is shown or not depending on whether it is hidden or not
// If the row is visible, return true. If it is hidden, return false

static bool Brw_WriteRowFileBrowser (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS],Brw_FileType_t IsFileOrFolder,ExpandTree_t ExpandTree,const char *PathInTree,const char *FileName,off_t FileSize,time_t TimeLastModif)
  {
   bool HiddenRow = false;
   bool IsRecent = false;
   bool SeeDownload     = Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_DOWNLOAD_CRS ||
                          Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_DOWNLOAD_GRP;
   bool AdminDownload   = Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS ||
                          Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP;
   bool CommonZone      = Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_CRS ||
                          Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP;
   bool AssignmentsZone = Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_USR ||
                          Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS;
   bool AdminMarks      = Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_CRS ||
                          Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP;

   Gbl.FileBrowser.Clipboard.IsThisFile = false;

   /***** Is this row hidden or visible? *****/
   if (SeeDownload)
      if ((HiddenRow = Brw_GetIfHiddenDownload (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree)) && Level)
         return false;

   /***** Get data of assignment using the name of the folder *****/
   if (AssignmentsZone && Level == 1)
     {
      strncpy (Gbl.FileBrowser.Asg.Folder,FileName,Asg_MAX_LENGTH_FOLDER);
      Gbl.FileBrowser.Asg.Folder[Asg_MAX_LENGTH_FOLDER] = '\0';
      Asg_GetDataOfAssignmentByFolder (&Gbl.FileBrowser.Asg);
      // The data of this assignment remains in Gbl.FileBrowser.Asg
      // for all subsequent rows with Level > 1 (files or folders inside this folder),
      // and they are overwritten on the next row with level == 1 (next assignment)
     }

   /***** Start this row *****/
   fprintf (Gbl.F.Out,"<tr>");

   /****** If current action allows file administration... ******/
   if (Brw_FileBrowserIsEditable[Gbl.FileBrowser.Type])
     {
      if (Gbl.FileBrowser.Clipboard.IsThisTree)
	 // If path in the clipboard is equal to complete path in tree or is the start of complete path in tree...
         if (Str_Path1BeginsByPath2 (Gbl.FileBrowser.Priv.FullPathInTree,Gbl.FileBrowser.Clipboard.Path))
            Gbl.FileBrowser.Clipboard.IsThisFile = true;

      /* Put icons to remove, copy and paste */
      Brw_PutIconsRemoveCopyPaste (Level,IsFileOrFolder,PathInTree,FileName);
     }

   /***** Indentation depending on level, icon, and file/folder name *****/
   /* Start of the column */
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"99%%\" bgcolor=\"#%s\" valign=\"top\" class=\"NO_BR\">" \
                      "<table cellspacing=\"0\" cellpadding=\"0\"><tr>",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);

   /* Indent depending on level */
   if (Level)
      Brw_IndentAndWriteIconExpandContract (Level,IsLastItemInLevel,ExpandTree,PathInTree,FileName);

   /* Put icon to hide file or folder */
   if (AdminDownload)
     {
      if (Brw_GetIfHiddenDownload (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree))
        {
         if (IsFileOrFolder == Brw_IS_FOLDER)
            Gbl.FileBrowser.HiddenLevels[Level] = true;
         Brw_PutIconShow (Level,IsFileOrFolder,PathInTree,FileName);
        }
      else
        {
         if (IsFileOrFolder == Brw_IS_FOLDER)
            Gbl.FileBrowser.HiddenLevels[Level] = false;
         Brw_PutIconHide (Level,IsFileOrFolder,PathInTree,FileName);
        }
     }

   /***** File or folder icon *****/
   if (IsFileOrFolder == Brw_IS_FILE)
      /* Icon with file type */
      Brw_PutIconFile (FileName);
   else	// Brw_IS_FOLDER
      /* Icon with folder */
      Brw_PutIconFolder (Level,PathInTree,FileName,ExpandTree);

   /* Check if is a recent file or folder */
   // If less than a week since last modify ==> indicate the file is recent by writting its name in green
   if (Gbl.TimeStartExecution < TimeLastModif + (7L*24L*60L*60L))
      IsRecent = true;
   /* Check if is a new file or folder */
   // If our last access was before the last modify ==> indicate the file is new by putting a blinking star
   if (Gbl.Usrs.Me.TimeLastAccToThisFileBrowser < TimeLastModif)
      Brw_PutIconNewFileOrFolder ();

   /* File or folder name */
   Brw_WriteFileName (Level,IsFileOrFolder,PathInTree,FileName,IsRecent);

   /* End of the column */
   fprintf (Gbl.F.Out,"</tr></table></td>");

   /***** Extra columns *****/
   if (CommonZone)
      /* User who created the file or folder */
      Brw_WriteFileOrFolderCreator (Level,Gbl.FileBrowser.Priv.FullPathInTree,IsRecent);
   else if (AssignmentsZone)
      /* Start and end dates of assignment */
      Brw_WriteDatesAssignment (Level);
   else if (AdminMarks)
      /* Header and footer rows */
      Brw_GetAndWriteNumRowsHeaderAndFooter (IsFileOrFolder,Gbl.CurrentCrs.Grps.GrpCod,PathInTree,FileName,IsRecent);

   /***** File date and size *****/
   Brw_WriteFileSizeAndDate (IsFileOrFolder,FileSize,TimeLastModif,IsRecent);

   /***** End this row *****/
   fprintf (Gbl.F.Out,"</tr>");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;

   return !HiddenRow;
  }

/*****************************************************************************/
/*************** Construct full path in tree of file browser *****************/
/*****************************************************************************/
// If, for example, PathInTreeUntilFileOrFolder is "descarga/teoria" and FileOrFolderName is "tema1.pdf"
// then Gbl.FileBrowser.Priv.FullPathInTree will be "descarga/teoria/tema1.pdf"
// If FileOrFolderName is ".", then Gbl.FileBrowser.Priv.FullPathInTree will be equal to PathInTreeUntilFileOrFolder

static void Brw_ConstructCompletePathInTree (const char *PathInTreeUntilFileOrFolder,const char *FileOrFolderName)
  {
   if (PathInTreeUntilFileOrFolder[0] == '\0')
      Gbl.FileBrowser.Priv.FullPathInTree[0] = '\0';
   else if (strcmp (FileOrFolderName,"."))
      sprintf (Gbl.FileBrowser.Priv.FullPathInTree,"%s/%s",PathInTreeUntilFileOrFolder,FileOrFolderName);
   else	// It's the root folder
      strcpy (Gbl.FileBrowser.Priv.FullPathInTree,PathInTreeUntilFileOrFolder);
  }

/*****************************************************************************/
/****************** Put icons to remove, copy and paste **********************/
/*****************************************************************************/

static void Brw_PutIconsRemoveCopyPaste (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName)
  {
   /***** Icon to remove file or folder *****/
   if (IsFileOrFolder == Brw_IS_FILE)
      /* Icon to remove a file */
      Brw_PutIconRemoveFile (Level,PathInTree,FileName);
   else	// Brw_IS_FOLDER
      /* Icon to remove a folder */
      Brw_PutIconRemoveDir (Level,PathInTree,FileName);
   Brw_PutIconCopy (Level,PathInTree,FileName);

   /***** Icon to paste *****/
   if (IsFileOrFolder == Brw_IS_FILE)
      fprintf (Gbl.F.Out,"<td class=\"BM%d\"></td>",Gbl.GlobalRowEvenOdd);	// Can't paste in a file
   else	// Brw_IS_FOLDER
     {
      if (Brw_CheckIfCanPasteIn (Level))
         /* Icon to paste active */
         Brw_PutIconPasteOn (Level,PathInTree,FileName);
      else
         /* Icon to paste inactive */
         Brw_PutIconPasteOff ();
     }
  }

/*****************************************************************************/
/******* Check if the clipboard can be pasted into the current folder ********/
/*****************************************************************************/
// Return true if Gbl.FileBrowser.Clipboard.Path can be pasted into Gbl.FileBrowser.Priv.FullPathInTree

static bool Brw_CheckIfCanPasteIn (unsigned Level)
  {
   char PathDstWithFile[PATH_MAX+1];

   if (Gbl.FileBrowser.Clipboard.FileBrowser == Brw_FILE_BRW_UNKNOWN)	// If there is nothing in clipboard
      return false;

   if (!Brw_CheckIfICanCreateIntoFolder (Level))
      return false;	// Pasting into top level of assignments is forbidden

   if (Gbl.FileBrowser.Clipboard.IsThisTree)	// We are in the same tree of the clipboard ==> we can paste or not depending on the subtree
     {
      /***** Construct the name of the file or folder destination *****/
      sprintf (PathDstWithFile,"%s/%s",Gbl.FileBrowser.Priv.FullPathInTree,Gbl.FileBrowser.Clipboard.FileName);

      return !Str_Path1BeginsByPath2 (PathDstWithFile,Gbl.FileBrowser.Clipboard.Path);
     }

   return true;	// I can paste
  }

/*****************************************************************************/
/******************* Write link e icon to remove a file **********************/
/*****************************************************************************/

static void Brw_PutIconRemoveFile (unsigned Level,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Remove_file[Txt_NUM_LANGUAGES];
   bool ICanRemove;

   fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);

   if ((ICanRemove = Brw_CheckIfICanModifyFileOrFolder (Level)))
     {
      /***** Form to remove a file *****/
      Act_FormStart (Brw_ActAskRemoveFile[Gbl.FileBrowser.Type]);
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            Usr_HiddenParUsrCodAll (Brw_ActAskRemoveFile[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
            break;
         default:
            break;
        }
      Brw_ParamListFiles (PathInTree,FileName);
      sprintf (Gbl.MsgWindowStatus,"%s %s",
               Txt_Remove_file[Gbl.Prefs.Language],FileName);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
               Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
     }
   else
      fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
               Gbl.Prefs.IconsURL);
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/****************** Write link and icon to remove a folder *******************/
/*****************************************************************************/

static void Brw_PutIconRemoveDir (unsigned Level,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Remove_folder[Txt_NUM_LANGUAGES];
   bool ICanRemove;

   fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);

   if ((ICanRemove = Brw_CheckIfICanModifyFileOrFolder (Level)))
     {
      /***** Form to remove a folder *****/
      Act_FormStart (Brw_ActRemoveFolder[Gbl.FileBrowser.Type]);
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            Usr_HiddenParUsrCodAll (Brw_ActRemoveFolder[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
            break;
         default:
            break;
        }
      Brw_ParamListFiles (PathInTree,FileName);
      sprintf (Gbl.MsgWindowStatus,"%s %s",
               Txt_Remove_folder[Gbl.Prefs.Language],FileName);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
               Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
     }
   else
      fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
               Gbl.Prefs.IconsURL);
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/************** Write link e icon to copy a file o a folder ******************/
/*****************************************************************************/

static void Brw_PutIconCopy (unsigned Level,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Copy[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);

   /***** Form to copy into the clipboard *****/
   Act_FormStart (Brw_ActCopy[Gbl.FileBrowser.Type]);
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         Usr_HiddenParUsrCodAll (Brw_ActCopy[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
         Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
         break;
      default:
         break;
     }
   Brw_ParamListFiles (PathInTree,FileName);
   sprintf (Gbl.MsgWindowStatus,"%s %s",
            Txt_Copy[Gbl.Prefs.Language],
            Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/copy12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\" /></acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/*************** Write link e icon to paste a file o a folder ****************/
/*****************************************************************************/

static void Brw_PutIconPasteOn (unsigned Level,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Paste_in[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);

   /***** Form to paste the content of the clipboard *****/
   Act_FormStart (Brw_ActPaste[Gbl.FileBrowser.Type]);
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
         break;
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         Usr_HiddenParUsrCodAll (Brw_ActPaste[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
         Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
         break;
      default:
         break;
     }
   Brw_ParamListFiles (PathInTree,FileName);
   sprintf (Gbl.MsgWindowStatus,"%s %s",
            Txt_Paste_in[Gbl.Prefs.Language],
            Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/paste12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\" /></acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/**************** Write link e icon to paste a file o a folder ***************/
/*****************************************************************************/

static void Brw_PutIconPasteOff (void)
  {
   fprintf (Gbl.F.Out,"<td class=\"BM%d\"><img src=\"%s/paste_off12x12.gif\" width=\"12\" height=\"12\" /></td>",
            Gbl.GlobalRowEvenOdd,Gbl.Prefs.IconsURL);
  }

/*****************************************************************************/
/*************** Indent and write icon to expand/contract folder *************/
/*****************************************************************************/

static void Brw_IndentAndWriteIconExpandContract (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS],ExpandTree_t ExpandTree,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Expand_FOLDER[Txt_NUM_LANGUAGES];
   extern const char *Txt_Contract_FOLDER[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td align=\"left\"><table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   Brw_IndentDependingOnLevel (Level,IsLastItemInLevel);
   fprintf (Gbl.F.Out,"<td width=\"12\">");

   switch (ExpandTree)
     {
      case EXPAND_TREE_NOTHING:
         fprintf (Gbl.F.Out,"<img src=\"%s/%s12x16.gif\" width=\"12\" height=\"16\" /></td></tr></table>",
                  Gbl.Prefs.IconsURL,IsLastItemInLevel[Level] ? "subend" : "submid");
         break;
      case EXPAND_TREE_PLUS:
         /***** Form to expand folder *****/
         Act_FormStart (Brw_ActExpandFolder[Gbl.FileBrowser.Type]);
         switch (Gbl.FileBrowser.Type)
           {
            case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
            case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
            case Brw_FILE_BRW_COMMON_GRP:
            case Brw_FILE_BRW_SEE_MARKS_GRP:
            case Brw_FILE_BRW_ADMIN_MARKS_GRP:
               Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
               break;
            case Brw_FILE_BRW_ASSIGNMENTS_CRS:
            case Brw_FILE_BRW_WORKS_CRS:
               Usr_HiddenParUsrCodAll (Brw_ActExpandFolder[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
               Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
               break;
            default:
               break;
           }
         Brw_ParamListFiles (PathInTree,FileName);
         sprintf (Gbl.MsgWindowStatus,"%s %s",
                  Txt_Expand_FOLDER[Gbl.Prefs.Language],
                  Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/%s12x16.gif\" alt=\"%s\" width=\"12\" height=\"16\" /></acronym></a></form></td></tr></table>",
                  Gbl.Prefs.IconsURL,IsLastItemInLevel[Level] ? "subendplus" : "submidplus",Gbl.MsgWindowStatus);
         break;
      case EXPAND_TREE_MINUS:
         /***** Form to contract folder *****/
         Act_FormStart (Brw_ActContractFolder[Gbl.FileBrowser.Type]);
         switch (Gbl.FileBrowser.Type)
           {
            case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
            case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
            case Brw_FILE_BRW_COMMON_GRP:
            case Brw_FILE_BRW_SEE_MARKS_GRP:
            case Brw_FILE_BRW_ADMIN_MARKS_GRP:
               Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
               break;
            case Brw_FILE_BRW_ASSIGNMENTS_CRS:
            case Brw_FILE_BRW_WORKS_CRS:
               Usr_HiddenParUsrCodAll (Brw_ActContractFolder[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
               Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
               break;
            default:
               break;
           }
         Brw_ParamListFiles (PathInTree,FileName);
         sprintf (Gbl.MsgWindowStatus,"%s %s",
                  Txt_Contract_FOLDER[Gbl.Prefs.Language],
                  Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/%s12x16.gif\" alt=\"%s\" width=\"12\" height=\"16\" /></acronym></a></form></td></tr></table>",
                  Gbl.Prefs.IconsURL,IsLastItemInLevel[Level] ? "subendminus" : "submidminus",Gbl.MsgWindowStatus);
         break;
     }

   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/************************* Indent depending on level *************************/
/*****************************************************************************/

static void Brw_IndentDependingOnLevel (unsigned Level,bool IsLastItemInLevel[1+Brw_MAX_DIR_LEVELS])
  {
   unsigned i;

   for (i=1; i<Level; i++)
      fprintf (Gbl.F.Out,"<td width=\"12\"><img src=\"%s/%s12x16.gif\" width=\"12\" height=\"16\" /></td>",
               Gbl.Prefs.IconsURL,IsLastItemInLevel[i] ? "tr" : "subleft");
  }

/*****************************************************************************/
/****************** Put link and icon to show file or folder *****************/
/*****************************************************************************/

static void Brw_PutIconShow (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Show[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
   if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS)
      Act_FormStart (ActShoDowCrs);
   else	// Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
     {
      Act_FormStart (ActShoDowGrp);
      Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
     }
   Brw_ParamListFiles (PathInTree,FileName);
   sprintf (Gbl.MsgWindowStatus,"%s %s",
            Txt_Show[Gbl.Prefs.Language],
            (IsFileOrFolder == Brw_IS_FILE) ? FileName :
                                              Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/%s12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\" /></acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Brw_CheckIfAnyUpperLevelIsHidden (Level) ? "proh_off" : "proh",Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/****************** Put link and icon to hide file or folder *****************/
/*****************************************************************************/

static void Brw_PutIconHide (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName)
  {
   extern const char *Txt_Hide[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
   if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS)
      Act_FormStart (ActHidDowCrs);
   else	// Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
     {
      Act_FormStart (ActHidDowGrp);
      Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
     }
   Brw_ParamListFiles (PathInTree,FileName);
   sprintf (Gbl.MsgWindowStatus,"%s %s",
            Txt_Hide[Gbl.Prefs.Language],
            (IsFileOrFolder == Brw_IS_FILE) ? FileName :
                                              Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/%s12x12.gif\" alt=\"%s\" width=\"12\" height=\"12\" /></acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Brw_CheckIfAnyUpperLevelIsHidden (Level) ? "finp_off" : "finp",Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/**** Check if any level of folders superior to the current one is hidden ****/
/*****************************************************************************/

static bool Brw_CheckIfAnyUpperLevelIsHidden (unsigned CurrentLevel)
  {
   unsigned N;

   for (N=0; N<CurrentLevel; N++)
      if (Gbl.FileBrowser.HiddenLevels[N])
         return true;

   return false;
  }

/*****************************************************************************/
/**************** Write link e icon to create a file or folder ***************/
/*****************************************************************************/

static void Brw_PutIconFolder (unsigned Level,const char *PathInTree,const char *FileName,ExpandTree_t ExpandTree)
  {
   extern const char *Txt_Upload_file_or_create_folder_in_FOLDER[Txt_NUM_LANGUAGES];
   bool ICanCreate;

   /***** Start cell *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\">",Level * 12);

   /***** Put icon *****/
   if ((ICanCreate = Brw_CheckIfICanCreateIntoFolder (Level)))	// I can create a new file or folder
     {
      /***** Form to create a new file or folder *****/
      Act_FormStart (Brw_ActFormCreate[Gbl.FileBrowser.Type]);
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
	    Usr_HiddenParUsrCodAll (Brw_ActFormCreate[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
            break;
         default:
            break;
        }
      Brw_ParamListFiles (PathInTree,FileName);
      sprintf (Gbl.MsgWindowStatus,Txt_Upload_file_or_create_folder_in_FOLDER[Gbl.Prefs.Language],
               Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/folder_%s_plus12x16.gif\" alt=\"%s\" width=\"12\" height=\"16\" /></acronym></a></form>",
               Gbl.Prefs.IconsURL,(ExpandTree == EXPAND_TREE_PLUS) ? "closed" : "open",Gbl.MsgWindowStatus);
     }
   else	// I can't create a new file or folder
      fprintf (Gbl.F.Out,"<img src=\"%s/folder_%s12x16.gif\" width=\"12\" height=\"16\" />",
               Gbl.Prefs.IconsURL,(ExpandTree == EXPAND_TREE_PLUS) ? "closed" : "open");

   /***** End cell *****/
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/**************** Write icon to indicate that the file is new ****************/
/*****************************************************************************/

static void Brw_PutIconNewFileOrFolder (void)
  {
   /***** Icon that indicates new file *****/
   fprintf (Gbl.F.Out,"<td class=\"BM%d\"><img src=\"%s/nue12x12.gif\" width=\"12\" height=\"12\" /></td>",
            Gbl.GlobalRowEvenOdd,Gbl.Prefs.IconsURL);
  }

/*****************************************************************************/
/************ Write central part with the icon of a file or folder ***********/
/*****************************************************************************/

static void Brw_PutIconFile (const char *FileName)
  {
   int DocType;
   bool NotFound = true;

   /***** Icon depending on the file extension *****/
   fprintf (Gbl.F.Out,"<td class=\"BM%d\"><img src=\"%s/%s/",
            Gbl.GlobalRowEvenOdd,Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_FILEXT);
   for (DocType=0; DocType<Brw_NUM_FILE_EXT_ALLOWED && NotFound; DocType++)
      if (Str_FileIs (FileName,Brw_FileExtensionsAllowed[DocType]))
	{
	 fprintf (Gbl.F.Out,"%s12x12.gif\" alt=\"Archivo %s\"",
                  Brw_FileExtensionsAllowed[DocType],
                  Brw_FileExtensionsAllowed[DocType]);
         NotFound = false;
	}
   if (NotFound)
      fprintf (Gbl.F.Out,"xxx12x12.gif\"");
   fprintf (Gbl.F.Out," width=\"12\" height=\"12\" /></td>");
  }

/*****************************************************************************/
/********* Write parte central with the name of a file or folder *************/
/*****************************************************************************/

static void Brw_WriteFileName (unsigned Level,Brw_FileType_t IsFileOrFolder,const char *PathInTree,const char *FileName,bool IsRecent)
  {
   extern const char *Txt_Check_marks_in_file_X[Txt_NUM_LANGUAGES];
   bool ICanRename;
   char URLWithSpaces[PATH_MAX+1];
   char URLWithoutSpaces[PATH_MAX+1];

   /***** Name and link of the file or folder *****/
   if (IsFileOrFolder == Brw_IS_FILE)
     {
      if (Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_CRS ||
          Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_GRP)
        {
	 /***** On the screen a link will be shown to view marks *****/
         fprintf (Gbl.F.Out,"<td align=\"left\" width=\"99%%\"");
         if (Gbl.FileBrowser.Clipboard.IsThisFile)
            fprintf (Gbl.F.Out," bgcolor=\"#%s\"",LIGHT_GREEN);
         fprintf (Gbl.F.Out,">");
         /* Form start */
         if (Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_CRS)
	    Act_FormStart (ActSeeMyAssCrs);
         else // Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_GRP
           {
	    Act_FormStart (ActSeeMyAssGrp);
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
           }
	 Brw_ParamListFiles (PathInTree,FileName);
         /* Start link */
         sprintf (Gbl.MsgWindowStatus,Txt_Check_marks_in_file_X[Gbl.Prefs.Language],
                  FileName);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,IsRecent ? "LST_REC" : "LST");
	 /* Name of the file of marks and end of the link and of the form */
         fprintf (Gbl.F.Out,"&nbsp;%s</acronym></a></form></td>",FileName);
        }
      else
        {
	 /***** On the screen a link will be shown to download the file *****/
         /* Create symbolic link from temporary public directory to private file in order to gain access to it for downloading */
         Brw_CreateTmpLinkFileBrowser (PathInTree,FileName);

         /* Create href to symbolic link */
         sprintf (URLWithSpaces,"%s/%s/%s/%s",Cfg_URL_FILE_BROWSER_TMP_PUBLIC,Gbl.FileBrowser.TmpPubDir,PathInTree,
                  Level ? FileName : Brw_RootFolderInternalNames[Gbl.FileBrowser.Type]);
         Brw_CopyStrChangingSpaces (URLWithSpaces,URLWithoutSpaces,PATH_MAX);	// In XHTML, URL must have no spaces

         /* Put anchor and filename */
         fprintf (Gbl.F.Out,"<td align=\"left\" width=\"99%%\"");
         if (Gbl.FileBrowser.Clipboard.IsThisFile)
            fprintf (Gbl.F.Out," bgcolor=\"#%s\"",LIGHT_GREEN);
         fprintf (Gbl.F.Out,"><a href=\"%s\" class=\"%s\" target=\"_blank\">" \
                            "&nbsp;%s</a></td>",
                  URLWithoutSpaces,IsRecent ? "LST_REC" : "LST",FileName);
        }
     }
   else	// Brw_IS_FOLDER
     {
      /***** Start of cell *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"99%%\"");
      if (Gbl.FileBrowser.Clipboard.IsThisFile)
         fprintf (Gbl.F.Out," bgcolor=\"#%s\"",LIGHT_GREEN);
      fprintf (Gbl.F.Out," class=\"%s\">",
               IsRecent ? "LST_REC" : "LST");

      /***** Form to rename folder *****/
      if ((ICanRename = Brw_CheckIfICanModifyFileOrFolder (Level)))
	{
         Act_FormStart (Brw_ActRenameFolder[Gbl.FileBrowser.Type]);
         switch (Gbl.FileBrowser.Type)
           {
            case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
            case Brw_FILE_BRW_COMMON_GRP:
            case Brw_FILE_BRW_ADMIN_MARKS_GRP:
               Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
               break;
            case Brw_FILE_BRW_ASSIGNMENTS_CRS:
            case Brw_FILE_BRW_WORKS_CRS:
	       Usr_HiddenParUsrCodAll (Brw_ActRenameFolder[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
               Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
               break;
            default:
               break;
           }
	 Brw_ParamListFiles (PathInTree,FileName);
	}

      /***** Create folder in the public temporary place *****/
      if (Gbl.FileBrowser.Type != Brw_FILE_BRW_SEE_MARKS_CRS &&
          Gbl.FileBrowser.Type != Brw_FILE_BRW_SEE_MARKS_GRP)
         Brw_CreatePublicDirForDownload (PathInTree,FileName);

      /***** Write name of the folder *****/
      fprintf (Gbl.F.Out,"&nbsp;");
      if (ICanRename)
      	 fprintf (Gbl.F.Out,"<input type=\"text\" name=\"NewFolderName\" size=\"40\" maxlength=\"40\" value=\"%s\"" \
                            " class=\"%s\" style=\"background-color:#%s\" onchange=\"javascript:document.forms[%d].submit();\" /></form>",
		  FileName,IsRecent ? "LST_EDIT_REC" : "LST_EDIT",
                  Gbl.FileBrowser.Clipboard.IsThisFile ? LIGHT_GREEN : Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
                  Gbl.NumForm);
      else
        {
         if ((Level == 1) &&
             (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_USR ||
              Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS))
            fprintf (Gbl.F.Out,"<span title=\"%s\">",
                     Gbl.FileBrowser.Asg.Title); 
         fprintf (Gbl.F.Out,"<strong>%s</strong>",
                  Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Level,FileName));
         if ((Level == 1) &&
             (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_USR ||
              Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS))
            fprintf (Gbl.F.Out,"</span>"); 
        }

      /***** End of cell *****/
      fprintf (Gbl.F.Out,"</td>");
     }
  }

/*****************************************************************************/
/*********************** Which filename must be shown? ***********************/
/*****************************************************************************/

static const char *Brw_GetFileNameToShow (Brw_FileBrowser_t FileBrowser,unsigned Level,const char *FileName)
  {
   extern const char *Txt_ROOT_FOLDER_EXTERNAL_NAMES[Brw_NUM_TYPES_FILE_BROWSER][Txt_NUM_LANGUAGES];

   return (Level ? FileName : Txt_ROOT_FOLDER_EXTERNAL_NAMES[FileBrowser][Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/************* Copy a string to another changing ' ' to '%20' ****************/
/*****************************************************************************/

static void Brw_CopyStrChangingSpaces (const char *StringWithSpaces,char *StringWithoutSpaces,unsigned MaxLength)
  {
   const char *PtrSrc;
   char *PtrDst;
   unsigned Length = 0;

   for (PtrSrc = StringWithSpaces, PtrDst = StringWithoutSpaces; *PtrSrc && Length<=MaxLength; PtrSrc++)
      if (*PtrSrc == ' ')
        {
         Length += 3;
         if (Length <= MaxLength)
           {
            *PtrDst++ = '%';
            *PtrDst++ = '2';
            *PtrDst++ = '0';
           }
        }
      else
        {
         Length++;
         if (Length <= MaxLength)
            *PtrDst++ = *PtrSrc;
        }
   *PtrDst = '\0';

   if (Length > MaxLength)
      Lay_ShowErrorAndExit ("Path is too long.");
  }

/*****************************************************************************/
/********** Create a link public temporary to download          **************/
/********** and otro link temporary to compression zip of works **************/
/*****************************************************************************/

static void Brw_CreateTmpLinkFileBrowser (const char *PathInTree,const char *FileName)
  {
   char PathAbsPrivate[PATH_MAX+1];
   char LinkRelFromCGI_BIN[PATH_MAX+1];

   /***** Construct absolute path to file in the private directory *****/
   sprintf (PathAbsPrivate,"%s/%s/%s",Gbl.FileBrowser.Priv.PathAbsAboveRootFolder,PathInTree,FileName);

   /***** Create, into temporary download directory, a symbolic link to file *****/
   sprintf (LinkRelFromCGI_BIN,"%s/%s/%s/%s",Cfg_PATH_REL_FILE_BROWSER_TMP_PUBLIC,Gbl.FileBrowser.TmpPubDir,PathInTree,FileName);
   if (symlink (PathAbsPrivate,LinkRelFromCGI_BIN) != 0)
      Lay_ShowErrorAndExit ("Can not create temorary link for download.");
  }

/*****************************************************************************/
/************ Create a temporary public folder for downloading ***************/
/*****************************************************************************/

static void Brw_CreatePublicDirForDownload (const char *PathInTree,const char *FileName)
  {
   char PathRelFromCGI_BIN[PATH_MAX+1];

   /***** Create, inside the public temporary directory of download, a folder *****/
   sprintf (PathRelFromCGI_BIN,"%s/%s/%s/%s",Cfg_PATH_REL_FILE_BROWSER_TMP_PUBLIC,Gbl.FileBrowser.TmpPubDir,PathInTree,FileName);
   Fil_CreateDirIfNotExists (PathRelFromCGI_BIN);
  }

/*****************************************************************************/
/***************** Write parameters of a row of file list ********************/
/*****************************************************************************/

static void Brw_ParamListFiles (const char *PathInTree,const char *FileName)
  {
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
	 Brw_ParamSeeDownloadZoneCrs ();
	 break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
	 Brw_ParamSeeDownloadZoneGrp ();
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
	 Brw_ParamAdminDownloadZoneCrs ();
	 break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
	 Brw_ParamAdminDownloadZoneGrp ();
	 break;
      case Brw_FILE_BRW_COMMON_CRS:
	 Brw_ParamAdminCommonZoneCrs ();
	 break;
      case Brw_FILE_BRW_COMMON_GRP:
	 Brw_ParamAdminCommonZoneGrp ();
	 break;
      case Brw_FILE_BRW_SEE_MARKS_CRS:
	 Brw_ParamSeeAssZoneCrs ();
	 break;
      case Brw_FILE_BRW_SEE_MARKS_GRP:
	 Brw_ParamSeeAssZoneGrp ();
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
	 Brw_ParamAdmAssZoneCrs ();
	 break;
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
	 Brw_ParamAdmAssZoneGrp ();
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
	 Brw_ParamAssignmentsZoneUsr ();
	 break;
      case Brw_FILE_BRW_WORKS_USR:
	 Brw_ParamWorksZoneUsr ();
	 break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
	 Brw_ParamAssignmentsZoneCrs ();
	 break;
      case Brw_FILE_BRW_WORKS_CRS:
	 Brw_ParamWorksZoneCrs ();
	 break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
	 Brw_ParamBriefZoneUsr ();
	 break;
      default:
	 break;
     }
   Brw_PutParamsPathAndFile (PathInTree,FileName);
   if (Gbl.FileBrowser.FullTree)
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FullTree\" value=\"Y\" />");
  }

static void Brw_ParamSeeDownloadZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_SEE_DOWNLOAD_CRS);
  }

static void Brw_ParamSeeDownloadZoneGrp (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_SEE_DOWNLOAD_GRP);
  }

static void Brw_ParamAdminDownloadZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS);
  }

static void Brw_ParamAdminDownloadZoneGrp (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP);
  }

static void Brw_ParamAdminCommonZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_COMMON_CRS);
  }

static void Brw_ParamAdminCommonZoneGrp (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_COMMON_GRP);
  }

void Brw_ParamSeeAssZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_SEE_MARKS_CRS);
  }

void Brw_ParamSeeAssZoneGrp (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_SEE_MARKS_GRP);
  }

void Brw_ParamAdmAssZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_ADMIN_MARKS_CRS);
  }

void Brw_ParamAdmAssZoneGrp (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_ADMIN_MARKS_GRP);
  }

void Brw_ParamAssignmentsZoneUsr (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_ASSIGNMENTS_USR);
  }

void Brw_ParamWorksZoneUsr (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_WORKS_USR);
  }

void Brw_ParamAssignmentsZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_ASSIGNMENTS_CRS);
  }

void Brw_ParamWorksZoneCrs (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_WORKS_CRS);
  }

void Brw_ParamBriefZoneUsr (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"BrwTyp\" value=\"%d\" />",
            Brw_FILE_BRW_BRIEFCASE_USR);
  }

/*****************************************************************************/
/******** Get number of rows of header and of footer of a file of marks ******/
/*****************************************************************************/

static void Brw_GetNumRowsHeaderAndFooter (unsigned *Header,unsigned *Footer)
  {
   char Query[256+PATH_MAX];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get number of rows of header and footer from database *****/
   sprintf (Query,"SELECT %s,%s FROM marks" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            HeadOrFootStr[Brw_HEADER],HeadOrFootStr[Brw_FOOTER],
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the number of rows in header and footer");

   /***** The result of the query must have only one row *****/
   if (NumRows == 1)
     {
      /***** Get number of header and footer rows *****/
      row = mysql_fetch_row (mysql_res);

      /* Header (row[0]) */
      if (sscanf (row[0],"%u",Header) != 1)
         Lay_ShowErrorAndExit ("Wrong number of header rows.");

      /* Footer (row[1]) */
      if (sscanf (row[1],"%u",Footer) != 1)
         Lay_ShowErrorAndExit ("Wrong number of footer rows.");
     }
   else if (NumRows == 0)
     {
      Brw_AddMarksToDB (Gbl.Marks.Header,Gbl.Marks.Footer);
      *Header = Gbl.Marks.Header;
      *Footer = Gbl.Marks.Footer;
     }
   else	// NumRows > 1
      Lay_ShowErrorAndExit ("Error when getting the number of rows in header and footer.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/****************** Add a new entry of marks into database *******************/
/*****************************************************************************/

static void Brw_AddMarksToDB (unsigned Header,unsigned Footer)
  {
   char Query[512+PATH_MAX];

   /***** Add file of marks to the database *****/
   sprintf (Query,"INSERT INTO marks (CrsCod,GrpCod,Path,%s,%s)" \
                  " VALUES ('%ld','%ld','%s','%u','%u')",
            HeadOrFootStr[Brw_HEADER],HeadOrFootStr[Brw_FOOTER],
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,
            Gbl.FileBrowser.Priv.FullPathInTree,Header,Footer);
   DB_QueryINSERT (Query,"can not add file of marks to database");
  }

/*****************************************************************************/
/**************** Remove from database an entry of marks *********************/
/*****************************************************************************/

static void Brw_RemoveOneGradesFileFromDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove a file of marks from the database *****/
   sprintf (Query,"DELETE FROM marks WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryDELETE (Query,"can not remove file of marks from database");
  }

/*****************************************************************************/
/******** Remove from database the children of a folder of marks *************/
/*****************************************************************************/

static void Brw_RemoveChildrenOfMarksFolderFromDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove children of a folder of marks from the database *****/
   sprintf (Query,"DELETE FROM marks WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryDELETE (Query,"can not remove files of marks from database");
  }

/*****************************************************************************/
/*********** Rename a folder of marks in table of files of marks *************/
/*****************************************************************************/

static void Brw_RenameChildrenMarksFileInDB (long GrpCod,const char *OldPath,const char *NewPath)
  {
   char Query[512+PATH_MAX*2];
   unsigned StartFinalSubpathNotChanged = strlen (OldPath) + 2;

   /***** Update the children of a folder in the table of files of marks *****/
   sprintf (Query,"UPDATE marks SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
            NewPath,StartFinalSubpathNotChanged,Gbl.CurrentCrs.Crs.CrsCod,GrpCod,OldPath);
   DB_QueryUPDATE (Query,"can not update files of marks in database");
  }

/*****************************************************************************/
/********* Change the number of rows of header of a file of marks ************/
/*****************************************************************************/

void Brw_ChangeNumRowsHeader (void)
  {
   Brw_ChangeNumRowsHeaderOrFooter (Brw_HEADER);
  }

/*****************************************************************************/
/********** Change the number of rows of footer of a file of marks ***********/
/*****************************************************************************/

void Brw_ChangeNumRowsFooter (void)
  {
   Brw_ChangeNumRowsHeaderOrFooter (Brw_FOOTER);
  }

/*****************************************************************************/
/***** Change the number of rows of header or footer of a file of marks ******/
/*****************************************************************************/

static void Brw_ChangeNumRowsHeaderOrFooter (Brw_HeadOrFoot_t HeaderOrFooter)
  {
   extern const char *Txt_The_number_of_rows_is_now_X[Txt_NUM_LANGUAGES];
   char UnsignedStr[10+1];
   char Query[256+PATH_MAX];
   unsigned NumRows;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Get the number of rows of the header or footer of the table of marks *****/
   Par_GetParam (Par_PARAM_SINGLE,HeadOrFootStr[HeaderOrFooter],UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&NumRows) == 1)
     {
      /***** Update file of marks in the database *****/
      sprintf (Query,"UPDATE marks SET %s='%u' WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
               HeadOrFootStr[HeaderOrFooter],NumRows,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
      DB_QueryUPDATE (Query,"can not update number of rows in header");

      /***** Write message of success *****/
      sprintf (Gbl.Message,Txt_The_number_of_rows_is_now_X[Gbl.Prefs.Language],
               NumRows);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
      Lay_ShowErrorAndExit ("Wrong number of rows.");

   /***** Show again the file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/************ Write start and end dates of a folder of assignment ************/
/*****************************************************************************/

static void Brw_WriteDatesAssignment (unsigned Level)
  {
   extern const char *Txt_unknown_assignment[Txt_NUM_LANGUAGES];

   if (Level == 1)
     {
      if (Gbl.FileBrowser.Asg.AsgCod >= 0)
        {
         /***** Write start date *****/
         fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">&nbsp;%02u/%02u/%02u&nbsp;%02u:%02u</td>",
                  Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
                  Gbl.FileBrowser.Asg.Open ? "ASG_LST_DATE_GREEN" : "ASG_LST_DATE_RED",
                  Gbl.FileBrowser.Asg.DateTimes[Asg_START_TIME].Day,
                  Gbl.FileBrowser.Asg.DateTimes[Asg_START_TIME].Month,
	          Gbl.FileBrowser.Asg.DateTimes[Asg_START_TIME].Year % 100,
	          Gbl.FileBrowser.Asg.DateTimes[Asg_START_TIME].Hour,
	          Gbl.FileBrowser.Asg.DateTimes[Asg_START_TIME].Minute);

         /***** Arrow *****/
         fprintf (Gbl.F.Out,"<td align=\"center\" width=\"16\" bgcolor=\"#%s\">" \
                            "<img src=\"%s/arrow%s16x12.gif\" width=\"16\" height=\"12\" alt=\"\" align=\"bottom\" />" \
                            "</td>",
                  Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
                  Gbl.Prefs.IconsURL,
                  Gbl.FileBrowser.Asg.Open ? "green" : "red");

         /***** Write start date *****/
         fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">%02u/%02u/%02u&nbsp;%02u:%02u</td>",
                  Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
                  Gbl.FileBrowser.Asg.Open ? "ASG_LST_DATE_GREEN" : "ASG_LST_DATE_RED",
                  Gbl.FileBrowser.Asg.DateTimes[Asg_END_TIME  ].Day,
                  Gbl.FileBrowser.Asg.DateTimes[Asg_END_TIME  ].Month,
                  Gbl.FileBrowser.Asg.DateTimes[Asg_END_TIME  ].Year % 100,
	          Gbl.FileBrowser.Asg.DateTimes[Asg_END_TIME  ].Hour,
	          Gbl.FileBrowser.Asg.DateTimes[Asg_END_TIME  ].Minute);
        }
      else
         fprintf (Gbl.F.Out,"<td colspan=\"3\" align=\"left\" bgcolor=\"#%s\" class=\"ASG_LST_DATE_GREEN\">&nbsp;(%s)</td>",
                  Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
                  Txt_unknown_assignment[Gbl.Prefs.Language]);
     }
   else
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\"></td>" \
                         "<td align=\"center\" width=\"16\" bgcolor=\"#%s\"></td>" \
                         "<td align=\"left\" valign=\"top\" bgcolor=\"#%s\"></td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
  }

/*****************************************************************************/
/********* Write number of header and footer rows of a file of marks *********/
/*****************************************************************************/

static void Brw_GetAndWriteNumRowsHeaderAndFooter (Brw_FileType_t IsFileOrFolder,long GrpCod,const char *PathInTree,const char *FileName,bool IsRecent)
  {
   extern const char *The_ClassFormulNB[The_NUM_THEMES];
   extern const char *Txt_TABLE_Header[Txt_NUM_LANGUAGES];
   extern const char *Txt_TABLE_Footer[Txt_NUM_LANGUAGES];
   unsigned Header;
   unsigned Footer;

   if (IsFileOrFolder == Brw_IS_FILE)
     {
      /***** Get number of rows in header or footer *****/
      Brw_GetNumRowsHeaderAndFooter (&Header,&Footer);

      /***** Write the number of rows of header *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],The_ClassFormulNB[Gbl.Prefs.Theme]);

      if (GrpCod >= 0)	// Group zone
        {
         Act_FormStart (ActChgNumRowHeaGrp);
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
        }
      else			// Course zone
         Act_FormStart (ActChgNumRowHeaCrs);

      fprintf (Gbl.F.Out,"&nbsp;%s: " \
                         "<input type=\"text\" name=\"%s\" size=\"1\" maxlength=\"5\" value=\"%u\"" \
                         " class=\"%s\" style=\"background-color:#%s\" onchange=\"javascript:document.forms[%d].submit();\" />",
               Txt_TABLE_Header[Gbl.Prefs.Language],
               HeadOrFootStr[Brw_HEADER],Header,
               IsRecent ? "LST_EDIT_REC" : "LST_EDIT",Gbl.ColorRows[Gbl.GlobalRowEvenOdd],Gbl.NumForm);
      Brw_ParamListFiles (PathInTree,FileName);
      fprintf (Gbl.F.Out,"</form></td>");

      /***** Write the number of rows of footer *****/
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],The_ClassFormulNB[Gbl.Prefs.Theme]);

      if (GrpCod >= 0)	// Group zone
        {
         Act_FormStart (ActChgNumRowFooGrp);
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
        }
      else			// Course zone
         Act_FormStart (ActChgNumRowFooCrs);

      fprintf (Gbl.F.Out,"&nbsp;%s: " \
                         "<input type=\"text\" name=\"%s\" size=\"1\" maxlength=\"5\" value=\"%u\"" \
                         " class=\"%s\" style=\"background-color:#%s\" onchange=\"javascript:document.forms[%d].submit();\" />",
               Txt_TABLE_Footer[Gbl.Prefs.Language],
               HeadOrFootStr[Brw_FOOTER],Footer,
               IsRecent ? "LST_EDIT_REC" : "LST_EDIT",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],Gbl.NumForm);
      Brw_ParamListFiles (PathInTree,FileName);
      fprintf (Gbl.F.Out,"</form></td>");
     }
   else	// IsFileOrFolder == Brw_IS_FOLDER
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" bgcolor=\"#%s\"></td>" \
                         "<td align=\"right\" valign=\"top\" bgcolor=\"#%s\"></td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
  }

/*****************************************************************************/
/****************** Write size and date of a file or folder ******************/
/*****************************************************************************/

static void Brw_WriteFileSizeAndDate (Brw_FileType_t IsFileOrFolder,off_t FileSize,time_t TimeLastModif,bool IsRecent)
  {
   /***** Write the file size *****/
   /* fprintf (Gbl.F.Out,"<td align=\"right\" width=\"50\" bgcolor=\"#%s\" class=\"%s\">&nbsp;",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd],IsRecent ? "LST_REC" : "LST"); */
   fprintf (Gbl.F.Out,"<td align=\"right\" bgcolor=\"#%s\" class=\"%s\">&nbsp;",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd],IsRecent ? "LST_REC" : "LST");
   if (IsFileOrFolder == Brw_IS_FILE)
      Str_WriteSizeInBytes ((double) FileSize);
   fprintf (Gbl.F.Out,"</td>");

   /***** Write the date *****/
   Dat_GetLocalTimeFromClock (&TimeLastModif);
   /* fprintf (Gbl.F.Out,"<td align=\"left\" width=\"75\" bgcolor=\"#%s\" class=\"%s\">&nbsp;",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd],IsRecent ? "LST_REC" : "LST"); */
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">&nbsp;",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd],IsRecent ? "LST_REC" : "LST");
   Dat_WriteDateFromtblock ();
   fprintf (Gbl.F.Out,"&nbsp;</td>");
  }

/*****************************************************************************/
/*************** Write the user who created the file or folder ***************/
/*****************************************************************************/

static void Brw_WriteFileOrFolderCreator (unsigned Level,const char *Path,bool IsRecent)
  {
   bool ShowUsr = false;
   bool ShowPhoto;
   char PhotoURL[PATH_MAX+1];
   char FirstName[Usr_MAX_BYTES_NAME+1];
   char Surnames[2*(Usr_MAX_BYTES_NAME+1)];

   if (Level)
     {
      /***** Get data of file/folder owner *****/
      Gbl.Usrs.Other.UsrDat.UsrCod = Brw_GetCreatorFileOrFolderCommon (Gbl.CurrentCrs.Grps.GrpCod,Path);
      ShowUsr = Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat);	// Get user's data from database
     }

   if (ShowUsr)
     {
      /***** Show photo *****/
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (&Gbl.Usrs.Other.UsrDat,PhotoURL);
      fprintf (Gbl.F.Out,"<td class=\"BM%d\">",Gbl.GlobalRowEvenOdd);
      Usr_ShowUsrPhoto (&Gbl.Usrs.Other.UsrDat,ShowPhoto ? PhotoURL : NULL,12,16,true);
      fprintf (Gbl.F.Out,"</td>");

      /***** Write name *****/
      strcpy (FirstName,Gbl.Usrs.Other.UsrDat.FirstName);
      strcpy (Surnames,Gbl.Usrs.Other.UsrDat.Surname1);
      if (Gbl.Usrs.Other.UsrDat.Surname2[0])
        {
         strcat (Surnames,"&nbsp;");
         strcat (Surnames,Gbl.Usrs.Other.UsrDat.Surname2);
        }
      Str_LimitLengthHTMLStr (FirstName,10);
      Str_LimitLengthHTMLStr (Surnames,15);
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">%s,&nbsp;%s</td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
               IsRecent ? "LST_REC" : "LST",
               Surnames,FirstName);
     }
   else
      /***** Don't show user's data *****/
      fprintf (Gbl.F.Out,"<td class=\"BM%d\"></td>" \
                         "<td align=\"left\" bgcolor=\"#%s\"></td>",
               Gbl.GlobalRowEvenOdd,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
  }

/*****************************************************************************/
/********************* Remove old temporary directories **********************/
/*****************************************************************************/

void Brw_RemoveOldTmpFiles (const char *Path,bool RemoveDirectory)
  {
   struct dirent **FileList;
   int NumFile,NumFilesInsideDir;
   char Path2[PATH_MAX+1];
   struct stat FileStatus;

   lstat (Path,&FileStatus);
   if (S_ISDIR (FileStatus.st_mode))		// It's a directory
     {
      /***** Scan the directory *****/
      NumFilesInsideDir = scandir (Path,&FileList,NULL,NULL);

      /* Loop over files */
      for (NumFile=0; NumFile<NumFilesInsideDir; NumFile++)
	 if (strcmp (FileList[NumFile]->d_name,".") &&
             strcmp (FileList[NumFile]->d_name,".."))	// Skip directories "." and ".."
	   {
	    sprintf (Path2,"%s/%s",Path,FileList[NumFile]->d_name);
	    Brw_RemoveOldTmpFiles (Path2,true);
	   }
      if (RemoveDirectory)
	 /* Remove the directory itself */
	 if (Gbl.TimeStartExecution - FileStatus.st_mtime > Cfg_TIME_TO_DELETE_BROWSER_TMP_FILES)
	    rmdir (Path);
     }
   else
      if (Gbl.TimeStartExecution - FileStatus.st_mtime > Cfg_TIME_TO_DELETE_BROWSER_TMP_FILES)
	 unlink (Path);
  }

/*****************************************************************************/
/*********** Ask for confirmation of removing of a folder no empty ***********/
/*****************************************************************************/

void Brw_AskRemFileFromTree (void)
  {
   extern const char *Txt_Do_you_really_want_to_remove_the_file_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_file[Txt_NUM_LANGUAGES];
   bool ICanRemove;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Button of confirmation of removing *****/
   if ((ICanRemove = Brw_CheckIfICanModifyFileOrFolder (Gbl.FileBrowser.Level)))
     {
      /***** Form to ask for confirmation to remove a file *****/
      Act_FormStart (Brw_ActRemoveFile[Gbl.FileBrowser.Type]);
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            Usr_HiddenParUsrCodAll (Brw_ActRemoveFile[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
            break;
         default:
            break;
        }
      Brw_ParamListFiles (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);
      sprintf (Gbl.Message,Txt_Do_you_really_want_to_remove_the_file_X[Gbl.Prefs.Language],
               Gbl.FileBrowser.FileOrFolderName);
      Lay_ShowAlert (WARNING,Gbl.Message);
      fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"%s\" /></div></form>\n",
               Txt_Remove_file[Gbl.Prefs.Language]);
     }
   else
      Lay_ShowErrorAndExit ("You can not remove this file.");

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/************************ Remove a file of a file browser ********************/
/*****************************************************************************/

void Brw_RemFileFromTree (void)
  {
   char PathPrivateRelFromCGI_BIN[PATH_MAX+1];
   extern const char *Txt_File_X_removed[Txt_NUM_LANGUAGES];
   struct stat FileStatus;
   bool ICanRemove;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   if ((ICanRemove = Brw_CheckIfICanModifyFileOrFolder (Gbl.FileBrowser.Level)))
     {
      sprintf (PathPrivateRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.FileBrowser.Priv.FullPathInTree);

      /***** Check if is a file o a folder *****/
      lstat (PathPrivateRelFromCGI_BIN,&FileStatus);
      if (S_ISREG (FileStatus.st_mode))		// It's a file
        {
         if (unlink (PathPrivateRelFromCGI_BIN))
            Lay_ShowErrorAndExit ("Can not remove file.");

         switch (Gbl.FileBrowser.Type)
           {
            case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
            case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
               /* If we remove a file of download, also we must remove it as hidden download from the database */
               Brw_RemoveOneHiddenDownloadFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
               break;
            case Brw_FILE_BRW_COMMON_CRS:
            case Brw_FILE_BRW_COMMON_GRP:
               /* If we remove a file of the common zone,
                  also we must remove su owner of the database */
               Brw_RemoveOneFileOrFolderCommonFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
               break;
            case Brw_FILE_BRW_ADMIN_MARKS_CRS:
            case Brw_FILE_BRW_ADMIN_MARKS_GRP:
               /* If we remove a file of marks, also we must remove from the database */
               Brw_RemoveOneGradesFileFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
               break;
            default:
               break;
           }

         /* Remove affected clipboards */
         Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

	 /* Message of confirmation of removing */
	 sprintf (Gbl.Message,Txt_File_X_removed[Gbl.Prefs.Language],
                  Gbl.FileBrowser.FileOrFolderName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
      else		// File not found
         Lay_ShowErrorAndExit ("File not found.");
     }
   else
      Lay_ShowErrorAndExit ("You can not remove this file.");

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/******************* Remove a folder of a file browser ***********************/
/*****************************************************************************/

void Brw_RemFolderFromTree (void)
  {
   extern const char *Txt_Folder_X_removed[Txt_NUM_LANGUAGES];
   char PathPrivateRelFromCGI_BIN[PATH_MAX+1];
   struct stat FileStatus;
   bool ICanRemove;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   if ((ICanRemove = Brw_CheckIfICanModifyFileOrFolder (Gbl.FileBrowser.Level)))
     {
      sprintf (PathPrivateRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.FileBrowser.Priv.FullPathInTree);

      /***** Check if it's a file or a folder *****/
      lstat (PathPrivateRelFromCGI_BIN,&FileStatus);
      if (S_ISDIR (FileStatus.st_mode))		// It's a directory
         if (rmdir (PathPrivateRelFromCGI_BIN))
           {
	    if (errno == ENOTEMPTY)	// The directory is not empty
	      {
	       Brw_AskConfirmRemoveFolderNotEmpty ();
	       Gbl.Message[0] = '\0';
	      }
	    else	// The directory is empty
               Lay_ShowErrorAndExit ("Can not remove folder.");
           }
         else
           {
            switch (Gbl.FileBrowser.Type)
              {
               case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
               case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
                  /* If we remove an empty folder of download,
                     also we must remove it as hidden download from the database */
                  Brw_RemoveOneHiddenDownloadFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
                  break;
               case Brw_FILE_BRW_COMMON_CRS:
               case Brw_FILE_BRW_COMMON_GRP:
                  /* If we remove a folder of the common zone,
                     also we must remove its owner from the database */
                  Brw_RemoveOneFileOrFolderCommonFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
                  break;
               default:
                  break;
              }
            /* Remove affected clipboards */
            Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

            /* Remove affected expanded folders */
            Brw_RemoveAffectedExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

            /* Message of confirmation of successfull removing */
	    sprintf (Gbl.Message,Txt_Folder_X_removed[Gbl.Prefs.Language],
                     Gbl.FileBrowser.FileOrFolderName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
      else		// Folder not found
         Lay_ShowErrorAndExit ("Folder not found.");
     }
   else
      Lay_ShowErrorAndExit ("You can not remove this folder.");

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/*********** Ask for confirmation of removing of a folder no empty ***********/
/*****************************************************************************/

static void Brw_AskConfirmRemoveFolderNotEmpty (void)
  {
   extern const char *Txt_Do_you_really_want_to_remove_the_folder_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_folder[Txt_NUM_LANGUAGES];

   /***** Form to remove a not empty folder *****/
   Act_FormStart (Brw_ActRemoveFolderNotEmpty[Gbl.FileBrowser.Type]);
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         Usr_HiddenParUsrCodAll (Brw_ActRemoveFolderNotEmpty[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
         Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
         break;
      default:
         break;
     }
   Brw_ParamListFiles (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);
   sprintf (Gbl.Message,Txt_Do_you_really_want_to_remove_the_folder_X[Gbl.Prefs.Language],
            Gbl.FileBrowser.FileOrFolderName);
   Lay_ShowAlert (WARNING,Gbl.Message);
   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<input type=\"submit\" value=\"%s\" /></div></form>\n",
            Txt_Remove_folder[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/********* Complete removing of a not empty folder in a file browser *********/
/*****************************************************************************/

void Brw_RemSubtreeInFileBrowser (void)
  {
   extern const char *Txt_Folder_X_and_all_its_contents_removed[Txt_NUM_LANGUAGES];
   char PathPrivateRelFromCGI_BIN[PATH_MAX+1];
   bool ICanRemove;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   if ((ICanRemove = Brw_CheckIfICanModifyFileOrFolder (Gbl.FileBrowser.Level)))
     {
      sprintf (PathPrivateRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.FileBrowser.Priv.FullPathInTree);

      /***** Remove the whole tree *****/
      Brw_RemoveTree (PathPrivateRelFromCGI_BIN);

      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
            /* If we remove a download folder, we must also remove from the database all the hidden downloads under that folder */
            Brw_RemoveOneHiddenDownloadFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
            Brw_RemoveChildrenOfHiddenDownloadFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
            break;
         case Brw_FILE_BRW_COMMON_CRS:
         case Brw_FILE_BRW_COMMON_GRP:
            /* If we remove a folder of the common zone,
               we must also remove its owner from the database in all the files o folders under that folser */
            Brw_RemoveOneFileOrFolderCommonFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
            Brw_RemoveChildrenOfFolderCommonFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
            break;
         case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            /* If we remove a folder of marks, we must also remove from the database all the files under that folder */
            Brw_RemoveChildrenOfMarksFolderFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);
            break;
         default:
            break;
        }

      /* Remove affected clipboards */
      Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

      /* Remove affected expanded folders */
      Brw_RemoveAffectedExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

      /***** Write message of confirmation *****/
      sprintf (Gbl.Message,Txt_Folder_X_and_all_its_contents_removed[Gbl.Prefs.Language],
               Gbl.FileBrowser.FileOrFolderName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/************************* Remove a folder recursively ***********************/
/*****************************************************************************/
// If the tree of directories and files exists, remove it

void Brw_RemoveTree (const char *PathRelFromCGI_BIN)
  {
   struct stat FileStatus;
   struct dirent **FileList;
   int NumFile,NumFiles;
   char PathFileRel[PATH_MAX+1];
   bool Error;

   if (Fil_CheckIfPathExists (PathRelFromCGI_BIN))
     {
      lstat (PathRelFromCGI_BIN,&FileStatus);
      if (S_ISDIR (FileStatus.st_mode))		// It's a directory
	{
	 if (rmdir (PathRelFromCGI_BIN))
	   {
	    Error = false;
	    if (errno == ENOTEMPTY)
	      {
	       /***** Remove each directory and file under this directory *****/
	       /* Scan the directory */
	       NumFiles = scandir (PathRelFromCGI_BIN,&FileList,NULL,NULL);

	       /* Remove recursively all the directories and files */
	       for (NumFile=0; NumFile<NumFiles; NumFile++)
		  if (strcmp (FileList[NumFile]->d_name,".") &&
                      strcmp (FileList[NumFile]->d_name,".."))	// Skip directories "." and ".."
		    {
		     sprintf (PathFileRel,"%s/%s",PathRelFromCGI_BIN,FileList[NumFile]->d_name);
		     Brw_RemoveTree (PathFileRel);
		    }

	       /***** Remove of new the directory, now empty *****/
	       if (rmdir (PathRelFromCGI_BIN))
		  Error = true;
	      }
	    else
	       Error = true;
	    if (Error)
	       Lay_ShowErrorAndExit ("Can not remove folder.");
	   }
	}
      else					// It's a file
	 if (unlink (PathRelFromCGI_BIN))
	    Lay_ShowErrorAndExit ("Can not remove file.");
     }
  }

/*****************************************************************************/
/********************* Expand a folder in a file browser *********************/
/*****************************************************************************/

void Brw_ExpandFileTree (void)
  {
   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Add path to table of expanded folders *****/
   Brw_InsThisFolderAndUpdOtherFoldersInExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

   /***** Remove old expanded folders (from all users) *****/
   Brw_RemoveExpiredExpandedFolders ();   // Someone must do this work. Let's do it when a user click in an expand or contract button

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/******************* Contract a folder in a file browser *********************/
/*****************************************************************************/

void Brw_ContractFileTree (void)
  {
   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Remove path where the user has clicked from table of expanded folders *****/
   Brw_RemThisFolderAndUpdOtherFoldersFromExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

   /***** Remove old expanded folders (from all users) *****/
   Brw_RemoveExpiredExpandedFolders ();   // Someone must do this work. Let's do it when a user click in an expand or contract button

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/******* Copy the path of a arch/carp of a file browser in clipboard *********/
/*****************************************************************************/

void Brw_CopyFromFileBrowser (void)
  {
   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Remove old clipboards (from all users) *****/
   Brw_RemoveExpiredClipboards ();   // Someone must do this work. Let's do it whenever a user click in a copy button

   /***** Poner the path in the clipboard *****/
   if (Brw_GetMyClipboard ())
      Brw_UpdatePathInClipboard (Gbl.FileBrowser.Priv.FullPathInTree);
   else
      Brw_AddPathToClipboards (Gbl.FileBrowser.Priv.FullPathInTree);

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/**************** Get the path of my clipboard and write it ******************/
/*****************************************************************************/

static inline void Brw_GetAndWriteClipboard (void)
  {
   if (Brw_GetMyClipboard ())
      Brw_WriteCurrentClipboard ();
  }

/*****************************************************************************/
/********* Write a title with the content of the current clipboard ***********/
/*****************************************************************************/

static void Brw_WriteCurrentClipboard (void)
  {
   extern const char *Txt_Copy_source[Txt_NUM_LANGUAGES];
   extern const char *Txt_download_management_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_common_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_assignments_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_works_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_marks_management_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_private_storage_zone[Txt_NUM_LANGUAGES];
   extern const char *Txt_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_group[Txt_NUM_LANGUAGES];
   extern const char *Txt_user[Txt_NUM_LANGUAGES];
   extern const char *Txt_all_files[Txt_NUM_LANGUAGES];
   extern const char *Txt_file_folder[Txt_NUM_LANGUAGES];
   struct Course Crs;
   struct GroupData GrpDat;
   struct UsrData UsrDat;
   unsigned LevelClipboard;
   const char *Ptr;

   if (Gbl.FileBrowser.Clipboard.FileBrowser != Brw_FILE_BRW_BRIEFCASE_USR)
     {
      Crs.CrsCod = Gbl.FileBrowser.Clipboard.CrsCod;
      Deg_GetDataOfCourseByCod (&Crs);
     }

   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"TITULO\">%s: ",
            Txt_Copy_source[Gbl.Prefs.Language]);
   switch (Gbl.FileBrowser.Clipboard.FileBrowser)
     {
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>",
                  Txt_download_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName);
         break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         GrpDat.GrpCod = Gbl.FileBrowser.Clipboard.GrpCod;
         Grp_GetDataOfGroupByCod (&GrpDat);
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s %s</strong>",
                  Txt_download_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
         break;
      case Brw_FILE_BRW_COMMON_CRS:
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>",
                  Txt_common_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName);
         break;
      case Brw_FILE_BRW_COMMON_GRP:
         GrpDat.GrpCod = Gbl.FileBrowser.Clipboard.GrpCod;
         Grp_GetDataOfGroupByCod (&GrpDat);
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s %s</strong>",
                  Txt_common_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s</strong>",
                  Txt_assignments_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_user[Gbl.Prefs.Language],Gbl.Usrs.Me.UsrDat.FullName);
         break;
      case Brw_FILE_BRW_WORKS_USR:
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s</strong>",
                  Txt_works_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_user[Gbl.Prefs.Language],Gbl.Usrs.Me.UsrDat.FullName);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         Usr_UsrDataConstructor (&UsrDat);
         UsrDat.UsrCod = Gbl.FileBrowser.Clipboard.WorksUsrCod;
         Usr_GetAllUsrDataFromUsrCod (&UsrDat);
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s</strong>",
                  Txt_assignments_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_user[Gbl.Prefs.Language],UsrDat.FullName);
         Usr_UsrDataDestructor (&UsrDat);
         break;
      case Brw_FILE_BRW_WORKS_CRS:
         Usr_UsrDataConstructor (&UsrDat);
         UsrDat.UsrCod = Gbl.FileBrowser.Clipboard.WorksUsrCod;
         Usr_GetAllUsrDataFromUsrCod (&UsrDat);
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s</strong>",
                  Txt_works_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_user[Gbl.Prefs.Language],UsrDat.FullName);
         Usr_UsrDataDestructor (&UsrDat);
         break;
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>",
                  Txt_marks_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName);
         break;
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         GrpDat.GrpCod = Gbl.FileBrowser.Clipboard.GrpCod;
         Grp_GetDataOfGroupByCod (&GrpDat);
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>, %s <strong>%s %s</strong>",
                  Txt_marks_management_zone[Gbl.Prefs.Language],
                  Txt_course[Gbl.Prefs.Language],Crs.ShortName,
                  Txt_group[Gbl.Prefs.Language],GrpDat.GrpTypName,GrpDat.GrpName);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         fprintf (Gbl.F.Out,"%s, %s <strong>%s</strong>",
                  Txt_private_storage_zone[Gbl.Prefs.Language],
                  Txt_user[Gbl.Prefs.Language],Gbl.Usrs.Me.UsrDat.FullName);
      default:
         break;
     }
   fprintf (Gbl.F.Out,", ");

   // LevelClipboard == number-of-slashes-in-full-path-including-file-or-folder
   for (LevelClipboard = 0, Ptr=Gbl.FileBrowser.Clipboard.Path; *Ptr; Ptr++)
      if (*Ptr == '/')
         LevelClipboard++;
   if (LevelClipboard)									// Is not the root folder?
      fprintf (Gbl.F.Out,"%s <strong>%s</strong>",
               Txt_file_folder[Gbl.Prefs.Language],
               Brw_GetFileNameToShow (Gbl.FileBrowser.Clipboard.FileBrowser,LevelClipboard,Gbl.FileBrowser.Clipboard.FileName));			// It's not the root folder
   else
      fprintf (Gbl.F.Out,Txt_all_files[Gbl.Prefs.Language]);				// It's the root folder
   fprintf (Gbl.F.Out,".</div>");
  }

/*****************************************************************************/
/********************** Get data of my current clipboard *********************/
/*****************************************************************************/
// Returns true if something found
// Returns false and void data if nothing found

static bool Brw_GetMyClipboard (void)
  {
   char Query[256+PATH_MAX];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;
   unsigned UnsignedNum;

   /***** Get my current clipboard from database *****/
   sprintf (Query,"SELECT FileBrowser,CrsCod,GrpCod,WorksUsrCod,Path FROM clipboard WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get source of copy from clipboard");

   /***** Depending on the number of rows of the result... *****/
   if (NumRows == 0)
     {
      /***** Clear clipboard data *****/
      Gbl.FileBrowser.Clipboard.FileBrowser = Brw_FILE_BRW_UNKNOWN;
      Gbl.FileBrowser.Clipboard.CrsCod      = -1L;
      Gbl.FileBrowser.Clipboard.GrpCod      = -1L;
      Gbl.FileBrowser.Clipboard.WorksUsrCod = -1L;
      Gbl.FileBrowser.Clipboard.Path[0]     = '\0';
      Gbl.FileBrowser.Clipboard.FileName[0] = '\0';
     }
   else if (NumRows == 1)
     {
      /***** Get clipboard data *****/
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",&UnsignedNum) == 1)
        {
         Gbl.FileBrowser.Clipboard.FileBrowser = (Brw_FileBrowser_t) UnsignedNum;
         Gbl.FileBrowser.Clipboard.CrsCod = Str_ConvertStrCodToLongCod (row[1]);
         if (sscanf (row[2],"%ld",&Gbl.FileBrowser.Clipboard.GrpCod) != 1)
            Gbl.FileBrowser.Clipboard.GrpCod = -1L;
         Gbl.FileBrowser.Clipboard.WorksUsrCod = Str_ConvertStrCodToLongCod (row[3]);
         strcpy (Gbl.FileBrowser.Clipboard.Path,row[4]);
         Str_GetFileNameFromPath (Gbl.FileBrowser.Clipboard.Path,Gbl.FileBrowser.Clipboard.FileName);
        }
      else
        {
         Gbl.FileBrowser.Clipboard.FileBrowser = Brw_FILE_BRW_UNKNOWN;
         Gbl.FileBrowser.Clipboard.CrsCod      = -1L;
         Gbl.FileBrowser.Clipboard.GrpCod      = -1L;
         Gbl.FileBrowser.Clipboard.WorksUsrCod = -1L;
         Gbl.FileBrowser.Clipboard.Path[0]     = '\0';
         Gbl.FileBrowser.Clipboard.FileName[0] = '\0';
        }
     }
   else
      Lay_ShowErrorAndExit ("Error when getting source of copy.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/*****************************************************************************/
/********* Check if the clipboard is in the current file browser *************/
/*****************************************************************************/
// Asigna false o true a Gbl.FileBrowser.Clipboard.IsThisTree

static bool Brw_CheckIfClipboardIsInThisTree (void)
  {
   bool IsThisTree = false;

   if (Gbl.FileBrowser.Clipboard.FileBrowser == Gbl.FileBrowser.Type)
     {
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_CRS:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         case Brw_FILE_BRW_ASSIGNMENTS_USR:
         case Brw_FILE_BRW_WORKS_USR:
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            if (Gbl.FileBrowser.Clipboard.CrsCod == Gbl.CurrentCrs.Crs.CrsCod)
               IsThisTree = true;		// We are in the course of the clipboard
            break;
         case Brw_FILE_BRW_BRIEFCASE_USR:
            IsThisTree = true;
            break;
         default:
            break;
        }
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            if (Gbl.FileBrowser.Clipboard.GrpCod != Gbl.CurrentCrs.Grps.GrpCod)
               IsThisTree = false;	// We are not in the group zone of the clipboard
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            if (Gbl.FileBrowser.Clipboard.WorksUsrCod != Gbl.Usrs.Other.UsrDat.UsrCod)
               IsThisTree = false;	// We are not in the student's works of the clipboard
            break;
         default:
            break;
        }
     }
   return IsThisTree;
  }

/*****************************************************************************/
/***************************** Add path to clipboards ************************/
/*****************************************************************************/

static void Brw_AddPathToClipboards (const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Add path to clipboards *****/
   sprintf (Query,"INSERT INTO clipboard (UsrCod,FileBrowser,CrsCod,GrpCod,WorksUsrCod,Path)" \
                  " VALUES ('%ld','%u','%ld','%ld','%ld','%s')",
            Gbl.Usrs.Me.UsrDat.UsrCod,
            (unsigned) Gbl.FileBrowser.Type,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_BRIEFCASE_USR) ? -1L : Gbl.CurrentCrs.Crs.CrsCod,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP) ? Gbl.CurrentCrs.Grps.GrpCod : -1L,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_WORKS_CRS) ? Gbl.Usrs.Other.UsrDat.UsrCod : -1L,
            Path);
   DB_QueryINSERT (Query,"can not add source of copy to clipboard");
  }

/*****************************************************************************/
/************************** Update path in my clipboard **********************/
/*****************************************************************************/

static void Brw_UpdatePathInClipboard (const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Update path in my clipboard *****/
   sprintf (Query,"UPDATE clipboard SET FileBrowser='%u',CrsCod='%ld',GrpCod='%ld',WorksUsrCod='%ld',Path='%s'" \
                  " WHERE UsrCod='%ld'",
	    (unsigned) Gbl.FileBrowser.Type,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_BRIEFCASE_USR) ? -1L : Gbl.CurrentCrs.Crs.CrsCod,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP) ? Gbl.CurrentCrs.Grps.GrpCod : -1L,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_WORKS_CRS) ? Gbl.Usrs.Other.UsrDat.UsrCod : -1L,
            Path,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update source of copy in clipboard");
  }

/*****************************************************************************/
/******* Insert specified folder from table of expanded folders       ********/
/******* and update click time of the other folders in the expl. tree ********/
/*****************************************************************************/

static void Brw_InsThisFolderAndUpdOtherFoldersInExpandedFolders (const char *Path)
  {
   if (strcmp (Path,Brw_RootFolderInternalNames[Gbl.FileBrowser.Type]))	// Don't insert root folder
     {
      /***** Insert this path in table of expanded folders if it's not yet there *****/
      if (!Brw_GetIfExpandedTree (Path))
         Brw_InsertFolderInExpandedFolders (Path);

      /***** Update paths of the current file browser in table of expanded folders *****/
      Brw_UpdateClickTimeOfThisFileBrowserInExpandedFolders ();
     }
  }

/*****************************************************************************/
/******* Remove specified folder from table of expanded folders       ********/
/******* and update click time of the other folders in the expl. tree ********/
/*****************************************************************************/

static void Brw_RemThisFolderAndUpdOtherFoldersFromExpandedFolders (const char *Path)
  {
   /***** Remove Path from expanded folders table *****/
   Brw_RemoveFolderFromExpandedFolders (Path);

   /***** Update paths of the current file browser in table of expanded folders *****/
   Brw_UpdateClickTimeOfThisFileBrowserInExpandedFolders ();
  }

/*****************************************************************************/
/************************* Insert path in expanded folders *******************/
/*****************************************************************************/

static void Brw_InsertFolderInExpandedFolders (const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Add path to expanded folders *****/
   // Path must be stored with final '/'
   sprintf (Query,"INSERT INTO expanded_folders (UsrCod,FileBrowser,CrsCod,GrpCod,WorksUsrCod,Path,ClickTime)" \
                  " VALUES ('%ld','%u','%ld','%ld','%ld','%s/',NOW())",
            Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_BRIEFCASE_USR) ? -1L : Gbl.CurrentCrs.Crs.CrsCod,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_DOWNLOAD_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_SEE_MARKS_GRP ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP) ? Gbl.CurrentCrs.Grps.GrpCod : -1L,
            (Gbl.FileBrowser.Type == Brw_FILE_BRW_ASSIGNMENTS_CRS ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_WORKS_CRS) ? Gbl.Usrs.Other.UsrDat.UsrCod : -1L,Path);
   DB_QueryINSERT (Query,"can not expand the content of a folder");
  }

/*****************************************************************************/
/******* Update paths of the current file browser in expanded folders ********/
/*****************************************************************************/

static void Brw_UpdateClickTimeOfThisFileBrowserInExpandedFolders (void)
  {
   char Query[512+PATH_MAX];

   /***** Update path in table of expanded folders *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Query,"UPDATE expanded_folders SET ClickTime=NOW()" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod);
         break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query,"UPDATE expanded_folders SET ClickTime=NOW()" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND GrpCod='%ld'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query,"UPDATE expanded_folders SET ClickTime=NOW()" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND WorksUsrCod='%ld'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Other.UsrDat.UsrCod);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query,"UPDATE expanded_folders SET ClickTime=NOW()" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type);
         break;
      default:
         break;
     }
   DB_QueryUPDATE (Query,"can not update expanded folder");
  }

/*****************************************************************************/
/********************** Remove path from expanded folders ********************/
/*****************************************************************************/

static void Brw_RemoveFolderFromExpandedFolders (const char *Path)
  {
   char Query[512+PATH_MAX];

   /***** Remove expanded folders associated to a file browser from a course or from a user *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Path);
         break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND GrpCod='%ld' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,Path);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND WorksUsrCod='%ld' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Other.UsrDat.UsrCod,Path);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Path);
         break;
      default:
         break;
     }
   DB_QueryDELETE (Query,"can not contract the content of a folder");
  }

/*****************************************************************************/
/************************* Remove clipboard of a user ************************/
/*****************************************************************************/

void Brw_RemoveUsrClipboard (long UsrCod)
  {
   char Query[512];

   /***** Remove clipboard of specified user *****/
   sprintf (Query,"DELETE FROM clipboard WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove source of copy for a user");
  }

/*****************************************************************************/
/************************ Remove clipboards of a group ***********************/
/*****************************************************************************/

void Brw_RemoveGrpClipboards (long GrpCod)
  {
   char Query[512];

   /***** Remove clipboard of specified user *****/
   sprintf (Query,"DELETE FROM clipboard WHERE GrpCod='%ld'",GrpCod);
   DB_QueryDELETE (Query,"can not remove source of copy for a group");
  }

/*****************************************************************************/
/****************** Remove clipboards of a user and a coruse *****************/
/*****************************************************************************/

void Brw_RemoveUsrClipboardInCrs (long UsrCod,long CrsCod)
  {
   char Query[512];

   /***** Remove clipboard of specified user *****/
   sprintf (Query,"DELETE FROM clipboard WHERE UsrCod='%ld' AND CrsCod='%ld'",
            UsrCod,CrsCod);
   DB_QueryDELETE (Query,"can not remove source of copy for a user and a course");
  }

/*****************************************************************************/
/****************** Remove clipboards of a user and a group ******************/
/*****************************************************************************/

void Brw_RemoveUsrGrpClipboard (long UsrCod,long GrpCod)
  {
   char Query[512];

   /***** Remove clipboard of specified user *****/
   sprintf (Query,"DELETE FROM clipboard WHERE UsrCod='%ld' AND GrpCod='%ld'",
            UsrCod,GrpCod);
   DB_QueryDELETE (Query,"can not remove source of copy for a user and a group");
  }

/*****************************************************************************/
/******************* Remove all expanded folders of a user *******************/
/*****************************************************************************/

void Brw_RemoveUsrExpandedFolders (long UsrCod)
  {
   char Query[512];

   /***** Remove expanded folders of specified user *****/
   sprintf (Query,"DELETE FROM expanded_folders WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove expanded folders for a user");
  }

/*****************************************************************************/
/********** Remove all expanded folders of a user in this course *************/
/*****************************************************************************/

void Brw_RemoveUsrExpandedFoldersInCrs (long UsrCod,long CrsCod)
  {
   char Query[512];

   /***** Remove expanded folders of specified user *****/
   sprintf (Query,"DELETE FROM expanded_folders" \
                  " WHERE UsrCod='%ld' AND CrsCod='%ld'",
            UsrCod,CrsCod);
   DB_QueryDELETE (Query,"can not remove expanded folders for a user in a course");
  }

/*****************************************************************************/
/****************** Remove expired clipboards (from all users) ***************/
/*****************************************************************************/

static void Brw_RemoveExpiredClipboards (void)
  {
   char Query[512];

   /***** Remove all expired clipboards *****/
   sprintf (Query,"DELETE LOW_PRIORITY FROM clipboard" \
                  " WHERE UNIX_TIMESTAMP() > UNIX_TIMESTAMP(CopyTime)+%ld",
            Cfg_TIME_TO_DELETE_BROWSER_CLIPBOARD);
   DB_QueryDELETE (Query,"can not remove old paths from clipboard");
  }

/*****************************************************************************/
/************* Remove expired expanded folders (from all users) **************/
/*****************************************************************************/

static void Brw_RemoveExpiredExpandedFolders (void)
  {
   char Query[512];

   if (!(Gbl.PID % 1000))	// Do this only one of 1000 times
     {
      /***** Remove all expired clipboards *****/
      sprintf (Query,"DELETE LOW_PRIORITY FROM expanded_folders" \
                     " WHERE UNIX_TIMESTAMP() > UNIX_TIMESTAMP(ClickTime)+%ld",
               Cfg_TIME_TO_DELETE_BROWSER_EXPANDED_FOLDERS);
      DB_QueryDELETE (Query,"can not remove old expanded folders");
     }
  }

/*****************************************************************************/
/********* Remove clipboards with paths from a course or from a user *********/
/*****************************************************************************/

static void Brw_RemoveAffectedClipboards (Brw_FileBrowser_t FileBrowser,long MyUsrCod,long WorksUsrCod)
  {
   char Query[512];

   /***** Remove clipboards associated to a file browser from a course or from a user *****/
   switch (FileBrowser)
     {
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         sprintf (Query,"DELETE FROM clipboard WHERE FileBrowser='%u' AND CrsCod='%ld'",
                  (unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod);
         break;
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query,"DELETE FROM clipboard WHERE FileBrowser='%u' AND CrsCod='%ld' AND GrpCod='%ld'",
                  (unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Query,"DELETE FROM clipboard WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld'",
                  MyUsrCod,(unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query,"DELETE FROM clipboard WHERE FileBrowser='%u' AND CrsCod='%ld' AND WorksUsrCod='%ld'",
                  (unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod,WorksUsrCod);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query,"DELETE FROM clipboard WHERE UsrCod='%ld' AND FileBrowser='%u'",
                  MyUsrCod,(unsigned) FileBrowser);
         break;
      default:
         break;
     }
   DB_QueryDELETE (Query,"can not remove source of copy");
  }

/*****************************************************************************/
/***** Remove expanded folders with paths from a course or from a user *******/
/*****************************************************************************/

static void Brw_RemoveAffectedExpandedFolders (const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove expanded folders associated to a file browser from a course or from a user *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE FileBrowser='%u' AND CrsCod='%ld' AND Path LIKE '%s/%%'",
                  (unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Path);
         break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE FileBrowser='%u' AND CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
                  (unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,Path);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND Path LIKE '%s/%%'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Path);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE FileBrowser='%u' AND CrsCod='%ld' AND WorksUsrCod='%ld' AND Path LIKE '%s/%%'",
                  (unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Other.UsrDat.UsrCod,Path);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query,"DELETE FROM expanded_folders" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND Path LIKE '%s/%%'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Path);
         break;
      default:
         break;
     }
   DB_QueryDELETE (Query,"can not remove expanded folders");
  }

/*****************************************************************************/
/***** Remove expanded folders with paths from a course or from a user *******/
/*****************************************************************************/

static void Brw_RenameAffectedExpandedFolders (Brw_FileBrowser_t FileBrowser,long MyUsrCod,long WorksUsrCod,const char *OldPath,const char *NewPath)
  {
   char Query[512+PATH_MAX*2];
   unsigned StartFinalSubpathNotChanged = strlen (OldPath) + 2;

   /***** Update expanded folders associated to a file browser from a course or from a user *****/
   switch (FileBrowser)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         sprintf (Query,"UPDATE expanded_folders SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                        " WHERE FileBrowser='%u' AND CrsCod='%ld' AND Path LIKE '%s/%%'",
                  NewPath,StartFinalSubpathNotChanged,(unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod,OldPath);
         break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query,"UPDATE expanded_folders SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                        " WHERE FileBrowser='%u' AND CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
                  NewPath,StartFinalSubpathNotChanged,(unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,OldPath);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Query,"UPDATE expanded_folders SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND Path LIKE '%s/%%'",
                  NewPath,StartFinalSubpathNotChanged,MyUsrCod,(unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod,OldPath);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query,"UPDATE expanded_folders SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                        " WHERE FileBrowser='%u' AND CrsCod='%ld' AND WorksUsrCod='%ld' AND Path LIKE '%s/%%'",
                  NewPath,StartFinalSubpathNotChanged,(unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod,WorksUsrCod,OldPath);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query,"UPDATE expanded_folders SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                        " WHERE UsrCod='%ld' AND FileBrowser='%u' AND Path LIKE '%s/%%'",
                  NewPath,StartFinalSubpathNotChanged,MyUsrCod,(unsigned) FileBrowser,OldPath);
         break;
      default:
         break;
     }
   DB_QueryUPDATE (Query,"can not remove expanded folders");
  }

/*****************************************************************************/
/**** Paste the arch/carp indicado in the portapapelesde in file browser *****/
/*****************************************************************************/

void Brw_PasteIntoFileBrowser (void)
  {
   extern const char *Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_[Txt_NUM_LANGUAGES];
   struct GroupData GrpDat;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   if (Brw_GetMyClipboard ())
     {
      if (Gbl.FileBrowser.Clipboard.GrpCod >= 0)	// Clipboard in a group zone
        {
         GrpDat.GrpCod = Gbl.FileBrowser.Clipboard.GrpCod;
	 Brw_GetSelectedGroupData (&GrpDat,true);
        }

      /***** Write the origin of the copy *****/
      Brw_WriteCurrentClipboard ();

      /***** Copy files recursively *****/
      Brw_PasteClipboard ();

      /***** Remove the affected clipboards *****/
      Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);
     }
   else
      /***** Write message ******/
      Lay_ShowAlert (WARNING,Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_[Gbl.Prefs.Language]);

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/********* Paste todo the content of the clipboard in the lugar actual *******/
/*****************************************************************************/
// Origen:
//	Type of file browser:		Gbl.FileBrowser.Clipboard.FileBrowser
//	Possible course:		Gbl.FileBrowser.Clipboard.CrsCod
//	Possible student in works:	Gbl.FileBrowser.Clipboard.WorksUsrCod
//	Path (file or folder):		Gbl.FileBrowser.Clipboard.Path
// Destino:
//	Type of file browser:		Gbl.FileBrowser.Type
//	Possible course:		Gbl.CurrentCrs.Crs.CrsCod
//	Possible student in works:	Gbl.Usrs.Other.UsrDat.UsrCod
//	Path (debera ser folder):	Gbl.FileBrowser.Priv.FullPathInTree

static void Brw_PasteClipboard (void)
  {
   extern const char *Txt_The_copy_has_been_successful[Txt_NUM_LANGUAGES];
   extern const char *Txt_Folders_copied[Txt_NUM_LANGUAGES];
   extern const char *Txt_Files_copied[Txt_NUM_LANGUAGES];
   struct Course Crs;
   char PathRelPrivCrsOrg[PATH_MAX+1];
   char PathOrgFromCGI_BIN[PATH_MAX+1];
   struct UsrData UsrDat;
   unsigned NumFilesPasted = 0;
   unsigned NumFoldsPasted = 0;
   unsigned NumUsrsToBeNotified;

   Gbl.FileBrowser.Clipboard.IsThisTree = Brw_CheckIfClipboardIsInThisTree ();
   if (Brw_CheckIfCanPasteIn (Gbl.FileBrowser.Level))
     {
      /***** Construct the relative path of the file or folder origen *****/
      switch (Gbl.FileBrowser.Clipboard.FileBrowser)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_CRS:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_CRS:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         case Brw_FILE_BRW_ASSIGNMENTS_USR:
         case Brw_FILE_BRW_WORKS_USR:
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            Crs.CrsCod = Gbl.FileBrowser.Clipboard.CrsCod;
            if (Deg_GetDataOfCourseByCod (&Crs))
               sprintf (PathRelPrivCrsOrg,"%s/%ld",Cfg_PATH_REL_CRS_PRIVATE,Crs.CrsCod);
            else
               Lay_ShowErrorAndExit ("The course of copy source doesn't exist.");
            break;
         default:
            break;
        }
      switch (Gbl.FileBrowser.Clipboard.FileBrowser)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
         case Brw_FILE_BRW_COMMON_CRS:
         case Brw_FILE_BRW_ADMIN_MARKS_CRS:
            sprintf (PathOrgFromCGI_BIN,"%s/%s",PathRelPrivCrsOrg,Gbl.FileBrowser.Clipboard.Path);
            break;
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            sprintf (PathOrgFromCGI_BIN,"%s/grp/%ld/%s",PathRelPrivCrsOrg,Gbl.FileBrowser.Clipboard.GrpCod,Gbl.FileBrowser.Clipboard.Path);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_USR:
         case Brw_FILE_BRW_WORKS_USR:
            sprintf (PathOrgFromCGI_BIN,"%s/usr/%s/%s",PathRelPrivCrsOrg,Gbl.Usrs.Me.UsrDat.DNI,Gbl.FileBrowser.Clipboard.Path);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            UsrDat.UsrCod = Gbl.FileBrowser.Clipboard.WorksUsrCod;
            Usr_GetDNIFromUsrCod (&UsrDat);
            sprintf (PathOrgFromCGI_BIN,"%s/usr/%s/%s",PathRelPrivCrsOrg,UsrDat.DNI,Gbl.FileBrowser.Clipboard.Path);
            break;
         case Brw_FILE_BRW_BRIEFCASE_USR:
            sprintf (PathOrgFromCGI_BIN,"%s/%s",Gbl.Usrs.Me.PathRelDir,Gbl.FileBrowser.Clipboard.Path);
            break;
         default:
            break;
        }

      /***** Paste tree (path in clipboard) into folder *****/
      Brw_CalcSizeOfDir (Gbl.FileBrowser.Priv.PathRelRootFolder);
      Brw_SetMaxQuota ();
      if (Brw_PasteTreeIntoFolder (PathOrgFromCGI_BIN,Gbl.FileBrowser.Priv.FullPathInTree,&NumFilesPasted,&NumFoldsPasted))
        {
         /* Write message of success */
         sprintf (Gbl.Message,"%s<br />%s: %u<br />%s: %u",
                  Txt_The_copy_has_been_successful[Gbl.Prefs.Language],
                  Txt_Folders_copied[Gbl.Prefs.Language],NumFoldsPasted,
                  Txt_Files_copied[Gbl.Prefs.Language],NumFilesPasted);
         Lay_ShowAlert (INFO,Gbl.Message);
        }

      /***** Add path where new tree is pasted to table of expanded folders *****/
      Brw_InsThisFolderAndUpdOtherFoldersInExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

      /***** Notify new file of marks to all users in course *****/
      if (NumFilesPasted && (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_CRS ||
                             Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP))
        {
         NumUsrsToBeNotified = Ntf_StoreNotifyEventsToAllUsrs (Ntf_NEW_MARKS_FILE,-1L,-1L);
         Ntf_ShowAlertNumUsrsToBeNotified (NumUsrsToBeNotified);
        }
     }
   else
      Lay_ShowErrorAndExit ("You can not paste here.");
  }

/*****************************************************************************/
/****************** Compute number of levels in a path ***********************/
/*****************************************************************************/

static unsigned Brw_NumLevelsInPath (const char *Path)
  {
   unsigned NumLevls = 0;

   while (*Path)
      if (*Path++ == '/')
         NumLevls++;

   return NumLevls;
  }

/*****************************************************************************/
/*********** Copy a source file o tree in the destination folder *************/
/*****************************************************************************/
// Overwrite the files (not the folders) found in the destination with the same name
// Return true if the copy has been made successfully, and false if not

static bool Brw_PasteTreeIntoFolder (const char *PathOrg,const char *PathDstEnTree,unsigned *NumFilesPasted,unsigned *NumFoldsPasted)
  {
   extern const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_maximum_allowed_number_of_levels[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_maximum_allowed_number_of_levels[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_there_is_already_another_file_with_that_name[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML[Txt_NUM_LANGUAGES];
   bool IsFile;
   bool IsFolder;
   char FileNameOrg[NAME_MAX+1];
   char PathInFolderOrg[PATH_MAX+1];
   char PathDstInTreeWithFile[PATH_MAX+1];
   char PathDstWithFile[PATH_MAX+1];
   struct stat FileStatus;
   struct dirent **FileList;
   int NumFileInThisDir;
   int NumFilesInThisDir;
   unsigned NumLevls;

   /***** Is it a file or a folder? *****/
   lstat (PathOrg,&FileStatus);
   IsFile   = S_ISREG (FileStatus.st_mode);	// It's a regular file
   IsFolder = S_ISDIR (FileStatus.st_mode);	// It's a directory

   /***** Get the name (only the name) of the origin file or folder *****/
   Str_GetFileNameFromPath (PathOrg,FileNameOrg);

   /***** Construct the name of the destination file or folder *****/
   sprintf (PathDstInTreeWithFile,"%s/%s",PathDstEnTree,FileNameOrg);

   /***** Construct the relative path of the destination file or folder *****/
   sprintf (PathDstWithFile,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,PathDstInTreeWithFile);

   /***** Update and check number of levels *****/
   // The number of levels is counted starting on the root folder raz, not included.
   // Example:	If PathDstInTreeWithFile is "maletin/1/2/3/4/FileNameOrg", then NumLevls=5
   if ((NumLevls = Brw_NumLevelsInPath (PathDstInTreeWithFile)) > Gbl.FileBrowser.Size.NumLevls)
      Gbl.FileBrowser.Size.NumLevls = NumLevls;
   if (Brw_CheckIfQuotaExceded ())
     {
      if (IsFile)
         sprintf (Gbl.Message,Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_maximum_allowed_number_of_levels[Gbl.Prefs.Language],
                  FileNameOrg);
      else if (IsFolder)
         sprintf (Gbl.Message,Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_maximum_allowed_number_of_levels[Gbl.Prefs.Language],
                  FileNameOrg);
      else
         Lay_ShowErrorAndExit ("Can not paste unknown file type.");
      Lay_ShowAlert (WARNING,Gbl.Message);
      return false;
     }

   /***** Copy file or folder *****/
   if (IsFile)	// It's a regular file
     {
      /***** Check if exists the destination file */
      if (Fil_CheckIfPathExists (PathDstWithFile))
        {
         sprintf (Gbl.Message,Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_there_is_already_another_file_with_that_name[Gbl.Prefs.Language],
                  FileNameOrg);
         Lay_ShowAlert (WARNING,Gbl.Message);
         return false;
        }

      /***** If the target file browser is that of marks, only HTML files are allowed *****/
      if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_CRS ||
          Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP)
         /* Check extension of the file */
         if (!Str_FileIsHTML (FileNameOrg))
           {
            sprintf (Gbl.Message,Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML[Gbl.Prefs.Language],
                     FileNameOrg);
            Lay_ShowAlert (WARNING,Gbl.Message);
            return false;
           }

      /***** Update and check the quota before copying the file *****/
      Gbl.FileBrowser.Size.NumFiles++;
      Gbl.FileBrowser.Size.TotalSiz += (unsigned long long int) FileStatus.st_size;
      if (Brw_CheckIfQuotaExceded ())
        {
         sprintf (Gbl.Message,Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_disk_quota[Gbl.Prefs.Language],
                  FileNameOrg);
         Lay_ShowAlert (WARNING,Gbl.Message);
         return false;
        }

      /***** Quota will not be exceeded ==> copy the origin file to the destination file *****/
      Fil_FastCopyOfFiles (PathOrg,PathDstWithFile);

      if (Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_CRS ||
          Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP)
         /* Add entry to the table of creators of files/folders */
         Brw_AddPathCommonToDB (Gbl.CurrentCrs.Grps.GrpCod,PathDstInTreeWithFile);

      (*NumFilesPasted)++;
     }
   else	// It's a directory
     {
      /***** Scan the source directory *****/
      NumFilesInThisDir = scandir (PathOrg,&FileList,NULL,alphasort);

      /***** Create the folder in the destination *****/
      if (!Fil_CheckIfPathExists (PathDstWithFile))	// If already exists, don't overwrite
        {
	 /* The directory doesn't exists ==> create it.
            First, update and check the quota */
	 Gbl.FileBrowser.Size.NumFolds++;
	 Gbl.FileBrowser.Size.TotalSiz += (unsigned long long int) FileStatus.st_size;
         if (Brw_CheckIfQuotaExceded ())
           {
            sprintf (Gbl.Message,Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_disk_quota[Gbl.Prefs.Language],
                     FileNameOrg);
            Lay_ShowAlert (WARNING,Gbl.Message);
            return false;
           }
	 /* The quota will not be exceded ==> create the directory */
         if (mkdir (PathDstWithFile,(mode_t) 0xFFF) != 0)
            Lay_ShowErrorAndExit ("Can not create folder.");

         if (Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_CRS ||
             Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP)
            /* Add entry to the table of owners of files/folders */
            Brw_AddPathCommonToDB (Gbl.CurrentCrs.Grps.GrpCod,PathDstInTreeWithFile);
        }

      /***** Copy each of the files and folders from the origin to the destination *****/
      for (NumFileInThisDir=0; NumFileInThisDir<NumFilesInThisDir; NumFileInThisDir++)
         if (strcmp (FileList[NumFileInThisDir]->d_name,".") &&
             strcmp (FileList[NumFileInThisDir]->d_name,".."))	// Skip directories "." and ".."
	   {
	    sprintf (PathInFolderOrg,"%s/%s",PathOrg,FileList[NumFileInThisDir]->d_name);
            if (!Brw_PasteTreeIntoFolder (PathInFolderOrg,PathDstInTreeWithFile,NumFilesPasted,NumFoldsPasted))
               return false;
	   }

      (*NumFoldsPasted)++;
     }
   return true;
  }

/*****************************************************************************/
/************** Form to add a file or folder to a file browser ***************/
/*****************************************************************************/

void Brw_ShowFormFileBrowser (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_You_can_create_a_new_folder_inside_the_folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_Folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_or_you_can_send_a_new_file_to_the_folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_File[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_file[Txt_NUM_LANGUAGES];
   extern const char *Txt_or_you_can_make_a_file_copy_to_the_folder[Txt_NUM_LANGUAGES];
   extern const char *Txt_Paste[Txt_NUM_LANGUAGES];
   const char *FileNameToShow;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Check if creating a new folder or file is allowed *****/
   if (Brw_CheckIfICanCreateIntoFolder (Gbl.FileBrowser.Level))
     {
      /***** Name of the folder to be shown ****/
      FileNameToShow = Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Gbl.FileBrowser.Level,Gbl.FileBrowser.FileOrFolderName);

      /***** Form to create a new folder *****/
      Act_FormStart (Brw_ActCreateFolder[Gbl.FileBrowser.Type]);
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
            Usr_HiddenParUsrCodAll (Brw_ActCreateFolder[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
            break;
         default:
            break;
        }
      Brw_ParamListFiles (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);

      /* Folder */
      sprintf (Gbl.Message,Txt_You_can_create_a_new_folder_inside_the_folder[Gbl.Prefs.Language],
               FileNameToShow);
      Lay_ShowAlert (INFO,Gbl.Message);

      fprintf (Gbl.F.Out,"<table width=\"100%%\"><tr>" \
                         "<td width=\"30%%\" align=\"right\" class=\"%s\">%s:<br /></td>" \
                         "<td align=\"left\" width=\"70%%\">" \
                         "<input type=\"text\" name=\"NewFolderName\" size=\"32\" maxlength=\"40\" value=\"\" />" \
                         "</td>" \
                         "</tr></table>",
               The_ClassFormul[Gbl.Prefs.Theme],Txt_Folder[Gbl.Prefs.Language]);

      /* Button to send */
      Lay_PutSendButton (Txt_Create_folder[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form>\n");

      /***** Form to send a file *****/
      Act_FormStart (Brw_ActCreateFile[Gbl.FileBrowser.Type]);
      switch (Gbl.FileBrowser.Type)
        {
         case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
         case Brw_FILE_BRW_COMMON_GRP:
         case Brw_FILE_BRW_ADMIN_MARKS_GRP:
            Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
            break;
         case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         case Brw_FILE_BRW_WORKS_CRS:
	    Usr_HiddenParUsrCodAll (Brw_ActCreateFile[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
            break;
         default:
            break;
        }
      Brw_ParamListFiles (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);

      /* File */
      sprintf (Gbl.Message,Txt_or_you_can_send_a_new_file_to_the_folder[Gbl.Prefs.Language],
               FileNameToShow);
      Lay_ShowAlert (INFO,Gbl.Message);

      fprintf (Gbl.F.Out,"<table width=\"100%%\"><tr>" \
                         "<td width=\"30%%\" align=\"right\" class=\"%s\">%s:<br /></td>" \
                         "<td align=\"left\" width=\"70%%\">" \
                         "<input type=\"file\" name=\"%s\" size=\"50\" maxlength=\"100\" value=\"\" />" \
                         "</td>" \
                         "</tr></table>",
               The_ClassFormul[Gbl.Prefs.Theme],Txt_File[Gbl.Prefs.Language],
               Brw_NAME_OF_PARAM_FILENAME_ORG);

      /* Button to send */
      Lay_PutSendButton (Txt_Upload_file[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form>\n");

      if (Brw_GetMyClipboard ())
        {
         /***** Check if we can paste in this folder *****/
         Gbl.FileBrowser.Clipboard.IsThisTree = Brw_CheckIfClipboardIsInThisTree ();
         if (Brw_CheckIfCanPasteIn (Gbl.FileBrowser.Level))
           {
            /***** Form to paste the content of the clipboard *****/
            Act_FormStart (Brw_ActPaste[Gbl.FileBrowser.Type]);
            switch (Gbl.FileBrowser.Type)
              {
               case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
               case Brw_FILE_BRW_COMMON_GRP:
               case Brw_FILE_BRW_ADMIN_MARKS_GRP:
                  Grp_PutParamGrpCod (Gbl.CurrentCrs.Grps.GrpCod);
                  break;
               case Brw_FILE_BRW_ASSIGNMENTS_CRS:
               case Brw_FILE_BRW_WORKS_CRS:
	          Usr_HiddenParUsrCodAll (Brw_ActPaste[Gbl.FileBrowser.Type],Gbl.Usrs.Select.All);
                  Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
                  break;
               default:
                  break;
              }
            Brw_ParamListFiles (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);

            /* Path in the clipboard */
            sprintf (Gbl.Message,Txt_or_you_can_make_a_file_copy_to_the_folder[Gbl.Prefs.Language],
                     FileNameToShow);
            Lay_ShowAlert (INFO,Gbl.Message);

            /* Button to send */
            Lay_PutSendButton (Txt_Paste[Gbl.Prefs.Language]);
            fprintf (Gbl.F.Out,"</form>\n");
           }
        }
     }
   else
      Lay_ShowErrorAndExit ("You can not paste here.");

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/*********** Receive the name of a new folder in a file browser **************/
/*****************************************************************************/

void Brw_RecFolderFileBrowser (void)
  {
   extern const char *Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES];
   extern const char *Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_folder_X_has_been_created_inside_the_folder_Y[Txt_NUM_LANGUAGES];
   char PathPrivateRelFromCGI_BIN[PATH_MAX+1];
   char PathCompleteInTreeIncludingFolder[PATH_MAX+1];

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Check if creating a new folder is allowed *****/
   if (Brw_CheckIfICanCreateIntoFolder (Gbl.FileBrowser.Level))
     {
      if (Brw_CheckIfFileOrFolderNameIsValid (Gbl.FileBrowser.NewFileOrFolderName))
        {
         /* In Gbl.FileBrowser.NewFileOrFolderName is the name of the new folder */
         sprintf (PathPrivateRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.FileBrowser.Priv.FullPathInTree);

         if (strlen (PathPrivateRelFromCGI_BIN) + 1 + strlen (Gbl.FileBrowser.NewFileOrFolderName) > PATH_MAX)
	    Lay_ShowErrorAndExit ("Path is too long.");
         strcat (PathPrivateRelFromCGI_BIN,"/");
         strcat (PathPrivateRelFromCGI_BIN,Gbl.FileBrowser.NewFileOrFolderName);

         /* Create the new directory */
         if (mkdir (PathPrivateRelFromCGI_BIN,(mode_t) 0xFFF) == 0)
	   {
	    /* Check if se ha sobrepasado the quota */
	    Brw_CalcSizeOfDir (Gbl.FileBrowser.Priv.PathRelRootFolder);
	    Brw_SetMaxQuota ();
            if (Brw_CheckIfQuotaExceded ())
	      {
	       Brw_RemoveTree (PathPrivateRelFromCGI_BIN);
               sprintf (Gbl.Message,Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota[Gbl.Prefs.Language],
                        Gbl.FileBrowser.NewFileOrFolderName);
               Lay_ShowAlert (WARNING,Gbl.Message);
	      }
	    else
              {
               /* Remove affected clipboards */
               Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

               /* Add path where new file is created to table of expanded folders */
               Brw_InsThisFolderAndUpdOtherFoldersInExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

               if (Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_CRS ||
                   Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP)
                 {
                  /* Add entry to the table of creators of files/folders */
                  sprintf (PathCompleteInTreeIncludingFolder,"%s/%s",Gbl.FileBrowser.Priv.FullPathInTree,Gbl.FileBrowser.NewFileOrFolderName);
                  Brw_AddPathCommonToDB (Gbl.CurrentCrs.Grps.GrpCod,PathCompleteInTreeIncludingFolder);
                 }

	       /* The folder has been created sucessfully */
	       sprintf (Gbl.Message,Txt_The_folder_X_has_been_created_inside_the_folder_Y[Gbl.Prefs.Language],
		        Gbl.FileBrowser.NewFileOrFolderName,
                        Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Gbl.FileBrowser.Level,Gbl.FileBrowser.FileOrFolderName));
               Lay_ShowAlert (INFO,Gbl.Message);
              }
	   }
         else
	   {
	    switch (errno)
	      {
	       case EEXIST:
                  sprintf (Gbl.Message,Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name[Gbl.Prefs.Language],
		   	   Gbl.FileBrowser.NewFileOrFolderName);
                  Lay_ShowAlert (WARNING,Gbl.Message);
	          break;
	       case EACCES:
	          Lay_ShowErrorAndExit ("Write forbidden.");
	       default:
	          Lay_ShowErrorAndExit ("Can not create folder.");
	      }
	   }
        }
      else	// Folder name not valid
         Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
      Lay_ShowErrorAndExit ("You can not create files or folders here.");

   /***** Show again the file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/**************** Rename an existing folder in a file browser ****************/
/*****************************************************************************/

void Brw_RenFolderFileBrowser (void)
  {
   extern const char *Txt_The_folder_name_X_has_changed_to_Y[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_folder_name_X_has_not_changed[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y[Txt_NUM_LANGUAGES];
   char OldPath[PATH_MAX+1],NewPath[PATH_MAX+1];
   char OldPathPrivRelFromCGI_BIN[PATH_MAX+1],
	NewPathPrivRelFromCGI_BIN[PATH_MAX+1];
   bool ICanRename;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   if ((ICanRename = Brw_CheckIfICanModifyFileOrFolder (Gbl.FileBrowser.Level)))
     {
      if (Brw_CheckIfFileOrFolderNameIsValid (Gbl.FileBrowser.NewFileOrFolderName))
        {
         if (strcmp (Gbl.FileBrowser.FileOrFolderName,Gbl.FileBrowser.NewFileOrFolderName))	// The name has changed
           {
            /* Gbl.FileBrowser.FileOrFolderName holds the new name of the folder */
            sprintf (OldPath,"%s/%s",Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);
            sprintf (OldPathPrivRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,OldPath);

            /* Gbl.FileBrowser.NewFileOrFolderName holds the new name of the folder */
            if (strlen (Gbl.FileBrowser.Priv.PathRelAboveRootFolder)+1+
                strlen (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder)+1+
                strlen (Gbl.FileBrowser.NewFileOrFolderName) > PATH_MAX)
	       Lay_ShowErrorAndExit ("Path is too long.");
            sprintf (NewPath,"%s/%s",Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.NewFileOrFolderName);
            sprintf (NewPathPrivRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,NewPath);

            /* We should check here that a folder with the same name doesn't exists.
	       but we leave this work to the system */

            /* Rename the directory. If a empty folder existed with the name new, overwrite it! */
            if (rename (OldPathPrivRelFromCGI_BIN,NewPathPrivRelFromCGI_BIN) == 0)
              {
               switch (Gbl.FileBrowser.Type)
                 {
                  case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
                  case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
	             /* If we rename a download folder, we must rename all the entries in the table of hidden downloads
                        that belong to the subtree starting at that folder */
                     Brw_RenameOneHiddenDownloadFolderInDB (Gbl.CurrentCrs.Grps.GrpCod,OldPath,NewPath);
                     Brw_RenameChildrenHiddenDownloadsInDB (Gbl.CurrentCrs.Grps.GrpCod,OldPath,NewPath);
                     break;
                  case Brw_FILE_BRW_COMMON_CRS:
                  case Brw_FILE_BRW_COMMON_GRP:
	             /* If we rename a common folder, we must rename all the entries in the table of common files
                        that belong to the subtree starting at that folder */
                     Brw_RenameOneFolderCommonInDB (Gbl.CurrentCrs.Grps.GrpCod,OldPath,NewPath);
                     Brw_RenameChildrenFilesOrFoldersCommonInDB (Gbl.CurrentCrs.Grps.GrpCod,OldPath,NewPath);
                     break;
                  case Brw_FILE_BRW_ADMIN_MARKS_CRS:
                  case Brw_FILE_BRW_ADMIN_MARKS_GRP:
	             /* If we rename a folder of marks, we must rename all the entries in the table of files of marks
                        that belong to the subtree starting at that folder */
                     Brw_RenameChildrenMarksFileInDB (Gbl.CurrentCrs.Grps.GrpCod,OldPath,NewPath);
                     break;
                  default:
                     break;
                 }
               /* Remove affected clipboards */
               Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

               /* Remove affected expanded folders */
               Brw_RenameAffectedExpandedFolders (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod,OldPath,NewPath);

               /* Write message of confirmation */
	       sprintf (Gbl.Message,Txt_The_folder_name_X_has_changed_to_Y[Gbl.Prefs.Language],
                        Gbl.FileBrowser.FileOrFolderName,Gbl.FileBrowser.NewFileOrFolderName);
               Lay_ShowAlert (INFO,Gbl.Message);
              }
            else
	      {
	       switch (errno)
	         {
	          case ENOTEMPTY:
	          case EEXIST:
	          case ENOTDIR:
	             sprintf (Gbl.Message,Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y[Gbl.Prefs.Language],
			      Gbl.FileBrowser.FileOrFolderName,Gbl.FileBrowser.NewFileOrFolderName);
	             break;
	          case EACCES:
	             Lay_ShowErrorAndExit ("Write forbidden.");
	          default:
	             Lay_ShowErrorAndExit ("Can not rename folder.");
	         }
               Lay_ShowAlert (WARNING,Gbl.Message);
	      }
           }
         else	// Names are equal. This may happens if we have press INTRO without changing the name
           {
	    sprintf (Gbl.Message,Txt_The_folder_name_X_has_not_changed[Gbl.Prefs.Language],
                     Gbl.FileBrowser.FileOrFolderName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// Folder name not valid
         Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
      Lay_ShowErrorAndExit ("You can not rename folder.");

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/****************** Receive a new file in a file browser *********************/
/*****************************************************************************/

void Brw_ReceiveFileInFileBrowser (void)
  {
   extern const char *Txt_The_MIME_type_X_of_file_Y_is_not_allowed[Txt_NUM_LANGUAGES];
   extern const char *Txt_X_file_type_is_not_HTML[Txt_NUM_LANGUAGES];
   extern const char *Txt_Try_sending_a_compressed_file[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_file_X_has_an_extension_not_allowed[Txt_NUM_LANGUAGES];
   extern const char *Txt_Can_not_create_the_file_X_because_there_is_already_a_folder_or_a_file_with_that_name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Can_not_create_the_file_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_file_X_has_been_placed_inside_the_folder_Y[Txt_NUM_LANGUAGES];
   extern const char *Txt_You_must_specify_the_file[Txt_NUM_LANGUAGES];
   char SrcFileName[PATH_MAX+1];
   char PathPrivateRelFromCGI_BIN[PATH_MAX+1];
   char PathPrivateRelFromCGI_BINTmp[PATH_MAX+1];
   char PathCompleteInTreeIncludingFile[PATH_MAX+1];
   char FileType[Brw_MAX_BYTES_MIME_TYPE+1];
   int Type;
   bool MIMETypeIsAllowed = false,
	ExtensionIsAllowed = false,
	FileIsValid = true;
   unsigned NumRowsHeader;
   unsigned NumRowsFooter;
   unsigned NumUsrsToBeNotified;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Check if creating a new file is allowed *****/
   if (Brw_CheckIfICanCreateIntoFolder (Gbl.FileBrowser.Level))
     {
      /***** First, we save in disk the file from stdin (really from Gbl.F.Tmp) *****/
      Fil_StartReceptionOfFile (SrcFileName,FileType);

      /***** Get filename from path *****/
      Str_GetFileNameFromPath (SrcFileName,Gbl.FileBrowser.NewFileOrFolderName);

      if (Gbl.FileBrowser.NewFileOrFolderName[0])
        {
         /***** Check if uploading this kind of file is allowed *****/
         switch (Gbl.FileBrowser.Type)
           {
            case Brw_FILE_BRW_ADMIN_MARKS_CRS:
            case Brw_FILE_BRW_ADMIN_MARKS_GRP:
               /* Check file extension */
               if ((ExtensionIsAllowed = Str_FileIsHTML (Gbl.FileBrowser.NewFileOrFolderName)))
                 {
	          /* Check MIME type*/
                  if (!strcmp (FileType,"text/html") ||
                      !strcmp (FileType,"text/plain") ||
                      !strcmp (FileType,"application/octet-stream"))
	             MIMETypeIsAllowed = true;
                  else	// MIME type forbidden
                    {
                     sprintf (Gbl.Message,Txt_The_MIME_type_X_of_file_Y_is_not_allowed[Gbl.Prefs.Language],
                              FileType,Gbl.FileBrowser.NewFileOrFolderName);
                     Lay_ShowAlert (WARNING,Gbl.Message);
                    }
	         }
	       else
                 {
                  sprintf (Gbl.Message,Txt_X_file_type_is_not_HTML[Gbl.Prefs.Language],
                           Gbl.FileBrowser.NewFileOrFolderName);
                  Lay_ShowAlert (WARNING,Gbl.Message);
                 }
               break;
            default:
               /* Check the file extension */
               for (Type=0; Type<Brw_NUM_FILE_EXT_ALLOWED; Type++)
                  if (Str_FileIs (Gbl.FileBrowser.NewFileOrFolderName,Brw_FileExtensionsAllowed[Type]))
 	            {
	             ExtensionIsAllowed = true;
	             break;
	            }
               if (ExtensionIsAllowed)
                 {
	          /* Check type MIME */
                  for (Type=0; Type<Brw_NUM_MIME_TYPES_ALLOWED; Type++)
                     if (!strcmp (FileType,Brw_MIMETypesAllowed[Type]))
	               {
	                MIMETypeIsAllowed = true;
	                break;
	               }
                  if (!MIMETypeIsAllowed)
                    {
                     sprintf (Gbl.Message,Txt_The_MIME_type_X_of_file_Y_is_not_allowed[Gbl.Prefs.Language],
                              FileType,Gbl.FileBrowser.NewFileOrFolderName);
                     Lay_ShowAlert (WARNING,Gbl.Message);
                     Lay_ShowAlert (INFO,Txt_Try_sending_a_compressed_file[Gbl.Prefs.Language]);
                    }
                 }
               else
                 {
                  sprintf (Gbl.Message,Txt_The_file_X_has_an_extension_not_allowed[Gbl.Prefs.Language],
	                   Gbl.FileBrowser.NewFileOrFolderName);
                  Lay_ShowAlert (WARNING,Gbl.Message);
                  Lay_ShowAlert (INFO,Txt_Try_sending_a_compressed_file[Gbl.Prefs.Language]);
                 }
           }
         if (ExtensionIsAllowed && MIMETypeIsAllowed)
           {
            if (Brw_CheckIfFileOrFolderNameIsValid (Gbl.FileBrowser.NewFileOrFolderName))
              {
               /* In Gbl.FileBrowser.NewFileOrFolderName is the name of the new file */
               sprintf (PathPrivateRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.FileBrowser.Priv.FullPathInTree);
               if (strlen (PathPrivateRelFromCGI_BIN) + 1 + strlen (Gbl.FileBrowser.NewFileOrFolderName) + strlen (".tmp") > PATH_MAX)
	          Lay_ShowErrorAndExit ("Path is too long.");
               strcat (PathPrivateRelFromCGI_BIN,"/");
               strcat (PathPrivateRelFromCGI_BIN,Gbl.FileBrowser.NewFileOrFolderName);

               /* Check if the destination file exists */
               if (Fil_CheckIfPathExists (PathPrivateRelFromCGI_BIN))
                 {
                  sprintf (Gbl.Message,Txt_Can_not_create_the_file_X_because_there_is_already_a_folder_or_a_file_with_that_name[Gbl.Prefs.Language],
                           Gbl.FileBrowser.NewFileOrFolderName);
                  Lay_ShowAlert (WARNING,Gbl.Message);
                 }
               else
                 {
                  /* End receiving the file */
                  sprintf (PathPrivateRelFromCGI_BINTmp,"%s.tmp",PathPrivateRelFromCGI_BIN);
                  FileIsValid = Fil_EndReceptionOfFile (PathPrivateRelFromCGI_BINTmp);

                  /* Check if the content of the file of marks is valid */
                  if (FileIsValid)
                     if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_CRS ||
                         Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP)
                       {
                        if (Brw_CheckMarksFile (PathPrivateRelFromCGI_BINTmp,&NumRowsHeader,&NumRowsFooter))
                          {
                           Gbl.Marks.Header = NumRowsHeader;
                           Gbl.Marks.Footer = NumRowsFooter;
                          }
                        else
                           FileIsValid = false;
                       }

                  if (FileIsValid)
                    {
                     /* Rename the temporary */
                     if (rename (PathPrivateRelFromCGI_BINTmp,PathPrivateRelFromCGI_BIN))
	               {
	                Brw_RemoveTree (PathPrivateRelFromCGI_BINTmp);
                        Lay_ShowErrorAndExit ("Can not create file.");
	               }
                     else
	               {
	                /* Check if quota has been exceeded */
	                Brw_CalcSizeOfDir (Gbl.FileBrowser.Priv.PathRelRootFolder);
	                Brw_SetMaxQuota ();
                        if (Brw_CheckIfQuotaExceded ())
	                  {
	                   Brw_RemoveTree (PathPrivateRelFromCGI_BIN);
	                   sprintf (Gbl.Message,Txt_Can_not_create_the_file_X_because_it_would_exceed_the_disk_quota[Gbl.Prefs.Language],
		                    Gbl.FileBrowser.NewFileOrFolderName);
                           Lay_ShowAlert (WARNING,Gbl.Message);
	                  }
	                else
                          {
                           /* Remove affected clipboards */
                           Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

                           /* Add path where new file is created to table of expanded folders */
                           Brw_InsThisFolderAndUpdOtherFoldersInExpandedFolders (Gbl.FileBrowser.Priv.FullPathInTree);

                           if (Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_CRS ||
                               Gbl.FileBrowser.Type == Brw_FILE_BRW_COMMON_GRP)
                             {
                              /* Add entry to the table of creators of files/folders */
                              sprintf (PathCompleteInTreeIncludingFile,"%s/%s",Gbl.FileBrowser.Priv.FullPathInTree,Gbl.FileBrowser.NewFileOrFolderName);
                              Brw_AddPathCommonToDB (Gbl.CurrentCrs.Grps.GrpCod,PathCompleteInTreeIncludingFile);
                             }

                           /* Show message of confirmation */
	                   sprintf (Gbl.Message,Txt_The_file_X_has_been_placed_inside_the_folder_Y[Gbl.Prefs.Language],
	    	                    Gbl.FileBrowser.NewFileOrFolderName,
                                    Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Gbl.FileBrowser.Level,Gbl.FileBrowser.FileOrFolderName));
                           Lay_ShowAlert (INFO,Gbl.Message);

                           /* Notify new file of marks to all users in course */
                           if (Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_CRS ||
                               Gbl.FileBrowser.Type == Brw_FILE_BRW_ADMIN_MARKS_GRP)
                             {
                              NumUsrsToBeNotified = Ntf_StoreNotifyEventsToAllUsrs (Ntf_NEW_MARKS_FILE,-1L,-1L);
                              Ntf_ShowAlertNumUsrsToBeNotified (NumUsrsToBeNotified);
                             }
                          }
                       }
	            }
                  else
	             Brw_RemoveTree (PathPrivateRelFromCGI_BINTmp);
                 }
              }
            else	// File name not valid
               Lay_ShowAlert (WARNING,Gbl.Message);
           }
        }
      else	// Empty filename
         Lay_ShowAlert (WARNING,Txt_You_must_specify_the_file[Gbl.Prefs.Language]);
     }
   else
      Lay_ShowErrorAndExit ("You can not create files or folders here.");

   /***** Show again file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/******************* Show file or folder in a file browser *******************/
/*****************************************************************************/

void Brw_ShowDownload (void)
  {
   extern const char *Txt_File_folder_X_is_now_visible[Txt_NUM_LANGUAGES];

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** If the file or folder is hidden, show it *****/
   if (Brw_GetIfHiddenDownload (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree))
      Brw_RemoveOneHiddenDownloadFromDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);

   /***** Remove the affected clipboards *****/
   Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

   /***** Write message of confirmation *****/
   sprintf (Gbl.Message,Txt_File_folder_X_is_now_visible[Gbl.Prefs.Language],
            Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Gbl.FileBrowser.Level,Gbl.FileBrowser.FileOrFolderName));
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show again the file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/***************** Hide file or folder in a file browser *********************/
/*****************************************************************************/

void Brw_HideDownload (void)
  {
   extern const char *Txt_File_folder_X_is_now_hidden[Txt_NUM_LANGUAGES];

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** If the file or folder is not already hidden, hide it *****/
   if (!Brw_GetIfHiddenDownload (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree))
      Brw_AddHiddenDownloadToDB (Gbl.CurrentCrs.Grps.GrpCod,Gbl.FileBrowser.Priv.FullPathInTree);

   /***** Remove the affected clipboards *****/
   Brw_RemoveAffectedClipboards (Gbl.FileBrowser.Type,Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Other.UsrDat.UsrCod);

   /***** Write confirmation message *****/
   sprintf (Gbl.Message,Txt_File_folder_X_is_now_hidden[Gbl.Prefs.Language],
            Brw_GetFileNameToShow (Gbl.FileBrowser.Type,Gbl.FileBrowser.Level,Gbl.FileBrowser.FileOrFolderName));
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show again the file browser *****/
   Brw_ShowAgainFileBrowserOrWorks ();
  }

/*****************************************************************************/
/********* Check if a file or folder from the download zone is hidden ********/
/*****************************************************************************/

static bool Brw_GetIfHiddenDownload (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a download is hidden from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM hidden_downloads WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a download is hidden");

   /***** Get 0 (not hidden) or 1 (hidden) from query result *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a download is hidden.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/*****************************************************************************/
/************* Check if a folder from a file browser is expanded *************/
/*****************************************************************************/

static bool Brw_GetIfExpandedTree (const char *Path)
  {
   char Query[512+PATH_MAX];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a folder is expanded from database *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_SEE_DOWNLOAD_CRS:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS:
      case Brw_FILE_BRW_COMMON_CRS:
      case Brw_FILE_BRW_SEE_MARKS_CRS:
      case Brw_FILE_BRW_ADMIN_MARKS_CRS:
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_WORKS_USR:
         sprintf (Query,"SELECT COUNT(*) FROM expanded_folders WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Path);
         break;
      case Brw_FILE_BRW_SEE_DOWNLOAD_GRP:
      case Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP:
      case Brw_FILE_BRW_COMMON_GRP:
      case Brw_FILE_BRW_SEE_MARKS_GRP:
      case Brw_FILE_BRW_ADMIN_MARKS_GRP:
         sprintf (Query,"SELECT COUNT(*) FROM expanded_folders WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND GrpCod='%ld' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Grps.GrpCod,Path);
         break;
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
      case Brw_FILE_BRW_WORKS_CRS:
         sprintf (Query,"SELECT COUNT(*) FROM expanded_folders WHERE UsrCod='%ld' AND FileBrowser='%u' AND CrsCod='%ld' AND WorksUsrCod='%ld' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Other.UsrDat.UsrCod,Path);
         break;
      case Brw_FILE_BRW_BRIEFCASE_USR:
         sprintf (Query,"SELECT COUNT(*) FROM expanded_folders WHERE UsrCod='%ld' AND FileBrowser='%u' AND Path='%s/'",
                  Gbl.Usrs.Me.UsrDat.UsrCod,(unsigned) Gbl.FileBrowser.Type,Path);
         break;
      default:
         break;
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get check if a folder is expanded");

   /***** Get 0 (not hidden) or 1 (hidden) from query result *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a folder is expanded.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/*****************************************************************************/
/****************** Add a hidden download to the database ********************/
/*****************************************************************************/

static void Brw_AddHiddenDownloadToDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Add hidden download file or folder to the database *****/
   sprintf (Query,"INSERT INTO hidden_downloads (CrsCod,GrpCod,Path)" \
                  " VALUES ('%ld','%ld','%s')",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryINSERT (Query,"can not add a hidden download to database");
  }

/*****************************************************************************/
/************** Remove a hidden download from the database *******************/
/*****************************************************************************/

static void Brw_RemoveOneHiddenDownloadFromDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove hidden download file or folder from the database *****/
   sprintf (Query,"DELETE FROM hidden_downloads" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryDELETE (Query,"can not remove a hidden download from database");
  }

/*****************************************************************************/
/********* Remove the children of a hidden download from the database ********/
/*****************************************************************************/

static void Brw_RemoveChildrenOfHiddenDownloadFromDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove children of a hidden download folder from the database *****/
   sprintf (Query,"DELETE FROM hidden_downloads WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryDELETE (Query,"can not remove hidden downloads from database");
  }

/*****************************************************************************/
/************ Rename a path in in the table of hidden downloads **************/
/*****************************************************************************/

static void Brw_RenameOneHiddenDownloadFolderInDB (long GrpCod,const char *OldPath,const char *NewPath)
  {
   char Query[256+PATH_MAX*2];

   /***** Update a hidden download in the table of hidden downloads *****/
   sprintf (Query,"UPDATE hidden_downloads SET Path='%s' WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            NewPath,Gbl.CurrentCrs.Crs.CrsCod,GrpCod,OldPath);
   DB_QueryUPDATE (Query,"can not update hidden download in database");
  }

/*****************************************************************************/
/***** Rename the children of a folder in the table of hidden downloads ******/
/*****************************************************************************/

static void Brw_RenameChildrenHiddenDownloadsInDB (long GrpCod,const char *OldPath,const char *NewPath)
  {
   char Query[256+PATH_MAX*2];
   unsigned StartFinalSubpathNotChanged = strlen (OldPath) + 2;

   /***** Update the children of a folder in the table of hidden downloads *****/
   sprintf (Query,"UPDATE hidden_downloads SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
            NewPath,StartFinalSubpathNotChanged,Gbl.CurrentCrs.Crs.CrsCod,GrpCod,OldPath);
   DB_QueryUPDATE (Query,"can not update hidden downloads in database");
  }

/*****************************************************************************/
/********** Add a path of the course common zone to the database *************/
/*****************************************************************************/

static void Brw_AddPathCommonToDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Add owner of the path to the database *****/
   sprintf (Query,"INSERT INTO crs_common_files (CrsCod,GrpCod,Path,UsrCod) VALUES ('%ld','%ld','%s','%ld')",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryINSERT (Query,"can not add common zone path to database");
  }

/*****************************************************************************/
/****** Get the user's code of the owner of a file common of the course ******/
/*****************************************************************************/

static long Brw_GetCreatorFileOrFolderCommon (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long UsrCod;

   /***** Get the owner of a file common of this course from database *****/
   sprintf (Query,"SELECT UsrCod FROM crs_common_files" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the owner of a common file");

   /***** Get user's code *****/
   if (NumRows == 0)
      UsrCod = -1L;
   else
     {
      /* Get user's code */
      row = mysql_fetch_row (mysql_res);
      UsrCod = Str_ConvertStrCodToLongCod (row[0]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return UsrCod;
  }

/*****************************************************************************/
/******** Remove a file or folder of a common zone from the database *********/
/*****************************************************************************/

static void Brw_RemoveOneFileOrFolderCommonFromDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove from database the entry that stores the owner of a common file *****/
   sprintf (Query,"DELETE FROM crs_common_files WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryDELETE (Query,"can not remove common zone path from database");
  }

/*****************************************************************************/
/****** Remove children of a folder of a common zone from the database *******/
/*****************************************************************************/

static void Brw_RemoveChildrenOfFolderCommonFromDB (long GrpCod,const char *Path)
  {
   char Query[256+PATH_MAX];

   /***** Remove from database the entries that store the owners of common files *****/
   sprintf (Query,"DELETE FROM crs_common_files" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
            Gbl.CurrentCrs.Crs.CrsCod,GrpCod,Path);
   DB_QueryDELETE (Query,"can not remove common zone paths from database");
  }

/*****************************************************************************/
/********* Rename a common file or folder in table of common files ***********/
/*****************************************************************************/

static void Brw_RenameOneFolderCommonInDB (long GrpCod,const char *OldPath,const char *NewPath)
  {
   char Query[256+PATH_MAX*2];

   /***** Update common file or folder in table of common files *****/
   sprintf (Query,"UPDATE crs_common_files SET Path='%s'" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path='%s'",
            NewPath,Gbl.CurrentCrs.Crs.CrsCod,GrpCod,OldPath);
   DB_QueryUPDATE (Query,"can not update folder name in a common zone");
  }

/*****************************************************************************/
/******** Rename childrens a common folder in table of common files **********/
/*****************************************************************************/

static void Brw_RenameChildrenFilesOrFoldersCommonInDB (long GrpCod,const char *OldPath,const char *NewPath)
  {
   char Query[256+PATH_MAX*2];
   unsigned StartFinalSubpathNotChanged = strlen (OldPath) + 2;

   /***** Update children of a common folder in table of common files *****/
   sprintf (Query,"UPDATE crs_common_files SET Path=CONCAT('%s','/',SUBSTRING(Path,%u))" \
                  " WHERE CrsCod='%ld' AND GrpCod='%ld' AND Path LIKE '%s/%%'",
            NewPath,StartFinalSubpathNotChanged,Gbl.CurrentCrs.Crs.CrsCod,GrpCod,OldPath);
   DB_QueryUPDATE (Query,"can not rename file or folder names in a common zone");
  }

/*****************************************************************************/
/********** Check if I have permission to modify a file or folder ************/
/*****************************************************************************/

static bool Brw_CheckIfICanModifyFileOrFolder (unsigned Level)
  {
   /***** Have I premission to:
          - remove the file or folder?
          - rename the folder? *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_COMMON_CRS:
         // Check if I am the owner of the folder
         return (Level ? Brw_CheckIfIHavePermissionFileOrFolderCommon (-1L) : false);
      case Brw_FILE_BRW_COMMON_GRP:
         // Check if I am the owner of the folder
         return (Level ? Brw_CheckIfIHavePermissionFileOrFolderCommon (Gbl.CurrentCrs.Grps.GrpCod) : false);
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         return ((Level == 1 &&
                  Gbl.FileBrowser.Asg.AsgCod < 0) ||	// If folder doesn't correspond to any assignment
                 (Level > 1 &&
                  (Gbl.FileBrowser.Asg.AsgCod < 0 ||	// If folder doesn't correspond to any assignment
                   (Gbl.FileBrowser.Asg.ICanDo &&	// If I can do this assignment
                    (Gbl.FileBrowser.Asg.Open || Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER)))));
      default:
         return (Level != 0 &&
                 Brw_FileBrowserIsEditable[Gbl.FileBrowser.Type]);
     }
  }

/*****************************************************************************/
/**** Check if I have permission to create a file or folder into a folder ****/
/*****************************************************************************/

static bool Brw_CheckIfICanCreateIntoFolder (unsigned Level)
  {
   /***** Have I premission to create/paste a new file or folder into the folder? *****/
   switch (Gbl.FileBrowser.Type)
     {
      case Brw_FILE_BRW_ASSIGNMENTS_USR:
      case Brw_FILE_BRW_ASSIGNMENTS_CRS:
         return (Level != 0 &&
                 (Gbl.FileBrowser.Asg.AsgCod < 0 ||	// If folder doesn't correspond to any assignment
                  (Gbl.FileBrowser.Asg.ICanDo &&	// If I can do this assignment
                   (Gbl.FileBrowser.Asg.Open || Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER))));
      default:
         return Brw_FileBrowserIsEditable[Gbl.FileBrowser.Type];
     }
  }

/*****************************************************************************/
/********** Check if I have permission to modify a file or folder ************/
/********** in the current common zone                            ************/
/*****************************************************************************/
// Returns true if the current user can remove or rename Gbl.FileBrowser.Priv.FullPathInTree, and false if he have not permission
// A user can remove or rename a file if he's the owner
// A user can remove or rename a folder if he's the unique owner of all the files and folders in the subtree starting there

static bool Brw_CheckIfIHavePermissionFileOrFolderCommon (long GrpCod)	// Use GrpCod = -1L to select commons files for the whole course
  {
   char Query[512+PATH_MAX*2];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long OwnerUsrCod = -1L;

   switch (Gbl.Usrs.Me.UsrTypeLog)
     {
      case Usr_STUDENT:	// If I am a student, I can modify the file/folder if I am the owner
         /***** Get all the distinct owners of files starting by Gbl.FileBrowser.Priv.FullPathInTree from database *****/
         sprintf (Query,"SELECT DISTINCT(UsrCod) FROM crs_common_files" \
                        " WHERE CrsCod='%ld' AND GrpCod='%ld' AND (Path='%s' OR Path LIKE '%s/%%')",
                  Gbl.CurrentCrs.Crs.CrsCod,GrpCod,
                  Gbl.FileBrowser.Priv.FullPathInTree,Gbl.FileBrowser.Priv.FullPathInTree);
         NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get owners of common zone files");

         /***** Check all common files that are equal to Gbl.FileBrowser.Priv.FullPathInTree
                or that are under the folder Gbl.FileBrowser.Priv.FullPathInTree *****/
         if (NumRows == 1)	// Get the owner of the file(s)
           {
            row = mysql_fetch_row (mysql_res);
            OwnerUsrCod = Str_ConvertStrCodToLongCod (row[0]);
           }

         /***** Free structure that stores the query result *****/
         DB_FreeMySQLResult (&mysql_res);

         return (Gbl.Usrs.Me.UsrDat.UsrCod == OwnerUsrCod);	// Am I the owner of subtree?
      case Usr_TEACHER:
      case Usr_DEG_ADMIN:
      case Usr_SUPERUSER:
         return true;
      default:
         return false;
     }
  }

/*****************************************************************************/
/****************** Check if the name of a file is allowed *******************/
/*****************************************************************************/
// Return true if the name of the file o folder is valid
// If the name is not valid, put in Gbl.Message a message of error and return false

bool Brw_CheckIfFileOrFolderNameIsValid (const char *FileName)
  {
   const char *Ptr;
   unsigned NumAlfanum = 0,NumSpaces = 0,NumPoints = 0;

   for (Ptr = FileName; *Ptr; Ptr++)
     {
      if ((*Ptr >= 'a' && *Ptr <= 'z') || (*Ptr >= 'A' && *Ptr <= 'Z') || (*Ptr >= '0' && *Ptr <= '9') || *Ptr == '_' || *Ptr == '-')
        {
         NumSpaces = NumPoints = 0;
         NumAlfanum++;
        }
      else if (*Ptr == ' ')
        {
         NumPoints = 0;
         if (++NumSpaces > 1)
           {
            strcpy (Gbl.Message,"El nombre de un archivo o carpeta no puede contener dos o m&aacute;s espacios consecutivos.");	// Need translation!
            return false;
           }
        }
      else if (*Ptr == '.')
        {
         NumSpaces = 0;
         if (++NumPoints > 1)
           {
            strcpy (Gbl.Message,"El nombre de un archivo o carpeta no puede contener dos o m&aacute;s puntos consecutivos.");	// Need translation!
            return false;
           }
        }
      else
        {
         strcpy (Gbl.Message,"El nombre de un archivo o carpeta no puede contener tildes, e&ntilde;es ni signos de puntuaci&oacute;n" \
                             " (con la excepci&oacute;n de &quot;.&quot;, &quot;&nbsp;&quot;, &quot;_&quot; y &quot;-&quot;)," \
                             " ya que algunos navegadores no permiten la descarga con esos caracteres.");	// Need translation!
         return false;
        }
     }
   if (NumAlfanum == 0)
     {
      strcpy (Gbl.Message,"El nombre de un archivo o carpeta tiene que contener al menos un car&aacute;cter alfanum&eacute;rico.");	// Need translation!
      return false;
     }
   return true;
  }

/****************************************************************************/
/************ Remove common zones of all the groups of a type ***************/
/****************************************************************************/

void Brw_RemoveZonesOfGroupsOfType (long GrpTypCod)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct GroupData GrpDat;

   /***** Query database *****/
   if ((NumRows = Grp_GetGrpsOfType (GrpTypCod,&mysql_res)))	// If there exists groups...
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
	 /* Get next group */
	 row = mysql_fetch_row (mysql_res);

         /* Group code is in row[0] */
         if (sscanf (row[0],"%ld",&(GrpDat.GrpCod)) != 1)
            Lay_ShowErrorAndExit ("Wrong group code.");

         /* Get name and type of the group from database */
         Grp_GetDataOfGroupByCod (&GrpDat);

         /* Remove file zones of this group */
         Brw_RemoveGrpZonesVerbose (&GrpDat);
	}

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/************* Remove file zones of a group and show message ****************/
/****************************************************************************/

void Brw_RemoveGrpZonesVerbose (struct GroupData *GrpDat)
  {
   extern const char *Txt_File_zones_of_the_group_X_Y_removed[Txt_NUM_LANGUAGES];

   /***** Remove group zones and clipboards *****/
   Brw_RemoveGrpZones (Gbl.CurrentCrs.Crs.CrsCod,GrpDat->GrpCod);

   /***** Print message *****/
   sprintf (Gbl.Message,Txt_File_zones_of_the_group_X_Y_removed[Gbl.Prefs.Language],
            GrpDat->GrpTypName,GrpDat->GrpName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/********************** Remove file zones of a group ************************/
/****************************************************************************/

void Brw_RemoveGrpZones (long CrsCod,long GrpCod)
  {
   char PathGrpFileZones[PATH_MAX+1];

   /***** Remove group zones *****/
   sprintf (PathGrpFileZones,"%s/%ld/grp/%ld",Cfg_PATH_REL_CRS_PRIVATE,CrsCod,GrpCod);
   Brw_RemoveTree (PathGrpFileZones);

   /***** Remove size of file zones in the group from database *****/
   Brw_RemoveSizeOfFileTreeFromDB (CrsCod,GrpCod,-1L);

   /***** Remove clipboard entries in this group *****/
   Brw_RemoveGrpClipboards (GrpCod);
  }

/****************************************************************************/
/**************** Remove the works of a user in a course ********************/
/****************************************************************************/

void Brw_RemoveUsrWorksInCrs (struct UsrData *UsrDat,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   extern const char *Txt_Works_of_the_user_with_ID_X_in_Y_removed[Txt_NUM_LANGUAGES];
   char PathRelUsrEnAsg[PATH_MAX+1];

   /***** Remove the folder usr/DNI inside the course *****/
   sprintf (PathRelUsrEnAsg,"%s/%ld/usr/%s",Cfg_PATH_REL_CRS_PRIVATE,Crs->CrsCod,UsrDat->DNI);
   Brw_RemoveTree (PathRelUsrEnAsg);

   /***** Remove size of file zone of the course from database *****/
   Brw_RemoveSizeOfFileTreeFromDB (Crs->CrsCod,-1L,UsrDat->UsrCod);

   /***** Write message *****/
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,Txt_Works_of_the_user_with_ID_X_in_Y_removed[Gbl.Prefs.Language],
               UsrDat->DNI,Crs->FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
  }

/****************************************************************************/
/************* Remove the works of a user in all of his courses *************/
/****************************************************************************/

void Brw_RemoveUsrWorksInAllCrss (struct UsrData *UsrDat,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows,NumAsgsBorrTrbj=0;
   struct Course Crs;

   /***** Query database *****/
   if ((NumRows = Usr_GetCrssFromUsr (UsrDat->UsrCod,-1L,&mysql_res)) > 0) // If courses found
     {
      /***** Remove the zone of works of the user in the courses he/she belongs to *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
	 /* Get the next course */
	 row = mysql_fetch_row (mysql_res);
         Crs.CrsCod = Str_ConvertStrCodToLongCod (row[0]);
         /* Get data of course */
         Deg_GetDataOfCourseByCod (&Crs);
         Brw_RemoveUsrWorksInCrs (UsrDat,&Crs,QuietOrVerbose);
         NumAsgsBorrTrbj++;
	}
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Write message *****/
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"Trabajos del usuario" \
                           " con DNI <strong>%s</strong>" \
                           " en un total de %lu de sus %lu asignaturas eliminados.",	// Need translation!
               UsrDat->DNI,NumAsgsBorrTrbj,NumRows);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
  }

/*****************************************************************************/
/************************ Receive a new file of marks ************************/
/*****************************************************************************/

bool Brw_CheckMarksFile (const char *Path,unsigned *NumRowsHead,unsigned *NumRowsFoot)
  {
   FILE *FileMarks;
   char CadenaLeida[Usr_MAX_LENGTH_USR_ID+1];
   bool EndOfHead = false,EndOfTable = false,FileIsCorrect = true;
   unsigned NumRowsStds = 0;

   *NumRowsHead = *NumRowsFoot = 0;

   /***** Open file with the table of marks *****/
   if ((FileMarks = fopen (Path,"rt")))
     {
      /***** Check if there is a table in the received file ******/
      if (Str_FindStrInFile (FileMarks,"<table",Str_NO_SKIP_HTML_COMMENTS))
        {
         // There is a table
         Str_FindStrInFile (FileMarks,">",Str_NO_SKIP_HTML_COMMENTS);

         // Only one table is allowed
         if (Str_FindStrInFile (FileMarks,"<table",Str_NO_SKIP_HTML_COMMENTS))
           {
            Lay_ShowAlert (WARNING,"Hay m&aacute;s de una tabla en el archivo de calificaciones.");	// Need translation!
            FileIsCorrect = false;
           }
         else
           {
            /* Return to table start */
            rewind (FileMarks);
            Str_FindStrInFile (FileMarks,"<table",Str_NO_SKIP_HTML_COMMENTS);
            Str_FindStrInFile (FileMarks,">",Str_NO_SKIP_HTML_COMMENTS);

            /* We assume that the structure of the table has several rows of header until the first row of students is found,
	       then it has a number of rows of students, including some dummy rows without students,
               and finally it has several rows of footer from the last row of students until the end of the table */
            /* Count rows of header */
            while (!EndOfHead)
               if (Str_FindStrInFile (FileMarks,"<tr",Str_NO_SKIP_HTML_COMMENTS))   // Nos vamos to the next row
                 {
                  // Every DNI must be in the first column of the row
                  if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Str_GetCellFromHTMLTableSkipComments (FileMarks,CadenaLeida,Usr_MAX_LENGTH_USR_ID)))
                    {
	             EndOfHead = true;
                     NumRowsStds++;
                    }
                  else
                     (*NumRowsHead)++;
                 }
               else
	          EndOfHead = true;	// No more rows

            /* Count rows of students and rows of footer */
            while (!EndOfTable)
               if (Str_FindStrInFile (FileMarks,"<tr",Str_NO_SKIP_HTML_COMMENTS))   // Go to the next row
                 {
                  // Each DNI must be in the first column of the row
                  if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Str_GetCellFromHTMLTableSkipComments (FileMarks,CadenaLeida,Usr_MAX_LENGTH_USR_ID)))
                    {
                     NumRowsStds++;
                     *NumRowsFoot = 0;
                    }
                  else
                     (*NumRowsFoot)++;
                 }
               else
	          EndOfTable = true;	// No more rows
           }
        }
      else
        {
         Lay_ShowAlert (WARNING,"No hay ninguna tabla en el archivo de calificaciones.");	// Need translation!
         FileIsCorrect = false;
        }

      /***** The file of marks is no more necessary. Close it. *****/
      fclose (FileMarks);
     }
   if (FileIsCorrect)
     {
      sprintf (Gbl.Message,"Se han detectado %u fila(s) de cabecera, %u fila(s) de alumnos y %u fila(s) de pie.",	// Need translation!
               *NumRowsHead,NumRowsStds,*NumRowsFoot);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   return FileIsCorrect;
  }

/****************************************************************************/
/*************************** Show the marks of a user ***********************/
/****************************************************************************/

void Brw_ShowMyMarks (void)
  {
   unsigned Row;
   unsigned Header;
   unsigned Footer;
   char DNI[Usr_MAX_LENGTH_USR_ID+1];
   char PathPrivateRelFromCGI_BIN[PATH_MAX+1];
   FILE *FileMarks;
   bool DNIFound = true;

   /***** Get parameters related to file browser *****/
   Brw_GetParAndInitFileBrowser ();

   /***** Get the path of the file of marks *****/
   Brw_ConstructCompletePathInTree (Gbl.FileBrowser.Priv.PathInTreeExceptFileOrFolder,Gbl.FileBrowser.FileOrFolderName);
   sprintf (PathPrivateRelFromCGI_BIN,"%s/%s",Gbl.FileBrowser.Priv.PathRelAboveRootFolder,Gbl.FileBrowser.Priv.FullPathInTree);

   /***** Get number of rows of header or footer *****/
   Brw_GetNumRowsHeaderAndFooter (&Header,&Footer);

   /***** Open file with the table of marks *****/
   if ((FileMarks = fopen (PathPrivateRelFromCGI_BIN,"rt")))
     {
      /***** Assign the DNI of the student whose marks will be shown *****/
      if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)		// If I am logged as student...
         strcpy (DNI,Gbl.Usrs.Me.UsrDat.DNI);			// ...use my DNI
      else							// If I am logged as teacher, administrator, superuser...
        {
         /* Select a random student from the course */
         if (Gbl.CurrentCrs.Grps.GrpCod >= 0)	// Group zone
           {
            if (Grp_GetNumStdsInGrp (Gbl.CurrentCrs.Grps.GrpCod))
               Usr_GetDNIRandomStdInGrp (Gbl.CurrentCrs.Grps.GrpCod,DNI);
            else
              {
               DNIFound = false;
               Usr_ShowWarningNoUsersFound (Usr_STUDENT);
              }
           }
         else					// Course zone
           {
            if (Gbl.CurrentCrs.Crs.NumStds)	// If there are student in this course
               Usr_GetDNIRandomStdInCrs (DNI);
            else
              {
               DNIFound = false;
               Usr_ShowWarningNoUsersFound (Usr_STUDENT);
              }
           }
        }

      if (DNIFound)
        {
         /***** Check if it exists a DNI in the first column of the table of marks *****/
         /* Jump to table start */
         Str_FindStrInFile (FileMarks,"<table",Str_NO_SKIP_HTML_COMMENTS);
         Str_FindStrInFile (FileMarks,">",Str_NO_SKIP_HTML_COMMENTS);
         if (!Str_FindStrInFirstColHTMLTable (FileMarks,DNI)) // The file gets located after the DNI
           {
	    sprintf (Gbl.Message,"No se ha encontrado el alumno con DNI <strong>%s</strong>" \
                                 " en el archivo de calificaciones.",	// Need translation!
                     DNI);
	    Lay_ShowAlert (WARNING,Gbl.Message);
	    return;
           }

         /***** Start HTML output *****/
         fprintf (Gbl.F.Out,"Content-type: text/html; charset=ISO-8859-1\n\n"); /* Two carriage returns aree mandatory at the end */
         Gbl.HTMLStartWritten = true;

         /***** Write all until the header (included) *****/
         /* Go to start of file */
         rewind (FileMarks);

         /* Write until table start */
         Str_WriteUntilStrFoundInFileIncludingStr (Gbl.F.Out,FileMarks,"<table",Str_SKIP_HTML_COMMENTS);
         Str_WriteUntilStrFoundInFileIncludingStr (Gbl.F.Out,FileMarks,">",Str_NO_SKIP_HTML_COMMENTS);

         /* Write header */
         for (Row=1; Row<=Header; Row++)
            Str_WriteUntilStrFoundInFileIncludingStr (Gbl.F.Out,FileMarks,"</tr>",Str_SKIP_HTML_COMMENTS);

         /****** Write the row corresponding to the student *****/
         /* Find DNI */
         Str_FindStrInFirstColHTMLTable (FileMarks,DNI);

         /* Find backward until "<tr" */
         Str_FindStrInFileBack (FileMarks,"<tr",Str_SKIP_HTML_COMMENTS);

         /* Write until "</tr>" */
         Str_WriteUntilStrFoundInFileIncludingStr (Gbl.F.Out,FileMarks,"</tr>",Str_SKIP_HTML_COMMENTS);

         /***** Write the footer and all until the end *****/
         /* Find the footer of the table */
         Str_FindStrInFile (FileMarks,"</table>",Str_NO_SKIP_HTML_COMMENTS);
         Str_FindStrInFileBack (FileMarks,"</table>",Str_NO_SKIP_HTML_COMMENTS);

         for (Row=1; Row<=Footer; Row++)
	    Str_FindStrInFileBack (FileMarks,"<tr",Str_SKIP_HTML_COMMENTS);

         /* Write the footer of the table */
         for (Row=1; Row<=Footer; Row++)
            Str_WriteUntilStrFoundInFileIncludingStr (Gbl.F.Out,FileMarks,"</tr>",Str_SKIP_HTML_COMMENTS);

         /* Write the end */
         Str_FindStrInFile (FileMarks,"</table>",Str_NO_SKIP_HTML_COMMENTS);
         Str_FindStrInFileBack (FileMarks,"</table>",Str_NO_SKIP_HTML_COMMENTS);
         Fil_FastCopyOfOpenFiles (FileMarks,Gbl.F.Out);
        }

      /***** The file of marks is no more needed. Close it. *****/
      fclose (FileMarks);

      Gbl.TablEndWritten = Gbl.HTMLEndWritten = true;
     }
   else	// Can't open the file with the table of marks
      Lay_ShowErrorAndExit ("Can not open file of marks.");
  }
