// swad_txt.c: text messages on screen, in several languages

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include "swad_asg.h"
#include "swad_brw.h"
#include "swad_crs.h"
#include "swad_hld.h"
#include "swad_ntf.h"
#include "swad_rec.h"
#include "swad_sta.h"
#include "swad_tst.h"
#include "swad_txt.h"

/****************************************************************************/
/******************************** Constants *********************************/
/****************************************************************************/

// The HTML entity for "" is &szlig; It stants for "S-Z ligature", because this symbol comes from the ligature of a gothic S and Z.

const char *Txt_StrLangId[Txt_NUM_LANGUAGES] =
	{
	"de",
	"en",
	"es",
	"fr",
	"it",
	"pt",
	};
const char *Txt_StrLangName[Txt_NUM_LANGUAGES] =
	{
	"Deutsch",
	"English",
	"Espa&ntilde;ol",
	"Fran&ccedil;ais",
	"Italiano",
	"Portugu&ecirc;s",
	};
const char *Txt_A_student_can_belong_to_several_groups[Txt_NUM_LANGUAGES] =
	{
	"Ein Student kann zu mehreren Gruppen",
	"A student can belong to several groups",
	"Un alumno puede pertenecer a varios grupos",
	"Un &eacute;tudiant peut appartenir &agrave; plusieurs groupes",
	"Uno studente pu&ograve; appartenere a pi&ugrave; gruppi",
	"Um aluno pode pertencer a v&aacute;rios grupos",
	};
const char *Txt_A_student_can_only_belong_to_one_group[Txt_NUM_LANGUAGES] =
	{
	"Ein Student kann nur zu einer Gruppe",
	"A student can only belong to one group",
	"Un alumno s&oacute;lo puede pertenecer a un grupo",
	"Un &eacute;tudiant ne peut appartenir &agrave; un groupe",
	"Uno studente non pu&ograve; che appartenere a un gruppo",
	"Um aluno pode pertencer a um grupo",
	};
const char *Txt_About[Txt_NUM_LANGUAGES] =
	{
	"&Uuml;ber",
	"About",
	"Acerca de",
	"&Agrave; propos de",
	"Chi",
	"Sobre",
	};
const char *Txt_Academic_year[Txt_NUM_LANGUAGES] =
	{
	"Akademisches Jahr",
	"Academic year",
	"Curso",
	"Ann&eacute;e scolaire",
	"Anno accademico",
	"Ano lectivo de",
	};
const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"zug&auml;nglich f&uuml;r das Lesen und Schreiben von Studenten und Dozenten des Vorlesung",
	"accessible for reading and writing by students and teachers of the course",
	"accesible para lectura por alumnos y profesores de la asignatura",
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de la mati&egrave;re",
	"accessibile la lettura e la scrittura da parte degli studenti e degli insegnanti della materia",
	"acess&iacute;vel para leitura e escrita pelos alunos e professores da disciplina",
	};
const char *Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"zug&auml;nglich f&uuml;r das Lesen und Schreiben von Studenten des Gruppe und Dozenten des Vorlesung",
	"accessible for reading and writing by students of the group and teachers of the course",
	"accesible para lectura por alumnos del grupo y profesores de la asignatura",
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re",
	"accessibile la lettura e la scrittura da parte degli studenti del gruppo e degli insegnanti della materia",
	"acess&iacute;vel para leitura e escrita pelos alunos do grupo e professores da disciplina",
	};
const char *Txt_accessible_for_reading_and_writing_by_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"zug&auml;nglich f&uuml;r das Lesen und Schreiben von Dozenten des Vorlesung",
	"accessible for reading and writing by teachers of the course",
	"accesible para lectura por profesores de la asignatura",
	"accessible en lecture et en &eacute;criture par les enseignants de la mati&egrave;re",
	"accessibile la lettura e la scrittura da parte degli insegnanti della materia",
	"acess&iacute;vel para leitura e escrita pelos professores da disciplina",
	};
const char *Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"zug&auml;nglich f&uuml;r das Lesen und Schreiben von Ihnen und dem Dozenten des Vorlesung",
	"accessible for reading and writing by you and the teachers of the course",
	"accesible para lectura por usted y los profesores de la asignatura",
	"accessible en lecture et en &eacute;criture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re",
	"accessibile la lettura e la scrittura da parte da voi e degli insegnanti della materia",
	"acess&iacute;vel para leitura e escrita por voc&ecirc; e os professores da disciplina",
	};
const char *Txt_accessible_only_for_reading_by_you_and_the_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"nur f&uuml;r das Lesen von Ihnen und dem Dozenten des Vorlesung",
	"accessible only for reading by you and the teachers of the course",
	"accesible s&oacute;lo para lectura por usted y los profesores de la asignatura",
	"accessible uniquement pour la lecture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re",
	"accessibile solo per la lettura da parte da voi e degli insegnanti della materia",
	"acess&iacute;vel apenas para leitura por voc&ecirc; e os professores da disciplina",
	};
const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"nur f&uuml;r das Lesen von Studenten und Dozenten des Vorlesung",
	"accessible only for reading by students and teachers of the course",
	"accesible s&oacute;lo para lectura por alumnos y profesores de la asignatura",
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de la mati&egrave;re",
	"accessibile solo per la lettura da parte degli studenti e degli insegnanti della materia",
	"acess&iacute;vel apenas para leitura pelos alunos e professores da disciplina",
	};
const char *Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"nur f&uuml;r das Lesen von Studenten des Gruppe und Dozenten des Vorlesung",
	"accessible only for reading by students of the group and teachers of the course",
	"accesible s&oacute;lo para lectura por alumnos del grupo y profesores de la asignatura",
	"accessible uniquement pour la lecture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re",
	"accessibile solo per la lettura da parte degli studenti del gruppo e degli insegnanti della materia",
	"acess&iacute;vel apenas para leitura pelos alunos do grupo e professores da disciplina",
	};
const char *Txt_Action[Txt_NUM_LANGUAGES] =
	{
	"Aktion",
	"Action",
	"Acci&oacute;n",
	"Action",
	"Azione",
	"A&ccedil;&atilde;o",
	};
const char *Txt_Assignments[Txt_NUM_LANGUAGES] =
	{
	"Aktivit&auml;ten",
	"Assignments",
	"Actividades",
	"Activit&eacute;s",
	"Attivit&agrave;",
	"Atividades",
        };
const char *Txt_ASSIGNMENTS_HELP_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Sortieren nach Beginn",
		"Sort by start date",
		"Ordenar por fecha de comienzo",
		"Trier par date de d&eacute;but",
		"Ordina per data di inizio",
		"Classificar por data de in&iacute;cio",
		},
		{
		"Sortieren nach Enddatum",
		"Sort by end date",
		"Ordenar por fecha de finalizaci&oacute;n",
		"Trier par date de fin",
		"Ordina per data di fine",
		"Classificar por data de t&eacute;rmino",
		},
	};
const char *Txt_ASSIGNMENTS_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Startdatum",
		"Start<br />date",
		"Fecha<br />inicial",
		"Date de<br />d&eacute;but",
		"Data di<br />inizio",
		"Data de<br />in&iacute;cio",
		},
		{
		"Enddatum",
		"End<br />date",
		"Fecha<br />final",
		"Date de<br />fin",
		"Data di<br />fine",
		"Data de<br />t&eacute;rmino",
		},
	};
const char *Txt_Assignments_zone[Txt_NUM_LANGUAGES] =
        {
	"Aktivit&auml;ten-Zone",
	"Assignments zone",
	"Zona de actividades",
	"Zone d'activit&eacute;s",
	"Zona di attivit&agrave;",
	"Zona de atividades",
        };
const char *Txt_assignments_zone[Txt_NUM_LANGUAGES] =
        {
	"Aktivit&auml;ten-Zone",
	"assignments zone",
	"zona de actividades",
	"zone d'activit&eacute;s",
	"zona di attivit&agrave;",
	"zona de atividades",
        };
const char *Txt_Assignment[Txt_NUM_LANGUAGES] =
	{
	"Aktivit&auml;t",
	"Assignment",
	"Actividad",
	"Activit&eacute;",
	"Attivit&agrave;",
	"Atividade",
        };
const char *Txt_ACTIVITY_TYPES[Asg_NUM_TYPES_SEND_WORK][Txt_NUM_LANGUAGES] =
	{
		{
		"Melden Sie keine Hausarbeiten (files)",
		"Do not submit works (files)",
		"No hay que enviar trabajos (archivos)",
		"Ne soumettez pas de travail (fichiers)",
		"Non presenta lavori (files)",
		"N&atilde;o submeter trabalhos (arquivos)",
		},
		{
		"Es ist zwingend notwendig Hausarbeiten (files)",
		"It is mandatory to submit works (files)",
		"Es obligatorio enviar trabajos (archivos)",
		"Il est obligatoire de soumetter le travail (fichiers)",
		"E 'obbligatorio presentare il lavoro (files)",
		"&Eacute; obrigat&oacute;ria a apresenta&ccedil;&atilde;o de trabalhos (arquivos)",
		}
	};
const char *Txt_all_files[Txt_NUM_LANGUAGES] =
	{
	"alle Dateien",
	"all files",
	"todos los archivos",
	"tous les fichiers",
	"tutti i file",
	"todos os arquivos",
	};
const char *Txt_All_groups[Txt_NUM_LANGUAGES] =
	{
	"Alle Gruppen",
	"All groups",
	"Todos los grupos",
	"Tous les groupes",
	"Tutti i gruppi",
	"Todos os grupos",
	};
const char *Txt_All_places[Txt_NUM_LANGUAGES] =
	{
	"Alle Standorte",
	"All places",
	"Todos los lugares",
	"Tous les emplacements",
	"Tutti i localit&agrave;",
	"Todas as localiza&ccedil;oes",
	};
const char *Txt_All_tags[Txt_NUM_LANGUAGES] =	// Tags used in self-assessment test
	{
	"Alle Tags",
	"All tags",
	"Todos los descriptores",
	"Tous les descripteurs",
	"Tutti i descrittori",
	"Todos os descritores",
	};
const char *Txt_All_types_of_answers[Txt_NUM_LANGUAGES] =	// Answers of a self-assessment test
	{
	"Alle Arten von Antworten",
	"All types of answers",
	"Todos los tipos de respuestas",
	"Tous les types de r&eacute;ponses",
	"Tutti i tipi di risposte",
	"Todos os tipos de respostas",
	};
const char *Txt_Already_existed_an_assignment_with_the_folder_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Gab es bereits einen Aktivit&auml;t mit die Verzeichnis <strong>%s</strong>.",
	"Already existed an assignment with the folder <strong>%s</strong>.",
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.",
	"Il existe d&eacute;j&agrave; une activit&eacute; avec le r&eacute;pertoire <strong>%s</strong>.",
	"Esiste gi&agrave; una attivit&agrave; con la cartella <strong>%s</strong>.",
	"J&aacute; existe uma atividade com o diret&oacute;rio <strong>%s</strong>.",
	};
const char *Txt_Already_existed_an_assignment_with_the_title_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Gab es bereits einen Aktivit&auml;t mit dem Titel <strong>%s</strong>.",
	"Already existed an assignment with the title <strong>%s</strong>.",
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.",
	"Il existe d&eacute;j&agrave; une activit&eacute; du titre <strong>%s</strong>.",
	"Esiste gi&agrave; una attivit&agrave; con il titolo <strong>%s</strong>.",
	"J&aacute; existe uma atividade com o t&iacute;tulo <strong>%s</strong>.",
	};
const char *Txt_and[Txt_NUM_LANGUAGES] =
	{
	"und",
	"and",
	"y",
	"et",
	"e",
	"e",
	};
const char *Txt_and_X_other_recipients[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u in the following sentences
	{
	"...und %u andere Empf&auml;nger",
	"...and %u other recipients",
	"...y otros %u destinatarios",
	"...et %u autres destinataires",
	"...e altri %u destinatari",
	"...e %u outros destinat&aacute;rios",
        };
const char *Txt_Announcement_of_exam_removed[Txt_NUM_LANGUAGES] =
	{
        "Anruf f&uuml;r Pr&uuml;fung entfernt.",
	"Announcement of exam removed.",
	"Convocatoria de examen eliminada.",
	"Convocation pour l'examen supprim&eacute;.",
	"Chiamata per esame rimossa.",
	"Chamada para exame removida.",
	};
const char *Txt_Another_centre[Txt_NUM_LANGUAGES] =
	{
	"Ein weiteres Zentrum",
	"Another centre",
	"Otro centro",
	"Un autre centre",
	"Un altro centro",
	"Outro centro",
	};
const char *Txt_Another_department[Txt_NUM_LANGUAGES] =
	{
	"Ein weiteres Fachbereich",
	"Another department",
	"Otro departamento",
	"Une autre d&eacute;partement",
	"Un altro dipartimento",
	"Outro departamento",
	};
const char *Txt_Another_institution[Txt_NUM_LANGUAGES] =
	{
	"Ein anderes Institution",
	"Another institution",
	"Otra instituci&oacute;n",
	"Un autre &eacute;tablissement",
	"Un'altra istituzione",
	"Outra institui&ccedil;&atilde;o",
	};
const char *Txt_Another_place[Txt_NUM_LANGUAGES] =
	{
	"Ein weiteres Standort",
	"Another place",
	"Otro lugar",
	"Un autre emplacement",
	"Un'altra localit&grave;",
	"Outra localiza&ccedil;&atilde;o",
	};
const char *Txt_Another_upload_problem[Txt_NUM_LANGUAGES] =
	{
	"Ein weiteres Upload Problem",
	"Another upload problem",
	"Otro problema de env&iacute;o",
	"Un autre probl&egrave;me de t&eacute;l&eacute;chargement",
	"Un altro problema di caricamento",
	"Outro problema de upload",
	};
const char *Txt_Answers[Txt_NUM_LANGUAGES] =	// Answers of self-assessment test
	{
	"Antworten",
	"Answers",
	"Respuestas",
	"R&eacute;ponses",
	"Risposte",
	"Respostas",
	};
const char *Txt_any_course[Txt_NUM_LANGUAGES] =
       {
	"jedem Vorlesung",
	"any course",
	"cualquier asignatura",
	"toute mati&egrave;re",
	"ogni materia",
	"qualquer disciplina",
	};
const char *Txt_any_course_degree[Txt_NUM_LANGUAGES] =
       {
	"jedem Vorlesung/Grad",
	"any course/degree",
	"cualquier asignatura/titulaci&oacute;n",
	"toute mati&egrave;re/&eacute;tude",
	"ogni materia/laurea",
	"qualquer disciplina/titula&ccedil;&atilde;o",
	};
const char *Txt_any_degree[Txt_NUM_LANGUAGES] =
       {
	"jedem Grad",
	"any degree",
	"cualquier titulaci&oacute;n",
	"tout &eacute;tude",
	"ogni laurea",
	"qualquer titula&ccedil;&atilde;o",
	};
const char *Txt_any_user[Txt_NUM_LANGUAGES] =
       {
	"jedem Benutzer",
	"any user",
	"cualquier usuario",
	"n'importe qu'utilisateur",
	"qualsiasi utente",
	"qualquer utilizador",
	};
const char *Txt_Assessment[Txt_NUM_LANGUAGES] =
	{
	"Bewertungs",
	"Assessment",
	"Evaluaci&oacute;n",
	"&Eacute;valuation",
	"Valutazione",
	"Avalia&ccedil;&atilde;o"
	};
const char *Txt_Assessment_system[Txt_NUM_LANGUAGES] =
	{
	"Bewertungs Kriterien",
	"Assessment system",
	"Sistema de evaluaci&oacute;n",
	"Syst&egrave;me d'&eacute;valuation",
	"Sistema di valutazione",
	"Sistema de avalia&ccedil;&atilde;o",
	};
/*
const char *Txt_Author[Txt_NUM_LANGUAGES] =
	{
	"Autor",
	"Author",
	"Autor",
	"Auteur",
	"Autore",
	"Autor",
	};
*/
const char *Txt_Average_number_of_courses_to_which_a_user_belongs[Txt_NUM_LANGUAGES] =
       {
        "Durchschnittliche Anzahl von Vorlesung, mit dem der Benutzer geh&ouml;rt (nur Benutzer mit Vorlesung)",
	"Average number of courses to which a user belongs (only users with courses) ",
	"N&ordm; medio de asignaturas a las que pertenece cada usuario (s&oacute;lo usuarios con asignaturas)",
        "Nombre moyen de cours &agrave; qui appartient un utilisateur (seulement utilisateurs avec mati&egrave;res)",
	"Numero medio di materie a cui appartiene un utente (solo utenti con materie)",
	"N&ordm; medio de disciplinas em que cada utilizador pertence (apenas utilizadores com disciplinas)",
	};
const char *Txt_Average_number_of_users_belonging_to_a_course[Txt_NUM_LANGUAGES] =
       {
        "Durchschnittliche Anzahl der Benutzer, die einen Vorlesung (nur Vorlesung mit Benutzernn)",
	"Average number of users belonging to a course (only courses with users)",
	"N&ordm; medio de usuarios que pertenecen a cada asignatura (s&oacute;lo asignaturas con usuarios)",
        "Nombre moyen d'utilisateurs appartenant &agrave; un cours (seulement utilisateurs avec mati&egrave;res)",
	"Numero medio di utenti appartenenti a un corso (solo utenti con materie)",
	"N&ordm; medio de utilizadores que pertencem a uma disciplina (apenas utilizadores com disciplinas)",
	};
const char *Txt_Average_type[Txt_NUM_LANGUAGES] =
	{
	"Art der Durchschnitt",
	"Average type",
	"Tipo de promedio",
	"Type de moyenne",
	"Tipo di media",
	"Tipo de m&eacute;dia",
	};
const char *Txt_Can_not_create_the_file_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Can not create the file <strong>%s</ strong> because it would exceed the disk quota.",	// Need bersetzung
	"Can not create the file <strong>%s</ strong> because it would exceed the disk quota.",
	"No se ha creado el archivo <strong>%s</strong> porque se exceder&iacute;a la cuota de disco."
	"Can not create the file <strong>%s</ strong> because it would exceed the disk quota.",	// Besoin de traduction
	"Can not create the file <strong>%s</ strong> because it would exceed the disk quota.",	// Bisogno di traduzione
	"Can not create the file <strong>%s</ strong> because it would exceed the disk quota.",	// Necessita de traduo
	};
const char *Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Can not create the folder <strong>%s</ strong> because it would exceed the disk quota.",	// Need bersetzung
	"Can not create the folder <strong>%s</ strong> because it would exceed the disk quota.",
	"No se ha creado la carpeta <strong>%s</strong> porque se exceder&iacute;a la cuota de disco."
	"Can not create the folder <strong>%s</ strong> because it would exceed the disk quota.",	// Besoin de traduction
	"Can not create the folder <strong>%s</ strong> because it would exceed the disk quota.",	// Bisogno di traduzione
	"Can not create the folder <strong>%s</ strong> because it would exceed the disk quota.",	// Necessita de traduo
	};
const char *Txt_Can_not_create_the_file_X_because_there_is_already_a_folder_or_a_file_with_that_name[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Can not create the file <strong>%s</ strong> because there is already a folder or a file with that name.",	// Need bersetzung
	"Can not create the file <strong>%s</ strong> because there is already a folder or a file with that name.",
	"No se ha creado el archivo <strong>%s</strong> porque ya existe una carpeta o un archivo con ese nombre."
	"Can not create the file <strong>%s</ strong> because there is already a folder or a file with that name.",	// Besoin de traduction
	"Can not create the file <strong>%s</ strong> because there is already a folder or a file with that name.",	// Bisogno di traduzione
	"Can not create the file <strong>%s</ strong> because there is already a folder or a file with that name.",	// Necessita de traduo
	};
const char *Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Can not create the folder <strong>%s</ strong> because there is already a folder or a file with that name.",	// Need bersetzung
	"Can not create the folder <strong>%s</ strong> because there is already a folder or a file with that name.",
	"No se ha creado la carpeta <strong>%s</strong> porque ya existe una carpeta o un archivo con ese nombre."
	"Can not create the folder <strong>%s</ strong> because there is already a folder or a file with that name.",	// Besoin de traduction
	"Can not create the folder <strong>%s</ strong> because there is already a folder or a file with that name.",	// Bisogno di traduzione
	"Can not create the folder <strong>%s</ strong> because there is already a folder or a file with that name.",	// Necessita de traduo
	};
const char *Txt_Can_not_rename_a_folder_of_assignment[Txt_NUM_LANGUAGES] =
	{
	"Can't Umbenennen ein Verzeichnis der Aktivit&auml;t.",
	"Can not rename a folder of assignment.",
	"No se puede cambiar el nombre de una carpeta de actividad.",
	"Impossible de renommer un r&eacute;pertoire d'activit&eacute;.",
	"Non &egrave; possibile rinominare una cartella di attivit&agrave;.",
	"N&atilde;o &eacute; possvel renomear um diret&oacute;rio de actividade.",
	};
const char *Txt_Centre[Txt_NUM_LANGUAGES] =
	{
	"Zentrum",
	"Centre",
	"Centro",
	"Centre",
	"Centro",
	"Centro",
	};
const char *Txt_Centre_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Centre <strong>%s</strong> removed.",	// Need bersetzung
	"Centre <strong>%s</strong> removed.",
	"Centro <strong>%s</strong> eliminado."
	"Centre <strong>%s</strong> removed.",	// Besoin de traduction
	"Centre <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Centre <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Centre_unspecified[Txt_NUM_LANGUAGES] =
	{
	"Zentrum nicht spezifiziert",
	"Centre unspecified",
	"Centro sin especificar",
	"Centre non sp&eacute;cifi&eacute;",
	"Centro non specificato",
	"Centro indeterminado",
	};
const char *Txt_Centres[Txt_NUM_LANGUAGES] =
	{
	"Zentren",
	"Centres",
	"Centros",
	"Centres",
	"Centri",
	"Centros"
	};
const char *Txt_centres[Txt_NUM_LANGUAGES] =
	{
	"Zentren",
	"centres",
	"centros",
	"centres",
	"centri",
	"centros",
	};
const char *Txt_Change_my_groups[Txt_NUM_LANGUAGES] =
	{
        "Gruppen &auml;ndern",
	"Change my groups",
	"Cambiar de grupos",
	"Modifier mes groupes",
	"Cambia miei gruppi",
	"Alterar meus grupos",
	};
const char *Txt_Change_my_record[Txt_NUM_LANGUAGES] =
	{
        "Profil speichern",
	"Change my record",
	"Cambiar mi ficha",
	"Modifier ma carte",
	"Cambia la mia scheda",
	"Alterar o meu cart&atilde;o",
	};
const char *Txt_Change_nickname[Txt_NUM_LANGUAGES] =
	{
	"Benutzername &auml;ndern",
	"Change nickname",
	"Cambiar apodo",
	"Changer le surnom",
	"Cambiare soprannome",
	"Alterar alcunha"
	};
const char *Txt_Change_password[Txt_NUM_LANGUAGES] =
	{
	"Passwort &auml;ndern",
	"Change password",
	"Cambiar contrase&ntilde;a",
	"Changer le mot de passe",
	"Cambiare password",
	"Alterar senha",
	};
const char *Txt_Change_preferences[Txt_NUM_LANGUAGES] =
	{
	"Einstellungen &auml;ndern",
	"Change preferences",
	"Cambiar preferencias",
	"Changer mes pr&eacute;f&eacute;rences",
	"Cambiare preferenze",
	"Alterar prefer&ecirc;ncias",
	};
const char *Txt_CHAT_Room[Txt_NUM_LANGUAGES] =
	{
	"Raum",
	"Room",
	"Sala",
	"Salle",
	"Room",
	"Room",
        };
const char *Txt_Chat_rooms[Txt_NUM_LANGUAGES] =
	{
	"Chat-R&auml;ume",
	"Chat rooms",
	"Salas de chat",
	"Salles de chat",
	"Chat rooms",
	"Chat rooms",
        };
const char *Txt_Check_marks_in_file_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Check marks in file %s",	// Need bersetzung
	"Check marks in file %s",
	"Consultar calificaciones en el archivo %s",
	"Check marks in file %s",	// Besoin de traduction
	"Check marks in file %s",	// Bisogno di traduzione
	"Check marks in file %s",	// Necessita de traduo
	};
const char *Txt_Code[Txt_NUM_LANGUAGES] =
	{
	"Code",
	"Code",
	"C&oacute;d.",
	"Code",
	"Cod.",
	"C&oacute;d.",
	};
const char *Txt_Columns[Txt_NUM_LANGUAGES] =
	{
	"Spalten",
	"Columns",
	"Columnas",
	"Colonnes",
	"Colonne",
	"Colunas",
	};
const char *Txt_columns[Txt_NUM_LANGUAGES] =
	{
	"Spalten",
	"columns",
	"columnas",
	"colonnes",
	"colonne",
	"colunas",
	};
const char *Txt_Connected_PLURAL[Txt_NUM_LANGUAGES] =
	{
	"Connected",
	"Connected",
	"Conectados",
	"Connect&eacute;s",
	"Collegati",
	"Conectados",
	};
const char *Txt_connected_PLURAL[Txt_NUM_LANGUAGES] =
	{
	"Connected",
	"connected",
	"conectados",
	"connect&eacute;s",
	"collegati",
	"conectados",
	};
const char *Txt_connected_SINGULAR[Txt_NUM_LANGUAGES] =
	{
	"Connected",
	"connected",
	"conectado",
	"connect&eacute;",
	"collegato",
	"conectado",
	};
const char *Txt_Continue[Txt_NUM_LANGUAGES] =
	{
	"Weiter",
	"Continue",
	"Continuar",
	"Continuer",
	"Continua",
	"Continuar",
	};
const char *Txt_Control_of_class_attendance[Txt_NUM_LANGUAGES] =
	{
	"Kontrolle der Teilnahme",
	"Control of class attendance",
	"Control de asistencia a clase",
	"Contr&ouml;le de la fr&eacute;quentation scolaire",
	"Controllo delle presenze per classe",
	"Controlo de assiduidade de classe",
	};
const char *Txt_Copy[Txt_NUM_LANGUAGES] =	// Verb
	{
	"Kopieren",
	"Copy",
	"Copiar",
	"Copier",
	"Copiare",
	"Copiar",
	};
const char *Txt_Copy_source[Txt_NUM_LANGUAGES] =
	{
	"Kopieren Quelle",
	"Copy source",
	"Origen de copia",
	"Source de copie",
	"Origine della copia",
	"Origem da copia",
	};
const char *Txt_Course[Txt_NUM_LANGUAGES] =
	{
	"Vorlesung",
	"Course",
	"Asignatura",
	"Mati&egrave;re",
	"Materia",
	"Disciplina"
	};
const char *Txt_Course_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Vorlesung",
	"Course",
	"Asignatura",
	"Matire",
	"Materia",
	"Disciplina"
	};
const char *Txt_course[Txt_NUM_LANGUAGES] =
	{
	"Vorlesung",
	"course",
	"asignatura",
	"mati&egrave;re",
	"materia",
	"disciplina",
	};
const char *Txt_crs[Txt_NUM_LANGUAGES] =	// Abbreviation of course (three characters + dot)
	{
	"Vor.",
	"crs.",
	"asg.",
	"mat.",
	"mat.",
	"crs.",
	};
const char *Txt_STAT_COLOR_TYPES[Sta_NUM_COLOR_TYPES][Txt_NUM_LANGUAGES] =
	{
		{
		"Farbe ",
		"color",
		"color",
		"couleur",
		"colore",
		"cor",
		},
		{
		"schwarz auf wei&szlig;",
		"black to white",
		"de negro a blanco",
		"du noir au blanc",
		"dal nero al bianco",
		"do preto ao branco",
		},
		{
		"wei&szlig; auf schwarz",
		"white to black",
		"de blanco a negro",
		"du blanc au noir",
		"dal bianco al nero",
		"do branco ao preto",
		},
	};
const char *Txt_Color_of_the_graphic[Txt_NUM_LANGUAGES] =
	{
	"Farbe der Grafik",
	"Color of the graphic",
	"Color de la gr&aacute;fica",
	"Couleur de l'image",
	"Colore del grafico",
	"Cor do gr&aacute;fico",
	};
const char *Txt_Common_zone[Txt_NUM_LANGUAGES] =
        {
	"Gemeinsame Zone",
	"Common zone",
	"Zona com&uacute;n",
	"Zone commune",
	"Zona comune",
	"Zona comum",
        };
const char *Txt_common_zone[Txt_NUM_LANGUAGES] =
        {
	"Gemeinsame Zone",
	"common zone",
	"zona com&uacute;n",
	"zone commune",
	"zona comune",
	"zona comum",
        };
/*
const char *Txt_Content[Txt_NUM_LANGUAGES] =
	{
	"Inhalt",
	"Content",
	"Contenido",
	"Contenu",
	"Contenuto",
	"Conte&uacute;do",
	};
*/
const char *Txt_Contract_FOLDER[Txt_NUM_LANGUAGES] =	// Hide the content (the files inside) of the folder
	{
	"Zusammenziehen",
	"Contract",
	"Contraer",
	"Contracter",
	"Contrarre",
	"Contrair",
	};
const char *Txt_Course_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Course <strong>%s</strong> removed.",	// Need bersetzung
	"Course <strong>%s</strong> removed.",
	"Asignatura <strong>%s</strong> eliminada."
	"Course <strong>%s</strong> removed.",	// Besoin de traduction
	"Course <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Course <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_COURSE_SELECTED_None[Txt_NUM_LANGUAGES] =
	{
	"Keine",
	"None",
	"Ninguna",
	"Aucune",
	"Nessuna",
	"Nenhum",
	};
const char *Txt_Courses[Txt_NUM_LANGUAGES] =
	{
	"Vorlesungen",
	"Courses",
	"Asignaturas",
	"Mati&egrave;res",
	"Materie",
	"Disciplinas",
	};
const char *Txt_Courses_with_students[Txt_NUM_LANGUAGES] =
	{
	"Vorlesungen mit Studenten",
	"Courses with students",
	"Asignaturas con alumnos",
	"Mati&egrave;res avec &eacute;tudiants",
	"Materie con studenti",
	"Disciplinas com estudantes",
	};
const char *Txt_Courses_with_teachers[Txt_NUM_LANGUAGES] =
	{
	"Vorlesungen mit Dozenten",
	"Courses with teachers",
	"Asignaturas con profesores",
	"Mati&egrave;res avec enseignants",
	"Materie con insegnanti",
	"Disciplinas com professores",
	};
const char *Txt_courses[Txt_NUM_LANGUAGES] =
	{
	"Vorlesungen",
	"courses",
	"asignaturas",
	"mati&egrave;res",
	"materie",
	"disciplinas",
	};
const char *Txt_Create_assignment[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Aktivit&auml;t",
	"Create assignment",
	"Crear actividad",
	"Cr&eacute;er activit&eacute;",
	"Crea attivit&agrave;",
	"Criar atividade",
        };
const char *Txt_Create_centre[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Zentrum",
	"Create centre",
	"Crear centro",
	"Cr&eacute;er centre",
	"Crea centro",
	"Criar centro",
        };
const char *Txt_Create_course[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Vorlesung",
	"Create course",
	"Crear asignatura",
	"Cr&eacute;er mati&egrave;re",
	"Crea materia",
	"Criar disciplina",
        };
const char *Txt_Create_degree[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Grad",
	"Create degree",
	"Crear titulaci&oacute;n",
	"Cr&eacute;er &eacute;tude",
	"Crea laurea",
	"Criar titula&ccedil;&atilde;o",
        };
const char *Txt_Create_department[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Fachbereich",
	"Create department",
	"Crear departamento",
	"Cr&eacute;er d&eacute;partement",
	"Crea dipartimento",
	"Criar departamento",
        };
const char *Txt_Create_e_mail_message[Txt_NUM_LANGUAGES] =
	{
	"Erstellen E-Mail-Nachricht",
	"Create e-mail message",
	"Crear mensaje de correo electr&oacute;nico",
	"Cr&eacute;er un e-mail message",
	"Crea messaggio e-mail",
	"Criar uma mensagem de e-mail",
        };
const char *Txt_Create_folder[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Verzeichnis",
	"Create folder",
	"Crear carpeta",
	"Cr&eacute;er r&eacute;pertoire",
	"Crea cartella",
	"Criar diret&oacute;rio",
        };
const char *Txt_Create_group[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Gruppe",
	"Create group",
	"Crear grupo",
	"Cr&eacute;er groupe",
	"Crea gruppo",
	"Criar grupo",
        };
const char *Txt_Create_holiday[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Feiertag",
	"Create holiday",
	"Crear festividad",
	"Cr&eacute;er f&ecirc;te",
	"Crea vacanza",
	"Criar f&eacute;ria",
        };
const char *Txt_Create_institution[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Institution",
	"Create institution",
	"Crear instituci&oacute;n",
	"Cr&eacute;er &eacute;tablissement",
	"Crea istituzione",
	"Criar institu&ccedil;&atilde;o",
        };
const char *Txt_Create_link[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Link",
	"Create link",
	"Crear enlace",
	"Cr&eacute;er lien",
	"Crea link",
	"Criar liga&ccedil;&atilde;o",
        };
const char *Txt_Create_place[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Standort",
	"Create place",
	"Crear lugar",
	"Cr&eacute;er emplacement",
	"Crea localit&grave;",
	"Criar localiza&ccedil;&atilde;o",
        };
const char *Txt_Create_plugin[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Plugin",
	"Create plugin",
	"Crear complemento",
	"Cr&eacute;er plugin",
	"Crea plugin",
	"Criar plugin",
        };
const char *Txt_Create_record_field[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Datenfeld",
	"Create record field",
	"Crear campo de ficha",
	"Cr&eacute;er champ de carte",
	"Crea campo di schede",
	"Criar campo de cart&atilde;o",
        };
const char *Txt_Create_type_of_degree[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Abschlussart",
	"Create type of degree",
	"Crear tipo de titulaci&oacute;n",
	"Cr&eacute;er type d'&eacute;tude",
	"Crea tipo di laurea",
	"Criar tipo de titula&ccedil;&atilde;o",
        };
const char *Txt_Create_type_of_group[Txt_NUM_LANGUAGES] =
	{
	"Erstellen Art der Gruppe",
	"Create type of group",
	"Crear tipo de grupo",
	"Cr&eacute;er type de groupe",
	"Crea tipo di gruppo",
	"Criar tipo de grupo",
        };
const char *Txt_Create_ZIP_file[Txt_NUM_LANGUAGES] =
	{
	"Erstellen ZIP-Datei",
	"Create ZIP file",
	"Crear archivo ZIP",
	"Cr&eacute;er fichier ZIP",
	"Crea file ZIP",
	"Criar arquivo ZIP",
	};
const char *Txt_Created_new_announcement_of_exam[Txt_NUM_LANGUAGES] =
	{
	"Erstellt neue Anrufe f&uuml;r Pr&uuml;fung.",
	"Created new announcement of exam.",
	"Creado nueva convocatoria de examen.",
	"Cr&eacute;&eacute; nouvelle convocation &agrave; un examen.",
	"Creato nuova chiamata per esame.",
	"Criado nova chamada para exame.",
	};
const char *Txt_Created_new_assignment_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Aktivit&auml;t <strong>%s</strong>.",
	"Created new assignment <strong>%s</strong>.",
	"Creada nueva actividad <strong>%s</strong>.",
	"Cr&eacute;&eacute;e nouvelle activit&eacute; <strong>%s</strong>.",
	"Creata nuova attivit&agrave; <strong>%s</strong>.",
	"Criada nova atividade <strong>%s</strong>.",
	};
const char *Txt_Created_new_centre_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Zentrum <strong>%s</strong>.",
	"Created new centre <strong>%s</strong>.",
	"Creado nuevo centro <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau centre <strong>%s</strong>.",
	"Creato nuovo centro <strong>%s</strong>.",
	"Criado novo centro <strong>%s</strong>.",
	};
const char *Txt_Created_new_course_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Vorlesung <strong>%s</strong>.",
	"Created new course <strong>%s</strong>.",
	"Creada nueva asignatura <strong>%s</strong>.",
	"Cr&eacute;&eacute;e nouvelle mati&egrave;re <strong>%s</strong>.",
	"Creata nuova materia <strong>%s</strong>.",
	"Criada nova disciplina <strong>%s</strong>.",
	};
const char *Txt_Created_new_degree_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Grad <strong>%s</strong>.",
	"Created new degree <strong>%s</strong>.",
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.",
	"Cr&eacute;&eacute;e nouvel &eacute;tude <strong>%s</strong>.",
	"Creata nuova laurea <strong>%s</strong>.",
	"Criada nova titula&ccedil;&atilde;o <strong>%s</strong>.",
	};
const char *Txt_Created_new_department_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Fachbereich <strong>%s</strong>.",
	"Created new department <strong>%s</strong>.",
	"Creado nuevo departamento <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau d&eacute;partement <strong>%s</strong>.",
	"Creato nuovo dipartimento <strong>%s</strong>.",
	"Criado novo departamento <strong>%s</strong>.",
	};
const char *Txt_Created_new_group_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Gruppe <strong>%s</strong>.",
	"Created new group <strong>%s</strong>.",
	"Creado nuevo grupo <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau groupe <strong>%s</strong>.",
	"Creato nuovo gruppo <strong>%s</strong>.",
	"Criado novo grupo <strong>%s</strong>.",
	};
const char *Txt_Created_new_holiday_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Feiertag <strong>%s</strong>.",
	"Created new holiday <strong>%s</strong>.",
	"Creado nueva festividad <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouvelle f&ecirc;te <strong>%s</strong>.",
	"Creato nuova vacanza <strong>%s</strong>.",
	"Criado nova f&eacute;ria <strong>%s</strong>.",
	};
const char *Txt_Created_new_institution_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Institution <strong>%s</strong>.",
	"Created new institution <strong>%s</strong>.",
	"Creada nueva instituci&oacute;n <strong>%s</strong>.",
	"Cr&eacute;&eacute;e nouvel &eacute;tablissement <strong>%s</strong>.",
	"Creata nuova istituzione <strong>%s</strong>.",
	"Criada nova institu&ccedil;&atilde;o <strong>%s</strong>.",
	};
const char *Txt_Created_new_link_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Link <strong>%s</strong>.",
	"Created new link <strong>%s</strong>.",
	"Creado nuevo enlace <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau lien <strong>%s</strong>.",
	"Creato nuovo link <strong>%s</strong>.",
	"Criado nova liga&ccedil;&atilde;o <strong>%s</strong>.",
	};
const char *Txt_Created_new_place_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Standort <strong>%s</strong>.",
	"Created new place <strong>%s</strong>.",
	"Creado nuevo lugar <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouvel emplacement <strong>%s</strong>.",
	"Creato nuova localit&grave; <strong>%s</strong>.",
	"Criado nova localiza&ccedil;&atilde;o <strong>%s</strong>.",
	};
const char *Txt_Created_new_plugin_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Plugin <strong>%s</strong>.",
	"Created new plugin <strong>%s</strong>.",
	"Creado nuevo complemento <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau plugin <strong>%s</strong>.",
	"Creato nuovo plugin <strong>%s</strong>.",
	"Criado novo plugin <strong>%s</strong>.",
	};
const char *Txt_Created_new_record_field_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Datenfeld <strong>%s</strong>.",
	"Created new record field <strong>%s</strong>.",
	"Creado nuevo campo de ficha <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau champ de carte <strong>%s</strong>.",
	"Creato nuovo campo di schede <strong>%s</strong>.",
	"Criado novo plugin <strong>%s</strong>.",
	};
const char *Txt_Created_new_type_of_degree_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Abschlussart <strong>%s</strong>.",
	"Created new type of degree <strong>%s</strong>.",
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau type d'&eacute;tude <strong>%s</strong>.",
	"Creato nuovo tipo di laurea <strong>%s</strong>.",
	"Criado novo tipo de titula&ccedil;&atilde;o <strong>%s</strong>.",
	};
const char *Txt_Created_new_type_of_group_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Erstellt neue Art der Gruppe <strong>%s</strong>.",
	"Created new type of group <strong>%s</strong>.",
	"Creado nuevo tipo de grupo <strong>%s</strong>.",
	"Cr&eacute;&eacute; nouveau type de groupe <strong>%s</strong>.",
	"Creato nuovo tipo di gruppo <strong>%s</strong>.",
	"Criado novo tipo de grupo <strong>%s</strong>.",
	};
const char *Txt_Created_tables_in_the_database_that_did_not_exist[Txt_NUM_LANGUAGES] =
	{
	"Erstellt Tabellen in der Datenbank, die es nicht gab.",
	"Created tables in the database that did not exist.",
	"Creadas las tablas de la base de datos que no exist&iacute;an.",
	"Tables cr&eacute;&eacute;es dans la base de donn&eacute;es qui n'existaient pas.",
	"Tabelle create nel database che non esisteva.",
	"Criadas tabelas no banco de dados que n&atilde;o existiam.",
	};
const char *Txt_Current_nickname[Txt_NUM_LANGUAGES] =
	{
	"Aktuelles Benutzername",
	"Current nickname",
	"Apodo actual",
	"Surnom actuel",
	"Soprannome corrente",
	"Alcunha atual",
	};
const char *Txt_Current_password[Txt_NUM_LANGUAGES] =
	{
	"Aktuelles Passwort",
	"Current password",
	"Contrase&ntilde;a actual",
	"Mot de passe actuel",
	"Password corrente",
	"Senha atual",
	};
const char *Txt_Date[Txt_NUM_LANGUAGES] =
	{
	"Datum",
	"Date",
	"Fecha",
	"Date",
	"Data",
	"Data",
	};
const char *Txt_Date_of_birth[Txt_NUM_LANGUAGES] =
	{
	"Geburtsdatum",
	"Date of birth",
	"Fecha nacimiento",
	"Date naissance",
	"Data nascita",
	"Data nascimento",
	};
const char *Txt_Dates[Txt_NUM_LANGUAGES] =
	{
	"Datum",
	"Dates",
	"Fechas",
	"Dates",
	"Date",
	"Datas",
	};
const char *Txt_DATES_RANGE[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
	"von %s bis %s",
	"from %s to %s",
	"desde el %s hasta el %s",
	"de %s jusqu'&agrave; %s",
	"dal %s fino al %s",
	"%s at&eacute; %s",
	};
const char *Txt_Day[Txt_NUM_LANGUAGES] =
	{
	"Tag",
	"Day",
	"D&iacute;a",
	"Jour",
	"Giorno",
	"Dia",
	};
const char *Txt_days[Txt_NUM_LANGUAGES] =
	{
	"Tage",
	"days",
	"d&iacute;as",
	"jours",
	"giorni",
	"dias",
	};
const char *Txt_Dear_NO_HTML[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
  {
      /* Usr_GENDER_UNKNOWN */	{"Sehr geehrte/r"	,"Dear"	,"Estimado/a"	,"Cher/Chre"	,"Caro/i"	,"Prezado/a"	},
      /* Usr_GENDER_FEMALE */	{"Sehr geehrte"		,"Dear"	,"Estimada"	,"Chre"	,"Cari"		,"Prezada"	},
      /* Usr_GENDER_MALE */		{"Sehr geehrter"	,"Dear"	,"Estimado"	,"Cher"		,"Caro"		,"Prezado"	},
      /* Usr_GENDER_ALL */		{"Sehr geehrte/r"	,"Dear"	,"Estimado/a"	,"Cher/Ch;re"	,"Caro/i"	,"Prezado/a"	},
  };
const char *Txt_default[Txt_NUM_LANGUAGES] =
	{
	"Vorgabe",
	"default",
	"predeterminado",
	"d&eacute;faut",
	"predefinito",
	"padr&atilde;o",
	};
const char *Txt_Degree[Txt_NUM_LANGUAGES] =
	{
	"Grad",
	"Degree",
	"Titulaci&oacute;n",
	"&Eacute;tude",
	"Laurea",
	"Titula&ccedil;&atilde;o",
	};
const char *Txt_degree[Txt_NUM_LANGUAGES] =
	{
	"Grad",
	"degree",
	"titulaci&oacute;n",
	"&eacute;tude",
	"laurea",
	"titula&ccedil;&atilde;o",
	};
const char *Txt_Degree_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Degree <strong>%s</strong> removed.",	// Need bersetzung
	"Degree <strong>%s</strong> removed.",
	"Titulaci&oacute;n <strong>%s</strong> eliminada."
	"Degree <strong>%s</strong> removed.",	// Besoin de traduction
	"Degree <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Degree <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Degrees[Txt_NUM_LANGUAGES] =
	{
	"Grad",
	"Degrees",
	"Titulaciones",
	"&Eacute;tudes",
	"Lauree",
	"Titula&ccedil;oes",
	};
const char *Txt_degrees[Txt_NUM_LANGUAGES] =
	{
	"Grad",
	"degrees",
	"titulaciones",
	"&eacute;tudes",
	"lauree",
	"titula&ccedil;oes",
	};
const char *Txt_Degrees_of_type[Txt_NUM_LANGUAGES] =
	{
	"Grad der Typ",
	"Degrees of type",
	"Titulaciones de tipo",
	"&Eacute;tudes de type",
	"Lauree di tipo",
	"Titula&ccedil;oes de tipo",
	};
const char *Txt_Department[Txt_NUM_LANGUAGES] =
	{
	"Fachbereich",
	"Department",
	"Departamento",
	"D&eacute;partement",
	"Dipartimento",
	"Departamento",
	};
const char *Txt_Department_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Department <strong>%s</strong> removed.",	// Need bersetzung
	"Department <strong>%s</strong> removed.",
	"Departamento <strong>%s</strong> eliminado."
	"Department <strong>%s</strong> removed.",	// Besoin de traduction
	"Department <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Department <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Department_unspecified[Txt_NUM_LANGUAGES] =
	{
	"Fachbereich nicht spezifiziert",
	"Department unspecified",
	"Departamento sin especificar",
	"D&eacute;partement non sp&eacute;cifi&eacute;",
	"Dipartimento non specificato",
	"Departamento indeterminado",
	};
const char *Txt_Departments[Txt_NUM_LANGUAGES] =
	{
	"Fachbereiche",
	"Departments",
	"Departamentos",
	"D&eacute;partements",
	"Dipartimenti",
	"Departamentos",
	};
const char *Txt_Description[Txt_NUM_LANGUAGES] =
	{
	"Beschreibung",
	"Description",
	"Descripci&oacute;n",
	"Description",
	"Descrizione",
	"Descri&ccedil;&atilde;o",
	};
const char *Txt_Design[Txt_NUM_LANGUAGES] =
	{
	"Design",
	"Design",
	"Dise&ntilde;o",
	"Conception",
	"Design",
	"Design",
	};
const char *Txt_Display_photos[Txt_NUM_LANGUAGES] =
	{
	"Bilder anzeigen",
	"Display photos",
	"Incluir fotos",
	"Inclure des photos",
	"Includere foto",
	"Incluir fotos",
	};
const char *Txt_distributed_by[Txt_NUM_LANGUAGES] =
	{
	"ausgesch&uuml;ttet von",
	"distributed by",
	"distribuido por",
	"distribu&eacute; par",
	"distribuito di",
	"distribu&iacute;do por",
	};
const char *Txt_Do_you_really_want_to_remove_the_assignment_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
        {
	"Wollen Sie wirklich Entfernen Sie die Aktivit&auml;t <strong>%s</strong>?",
	"Do you really want to remove the assignment <strong>%s</strong>?",
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?",
	"Voulez-vous vraiment supprimer l'activit&eacute; <strong>%s</strong>?",
	"Vuoi veramente rimuovere l'attivit&agrave; <strong>%s</strong>?",
	"Voc&ecirc; realmente deseja remover a atividade <strong>%s</strong>?",
        };
const char *Txt_Do_you_really_want_to_remove_the_file_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
        {
	"Wollen Sie wirklich Entfernen Sie die Datei <strong>%s</strong>?",
	"Do you really want to remove the file <strong>%s</strong>?",
	"&iquest;Realmente desea eliminar el archivo <strong>%s</strong>?",
	"Voulez-vous vraiment supprimer le fichier <strong>%s</strong>?",
	"Vuoi veramente rimuovere il file <strong>%s</strong>?",
	"Voc&ecirc; realmente deseja remover o arquivo <strong>%s</strong>?",
        };
const char *Txt_Do_you_really_want_to_remove_the_folder_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
        {
	"Wollen Sie wirklich Entfernen Sie die Verzeichnis <strong>%s</strong> und alle Inhalte?",
	"Do you really want to remove the folder <strong>%s</strong> and all its contents?",
	"&iquest;Realmente desea eliminar la carpeta <strong>%s</strong> y todo su contenido?",
	"Voulez-vous vraiment supprimer le r&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu?",
	"Vuoi veramente rimuovere la cartella <strong>%s</strong> e tutto il suo contenuto?",
	"Voc&ecirc; realmente deseja remover o diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do?",
        };
const char *Txt_Download_zone[Txt_NUM_LANGUAGES] =
        {
	"Download-Zone",
	"Download zone",
	"Zona de descarga",
	"Zone de t&eacute;l&eacute;chargements",
	"Zona di download",
	"Zona de downloads",
        };
const char *Txt_Download_management_zone[Txt_NUM_LANGUAGES] =
        {
	"Download Bewirtschaftungszone",
	"Download management zone",
	"Zona de administraci&oacute;n de descargas",
	"Zone de gestion de t&eacute;l&eacute;chargements",
	"Zona di gestione dei download",
	"Zona de gest&atilde;o de downloads",
        };
const char *Txt_download_management_zone[Txt_NUM_LANGUAGES] =
        {
	"Download Bewirtschaftungszone",
	"download management zone",
	"zona de administraci&oacute;n de descargas",
	"zone de gestion de t&eacute;l&eacute;chargements",
	"zona di gestione dei download",
	"zona de gest&atilde;o de downloads",
        };
const char *Txt_Edit[Txt_NUM_LANGUAGES] =
	{
	"Bearbeiten",
	"Edit",
	"Editar",
	"&Eacute;diter",
	"Editare",
	"Editar",
	};
const char *Txt_Edit_assignment[Txt_NUM_LANGUAGES] =
	{
	"Aktivit&auml;t bearbeiten",
	"Edit assignment",
	"Editar actividad",
	"&Eacute;diter activit&eacute;",
	"Editare attivit&agrave;",
	"Editar atividade",
	};
const char *Txt_Edit_text[Txt_NUM_LANGUAGES] =
	{
	"Text bearbeiten",
	"Edit text",
	"Editar texto",
	"&Eacute;diter texte",
	"Editare testo",
	"Editar texto",
	};
const char *Txt_eg_A_B[Txt_NUM_LANGUAGES] =
	{
	"z. B.: A, B,...",
	"e.g., A, B,...",
	"por ej.: A, B,...",
	"par exemple: A, B,...",
	"ad esempio: A, B,...",
	"ex.: A, B,...",
	};
const char *Txt_eg_Theory_Practice[Txt_NUM_LANGUAGES] =
	{
	"z. B.: Theorie, Praxis,...",
	"e.g., Theory, Practice,...",
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,...",
	"par exemple: Th&eacute;orie, Pratique,...",
	"ad esempio: Teoria, Pratica,...",
	"ex.: Teoria, Pr&aacute;tica,...",
	};
const char *Txt_Eliminate[Txt_NUM_LANGUAGES] =
	{
	"Fr&uuml;here",
	"Eliminate",
	"Eliminar",
	"&Eacute;liminer",
	"Eliminare",
	"Eliminar",
	};
const char *Txt_E_mail[Txt_NUM_LANGUAGES] =
	{
	"E-Mail",
	"E-mail",
	"Correo electr&oacute;nico",
	"Courrier &eacute;lectronique",
	"E-mail",
	"E-mail",	
	};
const char *Txt_E_mail_new_password[Txt_NUM_LANGUAGES] =
	{
	"Neues Passwort beantragen",
	"E-mail new password",
	"Enviarme una nueva contrase&ntilde;a",
	"Envoyez-moi un nouveau mot de passe",
	"Invia una nuova password",
	"Enviar uma nova senha",
	};
const char *Txt_End_date[Txt_NUM_LANGUAGES] =
	{
	"Enddatum",
	"End date",
	"Fecha final",
	"Date finale",
	"Data finale",
	"Data final",
        };
const char *Txt_Enter_a_new_item_here[Txt_NUM_LANGUAGES] =
	{
	"Geben Sie hier ein neues Element ein",
	"Enter a new item here",
	"Escriba aqu&iacute; un nuevo apartado",
	"Ecrivez ici un nouveau point",
	"Scrivi qui un nuovo punto",
	"Escreva aqui uma nova rubrica",
        };
const char *Txt_Event[Txt_NUM_LANGUAGES] =
	{
	"Event",
	"Event",
	"Evento",
	"Ev&eacute;nement",
	"Evento",
	"Evento",
        };
const char *Txt_EXAM_ANNOUNCEMENT[Txt_NUM_LANGUAGES] =
	{
	"ANRUF F&Uuml;R PR&Uuml;FUNG",
	"ANNOUNCEMENT OF EXAM",
	"CONVOCATORIA DE EXAMEN",
	"CONVOCATION &Agrave; UN EXAMEN",
	"CHIAMATA PER ESAME",
	"CHAMADA DE EXAME",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Approximate_duration[Txt_NUM_LANGUAGES] =
	{
	"Voraussichtliche Dauer",
	"Approximate duration",
	"Duraci&oacute;n aproximada",
	"Dur&eacute;e approximative",
	"Durata approssimativa",
	"Dura&ccedil;&atilde;o aproximada",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Course[Txt_NUM_LANGUAGES] =
	{
	"Vorlesung",
	"Course",
	"Asignatura",
	"Mati&egrave;re",
	"Materia",
	"Disciplina",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Documentation_required[Txt_NUM_LANGUAGES] =
	{
	"Dokumentation erforderlich",
	"Documentation required",
	"Documentaci&oacute;n exigida",
	"Documentation n&eacute;cessaire",
	"Documentazione necessaria",
	"Documenta&ccedil;&atilde;o necess&aacute;ria",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Exam_date[Txt_NUM_LANGUAGES] =
	{
	"Datum der Pr&uuml;fung",
	"Exam date",
	"Fecha del examen",
	"Date de l'examen",
	"Data di esame",
	"Exame data",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Material_allowed[Txt_NUM_LANGUAGES] =
	{
	"Material erlaubt",
	"Material allowed",
	"Material permitido",
	"Mat&eacute;riel autoris&eacute;",
	"Materiale consentito",
	"Material autorizado",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Material_required[Txt_NUM_LANGUAGES] =
	{
	"Material zwingend",
	"Material required",
	"Material obligatorio",
	"Mat&eacute;riel n&eacute;cessaire",
	"Materiale richiesto",
	"Material obrigat&oacute;rio",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Mode[Txt_NUM_LANGUAGES] =
	{
	"Modalit&auml;t<br />(schriftlich, m&uuml;ndlich,...)",
	"Mode<br />(written, oral,...)",
	"Modalidad<br />(escrito, oral,...)",
	"Mode<br />(&eacute;crit, oral,...)",
	"Modalit&agrave;<br />(scritto, orale,...)",
	"Modalidade<br />(escrita, oral,...)",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Other_information[Txt_NUM_LANGUAGES] =
	{
	"Weitere Informationen",
	"Other information",
	"Otras indicaciones",
	"Autres informations",
	"Altre informazioni",
	"Outras informa&ccedil;&otilde;es",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Place_of_exam[Txt_NUM_LANGUAGES] =
	{
	"Ort der Pr&uuml;fung",
	"Place of exam",
	"Lugar de realizaci&oacute;n",
	"Lieu de l'examen",
	"Luogo di esame",
	"Local do exame",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Session[Txt_NUM_LANGUAGES] =
	{
	"Zeitraum<br />(Juni, September,...)",
	"Session<br />(june, september,...)",
	"Convocatoria<br />(junio, septiembre,...)",
	"P&eacute;riode<br />(juin, septembre,...)",
	"Periodo<br />(giugno, settembre,...)",
	"Per&iacute;odo<br />(junho, setembro,...)",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Start_time[Txt_NUM_LANGUAGES] =
	{
	"Beginn",
	"Start time",
	"Hora de inicio",
	"Heure de d&eacute;but",
	"Ora di inizio",
	"Hora de in&iacute;cio",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Structure_of_the_exam[Txt_NUM_LANGUAGES] =
	{
	"Struktur der Pr&uuml;fung",
	"Structure of the exam",
	"Estructura del examen",
	"Structure de l'examen",
	"Struttura di esame",
	"Estrutura do exame",
	};
const char *Txt_EXAM_ANNOUNCEMENT_Year[Txt_NUM_LANGUAGES] =	// 1, 2, 3..., not 1984, 2009...
	{
	"Jahr",
	"Year",
	"Curso",
	"Ann&eacute;e",
	"Anno",
	"Ano",
	};
const char *Txt_Expand_FOLDER[Txt_NUM_LANGUAGES] =	// Show all the files inside the folder
	{
	"Erweitern",
	"Expand",
	"Expandir",
	"&Eacute;largir",
	"Espandere",
	"Expandir",
	};
const char *Txt_Family_address[Txt_NUM_LANGUAGES] =
	{
	"Familie-Adresse",
	"Family address",
	"Domicilio familiar",
	"Adresse famille",
	"Indirizzo famiglia",
	"Endere&ccedil;o Fam&iacute;lia",
	};
const char *Txt_Feedback_to_students[Txt_NUM_LANGUAGES] =
	{
	"Feedback an die Studenten",
	"Feedback to students",
	"Realimentaci&oacute;n para alumnos",
	"Feedback pour les &eacute;tudiants",
	"Feedback per gli studenti",
	"Feedback para estudantes",
	};
/*
const char *Txt_Festivities[Txt_NUM_LANGUAGES] =
	{
	"Feste",
	"Festivities",
	"Festividades",
	"F&ecirc;tes",
	"Feste",
	"Festividades",
        };
*/
/*
const char *Txt_Festivity[Txt_NUM_LANGUAGES] =
	{
	"Fest",
	"Festivity",
	"Festividad",
	"F&ecirc;te",
	"Festa",
	"Festividade",
        };
*/
const char *Txt_Field_BR_name[Txt_NUM_LANGUAGES] =
	{
	"Feldname",
	"Field<br />name",
	"Nombre<br />del campo",
	"Nom<br />du champ",
	"Nome<br />del campo",
	"Nome<br />do campo",
        };
const char *Txt_File[Txt_NUM_LANGUAGES] =
	{
	"Datei",
	"File",
	"Archivo",
	"Fichier",
	"File",
	"Arquivo",
        };
const char *Txt_file[Txt_NUM_LANGUAGES] =
	{
	"Datei",
	"file",
	"archivo",
	"fichier",
	"file",
	"arquivo",
        };
const char *Txt_file_folder[Txt_NUM_LANGUAGES] =
	{
	"Datei/Verzeichnis",
	"file/folder",
	"archivo/carpeta",
	"fichier/r&eacute;pertoire",
	"file/cartella",
	"arquivo/diret&oacute;rio",
        };
const char *Txt_File_folder_X_is_now_hidden[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Das Datei/Verzeichnis <strong>%s</strong> ist jetzt versteckt.",
	"File/folder <strong>%s</strong> is now hidden.",
	"El archivo/carpeta <strong>%s</strong> ahora est&aacute; oculto.",
	"Le fichier/r&eacute;pertoire <strong>%s</strong> est maintenant cach&eacute;.",
	"Il file/cartella <strong>%s</strong> &egrave; ora nascosto.",
	"O arquivo/diret&oacute;rio <strong>%s</strong> &eacute; agora oculto.",
        };
const char *Txt_File_folder_X_is_now_visible[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Das Datei/Verzeichnis <strong>%s</strong> ist jetzt sichtbar.",
	"File/folder <strong>%s</strong> is now visible.",
	"El archivo/carpeta <strong>%s</strong> ahora est&aacute; visible.",
	"Le fichier/r&eacute;pertoire <strong>%s</strong> est maintenant visible.",
	"Il file/cartella <strong>%s</strong> &egrave; ora visibile.",
	"O arquivo/diret&oacute;rio <strong>%s</strong> &eacute; agora vis&iacute;vel.",
        };
const char *Txt_File_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Datei <strong>%s</strong> entfernt.",
	"File <strong>%s</strong> removed.",
	"Archivo <strong>%s</strong> eliminado.",
	"Fichier <strong>%s</strong> supprim&eacute;.",
	"File <strong>%s</strong> rimosso.",
	"Arquivo <strong>%s</strong> removido.",
        };
const char *Txt_File_too_large[Txt_NUM_LANGUAGES] =
        {
         "Datei zu gro&szlig;",
         "File too large",
         "Archivo demasiado grande",
         "Fichier est trop gros",
         "File troppo grande",
         "Arquivo muito grande",
        };
const char *Txt_X_file_type_is_not_HTML[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"<strong>%s</strong> file type is not HTML.",	// Need bersetzung
	"<strong>%s</strong> file type is not HTML.",
	"El archivo <strong>%s</strong> no es de tipo HTML.",
	"<strong>%s</strong> file type is not HTML.",	// Besoin de traduction
	"<strong>%s</strong> file type is not HTML.",	// Bisogno di traduzione
	"<strong>%s</strong> file type is not HTML.",	// Necessita de traduo
	};
const char *Txt_File_with_list_of_IDs[Txt_NUM_LANGUAGES] =
        {
         "Datei mit einer Liste von Ausweis-Nr.",
         "File with list of IDs",
         "Archivo con lista de DNI",
         "Fichier avec la liste de num&eacute;ros d'identit&eacute;",
         "File con lista di numeri d'identit&agrave;",
         "Arquivo com lista de n&ordm;s. identif.",
        };
const char *Txt_File_with_your_photo[Txt_NUM_LANGUAGES] =
	{
	"Datei mit Ihrem Foto",
	"File with your photo",
	"Archivo con la foto",
	"Fichier avec votre photo",
	"File con la tua foto",
	"Arquivo com a sua foto",
        };
const char *Txt_File_zones[Txt_NUM_LANGUAGES] =
       {
	"Datei Zonen",
	"File zones",
	"Zonas de archivos",
	"Zones de fichiers",
	"Zone di file",
	"Zonas arquivos",
	};
const char *Txt_File_zones_of_the_group_X_Y_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
	"File zones of the group <strong>%s %s</strong> removed.",	// Need bersetzung
	"File zones of the group <strong>%s %s</strong> removed.",
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas."
	"File zones of the group <strong>%s %s</strong> removed.",	// Besoin de traduction
	"File zones of the group <strong>%s %s</strong> removed.",	// Bisogno di traduzione
	"File zones of the group <strong>%s %s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Files[Txt_NUM_LANGUAGES] =
	{
	"Dateien",
	"Files",
	"Archivos",
	"Fichiers",
	"File",
	"Arquivos",
        };
const char *Txt_files[Txt_NUM_LANGUAGES] =
	{
	"Dateien",
	"files",
	"archivos",
	"fichiers",
	"file",
	"arquivos",
        };
const char *Txt_Files_assignments[Txt_NUM_LANGUAGES] =
	{
	"Dateien Aktivitaten",
	"Files assignments",
	"Archivos actividades",
	"Fichiers activites",
	"File attivita",
	"Arquivos atividades",
        };
const char *Txt_Files_copied[Txt_NUM_LANGUAGES] =
	{
	"Dateien kopiert",
	"Files copied",
	"Archivos copiados",
	"Fichiers copi&eacute;s",
	"File copiati",
	"Arquivos copiados",
        };
const char *Txt_Files_of_marks_must_contain_a_table_in_HTML_format_[Txt_NUM_LANGUAGES] =
	{
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures." \
		" Each HTML file may be generated, for example, from a spreadsheet," \
		" by selecting the desired portion of the table and saving it as web page.<br />" \
		" A table can have <strong>multiple header and footer rows</strong>." \
		" Between them it should be <strong>a single row for each student</strong>," \
		" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />" \
		" A student will see a table with header rows, followed by the row with their marks, and footer rows.",	// Need bersetzung
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures." \
		" Each HTML file may be generated, for example, from a spreadsheet," \
		" by selecting the desired portion of the table and saving it as web page.<br />" \
		" A table can have <strong>multiple header and footer rows</strong>." \
		" Between them it should be <strong>a single row for each student</strong>," \
		" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />" \
		" A student will see a table with header rows, followed by the row with their marks, and footer rows.",
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes." \
		" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo," \
		" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />" \
		" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>." \
		" Entre ellas debe haber <strong>una &uacute;nica fila para cada alumno</strong>," \
		" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el DNI del alumno</strong>.<br />" \
		" Un alumno ver&aacute; una tabla con las filas de cabecera," \
		" seguidas por la fila con sus calificaciones," \
		" y por las filas de pie.",
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures." \
		" Each HTML file may be generated, for example, from a spreadsheet," \
		" by selecting the desired portion of the table and saving it as web page.<br />" \
		" A table can have <strong>multiple header and footer rows</strong>." \
		" Between them it should be <strong>a single row for each student</strong>," \
		" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />" \
		" A student will see a table with header rows, followed by the row with their marks, and footer rows.",	// Besoin de traduction
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures." \
		" Each HTML file may be generated, for example, from a spreadsheet," \
		" by selecting the desired portion of the table and saving it as web page.<br />" \
		" A table can have <strong>multiple header and footer rows</strong>." \
		" Between them it should be <strong>a single row for each student</strong>," \
		" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />" \
		" A student will see a table with header rows, followed by the row with their marks, and footer rows.",	// Bisogno di traduzione
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures." \
		" Each HTML file may be generated, for example, from a spreadsheet," \
		" by selecting the desired portion of the table and saving it as web page.<br />" \
		" A table can have <strong>multiple header and footer rows</strong>." \
		" Between them it should be <strong>a single row for each student</strong>," \
		" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />" \
		" A student will see a table with header rows, followed by the row with their marks, and footer rows.",	// Necessita de traduo
	};
const char *Txt_Files_works[Txt_NUM_LANGUAGES] =
	{
	"Dateien Hausarbeiten",
	"Files works",
	"Archivos trabajos",
	"Fichiers travaux",
	"File lavori",
	"Arquivos trabalhos",
        };
const char *Txt_first_message_not_allowed[Txt_NUM_LANGUAGES] =
       {
	"erste Nachricht nicht erlaubt",
	"first message not allowed",
	"primer mensaje no permitido",
	"premier message non admis",
	"primo messaggio non ammessi",
	"primeira mensagem n&atilde;o permitida",
	};
const char *Txt_First_name[Txt_NUM_LANGUAGES] =
	{
	"Vorname",
	"First name",
	"Nombre",
	"Pr&eacute;nom",
	"Nome",
	"Nome",
        };
const char *Txt_Folder[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnis",
	"Folder",
	"Carpeta",
	"R&eacute;pertoire",
	"Cartella",
	"Diret&oacute;rio",
        };
const char *Txt_folder[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnis",
	"folder",
	"carpeta",
	"r&eacute;pertoire",
	"cartella",
	"diret&oacute;rio",
        };
const char *Txt_Folder_X_and_all_its_contents_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
        {
	"Verzeichnis <strong>%s</strong> und alle Inhalte entfernt.",
	"Folder <strong>%s</strong> and all its contents removed.",
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.",
	"R&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu supprim&eacute;s.",
	"Cartella <strong>%s</strong> e tutti i suoi contenuti rimossi.",
	"Diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do removidos.",
        };
const char *Txt_Folder_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Verzeichnis <strong>%s</strong> entfernt.",
	"Folder <strong>%s</strong> removed.",
	"Carpeta <strong>%s</strong> eliminada.",
	"R&eacute;pertoire <strong>%s</strong> supprim&eacute;.",
	"Cartella <strong>%s</strong> rimossa.",
	"Diret&oacute;rio <strong>%s</strong> removido.",
        };
const char *Txt_Folders[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnisse",
	"Folders",
	"Carpetas",
	"R&eacute;pertoires",
	"Cartelle",
	"Diret&oacute;rios",
        };
const char *Txt_folders[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnisse",
	"folders",
	"carpetas",
	"r&eacute;pertoires",
	"cartelle",
	"diret&oacute;rios",
        };
const char *Txt_Folders_copied[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnisse kopiert",
	"Folders copied",
	"Carpetas copiadas",
	"R&eacute;pertoires copi&eacute;s",
	"Cartelle copiato",
	"Diret&oacute;rios copiados",
        };
const char *Txt_Folders_renamed[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnisse umbenannt",
	"Folders renamed",
	"Carpetas renombradas",
	"R&eacute;pertoires renomm&eacute;s",
	"Cartelle rinominate",
	"Diret&oacute;rios renomeados",
        };
const char *Txt_Folders_not_renamed[Txt_NUM_LANGUAGES] =
	{
	"Verzeichnisse nicht umbenannt",
	"Folders not renamed",
	"Carpetas no renombradas",
	"R&eacute;pertoires pas renomm&eacute;s",
	"Cartelle non rinominate",
	"Diret&oacute;rios n&atilde;o renomeados",
        };
/*
const char *Txt_Forum[Txt_NUM_LANGUAGES] =
	{
	"Forum",
	"Forum",
	"Foro",
	"Forum",
	"Forum",
	"F&oacute;rum",
        };
*/
const char *Txt_Forum_posts[Txt_NUM_LANGUAGES] =
       {
	"Forum-Beitr&auml;ge",
	"Forum posts",
	"Mensajes de foro",
	"Messages de forum",
	"Post del forum",
	"Posts de f&oacute;rum",
	};
const char *Txt_Forum_threads[Txt_NUM_LANGUAGES] =
       {
	"Forum-Threads",
	"Forum threads",
	"Hebras de foro",
	"Fils de forum",
	"Thread del forum",
	"Threads de f&oacute;rum",
	};
const char *Txt_Forum_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
	{
	"Forum",
	"Forum",
	"Foro",
	"Forum",
	"Forum",
	"Frum",
        };
const char *Txt_Forums[Txt_NUM_LANGUAGES] =
	{
	"Foren",
	"Forums",
	"Foros",
	"Forums",
	"Fori",
	"F&oacute;runs",
        };
const char *Txt_from[Txt_NUM_LANGUAGES] =
	{
	"von",
	"from",
	"de",
	"de",
	"da",
	"de",
        };
const char *Txt_from_A_COURSE[Txt_NUM_LANGUAGES] =
	{
	"von",
	"from",
	"desde",
	"de",
	"da",
	"de",
        };
const char *Txt_from_this_BR_course[Txt_NUM_LANGUAGES] =
	{
	"von diesem<br />Vorlesung",
	"from this<br />course",
	"desde esta<br />asignatura",
	"de cette<br />mati&egrave;re",
	"da questa<br />materia",
	"desta<br />disciplina",
        };
const char *Txt_Full_Name[Txt_NUM_LANGUAGES] =
	{
	"Vollst&auml;ndiger Name",
	"Full name",
	"Nombre completo",
	"Nom complet",
	"Nome completo",
	"Nome completo",
	};
const char *Txt_Full_BR_Name[Txt_NUM_LANGUAGES] =
	{
	"Vollst&auml;ndiger<br />Name",
	"Full<br />name",
	"Nombre<br />completo",
	"Nom<br />complet",
	"Nome<br />completo",
	"Nome<br />completo",
	};
const char *Txt_General[Txt_NUM_LANGUAGES] =
	{
	"Allgemein",
	"General",
	"General",
	"G&eacute;n&eacute;ral",
	"Generale",
	"Geral",
        };
const char *Txt_General_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &eacute; here
	{
	"Allgemein",
	"General",
	"General",
	"Gnral",
	"Generale",
	"Geral",
        };
/*
const char *Txt_General_topics[Txt_NUM_LANGUAGES] =
	{
	"Allgemeine Themen",
	"General topics",
	"Temas generales",
	"Affaires g&eacute;n&eacute;rales",
	"Temi generali",
	"Temas gerais",
        }; */
const char *Txt_general_topics[Txt_NUM_LANGUAGES] =
	{
	"allgemeine Themen",
	"general topics",
	"temas generales",
	"affaires g&eacute;n&eacute;rales",
	"temi generali",
	"temas gerais",
        };
const char *Txt_Generate_test[Txt_NUM_LANGUAGES] =
	{
	"Generieren Test",
	"Generate test",
	"Generar test",
	"G&eacute;n&eacute;rer le test",
	"Genera test",
	"Gerar teste",
        };
const char *Txt_Go_to_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Gehe zum %s",
	"Go to %s",
	"Ir a %s",
	"Aller &agrave; %s",
	"Vai a %s",
	"Ir para %s",
	};
const char *Txt_Go_to_BR_course_BR_and_reply[Txt_NUM_LANGUAGES] =
	{
	"Gehe zum<br />Vorlesung und Antworten",
	"Go to<br />course<br />and reply",
	"Ir a<br />asignatura<br />y responder",
	"Aller &agrave; la<br />mati&egrave;re<br />et r&eacute;pondre",
	"Vai alla<br />materia<br />e rispondere",
	"Ir para<br />a disciplina<br />e responder",
	};
const char *Txt_Go_to_BR_course_BR_and_reply_BR_again[Txt_NUM_LANGUAGES] =
	{
	"Gehe zum<br />Vorlesung<br />und Antworten<br />wieder",
	"Go to<br />course<br />and reply<br />again",
	"Ir a<br />asignatura<br />y responder<br />otra vez",
	"Aller &agrave; la<br />mati&egrave;re<br />et r&eacute;pondre<br />&agrave; nouveau",
	"Vai alla<br />materia<br />e rispondere<br />nuovamente",
	"Ir para<br />a disciplina<br />e responder<br />novamente",
	};
const char *Txt_Go_to_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &agrave; here
	{
	"Gehe zum",
	"Go to",
	"Ir a",
	"Aller ",
	"Vai a",
	"Ir para",
	};
const char *Txt_Group[Txt_NUM_LANGUAGES] =
	{
	"Gruppe",
	"Group",
	"Grupo",
	"Groupe",
	"Gruppo",
	"Grupo",
        };
const char *Txt_group[Txt_NUM_LANGUAGES] =
	{
	"Gruppe",
	"group",
	"grupo",
	"groupe",
	"gruppo",
	"grupo",
        };
const char *Txt_Group_name[Txt_NUM_LANGUAGES] =
	{
	"Name der Gruppe",
	"Group name",
	"Nombre del grupo",
	"Nom du groupe",
	"Nome del gruppo",
	"Nome do grupo",
        };
const char *Txt_Group_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Group <strong>%s</strong> removed.",	// Need bersetzung
	"Group <strong>%s</strong> removed.",
	"Grupo <strong>%s</strong> eliminado."
	"Group <strong>%s</strong> removed.",	// Besoin de traduction
	"Group <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Group <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Groups[Txt_NUM_LANGUAGES] =
	{
	"Gruppen",
	"Groups",
	"Grupos",
	"Groupes",
	"Gruppi",
	"Grupos",
        };
const char *Txt_Groups_in_the_course_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Gruppen in das Vorlesung <strong>%s</strong>:",
	"Groups in the course <strong>%s</strong>:",
	"Grupos en la asignatura <strong>%s</strong>:",
	"Groupes dans la mati&egrave;re <strong>%s</strong>:",
	"Gruppi nella materia <strong>%s</strong>:",
	"Grupos na disciplina <strong>%s</strong>:",
        };
const char *Txt_Groups_in_the_course_X_to_which_you_belong[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Gruppen in das Vorlesung <strong>%s</strong>, zu denen Sie geh&ouml;ren:",
	"Groups in the course <strong>%s</strong> to which you belong:",
	"Grupos en la asignatura <strong>%s</strong> a los que usted pertenece:",
	"Groupes dans la mati&egrave;re <strong>%s</strong> auxquels vous appartenez:",
	"Gruppi nella materia <strong>%s</strong> a cui voi appartenete:",
	"Grupos na disciplina <strong>%s</strong> aos quais voc&ecirc; pertence:",
        };
const char *Txt_Help_on_the_file_browser[Txt_NUM_LANGUAGES] =
	{
	"Hilfe auf dem Datei-Browser",
	"Help on the file browser",
	"Ayuda sobre el explorador de archivos",
	"Aide sur le navigateur de fichiers",
	"Guida sul file browser",
	"Ajuda sobre o arquivo navegador ",
        };
const char *Txt_Hide[Txt_NUM_LANGUAGES] =
	{
	"Ausblenden",
	"Hide",
	"Ocultar",
	"Masquer",
	"Nascondi",
	"Ocultar",
        };
const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY][Txt_NUM_LANGUAGES] =
	{
		{"Urlaub"		,"Holiday"		,"D&iacute;a festivo"		,"Jour f&eacute;ri&eacute;"		,"Vacanza"		,"Feriado"			},
		{"Non-Schule Zeitraum"	,"Non school period"	,"Per&iacute;odo no lectivo"	,"P&eacute;riode sans &eacute;cole"	,"Periodo senza scuola"	,"Per&iacute;odo sem escola"	}
	};
const char *Txt_Holiday[Txt_NUM_LANGUAGES] =
	{
	"Feiertag",
	"Holiday",
	"Festividad",
	"F&ecirc;te",
	"Vacanze",
	"F&eacute;ria",
        };
const char *Txt_Holiday_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Holiday <strong>%s</strong> removed.",	// Need bersetzung
	"Holiday <strong>%s</strong> removed.",
	"Festividad <strong>%s</strong> eliminada."
	"Holiday <strong>%s</strong> removed.",	// Besoin de traduction
	"Holiday <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Holiday <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Holidays[Txt_NUM_LANGUAGES] =
	{
	"Feiertage",
	"Holidays",
	"D&iacute;as festivos",
	"Jours f&eacute;ri&eacute;s",
	"Vacanze",
	"F&eacute;rias",
        };
const char *Txt_I_forgot_my_password[Txt_NUM_LANGUAGES] =
	{
	"Ich habe mein Passwort vergessen",
	"I forgot my password",
	"He olvidado mi contrase&ntilde;a",
	"J'ai oubli&eacute; mon mot de passe",
	"Ho dimenticato la password",
	"Esqueci minha senha",
	};
const char *Txt_Icons[Txt_NUM_LANGUAGES] =
	{
	"Symbole",
	"Icons",
	"Iconos",
	"Ic&ocirc;nes",
	"Icone",
	"&Iacute;cones",
        };
const char *Txt_ID_or_nick[Txt_NUM_LANGUAGES] =
	{
	"Ausweis-Nr. oder Benutzername",
	"ID or nick",
	"DNI o apodo",
	"Num&eacute;ro d'identit&eacute; ou surnom",
	"Numero d'identit&agrave; o soprannome",
	"N&ordm; identif. ou alcunha",
	};
const char *Txt_IDs_or_nicknames_separated_by_commas[Txt_NUM_LANGUAGES] =
        {
        "Ausweis-Nr. oder Spitznamen durch Kommas getrennt",
        "IDs or nicknames separated by commas",
        "DNI o apodos separados por comas",
        "Num&eacute;ros d'identit&eacute; ou surnoms s&eacute;par&eacute;s par des virgules",
        "Numeri d'identit&agrave; o soprannomi separati da virgole",
	"N&ordm;s. identif. ou apelidos separados por v&iacute;rgulas",
        };
const char *Txt_If_you_have_forgotten_your_password_[Txt_NUM_LANGUAGES] =
	{
        "Wenn Sie Ihr Passwort vergessen haben," \
		" eine neue wird die E-Mail von Ihnen angegebene Adresse in Ihren Profil gesendet werden vergessen.",
        "If you have forgotten your password," \
		" a new one will be sent to the e-mail address you entered in your record.",
        "Si usted ha olvidado su contrase&ntilde;a," \
		" se le enviar&aacute; una nueva a la direcci&oacute;n de correo electr&oacute;nico que escribi&oacute; en su ficha personal.",
        "Si vous avez oubli&eacute; votre mot de passe," \
		" une nouvelle sera envoy&eacute; &agrave; l'adresse &eacute;lectronique que vous avez entr&eacute; dans votre carte.",
        "Se hai dimenticato la password," \
		" una nuova verr&agrave; inviata all'indirizzo e-mail che hai inserito nel vostra scheda.",
        "Se voc&ecirc; tiver esquecido sua senha," \
		" uma nova ser&aacute; enviada para o e-mail que voc&ecirc; inseriu em seu cart&atilde;o.",
        };
const char *Txt_If_you_have_written_your_ID_or_nickname_correctly_[Txt_NUM_LANGUAGES] =
	{
	"Wenn Sie Ihre Ausweis-Nr. oder Benutzername korrekt geschrieben wurden, und die E-Mail-Adresse auf Ihren Profil korrekt ist," \
		" erhalten Sie eine E-Mail mit einem neuen Passwort.",
	"If you have written your ID or nickname correctly, and the e-mail address on your record is correct," \
		" you will receive an e-mail with a new password.",
	"Si usted ha escrito correctamente su DNI o apodo, y la direccin de correo electr&oacute;nico en su ficha es correcta," \
		" recibir&aacute; un correo electr&oacute;nico con una nueva contrase&ntilde;a.",
	"Si vous avez &eacute;crit votre num&eacute;ro d'identit&eacute; ou surnom correctement, ainsi que l'adresse e-mail sur votre carte est correct," \
		" vous recevrez un e-mail avec un nouveau mot de passe.",
	"Se avete scritto il vostro numero d'identit&agrave; o soprannome correttamente, e l'indirizzo e-mail della tua scheda &egrave; corretto," \
		" riceverete una e-mail con una nuova password.",
	"Se voc&ecirc; ter escrito o suo n&ordm; identif. ou alcunha corretamente, bem como o endere&ccedil;o de e-mail em seu cart&atilde;o est&aacute; correto," \
		" voc&ecirc; receber&aacute; um e-mail com uma nova senha.",
	};
const char *Txt_If_you_no_longer_wish_to_receive_email_notifications_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Wenn Sie nicht mehr erhalten mchten, E-Mail-Benachrichtigungen, Deaktivieren Sie die Option in:",
	"If you no longer wish to receive e-mail notifications, uncheck the option in:",
	"Si no desea recibir ms avisos por correo electrnico, desactive la opcin correspondiente en:",
	"Si vous ne souhaitez plus recevoir de notifications par e-mail, dcochez l'option dans:",
	"Se non desideri pi ricevere notifiche via e-mail, deselezionare l'opzione in:",
	"Se j no desejar receber notificaes por e-mail, desmarque a opo em:",
        };
const char *Txt_If_you_send_this_message_you_will_appear_as_its_author[Txt_NUM_LANGUAGES] =
	{
	"Wenn Senden diese Nachricht, Sie wird als dessen Urheber",
	"if you send this message, you will appear as its author",
	"si env&iacute;a este mensaje, usted aparecer&aacute; como autor",
	"si vous envoyez ce message, vous appara&icirc;trez comme son auteur",
	"se si invia questo messaggio, voi apparir&agrave; come il suo autore",
	"se voc&ecirc; enviar esta mensagem, voc&ecirc; vai aparecer como o seu autor",
	};
const char *Txt_in[Txt_NUM_LANGUAGES] =
	{
	"in",
	"in",
	"en",
	"en",
	"in",
	"em",
	};
const char *Txt_in_any_degree[Txt_NUM_LANGUAGES] =
	{
	"in jedem Grad",
	"in any degree",
	"en cualquier titulaci&oacute;n",
	"en tout &eacute;tude",
	"in ogni laurea",
	"em qualquer titula&ccedil;ao",
	};
const char *Txt_Indicators[Txt_NUM_LANGUAGES] =
	{
	"Indikatoren",
	"Indicators",
	"Indicadores",
	"Indicateurs",
	"Indicatori",
	"Indocadores",
	};
const char *Txt_INFO_SRC_HELP[Crs_NUM_INFO_SOURCES][Txt_NUM_LANGUAGES] =
	{
		{
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		},
		{
		"Interner Editor, nur f&uuml;r Studienplan (Vorlesung und &Uuml;bungen)",
		"internal editor, only for syllabus (lectures and practicals)",
		"editor integrado, s&oacute;lo para los temarios de teor&iacute;a y pr&aacute;cticas",
		"&eacute;diteur interne, que pour les plans (th&eacute;orie et pratique)" ,
		"editor interno, solo per le programmazioni de classi teoriche e pratiche",
		"interno editor, apenas para programas de teoria y pr&aacute;tica",
		},
		{
		"Plain Text Editor",
		"plain text editor",
		"editor de texto plano",
		"&eacute;diteur de texte simple",
		"editor di testo normale",
		"editor de texto simples",
		},
		{
		"senden Sie eine HTML-Datei, oder als ZIP-Datei mit einer Datei index.html / index.htm und Bild-Dateien",
		"send an HTML file, or a ZIP file containing a file index.html / index.htm and image files",
		"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes",
		"envoyer un fichier HTML, ou un fichier ZIP contenant un fichier index.html / index.htm et les fichiers d'images",
		"inviare un file HTML, o un file ZIP contenente un file index.html / index.htm e file di immagine",
		"enviar um arquivo HTML, ou um arquivo ZIP que cont&eacute;m um arquivo index.html / index.htm e limas da imagem",
		},
		{
		"Geben Sie die Adresse einer Web-Seite geleitet werden, der sich auf einem anderen Server",
		"enter the address of a web page that is hosted on another server",
		"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor",
		"entrer l'adresse d'une page Web qui est h&eacute;berg&eacute;e sur un autre serveur",
		"immettere l'indirizzo di una pagina web che &egrave; ospitata su un altro server",
		"introduza o endere&ccedil;o de uma p&aacute;gina Web que est&aacute; hospedada em outro servidor",
		}
	};
const char *Txt_INFO_SRC_FULL_TEXT[Crs_NUM_INFO_SOURCES][Txt_NUM_LANGUAGES] =
	{
		{
		"Geben keine Auskunft",
		"Do not give any information",
		"No ofrecer ninguna informaci&oacute;n",
		"Ne pas donner d'information",
		"Non dare informazioni",
		"N&atilde;o d&ecirc; nenhuma informa&ccedil;&atilde;o",
		},
		{
		"Interner Editor",
		"Internal editor",
		"Editor interno",
		"&Eacute;diteur interne",
		"Editor interno",
		"Interno editor",
		},
		{
		"Plain Text",
		"Plain text",
		"Texto plano",
		"Plain text",
		"Testo normale",
		"Texto simples",
		},
		{
		"Webseite gehostet hier",
		"Web page hosted here",
		"P&aacute;gina web alojada aqu&iacute;",
		"Page Web h&eacute;berg&eacute;e ici",
		"Pagina Web ospitati qui",
		"P&aacute;gina da Web hospedada aqui",
		},
		{
		"Webseite auf einem anderen Server gehostet",
		"Web page hosted on another server",
		"P&aacute;gina web alojada en otro servidor",
		"Page Web h&eacute;berg&eacute;e sur un autre serveur",
		"Pagina Web ospitata su un altro server",
		"P&aacute;gina da Web hospedada em outro servidor",
		}
	};
const char *Txt_INFO_SRC_SHORT_TEXT[Crs_NUM_INFO_SOURCES][Txt_NUM_LANGUAGES] =
	{
		{
		"Keine Angaben",
		"No information",
		"Ninguna informaci&oacute;n",
		"Aucune information",
		"Nessuna informazione",
		"Sem informa&ccedil;&atilde;o",
		},
		{
		"Interner Editor",
		"Internal editor",
		"Editor interno",
		"&Eacute;diteur interne",
		"Editor interno",
		"Interno editor",
		},
		{
		"Plain Text",
		"Plain text",
		"Texto plano",
		"Plain text",
		"Testo normale",
		"Texto simples",
		},
		{
		"Interne Webseite",
		"Internal page",
		"P&aacute;gina interna",
		"Page interne",
		"Interne pagina",
		"Interna p&aacute;gina",
		},
		{
		"Externe Webseite",
		"External page",
		"P&aacute;gina externa",
		"Page externe",
		"Esterne pagina",
		"Externa p&aacute;gina",
		}
	};
const char *Txt_Institution[Txt_NUM_LANGUAGES] =
	{
	"Institution",
	"Institution",
	"Instituci&oacute;n",
	"&Eacute;tablissement",
	"Istituzione",
	"Institu&ccedil;&atilde;o",
	};
const char *Txt_Institution_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Institution <strong>%s</strong> removed.",	// Need bersetzung
	"Institution <strong>%s</strong> removed.",
	"Instituci&oacute;n <strong>%s</strong> eliminada."
	"Institution <strong>%s</strong> removed.",	// Besoin de traduction
	"Institution <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Institution <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Institution_unspecified[Txt_NUM_LANGUAGES] =
	{
	"Institution nicht spezifiziert",
	"Institution unspecified",
	"Instituci&oacute;n sin especificar",
	"&Eacute;tablissement non sp&eacute;cifi&eacute;",
	"Istituzione non specificato",
	"Institu&ccedil;&atilde;o indeterminado",
	};
const char *Txt_Institutions[Txt_NUM_LANGUAGES] =
	{
	"Institutionen",
	"Institutions",
	"Instituciones",
	"&Eacute;tablissements",
	"Istituzioni",
	"Institu&ccedil;&otilde;es",
	};
const char *Txt_institutions[Txt_NUM_LANGUAGES] =
	{
	"Institutionen",
	"institutions",
	"instituciones",
	"&eacute;tablissements",
	"istituzioni",
	"institu&ccedil;&otilde;es",
	};
const char *Txt_Institutional_course_code[Txt_NUM_LANGUAGES] =
	{
	"Institut.<br />Code",
	"Institutional<br />code",
	"C&oacute;digo<br />institucional",
	"Code<br />institut.",
	"Code<br />istituzionali",
	"C&oacute;digo<br />instit.",
	};
const char *Txt_Institutional_links[Txt_NUM_LANGUAGES] =
	{
	"Institut. Links",
	"Institutional links",
	"Enlaces instituc.",
	"Liens institut.",
	"Links istituzionali",
	"Liga&ccedil;&otilde;es instit.",
	};
const char *Txt_Integer_number[Txt_NUM_LANGUAGES] =
	{
	"Ganze Zahl",
	"Integer number",
	"N&ordm; entero",
	"Nombre entier",
	"Numero entero",
	"N&ordm; inteiro",
	};
const char *Txt_X_is_busy_at_the_moment_Try_again_in_a_few_seconds[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"%s ist im Moment besch&auml;ftigt. Wieder in ein paar Sekunden noch einmal.",
	"%s is busy at the moment. Try again in a few seconds.",
	"%s est&aacute; ocupado en este momento. Vuelva a intentarlo dentro de unos segundos.",
	"%s est occup&eacute; en ce moment. Essayez &agrave; nouveau dans quelques secondes.",
	"%s &egrave; occupato in questo momento. Try again in pochi secondi.",
	"%s est&aacute; ocupado no momento. Tente novamente em alguns segundos.",
	};
const char *Txt_It_is_mandatory_to_choose_a_group[Txt_NUM_LANGUAGES] =
	{
	"Ausw&auml;hlen einer Gruppe ist Pflicht",
	"It is mandatory to choose a group",
	"Elegir grupo es obligatorio",
	"Il est obligatoire de choisir un groupe",
	"E 'obbligatorio scegliere un gruppo",
	"Escolher um grupo &eacute; obrigat&oacute;rio",
	};
const char *Txt_It_is_optional_to_choose_a_group[Txt_NUM_LANGUAGES] =
	{
	"Ausw&auml;hlen einer Gruppe ist freiwillig",
	"It is optional to choose a group",
	"Elegir grupo es voluntario",
	"Il est volontaire de choisir un groupe",
	"E 'facoltativo scegliere un gruppo",
	"Escolher um grupo &eacute; volunt&aacute;rio",
	};
const char *Txt_Language[Txt_NUM_LANGUAGES] =
	{
	"Sprache",
	"Language",
	"Idioma",
	"Langue",
	"Lingua",
	"Lingua",
	};
const char *Txt_level[Txt_NUM_LANGUAGES] =
	{
	"Ebene",
	"level",
	"nivel",
	"niveau",
	"livello",
	"n&iacute;vel",
        };
const char *Txt_Levels[Txt_NUM_LANGUAGES] =
	{
	"Ebenen",
	"Levels",
	"Niveles",
	"Niveaux",
	"Livelli",
	"N&iacute;veis",
        };
const char *Txt_levels[Txt_NUM_LANGUAGES] =
	{
	"Ebenen",
	"levels",
	"niveles",
	"niveaux",
	"livelli",
	"n&iacute;veis",
        };
const char *Txt_Link_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Link <strong>%s</strong> removed.",	// Need bersetzung
	"Link <strong>%s</strong> removed.",
	"Enlace <strong>%s</strong> eliminado."
	"Link <strong>%s</strong> removed.",	// Besoin de traduction
	"Link <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Link <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_List_of_IDs[Txt_NUM_LANGUAGES] =
        {
         "Liste von Ausweis-Nr.",
         "List of IDs",
         "Lista de DNI",
         "Liste de num&eacute;ros d'identit&eacute;",
         "Lista di numeri d'identit&agrave;",
         "Lista de n&ordm;s. identif.",
        };
const char *Txt_Local_address[Txt_NUM_LANGUAGES] =
	{
	"Lokale Adresse",
	"Local address",
	"Domicilio local",
	"Adresse locale",
	"Indirizzo locale",
	"Local Address",
	};
const char *Txt_Log_in[Txt_NUM_LANGUAGES] =
	{
	"Anmelden",
	"Log in",
	"Entrar",
	"Entrer",
	"Accedere",
	"Entrar",
	};
const char *Txt_Log_out[Txt_NUM_LANGUAGES] =
	{
	"Abmelden",
	"Log out",
	"Cerrar sesi&oacute;n",
	"Fermer session",
	"Chiudi sessione",
	"Fechar sess&atilde;o",
	};
const char *Txt_logged[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
	/* Usr_GENDER_UNKNOWN */	{"angemeldet"	,"logged"	,"identificado/a"	,"connect&eacute;/e"	,"logatto/a"	,"logado/a"	},
	/* Usr_GENDER_FEMALE */	{"angemeldet"	,"logged"	,"identificada"		,"connect&eacute;e"	,"logatta"	,"logada"	},
	/* Usr_GENDER_MALE */	{"angemeldet"	,"logged"	,"identificado"		,"connect&eacute;"	,"logatto"	,"logado"	},
	/* Usr_GENDER_ALL */	{"angemeldet"	,"logged"	,"identificado/a"	,"connect&eacute;/e"	,"logatto/a"	,"logado/a"	},
	};
const char *Txt_Mandatory_registration[Txt_NUM_LANGUAGES] =	// (to a type of group)
	{
	"Obligatorische beitreten?",
	"Mandatory registration?",
	"Adscripci&oacute;n obligatoria?",
	"Inscription obligatoire?",
	"Iscrizione obbligatoria?",
	"Inscri&ccedil;&atilde;o obrigat&oacute;ria?",
	};
const char *Txt_Mandatory_rules_to_compose_messages[Txt_NUM_LANGUAGES] =
	{
	"Verbindliche Regeln zum Verfassen von Nachrichten (Netiquette)",
	"Mandatory rules to compose messages (netiquette)",
	"Normas obligatorias al redactar mensajes (netiquette)",
	"R&wgrave;gles imp&eacute;ratives de composer des messages (netiquette)",
	"Norme imperative di comporre messaggi (netiquette)",
	"Regras obrigat&oacute;rias para compor mensagens (netiquette)",
	};
const char *Txt_Marks_management_zone[Txt_NUM_LANGUAGES] =
        {
	"Noten Bewirtschaftungszone",
	"Marks management zone",
	"Zona de administraci&oacute;n de calificaciones",
	"Zone de gestion de notes",
	"Zona di gestione di voti",
	"Zona de gest&atilde;o de notas",
        };
const char *Txt_marks_management_zone[Txt_NUM_LANGUAGES] =
        {
	"Noten Bewirtschaftungszone",
	"marks management zone",
	"zona de administraci&oacute;n de calificaciones",
	"zone de gestion de notes",
	"zona di gestione di voti",
	"zona de gest&atilde;o de notas",
        };
const char *Txt_Marks_zone[Txt_NUM_LANGUAGES] =
        {
	"Noten-Zone",
	"Marks zone",
	"Zona de calificaciones",
	"Zone de notes",
	"Zona di voti",
	"Zona de notas",
        };
const char *Txt_Max_levels[Txt_NUM_LANGUAGES] =
	{
	"Max. Ebenen",
	"Max. levels",
	"M&aacute;x. niveles",
	"Max. niveaux",
	"Mas. livelli",
	"M&aacute;x. n&iacute;veis",
        };
const char *Txt_Max_stud[Txt_NUM_LANGUAGES] =
	{
	"Max.Stud.",
	"Max.stud.",
	"M&aacute;x.alum",
	"Max.&eacute;tud.",
	"Mas.stud.",
	"M&aacute;x.estu.",
        };
const char *Txt_maximum[Txt_NUM_LANGUAGES] =
	{
	"Maximum",
	"maximum",
	"m&aacute;ximo",
	"maximum",
	"massimo",
	"m&aacute;ximo",
	};
const char *Txt_message[Txt_NUM_LANGUAGES] =
	{
	"Nachricht",
	"message",
	"mensaje",
	"message",
	"messaggio",
	"mensagem",
	};
const char *Txt_message_received_from_A_USER[Txt_NUM_LANGUAGES] =
	{
	"empfangene Nachricht aus",
	"message received from",
	"mensaje recibido de",
	"message re&ccedil;u de",
	"messaggio ricevuto de",
	"mensagem recebida de",
	};
const char *Txt_Message_removed[Txt_NUM_LANGUAGES] =
	{
	"Message removed.",	// Need bersetzung
	"Message removed.",
	"Mensaje eliminado."
	"Message removed.",	// Besoin de traduction
	"Message removed.",	// Bisogno di traduzione
	"Message removed.",	// Necessita de traduo
	};
const char *Txt_message_sent_to[Txt_NUM_LANGUAGES] =
	{
	"gesendete Nachricht an",
	"message sent to",
	"mensaje enviado a",
	"message envoy&eacute; &agrave;",
	"messaggio inviato a",
	"mensagem enviada para",
	};
const char *Txt_Messages[Txt_NUM_LANGUAGES] =
	{
	"Nachrichten",
	"Messages",
	"Mensajes",
	"Messages",
	"Messaggi",
	"Mensagens",
	};
const char *Txt_messages[Txt_NUM_LANGUAGES] =
	{
	"Nachrichten",
	"messages",
	"mensajes",
	"messages",
	"messaggi",
	"mensagens",
	};
const char *Txt_Messages_received[Txt_NUM_LANGUAGES] =
	{
	"Empfangene Nachrichten",
	"Messages received",
	"Mensajes recibidos",
	"Messages re&ccedil;us",
	"Messaggi ricevuti",
	"Mensagens recebidas",
	};
const char *Txt_Messages_received_from_A_COURSE[Txt_NUM_LANGUAGES] =
	{
	"Empfangene Nachrichten aus",
	"Messages received from",
	"Mensajes recibidos desde",
	"Messages re&ccedil;us de",
	"Messaggi ricevuti de",
	"Mensagens recebidas de",
	};
const char *Txt_messages_received_from_A_USER[Txt_NUM_LANGUAGES] =
	{
	"empfangene Nachrichten aus",
	"messages received from",
	"mensajes recibidos de",
	"messages re&ccedil;us de",
	"messaggi ricevuti de",
	"mensagens recebidas de",
	};
const char *Txt_Messages_sent[Txt_NUM_LANGUAGES] =
	{
	"Gesendete Nachrichten",
	"Messages sent",
	"Mensajes enviados",
	"Messages envoy&eacute;s",
	"Messaggi inviati",
	"Mensagens enviadas",
	};
const char *Txt_Messages_sent_by_teachers[Txt_NUM_LANGUAGES] =
	{
	"Gesendete Nachrichten, die von Dozenten",
	"Messages sent by teachers",
	"Mensajes enviados por profesores",
	"Messages envoy&eacute;s par enseignants",
	"Messaggi inviati da parte degli insegnanti",
	"Mensagens enviadas por professores",
	};
const char *Txt_Messages_sent_from_A_COURSE[Txt_NUM_LANGUAGES] =
	{
	"Gesendete Nachrichten aus",
	"Messages sent from",
	"Mensajes enviados desde",
	"Messages envoy&eacute;s de",
	"Messaggi inviati de",
	"Mensagens enviadas de",
	};
const char *Txt_messages_sent_to_A_USER[Txt_NUM_LANGUAGES] =
	{
	"gesendete Nachrichten an",
	"messages sent to",
	"mensajes enviados a",
	"messages envoy&eacute;s &agrave;",
	"messaggios inviati a",
	"mensagens enviadas para",
	};
const char *Txt_minimum[Txt_NUM_LANGUAGES] =
	{
	"Mindest",
	"minimum",
	"m&iacute;nimo",
	"minimum",
	"minimo",
	"m&iacute;nimo",
	};
const char *Txt_Minimum_time_seconds_per_question_between_two_tests[Txt_NUM_LANGUAGES] =
	{
	"Minimale Zeit (Sekunden)<br />pro Frage<br />zwischen zwei Tests",
	"Minimum time (seconds)<br />per question<br />between two tests",
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests",
	"Minimum de temps (en secondes)<br />par domanda<br />entre deux tests",
	"Minimo tempo (secondo)<br />per questione<br />tra due teste",
	"Tempo m&iacute;nimo (segundos)<br />por quest&atilde;o<br />entre dois testes ",
	};
const char *Txt_minutes[Txt_NUM_LANGUAGES] =
	{
	"Minute",
	"minutes",
	"minutos",
	"minutes",
	"minuti",
	"minutos",
        };
const char *Txt_Modify_assignment[Txt_NUM_LANGUAGES] =
	{
	"&Auml;ndern Aktivit&auml;t",
	"Modify assignment",
	"Modificar actividad",
	"Modifier activit&eacute;",
	"Modifica attivit&agrave;",
	"Modificar atividade",
        };
const char *Txt_Month[Txt_NUM_LANGUAGES] =
	{
	"Monat",
	"Month",
	"Mes",
	"Mois",
	"Mese",
	"M&ecirc;s",
	};
const char *Txt_MSG_containing_the_text[Txt_NUM_LANGUAGES] =
	{
	"mit dem Text",
	"containing the text",
	"que contenga el texto",
	"contenant le texte",
	"contenente il testo",
	"contendo o texto",
        };
const char *Txt_MSGS_containing_the_text[Txt_NUM_LANGUAGES] =
	{
	"mit dem Text",
	"containing the text",
	"que contengan el texto",
	"contenant le texte",
	"contenenti il testo",
	"contendo o texto",
        };
const char *Txt_MSG_From[Txt_NUM_LANGUAGES] =
	{
	"Von",
	"From",
	"De",
	"De",
	"Da",
	"De",
        };
const char *Txt_MSG_From_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Von",
	"From",
	"De",
	"De",
	"Da",
	"De",
        };
const char *Txt_MSG_from[Txt_NUM_LANGUAGES] =
	{
	"von",
	"from",
	"de",
	"de",
	"da",
	"de",
        };
const char *Txt_MSG_Message[Txt_NUM_LANGUAGES] =
	{
	"Nachricht",
	"Message",
	"Mensaje",
	"Message",
	"Messaggio",
	"Mensagem",
        };
const char *Txt_MSG_Subject[Txt_NUM_LANGUAGES] =
	{
	"Betreff",
	"Subject",
	"Asunto",
	"Objet",
	"Oggetto",
	"Assunto",
        };
const char *Txt_MSG_To[Txt_NUM_LANGUAGES] =
	{
	"An",
	"To",
	"Para",
	"Pour",
	"A",
	"Para",
        };
const char *Txt_MSGS_Deleted[Txt_NUM_LANGUAGES] =
	{
	"Gel&ouml;scht",
	"Deleted",
	"Eliminados",
	"Supprim&eacute;s",
	"Eliminati",
	"Suprimidas",
        };
const char *Txt_MSGS_Not_deleted[Txt_NUM_LANGUAGES] =
	{
	"Nicht Gel&ouml;scht",
	"Not deleted",
	"No eliminados",
	"Non supprim&eacute;s",
	"Non eliminati",
	"N&atilde;o suprimidas",
        };
/*
const char *Txt_MSGS_Notified[Txt_NUM_LANGUAGES] =
	{
	"Gemeldeten",
	"Notified",
	"Notificados",
	"Notifi&eacute;s",
	"Notificatos",
	"Notificadas",
        };
*/
const char *Txt_MSGS_Received[Txt_NUM_LANGUAGES] =
	{
	"Empfangene",
	"Received",
	"Recibidos",
	"Re&ccedil;us",
	"Ricevuti",
	"Recebidas",
	};
const char *Txt_MSGS_Sent[Txt_NUM_LANGUAGES] =
	{
	"Gesendete",
	"Sent",
	"Enviados",
	"Envoy&eacute;s",
	"Inviati",
	"Enviadas",
	};
const char *Txt_Multiple_registration[Txt_NUM_LANGUAGES] =	// (to a type of group)
	{
	"Mehrere beitreten?",
	"Multiple registration?",
	"Adscripci&oacute;n m&uacute;ltiple?",
	"Inscription multiple?",
	"Iscrizione multiple?",
	"Inscri&ccedil;&atilde;o m&uacute;ltiplas?",
	};
const char *Txt_My_courses[Txt_NUM_LANGUAGES] =
	{
	"Meine Vorlesungen",
	"My courses",
	"Mis asignaturas",
	"Mes mati&egrave;res",
	"Mie materie",
	"As minha disciplinas",
	};
const char *Txt_Name[Txt_NUM_LANGUAGES] =
	{
	"Name",
	"Name",
	"Nombre",
	"Nom",
	"Nome",
	"Nome",
	};
const char *Txt_Name_on_Skype[Txt_NUM_LANGUAGES] =
	{
	"Name auf Skype&trade;",
	"Name on Skype&trade;",
	"Nombre en Skype&trade;",
	"Nom sur Skype&trade;",
	"Nome Skype&trade;",
	"Nome no Skype&trade;",
	};
const char *Txt_New_assignment[Txt_NUM_LANGUAGES] =
	{
	"Neue Aktivit&auml;t",
	"New assignment",
	"Nueva actividad",
	"Nouvelle activit&eacute;",
	"Nuova attivit&agrave;",
	"Nova atividade",
        };
const char *Txt_New_announcement_of_exam[Txt_NUM_LANGUAGES] =
	{
	"Neuer Anruf f&uuml;r Pr&uuml;fung",
	"New announcement of exam",
	"Nueva convocatoria de examen",
	"Nouvelle convocation &agrave; un examen",
	"Nuova chiamata per esame",
	"Nova chamada para exame",
	};
const char *Txt_New_record_field[Txt_NUM_LANGUAGES] =
	{
	"Neue Datenfeld",
	"New record field",
	"Nuevo campo de ficha",
	"Nouveau champ de carte",
	"Nuovo campo di schede",
	"Novo campo de cart&atilde;o",
	};
const char *Txt_New_centre[Txt_NUM_LANGUAGES] =
	{
	"Neue Zentrum",
	"New centre",
	"Nuevo centro",
	"Nouveau centre",
	"Nuovo centro",
	"Novo centro",
	};
const char *Txt_New_course[Txt_NUM_LANGUAGES] =
	{
	"Neue Vorlesung",
	"New course",
	"Nueva asignatura",
	"Nouvelle mati&egrave;re",
	"Nuova materia",
	"Nova disciplina",
	};
const char *Txt_New_degree[Txt_NUM_LANGUAGES] =
	{
	"Neue Grad",
	"New degree",
	"Nueva titulaci&oacute;n",
	"Nouvel &eacute;tude",
	"Nuova laurea",
	"Nova titula&ccedil;&atilde;o",
	};
const char *Txt_New_department[Txt_NUM_LANGUAGES] =
	{
	"Neue Fachbereich",
	"New department",
	"Nuevo departamento",
	"Nouveau d&eacute;partement",
	"Nuovo dipartimento",
	"Novo departamento",
	};
const char *Txt_New_group[Txt_NUM_LANGUAGES] =
	{
	"Neue Gruppe",
	"New group",
	"Nuevo grupo",
	"Nouveau groupe",
	"Nuovo gruppo",
	"Novo grupo",
	};
const char *Txt_New_holiday[Txt_NUM_LANGUAGES] =
	{
	"Neue Feiertag",
	"New holiday",
	"Nueva festividad",
	"Nouvelle f&ecirc;te",
	"Nuova vacanza",
	"Nova f&eacute;ria",
        };
const char *Txt_New_institution[Txt_NUM_LANGUAGES] =
	{
	"Neuer Institution",
	"New institution",
	"Nueva instituci&oacute;n",
	"Nouvel &eacute;tablissement",
	"Nuova istituzione",
	"Nova institu&ccedil;&atilde;o",
	};
const char *Txt_New_link[Txt_NUM_LANGUAGES] =
	{
	"Neue Link",
	"New link",
	"Nuevo enlace",
	"Nouveau lien",
	"Nuovo link",
	"Nova liga&ccedil;&atilde;o",
	};
const char *Txt_New_message[Txt_NUM_LANGUAGES] =
	{
	"Neue Nachricht",
	"New message",
	"Nuevo mensaje",
	"Nouveau message",
	"Nuovo messaggio",
	"Nova mensagem",
	};
const char *Txt_new_message[Txt_NUM_LANGUAGES] =
	{
	"neue Nachricht",
	"new message",
	"mensaje nuevo",
	"nouveau message",
	"nuovo messaggio",
	"nova mensagem",
	};
const char *Txt_new_MESSAGE[Txt_NUM_LANGUAGES] =
	{
	"neue",
	"new",
	"nuevo",
	"nouveau",
	"nuovo",
	"nova",
	};
const char *Txt_new_messages[Txt_NUM_LANGUAGES] =
	{
	"neue Nachrichten",
	"new messages",
	"mensajes nuevos",
	"nouveaux messages",
	"nuovi messaggi",
	"novas mensagens",
	};
const char *Txt_new_MESSAGES[Txt_NUM_LANGUAGES] =
	{
	"neue",
	"new",
	"nuevos",
	"nouveaux",
	"nuovi",
	"novas",
	};
const char *Txt_New_nickname[Txt_NUM_LANGUAGES] =
	{
	"Neues Benutzername",
	"New nickname",
	"Nuevo apodo",
	"Nouveau surnom",
	"Nuovo soprannome",
	"Nova alcunha",
	};
const char *Txt_New_notice[Txt_NUM_LANGUAGES] =
	{
	"Neue Ank&uuml;ndigung",
	"New notice",
	"Nuevo aviso",
	"Nouvel avis",
	"Nuovo avviso",
	"Novo aviso",
	};
const char *Txt_New_password[Txt_NUM_LANGUAGES] =
	{
	"Neues Passwort",
	"New password",
	"Nueva contrase&ntilde;a",
	"Nouveau mot de passe",
	"Nuova password",
	"Nova senha",
	};
const char *Txt_New_password_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &ntilde; here.
	{
	"Neues Passwort",
	"New password",
	"Nueva contrasea",
	"Nouveau mot de passe",
	"Nuova password",
	"Nova senha",
	};
const char *Txt_New_place[Txt_NUM_LANGUAGES] =
	{
	"Neue Standort",
	"New place",
	"Nuevo lugar",
	"Nouvel emplacement",
	"Nuova localit&grave;",
	"Nova localiza&ccedil;&atilde;o",
	};
const char *Txt_New_plugin[Txt_NUM_LANGUAGES] =
	{
	"Neue Plugin",
	"New plugin",
	"Nuevo complemento",
	"Nouveau plugin",
	"Nuovo plugin",
	"Novo plugin",
	};
const char *Txt_new_tag[Txt_NUM_LANGUAGES] =	// Tag used in self-assessment test
	{
	"neuen Tag",
	"new tag",
	"nuevo descriptor",
	"nouveau descripteur",
	"nuovo descrittore",
	"novo descritor",
	};
const char *Txt_New_thread[Txt_NUM_LANGUAGES] =
       {
	"Neuen Thread",
	"New thread",
	"Nueva hebra",
	"Nouveau fil",
	"Nuovo thread",
	"Novo thread",
	};
const char *Txt_New_type_of_degree[Txt_NUM_LANGUAGES] =
	{
	"Neue Abschlussart",
	"New type of degree",
	"Nuevo tipo de titulaci&oacute;n",
	"Nouveau type d'&eacute;tude",
	"Nuovo tipo di laurea",
	"Novo tipo de titula&ccedil;&atilde;o",
	};
const char *Txt_New_type_of_group[Txt_NUM_LANGUAGES] =
	{
	"Neue Art der Gruppe",
	"New type of group",
	"Nuevo tipo de grupo",
	"Nouveau type de groupe",
	"Nuovo tipo di gruppo",
	"Novo tipo de grupo",
	};
const char *Txt_NEW_YEAR_GREETING[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u in the following sentences
	{
	"Happy %u!",
	"Happy %u!",
	"&iexcl;Feliz %u!",
	"Happy %u!",
	"Felice %u!",
	"Feliz %u!",
	};
const char *Txt_No[Txt_NUM_LANGUAGES] =
	{
	"Nein",
	"No",
	"No",
	"Non",
	"No",
	"N&atilde;o",
	};
const char *Txt_No_assignments[Txt_NUM_LANGUAGES] =
	{
	"Keine Aktivit&auml;ten.",
	"No assignments.",
	"No hay actividades.",
	"Il n'y a pas d'activit&eacute;s.",
	"Non ci sono attivit&agrave;.",
	"N&atilde;o h&aacute; atividades.",
	};
const char *Txt_No_announcements_of_exams_of_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
        "Keine Anrufe f&uuml;r Pr&uuml;fung von <strong>%s</strong>.",
	"No announcements of exams of <strong>%s</strong>.",
	"No hay convocatorias de examen de <strong>%s</strong>.",
        "Aucune convocation &agrave; un examen de <strong>%s</strong>.",
        "Nessuna chiamata per esame di <strong>%s</strong>."
        "N&atilde;o chamadas para exame de <strong>%s</strong>.",
	};
const char *Txt_no_course_degree[Txt_NUM_LANGUAGES] =
       {
	"kein Vorlesung/Grad",
	"no course/degree",
	"ninguna asignatura/titulaci&oacute;n",
	"pas de mati&egrave;re/&eacute;tude",
	"nessuna materia/laurea",
	"nenhuma disciplina/titula&ccedil;&atilde;o",
	};
const char *Txt_No_information_available[Txt_NUM_LANGUAGES] =
	{
        "Keine Informationen verf&uuml;gbar.",
	"No information available.",
	"No hay informaci&oacute;n disponible.",
        "Aucune information disponible.",
        "Nessuna informazione disponibile."
        "N&atilde;o h&aacute; informa&ccedil;&atilde;o dispon&iacute;vel.",
	};
const char *Txt_no_new_messages[Txt_NUM_LANGUAGES] =
	{
	"keine neuen Nachrichten",
	"no new messages",
	"ning&uacute;n mensaje nuevo",
	"pas de nouveaux messages",
	"nessun nuovo messaggIl n'y a pas des avisio",
	"sem mensagens novas",
	};
const char *Txt_No_notices[Txt_NUM_LANGUAGES] =
	{
	"Keine Ank&uuml;ndigungen.",
	"No notices.",
	"No hay avisos.",
	"Il n'y a pas d'avis.",
	"Non ci sono avvisi.",
	"N&atilde;o h&aacute; avisos.",
	};
const char *Txt_no_subject[Txt_NUM_LANGUAGES] =
	{
	"kein Betreff",
	"no subject",
	"sin asunto",
	"pas de sujet",
	"nessun oggetto",
	"sem assunto",
	};
const char *Txt_no_tags[Txt_NUM_LANGUAGES] =	// Tags used in self-assessment test
	{
	"keine Tags",
	"no tags",
	"sin descriptores",
	"pas de descripteurs",
	"senza descrittori",
	"sem descritores",
	};
const char *Txt_No_users_found[Usr_NUM_USR_TYPES][Txt_NUM_LANGUAGES] =
	{
		{	// Usr_UNKNOWN
        	"Keine Benutzer gefunden.",
		"No users found.",
		"No hay usuarios.",
        	"Aucun utilisateur trouv&eacute;.",
        	"No utenti trovati."
        	"Nenhum utilizador encontrado.",
		},
		{	// Usr_INVITED
        	"Keine Eingeladen gefunden.",
		"No invited found.",
		"No hay invitados.",
        	"Aucun invit&eacute; trouv&eacute;.",
        	"No ospiti trovati."
        	"Nenhum convidado encontrado.",
		},
		{	// Usr_STUDENT
        	"Keine Studenten gefunden.",
		"No students found.",
		"No hay alumnos.",
        	"Aucun &eacute;tudiant trouv&eacute;.",
        	"No studenti trovati."
        	"Nenhum estudante encontrado.",
		},
		{	// Usr_TEACHER
        	"Keine Dozenten gefunden.",
		"No teachers found.",
		"No hay profesores.",
        	"Aucun enseignant trouv&eacute;.",
        	"No insegnanti trovati."
        	"Nenhum professor encontrado.",
		},
		{	// Usr_DEG_ADMIN
        	"Keine Administratoren gefunden.",
		"No administrators found.",
		"No hay administradores.",
        	"Aucun administrateur trouv&eacute;.",
        	"No amministratori trovati."
        	"Nenhum administrador encontrado.",
		},
		{	// Usr_SUPERUSER
        	"Keine Superuser gefunden.",
		"No superusers found.",
		"No hay superusuarios.",
        	"Aucun superutilisateur trouv&eacute;.",
        	"No superutenti trovati."
        	"Nenhum superutilizador encontrado.",
		},
	};
const char *Txt_No_INDEX[Txt_NUM_LANGUAGES] =	// Short version of "Number" (as an index)...
	{
	"Anzahl",
	"#",
	"N&ordm;",
	"Nombre",
	"Numero",
	"N&ordm;",
	};
const char *Txt_No_of_courses[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Vorlesungen",
	"No. of courses",
	"N&ordm; de asignat.",
	"Nombre de mati&egrave;res",
	"Numero di materie",
	"N&ordm; de disciplinas",
	};
const char *Txt_No_of_BR_degrees[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der<br />Grad",
	"No. of<br />degrees",
	"N&ordm; de<br />titul.",
	"Nombre<br />d'&eacute;tudes",
	"Numero di<br />lauree",
	"N&ordm; de<br />titul.",
        };
const char *Txt_No_of_files[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Dateien",
	"No. of files",
	"N&ordm; de archivos",
	"Nombre de fichiers",
	"Numero di file",
	"N&ordm; de arquivos",
        };
const char *Txt_No_of_folders[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Verzeichnisse",
	"No. of folders",
	"N&ordm; de carpetas",
	"Nombre de r&eacute;pertoires",
	"Numero di cartelle",
	"N&ordm; de diret&oacute;rios",
        };
const char *Txt_No_of_forums[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Foren",
	"No. of forums",
	"N&ordm; de foros",
	"Nombre de forums",
	"Numero di fori",
	"N&ordm; de f&oacute;runs",
        };
const char *Txt_No_of_BR_groups[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der<br />Gruppen",
	"No. of<br />groups",
	"N&ordm; de<br />grupos",
	"Nombre de<br />groupes",
	"Numero di<br />gruppi",
	"N&ordm; de<br />grupos",
        };
const char *Txt_No_of_BR_lines[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der<br />Zeilen",
	"No. of<br />lines",
	"N&ordm; de<br />l&iacute;neas",
	"Nombre de<br />lignes",
	"Numero di<br />linee",
	"N&ordm; de<br />linhas",
        };
const char *Txt_No_BR_msgs[Txt_NUM_LANGUAGES] =
	{
	"Anzahl<br />Nachr.",
	"No.<br />msgs.",
	"N&ordm;<br />mens.",
	"Nombre<br />msgs.",
	"Num.<br />msgi.",
	"N&ordm;<br />mens.",
        };
const char *Txt_No_of_messages[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Nachrichten",
	"No. of messages",
	"N&ordm; de mensajes",
	"Nombre de messages",
	"Numero di messaggi",
	"N&ordm; de mensagens",
        };
const char *Txt_No_of_messages_BR_per_forum[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Nachrichten<br />pro Forum",
	"No. of messages<br />per forum",
	"N&ordm; de mensajes<br />por foro",
	"Nombre de messages<br />par forum",
	"Numero di messaggi<br />al forum",
	"N&ordm; de mensagens<br />por f&oacute;rum",
        };
const char *Txt_No_of_messages_BR_per_thread[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Nachrichten<br />pro Thread",
	"No. of messages<br />per thread",
	"N&ordm; de mensajes<br />por hebra",
	"Nombre de messages<br />par fil",
	"Numero di messaggi<br />al thread",
	"N&ordm; de mensagens<br />por thread",
        };
const char *Txt_No_of_questions[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Fragen",
	"No. of questions",
	"N&ordm; de preguntas",
	"Nombre de questions",
	"Numero di domande",
	"N&ordm; de quest&otilde;es",
        };
const char *Txt_No_OF_stud[Txt_NUM_LANGUAGES] =
	{
	"Anz.Stud.",
	"No.stud.",
	"N&ordm;alum.",
	"Nom.&eacute;tud.",
	"Num.stud.",
	"N&ordm;estu.",
        };
const char *Txt_No_of_BR_teachers[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der<br />Dozenten",
	"No. of<br />teachers",
	"N&ordm; de<br />profesores",
	"Nombre<br />d'enseignants",
	"Numero di<br />insegnanti",
	"N&ordm; de<br />professores",
        };
const char *Txt_No_of_threads[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Threads",
	"No. of threads",
	"N&ordm; de hebras",
	"Nombre de fils",
	"Numero di thread",
	"N&ordm; de threads",
        };
const char *Txt_No_of_threads_BR_per_forum[Txt_NUM_LANGUAGES] =
	{
	"Anzahl der Threads<br />pro Forum",
	"No. of threads<br />per forum",
	"N&ordm; de hebras<br />por foro",
	"Nombre de fils<br />par forum",
	"Numero di thread<br />al forum",
	"N&ordm; de threads<br />por f&oacute;rum",
        };
const char *Txt_No_of_users[Txt_NUM_LANGUAGES] =
       {
	"Anzahl der Benutzer",
	"No. of users",
	"N&ordm; de usuarios",
	"Nombre d'utilisateurs",
	"Numero di utenti",
	"N&ordm; de utilizadores",
	};
const char *Txt_No_of_users_belonging_to_at_least_one_course[Txt_NUM_LANGUAGES] =
       {
	"Anzahl der Benutzer, die mindestens ein Vorlesung",
	"No. of users belonging to at least one course",
	"N&ordm; de usuarios pertenecientes a alguna asignatura",
	"Nombre d'utilisateurs appartenant au moins &agrave; une mati&egrave;re",
	"Numero di utenti appartenenti ad almeno una materia",
	"N&ordm; de utilizadores que pertencem a pelo menos uma disciplina",
	};
const char *Txt_No_of_users_not_belonging_to_any_course[Txt_NUM_LANGUAGES] =
       {
	"Anzahl der Benutzer ohne Zugeh&ouml;rigkeit zu einem Vorlesung",
	"No. of users not belonging to any course",
	"N&ordm; de usuarios que no pertenecen a ninguna asignatura",
	"Nombre d'utilisateurs n'appartenant pas &agrave; une mati&egrave;re",
	"Numero di utenti che non appartengono ad alcuna materia",
	"N&ordm; de utilizadores que n&atilde;o pertencem a nenhuma disciplina",
	};
const char *Txt_No_of_users_who_will_be_notified_by_e_mail[Txt_NUM_LANGUAGES] =
       {
	"Anzahl der Benutzer, um benachrichtigt zu werden per E-Mail",
	"No. of users who will be notified by e-mail",
	"N&ordm; de usuarios que ser&aacute;n notificados por correo electr&oacute;nico",
	"Nombre d'utilisateurs qui seront notifi&eacute;s par e-mail",
	"Numero di utenti che saranno notificati via e-mail",
	"N&ordm; de utilizadores que ser&atilde;o notificados por e-mail",
	};
const char *Txt_No_questions_found_matching_your_search_criteria[Txt_NUM_LANGUAGES] =
	{
        "Keine Testfragen gefunden, die mit Ihrer Suchanfrage entsprechen.",
	"No questions found matching your search criteria.",
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.",
        "Aucune question trouv&eacute;e correspondant &agrave; vos crit&egrave;res de recherche.",
        "No domande trovati corrispondenti ai tuoi criteri di ricerca."
        "N&atilde;o quest&otilde;es com os seus crit&eacute;rios de pesquisa selecionados.",
	};
const char *Txt_No_test_questions_in_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
        "Keine Testfragen im <strong>%s</strong>.",
	"No test questions in <strong>%s</strong>.",
	"No hay preguntas de test en <strong>%s</strong>.",
        "Il n'y a pas de questions de test dans <strong>%s</strong>.",
        "Non ci sono domande di test in <strong>%s</strong>."
        "N&atilde;o existem quest&otilde;es de test em <strong>%s</strong>.",
	};
const char *Txt_nobody_else_can_access_this_content[Txt_NUM_LANGUAGES] =
	{
	"Niemand sonst kann auf diese Inhalte",
	"nobody else can access this content",
	"nadie m&aacute;s puede acceder a este contenido",
	"personne d'autre ne peut acc&eacute;der &agrave; ce contenu",
	"nessun altro pu&ograve; accedere a questo contenuto",
	"ningu&eacute;m mais pode acessar este conte&uacute;do",
	};
const char *Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_[Txt_NUM_LANGUAGES] =
	{
	"Nothing has been pasted because the clipboard is empty." \
		"Try copy-paste again.",	// Need bersetzung
	"Nothing has been pasted because the clipboard is empty." \
		"Try copy-paste again.",
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o." \
		" Trate de copiar y pegar de nuevo.",
	"Nothing has been pasted because the clipboard is empty." \
		"Try copy-paste again.",	// Besoin de traduction
	"Nothing has been pasted because the clipboard is empty." \
		"Try copy-paste again.",	// Bisogno di traduzione
	"Nothing has been pasted because the clipboard is empty." \
		"Try copy-paste again.",	// Necessita de traduo
	};
const char *Txt_NOTICE_Active_SINGULAR[Txt_NUM_LANGUAGES] =
	{
	"Aktive",
	"Active",
	"Activo",
	"Active",
	"Attivo",
	"Ativo",
	};
const char *Txt_NOTICE_Active_BR_notices[Txt_NUM_LANGUAGES] =
	{
	"Aktive<br />Ank&uuml;ndigungen",
	"Active<br />notices",
	"Avisos<br />activos",
	"Avis<br />actives",
	"Avvisi<br />attivi",
	"Avisos<br />ativos",
	};
const char *Txt_NOTICE_Active_Mark_as_obsolete[Txt_NUM_LANGUAGES] =
	{
	"Aktive; marke als veraltet",
	"Active; mark as obsolete",
	"Activo; marcar como obsoleto",
	"Active; marquer comme obsol&egrave;te",
	"Attivo; marchiare come obsoleto",
	"Ativo; marcar como obsoleto",
	};
const char *Txt_NOTICE_Deleted_BR_notices[Txt_NUM_LANGUAGES] =
	{
	"Gel&ouml;scht<br />Ank&uuml;ndigungen",
	"Deleted<br />notices",
	"Avisos<br />eliminados",
	"Avis<br />supprim&eacute;s",
	"Avvisi<br />eliminati",
	"Avisos<br />suprimidos",
        };
const char *Txt_NOTICE_Obsolete_SINGULAR[Txt_NUM_LANGUAGES] =
	{
	"Veraltet",
	"Obsolete",
	"Obsoleto",
	"Obsol&egrave;te",
	"Obsoleto",
	"Obsoleto",
        };
const char *Txt_NOTICE_Obsolete_BR_notices[Txt_NUM_LANGUAGES] =
	{
	"Veraltet<br />Ank&uuml;ndigungen",
	"Obsolete<br />notices",
	"Avisos<br />obsoletos",
	"Avis<br />obsol&egrave;tes",
	"Avvisi<br />obsoleti",
	"Avisos<br />obsoletos",
        };
const char *Txt_NOTICE_Obsolete_Mark_as_active[Txt_NUM_LANGUAGES] =
	{
	"Veraltet; marke als aktive",
	"Obsolete; mark as active",
	"Obsoleto; marcar como activo",
	"Obsol&egrave;te; marquer comme active",
	"Obsoleto; marchiare come attivo",
	"Obsoleto; marcar como ativo",
        };
const char *Txt_Notice_created[Txt_NUM_LANGUAGES] =
	{
	"Ank&uuml;ndigung erstellt.",
	"Notice created.",
	"Aviso creado.",
	"Avis cr&eacute;&eacute;.",
	"Avviso creato.",
	"Aviso criado.",
	};
const char *Txt_Notices[Txt_NUM_LANGUAGES] =
	{
	"Ank&uuml;ndigungen",
	"Notices",
	"Avisos",
	"Avis",
	"Avvisi",
	"Avisos",
	};
const char *Txt_Notifications[Txt_NUM_LANGUAGES] =
	{
	"Anmeldungen",
	"Notifications",
	"Notificaciones",
	"Notifications",
	"Notifiche",
	"Notifica&ccedil;&otilde;es",
	};
const char *Txt_NOTIFY_EVENTS_There_is_a_new_event_NO_HTML[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{								 	// Don't use HTML entities like &uuml; here.
	"Es ist eine neue Ereignis in %s",	
	"There is a new event in %s",
	"Hay un nuevo evento en %s",
	"Il y un nouvel vnement dans %s",
	"Vi  un nuovo evento in %s",
	"Existe um novo evento em %s",
	};
const char *Txt_NOTIFY_EVENTS_There_are_X_new_events_NO_HTML[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u and %s in the following sentences
	{								 	// Don't use HTML entities like &uuml; here.
	"Es gibt %u neue Ereignisse in %s",	
	"There are %u new events in %s",
	"Hay %u nuevos eventos en %s",
	"Il y a %u nouveaux vnements dans %s",
	"Ci sono %u nuovi eventi in %s",
	"H %u novos eventos em %s",
	};
const char *Txt_NOTIFY_EVENTS[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES] =
	{
		{	// Ntf_NEW_EXAM_ANNOUNCEMENT
       		"Neue Anrufe f&uuml;r Pr&uuml;fung",
		"New announcements of exam",
		"Nuevas convocatorias de examen",
       		"Nouvelles convocations &agrave; un examen",
		"Nuovi chiamate per esame"
		"Novas chamadas para exame",
		},
		{	// Ntf_NEW_MARKS_FILE
		"Neue Dateien mit Noten",
		"New files with marks",
		"Nuevos archivos de calificaciones",
		"Nouveaux fichiers avec des notes",
		"Nuovi file con voti",
		"Novos arquivos de notas",
		},
		{	// Ntf_NEW_NOTICE
		"Neue Ank&uuml;ndigungen",
		"New notices",
		"Nuevos avisos",
		"Nouveaux avis",
		"Nuovi avvisi",
		"Novos avisos",
		},
		{	// Ntf_NEW_MESSAGE
		"Neue Nachrichten",
		"New messages",
		"Nuevos mensajes",
		"Nouveaux messages",
		"Nuovi messaggi",
		"Novas mensagens",
		},
		{	// Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS
		"Neue Antworten auf meine Beitr&auml;ge in den Foren",
		"New responses to my posts in forums",
		"Nuevas respuestas a mis mensajes en foros",
		"Nouvelles r&eacute;ponses &agrave; mes messages dans les forums",
		"Nuove risposte ai miei post nei forum",
		"Novas respostas aos meus posts em f&oacute;runs",
		},
	};
const char *Txt_NOTIFY_EVENTS_MAIL_SUBJECT_NO_HTML[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &uuml; here.
	{
		{	// Ntf_NEW_EXAM_ANNOUNCEMENT
		"Neue Anrufe fr Prfung",
		"New announcement of exam",
		"Nueva convocatoria de examen",
       		"Nouvelle convocation  un examen",
		"Nuova chiamata per esame",
		"Nova chamada para exame",
		},
		{	// Ntf_NEW_MARKS_FILE
		"Neue Datei mit Noten",
		"New file with marks",
		"Nuevo archivo de calificaciones",
		"Nouveau fichier avec des notes",
		"Nuovo file con voti",
		"Novo arquivo de notas",
		},
		{	// Ntf_NEW_NOTICE
		"Neuen Ankndigungen",
		"New notice",
		"Nuevo aviso",
		"Nouvel avis",
		"Nuovo avviso",
		"Novo anncio",
		},
		{	// Ntf_NEW_MESSAGE
		"Neue Nachricht",
		"New message",
		"Nuevo mensaje",
		"Nouveau message",
		"Nuovo messaggio",
		"Nova mensagem",
		},
		{	// Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS
		"Neue Antwort in einem Forum",
		"New response in a forum",
		"Nueva respuesta en un foro",
		"Nouvelle rponse dans un forum",
		"Nuova risposta in un forum",
		"Nova resposta em um frum",
		},
	};
/*
const char *Txt_NOTIFY_EVENTS_MAIL_CONTENT_NO_HTML[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &uuml; here, 
												// Warning: it is very important to include %s in the following sentences
	{
		{	// Ntf_NEW_EXAM_ANNOUNCEMENT
		"Es gibt eine neue Anrufe fr Prfung in %s.",
		"There is a new announcement of exam in %s.",
		"Hay una nueva convocatoria de examen en %s.",
       		"Il y a une nouvelle convocation  un examen dans %s.",
		"Vi  una nuova chiamata per esame in %s.",
		"Existe uma nova chamada para exame em %s.",
		},
		{	// Ntf_NEW_MARKS_FILE
		"Es gibt eine neue Datei mit Noten in %s.",
		"There is a new file with marks in %s.",
		"Hay un nuevo archivo de calificaciones en %s.",
		"Il existe un nouveau fichier avec des notes dans %s.",
		"Vi  un nuovo file con voti in %s.",
		"Existe um novo arquivo de notas em %s.",
		},
		{	// Ntf_NEW_NOTICE
		"Es gibt einen neuen Ankndigungen in %s.",
		"There is a new notice in %s.",
		"Hay un nuevo aviso en %s.",
		"Il y a un nouvel avis dans %s.",
		"Vi  un nuovo avviso in %s.",
		"H um novo anncio em %s.",
		},
		{	// Ntf_NEW_MESSAGE
		"Sie haben eine neue Nachricht in %s.",
		"You have a new message in %s.",
		"Tiene un nuevo mensaje en %s.",
		"Vous avez un nouveau message dans %s.",
		"Hai un nuovo messaggio in %s.",
		"Voc tem uma nova mensagem em %s.",
		},
		{	// Ntf_NEW_RESPONSE_TO_MY_POSTS_IN_FORUMS
		"Es ist eine neue Antwort auf eine Nachricht von Ihnen in einem %s-Forum.",
		"There is a new response to a post of yours in a %s forum.",
		"Hay una nueva respuesta a un mensaje suyo en un foro de %s.",
		"Il ya une nouvelle rponse  un message de vous dans un forum de %s.",
		"C' una nuova risposta a un post di lei in un forum di %s.",
		"Existe uma nova resposta a um posto de voc em um frum de %s.",
		},
	};
*/
const char *Txt_Notify_me_by_e_mail_about[Txt_NUM_LANGUAGES] =
	{
	"Benachrichtigt mich per E-Mail &uuml;ber",
	"Notify me by e-mail about",
	"Avisarme por correo electr&oacute;nico sobre",
	"Avertissez-moi par e-mail sur",
	"Inviate via e-mail sui",
	"Notifique-me por e-mail sobre",
	};
const char *Txt_Number_of_BR_events[Txt_NUM_LANGUAGES] =
  	{
	"Anzahl der<br />Ereignisse",
	"Number of<br />events",
	"N&ordm; de<br />eventos",
	"Nombre<br />d'&eacute;v&eacute;nements",
	"Numero di<br />eventi",
	"N&uacute;mero de<br />eventos",
	};
const char *Txt_Number_of_BR_e_mails[Txt_NUM_LANGUAGES] =
  	{
	"Anzahl der<br />E-Mails",
	"Number of<br />e-mails",
	"N&ordm; de<br />correos",
	"Nombre<br />d'e-mails",
	"Numero di<br />e-mail",
	"N&uacute;mero de<br />e-mails",
	};
const char *Txt_Number_of_BR_notifications[Txt_NUM_LANGUAGES] =
  	{
	"Anzahl der<br />Anmeldungen",
	"Number of<br />notifications",
	"N&ordm; de<br />notificaciones",
	"Nombre de<br />notifications",
	"Numero di<br />notifiche",
	"N&uacute;mero de<br />notifica&ccedil;&otilde;es",
	};
const char *Txt_of_PART_OF_A_TOTAL[Txt_NUM_LANGUAGES] =	// Example: transferred 2 of 3 files
	{
	"von",
	"of",
	"de",
	"de",
	"di",
	"de",
        };
const char *Txt_out_of_PART_OF_A_SCORE[Txt_NUM_LANGUAGES] =	// Example: score is 2 out of 10
	{
	"von",
	"out of",
	"sobre",
	"sur",
	"su",
	"out of",
        };
const char *Txt_Office[Txt_NUM_LANGUAGES] =
	{
	"Raum",
	"Office",
	"Despacho",
	"Bureau",
	"Ufficio",
	"Gabinete",
	};
const char *Txt_one_day[Txt_NUM_LANGUAGES] =
	{
	"ein Tag",
	"one day",
	"un d&iacute;a",
	"un jour",
	"uno giorno",
	"um dia",
	};
const char *Txt_Online_tutoring[Txt_NUM_LANGUAGES] =
	{
	"Virtual Anleitungen",
	"Online tutoring",
	"Tutor&iacute;as virtuales",
	"Tutorat en ligne",
	"Tutoraggio on-line",
	"Tutoria on-line",
	};
const char *Txt_only_students[Txt_NUM_LANGUAGES] =
	{
	", nur Studenten",
	", only students",
	", s&oacute;lo alumnos",
	", seulement &eacute;tudiants",
	", solo studenti",
	", apenas estudantes",
        };
const char *Txt_only_teachers[Txt_NUM_LANGUAGES] =
	{
	", nur Dozenten",
	", only teachers",
	", s&oacute;lo profesores",
	", seulement enseignants",
	", solo insegnanti",
	", apenas professores",
        };
const char *Txt_only_teachers_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
	{
	", nur Dozenten",
	", only teachers",
	", slo profesores",
	", seulement enseignants",
	", solo insegnanti",
	", apenas professores",
        };
const char *Txt_or_you_can_make_a_file_copy_to_the_folder[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
       {
	"...Oder Sie k&ouml;nnen die Kopie einer Datei in den Verzeichnis <strong>%s</strong>:",
	"...or you can make a file copy to the folder <strong>%s</strong>:",
	"...o puede realizar una copia de archivos a la carpeta <strong>%s</strong>:",
	"...ou vous pouvez faire une copie de fichiers vers le r&eacute;pertoire <strong>%s</strong>:",
	"...oppure &egrave; possibile effettuare una copia dei file nella cartella <strong>%s</strong>:",
	"...ou voc&ecirc; pode fazer uma c&oacute;pia de arquivo na diret&oacute;rio <strong>%s</strong>:",
	};
const char *Txt_or_you_can_send_a_new_file_to_the_folder[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
       {
	"...oder senden Sie eine neue Datei in den Verzeichnis <strong>%s</strong>:",
	"...or you can send a new file to the folder <strong>%s</strong>:",
	"...o puede enviar un nuevo archivo a la carpeta <strong>%s</strong>:",
	"...ou vous pouvez envoyer un nouveau fichier dans le r&eacute;pertoire <strong>%s</strong>:",
	"...oppure si pu&ograve; inviare un nuovo file nella cartella <strong>%s</strong>:",
	"...ou voc&ecirc; pode enviar um novo arquivo para o diret&oacute;rio <strong>%s</strong>:",
	};
const char *Txt_Other_centres[Txt_NUM_LANGUAGES] =
	{
	"Andere Zentren",
	"Other centres",
	"Otros centros",
	"Autres centres",
	"Altri centri",
	"Outros centros",
	};
const char *Txt_Other_departments[Txt_NUM_LANGUAGES] =
	{
	"Andere Fachbereiche",
	"Other departments",
	"Otros departamentos",
	"Autres d&eacute;partements",
	"Altri dipartimenti",
	"Outros departamentos",
	};
const char *Txt_Other_institutions[Txt_NUM_LANGUAGES] =
	{
	"Andere Institutionen",
	"Other institutions",
	"Otras instituciones",
	"Autres institutions",
	"Altre istituzioni",
	"Outras institui&ccedil;&otilde;es",
	};
const char *Txt_Other_places[Txt_NUM_LANGUAGES] =
	{
	"Andere Standorte",
	"Other places",
	"Otros lugares",
	"Autres emplacements",
	"Altri localit&agrave;",
	"Outras localiza&ccedil;oes",
	};
const char *Txt_Other_recipients[Txt_NUM_LANGUAGES] =
	{
	"Andere Empf&auml;nger",
	"Other recipients",
	"Otros destinatarios",
	"Autres destinataires",
	"Altri destinatari",
	"Outros destinat&aacute;rios",
        };
const char *Txt_Page[Txt_NUM_LANGUAGES] =
	{
	"Page",
	"Page",
	"P&aacute;g.",
	"Page",
	"Pag.",
	"P&aacute;g.",
	};
const char *Txt_PAGE1_Page_generated_in[Txt_NUM_LANGUAGES] =
	{
	"Page generated in",
	"Page generated in",
	"P&aacute;gina generada en",
	"Page g&eacute;n&eacute;r&eacute;e en",
	"Pagina generata in",
	"P&aacute;gina gerada em",
	};
const char *Txt_PAGE2_and_sent_in[Txt_NUM_LANGUAGES] =
	{
	"gesendet und in",
	"and sent in",
	"y enviada en",
	"et envoy&eacute;e en ",
	"e inviata in",
	"e enviada em",
	};
const char *Txt_Password[Txt_NUM_LANGUAGES] =
	{
	"Passwort",
	"Password",
	"Contrase&ntilde;a",
	"Mot de passe",
	"Password",
	"Senha",
	};
const char *Txt_Paste[Txt_NUM_LANGUAGES] =
	{
	"Einf&uuml;gen",
	"Paste",
	"Pegar",
	"Coller",
	"Incolla",
	"Colar",
	};
const char *Txt_Paste_in[Txt_NUM_LANGUAGES] =
	{
	"Einf&uuml;gen an",
	"Paste in",
	"Pegar en",
	"Coller dans",
	"Incolla in",
	"Colar em",
	};
const char *Txt_PERCENT_of_users[Txt_NUM_LANGUAGES] =
       {
	"% der Benutzer",
	"% of users",
	"% de usuarios",
	"% d'utilisateurs",
	"% di utenti",
	"% de utilizadores",
	};
const char *Txt_Phone[Txt_NUM_LANGUAGES] =
	{
	"Telefon",
	"Phone",
	"Tel&eacute;fono",
	"T&eacute;l&eacute;phone",
	"Telefono",
	"Telefone",
        };
const char *Txt_PHOTO_Your_photo_exists[Txt_NUM_LANGUAGES] =
       {
	"Ihr Foto existiert.",
	"Your photo exists.",
 	"Su fotograf&iacute;a ya existe.",
	"Votre photo existe.",
	"La foto esiste.",
	"A fotografia existe.",
	};
const char *Txt_PHOTO_The_photo_of_X_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
       {
	"Das Foto von %s existiert.",
	"The photo of %s exists.",
 	"La fotograf&iacute;a de %s ya existe.",
	"La photo de %s existe.",
	"La foto di %s esiste.",
	"A fotografia de %s existe.",
	};
const char *Txt_PHOTO_X_has_no_photo_yet[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
       {
	"%s hat noch kein Foto.",
	"%s has no photo yet.",
 	"%s a&uacute;n no tiene fotograf&iacute;a.",
	"%s n'a pas de photo pour le moment.",
	"%s non ha ancora fotografia.",
	"%s n&acirc;o tem nenhuma foto ainda.",
	};
const char *Txt_PHOTO_If_you_send_a_new_picture[Txt_NUM_LANGUAGES] =
       {
	"Wenn Sie ein neues Bild, es wird der aktuelle.",
	"If you send a new picture, it will overwrite the current one.",
 	"Si env&iacute;a una nueva imagen, sobrescribir&aacute; la actual.",
	"Si vous envoyez une nouvelle image, il va remplacer l'actuel.",
	"Se si invia una nuova immagine, si sovrascriver&agrave; quella attuale.",
	"Se voc&ecirc; enviar uma nova imagem, que ir&aacute; substituir a actual.",
	};
const char *Txt_Photo[Txt_NUM_LANGUAGES] =
	{
	"Foto",
	"Photo",
	"Foto",
	"Photo",
	"Foto",
	"Foto",
        };
const char *Txt_Photo_removed[Txt_NUM_LANGUAGES] =
	{
	"Photo removed.",	// Need bersetzung
	"Photo removed.",
	"Fotograf&iacute;a eliminada."
	"Photo removed.",	// Besoin de traduction
	"Photo removed.",	// Bisogno di traduzione
	"Photo removed.",	// Necessita de traduo
	};
const char *Txt_photos[Txt_NUM_LANGUAGES] =
	{
	"Fotos",
	"photos",
	"fotos",
	"photos",
	"foto",
	"fotos",
        };
const char *Txt_Place[Txt_NUM_LANGUAGES] =	// A particular spot or area
	{
	"Standort",
	"Place",
	"Lugar",
	"Emplacement",
	"Luogo",
	"Localiza&ccedil;&atilde;o"
        };
const char *Txt_Place_of_origin[Txt_NUM_LANGUAGES] =
	{
	"Herkunftsort",
	"Place of origin",
	"Lugar de origen",
	"Lieu d'origine",
	"Luogo di origine",
	"Lugar de origem",
        };
const char *Txt_Place_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Place <strong>%s</strong> removed.",	// Need bersetzung
	"Place <strong>%s</strong> removed.",
	"Lugar <strong>%s</strong> eliminado."
	"Place <strong>%s</strong> removed.",	// Besoin de traduction
	"Place <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Place <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Place_unspecified[Txt_NUM_LANGUAGES] =
	{
	"Standort nicht spezifiziert",
	"Place unspecified",
	"Lugar sin especificar",
	"Emplacement non sp&eacute;cifi&eacute;",
	"Luogo non specificato",
	"Localiza&ccedil;&atilde;o indeterminada",
	};
const char *Txt_Places[Txt_NUM_LANGUAGES] =
	{
	"Standorte",
	"Places",
	"Lugares",
	"Emplacements",
	"Localit&grave;",
	"Localiza&ccedil;oes"
        };
const char *Txt_Please_do_not_reply_to_this_automatically_generated_email_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Bitte, antworten Sie nicht auf diese automatisch generierte E-Mail.",
	"Please do not reply to this automatically generated e-mail.",
	"Por favor, no responda a este correo electrnico generado automticamente.",
	"S'il vous plat, ne pas rpondre  ce courrier lectronique gnr automatiquement.",
	"Per favore, non rispondere a questa e-mail generata automaticamente.",
	"Por favor, no responda a este e-mail gerado automaticamente."
        };
const char *Txt_Plugin[Txt_NUM_LANGUAGES] =
	{
	"Plugin",
	"Plugin",
	"Complemento",
	"Plugin",
	"Plugin",
	"Plugin"
	};
const char *Txt_Plugin_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Plugin <strong>%s</strong> removed.",	// Need bersetzung
	"Plugin <strong>%s</strong> removed.",
	"Complemento <strong>%s</strong> eliminado."
	"Plugin <strong>%s</strong> removed.",	// Besoin de traduction
	"Plugin <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Plugin <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Plugins[Txt_NUM_LANGUAGES] =
	{
	"Plugins",
	"Plugins",
	"Complementos",
	"Plugins",
	"Plugins",
	"Plugins"
	};
const char *Txt_Post_removed[Txt_NUM_LANGUAGES] =
	{
	"Post removed.",	// Need bersetzung
	"Post removed.",
	"Mensaje eliminado."
	"Post removed.",	// Besoin de traduction
	"Post removed.",	// Bisogno di traduzione
	"Post removed.",	// Necessita de traduo
	};
const char *Txt_Post_and_thread_removed[Txt_NUM_LANGUAGES] =
	{
	"Post and thread removed.",	// Need bersetzung
	"Post and thread removed.",
	"Mensaje y hebra eliminados."
	"Post and thread removed.",	// Besoin de traduction
	"Post and thread removed.",	// Bisogno di traduzione
	"Post and thread removed.",	// Necessita de traduo
	};
const char *Txt_Private_Photo[Txt_NUM_LANGUAGES] =
	{
	"Private Foto",
	"Private photo",
	"Foto privada",
	"Photo priv&eacute;e",
	"Privata foto",
        "Foto privada",
        };
const char *Txt_Private_storage_zone[Txt_NUM_LANGUAGES] =
        {
	"Private Lagerhaltung Zone",
	"Private storage zone",
	"Zona de almacenamiento privada",
	"Zone de stockage priv&eacute;e",
	"Zona privata di immagazzinaggio",
	"Zona privada de armazenamento",
        };
const char *Txt_private_storage_zone[Txt_NUM_LANGUAGES] =
        {
	"private Lagerhaltung Zone",
	"private storage zone",
	"zona de almacenamiento privada",
	"zone de stockage priv&eacute;e",
	"zona privata di immagazzinaggio",
	"zona privada de armazenamento",
        };
const char *Txt_post[Txt_NUM_LANGUAGES] =
       {
	"Beitr&auml;g",
	"post",
	"mensaje",
	"message",
	"post",
	"post",
	};
const char *Txt_posts[Txt_NUM_LANGUAGES] =
       {
	"Beitr&auml;ge",
	"posts",
	"mensajes",
	"messages",
	"post",
	"posts",
	};
const char *Txt_Public_Photo[Txt_NUM_LANGUAGES] =
	{
	"Foto &ouml;ffentlich",
	"Public photo",
	"Foto p&uacute;blica",
	"Photo public",
	"Pubblica foto",
	"Foto p&uacute;blica"
        };
const char *Txt_Quota_exceeded[Txt_NUM_LANGUAGES] =
	{
	"Kontingent &uuml;berschritten!",
	"Quota Exceeded!",
	"&iexcl;Cuota excedida!",
	"Quota d&eacute;pass&eacute;!",
	"Quota superata!",
	"Quota excedida!",
	};
const char *Txt_ReaBRders[Txt_NUM_LANGUAGES] =
	{
	"Leser",
	"Rea-<br />ders",
	"Lec-<br />tores",
	"Lec-<br />teurs ",
	"Le-<br />ttori",
	"Lei-<br />tores",
	};
const char *Txt_Real_number_between_A_and_B_1[Txt_NUM_LANGUAGES] =
	{
	"Reelle Zahl zwischen",
	"Real number between",
	"N&ordm; real entre",
	"Nombre r&eacute;el entre",
	"Numero reale compreso tra",
	"N&ordm; real entre",
        };
const char *Txt_Real_number_between_A_and_B_2[Txt_NUM_LANGUAGES] =
	{
	"und",
	"and",
	"y",
	"et",
	"e",
	"e",
        };
/* Types and constants related to fields of the students' course records */
const char *Txt_Record_field_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Record field <strong>%s</strong> removed.",	// Need bersetzung
	"Record field <strong>%s</strong> removed.",
	"Campo de ficha <strong>%s</strong> eliminado."
	"Record field <strong>%s</strong> removed.",	// Besoin de traduction
	"Record field <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Record field <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES] =
	{
		{
		"Versteckt",
		"Hidden",
		"Oculto",
		"Cach&eacute;",
		"Nascosto",
		"Oculto",
		},
		{
		"Sichtbar, aber nicht bearbeitet werden",
		"Visible but not editable",
		"Visible pero no editable",
		"Visible mais non modifiable ",
		"Visibile ma non modificabile",
		"Vis&iacute;vel, mas n&atilde;o edit&aacute;vel",
		},
		{
		"Sichtbar und bearbeitet werde",
		"Visible and editable",
		"Visible y editable",
		"Visible et modifiable",
		"Visibile e modificabile",
		"Vis&iacute;vel e edit&aacute;vel",
		},
	};
const char *Txt_RECORD_FIELD_VISIBILITY_MSG[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
		{
		"Von nun an k&ouml;nnen die Studenten nicht das Feld <strong>%s</strong> der Profile.",
		"From now on students can not see the field <strong>%s</strong> of the records.",
		"A partir de ahora los alumnos no podr&aacute;n ver el campo <strong>%s</strong> de las fichas.",
		"A partir de maintenant, les &eacute;tudiants ne peuvent pas voir le champ <strong>%s</strong> de les cartes.",
		"Da ora in poi gli studenti non possono visualizzare il campo <strong>%s</strong> de le schede.",
		"De agora em diante, os alunos n&atilde;o podem ver o campo <strong>%s</strong> dos cart&otilde;es.",
		},
		{
		"Von nun an k&ouml;nnen sich Studenten, aber nicht bearbeiten das Feld <strong>%s</strong> der Profile.",
		"From now on students can view but not edit the field <strong>%s</strong> of the records.",
		"A partir de ahora los alumnos podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas.",
		"A partir de maintenant, les &eacute;tudiants peuvent consulter mais pas modifier le champ <strong>%s</strong> de les cartes.",
		"Da ora in poi gli studenti possono visualizzare ma non modificare il campo <strong>%s</strong> de le schede.",
		"De agora em diante, os alunos podem visualizar, mas n&atilde;o editar o campo <strong>%s</strong> dos cart&otilde;es.",
		},
		{
		"Von nun an k&ouml;nnen die Studenten anzuzeigen und zu bearbeiten das Feld <strong>%s</strong> der Profile.",
		"From now on students can view and edit the field <strong>%s</strong> of the records.",
		"A partir de ahora los alumnos podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas.",
		"A partir de maintenant, les &eacute;tudiants peuvent consulter et modifier le champ <strong>%s</strong> de les cartes.",
		"Da ora in poi gli studenti possono visualizzare e modificare il campo <strong>%s</strong> de le schede.",
		"De agora em diante, os alunos podem visualizar e editar o campo <strong>%s</strong> dos cart&otilde;es.",
		},
	};
const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES] =
	{
		{
		"versteckt",
		"hidden",
		"oculto",
		"cach&eacute;",
		"nascosto",
		"oculto",
		},
		{
		"sichtbar",
		"visible",
		"visible",
		"visible",
		"visibile",
		"vis&iacute;vel",
		},
		{
		"bearbeitet werde",
		"editable",
		"editable",
		"modifiable",
		"modificabile",
		"edit&aacute;vel",
		},
	};
const char *Txt_Recipients[Txt_NUM_LANGUAGES] =
	{
	"Empf&auml;nger",
	"Recipients",
	"Destinatarios",
	"Destinataires",
	"Destinatari",
	"Destinat&aacute;rios",
        };
const char *Txt_Register_me_IN_A_COURSE[Txt_NUM_LANGUAGES] =
	{
	"Anmeldung Sie mir",
	"Register me",
	"Darme de alta",
	"Inscrivez-moi",
	"Registrati me",
	"Registrar me",
	};
const char *Txt_Register_students[Txt_NUM_LANGUAGES] =
	{
	"Anmeldung Studenten",
	"Register students",
	"Dar de alta alumnos",
	"Inscrire &eacute;tudiants",
	"Registrare studenti",
	"Registrar estudantes",
	};
const char *Txt_Register_remove[Txt_NUM_LANGUAGES] =
	{
	"Anmeldung/entfernen",
	"Register/remove",
	"Dar de alta/baja",
	"Inscrire/enlever",
	"Registrare/rimnuovere",
	"Registrar/tirar",
	};
const char *Txt_Register_users_who_are_on_the_list[Txt_NUM_LANGUAGES] =
	{
	"Anmeldung Sie die Benutzer, die auf der Liste",
	"Register the users who are on the list",
	"Dar de alta los usuarios que est&aacute;n en la lista",
	"Inscrire les utilisateurs qui sont sur la liste",
	"Registrare gli utenti che sono in lista",
	"Registrar os utilizadores que est&atilde;o na lista",
	};
const char *Txt_Register_user_IN_A_COURSE[Txt_NUM_LANGUAGES] =
	{
	"Anmeldung Benutzer",
	"Register user",
	"Dar de alta usuario",
	"Inscrire utilisateur",
	"Registrare utente",
	"Registrar utilizador",
	};
const char *Txt_Remove[Txt_NUM_LANGUAGES] =
	{
	"Entfernen",
	"Remove",
	"Eliminar",
	"Supprimer",
	"Rimuovere",
	"Remover",
        };
const char *Txt_Remove_announcement_of_exam[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Anruf f&uuml;r Pr&uuml;fung",
	"Remove announcement of exam",
	"Eliminar convocatoria de examen",
	"Supprimer convocation &agrave; l'examen",
	"Rimuovere invito per l'esame",
	"Remover chamada para o exame",
	};
const char *Txt_Remove_assignment[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Aktivit&auml;t",
	"Remove assignment",
	"Eliminar actividad",
	"Supprimer activit&eacute;",
	"Rimuovere attivit&agrave;",
	"Remover atividade",
	};
const char *Txt_Remove_centre[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Zentrum",
	"Remove centre",
	"Eliminar centro",
	"Supprimer centre",
	"Rimuovere centro",
	"Remover centro",
	};
const char *Txt_Remove_course[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Vorlesung",
	"Remove course",
	"Eliminar asignatura",
	"Supprimer mati&egrave;re",
	"Rimuovere materia",
	"Remover disciplina",
	};
const char *Txt_Remove_degree[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Grad",
	"Remove degree",
	"Eliminar titulaci&oacute;n",
	"Supprimer &eacute;tude",
	"Rimuovere laurea",
	"Remover titula&ccedil;&atilde;",
	};
const char *Txt_Remove_department[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Fachbereich",
	"Remove department",
	"Eliminar departamento",
	"Supprimer d&eacute;partement",
	"Rimuovere dipartimento",
	"Remover departamento",
	};
const char *Txt_Remove_file[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Datei",
	"Remove file",
	"Eliminar archivo",
	"Supprimer fichier",
	"Rimuovere file",
	"Remover arquivo",
	};
const char *Txt_Remove_folder[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Verzeichnis",
	"Remove folder",
	"Eliminar carpeta",
	"Supprimer r&eacute;pertoire",
	"Rimuovere cartella",
	"Remover diret&oacute;rio",
	};
const char *Txt_Remove_group[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Gruppe",
	"Remove group",
	"Eliminar grupo",
	"Supprimer groupe",
	"Rimuovere gruppo",
	"Remover grupo",
	};
const char *Txt_Remove_holiday[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Feiertag",
	"Remove holiday",
	"Eliminar festividad",
	"Supprimer f&ecirc;te",
	"Rimuovere vacanza",
	"Remover f&eacute;ria",
	};
const char *Txt_Remove_institution[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Institution",
	"Remove institution",
	"Eliminar instituci&oacute;n",
	"Supprimer &eacute;tablissement",
	"Rimuovere istituzione",
	"Remover institui&ccedil;&atilde;o",
	};
const char *Txt_Remove_link[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Link",
	"Remove link",
	"Eliminar enlace",
	"Supprimer lien",
	"Rimuovere link",
	"Remover liga&ccedil;&atilde;o",
	};
const char *Txt_Remove_me[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Sie mir",
	"Remove me",
	"Darme de baja",
	"Enlevez-moi",
	"Rimuovi me",
	"Tirar me",
	};
const char *Txt_Remove_message[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Nachricht",
	"Remove message",
	"Eliminar mensaje",
	"Supprimer message",
	"Rimuovere messaggio",
	"Remover mensagem",
	};
const char *Txt_Remove_notice[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Ank&uuml;ndigung",
	"Remove notice",
	"Eliminar aviso",
	"Supprimer avis",
	"Rimuovere avviso",
	"Remover aviso",
	};
const char *Txt_Remove_photo[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Foto",
	"Remove photo",
	"Eliminar foto",
	"Supprimer photo",
	"Rimuovere foto",
	"Remover foto",
        };
const char *Txt_Remove_place[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Standort",
	"Remove place",
	"Eliminar lugar",
	"Supprimer emplacement",
	"Rimuovere localit&grave;",
	"Remover localiza&ccedil;&atilde;o",
	};
const char *Txt_Remove_plugin[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Plugin",
	"Remove plugin",
	"Eliminar complemento",
	"Supprimer plugin",
	"Rimuovere plugin",
	"Remover plugin",
        };
const char *Txt_Remove_post[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Beitr&auml;g",
	"Remove post",
	"Eliminar mensaje",
	"Supprimer message",
	"Rimuovere post",
	"Remover post",
        };
const char *Txt_Remove_question[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Frage",
	"Remove question",
	"Eliminar pregunta",
	"Supprimer question",
	"Rimuovere questione",
	"Remover quest&atilde;o",
        };
const char *Txt_Remove_record_field[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Datenfeld",
	"Remove record field",
	"Eliminar campo de ficha",
	"Supprimer champ de carte",
	"Rimuovere campo di schede",
	"Remover campo de cart&atilde;o",
        };
const char *Txt_Remove_students[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Studenten",
	"Remove students",
	"Dar de baja alumnos",
	"Enlever &eacute;tudiants",
	"Rimnuovere studenti",
	"Tirar estudantes",
	};
const char *Txt_Remove_the_MESSAGE[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Sie die",
	"Remove the",
	"Eliminar el",
	"Supprimer le",
	"Rimuovere il",
	"Remover a",
	};
const char *Txt_Remove_thread[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Thread",
	"Remove thread",
	"Eliminar hebra",
	"Supprimer fil",
	"Rimuovere thread",
	"Remover thread",
        };
const char *Txt_Remove_type_of_degree[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Abschlussart",
	"Remove type of degree",
	"Eliminar tipo de titulaci&oacute;n",
	"Supprimer type d'&eacute;tude",
	"Rimuovere tipo di laurea",
	"Remover tipo de titula&ccedil;&atilde;",
	};
const char *Txt_Remove_type_of_group[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Art der Gruppe",
	"Remove type of group",
	"Eliminar tipo de grupo",
	"Supprimer type de groupe",
	"Rimuovere tipo di gruppo",
	"Remover tipo de grupo",
	};
const char *Txt_Remove_users_who_are_on_the_list[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Sie die Benutzer, die auf der Liste",
	"Remove the users who are on the list",
	"Dar de baja los usuarios que est&aacute;n en la lista",
	"Enlever les utilisateurs qui sont sur la liste",
	"Rimnuovere gli utenti che sono in lista",
	"Tirar os utilizadores que est&atilde;o na lista",
	};
const char *Txt_Remove_users_who_are_not_on_the_list[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste",
	"Remove the users who are <strong>not</strong> on the list",
	"Dar de baja los usuarios que <strong>no</strong> est&aacute;n en la lista",
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste",
	"Rimnuovere gli utenti che <strong>non</strong> sono sulla lista",
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista",
	};
const char *Txt_Remove_users_who_are_not_on_the_list_and_register_users_who_are_on_the_list[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste, und Anmeldung Sie die Benutzer, die auf der Liste",
	"Remove the users who are <strong>not</strong> on the list, and register the users who are on the list",
	"Dar de baja los usuarios que <strong>no</strong> est&aacute;n en la lista, y dar de alta los usuarios que est&aacute;n en la lista",
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste, et inscrire les utilisateurs qui sont sur la liste",
	"Rimnuovere gli utenti che <strong>non</strong> sono sulla lista, e registrare gli utenti che sono in lista",
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista, e registrar os utilizadores que est&atilde;o na lista",
	};
const char *Txt_Remove_the_X_MESSAGES[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Sie die",
	"Remove the",
	"Eliminar los",
	"Supprimer les",
	"Rimuovere le",
	"Remover as",
	};
const char *Txt_Remove_user[Txt_NUM_LANGUAGES] =
	{
	"Entfernen Benutzer",
	"Remove user",
	"Dar de baja usuario",
	"Enlever utilisateur",
	"Rimuovere utente",
	"Tirar utilizador",
	};
const char *Txt_Reply[Txt_NUM_LANGUAGES] =
	{
	"Antworten",
	"Reply",
	"Responder",
	"R&eacute;pondre",
	"Rispondere",
	"Responder",
	};
const char *Txt_Reply_BR_again[Txt_NUM_LANGUAGES] =
	{
	"Antworten<br />wieder",
	"Reply<br />again",
	"Responder<br />otra vez",
	"R&eacute;pondre<br />&agrave; nouveau",
	"Rispondere<br />nuovamente",
	"Responder<br />novamente",
	};
const char *Txt_Restrict_the_statistics_to[Txt_NUM_LANGUAGES] =
	{
	"Beschr&auml;nken Sie die Statistiken auf",
	"Restrict the statistics to",
	"Restringir las estad&iacute;sticas a",
	"Restreindre les statistiques &agrave;",
	"Limita le statistiche a",
	"Restringir as estat&iacute;sticas"
        };
const char *Txt_Retype_new_password[Txt_NUM_LANGUAGES] =
	{
	"Wiederholung neues Passwort",
	"Retype new password",
	"Reescriba la nueva contrase&ntilde;a",
	"Confirmez votre nouveau mot de passe",
	"Riscrivi la nuova password",
	"Redigite a nova senha",
	};
const char *Txt_ROOT_FOLDER_EXTERNAL_NAMES[Brw_NUM_TYPES_FILE_BROWSER][Txt_NUM_LANGUAGES] =
	{
		// Brw_FILE_BRW_UNKNOWN
		{
		"",
		"",
		"",
		"",
		"",
		"",
		},
		// Brw_FILE_BRW_SEE_DOWNLOAD_CRS
		{
		"Download",
		"download",
		"descarga",
		"download",
		"download",
		"download",
		},
		// Brw_FILE_BRW_SEE_MARKS_CRS
		{
		"Noten",
		"marks",
		"calificaciones",
		"notes",
		"voti",
		"notas",
		},
		// Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS
		{
		"Download",
		"download",
		"descarga",
		"download",
		"download",
		"download",
		},
		// Brw_FILE_BRW_COMMON_CRS
		{
		"Gemeinsamer",
		"common",
		"comun",
		"commun",
		"comune",
		"comum",
		},
		// Brw_FILE_BRW_COMMON_GRP
		{
		"Gemeinsamer",
		"common",
		"comun",
		"commun",
		"comune",
		"comum",
		},
		// Brw_FILE_BRW_WORKS_USR
		{
		"Hausarbeiten",
		"works",
		"trabajos",
		"travaux",
		"lavori",
		"trabalhos",
		},
		// Brw_FILE_BRW_WORKS_CRS
		{
		"Hausarbeiten",
		"works",
		"trabajos",
		"travaux",
		"lavori",
		"trabalhos",
		},
		// Brw_FILE_BRW_ADMIN_MARKS_CRS
		{
		"Noten",
		"marks",
		"calificaciones",
		"notes",
		"voti",
		"notas",
		},
		// Brw_FILE_BRW_BRIEFCASE_USR
		{
		"Aktentasche",
		"briefcase",
		"maletin",
		"porte-documents",
		"cartella",
		"pasta",
		},
		// Brw_FILE_BRW_SEE_DOWNLOAD_GRP
		{
		"Download",
		"download",
		"descarga",
		"download",
		"download",
		"download",
		},
		// Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP
		{
		"Download",
		"download",
		"descarga",
		"download",
		"download",
		"download",
		},
		// Brw_FILE_BRW_SEE_MARKS_GRP
		{
		"Noten",
		"marks",
		"calificaciones",
		"notes",
		"voti",
		"notas",
		},
		// Brw_FILE_BRW_ADMIN_MARKS_GRP
		{
		"Noten",
		"marks",
		"calificaciones",
		"notes",
		"voti",
		"notas",
		},
		// Brw_FILE_BRW_ASSIGNMENTS_USR
		{
		"Aktivitaten",
		"assignments",
		"actividades",
		"activites",
		"attivita",
		"atividades",
		},
		// Brw_FILE_BRW_ASSIGNMENTS_CRS
		{
		"Aktivitaten",
		"assignments",
		"actividades",
		"activites",
		"attivita",
		"atividades",
		},
	};
const char *Txt_Score[Txt_NUM_LANGUAGES] =
	{
	"Ergebnis",
	"Score",
	"Puntuaci&oacute;n",
	"Score",
	"Punteggio",
	"Pontua&ccedil;&atilde;o",
        };
const char *Txt_Search[Txt_NUM_LANGUAGES] =
	{
	"Suchen",
	"Search",
	"Buscar",
	"Chercher",
	"Cercare",
	"Procurar",
        };
const char *Txt_Search_course[Txt_NUM_LANGUAGES] =
	{
	"Suchen Vorlesung",
	"Search course",
	"Buscar asignatura",
	"Chercher mati&egrave;re",
	"Cercare materia",
	"Procurar disciplina",
        };
const char *Txt_SEARCH_Courses_of_DEG_whose_names_contain_X_taught_by_teachers_whose_names_contain_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include three %s in the following sentences
	{
	"Vorlesungen von <strong>%s</strong> deren Namen enthalten <strong>%s</strong>, unterrichtet von Dozenten deren Namen enthalten <strong>%s</strong>:",
	"Courses of <strong>%s</strong> whose names contain <strong>%s</strong>, taught by teachers whose names contain <strong>%s</strong>:",
	"Asignaturas de <strong>%s</strong> cuyos nombres contienen <strong>%s</strong>, impartidas por profesores cuyos nombres contienen <strong>%s</strong>:",
	"Mati&egrave;res de <strong>%s</strong> dont les noms contiennent <strong>%s</strong>, enseign&eacute;es par des enseignants dont les noms contiennent <strong>%s</strong>:",
	"Materie di <strong>%s</strong> i cui nomi contengono <strong>%s</strong>, insegnata da insegnanti i cui nomi contengono <strong>%s</strong>:",
	"Disciplinas de <strong>%s</strong> cujos nomes contenham <strong>%s</strong>, ministradas por professores cujos nomes contenham <strong>%s</strong>:",
	};
const char *Txt_SEARCH_Courses_of_DEG_whose_names_contain_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
	"Vorlesungen von <strong>%s</strong> deren Namen enthalten <strong>%s</strong>:",
	"Courses of <strong>%s</strong> whose names contain <strong>%s</strong>:",
	"Asignaturas de <strong>%s</strong> cuyos nombres contienen <strong>%s</strong>:",
	"Mati&egrave;res de <strong>%s</strong> dont les noms contiennent <strong>%s</strong>:",
	"Materie di <strong>%s</strong> i cui nomi contengono <strong>%s</strong>:",
	"Disciplinas de <strong>%s</strong> cujos nomes contenham <strong>%s</strong>:",
	};
const char *Txt_SEARCH_Teachers_whose_names_contain_X_and_teach_courses_of_DEG[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
	"Dozenten deren Namen enthalten <strong>%s</strong> und lehren Kurse von <strong>%s</strong>:",
	"Teachers whose names contain <strong>%s</strong> and teach courses of <strong>%s</strong>:",
	"Profesores cuyos nombres contienen <strong>%s</strong> e imparten asignaturas de <strong>%s</strong>:",
	"Enseignants dont les noms contiennent <strong>%s</strong> et enseignent des mati&egrave;res de <strong>%s</strong>:",
	"Insegnanti i cui nomi contengono <strong>%s</strong> e insegnano materie di <strong>%s</strong>:",
	"Professores cujos nomes contenham <strong>%s</strong> e leccionam disciplinas de <strong>%s</strong>:",
	};
const char *Txt_SEARCH_Courses_of_DEG[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Vorlesungen von <strong>%s</strong>:",
	"Courses of <strong>%s</strong>:",
	"Asignaturas de <strong>%s</strong>:",
	"Mati&egrave;res de <strong>%s</strong>:",
	"Materie di <strong>%s</strong>:",
	"Disciplinas de <strong>%s</strong>:",
	};
const char *Txt_See_page_X_of_Y[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %u in the following sentences
	{
	"Siehe Seite %u von %u",
	"See page %u of %u",
	"Ver p&aacute;gina %u de %u",
	"Voir page %u de %u",
	"Vedi pagina %u di %u",
	"Veja p&aacute;gina %u de %u",
	};
const char *Txt_Self_assessment_test[Txt_NUM_LANGUAGES] =
	{
	"Test erstellen",
	"Self-assessment test",
	"Test de autoevaluaci&oacute;n",
	"Test d'auto&eacute;valuation",
	"Test di autovalutazione",
	"Teste de autoavalia&ccedil;&atilde;o",
        };
const char *Txt_Semester[Txt_NUM_LANGUAGES] =
	{
	"Semester",
	"Semester",
	"Cuatrim.",
	"Semestre",
	"Semestre",
	"Semestre",
        };
const char *Txt_Send[Txt_NUM_LANGUAGES] =
	{
	"Senden",
	"Send",
	"Enviar",
	"Envoyer",
	"Invia",
	"Enviar",
        };
const char *Txt_Send_announcement_of_exam[Txt_NUM_LANGUAGES] =
	{
	"Senden Anruf f&uuml;r Pr&uuml;fung",
	"Send announcement of exam",
	"Enviar convocatoria de examen",
	"Envoyer convocation &agrave; l'examen",
	"Invia invito per l'esame",
	"Enviar chamada para o exame",
	};
const char *Txt_Send_message[Txt_NUM_LANGUAGES] =
	{
	"Senden Nachricht",
	"Send message",
	"Enviar mensaje",
	"Envoyer message",
	"Invia messaggio",
	"Enviar mensagem",
	};
const char *Txt_Send_notice[Txt_NUM_LANGUAGES] =
	{
	"Senden Ank&uuml;ndigung",
	"Send notice",
	"Enviar aviso",
	"Envoyer avis",
	"Invia avviso",
	"Enviar aviso",
	};
const char *Txt_Send_URL[Txt_NUM_LANGUAGES] =
	{
	"Senden URL",
	"Send URL",
	"Enviar URL",
	"Envoyer URL",
	"Invia URL",
	"Enviar URL",
        };
const char *Txt_session[Txt_NUM_LANGUAGES] =
	{
	"Session",
	"session",
	"sesi&oacute;n",
	"session",
	"sessione",
	"sess&atilde;o",
        };
const char *Txt_sessions[Txt_NUM_LANGUAGES] =
	{
	"Sessions",
	"sessions",
	"sesiones",
	"sessions",
	"sessioni",
	"sess&otilde;es",
        };
const char *Txt_Short_Name[Txt_NUM_LANGUAGES] =
	{
	"Kurzname",
	"Short name",
	"Nombre breve",
	"Nom abr&eacute;g&eacute;",
	"Nome breve",
	"Nome abreviado",
	};
const char *Txt_Short_BR_Name[Txt_NUM_LANGUAGES] =
	{
	"Kurzname",
	"Short<br />name",
	"Nombre<br />breve",
	"Nom<br />abr&eacute;g&eacute;",
	"Nome<br />breve",
	"Nome<br />abreviado",
	};
const char *Txt_shortcut[Txt_NUM_LANGUAGES] =
	{
	"Verkn&uuml;pfung",
	"shortcut",
	"acceso directo",
	"acc&egrave;s direct",
	"accesso diretto",
	"acesso directo",
	};
const char *Txt_Show[Txt_NUM_LANGUAGES] =
	{
	"Zeige",
	"Show",
	"Mostrar",
	"Voir",
	"Visualizza",
	"Mostrar",
        };
const char *Txt_Show_all_data[Txt_NUM_LANGUAGES] =
	{
	"Alle Daten",
	"Show all data",
	"Mostrar todos los datos",
	"Voir toutes les donn&eacute;es",
	"Visualizza tutti i dati",
	"Mostrar todos os dados",
        };
const char *Txt_Show_all_files[Txt_NUM_LANGUAGES] =
	{
	"Alle Dateien anzeigen",
	"Show all files",
	"Mostrar todos los archivos",
	"Afficher tous les fichiers",
	"Mostra tutti i file",
	"Mostrar todos os arquivos",
        };
const char *Txt_Show_only_unread_messages[Txt_NUM_LANGUAGES] =
	{
	"Zeige nur ungelesene Nachrichten",
	"Show only unread messages",
	"Mostrar &uacute;nicamente los mensajes no le&iacute;dos",
	"Afficher uniquement les messages non lus",
	"Mostra solo i messaggi non letti",
	"Mostrar apenas mensagens n&atilde;o lidas",
	};
const char *Txt_Show_WHICH_groups[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Zeige nur meine Gruppen",
		"Show only my groups",
		"Mostrar s&oacute;lo mis grupos",
		"Afficher uniquement mes groupes",
		"Mostra solo i miei gruppi",
		"Mostrar apenas os meus grupos",
		},
		{
		"Zeige alle Gruppen",
		"Show all groups",
		"Mostrar todos los grupos",
		"Afficher tous les groupes",
		"Mostra tutti i gruppi",
		"Mostrar todos os grupos",
		},
	};
const char *Txt_Show_print_view[Txt_NUM_LANGUAGES] =
	{
	"Druckansicht",
	"Show print view",
	"Mostrar vista de impresi&oacute;n",
	"Voir la vue d'impression",
	"Visualizza la stampa",
	"Visualizar impress&atilde;o vista",
        };
const char *Txt_Show_questions[Txt_NUM_LANGUAGES] =
	{
	"Zeige Testfragen",
	"Show questions",
	"Mostrar preguntas",
	"Voir les questions",
	"Visualizza le domande",
	"Mostrar quest&otilde;es",
        };
const char *Txt_Show_records[Txt_NUM_LANGUAGES] =
	{
	"Zeige Profile",
	"Show records",
	"Mostrar fichas",
	"Voir les cartes",
	"Visualizza le schede",
	"Mostrar cart&otilde;es",
        };
const char *Txt_Show_statistic[Txt_NUM_LANGUAGES] =
	{
	"Zeige Statistik",
	"Show statistic",
	"Mostrar estad&iacute;stica",
	"Voir statistique",
	"Visualizza statistica",
	"Mostrar estat&iacute;stica",
        };
const char *Txt_Show_statistics[Txt_NUM_LANGUAGES] =
	{
	"Zeige Statistiken",
	"Show statistics",
	"Mostrar estad&iacute;sticas",
	"Voir statistiques",
	"Visualizza statistiche",
	"Mostrar estat&iacute;sticas",
        };
const char *Txt_Show_visits[Txt_NUM_LANGUAGES] =
	{
	"Zeige Anmeldungen",
	"Show visits",
	"Ver accesos",
	"Voir visites",
	"Visualizza visite",
	"Mostrar visitas",
        };
const char *Txt_Size_of_photos[Txt_NUM_LANGUAGES] =
	{
	"Gr&ouml;&szlig;e des Fotos",
	"Size of photos",
	"Tama&ntilde;o de las fotos",
	"Taille des photos",
	"Dimensioni della foto",
	"Tamanho das fotos",
        };
const char *Txt_Shuffle[Txt_NUM_LANGUAGES] =
	{
	"Shuffle",
	"Shuffle",
	"Barajar",
	"M&eacute;langer",
	"Mischiare",
	"Baralhar",
        };
const char *Txt_Size[Txt_NUM_LANGUAGES] =	// ...of a file zone
	{
	"Gr&ouml;&szlig;e",
	"Size",
	"Tama&ntilde;o",
	"Taille",
	"Dimensione",
	"Tamanho",
        };
const char *Txt_Source_of_information[Txt_NUM_LANGUAGES] =
	{
	"Quelle von Informationen",
	"Source of information",
	"Fuente de informaci&oacute;n",
	"Source d'information",
	"Fonte delle informazioni",
	"Fonte de informa&ccedil;&atilde;o",
        };
const char *Txt_STA_CLICK_STAT_TYPES[Sta_NUM_TYPES_CLICK_STATS][Txt_NUM_LANGUAGES] =
	{
		{
		"Zeige detaillierte Zugang",
		"Show detailed access",
		"Mostrar accesos detallados",
		"Afficher acc&egrave;s d&eacute;taill&eacute;es",
		"Visualizza dettaglio di accessi",
		"Mostrar detalhadamente acessos",
		},
		{
		"Benutzer",
		"user",
		"usuario",
		"utilisateur",
		"utente",
		"utilizador",
		},
		{
		"Tag",
		"day",
		"d&iacute;a",
		"jour",
		"giorno",
		"dia",
		},
		{
		"Tag und Stunde",
		"day and hour",
		"d&iacute;a y hora",
		"jour et heure",
		"giorno e ora",
		"dia e hora",
		},
		{
		"Woche",
		"week",
		"semana",
		"semaine",
		"settimana",
		"semana",
		},
		{
		"Monat",
		"month",
		"mes",
		"mois",
		"mese",
		"m&ecirc;s",
		},
		{
		"Stunde",
		"hour",
		"hora",
		"heure",
		"ora",
		"hora",
		},
		{
		"Minute",
		"minute",
		"minuto",
		"minute",
		"minuto",
		"minuto",
		},
		{
		"Aktion",
		"action",
		"acci&oacute;n",
		"action",
		"azione",
		"a&ccedil;&atilde;o",
		},
		{
		"Tag",
		"day",
		"d&iacute;a",
		"jour",
		"giorno",
		"dia",
		},
		{
		"Tag und Stunde",
		"day and hour",
		"d&iacute;a y hora",
		"jour et heure",
		"giorno e ora",
		"dia e hora",
		},
		{
		"Woche",
		"week",
		"semana",
		"semaine",
		"settimana",
		"semana",
		},
		{
		"Monat",
		"month",
		"mes",
		"mois",
		"mese",
		"m&ecirc;s",
		},
		{
		"Stunde",
		"hour",
		"hora",
		"heure",
		"ora",
		"hora",
		},
		{
		"Minute",
		"minute",
		"minuto",
		"minute",
		"minuto",
		"minuto",
		},
		{
		"Aktion",
		"action",
		"acci&oacute;n",
		"action",
		"azione",
		"a&ccedil;&atilde;o",
		},
		{
		"Grad",
		"degree",
		"titulaci&oacute;n",
		"&eacute;tude",
		"laurea",
		"titula&ccedil;&atilde;o",
		},
		{
		"Vorlesung",
		"course",
		"asignatura",
		"mati&egrave;re",
		"materia",
		"disciplina",
		}
	};
const char *Txt_STA_USE_STAT_TYPES[Sta_NUM_TYPES_USE_STATS][Txt_NUM_LANGUAGES] =
	{
		{
		"Studienabschl&uuml;sse und Vorlesungen",
		"degrees and courses",
		"titulaciones y asignaturas",
		"&eacute;tudes et mati&egrave;res",
		"laurea e materie",
		"titula&ccedil;oes e disciplinas",
		},
		{
		"Benutzer",
		"users",
		"usuarios",
		"utilisateurs",
		"utenti",
		"utilizadores",
		},
		{
		"Verzeichnisse und Dateien",
		"folders and files",
		"carpetas y archivos",
		"r&eacute;pertoires et fichiers",
		"cartelle e file",
		"diret&oacute;rios e arquivos",
		},
		{
		"Nachrichten",
		"messages",
		"mensajes",
		"messages",
		"messaggi",
		"mensagens",
		},
		{
		"Foren",
		"forums",
		"foros",
		"forums",
		"fori",
		"f&oacute;runs",
		},
		{
		"Ank&uuml;ndigungen",
		"notices",
		"avisos",
		"avis",
		"avvisi",
		"avisos",
		},
		{
		"Sprache",
		"language",
		"idioma",
		"langue",
		"lingua",
		"lingua",
		},
		{
		"Design",
		"design",
		"dise&ntilde;o",
		"conception",
		"design",
		"design",
		},
		{
		"Spalten",
		"columns",
		"columnas",
		"colonnes",
		"colonne",
		"colunas",
		},
		{
		"Anmeldungen",
		"notifications",
		"notificaciones",
		"notifications",
		"notifiche",
		"notifica&ccedil;&otilde;es",
		},
	};
const char *Txt_Start_date[Txt_NUM_LANGUAGES] =
	{
	"Startdatum",
	"Start date",
	"Fecha inicial",
	"Date initiale",
	"Data iniziale",
	"Data de in&iacute;cio",
        };
const char *Txt_Stem[Txt_NUM_LANGUAGES] =	// Stem of a self-assessment question
	{
	"Stamm",
	"Stem",
	"Enunciado",
	"Question",
	"Domanda",
	"Quest&atilde;o",
        };
const char *Txt_Stud[Txt_NUM_LANGUAGES] =
	{
	"Stud.",
	"Stud.",
	"Alum.",
	"&Eacute;tud.",
	"Stud.",
	"Estu.",
        };
const char *Txt_stud[Txt_NUM_LANGUAGES] =
	{
	"Stud.",
	"stud.",
	"alum.",
	"&eacute;tud.",
	"stud.",
	"estu.",
        };
const char *Txt_X_students_who_have_e_mail[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u and %f in the following sentences
	{
	"%u Studenten mit E-Mail (%.1f%% von %u Studenten)",
	"%u students who have e-mail (%.1f%% of %u students)",
	"%u alumnos con correo electr&oacute;nico (%.1f%% de %u alumnos)",
	"%u &eacute;tudiants qui ont l'e-mail (%.1f%% de %u &eacute;tudiants)",
	"%u studenti che hanno e-mail (%.1f%% di %u studenti)",
	"%u estudantes que t&ecirc;m e-mail (%.1f%% dos %u estudantes)",
        };
const char *Txt_X_students_who_have_accepted_and_who_have_e_mail[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u and %f in the following sentences
	{
	"%u Studenten die akzeptiert wurden und die E-Mail (%.1f%% von %u Studenten)",
	"%u students who have accepted and who have e-mail (%.1f%% of %u students)",
	"%u alumnos que han aceptado el alta y que tienen correo electr&oacute;nico (%.1f%% de %u alumnos)",
	"%u &eacute;tudiants qui ont accept&eacute; et qui ont l'e-mail (%.1f%% de %u &eacute;tudiants)",
	"%u studenti che hanno aderito e che hanno e-mail (%.1f%% di %u studenti)",
	"%u estudantes que aceitaram e que t&ecirc;m e-mail (%.1f%% dos %u estudantes)",
        };
const char *Txt_students_with_no_group[Txt_NUM_LANGUAGES] =
	{
	"Studente mit nicht-Gruppe",
	"students with no group",
	"alumnos en ning&uacute;n grupo",
	"&eacute;tudiants avec aucun groupe",
	"studenti con nessun gruppo",
	"estudantes com nenhum grupo",
        };
const char *Txt_Surname_1[Txt_NUM_LANGUAGES] =
	{
	"Nachname 1",
	"Surname 1",
	"Primer apellido",
	"Nom de famille 1",
	"Primo cognome",
	"Primeiro apelido",
        };
const char *Txt_Surname_2[Txt_NUM_LANGUAGES] =
	{
	"Nachname 2",
	"Surname 2",
	"Segundo apellido",
	"Nom de famille 2",
	"Secondo cognome",
	"Segundo apelido",
        };
const char *Txt_Syllabus[Txt_NUM_LANGUAGES] =
	{
	"Studienplan",
	"Syllabus",
	"Programa",
	"Plan",
	"Programmaz.",
	"Programa",
	};
const char *Txt_Syllabus_lectures[Txt_NUM_LANGUAGES] =
	{
	"Studienplan (Vorlesung)",
	"Syllabus (lectures)",
	"Programa teor&iacute;a",
	"Plan th&eacute;orie",
	"Programmaz. classi teoriche",
	"Programa teoria",
	};
const char *Txt_Syllabus_practices[Txt_NUM_LANGUAGES] =
	{
	"Studienplan (&Uuml;bungen)",
	"Syllabus (practicals)",
	"Programa pr&aacute;cticas",
	"Plan pratique",
	"Programmaz. classi pratiche",
	"Programa pr&aacute;tica",
	};
const char *Txt_TABLE_Footer[Txt_NUM_LANGUAGES] =
	{
	"Fu&szlig;",
	"Footer",
	"Pie",
	"Pied",
	"Piede",
	"P&eacute;",
	};
const char *Txt_TABLE_Header[Txt_NUM_LANGUAGES] =
	{
	"Header",
	"Header",
	"Cabecera",
	"T&ecirc;te",
	"Intestazione",
	"Cabe&ccedil;alho",
	};
const char *Txt_Tags[Txt_NUM_LANGUAGES] =	// Tags used in self-assessment test
	{
	"Tags",
	"Tags",
	"Descriptores",
	"Descripteurs",
	"Descrittori",
	"Descritores",
	};
const char *Txt_Teach[Txt_NUM_LANGUAGES] =
	{
	"Dozen.",
	"Teach.",
	"Prof.",
	"Enseig.",
	"Inseg.",
	"Prof.",
	};
const char *Txt_Teaching_guide[Txt_NUM_LANGUAGES] =
	{
	"Teaching Guide",
	"Teaching guide",
	"Gu&iacute;a docente",
	"Guide p&eacute;dagogique",
	"Guida didattica",
	"Ensino de guia",
	};
const char *Txt_Test_result[Txt_NUM_LANGUAGES] =
	{
	"Testergebnis",
	"Test result",
	"Resultado del test",
	"R&eacute;sultat du test",
	"Risultato del test",
	"Resultado do teste",
        };
const char *Txt_Test_No_X_that_you_make_in_this_course[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u in the following sentences
	{
	"Test Nr. %u dass Sie sich in diesem Vorlesung",
	"Test No. %u that you make in this course",
	"Test n&ordm; %u que realiza usted en esta asignatura",
	"Test n&ordm; %u que vous faites dans cette mati&egrave;re",
	"Test n. %u che voi fa in questa materia",
	"Teste n&ordm; %u que voc&ecirc; faz nesta disciplina",
        };
const char *Txt_Assignment_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Aktivit&auml;t <strong>%s</strong> entfernt.",
	"Assignment <strong>%s</strong> removed.",
	"Actividad <strong>%s</strong> eliminada.",
	"Activit&eacute; <strong>%s</strong> supprim&eacute;e.",
	"Attivit&agrave; <strong>%s</strong> rimossa.",
	"Atividade <strong>%s</strong> removida.",
        };
const char *Txt_The_assignment_has_been_modified[Txt_NUM_LANGUAGES] =
	{
	"Die Aktivit&auml;t wurde ge&auml;ndert.",
	"The assignment has been modified.",
	"La actividad ha sido modificada.",
	"L'activit&eacute; a &eacute;t&eacute; modifi&eacute;e.",
	"L'attivit&agrave; &egrave; stata modificata.",
	"A atividade foi modificada.",
	};
const char *Txt_The_copy_has_been_successful[Txt_NUM_LANGUAGES] =
	{
        "Die Kopie wurde erfolgreich.",
	"The copy has been successful.",
	"La copia se ha realizado con &eacute;xito.",
	"La copie a &eacute;t&eacute; un succ&egrave;s.",
	"La copia &egrave; stata un successo.",
	"A c&oacute;pia foi feita com sucesso.",
	};
const char *Txt_The_centre_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The centre <strong>%s</strong> already exists.",	// Need bersetzung
	"The centre <strong>%s</strong> already exists.",
	"El centro <strong>%s</strong> ya existe.",
	"The centre <strong>%s</strong> already exists.",	// Besoin de traduction
	"The centre <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The centre <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Need bersetzung
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the disk quota.",
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>" \
		" porque se exceder&iacute;a la cuota de disco.",
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Besoin de traduction
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Bisogno di traduzione
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Necessita de traduo
	};
const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_disk_quota[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The copy has stopped when trying to paste the folder <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Need bersetzung
	"The copy has stopped when trying to paste the folder <strong>%s</strong>" \
		" because it would exceed the disk quota.",
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>" \
		" porque se exceder&iacute;a la cuota de disco.",
	"The copy has stopped when trying to paste the folder <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Besoin de traduction
	"The copy has stopped when trying to paste the folder <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Bisogno di traduzione
	"The copy has stopped when trying to paste the folder <strong>%s</strong>" \
		" because it would exceed the disk quota.",	// Necessita de traduo
	};
const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_maximum_allowed_number_of_levels[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Need bersetzung
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>" \
		" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.",
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Besoin de traduction
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Bisogno di traduzione
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Necessita de traduo
	};
const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_maximum_allowed_number_of_levels[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Need bersetzung
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>" \
		" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.",
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Besoin de traduction
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Bisogno di traduzione
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because it would exceed the maximum allowed number of levels.",	// Necessita de traduo
	};
const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_there_is_already_another_file_with_that_name[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because there is already a file with that name.",	// Need bersetzung
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because there is already a file with that name.",
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>" \
		" porque ya existe otro archivo con ese nombre.",
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because there is already a file with that name.",	// Besoin de traduction
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because there is already a file with that name.",	// Bisogno di traduzione
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because there is already a file with that name.",	// Necessita de traduo
	};
const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because you can not paste a file here of a type other than HTML.",	// Need bersetzung
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because you can not paste a file here of a type other than HTML.",
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>" \
		" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.",
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because you can not paste a file here of a type other than HTML.",	// Besoin de traduction
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because you can not paste a file here of a type other than HTML.",	// Bisogno di traduzione
	"The copy has stopped when trying to paste the file <strong>%s</strong>" \
		" because you can not paste a file here of a type other than HTML.",	// Necessita de traduo
	};
const char *Txt_The_course_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The course <strong>%s</strong> already exists.",	// Need bersetzung
	"The course <strong>%s</strong> already exists.",
	"La asignatura <strong>%s</strong> ya existe.",
	"The course <strong>%s</strong> already exists.",	// Besoin de traduction
	"The course <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The course <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_course_X_already_exists_in_year_Y[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
	"The course <strong>%s</strong> already exists in year %s.",	// Need bersetzung
	"The course <strong>%s</strong> already exists in year %s.",
	"La asignatura <strong>%s</strong> ya existe en el curso %s.",
	"The course <strong>%s</strong> already exists in year %s.",	// Besoin de traduction
	"The course <strong>%s</strong> already exists in year %s.",	// Bisogno di traduzione
	"The course <strong>%s</strong> already exists in year %s.",	// Necessita de traduo
	};
const char *Txt_The_degree_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The degree <strong>%s</strong> already exists.",	// Need bersetzung
	"The degree <strong>%s</strong> already exists.",
	"La titulaci&oacute;n <strong>%s</strong> ya existe.",
	"The degree <strong>%s</strong> already exists.",	// Besoin de traduction
	"The degree <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The degree <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_department_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The department <strong>%s</strong> already exists.",	// Need bersetzung
	"The department <strong>%s</strong> already exists.",
	"El departamento <strong>%s</strong> ya existe.",
	"The department <strong>%s</strong> already exists.",	// Besoin de traduction
	"The department <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The department <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_file_X_has_an_extension_not_allowed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The file <strong>%s</strong> has an extension not allowed.",	// Need bersetzung
	"The file <strong>%s</strong> has an extension not allowed.",
	"El archivo <strong>%s</strong> tiene una extensi&oacute;n</a> no permitida.",
	"The file <strong>%s</strong> has an extension not allowed.",	// Besoin de traduction
	"The file <strong>%s</strong> has an extension not allowed.",	// Bisogno di traduzione
	"The file <strong>%s</strong> has an extension not allowed.",	// Necessita de traduo
	};
const char *Txt_The_file_X_has_been_placed_inside_the_folder_Y[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
        "Die Datei <strong>%s</strong> wurde innerhalb des Verzeichnis <strong>%s</strong>.",
	"The file <strong>%s</strong> has been placed inside the folder <strong>%s</strong>.",
	"El archivo <strong>%s</strong> se ha colocado dentro de la carpeta <strong>%s</strong>.",
	"Le fichier <strong>%s</strong> a &eacute;t&eacute; plac&eacute; dans le r&eacute;pertoire <strong>%s</strong>.",
	"Il file <strong>%s</strong> &egrave; stato messo all'interno della cartella <strong>%s</strong>.",
	"O arquivo <strong>%s</strong> foi colocado dentro do diret&oacute;rio <strong>%s</strong>.",
	};
const char *Txt_The_folder_X_has_been_created_inside_the_folder_Y[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
        "Die Verzeichnis <strong>%s</strong> erstellt wurde in den Verzeichnis <strong>%s</strong>.",
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.",
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.",
	"Le r&eacute;pertoire <strong>%s</strong> a &eacute;t&eacute; cr&eacute;&eacute; dans le r&eacute;pertoire <strong>%s</strong>.",
	"La cartella <strong>%s</strong> &egrave; stato creata all'interno della cartella <strong>%s</strong>.",
	"O diret&oacute;rio <strong>%s</strong> foi criado dentro do diret&oacute;rio <strong>%s</strong>.",
	};
const char *Txt_The_folder_name_X_has_changed_to_Y[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
        "The folder name <strong>%s</strong> has changed to <strong>%s</strong>.",	// Need bersetzung
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.",
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.",
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.",	// Besoin de traduction
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.",	// Bisogno di traduzione
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.",	// Necessita de traduo
	};
const char *Txt_The_folder_name_X_has_not_changed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
        "The folder name <strong>%s</strong> has not changed.",	// Need bersetzung
	"The folder name <strong>%s</strong> has not changed.",
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.",
	"The folder name <strong>%s</strong> has not changed.",	// Besoin de traduction
	"The folder name <strong>%s</strong> has not changed.",	// Bisogno di traduzione
	"The folder name <strong>%s</strong> has not changed.",	// Necessita de traduo
	};
const char *Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
        "The folder name <strong>%s</strong> has not changed" \
		" because there is already a folder or a file with the name <strong>%s</strong>.",	// Need bersetzung
	"The folder name <strong>%s</strong> has not changed" \
		" because there is already a folder or a file with the name <strong>%s</strong>.",
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado" \
		" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.",
	"The folder name <strong>%s</strong> has not changed" \
		" because there is already a folder or a file with the name <strong>%s</strong>.",	// Besoin de traduction
	"The folder name <strong>%s</strong> has not changed" \
		" because there is already a folder or a file with the name <strong>%s</strong>.",	// Bisogno di traduzione
	"The folder name <strong>%s</strong> has not changed" \
		" because there is already a folder or a file with the name <strong>%s</strong>.",	// Necessita de traduo
	};
const char *Txt_The_graph_shows_the_NUMBER[Txt_NUM_LANGUAGES] =
       {
	"Die Grafik zeigt die",
	"The graph shows the",
	"La gr&aacute;fica muestra el",
	"Le graphique montre le",
	"Il grafico mostra il",
	"O gr&aacute;fico mostra o",
	};
const char *Txt_The_group_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The group <strong>%s</strong> already exists.",	// Need bersetzung
	"The group <strong>%s</strong> already exists.",
	"El grupo <strong>%s</strong> ya existe.",
	"The group <strong>%s</strong> already exists.",	// Besoin de traduction
	"The group <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The group <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_holiday_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The holiday <strong>%s</strong> already exists.",	// Need bersetzung
	"The holiday <strong>%s</strong> already exists.",
	"La festividad <strong>%s</strong> ya existe.",
	"The holiday <strong>%s</strong> already exists.",	// Besoin de traduction
	"The holiday <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The holiday <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_institution_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The institution <strong>%s</strong> already exists.",	// Need bersetzung
	"The institution <strong>%s</strong> already exists.",
	"La instituci&oacute;n <strong>%s</strong> ya existe.",
	"The institution <strong>%s</strong> already exists.",	// Besoin de traduction
	"The institution <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The institution <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_integrated_editor_is_not_yet_available[Txt_NUM_LANGUAGES] =
       {
	"The integrated editor is not yet available.",	// Need bersetzung
	"The integrated editor is not yet available.",
	"El editor integrado a&uacute;n no est&aacute; disponible.",
	"The integrated editor is not yet available.",	// Besoin de traduction
	"The integrated editor is not yet available.",	// Bisogno di traduzione
	"The integrated editor is not yet available.",	// Necessita de traduo
	};
const char *Txt_The_link_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The link <strong>%s</strong> already exists.",	// Need bersetzung
	"The link <strong>%s</strong> already exists.",
	"El enlace <strong>%s</strong> ya existe.",
	"The link <strong>%s</strong> already exists.",	// Besoin de traduction
	"The link <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The link <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_the_marks_of_a_student_chosen_at_random_[Txt_NUM_LANGUAGES] =
	{
	"die Noten von einem Student nach dem Zufallsprinzip aus dem Vorlesung werden gezeigt;<br />" \
	" wenn Sie mehr als einen Student bedeutet dies, dass die Anzahl der Kopf oder Fu von Zeilen ist nicht korrekt",

	"the marks of a student chosen at random from the course will be shown;<br />" \
	" if you see more than one student, it means that the number of header or footer rows is not correct",

	"se mostrar&aacute;n las calificaciones de un alumno de la asignatura escogido al azar;<br />" \
	" si ve m&aacute;s de un alumno, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos",

	"les notes d'un &eacute;tudiant choisi au hasard dans la mati&egrave;re seront pr&eacute;sent&eacute;s;<br />" \
	" si vous voyez plus d'un &eacute;tudiant, cela signifie que le nombre de ligned de la t&ecirc;te ou du pied n'est pas correcte",

	"i voti di uno studente scelto a caso dalla materia verranno mostrati;<br />" \
	" se si vede pi&uacute; di uno studente, significa che il numero di testa o piedi righe non &egrave; corretto",

	"vai mostrar as notas de um estudante da disciplina escolhido ao acaso;<br />" \
	" se voc&ecirc; vir mais de um estudante, isso significa que o n&uacute;mero de cabe&ccedil;a ou p&eacute; linhas n&atilde;o est&aacute; correto",
        };
const char *Txt_The_MIME_type_X_of_file_Y_is_not_allowed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
        "The MIME type <strong>%s</strong> of file <strong>%s</strong> is not allowed.",	// Need bersetzung
	"The MIME type <strong>%s</strong> of file <strong>%s</strong> is not allowed.",
	"El tipo MIME <strong>%s</strong> del archivo <strong>%s</strong> no est&aacute; permitido.",
	"The MIME type <strong>%s</strong> of file <strong>%s</strong> is not allowed.",	// Besoin de traduction
	"The MIME type <strong>%s</strong> of file <strong>%s</strong> is not allowed.",	// Bisogno di traduzione
	"The MIME type <strong>%s</strong> of file <strong>%s</strong> is not allowed.",	// Necessita de traduo
	};
const char *Txt_The_number_of_questions_must_be_in_the_interval_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %u in the following sentences
	{
	"The number of questions must be in the interval [%u,%u].",	// Need bersetzung
	"The number of questions must be in the interval [%u,%u].",
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].",
	"The number of questions must be in the interval [%u,%u].",	// Besoin de traduction
	"The number of questions must be in the interval [%u,%u].",	// Bisogno di traduzione
	"The number of questions must be in the interval [%u,%u].",	// Necessita de traduo
	};
const char *Txt_The_number_of_rows_is_now_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u in the following sentences
	{
	"The number of rows is now <strong>%u</ strong>.",	// Need bersetzung
	"The number of rows is now <strong>%u</ strong>.",
	"El n&uacute;mero de filas es ahora <strong>%u</strong>.",
	"The number of rows is now <strong>%u</ strong>.",	// Besoin de traduction
	"The number of rows is now <strong>%u</ strong>.",	// Bisogno di traduzione
	"The number of rows is now <strong>%u</ strong>.",	// Necessita de traduo
	};
const char *Txt_The_place_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The place <strong>%s</strong> already exists.",	// Need bersetzung
	"The place <strong>%s</strong> already exists.",
	"El lugar <strong>%s</strong> ya existe.",
	"The place <strong>%s</strong> already exists.",	// Besoin de traduction
	"The place <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The place <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_plugin_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The plugin <strong>%s</strong> already exists.",	// Need bersetzung
	"The plugin <strong>%s</strong> already exists.",
	"El complemento <strong>%s</strong> ya existe.",
	"The plugin <strong>%s</strong> already exists.",	// Besoin de traduction
	"The plugin <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The plugin <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_record_field_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The record field <strong>%s</strong> already exists.",	// Need bersetzung
	"The record field <strong>%s</strong> already exists.",
	"El campo de ficha <strong>%s</strong> ya existe.",
	"The record field <strong>%s</strong> already exists.",	// Besoin de traduction
	"The record field <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The record field <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_type_of_degree_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The type of degree <strong>%s</strong> already exists.",	// Need bersetzung
	"The type of degree <strong>%s</strong> already exists.",
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.",
	"The type of degree <strong>%s</strong> already exists.",	// Besoin de traduction
	"The type of degree <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The type of degree <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_type_of_group_X_already_exists[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"The type of group <strong>%s</strong> already exists.",	// Need bersetzung
	"The type of group <strong>%s</strong> already exists.",
	"El tipo de grupo <strong>%s</strong> ya existe.",
	"The type of group <strong>%s</strong> already exists.",	// Besoin de traduction
	"The type of group <strong>%s</strong> already exists.",	// Bisogno di traduzione
	"The type of group <strong>%s</strong> already exists.",	// Necessita de traduo
	};
const char *Txt_The_whole_course[Txt_NUM_LANGUAGES] =
	{
	"Der gesamte Vorlesung",
	"The whole course",
	"Toda la asignatura",
	"L'ensemble de la mati&egrave;re",
	"Tutta la materia",
	"Toda a disciplina",
	};
const char *Txt_thread[Txt_NUM_LANGUAGES] =
       {
	"Thread",
	"thread",
	"hebra",
	"fil",
	"thread",
	"thread",
	};
const char *Txt_Thread_removed[Txt_NUM_LANGUAGES] =
	{
	"Thread removed.",	// Need bersetzung
	"Thread removed.",
	"Hebra eliminada."
	"Thread removed.",	// Besoin de traduction
	"Thread removed.",	// Bisogno di traduzione
	"Thread removed.",	// Necessita de traduo
	};
const char *Txt_Thread_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Thread <strong>%s</strong> removed.",	// Need bersetzung
	"Thread <strong>%s</strong> removed.",
	"Hebra <strong>%s</strong> eliminada."
	"Thread <strong>%s</strong> removed.",	// Besoin de traduction
	"Thread <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Thread <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Threads[Txt_NUM_LANGUAGES] =
       {
	"Threads",
	"Threads",
	"Hebras",
	"Fils",
	"Thread",
	"Threads",
	};
const char *Txt_threads[Txt_NUM_LANGUAGES] =
       {
	"Threads",
	"threads",
	"hebras",
	"fils",
	"thread",
	"threads",
	};
const char *Txt_Title[Txt_NUM_LANGUAGES] =
       {
	"Titel",
	"Title",
	"T&iacute;tulo",
	"Titre",
	"Titolo",
	"T&iacute;tulo",
	};
const char *Txt_To_use_chat_you_must_have_installed_the_software[Txt_NUM_LANGUAGES] =
	{
	"So verwenden Sie chatten, m&uuml;ssen Sie die Software installiert haben",
	"To use chat you must have installed the software",
	"Para usar el chat es necesario tener instalado el software",
	"Pour utiliser le chat, vous devez avoir install&eacute; le logiciel",
	"Per utilizzare la chat bisogna avere installato il software",
	"Para usar o chat &eacute; necess&aacute;rio ter instalado o software",
        };
const char *Txt_Today[Txt_NUM_LANGUAGES] =
       {
	"Heute",
	"Today",
	"Hoy",
	"Aujourd'hui",
	"Oggi",
	"Hoje",
	};
const char *Txt_Total[Txt_NUM_LANGUAGES] =
       {
	"Gesamtzahl",
	"Total",
	"Total",
	"Total",
	"Totale",
	"Total",
	};
const char *Txt_Total_number_of_users[Txt_NUM_LANGUAGES] =
       {
	"Gesamtzahl der Benutzer",
	"Total number of users",
	"N&ordm; total de usuarios",
	"Nombre total d'utilisateurs",
	"Numero totale di utenti",
	"N&ordm; total de utilizadores",
	};
const char *Txt_Try_sending_a_compressed_file[Txt_NUM_LANGUAGES] =
	{
	"Try sending a compressed file.",	// Need bersetzung
	"Try sending a compressed file.",
	"Pruebe a enviar un archivo comprimido.",
	"Try sending a compressed file.",	// Besoin de traduction
	"Try sending a compressed file.",	// Bisogno di traduzione
	"Try sending a compressed file.",	// Necessita de traduo
	};
const char *Txt_TST_Correct_ANSWER[Txt_NUM_LANGUAGES] =
	{
	"Richtige",
	"Correct",
	"Correcta",
	"Bonne",
	"Corretta",
	"Correta",
	};
const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES][Txt_NUM_LANGUAGES] =
	{
		{
		"R/F",
		"T/F",
		"V/F",
		"V/F",
		"V/F",
		"V/F",
		},
		{
		"Unique-Choice",
		"Unique choice",
		"Elecci&oacute;n &uacute;nica",
		"Choix unique",
		"Scelta unica",
		"&Uacute;nica escolha",
		},
		{
		"Multiple-Choice",
		"Multiple choice",
		"Elecci&oacute;n m&uacute;ltiple",
		"Choix multiple",
		"Scelta multipla",
		"M&uacute;ltipla escolha",
		},
		{
		"Text",
		"Text",
		"Texto",
		"Texte",
		"Testo",
		"Texto",
		},
		{
		"Ganze Zahl",
		"Integer number",
		"N&ordm; entero",
		"Nombre entier",
		"Numero entero",
		"N&ordm; inteiro",
		},
		{
		"Reelle Zahl",
		"Real number",
		"N&ordm; real",
		"Nombre r&eacute;el",
		"Numero reale",
		"N&ordm; real",
		},
	};
const char *Txt_TST_STR_ORDER_FULL[Tst_NUM_CRIT_ORDEN_TST][Txt_NUM_LANGUAGES] =
	{
		{	// Tst_ORDER_HEADING
		"Sortieren nach Stammzellen in Frage",
		"Sort by stem of the question",
		"Ordenar por enunciado de la pregunta",
		"Trier par question",
		"Ordina per questione",
		"Classificar por formula&ccedil;&atilde;o da quest&atilde;o",
		},
		{	// TST_ORDER_NUM_TIMES_REPLIED
		"Sortieren nach Anzahl der mal die Frage beantwortet wurde",
		"Sort by number of times the question has been answered",
		"Ordenar por n&ordm; de veces que se ha respondido la pregunta",
		"Trier par nombre de fois qu'on a r&eacute;pondu &agrave; la question",
		"Ordina per numero di volte che avete risposto alla domanda",
		"Classificar por n&uacute;mero de vezes que a quest&atilde;o foi respondida",
		},
		{	// TST_ORDER_AVERAGE_SCORE
		"Sortieren nach Durchschnittsnote",
		"Sort by average score",
		"Ordenar por puntuaci&oacute;n media",
		"Trier par note moyenne",
		"Ordina per punteggio medio",
		"Classificar por pontua&ccedil;&atilde;o m&eacute;dia",
		},
	};
const char *Txt_TST_STR_ORDER_SHORT[Tst_NUM_CRIT_ORDEN_TST][Txt_NUM_LANGUAGES] =
	{
		{	// Tst_ORDER_HEADING
		"Frage",
		"Question",
		"Pregunta",
		"Question",
		"Questione",
		"Quest&atilde;o",
		},
		{	// TST_ORDER_NUM_TIMES_REPLIED
		"Anzahl<br />Antw.",
		"No.<br />ans.",
		"N&ordm;<br />resp.",
		"Nombre<br />r&eacute;p.",
		"Num.<br />risp.",
		"N&ordm;<br />resp.",
		},
		{	// TST_ORDER_AVERAGE_SCORE
		"Durchschnitt",
		"Average<br />score",
		"Puntuaci&oacute;n<br />media",
		"Note<br />moyenne",
		"Punteggio<br />medio",
		"Pontua&ccedil;&atilde;o<br />m&eacute;dia",
		},
	};
const char *Txt_TST_STR_FEEDBACK[Tst_NUM_TYPES_FEEDBACK][Txt_NUM_LANGUAGES] =
	{
		{
		"Keine Feedback",
		"No feedback",
		"Ninguna realimentaci&oacute;n",
		"Pas de feedback",
		"No feedback",
		"Nenhum feedback",
		},
		{
		"Minimale Detail: nur die Gesamtnote",
		"Minimum feedback: only the total score",
		"M&iacute;nimo detalle: s&oacute;lo puntuaci&oacute;n total",
		"Moindre d&eacute;tail: que le score total",
		"Minimo dettaglio: solo il punteggio totale",
		"M&iacute;n&oiacute;mo detalhe: apenas a pontua&ccedil;&atilde;o total",
		},
		{
		"Medium Detail: Ergebnisse der einzelnen Frage",
		"Medium feedback: result of each question",
		"Detalle medio: puntuaci&oacute;n de cada pregunta",
		"Medium d&eacute;tail: le r&eacute;sultat de chaque question",
		"Medio dettaglio: risultato di ogni questione",
		"M&eacute;dio detalhe: pontua&ccedil;&atilde;o de cada quest&atilde;o",
		},
		{
		"Maximale Details: die richtige Antwort f&uuml;r jede Frage",
		"Maximum feedback: correct answer for each question",
		"M&aacute;ximo detalle: respuesta correcta en cada pregunta",
		"Maximum de d&eacute;tails: bonne r&eacute;ponse pour chaque question",
		"Massimo dettaglio: la corretta risposta per ogni domanda",
		"M&aacute;ximo detalhe: resposta correta para cada quest&atilde;o",
		},
	};
const char *Txt_TST_TF_QST[2][Txt_NUM_LANGUAGES] =
	{
		{	// True
		"R",
		"T",
		"V",
		"V",
		"V",
		"V",
		},
		{	// False
		"F",
		"F",
		"F",
		"F",
		"F",
		"F",
		}
	};
const char *Txt_Type[Txt_NUM_LANGUAGES] =
	{
	"Art",
	"Type",
	"Tipo",
	"Type",
	"Tipo",
	"Tipo",
	};
const char *Txt_Type_of_degree[Txt_NUM_LANGUAGES] =
	{
	"Abschlussart",
	"Type of degree",
	"Tipo de titulaci&oacute;n",
	"Type d'&eacute;tude",
	"Tipo di laurea",
	"Tipo de titula&ccedil;&atilde;o",
	};
const char *Txt_Type_of_BR_degree[Txt_NUM_LANGUAGES] =
	{
	"Abschlussart",
	"Type of<br />degree",
	"Tipo de<br />titulaci&oacute;n",
	"Type<br />d'&eacute;tude",
	"Tipo di<br />laurea",
	"Tipo de<br />titula&ccedil;&atilde;o",
	};
const char *Txt_Type_of_degree_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Type of degree <strong>%s</strong> removed.",	// Need bersetzung
	"Type of degree <strong>%s</strong> removed.",
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada."
	"Type of degree <strong>%s</strong> removed.",	// Besoin de traduction
	"Type of degree <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Type of degree <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Types_of_degree[Txt_NUM_LANGUAGES] =
	{
	"Abschlussart",
	"Types of degree",
	"Tipos de titulaci&oacute;n",
	"Types d'&eacute;tude",
	"Tipi di laurea",
	"Tipos de titula&ccedil;&atilde;o",
	};
const char *Txt_Type_of_group[Txt_NUM_LANGUAGES] =
	{
	"Art der Gruppe",
	"Type of group",
	"Tipo de grupo",
	"Type de groupe",
	"Tipo di gruppo",
	"Tipo de grupo",
        };
const char *Txt_Type_BR_of_group[Txt_NUM_LANGUAGES] =
	{
	"Art<br />der Gruppe",
	"Type<br />of group",
	"Tipo<br />de grupo",
	"Type<br />de groupe",
	"Tipo<br />di gruppo",
	"Tipo<br />de grupo",
        };
const char *Txt_Type_of_group_X_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Type of group <strong>%s</strong> removed.",	// Need bersetzung
	"Type of group <strong>%s</strong> removed.",
	"Tipo de grupo <strong>%s</strong> eliminado."
	"Type of group <strong>%s</strong> removed.",	// Besoin de traduction
	"Type of group <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Type of group <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Types_of_group_in_the_course_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Arten von Gruppe in das Vorlesung <strong>%s</strong>:",
	"Types of group in the course <strong>%s</strong>:",
	"Tipos de grupo en la asignatura <strong>%s</strong>:",
	"Types de groupe dans la mati&egrave;re <strong>%s</strong>:",
	"Tipi di gruppo nella materia <strong>%s</strong>:",
	"Tipos de grupo na disciplina <strong>%s</strong>:",
        };
const char *Txt_Type_of_user[Txt_NUM_LANGUAGES] =
	{
	"Benutzertyp",
	"Type of user",
	"Tipo de usuario",
	"Type d'utilisateur",
	"Tipo di utente",
	"Tipo de utilizador",
	};
const char *Txt_Undo[Txt_NUM_LANGUAGES] =
	{
	"R&uuml;ckg&auml;ngig",
	"Undo",
	"Deshacer",
	"Annuler",
	"Annulla",
	"Desfazer",
	};
const char *Txt_unknown_assignment[Txt_NUM_LANGUAGES] =
	{
	"Aktivit&auml;t unbekannt",
	"unknown assignment",
	"actividad desconocida",
	"activit&eacute; inconnue",
	"attivit&agrave; sconosciuta",
	"atividade desconhecida",
        };
const char *Txt_unknown_recipient[Txt_NUM_LANGUAGES] =
	{
	"Empf&auml;nger unbekannt",
	"unknown recipient",
	"destinatario desconocido",
	"destinataire inconnu",
	"destinatario sconosciuto",
	"destinat&aacute;rio desconhecido",
        };
const char *Txt_unknown_recipients[Txt_NUM_LANGUAGES] =
	{
	"Empf&auml;nger unbekannt",
	"unknown recipients",
	"destinatarios desconocidos",
	"destinataires inconnus",
	"destinatari sconosciuti",
	"destinat&aacute;rios desconhecidos",
        };
const char *Txt_unread_MESSAGE[Txt_NUM_LANGUAGES] =
	{
	"ungelesene",
	"unread",
	"no le&iacute;do",
	"non lu",
	"non letto",
	"n&atilde;o lida",
        };
const char *Txt_unread_MESSAGES[Txt_NUM_LANGUAGES] =
	{
	"ungelesene",
	"unread",
	"no le&iacute;dos",
	"non lus",
	"non letti",
	"n&atilde;o lidas",
        };
const char *Txt_Unread_BR_msgs[Txt_NUM_LANGUAGES] =
	{
	"Ungel.<br />Nachr.",
	"Unread<br />msgs.",
	"No<br />le&iacute;dos",
	"Non<br />lus",
	"Non<br />letti",
	"N&atilde;o<br />lidas",
	};
const char *Txt_unread_messages_received_from[Txt_NUM_LANGUAGES] =
	{
	"ungelesene empfangene Nachrichten aus",
	"unread messages received from",
	"mensajes no le&iacute;dos recibidos desde",
	"messages non lus re&ccedil;us de",
	"messaggi non letti ricevuti de",
	"mensagens n&atilde;o lidas recebidas de",
	};
const char *Txt_Update_students_according_to_selected_groups[Txt_NUM_LANGUAGES] =
	{
	"Update Studenten nach ausgew&auml;hlten Gruppen",
	"Update students according to selected groups",
	"Actualizar alumnos seg&uacute;n los grupos seleccionados",
	"Mise &agrave; jour des &eacute;tudiants en fonction de groupes s&eacute;lectionnes",
	"Aggiornamento studenti in base a gruppi selezionati",
	"Atualizar os estudantes de acordo com grupos selecionados",
	};
const char *Txt_Upload_file[Txt_NUM_LANGUAGES] =
	{
	"Upload Datei",
	"Upload file",
	"Enviar archivo",
	"T&eacute;l&eacute;charger fichier",
	"Caricare file",
	"Fazer upload de arquivo",
        };
const char *Txt_Upload_file_or_create_folder_in_FOLDER[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Upload Datei oder erstellen Verzeichnis in %s",
	"Upload file or create folder in %s",
	"A&ntilde;adir archivo o crear carpeta en %s",
	"Charger fichier ou cr&eacute;er r&eacute;pertoire dans %s",
	"Caricare file o creare cartella in %s",
	"Fazer upload de arquivo ou criar diret&oacute;rio em %s",
	};
const char *Txt_Upload_files[Txt_NUM_LANGUAGES] =
	{
	"Upload<br />Dateien?",
	"Upload<br />files?",
	"&iquest;Enviar<br />archivos?",
	"T&eacute;l&eacute;charger<br />fichiers?",
	"Caricare<br />file?",
	"Fazer upload<br />de arquivos?",
	};
const char *Txt_Upload_photo[Txt_NUM_LANGUAGES] =
	{
	"Upload Foto",
	"Upload photo",
	"Enviar foto",
	"T&eacute;l&eacute;charger photo",
	"Caricare foto",
	"Fazer upload de foto",
        };
const char *Txt_Upload_time_too_long[Txt_NUM_LANGUAGES] =
        {
         "Upload ist zu lang",
         "Upload time too long",
         "Tiempo de env&iacute;o demasiado largo",
         "Temps de t&eacute;l&eacute;chargement trop longs",
         "Durata del caricamento troppo lunghi",
         "Tempo de upload muito longo",
        };
const char *Txt_User[Txt_NUM_LANGUAGES] =
       {
	"Benutzer",
	"User",
	"Usuario",
	"Utilisateur",
	"Utente",
	"Utilizador",
	};
const char *Txt_user[Txt_NUM_LANGUAGES] =
       {
	"Benutzer",
	"user",
	"usuario",
	"utilisateur",
	"utente",
	"utilizador",
	};
const char *Txt_user_NO_HTML[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	/* Usr_GENDER_UNKNOWN */	{"Benutzer"	,"user"	,"usuario/a"	,"utilisateur/se"	,"utente"	,"utilizador/a"	},
	/* Usr_GENDER_FEMALE */	{"Benutzer"	,"user"	,"usuaria"	,"utilisateuse"		,"utente"	,"utilizadora"	},
	/* Usr_GENDER_MALE */	{"Benutzer"	,"user"	,"usuario"	,"utilisateur"		,"utente"	,"utilizador"	},
	/* Usr_GENDER_ALL */	{"Benutzer"	,"user"	,"usuario/a"	,"utilisateur/se"	,"utente"	,"utilizador/a"	},
	};
const char *Txt_usr[Txt_NUM_LANGUAGES] =	// Abbreviation of user (three characters + dot)
	{
	"Ben.",
	"usr.",
	"usr.",
	"uti.",
	"ute.",
	"uti.",
	};
const char *Txt_USR_TYPE_STATS[Sta_NUM_USR_TYPES_STAT][Txt_NUM_LANGUAGES] =
	{
		{
		"Identifiziert",
		"identified",
		"identificados",
		"identifi&eacute;s",
		"identificati",
		"identificados",
		},
		{
		"Identifiziert oder Unbekannte",
		"identified or unknown",
		"identificados o desconocidos",
		"identifi&eacute;s ou inconnus",
		"identificati o sconosciuti",
		"identificados ou desconhecidos",
		},
		{
     		"Administratoren",
		"administrators",
		"administradores",
		"administrateurs",
		"amministratori",
		"administradores",
		},
		{
		"Dozenten",
		"teachers",
		"profesores",
		"enseignants",
		"insegnanti",
		"professores",
		},
		{
		"Studenten",
		"students",
		"alumnos",
		"&eacute;tudiants",
		"studenti",
		"estudantes",
		},
		{
		"Eingeladen",
		"invited",
		"invitados",
		"invit&eacute;s",
		"ospiti",
		"convidados",
		},
		{
		"Unbekannte",
		"unknown",
		"desconocidos",
		"inconnus",
		"sconosciuti",
		"desconhecidos",
		},
		{
		"Mich",
		"me",
		"yo",
		"moi",
		"me",
		"eu",
		},
	};
const char *Txt_Users[Txt_NUM_LANGUAGES] =
       {
	"Benutzer",
	"Users",
	"Usuarios",
	"Utilisateurs",
	"Utenti",
	"Utilizadores",
	};
const char *Txt_users[Txt_NUM_LANGUAGES] =
       {
	"Benutzer",
	"users",
	"usuarios",
	"utilisateurs",
	"utenti",
	"utilizadores",
	};
const char *Txt_USER_comments[Txt_NUM_LANGUAGES] =
       {
	"Kommentare",
	"Comments",
	"Observaciones",
	"Commentaires",
	"Commenti",
	"Coment&aacute;rios",
	};
const char *Txt_User_DNI[Txt_NUM_LANGUAGES] =
	{
	"Ausweis-Nr.",
	"User ID",
	"DNI del usuario",
	"Num&eacute;ro d'identit&eacute; d'utilisateur",
	"Numero d'identit&agrave; utente",
	"N&ordm; de identif. do utilizador",
	};
const char *Txt_Vacants[Txt_NUM_LANGUAGES] =
	{
	"Frei",
	"Vacants",
	"Vacantes",
	"Vacants",
	"Liberi",
	"Desocupados",
        };
const char *Txt_View_all_recipients[Txt_NUM_LANGUAGES] =
	{
	"Anzeigen alle Empf&auml;nger",
	"View all recipients",
	"Ver todos los destinatarios",
	"Voir tous les destinataires",
	"Vedi tutti i destinatari",
	"Ver todos os estinat&aacute;rios",
        };
const char *Txt_View_in_a_new_window[Txt_NUM_LANGUAGES] =
	{
	"Ansicht in einem neuen Fenster",
	"View in a new window",
	"Mostrar en una ventana nueva",
	"Voir dans une nouvelle fen&ecirc;tre",
	"Visualizza in una nuova finestra",
	"Ver em uma nova janela",
        };
const char *Txt_View_messages[Txt_NUM_LANGUAGES] =
	{
	"Anzeigen Nachrichten",
	"View messages",
	"Ver mensajes",
	"Voir messages",
	"Vedi messaggi",
	"Ver mensagens",
	};
const char *Txt_View_works[Txt_NUM_LANGUAGES] =
	{
	"Anzeigen Hausarbeiten",
	"View works",
	"Ver trabajos",
	"Voir travaux",
	"Vedi lavori",
	"Ver trabalhos",
	};
const char *Txt_Virtual_pendrives[Txt_NUM_LANGUAGES] =
	{
	"Aktentasche",
	"Virtual pendrives",
	"Maletines",
	"Porte-documents",
	"Pendrive virtuale",
	"Pasta",
	};
const char *Txt_Visible_by_BR_the_student[Txt_NUM_LANGUAGES] =
	{
	"Sichtbare durch<br />die Student?",
	"Visible by<br />the student?",
	"&iquest;Visible por<br />el alumno/a?",
	"Visible par<br />l'&eacute;tudiant/e?",
	"Visibilidallo<br />studente?",
	"Vis&iacute;vel pelo<br />estudante?",
	};
const char *Txt_We_need_your_help[Txt_NUM_LANGUAGES] =
	{
	"Wir brauchen Ihre Hilfe!",
	"We need your help!",
	"&iexcl;Necesitamos su ayuda!",
	"Nous avons besoin de votre aide!",
	"Abbiamo bisogno del vostro aiuto!",
	"N&oacute;s precisamos de sua ajuda!",
	};
const char *Txt_Web_page[Txt_NUM_LANGUAGES] =
	{
	"Webseite",
	"Web page",
	"P&aacute;gina web",
	"Page Web",
	"Pagina Web",
	"P&aacute;gina da Web",		
	};
const char *Txt_Web_page_of_the_course[Txt_NUM_LANGUAGES] =
	{
	"Webseite des Vorlesung",
	"Web page of the course",
	"P&aacute;gina web de la asignatura",
	"Page Web de la mati&egrave;re",
	"Pagina Web della materia",
	"P&aacute;gina da Web da disciplina",
	};
const char *Txt_Week[Txt_NUM_LANGUAGES] =
	{
	"Woche",
	"Week",
	"Semana",
	"Semaine",
	"Settimana",
	"Semana",
	};
const char *Txt_with_new_posts[Txt_NUM_LANGUAGES] =
	{
	"mit neuen Beitr&auml;ge",
	"with new posts",
	"con mensajes nuevos",
	"avec de nouveaux messages",
	"con nuovi post",
	"com novas posts",
	};
const char *Txt_Works_of_the_user_with_ID_X_in_Y_removed[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include two %s in the following sentences
	{
	"Works of the user with ID <strong>%s</strong> in <strong>%s</strong> removed.",	// Need bersetzung
	"Works of the user with ID <strong>%s</strong> in <strong>%s</strong> removed.",
	"Trabajos del usuario con DNI <strong>%s</strong> en <strong>%s</strong> eliminados.",
	"Works of the user with ID <strong>%s</strong> in <strong>%s</strong> removed.",	// Besoin de traduction
	"Works of the user with ID <strong>%s</strong> in <strong>%s</strong> removed.",	// Bisogno di traduzione
	"Works of the user with ID <strong>%s</strong> in <strong>%s</strong> removed.",	// Necessita de traduo
	};
const char *Txt_Works_zone[Txt_NUM_LANGUAGES] =
        {
	"Hausarbeiten-Zone",
	"Works zone",
	"Zona de trabajos",
	"Zone de travaux",
	"Zona di lavori",
	"Zona de trabalhos",
        };
const char *Txt_works_zone[Txt_NUM_LANGUAGES] =
        {
	"Hausarbeiten-Zone",
	"works zone",
	"zona de trabajos",
	"zone de travaux",
	"zona di lavori",
	"zona de trabalhos",
        };
const char *Txt_WriBRters[Txt_NUM_LANGUAGES] =
	{
	"Schrift-<br />steller",
	"Wri-<br />ters",
	"Escri-<br />tores",
	"&Eacute;cri-<br />vains",
	"Scri-<br />ttori",
	"Escri-<br />tores",
	};
const char *Txt_Write_a_message_to_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Write a message to %s",	// Need bersetzung
	"Write a message to %s",
	"Escribir un mensaje a %s",
	"Write a message to %s",	// Besoin de traduction
	"Write a message to %s",	// Bisogno di traduzione
	"Write a message to %s",	// Necessita de traduo
	};
const char *Txt_Year[Txt_NUM_LANGUAGES] =	// 1, 2, 3..., not 1984, 2009...
	{
	"Jahr",
	"Year",
	"Curso",
	"Ann&eacute;e",
	"Anno",
	"Ano",
	};
const char *Txt_Yes[Txt_NUM_LANGUAGES] =
	{
	"Ja",
	"Yes",
	"S&iacute;",
	"Oui",
	"S&igrave;",
	"Sim",
	};
const char *Txt_Yesterday[Txt_NUM_LANGUAGES] =
       {
	"Gestern",
	"Yesterday",
	"Ayer",
	"Hier",
	"Ieri",
	"Ontem",
	};
const char *Txt_You_are_now_LOGGED_as_X[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Sie sind jetzt %s als <strong>%s</strong>.",
	"You are now %s as <strong>%s</strong>.",
	"Usted est&aacute; ahora %s como <strong>%s</strong>.",
	"Vous &ecirc;tes maintenant %s comme <strong>%s</strong>.",
	"Si &egrave; ora %s come <strong>%s</strong>.",
	"Agora voc&ecirc; est&aacute; %s como <strong>%s</strong>.",
	};
const char *Txt_You_can_create_a_new_folder_inside_the_folder[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
       {
	"Sie k&ouml;nnen einen neuen Verzeichnis im Verzeichnis <strong>%s</strong>:",
	"You can create a new folder inside the folder <strong>%s</strong>:",
	"Puede crear una nueva carpeta dentro de la carpeta <strong>%s</strong>:",
	"Vous pouvez cr&eacute;er un nouveau r&eacute;pertoire dans le r&eacute;pertoire <strong>%s</strong>:",
	"&Egrave; possibile creare una nuova cartella all'interno della cartella <strong>%s</strong>:",
	"Voc&ecirc; pode criar um novo diret&oacute;rio dentro do diret&oacute;rio <strong>%s</strong>:",
	};
const char *Txt_You_cant_disable_file_uploading_once_folders_have_been_created[Txt_NUM_LANGUAGES] =
	{
	"Sie k&ouml;nnen nicht verhindern, Datei hochladen, wenn Verzeichnisse erstellt worden sind.",
	"You can't disable file uploading once folders have been created.",
	"No puede desactivar el env&iacute;o de archivos una vez que se han creado carpetas.",
	"Vous ne pouvez pas d&eacute;sactiver le t&eacute;l&eacute;chargement de fichiers, une fois des r&eacute;pertoires ont &eacute;t&eacute; cr&eacute;&eacute;s.",
	"Non &egrave; possibile disabilitare il caricamento dei file una volta che le cartelle sono state create.",
	"Voc&ecirc; n&atilde;o pode desativar o upload de arquivos, uma vez diret&oacute;rios foram criados.",
	};
const char *Txt_You_dont_have_permission_to_perform_this_action[Txt_NUM_LANGUAGES] =
	{
	"Sie haben keine Berechtigung zum Ausf&uuml;hren dieser Aktion",
	"You don't have permission to perform this action",
	"No tiene permiso para ejecutar esta acci&oacute;n",
	"Vous n'avez pas la permission d'effectuer cette action",
	"Non si dispone dell'autorizzazione per eseguire questa azione",
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o para realizar esta a&ccedil;&atilde;o",
	};
const char *Txt_You_must_enter_your_ID_or_your_nickname[Txt_NUM_LANGUAGES] =
	{
	"Bitte geben Sie Ihre Ausweis-Nr. oder Ihren Benutzername.",
	"You must enter your ID or your nickname.",
	"Debe escribir su DNI o su apodo.",
	"Vous devez entrer votre num&eacute;ro d'identit&eacute; ou votre surnom.",
	"Dovete inserire il vostro numero d'identit&agrave; o il vostro soprannome.",
	"Voc&ecirc; deve digitar seu n&ordm; identif. ou seu alcunha.",
	};
const char *Txt_You_must_select_one_ore_more_recipients[Txt_NUM_LANGUAGES] =
	{
	"You must select one or more recipients.",	// Need bersetzung
	"You must select one or more recipients.",
	"Debe seleccionar uno o m&aacute;s destinatarios.",
	"You must select one or more recipients.",	// Besoin de traduction
	"You must select one or more recipients.",	// Bisogno di traduzione
	"You must select one or more recipients.",	// Necessita de traduo
	};
const char *Txt_You_must_select_one_ore_more_students[Txt_NUM_LANGUAGES] =
	{
	"You must select one or more students.",	// Need bersetzung
	"You must select one or more students.",
	"Debe seleccionar uno o m&aacute;s alumnos.",
	"You must select one or more students.",	// Besoin de traduction
	"You must select one or more students.",	// Bisogno di traduzione
	"You must select one or more students.",	// Necessita de traduo
	};
const char *Txt_You_must_select_one_ore_more_tags[Txt_NUM_LANGUAGES] =
	{
	"You must select one or more tags.",	// Need bersetzung
	"You must select one or more tags.",
	"Debe seleccionar uno o m&aacute;s descriptores.",
	"You must select one or more tags.",	// Besoin de traduction
	"You must select one or more tags.",	// Bisogno di traduzione
	"You must select one or more tags.",	// Necessita de traduo
	};
const char *Txt_You_must_select_one_ore_more_types_of_answer[Txt_NUM_LANGUAGES] =
	{
	"You must select one or more types of answer.",	// Need bersetzung
	"You must select one or more types of answer.",
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.",
	"You must select one or more types of answer.",	// Besoin de traduction
	"You must select one or more types of answer.",	// Bisogno di traduzione
	"You must select one or more types of answer.",	// Necessita de traduo
	};
const char *Txt_You_must_select_one_ore_more_users[Txt_NUM_LANGUAGES] =
	{
	"You must select one or more users.",	// Need bersetzung
	"You must select one or more users.",
	"Debe seleccionar uno o m&aacute;s usuarios.",
	"You must select one or more users.",	// Besoin de traduction
	"You must select one or more users.",	// Bisogno di traduzione
	"You must select one or more users.",	// Necessita de traduo
	};
const char *Txt_You_must_specify_the_file[Txt_NUM_LANGUAGES] =
	{
	"Sie m&uuml;ssen den Datei.",
	"You must specify the file.",
	"Debe especificar el archivo.",
	"Vous devez sp&eacute;cifier le fichier.",
	"&Egrave; necessario specificare il file.",
	"Voc&ecirc; deve especificar o arquivo.",
	};
const char *Txt_You_must_specify_the_title_of_the_assignment[Txt_NUM_LANGUAGES] =
	{
	"Sie m&uuml;ssen den Titel der Aktivit&auml;t.",
	"You must specify the title of the assignment.",
	"Debe especificar el t&iacute;tulo de la actividad.",
	"Vous devez sp&eacute;cifier le titre de l'activit&eacute;.",
	"&Egrave; necessario specificare il titolo dell'attivit&agrave;.",
	"Voc&ecirc; deve especificar o t&iacute;tulo da atividade.",
	};
const char *Txt_Your_registration_in_course_X_has_been_requested[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{
	"Ihre Anmeldung im Vorlesung <strong>%s</strong> beantragt worden ist." \
		" M&ouml;chten Sie, dass die Registrierung oder bevorzugen Sie, um aus dem Vorlesung?",
	"Your registration in course <strong>%s</strong> has been requested." \
		" Do you want to confirm the registration or do you prefer to be removed from the course?",
	"Se ha solicitado su alta en la asignatura <strong>%s</strong>." \
		" &iquest;Confirma el alta o prefiere darse de baja de la asignatura?",
	"Votre inscription &agrave; la mati&egrave;re <strong>%s</strong> a &eacute;t&eacute; demand&eacute;." \
		" Voulez-vous confirmer l'enregistrement ou pr&eacute;f&eacute;rez-vous &ecirc;tre enlev&eacute; de la mati&egrave;re?",
	"La sua registrazione in materia <strong>%s</strong> &egrave; stato richiesta." \
		" Vuoi per confermare la registrazione o si preferisce essere rimosso da questa materia?",
	"A sua inscri&ccedil;&atilde;o na disciplina <strong>%s</strong> tenha sido solicitada." \
		" Voc&ecirc; quer para confirmar a inscri&ccedil;&atilde;o ou voc&ecirc; prefere para ser removido da disciplina?",
	};
const char *Txt_Your_session_has_expired_due_to_inactivity[Txt_NUM_LANGUAGES] =
	{
	"Ihre Sitzung ist abgelaufen wegen Inaktivit&auml;t.",
	"Your session has expired due to inactivity.",
	"La sesi&oacute;n ha expirado por inactividad.",
	"Votre session a expir&eacute; d&ucirc; &agrave; l'inactivit&eacute;.",
	"La sessione &egrave; scaduta per inattivit&agrave;.",
	"Sua sess&atilde;o expirou devido &agrave; inatividade.",
	};
const char *Txt_ZIP_file[Txt_NUM_LANGUAGES] =
	{
	"ZIP-Datei",
	"ZIP file",
	"Archivo ZIP",
	"Fichier ZIP",
	"File ZIP",
	"Arquivo ZIP",
	};
const char *Txt_STAT_COURSE_FILE_ZONES[Sta_NUM_STAT_CRS_FILE_ZONES][Txt_NUM_LANGUAGES] =	// Use &nbsp; instead of space
	{
	/* Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS */	{"Downloads&nbsp;(Vor.)","Download&nbsp;(crs.)"	,"Descarg.&nbsp;(asg.)"	,"Documents&nbsp;(mat.)","Scarica&nbsp;(mat.)"	,"Documentos&nbsp;(crs.)"	},
	/* Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP */	{"Downloads&nbsp;(Grp.)","Download&nbsp;(grp.)"	,"Descarg.&nbsp;(grp.)"	,"Documents&nbsp;(grp.)","Scarica&nbsp;(grp.)"	,"Documentos&nbsp;(grp.)"	},
	/* Brw_FILE_BRW_COMMON_CRS */		{"Gemein.&nbsp;(Vor.)"	,"Common&nbsp;(crs.)"	,"Comunes&nbsp;(asg.)"	,"Communes&nbsp;(mat.)"	,"Comuni&nbsp;(mat.)"	,"Comumes&nbsp;(crs.)"		},
	/* Brw_FILE_BRW_COMMON_GRP */		{"Gemein.&nbsp;(Grp.)"	,"Common&nbsp;(grp.)"	,"Comunes&nbsp;(grp.)"	,"Communes&nbsp;(grp.)"	,"Comuni&nbsp;(grp.)"	,"Comumes&nbsp;(grp.)"		},
	/* Brw_FILE_BRW_ADMIN_MARKS_CRS */	{"Noten&nbsp;(Vor.)"	,"Marks&nbsp;(crs.)"	,"Calific.&nbsp;(asg.)"	,"Notes&nbsp;(mat.)"	,"Voti&nbsp;(mat.)"	,"Notas&nbsp;(crs.)"		},
	/* Brw_FILE_BRW_ADMIN_MARKS_GRP */	{"Noten&nbsp;(Grp.)"	,"Marks&nbsp;(grp.)"	,"Calific.&nbsp;(grp.)"	,"Notes&nbsp;(grp.)"	,"Voti&nbsp;(grp.)"	,"Notas&nbsp;(grp.)"		},
	/* Brw_FILE_BRW_ASSIGNMENTS_USR */	{"Aktivit&auml;ten"	,"Assignments"		,"Actividades"		,"Activit&eacute;s"	,"Attivit&agrave;"	,"Atividades"			},
	/* Brw_FILE_BRW_WORKS_USR */		{"Hausarbeiten"		,"Works"		,"Trabajos"		,"Travails"		,"Lavori"		,"Trabalhos"			},
	/* Brw_FILE_BRW_UNKNOWN */		{"Gesamtzahl"		,"Total"		,"Total"		,"Total"		,"Totale"		,"Total"			},
	};
const char *Txt_FORUM_WHICH_FORUM[2][Txt_NUM_LANGUAGES]=
	{
		{
		"Alle meine Foren",
		"All my forums",
		"Todos mis foros",
	        "Tous mes forums",
	        "Tutti i miei fori",
	        "Todos os meus f&oacute;runs",
		},
		{
		"Foren von diesem Standort",
		"Forums of this location",
		"Foros de este sitio",
		"Forums de cet emplacement",
		"Fori di questo sito",
		"F&oacute;runs deste site",
		},
	};
const char *Txt_FORUM_THREAD_HELP_ORDER[2][Txt_NUM_LANGUAGES]=
	{
		{
		"Sortieren nach Datum der ersten Nachricht",
		"Sort by date of first message",
		"Ordenar por fecha del primer mensaje",
		"Trier par date du premier message",
		"Ordina per data del primo messaggio",
		"Classificar por data da primeira mensagem",
		},
		{
		"Sortieren nach Datum der letzten Nachricht",
		"Sort by date of the last message",
		"Ordenar por fecha del &uacute;ltimo mensaje",
		"Trier par date du dernier message",
		"Ordina per data di ultimo messaggio",
		"Classificar pela data da &uacute;ltima mensagem",
		},
	};
const char *Txt_FORUM_THREAD_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{"Erste Nachricht"	,"First message"	,"Primer mensaje"		,"Premier message"	,"Primo messaggio"	,"Primeira mensagem"		},
		{"Letzte Nachricht"	,"Last message"		,"&Uacute;ltimo mensaje"	,"Dernier message"	,"Ultimo messaggio"	,"&Uacute;ltima mensagem"	},
	};
const char *Txt_CENTRES_HELP_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Sortieren nach Zentrum",
		"Sort by centre",
		"Ordenar por centro",
		"Trier par centre",
		"Ordina per centro",
		"Classificar por centro",
		},
		{
		"Sortieren nach Anzahl der Dozenten",
		"Sort by no. of teachers",
		"Ordenar por n&ordm; de profesores",
		"Trier par nombre d'enseignants",
		"Ordina per numero di insegnanti",
		"Classificar por n&ordm; de professores",
		},
	};
const char *Txt_CENTRES_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{"Zentrum"			,"Centre"		,"Centro"			,"Centre"			,"Centro"			,"Centro"			},
		{"Anzahl der<br />Dozenten"	,"No. of<br />teachers"	,"N&ordm; de<br />profesores"	,"Nombre<br />d'enseignants"	,"Numero di<br />insegnanti"	,"N&ordm; de<br />professores"	},
	};
const char *Txt_DEPARTMENTS_HELP_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Sortieren nach Fachbereich",
		"Sort by department",
		"Ordenar por departamento",
		"Trier par d&eacute;partement",
		"Ordina per dipartimento",
		"Classificar por departamento",
		},
		{
		"Sortieren nach Anzahl der Dozenten",
		"Sort by no. of teachers",
		"Ordenar por n&ordm; de profesores",
		"Trier par nombre d'enseignants",
		"Ordina per numero di insegnanti",
		"Classificar por n&ordm; de professores",
		},
	};
const char *Txt_DEPARTMENTS_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{"Fachbereich"			,"Department"		,"Departamento"			,"D&eacute;partement"		,"Dipartimento"			,"Departamento"			},
		{"Anzahl der<br />Dozenten"	,"No. of<br />teachers"	,"N&ordm; de<br />profesores"	,"Nombre<br />d'enseignants"	,"Numero di<br />insegnanti"	,"N&ordm; de<br />professores"	},
	};
const char *Txt_HOLIDAYS_HELP_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Sortieren nach Standort",
		"Sort by place",
		"Ordenar por lugar",
		"Trier par emplacement",
		"Ordina per localit&grave;",
		"Classificar por localiza&ccedil;&atilde;o",
		},
		{
		"Sortieren nach Datum",
		"Sort by date",
		"Ordenar por fecha",
		"Trier par date",
		"Ordina per data",
		"Classificar por data",
		},
	};
const char *Txt_HOLIDAYS_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Standort",
		"Place"	,
		"Lugar",
		"Emplacement",
		"Luogo",
		"Localiza&ccedil;&atilde;o",
		},
		{
		"Startdatum",
		"Start date",
		"Fecha inicial",
		"Date initiale",
		"Data iniziale",
		"Data de in&iacute;cio",
		},
	};
const char *Txt_INSTITUTIONS_HELP_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Sortieren nach Institution",
		"Sort by institution",
		"Ordenar por instituci&oacute;n",
		"Trier par &eacute;tablissement",
		"Ordina per istituzione",
		"Classificar por institui&ccedil;&atilde;o",
		},
		{
		"Sortieren nach Anzahl der Benutzer",
		"Sort by no. of users",
		"Ordenar por n&ordm; de usuarios",
		"Trier par nombre d'utilisateurs",
		"Ordina per numero di utenti",
		"Classificar por n&ordm; de utilizadores",
		},
	};
const char *Txt_INSTITUTIONS_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Institution",
		"Institution",
		"Instituci&oacute;n",
		"&Eacute;tablissement",
		"Istituzione",
		"Institui&ccedil;&atilde;o",
		},
		{
		"Anzahl der<br />Benutzer",
		"No. of<br />users",
		"N&ordm; de<br />usuarios",
		"Nombre<br />d'utilisateurs",
		"Numero di<br />utenti",
		"N&ordm; de<br />utilizadores",
		},
	};
const char *Txt_PLACES_HELP_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Sortieren nach Standort",
		"Sort by place",
		"Ordenar por lugar",
		"Trier par emplacement",
		"Ordina per localit&grave;",
		"Classificar por localiza&ccedil;&atilde;o",
		},
		{
		"Sortieren nach Anzahl der Grad",
		"Sort by no. of degrees",
		"Ordenar por n&ordm; de titulaciones",
		"Trier par nombre d'&eacute;tudes",
		"Ordina per numero di lauree",
		"Classificar por n&ordm; de titula&ccedil;oes",
		},
	};
const char *Txt_PLACES_ORDER[2][Txt_NUM_LANGUAGES] =
	{
		{
		"Standort",
		"Place",
		"Lugar",
		"Emplacement",
		"Luogo",
		"Localiza&ccedil;&atilde;o",
		},
		{
		"Anzahl der<br />Grad",
		"No. of<br />degrees",
		"N&ordm; de<br />titulaciones",
		"Nombre<br />d'&eacute;tudes",
		"Numero di<br />lauree",
		"N&ordm; de<br />titula&ccedil;oes",
		},
	};
const char *Txt_DAYS_SMALL[7][Txt_NUM_LANGUAGES] =
	{
		{"Montag"	,"Monday"	,"lunes"		,"Lundi"	,"Luned&igrave;"	,"Segunda-feira"	},
		{"Dienstag"	,"Tuesday"	,"martes"		,"Mardi"	,"Marted&igrave;"	,"Ter&ccedil;a-feira"	},
		{"Mittwoch"	,"Wednesday"	,"mi&eacute;rcoles"	,"Mercredi"	,"Mercoled&igrave;"	,"Quarta-feira"		},
		{"Donnerstag"	,"Thursday"	,"jueves"		,"Jeudi"	,"Gioved&igrave;"	,"Quinta-feira"		},
		{"Freitag"	,"Friday"	,"viernes"		,"Vendredi"	,"Venerd&igrave;"	,"Sexta-feira"		},
		{"Samstag"	,"Saturday"	,"s&aacute;bado"	,"Samedi"	,"Sabato"		,"Sabado"		},
		{"Sonntag"	,"Sunday"	,"domingo"		,"Dimanche"	,"Domenica"		,"Domingo"		},
	};
const char *Txt_DAYS_CAPS[7][Txt_NUM_LANGUAGES] =
	{
		{"MONTAG"	,"MONDAY"	,"LUNES"		,"LUNDI"	,"LUNED&Igrave;"	,"SEGUNDA-FEIRA"	},
		{"DIENSTAG"	,"TUESDAY"	,"MARTES"		,"MARDI"	,"MARTED&Igrave;"	,"TER&Ccedil;A-FEIRA"	},
		{"MITTWOCH"	,"WEDNESDAY"	,"MI&Eacute;RCOLES"	,"MERCREDI"	,"MERCOLED&Igrave;"	,"QUARTA-FEIRA"		},
		{"DONNERSTAG"	,"THURSDAY"	,"JUEVES"		,"JEUDI"	,"GIOVED&Igrave;"	,"QUINTA-FEIRA"		},
		{"FREITAG"	,"FRIDAY"	,"VIERNES"		,"VENDREDI"	,"VENERD&Igrave;"	,"SEXTA-FEIRA"		},
		{"SAMSTAG"	,"SATURDAY"	,"S&Aacute;BADO"	,"SAMEDI"	,"SABATO"		,"SABADO"		},
		{"SONNTAG"	,"SUNDAY"	,"DOMINGO"		,"DIMANCHE"	,"DOMENICA"		,"DOMINGO"		},
	};
const char *Txt_MONTHS_SMALL[12][Txt_NUM_LANGUAGES] =
	{
		{"Januar"	,"January"	,"enero"	,"janvier"		,"gennaio"	,"janeiro"	},
		{"Februar"	,"February"	,"febrero"	,"f&eacute;vrier"	,"febbraio"	,"fevereiro"	},
		{"M;&auml;rz"	,"March"	,"marzo"	,"mars"			,"marzo"	,"mar&ccedil;o"	},
		{"April"	,"April"	,"abril"	,"avril"		,"aprile"	,"abril"	},
		{"Mai"		,"May"		,"mayo"		,"mai"			,"maggio"	,"maio"		},
		{"Juni"		,"June"		,"junio"	,"juin"			,"giugno"	,"junho"	},
		{"Juli"		,"July"		,"julio"	,"juillet"		,"luglio"	,"julho"	},
		{"August"	,"August"	,"agosto"	,"ao&ucirc;t"		,"agosto"	,"agosto"	},
		{"September"	,"September"	,"septiembre"	,"septembre"		,"settembre"	,"setembro"	},
		{"Oktober"	,"October"	,"octubre"	,"octobre"		,"ottobre"	,"outubro"	},
		{"November"	,"November"	,"noviembre"	,"novembre"		,"novembre"	,"novembro"	},
		{"Dezember"	,"December"	,"diciembre"	,"d&eacute;cembre"	,"dicembre"	,"dezembro"	},
	};
const char *Txt_MONTHS_CAPS[12][Txt_NUM_LANGUAGES] =
	{
		{"JANUAR"	,"JANUARY"	,"ENERO"	,"JANVIER"		,"GENNAIO"	,"JANEIRO"	},
		{"FEBRUAR"	,"FEBRUARY"	,"FEBRERO"	,"F&Eacute;VRIER"	,"FEBBRAIO"	,"FEVEREIRO"	},
		{"M&Auml;RZ"	,"MARCH"	,"MARZO"	,"MARS"			,"MARZO"	,"MAR&Ccedil;O"	},
		{"APRIL"	,"APRIL"	,"ABRIL"	,"AVRIL"		,"APRILE"	,"ABRIL"	},
		{"MAI"		,"MAY"		,"MAYO"		,"MAI"			,"MAGGIO"	,"MAIO"		},
		{"JUNI"		,"JUNE"		,"JUNIO"	,"JUIN"			,"GIUGNO"	,"JUNHO"	},
		{"JULI"		,"JULY"		,"JULIO"	,"JUILLET"		,"LUGLIO"	,"JULHO"	},
		{"AUGUST"	,"AUGUST"	,"AGOSTO"	,"AO&Ucirc;T"		,"AGOSTO"	,"AGOSTO"	},
		{"SEPTEMBER"	,"SEPTEMBER"	,"SEPTIEMBRE"	,"SEPTEMBRE"		,"SETTEMBRE"	,"SETEMBRO"	},
		{"OKTOBER"	,"OCTOBER"	,"OCTUBRE"	,"OCTOBRE"		,"OTTOBRE"	,"OUTUBRO"	},
		{"NOVEMBER"	,"NOVEMBER"	,"NOVIEMBRE"	,"NOVEMBRE"		,"NOVEMBRE"	,"NOVEMBRO"	},
		{"DEZEMBER"	,"DECEMBER"	,"DICIEMBRE"	,"D&Eacute;CEMBRE"	,"DICEMBRE"	,"DEZEMBRO"	},
	};
const char *Txt_LAYOUT_SIDE_COLUMNS[4][Txt_NUM_LANGUAGES] =
	{
		{"Nur mittleren Bereich anzeigen"	,"Display central area only"		,"S&oacute;lo zona central"		,"S&oacute;lo zona central"		,"S&oacute;lo zona central"		,"S&oacute;lo zona central"		},
		{"Rechte Spalte anzeigen"		,"Display right column"			,"Columna derecha visible"		,"Columna derecha visible"		,"Columna derecha visible"		,"Columna derecha visible"		},
		{"Linke Spalte anzeigen"		,"Display left column"			,"Columna izquierda visible"		,"Columna izquierda visible"		,"Columna izquierda visible"		,"Columna izquierda visible"		},
		{"Linke und rechte Spalte anzeigen"	,"Display left and right columns"	,"Columnas izquierda y derecha visibles","Columnas izquierda y derecha visibles","Columnas izquierda y derecha visibles","Columnas izquierda y derecha visibles"},
	};
const char *Txt_TYPE_COUNT_STAT_SMALL[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES] =
	{
		{"Gesamtzahl der Klicks"			,"total number of clicks"			,"n&ordm; total de clics"			,"nombre total de clics"			,"numero totale di clic"	,"n&uacute;mero total de cliques"		},
		{"Anzahl der verschiedenen Benutzer"		,"number of distinct users"			,"n&ordm; de usuarios distintos"		,"nombre des diff&eacute;rents utilisateurs"	,"numero di utenti diversi"	,"n&uacute;mero de utilizadores diferentes"	},
		{"Anzahl der Klicks pro Benutzer"		,"number of clicks per user"			,"n&ordm; de clics por usuario"			,"nombre de clics par utilisateur"		,"numero di clic per utente"	,"n&uacute;mero de cliques por utilizador"	},
		{"Zeit zur Erzeugung von Seite"			,"time to generate page"			,"tiempo de generaci&oacute;n de p&aacute;gina"	,"temps pour g&eacute;n&eacute;rer la page"	,"tempo per generare pagina"	,"tempo para gerar p&aacute;gina"		},
		{"Zeit zur Seite senden"			,"time to send page"				,"tiempo de env&iacute;o de p&aacute;gina"	,"temps pour envoyer la page"			,"tempo per inviare pagina"	,"Tempo para enviar a p&aacute;gina"		},
	};
const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES] =
	{
		{"Gesamtzahl der Klicks"			,"Total number of clicks"			,"N&ordm; total de clics"			,"Nombre total de clics"			,"Numero totale di clic"	,"N&uacute;mero total de cliques"		},
		{"Anzahl der verschiedenen Benutzer"		,"Number of distinct users"			,"N&ordm; de usuarios distintos"		,"Nombre des diff&eacute;rents utilisateurs"	,"Numero di utenti diversi"	,"N&uacute;mero de utilizadores diferentes"	},
		{"Anzahl der Klicks pro Benutzer"		,"Number of clicks per user"			,"N&ordm; de clics por usuario"			,"Nombre de clics par utilisateur"		,"Numero di clic per utente"	,"n&uacute;mero de cliques por utilizador"	},
		{"Zeit zur Erzeugung von Seite"			,"Time to generate page"			,"Tiempo de generaci&oacute;n de p&aacute;gina"	,"Temps pour g&eacute;n&eacute;rer la page"	,"Tempo per generare pagina"	,"Tempo para gerar p&aacute;gina"		},
		{"Zeit zur Seite senden"			,"Time to send page"				,"Tiempo de env&iacute;o de p&aacute;gina"	,"Temps pour envoyer la page"			,"Ttempo per inviare pagina"	,"Tempo para enviar a p&aacute;gina"		},
	};
const char *Txt_GENDER_PLURAL_Abc[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
	/* Usr_GENDER_UNKNOWN */	{"Unbekannte"	,"Unknown"	,"Desconocidos"	,"Inconnus"	,"Sconosciuti"	,"Desconhecidos"},
	/* Usr_GENDER_FEMALE */	{"Frauen"	,"Women"	,"Mujeres"	,"Femmes"	,"Donne"	,"Mulheres"	},
	/* Usr_GENDER_MALE */	{"M&auml;nner"	,"Men"		,"Hombres"	,"Hommes"	,"Uomini"	,"Homens"	},
	/* Usr_GENDER_ALL */	{"Alle"		,"All"		,"Todos"	,"Tous"		,"Tutti"	,"Todos"	},
	};
const char *Txt_GENDER_PLURAL_abc[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
	/* Usr_GENDER_UNKNOWN */	{"unbekannte"	,"unknown"	,"desconocidos"	,"inconnus"	,"sconosciuti"	,"desconhecidos"},
	/* Usr_GENDER_FEMALE */	{"Frauen"	,"women"	,"mujeres"	,"femmes"	,"donne"	,"mulheres"	},
	/* Usr_GENDER_MALE */	{"M&auml;nner"	,"men"		,"hombres"	,"hommes"	,"uomini"	,"homens"	},
	/* Usr_GENDER_ALL */	{"alle"		,"all"		,"todos"	,"tous"		,"tutti"	,"todos"	},
	};
const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
		{
		/* Usr_UNKNOWN - Usr_GENDER_UNKNOWN */	{"Unbekannter/e"	,"Unknown"		,"Desconocido/a"	,"Inconnu/e"		,"Sconosciuto/a"	,"Desconhecido/a"	},
		/* Usr_UNKNOWN - Usr_GENDER_FEMALE */	{"Unbekannte"		,"Unknown"		,"Desconocida"		,"Inconnue"		,"Sconosciuta"		,"Desconhecida"		},
		/* Usr_UNKNOWN - Usr_GENDER_MALE */		{"Unbekannter"		,"Unknown"		,"Desconocido"		,"Inconnu"		,"Sconosciuto"		,"Desconhecido"		},
		/* Usr_UNKNOWN - Usr_GENDER_ALL */		{"Unbekannter/e"	,"Unknown"		,"Desconocido/a"	,"Inconnu/e"		,"Sconosciuto/a"	,"Desconhecido/a"	},
		},
		{
		/* Usr_INVITED - Usr_GENDER_UNKNOWN */	{"Eingeladen"		,"Invited"		,"Invitado/a"		,"Invit&eacute;/e"	,"Ospite"		,"Convidado/a"		},
		/* Usr_INVITED - Usr_GENDER_FEMALE */	{"Eingeladen"		,"Invited"		,"Invitada"		,"Invit&eacute;e"	,"Ospite"		,"Convidada"		},
		/* Usr_INVITED - Usr_GENDER_MALE */		{"Eingeladen"		,"Invited"		,"Invitado"		,"Invit&eacute;"	,"Ospite"		,"Convidado"		},
		/* Usr_INVITED - Usr_GENDER_ALL */		{"Eingeladen"		,"Invited"		,"Invitado/a"		,"Invit&eacute;/e"	,"Ospite"		,"Convidado/a"		},
		},
		{
		/* Usr_STUDENT - Usr_GENDER_UNKNOWN */	{"Student"		,"Student"		,"Alumno/a"		,"&Eacute;tudiant/e"	,"Studente"		,"Estudante"		},
		/* Usr_STUDENT - Usr_GENDER_FEMALE */	{"Student"		,"Student"		,"Alumna"		,"&Eacute;tudiante"	,"Studente"		,"Estudante"		},
		/* Usr_STUDENT - Usr_GENDER_MALE */		{"Student"		,"Student"		,"Alumno"		,"&Eacute;tudiant"	,"Studente"		,"Estudante"		},
		/* Usr_STUDENT - Usr_GENDER_ALL */		{"Student"		,"Student"		,"Alumno/a"		,"&Eacute;tudiant/e"	,"Studente"		,"Estudante"		},
		},
		{
		/* Usr_TEACHER - Usr_GENDER_UNKNOWN */	{"Dozent"		,"Teacher"		,"Profesor/a"		,"Enseignant/e"		,"Insegnante"		,"Professor/a"		},
		/* Usr_TEACHER - Usr_GENDER_FEMALE */	{"Dozent"		,"Teacher"		,"Profesora"		,"Enseignante"		,"Insegnante"		,"Professora"		},
		/* Usr_TEACHER - Usr_GENDER_MALE */		{"Dozent"		,"Teacher"		,"Profesor"		,"Enseignant"		,"Insegnante"		,"Professor"		},
		/* Usr_TEACHER - Usr_GENDER_ALL */		{"Dozent"		,"Teacher"		,"Profesor/a"		,"Enseignant/e"		,"Insegnante"		,"Professor/a"		},
		},
		{
		/* Usr_DEG_ADMIN - Usr_GENDER_UNKNOWN */	{"Administrator"	,"Administrator"	,"Administrador/a"	,"Administrateur/se"	,"Amministratore"	,"Administrador/a"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_FEMALE */	{"Administrator"	,"Administrator"	,"Administradora"	,"Administrateuse"	,"Amministratore"	,"Administradora"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_MALE */	{"Administrator"	,"Administrator"	,"Administrador"	,"Administrateur"	,"Amministratore"	,"Administrador"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_ALL */	{"Administrator"	,"Administrator"	,"Administrador/a"	,"Administrateur/se"	,"Amministratore"	,"Administrador/a"	},
		},
		{
		/* Usr_SUPERUSER - Usr_GENDER_UNKNOWN */	{"Superuser"		,"Superuser"		,"Superusuario/a"	,"Superutilisateur/se"	,"Superutente"		,"Superutilizador/a"	},
		/* Usr_SUPERUSER - Usr_GENDER_FEMALE */	{"Superuser"		,"Superuser"		,"Superusuaria"		,"Superutilisateuse"	,"Superutente"		,"Superutilizadora"	},
		/* Usr_SUPERUSER - Usr_GENDER_MALE  */	{"Superuser"		,"Superuser"		,"Superusuario"		,"Superutilisateur"	,"Superutente"		,"Superutilizador"	},
		/* Usr_SUPERUSER - Usr_GENDER_ALL */	{"Superuser"		,"Superuser"		,"Superusuario/a"	,"Superutilisateur/se"	,"Superutente"		,"Superutilizador/a"	},
		}
	};
const char *Txt_USR_TYPES_SINGULAR_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
		{
		/* Usr_UNKNOWN - Usr_GENDER_UNKNOWN */	{"Unbekannter/e"	,"unknown"		,"desconocido/a"	,"inconnu/e"		,"sconosciuto/a"	,"desconhecido/a"	},
		/* Usr_UNKNOWN - Usr_GENDER_FEMALE */	{"Unbekannte"		,"unknown"		,"desconocida"		,"inconnue"		,"sconosciuta"		,"desconhecida"		},
		/* Usr_UNKNOWN - Usr_GENDER_MALE */		{"Unbekannter"		,"unknown"		,"desconocido"		,"inconnu"		,"sconosciuto"		,"desconhecido"		},
		/* Usr_UNKNOWN - Usr_GENDER_ALL */		{"Unbekannter/e"	,"unknown"		,"desconocido/a"	,"inconnu/e"		,"sconosciuto/a"	,"desconhecido/a"	},
		},
		{
		/* Usr_INVITED - Usr_GENDER_UNKNOWN */	{"Eingeladen"		,"invited"		,"invitado/a"		,"invit&eacute;/e"	,"ospite"		,"convidado/a"		},
		/* Usr_INVITED - Usr_GENDER_FEMALE */	{"Eingeladen"		,"invited"		,"invitada"		,"invit&eacute;e"	,"ospite"		,"convidada"		},
		/* Usr_INVITED - Usr_GENDER_MALE */		{"Eingeladen"		,"invited"		,"invitado"		,"invit&eacute;"	,"ospite"		,"convidado"		},
		/* Usr_INVITED - Usr_GENDER_ALL */		{"Eingeladen"		,"invited"		,"invitado/a"		,"invit&eacute;/e"	,"ospite"		,"convidado/a"		},
		},
		{
		/* Usr_STUDENT - Usr_GENDER_UNKNOWN */	{"Student"		,"student"		,"alumno/a"		,"&eacute;tudiant/e"	,"studente"		,"estudante"		},
		/* Usr_STUDENT - Usr_GENDER_FEMALE */	{"Student"		,"student"		,"alumna"		,"&eacute;tudiante"	,"studente"		,"estudante"		},
		/* Usr_STUDENT - Usr_GENDER_MALE */		{"Student"		,"student"		,"alumno"		,"&eacute;tudiant"	,"studente"		,"estudante"		},
		/* Usr_STUDENT - Usr_GENDER_ALL */		{"Student"		,"student"		,"alumno/a"		,"&eacute;tudiant/e"	,"studente"		,"estudante"		},
		},
		{
		/* Usr_TEACHER - Usr_GENDER_UNKNOWN */	{"Dozent"		,"teacher"		,"profesor/a"		,"enseignant/e"		,"insegnante"		,"professor/a"		},
		/* Usr_TEACHER - Usr_GENDER_FEMALE */	{"Dozent"		,"teacher"		,"profesora"		,"enseignante"		,"insegnante"		,"professora"		},
		/* Usr_TEACHER - Usr_GENDER_MALE */		{"Dozent"		,"teacher"		,"profesor"		,"enseignant"		,"insegnante"		,"professor"		},
		/* Usr_TEACHER - Usr_GENDER_ALL */		{"Dozent"		,"teacher"		,"profesor/a"		,"enseignant/e"		,"insegnante"		,"professor/a"		},
		},
		{
		/* Usr_DEG_ADMIN - Usr_GENDER_UNKNOWN */	{"Administrator"	,"administrator"	,"administrador/a"	,"administrateur/se"	,"amministratore"	,"administrador/a"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_FEMALE */	{"Administrator"	,"administrator"	,"administradora"	,"administrateuse"	,"amministratore"	,"administradora"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_MALE */	{"Administrator"	,"administrator"	,"administrador"	,"administrateur"	,"amministratore"	,"administrador"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_ALL */	{"Administrator"	,"administrator"	,"administrador/a"	,"administrateur/se"	,"amministratore"	,"administrador/a"	},
		},
		{
		/* Usr_SUPERUSER - Usr_GENDER_UNKNOWN */	{"Superuser"		,"superuser"		,"superusuario/a"	,"superutilisateur/se"	,"superutente"		,"superutilizador/a"	},
		/* Usr_SUPERUSER - Usr_GENDER_FEMALE */	{"Superuser"		,"superuser"		,"superusuaria"		,"superutilisateuse"	,"superutente"		,"superutilizadora"	},
		/* Usr_SUPERUSER - Usr_GENDER_MALE  */	{"Superuser"		,"superuser"		,"superusuario"		,"superutilisateur"	,"superutente"		,"superutilizador"	},
		/* Usr_SUPERUSER - Usr_GENDER_ALL */	{"Superuser"		,"superuser"		,"superusuario/a"	,"superutilisateur/se"	,"superutente"		,"superutilizador/a"	},
		}
	};
const char *Txt_USR_TYPES_PLURAL_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
		{
		/* Usr_UNKNOWN - Usr_GENDER_UNKNOWN */	{"Unbekannte"		,"Unknown"		,"Desconocidos"		,"Inconnus"		,"Sconosciuti"		,"Desconhecidos"	},
		/* Usr_UNKNOWN - Usr_GENDER_FEMALE */	{"Unbekannte"		,"Unknown"		,"Desconocidas"		,"Inconnues"		,"Sconosciuti"		,"Desconhecidas"	},
		/* Usr_UNKNOWN - Usr_GENDER_MALE */		{"Unbekannte"		,"Unknown"		,"Desconocidos"		,"Inconnus"		,"Sconosciuti"		,"Desconhecidos"	},
		/* Usr_UNKNOWN - Usr_GENDER_ALL */		{"Unbekannte"		,"Unknown"		,"Desconocidos"		,"Inconnus"		,"Sconosciuti"		,"Desconhecidos"	},
		},
		{
		/* Usr_INVITED - Usr_GENDER_UNKNOWN */	{"Eingeladen"		,"Invited"		,"Invitados"		,"Invit&eacute;s"	,"Ospiti"		,"Convidados"		},
		/* Usr_INVITED - Usr_GENDER_FEMALE */	{"Eingeladen"		,"Invited"		,"Invitadas"		,"Invit&eacute;es"	,"Ospiti"		,"Convidadas"		},
		/* Usr_INVITED - Usr_GENDER_MALE */		{"Eingeladen"		,"Invited"		,"Invitados"		,"Invit&eacute;s"	,"Ospiti"		,"Convidados"		},
		/* Usr_INVITED - Usr_GENDER_ALL */		{"Eingeladen"		,"Invited"		,"Invitados"		,"Invit&eacute;s"	,"Ospiti"		,"Convidados"		},
		},
		{
		/* Usr_STUDENT - Usr_GENDER_UNKNOWN */	{"Studenten"		,"Students"		,"Alumnos"		,"&Eacute;tudiants"	,"Studenti"		,"Estudantes"		},
		/* Usr_STUDENT - Usr_GENDER_FEMALE */	{"Studenten"		,"Students"		,"Alumnas"		,"&Eacute;tudiantes"	,"Studenti"		,"Estudantes"		},
		/* Usr_STUDENT - Usr_GENDER_MALE */		{"Studenten"		,"Students"		,"Alumnos"		,"&Eacute;tudiants"	,"Studenti"		,"Estudantes"		},
		/* Usr_STUDENT - Usr_GENDER_ALL */		{"Studenten"		,"Students"		,"Alumnos"		,"&Eacute;tudiants"	,"Studenti"		,"Estudantes"		},
		},
		{
		/* Usr_TEACHER - Usr_GENDER_UNKNOWN */	{"Dozent"		,"Teachers"		,"Profesores"		,"Enseignants"		,"Insegnanti"		,"Professores"		},
		/* Usr_TEACHER - Usr_GENDER_FEMALE */	{"Dozent"		,"Teachers"		,"Profesoras"		,"Enseignantes"		,"Insegnanti"		,"Professoras"		},
		/* Usr_TEACHER - Usr_GENDER_MALE */		{"Dozent"		,"Teachers"		,"Profesores"		,"Enseignants"		,"Insegnanti"		,"Professores"		},
		/* Usr_TEACHER - Usr_GENDER_ALL */		{"Dozent"		,"Teachers"		,"Profesores"		,"Enseignants"		,"Insegnanti"		,"Professores"		},
		},
		{
		/* Usr_DEG_ADMIN - Usr_GENDER_UNKNOWN */	{"Administratoren"	,"Administrators"	,"Administradores"	,"Administrateurs"	,"Amministratori"	,"Administradores"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_FEMALE */	{"Administratoren"	,"Administrators"	,"Administradoras"	,"Administrateuses"	,"Amministratori"	,"Administradoras"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_MALE */	{"Administratoren"	,"Administrators"	,"Administradores"	,"Administrateurs"	,"Amministratori"	,"Administradores"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_ALL */	{"Administratoren"	,"Administrators"	,"Administradores"	,"Administrateurs"	,"Amministratori"	,"Administradores"	},
		},
		{
		/* Usr_SUPERUSER - Usr_GENDER_UNKNOWN */	{"Superuser"		,"Superusers"		,"Superusuarios"	,"Superutilisateurs"	,"Superutenti"		,"Superutilizadores"	},
		/* Usr_SUPERUSER - Usr_GENDER_FEMALE */	{"Superuser"		,"Superusers"		,"Superusuarias"	,"Superutilisateuses"	,"Superutenti"		,"Superutilizadoras"	},
		/* Usr_SUPERUSER - Usr_GENDER_MALE  */	{"Superuser"		,"Superusers"		,"Superusuarios"	,"Superutilisateurs"	,"Superutenti"		,"Superutilizadores"	},
		/* Usr_SUPERUSER - Usr_GENDER_ALL */	{"Superuser"		,"Superusers"		,"Superusuarios"	,"Superutilisateurs"	,"Superutenti"		,"Superutilizadores"	},
		}
	};
const char *Txt_USR_TYPES_PLURAL_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES] =
	{
		{
		/* Usr_UNKNOWN - Usr_GENDER_UNKNOWN */	{"Unbekannte"		,"unknown"		,"desconocidos"		,"inconnus"		,"sconosciuti"		,"desconhecidos"	},
		/* Usr_UNKNOWN - Usr_GENDER_FEMALE */	{"Unbekannte"		,"unknown"		,"desconocidas"		,"inconnues"		,"sconosciuti"		,"desconhecidas"	},
		/* Usr_UNKNOWN - Usr_GENDER_MALE */		{"Unbekannte"		,"unknown"		,"desconocidos"		,"inconnus"		,"sconosciuti"		,"desconhecidos"	},
		/* Usr_UNKNOWN - Usr_GENDER_ALL */		{"Unbekannte"		,"unknown"		,"desconocidos"		,"inconnus"		,"sconosciuti"		,"desconhecidos"	},
		},
		{
		/* Usr_INVITED - Usr_GENDER_UNKNOWN */	{"Eingeladen"		,"invited"		,"invitados"		,"invit&eacute;s"	,"ospiti"		,"convidados"		},
		/* Usr_INVITED - Usr_GENDER_FEMALE */	{"Eingeladen"		,"invited"		,"invitadas"		,"invit&eacute;es"	,"ospiti"		,"convidadas"		},
		/* Usr_INVITED - Usr_GENDER_MALE */		{"Eingeladen"		,"invited"		,"invitados"		,"invit&eacute;s"	,"ospiti"		,"convidados"		},
		/* Usr_INVITED - Usr_GENDER_ALL */		{"Eingeladen"		,"invited"		,"invitados"		,"invit&eacute;s"	,"ospiti"		,"convidados"		},
		},
		{
		/* Usr_STUDENT - Usr_GENDER_UNKNOWN */	{"Studenten"		,"students"		,"alumnos"		,"&eacute;tudiants"	,"studenti"		,"estudantes"		},
		/* Usr_STUDENT - Usr_GENDER_FEMALE */	{"Studenten"		,"students"		,"alumnas"		,"&eacute;tudiantes"	,"studenti"		,"estudantes"		},
		/* Usr_STUDENT - Usr_GENDER_MALE */		{"Studenten"		,"students"		,"alumnos"		,"&eacute;tudiants"	,"studenti"		,"estudantes"		},
		/* Usr_STUDENT - Usr_GENDER_ALL */		{"Studenten"		,"students"		,"alumnos"		,"&eacute;tudiants"	,"studenti"		,"estudantes"		},
		},
		{
		/* Usr_TEACHER - Usr_GENDER_UNKNOWN */	{"Dozenten"		,"teachers"		,"profesores"		,"enseignants"		,"insegnanti"		,"professores"		},
		/* Usr_TEACHER - Usr_GENDER_FEMALE */	{"Dozenten"		,"teachers"		,"profesoras"		,"enseignantes"		,"insegnanti"		,"professoras"		},
		/* Usr_TEACHER - Usr_GENDER_MALE */		{"Dozenten"		,"teachers"		,"profesores"		,"enseignants"		,"insegnanti"		,"professores"		},
		/* Usr_TEACHER - Usr_GENDER_ALL */		{"Dozenten"		,"teachers"		,"profesores"		,"enseignants"		,"insegnanti"		,"professores"		},
		},
		{
		/* Usr_DEG_ADMIN - Usr_GENDER_UNKNOWN */	{"Administratoren"	,"administrators"	,"administradores"	,"administrateurs"	,"amministratori"	,"administradores"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_FEMALE */	{"Administratoren"	,"administrators"	,"administradoras"	,"administrateuses"	,"amministratori"	,"administradoras"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_MALE */	{"Administratoren"	,"administrators"	,"administradores"	,"administrateurs"	,"amministratori"	,"administradores"	},
		/* Usr_DEG_ADMIN - Usr_GENDER_ALL */	{"Administratoren"	,"administrators"	,"administradores"	,"administrateurs"	,"amministratori"	,"administradores"	},
		},
		{
		/* Usr_SUPERUSER - Usr_GENDER_UNKNOWN */	{"Superuser"		,"superusers"		,"superusuarios"	,"superutilisateurs"	,"superutenti"		,"superutilizadores"	},
		/* Usr_SUPERUSER - Usr_GENDER_FEMALE */	{"Superuser"		,"superusers"		,"superusuarias"	,"superutilisateuses"	,"superutenti"		,"superutilizadoras"	},
		/* Usr_SUPERUSER - Usr_GENDER_MALE  */	{"Superuser"		,"superusers"		,"superusuarios"	,"superutilisateurs"	,"superutenti"		,"superutilizadores"	},
		/* Usr_SUPERUSER - Usr_GENDER_ALL */	{"Superuser"		,"superusers"		,"superusuarios"	,"superutilisateurs"	,"superutenti"		,"superutilizadores"	},
		}
	};
const char *Txt_NAMES_MAIN_FIELDS_DATA_USR[Usr_NUM_MAIN_FIELDS_DATA_USR][Txt_NUM_LANGUAGES] =
	{
		{"&nbsp;"	,"&nbsp;"	,"&nbsp;"		,"&nbsp;"				,"&nbsp;"			,"&nbsp;"			},
		{"Anzahl"	,"#"		,"N&ordm;"		,"Nombre"				,"Numero"			,"N&ordm;"			},
		{"Foto"		,"Photo"	,"Foto"			,"Photo"				,"Foto"				,"Foto"				},
		{"Ausweis-Nr."	,"ID"		,"DNI"			,"Num&eacute;ro d'identit&eacute;"	,"Num. d'identit&agrave;"	,"N&ordm; identif."		},
		{"Nachname 1"	,"Surname 1"	,"1er. apellido"	,"Nom de famille 1"			,"Primo cognome"		,"Primeiro apelido"		},
		{"Nachname 2"	,"Surname 2"	,"2&ordm; apellido"	,"Nom de famille 2"			,"Secondo cognome"		,"Segundo apelido"		},
		{"Vorname"	,"First name"	,"Nombre"		,"Pr&eacute;nom"			,"Nome"				,"Nome"				},
		{"E-mail"	,"E-mail"	,"E-mail"		,"E-mail"				,"E-mail"			,"E-mail"			},
		{"Web"		,"Web"		,"Web"			,"Web"					,"Web"				,"Web"				},
		{"Skype"	,"Skype"	,"Skype"		,"Skype"				,"Skype"			,"Skype"			},
		{"Institution"	,"Institution"	,"Instituci&oacute;n"	,"&Eacute;tablissement"			,"Istituzione"			,"Institu&ccedil;&atilde;o"	},
	};
const char *Txt_NAMES_MAIN_FIELDS_DATA_ADM[Usr_NUM_MAIN_FIELDS_DATA_ADM][Txt_NUM_LANGUAGES] =
	{
		{"Anzahl"	,"#"		,"N&ordm;"		,"Nombre"				,"Numero"			,"N&ordm;"			},
		{"Foto"		,"Photo"	,"Foto"			,"Photo"				,"Foto"				,"Foto"				},
		{"Ausweis-Nr."	,"ID"		,"DNI"			,"Num&eacute;ro d'identit&eacute;"	,"Numero d'identit&agrave;"	,"N&ordm; identif."		},
		{"Nachname 1"	,"Surname 1"	,"1er. apellido"	,"Nom de famille 1"			,"Primo cognome"		,"Primeiro apelido"		},
		{"Nachname 2"	,"Surname 2"	,"2&ordm; apellido"	,"Nom de famille 2"			,"Secondo cognome"		,"Segundo apelido"		},
		{"Vorname"	,"First name"	,"Nombre"		,"Pr&eacute;nom"			,"Nome"				,"Nome"				},
		{"E-mail"	,"E-mail"	,"E-mail"		,"E-mail"				,"E-mail"			,"E-mail"			},
		{"Web"		,"Web"		,"Web"			,"Web"					,"Web"				,"Web"				},
		{"Skype"	,"Skype"	,"Skype"		,"Skype"				,"Skype"			,"Skype"			},
		{"Institution"	,"Institution"	,"Instituci&oacute;n"	,"&Eacute;tablissement"			,"Istituzione"			,"Institu&ccedil;&atilde;o"	},
	};
const char *Txt_NAMES_ALL_FIELDS_DATA_STD[Usr_NUM_ALL_FIELDS_DATA_STD][Txt_NUM_LANGUAGES] =
	{
		{"Foto"			,"Photo"		,"Foto"			,"Photo"				,"Foto"				,"Foto"					},
		{"Ausweis-Nr."		,"ID"			,"DNI"			,"Num&eacute;ro d'identit&eacute;"	,"Numero d'identit&agrave;"	,"N&ordm; identif."			},
		{"Nachname 1"		,"Surname 1"		,"1er. apellido"	,"Nom de famille 1"			,"Primo cognome"		,"Primeiro apelido"			},
		{"Nachname 2"		,"Surname 2"		,"2&ordm; apellido"	,"Nom de famille 2"			,"Secondo cognome"		,"Segundo apelido"			},
		{"Vorname"		,"First name"		,"Nombre"		,"Pr&eacute;nom"			,"Nome"				,"Nome"					},
		{"E-mail"		,"E-mail"		,"E-mail"		,"E-mail"				,"E-mail"			,"E-mail"				},
		{"Web"			,"Web"			,"Web"			,"Web"					,"Web"				,"Web"					},
		{"Skype"		,"Skype"		,"Skype"		,"Skype"				,"Skype"			,"Skype"				},
		{"Institution"		,"Institution"		,"Instituci&oacute;n"	,"&Eacute;tablissement"			,"Istituzione"			,"Institu&ccedil;&atilde;o"		},
		{"Lokale Adresse"	,"Local address"	,"Domicilio local"	,"Adresse locale"			,"Indirizzo locale"		,"Local Address"			},
		{"Telefon"		,"Phone"		,"Tel&eacute;fono"	,"T&eacute;l&eacute;phone"		,"Telefono"			,"Telefone"				},
		{"Familie-Adresse"	,"Family address"	,"Domicilio familiar"	,"Adresse famille"			,"Indirizzo famiglia"		,"Endere&ccedil;o Fam&iacute;lia"	},
		{"Telefon"		,"Phone"		,"Tel&eacute;fono"	,"T&eacute;l&eacute;phone"		,"Telefono"			,"Telefone"				},
		{"Herkunftsort"		,"Place of origin"	,"Lugar de origen"	,"Lieu d'origine"			,"Luogo di origine"		,"Lugar de origem"			},
		{"Geburtsdatum"		,"Date of birth"	,"Fecha nacimiento"	,"Date naissance"			,"Data nascita"			,"Data nascimento"			},
	};
const char *Txt_NAMES_ALL_FIELDS_DATA_TCH[Usr_NUM_ALL_FIELDS_DATA_TCH][Txt_NUM_LANGUAGES] =
	{
		{"Foto"		,"Photo"	,"Foto"			,"Photo"				,"Foto"				,"Foto"				},
		{"Ausweis-Nr."	,"ID"		,"DNI"			,"Num&eacute;ro d'identit&eacute;"	,"Numero d'identit&agrave;"	,"N&ordm; identif."		},
		{"Nachname 1"	,"Surname 1"	,"1er. apellido"	,"Nom de famille 1"			,"Primo cognome"		,"Primeiro apelido"		},
		{"Nachname 2"	,"Surname 2"	,"2&ordm; apellido"	,"Nom de famille 2"			,"Secondo cognome"		,"Segundo apelido"		},
		{"Vorname"	,"First name"	,"Nombre"		,"Pr&eacute;nom"			,"Nome"				,"Nome"				},
		{"E-mail"	,"E-mail"	,"E-mail"		,"E-mail"				,"E-mail"			,"E-mail"			},
		{"Web"		,"Web"		,"Web"			,"Web"					,"Web"				,"Web"				},
		{"Skype"	,"Skype"	,"Skype"		,"Skype"				,"Skype"			,"Skype"			},
		{"Institution"	,"Institution"	,"Instituci&oacute;n"	,"&Eacute;tablissement"			,"Istituzione"			,"Institu&ccedil;&atilde;o"	},
		{"Zentrum"	,"Centre"	,"Centro"		,"Centre"				,"Centro"			,"Centro"			},
		{"Fachbereich"	,"Department"	,"Departamento"		,"D&eacute;partement"			,"Dipartimento"			,"Departamento"			},
		{"Raum"		,"Office"	,"Despacho"		,"Bureau"				,"Ufficio"			,"Gabinete"			},
		{"Telefon"	,"Phone"	,"Tel&eacute;fono"	,"T&eacute;l&eacute;phone"		,"Telefono"			,"Telefone"			},
	};
const char *Txt_TABS_FULL_TXT[Act_NUM_TABS][Txt_NUM_LANGUAGES] =
	{
	/* TabUnk */	{NULL			,NULL		,NULL			,NULL			,NULL		,NULL				},       
	/* TabDeg */	{"Grad"			,"Degree"	,"Titulaci&oacute;n"	,"&Eacute;tude"		,"Laurea"	,"Titula&ccedil;&atilde;o"	},
	/* TabCrs */	{"Vorlesung"		,"Course"	,"Asignatura"		,"Mati&egrave;re"	,"Materia"	,"Disciplina"			},
	/* TabAss */	{"Bewertung"		,"Assessment"	,"Evaluaci&oacute;n"	,"&Eacute;valuation"	,"Valutazione"	,"Avalia&ccedil;&atilde;o"	},
	/* TabUsr */	{"Benutzer"		,"Users"	,"Usuarios"		,"Utilisateurs"		,"Utenti"	,"Utilizadores"			},
	/* TabMsg */	{"Nachrichten"		,"Messages"	,"Mensajes"		,"Messages"		,"Messaggi"	,"Mensagens"			},
	/* TabSta */	{"Statistik"		,"Statistics"	,"Estad&iacute;sticas"	,"Statistiques"		,"Statistiche"	,"Estat&iacute;sticas"		},
	/* TabPer */	{"Privat"		,"Personal"	,"Personal"		,"Personnel"		,"Personale"	,"Pessoal"			},
	/* TabPlg */	{"Plugins"		,"Plugins"	,"Complementos"		,"Plugins"		,"Plugins"	,"Plugins"			},
	};
const char *Txt_TABS_SHORT_TXT[Act_NUM_TABS][Txt_NUM_LANGUAGES] =
	{
	/* TabUnk */	{NULL			,NULL		,NULL			,NULL			,NULL		,NULL				},       
	/* TabDeg */	{"Grad"			,"Degree"	,"Titul."		,"&Eacute;tude"		,"Laurea"	,"Titula&ccedil;."		},
	/* TabCrs */	{"Vorlesung"		,"Course"	,"Asignatura"		,"Mati&egrave;re"	,"Materia"	,"Disciplina"			},
	/* TabAss */	{"Bewertung"		,"Assess."	,"Evaluac."		,"&Eacute;valuat."	,"Valutaz."	,"Avalia&ccedil;."		},
	/* TabUsr */	{"Benutzer"		,"Users"	,"Usuarios"		,"Utilis."		,"Utenti"	,"Utiliz."			},
	/* TabMsg */	{"Nachrich."		,"Msgs."	,"Mensajes"		,"Messages"		,"Messaggi"	,"Mensag."			},
	/* TabSta */	{"Stat."		,"Stats"	,"Estad."		,"Stat."		,"Stat."	,"Estat."			},
	/* TabPer */	{"Privat"		,"Personal"	,"Personal"		,"Personnel"		,"Personale"	,"Pessoal"			},
	/* TabPlg */	{"Plugins"		,"Plugins"	,"Comp."		,"Plugins"		,"Plugins"	,"Plugins"			},
	};
const char *Txt_MENU[Act_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB][Txt_NUM_LANGUAGES] =
	{
		{
		 // TabUnk *********************************************************************
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		 // TabDeg *********************************************************************
		/* ActReqSch		*/	{"Suchen"					,"Search"				,"Buscar"						,"Chercher"						,"Cercare"				,"Procurar"					},
		/* ActSeeIns		*/	{"Institutions"					,"Institutions"				,"Instituciones"					,"&Eacute;tablissement"					,"Istituzioni"				,"Institu&ccedil;&atilde;os"			},
		/* ActSeeCtr		*/	{"Zentren"					,"Centres"				,"Centros"						,"Centres"						,"Centri"				,"Centros"					},
		/* ActSeeDpt		*/	{"Fachbereiche"					,"Departments"				,"Departamentos"					,"D&eacute;partements"					,"Dipartimenti"				,"Departamentos"				},
		/* ActSeePlc		*/	{"Standorte"					,"Places"				,"Lugares"						,"Emplacements"						,"Localit&agrave;"			,"Localiza&ccedil;oes"				},
		/* ActSeeHld		*/	{"Feiertage"					,"Holidays"				,"D&iacute;as festivos"					,"Jours f&eacute;ri&eacute;s"				,"Vacanze"				,"F&eacute;rias"				},
		/* ActLstAdm		*/	{"Liste der<br />Verwalter"			,"Administrators"			,"Lista admi-<br />nistradores"				,"Liste admi-<br />nistrateurs"				,"Lista degli<br />amministratori"	,"Lista admi-<br />nistradores"			},
		/* ActReqNewAdm		*/	{"Neuer<br />Verwalter"				,"New admi-<br />nistrator"		,"Alta admi-<br />nistrador"				,"Nouvel<br />administrateur"				,"Nuovo ammi-<br />nistratore"		,"Novo admi-<br />nistrador"			},
		/* ActReqRemAdm		*/	{"Verwalter<br />entfernen"			,"Remove<br />administrator"		,"Baja admi-<br />nistrador"				,"Retirer<br />administrateur"				,"Rimuovere<br />amminist."		,"Tirar admi-<br />nistrador"			},
		/* ActReqEdiDegTyp	*/	{"Abschlussarten<br />bearbeiten"		,"Edit types<br />of degree"		,"Editar tipos<br />titulaci&oacute;n"			,"&Eacute;diter types<br />&eacute;tude"		,"Editare<br />tipi laurea"		,"Editar tipos<br />titula&ccedil;&atilde;o"	},
		/* ActReqEdiDeg		*/	{"Studienabschl&uuml;sse<br />bearbeiten"	,"Edit<br />degrees"			,"Editar<br />titulaciones"				,"&Eacute;diter<br />&eacute;tudes"			,"Editare<br />laurea"			,"Editar<br />titula&ccedil;&atilde;os"		},
		/* ActReqEdiCrs		*/	{"Vorlesungen<br />bearbeiten"			,"Edit<br />courses"			,"Editar<br />asignaturas"				,"&Eacute;diter<br />mati&egrave;res"			,"Editare<br />materie"			,"Editar<br />disciplinas"			},
		/* ActSeeLnk		*/	{"Links"					,"Links"				,"Enlaces"						,"Liens"						,"Links"				,"Liga&ccedil;&otilde;es"			},
		/* ActSetUp		*/	{"Eingerichtet"					,"Set up"				,"Instalar"						,"Installer"						,"Set up"				,"Instalar"					},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabCrs **********************************************************************
		/* ActSeeTchGui		*/	{"Teaching<br />Guide"				,"Teaching<br />guide"			,"Gu&iacute;a<br />docente"				,"Guide<br />p&eacute;dagogique"			,"Guida<br />didattica"			,"Ensino<br />de guia"				},
		/* ActSeeCal		*/	{"Kalender"					,"Calendar"				,"Calendario"						,"Calendrier"						,"Calendario<br />del corso"		,"Calend&aacute;rio"				},
		/* ActSeeCrsTimTbl	*/	{"Stundenplan"					,"Timetable"				,"Horario"						,"Horaire"						,"Orario"				,"Hor&aacute;rio"				},
		/* ActSeeSylLec		*/	{"Studienplan<br />(Vorlesung)"			,"Syllabus<br />(lectures)"		,"Programa<br />teor&iacute;a"				,"Plan<br />th&eacute;orie"				,"Programmaz.<br />classi teoriche" 	 ,"Programa<br />teoria"			},
		/* ActSeeSylPra		*/	{"Studienplan<br />(&Uuml;bungen)"		,"Syllabus<br />(practicals)"		,"Programa<br />pr&aacute;cticas"			,"Plan<br />pratique"					,"Programmaz.<br />classi pratiche"	,"Programa<br />pr&aacute;tica"			},
		/* ActAskSeeDow		*/	{"Downloads"					,"Download<br />zone"			,"Descargas"						,"Documents"						,"Scarica di<br />files utili"		,"Documentos"					},
		/* ActAskAdmDow		*/	{"Downloads<br />verwalten"			,"Manage<br />downloads"		,"Administrar<br />descargas"				,"Administrer<br />documents"				,"Amministrare<br />files"		,"Administrar<br />documentos"			},
		/* ActAskAdmCom		*/	{"Gemeinsamer<br />Bereich"			,"Common<br />zones"			,"Zonas<br />comunes"					,"Zones<br />communes"					,"Zone<br />comuni"			,"Zonas<br />comumes"				},
		/* ActSeeBib		*/	{"Literatur"					,"Bibliography"				,"Bibliograf&iacute;a"					,"Bibliographie"					,"Bibliografia"				,"Bibliografia"					},
		/* ActSeeFAQ		*/	{"FAQ"						,"FAQ"					,"FAQ"							,"FAQ"							,"FAQ"					,"FAQ"						},
		/* ActSeeCrsLnk		*/	{"Links"					,"Links"				,"Enlaces"						,"Liens"						,"Links"				,"Liga&ccedil;&otilde;es"			},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabAss **********************************************************************
		/* ActSeeAss		*/	{"Bewertungs-<br />Kriterien"			,"Assessment<br />system"		,"Sistema de<br />evaluaci&oacute;n"			,"Syst&egrave;me<br />d'&eacute;valuation"		,"Sistema di<br />valutazione"		,"Sistema de<br />avalia&ccedil;&atilde;o"	},
		/* ActSeeAsg		*/	{"Aktivit&auml;ten"				,"Assignments"				,"Actividades"						,"Activit&eacute;s"					,"Attivit&agrave;"			,"Atividades"					},
		/* ActSeeCrdCrs		*/	{"Profil bei<br />Vorlesung"			,"My record for<br />this course"	,"Mi ficha en<br />asignatura"				,"Ma carte<br />dans mati&egrave;re"			,"La mia scheda<br />in materia"	,"Meu cart&atilde;o<br />em disciplina"		},
		/* ActEdiAsgWrkUsr	*/	{"Eingereichte<br />Hausarbeiten"		,"My<br />works"			,"Mis<br />trabajos"					,"Mes<br />travaux"					,"Miei<br />lavori"			,"Os meus<br />trabalhos"			},
		/* ActReqAsgWrkCrs	*/	{"Hausarbeiten<br />der Benutzer"		,"User<br />works"			,"Trabajos<br />usuarios"				,"Travails<br />utilisateurs"				,"Lavori<br />utenti"			,"Trabalhos<br />utilizadores"			},
		/* ActReqTst		*/	{"Test<br />erstellen"				,"Self-assess-<br />ment test"		,"Test de<br />autoevaluaci&oacute;n"			,"Test<br />d'auto&eacute;valuation"			,"Test di<br />autovalutazione"		,"Teste de<br />autoavalia&ccedil;&atilde;o"	},
		/* ActWriTstQst		*/	{"Testfrage<br />eingeben"			,"Enter<br />question"			,"Escribir<br />pregunta"				,"&Eacute;crire<br />question"				,"Scrivere<br />domanda"		,"Escrever<br />quest&atilde;o"			},
		/* ActEdiTstQst		*/	{"Testfragen<br />bearbeiten"			,"Edit<br />questions"			,"Editar<br />preguntas"				,"&Eacute;diter<br />questions"				,"Editare<br />domande"			,"Editar<br />quest&otilde;es"			},
		/* ActCfgTst		*/	{"Test-<br />Einstellungen"			,"Configure<br />test"			,"Configurar<br />test"					,"Configurer<br />test"					,"Configurare<br />test"		,"Configurar<br />teste"			},
		/* ActSeeExaAnn		*/	{"Pr&uuml;fungs-<br />Ank&uuml;ndigung"		,"Announcements<br />of exams"		,"Convocatorias<br />de ex&aacute;menes"		,"Convocations<br />&agrave; examens"			,"Convocazioni<br />di esami"		,"Chamadas<br />exames"				},
		/* ActAskSeeAss		*/	{"Noten"					,"Marks"				,"Califica-<br />ciones"				,"Notes"						,"Voti"					,"Notas"					},
		/* ActAskAdmAss		*/	{"Noten<br />verwalten"				,"Manage<br />marks"			,"Administrar<br />calific."				,"Administrer<br />notes"				,"Amministrare<br />voti"		,"Administrar<br />notas"			},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabUsr **********************************************************************
		/* ActSeeStd		*/	{"Studenten-<br />Verzeichnis"			,"Students'<br />list"			,"Lista<br />alumnos"					,"Liste<br />&eacute;tudiants"				,"Elenco<br />studenti"			,"Lista<br />estudantes"			},
		/* ActGetExtLstStd	*/	{"Offizielle<br />Studenten"			,"Official<br />students"		,"Alumnos<br />oficiales"				,"&Eacute;tudiants<br />officiels"			,"Studenti<br />ufficiali"		,"Estudantes<br />oficiais"			},
		/* ActSeeTch		*/	{"Dozenten-<br />Verzeichnis"			,"Teachers'<br />list"			,"Lista<br />profesores"				,"Liste<br />enseignants"				,"Elenco<br />insegnanti"		,"Lista<br />professores"			},
		/* ActReqCrdCrsStd	*/	{"Studenten-<br />Profile"			,"Students'<br />records"		,"Fichas<br />alumnos"					,"Cartes<br />&eacute;tudiants"				,"Schede<br />studenti"			,"Cart&otilde;es<br />estudantes"		},
		/* ActEdiFieCrd		*/	{"Datenfelder"					,"Record<br />fields"			,"Campos<br />fichas"					,"Champs<br />carte"					,"Campi<br />schede"			,"Campos<br />cart&atilde;o"			},
		/* ActSeeCrdCrsTch	*/	{"Dozenten-<br />Profile"			,"Teachers'<br />records"		,"Fichas<br />profesores"				,"Cartes<br />enseignants"				,"Schede<br />insegnanti"		,"Cart&atilde;o<br />professores"		},
		/* ActSeeTut		*/	{"Sprechstunden"				,"Office<br />hours"			,"Horarios<br />tutor&iacute;as"			,"Horaires<br />tuteur"					,"Orari delle<br />tutorie"		,"Hor&aacute;rios<br />tutor"			},
		/* ActReqEdiGrp		*/	{"Gruppen<br />bearbeiten"			,"Edit<br />groups"			,"Editar<br />grupos"					,"&Eacute;diter<br />groupes"				,"Editare<br />gruppi"			,"Editar<br />grupos"				},
		/* ActReqSelGrp		*/	{"Meine<br />Gruppen"				,"My<br />groups"			,"Mis<br />grupos"					,"Mes<br />groupes"					,"Miei<br />gruppi"			,"Os meus<br />grupos"				},
		/* ActReqNewUsrCrs	*/	{"Benutzer<br />anlegen/bearb."			,"New/modify<br />user"			,"Alta/modif.<br />usuario"				,"Nouvel/modif.<br />utilisateur"			,"Nuovo/modif.<br />utente"		,"Novo/modif.<br />utilizador"			},
		/* ActMdfSevStdCrs	*/	{"Benutzer<br />anmelden/entfernen"		,"New/remove<br />users"		,"Alta/baja<br />usuarios"				,"Nouveaux/enlever<br />utilisateurs"			,"Nuovi/rimuovere<br />utenti"		,"Novos/tirar<br />utilizadores"			},
		/* ActReqPwdOthUsr	*/	{"Passw. eines<br />Benutzers"			,"Password of<br />another user"	,"Contrase&ntilde;a<br />otro usuario"			,"Mot de passe<br />autre utilis."			,"Password<br />altro utente"		,"Senha<br />outro utilizador"			},
		/* ActReqPhoUsr		*/	{"Foto eines<br />Benutzers"			,"Photo of<br />another user"		,"Foto otro<br />usuario"				,"Photo autre<br />utilisateur"				,"Foto<br />altro utente"		,"Foto outro<br />utilizador"			},
		/* ActReqRemUsrCrs	*/	{"Benutzer<br />entfernen"			,"Remove<br />user"			,"Baja<br />usuario"					,"Enlever<br />utilisateur"				,"Rimuovere<br />utente"		,"Tirar<br />utilizador"			},
		/* ActAskRemAllStd	*/	{"Alle Studenten<br />entfernen"		,"Remove all<br />students"		,"Baja todos<br />alumnos"				,"Enlever tous<br />&eacute;tudiants"			,"Rimuovere<br />tutti gli stud."	,"Tirar todos<br />estudantes"			},
		/* ActReqDNIRemGbl	*/	{"Benutzer<br />l&ouml;schen"			,"Eliminate<br />user"			,"Baja total<br />usuario"				,"&Eacute;liminer<br />utilisateur"			,"Eliminare<br />utente"		,"Eliminar<br />utilizador"			},
		/* ActAskRemOldUsr	*/	{"Fr&uuml;here<br />Benutzer<br />l&ouml;schen"	,"Eliminate<br />old users"		,"Baja usr.<br />antiguos"				,"&Eacute;liminer anciens<br />utilisateurs"		,"Eliminare<br />anticos utenti"	,"Eliminar<br />antigos uti."			},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabMsg **********************************************************************
		/* ActSeeNot		*/	{"Ank&uuml;ndigungen"				,"Notices"				,"Avisos"						,"Avis"							,"Avvisi"				,"Avisos"					},
		/* ActReqMsgUsr		*/	{"Neue<br />Nachricht"				,"New<br />message"			,"Escribir<br />mensaje"				,"Nouveau<br />message"					,"Nuovo<br />messaggio"			,"Nova<br />mensagem"				},
		/* ActSeeSntMsg		*/	{"Gesendete<br />Nachrichten"			,"Messages<br />sent"			,"Mensajes<br />enviados"				,"Messages<br />envoy&eacute;s"				,"Messaggi<br />inviati"		,"Mensagens<br />enviadas"			},
		/* ActSeeRecMsg		*/	{"Empfangene<br />Nachrichten"			,"Messages<br />received"		,"Mensajes<br />recibidos"				,"Messages<br />re&ccedil;us"				,"Messaggi<br />ricevuti"		,"Mensagens<br />recebidas"			},
		/* ActMaiStd		*/	{"E-Mail"					,"E-mail"				,"Correo<br />electr&oacute;nico"			,"Courrier<br />&eacute;lectronique"			,"E-mail"				,"E-mail"					},
		/* ActSeeFor		*/	{"Foren"					,"Forums"				,"Foros"						,"Forums"						,"Fori di<br />discussione"		,"F&oacute;runs"				},
		/* ActSeeChtRms		*/	{"Chat"						,"Chat"					,"Chat"							,"Chat"							,"Chat"					,"Chat"						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabSta **********************************************************************
		/* ActReqAccCrs		*/	{"Anmeldungen<br />bei Vorlesung"		,"Visits to<br />course"		,"Accesos<br />asignatura"				,"Visites<br />mati&egrave;re"				,"Visite<br />materia"			,"Visitas<br />disciplina"			},
		/* ActReqAccGbl		*/	{"Anmeldungen<br />bei SWAD"			,"Logins to<br />SWAD"			,"Accesos<br />SWAD"					,"Visites<br />SWAD"					,"Visite<br />SWAD"			,"Visitas<br />SWAD"				},
		/* ActReqUseGbl		*/	{"Nutzung<br />von SWAD"			,"Use of<br />SWAD"			,"Uso de<br />SWAD"					,"Utilisation<br />de SWAD"				,"Uso di<br />SWAD"			,"Utiliza&ccedil;&atilde;o<br />de SWAD"	},
		/* ActReqStaCrs		*/	{"Statistiken<br />Vorlesungen"			,"Statistics<br />courses"		,"Estad&iacute;sticas<br />asignaturas"			,"Statistiques<br />mati&egrave;res"			,"Statistiche<br />materie"		,"Estat&iacute;sticas<br />disciplinas"		},
		/* ActSeePhoDeg		*/	{"Mischbilder der<br />Studienabschl&uuml;sse"	,"Photos<br />of degrees"		,"Orla<br />titulaciones"				,"Photographie<br />&eacute;tudes"			,"Fotografia<br />di laurea"		,"Fotografia<br />titula&ccedil;oes"		},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabPer **********************************************************************
		/* ActFrmSndNewPwd	*/	{"Neues<br />Passwort"				,"New<br />password"			,"Nueva<br />contrase&ntilde;a"				,"Nouveau<br />mot de passe"				,"Nuova<br />password"			,"Nova<br />senha"				},
		/* ActFrmChgPwd		*/	{"Passwort"					,"Password"				,"Contrase&ntilde;a"					,"Mot<br />de passe"					,"Password"				,"Senha"					},
		/* ActFrmNic		*/	{"Benutzername"					,"Nickname"				,"Apodo"						,"Surnom"						,"Soprannome"				,"Alcunha"					},
		/* ActReqEdiCrdCom	*/	{"Profil"					,"Record<br />card"			,"Ficha"						,"Carte"						,"Scheda"				,"Cart&atilde;o"				},
		/* ActReqMyPho_new	*/	{"Foto"						,"Photo"				,"Foto"							,"Photo 2.0<br />(beta)"				,"Foto"					,"Foto"						},
		/* ActEdiPrf		*/	{"Einstellungen"				,"Preferences"				,"Preferencias"						,"Pr&eacute;f&eacute;rences"				,"Preferenze"				,"Prefer&ecirc;ncias"				},
		/* ActSeeMyTimTbl	*/	{"Stundenplan"					,"My timetable"				,"Horario mis<br />asignaturas"				,"Horaire mes<br />mati&egrave;res"			,"Orario delle<br />mie materie"	,"Hor&aacute;rio<br />minha disciplinas"	},
		/* ActEdiTut		*/	{"Stundenplan<br />(Sprechstunden)"		,"Timetable of<br />office hours"	,"Horario<br />tutor&iacute;as"				,"Horaire<br />tuteur"					,"Orario della<br />tutoria"		,"Hor&aacute;rio<br />tutor"			},
		/* AccAdmMlt		*/	{"Aktentasche"					,"Virtual<br />pendrive"		,"Malet&iacute;n"					,"Porte-<br />documents"				,"Pendrive<br />virtuale"		,"Pasta"					},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabPlg *******************************************************************
		/* ActLstPlg		*/	{"Liste<br />plugins"				,"List<br />plugins"			,"Lista<br />complem."					,"Liste<br />plugins"					,"Lista<br />plugins"			,"Lista<br />plugins"				},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
	};
const char *Txt_OPTION_TITLE[Act_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB][Txt_NUM_LANGUAGES] =
	{
		{
		 // TabUnk *********************************************************************
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabDeg *************************************************************************
		/* ActReqSch		*/	{"Suchen"					,"Search of courses / teachers"		,"B&uacute;squeda de asignaturas / profesores"		,"Chercher"						,"Cercare"				,"Procurar"					},
		/* ActSeeIns		*/	{"Institutions"					,"Institutions"				,"Instituciones"					,"&Eacute;tablissement"					,"Istituzioni"				,"Institu&ccedil;&atilde;os"			},
		/* ActSeeCtr		*/	{"Zentren"					,"Centres"				,"Centros"						,"Centres"						,"Centri"				,"Centros"					},
		/* ActSeeDpt		*/	{"Fachbereiche"					,"Departments"				,"Departamentos"					,"D&eacute;partements"					,"Dipartimenti"				,"Departamentos"				},
		/* ActSeePlc		*/	{"Standorte"					,"Places"				,"Lugares"						,"Emplacements"						,"Localit&agrave;"			,"Localiza&ccedil;oes"				},
		/* ActSeeHld		*/	{"Urlaub und Non-Schule Zeitraum"		,"Holidays and non school days"		,"D&iacute;as festivos y no lectivos"			,"Jours f&eacute;ri&eacute;s et sans &eacute;cole"	,"Vacanze e giorni senza scuola"	,"Feriados e dias sem escola"			},
		/* ActLstAdm		*/	{"Liste der Verwalter"				,"Administrators"			,"Lista de administradores"				,"Liste administrateurs"				,"Lista degli amministratori"		,"Lista admi- nistradores"			},
		/* ActReqNewAdm		*/	{"Neuer Verwalter"				,"New administrator"			,"Alta de administrador"				,"Nouvel administrateur"				,"Nuovo amministratore"			,"Novo administrador"				},
		/* ActReqRemAdm		*/	{"Verwalter entfernen"				,"Remove administrator"			,"Baja de administrador"				,"Retirer administrateur"				,"Rimuovere amminist."			,"Tirar administrador"				},
		/* ActReqEdiDegTyp	*/	{"Abschlussarten bearbeiten"			,"Edit types of degree"			,"Edici&oacute;n de tipos de titulaci&oacute;n"		,"&Eacute;diter types &eacute;tude"			,"Editare tipi laurea"			,"Editar tipos titula&ccedil;&atilde;o"		},
		/* ActReqEdiDeg		*/	{"Studienabschl&uuml;sse bearbeiten"		,"Edit degrees"				,"Edici&oacute;n de titulaciones"			,"&Eacute;diter &eacute;tudes"				,"Editare laurea"			,"Editar titula&ccedil;&atilde;os"		},
		/* ActReqEdiCrs		*/	{"Vorlesungen bearbeiten"			,"Edit courses"				,"Edici&oacute;n de asignaturas"			,"&Eacute;diter mati&egrave;res"			,"Editare materie"			,"Editar disciplinas"				},
		/* ActSeeLnk		*/	{"Institutionelle Links"			,"Institutional links"			,"Enlaces institucionales"				,"Liens institutionnels"				,"Links istituzionali"			,"Liga&ccedil;&otilde;es institucionais"	},
		/* ActSetUp		*/	{"Eingerichtet"					,"Set up"				,"Instalar"						,"Installer"						,"Set up"				,"Instalar"					},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabCrs *************************************************************************
		/* ActSeeTchGui		*/	{"Teaching Guide"				,"Teaching guide"			,"Gu&iacute;a docente"					,"Guide p&eacute;dagogique de la mati&egrave;re"	,"Guida didattica"			,"Ensino de guia"				},
		/* ActSeeCal		*/	{"Kalender"					,"Academic calendar"			,"Calendario acad&eacute;mico"				,"Calendrier"						,"Calendario del corso"			,"Calend&aacute;rio"				},
		/* ActSeeCrsTimTbl	*/	{"Stundenplan"					,"Class timetable"			,"Horario de clases"					,"Horaire"						,"Orario"				,"Hor&aacute;rio"				},
		/* ActSeeSylLec		*/	{"Studienplan (Vorlesung)"			,"Syllabus (lectures)"			,"Programa de teor&iacute;a"				,"Plan th&eacute;orie"					,"Programmaz. classi teoriche"		,"Programa teoria"				},
		/* ActSeeSylPra		*/	{"Studienplan (&Uuml;bungen)"			,"Syllabus (practicals)"		,"Programa de pr&aacute;cticas"				,"Plan pratique"					,"Programmaz. classi pratiche"		,"Programa pr&aacute;tica"			},
		/* ActAskSeeDow		*/	{"Downloads"					,"Download zone"			,"Descarga de archivos"					,"Documents"						,"Scarica di files utili"		,"Documentos"					},
		/* ActAskAdmDow		*/	{"Downloads verwalten"				,"Manage downloads"			,"Administraci&oacute;n de archivos para descarga"	,"Administrer documents"				,"Amministrare files"			,"Administrar documentos"			},
		/* ActAskAdmCom		*/	{"Gemeinsamer Bereich"				,"Common zones"				,"Zonas comunes"					,"Zones communes"					,"Zone comuni"				,"Zonas comumes"				},
		/* ActSeeBib		*/	{"Literatur"					,"Bibliography"				,"Bibliograf&iacute;a"					,"Bibliographie"					,"Bibliografia"				,"Bibliografia"					},
		/* ActSeeFAQ		*/	{"FAQ"						,"FAQ (Frequently Asked Questions)"	,"FAQ (preguntas frecuentes)"				,"FAQ"							,"FAQ"					,"FAQ"						},
		/* ActSeeCrsLnk		*/	{"Links"					,"Links"				,"Enlaces"						,"Liens"						,"Links"				,"Liga&ccedil;&otilde;es"			},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabAss *************************************************************************
		/* ActSeeAss		*/	{"BewertungsKriterien"				,"Assessment system"			,"Sistema de evaluaci&oacute;n"				,"Syst&egrave;me d'&eacute;valuation"			,"Sistema di valutazione"		,"Sistema de avalia&ccedil;&atilde;o"		},
		/* ActSeeAsg		*/	{"Aktivit&auml;ten"				,"Assignments"				,"Actividades"						,"Activit&eacute;s"					,"Attivit&agrave;"			,"Atividades"					},
		/* ActSeeCrdCrs		*/	{"Profil bei Vorlesung"				,"My record for this course"		,"Mi ficha en esta asignatura"				,"Ma carte dans mati&egrave;re"				,"La mia scheda in materia"		,"Meu cart&atilde;o em disciplina"		},
		/* ActEdiAsgWrkUsr	*/	{"Eingereichte Hausarbeiten"			,"My works"				,"Mis trabajos"						,"Mes travaux"						,"Miei lavori"				,"Os meus trabalhos"				},
		/* ActReqAsgWrkCrs	*/	{"Hausarbeiten der Benutzer"			,"User works"				,"Trabajos de usuarios"					,"Travails d'utilisateurs"				,"Lavori di utenti"			,"Trabalhos de utilizadores"			},
		/* ActReqTst		*/	{"Test erstellen"				,"Self-assessment test"			,"Test de autoevaluaci&oacute;n"			,"Test d'auto&eacute;valuation"				,"Test di autovalutazione"		,"Teste de autoavalia&ccedil;&atilde;o"		},
		/* ActWriTstQst		*/	{"Testfrage eingeben"				,"Edition of a question"		,"Edici&oacute;n de una pregunta tipo test"		,"&Eacute;crire question"				,"Scrivere domanda"			,"Escrever quest&atilde;o"			},
		/* ActEdiTstQst		*/	{"Testfragen bearbeiten"			,"Edition of existing questions"	,"Edici&oacute;n de preguntas tipo test existentes"	,"&Eacute;diter questions"				,"Editare domande"			,"Editar quest&otilde;es"			},
		/* ActCfgTst		*/	{"TestEinstellungen"				,"Configure test"			,"Configurar test de autoevaluaci&oacute;n"		,"Configurer test"					,"Configurare test"			,"Configurar teste"				},
		/* ActSeeExaAnn		*/	{"Pr&uuml;fungsAnk&uuml;ndigung"		,"Announcements of exams"		,"Convocatorias de ex&aacute;menes"			,"Convocations &agrave; examens"			,"Convocazioni di esami"		,"Chamadas para exames"				},
		/* ActAskSeeAss		*/	{"Noten"					,"Marks"				,"Consulta de calificaciones"				,"Notes"						,"Voti"					,"Notas"					},
		/* ActAskAdmAss		*/	{"Noten verwalten"				,"Manage marks"				,"Administraci&oacute;n de archivos de calificaciones"	,"Administrer notes"					,"Amministrare voti"			,"Administrar notas"				},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabUsr *******************************************************************
		/* ActSeeStd		*/	{"Studentenverzeichnis"				,"Students' list"			,"Orla o lista de alumnos"				,"Liste &eacute;tudiants"				,"Elenco studenti"			,"Lista estudantes"				},
		/* ActGetExtLstStd	*/	{"Meine offizielle Gruppen und Studenten"	,"My official groups and students"	,"Mis grupos y alumnos oficiales"			,"Mes groupes et &eacute;tudiants officiels"		,"I miei gruppi e studenti ufficiali"	,"Meus grupos e estudantes oficiais"		},
		/* ActSeeTch		*/	{"Dozentenverzeichnis"				,"Teachers' list"			,"Orla o lista de profesores"				,"Liste enseignants"					,"Elenco insegnanti"			,"Lista professores"				},
		/* ActReqCrdCrsStd	*/	{"Studentenprofile"				,"Students' records"			,"Fichas de alumnos"					,"Cartes &eacute;tudiants"				,"Schede studenti"			,"Cart&otilde;es estudantes"			},
		/* ActEdiFieCrd		*/	{"Datenfelder"					,"Record fields"			,"Campos de fichas para esta asignatura"		,"Champs carte"						,"Campi schede"				,"Campos cart&atilde;o"				},
		/* ActSeeCrdCrsTch	*/	{"Dozentenprofile"				,"Teachers' records"			,"Fichas de profesores"					,"Cartes enseignants"					,"Schede insegnanti"			,"Cart&otilde;es professores"			},
		/* ActSeeTut		*/	{"Sprechstunden"				,"Office hours"				,"Horarios de tutor&iacute;as"				,"Horaires tuteur"					,"Orari delle tutorie"			,"Hor&aacute;rios tutor"			},
		/* ActReqEdiGrp		*/	{"Gruppen bearbeiten"				,"Edit groups"				,"Edici&oacute;n de grupos"				,"&Eacute;diter groupes"				,"Editare gruppi"			,"Editar grupos"				},
		/* ActReqSelGrp		*/	{"Meine Gruppen"				,"My groups"				,"Mis grupos"						,"Mes groupes"						,"Miei gruppi"				,"Os meus grupos"				},
		/* ActReqNewUsrCrs	*/	{"Benutzer anlegen/bearb."			,"New / modify user"			,"Alta / modificaci&oacute;n de usuario"		,"Nouvel/modif. utilisateur"				,"Nuovo / modif. utente"		,"Novo / modif. utilizador"			},
		/* ActMdfSevStdCrs	*/	{"Benutzer anmelden/entfernen"			,"New/remove users"			,"Alta/baja varios usuarios"				,"Nouveaux/enlever utilisateurs"			,"Nuovi/rimuovere utenti"		,"Novos/tirar utilizadores"			},
		/* ActReqPwdOthUsr	*/	{"Passw. eines Benutzers"			,"Password of another user"		,"Cambio de contrase&ntilde;a de otro usuario"		,"Mot de passe autre utilis."				,"Password altro utente"		,"Senha outro utilizador"			},
		/* ActReqPhoUsr		*/	{"Foto eines Benutzers"				,"Photo of another user"		,"Fotograf&iacute;a de otro usuario"			,"Photo autre utilisateur"				,"Foto altro utente"			,"Foto outro utilizador"			},
		/* ActReqRemUsrCrs	*/	{"Benutzer entfernen"				,"Remove user"				,"Baja de usuario"					,"Enlever utilisateur"					,"Rimuovere utente"			,"Tirar utilizador"				},
		/* ActAskRemAllStd	*/	{"Alle Studenten entfernen"			,"Remove all students"			,"Baja de todos los alumnos"				,"Enlever tous &eacute;tudiants"			,"Rimuovere tutti gli studenti"		,"Tirar todos estudantes"			},
		/* ActReqDNIRemGbl	*/	{"Benutzer l&ouml;schen"			,"Eliminate user"			,"Eliminaci&oacute;n total de un usuario"		,"&Eacute;liminer utilisateur"				,"Eliminare utente"			,"Eliminar utilizador"				},
		/* ActAskRemOldUsr	*/	{"Fr&uuml;here Benutzer l&ouml;schen"		,"Eliminate old users"			,"Eliminaci&oacute;n total de usuarios antiguos"	,"&Eacute;liminer anciens utilisateurs"			,"Eliminare anticos utenti"		,"Eliminar antigos uti."			},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabMsg *******************************************************************
		/* ActSeeNot		*/	{"Ank&uuml;ndigungen"				,"Notices"				,"Avisos"						,"Avis"							,"Avvisi"				,"Avisos"					},
		/* ActReqMsgUsr		*/	{"Neue Nachricht"				,"New message"				,"Nuevo mensaje"					,"Nouveau message"					,"Nuovo messaggio"			,"Nova mensagem"				},
		/* ActSeeSntMsg		*/	{"Gesendete Nachrichten"			,"Messages sent"			,"Mensajes enviados"					,"Messages envoy&eacute;s"				,"Messaggi inviati"			,"Mensagens enviadas"				},
		/* ActSeeRecMsg		*/	{"Empfangene Nachrichten"			,"Messages received"			,"Mensajes recibidos"					,"Messages re&ccedil;us"				,"Messaggi ricevuti"			,"Mensagens recebidas"				},
		/* ActMaiStd		*/	{"E-Mail"					,"E-mail"				,"Env&iacute;o de correo electr&oacute;nico a alumnos"	,"Courrier &eacute;lectronique"				,"E-mail"				,"E-mail"					},
		/* ActSeeFor		*/	{"Foren"					,"Forums"				,"Foros de discusi&oacute;n"				,"Forums"						,"Fori di discussione"			,"F&oacute;runs"				},
		/* ActSeeChtRms		*/	{"Chat"						,"Chat"					,"Chat"							,"Chat"							,"Chat"					,"Chat"						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabSta *******************************************************************
		/* ActReqAccCrs		*/	{"Anmeldungen bei Vorlesung"			,"Visits to course"			,"Accesos a la asignatura"				,"Visites mati&egrave;re"				,"Visite materia"			,"Visitas disciplina"				},
		/* ActReqAccGbl		*/	{"Anmeldungen bei SWAD"				,"Logins to SWAD"			,"Accesos a todo SWAD"					,"Visites SWAD"						,"Visite SWAD"				,"Visitas SWAD"					},
		/* ActReqUseGbl		*/	{"Nutzung von SWAD"				,"Use of SWAD"				,"Uso actual de SWAD"					,"Utilisation de SWAD"					,"Uso di SWAD"				,"Utiliza&ccedil;&atilde;o de SWAD"		},
		/* ActReqStaCrs		*/	{"Statistiken von Vorlesungen"			,"Statistics of courses"		,"Estad&iacute;sticas de asignaturas"			,"Statistiques des mati&egrave;res"			,"Statistiche di materie"		,"Estat&iacute;sticas de disciplinas"		},
		/* ActSeePhoDeg		*/	{"Mischbilder der Studienabschl&uuml;sse"	,"Photos of degrees"			,"Fotos promedio de cada titulaci&oacute;n"		,"Photographie &eacute;tudes"				,"Fotografia di laurea"			,"Fotografia titula&ccedil;oes"			},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabPer *******************************************************************
		/* ActFrmSndNewPwd	*/	{"Neues Passwort"				,"New password"				,"Env&iacute;o de una nueva contrase&ntilde;a de acceso","Nouveau mot de passe"					,"Nuova password"			,"Nova senha"					},
		/* ActFrmChgPwd		*/	{"Passwort"					,"Password"				,"Cambio de contrase&ntilde;a de acceso"		,"Mot de passe"						,"Password"				,"Senha"					},
		/* ActFrmNic		*/	{"Benutzername"					,"Nickname"				,"Apodo"						,"Surnom"						,"Soprannome"				,"Alcunha"					},
		/* ActReqEdiCrdCom	*/	{"Profil"					,"Record"				,"Ficha personal"					,"Carte"						,"Scheda"				,"Cart&atilde;o"				},
		/* ActReqMyPho		*/	{"Foto"						,"Photo"				,"Fotograf&iacute;a"					,"Photo 2.0 (beta)"					,"Foto"					,"Foto"						},
		/* ActEdiPrf		*/	{"Einstellungen"				,"Preferences"				,"Preferencias"						,"Pr&eacute;f&eacute;rences"				,"Preferenze"				,"Prefer&ecirc;ncias"				},
		/* ActSeeMyTimTbl	*/	{"Stundenplan"					,"My timetable"				,"Horario de mis asignaturas"				,"Horaire mes mati&egrave;res"				,"Orario delle mie materie"		,"Hor&aacute;rio minha disciplinas"		},
		/* ActEdiTut		*/	{"Stundenplan (Sprechstunden)"			,"Timetable of office hours"		,"Horario de tutor&iacute;as"				,"Horaire tuteur"					,"Orario della tutoria"			,"Hor&aacute;rio tutor"				},
		/* AccAdmMlt		*/	{"Aktentasche"					,"Virtual pendrive"			,"Malet&iacute;n de almacenamiento personal"		,"Portedocuments"					,"Pendrive virtuale"			,"Pasta"					},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
		{
		// TabPlg *******************************************************************
		/* ActLstPlg		*/	{"Liste von plugins"				,"List of plugins"			,"Lista de complementos"				,"Liste de plugins"					,"Lista di plugins"			,"Lista de plugins"				},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
						{NULL						,NULL					,NULL							,NULL							,NULL					,NULL						},
		},
	};
